	{ "acst",  1, -14400, "America/Porto_Acre"            },
	{ "acst",  1, -14400, "America/Eirunepe"              },
	{ "acst",  1, -14400, "America/Rio_Branco"            },
	{ "acst",  1, -14400, "Brazil/Acre"                   },
	{ "act",   0, -18000, "America/Porto_Acre"            },
	{ "act",   0, -18000, "America/Eirunepe"              },
	{ "act",   0, -18000, "America/Rio_Branco"            },
	{ "act",   0, -18000, "Brazil/Acre"                   },
	{ "addt",  1,  -7200, "America/Goose_Bay"             },
	{ "addt",  1,  -7200, "America/Pangnirtung"           },
	{ "admt",  0,   9320, "Africa/Addis_Ababa"            },
	{ "admt",  0,   9320, "Africa/Asmara"                 },
	{ "admt",  0,   9320, "Africa/Asmera"                 },
	{ "adt",   1, -10800, "America/Halifax"               },
	{ "adt",   1, -10800, "America/Barbados"              },
	{ "adt",   1, -10800, "America/Blanc-Sablon"          },
	{ "adt",   1, -10800, "America/Glace_Bay"             },
	{ "adt",   1, -10800, "America/Goose_Bay"             },
	{ "adt",   1, -10800, "America/Martinique"            },
	{ "adt",   1, -10800, "America/Moncton"               },
	{ "adt",   1, -10800, "America/Pangnirtung"           },
	{ "adt",   1, -10800, "America/Thule"                 },
	{ "adt",   1, -10800, "Atlantic/Bermuda"              },
	{ "adt",   1, -10800, "Canada/Atlantic"               },
	{ "adt",   1,  14400, "Asia/Baghdad"                  },
	{ "aft",   0,  14400, "Asia/Kabul"                    },
	{ "aft",   0,  16200, "Asia/Kabul"                    },
	{ "ahdt",  1, -32400, "America/Anchorage"             },
	{ "ahst",  0, -36000, "America/Anchorage"             },
	{ "ahst",  0, -36000, "America/Adak"                  },
	{ "ahst",  0, -36000, "America/Atka"                  },
	{ "akdt",  1, -28800, "America/Anchorage"             },
	{ "akdt",  1, -28800, "America/Juneau"                },
	{ "akdt",  1, -28800, "America/Nome"                  },
	{ "akdt",  1, -28800, "America/Yakutat"               },
	{ "akst",  0, -32400, "America/Anchorage"             },
	{ "akst",  0, -32400, "America/Juneau"                },
	{ "akst",  0, -32400, "America/Nome"                  },
	{ "akst",  0, -32400, "America/Yakutat"               },
	{ "aktst", 1,  21600, "Asia/Aqtobe"                   },
	{ "aktt",  0,  14400, "Asia/Aqtobe"                   },
	{ "aktt",  0,  18000, "Asia/Aqtobe"                   },
	{ "aktt",  0,  21600, "Asia/Aqtobe"                   },
	{ "almst", 1,  25200, "Asia/Almaty"                   },
	{ "almt",  0,  18000, "Asia/Almaty"                   },
	{ "almt",  0,  21600, "Asia/Almaty"                   },
	{ "amst",  1,  14400, "Asia/Yerevan"                  },
	{ "amst",  1,  18000, "Asia/Yerevan"                  },
	{ "amst",  1, -10800, "America/Boa_Vista"             },
	{ "amst",  1, -10800, "America/Campo_Grande"          },
	{ "amst",  1, -10800, "America/Cuiaba"                },
	{ "amst",  1, -10800, "America/Manaus"                },
	{ "amst",  1, -10800, "America/Porto_Velho"           },
	{ "amst",  1, -10800, "America/Santarem"              },
	{ "amst",  1, -10800, "Brazil/West"                   },
	{ "amt",   0,  10800, "Asia/Yerevan"                  },
	{ "amt",   0,  14400, "Asia/Yerevan"                  },
	{ "amt",   0, -13840, "America/Asuncion"              },
	{ "amt",   0, -14400, "America/Boa_Vista"             },
	{ "amt",   0, -14400, "America/Campo_Grande"          },
	{ "amt",   0, -14400, "America/Cuiaba"                },
	{ "amt",   0, -14400, "America/Eirunepe"              },
	{ "amt",   0, -14400, "America/Manaus"                },
	{ "amt",   0, -14400, "America/Porto_Acre"            },
	{ "amt",   0, -14400, "America/Porto_Velho"           },
	{ "amt",   0, -14400, "America/Rio_Branco"            },
	{ "amt",   0, -14400, "America/Santarem"              },
	{ "amt",   0, -14400, "Brazil/Acre"                   },
	{ "amt",   0, -14400, "Brazil/West"                   },
	{ "amt",   0,   1172, "Europe/Amsterdam"              },
	{ "amt",   0,   5692, "Europe/Athens"                 },
	{ "anast", 1,  43200, "Asia/Anadyr"                   },
	{ "anast", 1,  46800, "Asia/Anadyr"                   },
	{ "anast", 1,  50400, "Asia/Anadyr"                   },
	{ "anat",  0,  39600, "Asia/Anadyr"                   },
	{ "anat",  0,  43200, "Asia/Anadyr"                   },
	{ "anat",  0,  46800, "Asia/Anadyr"                   },
	{ "ant",   0, -16200, "America/Curacao"               },
	{ "ant",   0, -16200, "America/Aruba"                 },
	{ "aot",   0,   3124, "Africa/Luanda"                 },
	{ "apt",   1, -10800, "America/Halifax"               },
	{ "apt",   1, -10800, "America/Blanc-Sablon"          },
	{ "apt",   1, -10800, "America/Glace_Bay"             },
	{ "apt",   1, -10800, "America/Moncton"               },
	{ "apt",   1, -10800, "America/Pangnirtung"           },
	{ "apt",   1, -10800, "America/Puerto_Rico"           },
	{ "apt",   1, -10800, "Canada/Atlantic"               },
	{ "aqtst", 1,  18000, "Asia/Aqtau"                    },
	{ "aqtst", 1,  21600, "Asia/Aqtau"                    },
	{ "aqtst", 1,  21600, "Asia/Aqtobe"                   },
	{ "aqtt",  0,  14400, "Asia/Aqtau"                    },
	{ "aqtt",  0,  18000, "Asia/Aqtau"                    },
	{ "aqtt",  0,  18000, "Asia/Aqtobe"                   },
	{ "arst",  1, -10800, "America/Buenos_Aires"          },
	{ "arst",  1,  -7200, "America/Buenos_Aires"          },
	{ "arst",  1, -10800, "America/Argentina/Buenos_Aires" },
	{ "arst",  1, -10800, "America/Argentina/Catamarca"   },
	{ "arst",  1, -10800, "America/Argentina/ComodRivadavia" },
	{ "arst",  1, -10800, "America/Argentina/Cordoba"     },
	{ "arst",  1, -10800, "America/Argentina/Jujuy"       },
	{ "arst",  1, -10800, "America/Argentina/La_Rioja"    },
	{ "arst",  1, -10800, "America/Argentina/Mendoza"     },
	{ "arst",  1, -10800, "America/Argentina/Rio_Gallegos" },
	{ "arst",  1, -10800, "America/Argentina/Salta"       },
	{ "arst",  1, -10800, "America/Argentina/San_Juan"    },
	{ "arst",  1, -10800, "America/Argentina/San_Luis"    },
	{ "arst",  1, -10800, "America/Argentina/Tucuman"     },
	{ "arst",  1, -10800, "America/Argentina/Ushuaia"     },
	{ "arst",  1, -10800, "America/Catamarca"             },
	{ "arst",  1, -10800, "America/Cordoba"               },
	{ "arst",  1, -10800, "America/Jujuy"                 },
	{ "arst",  1, -10800, "America/Mendoza"               },
	{ "arst",  1, -10800, "America/Rosario"               },
	{ "arst",  1, -10800, "Antarctica/Palmer"             },
	{ "arst",  1,  -7200, "America/Argentina/Buenos_Aires" },
	{ "arst",  1,  -7200, "America/Argentina/Catamarca"   },
	{ "arst",  1,  -7200, "America/Argentina/ComodRivadavia" },
	{ "arst",  1,  -7200, "America/Argentina/Cordoba"     },
	{ "arst",  1,  -7200, "America/Argentina/Jujuy"       },
	{ "arst",  1,  -7200, "America/Argentina/La_Rioja"    },
	{ "arst",  1,  -7200, "America/Argentina/Mendoza"     },
	{ "arst",  1,  -7200, "America/Argentina/Rio_Gallegos" },
	{ "arst",  1,  -7200, "America/Argentina/Salta"       },
	{ "arst",  1,  -7200, "America/Argentina/San_Juan"    },
	{ "arst",  1,  -7200, "America/Argentina/San_Luis"    },
	{ "arst",  1,  -7200, "America/Argentina/Tucuman"     },
	{ "arst",  1,  -7200, "America/Argentina/Ushuaia"     },
	{ "arst",  1,  -7200, "America/Catamarca"             },
	{ "arst",  1,  -7200, "America/Cordoba"               },
	{ "arst",  1,  -7200, "America/Jujuy"                 },
	{ "arst",  1,  -7200, "America/Mendoza"               },
	{ "arst",  1,  -7200, "America/Rosario"               },
	{ "arst",  1,  -7200, "Antarctica/Palmer"             },
	{ "art",   0, -10800, "America/Buenos_Aires"          },
	{ "art",   0, -14400, "America/Buenos_Aires"          },
	{ "art",   0, -10800, "America/Argentina/Buenos_Aires" },
	{ "art",   0, -10800, "America/Argentina/Catamarca"   },
	{ "art",   0, -10800, "America/Argentina/ComodRivadavia" },
	{ "art",   0, -10800, "America/Argentina/Cordoba"     },
	{ "art",   0, -10800, "America/Argentina/Jujuy"       },
	{ "art",   0, -10800, "America/Argentina/La_Rioja"    },
	{ "art",   0, -10800, "America/Argentina/Mendoza"     },
	{ "art",   0, -10800, "America/Argentina/Rio_Gallegos" },
	{ "art",   0, -10800, "America/Argentina/Salta"       },
	{ "art",   0, -10800, "America/Argentina/San_Juan"    },
	{ "art",   0, -10800, "America/Argentina/San_Luis"    },
	{ "art",   0, -10800, "America/Argentina/Tucuman"     },
	{ "art",   0, -10800, "America/Argentina/Ushuaia"     },
	{ "art",   0, -10800, "America/Catamarca"             },
	{ "art",   0, -10800, "America/Cordoba"               },
	{ "art",   0, -10800, "America/Jujuy"                 },
	{ "art",   0, -10800, "America/Mendoza"               },
	{ "art",   0, -10800, "America/Rosario"               },
	{ "art",   0, -10800, "Antarctica/Palmer"             },
	{ "art",   0, -14400, "America/Argentina/Buenos_Aires" },
	{ "art",   0, -14400, "America/Argentina/Catamarca"   },
	{ "art",   0, -14400, "America/Argentina/ComodRivadavia" },
	{ "art",   0, -14400, "America/Argentina/Cordoba"     },
	{ "art",   0, -14400, "America/Argentina/Jujuy"       },
	{ "art",   0, -14400, "America/Argentina/La_Rioja"    },
	{ "art",   0, -14400, "America/Argentina/Mendoza"     },
	{ "art",   0, -14400, "America/Argentina/Rio_Gallegos" },
	{ "art",   0, -14400, "America/Argentina/Salta"       },
	{ "art",   0, -14400, "America/Argentina/San_Juan"    },
	{ "art",   0, -14400, "America/Argentina/San_Luis"    },
	{ "art",   0, -14400, "America/Argentina/Tucuman"     },
	{ "art",   0, -14400, "America/Argentina/Ushuaia"     },
	{ "art",   0, -14400, "America/Catamarca"             },
	{ "art",   0, -14400, "America/Cordoba"               },
	{ "art",   0, -14400, "America/Jujuy"                 },
	{ "art",   0, -14400, "America/Mendoza"               },
	{ "art",   0, -14400, "America/Rosario"               },
	{ "art",   0, -14400, "Antarctica/Palmer"             },
	{ "ashst", 1,  18000, "Asia/Ashkhabad"                },
	{ "ashst", 1,  21600, "Asia/Ashkhabad"                },
	{ "ashst", 1,  18000, "Asia/Ashgabat"                 },
	{ "ashst", 1,  21600, "Asia/Ashgabat"                 },
	{ "asht",  0,  14400, "Asia/Ashkhabad"                },
	{ "asht",  0,  18000, "Asia/Ashkhabad"                },
	{ "asht",  0,  14400, "Asia/Ashgabat"                 },
	{ "asht",  0,  18000, "Asia/Ashgabat"                 },
	{ "ast",   0,  10800, "Asia/Riyadh"                   },
	{ "ast",   0, -14400, "America/Anguilla"              },
	{ "ast",   0, -14400, "America/Antigua"               },
	{ "ast",   0, -14400, "America/Aruba"                 },
	{ "ast",   0, -14400, "America/Barbados"              },
	{ "ast",   0, -14400, "America/Blanc-Sablon"          },
	{ "ast",   0, -14400, "America/Curacao"               },
	{ "ast",   0, -14400, "America/Dominica"              },
	{ "ast",   0, -14400, "America/Glace_Bay"             },
	{ "ast",   0, -14400, "America/Goose_Bay"             },
	{ "ast",   0, -14400, "America/Grenada"               },
	{ "ast",   0, -14400, "America/Guadeloupe"            },
	{ "ast",   0, -14400, "America/Halifax"               },
	{ "ast",   0, -14400, "America/Marigot"               },
	{ "ast",   0, -14400, "America/Martinique"            },
	{ "ast",   0, -14400, "America/Miquelon"              },
	{ "ast",   0, -14400, "America/Moncton"               },
	{ "ast",   0, -14400, "America/Montserrat"            },
	{ "ast",   0, -14400, "America/Pangnirtung"           },
	{ "ast",   0, -14400, "America/Port_of_Spain"         },
	{ "ast",   0, -14400, "America/Puerto_Rico"           },
	{ "ast",   0, -14400, "America/Santo_Domingo"         },
	{ "ast",   0, -14400, "America/St_Barthelemy"         },
	{ "ast",   0, -14400, "America/St_Kitts"              },
	{ "ast",   0, -14400, "America/St_Lucia"              },
	{ "ast",   0, -14400, "America/St_Thomas"             },
	{ "ast",   0, -14400, "America/St_Vincent"            },
	{ "ast",   0, -14400, "America/Thule"                 },
	{ "ast",   0, -14400, "America/Tortola"               },
	{ "ast",   0, -14400, "America/Virgin"                },
	{ "ast",   0, -14400, "Atlantic/Bermuda"              },
	{ "ast",   0, -14400, "Canada/Atlantic"               },
	{ "ast",   0,  10800, "Asia/Aden"                     },
	{ "ast",   0,  10800, "Asia/Baghdad"                  },
	{ "ast",   0,  10800, "Asia/Bahrain"                  },
	{ "ast",   0,  10800, "Asia/Kuwait"                   },
	{ "ast",   0,  10800, "Asia/Qatar"                    },
	{ "awt",   1, -10800, "America/Halifax"               },
	{ "awt",   1, -10800, "America/Blanc-Sablon"          },
	{ "awt",   1, -10800, "America/Glace_Bay"             },
	{ "awt",   1, -10800, "America/Moncton"               },
	{ "awt",   1, -10800, "America/Pangnirtung"           },
	{ "awt",   1, -10800, "America/Puerto_Rico"           },
	{ "awt",   1, -10800, "Canada/Atlantic"               },
	{ "azomt", 1,      0, "Atlantic/Azores"               },
	{ "azost", 1,  -3600, "Atlantic/Azores"               },
	{ "azost", 1,      0, "Atlantic/Azores"               },
	{ "azot",  0,  -3600, "Atlantic/Azores"               },
	{ "azot",  0,  -7200, "Atlantic/Azores"               },
	{ "azst",  1,  14400, "Asia/Baku"                     },
	{ "azst",  1,  18000, "Asia/Baku"                     },
	{ "azt",   0,  10800, "Asia/Baku"                     },
	{ "azt",   0,  14400, "Asia/Baku"                     },
	{ "bakst", 1,  14400, "Asia/Baku"                     },
	{ "bakst", 1,  18000, "Asia/Baku"                     },
	{ "bakt",  0,  10800, "Asia/Baku"                     },
	{ "bakt",  0,  14400, "Asia/Baku"                     },
	{ "bdst",  1,   7200, "Europe/London"                 },
	{ "bdst",  1,  25200, "Asia/Dacca"                    },
	{ "bdst",  1,  25200, "Asia/Dhaka"                    },
	{ "bdst",  1,   7200, "Europe/Belfast"                },
	{ "bdst",  1,   7200, "Europe/Gibraltar"              },
	{ "bdst",  1,   7200, "Europe/Guernsey"               },
	{ "bdst",  1,   7200, "Europe/Isle_of_Man"            },
	{ "bdst",  1,   7200, "Europe/Jersey"                 },
	{ "bdst",  1,   7200, "GB"                            },
	{ "bdt",   1, -36000, "America/Adak"                  },
	{ "bdt",   1, -36000, "America/Atka"                  },
	{ "bdt",   1, -36000, "America/Nome"                  },
	{ "bdt",   0,  21600, "Asia/Dacca"                    },
	{ "bdt",   0,  21600, "Asia/Dhaka"                    },
	{ "beat",  0,   9000, "Africa/Mogadishu"              },
	{ "beat",  0,   9000, "Africa/Kampala"                },
	{ "beat",  0,   9000, "Africa/Nairobi"                },
	{ "beaut", 0,   9885, "Africa/Nairobi"                },
	{ "beaut", 0,   9885, "Africa/Dar_es_Salaam"          },
	{ "beaut", 0,   9885, "Africa/Kampala"                },
	{ "bmt",   0, -14308, "America/Barbados"              },
	{ "bmt",   0, -17780, "America/Bogota"                },
	{ "bmt",   0,  -3996, "Africa/Banjul"                 },
	{ "bmt",   0,  10656, "Asia/Baghdad"                  },
	{ "bmt",   0,  24124, "Asia/Bangkok"                  },
	{ "bmt",   0,   6264, "Europe/Tiraspol"               },
	{ "bmt",   0,   6264, "Europe/Bucharest"              },
	{ "bmt",   0,   6264, "Europe/Chisinau"               },
	{ "bnt",   0,  27000, "Asia/Brunei"                   },
	{ "bnt",   0,  28800, "Asia/Brunei"                   },
	{ "bortst", 1,  30000, "Asia/Kuching"                  },
	{ "bort",  0,  27000, "Asia/Kuching"                  },
	{ "bort",  0,  28800, "Asia/Kuching"                  },
	{ "bost",  1, -12756, "America/La_Paz"                },
	{ "bot",   0, -14400, "America/La_Paz"                },
	{ "brst",  1,  -7200, "America/Sao_Paulo"             },
	{ "brst",  1,  -7200, "America/Araguaina"             },
	{ "brst",  1,  -7200, "America/Bahia"                 },
	{ "brst",  1,  -7200, "America/Belem"                 },
	{ "brst",  1,  -7200, "America/Fortaleza"             },
	{ "brst",  1,  -7200, "America/Maceio"                },
	{ "brst",  1,  -7200, "America/Recife"                },
	{ "brst",  1,  -7200, "Brazil/East"                   },
	{ "brt",   0, -10800, "America/Sao_Paulo"             },
	{ "brt",   0, -10800, "America/Araguaina"             },
	{ "brt",   0, -10800, "America/Bahia"                 },
	{ "brt",   0, -10800, "America/Belem"                 },
	{ "brt",   0, -10800, "America/Fortaleza"             },
	{ "brt",   0, -10800, "America/Maceio"                },
	{ "brt",   0, -10800, "America/Recife"                },
	{ "brt",   0, -10800, "America/Santarem"              },
	{ "brt",   0, -10800, "Brazil/East"                   },
	{ "bst",   0,   3600, "Europe/London"                 },
	{ "bst",   1,   3600, "Europe/London"                 },
	{ "bst",   0, -39600, "America/Adak"                  },
	{ "bst",   0, -39600, "America/Atka"                  },
	{ "bst",   0, -39600, "America/Nome"                  },
	{ "bst",   0, -39600, "Pacific/Midway"                },
	{ "bst",   0, -39600, "Pacific/Pago_Pago"             },
	{ "bst",   0, -39600, "Pacific/Samoa"                 },
	{ "bst",   0,   3600, "Europe/Belfast"                },
	{ "bst",   0,   3600, "Europe/Guernsey"               },
	{ "bst",   0,   3600, "Europe/Isle_of_Man"            },
	{ "bst",   0,   3600, "Europe/Jersey"                 },
	{ "bst",   0,   3600, "GB"                            },
	{ "bst",   1,   3600, "Europe/Belfast"                },
	{ "bst",   1,   3600, "Europe/Dublin"                 },
	{ "bst",   1,   3600, "Europe/Gibraltar"              },
	{ "bst",   1,   3600, "Europe/Guernsey"               },
	{ "bst",   1,   3600, "Europe/Isle_of_Man"            },
	{ "bst",   1,   3600, "Europe/Jersey"                 },
	{ "bst",   1,   3600, "GB"                            },
	{ "btt",   0,  21600, "Asia/Thimbu"                   },
	{ "btt",   0,  21600, "Asia/Thimphu"                  },
	{ "burt",  0,  23400, "Asia/Kolkata"                  },
	{ "burt",  0,  23400, "Asia/Calcutta"                 },
	{ "burt",  0,  23400, "Asia/Dacca"                    },
	{ "burt",  0,  23400, "Asia/Dhaka"                    },
	{ "burt",  0,  23400, "Asia/Rangoon"                  },
	{ "cant",  0,  -3600, "Atlantic/Canary"               },
	{ "capt",  1, -32400, "America/Anchorage"             },
	{ "cast",  0,  34200, "Australia/Adelaide"            },
	{ "cast",  1,  10800, "Africa/Gaborone"               },
	{ "cast",  1,  10800, "Africa/Khartoum"               },
	{ "cast",  0,  39600, "Antarctica/Casey"              },
	{ "cat",   0, -36000, "America/Anchorage"             },
	{ "cat",   0,   7200, "Africa/Khartoum"               },
	{ "cat",   0,   7200, "Africa/Blantyre"               },
	{ "cat",   0,   7200, "Africa/Bujumbura"              },
	{ "cat",   0,   7200, "Africa/Gaborone"               },
	{ "cat",   0,   7200, "Africa/Harare"                 },
	{ "cat",   0,   7200, "Africa/Kigali"                 },
	{ "cat",   0,   7200, "Africa/Lubumbashi"             },
	{ "cat",   0,   7200, "Africa/Lusaka"                 },
	{ "cat",   0,   7200, "Africa/Maputo"                 },
	{ "cat",   0,   7200, "Africa/Windhoek"               },
	{ "cawt",  1, -32400, "America/Anchorage"             },
	{ "cct",   0,  23400, "Indian/Cocos"                  },
	{ "cddt",  1, -14400, "America/Rankin_Inlet"          },
	{ "cddt",  1, -14400, "America/Resolute"              },
	{ "cdt",   1, -18000, "America/Chicago"               },
	{ "cdt",   1, -14400, "America/Havana"                },
	{ "cdt",   0, -18000, "America/Resolute"              },
	{ "cdt",   1, -18000, "America/Atikokan"              },
	{ "cdt",   1, -18000, "America/Bahia_Banderas"        },
	{ "cdt",   1, -18000, "America/Belize"                },
	{ "cdt",   1, -18000, "America/Cambridge_Bay"         },
	{ "cdt",   1, -18000, "America/Cancun"                },
	{ "cdt",   1, -18000, "America/Chihuahua"             },
	{ "cdt",   1, -18000, "America/Coral_Harbour"         },
	{ "cdt",   1, -18000, "America/Costa_Rica"            },
	{ "cdt",   1, -18000, "America/El_Salvador"           },
	{ "cdt",   1, -18000, "America/Fort_Wayne"            },
	{ "cdt",   1, -18000, "America/Guatemala"             },
	{ "cdt",   1, -18000, "America/Indiana/Indianapolis"  },
	{ "cdt",   1, -18000, "America/Indiana/Knox"          },
	{ "cdt",   1, -18000, "America/Indiana/Marengo"       },
	{ "cdt",   1, -18000, "America/Indiana/Petersburg"    },
	{ "cdt",   1, -18000, "America/Indiana/Tell_City"     },
	{ "cdt",   1, -18000, "America/Indiana/Vevay"         },
	{ "cdt",   1, -18000, "America/Indiana/Vincennes"     },
	{ "cdt",   1, -18000, "America/Indiana/Winamac"       },
	{ "cdt",   1, -18000, "America/Indianapolis"          },
	{ "cdt",   1, -18000, "America/Iqaluit"               },
	{ "cdt",   1, -18000, "America/Kentucky/Louisville"   },
	{ "cdt",   1, -18000, "America/Kentucky/Monticello"   },
	{ "cdt",   1, -18000, "America/Knox_IN"               },
	{ "cdt",   1, -18000, "America/Louisville"            },
	{ "cdt",   1, -18000, "America/Managua"               },
	{ "cdt",   1, -18000, "America/Matamoros"             },
	{ "cdt",   1, -18000, "America/Menominee"             },
	{ "cdt",   1, -18000, "America/Merida"                },
	{ "cdt",   1, -18000, "America/Mexico_City"           },
	{ "cdt",   1, -18000, "America/Monterrey"             },
	{ "cdt",   1, -18000, "America/North_Dakota/Center"   },
	{ "cdt",   1, -18000, "America/North_Dakota/New_Salem" },
	{ "cdt",   1, -18000, "America/Ojinaga"               },
	{ "cdt",   1, -18000, "America/Pangnirtung"           },
	{ "cdt",   1, -18000, "America/Rainy_River"           },
	{ "cdt",   1, -18000, "America/Rankin_Inlet"          },
	{ "cdt",   1, -18000, "America/Resolute"              },
	{ "cdt",   1, -18000, "America/Tegucigalpa"           },
	{ "cdt",   1, -18000, "America/Winnipeg"              },
	{ "cdt",   1, -18000, "Canada/Central"                },
	{ "cdt",   1, -18000, "Mexico/General"                },
	{ "cdt",   1,  32400, "Asia/Shanghai"                 },
	{ "cdt",   1,  32400, "Asia/Chongqing"                },
	{ "cdt",   1,  32400, "Asia/Chungking"                },
	{ "cdt",   1,  32400, "Asia/Harbin"                   },
	{ "cdt",   1,  32400, "Asia/Kashgar"                  },
	{ "cdt",   1,  32400, "Asia/Taipei"                   },
	{ "cdt",   1,  32400, "Asia/Urumqi"                   },
	{ "cdt",   1,  32400, "PRC"                           },
	{ "cdt",   1,  32400, "ROC"                           },
	{ "cemt",  1,  10800, "Europe/Berlin"                 },
	{ "cemt",  1,  10800, "CET"                           },
	{ "cest",  1,   7200, "Europe/Berlin"                 },
	{ "cest",  1,  10800, "Europe/Kaliningrad"            },
	{ "cest",  1,   7200, "Africa/Algiers"                },
	{ "cest",  1,   7200, "Africa/Ceuta"                  },
	{ "cest",  1,   7200, "Africa/Tripoli"                },
	{ "cest",  1,   7200, "Africa/Tunis"                  },
	{ "cest",  1,   7200, "Arctic/Longyearbyen"           },
	{ "cest",  1,   7200, "Atlantic/Jan_Mayen"            },
	{ "cest",  1,   7200, "CET"                           },
	{ "cest",  1,   7200, "Europe/Amsterdam"              },
	{ "cest",  1,   7200, "Europe/Andorra"                },
	{ "cest",  1,   7200, "Europe/Athens"                 },
	{ "cest",  1,   7200, "Europe/Belgrade"               },
	{ "cest",  1,   7200, "Europe/Bratislava"             },
	{ "cest",  1,   7200, "Europe/Brussels"               },
	{ "cest",  1,   7200, "Europe/Budapest"               },
	{ "cest",  1,   7200, "Europe/Chisinau"               },
	{ "cest",  1,   7200, "Europe/Copenhagen"             },
	{ "cest",  1,   7200, "Europe/Gibraltar"              },
	{ "cest",  1,   7200, "Europe/Kaliningrad"            },
	{ "cest",  1,   7200, "Europe/Kiev"                   },
	{ "cest",  1,   7200, "Europe/Lisbon"                 },
	{ "cest",  1,   7200, "Europe/Ljubljana"              },
	{ "cest",  1,   7200, "Europe/Luxembourg"             },
	{ "cest",  1,   7200, "Europe/Madrid"                 },
	{ "cest",  1,   7200, "Europe/Malta"                  },
	{ "cest",  1,   7200, "Europe/Minsk"                  },
	{ "cest",  1,   7200, "Europe/Monaco"                 },
	{ "cest",  1,   7200, "Europe/Oslo"                   },
	{ "cest",  1,   7200, "Europe/Paris"                  },
	{ "cest",  1,   7200, "Europe/Podgorica"              },
	{ "cest",  1,   7200, "Europe/Prague"                 },
	{ "cest",  1,   7200, "Europe/Riga"                   },
	{ "cest",  1,   7200, "Europe/Rome"                   },
	{ "cest",  1,   7200, "Europe/San_Marino"             },
	{ "cest",  1,   7200, "Europe/Sarajevo"               },
	{ "cest",  1,   7200, "Europe/Simferopol"             },
	{ "cest",  1,   7200, "Europe/Skopje"                 },
	{ "cest",  1,   7200, "Europe/Sofia"                  },
	{ "cest",  1,   7200, "Europe/Stockholm"              },
	{ "cest",  1,   7200, "Europe/Tallinn"                },
	{ "cest",  1,   7200, "Europe/Tirane"                 },
	{ "cest",  1,   7200, "Europe/Tiraspol"               },
	{ "cest",  1,   7200, "Europe/Uzhgorod"               },
	{ "cest",  1,   7200, "Europe/Vaduz"                  },
	{ "cest",  1,   7200, "Europe/Vatican"                },
	{ "cest",  1,   7200, "Europe/Vienna"                 },
	{ "cest",  1,   7200, "Europe/Vilnius"                },
	{ "cest",  1,   7200, "Europe/Warsaw"                 },
	{ "cest",  1,   7200, "Europe/Zagreb"                 },
	{ "cest",  1,   7200, "Europe/Zaporozhye"             },
	{ "cest",  1,   7200, "Europe/Zurich"                 },
	{ "cest",  1,   7200, "WET"                           },
	{ "cet",   0,   3600, "Europe/Berlin"                 },
	{ "cet",   0,   3600, "Africa/Algiers"                },
	{ "cet",   0,   3600, "Africa/Casablanca"             },
	{ "cet",   0,   3600, "Africa/Ceuta"                  },
	{ "cet",   0,   3600, "Africa/Tripoli"                },
	{ "cet",   0,   3600, "Africa/Tunis"                  },
	{ "cet",   0,   3600, "Arctic/Longyearbyen"           },
	{ "cet",   0,   3600, "Atlantic/Jan_Mayen"            },
	{ "cet",   0,   3600, "CET"                           },
	{ "cet",   0,   3600, "Europe/Amsterdam"              },
	{ "cet",   0,   3600, "Europe/Andorra"                },
	{ "cet",   0,   3600, "Europe/Athens"                 },
	{ "cet",   0,   3600, "Europe/Belgrade"               },
	{ "cet",   0,   3600, "Europe/Bratislava"             },
	{ "cet",   0,   3600, "Europe/Brussels"               },
	{ "cet",   0,   3600, "Europe/Budapest"               },
	{ "cet",   0,   3600, "Europe/Chisinau"               },
	{ "cet",   0,   3600, "Europe/Copenhagen"             },
	{ "cet",   0,   3600, "Europe/Gibraltar"              },
	{ "cet",   0,   3600, "Europe/Kaliningrad"            },
	{ "cet",   0,   3600, "Europe/Kiev"                   },
	{ "cet",   0,   3600, "Europe/Lisbon"                 },
	{ "cet",   0,   3600, "Europe/Ljubljana"              },
	{ "cet",   0,   3600, "Europe/Luxembourg"             },
	{ "cet",   0,   3600, "Europe/Madrid"                 },
	{ "cet",   0,   3600, "Europe/Malta"                  },
	{ "cet",   0,   3600, "Europe/Minsk"                  },
	{ "cet",   0,   3600, "Europe/Monaco"                 },
	{ "cet",   0,   3600, "Europe/Oslo"                   },
	{ "cet",   0,   3600, "Europe/Paris"                  },
	{ "cet",   0,   3600, "Europe/Podgorica"              },
	{ "cet",   0,   3600, "Europe/Prague"                 },
	{ "cet",   0,   3600, "Europe/Riga"                   },
	{ "cet",   0,   3600, "Europe/Rome"                   },
	{ "cet",   0,   3600, "Europe/San_Marino"             },
	{ "cet",   0,   3600, "Europe/Sarajevo"               },
	{ "cet",   0,   3600, "Europe/Simferopol"             },
	{ "cet",   0,   3600, "Europe/Skopje"                 },
	{ "cet",   0,   3600, "Europe/Sofia"                  },
	{ "cet",   0,   3600, "Europe/Stockholm"              },
	{ "cet",   0,   3600, "Europe/Tallinn"                },
	{ "cet",   0,   3600, "Europe/Tirane"                 },
	{ "cet",   0,   3600, "Europe/Tiraspol"               },
	{ "cet",   0,   3600, "Europe/Uzhgorod"               },
	{ "cet",   0,   3600, "Europe/Vaduz"                  },
	{ "cet",   0,   3600, "Europe/Vatican"                },
	{ "cet",   0,   3600, "Europe/Vienna"                 },
	{ "cet",   0,   3600, "Europe/Vilnius"                },
	{ "cet",   0,   3600, "Europe/Warsaw"                 },
	{ "cet",   0,   3600, "Europe/Zagreb"                 },
	{ "cet",   0,   3600, "Europe/Zaporozhye"             },
	{ "cet",   0,   3600, "Europe/Zurich"                 },
	{ "cet",   0,   3600, "WET"                           },
	{ "cet",   0,   7200, "Europe/Kaliningrad"            },
	{ "cgst",  1,  -3600, "America/Scoresbysund"          },
	{ "cgt",   0,  -7200, "America/Scoresbysund"          },
	{ "chadt", 1,  49500, "Pacific/Chatham"               },
	{ "chast", 0,  45900, "Pacific/Chatham"               },
	{ "chat",  0,  30600, "Asia/Harbin"                   },
	{ "chat",  0,  32400, "Asia/Harbin"                   },
	{ "chdt",  1, -19800, "America/Belize"                },
	{ "chost", 1,  36000, "Asia/Choibalsan"               },
	{ "chot",  0,  28800, "Asia/Choibalsan"               },
	{ "chot",  0,  32400, "Asia/Choibalsan"               },
	{ "chut",  0,  36000, "Pacific/Chuuk"                 },
	{ "chut",  0,  36000, "Pacific/Truk"                  },
	{ "chut",  0,  36000, "Pacific/Yap"                   },
	{ "cit",   0,  28800, "Asia/Dili"                     },
	{ "cit",   0,  28800, "Asia/Makassar"                 },
	{ "cit",   0,  28800, "Asia/Pontianak"                },
	{ "cit",   0,  28800, "Asia/Ujung_Pandang"            },
	{ "cjt",   0,  32400, "Asia/Sakhalin"                 },
	{ "cjt",   0,  32400, "Asia/Tokyo"                    },
	{ "ckhst", 1, -34200, "Pacific/Rarotonga"             },
	{ "ckt",   0, -36000, "Pacific/Rarotonga"             },
	{ "ckt",   0, -37800, "Pacific/Rarotonga"             },
	{ "clst",  1, -10800, "America/Santiago"              },
	{ "clst",  1, -14400, "America/Santiago"              },
	{ "clst",  1, -10800, "Antarctica/Palmer"             },
	{ "clst",  1, -10800, "Chile/Continental"             },
	{ "clst",  1, -14400, "Chile/Continental"             },
	{ "clt",   0, -14400, "America/Santiago"              },
	{ "clt",   0, -18000, "America/Santiago"              },
	{ "clt",   0, -14400, "Antarctica/Palmer"             },
	{ "clt",   0, -14400, "Chile/Continental"             },
	{ "clt",   0, -18000, "Chile/Continental"             },
	{ "cmt",   0, -14640, "America/St_Lucia"              },
	{ "cmt",   0, -15408, "America/Argentina/Buenos_Aires" },
	{ "cmt",   0, -15408, "America/Argentina/Catamarca"   },
	{ "cmt",   0, -15408, "America/Argentina/ComodRivadavia" },
	{ "cmt",   0, -15408, "America/Argentina/Cordoba"     },
	{ "cmt",   0, -15408, "America/Argentina/Jujuy"       },
	{ "cmt",   0, -15408, "America/Argentina/La_Rioja"    },
	{ "cmt",   0, -15408, "America/Argentina/Mendoza"     },
	{ "cmt",   0, -15408, "America/Argentina/Rio_Gallegos" },
	{ "cmt",   0, -15408, "America/Argentina/Salta"       },
	{ "cmt",   0, -15408, "America/Argentina/San_Juan"    },
	{ "cmt",   0, -15408, "America/Argentina/San_Luis"    },
	{ "cmt",   0, -15408, "America/Argentina/Tucuman"     },
	{ "cmt",   0, -15408, "America/Argentina/Ushuaia"     },
	{ "cmt",   0, -15408, "America/Buenos_Aires"          },
	{ "cmt",   0, -15408, "America/Catamarca"             },
	{ "cmt",   0, -15408, "America/Cordoba"               },
	{ "cmt",   0, -15408, "America/Jujuy"                 },
	{ "cmt",   0, -15408, "America/Mendoza"               },
	{ "cmt",   0, -15408, "America/Rosario"               },
	{ "cmt",   0, -16060, "America/Caracas"               },
	{ "cmt",   0, -16356, "America/La_Paz"                },
	{ "cmt",   0, -19176, "America/Panama"                },
	{ "cmt",   0,   6900, "Europe/Chisinau"               },
	{ "cmt",   0,   6900, "Europe/Tiraspol"               },
	{ "cost",  1, -14400, "America/Bogota"                },
	{ "cot",   0, -18000, "America/Bogota"                },
	{ "cpt",   1, -18000, "America/Chicago"               },
	{ "cpt",   1, -18000, "America/Atikokan"              },
	{ "cpt",   1, -18000, "America/Coral_Harbour"         },
	{ "cpt",   1, -18000, "America/Fort_Wayne"            },
	{ "cpt",   1, -18000, "America/Indiana/Indianapolis"  },
	{ "cpt",   1, -18000, "America/Indiana/Knox"          },
	{ "cpt",   1, -18000, "America/Indiana/Marengo"       },
	{ "cpt",   1, -18000, "America/Indiana/Petersburg"    },
	{ "cpt",   1, -18000, "America/Indiana/Tell_City"     },
	{ "cpt",   1, -18000, "America/Indiana/Vevay"         },
	{ "cpt",   1, -18000, "America/Indiana/Vincennes"     },
	{ "cpt",   1, -18000, "America/Indiana/Winamac"       },
	{ "cpt",   1, -18000, "America/Indianapolis"          },
	{ "cpt",   1, -18000, "America/Kentucky/Louisville"   },
	{ "cpt",   1, -18000, "America/Kentucky/Monticello"   },
	{ "cpt",   1, -18000, "America/Knox_IN"               },
	{ "cpt",   1, -18000, "America/Louisville"            },
	{ "cpt",   1, -18000, "America/Menominee"             },
	{ "cpt",   1, -18000, "America/Rainy_River"           },
	{ "cpt",   1, -18000, "America/Winnipeg"              },
	{ "cpt",   1, -18000, "Canada/Central"                },
	{ "cst",   0, -21600, "America/Chicago"               },
	{ "cst",   0, -18000, "America/Havana"                },
	{ "cst",   0, -21600, "America/Atikokan"              },
	{ "cst",   0, -21600, "America/Bahia_Banderas"        },
	{ "cst",   0, -21600, "America/Belize"                },
	{ "cst",   0, -21600, "America/Cambridge_Bay"         },
	{ "cst",   0, -21600, "America/Cancun"                },
	{ "cst",   0, -21600, "America/Chihuahua"             },
	{ "cst",   0, -21600, "America/Coral_Harbour"         },
	{ "cst",   0, -21600, "America/Costa_Rica"            },
	{ "cst",   0, -21600, "America/Detroit"               },
	{ "cst",   0, -21600, "America/El_Salvador"           },
	{ "cst",   0, -21600, "America/Fort_Wayne"            },
	{ "cst",   0, -21600, "America/Guatemala"             },
	{ "cst",   0, -21600, "America/Hermosillo"            },
	{ "cst",   0, -21600, "America/Indiana/Indianapolis"  },
	{ "cst",   0, -21600, "America/Indiana/Knox"          },
	{ "cst",   0, -21600, "America/Indiana/Marengo"       },
	{ "cst",   0, -21600, "America/Indiana/Petersburg"    },
	{ "cst",   0, -21600, "America/Indiana/Tell_City"     },
	{ "cst",   0, -21600, "America/Indiana/Vevay"         },
	{ "cst",   0, -21600, "America/Indiana/Vincennes"     },
	{ "cst",   0, -21600, "America/Indiana/Winamac"       },
	{ "cst",   0, -21600, "America/Indianapolis"          },
	{ "cst",   0, -21600, "America/Iqaluit"               },
	{ "cst",   0, -21600, "America/Kentucky/Louisville"   },
	{ "cst",   0, -21600, "America/Kentucky/Monticello"   },
	{ "cst",   0, -21600, "America/Knox_IN"               },
	{ "cst",   0, -21600, "America/Louisville"            },
	{ "cst",   0, -21600, "America/Managua"               },
	{ "cst",   0, -21600, "America/Matamoros"             },
	{ "cst",   0, -21600, "America/Mazatlan"              },
	{ "cst",   0, -21600, "America/Menominee"             },
	{ "cst",   0, -21600, "America/Merida"                },
	{ "cst",   0, -21600, "America/Mexico_City"           },
	{ "cst",   0, -21600, "America/Monterrey"             },
	{ "cst",   0, -21600, "America/North_Dakota/Center"   },
	{ "cst",   0, -21600, "America/North_Dakota/New_Salem" },
	{ "cst",   0, -21600, "America/Ojinaga"               },
	{ "cst",   0, -21600, "America/Pangnirtung"           },
	{ "cst",   0, -21600, "America/Rainy_River"           },
	{ "cst",   0, -21600, "America/Rankin_Inlet"          },
	{ "cst",   0, -21600, "America/Regina"                },
	{ "cst",   0, -21600, "America/Resolute"              },
	{ "cst",   0, -21600, "America/Swift_Current"         },
	{ "cst",   0, -21600, "America/Tegucigalpa"           },
	{ "cst",   0, -21600, "America/Thunder_Bay"           },
	{ "cst",   0, -21600, "America/Winnipeg"              },
	{ "cst",   0, -21600, "Canada/Central"                },
	{ "cst",   0, -21600, "Canada/East-Saskatchewan"      },
	{ "cst",   0, -21600, "Canada/Saskatchewan"           },
	{ "cst",   0, -21600, "Mexico/BajaSur"                },
	{ "cst",   0, -21600, "Mexico/General"                },
	{ "cst",   0,  28800, "Asia/Chongqing"                },
	{ "cst",   0,  28800, "Asia/Chungking"                },
	{ "cst",   0,  28800, "Asia/Harbin"                   },
	{ "cst",   0,  28800, "Asia/Kashgar"                  },
	{ "cst",   0,  28800, "Asia/Macao"                    },
	{ "cst",   0,  28800, "Asia/Macau"                    },
	{ "cst",   0,  28800, "Asia/Shanghai"                 },
	{ "cst",   0,  28800, "Asia/Taipei"                   },
	{ "cst",   0,  28800, "Asia/Urumqi"                   },
	{ "cst",   0,  28800, "PRC"                           },
	{ "cst",   0,  28800, "ROC"                           },
	{ "cst",   0,  34200, "Asia/Jayapura"                 },
	{ "cst",   0,  34200, "Australia/Adelaide"            },
	{ "cst",   0,  34200, "Australia/Broken_Hill"         },
	{ "cst",   0,  34200, "Australia/Darwin"              },
	{ "cst",   0,  34200, "Australia/North"               },
	{ "cst",   0,  34200, "Australia/South"               },
	{ "cst",   0,  34200, "Australia/Yancowinna"          },
	{ "cst",   1,  37800, "Australia/Adelaide"            },
	{ "cst",   1,  37800, "Australia/Broken_Hill"         },
	{ "cst",   1,  37800, "Australia/Darwin"              },
	{ "cst",   1,  37800, "Australia/North"               },
	{ "cst",   1,  37800, "Australia/South"               },
	{ "cst",   1,  37800, "Australia/Yancowinna"          },
	{ "cut",   0,   8400, "Europe/Zaporozhye"             },
	{ "cvst",  1,  -3600, "Atlantic/Cape_Verde"           },
	{ "cvt",   0,  -3600, "Atlantic/Cape_Verde"           },
	{ "cvt",   0,  -7200, "Atlantic/Cape_Verde"           },
	{ "cwst",  0,  31500, "Australia/Eucla"               },
	{ "cwst",  1,  35100, "Australia/Eucla"               },
	{ "cwt",   1, -18000, "America/Chicago"               },
	{ "cwt",   1, -18000, "America/Atikokan"              },
	{ "cwt",   1, -18000, "America/Coral_Harbour"         },
	{ "cwt",   1, -18000, "America/Fort_Wayne"            },
	{ "cwt",   1, -18000, "America/Indiana/Indianapolis"  },
	{ "cwt",   1, -18000, "America/Indiana/Knox"          },
	{ "cwt",   1, -18000, "America/Indiana/Marengo"       },
	{ "cwt",   1, -18000, "America/Indiana/Petersburg"    },
	{ "cwt",   1, -18000, "America/Indiana/Tell_City"     },
	{ "cwt",   1, -18000, "America/Indiana/Vevay"         },
	{ "cwt",   1, -18000, "America/Indiana/Vincennes"     },
	{ "cwt",   1, -18000, "America/Indiana/Winamac"       },
	{ "cwt",   1, -18000, "America/Indianapolis"          },
	{ "cwt",   1, -18000, "America/Kentucky/Louisville"   },
	{ "cwt",   1, -18000, "America/Kentucky/Monticello"   },
	{ "cwt",   1, -18000, "America/Knox_IN"               },
	{ "cwt",   1, -18000, "America/Louisville"            },
	{ "cwt",   1, -18000, "America/Menominee"             },
	{ "cwt",   1, -18000, "America/Mexico_City"           },
	{ "cwt",   1, -18000, "America/Rainy_River"           },
	{ "cwt",   1, -18000, "America/Winnipeg"              },
	{ "cwt",   1, -18000, "Canada/Central"                },
	{ "cwt",   1, -18000, "Mexico/General"                },
	{ "cxt",   0,  25200, "Indian/Christmas"              },
	{ "chst",  0,  36000, "Pacific/Guam"                  },
	{ "chst",  0,  36000, "Pacific/Saipan"                },
	{ "dact",  0,  21600, "Asia/Dacca"                    },
	{ "dact",  0,  21600, "Asia/Dhaka"                    },
	{ "davt",  0,  18000, "Antarctica/Davis"              },
	{ "davt",  0,  25200, "Antarctica/Davis"              },
	{ "ddut",  0,  36000, "Antarctica/DumontDUrville"     },
	{ "dmt",   0,  -1521, "Europe/Dublin"                 },
	{ "dusst", 1,  21600, "Asia/Dushanbe"                 },
	{ "dusst", 1,  25200, "Asia/Dushanbe"                 },
	{ "dust",  0,  18000, "Asia/Dushanbe"                 },
	{ "dust",  0,  21600, "Asia/Dushanbe"                 },
	{ "easst", 1, -18000, "Chile/EasterIsland"            },
	{ "easst", 1, -21600, "Chile/EasterIsland"            },
	{ "easst", 1, -18000, "Pacific/Easter"                },
	{ "easst", 1, -21600, "Pacific/Easter"                },
	{ "east",  0, -21600, "Chile/EasterIsland"            },
	{ "east",  0, -25200, "Chile/EasterIsland"            },
	{ "east",  0, -21600, "Pacific/Easter"                },
	{ "east",  0, -25200, "Pacific/Easter"                },
	{ "east",  1,  14400, "Indian/Antananarivo"           },
	{ "eat",   0,  10800, "Africa/Khartoum"               },
	{ "eat",   0,  10800, "Africa/Addis_Ababa"            },
	{ "eat",   0,  10800, "Africa/Asmara"                 },
	{ "eat",   0,  10800, "Africa/Asmera"                 },
	{ "eat",   0,  10800, "Africa/Dar_es_Salaam"          },
	{ "eat",   0,  10800, "Africa/Djibouti"               },
	{ "eat",   0,  10800, "Africa/Kampala"                },
	{ "eat",   0,  10800, "Africa/Mogadishu"              },
	{ "eat",   0,  10800, "Africa/Nairobi"                },
	{ "eat",   0,  10800, "Indian/Antananarivo"           },
	{ "eat",   0,  10800, "Indian/Comoro"                 },
	{ "eat",   0,  10800, "Indian/Mayotte"                },
	{ "ect",   0, -18000, "America/Guayaquil"             },
	{ "ect",   0, -18000, "Pacific/Galapagos"             },
	{ "eddt",  1, -10800, "America/Iqaluit"               },
	{ "edt",   1, -14400, "America/New_York"              },
	{ "edt",   1, -14400, "America/Cancun"                },
	{ "edt",   1, -14400, "America/Detroit"               },
	{ "edt",   1, -14400, "America/Fort_Wayne"            },
	{ "edt",   1, -14400, "America/Grand_Turk"            },
	{ "edt",   1, -14400, "America/Indiana/Indianapolis"  },
	{ "edt",   1, -14400, "America/Indiana/Marengo"       },
	{ "edt",   1, -14400, "America/Indiana/Petersburg"    },
	{ "edt",   1, -14400, "America/Indiana/Tell_City"     },
	{ "edt",   1, -14400, "America/Indiana/Vevay"         },
	{ "edt",   1, -14400, "America/Indiana/Vincennes"     },
	{ "edt",   1, -14400, "America/Indiana/Winamac"       },
	{ "edt",   1, -14400, "America/Indianapolis"          },
	{ "edt",   1, -14400, "America/Iqaluit"               },
	{ "edt",   1, -14400, "America/Jamaica"               },
	{ "edt",   1, -14400, "America/Kentucky/Louisville"   },
	{ "edt",   1, -14400, "America/Kentucky/Monticello"   },
	{ "edt",   1, -14400, "America/Louisville"            },
	{ "edt",   1, -14400, "America/Montreal"              },
	{ "edt",   1, -14400, "America/Nassau"                },
	{ "edt",   1, -14400, "America/Nipigon"               },
	{ "edt",   1, -14400, "America/Pangnirtung"           },
	{ "edt",   1, -14400, "America/Port-au-Prince"        },
	{ "edt",   1, -14400, "America/Santo_Domingo"         },
	{ "edt",   1, -14400, "America/Thunder_Bay"           },
	{ "edt",   1, -14400, "America/Toronto"               },
	{ "edt",   1, -14400, "Canada/Eastern"                },
	{ "edt",   1, -14400, "EST"                           },
	{ "eest",  1,  10800, "Europe/Helsinki"               },
	{ "eest",  1,  10800, "Africa/Cairo"                  },
	{ "eest",  1,  10800, "Asia/Amman"                    },
	{ "eest",  1,  10800, "Asia/Beirut"                   },
	{ "eest",  1,  10800, "Asia/Damascus"                 },
	{ "eest",  1,  10800, "Asia/Gaza"                     },
	{ "eest",  1,  10800, "Asia/Istanbul"                 },
	{ "eest",  1,  10800, "Asia/Nicosia"                  },
	{ "eest",  1,  10800, "EET"                           },
	{ "eest",  1,  10800, "Europe/Athens"                 },
	{ "eest",  1,  10800, "Europe/Bucharest"              },
	{ "eest",  1,  10800, "Europe/Chisinau"               },
	{ "eest",  1,  10800, "Europe/Istanbul"               },
	{ "eest",  1,  10800, "Europe/Kaliningrad"            },
	{ "eest",  1,  10800, "Europe/Kiev"                   },
	{ "eest",  1,  10800, "Europe/Mariehamn"              },
	{ "eest",  1,  10800, "Europe/Minsk"                  },
	{ "eest",  1,  10800, "Europe/Moscow"                 },
	{ "eest",  1,  10800, "Europe/Nicosia"                },
	{ "eest",  1,  10800, "Europe/Riga"                   },
	{ "eest",  1,  10800, "Europe/Simferopol"             },
	{ "eest",  1,  10800, "Europe/Sofia"                  },
	{ "eest",  1,  10800, "Europe/Tallinn"                },
	{ "eest",  1,  10800, "Europe/Tiraspol"               },
	{ "eest",  1,  10800, "Europe/Uzhgorod"               },
	{ "eest",  1,  10800, "Europe/Vilnius"                },
	{ "eest",  1,  10800, "Europe/Warsaw"                 },
	{ "eest",  1,  10800, "Europe/Zaporozhye"             },
	{ "eet",   0,   7200, "Europe/Helsinki"               },
	{ "eet",   1,  10800, "Asia/Gaza"                     },
	{ "eet",   0,   7200, "Africa/Cairo"                  },
	{ "eet",   0,   7200, "Africa/Tripoli"                },
	{ "eet",   0,   7200, "Asia/Amman"                    },
	{ "eet",   0,   7200, "Asia/Beirut"                   },
	{ "eet",   0,   7200, "Asia/Damascus"                 },
	{ "eet",   0,   7200, "Asia/Gaza"                     },
	{ "eet",   0,   7200, "Asia/Istanbul"                 },
	{ "eet",   0,   7200, "Asia/Nicosia"                  },
	{ "eet",   0,   7200, "EET"                           },
	{ "eet",   0,   7200, "Europe/Athens"                 },
	{ "eet",   0,   7200, "Europe/Bucharest"              },
	{ "eet",   0,   7200, "Europe/Chisinau"               },
	{ "eet",   0,   7200, "Europe/Istanbul"               },
	{ "eet",   0,   7200, "Europe/Kaliningrad"            },
	{ "eet",   0,   7200, "Europe/Kiev"                   },
	{ "eet",   0,   7200, "Europe/Mariehamn"              },
	{ "eet",   0,   7200, "Europe/Minsk"                  },
	{ "eet",   0,   7200, "Europe/Moscow"                 },
	{ "eet",   0,   7200, "Europe/Nicosia"                },
	{ "eet",   0,   7200, "Europe/Riga"                   },
	{ "eet",   0,   7200, "Europe/Simferopol"             },
	{ "eet",   0,   7200, "Europe/Sofia"                  },
	{ "eet",   0,   7200, "Europe/Tallinn"                },
	{ "eet",   0,   7200, "Europe/Tiraspol"               },
	{ "eet",   0,   7200, "Europe/Uzhgorod"               },
	{ "eet",   0,   7200, "Europe/Vilnius"                },
	{ "eet",   0,   7200, "Europe/Warsaw"                 },
	{ "eet",   0,   7200, "Europe/Zaporozhye"             },
	{ "egst",  1,      0, "America/Scoresbysund"          },
	{ "egt",   0,  -3600, "America/Scoresbysund"          },
	{ "ehdt",  1, -16200, "America/Santo_Domingo"         },
	{ "eit",   0,  32400, "Asia/Jayapura"                 },
	{ "emt",   0, -26248, "Chile/EasterIsland"            },
	{ "emt",   0, -26248, "Pacific/Easter"                },
	{ "ept",   1, -14400, "America/New_York"              },
	{ "ept",   1, -14400, "America/Detroit"               },
	{ "ept",   1, -14400, "America/Iqaluit"               },
	{ "ept",   1, -14400, "America/Montreal"              },
	{ "ept",   1, -14400, "America/Nipigon"               },
	{ "ept",   1, -14400, "America/Thunder_Bay"           },
	{ "ept",   1, -14400, "America/Toronto"               },
	{ "ept",   1, -14400, "Canada/Eastern"                },
	{ "ept",   1, -14400, "EST"                           },
	{ "est",   0, -18000, "America/New_York"              },
	{ "est",   0, -18000, "America/Antigua"               },
	{ "est",   0, -18000, "America/Atikokan"              },
	{ "est",   0, -18000, "America/Cambridge_Bay"         },
	{ "est",   0, -18000, "America/Cancun"                },
	{ "est",   0, -18000, "America/Cayman"                },
	{ "est",   0, -18000, "America/Chicago"               },
	{ "est",   0, -18000, "America/Coral_Harbour"         },
	{ "est",   0, -18000, "America/Detroit"               },
	{ "est",   0, -18000, "America/Fort_Wayne"            },
	{ "est",   0, -18000, "America/Grand_Turk"            },
	{ "est",   0, -18000, "America/Indiana/Indianapolis"  },
	{ "est",   0, -18000, "America/Indiana/Knox"          },
	{ "est",   0, -18000, "America/Indiana/Marengo"       },
	{ "est",   0, -18000, "America/Indiana/Petersburg"    },
	{ "est",   0, -18000, "America/Indiana/Tell_City"     },
	{ "est",   0, -18000, "America/Indiana/Vevay"         },
	{ "est",   0, -18000, "America/Indiana/Vincennes"     },
	{ "est",   0, -18000, "America/Indiana/Winamac"       },
	{ "est",   0, -18000, "America/Indianapolis"          },
	{ "est",   0, -18000, "America/Iqaluit"               },
	{ "est",   0, -18000, "America/Jamaica"               },
	{ "est",   0, -18000, "America/Kentucky/Louisville"   },
	{ "est",   0, -18000, "America/Kentucky/Monticello"   },
	{ "est",   0, -18000, "America/Knox_IN"               },
	{ "est",   0, -18000, "America/Louisville"            },
	{ "est",   0, -18000, "America/Managua"               },
	{ "est",   0, -18000, "America/Menominee"             },
	{ "est",   0, -18000, "America/Merida"                },
	{ "est",   0, -18000, "America/Moncton"               },
	{ "est",   0, -18000, "America/Montreal"              },
	{ "est",   0, -18000, "America/Nassau"                },
	{ "est",   0, -18000, "America/Nipigon"               },
	{ "est",   0, -18000, "America/Panama"                },
	{ "est",   0, -18000, "America/Pangnirtung"           },
	{ "est",   0, -18000, "America/Port-au-Prince"        },
	{ "est",   0, -18000, "America/Rankin_Inlet"          },
	{ "est",   0, -18000, "America/Resolute"              },
	{ "est",   0, -18000, "America/Santo_Domingo"         },
	{ "est",   0, -18000, "America/Thunder_Bay"           },
	{ "est",   0, -18000, "America/Toronto"               },
	{ "est",   0, -18000, "Canada/Eastern"                },
	{ "est",   0, -18000, "EST"                           },
	{ "est",   0,  36000, "Antarctica/Macquarie"          },
	{ "est",   0,  36000, "Australia/ACT"                 },
	{ "est",   0,  36000, "Australia/Brisbane"            },
	{ "est",   0,  36000, "Australia/Canberra"            },
	{ "est",   0,  36000, "Australia/Currie"              },
	{ "est",   0,  36000, "Australia/Hobart"              },
	{ "est",   0,  36000, "Australia/LHI"                 },
	{ "est",   0,  36000, "Australia/Lindeman"            },
	{ "est",   0,  36000, "Australia/Lord_Howe"           },
	{ "est",   0,  36000, "Australia/Melbourne"           },
	{ "est",   0,  36000, "Australia/NSW"                 },
	{ "est",   0,  36000, "Australia/Queensland"          },
	{ "est",   0,  36000, "Australia/Sydney"              },
	{ "est",   0,  36000, "Australia/Tasmania"            },
	{ "est",   0,  36000, "Australia/Victoria"            },
	{ "est",   1,  39600, "Australia/Melbourne"           },
	{ "est",   1,  39600, "Antarctica/Macquarie"          },
	{ "est",   1,  39600, "Australia/ACT"                 },
	{ "est",   1,  39600, "Australia/Brisbane"            },
	{ "est",   1,  39600, "Australia/Canberra"            },
	{ "est",   1,  39600, "Australia/Currie"              },
	{ "est",   1,  39600, "Australia/Hobart"              },
	{ "est",   1,  39600, "Australia/Lindeman"            },
	{ "est",   1,  39600, "Australia/NSW"                 },
	{ "est",   1,  39600, "Australia/Queensland"          },
	{ "est",   1,  39600, "Australia/Sydney"              },
	{ "est",   1,  39600, "Australia/Tasmania"            },
	{ "est",   1,  39600, "Australia/Victoria"            },
	{ "ewt",   1, -14400, "America/New_York"              },
	{ "ewt",   1, -14400, "America/Detroit"               },
	{ "ewt",   1, -14400, "America/Iqaluit"               },
	{ "ewt",   1, -14400, "America/Montreal"              },
	{ "ewt",   1, -14400, "America/Nipigon"               },
	{ "ewt",   1, -14400, "America/Thunder_Bay"           },
	{ "ewt",   1, -14400, "America/Toronto"               },
	{ "ewt",   1, -14400, "Canada/Eastern"                },
	{ "ewt",   1, -14400, "EST"                           },
	{ "ffmt",  0, -14660, "America/Martinique"            },
	{ "fjst",  1,  46800, "Pacific/Fiji"                  },
	{ "fjt",   0,  43200, "Pacific/Fiji"                  },
	{ "fkst",  1, -10800, "Atlantic/Stanley"              },
	{ "fkst",  1,  -7200, "Atlantic/Stanley"              },
	{ "fkt",   0, -10800, "Atlantic/Stanley"              },
	{ "fkt",   0, -14400, "Atlantic/Stanley"              },
	{ "fmt",   0,  -3180, "Africa/Freetown"               },
	{ "fmt",   0,  -4056, "Atlantic/Madeira"              },
	{ "fnst",  1,  -3600, "America/Noronha"               },
	{ "fnst",  1,  -3600, "Brazil/DeNoronha"              },
	{ "fnt",   0,  -7200, "America/Noronha"               },
	{ "fnt",   0,  -7200, "Brazil/DeNoronha"              },
	{ "fort",  0,  14400, "Asia/Aqtau"                    },
	{ "fort",  0,  18000, "Asia/Aqtau"                    },
	{ "frust", 1,  21600, "Asia/Bishkek"                  },
	{ "frust", 1,  25200, "Asia/Bishkek"                  },
	{ "frut",  0,  18000, "Asia/Bishkek"                  },
	{ "frut",  0,  21600, "Asia/Bishkek"                  },
	{ "galt",  0, -21600, "Pacific/Galapagos"             },
	{ "gamt",  0, -32400, "Pacific/Gambier"               },
	{ "gbgt",  0, -13500, "America/Guyana"                },
	{ "gest",  1,  14400, "Asia/Tbilisi"                  },
	{ "gest",  1,  18000, "Asia/Tbilisi"                  },
	{ "get",   0,  10800, "Asia/Tbilisi"                  },
	{ "get",   0,  14400, "Asia/Tbilisi"                  },
	{ "gft",   0, -10800, "America/Cayenne"               },
	{ "gft",   0, -14400, "America/Cayenne"               },
	{ "ghst",  1,   1200, "Africa/Accra"                  },
	{ "gilt",  0,  43200, "Pacific/Tarawa"                },
	{ "gmt",   0,      0, "Africa/Abidjan"                },
	{ "gmt",   0,      0, "Africa/Accra"                  },
	{ "gmt",   0,      0, "Africa/Bamako"                 },
	{ "gmt",   0,      0, "Africa/Banjul"                 },
	{ "gmt",   0,      0, "Africa/Bissau"                 },
	{ "gmt",   0,      0, "Africa/Conakry"                },
	{ "gmt",   0,      0, "Africa/Dakar"                  },
	{ "gmt",   0,      0, "Africa/Freetown"               },
	{ "gmt",   0,      0, "Africa/Lome"                   },
	{ "gmt",   0,      0, "Africa/Malabo"                 },
	{ "gmt",   0,      0, "Africa/Monrovia"               },
	{ "gmt",   0,      0, "Africa/Niamey"                 },
	{ "gmt",   0,      0, "Africa/Nouakchott"             },
	{ "gmt",   0,      0, "Africa/Ouagadougou"            },
	{ "gmt",   0,      0, "Africa/Porto-Novo"             },
	{ "gmt",   0,      0, "Africa/Sao_Tome"               },
	{ "gmt",   0,      0, "Africa/Timbuktu"               },
	{ "gmt",   0,      0, "America/Danmarkshavn"          },
	{ "gmt",   0,      0, "Atlantic/Reykjavik"            },
	{ "gmt",   0,      0, "Atlantic/St_Helena"            },
	{ "gmt",   0,      0, "Etc/GMT"                       },
	{ "gmt",   0,      0, "Etc/Greenwich"                 },
	{ "gmt",   0,      0, "Europe/Belfast"                },
	{ "gmt",   0,      0, "Europe/Dublin"                 },
	{ "gmt",   0,      0, "Europe/Gibraltar"              },
	{ "gmt",   0,      0, "Europe/Guernsey"               },
	{ "gmt",   0,      0, "Europe/Isle_of_Man"            },
	{ "gmt",   0,      0, "Europe/Jersey"                 },
	{ "gmt",   0,      0, "Europe/London"                 },
	{ "gmt",   0,      0, "GB"                            },
	{ "gst",   0,  14400, "Asia/Dubai"                    },
	{ "gst",   0,  -7200, "Atlantic/South_Georgia"        },
	{ "gst",   0,  14400, "Asia/Bahrain"                  },
	{ "gst",   0,  14400, "Asia/Muscat"                   },
	{ "gst",   0,  14400, "Asia/Qatar"                    },
	{ "gst",   0,  36000, "Pacific/Guam"                  },
	{ "gyt",   0, -10800, "America/Guyana"                },
	{ "gyt",   0, -13500, "America/Guyana"                },
	{ "gyt",   0, -14400, "America/Guyana"                },
	{ "hadt",  1, -32400, "America/Adak"                  },
	{ "hadt",  1, -32400, "America/Atka"                  },
	{ "hast",  0, -36000, "America/Adak"                  },
	{ "hast",  0, -36000, "America/Atka"                  },
	{ "hdt",   1, -34200, "Pacific/Honolulu"              },
	{ "hdt",   1, -34200, "HST"                           },
	{ "hkst",  1,  32400, "Asia/Hong_Kong"                },
	{ "hkt",   0,  28800, "Asia/Hong_Kong"                },
	{ "hmt",   0, -19776, "America/Havana"                },
	{ "hmt",   0,  -6872, "Atlantic/Azores"               },
	{ "hmt",   0,  21200, "Asia/Calcutta"                 },
	{ "hmt",   0,  21200, "Asia/Dacca"                    },
	{ "hmt",   0,  21200, "Asia/Dhaka"                    },
	{ "hmt",   0,  21200, "Asia/Kolkata"                  },
	{ "hmt",   0,   5992, "EET"                           },
	{ "hmt",   0,   5992, "Europe/Helsinki"               },
	{ "hmt",   0,   5992, "Europe/Mariehamn"              },
	{ "hovst", 1,  28800, "Asia/Hovd"                     },
	{ "hovt",  0,  21600, "Asia/Hovd"                     },
	{ "hovt",  0,  25200, "Asia/Hovd"                     },
	{ "hpt",   1, -34200, "Pacific/Honolulu"              },
	{ "hpt",   1, -34200, "HST"                           },
	{ "hst",   0, -36000, "Pacific/Honolulu"              },
	{ "hst",   0, -37800, "Pacific/Honolulu"              },
	{ "hst",   0, -36000, "HST"                           },
	{ "hst",   0, -36000, "Pacific/Johnston"              },
	{ "hst",   0, -37800, "HST"                           },
	{ "hwt",   1, -34200, "Pacific/Honolulu"              },
	{ "hwt",   1, -34200, "HST"                           },
	{ "ict",   0,  25200, "Asia/Bangkok"                  },
	{ "ict",   0,  25200, "Asia/Ho_Chi_Minh"              },
	{ "ict",   0,  25200, "Asia/Phnom_Penh"               },
	{ "ict",   0,  25200, "Asia/Saigon"                   },
	{ "ict",   0,  25200, "Asia/Vientiane"                },
	{ "ict",   0,  28800, "Asia/Ho_Chi_Minh"              },
	{ "ict",   0,  28800, "Asia/Phnom_Penh"               },
	{ "ict",   0,  28800, "Asia/Saigon"                   },
	{ "ict",   0,  28800, "Asia/Vientiane"                },
	{ "iddt",  1,  14400, "Asia/Jerusalem"                },
	{ "iddt",  1,  14400, "Asia/Tel_Aviv"                 },
	{ "idt",   1,  10800, "Asia/Jerusalem"                },
	{ "idt",   1,  10800, "Asia/Gaza"                     },
	{ "idt",   1,  10800, "Asia/Tel_Aviv"                 },
	{ "ihst",  1,  21600, "Asia/Colombo"                  },
	{ "imt",   0,  25040, "Asia/Irkutsk"                  },
	{ "imt",   0,   7016, "Asia/Istanbul"                 },
	{ "imt",   0,   7016, "Europe/Istanbul"               },
	{ "iot",   0,  18000, "Indian/Chagos"                 },
	{ "iot",   0,  21600, "Indian/Chagos"                 },
	{ "irdt",  1,  16200, "Asia/Tehran"                   },
	{ "irdt",  1,  18000, "Asia/Tehran"                   },
	{ "irkst", 1,  28800, "Asia/Irkutsk"                  },
	{ "irkst", 1,  32400, "Asia/Irkutsk"                  },
	{ "irkt",  0,  25200, "Asia/Irkutsk"                  },
	{ "irkt",  0,  28800, "Asia/Irkutsk"                  },
	{ "irst",  0,  12600, "Asia/Tehran"                   },
	{ "irst",  0,  14400, "Asia/Tehran"                   },
	{ "isst",  1,      0, "Atlantic/Reykjavik"            },
	{ "ist",   0,   7200, "Asia/Jerusalem"                },
	{ "ist",   0,  -3600, "Atlantic/Reykjavik"            },
	{ "ist",   0,  19800, "Asia/Calcutta"                 },
	{ "ist",   0,  19800, "Asia/Colombo"                  },
	{ "ist",   0,  19800, "Asia/Dacca"                    },
	{ "ist",   0,  19800, "Asia/Dhaka"                    },
	{ "ist",   0,  19800, "Asia/Karachi"                  },
	{ "ist",   0,  19800, "Asia/Kathmandu"                },
	{ "ist",   0,  19800, "Asia/Katmandu"                 },
	{ "ist",   0,  19800, "Asia/Kolkata"                  },
	{ "ist",   0,  19800, "Asia/Thimbu"                   },
	{ "ist",   0,  19800, "Asia/Thimphu"                  },
	{ "ist",   1,   2079, "Europe/Dublin"                 },
	{ "ist",   1,  23400, "Asia/Calcutta"                 },
	{ "ist",   1,  23400, "Asia/Colombo"                  },
	{ "ist",   1,  23400, "Asia/Karachi"                  },
	{ "ist",   1,  23400, "Asia/Kolkata"                  },
	{ "ist",   0,   3600, "Europe/Dublin"                 },
	{ "ist",   1,   3600, "Europe/Dublin"                 },
	{ "ist",   0,   7200, "Asia/Gaza"                     },
	{ "ist",   0,   7200, "Asia/Tel_Aviv"                 },
	{ "javt",  0,  26400, "Asia/Jakarta"                  },
	{ "jdt",   1,  36000, "Asia/Tokyo"                    },
	{ "jmt",   0,  -1368, "Atlantic/St_Helena"            },
	{ "jmt",   0,  25632, "Asia/Jakarta"                  },
	{ "jmt",   0,   8440, "Asia/Jerusalem"                },
	{ "jmt",   0,   8440, "Asia/Tel_Aviv"                 },
	{ "jst",   0,  32400, "Asia/Tokyo"                    },
	{ "jst",   0,  32400, "Asia/Dili"                     },
	{ "jst",   0,  32400, "Asia/Hong_Kong"                },
	{ "jst",   0,  32400, "Asia/Jakarta"                  },
	{ "jst",   0,  32400, "Asia/Kuala_Lumpur"             },
	{ "jst",   0,  32400, "Asia/Kuching"                  },
	{ "jst",   0,  32400, "Asia/Makassar"                 },
	{ "jst",   0,  32400, "Asia/Manila"                   },
	{ "jst",   0,  32400, "Asia/Pontianak"                },
	{ "jst",   0,  32400, "Asia/Rangoon"                  },
	{ "jst",   0,  32400, "Asia/Sakhalin"                 },
	{ "jst",   0,  32400, "Asia/Singapore"                },
	{ "jst",   0,  32400, "Asia/Ujung_Pandang"            },
	{ "jst",   0,  32400, "Pacific/Nauru"                 },
	{ "kart",  0,  18000, "Asia/Karachi"                  },
	{ "kast",  0,  18000, "Asia/Kashgar"                  },
	{ "kast",  0,  19800, "Asia/Kashgar"                  },
	{ "kdt",   1,  32400, "Asia/Seoul"                    },
	{ "kdt",   1,  36000, "Asia/Seoul"                    },
	{ "kdt",   1,  32400, "ROK"                           },
	{ "kdt",   1,  36000, "ROK"                           },
	{ "kgst",  1,  21600, "Asia/Bishkek"                  },
	{ "kgt",   0,  18000, "Asia/Bishkek"                  },
	{ "kgt",   0,  21600, "Asia/Bishkek"                  },
	{ "kizst", 1,  21600, "Asia/Qyzylorda"                },
	{ "kizt",  0,  14400, "Asia/Qyzylorda"                },
	{ "kizt",  0,  18000, "Asia/Qyzylorda"                },
	{ "kizt",  0,  21600, "Asia/Qyzylorda"                },
	{ "kmt",   0,   5736, "Europe/Vilnius"                },
	{ "kmt",   0, -14696, "America/St_Vincent"            },
	{ "kmt",   0, -18432, "America/Cayman"                },
	{ "kmt",   0, -18432, "America/Grand_Turk"            },
	{ "kmt",   0, -18432, "America/Jamaica"               },
	{ "kmt",   0,   7324, "Europe/Kiev"                   },
	{ "kost",  0,  39600, "Pacific/Kosrae"                },
	{ "kost",  0,  43200, "Pacific/Kosrae"                },
	{ "krast", 1,  25200, "Asia/Krasnoyarsk"              },
	{ "krast", 1,  28800, "Asia/Krasnoyarsk"              },
	{ "krast", 1,  25200, "Asia/Novokuznetsk"             },
	{ "krast", 1,  28800, "Asia/Novokuznetsk"             },
	{ "krat",  0,  21600, "Asia/Krasnoyarsk"              },
	{ "krat",  0,  25200, "Asia/Krasnoyarsk"              },
	{ "krat",  0,  21600, "Asia/Novokuznetsk"             },
	{ "krat",  0,  25200, "Asia/Novokuznetsk"             },
	{ "kst",   0,  28800, "Asia/Seoul"                    },
	{ "kst",   0,  30600, "Asia/Seoul"                    },
	{ "kst",   0,  32400, "Asia/Seoul"                    },
	{ "kst",   0,  28800, "Asia/Pyongyang"                },
	{ "kst",   0,  28800, "ROK"                           },
	{ "kst",   0,  30600, "Asia/Pyongyang"                },
	{ "kst",   0,  30600, "ROK"                           },
	{ "kst",   0,  32400, "Asia/Pyongyang"                },
	{ "kst",   0,  32400, "ROK"                           },
	{ "kuyst", 1,  10800, "Europe/Samara"                 },
	{ "kuyst", 1,  14400, "Europe/Samara"                 },
	{ "kuyst", 1,  18000, "Europe/Samara"                 },
	{ "kuyt",  0,  10800, "Europe/Samara"                 },
	{ "kuyt",  0,  14400, "Europe/Samara"                 },
	{ "kwat",  0, -43200, "Pacific/Kwajalein"             },
	{ "lhst",  0,  37800, "Australia/Lord_Howe"           },
	{ "lhst",  1,  39600, "Australia/Lord_Howe"           },
	{ "lhst",  1,  41400, "Australia/Lord_Howe"           },
	{ "lhst",  0,  37800, "Australia/LHI"                 },
	{ "lhst",  1,  39600, "Australia/LHI"                 },
	{ "lhst",  1,  41400, "Australia/LHI"                 },
	{ "lint",  0, -36000, "Pacific/Kiritimati"            },
	{ "lint",  0, -38400, "Pacific/Kiritimati"            },
	{ "lint",  0,  50400, "Pacific/Kiritimati"            },
	{ "lkt",   0,  21600, "Asia/Colombo"                  },
	{ "lkt",   0,  23400, "Asia/Colombo"                  },
	{ "lmt",   0, -11188, "America/Sao_Paulo"             },
	{ "lmt",   0, -11188, "Brazil/East"                   },
	{ "lmt",   0, -11568, "America/Araguaina"             },
	{ "lmt",   0, -11636, "America/Belem"                 },
	{ "lmt",   0, -12416, "America/Godthab"               },
	{ "lmt",   0, -12560, "America/Cayenne"               },
	{ "lmt",   0, -13108, "America/Campo_Grande"          },
	{ "lmt",   0, -13128, "America/Santarem"              },
	{ "lmt",   0, -13240, "America/Paramaribo"            },
	{ "lmt",   0, -13460, "America/Cuiaba"                },
	{ "lmt",   0, -13480, "America/Miquelon"              },
	{ "lmt",   0, -13960, "America/Guyana"                },
	{ "lmt",   0, -14308, "America/Barbados"              },
	{ "lmt",   0, -14388, "America/Glace_Bay"             },
	{ "lmt",   0, -14404, "America/Manaus"                },
	{ "lmt",   0, -14404, "Brazil/West"                   },
	{ "lmt",   0, -14560, "America/Boa_Vista"             },
	{ "lmt",   0, -14736, "America/Dominica"              },
	{ "lmt",   0, -14764, "America/Port_of_Spain"         },
	{ "lmt",   0, -14768, "America/Guadeloupe"            },
	{ "lmt",   0, -14768, "America/Marigot"               },
	{ "lmt",   0, -14768, "America/St_Barthelemy"         },
	{ "lmt",   0, -14820, "America/Grenada"               },
	{ "lmt",   0, -14832, "America/Antigua"               },
	{ "lmt",   0, -14932, "America/Montserrat"            },
	{ "lmt",   0, -15052, "America/St_Kitts"              },
	{ "lmt",   0, -15136, "America/Anguilla"              },
	{ "lmt",   0, -15264, "America/Halifax"               },
	{ "lmt",   0, -15264, "Canada/Atlantic"               },
	{ "lmt",   0, -15336, "America/Porto_Velho"           },
	{ "lmt",   0, -15508, "America/Tortola"               },
	{ "lmt",   0, -15544, "Atlantic/Bermuda"              },
	{ "lmt",   0, -15584, "America/St_Thomas"             },
	{ "lmt",   0, -15584, "America/Virgin"                },
	{ "lmt",   0,  -1624, "Atlantic/Faeroe"               },
	{ "lmt",   0,  -1624, "Atlantic/Faroe"                },
	{ "lmt",   0, -16272, "America/Porto_Acre"            },
	{ "lmt",   0, -16272, "America/Rio_Branco"            },
	{ "lmt",   0, -16272, "Brazil/Acre"                   },
	{ "lmt",   0, -16508, "America/Thule"                 },
	{ "lmt",   0, -16544, "America/Curacao"               },
	{ "lmt",   0, -16768, "America/Eirunepe"              },
	{ "lmt",   0, -16824, "America/Aruba"                 },
	{ "lmt",   0,  -1820, "Africa/Casablanca"             },
	{ "lmt",   0, -18516, "America/Lima"                  },
	{ "lmt",   0, -18564, "America/Nassau"                },
	{ "lmt",   0,  -1920, "Africa/Bamako"                 },
	{ "lmt",   0,  -1920, "Africa/Timbuktu"               },
	{ "lmt",   0, -19931, "America/Detroit"               },
	{ "lmt",   0, -20824, "America/Cancun"                },
	{ "lmt",   0, -20932, "America/Tegucigalpa"           },
	{ "lmt",   0, -21168, "America/Belize"                },
	{ "lmt",   0, -21408, "America/El_Salvador"           },
	{ "lmt",   0, -21504, "Pacific/Galapagos"             },
	{ "lmt",   0, -21508, "America/Merida"                },
	{ "lmt",   0, -21724, "America/Guatemala"             },
	{ "lmt",   0,  -2192, "Africa/Sao_Tome"               },
	{ "lmt",   0,  -2192, "Europe/Lisbon"                 },
	{ "lmt",   0, -23796, "America/Mexico_City"           },
	{ "lmt",   0, -23796, "Mexico/General"                },
	{ "lmt",   0, -24000, "America/Matamoros"             },
	{ "lmt",   0, -24076, "America/Monterrey"             },
	{ "lmt",   0, -25060, "America/Ojinaga"               },
	{ "lmt",   0, -25116, "America/Regina"                },
	{ "lmt",   0, -25116, "Canada/East-Saskatchewan"      },
	{ "lmt",   0, -25116, "Canada/Saskatchewan"           },
	{ "lmt",   0, -25260, "America/Bahia_Banderas"        },
	{ "lmt",   0, -25460, "America/Chihuahua"             },
	{ "lmt",   0, -25540, "America/Mazatlan"              },
	{ "lmt",   0, -25540, "Mexico/BajaSur"                },
	{ "lmt",   0, -25880, "America/Swift_Current"         },
	{ "lmt",   0, -26632, "America/Hermosillo"            },
	{ "lmt",   0, -27232, "America/Edmonton"              },
	{ "lmt",   0, -27232, "Canada/Mountain"               },
	{ "lmt",   0, -27568, "America/Santa_Isabel"          },
	{ "lmt",   0, -28084, "America/Ensenada"              },
	{ "lmt",   0, -28084, "America/Tijuana"               },
	{ "lmt",   0, -28084, "Mexico/BajaNorte"              },
	{ "lmt",   0,  -3168, "Africa/El_Aaiun"               },
	{ "lmt",   0, -32388, "Pacific/Gambier"               },
	{ "lmt",   0,  -3292, "Africa/Conakry"                },
	{ "lmt",   0, -33480, "Pacific/Marquesas"             },
	{ "lmt",   0, -35896, "Pacific/Tahiti"                },
	{ "lmt",   0,   -364, "Africa/Ouagadougou"            },
	{ "lmt",   0,  -3696, "Atlantic/Canary"               },
	{ "lmt",   0,  -3740, "Africa/Bissau"                 },
	{ "lmt",   0,  -3828, "Africa/Nouakchott"             },
	{ "lmt",   0,  -3996, "Africa/Banjul"                 },
	{ "lmt",   0, -40968, "Pacific/Pago_Pago"             },
	{ "lmt",   0, -40968, "Pacific/Samoa"                 },
	{ "lmt",   0, -41216, "Pacific/Apia"                  },
	{ "lmt",   0,  -4184, "Africa/Dakar"                  },
	{ "lmt",   0,  -4480, "America/Danmarkshavn"          },
	{ "lmt",   0,  -5272, "America/Scoresbysund"          },
	{ "lmt",   0,    -52, "Africa/Accra"                  },
	{ "lmt",   0,  -5644, "Atlantic/Cape_Verde"           },
	{ "lmt",   0,  -7780, "America/Noronha"               },
	{ "lmt",   0,  -7780, "Brazil/DeNoronha"              },
	{ "lmt",   0,  -8376, "America/Recife"                },
	{ "lmt",   0,  -8572, "America/Maceio"                },
	{ "lmt",   0,  -9240, "America/Fortaleza"             },
	{ "lmt",   0,  -9244, "America/Bahia"                 },
	{ "lmt",   0,   -968, "Africa/Abidjan"                },
	{ "lmt",   0,  10356, "Africa/Djibouti"               },
	{ "lmt",   0,  10384, "Indian/Comoro"                 },
	{ "lmt",   0,  10660, "Europe/Volgograd"              },
	{ "lmt",   0,  10680, "Asia/Yerevan"                  },
	{ "lmt",   0,  10848, "Asia/Aden"                     },
	{ "lmt",   0,  10856, "Indian/Mayotte"                },
	{ "lmt",   0,  11212, "Asia/Riyadh"                   },
	{ "lmt",   0,  11404, "Indian/Antananarivo"           },
	{ "lmt",   0,  11516, "Asia/Kuwait"                   },
	{ "lmt",   0,  11964, "Asia/Baku"                     },
	{ "lmt",   0,  12036, "Europe/Samara"                 },
	{ "lmt",   0,  12064, "Asia/Aqtau"                    },
	{ "lmt",   0,  12140, "Asia/Bahrain"                  },
	{ "lmt",   0,  12324, "Asia/Oral"                     },
	{ "lmt",   0,  12344, "Asia/Tehran"                   },
	{ "lmt",   0,  12368, "Asia/Qatar"                    },
	{ "lmt",   0,  13272, "Asia/Dubai"                    },
	{ "lmt",   0,  13308, "Indian/Mahe"                   },
	{ "lmt",   0,  13312, "Indian/Reunion"                },
	{ "lmt",   0,  13720, "Asia/Aqtobe"                   },
	{ "lmt",   0,  13800, "Indian/Mauritius"              },
	{ "lmt",   0,  14012, "Asia/Ashgabat"                 },
	{ "lmt",   0,  14012, "Asia/Ashkhabad"                },
	{ "lmt",   0,  14060, "Asia/Muscat"                   },
	{ "lmt",   0,  14544, "Asia/Yekaterinburg"            },
	{ "lmt",   0,   1476, "Europe/Luxembourg"             },
	{ "lmt",   0,  15712, "Asia/Qyzylorda"                },
	{ "lmt",   0,  16032, "Asia/Samarkand"                },
	{ "lmt",   0,  16092, "Asia/Karachi"                  },
	{ "lmt",   0,  16512, "Asia/Dushanbe"                 },
	{ "lmt",   0,  16632, "Asia/Tashkent"                 },
	{ "lmt",   0,  17380, "Indian/Chagos"                 },
	{ "lmt",   0,  17616, "Asia/Omsk"                     },
	{ "lmt",   0,  17904, "Asia/Bishkek"                  },
	{ "lmt",   0,  18236, "Asia/Kashgar"                  },
	{ "lmt",   0,  18468, "Asia/Almaty"                   },
	{ "lmt",   0,  19900, "Asia/Novosibirsk"              },
	{ "lmt",   0,  20476, "Asia/Kathmandu"                },
	{ "lmt",   0,  20476, "Asia/Katmandu"                 },
	{ "lmt",   0,  21020, "Asia/Urumqi"                   },
	{ "lmt",   0,   2108, "Africa/Malabo"                 },
	{ "lmt",   0,  21516, "Asia/Thimbu"                   },
	{ "lmt",   0,  21516, "Asia/Thimphu"                  },
	{ "lmt",   0,  21996, "Asia/Hovd"                     },
	{ "lmt",   0,  22280, "Asia/Krasnoyarsk"              },
	{ "lmt",   0,   2268, "Africa/Libreville"             },
	{ "lmt",   0,   2328, "Africa/Douala"                 },
	{ "lmt",   0,  24624, "Asia/Vientiane"                },
	{ "lmt",   0,  25180, "Asia/Phnom_Penh"               },
	{ "lmt",   0,  25580, "Asia/Chongqing"                },
	{ "lmt",   0,  25580, "Asia/Chungking"                },
	{ "lmt",   0,  25600, "Asia/Ho_Chi_Minh"              },
	{ "lmt",   0,  25600, "Asia/Saigon"                   },
	{ "lmt",   0,  25652, "Asia/Ulaanbaatar"              },
	{ "lmt",   0,  25652, "Asia/Ulan_Bator"               },
	{ "lmt",   0,  26240, "Asia/Pontianak"                },
	{ "lmt",   0,  26480, "Asia/Kuching"                  },
	{ "lmt",   0,  27260, "Asia/Macao"                    },
	{ "lmt",   0,  27260, "Asia/Macau"                    },
	{ "lmt",   0,  27396, "Asia/Hong_Kong"                },
	{ "lmt",   0,  27480, "Asia/Choibalsan"               },
	{ "lmt",   0,  27580, "Asia/Brunei"                   },
	{ "lmt",   0,  28656, "Asia/Makassar"                 },
	{ "lmt",   0,  28656, "Asia/Ujung_Pandang"            },
	{ "lmt",   0,  29152, "Asia/Shanghai"                 },
	{ "lmt",   0,  29152, "PRC"                           },
	{ "lmt",   0,  30140, "Asia/Dili"                     },
	{ "lmt",   0,  30404, "Asia/Harbin"                   },
	{ "lmt",   0,  31120, "Asia/Yakutsk"                  },
	{ "lmt",   0,   3164, "Africa/Tripoli"                },
	{ "lmt",   0,  31664, "Asia/Vladivostok"              },
	{ "lmt",   0,  33768, "Asia/Jayapura"                 },
	{ "lmt",   0,  34248, "Asia/Sakhalin"                 },
	{ "lmt",   0,   3612, "Africa/Ndjamena"               },
	{ "lmt",   0,  36192, "Asia/Magadan"                  },
	{ "lmt",   0,   3668, "Africa/Brazzaville"            },
	{ "lmt",   0,  38076, "Asia/Kamchatka"                },
	{ "lmt",   0,  38388, "Pacific/Guadalcanal"           },
	{ "lmt",   0,  39948, "Pacific/Noumea"                },
	{ "lmt",   0,  40060, "Pacific/Nauru"                 },
	{ "lmt",   0,  40396, "Pacific/Efate"                 },
	{ "lmt",   0,  42596, "Asia/Anadyr"                   },
	{ "lmt",   0,  42820, "Pacific/Fiji"                  },
	{ "lmt",   0,  44028, "Pacific/Chatham"               },
	{ "lmt",   0,   4460, "Africa/Bangui"                 },
	{ "lmt",   0,   4760, "Europe/Tirane"                 },
	{ "lmt",   0,    508, "Africa/Niamey"                 },
	{ "lmt",   0,    628, "Africa/Porto-Novo"             },
	{ "lmt",   0,   6600, "Africa/Maseru"                 },
	{ "lmt",   0,   6788, "Africa/Lusaka"                 },
	{ "lmt",   0,   7216, "Africa/Kigali"                 },
	{ "lmt",   0,   7452, "Africa/Harare"                 },
	{ "lmt",   0,   7464, "Africa/Mbabane"                },
	{ "lmt",   0,   7780, "Africa/Kampala"                },
	{ "lmt",   0,   7808, "Africa/Khartoum"               },
	{ "lmt",   0,   7820, "Africa/Maputo"                 },
	{ "lmt",   0,   8008, "Asia/Nicosia"                  },
	{ "lmt",   0,   8008, "Europe/Nicosia"                },
	{ "lmt",   0,    816, "Africa/Lagos"                  },
	{ "lmt",   0,   8400, "Africa/Blantyre"               },
	{ "lmt",   0,   8624, "Asia/Amman"                    },
	{ "lmt",   0,   8712, "Asia/Damascus"                 },
	{ "lmt",   0,   8836, "Africa/Nairobi"                },
	{ "lmt",   0,   9428, "Africa/Dar_es_Salaam"          },
	{ "lont",  0,  25200, "Asia/Chongqing"                },
	{ "lont",  0,  25200, "Asia/Chungking"                },
	{ "lrt",   0,  -2670, "Africa/Monrovia"               },
	{ "lst",   1,   9384, "Europe/Riga"                   },
	{ "madmt", 1,   3600, "Atlantic/Madeira"              },
	{ "madst", 1,      0, "Atlantic/Madeira"              },
	{ "madt",  0,  -3600, "Atlantic/Madeira"              },
	{ "magst", 1,  39600, "Asia/Magadan"                  },
	{ "magst", 1,  43200, "Asia/Magadan"                  },
	{ "magt",  0,  36000, "Asia/Magadan"                  },
	{ "magt",  0,  39600, "Asia/Magadan"                  },
	{ "malst", 1,  26400, "Asia/Singapore"                },
	{ "malst", 1,  26400, "Asia/Kuala_Lumpur"             },
	{ "malt",  0,  25200, "Asia/Singapore"                },
	{ "malt",  0,  26400, "Asia/Singapore"                },
	{ "malt",  0,  27000, "Asia/Singapore"                },
	{ "malt",  0,  25200, "Asia/Kuala_Lumpur"             },
	{ "malt",  0,  26400, "Asia/Kuala_Lumpur"             },
	{ "malt",  0,  27000, "Asia/Kuala_Lumpur"             },
	{ "mart",  0, -34200, "Pacific/Marquesas"             },
	{ "mawt",  0,  18000, "Antarctica/Mawson"             },
	{ "mawt",  0,  21600, "Antarctica/Mawson"             },
	{ "mddt",  1, -18000, "America/Cambridge_Bay"         },
	{ "mddt",  1, -18000, "America/Yellowknife"           },
	{ "mdst",  1,  16248, "Europe/Moscow"                 },
	{ "mdt",   1, -21600, "America/Denver"                },
	{ "mdt",   1, -21600, "America/Bahia_Banderas"        },
	{ "mdt",   1, -21600, "America/Boise"                 },
	{ "mdt",   1, -21600, "America/Cambridge_Bay"         },
	{ "mdt",   1, -21600, "America/Chihuahua"             },
	{ "mdt",   1, -21600, "America/Edmonton"              },
	{ "mdt",   1, -21600, "America/Hermosillo"            },
	{ "mdt",   1, -21600, "America/Inuvik"                },
	{ "mdt",   1, -21600, "America/Mazatlan"              },
	{ "mdt",   1, -21600, "America/North_Dakota/Center"   },
	{ "mdt",   1, -21600, "America/North_Dakota/New_Salem" },
	{ "mdt",   1, -21600, "America/Ojinaga"               },
	{ "mdt",   1, -21600, "America/Phoenix"               },
	{ "mdt",   1, -21600, "America/Regina"                },
	{ "mdt",   1, -21600, "America/Shiprock"              },
	{ "mdt",   1, -21600, "America/Swift_Current"         },
	{ "mdt",   1, -21600, "America/Yellowknife"           },
	{ "mdt",   1, -21600, "Canada/East-Saskatchewan"      },
	{ "mdt",   1, -21600, "Canada/Mountain"               },
	{ "mdt",   1, -21600, "Canada/Saskatchewan"           },
	{ "mdt",   1, -21600, "Mexico/BajaSur"                },
	{ "mdt",   1, -21600, "MST"                           },
	{ "mest",  1,   7200, "MET"                           },
	{ "met",   0,   3600, "MET"                           },
	{ "mht",   0,  39600, "Pacific/Kwajalein"             },
	{ "mht",   0,  43200, "Pacific/Kwajalein"             },
	{ "mht",   0,  39600, "Pacific/Majuro"                },
	{ "mht",   0,  43200, "Pacific/Majuro"                },
	{ "mist",  0,  39600, "Antarctica/Macquarie"          },
	{ "mmt",   0,   9000, "Europe/Moscow"                 },
	{ "mmt",   0,   9048, "Europe/Moscow"                 },
	{ "mmt",   0, -13484, "America/Montevideo"            },
	{ "mmt",   0, -20712, "America/Managua"               },
	{ "mmt",   0,  -2588, "Africa/Monrovia"               },
	{ "mmt",   0,  17640, "Indian/Maldives"               },
	{ "mmt",   0,  19172, "Asia/Colombo"                  },
	{ "mmt",   0,  23400, "Asia/Rangoon"                  },
	{ "mmt",   0,  28656, "Asia/Makassar"                 },
	{ "mmt",   0,  28656, "Asia/Ujung_Pandang"            },
	{ "mmt",   0,   6600, "Europe/Minsk"                  },
	{ "most",  1,  32400, "Asia/Macao"                    },
	{ "most",  1,  32400, "Asia/Macau"                    },
	{ "mot",   0,  28800, "Asia/Macao"                    },
	{ "mot",   0,  28800, "Asia/Macau"                    },
	{ "mpt",   1, -21600, "America/Denver"                },
	{ "mpt",   1, -21600, "America/Boise"                 },
	{ "mpt",   1, -21600, "America/Cambridge_Bay"         },
	{ "mpt",   1, -21600, "America/Edmonton"              },
	{ "mpt",   1, -21600, "America/North_Dakota/Center"   },
	{ "mpt",   1, -21600, "America/North_Dakota/New_Salem" },
	{ "mpt",   1, -21600, "America/Regina"                },
	{ "mpt",   1, -21600, "America/Shiprock"              },
	{ "mpt",   1, -21600, "America/Swift_Current"         },
	{ "mpt",   1, -21600, "America/Yellowknife"           },
	{ "mpt",   1, -21600, "Canada/East-Saskatchewan"      },
	{ "mpt",   1, -21600, "Canada/Mountain"               },
	{ "mpt",   1, -21600, "Canada/Saskatchewan"           },
	{ "mpt",   1, -21600, "MST"                           },
	{ "mpt",   0,  32400, "Pacific/Saipan"                },
	{ "mpt",   0,  36000, "Pacific/Saipan"                },
	{ "msd",   1,  14400, "Europe/Moscow"                 },
	{ "msd",   1,  18000, "Europe/Moscow"                 },
	{ "msd",   1,  14400, "Europe/Chisinau"               },
	{ "msd",   1,  14400, "Europe/Kaliningrad"            },
	{ "msd",   1,  14400, "Europe/Kiev"                   },
	{ "msd",   1,  14400, "Europe/Minsk"                  },
	{ "msd",   1,  14400, "Europe/Riga"                   },
	{ "msd",   1,  14400, "Europe/Simferopol"             },
	{ "msd",   1,  14400, "Europe/Tallinn"                },
	{ "msd",   1,  14400, "Europe/Tiraspol"               },
	{ "msd",   1,  14400, "Europe/Uzhgorod"               },
	{ "msd",   1,  14400, "Europe/Vilnius"                },
	{ "msd",   1,  14400, "Europe/Zaporozhye"             },
	{ "msk",   0,  10800, "Europe/Moscow"                 },
	{ "msk",   0,  10800, "Europe/Chisinau"               },
	{ "msk",   0,  10800, "Europe/Kaliningrad"            },
	{ "msk",   0,  10800, "Europe/Kiev"                   },
	{ "msk",   0,  10800, "Europe/Minsk"                  },
	{ "msk",   0,  10800, "Europe/Riga"                   },
	{ "msk",   0,  10800, "Europe/Simferopol"             },
	{ "msk",   0,  10800, "Europe/Tallinn"                },
	{ "msk",   0,  10800, "Europe/Tiraspol"               },
	{ "msk",   0,  10800, "Europe/Uzhgorod"               },
	{ "msk",   0,  10800, "Europe/Vilnius"                },
	{ "msk",   0,  10800, "Europe/Zaporozhye"             },
	{ "mst",   0, -25200, "America/Denver"                },
	{ "mst",   0, -25200, "America/Bahia_Banderas"        },
	{ "mst",   0, -25200, "America/Boise"                 },
	{ "mst",   0, -25200, "America/Cambridge_Bay"         },
	{ "mst",   0, -25200, "America/Chihuahua"             },
	{ "mst",   0, -25200, "America/Dawson_Creek"          },
	{ "mst",   0, -25200, "America/Edmonton"              },
	{ "mst",   0, -25200, "America/Ensenada"              },
	{ "mst",   0, -25200, "America/Hermosillo"            },
	{ "mst",   0, -25200, "America/Inuvik"                },
	{ "mst",   0, -25200, "America/Mazatlan"              },
	{ "mst",   0, -25200, "America/Mexico_City"           },
	{ "mst",   0, -25200, "America/North_Dakota/Center"   },
	{ "mst",   0, -25200, "America/North_Dakota/New_Salem" },
	{ "mst",   0, -25200, "America/Ojinaga"               },
	{ "mst",   0, -25200, "America/Phoenix"               },
	{ "mst",   0, -25200, "America/Regina"                },
	{ "mst",   0, -25200, "America/Santa_Isabel"          },
	{ "mst",   0, -25200, "America/Shiprock"              },
	{ "mst",   0, -25200, "America/Swift_Current"         },
	{ "mst",   0, -25200, "America/Tijuana"               },
	{ "mst",   0, -25200, "America/Yellowknife"           },
	{ "mst",   0, -25200, "Canada/East-Saskatchewan"      },
	{ "mst",   0, -25200, "Canada/Mountain"               },
	{ "mst",   0, -25200, "Canada/Saskatchewan"           },
	{ "mst",   0, -25200, "Mexico/BajaNorte"              },
	{ "mst",   0, -25200, "Mexico/BajaSur"                },
	{ "mst",   0, -25200, "Mexico/General"                },
	{ "mst",   0, -25200, "MST"                           },
	{ "mst",   1,  12648, "Europe/Moscow"                 },
	{ "must",  1,  18000, "Indian/Mauritius"              },
	{ "mut",   0,  14400, "Indian/Mauritius"              },
	{ "mvt",   0,  18000, "Indian/Maldives"               },
	{ "mwt",   1, -21600, "America/Denver"                },
	{ "mwt",   1, -21600, "America/Boise"                 },
	{ "mwt",   1, -21600, "America/Cambridge_Bay"         },
	{ "mwt",   1, -21600, "America/Edmonton"              },
	{ "mwt",   1, -21600, "America/North_Dakota/Center"   },
	{ "mwt",   1, -21600, "America/North_Dakota/New_Salem" },
	{ "mwt",   1, -21600, "America/Phoenix"               },
	{ "mwt",   1, -21600, "America/Regina"                },
	{ "mwt",   1, -21600, "America/Shiprock"              },
	{ "mwt",   1, -21600, "America/Swift_Current"         },
	{ "mwt",   1, -21600, "America/Yellowknife"           },
	{ "mwt",   1, -21600, "Canada/East-Saskatchewan"      },
	{ "mwt",   1, -21600, "Canada/Mountain"               },
	{ "mwt",   1, -21600, "Canada/Saskatchewan"           },
	{ "mwt",   1, -21600, "MST"                           },
	{ "myt",   0,  28800, "Asia/Kuala_Lumpur"             },
	{ "myt",   0,  28800, "Asia/Kuching"                  },
	{ "ncst",  1,  43200, "Pacific/Noumea"                },
	{ "nct",   0,  39600, "Pacific/Noumea"                },
	{ "nddt",  1,  -5400, "America/St_Johns"              },
	{ "nddt",  1,  -5400, "Canada/Newfoundland"           },
	{ "ndt",   1,  -9000, "America/St_Johns"              },
	{ "ndt",   1,  -9052, "America/St_Johns"              },
	{ "ndt",   1, -36000, "Pacific/Midway"                },
	{ "ndt",   1,  -9000, "America/Goose_Bay"             },
	{ "ndt",   1,  -9000, "Canada/Newfoundland"           },
	{ "ndt",   1,  -9052, "America/Goose_Bay"             },
	{ "ndt",   1,  -9052, "Canada/Newfoundland"           },
	{ "negt",  0, -12600, "America/Paramaribo"            },
	{ "nest",  1,   4800, "Europe/Amsterdam"              },
	{ "net",   0,   1200, "Europe/Amsterdam"              },
	{ "nft",   0,  41400, "Pacific/Norfolk"               },
	{ "nmt",   0,  20928, "Asia/Novokuznetsk"             },
	{ "nmt",   0,  40320, "Pacific/Norfolk"               },
	{ "novst", 1,  25200, "Asia/Novosibirsk"              },
	{ "novst", 1,  28800, "Asia/Novosibirsk"              },
	{ "novst", 1,  25200, "Asia/Novokuznetsk"             },
	{ "novt",  0,  21600, "Asia/Novosibirsk"              },
	{ "novt",  0,  25200, "Asia/Novosibirsk"              },
	{ "novt",  0,  21600, "Asia/Novokuznetsk"             },
	{ "npt",   1,  -9000, "America/St_Johns"              },
	{ "npt",   1, -36000, "America/Adak"                  },
	{ "npt",   1, -36000, "America/Atka"                  },
	{ "npt",   1, -36000, "America/Nome"                  },
	{ "npt",   1,  -9000, "America/Goose_Bay"             },
	{ "npt",   1,  -9000, "Canada/Newfoundland"           },
	{ "npt",   0,  20700, "Asia/Katmandu"                 },
	{ "npt",   0,  20700, "Asia/Kathmandu"                },
	{ "nrt",   0,  41400, "Pacific/Nauru"                 },
	{ "nrt",   0,  43200, "Pacific/Nauru"                 },
	{ "nst",   0, -12600, "America/St_Johns"              },
	{ "nst",   0, -12652, "America/St_Johns"              },
	{ "nst",   0, -12600, "America/Goose_Bay"             },
	{ "nst",   0, -12600, "Canada/Newfoundland"           },
	{ "nst",   0, -12652, "America/Goose_Bay"             },
	{ "nst",   0, -12652, "Canada/Newfoundland"           },
	{ "nst",   0, -39600, "America/Adak"                  },
	{ "nst",   0, -39600, "America/Atka"                  },
	{ "nst",   0, -39600, "America/Nome"                  },
	{ "nst",   0, -39600, "Pacific/Midway"                },
	{ "nst",   0, -39600, "Pacific/Pago_Pago"             },
	{ "nst",   0, -39600, "Pacific/Samoa"                 },
	{ "nst",   1,   4772, "Europe/Amsterdam"              },
	{ "nut",   0, -39600, "Pacific/Niue"                  },
	{ "nut",   0, -40800, "Pacific/Niue"                  },
	{ "nut",   0, -41400, "Pacific/Niue"                  },
	{ "nwt",   1,  -9000, "America/St_Johns"              },
	{ "nwt",   1, -36000, "America/Adak"                  },
	{ "nwt",   1, -36000, "America/Atka"                  },
	{ "nwt",   1, -36000, "America/Nome"                  },
	{ "nwt",   1,  -9000, "America/Goose_Bay"             },
	{ "nwt",   1,  -9000, "Canada/Newfoundland"           },
	{ "nzdt",  1,  46800, "Pacific/Auckland"              },
	{ "nzdt",  1,  46800, "Antarctica/McMurdo"            },
	{ "nzdt",  1,  46800, "Antarctica/South_Pole"         },
	{ "nzdt",  1,  46800, "NZ"                            },
	{ "nzmt",  0,  41400, "Pacific/Auckland"              },
	{ "nzmt",  0,  41400, "NZ"                            },
	{ "nzst",  0,  43200, "Pacific/Auckland"              },
	{ "nzst",  1,  43200, "Pacific/Auckland"              },
	{ "nzst",  1,  45000, "Pacific/Auckland"              },
	{ "nzst",  0,  43200, "Antarctica/McMurdo"            },
	{ "nzst",  0,  43200, "Antarctica/South_Pole"         },
	{ "nzst",  0,  43200, "NZ"                            },
	{ "nzst",  1,  43200, "NZ"                            },
	{ "nzst",  1,  45000, "NZ"                            },
	{ "omsst", 1,  21600, "Asia/Omsk"                     },
	{ "omsst", 1,  25200, "Asia/Omsk"                     },
	{ "omst",  0,  18000, "Asia/Omsk"                     },
	{ "omst",  0,  21600, "Asia/Omsk"                     },
	{ "orast", 1,  18000, "Asia/Oral"                     },
	{ "orat",  0,  14400, "Asia/Oral"                     },
	{ "orat",  0,  18000, "Asia/Oral"                     },
	{ "pddt",  1, -21600, "America/Inuvik"                },
	{ "pdt",   1, -25200, "America/Los_Angeles"           },
	{ "pdt",   1, -25200, "America/Boise"                 },
	{ "pdt",   1, -25200, "America/Dawson"                },
	{ "pdt",   1, -25200, "America/Dawson_Creek"          },
	{ "pdt",   1, -25200, "America/Ensenada"              },
	{ "pdt",   1, -25200, "America/Juneau"                },
	{ "pdt",   1, -25200, "America/Santa_Isabel"          },
	{ "pdt",   1, -25200, "America/Tijuana"               },
	{ "pdt",   1, -25200, "America/Vancouver"             },
	{ "pdt",   1, -25200, "America/Whitehorse"            },
	{ "pdt",   1, -25200, "Canada/Pacific"                },
	{ "pdt",   1, -25200, "Canada/Yukon"                  },
	{ "pdt",   1, -25200, "Mexico/BajaNorte"              },
	{ "pest",  1, -14400, "America/Lima"                  },
	{ "petst", 1,  43200, "Asia/Kamchatka"                },
	{ "petst", 1,  46800, "Asia/Kamchatka"                },
	{ "pett",  0,  39600, "Asia/Kamchatka"                },
	{ "pett",  0,  43200, "Asia/Kamchatka"                },
	{ "pet",   0, -18000, "America/Lima"                  },
	{ "pgt",   0,  36000, "Pacific/Port_Moresby"          },
	{ "phot",  0, -39600, "Pacific/Enderbury"             },
	{ "phot",  0, -43200, "Pacific/Enderbury"             },
	{ "phot",  0,  46800, "Pacific/Enderbury"             },
	{ "phst",  1,  32400, "Asia/Manila"                   },
	{ "pht",   0,  28800, "Asia/Manila"                   },
	{ "pkst",  1,  21600, "Asia/Karachi"                  },
	{ "pkt",   0,  18000, "Asia/Karachi"                  },
	{ "pmdt",  1,  -7200, "America/Miquelon"              },
	{ "pmst",  0, -10800, "America/Miquelon"              },
	{ "pmt",   0, -13236, "America/Paramaribo"            },
	{ "pmt",   0, -13252, "America/Paramaribo"            },
	{ "pmt",   0,  26240, "Asia/Pontianak"                },
	{ "pmt",   0,  36000, "Antarctica/DumontDUrville"     },
	{ "pmt",   0,    561, "Africa/Algiers"                },
	{ "pmt",   0,    561, "Africa/Tunis"                  },
	{ "pmt",   0,    561, "Europe/Monaco"                 },
	{ "pmt",   0,    561, "Europe/Paris"                  },
	{ "pmt",   0,    561, "WET"                           },
	{ "pnt",   0, -30600, "Pacific/Pitcairn"              },
	{ "pont",  0,  39600, "Pacific/Pohnpei"               },
	{ "pont",  0,  39600, "Pacific/Ponape"                },
	{ "ppmt",  0, -17340, "America/Port-au-Prince"        },
	{ "ppt",   1, -25200, "America/Los_Angeles"           },
	{ "ppt",   1, -25200, "America/Dawson_Creek"          },
	{ "ppt",   1, -25200, "America/Ensenada"              },
	{ "ppt",   1, -25200, "America/Juneau"                },
	{ "ppt",   1, -25200, "America/Santa_Isabel"          },
	{ "ppt",   1, -25200, "America/Tijuana"               },
	{ "ppt",   1, -25200, "America/Vancouver"             },
	{ "ppt",   1, -25200, "Canada/Pacific"                },
	{ "ppt",   1, -25200, "Mexico/BajaNorte"              },
	{ "pst",   0, -28800, "America/Los_Angeles"           },
	{ "pst",   0, -28800, "America/Bahia_Banderas"        },
	{ "pst",   0, -28800, "America/Boise"                 },
	{ "pst",   0, -28800, "America/Dawson"                },
	{ "pst",   0, -28800, "America/Dawson_Creek"          },
	{ "pst",   0, -28800, "America/Ensenada"              },
	{ "pst",   0, -28800, "America/Hermosillo"            },
	{ "pst",   0, -28800, "America/Inuvik"                },
	{ "pst",   0, -28800, "America/Juneau"                },
	{ "pst",   0, -28800, "America/Mazatlan"              },
	{ "pst",   0, -28800, "America/Santa_Isabel"          },
	{ "pst",   0, -28800, "America/Tijuana"               },
	{ "pst",   0, -28800, "America/Vancouver"             },
	{ "pst",   0, -28800, "America/Whitehorse"            },
	{ "pst",   0, -28800, "Canada/Pacific"                },
	{ "pst",   0, -28800, "Canada/Yukon"                  },
	{ "pst",   0, -28800, "Mexico/BajaNorte"              },
	{ "pst",   0, -28800, "Mexico/BajaSur"                },
	{ "pst",   0, -28800, "Pacific/Pitcairn"              },
	{ "pwt",   1, -25200, "America/Los_Angeles"           },
	{ "pwt",   1, -25200, "America/Dawson_Creek"          },
	{ "pwt",   1, -25200, "America/Ensenada"              },
	{ "pwt",   1, -25200, "America/Juneau"                },
	{ "pwt",   1, -25200, "America/Santa_Isabel"          },
	{ "pwt",   1, -25200, "America/Tijuana"               },
	{ "pwt",   1, -25200, "America/Vancouver"             },
	{ "pwt",   1, -25200, "Canada/Pacific"                },
	{ "pwt",   1, -25200, "Mexico/BajaNorte"              },
	{ "pwt",   0,  32400, "Pacific/Palau"                 },
	{ "pyst",  1, -10800, "America/Asuncion"              },
	{ "pyt",   0, -10800, "America/Asuncion"              },
	{ "pyt",   0, -14400, "America/Asuncion"              },
	{ "qmt",   0, -18840, "America/Guayaquil"             },
	{ "qyzst", 1,  25200, "Asia/Qyzylorda"                },
	{ "qyzt",  0,  18000, "Asia/Qyzylorda"                },
	{ "qyzt",  0,  21600, "Asia/Qyzylorda"                },
	{ "ret",   0,  14400, "Indian/Reunion"                },
	{ "rmt",   0,   5784, "Europe/Riga"                   },
	{ "rmt",   0,  -5268, "Atlantic/Reykjavik"            },
	{ "rmt",   0,  23076, "Asia/Rangoon"                  },
	{ "rott",  0, -10800, "Antarctica/Rothera"            },
	{ "sakst", 1,  39600, "Asia/Sakhalin"                 },
	{ "sakst", 1,  43200, "Asia/Sakhalin"                 },
	{ "sakt",  0,  36000, "Asia/Sakhalin"                 },
	{ "sakt",  0,  39600, "Asia/Sakhalin"                 },
	{ "samst", 1,  21600, "Asia/Samarkand"                },
	{ "samst", 1,  14400, "Europe/Samara"                 },
	{ "samst", 1,  18000, "Europe/Samara"                 },
	{ "samt",  0,  14400, "Asia/Samarkand"                },
	{ "samt",  0,  18000, "Asia/Samarkand"                },
	{ "samt",  0, -41400, "Pacific/Apia"                  },
	{ "samt",  0, -41400, "Pacific/Pago_Pago"             },
	{ "samt",  0, -41400, "Pacific/Samoa"                 },
	{ "samt",  0,  10800, "Europe/Samara"                 },
	{ "samt",  0,  14400, "Europe/Samara"                 },
	{ "sast",  1,  10800, "Africa/Johannesburg"           },
	{ "sast",  0,   5400, "Africa/Johannesburg"           },
	{ "sast",  0,   7200, "Africa/Johannesburg"           },
	{ "sast",  1,  10800, "Africa/Maseru"                 },
	{ "sast",  1,  10800, "Africa/Windhoek"               },
	{ "sast",  0,   7200, "Africa/Maseru"                 },
	{ "sast",  0,   7200, "Africa/Mbabane"                },
	{ "sast",  0,   7200, "Africa/Windhoek"               },
	{ "sbt",   0,  39600, "Pacific/Guadalcanal"           },
	{ "sct",   0,  14400, "Indian/Mahe"                   },
	{ "sdmt",  0, -16800, "America/Santo_Domingo"         },
	{ "sgt",   0,  27000, "Asia/Singapore"                },
	{ "sgt",   0,  28800, "Asia/Singapore"                },
	{ "shest", 1,  21600, "Asia/Aqtau"                    },
	{ "shet",  0,  18000, "Asia/Aqtau"                    },
	{ "shet",  0,  21600, "Asia/Aqtau"                    },
	{ "sjmt",  0, -20180, "America/Costa_Rica"            },
	{ "slst",  1,  -1200, "Africa/Freetown"               },
	{ "slst",  1,   3600, "Africa/Freetown"               },
	{ "smt",   0,  25580, "Asia/Saigon"                   },
	{ "smt",   0, -13884, "Atlantic/Stanley"              },
	{ "smt",   0, -16966, "America/Santiago"              },
	{ "smt",   0, -16966, "Chile/Continental"             },
	{ "smt",   0,  24925, "Asia/Kuala_Lumpur"             },
	{ "smt",   0,  24925, "Asia/Singapore"                },
	{ "smt",   0,  25580, "Asia/Ho_Chi_Minh"              },
	{ "smt",   0,  25580, "Asia/Phnom_Penh"               },
	{ "smt",   0,  25580, "Asia/Vientiane"                },
	{ "smt",   0,   8160, "Europe/Simferopol"             },
	{ "srt",   0, -10800, "America/Paramaribo"            },
	{ "srt",   0, -12600, "America/Paramaribo"            },
	{ "sst",   0, -39600, "Pacific/Samoa"                 },
	{ "sst",   0, -39600, "Pacific/Midway"                },
	{ "sst",   0, -39600, "Pacific/Pago_Pago"             },
	{ "stat",  0,  10800, "Europe/Volgograd"              },
	{ "stat",  0,  14400, "Europe/Volgograd"              },
	{ "svest", 1,  18000, "Asia/Yekaterinburg"            },
	{ "svest", 1,  21600, "Asia/Yekaterinburg"            },
	{ "svet",  0,  14400, "Asia/Yekaterinburg"            },
	{ "svet",  0,  18000, "Asia/Yekaterinburg"            },
	{ "swat",  0,   5400, "Africa/Windhoek"               },
	{ "syot",  0,  10800, "Antarctica/Syowa"              },
	{ "taht",  0, -36000, "Pacific/Tahiti"                },
	{ "tasst", 1,  25200, "Asia/Samarkand"                },
	{ "tasst", 1,  21600, "Asia/Tashkent"                 },
	{ "tasst", 1,  25200, "Asia/Tashkent"                 },
	{ "tast",  0,  21600, "Asia/Samarkand"                },
	{ "tast",  0,  18000, "Asia/Tashkent"                 },
	{ "tast",  0,  21600, "Asia/Tashkent"                 },
	{ "tbist", 1,  14400, "Asia/Tbilisi"                  },
	{ "tbist", 1,  18000, "Asia/Tbilisi"                  },
	{ "tbit",  0,  10800, "Asia/Tbilisi"                  },
	{ "tbit",  0,  14400, "Asia/Tbilisi"                  },
	{ "tbmt",  0,  10756, "Asia/Tbilisi"                  },
	{ "tft",   0,  18000, "Indian/Kerguelen"              },
	{ "tjt",   0,  18000, "Asia/Dushanbe"                 },
	{ "tkt",   0, -36000, "Pacific/Fakaofo"               },
	{ "tlt",   0,  28800, "Asia/Dili"                     },
	{ "tlt",   0,  32400, "Asia/Dili"                     },
	{ "tmt",   0,  12344, "Asia/Tehran"                   },
	{ "tmt",   0,  14400, "Asia/Ashgabat"                 },
	{ "tmt",   0,  14400, "Asia/Ashkhabad"                },
	{ "tmt",   0,  18000, "Asia/Ashgabat"                 },
	{ "tmt",   0,  18000, "Asia/Ashkhabad"                },
	{ "tmt",   0,   5940, "Europe/Tallinn"                },
	{ "tost",  1,  50400, "Pacific/Tongatapu"             },
	{ "tot",   0,  44400, "Pacific/Tongatapu"             },
	{ "tot",   0,  46800, "Pacific/Tongatapu"             },
	{ "trst",  1,  14400, "Europe/Istanbul"               },
	{ "trst",  1,  14400, "Asia/Istanbul"                 },
	{ "trt",   0,  10800, "Europe/Istanbul"               },
	{ "trt",   0,  10800, "Asia/Istanbul"                 },
	{ "tsat",  0,  10800, "Europe/Volgograd"              },
	{ "tvt",   0,  43200, "Pacific/Funafuti"              },
	{ "uct",   0,      0, "Etc/UCT"                       },
	{ "uct",   0,      0, "UCT"                           },
	{ "ulast", 1,  32400, "Asia/Ulaanbaatar"              },
	{ "ulast", 1,  32400, "Asia/Ulan_Bator"               },
	{ "ulat",  0,  25200, "Asia/Ulaanbaatar"              },
	{ "ulat",  0,  28800, "Asia/Ulaanbaatar"              },
	{ "ulat",  0,  25200, "Asia/Choibalsan"               },
	{ "ulat",  0,  25200, "Asia/Ulan_Bator"               },
	{ "ulat",  0,  28800, "Asia/Choibalsan"               },
	{ "ulat",  0,  28800, "Asia/Ulan_Bator"               },
	{ "urast", 1,  18000, "Asia/Oral"                     },
	{ "urast", 1,  21600, "Asia/Oral"                     },
	{ "urat",  0,  14400, "Asia/Oral"                     },
	{ "urat",  0,  18000, "Asia/Oral"                     },
	{ "urat",  0,  21600, "Asia/Oral"                     },
	{ "urut",  0,  21600, "Asia/Urumqi"                   },
	{ "utc",   0,      0, "Etc/Universal"                 },
	{ "utc",   0,      0, "Etc/UTC"                       },
	{ "utc",   0,      0, "Etc/Zulu"                      },
	{ "utc",   0,      0, "GMT"                           },
	{ "utc",   0,      0, "UTC"                           },
	{ "utc",   0,      0, "UTC"                           },
	{ "uyhst", 1, -10800, "America/Montevideo"            },
	{ "uyhst", 1,  -9000, "America/Montevideo"            },
	{ "uyst",  1,  -7200, "America/Montevideo"            },
	{ "uyt",   0, -10800, "America/Montevideo"            },
	{ "uyt",   0, -12600, "America/Montevideo"            },
	{ "uzst",  1,  21600, "Asia/Samarkand"                },
	{ "uzst",  1,  21600, "Asia/Tashkent"                 },
	{ "uzt",   0,  18000, "Asia/Samarkand"                },
	{ "uzt",   0,  18000, "Asia/Tashkent"                 },
	{ "vet",   0, -14400, "America/Caracas"               },
	{ "vet",   0, -16200, "America/Caracas"               },
	{ "vlasst", 1,  36000, "Asia/Vladivostok"              },
	{ "vlast", 0,  32400, "Asia/Vladivostok"              },
	{ "vlast", 1,  39600, "Asia/Vladivostok"              },
	{ "vlat",  0,  32400, "Asia/Vladivostok"              },
	{ "vlat",  0,  36000, "Asia/Vladivostok"              },
	{ "volst", 1,  14400, "Europe/Volgograd"              },
	{ "volst", 1,  18000, "Europe/Volgograd"              },
	{ "volt",  0,  10800, "Europe/Volgograd"              },
	{ "volt",  0,  14400, "Europe/Volgograd"              },
	{ "vost",  0,  21600, "Antarctica/Vostok"             },
	{ "vust",  1,  43200, "Pacific/Efate"                 },
	{ "vut",   0,  39600, "Pacific/Efate"                 },
	{ "wakt",  0,  43200, "Pacific/Wake"                  },
	{ "warst", 1, -10800, "America/Mendoza"               },
	{ "warst", 1, -10800, "America/Argentina/Jujuy"       },
	{ "warst", 1, -10800, "America/Argentina/Mendoza"     },
	{ "warst", 1, -10800, "America/Argentina/San_Luis"    },
	{ "warst", 1, -10800, "America/Jujuy"                 },
	{ "wart",  0, -14400, "America/Mendoza"               },
	{ "wart",  0, -14400, "America/Argentina/Catamarca"   },
	{ "wart",  0, -14400, "America/Argentina/ComodRivadavia" },
	{ "wart",  0, -14400, "America/Argentina/Cordoba"     },
	{ "wart",  0, -14400, "America/Argentina/Jujuy"       },
	{ "wart",  0, -14400, "America/Argentina/La_Rioja"    },
	{ "wart",  0, -14400, "America/Argentina/Mendoza"     },
	{ "wart",  0, -14400, "America/Argentina/Rio_Gallegos" },
	{ "wart",  0, -14400, "America/Argentina/Salta"       },
	{ "wart",  0, -14400, "America/Argentina/San_Juan"    },
	{ "wart",  0, -14400, "America/Argentina/San_Luis"    },
	{ "wart",  0, -14400, "America/Argentina/Tucuman"     },
	{ "wart",  0, -14400, "America/Argentina/Ushuaia"     },
	{ "wart",  0, -14400, "America/Catamarca"             },
	{ "wart",  0, -14400, "America/Cordoba"               },
	{ "wart",  0, -14400, "America/Jujuy"                 },
	{ "wart",  0, -14400, "America/Rosario"               },
	{ "wast",  1,   7200, "Africa/Windhoek"               },
	{ "wast",  1,   7200, "Africa/Ndjamena"               },
	{ "wat",   0,  -3600, "Africa/Dakar"                  },
	{ "wat",   0,  -3600, "Africa/Bamako"                 },
	{ "wat",   0,  -3600, "Africa/Banjul"                 },
	{ "wat",   0,  -3600, "Africa/Bissau"                 },
	{ "wat",   0,  -3600, "Africa/Conakry"                },
	{ "wat",   0,  -3600, "Africa/El_Aaiun"               },
	{ "wat",   0,  -3600, "Africa/Freetown"               },
	{ "wat",   0,  -3600, "Africa/Niamey"                 },
	{ "wat",   0,  -3600, "Africa/Nouakchott"             },
	{ "wat",   0,  -3600, "Africa/Timbuktu"               },
	{ "wat",   0,      0, "Africa/Freetown"               },
	{ "wat",   0,   3600, "Africa/Brazzaville"            },
	{ "wat",   0,   3600, "Africa/Bangui"                 },
	{ "wat",   0,   3600, "Africa/Douala"                 },
	{ "wat",   0,   3600, "Africa/Kinshasa"               },
	{ "wat",   0,   3600, "Africa/Lagos"                  },
	{ "wat",   0,   3600, "Africa/Libreville"             },
	{ "wat",   0,   3600, "Africa/Luanda"                 },
	{ "wat",   0,   3600, "Africa/Malabo"                 },
	{ "wat",   0,   3600, "Africa/Ndjamena"               },
	{ "wat",   0,   3600, "Africa/Niamey"                 },
	{ "wat",   0,   3600, "Africa/Porto-Novo"             },
	{ "wat",   0,   3600, "Africa/Windhoek"               },
	{ "wemt",  1,   7200, "Europe/Lisbon"                 },
	{ "wemt",  1,   7200, "Europe/Madrid"                 },
	{ "wemt",  1,   7200, "Europe/Monaco"                 },
	{ "wemt",  1,   7200, "Europe/Paris"                  },
	{ "wemt",  1,   7200, "WET"                           },
	{ "west",  1,   3600, "Europe/Paris"                  },
	{ "west",  1,   3600, "Africa/Algiers"                },
	{ "west",  1,   3600, "Africa/Casablanca"             },
	{ "west",  1,   3600, "Africa/Ceuta"                  },
	{ "west",  1,   3600, "Atlantic/Canary"               },
	{ "west",  1,   3600, "Atlantic/Faeroe"               },
	{ "west",  1,   3600, "Atlantic/Faroe"                },
	{ "west",  1,   3600, "Atlantic/Madeira"              },
	{ "west",  1,   3600, "Europe/Brussels"               },
	{ "west",  1,   3600, "Europe/Lisbon"                 },
	{ "west",  1,   3600, "Europe/Luxembourg"             },
	{ "west",  1,   3600, "Europe/Madrid"                 },
	{ "west",  1,   3600, "Europe/Monaco"                 },
	{ "west",  1,   3600, "WET"                           },
	{ "west",  1,   7200, "Europe/Luxembourg"             },
	{ "wet",   0,      0, "Europe/Paris"                  },
	{ "wet",   0,      0, "Africa/Algiers"                },
	{ "wet",   0,      0, "Africa/Casablanca"             },
	{ "wet",   0,      0, "Africa/Ceuta"                  },
	{ "wet",   0,      0, "Africa/El_Aaiun"               },
	{ "wet",   0,      0, "Atlantic/Azores"               },
	{ "wet",   0,      0, "Atlantic/Canary"               },
	{ "wet",   0,      0, "Atlantic/Faeroe"               },
	{ "wet",   0,      0, "Atlantic/Faroe"                },
	{ "wet",   0,      0, "Atlantic/Madeira"              },
	{ "wet",   0,      0, "Europe/Andorra"                },
	{ "wet",   0,      0, "Europe/Brussels"               },
	{ "wet",   0,      0, "Europe/Lisbon"                 },
	{ "wet",   0,      0, "Europe/Luxembourg"             },
	{ "wet",   0,      0, "Europe/Madrid"                 },
	{ "wet",   0,      0, "Europe/Monaco"                 },
	{ "wet",   0,      0, "WET"                           },
	{ "wet",   0,   3600, "Europe/Luxembourg"             },
	{ "wft",   0,  43200, "Pacific/Wallis"                },
	{ "wgst",  1,  -7200, "America/Godthab"               },
	{ "wgst",  1,  -7200, "America/Danmarkshavn"          },
	{ "wgt",   0, -10800, "America/Godthab"               },
	{ "wgt",   0, -10800, "America/Danmarkshavn"          },
	{ "wit",   0,  25200, "Asia/Jakarta"                  },
	{ "wit",   0,  27000, "Asia/Jakarta"                  },
	{ "wit",   0,  28800, "Asia/Jakarta"                  },
	{ "wit",   0,  25200, "Asia/Pontianak"                },
	{ "wit",   0,  27000, "Asia/Pontianak"                },
	{ "wit",   0,  28800, "Asia/Pontianak"                },
	{ "wmt",   0,   5040, "Europe/Vilnius"                },
	{ "wmt",   0,   5040, "Europe/Warsaw"                 },
	{ "wsdt",  1, -36000, "Pacific/Apia"                  },
	{ "wst",   0,  28800, "Australia/Perth"               },
	{ "wst",   1,  32400, "Australia/Perth"               },
	{ "wst",   0, -39600, "Pacific/Apia"                  },
	{ "wst",   0,  28800, "Antarctica/Casey"              },
	{ "wst",   0,  28800, "Australia/West"                },
	{ "wst",   1,  32400, "Australia/West"                },
	{ "yakst", 1,  32400, "Asia/Yakutsk"                  },
	{ "yakst", 1,  36000, "Asia/Yakutsk"                  },
	{ "yakt",  0,  28800, "Asia/Yakutsk"                  },
	{ "yakt",  0,  32400, "Asia/Yakutsk"                  },
	{ "yddt",  1, -25200, "America/Dawson"                },
	{ "yddt",  1, -25200, "America/Whitehorse"            },
	{ "yddt",  1, -25200, "Canada/Yukon"                  },
	{ "ydt",   1, -28800, "America/Dawson"                },
	{ "ydt",   1, -28800, "America/Whitehorse"            },
	{ "ydt",   1, -28800, "America/Yakutat"               },
	{ "ydt",   1, -28800, "Canada/Yukon"                  },
	{ "yekst", 1,  21600, "Asia/Yekaterinburg"            },
	{ "yekt",  0,  18000, "Asia/Yekaterinburg"            },
	{ "yerst", 1,  14400, "Asia/Yerevan"                  },
	{ "yerst", 1,  18000, "Asia/Yerevan"                  },
	{ "yert",  0,  10800, "Asia/Yerevan"                  },
	{ "yert",  0,  14400, "Asia/Yerevan"                  },
	{ "ypt",   1, -28800, "America/Dawson"                },
	{ "ypt",   1, -28800, "America/Whitehorse"            },
	{ "ypt",   1, -28800, "America/Yakutat"               },
	{ "ypt",   1, -28800, "Canada/Yukon"                  },
	{ "yst",   0, -32400, "America/Anchorage"             },
	{ "yst",   0, -32400, "America/Dawson"                },
	{ "yst",   0, -32400, "America/Juneau"                },
	{ "yst",   0, -32400, "America/Nome"                  },
	{ "yst",   0, -32400, "America/Whitehorse"            },
	{ "yst",   0, -32400, "America/Yakutat"               },
	{ "yst",   0, -32400, "Canada/Yukon"                  },
	{ "ywt",   1, -28800, "America/Dawson"                },
	{ "ywt",   1, -28800, "America/Whitehorse"            },
	{ "ywt",   1, -28800, "America/Yakutat"               },
	{ "ywt",   1, -28800, "Canada/Yukon"                  },
	{ "a",     0,   3600, NULL                            },
	{ "b",     0,   7200, NULL                            },
	{ "c",     0,  10800, NULL                            },
	{ "d",     0,  14400, NULL                            },
	{ "e",     0,  18000, NULL                            },
	{ "f",     0,  21600, NULL                            },
	{ "g",     0,  25200, NULL                            },
	{ "h",     0,  28800, NULL                            },
	{ "i",     0,  32400, NULL                            },
	{ "k",     0,  36000, NULL                            },
	{ "l",     0,  39600, NULL                            },
	{ "m",     0,  43200, NULL                            },
	{ "n",     0,  -3600, NULL                            },
	{ "o",     0,  -7200, NULL                            },
	{ "p",     0, -10800, NULL                            },
	{ "q",     0, -14400, NULL                            },
	{ "r",     0, -18000, NULL                            },
	{ "s",     0, -21600, NULL                            },
	{ "t",     0, -25200, NULL                            },
	{ "u",     0, -28800, NULL                            },
	{ "v",     0, -32400, NULL                            },
	{ "w",     0, -36000, NULL                            },
	{ "x",     0, -39600, NULL                            },
	{ "y",     0, -43200, NULL                            },
	{ "zzz",   0,      0, "Antarctica/Davis"              },
	{ "zzz",   0,      0, "America/Cambridge_Bay"         },
	{ "zzz",   0,      0, "America/Inuvik"                },
	{ "zzz",   0,      0, "America/Iqaluit"               },
	{ "zzz",   0,      0, "America/Pangnirtung"           },
	{ "zzz",   0,      0, "America/Rankin_Inlet"          },
	{ "zzz",   0,      0, "America/Resolute"              },
	{ "zzz",   0,      0, "America/Yellowknife"           },
	{ "zzz",   0,      0, "Antarctica/Casey"              },
	{ "zzz",   0,      0, "Antarctica/DumontDUrville"     },
	{ "zzz",   0,      0, "Antarctica/Macquarie"          },
	{ "zzz",   0,      0, "Antarctica/Mawson"             },
	{ "zzz",   0,      0, "Antarctica/McMurdo"            },
	{ "zzz",   0,      0, "Antarctica/Palmer"             },
	{ "zzz",   0,      0, "Antarctica/Rothera"            },
	{ "zzz",   0,      0, "Antarctica/South_Pole"         },
	{ "zzz",   0,      0, "Antarctica/Syowa"              },
	{ "zzz",   0,      0, "Antarctica/Vostok"             },
	{ "zzz",   0,      0, "Indian/Kerguelen"              },
	{ "z",     0,      0, NULL                            },
