struct _IO_FILE;
struct _IO_FILE *_coverage_fout ;
typedef struct _IO_FILE FILE;
extern int fclose(FILE *__stream ) ;
extern int fflush(FILE *__stream ) ;
extern int fprintf(FILE * __restrict  __stream ,
                   char const   * __restrict  __format  , ...) ;
extern FILE *fopen(char const   * __restrict  __filename ,
                   char const   * __restrict  __modes ) ;
typedef unsigned int size_t;
size_t __attribute__((__visibility__("default")))  zend_dirname(char *path ,
                                                                size_t len ) 
{ register char *end ;
  unsigned int len_adjust ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8555\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  end = (path + len) - 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8556\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  len_adjust = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8557\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (len == 0U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8529\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8530\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8558\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8533\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )end >= (unsigned int )path) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8532\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*end == 47) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8531\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8534\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    end --;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8559\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )end < (unsigned int )path) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8535\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(path + 0) = (char )'/';
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8536\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(path + 1) = (char )'\000';
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8537\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned int __attribute__((__visibility__("default")))  )(1U + len_adjust));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8538\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8560\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8541\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )end >= (unsigned int )path) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8540\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! ((int )*end == 47)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8539\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8542\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    end --;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8561\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )end < (unsigned int )path) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8543\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(path + 0) = (char )'.';
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8544\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(path + 1) = (char )'\000';
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8545\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned int __attribute__((__visibility__("default")))  )(1U + len_adjust));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8546\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8562\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8549\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )end >= (unsigned int )path) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8548\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*end == 47) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8547\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8550\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    end --;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8563\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )end < (unsigned int )path) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8551\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(path + 0) = (char )'/';
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8552\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(path + 1) = (char )'\000';
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8553\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned int __attribute__((__visibility__("default")))  )(1U + len_adjust));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8554\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8564\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *(end + 1) = (char )'\000';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8565\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned int __attribute__((__visibility__("default")))  )((unsigned int )((end + 1) - path) + len_adjust));
}
}
size_t __attribute__((__visibility__("default")))  zend_dirname(char *path ,
                                                                size_t len ) ;
void zend_do_end_namespace(void) ;
struct _zend_compiler_globals;
struct _zend_compiler_globals  __attribute__((__visibility__("default"))) compiler_globals ;
typedef unsigned char zend_bool;
struct _zend_encoding;
typedef struct _zend_encoding zend_encoding;
struct bucket;
typedef struct bucket Bucket;
typedef unsigned long ulong;
typedef unsigned int uint;
struct _hashtable {
   uint nTableSize ;
   uint nTableMask ;
   uint nNumOfElements ;
   ulong nNextFreeElement ;
   Bucket *pInternalPointer ;
   Bucket *pListHead ;
   Bucket *pListTail ;
   Bucket **arBuckets ;
   void (*pDestructor)(void *pDest ) ;
   zend_bool persistent ;
   unsigned char nApplyCount ;
   zend_bool bApplyProtection ;
};
typedef struct _hashtable HashTable;
struct _zend_stack {
   int top ;
   int max ;
   void **elements ;
};
typedef struct _zend_stack zend_stack;
struct _hashtable;
typedef unsigned int zend_uint;
struct _zend_compiler_context {
   zend_uint opcodes_size ;
   int vars_size ;
   int literals_size ;
   int current_brk_cont ;
   int backpatch_count ;
   HashTable *labels ;
};
typedef struct _zend_compiler_context zend_compiler_context;
struct _zval_struct;
typedef struct _zval_struct zval;
struct _zend_op_array;
typedef struct _zend_op_array zend_op_array;
typedef unsigned char zend_uchar;
struct _zend_object_handlers;
typedef struct _zend_object_handlers zend_object_handlers;
typedef unsigned int zend_object_handle;
struct _zend_object_value {
   zend_object_handle handle ;
   zend_object_handlers const   *handlers ;
};
typedef struct _zend_object_value zend_object_value;
struct __anonstruct_str_39 {
   char *val ;
   int len ;
};
union _zvalue_value {
   long lval ;
   double dval ;
   struct __anonstruct_str_39 str ;
   HashTable *ht ;
   zend_object_value obj ;
};
typedef union _zvalue_value zvalue_value;
struct _zval_struct {
   zvalue_value value ;
   zend_uint refcount__gc ;
   zend_uchar type ;
   zend_uchar is_ref__gc ;
};
struct _zend_literal;
typedef struct _zend_literal zend_literal;
struct _zend_op;
typedef struct _zend_op zend_op;
typedef unsigned long zend_ulong;
union _znode_op {
   zend_uint constant ;
   zend_uint var ;
   zend_uint num ;
   zend_ulong hash ;
   zend_uint opline_num ;
   zend_op *jmp_addr ;
   zval *zv ;
   zend_literal *literal ;
   void *ptr ;
};
typedef union _znode_op znode_op;
union __anonunion_u_49 {
   znode_op op ;
   zval constant ;
   zend_op_array *op_array ;
};
struct _znode {
   int op_type ;
   union __anonunion_u_49 u ;
   zend_uint EA ;
};
typedef struct _znode znode;
struct _zend_ini_parser_param;
struct _zend_llist_element;
typedef struct _zend_llist_element zend_llist_element;
struct _zend_llist {
   zend_llist_element *head ;
   zend_llist_element *tail ;
   size_t count ;
   size_t size ;
   void (*dtor)(void * ) ;
   unsigned char persistent ;
   zend_llist_element *traverse_ptr ;
};
typedef struct _zend_llist zend_llist;
struct _zend_declarables {
   zval ticks ;
};
typedef struct _zend_declarables zend_declarables;
struct _zend_class_entry;
typedef struct _zend_class_entry zend_class_entry;
struct _zend_compiler_globals {
   zend_stack bp_stack ;
   zend_stack switch_cond_stack ;
   zend_stack foreach_copy_stack ;
   zend_stack object_stack ;
   zend_stack declare_stack ;
   zend_class_entry *active_class_entry ;
   zend_llist list_llist ;
   zend_llist dimension_llist ;
   zend_stack list_stack ;
   zend_stack function_call_stack ;
   char *compiled_filename ;
   int zend_lineno ;
   char *heredoc ;
   int heredoc_len ;
   zend_op_array *active_op_array ;
   HashTable *function_table ;
   HashTable *class_table ;
   HashTable filenames_table ;
   HashTable *auto_globals ;
   zend_bool parse_error ;
   zend_bool in_compilation ;
   zend_bool short_tags ;
   zend_bool asp_tags ;
   zend_declarables declarables ;
   zend_bool unclean_shutdown ;
   zend_bool ini_parser_unbuffered_errors ;
   zend_llist open_files ;
   long catch_begin ;
   struct _zend_ini_parser_param *ini_parser_param ;
   int interactive ;
   zend_uint start_lineno ;
   zend_bool increment_lineno ;
   znode implementing_class ;
   zend_uint access_type ;
   char *doc_comment ;
   zend_uint doc_comment_len ;
   zend_uint compiler_options ;
   zval *current_namespace ;
   HashTable *current_import ;
   zend_bool in_namespace ;
   zend_bool has_bracketed_namespaces ;
   zend_compiler_context context ;
   zend_stack context_stack ;
   char *interned_strings_start ;
   char *interned_strings_end ;
   char *interned_strings_top ;
   char *interned_strings_snapshot_top ;
   HashTable interned_strings ;
   zend_encoding const   **script_encoding_list ;
   size_t script_encoding_list_size ;
   zend_bool multibyte ;
   zend_bool detect_unicode ;
   zend_bool encoding_declared ;
};
void zend_do_end_compilation(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8526\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.has_bracketed_namespaces = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8527\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_end_namespace();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8528\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_end_compilation(void) ;
extern void __attribute__((__visibility__("default")))  _efree(void *ptr ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_destroy(HashTable *ht ) ;
extern void __attribute__((__visibility__("default")))  gc_remove_zval_from_buffer(zval *zv ) ;
struct _zval_gc_info;
typedef struct _zval_gc_info zval_gc_info;
struct _gc_root_buffer;
typedef struct _gc_root_buffer gc_root_buffer;
union __anonunion_u_46 {
   gc_root_buffer *buffered ;
   struct _zval_gc_info *next ;
};
struct _zval_gc_info {
   zval z ;
   union __anonunion_u_46 u ;
};
__inline static void ( __attribute__((__always_inline__)) _zval_dtor)(zval *zvalue ) ;
void zend_do_end_namespace(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8522\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.in_namespace = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8523\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.current_namespace) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8514\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_dtor(compiler_globals.current_namespace);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8515\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8512\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((gc_root_buffer *)((unsigned long )((zval_gc_info *)compiler_globals.current_namespace)->u.buffered & 4294967292UL)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8510\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        gc_remove_zval_from_buffer(compiler_globals.current_namespace);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8511\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8513\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)compiler_globals.current_namespace);
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8516\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.current_namespace = (zval *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8517\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8524\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.current_import) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8518\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_hash_destroy(compiler_globals.current_import);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8519\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)compiler_globals.current_import);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8520\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.current_import = (HashTable *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8521\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8525\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  zend_error)(int type ,
                                                                                              char const   *format 
                                                                                              , ...) ;
void zend_verify_namespace(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8508\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.has_bracketed_namespaces) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8506\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! compiler_globals.in_namespace) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8504\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "No code may exist outside of namespace {}");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8505\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8507\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8509\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_verify_namespace(void) ;
struct _znode;
struct _zend_execute_data;
typedef struct _zend_execute_data zend_execute_data;
struct _zend_op {
   int ( __attribute__((__fastcall__)) (*handler))(zend_execute_data *execute_data ) ;
   znode_op op1 ;
   znode_op op2 ;
   znode_op result ;
   ulong extended_value ;
   uint lineno ;
   zend_uchar opcode ;
   zend_uchar op1_type ;
   zend_uchar op2_type ;
   zend_uchar result_type ;
};
int zend_add_literal(zend_op_array *op_array , zval const   *zv ) ;
extern zend_op *get_next_op(zend_op_array *op_array ) ;
void zend_do_build_namespace_name(znode *result , znode *prefix , znode *name ) ;
extern char __attribute__((__visibility__("default")))  *zend_str_tolower_dup(char const   *source ,
                                                                              unsigned int length ) ;
struct _zend_constant;
typedef struct _zend_constant zend_constant;
static zend_constant *zend_get_ct_const(zval const   *const_name ,
                                        int all_internal_constants_substitution ) ;
void zend_do_declare_constant(znode *name , znode *value ) 
{ zend_op *opline ;
  zend_constant *tmp ;
  znode tmp___0 ;
  char __attribute__((__visibility__("default")))  *tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8494\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )value->u.constant.type == 9) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8473\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Arrays are not allowed as constants");
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8474\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8495\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_get_ct_const((zval const   *)(& name->u.constant), 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8496\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8475\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Cannot redeclare constant \'%s\'",
               name->u.constant.value.str.val);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8476\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8497\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.current_namespace) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8477\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0.op_type = 1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8478\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0.u.constant = *(compiler_globals.current_namespace);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8479\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = zend_str_tolower_dup((char const   *)tmp___0.u.constant.value.str.val,
                                   (unsigned int )tmp___0.u.constant.value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8480\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0.u.constant.value.str.val = (char *)tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8481\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_build_namespace_name(& tmp___0, & tmp___0, name);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8482\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *name = tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8483\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8498\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8499\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)143;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8500\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8501\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8487\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )name->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8488\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (name->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8484\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& name->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8485\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___2;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8486\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = name->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8502\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8492\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )value->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8493\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (value->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8489\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___3 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& value->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8490\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___3;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8491\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2 = value->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8503\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_declare_constant(znode *name , znode *value ) ;
extern  __attribute__((__nothrow__)) size_t strlen(char const   *__s )  __attribute__((__pure__,
__nonnull__(1))) ;
extern int __attribute__((__visibility__("default")))  _zend_hash_add_or_update(HashTable *ht ,
                                                                                char const   *arKey ,
                                                                                uint nKeyLength ,
                                                                                void *pData ,
                                                                                uint nDataSize ,
                                                                                void **pDest ,
                                                                                int flag ) ;
extern  __attribute__((__nothrow__)) int memcmp(void const   *__s1 ,
                                                void const   *__s2 , size_t __n )  __attribute__((__pure__,
__nonnull__(1,2))) ;
struct _zend_module_entry;
struct _zend_function_entry;
struct __anonstruct_internal_42 {
   struct _zend_function_entry  const  *builtin_functions ;
   struct _zend_module_entry *module ;
};
struct __anonstruct_user_41 {
   char const   *filename ;
   zend_uint line_start ;
   zend_uint line_end ;
   char const   *doc_comment ;
   zend_uint doc_comment_len ;
};
union __anonunion_info_40 {
   struct __anonstruct_user_41 user ;
   struct __anonstruct_internal_42 internal ;
};
struct _zend_trait_precedence;
typedef struct _zend_trait_precedence zend_trait_precedence;
struct _zend_trait_alias;
typedef struct _zend_trait_alias zend_trait_alias;
struct _zend_unserialize_data;
typedef struct _zend_unserialize_data zend_unserialize_data;
struct _zend_serialize_data;
typedef struct _zend_serialize_data zend_serialize_data;
union _zend_function;
struct _zend_object_iterator;
typedef struct _zend_object_iterator zend_object_iterator;
struct _zend_object_iterator_funcs;
typedef struct _zend_object_iterator_funcs zend_object_iterator_funcs;
struct _zend_class_iterator_funcs {
   zend_object_iterator_funcs *funcs ;
   union _zend_function *zf_new_iterator ;
   union _zend_function *zf_valid ;
   union _zend_function *zf_current ;
   union _zend_function *zf_key ;
   union _zend_function *zf_next ;
   union _zend_function *zf_rewind ;
};
typedef struct _zend_class_iterator_funcs zend_class_iterator_funcs;
struct _zend_class_entry {
   char type ;
   char const   *name ;
   zend_uint name_length ;
   struct _zend_class_entry *parent ;
   int refcount ;
   zend_uint ce_flags ;
   HashTable function_table ;
   HashTable properties_info ;
   zval **default_properties_table ;
   zval **default_static_members_table ;
   zval **static_members_table ;
   HashTable constants_table ;
   int default_properties_count ;
   int default_static_members_count ;
   union _zend_function *constructor ;
   union _zend_function *destructor ;
   union _zend_function *clone ;
   union _zend_function *__get ;
   union _zend_function *__set ;
   union _zend_function *__unset ;
   union _zend_function *__isset ;
   union _zend_function *__call ;
   union _zend_function *__callstatic ;
   union _zend_function *__tostring ;
   union _zend_function *serialize_func ;
   union _zend_function *unserialize_func ;
   zend_class_iterator_funcs iterator_funcs ;
   zend_object_value (*create_object)(zend_class_entry *class_type ) ;
   zend_object_iterator *(*get_iterator)(zend_class_entry *ce , zval *object ,
                                         int by_ref ) ;
   int (*interface_gets_implemented)(zend_class_entry *iface ,
                                     zend_class_entry *class_type ) ;
   union _zend_function *(*get_static_method)(zend_class_entry *ce ,
                                              char *method , int method_len ) ;
   int (*serialize)(zval *object , unsigned char **buffer , zend_uint *buf_len ,
                    zend_serialize_data *data ) ;
   int (*unserialize)(zval **object , zend_class_entry *ce ,
                      unsigned char const   *buf , zend_uint buf_len ,
                      zend_unserialize_data *data ) ;
   zend_class_entry **interfaces ;
   zend_uint num_interfaces ;
   zend_class_entry **traits ;
   zend_uint num_traits ;
   zend_trait_alias **trait_aliases ;
   zend_trait_precedence **trait_precedences ;
   union __anonunion_info_40 info ;
};
extern int __attribute__((__visibility__("default")))  zend_hash_find(HashTable const   *ht ,
                                                                      char const   *arKey ,
                                                                      uint nKeyLength ,
                                                                      void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_exists(HashTable const   *ht ,
                                                                        char const   *arKey ,
                                                                        uint nKeyLength ) ;
extern  __attribute__((__nothrow__)) void *memcpy(void * __restrict  __dest ,
                                                  void const   * __restrict  __src ,
                                                  size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern char __attribute__((__visibility__("default")))  *zend_str_tolower_copy(char *dest ,
                                                                               char const   *source ,
                                                                               unsigned int length ) ;
extern void __attribute__((__visibility__("default")))  *_emalloc(size_t size )  __attribute__((__malloc__)) ;
__inline static void ( __attribute__((__always_inline__)) _zval_copy_ctor)(zval *zvalue ) ;
extern char __attribute__((__visibility__("default")))  *_estrndup(char const   *s ,
                                                                   unsigned int length )  __attribute__((__malloc__)) ;
__inline static void const   *zend_memrchr(void const   *s , int c , size_t n ) ;
extern void __attribute__((__visibility__("default")))  _zval_ptr_dtor(zval **zval_ptr ) ;
extern int __attribute__((__visibility__("default")))  _zend_hash_init(HashTable *ht ,
                                                                       uint nSize ,
                                                                       ulong (*pHashFunction)(char const   *arKey ,
                                                                                              uint nKeyLength ) ,
                                                                       void (*pDestructor)(void *pDest ) ,
                                                                       zend_bool persistent ) ;
void zend_do_use(znode *ns_name , znode *new_name , int is_global ) 
{ char *lcname ;
  zval *name ;
  zval *ns ;
  zval tmp ;
  zend_bool warn ;
  zend_class_entry **pce ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  char const   *p ;
  void const   *tmp___2 ;
  char const   *__s ;
  zval *__z ;
  size_t tmp___3 ;
  char __attribute__((__visibility__("default")))  *tmp___4 ;
  int tmp___5 ;
  char __attribute__((__visibility__("default")))  *tmp___6 ;
  int tmp___7 ;
  int tmp___8 ;
  char *c_ns_name ;
  void __attribute__((__visibility__("default")))  *tmp___9 ;
  char *tmp2 ;
  char __attribute__((__visibility__("default")))  *tmp___10 ;
  int tmp___11 ;
  int __attribute__((__visibility__("default")))  tmp___12 ;
  char *c_tmp ;
  char __attribute__((__visibility__("default")))  *tmp___13 ;
  int tmp___14 ;
  int __attribute__((__visibility__("default")))  tmp___15 ;
  int __attribute__((__visibility__("default")))  tmp___16 ;
  size_t __s1_len ;
  size_t __s2_len ;
  int tmp___17 ;
  int tmp___18 ;
  int tmp___19 ;
  unsigned char const   *__s2 ;
  register int __result ;
  int tmp___20 ;
  unsigned char const   *__s1 ;
  register int __result___0 ;
  int tmp___21 ;
  int tmp___22 ;
  int tmp___23 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8458\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  warn = (zend_bool )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8459\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! compiler_globals.current_import) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8362\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = _emalloc(sizeof(HashTable ));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8363\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.current_import = (HashTable *)tmp___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8364\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zend_hash_init(compiler_globals.current_import, 0U,
                    (ulong (*)(char const   *arKey , uint nKeyLength ))((void *)0),
                    (void (*)(void * ))(& _zval_ptr_dtor), (unsigned char)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8365\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8460\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8366\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = _emalloc(sizeof(zval_gc_info ));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8367\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ns = (zval *)tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8368\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((zval_gc_info *)ns)->u.buffered = (gc_root_buffer *)((void *)0);
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8461\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *ns = ns_name->u.constant;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8462\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (new_name) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8369\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    name = & new_name->u.constant;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8386\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    name = & tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8387\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = zend_memrchr((void const   *)ns->value.str.val, '\\',
                           (unsigned int )ns->value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8388\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p = (char const   *)tmp___2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8389\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (p) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8377\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8370\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __s = p + 1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8371\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z = name;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8372\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = strlen(__s);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8373\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z->value.str.len = (int )tmp___3;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8374\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___4 = _estrndup(__s, (unsigned int )__z->value.str.len);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8375\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z->value.str.val = (char *)tmp___4;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8376\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z->type = (unsigned char)6;
        break;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8382\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *name = *ns;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8383\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zval_copy_ctor(name);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8384\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! is_global) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8380\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (! compiler_globals.current_namespace) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8378\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___5 = 1;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8379\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___5 = 0;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8381\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___5 = 0;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8385\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      warn = (unsigned char )tmp___5;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8463\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___6 = zend_str_tolower_dup((char const   *)name->value.str.val,
                                 (unsigned int )name->value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8464\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lcname = (char *)tmp___6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8465\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )name->value.str.len == sizeof("self") - 1U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8391\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___7 = memcmp((void const   *)lcname, (void const   *)"self",
                     sizeof("self") - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8392\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___7) {
      goto _L;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8390\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L,
                 "Cannot use %s as %s because \'%s\' is a special class name",
                 ns->value.str.val, name->value.str.val, name->value.str.val);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8398\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _L: /* CIL Label */ 
    if ((unsigned int )name->value.str.len == sizeof("parent") - 1U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8395\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___8 = memcmp((void const   *)lcname, (void const   *)"parent",
                       sizeof("parent") - 1U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8396\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___8) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8393\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8394\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L,
                   "Cannot use %s as %s because \'%s\' is a special class name",
                   ns->value.str.val, name->value.str.val, name->value.str.val);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8397\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8466\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.current_namespace) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8409\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___9 = _emalloc((unsigned int )((((compiler_globals.current_namespace)->value.str.len + 1) + name->value.str.len) + 1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8410\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    c_ns_name = (char *)tmp___9;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8411\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_str_tolower_copy(c_ns_name,
                          (char const   *)(compiler_globals.current_namespace)->value.str.val,
                          (unsigned int )(compiler_globals.current_namespace)->value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8412\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(c_ns_name + (compiler_globals.current_namespace)->value.str.len) = (char )'\\';
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8413\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    memcpy((void */* __restrict  */)((c_ns_name + (compiler_globals.current_namespace)->value.str.len) + 1),
           (void const   */* __restrict  */)lcname,
           (unsigned int )(name->value.str.len + 1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8414\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___12 = zend_hash_exists((HashTable const   *)compiler_globals.class_table,
                                (char const   *)c_ns_name,
                                (unsigned int )((((compiler_globals.current_namespace)->value.str.len + 1) + name->value.str.len) + 1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8415\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___12) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8404\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___10 = zend_str_tolower_dup((char const   *)ns->value.str.val,
                                      (unsigned int )ns->value.str.len);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8405\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp2 = (char *)tmp___10;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8406\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (ns->value.str.len != ((compiler_globals.current_namespace)->value.str.len + 1) + name->value.str.len) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8399\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L,
                   "Cannot use %s as %s because the name is already in use",
                   ns->value.str.val, name->value.str.val);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8402\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___11 = memcmp((void const   *)tmp2, (void const   *)c_ns_name,
                          (unsigned int )ns->value.str.len);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8403\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___11) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8400\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L,
                     "Cannot use %s as %s because the name is already in use",
                     ns->value.str.val, name->value.str.val);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8401\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8407\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)tmp2);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8408\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8416\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)c_ns_name);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8431\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___15 = zend_hash_find((HashTable const   *)compiler_globals.class_table,
                              (char const   *)lcname,
                              (unsigned int )(name->value.str.len + 1),
                              (void **)(& pce));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8432\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___15 == (int __attribute__((__visibility__("default")))  )0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8429\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )(*pce)->type == 2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8427\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )(*pce)->info.user.filename == (unsigned int )compiler_globals.compiled_filename) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8422\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___13 = zend_str_tolower_dup((char const   *)ns->value.str.val,
                                          (unsigned int )ns->value.str.len);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8423\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          c_tmp = (char *)tmp___13;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8424\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (ns->value.str.len != name->value.str.len) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8417\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_error(1 << 6L,
                       "Cannot use %s as %s because the name is already in use",
                       ns->value.str.val, name->value.str.val);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8420\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___14 = memcmp((void const   *)c_tmp, (void const   *)lcname,
                              (unsigned int )ns->value.str.len);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8421\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (tmp___14) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8418\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              zend_error(1 << 6L,
                         "Cannot use %s as %s because the name is already in use",
                         ns->value.str.val, name->value.str.val);
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8419\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8425\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _efree((void *)c_tmp);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8426\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8428\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8430\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8467\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___16 = _zend_hash_add_or_update(compiler_globals.current_import,
                                      (char const   *)lcname,
                                      (unsigned int )(name->value.str.len + 1),
                                      (void *)(& ns), sizeof(zval *),
                                      (void **)((void *)0), 1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8468\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___16 != (int __attribute__((__visibility__("default")))  )0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8433\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L,
               "Cannot use %s as %s because the name is already in use",
               ns->value.str.val, name->value.str.val);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8434\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8469\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (warn) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8454\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8446\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __s1_len = strlen((char const   *)name->value.str.val);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8447\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __s2_len = strlen("strict");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8448\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! ((unsigned int )((void const   *)(name->value.str.val + 1)) - (unsigned int )((void const   *)name->value.str.val) == 1U)) {
        goto _L___1;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8441\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (__s1_len >= 4U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8439\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L___1: /* CIL Label */ 
          if (! ((unsigned int )((void const   *)("strict" + 1)) - (unsigned int )((void const   *)"strict") == 1U)) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8435\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___23 = 1;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8438\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (__s2_len >= 4U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8436\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___23 = 1;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8437\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___23 = 0;
            }
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8440\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___23 = 0;
        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8449\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___23) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8442\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___18 = __builtin_strcmp((char const   *)name->value.str.val,
                                    "strict");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8443\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___22 = tmp___18;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8444\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___21 = __builtin_strcmp((char const   *)name->value.str.val,
                                    "strict");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8445\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___22 = tmp___21;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8450\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___21 = __builtin_strcmp((char const   *)name->value.str.val, "strict");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8451\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___22 = tmp___21;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8455\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___22) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8452\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8453\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "You seem to be trying to use a different language...");
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8456\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 1L,
               "The use statement with non-compound name \'%s\' has no effect",
               name->value.str.val);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8457\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8470\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _efree((void *)lcname);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8471\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_dtor(name);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8472\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_use(znode *ns_name , znode *new_name , int is_global ) ;
struct _zend_try_catch_element;
typedef struct _zend_try_catch_element zend_try_catch_element;
struct _zend_brk_cont_element;
typedef struct _zend_brk_cont_element zend_brk_cont_element;
struct _zend_compiled_variable;
typedef struct _zend_compiled_variable zend_compiled_variable;
struct _zend_arg_info;
typedef struct _zend_arg_info zend_arg_info;
struct _zend_op_array {
   zend_uchar type ;
   char const   *function_name ;
   zend_class_entry *scope ;
   zend_uint fn_flags ;
   union _zend_function *prototype ;
   zend_uint num_args ;
   zend_uint required_num_args ;
   zend_arg_info *arg_info ;
   zend_uint *refcount ;
   zend_op *opcodes ;
   zend_uint last ;
   zend_compiled_variable *vars ;
   int last_var ;
   zend_uint T ;
   zend_brk_cont_element *brk_cont_array ;
   int last_brk_cont ;
   zend_try_catch_element *try_catch_array ;
   int last_try_catch ;
   HashTable *static_variables ;
   zend_uint this_var ;
   char const   *filename ;
   zend_uint line_start ;
   zend_uint line_end ;
   char const   *doc_comment ;
   zend_uint doc_comment_len ;
   zend_uint early_binding ;
   zend_literal *literals ;
   int last_literal ;
   void **run_time_cache ;
   int last_cache_slot ;
   void *reserved[4] ;
};
void zend_do_begin_namespace(znode const   *name , zend_bool with_bracket ) 
{ char *lcname ;
  int num ;
  char __attribute__((__visibility__("default")))  *tmp ;
  int tmp___0 ;
  int tmp___1 ;
  void __attribute__((__visibility__("default")))  *tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8354\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! compiler_globals.has_bracketed_namespaces) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8289\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.current_namespace) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8287\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (with_bracket) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8285\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L,
                   "Cannot mix bracketed namespace declarations with unbracketed namespace declarations");
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8286\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8288\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8296\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! with_bracket) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8290\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L,
                 "Cannot mix bracketed namespace declarations with unbracketed namespace declarations");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8295\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (compiler_globals.current_namespace) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8291\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L, "Namespace declarations cannot be nested");
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8294\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (compiler_globals.in_namespace) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8292\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L, "Namespace declarations cannot be nested");
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8293\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8355\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! with_bracket) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8297\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! compiler_globals.current_namespace) {
      goto _L;
    } else {
      goto _L___0;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8314\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _L___0: /* CIL Label */ 
    if (with_bracket) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8312\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! compiler_globals.has_bracketed_namespaces) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8310\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L: /* CIL Label */ 
        if ((compiler_globals.active_op_array)->last > 0U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8306\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          num = (int )(compiler_globals.active_op_array)->last;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8307\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8302\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (num > 0) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "8301\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int )((compiler_globals.active_op_array)->opcodes + (num - 1))->opcode == 101) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8298\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "8300\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )((compiler_globals.active_op_array)->opcodes + (num - 1))->opcode == 105) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "8299\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  break;
                }
              }
            } else {
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8303\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            num --;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8308\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (num > 0) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8304\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_error(1 << 6L,
                       "Namespace declaration statement has to be the very first statement in the script");
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "8305\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8309\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8311\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8313\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8356\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.in_namespace = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8357\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (with_bracket) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8315\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.has_bracketed_namespaces = (unsigned char)1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8316\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8358\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (name) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8331\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_str_tolower_dup((char const   *)name->u.constant.value.str.val,
                               (unsigned int )name->u.constant.value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8332\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    lcname = (char *)tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8333\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )name->u.constant.value.str.len == sizeof("self") - 1U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8318\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = memcmp((void const   *)lcname, (void const   *)"self",
                       sizeof("self") - 1U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8319\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___0) {
        goto _L___1;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8317\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L, "Cannot use \'%s\' as namespace name",
                   name->u.constant.value.str.val);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8325\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L___1: /* CIL Label */ 
      if ((unsigned int )name->u.constant.value.str.len == sizeof("parent") - 1U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8322\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = memcmp((void const   *)lcname, (void const   *)"parent",
                         sizeof("parent") - 1U);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8323\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8320\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8321\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L, "Cannot use \'%s\' as namespace name",
                     name->u.constant.value.str.val);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8324\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8334\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)lcname);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8335\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.current_namespace) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8326\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zval_dtor(compiler_globals.current_namespace);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8330\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8327\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___2 = _emalloc(sizeof(zval_gc_info ));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8328\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        compiler_globals.current_namespace = (zval *)tmp___2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8329\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zval_gc_info *)compiler_globals.current_namespace)->u.buffered = (gc_root_buffer *)((void *)0);
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8336\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(compiler_globals.current_namespace) = (struct _zval_struct )name->u.constant;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8345\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.current_namespace) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8341\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zval_dtor(compiler_globals.current_namespace);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8342\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8339\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((gc_root_buffer *)((unsigned long )((zval_gc_info *)compiler_globals.current_namespace)->u.buffered & 4294967292UL)) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8337\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          gc_remove_zval_from_buffer(compiler_globals.current_namespace);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8338\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8340\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _efree((void *)compiler_globals.current_namespace);
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8343\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      compiler_globals.current_namespace = (zval *)((void *)0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8344\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8359\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.current_import) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8346\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_hash_destroy(compiler_globals.current_import);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8347\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)compiler_globals.current_import);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8348\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.current_import = (HashTable *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8349\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8360\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.doc_comment) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8350\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)compiler_globals.doc_comment);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8351\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment = (char *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8352\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment_len = 0U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8353\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8361\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_begin_namespace(znode const   *name , zend_bool with_bracket ) ;
void zend_do_build_full_name(znode *result , znode *prefix , znode *name ,
                             int is_class_member ) ;
void zend_do_build_namespace_name(znode *result , znode *prefix , znode *name ) 
{ znode tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8282\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (prefix) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8276\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *result = *prefix;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8277\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )result->u.constant.type == 6) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8274\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (result->u.constant.value.str.len == 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8272\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (compiler_globals.current_namespace) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8266\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _zval_dtor(& result->u.constant);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8267\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp.op_type = 1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8268\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp.u.constant = *(compiler_globals.current_namespace);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8269\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _zval_copy_ctor(& tmp.u.constant);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8270\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_do_build_namespace_name(result, (znode *)((void *)0), & tmp);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8271\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8273\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8275\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8278\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = 1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8279\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->u.constant.type = (unsigned char)6;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8280\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->u.constant.value.str.val = (char *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8281\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->u.constant.value.str.len = 0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8283\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_build_full_name((znode *)((void *)0), result, name, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8284\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
struct _zend_compiled_variable {
   char const   *name ;
   int name_len ;
   ulong hash_value ;
};
char const __attribute__((__visibility__("default")))  *zend_get_compiled_variable_name(zend_op_array const   *op_array ,
                                                                                        zend_uint var ,
                                                                                        int *name_len ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8264\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (name_len) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8262\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *name_len = (op_array->vars + var)->name_len;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8263\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8265\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((char const __attribute__((__visibility__("default")))  *)(op_array->vars + var)->name);
}
}
char const __attribute__((__visibility__("default")))  *zend_get_compiled_variable_name(zend_op_array const   *op_array ,
                                                                                        zend_uint var ,
                                                                                        int *name_len ) ;
int zend_get_class_fetch_type(char const   *class_name , uint class_name_len ) 
{ int tmp ;
  int tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8261\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_name_len == sizeof("self") - 1U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8249\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = memcmp((void const   *)class_name, (void const   *)"self",
                     sizeof("self") - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8250\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___1) {
      goto _L___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8248\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (1);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8260\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _L___0: /* CIL Label */ 
    if (class_name_len == sizeof("parent") - 1U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8252\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = memcmp((void const   *)class_name, (void const   *)"parent",
                       sizeof("parent") - 1U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8253\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___0) {
        goto _L;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8251\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (2);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8259\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L: /* CIL Label */ 
      if (class_name_len == sizeof("static") - 1U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8256\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = memcmp((void const   *)class_name, (void const   *)"static",
                     sizeof("static") - 1U);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8257\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8254\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (0);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "8255\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (7);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8258\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (0);
      }
    }
  }
}
}
int zend_get_class_fetch_type(char const   *class_name , uint class_name_len ) ;
typedef union _zend_function zend_function;
extern void __attribute__((__visibility__("default")))  zend_function_dtor(zend_function *function ) ;
extern int __attribute__((__visibility__("default")))  _zend_hash_init_ex(HashTable *ht ,
                                                                          uint nSize ,
                                                                          ulong (*pHashFunction)(char const   *arKey ,
                                                                                                 uint nKeyLength ) ,
                                                                          void (*pDestructor)(void *pDest ) ,
                                                                          zend_bool persistent ,
                                                                          zend_bool bApplyProtection ) ;
struct _zend_property_info;
typedef struct _zend_property_info zend_property_info;
static void zend_destroy_property_info(zend_property_info *property_info ) ;
static void zend_destroy_property_info_internal(zend_property_info *property_info ) ;
extern void __attribute__((__visibility__("default")))  _zval_internal_ptr_dtor(zval **zvalue ) ;
void __attribute__((__visibility__("default")))  zend_initialize_class_data(zend_class_entry *ce ,
                                                                            zend_bool nullify_handlers ) 
{ zend_bool persistent_hashes ;
  int tmp ;
  void (*zval_ptr_dtor_func)(void *pDest ) ;
  void (*tmp___0)(void * ) ;
  void (*tmp___1)(zend_property_info *property_info ) ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8231\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )ce->type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8190\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = 1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8191\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = 0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8232\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  persistent_hashes = (zend_bool )tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8233\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (persistent_hashes) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8192\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = (void (*)(void * ))(& _zval_internal_ptr_dtor);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8193\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = (void (*)(void * ))(& _zval_ptr_dtor);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8234\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zval_ptr_dtor_func = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8235\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce->refcount = 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8236\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce->ce_flags = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8237\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce->default_properties_table = (zval **)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8238\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce->default_static_members_table = (zval **)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8239\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (persistent_hashes) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8194\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = & zend_destroy_property_info_internal;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8195\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = & zend_destroy_property_info;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8240\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zend_hash_init_ex(& ce->properties_info, 0U,
                     (ulong (*)(char const   *arKey , uint nKeyLength ))((void *)0),
                     (void (*)(void *pDest ))tmp___1, persistent_hashes,
                     (unsigned char)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8241\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zend_hash_init_ex(& ce->constants_table, 0U,
                     (ulong (*)(char const   *arKey , uint nKeyLength ))((void *)0),
                     zval_ptr_dtor_func, persistent_hashes, (unsigned char)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8242\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zend_hash_init_ex(& ce->function_table, 0U,
                     (ulong (*)(char const   *arKey , uint nKeyLength ))((void *)0),
                     (void (*)(void * ))(& zend_function_dtor),
                     persistent_hashes, (unsigned char)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8243\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )ce->type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8196\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->static_members_table = (zval **)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8197\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->static_members_table = ce->default_static_members_table;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8198\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->info.user.doc_comment = (char const   *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8199\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->info.user.doc_comment_len = 0U;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8244\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce->default_properties_count = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8245\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce->default_static_members_count = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8246\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (nullify_handlers) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8203\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->constructor = (union _zend_function *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8204\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->destructor = (union _zend_function *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8205\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->clone = (union _zend_function *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8206\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__get = (union _zend_function *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8207\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__set = (union _zend_function *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8208\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__unset = (union _zend_function *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8209\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__isset = (union _zend_function *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8210\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__call = (union _zend_function *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8211\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__callstatic = (union _zend_function *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8212\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__tostring = (union _zend_function *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8213\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->create_object = (zend_object_value (*)(zend_class_entry *class_type ))((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8214\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->get_iterator = (zend_object_iterator *(*)(zend_class_entry *ce ,
                                                  zval *object , int by_ref ))((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8215\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->iterator_funcs.funcs = (zend_object_iterator_funcs *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8216\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->interface_gets_implemented = (int (*)(zend_class_entry *iface ,
                                              zend_class_entry *class_type ))((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8217\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->get_static_method = (union _zend_function *(*)(zend_class_entry *ce ,
                                                       char *method ,
                                                       int method_len ))((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8218\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->parent = (struct _zend_class_entry *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8219\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->num_interfaces = 0U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8220\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->interfaces = (zend_class_entry **)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8221\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->num_traits = 0U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8222\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->traits = (zend_class_entry **)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8223\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->trait_aliases = (zend_trait_alias **)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8224\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->trait_precedences = (zend_trait_precedence **)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8225\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->serialize = (int (*)(zval *object , unsigned char **buffer ,
                             zend_uint *buf_len , zend_serialize_data *data ))((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8226\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->unserialize = (int (*)(zval **object , zend_class_entry *ce ,
                               unsigned char const   *buf , zend_uint buf_len ,
                               zend_unserialize_data *data ))((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8227\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->serialize_func = (union _zend_function *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8228\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->unserialize_func = (union _zend_function *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8229\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )ce->type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8200\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->info.internal.module = (struct _zend_module_entry *)((void *)0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8201\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->info.internal.builtin_functions = (struct _zend_function_entry  const  *)((void *)0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8202\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8230\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8247\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void __attribute__((__visibility__("default")))  zend_initialize_class_data(zend_class_entry *ce ,
                                                                            zend_bool nullify_handlers ) ;
struct _zend_php_scanner_globals;
typedef struct _zend_php_scanner_globals zend_php_scanner_globals;
extern zend_php_scanner_globals __attribute__((__visibility__("default")))  language_scanner_globals ;
struct _zend_file_handle;
typedef struct _zend_file_handle zend_file_handle;
struct _zend_php_scanner_globals {
   zend_file_handle *yy_in ;
   zend_file_handle *yy_out ;
   unsigned int yy_leng ;
   unsigned char *yy_start ;
   unsigned char *yy_text ;
   unsigned char *yy_cursor ;
   unsigned char *yy_marker ;
   unsigned char *yy_limit ;
   int yy_state ;
   zend_stack state_stack ;
   unsigned char *script_org ;
   size_t script_org_size ;
   unsigned char *script_filtered ;
   size_t script_filtered_size ;
   size_t (*input_filter)(unsigned char **str , size_t *str_length ,
                          unsigned char const   *buf , size_t length ) ;
   size_t (*output_filter)(unsigned char **str , size_t *str_length ,
                           unsigned char const   *buf , size_t length ) ;
   zend_encoding const   *script_encoding ;
};
extern int __attribute__((__visibility__("default")))  lex_scan(zval *zendlval ) ;
int zendlex(znode *zendlval ) 
{ int retval ;
  int __attribute__((__visibility__("default")))  tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8182\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.increment_lineno) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8169\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.zend_lineno) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8170\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.increment_lineno = (unsigned char)0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8171\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8183\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  again: 
  zendlval->u.constant.type = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8184\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = lex_scan(& zendlval->u.constant);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8185\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  retval = (int )tmp;
  switch (retval) {
  case 370: 
  case 371: 
  case 372: 
  case 375: 
  goto again;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8177\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 374: 
  if ((int )*(language_scanner_globals.yy_text + (language_scanner_globals.yy_leng - 1U)) != 62) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8172\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.increment_lineno = (unsigned char)1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8173\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8178\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.has_bracketed_namespaces) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8175\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! compiler_globals.in_namespace) {
      goto again;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8174\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8176\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8179\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  retval = ';';
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8180\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 373: 
  retval = 316;
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8181\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 377: 
  _efree((void *)zendlval->u.constant.value.str.val);
  break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8186\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->u.constant.refcount__gc = 1U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8187\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->u.constant.is_ref__gc = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8188\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zendlval->op_type = 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8189\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (retval);
}
}
int zendlex(znode *zendlval ) ;
struct _zend_auto_global;
typedef struct _zend_auto_global zend_auto_global;
static int zend_auto_global_init(zend_auto_global *auto_global ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_apply(HashTable *ht ,
                                                                        int (*apply_func)(void *pDest ) ) ;
void __attribute__((__visibility__("default")))  zend_activate_auto_globals(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8167\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_hash_apply(compiler_globals.auto_globals,
                  (int (*)(void *pDest ))(& zend_auto_global_init));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8168\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void __attribute__((__visibility__("default")))  zend_activate_auto_globals(void) ;
struct _zend_auto_global {
   char const   *name ;
   uint name_len ;
   zend_bool (*auto_global_callback)(char const   *name , uint name_len ) ;
   zend_bool jit ;
   zend_bool armed ;
};
static int zend_auto_global_init(zend_auto_global *auto_global ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8165\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (auto_global->jit) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8161\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    auto_global->armed = (unsigned char)1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8164\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (auto_global->auto_global_callback) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8162\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      auto_global->armed = (*(auto_global->auto_global_callback))(auto_global->name,
                                                                  auto_global->name_len);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8163\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      auto_global->armed = (unsigned char)0;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8166\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (0);
}
}
extern char const __attribute__((__visibility__("default")))  *(*zend_new_interned_string)(char const   *str ,
                                                                                           int len ,
                                                                                           int free_src ) ;
int __attribute__((__visibility__("default")))  zend_register_auto_global(char const   *name ,
                                                                          uint name_len ,
                                                                          zend_bool jit ,
                                                                          zend_bool (*auto_global_callback)(char const   *name ,
                                                                                                            uint name_len ) ) 
{ zend_auto_global auto_global ;
  char const __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8154\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = (*zend_new_interned_string)((char const   *)((char *)name),
                                    (int )(name_len + 1U), 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8155\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  auto_global.name = (char const   *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8156\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  auto_global.name_len = name_len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8157\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  auto_global.auto_global_callback = auto_global_callback;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8158\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  auto_global.jit = jit;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8159\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = _zend_hash_add_or_update(compiler_globals.auto_globals, name,
                                     name_len + 1U, (void *)(& auto_global),
                                     sizeof(zend_auto_global ),
                                     (void **)((void *)0), 1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8160\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___0);
}
}
int __attribute__((__visibility__("default")))  zend_register_auto_global(char const   *name ,
                                                                          uint name_len ,
                                                                          zend_bool jit ,
                                                                          zend_bool (*auto_global_callback)(char const   *name ,
                                                                                                            uint name_len ) ) ;
zend_bool __attribute__((__visibility__("default")))  zend_is_auto_global_quick(char const   *name ,
                                                                                uint name_len ,
                                                                                ulong hashval ) ;
zend_bool __attribute__((__visibility__("default")))  zend_is_auto_global(char const   *name ,
                                                                          uint name_len ) 
{ zend_bool __attribute__((__visibility__("default")))  tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8152\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_is_auto_global_quick(name, name_len, 0UL);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8153\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
zend_bool __attribute__((__visibility__("default")))  zend_is_auto_global(char const   *name ,
                                                                          uint name_len ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_quick_find(HashTable const   *ht ,
                                                                            char const   *arKey ,
                                                                            uint nKeyLength ,
                                                                            ulong h ,
                                                                            void **pData ) ;
extern ulong __attribute__((__visibility__("default")))  zend_hash_func(char const   *arKey ,
                                                                        uint nKeyLength ) ;
zend_bool __attribute__((__visibility__("default")))  zend_is_auto_global_quick(char const   *name ,
                                                                                uint name_len ,
                                                                                ulong hashval ) 
{ zend_auto_global *auto_global ;
  ulong hash ;
  ulong __attribute__((__visibility__("default")))  tmp ;
  ulong tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8147\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (hashval) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8139\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = hashval;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8140\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_hash_func(name, name_len + 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8141\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = (unsigned long )tmp;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8148\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8149\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_hash_quick_find((HashTable const   *)compiler_globals.auto_globals,
                                 name, name_len + 1U, hash,
                                 (void **)(& auto_global));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8150\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8144\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (auto_global->armed) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8142\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      auto_global->armed = (*(auto_global->auto_global_callback))(auto_global->name,
                                                                  auto_global->name_len);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8143\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8145\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char __attribute__((__visibility__("default")))  )1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8146\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8151\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned char __attribute__((__visibility__("default")))  )0);
}
}
void zend_do_ticks(void) 
{ zend_op *opline ;
  zend_op *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8132\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8133\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8134\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)105;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8135\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8136\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8137\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = (unsigned long )compiler_globals.declarables.ticks.value.lval;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8138\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_ticks(void) ;
void zend_do_extended_fcall_end(void) 
{ zend_op *opline ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8126\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! (compiler_globals.compiler_options & 1U)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8124\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8125\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8127\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8128\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)103;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8129\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8130\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8131\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_extended_fcall_end(void) ;
void zend_do_extended_fcall_begin(void) 
{ zend_op *opline ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8118\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! (compiler_globals.compiler_options & 1U)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8116\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8117\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8119\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8120\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)102;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8121\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8122\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8123\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_extended_fcall_begin(void) ;
void zend_do_extended_info(void) 
{ zend_op *opline ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8110\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! (compiler_globals.compiler_options & 1U)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8108\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8109\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8111\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8112\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)101;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8113\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8114\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8115\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_extended_info(void) ;
struct _zend_literal {
   zval constant ;
   zend_ulong hash_value ;
   zend_uint cache_slot ;
};
extern int get_next_op_number(zend_op_array *op_array ) ;
void zend_do_qm_false(znode *result , znode const   *false_value ,
                      znode const   *qm_token , znode const   *colon_token ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8097\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8098\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8099\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8071\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )qm_token->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8072\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (qm_token->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8068\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & qm_token->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8069\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8070\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result = (union _znode_op )qm_token->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8100\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (qm_token->op_type == (int const   )(1 << 1)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8083\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (false_value->op_type == (int const   )(1 << 2)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8073\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((compiler_globals.active_op_array)->opcodes + (colon_token->u.op.opline_num - 1U))->opcode = (unsigned char)157;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8074\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((compiler_globals.active_op_array)->opcodes + (colon_token->u.op.opline_num - 1U))->result_type = (unsigned char )(1 << 2);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8075\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char)157;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8076\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result_type = (unsigned char )(1 << 2);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8082\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (false_value->op_type == (int const   )(1 << 4)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8077\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->opcodes + (colon_token->u.op.opline_num - 1U))->opcode = (unsigned char)157;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8078\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->opcodes + (colon_token->u.op.opline_num - 1U))->result_type = (unsigned char )(1 << 2);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8079\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->opcode = (unsigned char)157;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8080\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->result_type = (unsigned char )(1 << 2);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "8081\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->opcode = (unsigned char)22;
      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8084\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)157;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8101\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8088\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )false_value->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8089\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (false_value->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8085\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & false_value->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8086\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8087\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )false_value->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8102\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8103\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8104\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + colon_token->u.op.opline_num)->op1.opline_num = (unsigned int )tmp___2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8105\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8093\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8094\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8090\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8091\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8092\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8106\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8095\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) --;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8096\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8107\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_qm_false(znode *result , znode const   *false_value ,
                      znode const   *qm_token , znode const   *colon_token ) ;
static zend_uint get_temporary_variable(zend_op_array *op_array ) ;
void zend_do_qm_true(znode const   *true_value , znode *qm_token ,
                     znode *colon_token ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8052\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8053\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8054\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8055\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + qm_token->u.op.opline_num)->op2.opline_num = (unsigned int )(tmp___0 + 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8056\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (true_value->op_type == (int const   )(1 << 2)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8035\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)157;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8036\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )(1 << 2);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8041\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (true_value->op_type == (int const   )(1 << 4)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8037\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char)157;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8038\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result_type = (unsigned char )(1 << 2);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8039\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char)22;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8040\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result_type = (unsigned char )(1 << 1);
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8057\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8058\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8045\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )true_value->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8046\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (true_value->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8042\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & true_value->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8043\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8044\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )true_value->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8059\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8060\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8050\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    qm_token->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8051\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (qm_token->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8047\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      qm_token->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8048\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      qm_token->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8049\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      qm_token->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8061\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8062\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  colon_token->u.op.opline_num = (unsigned int )tmp___2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8063\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8064\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)42;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8065\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8066\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8067\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_qm_true(znode const   *true_value , znode *qm_token ,
                     znode *colon_token ) ;
void zend_do_begin_qm_op(znode const   *cond , znode *qm_token ) 
{ int jmpz_op_number ;
  int tmp ;
  zend_op *opline ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8025\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8026\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  jmpz_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8027\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8028\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)43;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8029\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8016\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )cond->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8017\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (cond->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8013\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & cond->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8014\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8015\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )cond->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8030\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8031\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2.opline_num = (unsigned int )jmpz_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8032\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8021\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    qm_token->op_type = (int )opline->op2_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8022\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (qm_token->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8018\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      qm_token->u.constant = ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8019\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      qm_token->u.op = opline->op2;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "8020\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      qm_token->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8033\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8023\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) ++;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8024\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8034\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_begin_qm_op(znode const   *cond , znode *qm_token ) ;
void zend_do_jmp_set_else(znode *result , znode const   *false_value ,
                          znode const   *jmp_token , znode const   *colon_token ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8001\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8002\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8003\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7975\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )colon_token->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7976\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (colon_token->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7972\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & colon_token->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7973\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7974\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result = (union _znode_op )colon_token->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8004\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (colon_token->op_type == (int const   )(1 << 1)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7987\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (false_value->op_type == (int const   )(1 << 2)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7977\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((compiler_globals.active_op_array)->opcodes + jmp_token->u.op.opline_num)->opcode = (unsigned char)158;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7978\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((compiler_globals.active_op_array)->opcodes + jmp_token->u.op.opline_num)->result_type = (unsigned char )(1 << 2);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7979\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char)157;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7980\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result_type = (unsigned char )(1 << 2);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7986\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (false_value->op_type == (int const   )(1 << 4)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7981\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->opcodes + jmp_token->u.op.opline_num)->opcode = (unsigned char)158;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7982\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->opcodes + jmp_token->u.op.opline_num)->result_type = (unsigned char )(1 << 2);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7983\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->opcode = (unsigned char)157;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7984\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->result_type = (unsigned char )(1 << 2);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7985\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->opcode = (unsigned char)22;
      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7988\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)157;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8005\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = 0UL;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8006\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7992\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )false_value->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7993\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (false_value->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7989\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & false_value->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7990\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7991\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )false_value->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8007\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8008\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7997\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7998\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7994\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7995\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7996\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8009\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8010\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + jmp_token->u.op.opline_num)->op2.opline_num = (unsigned int )tmp___2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8011\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7999\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) --;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "8000\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8012\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_jmp_set_else(znode *result , znode const   *false_value ,
                          znode const   *jmp_token , znode const   *colon_token ) ;
void zend_do_jmp_set(znode const   *value , znode *jmp_token ,
                     znode *colon_token ) 
{ int op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7960\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7961\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7962\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7963\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7964\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (value->op_type == (int const   )(1 << 2)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7941\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)158;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7942\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )(1 << 2);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7947\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (value->op_type == (int const   )(1 << 4)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7943\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char)158;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7944\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result_type = (unsigned char )(1 << 2);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7945\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char)152;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7946\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result_type = (unsigned char )(1 << 1);
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7965\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7966\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7951\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )value->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7952\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (value->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7948\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & value->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7949\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7950\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )value->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7967\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7968\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7956\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    colon_token->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7957\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (colon_token->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7953\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      colon_token->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7954\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      colon_token->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7955\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      colon_token->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7969\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  jmp_token->u.op.opline_num = (unsigned int )op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7970\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7958\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) ++;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7959\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7971\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_jmp_set(znode const   *value , znode *jmp_token ,
                     znode *colon_token ) ;
void zend_do_end_silence(znode const   *strudel_token ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7935\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7936\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7937\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)58;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7938\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7933\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )strudel_token->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7934\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (strudel_token->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7930\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & strudel_token->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7931\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7932\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )strudel_token->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7939\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7940\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_end_silence(znode const   *strudel_token ) ;
void zend_do_begin_silence(znode *strudel_token ) 
{ zend_op *opline ;
  zend_op *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7921\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7922\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7923\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)57;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7924\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7925\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7926\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7927\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7928\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7919\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    strudel_token->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7920\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (strudel_token->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7916\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      strudel_token->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7917\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      strudel_token->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7918\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      strudel_token->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7929\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_begin_silence(znode *strudel_token ) ;
void zend_do_exit(znode *result , znode const   *message ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7907\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7908\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7909\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)79;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7910\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7905\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )message->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7906\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (message->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7902\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & message->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7903\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7904\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )message->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7911\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7912\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->op_type = 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7913\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->u.constant.type = (unsigned char)3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7914\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->u.constant.value.lval = 1L;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7915\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_exit(znode *result , znode const   *message ) ;
struct _zend_declarables;
struct _zend_stack;
extern int __attribute__((__visibility__("default")))  zend_stack_top(zend_stack const   *stack ,
                                                                      void **element ) ;
void zend_do_declare_end(znode const   *declare_token ) 
{ zend_declarables *declarables ;
  int tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7897\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.declare_stack),
                 (void **)(& declarables));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7898\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7899\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.declarables.ticks.value.lval) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7893\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = 1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7894\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = 0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7900\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (((zend_uint const   )tmp - declare_token->u.op.opline_num) - (zend_uint const   )tmp___0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7895\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.declarables = *declarables;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7896\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7901\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_declare_end(znode const   *declare_token ) ;
extern void __attribute__((__visibility__("default")))  convert_to_long(zval *op ) ;
extern void __attribute__((__visibility__("default")))  zend_multibyte_yyinput_again(size_t (*old_input_filter)(unsigned char **str ,
                                                                                                                size_t *str_length ,
                                                                                                                unsigned char const   *buf ,
                                                                                                                size_t length ) ,
                                                                                     zend_encoding const   *old_encoding ) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_set_filter(zend_encoding const   *onetime_encoding ) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_fetch_encoding(char const   *name ) ;
extern void __attribute__((__visibility__("default")))  _convert_to_string(zval *op ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_strcasecmp(char const   *s1 ,
                                                                              uint len1 ,
                                                                              char const   *s2 ,
                                                                              uint len2 ) ;
void zend_do_declare_stmt(znode *var , znode *val ) 
{ int num ;
  zend_encoding const   *new_encoding ;
  zend_encoding const   *old_encoding ;
  size_t (*old_input_filter)(unsigned char **str , size_t *str_length ,
                             unsigned char const   *buf , size_t length ) ;
  zend_encoding const __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7889\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_binary_strcasecmp((char const   *)var->u.constant.value.str.val,
                                   (unsigned int )var->u.constant.value.str.len,
                                   "ticks", sizeof("ticks") - 1U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7890\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7885\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = zend_binary_strcasecmp((char const   *)var->u.constant.value.str.val,
                                     (unsigned int )var->u.constant.value.str.len,
                                     "encoding", sizeof("encoding") - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7886\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7848\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 7L, "Unsupported declare \'%s\'",
                 var->u.constant.value.str.val);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7849\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zval_dtor(& val->u.constant);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7879\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (((int )val->u.constant.type & 0x00f) == 8) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7850\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L, "Cannot use constants as encoding");
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7851\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7880\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      num = (int )(compiler_globals.active_op_array)->last;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7881\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7856\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (num > 0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7855\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )((compiler_globals.active_op_array)->opcodes + (num - 1))->opcode == 101) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7852\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7854\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )((compiler_globals.active_op_array)->opcodes + (num - 1))->opcode == 105) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7853\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            } else {
              break;
            }
          }
        } else {
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7857\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        num --;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7882\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (num > 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7858\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L,
                   "Encoding declaration pragma must be the very first statement in the script");
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7859\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7883\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (compiler_globals.multibyte) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7873\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        compiler_globals.encoding_declared = (unsigned char)1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7874\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )val->u.constant.type != 6) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7860\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _convert_to_string(& val->u.constant);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7861\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7875\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = zend_multibyte_fetch_encoding((char const   *)val->u.constant.value.str.val);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7876\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        new_encoding = (zend_encoding const   *)tmp;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7877\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (! new_encoding) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7862\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 7L, "Unsupported encoding [%s]",
                     val->u.constant.value.str.val);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7869\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          old_input_filter = language_scanner_globals.input_filter;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7870\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          old_encoding = language_scanner_globals.script_encoding;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7871\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_multibyte_set_filter(new_encoding);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7872\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )old_input_filter != (unsigned int )language_scanner_globals.input_filter) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7863\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_multibyte_yyinput_again(old_input_filter, old_encoding);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7868\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (old_input_filter) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7866\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((unsigned int )new_encoding != (unsigned int )old_encoding) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7864\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                zend_multibyte_yyinput_again(old_input_filter, old_encoding);
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7865\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7867\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7878\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 7L,
                   "declare(encoding=...) ignored because Zend multibyte feature is turned off by settings");
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7884\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zval_dtor(& val->u.constant);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7887\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    convert_to_long(& val->u.constant);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7888\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.declarables.ticks = val->u.constant;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7891\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_dtor(& var->u.constant);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7892\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_declare_stmt(znode *var , znode *val ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_push(zend_stack *stack ,
                                                                       void const   *element ,
                                                                       int size ) ;
void zend_do_declare_begin(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7846\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.declare_stack,
                  (void const   *)(& compiler_globals.declarables),
                  (int )sizeof(zend_declarables ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7847\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_declare_begin(void) ;
extern int __attribute__((__visibility__("default")))  zend_stack_del_top(zend_stack *stack ) ;
static int generate_free_foreach_copy(zend_op const   *foreach_copy ) ;
__inline static void do_end_loop(int cont_addr , int has_loop_var ) ;
void zend_do_foreach_end(znode const   *foreach_token , znode const   *as_token ) 
{ zend_op *container_ptr ;
  zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7830\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7831\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7832\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)42;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7833\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1.opline_num = (unsigned int )as_token->u.op.opline_num;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7834\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7835\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7836\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7837\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + foreach_token->u.op.opline_num)->op2.opline_num = (unsigned int )tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7838\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7839\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + as_token->u.op.opline_num)->op2.opline_num = (unsigned int )tmp___1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7840\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  do_end_loop((int )as_token->u.op.opline_num, 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7841\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.foreach_copy_stack),
                 (void **)(& container_ptr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7842\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  generate_free_foreach_copy((zend_op const   *)container_ptr);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7843\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_del_top(& compiler_globals.foreach_copy_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7844\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7828\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) --;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7829\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7845\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_foreach_end(znode const   *foreach_token , znode const   *as_token ) ;
__inline static void do_begin_loop(void) ;
void zend_do_free(znode *op1 ) ;
void zend_do_assign(znode *result , znode *variable , znode *value ) ;
void zend_do_assign_ref(znode *result , znode const   *lvar ,
                        znode const   *rvar ) ;
void zend_do_end_variable_parse(znode *variable , int type , int arg_offset ) ;
union _znode_op;
extern  __attribute__((__nothrow__)) void *memset(void *__s , int __c ,
                                                  size_t __n )  __attribute__((__nonnull__(1))) ;
void zend_do_foreach_cont(znode *foreach_token ,
                          znode const   *open_brackets_token ,
                          znode const   *as_token , znode *value , znode *key ) 
{ zend_op *opline ;
  znode dummy ;
  znode value_node ;
  zend_bool assign_by_ref ;
  znode *tmp ;
  zend_op *foreach_copy ;
  zend_op *fetch ;
  zend_op *end ;
  zend_uchar tmp___0 ;
  zend_uchar tmp___1 ;
  znode key_node ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7817\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  assign_by_ref = (zend_bool )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7818\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = (compiler_globals.active_op_array)->opcodes + as_token->u.op.opline_num;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7819\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (key->op_type != 1 << 3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7754\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = key;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7755\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    key = value;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7756\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    value = tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7757\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value |= 2UL;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7758\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7820\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (key->op_type != 1 << 3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7761\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (key->EA & (unsigned int )(1 << 5)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7759\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Key element cannot be a reference");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7760\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7762\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7821\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (value->EA & (unsigned int )(1 << 5)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7765\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    assign_by_ref = (unsigned char)1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7766\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! (opline - 1)->extended_value) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7763\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L,
                 "Cannot create references to elements of a temporary array expression");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7764\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7767\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value |= 1UL;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7768\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->opcodes + foreach_token->u.op.opline_num)->extended_value |= (unsigned long )(1 << 1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7788\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    fetch = (compiler_globals.active_op_array)->opcodes + foreach_token->u.op.opline_num;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7789\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    end = (compiler_globals.active_op_array)->opcodes + open_brackets_token->u.op.opline_num;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7790\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    fetch->extended_value = 0UL;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7791\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7784\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )fetch != (unsigned int )end) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7769\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7785\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      fetch --;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7786\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )fetch->opcode == 84) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7772\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )fetch->op2_type == 1 << 3) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7770\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L, "Cannot use [] for reading");
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7771\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7773\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7787\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )fetch->opcode == 156) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7774\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        fetch->opcode = (unsigned char)0;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7775\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        memset((void *)(& fetch->result), 0, sizeof(fetch->result));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7776\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        memset((void *)(& fetch->op1), 0, sizeof(fetch->op1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7777\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        memset((void *)(& fetch->op2), 0, sizeof(fetch->op2));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7778\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = (unsigned char )(1 << 3);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7779\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        fetch->op2_type = tmp___1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7780\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = tmp___1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7781\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        fetch->op1_type = tmp___0;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7782\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        fetch->result_type = tmp___0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7783\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        fetch->opcode = (unsigned char )((int )fetch->opcode - 3);
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7792\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_stack_top((zend_stack const   *)(& compiler_globals.foreach_copy_stack),
                   (void **)(& foreach_copy));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7793\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    foreach_copy->op1_type = (unsigned char )(1 << 3);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7822\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7797\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    value_node.op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7798\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (value_node.op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7794\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      value_node.u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7795\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      value_node.u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7796\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      value_node.EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7823\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (assign_by_ref) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7799\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_end_variable_parse(value, 1, 0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7800\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_assign_ref((znode *)((void *)0), (znode const   *)value,
                       (znode const   *)(& value_node));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7801\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_assign(& dummy, value, & value_node);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7802\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_free(& dummy);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7824\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (key->op_type != 1 << 3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7808\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline = (compiler_globals.active_op_array)->opcodes + (as_token->u.op.opline_num + 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7809\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )(1 << 1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7810\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result.opline_num = get_temporary_variable(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7811\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7806\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      key_node.op_type = (int )opline->result_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7807\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (key_node.op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7803\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        key_node.u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7804\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        key_node.u.op = opline->result;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7805\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        key_node.EA = 0U;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7812\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_assign(& dummy, key, & key_node);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7813\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_free(& dummy);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7814\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7825\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  do_begin_loop();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7826\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7815\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) ++;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7816\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7827\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_foreach_cont(znode *foreach_token ,
                          znode const   *open_brackets_token ,
                          znode const   *as_token , znode *value , znode *key ) ;
__inline static zend_bool zend_is_function_or_method_call(znode const   *variable ) ;
void zend_do_foreach_begin(znode *foreach_token , znode *open_brackets_token ,
                           znode *array , znode *as_token , int variable ) 
{ zend_op *opline ;
  zend_bool is_variable ;
  zend_bool push_container ;
  zend_op dummy_opline ;
  zend_bool tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;
  int tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7725\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  push_container = (zend_bool )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7726\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (variable) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7701\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_is_function_or_method_call((znode const   *)array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7702\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7692\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      is_variable = (unsigned char)0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7693\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      is_variable = (unsigned char)1;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7703\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = get_next_op_number(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7704\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    open_brackets_token->u.op.opline_num = (unsigned int )tmp___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7705\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_end_variable_parse(array, 1, 0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7706\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((compiler_globals.active_op_array)->last > 0U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7699\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )((compiler_globals.active_op_array)->opcodes + ((compiler_globals.active_op_array)->last - 1U))->opcode == 85) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7697\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )((compiler_globals.active_op_array)->opcodes + ((compiler_globals.active_op_array)->last - 1U))->op1_type == 1 << 2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7694\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((compiler_globals.active_op_array)->opcodes + ((compiler_globals.active_op_array)->last - 1U))->extended_value |= 134217728UL;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7695\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          push_container = (unsigned char)1;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7696\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7698\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7700\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7707\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    is_variable = (unsigned char)0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7708\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = get_next_op_number(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7709\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    open_brackets_token->u.op.opline_num = (unsigned int )tmp___1;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7727\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7728\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  foreach_token->u.op.opline_num = (unsigned int )tmp___2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7729\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7730\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)77;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7731\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7732\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7733\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7713\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )array->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7714\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (array->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7710\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___3 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& array->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7711\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___3;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7712\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = array->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7734\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7735\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (is_variable) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7715\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value = 1UL;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7716\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value = 0UL;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7736\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7717\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    dummy_opline.result_type = opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7718\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    dummy_opline.result = opline->result;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7737\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (push_container) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7721\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7719\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      dummy_opline.op1_type = ((compiler_globals.active_op_array)->opcodes + ((compiler_globals.active_op_array)->last - 2U))->op1_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7720\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      dummy_opline.op1 = ((compiler_globals.active_op_array)->opcodes + ((compiler_globals.active_op_array)->last - 2U))->op1;
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7722\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    dummy_opline.op1_type = (unsigned char )(1 << 3);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7738\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.foreach_copy_stack,
                  (void const   *)((void *)(& dummy_opline)),
                  (int )sizeof(zend_op ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7739\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7740\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  as_token->u.op.opline_num = (unsigned int )tmp___4;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7741\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7742\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)78;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7743\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7744\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7745\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7723\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = dummy_opline.result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7724\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1 = dummy_opline.result;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7746\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = 0UL;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7747\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7748\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7749\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)137;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7750\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7751\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7752\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7753\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_foreach_begin(znode *foreach_token , znode *open_brackets_token ,
                           znode *array , znode *as_token , int variable ) ;
void zend_do_instanceof(znode *result , znode const   *expr ,
                        znode const   *class_znode , int type ) 
{ int last_op_number ;
  int tmp ;
  zend_op *opline ;
  int tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7680\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7681\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  last_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7682\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (last_op_number > 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7660\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline = (compiler_globals.active_op_array)->opcodes + (last_op_number - 1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7661\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )opline->opcode == 109) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7658\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->extended_value |= 128UL;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7659\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7662\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7683\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (expr->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7663\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "instanceof expects an object instance, constant given");
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7664\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7684\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7685\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)138;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7686\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7687\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7688\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7668\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )expr->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7669\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7665\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & expr->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7666\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7667\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )expr->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7689\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7673\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )class_znode->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7674\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (class_znode->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7670\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & class_znode->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7671\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7672\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2 = (union _znode_op )class_znode->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7690\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7678\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7679\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7675\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7676\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7677\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7691\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_instanceof(znode *result , znode const   *expr ,
                        znode const   *class_znode , int type ) ;
void zend_check_writable_variable(znode const   *variable ) ;
void zend_do_isset_or_isempty(int type , znode *result , znode *variable ) 
{ zend_op *last_op ;
  int tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7651\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_end_variable_parse(variable, 3, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7652\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_check_writable_variable((znode const   *)variable);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7653\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (variable->op_type == 1 << 4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7635\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op = get_next_op(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7636\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op->opcode = (unsigned char)114;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7637\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7633\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      last_op->op1_type = (unsigned char )variable->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7634\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (variable->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7630\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = zend_add_literal(compiler_globals.active_op_array,
                               (zval const   *)(& variable->u.constant));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7631\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        last_op->op1.constant = (unsigned int )tmp;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7632\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        last_op->op1 = variable->u.op;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7638\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op->op2_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7639\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op->result.var = get_temporary_variable(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7640\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op->extended_value = 276824064UL;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7644\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = get_next_op_number(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7645\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op = (compiler_globals.active_op_array)->opcodes + (tmp___0 - 1);
    switch ((int )last_op->opcode) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7641\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 89: 
    last_op->opcode = (unsigned char)114;
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7642\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 90: 
    last_op->opcode = (unsigned char)115;
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7643\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 91: 
    last_op->opcode = (unsigned char)148;
    break;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7654\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  last_op->result_type = (unsigned char )(1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7655\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  last_op->extended_value |= (unsigned long )type;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7656\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7649\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )last_op->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7650\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7646\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + last_op->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7647\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = last_op->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7648\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7657\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_isset_or_isempty(int type , znode *result , znode *variable ) ;
void zend_do_unset(znode const   *variable ) 
{ zend_op *last_op ;
  zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7627\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_check_writable_variable(variable);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7628\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (variable->op_type == (int const   )(1 << 4)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7612\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = get_next_op(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7613\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline = tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7614\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)74;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7615\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7610\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1_type = (unsigned char )variable->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7611\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (variable->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7607\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                   & variable->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7608\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1.constant = (unsigned int )tmp___0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7609\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1 = (union _znode_op )variable->u.op;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7616\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7617\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7618\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value = 276824064UL;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7625\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = get_next_op_number(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7626\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op = (compiler_globals.active_op_array)->opcodes + (tmp___1 - 1);
    switch ((int )last_op->opcode) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7619\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 95: 
    last_op->opcode = (unsigned char)74;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7620\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op->result_type = (unsigned char )(1 << 3);
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7621\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 96: 
    last_op->opcode = (unsigned char)75;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7622\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op->result_type = (unsigned char )(1 << 3);
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7623\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 97: 
    last_op->opcode = (unsigned char)76;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7624\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op->result_type = (unsigned char )(1 << 3);
    break;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7629\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_unset(znode const   *variable ) ;
static int lookup_cv(zend_op_array *op_array , char *name , int name_len ,
                     ulong hash ) ;
void fetch_simple_variable(znode *result , znode *varname , int bp ) ;
void zend_do_begin_variable_parse(void) ;
void fetch_simple_variable_ex(znode *result , znode *varname , int bp ,
                              zend_uchar op ) ;
void zend_do_indirect_references(znode *result , znode const   *num_references ,
                                 znode *variable ) 
{ int i ;
  char __attribute__((__visibility__("default")))  *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7600\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_end_variable_parse(variable, 0, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7601\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7602\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7590\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((long const   )i < num_references->u.constant.value.lval) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7589\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7591\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    fetch_simple_variable_ex(result, variable, 0, (unsigned char)80);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7592\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *variable = *result;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7593\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7603\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_begin_variable_parse();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7604\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fetch_simple_variable(result, variable, 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7605\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->scope) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7598\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((compiler_globals.active_op_array)->this_var == 4294967295U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7594\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = _estrndup("this", sizeof("this") - 1U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7595\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = lookup_cv(compiler_globals.active_op_array, (char *)tmp,
                          (int )(sizeof("this") - 1U), 275574653UL);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7596\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (compiler_globals.active_op_array)->this_var = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7597\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7599\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7606\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_indirect_references(znode *result , znode const   *num_references ,
                                 znode *variable ) ;
void zend_do_include_or_eval(int type , znode *result , znode const   *op1 ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7577\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_extended_fcall_begin();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7578\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7579\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7580\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)73;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7581\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7582\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7583\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7570\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )op1->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7571\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op1->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7567\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & op1->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7568\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7569\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )op1->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7584\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7585\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = (unsigned long )type;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7586\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7575\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7576\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7572\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7573\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7574\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7587\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_extended_fcall_end();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7588\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_include_or_eval(int type , znode *result , znode const   *op1 ) ;
void zend_do_cast(znode *result , znode const   *expr , int type ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7557\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7558\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7559\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)21;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7560\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7561\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7562\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7550\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )expr->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7551\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7547\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & expr->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7548\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7549\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )expr->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7563\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7564\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = (unsigned long )type;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7565\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7555\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7556\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7552\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7553\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7554\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7566\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_cast(znode *result , znode const   *expr , int type ) ;
struct bucket {
   ulong h ;
   uint nKeyLength ;
   void *pData ;
   void *pDataPtr ;
   struct bucket *pListNext ;
   struct bucket *pListLast ;
   struct bucket *pNext ;
   struct bucket *pLast ;
   char const   *arKey ;
};
void zend_do_fetch_global_variable(znode *varname ,
                                   znode const   *static_assignment ,
                                   int fetch_type ) 
{ zend_op *opline ;
  znode lval ;
  znode result ;
  int tmp ;
  ulong __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7532\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (varname->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7509\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )varname->u.constant.type != 6) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7507\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )varname->u.constant.type != 6) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7505\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _convert_to_string(& varname->u.constant);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7506\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7508\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7510\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7533\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7534\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)83;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7535\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7536\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7537\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7514\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )varname->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7515\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (varname->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7511\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = zend_add_literal(compiler_globals.active_op_array,
                             (zval const   *)(& varname->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7512\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7513\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = varname->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7538\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )opline->op1_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7523\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7522\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7519\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7516\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val - sizeof(Bucket )))->h;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7517\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___0 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                                   (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len + 1));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7518\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = (unsigned long )tmp___0;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7520\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                                 (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7521\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = (unsigned long )tmp___0;
      }
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7524\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7539\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7540\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = (unsigned long )fetch_type;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7541\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7528\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result.op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7529\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result.op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7525\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result.u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7526\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result.u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7527\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result.EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7542\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (varname->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7530\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_copy_ctor(& varname->u.constant);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7531\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7543\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fetch_simple_variable(& lval, varname, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7544\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_assign_ref((znode *)((void *)0), (znode const   *)(& lval),
                     (znode const   *)(& result));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7545\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + ((compiler_globals.active_op_array)->last - 1U))->result_type = (unsigned char )((int )((compiler_globals.active_op_array)->opcodes + ((compiler_globals.active_op_array)->last - 1U))->result_type | (1 << 5));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7546\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_fetch_global_variable(znode *varname ,
                                   znode const   *static_assignment ,
                                   int fetch_type ) ;
void zend_do_fetch_static_variable(znode *varname ,
                                   znode const   *static_assignment ,
                                   int fetch_type ) ;
__inline static zend_bool ( __attribute__((__always_inline__)) zval_unset_isref_p)(zval *pz ) ;
__inline static zend_uint ( __attribute__((__always_inline__)) zval_set_refcount_p)(zval *pz ,
                                                                                    zend_uint rc ) ;
void zend_do_fetch_lexical_variable(znode *varname , zend_bool is_ref ) 
{ znode value ;
  int tmp ;
  int tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7495\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )varname->u.constant.value.str.len == sizeof("this") - 1U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7488\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = memcmp((void const   *)varname->u.constant.value.str.val,
                 (void const   *)"this", sizeof("this") - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7489\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp == 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7485\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Cannot use $this as lexical variable");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7486\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7487\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7490\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7496\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  value.op_type = 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7497\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  value.u.constant.type = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7498\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (is_ref) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7491\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = 0x040;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7492\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = 0x020;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7499\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  value.u.constant.type = (unsigned char )((int )value.u.constant.type | tmp___0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7500\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zval_set_refcount_p(& value.u.constant, 1U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7501\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zval_unset_isref_p(& value.u.constant);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7502\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (is_ref) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7493\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = 0x20000000;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7494\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = 0x50000000;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7503\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_fetch_static_variable(varname, (znode const   *)(& value), tmp___1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7504\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_fetch_lexical_variable(znode *varname , zend_bool is_ref ) ;
extern zval __attribute__((__visibility__("default")))  zval_used_for_init ;
void zend_do_fetch_static_variable(znode *varname ,
                                   znode const   *static_assignment ,
                                   int fetch_type ) 
{ zval *tmp ;
  zend_op *opline ;
  znode lval ;
  znode result ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  int tmp___2 ;
  ulong __attribute__((__visibility__("default")))  tmp___3 ;
  znode dummy ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7466\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7421\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = _emalloc(sizeof(zval_gc_info ));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7422\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = (zval *)tmp___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7423\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((zval_gc_info *)tmp)->u.buffered = (gc_root_buffer *)((void *)0);
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7467\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (static_assignment) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7424\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *tmp = (struct _zval_struct )static_assignment->u.constant;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7425\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *tmp = (struct _zval_struct )zval_used_for_init;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7468\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! (compiler_globals.active_op_array)->static_variables) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7428\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((compiler_globals.active_op_array)->scope) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7426\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((compiler_globals.active_op_array)->scope)->ce_flags |= 8388608U;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7427\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7429\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = _emalloc(sizeof(HashTable ));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7430\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.active_op_array)->static_variables = (HashTable *)tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7431\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zend_hash_init((compiler_globals.active_op_array)->static_variables, 2U,
                    (ulong (*)(char const   *arKey , uint nKeyLength ))((void *)0),
                    (void (*)(void * ))(& _zval_ptr_dtor), (unsigned char)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7432\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7469\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zend_hash_add_or_update((compiler_globals.active_op_array)->static_variables,
                           (char const   *)varname->u.constant.value.str.val,
                           (unsigned int )(varname->u.constant.value.str.len + 1),
                           (void *)(& tmp), sizeof(zval *),
                           (void **)((void *)0), 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7470\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (varname->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7437\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )varname->u.constant.type != 6) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7435\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )varname->u.constant.type != 6) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7433\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _convert_to_string(& varname->u.constant);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7434\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7436\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7438\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7471\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7472\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (fetch_type == 0x50000000) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7439\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)80;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7440\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)83;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7473\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7474\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7475\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7444\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )varname->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7445\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (varname->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7441\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& varname->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7442\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___2;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7443\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = varname->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7476\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )opline->op1_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7453\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7452\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7449\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7446\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val - sizeof(Bucket )))->h;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7447\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___3 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                                   (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len + 1));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7448\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = (unsigned long )tmp___3;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7450\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                                 (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7451\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = (unsigned long )tmp___3;
      }
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7454\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7477\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7478\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = 536870912UL;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7479\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7458\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result.op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7459\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result.op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7455\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result.u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7456\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result.u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7457\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result.EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7480\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (varname->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7460\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_copy_ctor(& varname->u.constant);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7461\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7481\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fetch_simple_variable(& lval, varname, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7482\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (fetch_type == 0x50000000) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7462\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_begin_variable_parse();
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7463\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_assign(& dummy, & lval, & result);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7464\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_free(& dummy);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7465\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_assign_ref((znode *)((void *)0), (znode const   *)(& lval),
                       (znode const   *)(& result));
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7483\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + ((compiler_globals.active_op_array)->last - 1U))->result_type = (unsigned char )((int )((compiler_globals.active_op_array)->opcodes + ((compiler_globals.active_op_array)->last - 1U))->result_type | (1 << 5));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7484\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  *_erealloc(void *ptr ,
                                                                   size_t size ,
                                                                   int allow_failure ) ;
void zend_add_to_list(void *result , void *item ) 
{ void **list ;
  size_t n ;
  void __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7412\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  list = (void **)*((void **)result);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7413\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  n = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7414\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7410\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (list) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7409\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (*(list + n)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7408\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7411\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    n ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7415\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _erealloc((void *)list, sizeof(void *) * (n + 2U), 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7416\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  list = (void **)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7417\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *(list + n) = item;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7418\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *(list + (n + 1U)) = (void *)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7419\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *((void **)result) = (void *)list;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7420\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_add_to_list(void *result , void *item ) ;
void zend_init_list(void *result , void *item ) 
{ void **list ;
  void __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7402\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _emalloc(sizeof(void *) * 2U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7403\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  list = (void **)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7404\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *(list + 0) = item;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7405\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *(list + 1) = (void *)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7406\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *((void **)result) = (void *)list;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7407\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_init_list(void *result , void *item ) ;
struct _zend_llist;
extern void __attribute__((__visibility__("default")))  zend_llist_destroy(zend_llist *l ) ;
struct _zend_llist_element {
   struct _zend_llist_element *next ;
   struct _zend_llist_element *prev ;
   char data[1] ;
};
struct _list_llist_element;
typedef struct _list_llist_element list_llist_element;
struct _list_llist_element {
   znode var ;
   zend_llist dimensions ;
   znode value ;
};
void zend_do_list_end(znode *result , znode *expr ) 
{ zend_llist_element *le ;
  zend_llist_element *dimension ;
  zend_op *opline ;
  znode last_container ;
  zend_llist *tmp_dimension_llist ;
  int tmp ;
  zval _c ;
  zval *__z ;
  int tmp___0 ;
  zend_llist *p ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7390\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  le = compiler_globals.list_llist.head;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7391\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7381\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (le) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7347\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7382\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp_dimension_llist = & ((list_llist_element *)(le->data))->dimensions;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7383\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    dimension = tmp_dimension_llist->head;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7384\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7371\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (dimension) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7348\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7372\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline = get_next_op(compiler_globals.active_op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7373\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )dimension == (unsigned int )tmp_dimension_llist->head) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7353\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        last_container = *expr;
        switch (expr->op_type) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7349\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        case 1 << 2: 
        case 1 << 4: 
        opline->opcode = (unsigned char)81;
        break;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7350\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        case 1 << 1: 
        opline->opcode = (unsigned char)98;
        break;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7351\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        case 1: 
        _zval_copy_ctor(& expr->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7352\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->opcode = (unsigned char)98;
        break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7354\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->extended_value |= 134217728UL;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7355\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->opcode = (unsigned char)81;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7374\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result_type = (unsigned char )(1 << 2);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7375\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7376\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7359\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1_type = (unsigned char )last_container.op_type;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7360\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (last_container.op_type == 1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7356\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& last_container.u.constant));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7357\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op1.constant = (unsigned int )tmp;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7358\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op1 = last_container.u.op;
        }
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7377\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2_type = (unsigned char)1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7378\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7361\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z = & _c;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7362\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z->value.lval = (long )*((int *)(dimension->data));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7363\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z->type = (unsigned char)1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7364\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                   (zval const   *)(& _c));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7365\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2.constant = (unsigned int )tmp___0;
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7379\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7369\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        last_container.op_type = (int )opline->result_type;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7370\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (last_container.op_type == 1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7366\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          last_container.u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7367\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          last_container.u.op = opline->result;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7368\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          last_container.EA = 0U;
        }
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7380\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      dimension = dimension->next;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7385\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((list_llist_element *)(le->data))->value = last_container;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7386\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_llist_destroy(& ((list_llist_element *)(le->data))->dimensions);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7387\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_assign(result, & ((list_llist_element *)(le->data))->var,
                   & ((list_llist_element *)(le->data))->value);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7388\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_free(result);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7389\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    le = le->next;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7392\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_destroy(& compiler_globals.dimension_llist);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7393\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_destroy(& compiler_globals.list_llist);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7394\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *result = *expr;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7395\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.list_stack),
                 (void **)(& p));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7396\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.dimension_llist = *p;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7397\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_del_top(& compiler_globals.list_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7398\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.list_stack),
                 (void **)(& p));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7399\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.list_llist = *p;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7400\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_del_top(& compiler_globals.list_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7401\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_list_end(znode *result , znode *expr ) ;
void zend_do_new_list_begin(void) ;
extern void __attribute__((__visibility__("default")))  zend_llist_init(zend_llist *l ,
                                                                        size_t size ,
                                                                        void (*dtor)(void * ) ,
                                                                        unsigned char persistent ) ;
void zend_do_list_init(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7341\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.list_stack,
                  (void const   *)(& compiler_globals.list_llist),
                  (int )sizeof(zend_llist ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7342\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.list_stack,
                  (void const   *)(& compiler_globals.dimension_llist),
                  (int )sizeof(zend_llist ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7343\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_init(& compiler_globals.list_llist, sizeof(list_llist_element ),
                  (void (*)(void * ))((void *)0), (unsigned char)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7344\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_init(& compiler_globals.dimension_llist, sizeof(int ),
                  (void (*)(void * ))((void *)0), (unsigned char)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7345\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_new_list_begin();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7346\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_list_init(void) ;
extern void __attribute__((__visibility__("default")))  *zend_llist_remove_tail(zend_llist *l ) ;
void zend_do_new_list_end(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7338\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_remove_tail(& compiler_globals.dimension_llist);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7339\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (*((int *)((compiler_globals.dimension_llist.tail)->data))) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7340\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_new_list_end(void) ;
extern void __attribute__((__visibility__("default")))  zend_llist_add_element(zend_llist *l ,
                                                                               void *element ) ;
void zend_do_new_list_begin(void) 
{ int current_dimension ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7335\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  current_dimension = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7336\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_add_element(& compiler_globals.dimension_llist,
                         (void *)(& current_dimension));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7337\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  zend_llist_prepend_element(zend_llist *l ,
                                                                                   void *element ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_copy(zend_llist *dst ,
                                                                        zend_llist *src ) ;
void zend_do_add_list_element(znode const   *element ) 
{ list_llist_element lle ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7332\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (element) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7327\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_check_writable_variable(element);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7328\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    lle.var = (struct _znode )*element;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7329\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_llist_copy(& lle.dimensions, & compiler_globals.dimension_llist);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7330\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_llist_prepend_element(& compiler_globals.list_llist, (void *)(& lle));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7331\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7333\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (*((int *)((compiler_globals.dimension_llist.tail)->data))) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7334\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_add_list_element(znode const   *element ) ;
extern int __attribute__((__visibility__("default")))  _zend_hash_index_update_or_next_insert(HashTable *ht ,
                                                                                              ulong h ,
                                                                                              void *pData ,
                                                                                              uint nDataSize ,
                                                                                              void **pDest ,
                                                                                              int flag ) ;
__inline static long ( __attribute__((__always_inline__)) zend_dval_to_lval)(double d ) ;
__inline static int zend_symtable_update(HashTable *ht , char const   *arKey ,
                                         uint nKeyLength , void *pData ,
                                         uint nDataSize , void **pDest ) ;
void zend_do_add_static_array_element(znode *result , znode *offset ,
                                      znode const   *expr ) 
{ zval *element ;
  void __attribute__((__visibility__("default")))  *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  long tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7323\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7305\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = _emalloc(sizeof(zval_gc_info ));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7306\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    element = (zval *)tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7307\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((zval_gc_info *)element)->u.buffered = (gc_root_buffer *)((void *)0);
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7324\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *element = (struct _zval_struct )expr->u.constant;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7325\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (offset) {
    switch ((int )offset->u.constant.type & 0x00f) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7308\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 8: 
    element->type = (unsigned char )((int )element->type | 0x080);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7309\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = _erealloc((void *)offset->u.constant.value.str.val,
                        (unsigned int )(offset->u.constant.value.str.len + 3), 0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7310\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    offset->u.constant.value.str.val = (char *)tmp___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7311\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(offset->u.constant.value.str.val + (offset->u.constant.value.str.len + 1)) = (char )offset->u.constant.type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7312\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(offset->u.constant.value.str.val + (offset->u.constant.value.str.len + 2)) = (char)0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7313\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_symtable_update(result->u.constant.value.ht,
                         (char const   *)offset->u.constant.value.str.val,
                         (unsigned int )(offset->u.constant.value.str.len + 3),
                         (void *)(& element), sizeof(zval *),
                         (void **)((void *)0));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7314\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_dtor(& offset->u.constant);
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7315\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 6: 
    zend_symtable_update(result->u.constant.value.ht,
                         (char const   *)offset->u.constant.value.str.val,
                         (unsigned int )(offset->u.constant.value.str.len + 1),
                         (void *)(& element), sizeof(zval *),
                         (void **)((void *)0));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7316\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_dtor(& offset->u.constant);
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7317\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 0: 
    zend_symtable_update(result->u.constant.value.ht, "", 1U,
                         (void *)(& element), sizeof(zval *),
                         (void **)((void *)0));
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7318\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 1: 
    case 3: 
    _zend_hash_index_update_or_next_insert(result->u.constant.value.ht,
                                           (unsigned long )offset->u.constant.value.lval,
                                           (void *)(& element), sizeof(zval *),
                                           (void **)((void *)0), 1);
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7319\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 2: 
    tmp___1 = zend_dval_to_lval(offset->u.constant.value.dval);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7320\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zend_hash_index_update_or_next_insert(result->u.constant.value.ht,
                                           (unsigned long )tmp___1,
                                           (void *)(& element), sizeof(zval *),
                                           (void **)((void *)0), 1);
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7321\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 9: 
    zend_error(1, "Illegal offset type");
    break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7322\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zend_hash_index_update_or_next_insert(result->u.constant.value.ht, 0UL,
                                           (void *)(& element), sizeof(zval *),
                                           (void **)((void *)0), 1 << 2);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7326\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_add_static_array_element(znode *result , znode *offset ,
                                      znode const   *expr ) ;
void zend_do_add_array_element(znode *result , znode const   *expr ,
                               znode const   *offset , zend_bool is_ref ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;
  ulong index___0 ;
  int numeric ;
  register char const   *tmp___3 ;
  char const   *end ;
  zval *__z ;
  ulong __attribute__((__visibility__("default")))  tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7297\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7298\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7299\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)72;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7300\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7230\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )result->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7231\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7227\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& result->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7228\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7229\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result = result->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7301\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7235\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )expr->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7236\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7232\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & expr->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7233\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7234\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )expr->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7302\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (offset) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7294\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7240\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2_type = (unsigned char )offset->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7241\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (offset->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7237\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                   & offset->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7238\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2.constant = (unsigned int )tmp___2;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7239\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2 = (union _znode_op )offset->u.op;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7295\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )opline->op2_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7292\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.type == 6) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7288\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        numeric = 0;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7289\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7273\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___3 = (char const   *)((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7274\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*tmp___3 == 45) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7242\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___3 ++;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7243\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7275\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*tmp___3 >= 48) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7271\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*tmp___3 <= 57) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7265\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              end = (char const   *)(((((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val + ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len) + 1) - 1);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7266\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*end != 0) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7250\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp___3 == 48) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7244\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len + 1 > 2) {
                    break;
                  } else {
                    goto _L;
                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7249\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  _L: /* CIL Label */ 
                  if (end - tmp___3 > 10) {
                    break;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7248\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (end - tmp___3 == 10) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "7246\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if ((int const   )*tmp___3 > 50) {
                        break;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "7245\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }

                      }
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "7247\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }

                    }
                  }
                }
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7267\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              index___0 = (unsigned long )((int const   )*tmp___3 - 48);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7268\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              while (1) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7254\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___3 ++;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7255\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((unsigned int )tmp___3 != (unsigned int )end) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7253\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*tmp___3 >= 48) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7252\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if ((int const   )*tmp___3 <= 57) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "7251\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }

                    } else {
                      break;
                    }
                  } else {
                    break;
                  }
                } else {
                  break;
                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7256\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                index___0 = index___0 * 10UL + (ulong )((int const   )*tmp___3 - 48);
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7269\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((unsigned int )tmp___3 == (unsigned int )end) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7262\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )*(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val) == 45) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7258\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (index___0 - 1UL > 2147483647UL) {
                    break;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7257\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7259\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  index___0 = (unsigned long )(- ((long )index___0));
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7261\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (index___0 > 2147483647UL) {
                    break;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7260\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7263\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                numeric = 1;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7264\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7270\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7272\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7290\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (numeric) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7276\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _zval_dtor(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7277\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          __z = & ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7278\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          __z->value.lval = (long )index___0;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7279\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          __z->type = (unsigned char)1;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7287\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7286\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7283\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7280\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val - sizeof(Bucket )))->h;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7281\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___4 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val,
                                         (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len + 1));
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7282\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = (unsigned long )tmp___4;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7284\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___4 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val,
                                       (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len + 1));
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7285\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = (unsigned long )tmp___4;
            }
            break;
          }
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7291\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7293\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7296\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )(1 << 3);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7303\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = (unsigned long )is_ref;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7304\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_add_array_element(znode *result , znode const   *expr ,
                               znode const   *offset , zend_bool is_ref ) ;
void zend_do_init_array(znode *result , znode const   *expr ,
                        znode const   *offset , zend_bool is_ref ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;
  int tmp___1 ;
  ulong index___0 ;
  int numeric ;
  register char const   *tmp___2 ;
  char const   *end ;
  zval *__z ;
  ulong __attribute__((__visibility__("default")))  tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7218\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7219\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7220\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)71;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7221\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7222\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7223\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7147\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7148\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7144\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7145\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7146\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7224\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (expr) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7214\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7152\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1_type = (unsigned char )expr->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7153\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (expr->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7149\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                   & expr->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7150\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1.constant = (unsigned int )tmp___0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7151\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1 = (union _znode_op )expr->u.op;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7215\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (offset) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7211\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7157\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2_type = (unsigned char )offset->op_type;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7158\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (offset->op_type == 1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7154\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                     & offset->u.constant);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7155\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op2.constant = (unsigned int )tmp___1;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7156\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op2 = (union _znode_op )offset->u.op;
        }
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7212\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )opline->op2_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7209\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.type == 6) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7205\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          numeric = 0;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7206\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7190\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___2 = (char const   *)((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7191\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*tmp___2 == 45) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7159\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___2 ++;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7160\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7192\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*tmp___2 >= 48) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7188\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*tmp___2 <= 57) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7182\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                end = (char const   *)(((((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val + ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len) + 1) - 1);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7183\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*end != 0) {
                  break;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7167\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*tmp___2 == 48) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7161\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len + 1 > 2) {
                      break;
                    } else {
                      goto _L;
                    }
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7166\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    _L: /* CIL Label */ 
                    if (end - tmp___2 > 10) {
                      break;
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "7165\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (end - tmp___2 == 10) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "7163\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if ((int const   )*tmp___2 > 50) {
                          break;
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "7162\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }

                        }
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "7164\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }

                      }
                    }
                  }
                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7184\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                index___0 = (unsigned long )((int const   )*tmp___2 - 48);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7185\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                while (1) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7171\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___2 ++;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7172\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((unsigned int )tmp___2 != (unsigned int )end) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7170\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if ((int const   )*tmp___2 >= 48) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "7169\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if ((int const   )*tmp___2 <= 57) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "7168\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }

                      } else {
                        break;
                      }
                    } else {
                      break;
                    }
                  } else {
                    break;
                  }
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7173\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  index___0 = index___0 * 10UL + (ulong )((int const   )*tmp___2 - 48);
                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7186\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((unsigned int )tmp___2 == (unsigned int )end) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7179\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int )*(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val) == 45) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7175\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (index___0 - 1UL > 2147483647UL) {
                      break;
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "7174\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }

                    }
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7176\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    index___0 = (unsigned long )(- ((long )index___0));
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "7178\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (index___0 > 2147483647UL) {
                      break;
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "7177\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }

                    }
                  }
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7180\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  numeric = 1;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7181\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7187\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7189\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
            break;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7207\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (numeric) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7193\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _zval_dtor(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7194\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            __z = & ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7195\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            __z->value.lval = (long )index___0;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7196\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            __z->type = (unsigned char)1;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7204\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            while (1) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "7203\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7200\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7197\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val - sizeof(Bucket )))->h;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7198\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___3 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val,
                                           (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len + 1));
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "7199\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = (unsigned long )tmp___3;
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7201\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___3 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val,
                                         (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len + 1));
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "7202\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = (unsigned long )tmp___3;
              }
              break;
            }
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7208\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7210\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7213\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2_type = (unsigned char )(1 << 3);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7216\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7217\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )(1 << 3);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7225\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = (unsigned long )is_ref;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7226\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_init_array(znode *result , znode const   *expr ,
                        znode const   *offset , zend_bool is_ref ) ;
void zend_do_shell_exec(znode *result , znode const   *cmd ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;
  zval _c ;
  char const   *__s ;
  char __attribute__((__visibility__("default")))  *tmp___1 ;
  int __l ;
  zval *__z ;
  char __attribute__((__visibility__("default")))  *tmp___2 ;
  int tmp___3 ;
  ulong __attribute__((__visibility__("default")))  tmp___4 ;
  int tmp___5 ;
  void __attribute__((__visibility__("default")))  *tmp___6 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7126\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7127\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  switch ((int )cmd->op_type) {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7087\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 1: 
  case 1 << 1: 
  opline->opcode = (unsigned char)65;
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7088\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  default: 
  opline->opcode = (unsigned char)66;
  break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7128\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7092\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )cmd->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7093\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (cmd->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7089\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & cmd->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7090\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7091\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )cmd->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7129\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2.opline_num = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7130\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = 60UL;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7131\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7132\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7133\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)60;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7134\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7135\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7136\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7101\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7094\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = _estrndup("shell_exec", sizeof("shell_exec") - 1U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7095\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __s = (char const   *)tmp___1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7096\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __l = (int )(sizeof("shell_exec") - 1U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7097\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z = & _c;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7098\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->value.str.len = __l;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7099\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7100\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->type = (unsigned char)6;
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7102\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = zend_add_literal(compiler_globals.active_op_array,
                               (zval const   *)(& _c));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7103\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1.constant = (unsigned int )tmp___3;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7137\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7110\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7107\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7104\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val - sizeof(Bucket )))->h;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7105\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___4 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                                 (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7106\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = (unsigned long )tmp___4;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7108\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___4 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                               (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len + 1));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7109\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = (unsigned long )tmp___4;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7138\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7139\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7117\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___5 = (compiler_globals.active_op_array)->last_cache_slot;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7118\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->last_cache_slot) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7119\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->literals + opline->op1.constant)->cache_slot = (unsigned int )tmp___5;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7120\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((compiler_globals.active_op_array)->fn_flags & 16U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7115\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((compiler_globals.active_op_array)->run_time_cache) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7111\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___6 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                            (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                            0);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7112\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___6;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7113\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7114\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7116\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7140\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = 1UL;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7141\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7142\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7124\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7125\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7121\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7122\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7123\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7143\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_shell_exec(znode *result , znode const   *cmd ) ;
int zend_add_const_name_literal(zend_op_array *op_array , zval const   *zv ,
                                int unqualified ) ;
static int zend_constant_ct_subst(znode *result , zval *const_name ,
                                  int all_internal_constants_substitution ) ;
void zend_resolve_non_class_name(znode *element_name ,
                                 zend_bool check_namespace ) ;
extern  __attribute__((__nothrow__)) void *memchr(void const   *__s , int __c ,
                                                  size_t __n )  __attribute__((__pure__,
__nonnull__(1))) ;
int zend_add_class_name_literal(zend_op_array *op_array , zval const   *zv ) ;
void zend_do_fetch_class(znode *result , znode *class_name ) ;
void zend_resolve_class_name(znode *class_name , ulong fetch_type ,
                             int check_ns_name ) ;
void zend_do_fetch_constant(znode *result , znode *constant_container ,
                            znode *constant_name , int mode ,
                            zend_bool check_namespace ) 
{ znode tmp ;
  zend_op *opline ;
  int type ;
  char *compound ;
  ulong fetch_type ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;
  ulong __attribute__((__visibility__("default")))  tmp___4 ;
  int tmp___5 ;
  void __attribute__((__visibility__("default")))  *tmp___6 ;
  void __attribute__((__visibility__("default")))  *tmp___7 ;
  void *tmp___8 ;
  int tmp___9 ;
  void *tmp___10 ;
  int tmp___11 ;
  int tmp___12 ;
  int tmp___13 ;
  int tmp___14 ;
  int tmp___15 ;
  void __attribute__((__visibility__("default")))  *tmp___16 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7084\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fetch_type = (ulong )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7085\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (constant_container) {
    switch (mode) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7016\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 1: 
    type = zend_get_class_fetch_type((char const   *)constant_container->u.constant.value.str.val,
                                     (unsigned int )constant_container->u.constant.value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7017\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (7 == type) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6957\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1, "\"static::\" is not allowed in compile-time constants");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6960\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (0 == type) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6958\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_resolve_class_name(constant_container, fetch_type, 1);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6959\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7018\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_build_full_name((znode *)((void *)0), constant_container,
                            constant_name, 1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7019\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *result = *constant_container;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7020\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->u.constant.type = (unsigned char )(8UL | fetch_type);
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7021\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 1 << 1: 
    if (constant_container->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6964\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_get_class_fetch_type((char const   *)constant_container->u.constant.value.str.val,
                                          (unsigned int )constant_container->u.constant.value.str.len);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6965\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (0 == tmp___0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6961\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_resolve_class_name(constant_container, fetch_type, 1);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6962\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_do_fetch_class(& tmp, constant_container);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6963\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        constant_container = & tmp;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6966\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_do_fetch_class(& tmp, constant_container);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6967\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      constant_container = & tmp;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7022\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline = get_next_op(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7023\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)99;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7024\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )(1 << 1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7025\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7026\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (constant_container->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6968\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1_type = (unsigned char)1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6969\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_class_name_literal(compiler_globals.active_op_array,
                                            (zval const   *)(& constant_container->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6970\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6976\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6974\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1_type = (unsigned char )constant_container->op_type;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6975\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (constant_container->op_type == 1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6971\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                     (zval const   *)(& constant_container->u.constant));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6972\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op1.constant = (unsigned int )tmp___2;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6973\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op1 = constant_container->u.op;
        }
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7027\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6980\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2_type = (unsigned char )constant_name->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6981\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (constant_name->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6977\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = zend_add_literal(compiler_globals.active_op_array,
                                   (zval const   *)(& constant_name->u.constant));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6978\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2.constant = (unsigned int )tmp___3;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6979\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2 = constant_name->u.op;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7028\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6988\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6985\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6982\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val - sizeof(Bucket )))->h;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6983\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___4 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val,
                                   (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len + 1));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6984\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = (unsigned long )tmp___4;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6986\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___4 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val,
                                 (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6987\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = (unsigned long )tmp___4;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7029\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )opline->op1_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6999\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6995\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___5 = (compiler_globals.active_op_array)->last_cache_slot;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6996\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->last_cache_slot) ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6997\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->literals + opline->op2.constant)->cache_slot = (unsigned int )tmp___5;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6998\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((compiler_globals.active_op_array)->fn_flags & 16U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6993\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((compiler_globals.active_op_array)->run_time_cache) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6989\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___6 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                                (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                                0);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6990\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___6;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6991\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6992\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6994\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        break;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7010\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7007\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->literals + opline->op2.constant)->cache_slot = (unsigned int )(compiler_globals.active_op_array)->last_cache_slot;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7008\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (compiler_globals.active_op_array)->last_cache_slot += 2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7009\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((compiler_globals.active_op_array)->fn_flags & 16U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7005\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((compiler_globals.active_op_array)->run_time_cache) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7000\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___7 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                                (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                                0);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7001\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___7;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7002\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7003\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 2)) = (void *)0;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "7004\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "7006\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7030\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7014\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->op_type = (int )opline->result_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7015\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (result->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7011\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7012\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->u.op = opline->result;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7013\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->EA = 0U;
      }
      break;
    }
    break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7031\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7032\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  switch (mode) {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7062\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 1: 
  tmp___8 = memchr((void const   *)constant_name->u.constant.value.str.val,
                   '\\', (unsigned int )constant_name->u.constant.value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7063\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compound = (char *)tmp___8;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7064\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___9 = zend_constant_ct_subst(result, & constant_name->u.constant, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7065\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___9) {
    break;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7033\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7066\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_resolve_non_class_name(constant_name, check_namespace);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7067\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! compound) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7034\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    fetch_type |= 16UL;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7035\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7068\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *result = *constant_name;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7069\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->u.constant.type = (unsigned char )(8UL | fetch_type);
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7070\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 1 << 1: 
  tmp___10 = memchr((void const   *)constant_name->u.constant.value.str.val,
                    '\\', (unsigned int )constant_name->u.constant.value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7071\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compound = (char *)tmp___10;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7072\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_resolve_non_class_name(constant_name, check_namespace);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7073\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___11 = zend_constant_ct_subst(result, & constant_name->u.constant, 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7074\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___11) {
    break;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7036\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7075\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7076\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)99;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7077\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7078\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7079\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7040\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7041\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7037\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7038\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7039\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7080\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7081\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7082\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compound) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7042\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value = 0UL;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7043\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___12 = zend_add_const_name_literal(compiler_globals.active_op_array,
                                           (zval const   *)(& constant_name->u.constant),
                                           0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7044\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2.constant = (unsigned int )tmp___12;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7050\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value = 16UL;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7051\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.current_namespace) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7045\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->extended_value |= 256UL;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7046\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___13 = zend_add_const_name_literal(compiler_globals.active_op_array,
                                             (zval const   *)(& constant_name->u.constant),
                                             1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7047\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___13;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7048\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___14 = zend_add_const_name_literal(compiler_globals.active_op_array,
                                             (zval const   *)(& constant_name->u.constant),
                                             0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7049\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___14;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7083\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7058\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___15 = (compiler_globals.active_op_array)->last_cache_slot;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7059\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->last_cache_slot) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7060\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->literals + opline->op2.constant)->cache_slot = (unsigned int )tmp___15;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "7061\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((compiler_globals.active_op_array)->fn_flags & 16U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7056\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((compiler_globals.active_op_array)->run_time_cache) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7052\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___16 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                             (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                             0);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7053\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___16;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7054\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "7055\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "7057\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    break;
  }
  break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7086\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_fetch_constant(znode *result , znode *constant_container ,
                            znode *constant_name , int mode ,
                            zend_bool check_namespace ) ;
struct _zend_constant {
   zval value ;
   int flags ;
   char *name ;
   uint name_len ;
   int module_number ;
};
static int zend_constant_ct_subst(znode *result , zval *const_name ,
                                  int all_internal_constants_substitution ) 
{ zend_constant *c ;
  zend_constant *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6953\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_get_ct_const((zval const   *)const_name,
                          all_internal_constants_substitution);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6954\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  c = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6955\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (c) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6945\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_dtor(const_name);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6946\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = 1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6947\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->u.constant = c->value;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6948\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_copy_ctor(& result->u.constant);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6949\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->u.constant.refcount__gc = 1U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6950\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->u.constant.is_ref__gc = (unsigned char)0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6951\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return (1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6952\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6956\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (0);
}
}
struct _zend_executor_globals;
typedef struct _zend_executor_globals zend_executor_globals;
zend_executor_globals __attribute__((__visibility__("default")))  executor_globals ;
typedef unsigned short fpu_control_t;
struct _zend_property_info {
   zend_uint flags ;
   char const   *name ;
   int name_length ;
   ulong h ;
   int offset ;
   char const   *doc_comment ;
   int doc_comment_len ;
   zend_class_entry *ce ;
};
struct _zend_object_store_bucket;
typedef struct _zend_object_store_bucket zend_object_store_bucket;
struct _zend_objects_store {
   zend_object_store_bucket *object_buckets ;
   zend_uint top ;
   zend_uint size ;
   int free_list_head ;
};
typedef struct _zend_objects_store zend_objects_store;
struct _zend_ini_entry;
typedef struct _zend_ini_entry zend_ini_entry;
enum __anonenum_zend_error_handling_t_47 {
    EH_NORMAL = 0,
    EH_SUPPRESS = 1,
    EH_THROW = 2
} ;
typedef enum __anonenum_zend_error_handling_t_47 zend_error_handling_t;
struct _zend_ptr_stack {
   int top ;
   int max ;
   void **elements ;
   void **top_element ;
   zend_bool persistent ;
};
typedef struct _zend_ptr_stack zend_ptr_stack;
struct _zend_vm_stack;
typedef struct _zend_vm_stack *zend_vm_stack;
struct __anonstruct___sigset_t_8 {
   unsigned long __val[1024U / (8U * sizeof(unsigned long ))] ;
};
typedef struct __anonstruct___sigset_t_8 __sigset_t;
typedef int __jmp_buf[6];
struct __jmp_buf_tag {
   __jmp_buf __jmpbuf ;
   int __mask_was_saved ;
   __sigset_t __saved_mask ;
};
typedef struct __jmp_buf_tag jmp_buf[1];
struct _zend_executor_globals {
   zval **return_value_ptr_ptr ;
   zval uninitialized_zval ;
   zval *uninitialized_zval_ptr ;
   zval error_zval ;
   zval *error_zval_ptr ;
   zend_ptr_stack arg_types_stack ;
   HashTable *symtable_cache[32] ;
   HashTable **symtable_cache_limit ;
   HashTable **symtable_cache_ptr ;
   zend_op **opline_ptr ;
   HashTable *active_symbol_table ;
   HashTable symbol_table ;
   HashTable included_files ;
   jmp_buf *bailout ;
   int error_reporting ;
   int orig_error_reporting ;
   int exit_status ;
   zend_op_array *active_op_array ;
   HashTable *function_table ;
   HashTable *class_table ;
   HashTable *zend_constants ;
   zend_class_entry *scope ;
   zend_class_entry *called_scope ;
   zval *This ;
   long precision ;
   int ticks_count ;
   zend_bool in_execution ;
   HashTable *in_autoload ;
   zend_function *autoload_func ;
   zend_bool full_tables_cleanup ;
   zend_bool no_extensions ;
   HashTable regular_list ;
   HashTable persistent_list ;
   zend_vm_stack argument_stack ;
   int user_error_handler_error_reporting ;
   zval *user_error_handler ;
   zval *user_exception_handler ;
   zend_stack user_error_handlers_error_reporting ;
   zend_ptr_stack user_error_handlers ;
   zend_ptr_stack user_exception_handlers ;
   zend_error_handling_t error_handling ;
   zend_class_entry *exception_class ;
   int timeout_seconds ;
   int lambda_count ;
   HashTable *ini_directives ;
   HashTable *modified_ini_directives ;
   zend_ini_entry *error_reporting_ini_entry ;
   zend_objects_store objects_store ;
   zval *exception ;
   zval *prev_exception ;
   zend_op *opline_before_exception ;
   zend_op exception_op[3] ;
   struct _zend_execute_data *current_execute_data ;
   struct _zend_module_entry *current_module ;
   zend_property_info std_property_info ;
   zend_bool active ;
   zend_op *start_op ;
   void *saved_fpu_cw_ptr ;
   fpu_control_t saved_fpu_cw ;
   void *reserved[4] ;
};
static zend_constant *zend_get_ct_const(zval const   *const_name ,
                                        int all_internal_constants_substitution ) 
{ zend_constant *c ;
  char *lookup_name ;
  char __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  char *lookup_name___0 ;
  char __attribute__((__visibility__("default")))  *tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  int __attribute__((__visibility__("default")))  tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6940\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  c = (zend_constant *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6941\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*(const_name->value.str.val + 0) == 92) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6910\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = zend_hash_find((HashTable const   *)executor_globals.zend_constants,
                             (char const   *)(const_name->value.str.val + 1),
                             (unsigned int )const_name->value.str.len,
                             (void **)(& c));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6911\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6903\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = zend_str_tolower_dup((char const   *)(const_name->value.str.val + 1),
                                 (unsigned int )(const_name->value.str.len - 1));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6904\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      lookup_name = (char *)tmp;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6905\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_hash_find((HashTable const   *)executor_globals.zend_constants,
                               (char const   *)lookup_name,
                               (unsigned int )const_name->value.str.len,
                               (void **)(& c));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6906\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___0 == (int __attribute__((__visibility__("default")))  )0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6901\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (c->flags & (1 << 2)) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6899\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (! (c->flags & 1)) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6896\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _efree((void *)lookup_name);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6897\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return (c);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6898\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6900\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6902\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6907\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)lookup_name);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6908\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((zend_constant *)((void *)0));
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6909\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6926\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___4 = zend_hash_find((HashTable const   *)executor_globals.zend_constants,
                             (char const   *)const_name->value.str.val,
                             (unsigned int )(const_name->value.str.len + 1),
                             (void **)(& c));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6927\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___4 == (int __attribute__((__visibility__("default")))  )-1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6919\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_str_tolower_dup((char const   *)const_name->value.str.val,
                                     (unsigned int )const_name->value.str.len);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6920\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      lookup_name___0 = (char *)tmp___2;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6921\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___3 = zend_hash_find((HashTable const   *)executor_globals.zend_constants,
                               (char const   *)lookup_name___0,
                               (unsigned int )(const_name->value.str.len + 1),
                               (void **)(& c));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6922\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___3 == (int __attribute__((__visibility__("default")))  )0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6917\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (c->flags & (1 << 2)) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6915\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (! (c->flags & 1)) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6912\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _efree((void *)lookup_name___0);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6913\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return (c);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6914\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6916\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6918\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6923\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)lookup_name___0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6924\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((zend_constant *)((void *)0));
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6925\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6942\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (c->flags & (1 << 2)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6928\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return (c);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6929\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6943\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (all_internal_constants_substitution) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6938\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (c->flags & (1 << 1)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6936\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! (compiler_globals.compiler_options & (unsigned int )(1 << 5))) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6934\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )c->value.type != 8) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6932\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )c->value.type != 9) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6930\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return (c);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6931\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6933\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6935\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6937\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6939\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6944\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((zend_constant *)((void *)0));
}
}
void zend_do_end_function_call(znode *function_name , znode *result ,
                               znode const   *argument_list , int is_method ,
                               int is_dynamic_fcall ) ;
void zend_do_end_new_object(znode *result , znode const   *new_token ,
                            znode const   *argument_list ) 
{ znode ctor_result ;
  int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6890\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_end_function_call((znode *)((void *)0), & ctor_result, argument_list,
                            1, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6891\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_free(& ctor_result);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6892\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6893\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + new_token->u.op.opline_num)->op2.opline_num = (unsigned int )tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6894\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6888\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )((compiler_globals.active_op_array)->opcodes + new_token->u.op.opline_num)->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6889\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6885\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + ((compiler_globals.active_op_array)->opcodes + new_token->u.op.opline_num)->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6886\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = ((compiler_globals.active_op_array)->opcodes + new_token->u.op.opline_num)->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6887\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6895\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_end_new_object(znode *result , znode const   *new_token ,
                            znode const   *argument_list ) ;
void zend_do_begin_new_object(znode *new_token , znode *class_type ) 
{ zend_op *opline ;
  unsigned char *ptr ;
  int tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6874\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ptr = (unsigned char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6875\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6876\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  new_token->u.op.opline_num = (unsigned int )tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6877\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6878\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)68;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6879\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6880\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6881\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6872\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )class_type->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6873\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (class_type->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6869\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& class_type->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6870\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6871\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = class_type->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6882\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6883\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.function_call_stack,
                  (void const   *)((void *)(& ptr)),
                  (int )sizeof(unsigned char *));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6884\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_begin_new_object(znode *new_token , znode *class_type ) ;
void zend_do_pop_object(znode *object ) 
{ znode *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6866\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (object) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6863\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_stack_top((zend_stack const   *)(& compiler_globals.object_stack),
                   (void **)(& tmp));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6864\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *object = *tmp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6865\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6867\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_del_top(& compiler_globals.object_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6868\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_pop_object(znode *object ) ;
void zend_do_push_object(znode const   *object ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6861\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.object_stack, (void const   *)object,
                  (int )sizeof(znode ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6862\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_push_object(znode const   *object ) ;
extern void __attribute__((__visibility__("default")))  zend_register_long_constant(char const   *name ,
                                                                                    uint name_len ,
                                                                                    long lval ,
                                                                                    int flags ,
                                                                                    int module_number ) ;
extern size_t __attribute__((__visibility__("default")))  zend_get_scanned_file_offset(void) ;
void __attribute__((__visibility__("default")))  zend_mangle_property_name(char **dest ,
                                                                           int *dest_length ,
                                                                           char const   *src1 ,
                                                                           int src1_length ,
                                                                           char const   *src2 ,
                                                                           int src2_length ,
                                                                           int internal ) ;
char __attribute__((__visibility__("default")))  *zend_get_compiled_filename(void) ;
void zend_do_halt_compiler_register(void) 
{ char *name ;
  char *cfilename ;
  char haltoff[25] ;
  int len ;
  int clen ;
  char __attribute__((__visibility__("default")))  *tmp ;
  size_t tmp___0 ;
  size_t __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6825\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[0] = (char )'_';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6826\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[1] = (char )'_';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6827\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[2] = (char )'C';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6828\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[3] = (char )'O';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6829\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[4] = (char )'M';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6830\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[5] = (char )'P';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6831\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[6] = (char )'I';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6832\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[7] = (char )'L';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6833\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[8] = (char )'E';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6834\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[9] = (char )'R';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6835\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[10] = (char )'_';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6836\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[11] = (char )'H';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6837\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[12] = (char )'A';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6838\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[13] = (char )'L';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6839\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[14] = (char )'T';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6840\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[15] = (char )'_';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6841\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[16] = (char )'O';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6842\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[17] = (char )'F';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6843\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[18] = (char )'F';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6844\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[19] = (char )'S';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6845\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[20] = (char )'E';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6846\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[21] = (char )'T';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6847\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[22] = (char )'_';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6848\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[23] = (char )'_';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6849\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  haltoff[24] = (char )'\000';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6850\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.has_bracketed_namespaces) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6821\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.in_namespace) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6819\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L,
                 "__HALT_COMPILER() can only be used from the outermost scope");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6820\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6822\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6851\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_get_compiled_filename();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6852\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  cfilename = (char *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6853\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = strlen((char const   *)cfilename);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6854\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  clen = (int )tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6855\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_mangle_property_name(& name, & len, (char const   *)(haltoff),
                            (int )(sizeof(haltoff) - 1U),
                            (char const   *)cfilename, clen, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6856\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_get_scanned_file_offset();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6857\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_register_long_constant((char const   *)name, (unsigned int )(len + 1),
                              (long )tmp___1, 1, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6858\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _efree((void *)name);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6859\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.in_namespace) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6823\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_end_namespace();
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6824\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6860\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_halt_compiler_register(void) ;
extern void init_op(zend_op *op ) ;
void zend_del_literal(zend_op_array *op_array , int n ) ;
static zend_bool opline_is_fetch_this(zend_op const   *opline ) ;
void zend_do_fetch_property(znode *result , znode *object ,
                            znode const   *property ) 
{ zend_op opline ;
  zend_llist *fetch_list_ptr ;
  zend_llist_element *le ;
  zend_op *opline_ptr ;
  int tmp ;
  ulong __attribute__((__visibility__("default")))  tmp___0 ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  zend_bool tmp___2 ;
  int tmp___3 ;
  zend_bool tmp___4 ;
  int tmp___5 ;
  int tmp___6 ;
  ulong __attribute__((__visibility__("default")))  tmp___7 ;
  void __attribute__((__visibility__("default")))  *tmp___8 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6805\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.bp_stack),
                 (void **)(& fetch_list_ptr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6806\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (object->op_type == 1 << 4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6703\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (object->u.op.var == (compiler_globals.active_op_array)->this_var) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6701\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      object->op_type = 1 << 3;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6702\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6754\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (fetch_list_ptr->count == 1U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6749\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      le = fetch_list_ptr->head;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6750\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline_ptr = (zend_op *)(le->data);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6751\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = opline_is_fetch_this((zend_op const   *)opline_ptr);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6752\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6742\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_del_literal(compiler_globals.active_op_array,
                         (int )opline_ptr->op1.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6743\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline_ptr->op1_type = (unsigned char )(1 << 3);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6744\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6707\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline_ptr->op2_type = (unsigned char )property->op_type;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6708\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (property->op_type == 1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6704\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp = zend_add_literal(compiler_globals.active_op_array,
                                   & property->u.constant);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6705\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            opline_ptr->op2.constant = (unsigned int )tmp;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6706\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            opline_ptr->op2 = (union _znode_op )property->u.op;
          }
          break;
        }
        switch ((int )opline_ptr->opcode) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6709\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        case 83: 
        opline_ptr->opcode = (unsigned char)85;
        break;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6710\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        case 80: 
        opline_ptr->opcode = (unsigned char)82;
        break;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6711\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        case 86: 
        opline_ptr->opcode = (unsigned char)88;
        break;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6712\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        case 89: 
        opline_ptr->opcode = (unsigned char)91;
        break;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6713\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        case 95: 
        opline_ptr->opcode = (unsigned char)97;
        break;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6714\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        case 92: 
        opline_ptr->opcode = (unsigned char)94;
        break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6745\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )opline_ptr->op2_type == 1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6735\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )((compiler_globals.active_op_array)->literals + opline_ptr->op2.constant)->constant.type == 6) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6732\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            while (1) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "6721\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((unsigned int )((compiler_globals.active_op_array)->literals + opline_ptr->op2.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "6718\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((unsigned int )((compiler_globals.active_op_array)->literals + opline_ptr->op2.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "6715\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline_ptr->op2.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline_ptr->op2.constant)->constant.value.str.val - sizeof(Bucket )))->h;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "6716\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___0 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline_ptr->op2.constant)->constant.value.str.val,
                                           (unsigned int )(((compiler_globals.active_op_array)->literals + opline_ptr->op2.constant)->constant.value.str.len + 1));
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "6717\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline_ptr->op2.constant)->constant))->hash_value = (unsigned long )tmp___0;
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "6719\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___0 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline_ptr->op2.constant)->constant.value.str.val,
                                         (unsigned int )(((compiler_globals.active_op_array)->literals + opline_ptr->op2.constant)->constant.value.str.len + 1));
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "6720\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline_ptr->op2.constant)->constant))->hash_value = (unsigned long )tmp___0;
              }
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6733\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            while (1) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "6729\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              ((compiler_globals.active_op_array)->literals + opline_ptr->op2.constant)->cache_slot = (unsigned int )(compiler_globals.active_op_array)->last_cache_slot;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "6730\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              (compiler_globals.active_op_array)->last_cache_slot += 2;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "6731\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((compiler_globals.active_op_array)->fn_flags & 16U) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "6727\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((compiler_globals.active_op_array)->run_time_cache) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "6722\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___1 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                                      (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                                      0);
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "6723\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___1;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "6724\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "6725\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 2)) = (void *)0;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "6726\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "6728\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
              break;
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6734\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6736\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6746\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6740\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->op_type = (int )opline_ptr->result_type;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6741\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (result->op_type == 1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6737\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            result->u.constant = ((compiler_globals.active_op_array)->literals + opline_ptr->result.constant)->constant;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6738\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            result->u.op = opline_ptr->result;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6739\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            result->EA = 0U;
          }
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6747\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6748\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6753\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6807\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = zend_is_function_or_method_call((znode const   *)object);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6808\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6760\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    init_op(& opline);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6761\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.opcode = (unsigned char)156;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6762\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6758\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline.op1_type = (unsigned char )object->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6759\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (object->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6755\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = zend_add_literal(compiler_globals.active_op_array,
                                   (zval const   *)(& object->u.constant));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6756\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline.op1.constant = (unsigned int )tmp___3;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6757\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline.op1 = object->u.op;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6763\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.op2_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6764\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.result_type = (unsigned char )(1 << 2);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6765\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.result.var = opline.op1.var;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6766\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_llist_add_element(fetch_list_ptr, (void *)(& opline));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6767\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6809\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  init_op(& opline);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6810\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline.opcode = (unsigned char)85;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6811\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline.result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6812\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline.result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6813\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6771\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.op1_type = (unsigned char )object->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6772\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (object->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6768\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___5 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& object->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6769\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline.op1.constant = (unsigned int )tmp___5;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6770\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline.op1 = object->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6814\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6776\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.op2_type = (unsigned char )property->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6777\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (property->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6773\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___6 = zend_add_literal(compiler_globals.active_op_array,
                                 & property->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6774\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline.op2.constant = (unsigned int )tmp___6;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6775\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline.op2 = (union _znode_op )property->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6815\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )opline.op2_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6798\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.type == 6) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6795\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6784\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6781\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6778\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.val - sizeof(Bucket )))->h;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6779\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___7 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.val,
                                     (unsigned int )(((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.len + 1));
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6780\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant))->hash_value = (unsigned long )tmp___7;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6782\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___7 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.val,
                                   (unsigned int )(((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.len + 1));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6783\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant))->hash_value = (unsigned long )tmp___7;
        }
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6796\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6792\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->literals + opline.op2.constant)->cache_slot = (unsigned int )(compiler_globals.active_op_array)->last_cache_slot;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6793\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (compiler_globals.active_op_array)->last_cache_slot += 2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6794\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((compiler_globals.active_op_array)->fn_flags & 16U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6790\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((compiler_globals.active_op_array)->run_time_cache) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6785\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___8 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                                (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                                0);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6786\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___8;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6787\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6788\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 2)) = (void *)0;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6789\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6791\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        break;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6797\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6799\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6816\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6803\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline.result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6804\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6800\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline.result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6801\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline.result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6802\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6817\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_add_element(fetch_list_ptr, (void *)(& opline));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6818\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_fetch_property(znode *result , znode *object ,
                            znode const   *property ) ;
extern int __attribute__((__visibility__("default")))  _zend_hash_quick_add_or_update(HashTable *ht ,
                                                                                      char const   *arKey ,
                                                                                      uint nKeyLength ,
                                                                                      ulong h ,
                                                                                      void *pData ,
                                                                                      uint nDataSize ,
                                                                                      void **pDest ,
                                                                                      int flag ) ;
void zend_do_declare_class_constant(znode *var_name , znode const   *value ) 
{ zval *property ;
  char const   *cname ;
  int result ;
  void __attribute__((__visibility__("default")))  *tmp ;
  char const __attribute__((__visibility__("default")))  *tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6689\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  cname = (char const   *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6690\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )value->u.constant.type == 9) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6662\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Arrays are not allowed in class constants");
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6663\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6664\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6691\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (((compiler_globals.active_class_entry)->ce_flags & 288U) == 288U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6665\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Traits cannot have constants");
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6666\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6667\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6692\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6668\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = _emalloc(sizeof(zval_gc_info ));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6669\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    property = (zval *)tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6670\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((zval_gc_info *)property)->u.buffered = (gc_root_buffer *)((void *)0);
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6693\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *property = (struct _zval_struct )value->u.constant;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6694\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = (*zend_new_interned_string)((char const   *)var_name->u.constant.value.str.val,
                                        var_name->u.constant.value.str.len + 1,
                                        0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6695\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  cname = (char const   *)tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6696\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )cname >= (unsigned int )compiler_globals.interned_strings_start) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6675\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )cname < (unsigned int )compiler_globals.interned_strings_end) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6671\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = _zend_hash_quick_add_or_update(& (compiler_globals.active_class_entry)->constants_table,
                                               cname,
                                               (unsigned int )(var_name->u.constant.value.str.len + 1),
                                               ((Bucket *)((char *)cname - sizeof(Bucket )))->h,
                                               (void *)(& property),
                                               sizeof(zval *),
                                               (void **)((void *)0), 1 << 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6672\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result = (int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6673\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = _zend_hash_add_or_update(& (compiler_globals.active_class_entry)->constants_table,
                                         cname,
                                         (unsigned int )(var_name->u.constant.value.str.len + 1),
                                         (void *)(& property), sizeof(zval *),
                                         (void **)((void *)0), 1 << 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6674\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result = (int )tmp___2;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6676\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = _zend_hash_add_or_update(& (compiler_globals.active_class_entry)->constants_table,
                                       cname,
                                       (unsigned int )(var_name->u.constant.value.str.len + 1),
                                       (void *)(& property), sizeof(zval *),
                                       (void **)((void *)0), 1 << 1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6677\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = (int )tmp___2;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6697\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (result == -1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6682\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6680\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((gc_root_buffer *)((unsigned long )((zval_gc_info *)property)->u.buffered & 4294967292UL)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6678\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        gc_remove_zval_from_buffer(property);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6679\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6681\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)property);
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6683\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Cannot redefine class constant %s::%s",
               (compiler_globals.active_class_entry)->name,
               var_name->u.constant.value.str.val);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6684\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6698\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_dtor(& var_name->u.constant);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6699\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.doc_comment) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6685\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)compiler_globals.doc_comment);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6686\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment = (char *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6687\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment_len = 0U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6688\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6700\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_declare_class_constant(znode *var_name , znode const   *value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_ex(zend_class_entry *ce ,
                                                                                char const   *name ,
                                                                                int name_length ,
                                                                                zval *property ,
                                                                                int access_type ,
                                                                                char const   *doc_comment ,
                                                                                int doc_comment_len ) ;
void zend_do_declare_property(znode const   *var_name , znode const   *value ,
                              zend_uint access_type ) 
{ zval *property ;
  zend_property_info *existing_property_info ;
  char *comment ;
  int comment_len ;
  int __attribute__((__visibility__("default")))  tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  char const __attribute__((__visibility__("default")))  *tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6648\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  comment = (char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6649\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  comment_len = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6650\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_class_entry)->ce_flags & 128U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6628\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Interfaces may not include member variables");
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6629\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6651\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (access_type & 2U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6630\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Properties cannot be declared abstract");
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6631\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6652\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (access_type & 4U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6632\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L,
               "Cannot declare property %s::$%s final, the final modifier is allowed only for methods and classes",
               (compiler_globals.active_class_entry)->name,
               var_name->u.constant.value.str.val);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6633\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6653\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_hash_find((HashTable const   *)(& (compiler_globals.active_class_entry)->properties_info),
                       (char const   *)var_name->u.constant.value.str.val,
                       (unsigned int )(var_name->u.constant.value.str.len + 1),
                       (void **)(& existing_property_info));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6654\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp == (int __attribute__((__visibility__("default")))  )0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6634\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Cannot redeclare %s::$%s",
               (compiler_globals.active_class_entry)->name,
               var_name->u.constant.value.str.val);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6635\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6655\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6636\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = _emalloc(sizeof(zval_gc_info ));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6637\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    property = (zval *)tmp___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6638\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((zval_gc_info *)property)->u.buffered = (gc_root_buffer *)((void *)0);
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6656\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (value) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6639\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *property = (struct _zval_struct )value->u.constant;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6640\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    property->refcount__gc = 1U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6641\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    property->is_ref__gc = (unsigned char)0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6642\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    property->type = (unsigned char)0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6657\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.doc_comment) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6643\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    comment = compiler_globals.doc_comment;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6644\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    comment_len = (int )compiler_globals.doc_comment_len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6645\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment = (char *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6646\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment_len = 0U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6647\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6658\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = (*zend_new_interned_string)((char const   *)var_name->u.constant.value.str.val,
                                        (int )(var_name->u.constant.value.str.len + 1),
                                        0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6659\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_declare_property_ex(compiler_globals.active_class_entry,
                           (char const   *)tmp___1,
                           (int )var_name->u.constant.value.str.len, property,
                           (int )access_type, (char const   *)comment,
                           comment_len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6660\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _efree((void *)var_name->u.constant.value.str.val);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6661\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_declare_property(znode const   *var_name , znode const   *value ,
                              zend_uint access_type ) ;
static int zend_strnlen(char const   *s , int maxlen ) ;
int __attribute__((__visibility__("default")))  zend_unmangle_property_name(char const   *mangled_property ,
                                                                            int len ,
                                                                            char const   **class_name ,
                                                                            char const   **prop_name ) 
{ int class_name_len ;
  int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6618\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *class_name = (char const   *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6619\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )*(mangled_property + 0) != 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6599\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *prop_name = mangled_property;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6600\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6601\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6620\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (len < 3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6602\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 3L, "Illegal member variable name");
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6603\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *prop_name = mangled_property;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6604\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )-1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6609\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(mangled_property + 1) == 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6605\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 3L, "Illegal member variable name");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6606\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *prop_name = mangled_property;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6607\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((int __attribute__((__visibility__("default")))  )-1);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6608\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6621\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  len --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6622\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_strnlen(mangled_property + 1, len - 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6623\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  class_name_len = tmp + 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6624\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_name_len >= len) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6610\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 3L, "Corrupt member variable name");
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6611\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *prop_name = mangled_property;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6612\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )-1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6617\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(mangled_property + class_name_len) != 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6613\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 3L, "Corrupt member variable name");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6614\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *prop_name = mangled_property;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6615\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((int __attribute__((__visibility__("default")))  )-1);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6616\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6625\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *class_name = mangled_property + 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6626\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *prop_name = *class_name + class_name_len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6627\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )0);
}
}
int __attribute__((__visibility__("default")))  zend_unmangle_property_name(char const   *mangled_property ,
                                                                            int len ,
                                                                            char const   **class_name ,
                                                                            char const   **prop_name ) ;
static int zend_strnlen(char const   *s , int maxlen ) 
{ int len ;
  char const   *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6596\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  len = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6597\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6592\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = s;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6593\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    s ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6594\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (*tmp) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6589\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = maxlen;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6590\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      maxlen --;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6591\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6588\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6595\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    len ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6598\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (len);
}
}
__inline static void *__zend_malloc(size_t len ) ;
void __attribute__((__visibility__("default")))  zend_mangle_property_name(char **dest ,
                                                                           int *dest_length ,
                                                                           char const   *src1 ,
                                                                           int src1_length ,
                                                                           char const   *src2 ,
                                                                           int src2_length ,
                                                                           int internal ) 
{ char *prop_name ;
  int prop_name_length ;
  void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6580\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  prop_name_length = ((1 + src1_length) + 1) + src2_length;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6581\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (internal) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6576\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = __zend_malloc((unsigned int )(prop_name_length + 1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6577\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    prop_name = (char *)tmp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6578\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = _emalloc((unsigned int )(prop_name_length + 1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6579\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    prop_name = (char *)tmp___0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6582\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *(prop_name + 0) = (char )'\000';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6583\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  memcpy((void */* __restrict  */)(prop_name + 1),
         (void const   */* __restrict  */)src1, (unsigned int )(src1_length + 1));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6584\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  memcpy((void */* __restrict  */)(((prop_name + 1) + src1_length) + 1),
         (void const   */* __restrict  */)src2, (unsigned int )(src2_length + 1));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6585\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *dest = prop_name;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6586\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *dest_length = prop_name_length;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6587\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_implements_trait(znode *trait_name ) 
{ zend_op *opline ;
  int tmp ;
  int tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6564\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_class_entry)->ce_flags & 128U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6555\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L,
               "Cannot use traits inside of interfaces. %s is used in %s",
               trait_name->u.constant.value.str.val,
               (compiler_globals.active_class_entry)->name);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6556\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6565\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_get_class_fetch_type((char const   *)trait_name->u.constant.value.str.val,
                                  (unsigned int )trait_name->u.constant.value.str.len);
  switch (tmp) {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6557\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 1: 
  case 2: 
  case 7: 
  zend_error(1 << 6L, "Cannot use \'%s\' as trait name as it is reserved",
             trait_name->u.constant.value.str.val);
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6558\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  default: ;
  break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6566\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6567\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)154;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6568\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6562\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )compiler_globals.implementing_class.op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6563\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.implementing_class.op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6559\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& compiler_globals.implementing_class.u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6560\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6561\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = compiler_globals.implementing_class.u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6569\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_resolve_class_name(trait_name, opline->extended_value, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6570\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = 14UL;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6571\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6572\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_add_class_name_literal(compiler_globals.active_op_array,
                                        (zval const   *)(& trait_name->u.constant));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6573\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2.constant = (unsigned int )tmp___1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6574\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_class_entry)->num_traits) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6575\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_implements_trait(znode *trait_name ) ;
void zend_do_implements_interface(znode *interface_name ) 
{ zend_op *opline ;
  int tmp ;
  int tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6543\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (((compiler_globals.active_class_entry)->ce_flags & 288U) == 288U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6534\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L,
               "Cannot use \'%s\' as interface on \'%s\' since it is a Trait",
               interface_name->u.constant.value.str.val,
               (compiler_globals.active_class_entry)->name);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6535\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6544\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_get_class_fetch_type((char const   *)interface_name->u.constant.value.str.val,
                                  (unsigned int )interface_name->u.constant.value.str.len);
  switch (tmp) {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6536\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 1: 
  case 2: 
  case 7: 
  zend_error(1 << 6L, "Cannot use \'%s\' as interface name as it is reserved",
             interface_name->u.constant.value.str.val);
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6537\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  default: ;
  break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6545\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6546\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)144;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6547\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6541\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )compiler_globals.implementing_class.op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6542\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.implementing_class.op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6538\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& compiler_globals.implementing_class.u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6539\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6540\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = compiler_globals.implementing_class.u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6548\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_resolve_class_name(interface_name, opline->extended_value, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6549\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = (opline->extended_value & 4294967280UL) | 6UL;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6550\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6551\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_add_class_name_literal(compiler_globals.active_op_array,
                                        (zval const   *)(& interface_name->u.constant));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6552\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2.constant = (unsigned int )tmp___1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6553\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_class_entry)->num_interfaces) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6554\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_implements_interface(znode *interface_name ) ;
static void do_verify_abstract_class(void) ;
extern void zend_verify_abstract_class(zend_class_entry *ce ) ;
int __attribute__((__visibility__("default")))  zend_get_compiled_lineno(void) ;
struct _zend_internal_function {
   zend_uchar type ;
   char const   *function_name ;
   zend_class_entry *scope ;
   zend_uint fn_flags ;
   union _zend_function *prototype ;
   zend_uint num_args ;
   zend_uint required_num_args ;
   zend_arg_info *arg_info ;
   void (*handler)(int ht , zval *return_value , zval **return_value_ptr ,
                   zval *this_ptr , int return_value_used ) ;
   struct _zend_module_entry *module ;
};
typedef struct _zend_internal_function zend_internal_function;
struct __anonstruct_common_50 {
   zend_uchar type ;
   char const   *function_name ;
   zend_class_entry *scope ;
   zend_uint fn_flags ;
   union _zend_function *prototype ;
   zend_uint num_args ;
   zend_uint required_num_args ;
   zend_arg_info *arg_info ;
};
union _zend_function {
   zend_uchar type ;
   struct __anonstruct_common_50 common ;
   zend_op_array op_array ;
   zend_internal_function internal_function ;
};
void zend_do_end_class_declaration(znode const   *class_token ,
                                   znode const   *parent_token ) 
{ zend_class_entry *ce ;
  int __attribute__((__visibility__("default")))  tmp ;
  zend_op *opline ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6523\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce = compiler_globals.active_class_entry;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6524\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->constructor) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6486\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (ce->constructor)->common.fn_flags |= 8192U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6487\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((ce->constructor)->common.fn_flags & 1U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6484\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Constructor %s::%s() cannot be static", ce->name,
                 (ce->constructor)->common.function_name);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6485\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6488\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6525\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->destructor) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6491\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (ce->destructor)->common.fn_flags |= 16384U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6492\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((ce->destructor)->common.fn_flags & 1U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6489\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Destructor %s::%s() cannot be static", ce->name,
                 (ce->destructor)->common.function_name);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6490\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6493\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6526\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->clone) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6496\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (ce->clone)->common.fn_flags |= 32768U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6497\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((ce->clone)->common.fn_flags & 1U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6494\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Clone method %s::%s() cannot be static", ce->name,
                 (ce->clone)->common.function_name);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6495\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6498\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6527\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_get_compiled_lineno();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6528\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce->info.user.line_end = (unsigned int )tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6529\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->num_traits > 0U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6504\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->traits = (zend_class_entry **)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6505\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->num_traits = 0U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6506\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->ce_flags |= 4194304U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6507\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline = get_next_op(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6508\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)155;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6509\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6502\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1_type = (unsigned char )compiler_globals.implementing_class.op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6503\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (compiler_globals.implementing_class.op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6499\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                   (zval const   *)(& compiler_globals.implementing_class.u.constant));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6500\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1.constant = (unsigned int )tmp___0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6501\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1 = compiler_globals.implementing_class.u.op;
      }
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6510\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6530\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! (ce->ce_flags & 160U)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6517\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (parent_token->op_type != (int const   )(1 << 3)) {
      goto _L;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6516\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (ce->num_interfaces > 0U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6513\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L: /* CIL Label */ 
        zend_verify_abstract_class(ce);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6514\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (ce->num_interfaces) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6511\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          do_verify_abstract_class();
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6512\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6515\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6518\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6531\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->num_interfaces > 0U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6519\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->interfaces = (zend_class_entry **)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6520\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->num_interfaces = 0U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6521\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->ce_flags |= 524288U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6522\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6532\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.active_class_entry = (zend_class_entry *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6533\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_end_class_declaration(znode const   *class_token ,
                                   znode const   *parent_token ) ;
static void do_verify_abstract_class(void) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6478\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6479\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6480\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)146;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6481\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6476\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )compiler_globals.implementing_class.op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6477\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.implementing_class.op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6473\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& compiler_globals.implementing_class.u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6474\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6475\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = compiler_globals.implementing_class.u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6482\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6483\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static void build_runtime_defined_function_key(zval *result ,
                                               char const   *name ,
                                               int name_length ) ;
void zend_do_begin_class_declaration(znode const   *class_token ,
                                     znode *class_name ,
                                     znode const   *parent_class_name ) 
{ zend_op *opline ;
  int doing_inheritance ;
  zend_class_entry *new_class_entry ;
  char *lcname ;
  int error ;
  zval **ns_name ;
  zval key ;
  char __attribute__((__visibility__("default")))  *tmp ;
  size_t __s1_len ;
  size_t __s2_len ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;
  unsigned char const   *__s2 ;
  register int __result ;
  int tmp___3 ;
  unsigned char const   *__s1 ;
  register int __result___0 ;
  int tmp___4 ;
  int tmp___5 ;
  int tmp___6 ;
  size_t __s1_len___0 ;
  size_t __s2_len___0 ;
  int tmp___7 ;
  int tmp___8 ;
  int tmp___9 ;
  unsigned char const   *__s2___0 ;
  register int __result___1 ;
  int tmp___10 ;
  unsigned char const   *__s1___0 ;
  register int __result___2 ;
  int tmp___11 ;
  int tmp___12 ;
  int tmp___13 ;
  int __attribute__((__visibility__("default")))  tmp___14 ;
  znode tmp___15 ;
  char __attribute__((__visibility__("default")))  *tmp___16 ;
  char *tmp___17 ;
  char __attribute__((__visibility__("default")))  *tmp___18 ;
  int tmp___19 ;
  void __attribute__((__visibility__("default")))  *tmp___20 ;
  char const __attribute__((__visibility__("default")))  *tmp___21 ;
  char __attribute__((__visibility__("default")))  *tmp___22 ;
  int tmp___23 ;
  ulong __attribute__((__visibility__("default")))  tmp___24 ;
  zval _c ;
  char const   *__s ;
  int __l ;
  zval *__z ;
  char __attribute__((__visibility__("default")))  *tmp___25 ;
  int tmp___26 ;
  ulong __attribute__((__visibility__("default")))  tmp___27 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6433\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  doing_inheritance = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6434\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  error = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6435\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.active_class_entry) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6326\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Class declarations may not be nested");
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6327\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6328\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6436\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_str_tolower_dup((char const   *)class_name->u.constant.value.str.val,
                             (unsigned int )class_name->u.constant.value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6437\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lcname = (char *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6438\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6340\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s1_len = strlen((char const   *)lcname);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6341\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s2_len = strlen("self");
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6342\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! ((unsigned int )((void const   *)(lcname + 1)) - (unsigned int )((void const   *)lcname) == 1U)) {
      goto _L___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6335\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (__s1_len >= 4U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6333\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___0: /* CIL Label */ 
        if (! ((unsigned int )((void const   *)("self" + 1)) - (unsigned int )((void const   *)"self") == 1U)) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6329\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___6 = 1;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6332\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (__s2_len >= 4U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6330\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___6 = 1;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6331\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___6 = 0;
          }
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6334\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___6 = 0;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6343\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___6) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6336\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = __builtin_strcmp((char const   *)lcname, "self");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6337\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___5 = tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6338\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___4 = __builtin_strcmp((char const   *)lcname, "self");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6339\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___5 = tmp___4;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6344\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___4 = __builtin_strcmp((char const   *)lcname, "self");
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6345\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___5 = tmp___4;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6439\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___5) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6366\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6357\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __s1_len___0 = strlen((char const   *)lcname);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6358\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __s2_len___0 = strlen("parent");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6359\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! ((unsigned int )((void const   *)(lcname + 1)) - (unsigned int )((void const   *)lcname) == 1U)) {
        goto _L___2;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6352\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (__s1_len___0 >= 4U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6350\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L___2: /* CIL Label */ 
          if (! ((unsigned int )((void const   *)("parent" + 1)) - (unsigned int )((void const   *)"parent") == 1U)) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6346\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___13 = 1;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6349\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (__s2_len___0 >= 4U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "6347\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___13 = 1;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "6348\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___13 = 0;
            }
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6351\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___13 = 0;
        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6360\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___13) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6353\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___8 = __builtin_strcmp((char const   *)lcname, "parent");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6354\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___12 = tmp___8;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6355\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___11 = __builtin_strcmp((char const   *)lcname, "parent");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6356\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___12 = tmp___11;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6361\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___11 = __builtin_strcmp((char const   *)lcname, "parent");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6362\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___12 = tmp___11;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6367\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___12) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6363\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6364\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)lcname);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6365\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Cannot use \'%s\' as class name as it is reserved",
                 class_name->u.constant.value.str.val);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6368\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)lcname);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6369\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Cannot use \'%s\' as class name as it is reserved",
               class_name->u.constant.value.str.val);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6440\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.current_import) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6372\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___14 = zend_hash_find((HashTable const   *)compiler_globals.current_import,
                              (char const   *)lcname,
                              (unsigned int )(class_name->u.constant.value.str.len + 1),
                              (void **)(& ns_name));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6373\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___14 == (int __attribute__((__visibility__("default")))  )0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6370\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      error = 1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6371\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6374\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6441\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.current_namespace) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6375\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___15.u.constant = *(compiler_globals.current_namespace);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6376\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_copy_ctor(& tmp___15.u.constant);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6377\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_build_namespace_name(& tmp___15, & tmp___15, class_name);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6378\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    class_name = & tmp___15;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6379\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)lcname);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6380\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___16 = zend_str_tolower_dup((char const   *)class_name->u.constant.value.str.val,
                                    (unsigned int )class_name->u.constant.value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6381\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    lcname = (char *)tmp___16;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6382\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6442\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (error) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6388\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___18 = zend_str_tolower_dup((char const   *)(*ns_name)->value.str.val,
                                    (unsigned int )(*ns_name)->value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6389\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___17 = (char *)tmp___18;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6390\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((*ns_name)->value.str.len != class_name->u.constant.value.str.len) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6383\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L,
                 "Cannot declare class %s because the name is already in use",
                 class_name->u.constant.value.str.val);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6386\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___19 = memcmp((void const   *)tmp___17, (void const   *)lcname,
                        (unsigned int )class_name->u.constant.value.str.len);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6387\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___19) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6384\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L,
                   "Cannot declare class %s because the name is already in use",
                   class_name->u.constant.value.str.val);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6385\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6391\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)tmp___17);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6392\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6443\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___20 = _emalloc(sizeof(zend_class_entry ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6444\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  new_class_entry = (zend_class_entry *)tmp___20;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6445\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  new_class_entry->type = (char)2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6446\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___21 = (*zend_new_interned_string)((char const   *)class_name->u.constant.value.str.val,
                                         class_name->u.constant.value.str.len + 1,
                                         1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6447\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  new_class_entry->name = (char const   *)tmp___21;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6448\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  new_class_entry->name_length = (unsigned int )class_name->u.constant.value.str.len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6449\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_initialize_class_data(new_class_entry, (unsigned char)1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6450\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___22 = zend_get_compiled_filename();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6451\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  new_class_entry->info.user.filename = (char const   *)tmp___22;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6452\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  new_class_entry->info.user.line_start = (unsigned int )class_token->u.op.opline_num;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6453\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  new_class_entry->ce_flags |= (unsigned int )class_token->EA;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6454\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (parent_class_name) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6399\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (parent_class_name->op_type != (int const   )(1 << 3)) {
      switch ((int )parent_class_name->EA) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6393\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 1: 
      zend_error(1 << 6L, "Cannot use \'self\' as class name as it is reserved");
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6394\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 2: 
      zend_error(1 << 6L,
                 "Cannot use \'parent\' as class name as it is reserved");
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6395\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 7: 
      zend_error(1 << 6L,
                 "Cannot use \'static\' as class name as it is reserved");
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6396\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      default: ;
      break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6397\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      doing_inheritance = 1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6398\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6400\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6455\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6456\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6457\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  build_runtime_defined_function_key(& key, (char const   *)lcname,
                                     (int )new_class_entry->name_length);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6458\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___23 = zend_add_literal(compiler_globals.active_op_array,
                              (zval const   *)(& key));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6459\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1.constant = (unsigned int )tmp___23;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6460\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___24 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                            (unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6461\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = (unsigned long )tmp___24;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6462\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6463\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (doing_inheritance) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6403\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((new_class_entry->ce_flags & 288U) == 288U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6401\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "A trait (%s) cannot extend a class",
                 new_class_entry->name);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6402\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6404\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value = (unsigned long )parent_class_name->u.op.var;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6405\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)140;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6406\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)139;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6464\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6413\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6407\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __s = (char const   *)lcname;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6408\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __l = (int )new_class_entry->name_length;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6409\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z = & _c;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6410\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->value.str.len = __l;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6411\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6412\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->type = (unsigned char)6;
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6414\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___26 = zend_add_literal(compiler_globals.active_op_array,
                                (zval const   *)(& _c));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6415\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2.constant = (unsigned int )tmp___26;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6465\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6422\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6419\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6416\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val - sizeof(Bucket )))->h;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6417\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___27 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val,
                                  (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6418\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = (unsigned long )tmp___27;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6420\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___27 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val,
                                (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len + 1));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6421\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = (unsigned long )tmp___27;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6466\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zend_hash_quick_add_or_update(compiler_globals.class_table,
                                 (char const   *)key.value.str.val,
                                 (unsigned int )key.value.str.len,
                                 ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value,
                                 (void *)(& new_class_entry),
                                 sizeof(zend_class_entry *),
                                 (void **)((void *)0), 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6467\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.active_class_entry = new_class_entry;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6468\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6469\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6470\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6426\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.implementing_class.op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6427\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.implementing_class.op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6423\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      compiler_globals.implementing_class.u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6424\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      compiler_globals.implementing_class.u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6425\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      compiler_globals.implementing_class.EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6471\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.doc_comment) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6428\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.active_class_entry)->info.user.doc_comment = (char const   *)compiler_globals.doc_comment;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6429\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.active_class_entry)->info.user.doc_comment_len = compiler_globals.doc_comment_len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6430\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment = (char *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6431\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment_len = 0U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6432\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6472\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_begin_class_declaration(znode const   *class_token ,
                                     znode *class_name ,
                                     znode const   *parent_class_name ) ;
struct _zend_switch_entry;
typedef struct _zend_switch_entry zend_switch_entry;
struct _zend_switch_entry {
   znode cond ;
   int default_case ;
   int control_var ;
};
void zend_do_default_before_statement(znode const   *case_list ,
                                      znode *default_token ) 
{ int next_op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *tmp___0 ;
  zend_switch_entry *switch_entry_ptr ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6312\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6313\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  next_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6314\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6315\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6316\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.switch_cond_stack),
                 (void **)(& switch_entry_ptr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6317\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)42;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6318\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6319\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6320\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  default_token->u.op.opline_num = (unsigned int )next_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6321\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  next_op_number = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6322\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  switch_entry_ptr->default_case = next_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6323\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (case_list->op_type == (int const   )(1 << 3)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6310\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6311\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6324\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + case_list->u.op.opline_num)->op1.opline_num = (unsigned int )next_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6325\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_default_before_statement(znode const   *case_list ,
                                      znode *default_token ) ;
void zend_do_case_after_statement(znode *result , znode const   *case_token ) 
{ int next_op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6301\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6302\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  next_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6303\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6304\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6305\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)42;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6306\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6307\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6308\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->u.op.opline_num = (unsigned int )next_op_number;
  switch ((int )((compiler_globals.active_op_array)->opcodes + case_token->u.op.opline_num)->opcode) {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6297\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 42: 
  tmp___1 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6298\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + case_token->u.op.opline_num)->op1.opline_num = (unsigned int )tmp___1;
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6299\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 43: 
  tmp___2 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6300\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + case_token->u.op.opline_num)->op2.opline_num = (unsigned int )tmp___2;
  break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6309\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_case_after_statement(znode *result , znode const   *case_token ) ;
void zend_do_case_before_statement(znode const   *case_list ,
                                   znode *case_token , znode const   *case_expr ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int next_op_number ;
  zend_switch_entry *switch_entry_ptr ;
  znode result ;
  zend_uint tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6276\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6277\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6278\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.switch_cond_stack),
                 (void **)(& switch_entry_ptr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6279\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (switch_entry_ptr->control_var == -1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6249\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = get_temporary_variable(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6250\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    switch_entry_ptr->control_var = (int )tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6251\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6280\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)48;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6281\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = (unsigned int )switch_entry_ptr->control_var;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6282\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6283\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6255\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )switch_entry_ptr->cond.op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6256\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (switch_entry_ptr->cond.op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6252\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& switch_entry_ptr->cond.u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6253\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6254\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = switch_entry_ptr->cond.u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6284\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6260\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )case_expr->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6261\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (case_expr->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6257\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                 & case_expr->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6258\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___2;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6259\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2 = (union _znode_op )case_expr->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6285\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )opline->op1_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6262\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_copy_ctor(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6263\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6286\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6267\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result.op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6268\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result.op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6264\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result.u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6265\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result.u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6266\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result.EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6287\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  next_op_number = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6288\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6289\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)43;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6290\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6272\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )result.op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6273\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result.op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6269\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___3 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& result.u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6270\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___3;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6271\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = result.u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6291\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6292\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case_token->u.op.opline_num = (unsigned int )next_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6293\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (case_list->op_type == (int const   )(1 << 3)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6274\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6275\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6294\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  next_op_number = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6295\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + case_list->u.op.opline_num)->op1.opline_num = (unsigned int )next_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6296\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_case_before_statement(znode const   *case_list ,
                                   znode *case_token , znode const   *case_expr ) ;
struct _zend_brk_cont_element {
   int start ;
   int cont ;
   int brk ;
   int parent ;
};
void zend_do_switch_end(znode const   *case_list ) 
{ zend_op *opline ;
  zend_switch_entry *switch_entry_ptr ;
  int next_op_number ;
  int tmp ;
  int tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6237\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.switch_cond_stack),
                 (void **)(& switch_entry_ptr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6238\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (switch_entry_ptr->default_case != -1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6210\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline = get_next_op(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6211\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)42;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6212\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6213\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6214\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1.opline_num = (unsigned int )switch_entry_ptr->default_case;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6215\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6239\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (case_list->op_type != (int const   )(1 << 3)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6216\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = get_next_op_number(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6217\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    next_op_number = tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6218\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->opcodes + case_list->u.op.opline_num)->op1.opline_num = (unsigned int )next_op_number;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6219\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6240\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6241\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->brk_cont_array + compiler_globals.context.current_brk_cont)->brk = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6242\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->brk_cont_array + compiler_globals.context.current_brk_cont)->cont = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6243\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.context.current_brk_cont = ((compiler_globals.active_op_array)->brk_cont_array + compiler_globals.context.current_brk_cont)->parent;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6244\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (switch_entry_ptr->cond.op_type == 1 << 2) {
    goto _L;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6232\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (switch_entry_ptr->cond.op_type == 1 << 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6227\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L: /* CIL Label */ 
      opline = get_next_op(compiler_globals.active_op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6228\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (switch_entry_ptr->cond.op_type == 1 << 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6220\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->opcode = (unsigned char)70;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6221\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->opcode = (unsigned char)49;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6229\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6225\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1_type = (unsigned char )switch_entry_ptr->cond.op_type;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6226\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (switch_entry_ptr->cond.op_type == 1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6222\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                     (zval const   *)(& switch_entry_ptr->cond.u.constant));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6223\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op1.constant = (unsigned int )tmp___1;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6224\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op1 = switch_entry_ptr->cond.u.op;
        }
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6230\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2_type = (unsigned char )(1 << 3);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6231\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6245\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (switch_entry_ptr->cond.op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6233\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_dtor(& switch_entry_ptr->cond.u.constant);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6234\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6246\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_del_top(& compiler_globals.switch_cond_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6247\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6235\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) --;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6236\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6248\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_switch_end(znode const   *case_list ) ;
void zend_do_switch_cond(znode const   *cond ) 
{ zend_switch_entry switch_entry ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6203\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  switch_entry.cond = (struct _znode )*cond;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6204\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  switch_entry.default_case = -1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6205\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  switch_entry.control_var = -1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6206\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.switch_cond_stack,
                  (void const   *)((void *)(& switch_entry)),
                  (int )sizeof(switch_entry));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6207\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  do_begin_loop();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6208\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6201\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) ++;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6202\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6209\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_switch_cond(znode const   *cond ) ;
void zend_do_brk_cont(zend_uchar op , znode const   *expr ) 
{ zend_op *opline ;
  zend_op *tmp ;
  char const   *tmp___0 ;
  char const   *tmp___1 ;
  int tmp___2 ;
  zval _c ;
  zval *__z ;
  int tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6194\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6195\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6196\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = op;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6197\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1.opline_num = (unsigned int )compiler_globals.context.current_brk_cont;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6198\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6199\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (expr) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6185\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr->op_type != 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6171\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )op == 50) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6169\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = "break";
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6170\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = "continue";
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6172\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L,
                 "\'%s\' operator with non-constant operand is no longer supported",
                 tmp___0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6179\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )expr->u.constant.type != 1) {
        goto _L;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6178\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (expr->u.constant.value.lval < 1L) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6175\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L: /* CIL Label */ 
          if ((int )op == 50) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6173\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___1 = "break";
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "6174\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___1 = "continue";
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6176\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L, "\'%s\' operator accepts only positive numbers",
                     tmp___1);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "6177\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6186\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6183\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2_type = (unsigned char )expr->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6184\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (expr->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6180\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                   & expr->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6181\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2.constant = (unsigned int )tmp___2;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6182\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2 = (union _znode_op )expr->u.op;
      }
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6192\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6187\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z = & _c;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6188\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->value.lval = 1L;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6189\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->type = (unsigned char)1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6190\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___3 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& _c));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6191\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___3;
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6193\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char)1;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6200\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_brk_cont(zend_uchar op , znode const   *expr ) ;
void zend_do_do_while_end(znode const   *do_token ,
                          znode const   *expr_open_bracket ,
                          znode const   *expr ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6160\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6161\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6162\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)44;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6163\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6156\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )expr->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6157\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6153\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & expr->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6154\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6155\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )expr->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6164\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2.opline_num = (unsigned int )do_token->u.op.opline_num;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6165\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6166\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  do_end_loop((int )expr_open_bracket->u.op.opline_num, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6167\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6158\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) --;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6159\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6168\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_do_while_end(znode const   *do_token ,
                          znode const   *expr_open_bracket ,
                          znode const   *expr ) ;
void zend_do_do_while_begin(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6150\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  do_begin_loop();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6151\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6148\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) ++;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6149\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6152\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_do_while_begin(void) ;
void zend_do_boolean_and_end(znode *result , znode const   *expr1 ,
                             znode const   *expr2 , znode const   *op_token ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6138\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6139\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6140\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *result = (struct _znode )*expr1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6141\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)52;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6142\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6131\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )result->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6132\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6128\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& result->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6129\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6130\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result = result->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6143\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6136\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )expr2->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6137\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr2->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6133\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & expr2->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6134\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6135\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )expr2->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6144\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6145\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6146\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + op_token->u.op.opline_num)->op2.opline_num = (unsigned int )tmp___2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6147\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_boolean_and_end(znode *result , znode const   *expr1 ,
                             znode const   *expr2 , znode const   *op_token ) ;
void zend_do_boolean_and_begin(znode *expr1 , znode *op_token ) 
{ int next_op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6117\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6118\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  next_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6119\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6120\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6121\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)46;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6122\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (expr1->op_type == 1 << 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6104\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6102\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result_type = (unsigned char )expr1->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6103\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (expr1->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6099\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                   (zval const   *)(& expr1->u.constant));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6100\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->result.constant = (unsigned int )tmp___1;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6101\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->result = expr1->u.op;
      }
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6105\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6106\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )(1 << 1);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6123\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6110\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )expr1->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6111\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr1->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6107\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& expr1->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6108\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___2;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6109\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = expr1->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6124\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6125\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  op_token->u.op.opline_num = (unsigned int )next_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6126\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6115\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    expr1->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6116\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr1->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6112\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      expr1->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6113\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      expr1->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6114\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      expr1->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6127\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_boolean_and_begin(znode *expr1 , znode *op_token ) ;
void zend_do_boolean_or_end(znode *result , znode const   *expr1 ,
                            znode const   *expr2 , znode *op_token ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6089\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6090\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6091\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *result = (struct _znode )*expr1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6092\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)52;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6093\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6082\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )result->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6083\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6079\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& result->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6080\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6081\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result = result->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6094\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6087\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )expr2->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6088\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr2->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6084\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & expr2->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6085\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6086\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )expr2->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6095\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6096\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6097\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + op_token->u.op.opline_num)->op2.opline_num = (unsigned int )tmp___2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6098\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_boolean_or_end(znode *result , znode const   *expr1 ,
                            znode const   *expr2 , znode *op_token ) ;
void zend_do_boolean_or_begin(znode *expr1 , znode *op_token ) 
{ int next_op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6068\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6069\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  next_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6070\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6071\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6072\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)47;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6073\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (expr1->op_type == 1 << 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6055\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6053\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result_type = (unsigned char )expr1->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6054\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (expr1->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6050\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                   (zval const   *)(& expr1->u.constant));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6051\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->result.constant = (unsigned int )tmp___1;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6052\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->result = expr1->u.op;
      }
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6056\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6057\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )(1 << 1);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6074\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6061\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )expr1->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6062\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr1->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6058\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& expr1->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6059\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___2;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6060\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = expr1->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6075\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6076\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  op_token->u.op.opline_num = (unsigned int )next_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6077\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6066\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    expr1->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6067\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr1->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6063\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      expr1->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6064\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      expr1->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6065\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      expr1->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6078\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_boolean_or_begin(znode *expr1 , znode *op_token ) ;
zend_class_entry __attribute__((__visibility__("default")))  *do_bind_inherited_class(zend_op_array const   *op_array ,
                                                                                      zend_op const   *opline ,
                                                                                      HashTable *class_table ,
                                                                                      zend_class_entry *parent_ce ,
                                                                                      zend_bool compile_time ) ;
extern int __attribute__((__visibility__("default")))  zend_lookup_class(char const   *name ,
                                                                         int name_length ,
                                                                         zend_class_entry ***ce ) ;
void __attribute__((__visibility__("default")))  zend_do_delayed_early_binding(zend_op_array const   *op_array ) 
{ zend_bool orig_in_compilation ;
  zend_uint opline_num ;
  zend_class_entry **pce ;
  int __attribute__((__visibility__("default")))  tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6048\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (op_array->early_binding != 4294967295U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6042\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    orig_in_compilation = compiler_globals.in_compilation;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6043\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline_num = (zend_uint )op_array->early_binding;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6044\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.in_compilation = (unsigned char)1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6045\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6038\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (opline_num != 4294967295U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6035\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6039\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = zend_lookup_class((char const   *)((op_array->opcodes + (opline_num - 1U))->op2.zv)->value.str.val,
                              ((op_array->opcodes + (opline_num - 1U))->op2.zv)->value.str.len,
                              & pce);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6040\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp == (int __attribute__((__visibility__("default")))  )0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6036\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        do_bind_inherited_class(op_array,
                                (zend_op const   *)(op_array->opcodes + opline_num),
                                executor_globals.class_table, *pce,
                                (unsigned char)0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "6037\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "6041\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline_num = (op_array->opcodes + opline_num)->result.opline_num;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6046\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.in_compilation = orig_in_compilation;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "6047\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6049\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void __attribute__((__visibility__("default")))  zend_do_delayed_early_binding(zend_op_array const   *op_array ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_del_key_or_index(HashTable *ht ,
                                                                                  char const   *arKey ,
                                                                                  uint nKeyLength ,
                                                                                  ulong h ,
                                                                                  int flag ) ;
zend_class_entry __attribute__((__visibility__("default")))  *do_bind_class(zend_op_array const   *op_array ,
                                                                            zend_op const   *opline ,
                                                                            HashTable *class_table ,
                                                                            zend_bool compile_time ) ;
int __attribute__((__visibility__("default")))  do_bind_function(zend_op_array const   *op_array ,
                                                                 zend_op *opline ,
                                                                 HashTable *function_table ,
                                                                 zend_bool compile_time ) ;
void zend_do_early_binding(void) 
{ zend_op *opline ;
  HashTable *table ;
  int __attribute__((__visibility__("default")))  tmp ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp___0 ;
  zend_op *fetch_class_opline ;
  zval *parent_name ;
  zend_class_entry **pce ;
  zend_uint *opline_num ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp___2 ;
  zend_uchar tmp___3 ;
  zend_uchar tmp___4 ;
  zend_uchar tmp___5 ;
  zend_uchar tmp___6 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6020\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = (compiler_globals.active_op_array)->opcodes + ((compiler_globals.active_op_array)->last - 1U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6021\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5970\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )opline->opcode == 105) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5969\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )opline > (unsigned int )(compiler_globals.active_op_array)->opcodes) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5968\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5971\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline --;
  }
  switch ((int )opline->opcode) {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5994\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 141: 
  tmp = do_bind_function((zend_op_array const   *)compiler_globals.active_op_array,
                         opline, compiler_globals.function_table,
                         (unsigned char)1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5995\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp == (int __attribute__((__visibility__("default")))  )-1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5972\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5973\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5996\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  table = compiler_globals.function_table;
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5997\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 139: 
  tmp___0 = do_bind_class((zend_op_array const   *)compiler_globals.active_op_array,
                          (zend_op const   *)opline,
                          compiler_globals.class_table, (unsigned char)1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5998\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )tmp___0 == (unsigned int )((void *)0)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5974\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5975\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5999\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  table = compiler_globals.class_table;
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6000\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 140: 
  fetch_class_opline = opline - 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6001\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  parent_name = & ((compiler_globals.active_op_array)->literals + fetch_class_opline->op2.constant)->constant;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6002\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_lookup_class((char const   *)parent_name->value.str.val,
                              parent_name->value.str.len, & pce);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6003\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
    goto _L;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5991\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.compiler_options & (unsigned int )(1 << 3)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5989\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )(*pce)->type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5986\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L: /* CIL Label */ 
        if (compiler_globals.compiler_options & (unsigned int )(1 << 4)) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5979\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline_num = & (compiler_globals.active_op_array)->early_binding;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5980\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5977\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (*opline_num != 4294967295U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5976\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            } else {
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5978\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            opline_num = & ((compiler_globals.active_op_array)->opcodes + *opline_num)->result.opline_num;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5981\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *opline_num = (unsigned int )(opline - (compiler_globals.active_op_array)->opcodes);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5982\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->opcode = (unsigned char)145;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5983\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->result_type = (unsigned char )(1 << 3);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5984\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->result.opline_num = 4294967295U;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5985\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5987\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5988\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5990\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6004\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = do_bind_inherited_class((zend_op_array const   *)compiler_globals.active_op_array,
                                    (zend_op const   *)opline,
                                    compiler_globals.class_table, *pce,
                                    (unsigned char)1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6005\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )tmp___2 == (unsigned int )((void *)0)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5992\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5993\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6006\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_del_literal(compiler_globals.active_op_array,
                   (int )fetch_class_opline->op2.constant);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6007\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fetch_class_opline->opcode = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6008\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  memset((void *)(& fetch_class_opline->result), 0,
         sizeof(fetch_class_opline->result));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6009\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  memset((void *)(& fetch_class_opline->op1), 0, sizeof(fetch_class_opline->op1));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6010\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  memset((void *)(& fetch_class_opline->op2), 0, sizeof(fetch_class_opline->op2));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6011\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6012\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fetch_class_opline->op2_type = tmp___4;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6013\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = tmp___4;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6014\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fetch_class_opline->op1_type = tmp___3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6015\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fetch_class_opline->result_type = tmp___3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6016\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  table = compiler_globals.class_table;
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6017\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 146: 
  case 144: 
  case 154: 
  case 155: 
  return;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6018\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  default: 
  zend_error(1 << 6L, "Invalid binding type");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6019\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6022\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_hash_del_key_or_index(table,
                             (char const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                             (unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len,
                             ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value,
                             2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6023\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_del_literal(compiler_globals.active_op_array, (int )opline->op1.constant);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6024\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_del_literal(compiler_globals.active_op_array, (int )opline->op2.constant);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6025\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6026\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  memset((void *)(& opline->result), 0, sizeof(opline->result));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6027\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  memset((void *)(& opline->op1), 0, sizeof(opline->op1));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6028\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  memset((void *)(& opline->op2), 0, sizeof(opline->op2));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6029\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___6 = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6030\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = tmp___6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6031\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___5 = tmp___6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6032\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = tmp___5;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6033\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = tmp___5;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6034\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_early_binding(void) ;
void __attribute__((__visibility__("default")))  zend_do_inheritance(zend_class_entry *ce ,
                                                                     zend_class_entry *parent_ce ) ;
zend_class_entry __attribute__((__visibility__("default")))  *do_bind_inherited_class(zend_op_array const   *op_array ,
                                                                                      zend_op const   *opline ,
                                                                                      HashTable *class_table ,
                                                                                      zend_class_entry *parent_ce ,
                                                                                      zend_bool compile_time ) 
{ zend_class_entry *ce ;
  zend_class_entry **pce ;
  int found_ce ;
  zval *op1 ;
  zval *op2 ;
  int __attribute__((__visibility__("default")))  tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5958\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compile_time) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5943\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op1 = & (op_array->literals + opline->op1.constant)->constant;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5944\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op2 = & (op_array->literals + opline->op2.constant)->constant;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5945\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op1 = (zval *)opline->op1.zv;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5946\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op2 = (zval *)opline->op2.zv;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5959\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_hash_quick_find((HashTable const   *)class_table,
                             (char const   *)op1->value.str.val,
                             (unsigned int )op1->value.str.len,
                             ((zend_literal *)op1)->hash_value, (void **)(& pce));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5960\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  found_ce = (int )tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5961\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (found_ce == -1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5949\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! compile_time) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5947\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Cannot redeclare class %s", op2->value.str.val);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5948\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5950\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((zend_class_entry __attribute__((__visibility__("default")))  *)((void *)0));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5951\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce = *pce;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5962\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (parent_ce->ce_flags & 128U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5952\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Class %s cannot extend from interface %s", ce->name,
               parent_ce->name);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5955\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((parent_ce->ce_flags & 288U) == 288U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5953\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Class %s cannot extend from trait %s", ce->name,
                 parent_ce->name);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5954\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5963\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_inheritance(ce, parent_ce);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5964\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (ce->refcount) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5965\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = _zend_hash_quick_add_or_update(class_table,
                                           (char const   *)op2->value.str.val,
                                           (unsigned int )(op2->value.str.len + 1),
                                           ((zend_literal *)op2)->hash_value,
                                           (void *)pce,
                                           sizeof(zend_class_entry *),
                                           (void **)((void *)0), 1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5966\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5956\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Cannot redeclare class %s", ce->name);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5957\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5967\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((zend_class_entry __attribute__((__visibility__("default")))  *)ce);
}
}
zend_class_entry __attribute__((__visibility__("default")))  *do_bind_class(zend_op_array const   *op_array ,
                                                                            zend_op const   *opline ,
                                                                            HashTable *class_table ,
                                                                            zend_bool compile_time ) 
{ zend_class_entry *ce ;
  zend_class_entry **pce ;
  zval *op1 ;
  zval *op2 ;
  int __attribute__((__visibility__("default")))  tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5937\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compile_time) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5921\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op1 = & (op_array->literals + opline->op1.constant)->constant;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5922\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op2 = & (op_array->literals + opline->op2.constant)->constant;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5923\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op1 = (zval *)opline->op1.zv;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5924\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op2 = (zval *)opline->op2.zv;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5938\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_hash_quick_find((HashTable const   *)class_table,
                             (char const   *)op1->value.str.val,
                             (unsigned int )op1->value.str.len,
                             ((zend_literal *)op1)->hash_value, (void **)(& pce));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5939\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp == (int __attribute__((__visibility__("default")))  )-1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5925\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L,
               "Internal Zend error - Missing class information for %s",
               op1->value.str.val);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5926\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((zend_class_entry __attribute__((__visibility__("default")))  *)((void *)0));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5927\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce = *pce;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5940\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (ce->refcount) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5941\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = _zend_hash_quick_add_or_update(class_table,
                                           (char const   *)op2->value.str.val,
                                           (unsigned int )(op2->value.str.len + 1),
                                           ((zend_literal *)op2)->hash_value,
                                           (void *)(& ce),
                                           sizeof(zend_class_entry *),
                                           (void **)((void *)0), 1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5942\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5930\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (ce->refcount) --;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5931\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! compile_time) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5928\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Cannot redeclare class %s", ce->name);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5929\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5932\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((zend_class_entry __attribute__((__visibility__("default")))  *)((void *)0));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5935\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! (ce->ce_flags & 524416U)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5933\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_verify_abstract_class(ce);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5934\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5936\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((zend_class_entry __attribute__((__visibility__("default")))  *)ce);
  }
}
}
struct _zend_trait_method_reference;
typedef struct _zend_trait_method_reference zend_trait_method_reference;
struct _zend_trait_precedence {
   zend_trait_method_reference *trait_method ;
   zend_class_entry **exclude_from_classes ;
   union _zend_function *function ;
};
void zend_prepare_trait_precedence(znode *result , znode *method_reference ,
                                   znode *trait_list ) 
{ zend_trait_precedence *trait_precedence ;
  void __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5914\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _emalloc(sizeof(zend_trait_precedence ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5915\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  trait_precedence = (zend_trait_precedence *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5916\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  trait_precedence->trait_method = (zend_trait_method_reference *)method_reference->u.op.ptr;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5917\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  trait_precedence->exclude_from_classes = (zend_class_entry **)trait_list->u.op.ptr;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5918\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  trait_precedence->function = (union _zend_function *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5919\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->u.op.ptr = (void *)trait_precedence;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5920\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_prepare_trait_precedence(znode *result , znode *method_reference ,
                                   znode *trait_list ) ;
struct _zend_trait_alias {
   zend_trait_method_reference *trait_method ;
   char const   *alias ;
   unsigned int alias_len ;
   zend_uint modifiers ;
   union _zend_function *function ;
};
void zend_prepare_trait_alias(znode *result , znode *method_reference ,
                              znode *modifiers , znode *alias ) 
{ zend_trait_alias *trait_alias ;
  void __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5905\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _emalloc(sizeof(zend_trait_alias ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5906\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  trait_alias = (zend_trait_alias *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5907\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  trait_alias->trait_method = (zend_trait_method_reference *)method_reference->u.op.ptr;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5908\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  trait_alias->modifiers = (unsigned int )modifiers->u.constant.value.lval;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5909\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (modifiers->u.constant.value.lval == 1L) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5899\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Cannot use \'static\' as method modifier");
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5900\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5901\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5910\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (alias) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5902\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    trait_alias->alias = (char const   *)alias->u.constant.value.str.val;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5903\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    trait_alias->alias_len = (unsigned int )alias->u.constant.value.str.len;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5904\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    trait_alias->alias = (char const   *)((void *)0);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5911\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  trait_alias->function = (union _zend_function *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5912\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->u.op.ptr = (void *)trait_alias;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5913\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_prepare_trait_alias(znode *result , znode *method_reference ,
                              znode *modifiers , znode *alias ) ;
struct _zend_trait_method_reference {
   char const   *method_name ;
   unsigned int mname_len ;
   zend_class_entry *ce ;
   char const   *class_name ;
   unsigned int cname_len ;
};
void zend_prepare_reference(znode *result , znode *class_name ,
                            znode *method_name ) 
{ zend_trait_method_reference *method_ref ;
  void __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5890\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _emalloc(sizeof(zend_trait_method_reference ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5891\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  method_ref = (zend_trait_method_reference *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5892\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  method_ref->ce = (zend_class_entry *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5893\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_name) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5885\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_resolve_class_name(class_name, 4UL, 1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5886\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    method_ref->class_name = (char const   *)class_name->u.constant.value.str.val;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5887\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    method_ref->cname_len = (unsigned int )class_name->u.constant.value.str.len;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5888\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    method_ref->class_name = (char const   *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5889\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    method_ref->cname_len = 0U;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5894\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  method_ref->method_name = (char const   *)method_name->u.constant.value.str.val;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5895\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  method_ref->mname_len = (unsigned int )method_name->u.constant.value.str.len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5896\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->u.op.ptr = (void *)method_ref;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5897\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->op_type = 1 << 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5898\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_prepare_reference(znode *result , znode *class_name ,
                            znode *method_name ) ;
void zend_add_trait_alias(znode *alias_znode ) 
{ zend_class_entry *ce ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5882\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce = compiler_globals.active_class_entry;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5883\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_add_to_list((void *)(& ce->trait_aliases), alias_znode->u.op.ptr);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5884\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_add_trait_alias(znode *alias_znode ) ;
void zend_add_trait_precedence(znode *precedence_znode ) 
{ zend_class_entry *ce ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5879\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce = compiler_globals.active_class_entry;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5880\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_add_to_list((void *)(& ce->trait_precedences), precedence_znode->u.op.ptr);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5881\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_add_trait_precedence(znode *precedence_znode ) ;
int __attribute__((__visibility__("default")))  do_bind_function(zend_op_array const   *op_array ,
                                                                 zend_op *opline ,
                                                                 HashTable *function_table ,
                                                                 zend_bool compile_time ) 
{ zend_function *function ;
  zval *op1 ;
  zval *op2 ;
  int error_level ;
  int tmp ;
  zend_function *old_function ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5875\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compile_time) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5855\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op1 = & (op_array->literals + opline->op1.constant)->constant;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5856\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op2 = & (op_array->literals + opline->op2.constant)->constant;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5857\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op1 = opline->op1.zv;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5858\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op2 = opline->op2.zv;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5876\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_hash_quick_find((HashTable const   *)function_table,
                       (char const   *)op1->value.str.val,
                       (unsigned int )op1->value.str.len,
                       ((zend_literal *)op1)->hash_value,
                       (void **)((void *)(& function)));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5877\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = _zend_hash_quick_add_or_update(function_table,
                                           (char const   *)op2->value.str.val,
                                           (unsigned int )(op2->value.str.len + 1),
                                           ((zend_literal *)op2)->hash_value,
                                           (void *)function,
                                           sizeof(zend_function ),
                                           (void **)((void *)0), 1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5878\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5867\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compile_time) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5859\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = 1 << 6L;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5860\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = 1;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5868\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    error_level = tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5869\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = zend_hash_quick_find((HashTable const   *)function_table,
                                   (char const   *)op2->value.str.val,
                                   (unsigned int )(op2->value.str.len + 1),
                                   ((zend_literal *)op2)->hash_value,
                                   (void **)((void *)(& old_function)));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5870\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0 == (int __attribute__((__visibility__("default")))  )0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5865\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )old_function->type == 2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5863\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (old_function->op_array.last > 0U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5861\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(error_level,
                     "Cannot redeclare %s() (previously declared in %s:%d)",
                     function->common.function_name,
                     old_function->op_array.filename,
                     (old_function->op_array.opcodes + 0)->lineno);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5862\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(error_level, "Cannot redeclare %s()",
                     function->common.function_name);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5864\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(error_level, "Cannot redeclare %s()",
                   function->common.function_name);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5866\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(error_level, "Cannot redeclare %s()",
                 function->common.function_name);
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5871\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )-1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5872\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(function->op_array.refcount)) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5873\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    function->op_array.static_variables = (HashTable *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5874\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((int __attribute__((__visibility__("default")))  )0);
  }
}
}
static void zend_do_traits_property_binding(zend_class_entry *ce ) ;
static void zend_do_check_for_inconsistent_traits_aliasing(zend_class_entry *ce ) ;
static void zend_do_traits_method_binding(zend_class_entry *ce ) ;
static void zend_traits_init_trait_structures(zend_class_entry *ce ) ;
void __attribute__((__visibility__("default")))  zend_do_bind_traits(zend_class_entry *ce ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5847\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->num_traits <= 0U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5843\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5844\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5848\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_traits_init_trait_structures(ce);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5849\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_traits_method_binding(ce);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5850\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_check_for_inconsistent_traits_aliasing(ce);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5851\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_traits_property_binding(ce);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5852\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_verify_abstract_class(ce);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5853\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->ce_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5845\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->ce_flags -= 16U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5846\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5854\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void __attribute__((__visibility__("default")))  zend_do_bind_traits(zend_class_entry *ce ) ;
static void zend_do_check_for_inconsistent_traits_aliasing(zend_class_entry *ce ) 
{ int i ;
  zend_trait_alias *cur_alias ;
  char *lc_method_name ;
  char __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5840\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5841\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->trait_aliases) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5838\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5834\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (*(ce->trait_aliases + i)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5822\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5835\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      cur_alias = *(ce->trait_aliases + i);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5836\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! (cur_alias->trait_method)->ce) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5832\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (cur_alias->alias) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5823\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L,
                     "An alias (%s) was defined for method %s(), but this method does not exist",
                     cur_alias->alias, (cur_alias->trait_method)->method_name);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5828\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp = zend_str_tolower_dup((cur_alias->trait_method)->method_name,
                                     (cur_alias->trait_method)->mname_len);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5829\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          lc_method_name = (char *)tmp;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5830\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___0 = zend_hash_exists((HashTable const   *)(& ce->function_table),
                                     (char const   *)lc_method_name,
                                     (cur_alias->trait_method)->mname_len + 1U);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5831\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___0) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5824\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _efree((void *)lc_method_name);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5825\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_error(1 << 6L,
                       "The modifiers for the trait alias %s() need to be changed in the same statment in which the alias is defined. Error",
                       (cur_alias->trait_method)->method_name);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5826\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _efree((void *)lc_method_name);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5827\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_error(1 << 6L,
                       "The modifiers of the trait method %s() are changed, but this method does not exist. Error",
                       (cur_alias->trait_method)->method_name);
          }
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5833\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5837\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i ++;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5839\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5842\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
typedef Bucket *HashPosition;
extern int __attribute__((__visibility__("default")))  zend_hash_move_forward_ex(HashTable *ht ,
                                                                                 HashPosition *pos ) ;
__inline static zend_uint ( __attribute__((__always_inline__)) zval_addref_p)(zval *pz ) ;
static zend_class_entry *find_first_definition(zend_class_entry *ce ,
                                               size_t current_trait ,
                                               char const   *prop_name ,
                                               int prop_name_length ,
                                               ulong prop_hash ,
                                               zend_class_entry *coliding_ce ) ;
extern int __attribute__((__visibility__("default")))  compare_function(zval *result ,
                                                                        zval *op1 ,
                                                                        zval *op2 ) ;
static void zend_traits_register_private_property(zend_class_entry *ce ,
                                                  char const   *name ,
                                                  int name_len ,
                                                  zend_property_info *old_info ,
                                                  zval *property ) ;
extern ulong __attribute__((__visibility__("default")))  zend_get_hash_value(char const   *arKey ,
                                                                             uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_get_current_data_ex(HashTable *ht ,
                                                                                     void **pData ,
                                                                                     HashPosition *pos ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_internal_pointer_reset_ex(HashTable *ht ,
                                                                                            HashPosition *pos ) ;
static void zend_do_traits_property_binding(zend_class_entry *ce ) 
{ size_t i ;
  zend_property_info *property_info ;
  zend_property_info *coliding_prop ;
  zval compare_result ;
  char const   *prop_name ;
  int prop_name_length ;
  ulong prop_hash ;
  char const   *class_name_unused ;
  zend_bool not_compatible ;
  zval *prop_value ;
  char *doc_comment ;
  size_t tmp ;
  ulong __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  int tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  int tmp___4 ;
  zend_class_entry *tmp___5 ;
  zend_class_entry *tmp___6 ;
  int __attribute__((__visibility__("default")))  tmp___7 ;
  char __attribute__((__visibility__("default")))  *tmp___8 ;
  int __attribute__((__visibility__("default")))  tmp___9 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5819\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5820\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5815\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (i < ce->num_traits) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5757\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5816\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_hash_internal_pointer_reset_ex(& (*(ce->traits + i))->properties_info,
                                        (HashPosition *)((void *)0));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5817\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5805\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___9 = zend_hash_get_current_data_ex(& (*(ce->traits + i))->properties_info,
                                              (void **)((void *)(& property_info)),
                                              (HashPosition *)((void *)0));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5806\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___9 == (int __attribute__((__visibility__("default")))  )0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5758\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5807\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((property_info->flags & 1792U) == 256U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5759\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        prop_hash = property_info->h;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5760\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        prop_name = property_info->name;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5761\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        prop_name_length = property_info->name_length;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5762\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_unmangle_property_name(property_info->name,
                                    property_info->name_length,
                                    & class_name_unused, & prop_name);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5763\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = strlen(prop_name);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5764\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        prop_name_length = (int )tmp;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5765\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = zend_get_hash_value(prop_name,
                                      (unsigned int )(prop_name_length + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5766\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        prop_hash = (unsigned long )tmp___0;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5808\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___7 = zend_hash_quick_find((HashTable const   *)(& ce->properties_info),
                                     prop_name,
                                     (unsigned int )(prop_name_length + 1),
                                     prop_hash, (void **)(& coliding_prop));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5809\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___7 == (int __attribute__((__visibility__("default")))  )0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5796\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (coliding_prop->flags & 131072U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5773\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_hash_quick_find((HashTable const   *)(& (coliding_prop->ce)->properties_info),
                               prop_name, (unsigned int )(prop_name_length + 1),
                               prop_hash, (void **)(& coliding_prop));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5774\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (coliding_prop->flags & 1024U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5769\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (property_info->flags & 1U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5767\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              prop_value = *((*(ce->traits + i))->default_static_members_table + property_info->offset);
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5768\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              prop_value = *((*(ce->traits + i))->default_properties_table + property_info->offset);
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5770\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zval_addref_p(prop_value);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5771\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_traits_register_private_property(ce, prop_name,
                                                  prop_name_length,
                                                  property_info, prop_value);
            goto __Cont;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5772\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5775\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5797\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((coliding_prop->flags & 1793U) == (property_info->flags & 1793U)) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5790\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (property_info->flags & 1U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5780\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___1 = compare_function(& compare_result,
                                       *(ce->default_static_members_table + coliding_prop->offset),
                                       *((*(ce->traits + i))->default_static_members_table + property_info->offset));
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5781\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (-1 == (int )tmp___1) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5776\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___2 = 1;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5779\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (compare_result.value.lval != 0L) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5777\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___2 = 1;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5778\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___2 = 0;
              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5782\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            not_compatible = (unsigned char )tmp___2;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5787\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___3 = compare_function(& compare_result,
                                       *(ce->default_properties_table + coliding_prop->offset),
                                       *((*(ce->traits + i))->default_properties_table + property_info->offset));
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5788\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (-1 == (int )tmp___3) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5783\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___4 = 1;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5786\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (compare_result.value.lval != 0L) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5784\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___4 = 1;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5785\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___4 = 0;
              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5789\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            not_compatible = (unsigned char )tmp___4;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5791\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          not_compatible = (unsigned char)1;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5798\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (not_compatible) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5792\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___5 = find_first_definition(ce, i, prop_name, prop_name_length,
                                          prop_hash, coliding_prop->ce);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5793\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L,
                     "%s and %s define the same property ($%s) in the composition of %s. However, the definition differs and is considered incompatible. Class was composed",
                     tmp___5->name, (property_info->ce)->name, prop_name,
                     ce->name);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5794\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___6 = find_first_definition(ce, i, prop_name, prop_name_length,
                                          prop_hash, coliding_prop->ce);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5795\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 11L,
                     "%s and %s define the same property ($%s) in the composition of %s. This might be incompatible, to improve maintainability consider using accessor methods in traits instead. Class was composed",
                     tmp___6->name, (property_info->ce)->name, prop_name,
                     ce->name);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5799\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5810\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (property_info->flags & 1U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5800\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        prop_value = *((*(ce->traits + i))->default_static_members_table + property_info->offset);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5801\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        prop_value = *((*(ce->traits + i))->default_properties_table + property_info->offset);
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5811\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zval_addref_p(prop_value);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5812\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (property_info->doc_comment) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5802\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___8 = _estrndup(property_info->doc_comment,
                            (unsigned int )property_info->doc_comment_len);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5803\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        doc_comment = (char *)tmp___8;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5804\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        doc_comment = (char *)((void *)0);
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5813\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_declare_property_ex(ce, prop_name, prop_name_length, prop_value,
                               (int )property_info->flags,
                               (char const   *)doc_comment,
                               property_info->doc_comment_len);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5814\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __Cont: /* CIL Label */ 
      zend_hash_move_forward_ex(& (*(ce->traits + i))->properties_info,
                                (HashPosition *)((void *)0));
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5818\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5821\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern  __attribute__((__nothrow__)) void free(void *__ptr ) ;
__inline static void *__zend_realloc(void *p , size_t len ) ;
static void zend_traits_register_private_property(zend_class_entry *ce ,
                                                  char const   *name ,
                                                  int name_len ,
                                                  zend_property_info *old_info ,
                                                  zval *property ) 
{ char *priv_name ;
  int priv_name_length ;
  char const   *interned_name ;
  zend_property_info property_info ;
  ulong h ;
  ulong __attribute__((__visibility__("default")))  tmp ;
  int tmp___0 ;
  void *tmp___1 ;
  void __attribute__((__visibility__("default")))  *tmp___2 ;
  int tmp___3 ;
  void *tmp___4 ;
  void __attribute__((__visibility__("default")))  *tmp___5 ;
  char const __attribute__((__visibility__("default")))  *tmp___6 ;
  ulong __attribute__((__visibility__("default")))  tmp___7 ;
  char __attribute__((__visibility__("default")))  *tmp___8 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5741\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_get_hash_value(name, (unsigned int )(name_len + 1));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5742\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  h = (ulong )tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5743\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  property_info = *old_info;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5744\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (old_info->flags & 1U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5718\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = ce->default_static_members_count;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5719\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (ce->default_static_members_count) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5720\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    property_info.offset = tmp___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5721\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )ce->type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5712\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = __zend_realloc((void *)ce->default_static_members_table,
                               sizeof(zval *) * (unsigned int )ce->default_static_members_count);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5713\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->default_static_members_table = (zval **)tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5714\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = _erealloc((void *)ce->default_static_members_table,
                          sizeof(zval *) * (unsigned int )ce->default_static_members_count,
                          0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5715\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->default_static_members_table = (zval **)tmp___2;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5722\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(ce->default_static_members_table + property_info.offset) = property;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5723\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )ce->type == 2) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5716\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->static_members_table = ce->default_static_members_table;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5717\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5728\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = ce->default_properties_count;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5729\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (ce->default_properties_count) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5730\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    property_info.offset = tmp___3;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5731\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )ce->type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5724\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___4 = __zend_realloc((void *)ce->default_properties_table,
                               sizeof(zval *) * (unsigned int )ce->default_properties_count);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5725\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->default_properties_table = (zval **)tmp___4;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5726\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___5 = _erealloc((void *)ce->default_properties_table,
                          sizeof(zval *) * (unsigned int )ce->default_properties_count,
                          0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5727\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->default_properties_table = (zval **)tmp___5;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5732\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(ce->default_properties_table + property_info.offset) = property;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5745\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_mangle_property_name(& priv_name, & priv_name_length, ce->name,
                            (int )ce->name_length, name, name_len,
                            (int )ce->type & 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5746\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  property_info.name = (char const   *)priv_name;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5747\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  property_info.name_length = priv_name_length;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5748\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___6 = (*zend_new_interned_string)(property_info.name,
                                        property_info.name_length + 1, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5749\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  interned_name = (char const   *)tmp___6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5750\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )interned_name != (unsigned int )property_info.name) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5735\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )ce->type == 2) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5733\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)((char *)property_info.name));
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5734\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      free((void *)((char *)property_info.name));
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5736\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    property_info.name = interned_name;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5737\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5751\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___7 = zend_get_hash_value(property_info.name,
                                (unsigned int )(property_info.name_length + 1));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5752\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  property_info.h = (unsigned long )tmp___7;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5753\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  property_info.ce = ce;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5754\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (property_info.doc_comment) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5738\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___8 = _estrndup(property_info.doc_comment,
                        (unsigned int )property_info.doc_comment_len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5739\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    property_info.doc_comment = (char const   *)tmp___8;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5740\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5755\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zend_hash_quick_add_or_update(& ce->properties_info, name,
                                 (unsigned int )(name_len + 1), h,
                                 (void *)(& property_info),
                                 sizeof(zend_property_info ),
                                 (void **)((void *)0), 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5756\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static zend_class_entry *find_first_definition(zend_class_entry *ce ,
                                               size_t current_trait ,
                                               char const   *prop_name ,
                                               int prop_name_length ,
                                               ulong prop_hash ,
                                               zend_class_entry *coliding_ce ) 
{ size_t i ;
  zend_property_info *coliding_prop ;
  int __attribute__((__visibility__("default")))  tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5709\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5710\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5705\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (i < current_trait) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5702\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (i < ce->num_traits) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5701\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5706\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_hash_quick_find((HashTable const   *)(& (*(ce->traits + i))->properties_info),
                               prop_name, (unsigned int )(prop_name_length + 1),
                               prop_hash, (void **)(& coliding_prop));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5707\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp == (int __attribute__((__visibility__("default")))  )0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5703\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (*(ce->traits + i));
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5704\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5708\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5711\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (coliding_ce);
}
}
extern void __attribute__((__visibility__("default")))  zend_hash_graceful_destroy(HashTable *ht ) ;
struct _zend_hash_key;
typedef struct _zend_hash_key zend_hash_key;
typedef __builtin_va_list __gnuc_va_list;
typedef __gnuc_va_list va_list;
static int zend_traits_merge_functions_to_class(zend_function *fn ,
                                                int num_args , va_list args ,
                                                zend_hash_key *hash_key ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_apply_with_arguments(HashTable *ht ,
                                                                                       int (*apply_func)(void *pDest ,
                                                                                                         int num_args ,
                                                                                                         va_list args ,
                                                                                                         zend_hash_key *hash_key ) ,
                                                                                       int  
                                                                                       , ...) ;
static int zend_traits_merge_functions(zend_function *fn , int num_args ,
                                       va_list args , zend_hash_key *hash_key ) ;
static void zend_traits_copy_trait_function_table(HashTable *target ,
                                                  HashTable *source ,
                                                  zend_trait_alias **aliases ,
                                                  HashTable *exclude_table ) ;
static void zend_traits_compile_exclude_table(HashTable *exclude_table ,
                                              zend_trait_precedence **precedences ,
                                              zend_class_entry *trait ) ;
extern  __attribute__((__nothrow__)) void *malloc(size_t __size )  __attribute__((__malloc__)) ;
static void zend_do_traits_method_binding(zend_class_entry *ce ) 
{ HashTable **function_tables ;
  HashTable *resulting_table ;
  HashTable exclude_table ;
  size_t i ;
  void *tmp ;
  void *tmp___0 ;
  void *tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5685\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = malloc(sizeof(HashTable *) * ce->num_traits);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5686\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  function_tables = (HashTable **)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5687\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = malloc(sizeof(HashTable ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5688\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  resulting_table = (HashTable *)tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5689\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zend_hash_init_ex(resulting_table, 10U,
                     (ulong (*)(char const   *arKey ,
                                uint nKeyLength ))((void *)0),
                     (void (*)(void *pDest ))((void *)0), (unsigned char)1,
                     (unsigned char)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5690\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5691\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5670\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (i < ce->num_traits) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5664\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5671\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = malloc(sizeof(HashTable ));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5672\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(function_tables + i) = (HashTable *)tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5673\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zend_hash_init_ex(*(function_tables + i),
                       (*(ce->traits + i))->function_table.nNumOfElements,
                       (ulong (*)(char const   *arKey , uint nKeyLength ))((void *)0),
                       (void (*)(void *pDest ))((void *)0), (unsigned char)1,
                       (unsigned char)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5674\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (ce->trait_precedences) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5665\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zend_hash_init_ex(& exclude_table, 2U,
                         (ulong (*)(char const   *arKey ,
                                    uint nKeyLength ))((void *)0),
                         (void (*)(void *pDest ))((void *)0), (unsigned char)0,
                         (unsigned char)0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5666\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_traits_compile_exclude_table(& exclude_table, ce->trait_precedences,
                                        *(ce->traits + i));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5667\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_traits_copy_trait_function_table(*(function_tables + i),
                                            & (*(ce->traits + i))->function_table,
                                            ce->trait_aliases, & exclude_table);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5668\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_hash_destroy(& exclude_table);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5669\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_traits_copy_trait_function_table(*(function_tables + i),
                                            & (*(ce->traits + i))->function_table,
                                            ce->trait_aliases,
                                            (HashTable *)((void *)0));
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5675\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5692\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5693\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5677\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (i < ce->num_traits) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5676\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5678\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_hash_apply_with_arguments(*(function_tables + i),
                                   (int (*)(void *pDest , int num_args ,
                                            va_list args ,
                                            zend_hash_key *hash_key ))(& zend_traits_merge_functions),
                                   5, i, ce->num_traits, resulting_table,
                                   function_tables, ce);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5679\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5694\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_hash_apply_with_arguments(resulting_table,
                                 (int (*)(void *pDest , int num_args ,
                                          va_list args ,
                                          zend_hash_key *hash_key ))(& zend_traits_merge_functions_to_class),
                                 1, ce);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5695\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5696\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5681\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (i < ce->num_traits) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5680\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5682\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_hash_graceful_destroy(*(function_tables + i));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5683\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    free((void *)*(function_tables + i));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5684\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5697\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  free((void *)function_tables);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5698\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_hash_graceful_destroy(resulting_table);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5699\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  free((void *)resulting_table);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5700\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static void zend_traits_compile_exclude_table(HashTable *exclude_table ,
                                              zend_trait_precedence **precedences ,
                                              zend_class_entry *trait ) 
{ size_t i ;
  size_t j ;
  zend_uint lcname_len ;
  char *lcname ;
  char __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5660\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5661\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! precedences) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5637\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5638\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5662\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5657\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (*(precedences + i)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5639\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5658\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((*(precedences + i))->exclude_from_classes) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5654\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      j = 0U;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5655\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5651\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (*((*(precedences + i))->exclude_from_classes + j)) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5640\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5652\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )*((*(precedences + i))->exclude_from_classes + j) == (unsigned int )trait) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5644\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          lcname_len = ((*(precedences + i))->trait_method)->mname_len;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5645\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp = zend_str_tolower_dup(((*(precedences + i))->trait_method)->method_name,
                                     lcname_len);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5646\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          lcname = (char *)tmp;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5647\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___0 = _zend_hash_add_or_update(exclude_table,
                                             (char const   *)lcname, lcname_len,
                                             (void *)0, 0U,
                                             (void **)((void *)0), 1 << 1);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5648\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5641\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _efree((void *)lcname);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5642\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_error(1 << 6L,
                       "Failed to evaluate a trait precedence (%s). Method of trait %s was defined to be excluded multiple times",
                       ((*(precedences + i))->trait_method)->method_name,
                       trait->name);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5643\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5649\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _efree((void *)lcname);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5650\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5653\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        j ++;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5656\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5659\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5663\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_fetch_class(char const   *class_name ,
                                                                                      uint class_name_len ,
                                                                                      int fetch_type ) ;
static void zend_traits_init_trait_structures(zend_class_entry *ce ) 
{ size_t i ;
  size_t j ;
  zend_trait_precedence *cur_precedence ;
  zend_trait_method_reference *cur_method_ref ;
  char *lcname ;
  zend_bool method_exists ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp ;
  char __attribute__((__visibility__("default")))  *tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  char *class_name ;
  zend_uint name_length ;
  size_t tmp___2 ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp___3 ;
  zend_class_entry __attribute__((__visibility__("default")))  *tmp___4 ;
  char __attribute__((__visibility__("default")))  *tmp___5 ;
  int __attribute__((__visibility__("default")))  tmp___6 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5633\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  j = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5634\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->trait_precedences) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5611\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i = 0U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5612\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5607\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      cur_precedence = *(ce->trait_precedences + i);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5608\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (cur_precedence) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5583\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5609\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (cur_precedence->exclude_from_classes) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5595\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        cur_method_ref = cur_precedence->trait_method;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5596\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = zend_fetch_class(cur_method_ref->class_name,
                               cur_method_ref->cname_len, 14);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5597\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (cur_precedence->trait_method)->ce = (zend_class_entry *)tmp;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5598\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = zend_str_tolower_dup(cur_method_ref->method_name,
                                       cur_method_ref->mname_len);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5599\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        lcname = (char *)tmp___0;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5600\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = zend_hash_exists((HashTable const   *)(& (cur_method_ref->ce)->function_table),
                                   (char const   *)lcname,
                                   cur_method_ref->mname_len + 1U);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5601\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        method_exists = (unsigned char )tmp___1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5602\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _efree((void *)lcname);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5603\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (! method_exists) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5584\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L,
                     "A precedence rule was defined for %s::%s but this method does not exist",
                     (cur_method_ref->ce)->name, cur_method_ref->method_name);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5585\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5604\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        j = 0U;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5605\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5587\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (*(cur_precedence->exclude_from_classes + j)) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5586\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            break;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5588\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          class_name = (char *)*(cur_precedence->exclude_from_classes + j);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5589\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___2 = strlen((char const   *)class_name);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5590\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          name_length = tmp___2;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5591\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___3 = zend_fetch_class((char const   *)class_name, name_length, 14);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5592\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *(cur_precedence->exclude_from_classes + j) = (zend_class_entry *)tmp___3;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5593\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _efree((void *)class_name);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5594\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          j ++;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5606\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5610\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i ++;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5613\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5635\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->trait_aliases) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5630\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i = 0U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5631\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5627\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (*(ce->trait_aliases + i)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5614\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5628\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (((*(ce->trait_aliases + i))->trait_method)->class_name) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5617\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        cur_method_ref = (*(ce->trait_aliases + i))->trait_method;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5618\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___4 = zend_fetch_class(cur_method_ref->class_name,
                                   cur_method_ref->cname_len, 14);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5619\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        cur_method_ref->ce = (zend_class_entry *)tmp___4;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5620\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___5 = zend_str_tolower_dup(cur_method_ref->method_name,
                                       cur_method_ref->mname_len);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5621\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        lcname = (char *)tmp___5;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5622\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___6 = zend_hash_exists((HashTable const   *)(& (cur_method_ref->ce)->function_table),
                                   (char const   *)lcname,
                                   cur_method_ref->mname_len + 1U);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5623\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        method_exists = (unsigned char )tmp___6;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5624\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _efree((void *)lcname);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5625\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (! method_exists) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5615\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L,
                     "An alias was defined for %s::%s but this method does not exist",
                     (cur_method_ref->ce)->name, cur_method_ref->method_name);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5616\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5626\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5629\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i ++;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5632\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5636\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static int zend_traits_copy_functions(zend_function *fn , int num_args ,
                                      va_list args , zend_hash_key *hash_key ) ;
static void zend_traits_copy_trait_function_table(HashTable *target ,
                                                  HashTable *source ,
                                                  zend_trait_alias **aliases ,
                                                  HashTable *exclude_table ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5581\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_hash_apply_with_arguments(source,
                                 (int (*)(void *pDest , int num_args ,
                                          va_list args ,
                                          zend_hash_key *hash_key ))(& zend_traits_copy_functions),
                                 3, target, aliases, exclude_table);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5582\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void __attribute__((__visibility__("default")))  function_add_ref(zend_function *function ) ;
static int zend_traits_copy_functions(zend_function *fn , int num_args ,
                                      va_list args , zend_hash_key *hash_key ) 
{ HashTable *target ;
  zend_trait_alias **aliases ;
  HashTable *exclude_table ;
  char *lcname ;
  unsigned int fnname_len ;
  zend_function fn_copy ;
  void *dummy ;
  size_t i ;
  char __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  char __attribute__((__visibility__("default")))  *tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  int __attribute__((__visibility__("default")))  tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5570\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5571\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  target = __builtin_va_arg(args, HashTable *);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5572\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  aliases = __builtin_va_arg(args, zend_trait_alias **);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5573\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  exclude_table = __builtin_va_arg(args, HashTable *);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5574\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fnname_len = strlen(fn->common.function_name);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5575\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (aliases) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5532\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5529\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (*(aliases + i)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5499\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5530\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )(*(aliases + i))->alias != (unsigned int )((void *)0)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5527\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (! ((*(aliases + i))->trait_method)->ce) {
          goto _L;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5526\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )fn->common.scope == (unsigned int )((*(aliases + i))->trait_method)->ce) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5524\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L: /* CIL Label */ 
            if (((*(aliases + i))->trait_method)->mname_len == fnname_len) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5521\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___1 = zend_binary_strcasecmp(((*(aliases + i))->trait_method)->method_name,
                                               ((*(aliases + i))->trait_method)->mname_len,
                                               fn->common.function_name,
                                               fnname_len);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5522\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (tmp___1 == (int __attribute__((__visibility__("default")))  )0) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5510\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                fn_copy = *fn;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5511\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                function_add_ref(& fn_copy);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5512\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                fn_copy.common.function_name = (*(aliases + i))->alias;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5513\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((*(aliases + i))->modifiers) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5502\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  fn_copy.common.fn_flags = (*(aliases + i))->modifiers;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5503\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (! ((*(aliases + i))->modifiers & 1792U)) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5500\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    fn_copy.common.fn_flags |= 256U;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5501\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5504\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  fn_copy.common.fn_flags |= fn->common.fn_flags ^ (fn->common.fn_flags & 1792U);
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5505\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5514\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp = zend_str_tolower_dup((*(aliases + i))->alias,
                                           (*(aliases + i))->alias_len);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5515\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                lcname = (char *)tmp;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5516\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___0 = _zend_hash_add_or_update(target,
                                                   (char const   *)lcname,
                                                   (*(aliases + i))->alias_len + 1U,
                                                   (void *)(& fn_copy),
                                                   sizeof(zend_function ),
                                                   (void **)((void *)0), 1 << 1);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5517\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (tmp___0 == (int __attribute__((__visibility__("default")))  )-1) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5506\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  zend_error(1 << 6L,
                             "Failed to add aliased trait method (%s) to the trait table. There is probably already a trait method with the same name",
                             fn_copy.common.function_name);
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5507\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5518\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                _efree((void *)lcname);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5519\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (! ((*(aliases + i))->trait_method)->ce) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5508\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  ((*(aliases + i))->trait_method)->ce = fn->common.scope;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5509\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5520\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5523\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5525\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5528\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5531\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i ++;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5533\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5576\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = zend_str_tolower_dup(fn->common.function_name, fnname_len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5577\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lcname = (char *)tmp___2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5578\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )exclude_table == (unsigned int )((void *)0)) {
    goto _L___1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5568\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___5 = zend_hash_find((HashTable const   *)exclude_table,
                             (char const   *)lcname, fnname_len, & dummy);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5569\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___5 == (int __attribute__((__visibility__("default")))  )-1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5562\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L___1: /* CIL Label */ 
      fn_copy = *fn;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5563\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      function_add_ref(& fn_copy);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5564\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (aliases) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5557\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        i = 0U;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5558\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5554\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (*(aliases + i)) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5534\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            break;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5555\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )(*(aliases + i))->alias == (unsigned int )((void *)0)) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5552\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((*(aliases + i))->modifiers != 0U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5550\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (! ((*(aliases + i))->trait_method)->ce) {
                goto _L___0;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5549\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((unsigned int )fn->common.scope == (unsigned int )((*(aliases + i))->trait_method)->ce) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5547\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  _L___0: /* CIL Label */ 
                  if (((*(aliases + i))->trait_method)->mname_len == fnname_len) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5544\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___3 = zend_binary_strcasecmp(((*(aliases + i))->trait_method)->method_name,
                                                     ((*(aliases + i))->trait_method)->mname_len,
                                                     fn->common.function_name,
                                                     fnname_len);
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5545\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (tmp___3 == (int __attribute__((__visibility__("default")))  )0) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5539\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      fn_copy.common.fn_flags = (*(aliases + i))->modifiers;
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5540\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (! ((*(aliases + i))->modifiers & 1792U)) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5535\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        fn_copy.common.fn_flags |= 256U;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5536\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }

                      }
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5541\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      fn_copy.common.fn_flags |= fn->common.fn_flags ^ (fn->common.fn_flags & 1792U);
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5542\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (! ((*(aliases + i))->trait_method)->ce) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5537\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        ((*(aliases + i))->trait_method)->ce = fn->common.scope;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5538\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }

                      }
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5543\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }

                    }
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5546\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5548\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5551\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5553\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5556\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          i ++;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5559\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5565\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___4 = _zend_hash_add_or_update(target, (char const   *)lcname,
                                         fnname_len + 1U, (void *)(& fn_copy),
                                         sizeof(zend_function ),
                                         (void **)((void *)0), 1 << 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5566\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___4 == (int __attribute__((__visibility__("default")))  )-1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5560\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L,
                   "Failed to add trait method (%s) to the trait table. There is probably already a trait method with the same name",
                   fn_copy.common.function_name);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5561\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5567\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5579\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _efree((void *)lcname);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5580\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (0);
}
}
struct _zend_hash_key {
   char const   *arKey ;
   uint nKeyLength ;
   ulong h ;
};
static void zend_add_magic_methods(zend_class_entry *ce , char const   *mname ,
                                   uint mname_len , zend_function *fe ) ;
static void do_inheritance_check_on_method(zend_function *child ,
                                           zend_function *parent ) ;
static int zend_traits_merge_functions_to_class(zend_function *fn ,
                                                int num_args , va_list args ,
                                                zend_hash_key *hash_key ) 
{ zend_class_entry *ce ;
  zend_class_entry *tmp ;
  zend_function *existing_fn ;
  zend_function fn_copy ;
  zend_function *fn_copy_p ;
  zend_function *prototype ;
  zend_function *parent_function ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5492\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_va_arg(args, zend_class_entry *);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5493\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5494\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  existing_fn = (zend_function *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5495\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  prototype = (zend_function *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5496\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = zend_hash_quick_find((HashTable const   *)(& ce->function_table),
                                 hash_key->arKey, hash_key->nKeyLength,
                                 hash_key->h, (void **)(& existing_fn));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5497\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___2 == (int __attribute__((__visibility__("default")))  )-1) {
    goto _L___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5491\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )existing_fn->common.scope != (unsigned int )ce) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5475\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L___0: /* CIL Label */ 
      if (ce->parent) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5447\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = zend_hash_quick_find((HashTable const   *)(& (ce->parent)->function_table),
                                       hash_key->arKey, hash_key->nKeyLength,
                                       hash_key->h, (void **)(& parent_function));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5448\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___0 != (int __attribute__((__visibility__("default")))  )-1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5444\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          prototype = parent_function;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5445\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (fn->common.fn_flags & 2U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5441\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_function_dtor(fn);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5442\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return (1);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5443\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5446\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5449\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5476\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      fn->common.scope = ce;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5477\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      fn->common.prototype = prototype;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5478\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (prototype) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5453\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (prototype->common.fn_flags & 8U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5450\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          fn->common.fn_flags |= 8U;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5452\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (prototype->common.fn_flags & 2U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5451\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            fn->common.fn_flags |= 8U;
          } else {
            goto _L;
          }
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5456\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L: /* CIL Label */ 
        if (fn->common.fn_flags & 8U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5454\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          fn->common.fn_flags -= 8U;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5455\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5479\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (prototype) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5457\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        do_inheritance_check_on_method(fn, prototype);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5458\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5480\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (existing_fn) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5461\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (existing_fn->common.fn_flags & 2U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5459\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          do_inheritance_check_on_method(fn, existing_fn);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5460\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5462\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5481\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (existing_fn) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5467\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )existing_fn->common.scope != (unsigned int )ce) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5465\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((fn->common.fn_flags & 2U) == 0U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5463\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_hash_del_key_or_index(& ce->function_table, hash_key->arKey,
                                       hash_key->nKeyLength, hash_key->h, 2);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5464\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5466\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5468\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5482\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (fn->common.fn_flags & 2U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5469\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ce->ce_flags |= 16U;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5470\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5483\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (fn->op_array.static_variables) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5471\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ce->ce_flags |= 8388608U;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5472\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5484\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      fn_copy = *fn;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5485\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      function_add_ref(& fn_copy);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5486\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = _zend_hash_quick_add_or_update(& ce->function_table,
                                               hash_key->arKey,
                                               hash_key->nKeyLength,
                                               hash_key->h, (void *)(& fn_copy),
                                               sizeof(zend_function ),
                                               (void **)(& fn_copy_p), 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5487\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5473\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L,
                   "Trait method %s has not been applied, because failure occured during updating class method table",
                   hash_key->arKey);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5474\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5488\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_add_magic_methods(ce, hash_key->arKey, hash_key->nKeyLength,
                             fn_copy_p);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5489\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_function_dtor(fn);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5490\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_function_dtor(fn);
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5498\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (1);
}
}
extern  __attribute__((__nothrow__)) int strncmp(char const   *__s1 ,
                                                 char const   *__s2 ,
                                                 size_t __n )  __attribute__((__pure__,
__nonnull__(1,2))) ;
static void zend_add_magic_methods(zend_class_entry *ce , char const   *mname ,
                                   uint mname_len , zend_function *fe ) 
{ char *lowercase_name ;
  void __attribute__((__visibility__("default")))  *tmp ;
  char const __attribute__((__visibility__("default")))  *tmp___0 ;
  int tmp___1 ;
  size_t tmp___2 ;
  size_t tmp___3 ;
  size_t __s1_len ;
  size_t __s2_len ;
  int tmp___4 ;
  int tmp___5 ;
  int tmp___6 ;
  unsigned char const   *__s2 ;
  register int __result ;
  int tmp___7 ;
  unsigned char const   *__s1 ;
  register int __result___0 ;
  int tmp___8 ;
  int tmp___9 ;
  int tmp___10 ;
  int tmp___11 ;
  int tmp___12 ;
  size_t tmp___13 ;
  size_t tmp___14 ;
  size_t tmp___15 ;
  size_t tmp___16 ;
  size_t __s1_len___0 ;
  size_t __s2_len___0 ;
  int tmp___17 ;
  int tmp___18 ;
  int tmp___19 ;
  unsigned char const   *__s2___0 ;
  register int __result___1 ;
  int tmp___20 ;
  unsigned char const   *__s1___0 ;
  register int __result___2 ;
  int tmp___21 ;
  int tmp___22 ;
  int tmp___23 ;
  int tmp___24 ;
  int tmp___25 ;
  size_t tmp___26 ;
  size_t tmp___27 ;
  size_t tmp___28 ;
  size_t tmp___29 ;
  size_t __s1_len___1 ;
  size_t __s2_len___1 ;
  int tmp___30 ;
  int tmp___31 ;
  int tmp___32 ;
  unsigned char const   *__s2___1 ;
  register int __result___3 ;
  int tmp___33 ;
  unsigned char const   *__s1___1 ;
  register int __result___4 ;
  int tmp___34 ;
  int tmp___35 ;
  int tmp___36 ;
  int tmp___37 ;
  int tmp___38 ;
  size_t tmp___39 ;
  size_t tmp___40 ;
  size_t tmp___41 ;
  size_t tmp___42 ;
  size_t __s1_len___2 ;
  size_t __s2_len___2 ;
  int tmp___43 ;
  int tmp___44 ;
  int tmp___45 ;
  unsigned char const   *__s2___2 ;
  register int __result___5 ;
  int tmp___46 ;
  unsigned char const   *__s1___2 ;
  register int __result___6 ;
  int tmp___47 ;
  int tmp___48 ;
  int tmp___49 ;
  int tmp___50 ;
  int tmp___51 ;
  size_t tmp___52 ;
  size_t tmp___53 ;
  size_t tmp___54 ;
  size_t tmp___55 ;
  size_t __s1_len___3 ;
  size_t __s2_len___3 ;
  int tmp___56 ;
  int tmp___57 ;
  int tmp___58 ;
  unsigned char const   *__s2___3 ;
  register int __result___7 ;
  int tmp___59 ;
  unsigned char const   *__s1___3 ;
  register int __result___8 ;
  int tmp___60 ;
  int tmp___61 ;
  int tmp___62 ;
  int tmp___63 ;
  int tmp___64 ;
  size_t tmp___65 ;
  size_t tmp___66 ;
  size_t tmp___67 ;
  size_t tmp___68 ;
  size_t __s1_len___4 ;
  size_t __s2_len___4 ;
  int tmp___69 ;
  int tmp___70 ;
  int tmp___71 ;
  unsigned char const   *__s2___4 ;
  register int __result___9 ;
  int tmp___72 ;
  unsigned char const   *__s1___4 ;
  register int __result___10 ;
  int tmp___73 ;
  int tmp___74 ;
  int tmp___75 ;
  int tmp___76 ;
  int tmp___77 ;
  size_t tmp___78 ;
  size_t tmp___79 ;
  size_t tmp___80 ;
  size_t tmp___81 ;
  size_t __s1_len___5 ;
  size_t __s2_len___5 ;
  int tmp___82 ;
  int tmp___83 ;
  int tmp___84 ;
  unsigned char const   *__s2___5 ;
  register int __result___11 ;
  int tmp___85 ;
  unsigned char const   *__s1___5 ;
  register int __result___12 ;
  int tmp___86 ;
  int tmp___87 ;
  int tmp___88 ;
  int tmp___89 ;
  int tmp___90 ;
  size_t tmp___91 ;
  size_t tmp___92 ;
  size_t tmp___93 ;
  size_t tmp___94 ;
  size_t __s1_len___6 ;
  size_t __s2_len___6 ;
  int tmp___95 ;
  int tmp___96 ;
  int tmp___97 ;
  unsigned char const   *__s2___6 ;
  register int __result___13 ;
  int tmp___98 ;
  unsigned char const   *__s1___6 ;
  register int __result___14 ;
  int tmp___99 ;
  int tmp___100 ;
  int tmp___101 ;
  int tmp___102 ;
  int tmp___103 ;
  size_t tmp___104 ;
  size_t tmp___105 ;
  size_t tmp___106 ;
  size_t tmp___107 ;
  size_t __s1_len___7 ;
  size_t __s2_len___7 ;
  int tmp___108 ;
  int tmp___109 ;
  int tmp___110 ;
  unsigned char const   *__s2___7 ;
  register int __result___15 ;
  int tmp___111 ;
  unsigned char const   *__s1___7 ;
  register int __result___16 ;
  int tmp___112 ;
  int tmp___113 ;
  int tmp___114 ;
  int tmp___115 ;
  int tmp___116 ;
  size_t tmp___117 ;
  size_t tmp___118 ;
  size_t tmp___119 ;
  size_t tmp___120 ;
  size_t __s1_len___8 ;
  size_t __s2_len___8 ;
  int tmp___121 ;
  int tmp___122 ;
  int tmp___123 ;
  unsigned char const   *__s2___8 ;
  register int __result___17 ;
  int tmp___124 ;
  unsigned char const   *__s1___8 ;
  register int __result___18 ;
  int tmp___125 ;
  int tmp___126 ;
  int tmp___127 ;
  int tmp___128 ;
  int tmp___129 ;
  size_t tmp___130 ;
  size_t tmp___131 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5438\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5190\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5184\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __s1_len___8 = strlen(mname);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5185\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __s2_len___8 = strlen("__clone");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5186\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! ((unsigned int )((void const   *)(mname + 1)) - (unsigned int )((void const   *)mname) == 1U)) {
        goto _L___18;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5179\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (__s1_len___8 >= 4U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5177\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L___18: /* CIL Label */ 
          if (! ((unsigned int )((void const   *)("__clone" + 1)) - (unsigned int )((void const   *)"__clone") == 1U)) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5173\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___127 = 1;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5176\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (__s2_len___8 >= 4U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5174\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___127 = 1;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5175\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___127 = 0;
            }
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5178\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___127 = 0;
        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5187\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___127) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5180\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___122 = __builtin_strcmp(mname, "__clone");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5181\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___126 = tmp___122;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5182\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___125 = __builtin_strcmp(mname, "__clone");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5183\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___126 = tmp___125;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5188\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___125 = __builtin_strcmp(mname, "__clone");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5189\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___126 = tmp___125;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5191\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___129 = tmp___126;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5192\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___128 = strncmp(mname, "__clone", mname_len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5193\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___129 = tmp___128;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5439\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___129) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5434\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5211\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5205\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __s1_len___7 = strlen(mname);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5206\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __s2_len___7 = strlen("__construct");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5207\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (! ((unsigned int )((void const   *)(mname + 1)) - (unsigned int )((void const   *)mname) == 1U)) {
          goto _L___16;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5200\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (__s1_len___7 >= 4U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5198\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___16: /* CIL Label */ 
            if (! ((unsigned int )((void const   *)("__construct" + 1)) - (unsigned int )((void const   *)"__construct") == 1U)) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5194\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___114 = 1;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5197\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (__s2_len___7 >= 4U) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5195\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___114 = 1;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5196\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___114 = 0;
              }
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5199\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___114 = 0;
          }
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5208\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___114) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5201\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___109 = __builtin_strcmp(mname, "__construct");
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5202\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___113 = tmp___109;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5203\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___112 = __builtin_strcmp(mname, "__construct");
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5204\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___113 = tmp___112;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5209\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___112 = __builtin_strcmp(mname, "__construct");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5210\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___113 = tmp___112;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5212\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___116 = tmp___113;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5213\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___115 = strncmp(mname, "__construct", mname_len);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5214\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___116 = tmp___115;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5435\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___116) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5427\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5232\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5226\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          __s1_len___6 = strlen(mname);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5227\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          __s2_len___6 = strlen("__destruct");
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5228\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (! ((unsigned int )((void const   *)(mname + 1)) - (unsigned int )((void const   *)mname) == 1U)) {
            goto _L___14;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5221\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (__s1_len___6 >= 4U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5219\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L___14: /* CIL Label */ 
              if (! ((unsigned int )((void const   *)("__destruct" + 1)) - (unsigned int )((void const   *)"__destruct") == 1U)) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5215\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___101 = 1;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5218\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (__s2_len___6 >= 4U) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5216\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___101 = 1;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5217\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___101 = 0;
                }
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5220\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___101 = 0;
            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5229\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___101) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5222\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___96 = __builtin_strcmp(mname, "__destruct");
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5223\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___100 = tmp___96;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5224\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___99 = __builtin_strcmp(mname, "__destruct");
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5225\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___100 = tmp___99;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5230\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___99 = __builtin_strcmp(mname, "__destruct");
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5231\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___100 = tmp___99;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5233\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___103 = tmp___100;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5234\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___102 = strncmp(mname, "__destruct", mname_len);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5235\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___103 = tmp___102;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5428\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___103) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5423\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5253\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (0) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5247\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            __s1_len___5 = strlen(mname);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5248\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            __s2_len___5 = strlen("__get");
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5249\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (! ((unsigned int )((void const   *)(mname + 1)) - (unsigned int )((void const   *)mname) == 1U)) {
              goto _L___12;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5242\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (__s1_len___5 >= 4U) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5240\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                _L___12: /* CIL Label */ 
                if (! ((unsigned int )((void const   *)("__get" + 1)) - (unsigned int )((void const   *)"__get") == 1U)) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5236\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___88 = 1;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5239\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (__s2_len___5 >= 4U) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5237\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___88 = 1;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5238\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___88 = 0;
                  }
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5241\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___88 = 0;
              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5250\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (tmp___88) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5243\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___83 = __builtin_strcmp(mname, "__get");
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5244\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___87 = tmp___83;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5245\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___86 = __builtin_strcmp(mname, "__get");
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5246\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___87 = tmp___86;
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5251\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___86 = __builtin_strcmp(mname, "__get");
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5252\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___87 = tmp___86;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5254\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___90 = tmp___87;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5255\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___89 = strncmp(mname, "__get", mname_len);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5256\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___90 = tmp___89;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5424\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___90) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5420\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (0) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5274\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (0) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5268\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              __s1_len___4 = strlen(mname);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5269\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              __s2_len___4 = strlen("__set");
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5270\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (! ((unsigned int )((void const   *)(mname + 1)) - (unsigned int )((void const   *)mname) == 1U)) {
                goto _L___10;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5263\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (__s1_len___4 >= 4U) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5261\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  _L___10: /* CIL Label */ 
                  if (! ((unsigned int )((void const   *)("__set" + 1)) - (unsigned int )((void const   *)"__set") == 1U)) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5257\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___75 = 1;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5260\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (__s2_len___4 >= 4U) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5258\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___75 = 1;
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5259\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___75 = 0;
                    }
                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5262\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___75 = 0;
                }
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5271\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (tmp___75) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5264\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___70 = __builtin_strcmp(mname, "__set");
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5265\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___74 = tmp___70;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5266\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___73 = __builtin_strcmp(mname, "__set");
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5267\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___74 = tmp___73;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5272\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___73 = __builtin_strcmp(mname, "__set");
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5273\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___74 = tmp___73;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5275\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___77 = tmp___74;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5276\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___76 = strncmp(mname, "__set", mname_len);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5277\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___77 = tmp___76;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5421\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___77) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5417\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (0) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5295\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (0) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5289\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                __s1_len___3 = strlen(mname);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5290\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                __s2_len___3 = strlen("__call");
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5291\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (! ((unsigned int )((void const   *)(mname + 1)) - (unsigned int )((void const   *)mname) == 1U)) {
                  goto _L___8;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5284\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (__s1_len___3 >= 4U) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5282\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    _L___8: /* CIL Label */ 
                    if (! ((unsigned int )((void const   *)("__call" + 1)) - (unsigned int )((void const   *)"__call") == 1U)) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5278\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___62 = 1;
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5281\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (__s2_len___3 >= 4U) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5279\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___62 = 1;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5280\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___62 = 0;
                      }
                    }
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5283\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___62 = 0;
                  }
                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5292\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (tmp___62) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5285\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___57 = __builtin_strcmp(mname, "__call");
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5286\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___61 = tmp___57;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5287\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___60 = __builtin_strcmp(mname, "__call");
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5288\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___61 = tmp___60;
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5293\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___60 = __builtin_strcmp(mname, "__call");
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5294\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___61 = tmp___60;
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5296\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___64 = tmp___61;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5297\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___63 = strncmp(mname, "__call", mname_len);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5298\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___64 = tmp___63;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5418\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (tmp___64) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5414\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (0) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5316\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (0) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5310\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  __s1_len___2 = strlen(mname);
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5311\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  __s2_len___2 = strlen("__unset");
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5312\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (! ((unsigned int )((void const   *)(mname + 1)) - (unsigned int )((void const   *)mname) == 1U)) {
                    goto _L___6;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5305\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (__s1_len___2 >= 4U) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5303\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      _L___6: /* CIL Label */ 
                      if (! ((unsigned int )((void const   *)("__unset" + 1)) - (unsigned int )((void const   *)"__unset") == 1U)) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5299\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___49 = 1;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5302\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if (__s2_len___2 >= 4U) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5300\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp___49 = 1;
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5301\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp___49 = 0;
                        }
                      }
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5304\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___49 = 0;
                    }
                  }
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5313\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (tmp___49) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5306\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___44 = __builtin_strcmp(mname, "__unset");
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5307\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___48 = tmp___44;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5308\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___47 = __builtin_strcmp(mname, "__unset");
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5309\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___48 = tmp___47;
                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5314\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___47 = __builtin_strcmp(mname, "__unset");
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5315\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___48 = tmp___47;
                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5317\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___51 = tmp___48;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5318\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___50 = strncmp(mname, "__unset", mname_len);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5319\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___51 = tmp___50;
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5415\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (tmp___51) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5411\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (0) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5337\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (0) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5331\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    __s1_len___1 = strlen(mname);
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5332\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    __s2_len___1 = strlen("__isset");
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5333\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (! ((unsigned int )((void const   *)(mname + 1)) - (unsigned int )((void const   *)mname) == 1U)) {
                      goto _L___4;
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5326\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (__s1_len___1 >= 4U) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5324\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        _L___4: /* CIL Label */ 
                        if (! ((unsigned int )((void const   *)("__isset" + 1)) - (unsigned int )((void const   *)"__isset") == 1U)) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5320\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp___36 = 1;
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5323\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          if (__s2_len___1 >= 4U) {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "5321\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            tmp___36 = 1;
                          } else {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "5322\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            tmp___36 = 0;
                          }
                        }
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5325\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___36 = 0;
                      }
                    }
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5334\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (tmp___36) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5327\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___31 = __builtin_strcmp(mname, "__isset");
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5328\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___35 = tmp___31;
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5329\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___34 = __builtin_strcmp(mname, "__isset");
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5330\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___35 = tmp___34;
                    }
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5335\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___34 = __builtin_strcmp(mname, "__isset");
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5336\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___35 = tmp___34;
                  }
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5338\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___38 = tmp___35;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5339\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___37 = strncmp(mname, "__isset", mname_len);
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5340\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___38 = tmp___37;
                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5412\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (tmp___38) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5408\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (0) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5358\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (0) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5352\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      __s1_len___0 = strlen(mname);
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5353\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      __s2_len___0 = strlen("__callstatic");
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5354\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (! ((unsigned int )((void const   *)(mname + 1)) - (unsigned int )((void const   *)mname) == 1U)) {
                        goto _L___2;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5347\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if (__s1_len___0 >= 4U) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5345\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          _L___2: /* CIL Label */ 
                          if (! ((unsigned int )((void const   *)("__callstatic" + 1)) - (unsigned int )((void const   *)"__callstatic") == 1U)) {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "5341\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            tmp___23 = 1;
                          } else {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "5344\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            if (__s2_len___0 >= 4U) {
                              {
                              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                     "a");
                              fprintf(_coverage_fout, "5342\n");
                              fflush(_coverage_fout);
                              fclose(_coverage_fout);
                              }
                              tmp___23 = 1;
                            } else {
                              {
                              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                     "a");
                              fprintf(_coverage_fout, "5343\n");
                              fflush(_coverage_fout);
                              fclose(_coverage_fout);
                              }
                              tmp___23 = 0;
                            }
                          }
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5346\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp___23 = 0;
                        }
                      }
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5355\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (tmp___23) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5348\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___18 = __builtin_strcmp(mname, "__callstatic");
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5349\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___22 = tmp___18;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5350\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___21 = __builtin_strcmp(mname, "__callstatic");
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5351\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___22 = tmp___21;
                      }
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5356\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___21 = __builtin_strcmp(mname, "__callstatic");
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5357\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___22 = tmp___21;
                    }
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5359\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___25 = tmp___22;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5360\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___24 = strncmp(mname, "__callstatic", mname_len);
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5361\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___25 = tmp___24;
                  }
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5409\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (tmp___25) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5405\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (0) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5379\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (0) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5373\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        __s1_len = strlen(mname);
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5374\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        __s2_len = strlen("__tostring");
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5375\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if (! ((unsigned int )((void const   *)(mname + 1)) - (unsigned int )((void const   *)mname) == 1U)) {
                          goto _L___0;
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5368\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          if (__s1_len >= 4U) {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "5366\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            _L___0: /* CIL Label */ 
                            if (! ((unsigned int )((void const   *)("__tostring" + 1)) - (unsigned int )((void const   *)"__tostring") == 1U)) {
                              {
                              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                     "a");
                              fprintf(_coverage_fout, "5362\n");
                              fflush(_coverage_fout);
                              fclose(_coverage_fout);
                              }
                              tmp___10 = 1;
                            } else {
                              {
                              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                     "a");
                              fprintf(_coverage_fout, "5365\n");
                              fflush(_coverage_fout);
                              fclose(_coverage_fout);
                              }
                              if (__s2_len >= 4U) {
                                {
                                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                       "a");
                                fprintf(_coverage_fout, "5363\n");
                                fflush(_coverage_fout);
                                fclose(_coverage_fout);
                                }
                                tmp___10 = 1;
                              } else {
                                {
                                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                       "a");
                                fprintf(_coverage_fout, "5364\n");
                                fflush(_coverage_fout);
                                fclose(_coverage_fout);
                                }
                                tmp___10 = 0;
                              }
                            }
                          } else {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "5367\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            tmp___10 = 0;
                          }
                        }
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5376\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if (tmp___10) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5369\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp___5 = __builtin_strcmp(mname, "__tostring");
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5370\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp___9 = tmp___5;
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5371\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp___8 = __builtin_strcmp(mname, "__tostring");
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5372\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp___9 = tmp___8;
                        }
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5377\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___8 = __builtin_strcmp(mname, "__tostring");
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5378\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___9 = tmp___8;
                      }
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5380\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___12 = tmp___9;
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5381\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___11 = strncmp(mname, "__tostring", mname_len);
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5382\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___12 = tmp___11;
                    }
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5406\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (tmp___12) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5403\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (ce->name_length + 1U == mname_len) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5394\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp = _emalloc(ce->name_length + 1U);
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5395\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        lowercase_name = (char *)tmp;
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5396\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        zend_str_tolower_copy(lowercase_name, ce->name,
                                              ce->name_length);
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5397\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___0 = (*zend_new_interned_string)((char const   *)lowercase_name,
                                                              (int )(ce->name_length + 1U),
                                                              1);
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5398\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        lowercase_name = (char *)tmp___0;
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5399\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___1 = memcmp((void const   *)mname,
                                         (void const   *)lowercase_name,
                                         mname_len);
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5400\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if (tmp___1) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5383\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }

                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5386\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          if (ce->constructor) {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "5384\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            zend_error(1 << 6L,
                                       "%s has colliding constructor definitions coming from traits",
                                       ce->name);
                          } else {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "5385\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }

                          }
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5387\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          ce->constructor = fe;
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5388\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          fe->common.fn_flags |= 8192U;
                        }
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5401\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        while (1) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "5393\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          if ((unsigned int )lowercase_name >= (unsigned int )compiler_globals.interned_strings_start) {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "5391\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            if ((unsigned int )lowercase_name < (unsigned int )compiler_globals.interned_strings_end) {
                              {
                              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                     "a");
                              fprintf(_coverage_fout, "5389\n");
                              fflush(_coverage_fout);
                              fclose(_coverage_fout);
                              }

                            } else {
                              {
                              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                     "a");
                              fprintf(_coverage_fout, "5390\n");
                              fflush(_coverage_fout);
                              fclose(_coverage_fout);
                              }
                              _efree((void *)lowercase_name);
                            }
                          } else {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "5392\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            _efree((void *)lowercase_name);
                          }
                          break;
                        }
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "5402\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }

                      }
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "5404\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      ce->__tostring = fe;
                    }
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "5407\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    ce->__callstatic = fe;
                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "5410\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  ce->__isset = fe;
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "5413\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                ce->__unset = fe;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5416\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              ce->__call = fe;
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5419\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            ce->__set = fe;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5422\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ce->__get = fe;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5425\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ce->destructor = fe;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5426\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        fe->common.fn_flags |= 16384U;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5431\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (ce->constructor) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5429\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L,
                   "%s has colliding constructor definitions coming from traits",
                   ce->name);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5430\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5432\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->constructor = fe;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5433\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      fe->common.fn_flags |= 8192U;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5436\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->clone = fe;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5437\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    fe->common.fn_flags |= 32768U;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5440\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static char *zend_get_function_declaration(zend_function *fptr ) ;
static zend_bool zend_do_perform_implementation_check(zend_function const   *fe ,
                                                      zend_function const   *proto ) ;
static int zend_traits_merge_functions(zend_function *fn , int num_args ,
                                       va_list args , zend_hash_key *hash_key ) 
{ size_t current ;
  size_t i ;
  size_t count ;
  HashTable *resulting_table ;
  HashTable **function_tables ;
  zend_class_entry *ce ;
  size_t collision ;
  size_t abstract_solved ;
  zend_function *other_trait_fn ;
  char *tmp ;
  char *tmp___0 ;
  zend_bool tmp___1 ;
  zend_bool tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  zend_function *class_fn ;
  int __attribute__((__visibility__("default")))  tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5162\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  collision = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5163\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  abstract_solved = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5164\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  current = __builtin_va_arg(args, size_t );
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5165\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  count = __builtin_va_arg(args, size_t );
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5166\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  resulting_table = __builtin_va_arg(args, HashTable *);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5167\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  function_tables = __builtin_va_arg(args, HashTable **);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5168\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce = __builtin_va_arg(args, zend_class_entry *);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5169\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5170\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5144\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (i < count) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5119\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5145\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (i == current) {
      goto __Cont;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5120\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5146\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = zend_hash_quick_find((HashTable const   *)*(function_tables + i),
                                   hash_key->arKey, hash_key->nKeyLength,
                                   hash_key->h, (void **)(& other_trait_fn));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5147\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3 == (int __attribute__((__visibility__("default")))  )0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5142\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (other_trait_fn->common.fn_flags & 2U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5133\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (fn->common.fn_flags & 2U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5130\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___1 = zend_do_perform_implementation_check((zend_function const   *)fn,
                                                         (zend_function const   *)other_trait_fn);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5131\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5125\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___2 = zend_do_perform_implementation_check((zend_function const   *)other_trait_fn,
                                                           (zend_function const   *)fn);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5126\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (tmp___2) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5121\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5122\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp = zend_get_function_declaration(other_trait_fn);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5123\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___0 = zend_get_function_declaration(fn);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "5124\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              zend_error(1 << 6L,
                         "Declaration of %s must be compatible with %s",
                         tmp___0, tmp);
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5127\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp = zend_get_function_declaration(other_trait_fn);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5128\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___0 = zend_get_function_declaration(fn);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "5129\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_error(1 << 6L, "Declaration of %s must be compatible with %s",
                       tmp___0, tmp);
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5132\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          do_inheritance_check_on_method(fn, other_trait_fn);
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5134\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_function_dtor(other_trait_fn);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5135\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_hash_del_key_or_index(*(function_tables + i), hash_key->arKey,
                                   hash_key->nKeyLength, hash_key->h, 2);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5141\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (fn->common.fn_flags & 2U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5136\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          do_inheritance_check_on_method(other_trait_fn, fn);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5137\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          abstract_solved = 1U;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5138\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          collision ++;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5139\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_function_dtor(other_trait_fn);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5140\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_hash_del_key_or_index(*(function_tables + i), hash_key->arKey,
                                     hash_key->nKeyLength, hash_key->h, 2);
        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5143\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5148\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __Cont: /* CIL Label */ 
    i ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5171\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (collision) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5153\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___4 = zend_hash_quick_find((HashTable const   *)(& ce->function_table),
                                   hash_key->arKey, hash_key->nKeyLength,
                                   hash_key->h, (void **)(& class_fn));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5154\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___4 == (int __attribute__((__visibility__("default")))  )-1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5149\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L,
                 "Trait method %s has not been applied, because there are collisions with other trait methods on %s",
                 fn->common.function_name, ce->name);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5152\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )class_fn->common.scope != (unsigned int )ce) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5150\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L,
                   "Trait method %s has not been applied, because there are collisions with other trait methods on %s",
                   fn->common.function_name, ce->name);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5151\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5155\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_function_dtor(fn);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5161\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (abstract_solved) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5156\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_function_dtor(fn);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5159\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___5 = _zend_hash_quick_add_or_update(resulting_table, hash_key->arKey,
                                               hash_key->nKeyLength,
                                               hash_key->h, (void *)fn,
                                               sizeof(zend_function ),
                                               (void **)((void *)0), 1 << 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5160\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___5 == (int __attribute__((__visibility__("default")))  )-1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5157\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L,
                   "Trait method %s has not been applied, because failure occured during updating resulting trait method table",
                   fn->common.function_name);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5158\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5172\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (1);
}
}
extern  __attribute__((__nothrow__)) void *realloc(void *__ptr , size_t __size )  __attribute__((__warn_unused_result__)) ;
extern  __attribute__((__nothrow__)) void *memmove(void *__dest ,
                                                   void const   *__src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
void __attribute__((__visibility__("default")))  zend_do_implement_trait(zend_class_entry *ce ,
                                                                         zend_class_entry *trait ) 
{ zend_uint i ;
  zend_uint ignore ;
  zend_uint current_trait_num ;
  zend_uint parent_trait_num ;
  zend_uint tmp ;
  void *tmp___0 ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  zend_uint tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5111\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ignore = (zend_uint )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5112\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  current_trait_num = ce->num_traits;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5113\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->parent) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5084\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = (ce->parent)->num_traits;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5085\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = 0U;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5114\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  parent_trait_num = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5115\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5116\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5095\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (i < ce->num_traits) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5086\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5096\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )*(ce->traits + i) == (unsigned int )((void *)0)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5087\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (ce->num_traits) --;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5088\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      memmove((void *)(ce->traits + i), (void const   *)((ce->traits + i) + 1),
              sizeof(zend_class_entry *) * (ce->num_traits - i));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5089\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i --;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5094\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )*(ce->traits + i) == (unsigned int )trait) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5092\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (i < parent_trait_num) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5090\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ignore = 1U;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5091\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5093\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5097\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5117\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ignore) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5106\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (ce->num_traits >= current_trait_num) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5104\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )ce->type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5098\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        current_trait_num ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5099\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = realloc((void *)ce->traits,
                          sizeof(zend_class_entry *) * current_trait_num);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5100\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ce->traits = (zend_class_entry **)tmp___0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5101\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        current_trait_num ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5102\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = _erealloc((void *)ce->traits,
                            sizeof(zend_class_entry *) * current_trait_num, 0);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5103\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ce->traits = (zend_class_entry **)tmp___1;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5105\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5107\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = ce->num_traits;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5108\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (ce->num_traits) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5109\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(ce->traits + tmp___2) = trait;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5110\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5118\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void __attribute__((__visibility__("default")))  zend_do_implement_trait(zend_class_entry *ce ,
                                                                         zend_class_entry *trait ) ;
void __attribute__((__visibility__("default")))  zend_do_inherit_interfaces(zend_class_entry *ce ,
                                                                            zend_class_entry const   *iface ) ;
__inline static void do_implement_interface(zend_class_entry *ce ,
                                            zend_class_entry *iface ) ;
static zend_bool do_inherit_method_check(HashTable *child_function_table ,
                                         zend_function *parent ,
                                         zend_hash_key const   *hash_key ,
                                         zend_class_entry *child_ce ) ;
static void do_inherit_method(zend_function *function ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_merge_ex(HashTable *target ,
                                                                           HashTable *source ,
                                                                           void (*pCopyConstructor)(void *pElement ) ,
                                                                           uint size ,
                                                                           zend_bool (*pMergeSource)(HashTable *target_ht ,
                                                                                                     void *source_data ,
                                                                                                     zend_hash_key *hash_key ,
                                                                                                     void *pParam ) ,
                                                                           void *pParam ) ;
static zend_bool do_inherit_constant_check(HashTable *child_constants_table ,
                                           zval const   **parent_constant ,
                                           zend_hash_key const   *hash_key ,
                                           zend_class_entry const   *iface ) ;
extern void __attribute__((__visibility__("default")))  zval_add_ref(zval **p ) ;
static int do_interface_constant_check(zval **val , int num_args ,
                                       va_list args ,
                                       zend_hash_key const   *key ) ;
void __attribute__((__visibility__("default")))  zend_do_implement_interface(zend_class_entry *ce ,
                                                                             zend_class_entry *iface ) 
{ zend_uint i ;
  zend_uint ignore ;
  zend_uint current_iface_num ;
  zend_uint parent_iface_num ;
  zend_uint tmp ;
  void *tmp___0 ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  zend_uint tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5076\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ignore = (zend_uint )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5077\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  current_iface_num = ce->num_interfaces;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5078\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->parent) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5045\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = (ce->parent)->num_interfaces;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5046\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = 0U;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5079\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  parent_iface_num = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5080\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5081\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5056\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (i < ce->num_interfaces) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5047\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5057\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )*(ce->interfaces + i) == (unsigned int )((void *)0)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5048\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (ce->num_interfaces) --;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5049\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      memmove((void *)(ce->interfaces + i),
              (void const   *)((ce->interfaces + i) + 1),
              sizeof(zend_class_entry *) * (ce->num_interfaces - i));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5050\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i --;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5055\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )*(ce->interfaces + i) == (unsigned int )iface) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5053\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (i < parent_iface_num) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5051\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ignore = 1U;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "5052\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L,
                     "Class %s cannot implement previously implemented interface %s",
                     ce->name, iface->name);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5054\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5058\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5082\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ignore) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5059\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_hash_apply_with_arguments(& ce->constants_table,
                                   (int (*)(void *pDest , int num_args ,
                                            va_list args ,
                                            zend_hash_key *hash_key ))(& do_interface_constant_check),
                                   1, & iface);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5068\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (ce->num_interfaces >= current_iface_num) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5066\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )ce->type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5060\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        current_iface_num ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5061\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = realloc((void *)ce->interfaces,
                          sizeof(zend_class_entry *) * current_iface_num);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5062\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ce->interfaces = (zend_class_entry **)tmp___0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5063\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        current_iface_num ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5064\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = _erealloc((void *)ce->interfaces,
                            sizeof(zend_class_entry *) * current_iface_num, 0);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5065\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ce->interfaces = (zend_class_entry **)tmp___1;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5067\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5069\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = ce->num_interfaces;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5070\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (ce->num_interfaces) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5071\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *(ce->interfaces + tmp___2) = iface;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5072\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_hash_merge_ex(& ce->constants_table, & iface->constants_table,
                       (void (*)(void *pElement ))(& zval_add_ref),
                       sizeof(zval *),
                       (zend_bool (*)(HashTable *target_ht ,
                                      void *source_data ,
                                      zend_hash_key *hash_key ,
                                      void *pParam ))(& do_inherit_constant_check),
                       (void *)iface);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5073\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_hash_merge_ex(& ce->function_table, & iface->function_table,
                       (void (*)(void *pElement ))(& do_inherit_method),
                       sizeof(zend_function ),
                       (zend_bool (*)(HashTable *target_ht , void *source_data ,
                                      zend_hash_key *hash_key , void *pParam ))(& do_inherit_method_check),
                       (void *)ce);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5074\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    do_implement_interface(ce, iface);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5075\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_inherit_interfaces(ce, (zend_class_entry const   *)iface);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5083\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void __attribute__((__visibility__("default")))  zend_do_implement_interface(zend_class_entry *ce ,
                                                                             zend_class_entry *iface ) ;
static int do_interface_constant_check(zval **val , int num_args ,
                                       va_list args ,
                                       zend_hash_key const   *key ) 
{ zend_class_entry **iface ;
  zend_class_entry **tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5041\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_va_arg(args, zend_class_entry **);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5042\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  iface = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5043\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  do_inherit_constant_check(& (*iface)->constants_table, (zval const   **)val,
                            key, (zend_class_entry const   *)*iface);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5044\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (0);
}
}
static zend_bool do_inherit_constant_check(HashTable *child_constants_table ,
                                           zval const   **parent_constant ,
                                           zend_hash_key const   *hash_key ,
                                           zend_class_entry const   *iface ) 
{ zval **old_constant ;
  int __attribute__((__visibility__("default")))  tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5038\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_hash_quick_find((HashTable const   *)child_constants_table,
                             (char const   *)hash_key->arKey,
                             (unsigned int )hash_key->nKeyLength,
                             (unsigned long )hash_key->h,
                             (void **)(& old_constant));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5039\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp == (int __attribute__((__visibility__("default")))  )0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5035\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )*old_constant != (unsigned int )*parent_constant) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5033\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L,
                 "Cannot inherit previously-inherited or override constant %s from interface %s",
                 hash_key->arKey, iface->name);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5034\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5036\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5037\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5040\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned char)1);
}
}
static void do_inherit_parent_constructor(zend_class_entry *ce ) ;
extern void __attribute__((__visibility__("default")))  _zend_hash_merge(HashTable *target ,
                                                                         HashTable *source ,
                                                                         void (*pCopyConstructor)(void *pElement ) ,
                                                                         void *tmp ,
                                                                         uint size ,
                                                                         int overwrite ) ;
static zend_bool do_inherit_property_access_check(HashTable *target_ht ,
                                                  zend_property_info *parent_info ,
                                                  zend_hash_key const   *hash_key ,
                                                  zend_class_entry *ce ) ;
static void zend_duplicate_property_info(zend_property_info *property_info ) ;
static void zend_duplicate_property_info_internal(zend_property_info *property_info ) ;
__inline static zend_bool ( __attribute__((__always_inline__)) zval_set_isref_p)(zval *pz ) ;
__inline static zend_uint ( __attribute__((__always_inline__)) zval_delref_p)(zval *pz ) ;
__inline static zend_uint ( __attribute__((__always_inline__)) zval_refcount_p)(zval *pz ) ;
__inline static zend_bool ( __attribute__((__always_inline__)) zval_isref_p)(zval *pz ) ;
extern void __attribute__((__visibility__("default")))  zend_update_class_constants(zend_class_entry *class_type ) ;
void __attribute__((__visibility__("default")))  zend_do_inheritance(zend_class_entry *ce ,
                                                                     zend_class_entry *parent_ce ) 
{ zend_property_info *property_info ;
  int i ;
  void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  int tmp___1 ;
  int i___0 ;
  void __attribute__((__visibility__("default")))  *tmp___2 ;
  int tmp___3 ;
  zval *new_zv ;
  void __attribute__((__visibility__("default")))  *tmp___4 ;
  zend_uint tmp___5 ;
  zend_bool tmp___6 ;
  int i___1 ;
  void *tmp___7 ;
  void __attribute__((__visibility__("default")))  *tmp___8 ;
  int tmp___9 ;
  zval *new_zv___0 ;
  void __attribute__((__visibility__("default")))  *tmp___10 ;
  zend_uint tmp___11 ;
  zend_bool tmp___12 ;
  int __attribute__((__visibility__("default")))  tmp___13 ;
  void (*tmp___14)(zend_property_info *property_info ) ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5015\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->ce_flags & 128U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4872\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! (parent_ce->ce_flags & 128U)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4870\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Interface %s may not inherit from class (%s)",
                 ce->name, parent_ce->name);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4871\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4873\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5016\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (parent_ce->ce_flags & 64U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4874\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Class %s may not inherit from final class (%s)",
               ce->name, parent_ce->name);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4875\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5017\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce->parent = parent_ce;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5018\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->serialize) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4876\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->serialize = parent_ce->serialize;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4877\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5019\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->unserialize) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4878\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->unserialize = parent_ce->unserialize;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4879\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5020\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_inherit_interfaces(ce, (zend_class_entry const   *)parent_ce);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5021\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (parent_ce->default_properties_count) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4898\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i = ce->default_properties_count + parent_ce->default_properties_count;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4899\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )ce->type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4880\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __zend_realloc((void *)ce->default_properties_table,
                           sizeof(void *) * (unsigned int )i);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4881\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->default_properties_table = (zval **)tmp;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4882\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = _erealloc((void *)ce->default_properties_table,
                          sizeof(void *) * (unsigned int )i, 0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4883\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->default_properties_table = (zval **)tmp___0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4900\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (ce->default_properties_count) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4889\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4885\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = i;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4886\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        i --;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4887\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1 > parent_ce->default_properties_count) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4884\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4888\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *(ce->default_properties_table + i) = *(ce->default_properties_table + (i - parent_ce->default_properties_count));
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4890\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4901\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i = 0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4902\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4894\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (i < parent_ce->default_properties_count) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4891\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4895\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *(ce->default_properties_table + i) = *(parent_ce->default_properties_table + i);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4896\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (*(ce->default_properties_table + i)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4892\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zval_addref_p(*(ce->default_properties_table + i));
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4893\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4897\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i ++;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4903\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->default_properties_count += parent_ce->default_properties_count;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4904\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5022\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )parent_ce->type != (int )ce->type) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4947\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_update_class_constants(parent_ce);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4948\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (parent_ce->default_static_members_count) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4938\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i___0 = ce->default_static_members_count + parent_ce->default_static_members_count;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4939\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = _erealloc((void *)ce->default_static_members_table,
                          sizeof(void *) * (unsigned int )i___0, 0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4940\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->default_static_members_table = (zval **)tmp___2;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4941\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (ce->default_static_members_count) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4910\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4906\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___3 = i___0;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4907\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          i___0 --;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4908\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___3 > parent_ce->default_static_members_count) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4905\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            break;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4909\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *(ce->default_static_members_table + i___0) = *(ce->default_static_members_table + (i___0 - parent_ce->default_static_members_count));
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4911\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4942\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i___0 = 0;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4943\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4932\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (i___0 < parent_ce->default_static_members_count) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4912\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4933\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___6 = zval_isref_p(*(parent_ce->static_members_table + i___0));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4934\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___6) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4913\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4930\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4928\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___5 = zval_refcount_p(*(parent_ce->static_members_table + i___0));
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4929\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (tmp___5 > 1U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4922\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              zval_delref_p(*(parent_ce->static_members_table + i___0));
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4923\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              while (1) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4914\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___4 = _emalloc(sizeof(zval_gc_info ));
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4915\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                new_zv = (zval *)tmp___4;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4916\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                ((zval_gc_info *)new_zv)->u.buffered = (gc_root_buffer *)((void *)0);
                break;
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4924\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              while (1) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4919\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                while (1) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "4917\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  new_zv->value = (*(parent_ce->static_members_table + i___0))->value;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "4918\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  new_zv->type = (*(parent_ce->static_members_table + i___0))->type;
                  break;
                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4920\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                zval_set_refcount_p(new_zv, 1U);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4921\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                zval_unset_isref_p(new_zv);
                break;
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4925\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              *(parent_ce->static_members_table + i___0) = new_zv;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4926\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _zval_copy_ctor(new_zv);
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4927\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
            break;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4931\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zval_set_isref_p(*(parent_ce->static_members_table + i___0));
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4935\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *(ce->default_static_members_table + i___0) = *(parent_ce->static_members_table + i___0);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4936\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zval_addref_p(*(ce->default_static_members_table + i___0));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4937\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        i___0 ++;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4944\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->default_static_members_count += parent_ce->default_static_members_count;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4945\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->static_members_table = ce->default_static_members_table;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4946\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4996\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (parent_ce->default_static_members_count) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4988\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i___1 = ce->default_static_members_count + parent_ce->default_static_members_count;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4989\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )ce->type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4949\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___7 = __zend_realloc((void *)ce->default_static_members_table,
                                 sizeof(void *) * (unsigned int )i___1);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4950\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ce->default_static_members_table = (zval **)tmp___7;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4951\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___8 = _erealloc((void *)ce->default_static_members_table,
                            sizeof(void *) * (unsigned int )i___1, 0);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4952\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ce->default_static_members_table = (zval **)tmp___8;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4990\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (ce->default_static_members_count) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4958\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4954\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___9 = i___1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4955\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          i___1 --;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4956\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___9 > parent_ce->default_static_members_count) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4953\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            break;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4957\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *(ce->default_static_members_table + i___1) = *(ce->default_static_members_table + (i___1 - parent_ce->default_static_members_count));
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4959\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4991\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i___1 = 0;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4992\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4980\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (i___1 < parent_ce->default_static_members_count) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4960\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4981\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___12 = zval_isref_p(*(parent_ce->default_static_members_table + i___1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4982\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___12) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4961\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4978\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4976\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___11 = zval_refcount_p(*(parent_ce->default_static_members_table + i___1));
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4977\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (tmp___11 > 1U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4970\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              zval_delref_p(*(parent_ce->default_static_members_table + i___1));
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4971\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              while (1) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4962\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___10 = _emalloc(sizeof(zval_gc_info ));
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4963\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                new_zv___0 = (zval *)tmp___10;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4964\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                ((zval_gc_info *)new_zv___0)->u.buffered = (gc_root_buffer *)((void *)0);
                break;
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4972\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              while (1) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4967\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                while (1) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "4965\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  new_zv___0->value = (*(parent_ce->default_static_members_table + i___1))->value;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "4966\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  new_zv___0->type = (*(parent_ce->default_static_members_table + i___1))->type;
                  break;
                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4968\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                zval_set_refcount_p(new_zv___0, 1U);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4969\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                zval_unset_isref_p(new_zv___0);
                break;
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4973\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              *(parent_ce->default_static_members_table + i___1) = new_zv___0;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4974\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _zval_copy_ctor(new_zv___0);
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4975\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
            break;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4979\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zval_set_isref_p(*(parent_ce->default_static_members_table + i___1));
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4983\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *(ce->default_static_members_table + i___1) = *(parent_ce->default_static_members_table + i___1);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4984\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zval_addref_p(*(ce->default_static_members_table + i___1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4985\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        i___1 ++;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4993\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->default_static_members_count += parent_ce->default_static_members_count;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4994\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )ce->type == 2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4986\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ce->static_members_table = ce->default_static_members_table;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4987\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4995\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5023\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_hash_internal_pointer_reset_ex(& ce->properties_info,
                                      (HashPosition *)((void *)0));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5024\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5002\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___13 = zend_hash_get_current_data_ex(& ce->properties_info,
                                             (void **)((void *)(& property_info)),
                                             (HashPosition *)((void *)0));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5003\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___13 == (int __attribute__((__visibility__("default")))  )0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4997\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5004\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )property_info->ce == (unsigned int )ce) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5000\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (property_info->flags & 1U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4998\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        property_info->offset += parent_ce->default_static_members_count;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4999\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        property_info->offset += parent_ce->default_properties_count;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5001\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5005\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_hash_move_forward_ex(& ce->properties_info, (HashPosition *)((void *)0));
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5025\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )ce->type & 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5006\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___14 = & zend_duplicate_property_info_internal;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5007\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___14 = & zend_duplicate_property_info;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5026\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_hash_merge_ex(& ce->properties_info, & parent_ce->properties_info,
                     (void (*)(void *pElement ))tmp___14,
                     sizeof(zend_property_info ),
                     (zend_bool (*)(HashTable *target_ht , void *source_data ,
                                    zend_hash_key *hash_key , void *pParam ))(& do_inherit_property_access_check),
                     (void *)ce);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5027\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zend_hash_merge(& ce->constants_table, & parent_ce->constants_table,
                   (void (*)(void * ))(& zval_add_ref), (void *)0,
                   sizeof(zval *), 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5028\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_hash_merge_ex(& ce->function_table, & parent_ce->function_table,
                     (void (*)(void *pElement ))(& do_inherit_method),
                     sizeof(zend_function ),
                     (zend_bool (*)(HashTable *target_ht , void *source_data ,
                                    zend_hash_key *hash_key , void *pParam ))(& do_inherit_method_check),
                     (void *)ce);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5029\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  do_inherit_parent_constructor(ce);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5030\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->ce_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5009\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )ce->type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5008\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ce->ce_flags |= 32U;
    } else {
      goto _L;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "5014\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _L: /* CIL Label */ 
    if (! (ce->ce_flags & 524288U)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5012\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! (ce->ce_flags & 4194304U)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5010\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_verify_abstract_class(ce);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "5011\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "5013\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5031\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce->ce_flags |= parent_ce->ce_flags & 8388608U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5032\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void __attribute__((__visibility__("default")))  zend_do_inherit_interfaces(zend_class_entry *ce ,
                                                                            zend_class_entry const   *iface ) 
{ zend_uint i ;
  zend_uint ce_num ;
  zend_uint if_num ;
  zend_class_entry *entry ;
  void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  zend_uint tmp___1 ;
  zend_uint tmp___2 ;
  zend_uint tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4863\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if_num = (zend_uint )iface->num_interfaces;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4864\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (if_num == 0U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4835\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4836\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4865\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce_num = ce->num_interfaces;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4866\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )ce->type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4837\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = realloc((void *)ce->interfaces,
                  sizeof(zend_class_entry *) * (ce_num + if_num));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4838\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->interfaces = (zend_class_entry **)tmp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4839\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = _erealloc((void *)ce->interfaces,
                        sizeof(zend_class_entry *) * (ce_num + if_num), 0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4840\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->interfaces = (zend_class_entry **)tmp___0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4867\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4851\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = if_num;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4852\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if_num --;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4853\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___2) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4841\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4854\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    entry = *(iface->interfaces + if_num);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4855\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i = 0U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4856\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4844\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (i < ce_num) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4842\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4845\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )*(ce->interfaces + i) == (unsigned int )entry) {
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4843\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4846\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i ++;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4857\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (i == ce_num) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4847\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = ce->num_interfaces;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4848\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (ce->num_interfaces) ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4849\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *(ce->interfaces + tmp___1) = entry;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4850\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4868\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4859\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (ce_num < ce->num_interfaces) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4858\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4860\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = ce_num;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4861\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce_num ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4862\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    do_implement_interface(ce, *(ce->interfaces + tmp___3));
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4869\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void do_implement_interface(zend_class_entry *ce ,
                                            zend_class_entry *iface ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4832\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! (ce->ce_flags & 128U)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4828\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (iface->interface_gets_implemented) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4825\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = (*(iface->interface_gets_implemented))(iface, ce);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4826\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp == -1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4823\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 4L, "Class %s could not implement interface %s",
                   ce->name, iface->name);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4824\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4827\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4829\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4833\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )ce == (unsigned int )iface) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4830\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1, "Interface %s cannot implement itself", ce->name);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4831\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4834\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
char *zend_visibility_string(zend_uint fn_flags ) ;
static zend_bool do_inherit_property_access_check(HashTable *target_ht ,
                                                  zend_property_info *parent_info ,
                                                  zend_hash_key const   *hash_key ,
                                                  zend_class_entry *ce ) 
{ zend_property_info *child_info ;
  zend_class_entry *parent_ce ;
  int __attribute__((__visibility__("default")))  tmp ;
  char const   *tmp___0 ;
  char const   *tmp___1 ;
  char const   *tmp___2 ;
  char *tmp___3 ;
  int __attribute__((__visibility__("default")))  tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4819\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  parent_ce = ce->parent;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4820\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (parent_info->flags & 132096U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4789\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_hash_quick_find((HashTable const   *)(& ce->properties_info),
                               (char const   *)hash_key->arKey,
                               (unsigned int )hash_key->nKeyLength,
                               (unsigned long )hash_key->h,
                               (void **)(& child_info));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4790\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp == (int __attribute__((__visibility__("default")))  )0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4782\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      child_info->flags |= 2048U;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4785\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zend_hash_quick_add_or_update(& ce->properties_info,
                                     (char const   *)hash_key->arKey,
                                     (unsigned int )hash_key->nKeyLength,
                                     (unsigned long )hash_key->h,
                                     (void *)parent_info,
                                     sizeof(zend_property_info ),
                                     (void **)(& child_info), 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4786\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )ce->type & 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4783\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_duplicate_property_info_internal(child_info);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4784\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_duplicate_property_info(child_info);
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4787\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      child_info->flags &= 4294966271U;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4788\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      child_info->flags |= 131072U;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4791\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4792\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4821\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = zend_hash_quick_find((HashTable const   *)(& ce->properties_info),
                                 (char const   *)hash_key->arKey,
                                 (unsigned int )hash_key->nKeyLength,
                                 (unsigned long )hash_key->h,
                                 (void **)(& child_info));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4822\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4 == (int __attribute__((__visibility__("default")))  )0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4814\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((parent_info->flags & 1U) != (child_info->flags & 1U)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4797\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (child_info->flags & 1U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4793\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = "static ";
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4794\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = "non static ";
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4798\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (parent_info->flags & 1U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4795\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = "static ";
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4796\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = "non static ";
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4799\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Cannot redeclare %s%s::$%s as %s%s::$%s", tmp___1,
                 parent_ce->name, hash_key->arKey, tmp___0, ce->name,
                 hash_key->arKey);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4800\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4815\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (parent_info->flags & 2048U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4801\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      child_info->flags |= 2048U;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4802\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4816\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((child_info->flags & 1792U) > (parent_info->flags & 1792U)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4805\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (parent_info->flags & 256U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4803\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___2 = "";
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4804\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___2 = " or weaker";
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4806\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___3 = zend_visibility_string(parent_info->flags);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4807\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L,
                 "Access level to %s::$%s must be %s (as in class %s)%s",
                 ce->name, hash_key->arKey, tmp___3, parent_ce->name, tmp___2);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4813\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((child_info->flags & 1U) == 0U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4808\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zval_delref_p(*(ce->default_properties_table + parent_info->offset));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4809\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *(ce->default_properties_table + parent_info->offset) = *(ce->default_properties_table + child_info->offset);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4810\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *(ce->default_properties_table + child_info->offset) = (zval *)((void *)0);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4811\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        child_info->offset = parent_info->offset;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4812\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4817\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4818\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)1);
  }
}
}
static zend_bool do_inherit_method_check(HashTable *child_function_table ,
                                         zend_function *parent ,
                                         zend_hash_key const   *hash_key ,
                                         zend_class_entry *child_ce ) 
{ zend_uint parent_flags ;
  zend_function *child ;
  int __attribute__((__visibility__("default")))  tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4777\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  parent_flags = parent->common.fn_flags;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4778\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_hash_quick_find((HashTable const   *)child_function_table,
                             (char const   *)hash_key->arKey,
                             (unsigned int )hash_key->nKeyLength,
                             (unsigned long )hash_key->h, (void **)(& child));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4779\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp == (int __attribute__((__visibility__("default")))  )-1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4774\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (parent_flags & 2U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4772\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      child_ce->ce_flags |= 16U;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4773\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4775\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4776\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4780\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  do_inheritance_check_on_method(child, parent);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4781\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned char)0);
}
}
static void do_inheritance_check_on_method(zend_function *child ,
                                           zend_function *parent ) 
{ zend_uint child_flags ;
  zend_uint parent_flags ;
  char const   *tmp ;
  zend_class_entry *tmp___0 ;
  char const   *tmp___1 ;
  char const   *tmp___2 ;
  char const   *tmp___3 ;
  char const   *tmp___4 ;
  char const   *tmp___5 ;
  char const   *tmp___6 ;
  char const   *tmp___7 ;
  char const   *tmp___8 ;
  char const   *tmp___9 ;
  char *tmp___10 ;
  char const   *tmp___11 ;
  char *tmp___12 ;
  char const   *tmp___13 ;
  zend_bool tmp___14 ;
  char *method_prototype ;
  char *tmp___15 ;
  char const   *tmp___16 ;
  zend_bool tmp___17 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4762\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  parent_flags = parent->common.fn_flags;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4763\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (((parent->common.scope)->ce_flags & 128U) == 0U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4655\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (parent->common.fn_flags & 2U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4652\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (child->common.prototype) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4643\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = (child->common.prototype)->common.scope;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4644\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = child->common.scope;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4653\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )parent->common.scope != (unsigned int )tmp___0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4650\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (child->common.fn_flags & 10U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4647\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (child->common.prototype) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4645\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp = ((child->common.prototype)->common.scope)->name;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4646\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp = (child->common.scope)->name;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4648\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L,
                     "Can\'t inherit abstract function %s::%s() (previously declared abstract in %s)",
                     (parent->common.scope)->name, child->common.function_name,
                     tmp);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4649\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4651\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4654\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4656\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4764\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (parent_flags & 4U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4661\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (parent) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4659\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (parent->common.scope) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4657\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = (parent->common.scope)->name;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4658\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = "";
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4660\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = "";
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4662\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Cannot override final method %s::%s()", tmp___1,
               child->common.function_name);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4663\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4765\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  child_flags = child->common.fn_flags;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4766\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((child_flags & 1U) != (parent_flags & 1U)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4686\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (child->common.fn_flags & 1U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4672\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (child) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4666\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (child->common.scope) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4664\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___2 = (child->common.scope)->name;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4665\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___2 = "";
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4667\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___2 = "";
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4673\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (parent) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4670\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (parent->common.scope) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4668\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___3 = (parent->common.scope)->name;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4669\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___3 = "";
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4671\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = "";
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4674\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L,
                 "Cannot make non static method %s::%s() static in class %s",
                 tmp___3, child->common.function_name, tmp___2);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4683\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (child) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4677\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (child->common.scope) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4675\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___4 = (child->common.scope)->name;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4676\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___4 = "";
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4678\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___4 = "";
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4684\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (parent) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4681\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (parent->common.scope) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4679\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___5 = (parent->common.scope)->name;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4680\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___5 = "";
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4682\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___5 = "";
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4685\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L,
                 "Cannot make static method %s::%s() non static in class %s",
                 tmp___5, child->common.function_name, tmp___4);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4687\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4767\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (child_flags & 2U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4700\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! (parent_flags & 2U)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4696\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (child) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4690\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (child->common.scope) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4688\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___6 = (child->common.scope)->name;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4689\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___6 = "";
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4691\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___6 = "";
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4697\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (parent) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4694\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (parent->common.scope) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4692\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___7 = (parent->common.scope)->name;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4693\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___7 = "";
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4695\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___7 = "";
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4698\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L,
                 "Cannot make non abstract method %s::%s() abstract in class %s",
                 tmp___7, child->common.function_name, tmp___6);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4699\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4701\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4768\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (parent_flags & 2048U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4702\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    child->common.fn_flags |= 2048U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4723\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((child_flags & 1792U) > (parent_flags & 1792U)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4713\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (parent_flags & 256U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4703\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___8 = "";
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4704\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___8 = " or weaker";
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4714\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (parent) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4707\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (parent->common.scope) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4705\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___9 = (parent->common.scope)->name;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4706\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___9 = "";
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4708\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___9 = "";
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4715\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___10 = zend_visibility_string(parent_flags);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4716\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (child) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4711\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (child->common.scope) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4709\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___11 = (child->common.scope)->name;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4710\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___11 = "";
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4712\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___11 = "";
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4717\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L,
                 "Access level to %s::%s() must be %s (as in class %s)%s",
                 tmp___11, child->common.function_name, tmp___10, tmp___9,
                 tmp___8);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4722\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((child_flags & 1792U) < (parent_flags & 1792U)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4720\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((parent_flags & 1792U) & 1024U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4718\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          child->common.fn_flags |= 2048U;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4719\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4721\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4769\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (parent_flags & 1024U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4724\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    child->common.prototype = (union _zend_function *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4735\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (parent_flags & 2U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4725\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      child->common.fn_flags |= 8U;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4726\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      child->common.prototype = parent;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4734\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! (parent->common.fn_flags & 8192U)) {
        goto _L;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4733\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (parent->common.prototype) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4731\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (((parent->common.prototype)->common.scope)->ce_flags & 128U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4729\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L: /* CIL Label */ 
            if (parent->common.prototype) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4727\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              child->common.prototype = parent->common.prototype;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4728\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              child->common.prototype = parent;
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4730\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4732\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4770\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (child->common.prototype) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4746\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((child->common.prototype)->common.fn_flags & 2U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4744\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___14 = zend_do_perform_implementation_check((zend_function const   *)child,
                                                      (zend_function const   *)child->common.prototype);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4745\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___14) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4736\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4741\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___12 = zend_get_function_declaration(child->common.prototype);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4742\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (child) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4739\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (child->common.scope) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4737\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___13 = (child->common.scope)->name;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4738\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___13 = "";
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4740\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___13 = "";
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4743\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L,
                   "Declaration of %s::%s() must be compatible with %s",
                   tmp___13, child->common.function_name, tmp___12);
      }
    } else {
      goto _L___1;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4761\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _L___1: /* CIL Label */ 
    if (executor_globals.error_reporting & (1 << 11L)) {
      goto _L___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4760\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (executor_globals.user_error_handler) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4757\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___0: /* CIL Label */ 
        tmp___17 = zend_do_perform_implementation_check((zend_function const   *)child,
                                                        (zend_function const   *)parent);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4758\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___17) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4747\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4752\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___15 = zend_get_function_declaration(child->common.prototype);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4753\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          method_prototype = tmp___15;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4754\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (child) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4750\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (child->common.scope) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4748\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___16 = (child->common.scope)->name;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4749\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___16 = "";
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4751\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___16 = "";
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4755\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 11L,
                     "Declaration of %s::%s() should be compatible with %s",
                     tmp___16, child->common.function_name, method_prototype);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4756\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _efree((void *)method_prototype);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4759\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4771\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  zend_make_printable_zval(zval *expr ,
                                                                                 zval *expr_copy ,
                                                                                 int *use_copy ) ;
extern int __attribute__((__visibility__("default")))  zval_update_constant_ex(zval **pp ,
                                                                               void *arg ,
                                                                               zend_class_entry *scope ) ;
struct _zend_arg_info {
   char const   *name ;
   zend_uint name_len ;
   char const   *class_name ;
   zend_uint class_name_len ;
   zend_uchar type_hint ;
   zend_bool allow_null ;
   zend_bool pass_by_reference ;
};
extern char __attribute__((__visibility__("default")))  *zend_get_type_by_const(int type ) ;
extern  __attribute__((__nothrow__)) int strcasecmp(char const   *__s1 ,
                                                    char const   *__s2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
static char *zend_get_function_declaration(zend_function *fptr ) 
{ char *offset ;
  char *buf ;
  zend_uint length ;
  void __attribute__((__visibility__("default")))  *tmp ;
  char *tmp___0 ;
  char *tmp___1 ;
  char *tmp___2 ;
  char *tmp___3 ;
  size_t name_len ;
  size_t tmp___4 ;
  void __attribute__((__visibility__("default")))  *tmp___5 ;
  long tmp___6 ;
  char *tmp___7 ;
  zend_uint i ;
  zend_uint required ;
  zend_arg_info *arg_info ;
  char const   *class_name ;
  zend_uint class_name_len ;
  int tmp___8 ;
  int tmp___9 ;
  void __attribute__((__visibility__("default")))  *tmp___10 ;
  long tmp___11 ;
  char *tmp___12 ;
  zend_uint type_name_len ;
  char *type_name ;
  char __attribute__((__visibility__("default")))  *tmp___13 ;
  void __attribute__((__visibility__("default")))  *tmp___14 ;
  long tmp___15 ;
  char *tmp___16 ;
  char *tmp___17 ;
  char *tmp___18 ;
  void __attribute__((__visibility__("default")))  *tmp___19 ;
  long tmp___20 ;
  zend_uint idx ;
  char *tmp___21 ;
  char *tmp___22 ;
  char *tmp___23 ;
  char *tmp___24 ;
  zend_op *precv ;
  zend_uint idx___0 ;
  zend_op *op ;
  zend_op *end ;
  zval *zv ;
  zval zv_copy ;
  int use_copy ;
  void __attribute__((__visibility__("default")))  *tmp___25 ;
  char *tmp___26 ;
  int tmp___27 ;
  void __attribute__((__visibility__("default")))  *tmp___28 ;
  int tmp___29 ;
  long tmp___30 ;
  int tmp___31 ;
  int tmp___32 ;
  char *tmp___33 ;
  char *tmp___34 ;
  char *tmp___35 ;
  char *tmp___36 ;
  void __attribute__((__visibility__("default")))  *tmp___37 ;
  long tmp___38 ;
  char *tmp___39 ;
  char *tmp___40 ;
  void __attribute__((__visibility__("default")))  *tmp___41 ;
  long tmp___42 ;
  char *tmp___43 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4622\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  length = (zend_uint )1024;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4623\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _emalloc(length * sizeof(char ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4624\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  buf = (char *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4625\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  offset = buf;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4626\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (fptr->op_array.fn_flags & 67108864U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4403\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = offset;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4404\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    offset ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4405\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *tmp___0 = (char )'&';
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4406\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = offset;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4407\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    offset ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4408\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *tmp___1 = (char )' ';
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4409\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4627\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (fptr->common.scope) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4410\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    memcpy((void */* __restrict  */)offset,
           (void const   */* __restrict  */)(fptr->common.scope)->name,
           (fptr->common.scope)->name_length);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4411\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    offset += (fptr->common.scope)->name_length;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4412\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = offset;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4413\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    offset ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4414\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *tmp___2 = (char )':';
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4415\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = offset;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4416\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    offset ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4417\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *tmp___3 = (char )':';
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4418\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4628\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = strlen(fptr->common.function_name);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4629\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  name_len = tmp___4;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4630\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___6 = __builtin_expect((long )((size_t )(offset - buf) + name_len >= length),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4631\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___6) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4419\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    length += name_len + 1U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4420\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___5 = _erealloc((void *)buf, length, 0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4421\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    buf = (char *)tmp___5;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4422\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4632\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  memcpy((void */* __restrict  */)offset,
         (void const   */* __restrict  */)fptr->common.function_name, name_len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4633\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  offset += name_len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4634\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___7 = offset;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4635\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  offset ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4636\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___7 = (char )'(';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4637\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (fptr->common.arg_info) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4617\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arg_info = fptr->common.arg_info;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4618\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    required = fptr->common.required_num_args;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4619\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i = 0U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4620\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4604\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (i < fptr->common.num_args) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4423\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4605\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (arg_info->class_name) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4440\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___9 = strcasecmp(arg_info->class_name, "self");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4441\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___9) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4431\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L: /* CIL Label */ 
          tmp___8 = strcasecmp(arg_info->class_name, "parent");
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4432\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___8) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4424\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            class_name = arg_info->class_name;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4425\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            class_name_len = arg_info->class_name_len;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4430\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((fptr->common.scope)->parent) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4426\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              class_name = ((fptr->common.scope)->parent)->name;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4427\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              class_name_len = ((fptr->common.scope)->parent)->name_length;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4428\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              class_name = arg_info->class_name;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4429\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              class_name_len = arg_info->class_name_len;
            }
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4435\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (fptr->common.scope) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4433\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            class_name = (fptr->common.scope)->name;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4434\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            class_name_len = (fptr->common.scope)->name_length;
          } else {
            goto _L;
          }
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4442\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___11 = __builtin_expect((long )((zend_uint )(offset - buf) + class_name_len >= length),
                                    0L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4443\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___11) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4436\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          length += class_name_len + 1U;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4437\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___10 = _erealloc((void *)buf, length, 0);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4438\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          buf = (char *)tmp___10;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4439\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4444\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        memcpy((void */* __restrict  */)offset,
               (void const   */* __restrict  */)class_name, class_name_len);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4445\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        offset += class_name_len;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4446\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___12 = offset;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4447\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        offset ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4448\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp___12 = (char )' ';
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4464\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (arg_info->type_hint) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4453\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___13 = zend_get_type_by_const((int )arg_info->type_hint);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4454\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          type_name = (char *)tmp___13;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4455\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          type_name_len = strlen((char const   *)type_name);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4456\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___15 = __builtin_expect((long )((zend_uint )(offset - buf) + type_name_len >= length),
                                      0L);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4457\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___15) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4449\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            length += type_name_len + 1U;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4450\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___14 = _erealloc((void *)buf, length, 0);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4451\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            buf = (char *)tmp___14;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4452\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4458\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          memcpy((void */* __restrict  */)offset,
                 (void const   */* __restrict  */)type_name, type_name_len);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4459\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          offset += type_name_len;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4460\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___16 = offset;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4461\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          offset ++;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4462\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *tmp___16 = (char )' ';
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4463\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4606\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (arg_info->pass_by_reference) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4465\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___17 = offset;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4466\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        offset ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4467\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp___17 = (char )'&';
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4468\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4607\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___18 = offset;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4608\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      offset ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4609\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp___18 = (char )'$';
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4610\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (arg_info->name) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4473\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___20 = __builtin_expect((long )((zend_uint )(offset - buf) + arg_info->name_len >= length),
                                    0L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4474\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___20) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4469\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          length += arg_info->name_len + 1U;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4470\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___19 = _erealloc((void *)buf, length, 0);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4471\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          buf = (char *)tmp___19;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4472\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4475\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        memcpy((void */* __restrict  */)offset,
               (void const   */* __restrict  */)arg_info->name,
               arg_info->name_len);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4476\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        offset += arg_info->name_len;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4483\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        idx = i;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4484\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        memcpy((void */* __restrict  */)offset,
               (void const   */* __restrict  */)"param", 5U);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4485\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        offset += 5;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4486\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4478\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___21 = offset;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4479\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          offset ++;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4480\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *tmp___21 = (char )((int )((char )(idx % 10U)) + 48);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4481\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          idx /= 10U;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4482\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (idx > 0U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4477\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            break;
          }
        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4611\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (i >= required) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4582\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___22 = offset;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4583\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        offset ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4584\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp___22 = (char )' ';
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4585\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___23 = offset;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4586\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        offset ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4587\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp___23 = (char )'=';
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4588\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___24 = offset;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4589\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        offset ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4590\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp___24 = (char )' ';
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4591\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )fptr->type == 2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4573\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          precv = (zend_op *)((void *)0);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4574\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          idx___0 = i;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4575\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          op = ((zend_op_array *)fptr)->opcodes;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4576\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          end = op + ((zend_op_array *)fptr)->last;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4577\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          idx___0 ++;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4578\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4493\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )op < (unsigned int )end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4487\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            } else {
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4494\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )op->opcode == 63) {
              goto _L___0;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4492\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int )op->opcode == 64) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4490\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                _L___0: /* CIL Label */ 
                if ((unsigned long )op->op1.num == (unsigned long )((long )idx___0)) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "4488\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  precv = op;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "4489\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4491\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4495\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            op ++;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4579\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (precv) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4571\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )precv->opcode == 64) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4569\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int )precv->op2_type != 1 << 3) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4560\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                while (1) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "4496\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___25 = _emalloc(sizeof(zval_gc_info ));
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "4497\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  zv = (zval *)tmp___25;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "4498\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  ((zval_gc_info *)zv)->u.buffered = (gc_root_buffer *)((void *)0);
                  break;
                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4561\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                *zv = *(precv->op2.zv);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4562\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                _zval_copy_ctor(zv);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4563\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                zv->refcount__gc = 1U;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4564\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                zv->is_ref__gc = (unsigned char)0;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4565\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                zval_update_constant_ex(& zv, (void *)1, fptr->common.scope);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4566\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )zv->type == 3) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "4503\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (zv->value.lval) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "4499\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    memcpy((void */* __restrict  */)offset,
                           (void const   */* __restrict  */)"true", 4U);
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "4500\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    offset += 4;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "4501\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    memcpy((void */* __restrict  */)offset,
                           (void const   */* __restrict  */)"false", 5U);
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "4502\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    offset += 5;
                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "4559\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int )zv->type == 0) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "4504\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    memcpy((void */* __restrict  */)offset,
                           (void const   */* __restrict  */)"NULL", 4U);
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "4505\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    offset += 4;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "4558\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if ((int )zv->type == 6) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4529\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___26 = offset;
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4530\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      offset ++;
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4531\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      *tmp___26 = (char )'\'';
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4532\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (zv->value.str.len < 10) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4506\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___29 = zv->value.str.len;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4507\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___29 = 10;
                      }
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4533\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___30 = __builtin_expect((long )((zend_uint )((offset - buf) + tmp___29) >= length),
                                                  0L);
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4534\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (tmp___30) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4510\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if (zv->value.str.len < 10) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "4508\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp___27 = zv->value.str.len;
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "4509\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp___27 = 10;
                        }
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4511\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        length += (zend_uint )(tmp___27 + 1);
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4512\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___28 = _erealloc((void *)buf, length, 0);
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4513\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        buf = (char *)tmp___28;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4514\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }

                      }
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4535\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (zv->value.str.len < 10) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4515\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___31 = zv->value.str.len;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4516\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___31 = 10;
                      }
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4536\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      memcpy((void */* __restrict  */)offset,
                             (void const   */* __restrict  */)zv->value.str.val,
                             (unsigned int )tmp___31);
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4537\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (zv->value.str.len < 10) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4517\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___32 = zv->value.str.len;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4518\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___32 = 10;
                      }
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4538\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      offset += tmp___32;
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4539\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (zv->value.str.len > 10) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4519\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___33 = offset;
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4520\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        offset ++;
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4521\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        *tmp___33 = (char )'.';
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4522\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___34 = offset;
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4523\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        offset ++;
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4524\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        *tmp___34 = (char )'.';
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4525\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___35 = offset;
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4526\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        offset ++;
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4527\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        *tmp___35 = (char )'.';
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4528\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }

                      }
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4540\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___36 = offset;
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4541\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      offset ++;
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4542\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      *tmp___36 = (char )'\'';
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4557\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if ((int )zv->type == 4) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4543\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        memcpy((void */* __restrict  */)offset,
                               (void const   */* __restrict  */)"Array", 5U);
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4544\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        offset += 5;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4551\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        zend_make_printable_zval(zv, & zv_copy, & use_copy);
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4552\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___38 = __builtin_expect((long )((zend_uint )((offset - buf) + zv_copy.value.str.len) >= length),
                                                    0L);
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4553\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if (tmp___38) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "4545\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          length += (zend_uint )(zv_copy.value.str.len + 1);
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "4546\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp___37 = _erealloc((void *)buf, length, 0);
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "4547\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          buf = (char *)tmp___37;
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "4548\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }

                        }
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4554\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        memcpy((void */* __restrict  */)offset,
                               (void const   */* __restrict  */)zv_copy.value.str.val,
                               (unsigned int )zv_copy.value.str.len);
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4555\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        offset += zv_copy.value.str.len;
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4556\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if (use_copy) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "4549\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          _zval_dtor(& zv_copy);
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "4550\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }

                        }
                      }
                    }
                  }
                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4567\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                _zval_ptr_dtor(& zv);
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4568\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4570\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4572\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4580\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          memcpy((void */* __restrict  */)offset,
                 (void const   */* __restrict  */)"NULL", 4U);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4581\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          offset += 4;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4592\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4612\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4613\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (i < fptr->common.num_args) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4593\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___39 = offset;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4594\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        offset ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4595\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp___39 = (char )',';
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4596\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___40 = offset;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4597\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        offset ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4598\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp___40 = (char )' ';
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4599\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4614\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      arg_info ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4615\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___42 = __builtin_expect((long )((zend_uint )((offset - buf) + 32) >= length),
                                  0L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4616\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___42) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4600\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        length += 33U;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4601\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___41 = _erealloc((void *)buf, length, 0);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4602\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        buf = (char *)tmp___41;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4603\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4621\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4638\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___43 = offset;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4639\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  offset ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4640\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___43 = (char )')';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4641\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *offset = (char )'\000';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4642\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (buf);
}
}
static zend_bool zend_do_perform_implementation_check(zend_function const   *fe ,
                                                      zend_function const   *proto ) 
{ zend_uint i ;
  int tmp ;
  int tmp___0 ;
  char const   *fe_class_name ;
  char const   *proto_class_name ;
  zend_uint fe_class_name_len ;
  zend_uint proto_class_name_len ;
  int tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;
  int tmp___4 ;
  char const   *colon ;
  zend_class_entry **fe_ce ;
  zend_class_entry **proto_ce ;
  int found ;
  int found2 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;
  int __attribute__((__visibility__("default")))  tmp___6 ;
  void *tmp___7 ;
  char *tmp___8 ;
  void const   *tmp___9 ;
  int tmp___10 ;
  int tmp___11 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4394\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! proto) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4282\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4287\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! proto->common.arg_info) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4285\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )proto->common.type != 2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4283\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((unsigned char)1);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4284\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4286\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4395\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (fe->common.fn_flags & 8192U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4292\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (((proto->common.scope)->ce_flags & 128U) == 0U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4290\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((proto->common.fn_flags & 2U) == 0U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4288\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((unsigned char)1);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4289\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4291\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4293\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4396\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (proto->common.required_num_args < fe->common.required_num_args) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4294\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4297\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (proto->common.num_args > fe->common.num_args) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4295\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((unsigned char)0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4296\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4397\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )fe->common.type != 2) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4302\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((proto->common.fn_flags & 16777216U) != 0U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4300\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((fe->common.fn_flags & 16777216U) == 0U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4298\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((unsigned char)0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4299\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4301\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4303\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4398\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (proto->common.fn_flags & 67108864U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4306\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! (fe->common.fn_flags & 67108864U)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4304\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((unsigned char)0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4305\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4307\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4399\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4400\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4377\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (i < (zend_uint )proto->common.num_args) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4308\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4378\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((fe->common.arg_info + i)->class_name) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4309\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = 1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4310\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = 0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4379\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((proto->common.arg_info + i)->class_name) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4311\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = 1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4312\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = 0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4380\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp ^ tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4313\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((unsigned char)0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4314\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4381\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((fe->common.arg_info + i)->class_name) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4366\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = strcasecmp((fe->common.arg_info + i)->class_name, "parent");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4367\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4322\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L: /* CIL Label */ 
        tmp___1 = strcasecmp((fe->common.arg_info + i)->class_name, "self");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4323\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4315\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          fe_class_name = (fe->common.arg_info + i)->class_name;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4316\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          fe_class_name_len = (fe->common.arg_info + i)->class_name_len;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4321\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (fe->common.scope) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4317\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            fe_class_name = (fe->common.scope)->name;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4318\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            fe_class_name_len = (fe->common.scope)->name_length;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4319\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            fe_class_name = (fe->common.arg_info + i)->class_name;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4320\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            fe_class_name_len = (fe->common.arg_info + i)->class_name_len;
          }
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4326\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (proto->common.scope) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4324\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          fe_class_name = (proto->common.scope)->name;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4325\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          fe_class_name_len = (proto->common.scope)->name_length;
        } else {
          goto _L;
        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4368\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___4 = strcasecmp((proto->common.arg_info + i)->class_name, "parent");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4369\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___4) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4334\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___1: /* CIL Label */ 
        _L___0: /* CIL Label */ 
        tmp___3 = strcasecmp((proto->common.arg_info + i)->class_name, "self");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4335\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___3) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4327\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          proto_class_name = (proto->common.arg_info + i)->class_name;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4328\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          proto_class_name_len = (proto->common.arg_info + i)->class_name_len;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4333\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (proto->common.scope) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4329\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            proto_class_name = (proto->common.scope)->name;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4330\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            proto_class_name_len = (proto->common.scope)->name_length;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4331\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            proto_class_name = (proto->common.arg_info + i)->class_name;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4332\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            proto_class_name_len = (proto->common.arg_info + i)->class_name_len;
          }
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4339\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (proto->common.scope) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4338\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((proto->common.scope)->parent) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4336\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            proto_class_name = ((proto->common.scope)->parent)->name;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4337\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            proto_class_name_len = ((proto->common.scope)->parent)->name_length;
          } else {
            goto _L___1;
          }
        } else {
          goto _L___1;
        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4370\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___11 = strcasecmp(fe_class_name, proto_class_name);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4371\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___11 != 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4364\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )fe->common.type != 2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4340\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return ((unsigned char)0);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4362\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___8 = __builtin_strchr((char *)proto_class_name, '\\');
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4363\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )tmp___8 != (unsigned int )((void *)0)) {
            goto _L___2;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4359\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___9 = zend_memrchr((void const   *)fe_class_name, '\\',
                                   fe_class_name_len);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4360\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            colon = (char const   *)tmp___9;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4361\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )colon == (unsigned int )((void *)0)) {
              goto _L___2;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4357\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___10 = strcasecmp(colon + 1, proto_class_name);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "4358\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (tmp___10 != 0) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4351\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                _L___2: /* CIL Label */ 
                tmp___5 = zend_lookup_class(fe_class_name,
                                            (int )fe_class_name_len, & fe_ce);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4352\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                found = (int )tmp___5;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4353\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___6 = zend_lookup_class(proto_class_name,
                                            (int )proto_class_name_len,
                                            & proto_ce);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4354\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                found2 = (int )tmp___6;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4355\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (found != 0) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "4341\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  return ((unsigned char)0);
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "4350\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (found2 != 0) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "4342\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    return ((unsigned char)0);
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "4349\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if ((int )(*fe_ce)->type == 1) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4343\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      return ((unsigned char)0);
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "4348\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if ((int )(*proto_ce)->type == 1) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4344\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        return ((unsigned char)0);
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "4347\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if ((unsigned int )*fe_ce != (unsigned int )*proto_ce) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "4345\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          return ((unsigned char)0);
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "4346\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }

                        }
                      }
                    }
                  }
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "4356\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
          }
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4365\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4372\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4382\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )(fe->common.arg_info + i)->type_hint != (int )(proto->common.arg_info + i)->type_hint) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4373\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((unsigned char)0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4374\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4383\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )(fe->common.arg_info + i)->pass_by_reference != (int )(proto->common.arg_info + i)->pass_by_reference) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4375\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((unsigned char)0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4376\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4384\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4401\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (proto->common.fn_flags & 16777216U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4391\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i = (unsigned int )proto->common.num_args;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4392\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4388\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (i < (zend_uint )fe->common.num_args) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4385\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4389\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! (fe->common.arg_info + i)->pass_by_reference) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4386\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((unsigned char)0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4387\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4390\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      i ++;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4393\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4402\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned char)1);
}
}
static void do_inherit_method(zend_function *function ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4280\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  function_add_ref(function);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4281\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
char *zend_visibility_string(zend_uint fn_flags ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4276\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (fn_flags & 1024U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4270\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((char *)"private");
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4271\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4277\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (fn_flags & 512U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4272\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((char *)"protected");
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4273\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4278\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (fn_flags & 256U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4274\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((char *)"public");
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4275\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4279\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((char *)"");
}
}
static void do_inherit_parent_constructor(zend_class_entry *ce ) 
{ zend_function *function ;
  zend_function *new_function ;
  char *lc_class_name ;
  char *lc_parent_class_name ;
  char __attribute__((__visibility__("default")))  *tmp ;
  char __attribute__((__visibility__("default")))  *tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;
  int __attribute__((__visibility__("default")))  tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4250\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->parent) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4194\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4195\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4251\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce->create_object = (ce->parent)->create_object;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4252\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->get_iterator) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4196\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->get_iterator = (ce->parent)->get_iterator;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4197\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4253\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->iterator_funcs.funcs) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4198\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->iterator_funcs.funcs = (ce->parent)->iterator_funcs.funcs;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4199\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4254\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->__get) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4200\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__get = (ce->parent)->__get;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4201\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4255\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->__set) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4202\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__set = (ce->parent)->__set;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4203\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4256\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->__unset) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4204\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__unset = (ce->parent)->__unset;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4205\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4257\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->__isset) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4206\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__isset = (ce->parent)->__isset;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4207\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4258\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->__call) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4208\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__call = (ce->parent)->__call;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4209\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4259\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->__callstatic) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4210\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__callstatic = (ce->parent)->__callstatic;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4211\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4260\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->__tostring) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4212\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->__tostring = (ce->parent)->__tostring;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4213\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4261\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->clone) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4214\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->clone = (ce->parent)->clone;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4215\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4262\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->serialize) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4216\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->serialize = (ce->parent)->serialize;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4217\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4263\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->unserialize) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4218\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->unserialize = (ce->parent)->unserialize;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4219\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4264\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! ce->destructor) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4220\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ce->destructor = (ce->parent)->destructor;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4221\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4265\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ce->constructor) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4226\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((ce->parent)->constructor) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4224\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (((ce->parent)->constructor)->common.fn_flags & 4U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4222\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1, "Cannot override final %s::%s() with %s::%s()",
                   (ce->parent)->name,
                   ((ce->parent)->constructor)->common.function_name, ce->name,
                   (ce->constructor)->common.function_name);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4223\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4225\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4227\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4228\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4266\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = zend_hash_find((HashTable const   *)(& (ce->parent)->function_table),
                           "__construct", sizeof("__construct"),
                           (void **)(& function));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4267\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4 == (int __attribute__((__visibility__("default")))  )0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4229\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zend_hash_add_or_update(& ce->function_table, "__construct",
                             sizeof("__construct"), (void *)function,
                             sizeof(zend_function ), (void **)(& new_function),
                             1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4230\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    function_add_ref(new_function);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4245\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_str_tolower_dup(ce->name, ce->name_length);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4246\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    lc_class_name = (char *)tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4247\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = zend_hash_exists((HashTable const   *)(& ce->function_table),
                               (char const   *)lc_class_name,
                               ce->name_length + 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4248\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4231\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4240\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_str_tolower_dup((ce->parent)->name,
                                     (ce->parent)->name_length);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4241\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      lc_parent_class_name = (char *)tmp___0;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4242\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_hash_exists((HashTable const   *)(& ce->function_table),
                                 (char const   *)lc_parent_class_name,
                                 (ce->parent)->name_length + 1U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4243\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4232\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4238\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___2 = zend_hash_find((HashTable const   *)(& (ce->parent)->function_table),
                                 (char const   *)lc_parent_class_name,
                                 (ce->parent)->name_length + 1U,
                                 (void **)(& function));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4239\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___2 == (int __attribute__((__visibility__("default")))  )0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4236\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (function->common.fn_flags & 8192U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4233\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _zend_hash_add_or_update(& ce->function_table,
                                     (char const   *)lc_parent_class_name,
                                     (ce->parent)->name_length + 1U,
                                     (void *)function, sizeof(zend_function ),
                                     (void **)(& new_function), 1);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4234\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            function_add_ref(new_function);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "4235\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "4237\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4244\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)lc_parent_class_name);
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4249\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)lc_class_name);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4268\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ce->constructor = (ce->parent)->constructor;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4269\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  zend_hash_copy(HashTable *target ,
                                                                       HashTable *source ,
                                                                       void (*pCopyConstructor)(void *pElement ) ,
                                                                       void *tmp ,
                                                                       uint size ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_num_elements(HashTable const   *ht ) ;
void __attribute__((__visibility__("default")))  function_add_ref(zend_function *function ) 
{ zend_op_array *op_array ;
  HashTable *static_variables ;
  zval *tmp_zval ;
  void __attribute__((__visibility__("default")))  *tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4192\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )function->type == 2) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4187\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op_array = & function->op_array;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4188\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (*(op_array->refcount)) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4189\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op_array->static_variables) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4180\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      static_variables = op_array->static_variables;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4181\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = _emalloc(sizeof(HashTable ));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4182\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      op_array->static_variables = (HashTable *)tmp;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4183\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_hash_num_elements((HashTable const   *)static_variables);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4184\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zend_hash_init(op_array->static_variables, (unsigned int )tmp___0,
                      (ulong (*)(char const   *arKey , uint nKeyLength ))((void *)0),
                      (void (*)(void * ))(& _zval_ptr_dtor), (unsigned char)0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4185\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_hash_copy(op_array->static_variables, static_variables,
                     (void (*)(void *pElement ))(& zval_add_ref),
                     (void *)(& tmp_zval), sizeof(zval *));
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4186\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4190\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op_array->run_time_cache = (void **)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4191\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4193\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_throw(znode const   *expr ) 
{ zend_op *opline ;
  int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4175\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4176\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)108;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4177\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4173\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )expr->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4174\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4170\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = zend_add_literal(compiler_globals.active_op_array,
                             & expr->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4171\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4172\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )expr->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4178\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4179\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_throw(znode const   *expr ) ;
void zend_do_end_catch(znode const   *try_token ) 
{ int jmp_op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *tmp___0 ;
  zend_llist *jmp_list_ptr ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4158\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4159\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  jmp_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4160\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4161\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4162\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)42;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4163\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4164\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4165\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.bp_stack),
                 (void **)(& jmp_list_ptr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4166\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_add_element(jmp_list_ptr, (void *)(& jmp_op_number));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4167\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4168\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + try_token->u.op.opline_num)->extended_value = (unsigned long )tmp___1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4169\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_end_catch(znode const   *try_token ) ;
void zend_do_begin_catch(znode *try_token , znode *class_name ,
                         znode *catch_var , znode *first_catch ) 
{ long catch_op_number ;
  zend_op *opline ;
  znode catch_class ;
  int tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4142\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_name->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4137\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_get_class_fetch_type((char const   *)class_name->u.constant.value.str.val,
                                    (unsigned int )class_name->u.constant.value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4138\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (0 == tmp) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4134\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_resolve_class_name(class_name, 4UL, 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4135\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      catch_class = *class_name;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4136\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Bad class name in the catch statement");
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4139\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Bad class name in the catch statement");
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4143\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4144\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  catch_op_number = (long )tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4145\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (first_catch) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4140\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    first_catch->u.op.opline_num = (unsigned int )catch_op_number;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4141\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4146\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4147\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)107;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4148\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4149\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_add_class_name_literal(compiler_globals.active_op_array,
                                        (zval const   *)(& catch_class.u.constant));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4150\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1.constant = (unsigned int )tmp___1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4151\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 4);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4152\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = lookup_cv(compiler_globals.active_op_array,
                      catch_var->u.constant.value.str.val,
                      catch_var->u.constant.value.str.len, 0UL);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4153\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2.var = (unsigned int )tmp___2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4154\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  catch_var->u.constant.value.str.val = (char *)((compiler_globals.active_op_array)->vars + opline->op2.var)->name;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4155\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.num = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4156\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  try_token->u.op.opline_num = (unsigned int )catch_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4157\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_begin_catch(znode *try_token , znode *class_name ,
                         znode *catch_var , znode *first_catch ) ;
static int zend_add_try_element(zend_uint try_op ) ;
void zend_do_try(znode *try_token ) 
{ int tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4129\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4130\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = zend_add_try_element((unsigned int )tmp);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4131\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  try_token->u.op.opline_num = (unsigned int )tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4132\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4127\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) ++;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4128\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4133\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_try(znode *try_token ) ;
void zend_do_if_end(void) ;
void zend_do_mark_last_catch(znode const   *first_catch ,
                             znode const   *last_additional_catch ) 
{ int tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4122\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->last) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4123\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_if_end();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4124\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (last_additional_catch->u.op.opline_num == 4294967295U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4114\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->opcodes + first_catch->u.op.opline_num)->result.num = 1U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4115\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = get_next_op_number(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4116\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->opcodes + first_catch->u.op.opline_num)->extended_value = (unsigned long )tmp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4117\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->opcodes + last_additional_catch->u.op.opline_num)->result.num = 1U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4118\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = get_next_op_number(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4119\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->opcodes + last_additional_catch->u.op.opline_num)->extended_value = (unsigned long )tmp___0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4125\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4120\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) --;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4121\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4126\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_mark_last_catch(znode const   *first_catch ,
                             znode const   *last_additional_catch ) ;
static void zend_add_catch_element(int offset , zend_uint catch_op ) ;
void zend_initialize_try_catch_element(znode const   *try_token ) 
{ int jmp_op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *tmp___0 ;
  zend_llist jmp_list ;
  zend_llist *jmp_list_ptr ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4100\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4101\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  jmp_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4102\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4103\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4104\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)42;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4105\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4106\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4107\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_init(& jmp_list, sizeof(int ), (void (*)(void * ))((void *)0),
                  (unsigned char)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4108\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.bp_stack,
                  (void const   *)((void *)(& jmp_list)),
                  (int )sizeof(zend_llist ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4109\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.bp_stack),
                 (void **)(& jmp_list_ptr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4110\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_add_element(jmp_list_ptr, (void *)(& jmp_op_number));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4111\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4112\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_add_catch_element((int )try_token->u.op.opline_num,
                         (unsigned int )tmp___1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4113\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_initialize_try_catch_element(znode const   *try_token ) ;
void zend_do_first_catch(znode *open_parentheses ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4097\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4098\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  open_parentheses->u.op.opline_num = (unsigned int )tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4099\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_first_catch(znode *open_parentheses ) ;
struct _zend_try_catch_element {
   zend_uint try_op ;
   zend_uint catch_op ;
};
static void zend_add_catch_element(int offset , zend_uint catch_op ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4095\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->try_catch_array + offset)->catch_op = catch_op;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4096\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static int zend_add_try_element(zend_uint try_op ) 
{ int try_catch_offset ;
  int tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4088\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = (compiler_globals.active_op_array)->last_try_catch;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4089\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->last_try_catch) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4090\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  try_catch_offset = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4091\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = _erealloc((void *)(compiler_globals.active_op_array)->try_catch_array,
                      sizeof(zend_try_catch_element ) * (unsigned int )(compiler_globals.active_op_array)->last_try_catch,
                      0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4092\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (compiler_globals.active_op_array)->try_catch_array = (zend_try_catch_element *)tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4093\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->try_catch_array + try_catch_offset)->try_op = try_op;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4094\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (try_catch_offset);
}
}
extern void __attribute__((__visibility__("default")))  zend_stack_apply(zend_stack *stack ,
                                                                         int type ,
                                                                         int (*apply_function)(void *element ) ) ;
static int generate_free_switch_expr(zend_switch_entry const   *switch_entry ) ;
void zend_do_return(znode *expr , int do_end_vparse ) 
{ zend_op *opline ;
  int start_op_number ;
  int end_op_number ;
  zend_bool tmp ;
  int tmp___0 ;
  zend_bool tmp___1 ;
  zval _c ;
  int tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4077\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (do_end_vparse) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4052\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((compiler_globals.active_op_array)->fn_flags & 67108864U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4049\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = zend_is_function_or_method_call((znode const   *)expr);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4050\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4047\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_do_end_variable_parse(expr, 0, 0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4048\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_do_end_variable_parse(expr, 1, 0);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4051\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_do_end_variable_parse(expr, 0, 0);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4053\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4078\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  start_op_number = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4079\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_apply(& compiler_globals.switch_cond_stack, 1,
                   (int (*)(void *element ))(& generate_free_switch_expr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4080\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_apply(& compiler_globals.foreach_copy_stack, 1,
                   (int (*)(void *element ))(& generate_free_foreach_copy));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4081\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  end_op_number = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4082\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4055\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (start_op_number < end_op_number) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4054\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4056\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->opcodes + start_op_number)->extended_value |= (unsigned long )(1 << 2);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4057\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    start_op_number ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4083\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4084\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 67108864U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4058\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)111;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4059\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)62;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4085\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (expr) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4070\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4063\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1_type = (unsigned char )expr->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4064\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (expr->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4060\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                   (zval const   *)(& expr->u.constant));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4061\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1.constant = (unsigned int )tmp___0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4062\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1 = expr->u.op;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4071\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (do_end_vparse) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4067\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_is_function_or_method_call((znode const   *)expr);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4068\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4065\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->extended_value = 1UL;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "4066\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4069\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4075\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char)1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4076\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4072\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _c = (struct _zval_struct )zval_used_for_init;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4073\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& _c));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4074\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___2;
      break;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4086\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4087\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_return(znode *expr , int do_end_vparse ) ;
static int generate_free_foreach_copy(zend_op const   *foreach_copy ) 
{ zend_op *opline ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4039\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )foreach_copy->result_type == (int const   )(1 << 3)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4023\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )foreach_copy->op1_type == (int const   )(1 << 3)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4021\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (1);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4022\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4024\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4040\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4041\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )foreach_copy->result_type == (int const   )(1 << 1)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4025\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)70;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4026\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)49;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4042\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4027\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )foreach_copy->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4028\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1 = (union _znode_op )foreach_copy->result;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4043\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4044\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = 1UL;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4045\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )foreach_copy->op1_type != (int const   )(1 << 3)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4033\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline = get_next_op(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4034\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )foreach_copy->op1_type == (int const   )(1 << 1)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4029\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char)70;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4030\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char)49;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4035\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4031\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1_type = (unsigned char )foreach_copy->op1_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4032\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )foreach_copy->op1;
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4036\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4037\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value = 0UL;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4038\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4046\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (0);
}
}
static int generate_free_switch_expr(zend_switch_entry const   *switch_entry ) 
{ zend_op *opline ;
  int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4014\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (switch_entry->cond.op_type != (int const   )(1 << 2)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4005\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (switch_entry->cond.op_type != (int const   )(1 << 1)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4003\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (switch_entry->cond.op_type == (int const   )(1 << 3));
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4004\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4006\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4015\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4016\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (switch_entry->cond.op_type == (int const   )(1 << 1)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4007\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)70;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4008\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)49;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4017\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4012\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )switch_entry->cond.op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "4013\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (switch_entry->cond.op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4009\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = zend_add_literal(compiler_globals.active_op_array,
                             & switch_entry->cond.u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4010\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "4011\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )switch_entry->cond.u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4018\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4019\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = 0UL;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4020\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (0);
}
}
void zend_do_pass_param(znode *param , zend_uchar op , int offset ) 
{ zend_op *opline ;
  int original_op ;
  zend_function **function_ptr_ptr ;
  zend_function *function_ptr ;
  int send_by_reference ;
  int send_function ;
  unsigned int tmp ;
  zend_bool tmp___0 ;
  unsigned int tmp___1 ;
  unsigned int tmp___2 ;
  unsigned int tmp___3 ;
  zend_bool tmp___4 ;
  int tmp___5 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3987\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  original_op = (int )op;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3988\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  send_function = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3989\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.function_call_stack),
                 (void **)(& function_ptr_ptr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3990\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  function_ptr = *function_ptr_ptr;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3991\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (original_op == 67) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3921\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (function_ptr) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3919\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (function_ptr->common.function_name) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3917\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )function_ptr->common.type == 2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3915\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (function_ptr) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3912\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (function_ptr->common.arg_info) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3908\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((unsigned int )offset <= function_ptr->common.num_args) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3906\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp = (unsigned int )((int )(function_ptr->common.arg_info + ((unsigned int )offset - 1U))->pass_by_reference & 3);
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3907\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp = function_ptr->common.fn_flags & 50331648U;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3909\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp = function_ptr->common.fn_flags & 50331648U;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3913\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (tmp) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3910\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              zend_error(1 << 6L, "Call-time pass-by-reference has been removed");
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3911\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              zend_error(1 << 6L,
                         "Call-time pass-by-reference has been removed; If you would like to pass argument by reference, modify the declaration of %s().",
                         function_ptr->common.function_name);
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3914\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_error(1 << 6L,
                       "Call-time pass-by-reference has been removed; If you would like to pass argument by reference, modify the declaration of %s().",
                       function_ptr->common.function_name);
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3916\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L, "Call-time pass-by-reference has been removed");
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3918\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L, "Call-time pass-by-reference has been removed");
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3920\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Call-time pass-by-reference has been removed");
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3922\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3923\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3992\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (function_ptr) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3954\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (function_ptr) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3942\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (function_ptr->common.arg_info) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3926\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )offset <= function_ptr->common.num_args) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3924\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___3 = (unsigned int )((int )(function_ptr->common.arg_info + ((unsigned int )offset - 1U))->pass_by_reference & 2);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3925\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___3 = function_ptr->common.fn_flags & 33554432U;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3927\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = function_ptr->common.fn_flags & 33554432U;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3943\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___3) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3941\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (param->op_type & ((1 << 2) | (1 << 4))) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3938\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (original_op != 65) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3934\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            send_by_reference = 1;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3935\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )op == 66) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3931\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___0 = zend_is_function_or_method_call((znode const   *)param);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3932\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (tmp___0) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3928\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                op = (unsigned char)106;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3929\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                send_function = (1 << 2) | (1 << 3);
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3930\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3933\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3936\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            op = (unsigned char)65;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3937\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            send_by_reference = 0;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3939\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          op = (unsigned char)65;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3940\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          send_by_reference = 0;
        }
      } else {
        goto _L;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3953\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L: /* CIL Label */ 
      if (function_ptr) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3950\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (function_ptr->common.arg_info) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3946\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )offset <= function_ptr->common.num_args) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3944\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___2 = (unsigned int )((int )(function_ptr->common.arg_info + ((unsigned int )offset - 1U))->pass_by_reference & 3);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3945\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___2 = function_ptr->common.fn_flags & 50331648U;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3947\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___2 = function_ptr->common.fn_flags & 50331648U;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3951\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3948\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          send_by_reference = 1;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3949\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          send_by_reference = 0;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3952\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        send_by_reference = 0;
      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3955\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    send_by_reference = 0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3993\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )op == 66) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3958\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___4 = zend_is_function_or_method_call((znode const   *)param);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3959\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___4) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3956\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      op = (unsigned char)106;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3957\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      send_function = 1 << 2;
    } else {
      goto _L___0;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3964\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _L___0: /* CIL Label */ 
    if ((int )op == 65) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3962\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (param->op_type & ((1 << 2) | (1 << 4))) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3960\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        op = (unsigned char)106;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3961\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3963\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3994\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )op != 106) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3968\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (send_by_reference == 1) {
      switch (param->op_type) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3965\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 1 << 2: 
      case 1 << 4: 
      op = (unsigned char)67;
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3966\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      default: 
      zend_error(1 << 6L, "Only variables can be passed by reference");
      break;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3967\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3969\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3995\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (original_op == 66) {
    switch ((int )op) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3972\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 106: 
    zend_do_end_variable_parse(param, 0, 0);
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3973\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 66: 
    if (function_ptr) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3970\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_do_end_variable_parse(param, 0, 0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3971\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_do_end_variable_parse(param, 5, offset);
    }
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3974\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 67: 
    zend_do_end_variable_parse(param, 1, 0);
    break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3975\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3996\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3997\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )op == 106) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3978\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (function_ptr) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3976\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->extended_value = (unsigned long )(((1 << 1) | send_by_reference) | send_function);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3977\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->extended_value = (unsigned long )send_function;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3981\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (function_ptr) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3979\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->extended_value = 60UL;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3980\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->extended_value = 61UL;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3998\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = op;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3999\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3985\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )param->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3986\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (param->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3982\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___5 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& param->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3983\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___5;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3984\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = param->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4000\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2.opline_num = (unsigned int )offset;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4001\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4002\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_pass_param(znode *param , zend_uchar op , int offset ) ;
void zend_do_end_function_call(znode *function_name , znode *result ,
                               znode const   *argument_list , int is_method ,
                               int is_dynamic_fcall ) 
{ zend_op *opline ;
  int tmp ;
  ulong __attribute__((__visibility__("default")))  tmp___0 ;
  int tmp___1 ;
  void __attribute__((__visibility__("default")))  *tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3898\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (is_method) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3856\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (function_name) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3855\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (function_name->op_type == 1 << 3) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3853\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (argument_list->u.constant.value.lval != 0L) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3851\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 1L, "Clone method does not require arguments");
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3852\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3854\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline = (compiler_globals.active_op_array)->opcodes + function_name->u.constant.value.lval;
      } else {
        goto _L___0;
      }
    } else {
      goto _L___0;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3891\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _L___0: /* CIL Label */ 
    _L: /* CIL Label */ 
    opline = get_next_op(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3892\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! is_method) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3888\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! is_dynamic_fcall) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3885\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (function_name->op_type == 1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3879\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->opcode = (unsigned char)60;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3880\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3860\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            opline->op1_type = (unsigned char )function_name->op_type;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3861\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (function_name->op_type == 1) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3857\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp = zend_add_literal(compiler_globals.active_op_array,
                                     (zval const   *)(& function_name->u.constant));
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3858\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              opline->op1.constant = (unsigned int )tmp;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3859\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              opline->op1 = function_name->u.op;
            }
            break;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3881\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3868\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3865\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3862\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val - sizeof(Bucket )))->h;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3863\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___0 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                                         (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len + 1));
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3864\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = (unsigned long )tmp___0;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3866\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___0 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                                       (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len + 1));
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3867\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = (unsigned long )tmp___0;
            }
            break;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3882\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3875\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___1 = (compiler_globals.active_op_array)->last_cache_slot;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3876\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            ((compiler_globals.active_op_array)->last_cache_slot) ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3877\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            ((compiler_globals.active_op_array)->literals + opline->op1.constant)->cache_slot = (unsigned int )tmp___1;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3878\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((compiler_globals.active_op_array)->fn_flags & 16U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3873\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((compiler_globals.active_op_array)->run_time_cache) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3869\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___2 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                                    (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                                    0);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3870\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___2;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3871\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3872\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3874\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
            break;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3883\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->opcode = (unsigned char)61;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3884\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op1_type = (unsigned char )(1 << 3);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3886\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->opcode = (unsigned char)61;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3887\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1_type = (unsigned char )(1 << 3);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3889\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char)61;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3890\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1_type = (unsigned char )(1 << 3);
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3899\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3900\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3901\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3896\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3897\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3893\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3894\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3895\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3902\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3903\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_del_top(& compiler_globals.function_call_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3904\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = (unsigned long )argument_list->u.constant.value.lval;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3905\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
int zend_add_func_name_literal(zend_op_array *op_array , zval const   *zv ) ;
int zend_do_begin_class_member_function_call(znode *class_name ,
                                             znode *method_name ) 
{ znode class_node ;
  unsigned char *ptr ;
  zend_op *opline ;
  char *lcname ;
  char __attribute__((__visibility__("default")))  *tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;
  int tmp___4 ;
  int tmp___5 ;
  void __attribute__((__visibility__("default")))  *tmp___6 ;
  void __attribute__((__visibility__("default")))  *tmp___7 ;
  int tmp___8 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3842\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ptr = (unsigned char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3843\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (method_name->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3784\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )method_name->u.constant.type != 6) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3776\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Method name must be a string");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3777\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3785\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_str_tolower_dup((char const   *)method_name->u.constant.value.str.val,
                               (unsigned int )method_name->u.constant.value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3786\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    lcname = (char *)tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3787\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (sizeof("__construct") - 1U == (unsigned int )method_name->u.constant.value.str.len) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3781\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = memcmp((void const   *)lcname, (void const   *)"__construct",
                       sizeof("__construct") - 1U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3782\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___0 == 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3778\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _zval_dtor(& method_name->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3779\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        method_name->op_type = 1 << 3;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3780\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3783\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3788\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)lcname);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3789\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3844\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_name->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3796\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = zend_get_class_fetch_type((char const   *)class_name->u.constant.value.str.val,
                                        (unsigned int )class_name->u.constant.value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3797\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (0 == tmp___1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3790\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_resolve_class_name(class_name, 4UL, 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3791\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      class_node = *class_name;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3792\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline = get_next_op(compiler_globals.active_op_array);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3793\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_do_fetch_class(& class_node, class_name);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3794\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline = get_next_op(compiler_globals.active_op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3795\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->extended_value = (unsigned long )class_node.EA;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3798\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_fetch_class(& class_node, class_name);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3799\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline = get_next_op(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3800\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value = (unsigned long )class_node.EA;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3845\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)113;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3846\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_node.op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3801\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char)1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3802\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = zend_add_class_name_literal(compiler_globals.active_op_array,
                                          (zval const   *)(& class_node.u.constant));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3803\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1.constant = (unsigned int )tmp___2;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3809\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3807\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1_type = (unsigned char )class_node.op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3808\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (class_node.op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3804\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = zend_add_literal(compiler_globals.active_op_array,
                                   (zval const   *)(& class_node.u.constant));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3805\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1.constant = (unsigned int )tmp___3;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3806\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1 = class_node.u.op;
      }
      break;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3847\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (method_name->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3832\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char)1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3833\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___4 = zend_add_func_name_literal(compiler_globals.active_op_array,
                                         (zval const   *)(& method_name->u.constant));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3834\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2.constant = (unsigned int )tmp___4;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3835\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )opline->op1_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3820\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3816\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___5 = (compiler_globals.active_op_array)->last_cache_slot;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3817\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->last_cache_slot) ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3818\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->literals + opline->op2.constant)->cache_slot = (unsigned int )tmp___5;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3819\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((compiler_globals.active_op_array)->fn_flags & 16U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3814\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((compiler_globals.active_op_array)->run_time_cache) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3810\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___6 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                                (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                                0);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3811\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___6;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3812\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3813\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3815\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        break;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3831\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3828\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->literals + opline->op2.constant)->cache_slot = (unsigned int )(compiler_globals.active_op_array)->last_cache_slot;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3829\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (compiler_globals.active_op_array)->last_cache_slot += 2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3830\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((compiler_globals.active_op_array)->fn_flags & 16U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3826\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((compiler_globals.active_op_array)->run_time_cache) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3821\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___7 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                                (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                                0);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3822\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___7;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3823\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3824\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 2)) = (void *)0;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3825\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3827\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        break;
      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3841\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3839\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2_type = (unsigned char )method_name->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3840\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (method_name->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3836\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___8 = zend_add_literal(compiler_globals.active_op_array,
                                   (zval const   *)(& method_name->u.constant));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3837\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2.constant = (unsigned int )tmp___8;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3838\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2 = method_name->u.op;
      }
      break;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3848\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.function_call_stack,
                  (void const   *)((void *)(& ptr)),
                  (int )sizeof(zend_function *));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3849\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_extended_fcall_begin();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3850\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (1);
}
}
int zend_do_begin_class_member_function_call(znode *class_name ,
                                             znode *method_name ) ;
void zend_do_build_full_name(znode *result , znode *prefix , znode *name ,
                             int is_class_member ) 
{ zend_uint length ;
  void __attribute__((__visibility__("default")))  *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3773\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! result) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3745\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = prefix;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3746\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *result = *prefix;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3774\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (is_class_member) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3753\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    length = ((sizeof("::") - 1U) + (unsigned int )result->u.constant.value.str.len) + (unsigned int )name->u.constant.value.str.len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3754\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = _erealloc((void *)result->u.constant.value.str.val, length + 1U, 0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3755\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->u.constant.value.str.val = (char *)tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3756\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    memcpy((void */* __restrict  */)(result->u.constant.value.str.val + result->u.constant.value.str.len),
           (void const   */* __restrict  */)"::", sizeof("::") - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3757\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    memcpy((void */* __restrict  */)(result->u.constant.value.str.val + (((unsigned int )result->u.constant.value.str.len + sizeof("::")) - 1U)),
           (void const   */* __restrict  */)name->u.constant.value.str.val,
           (unsigned int )(name->u.constant.value.str.len + 1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3758\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (name->u.constant.value.str.val) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3751\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )name->u.constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3749\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )name->u.constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3747\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3748\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _efree((void *)name->u.constant.value.str.val);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3750\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _efree((void *)name->u.constant.value.str.val);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3752\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3759\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->u.constant.value.str.len = (int )length;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3766\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    length = ((sizeof("\\") - 1U) + (unsigned int )result->u.constant.value.str.len) + (unsigned int )name->u.constant.value.str.len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3767\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = _erealloc((void *)result->u.constant.value.str.val, length + 1U, 0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3768\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->u.constant.value.str.val = (char *)tmp___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3769\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    memcpy((void */* __restrict  */)(result->u.constant.value.str.val + result->u.constant.value.str.len),
           (void const   */* __restrict  */)"\\", sizeof("\\") - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3770\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    memcpy((void */* __restrict  */)(result->u.constant.value.str.val + (((unsigned int )result->u.constant.value.str.len + sizeof("\\")) - 1U)),
           (void const   */* __restrict  */)name->u.constant.value.str.val,
           (unsigned int )(name->u.constant.value.str.len + 1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3771\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (name->u.constant.value.str.val) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3764\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )name->u.constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3762\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )name->u.constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3760\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3761\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _efree((void *)name->u.constant.value.str.val);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3763\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _efree((void *)name->u.constant.value.str.val);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3765\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3772\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->u.constant.value.str.len = (int )length;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3775\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
struct _zend_compiler_context;
extern int __attribute__((__visibility__("default")))  zend_stack_is_empty(zend_stack const   *stack ) ;
void zend_release_labels(void) 
{ zend_compiler_context *ctx ;
  int __attribute__((__visibility__("default")))  tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3741\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.context.labels) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3734\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_hash_destroy(compiler_globals.context.labels);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3735\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)compiler_globals.context.labels);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3736\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3742\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_stack_is_empty((zend_stack const   *)(& compiler_globals.context_stack));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3743\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3737\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3738\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_stack_top((zend_stack const   *)(& compiler_globals.context_stack),
                   (void **)(& ctx));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3739\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.context = *ctx;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3740\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_stack_del_top(& compiler_globals.context_stack);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3744\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_release_labels(void) ;
void zend_resolve_goto_label(zend_op_array *op_array , zend_op *opline ,
                             int pass2 ) ;
void zend_do_goto(znode const   *label ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3726\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3727\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3728\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)100;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3729\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = (unsigned long )compiler_globals.context.current_brk_cont;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3730\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3731\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3724\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )label->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3725\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (label->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3721\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & label->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3722\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3723\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2 = (union _znode_op )label->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3732\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_resolve_goto_label(compiler_globals.active_op_array, opline, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3733\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_goto(znode const   *label ) ;
struct _zend_label;
typedef struct _zend_label zend_label;
struct _zend_label {
   int brk_cont ;
   zend_uint opline_num ;
};
void zend_resolve_goto_label(zend_op_array *op_array , zend_op *opline ,
                             int pass2 ) 
{ zend_label *dest ;
  long current ;
  long distance ;
  zval *label ;
  int __attribute__((__visibility__("default")))  tmp ;
  zval *__z ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3710\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (pass2) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3674\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    label = opline->op2.zv;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3675\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    label = & (op_array->literals + opline->op2.constant)->constant;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3711\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )compiler_globals.context.labels == (unsigned int )((void *)0)) {
    goto _L;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3686\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_hash_find((HashTable const   *)compiler_globals.context.labels,
                         (char const   *)label->value.str.val,
                         (unsigned int )(label->value.str.len + 1),
                         (void **)(& dest));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3687\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp == (int __attribute__((__visibility__("default")))  )-1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3684\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L: /* CIL Label */ 
      if (pass2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3676\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        compiler_globals.in_compilation = (unsigned char)1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3677\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        compiler_globals.active_op_array = op_array;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3678\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        compiler_globals.zend_lineno = (int )opline->lineno;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3679\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L, "\'goto\' to undefined label \'%s\'",
                   label->value.str.val);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3682\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (op_array->fn_flags & 16U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3680\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          (compiler_globals.context.backpatch_count) ++;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3681\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3683\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3685\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3712\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1.opline_num = dest->opline_num;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3713\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_dtor(label);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3714\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  label->type = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3715\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  current = (long )opline->extended_value;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3716\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  distance = 0L;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3717\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3696\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (current != (long )dest->brk_cont) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3688\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3697\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (current == -1L) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3693\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (pass2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3689\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        compiler_globals.in_compilation = (unsigned char)1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3690\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        compiler_globals.active_op_array = op_array;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3691\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        compiler_globals.zend_lineno = (int )opline->lineno;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3692\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3694\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "\'goto\' into loop or switch statement is disallowed");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3695\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3698\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    current = (long )(op_array->brk_cont_array + current)->parent;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3699\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    distance ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3718\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (distance == 0L) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3700\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)42;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3701\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value = 0UL;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3702\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )(1 << 3);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3703\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z = label;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3704\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->value.lval = distance;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3705\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->type = (unsigned char)1;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3719\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (pass2) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3708\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op_array->fn_flags & 16U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3706\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (compiler_globals.context.backpatch_count) --;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3707\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3709\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3720\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_label(znode *label ) 
{ zend_label dest ;
  void __attribute__((__visibility__("default")))  *tmp ;
  int tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3666\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! compiler_globals.context.labels) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3660\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = _emalloc(sizeof(HashTable ));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3661\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.context.labels = (HashTable *)tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3662\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zend_hash_init(compiler_globals.context.labels, 4U,
                    (ulong (*)(char const   *arKey , uint nKeyLength ))((void *)0),
                    (void (*)(void *pDest ))((void *)0), (unsigned char)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3663\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3667\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  dest.brk_cont = compiler_globals.context.current_brk_cont;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3668\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3669\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  dest.opline_num = (unsigned int )tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3670\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = _zend_hash_add_or_update(compiler_globals.context.labels,
                                     (char const   *)label->u.constant.value.str.val,
                                     (unsigned int )(label->u.constant.value.str.len + 1),
                                     (void *)((void **)(& dest)),
                                     sizeof(zend_label ), (void **)((void *)0),
                                     1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3671\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3664\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Label \'%s\' already defined",
               label->u.constant.value.str.val);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3665\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3672\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_dtor(& label->u.constant);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3673\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_label(znode *label ) ;
void zend_do_fetch_class(znode *result , znode *class_name ) 
{ long fetch_class_op_number ;
  zend_op *opline ;
  int tmp ;
  int fetch_type ;
  int tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3646\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_name->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3625\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )class_name->u.constant.type == 6) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3623\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (class_name->u.constant.value.str.len == 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3619\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _zval_dtor(& class_name->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3620\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L, "Cannot use \'namespace\' as a class name");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3621\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3622\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3624\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3626\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3647\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3648\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fetch_class_op_number = (long )tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3649\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3650\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)109;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3651\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3652\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->extended_value = 4UL;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3653\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.catch_begin = fetch_class_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3654\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_name->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3634\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    fetch_type = zend_get_class_fetch_type((char const   *)class_name->u.constant.value.str.val,
                                           (unsigned int )class_name->u.constant.value.str.len);
    switch (fetch_type) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3627\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 1: 
    case 2: 
    case 7: 
    opline->op2_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3628\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value = (unsigned long )fetch_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3629\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zval_dtor(& class_name->u.constant);
    break;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3630\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    default: 
    zend_resolve_class_name(class_name, opline->extended_value, 0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3631\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char)1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3632\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = zend_add_class_name_literal(compiler_globals.active_op_array,
                                          (zval const   *)(& class_name->u.constant));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3633\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2.constant = (unsigned int )tmp___0;
    break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3640\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3638\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2_type = (unsigned char )class_name->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3639\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (class_name->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3635\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                   (zval const   *)(& class_name->u.constant));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3636\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2.constant = (unsigned int )tmp___1;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3637\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2 = class_name->u.op;
      }
      break;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3655\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3656\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3657\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3644\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3645\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3641\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3642\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3643\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3658\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->EA = (unsigned int )opline->extended_value;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3659\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_resolve_class_name(znode *class_name , ulong fetch_type ,
                             int check_ns_name ) 
{ char *compound ;
  char *lcname ;
  zval **ns ;
  znode tmp ;
  int len ;
  void *tmp___0 ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  int tmp___2 ;
  char __attribute__((__visibility__("default")))  *tmp___3 ;
  int __attribute__((__visibility__("default")))  tmp___4 ;
  char __attribute__((__visibility__("default")))  *tmp___5 ;
  int __attribute__((__visibility__("default")))  tmp___6 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3615\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = memchr((void const   *)class_name->u.constant.value.str.val, '\\',
                   (unsigned int )class_name->u.constant.value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3616\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compound = (char *)tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3617\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compound) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3595\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )*(class_name->u.constant.value.str.val + 0) == 92) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3563\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (class_name->u.constant.value.str.len) --;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3564\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      memmove((void *)class_name->u.constant.value.str.val,
              (void const   *)(class_name->u.constant.value.str.val + 1),
              (unsigned int )(class_name->u.constant.value.str.len + 1));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3565\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = _erealloc((void *)class_name->u.constant.value.str.val,
                          (unsigned int )(class_name->u.constant.value.str.len + 1),
                          0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3566\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      class_name->u.constant.value.str.val = (char *)tmp___1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3567\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_get_class_fetch_type((char const   *)class_name->u.constant.value.str.val,
                                          (unsigned int )class_name->u.constant.value.str.len);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3568\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (0 != tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3561\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L, "\'\\%s\' is an invalid class name",
                   class_name->u.constant.value.str.val);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3562\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3593\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (compiler_globals.current_import) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3580\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        len = compound - class_name->u.constant.value.str.val;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3581\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = zend_str_tolower_dup((char const   *)class_name->u.constant.value.str.val,
                                       (unsigned int )len);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3582\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        lcname = (char *)tmp___3;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3583\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___4 = zend_hash_find((HashTable const   *)compiler_globals.current_import,
                                 (char const   *)lcname,
                                 (unsigned int )(len + 1), (void **)(& ns));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3584\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___4 == (int __attribute__((__visibility__("default")))  )0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3569\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp.op_type = 1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3570\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp.u.constant = *(*ns);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3571\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _zval_copy_ctor(& tmp.u.constant);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3572\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          len ++;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3573\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          class_name->u.constant.value.str.len -= len;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3574\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          memmove((void *)class_name->u.constant.value.str.val,
                  (void const   *)(class_name->u.constant.value.str.val + len),
                  (unsigned int )(class_name->u.constant.value.str.len + 1));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3575\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_do_build_namespace_name(& tmp, & tmp, class_name);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3576\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *class_name = tmp;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3577\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _efree((void *)lcname);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3578\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3579\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3585\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _efree((void *)lcname);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3586\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3594\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (compiler_globals.current_namespace) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3587\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp.op_type = 1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3588\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp.u.constant = *(compiler_globals.current_namespace);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3589\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _zval_copy_ctor(& tmp.u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3590\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_do_build_namespace_name(& tmp, & tmp, class_name);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3591\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *class_name = tmp;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3592\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3614\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.current_import) {
      goto _L___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3613\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (compiler_globals.current_namespace) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3608\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___0: /* CIL Label */ 
        tmp___5 = zend_str_tolower_dup((char const   *)class_name->u.constant.value.str.val,
                                       (unsigned int )class_name->u.constant.value.str.len);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3609\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        lcname = (char *)tmp___5;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3610\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (compiler_globals.current_import) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3599\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___6 = zend_hash_find((HashTable const   *)compiler_globals.current_import,
                                   (char const   *)lcname,
                                   (unsigned int )(class_name->u.constant.value.str.len + 1),
                                   (void **)(& ns));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3600\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___6 == (int __attribute__((__visibility__("default")))  )0) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3596\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _zval_dtor(& class_name->u.constant);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3597\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            class_name->u.constant = *(*ns);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3598\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _zval_copy_ctor(& class_name->u.constant);
          } else {
            goto _L;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3607\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L: /* CIL Label */ 
          if (compiler_globals.current_namespace) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3601\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp.op_type = 1;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3602\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp.u.constant = *(compiler_globals.current_namespace);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3603\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _zval_copy_ctor(& tmp.u.constant);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3604\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_do_build_namespace_name(& tmp, & tmp, class_name);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3605\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *class_name = tmp;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3606\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3611\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _efree((void *)lcname);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3612\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3618\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_resolve_non_class_name(znode *element_name ,
                                 zend_bool check_namespace ) 
{ znode tmp ;
  int len ;
  zval **ns ;
  char *lcname ;
  char *compound ;
  void *tmp___0 ;
  char __attribute__((__visibility__("default")))  *tmp___1 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;
  void __attribute__((__visibility__("default")))  *tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3554\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = memchr((void const   *)element_name->u.constant.value.str.val, '\\',
                   (unsigned int )element_name->u.constant.value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3555\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compound = (char *)tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3556\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*(element_name->u.constant.value.str.val + 0) == 92) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3512\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    memmove((void *)element_name->u.constant.value.str.val,
            (void const   *)(element_name->u.constant.value.str.val + 1),
            (unsigned int )element_name->u.constant.value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3513\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (element_name->u.constant.value.str.len) --;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3514\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3515\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3557\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! check_namespace) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3516\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3517\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3558\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compound) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3536\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.current_import) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3529\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      len = compound - element_name->u.constant.value.str.val;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3530\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_str_tolower_dup((char const   *)element_name->u.constant.value.str.val,
                                     (unsigned int )len);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3531\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      lcname = (char *)tmp___1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3532\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_hash_find((HashTable const   *)compiler_globals.current_import,
                               (char const   *)lcname, (unsigned int )(len + 1),
                               (void **)(& ns));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3533\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2 == (int __attribute__((__visibility__("default")))  )0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3518\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp.op_type = 1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3519\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp.u.constant = *(*ns);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3520\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _zval_copy_ctor(& tmp.u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3521\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        len ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3522\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        element_name->u.constant.value.str.len -= len;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3523\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        memmove((void *)element_name->u.constant.value.str.val,
                (void const   *)(element_name->u.constant.value.str.val + len),
                (unsigned int )(element_name->u.constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3524\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_do_build_namespace_name(& tmp, & tmp, element_name);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3525\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *element_name = tmp;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3526\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _efree((void *)lcname);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3527\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3528\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3534\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)lcname);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3535\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3537\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3559\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.current_namespace) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3544\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = *element_name;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3545\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp.u.constant.value.str.len = (int )(((sizeof("\\") - 1U) + (unsigned int )element_name->u.constant.value.str.len) + (unsigned int )(compiler_globals.current_namespace)->value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3546\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = _emalloc((unsigned int )(tmp.u.constant.value.str.len + 1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3547\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp.u.constant.value.str.val = (char *)tmp___3;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3548\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    memcpy((void */* __restrict  */)tmp.u.constant.value.str.val,
           (void const   */* __restrict  */)(compiler_globals.current_namespace)->value.str.val,
           (unsigned int )(compiler_globals.current_namespace)->value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3549\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    memcpy((void */* __restrict  */)(tmp.u.constant.value.str.val + (compiler_globals.current_namespace)->value.str.len),
           (void const   */* __restrict  */)"\\", sizeof("\\") - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3550\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    memcpy((void */* __restrict  */)(tmp.u.constant.value.str.val + (((unsigned int )(compiler_globals.current_namespace)->value.str.len + sizeof("\\")) - 1U)),
           (void const   */* __restrict  */)element_name->u.constant.value.str.val,
           (unsigned int )(element_name->u.constant.value.str.len + 1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3551\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (element_name->u.constant.value.str.val) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3542\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )element_name->u.constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3540\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )element_name->u.constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3538\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3539\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _efree((void *)element_name->u.constant.value.str.val);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3541\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _efree((void *)element_name->u.constant.value.str.val);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3543\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3552\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *element_name = tmp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3553\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3560\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
int zend_add_ns_func_name_literal(zend_op_array *op_array , zval const   *zv ) ;
void zend_do_begin_dynamic_function_call(znode *function_name , int ns_call ) 
{ unsigned char *ptr ;
  zend_op *opline ;
  int tmp ;
  int tmp___0 ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;
  void __attribute__((__visibility__("default")))  *tmp___4 ;
  int tmp___5 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3506\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ptr = (unsigned char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3507\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3508\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ns_call) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3477\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)69;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3478\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3479\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char)1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3480\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_add_ns_func_name_literal(compiler_globals.active_op_array,
                                        (zval const   *)(& function_name->u.constant));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3481\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2.constant = (unsigned int )tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3482\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3473\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = (compiler_globals.active_op_array)->last_cache_slot;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3474\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((compiler_globals.active_op_array)->last_cache_slot) ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3475\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((compiler_globals.active_op_array)->literals + opline->op2.constant)->cache_slot = (unsigned int )tmp___0;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3476\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((compiler_globals.active_op_array)->fn_flags & 16U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3471\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((compiler_globals.active_op_array)->run_time_cache) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3467\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___1 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                              (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                              0);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3468\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3469\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3470\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3472\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3503\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)59;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3504\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3505\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (function_name->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3493\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2_type = (unsigned char)1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3494\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_func_name_literal(compiler_globals.active_op_array,
                                           (zval const   *)(& function_name->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3495\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___2;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3496\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3489\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = (compiler_globals.active_op_array)->last_cache_slot;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3490\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->last_cache_slot) ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3491\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->literals + opline->op2.constant)->cache_slot = (unsigned int )tmp___3;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3492\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((compiler_globals.active_op_array)->fn_flags & 16U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3487\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((compiler_globals.active_op_array)->run_time_cache) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3483\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___4 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                                (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                                0);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3484\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___4;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3485\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3486\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3488\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        break;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3502\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3500\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2_type = (unsigned char )function_name->op_type;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3501\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (function_name->op_type == 1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3497\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___5 = zend_add_literal(compiler_globals.active_op_array,
                                     (zval const   *)(& function_name->u.constant));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3498\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op2.constant = (unsigned int )tmp___5;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3499\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op2 = function_name->u.op;
        }
        break;
      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3509\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.function_call_stack,
                  (void const   *)((void *)(& ptr)),
                  (int )sizeof(zend_function *));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3510\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_extended_fcall_begin();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3511\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_begin_dynamic_function_call(znode *function_name , int ns_call ) ;
void zend_do_clone(znode *result , znode const   *expr ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3458\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3459\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3460\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)110;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3461\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3451\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )expr->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3452\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3448\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & expr->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3449\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3450\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )expr->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3462\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3463\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3464\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3465\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3456\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3457\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3453\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3454\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3455\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3466\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_clone(znode *result , znode const   *expr ) ;
void zend_do_begin_method_call(znode *left_bracket ) 
{ zend_op *last_op ;
  int last_op_number ;
  unsigned char *ptr ;
  int tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  zval name ;
  char __attribute__((__visibility__("default")))  *tmp___1 ;
  int tmp___2 ;
  void __attribute__((__visibility__("default")))  *tmp___3 ;
  zend_op *opline ;
  zend_op *tmp___4 ;
  int tmp___5 ;
  int tmp___6 ;
  void __attribute__((__visibility__("default")))  *tmp___7 ;
  int tmp___8 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3437\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ptr = (unsigned char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3438\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_end_variable_parse(left_bracket, 0, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3439\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_begin_variable_parse();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3440\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3441\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  last_op_number = tmp - 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3442\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  last_op = (compiler_globals.active_op_array)->opcodes + last_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3443\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )last_op->op2_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3374\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )((compiler_globals.active_op_array)->literals + last_op->op2.constant)->constant.type == 6) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3372\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + last_op->op2.constant)->constant.value.str.len == sizeof("__clone") - 1U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3369\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = zend_binary_strcasecmp((char const   *)((compiler_globals.active_op_array)->literals + last_op->op2.constant)->constant.value.str.val,
                                         (unsigned int )((compiler_globals.active_op_array)->literals + last_op->op2.constant)->constant.value.str.len,
                                         "__clone", sizeof("__clone") - 1U);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3370\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3367\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3368\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L,
                     "Cannot call __clone() method on objects - use \'clone $obj\' instead");
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3371\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3373\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3375\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3444\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )last_op->opcode == 82) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3408\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )last_op->op2_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3400\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      name = ((compiler_globals.active_op_array)->literals + last_op->op2.constant)->constant;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3401\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )name.type != 6) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3376\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L, "Method name must be a string");
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3377\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3402\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )name.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3381\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )name.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3378\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3379\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___1 = _estrndup((char const   *)name.value.str.val,
                              (unsigned int )name.value.str.len);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3380\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          name.value.str.val = (char *)tmp___1;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3382\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = _estrndup((char const   *)name.value.str.val,
                            (unsigned int )name.value.str.len);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3383\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        name.value.str.val = (char *)tmp___1;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3403\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3389\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (((compiler_globals.active_op_array)->literals + last_op->op2.constant)->cache_slot != 4294967295U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3387\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (((compiler_globals.active_op_array)->literals + last_op->op2.constant)->cache_slot == (zend_uint )((compiler_globals.active_op_array)->last_cache_slot - 2)) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3384\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            ((compiler_globals.active_op_array)->literals + last_op->op2.constant)->cache_slot = 4294967295U;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3385\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.active_op_array)->last_cache_slot -= 2;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3386\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3388\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3404\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_func_name_literal(compiler_globals.active_op_array,
                                           (zval const   *)(& name));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3405\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      last_op->op2.constant = (unsigned int )tmp___2;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3406\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3397\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->literals + last_op->op2.constant)->cache_slot = (unsigned int )(compiler_globals.active_op_array)->last_cache_slot;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3398\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (compiler_globals.active_op_array)->last_cache_slot += 2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3399\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((compiler_globals.active_op_array)->fn_flags & 16U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3395\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((compiler_globals.active_op_array)->run_time_cache) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3390\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___3 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                                (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                                0);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3391\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___3;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3392\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3393\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 2)) = (void *)0;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3394\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3396\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        break;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3407\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3409\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op->opcode = (unsigned char)112;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3410\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op->result_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3411\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    left_bracket->u.constant.value.lval = 59L;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3432\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___4 = get_next_op(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3433\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline = tmp___4;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3434\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)59;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3435\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3436\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (left_bracket->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3422\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2_type = (unsigned char)1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3423\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___5 = zend_add_func_name_literal(compiler_globals.active_op_array,
                                           (zval const   *)(& left_bracket->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3424\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___5;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3425\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3418\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___6 = (compiler_globals.active_op_array)->last_cache_slot;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3419\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->last_cache_slot) ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3420\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((compiler_globals.active_op_array)->literals + opline->op2.constant)->cache_slot = (unsigned int )tmp___6;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3421\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((compiler_globals.active_op_array)->fn_flags & 16U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3416\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((compiler_globals.active_op_array)->run_time_cache) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3412\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___7 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                                (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                                0);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3413\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___7;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3414\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3415\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3417\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        break;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3431\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3429\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2_type = (unsigned char )left_bracket->op_type;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3430\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (left_bracket->op_type == 1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3426\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___8 = zend_add_literal(compiler_globals.active_op_array,
                                     (zval const   *)(& left_bracket->u.constant));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3427\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op2.constant = (unsigned int )tmp___8;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3428\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op2 = left_bracket->u.op;
        }
        break;
      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3445\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.function_call_stack,
                  (void const   *)((void *)(& ptr)),
                  (int )sizeof(zend_function *));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3446\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_extended_fcall_begin();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3447\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_begin_method_call(znode *left_bracket ) ;
int zend_do_begin_function_call(znode *function_name ,
                                zend_bool check_namespace ) 
{ zend_function *function ;
  char *lcname ;
  char *is_compound ;
  void *tmp ;
  char __attribute__((__visibility__("default")))  *tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3354\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = memchr((void const   *)function_name->u.constant.value.str.val, '\\',
               (unsigned int )function_name->u.constant.value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3355\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  is_compound = (char *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3356\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_resolve_non_class_name(function_name, check_namespace);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3357\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (check_namespace) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3342\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.current_namespace) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3340\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! is_compound) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3337\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_do_begin_dynamic_function_call(function_name, 1);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3338\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (1);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3339\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3341\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3343\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3358\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = zend_str_tolower_dup((char const   *)function_name->u.constant.value.str.val,
                                 (unsigned int )function_name->u.constant.value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3359\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lcname = (char *)tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3360\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_hash_find((HashTable const   *)compiler_globals.function_table,
                           (char const   *)lcname,
                           (unsigned int )(function_name->u.constant.value.str.len + 1),
                           (void **)(& function));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3361\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___1 == (int __attribute__((__visibility__("default")))  )-1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3344\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_begin_dynamic_function_call(function_name, 0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3345\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)lcname);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3346\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return (1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3353\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.compiler_options & (unsigned int )(1 << 2)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3351\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )function->type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3347\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_do_begin_dynamic_function_call(function_name, 0);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3348\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _efree((void *)lcname);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3349\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (1);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3350\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3352\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3362\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _efree((void *)function_name->u.constant.value.str.val);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3363\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  function_name->u.constant.value.str.val = lcname;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3364\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.function_call_stack,
                  (void const   *)((void *)(& function)),
                  (int )sizeof(zend_function *));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3365\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_extended_fcall_begin();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3366\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (0);
}
}
int zend_do_begin_function_call(znode *function_name ,
                                zend_bool check_namespace ) ;
void zend_do_receive_arg(zend_uchar op , znode *varname ,
                         znode const   *offset , znode const   *initialization ,
                         znode *class_type , unsigned char pass_by_reference ) 
{ zend_op *opline ;
  zend_arg_info *cur_arg_info ;
  znode var ;
  int tmp ;
  int tmp___0 ;
  zend_bool __attribute__((__visibility__("default")))  tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;
  int tmp___4 ;
  void __attribute__((__visibility__("default")))  *tmp___5 ;
  char __attribute__((__visibility__("default")))  *tmp___6 ;
  char const __attribute__((__visibility__("default")))  *tmp___7 ;
  int tmp___8 ;
  int tmp___9 ;
  int tmp___10 ;
  char const __attribute__((__visibility__("default")))  *tmp___11 ;
  int tmp___12 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3314\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_type->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3225\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )class_type->u.constant.type == 6) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3223\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (class_type->u.constant.value.str.len == 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3219\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _zval_dtor(& class_type->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3220\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L, "Cannot use \'namespace\' as a class name");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3221\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3222\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3224\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3226\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3315\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_is_auto_global_quick((char const   *)varname->u.constant.value.str.val,
                                      (unsigned int )varname->u.constant.value.str.len,
                                      0UL);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3316\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3227\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Cannot re-assign auto-global variable %s",
               varname->u.constant.value.str.val);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3240\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    var.op_type = 1 << 4;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3241\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = lookup_cv(compiler_globals.active_op_array,
                    varname->u.constant.value.str.val,
                    varname->u.constant.value.str.len, 0UL);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3242\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    var.u.op.var = (unsigned int )tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3243\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    varname->u.constant.value.str.val = (char *)((compiler_globals.active_op_array)->vars + var.u.op.var)->name;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3244\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    var.EA = 0U;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3245\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (((compiler_globals.active_op_array)->vars + var.u.op.var)->hash_value == 275574653UL) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3238\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )varname->u.constant.value.str.len == sizeof("this") - 1U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3235\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = memcmp((void const   *)varname->u.constant.value.str.val,
                         (void const   *)"this", sizeof("this") - 1U);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3236\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3228\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3233\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((compiler_globals.active_op_array)->scope) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3231\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (((compiler_globals.active_op_array)->fn_flags & 1U) == 0U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3229\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              zend_error(1 << 6L, "Cannot re-assign $this");
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3230\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3232\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3234\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          (compiler_globals.active_op_array)->this_var = var.u.op.var;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3237\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3239\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3317\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3318\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->num_args) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3319\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = op;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3320\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3249\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )var.op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3250\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (var.op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3246\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& var.u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3247\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result.constant = (unsigned int )tmp___2;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3248\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result = var.u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3321\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3254\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )offset->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3255\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (offset->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3251\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___3 = zend_add_literal(compiler_globals.active_op_array,
                                 & offset->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3252\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___3;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3253\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )offset->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3322\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )op == 64) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3261\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3259\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2_type = (unsigned char )initialization->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3260\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (initialization->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3256\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___4 = zend_add_literal(compiler_globals.active_op_array,
                                   & initialization->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3257\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2.constant = (unsigned int )tmp___4;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3258\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op2 = (union _znode_op )initialization->u.op;
      }
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3262\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.active_op_array)->required_num_args = (compiler_globals.active_op_array)->num_args;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3263\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )(1 << 3);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3323\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___5 = _erealloc((void *)(compiler_globals.active_op_array)->arg_info,
                      sizeof(zend_arg_info ) * (compiler_globals.active_op_array)->num_args,
                      0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3324\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (compiler_globals.active_op_array)->arg_info = (zend_arg_info *)tmp___5;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3325\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  cur_arg_info = (compiler_globals.active_op_array)->arg_info + ((compiler_globals.active_op_array)->num_args - 1U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3326\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___6 = _estrndup((char const   *)varname->u.constant.value.str.val,
                      (unsigned int )varname->u.constant.value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3327\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___7 = (*zend_new_interned_string)((char const   *)tmp___6,
                                        varname->u.constant.value.str.len + 1, 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3328\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  cur_arg_info->name = (char const   *)tmp___7;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3329\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  cur_arg_info->name_len = (unsigned int )varname->u.constant.value.str.len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3330\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  cur_arg_info->type_hint = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3331\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  cur_arg_info->allow_null = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3332\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  cur_arg_info->pass_by_reference = pass_by_reference;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3333\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  cur_arg_info->class_name = (char const   *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3334\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  cur_arg_info->class_name_len = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3335\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_type->op_type != 1 << 3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3311\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    cur_arg_info->allow_null = (unsigned char)0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3312\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )class_type->u.constant.type != 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3309\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )class_type->u.constant.type == 4) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3276\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        cur_arg_info->type_hint = (unsigned char)4;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3277\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )op == 64) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3274\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )initialization->u.constant.type == 0) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3264\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            cur_arg_info->allow_null = (unsigned char)1;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3273\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )initialization->u.constant.type == 8) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3266\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___8 = strcasecmp((char const   *)initialization->u.constant.value.str.val,
                                   "NULL");
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3267\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (tmp___8) {
                goto _L;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3265\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                cur_arg_info->allow_null = (unsigned char)1;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3272\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if ((int const   )initialization->u.constant.type != 4) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3270\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )initialization->u.constant.type != 9) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "3268\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  zend_error(1 << 6L,
                             "Default value for parameters with array type hint can only be an array or NULL");
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "3269\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3271\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3275\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3308\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )class_type->u.constant.type == 10) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3287\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          cur_arg_info->type_hint = (unsigned char)10;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3288\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )op == 64) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3285\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )initialization->u.constant.type == 0) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3278\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              cur_arg_info->allow_null = (unsigned char)1;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3284\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )initialization->u.constant.type == 8) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3281\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___9 = strcasecmp((char const   *)initialization->u.constant.value.str.val,
                                     "NULL");
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3282\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (tmp___9) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "3279\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  zend_error(1 << 6L,
                             "Default value for parameters with callable type hint can only be NULL");
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "3280\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  cur_arg_info->allow_null = (unsigned char)1;
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3283\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                zend_error(1 << 6L,
                           "Default value for parameters with callable type hint can only be NULL");
              }
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3286\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3300\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          cur_arg_info->type_hint = (unsigned char)5;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3301\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___10 = zend_get_class_fetch_type((char const   *)class_type->u.constant.value.str.val,
                                               (unsigned int )class_type->u.constant.value.str.len);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3302\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (0 == tmp___10) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3289\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_resolve_class_name(class_type, opline->extended_value, 1);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3290\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3303\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___11 = (*zend_new_interned_string)((char const   *)class_type->u.constant.value.str.val,
                                                 class_type->u.constant.value.str.len + 1,
                                                 1);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3304\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          class_type->u.constant.value.str.val = (char *)tmp___11;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3305\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          cur_arg_info->class_name = (char const   *)class_type->u.constant.value.str.val;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3306\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          cur_arg_info->class_name_len = (unsigned int )class_type->u.constant.value.str.len;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3307\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )op == 64) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3298\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )initialization->u.constant.type == 0) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3291\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              cur_arg_info->allow_null = (unsigned char)1;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3297\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )initialization->u.constant.type == 8) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3294\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___12 = strcasecmp((char const   *)initialization->u.constant.value.str.val,
                                      "NULL");
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3295\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (tmp___12) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "3292\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  zend_error(1 << 6L,
                             "Default value for parameters with a class type hint can only be NULL");
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "3293\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  cur_arg_info->allow_null = (unsigned char)1;
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3296\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                zend_error(1 << 6L,
                           "Default value for parameters with a class type hint can only be NULL");
              }
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3299\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3310\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3313\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3336\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_receive_arg(zend_uchar op , znode *varname ,
                         znode const   *offset , znode const   *initialization ,
                         znode *class_type , unsigned char pass_by_reference ) ;
extern void __attribute__((__visibility__("default")))  zend_check_magic_method_implementation(zend_class_entry const   *ce ,
                                                                                               zend_function const   *fptr ,
                                                                                               int error_type ) ;
extern int __attribute__((__visibility__("default")))  pass_two(zend_op_array *op_array ) ;
void zend_do_end_function_declaration(znode const   *function_token ) 
{ char lcname[16] ;
  int name_len ;
  size_t tmp ;
  unsigned int tmp___0 ;
  int tmp___1 ;
  int __attribute__((__visibility__("default")))  tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3208\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_extended_info();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3209\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_return((znode *)((void *)0), 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3210\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  pass_two(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3211\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_release_labels();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3212\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.active_class_entry) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3192\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_check_magic_method_implementation((zend_class_entry const   *)compiler_globals.active_class_entry,
                                           (zend_function const   *)((zend_function *)compiler_globals.active_op_array),
                                           1 << 6L);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3202\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = strlen((compiler_globals.active_op_array)->function_name);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3203\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    name_len = (int )tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3204\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )name_len < sizeof(lcname) - 1U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3193\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = (unsigned int )name_len;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3194\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = sizeof(lcname) - 1U;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3205\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_str_tolower_copy(lcname,
                          (compiler_globals.active_op_array)->function_name,
                          tmp___0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3206\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    lcname[sizeof(lcname) - 1U] = (char )'\000';
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3207\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )name_len == sizeof("__autoload") - 1U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3199\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = memcmp((void const   *)(lcname), (void const   *)"__autoload",
                       sizeof("__autoload"));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3200\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3195\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3198\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((compiler_globals.active_op_array)->num_args != 1U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3196\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L, "%s() must take exactly 1 argument", "__autoload");
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3197\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3201\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3213\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = zend_get_compiled_lineno();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3214\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (compiler_globals.active_op_array)->line_end = (unsigned int )tmp___2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3215\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.active_op_array = (zend_op_array *)function_token->u.op_array;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3216\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_del_top(& compiler_globals.switch_cond_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3217\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_del_top(& compiler_globals.foreach_copy_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3218\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_end_function_declaration(znode const   *function_token ) ;
void zend_do_handle_exception(void) 
{ zend_op *opline ;
  zend_op *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3186\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3187\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3188\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)149;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3189\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3190\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3191\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_handle_exception(void) ;
void zend_do_begin_function_declaration(znode *function_token ,
                                        znode *function_name , int is_method ,
                                        int return_reference ,
                                        znode *fn_flags_znode ) ;
void zend_do_begin_lambda_function_declaration(znode *result ,
                                               znode *function_token ,
                                               int return_reference ,
                                               int is_static ) 
{ znode function_name ;
  zend_op_array *current_op_array ;
  int current_op_number ;
  int tmp ;
  zend_op *current_op ;
  char const   *__s ;
  int __l ;
  zval *__z ;
  char __attribute__((__visibility__("default")))  *tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3170\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  current_op_array = compiler_globals.active_op_array;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3171\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3172\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  current_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3173\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  function_name.op_type = 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3174\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3156\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s = "{closure}";
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3157\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __l = (int )(sizeof("{closure}") - 1U);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3158\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z = & function_name.u.constant;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3159\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->value.str.len = __l;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3160\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = _estrndup(__s, (unsigned int )__l);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3161\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->value.str.val = (char *)tmp___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3162\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->type = (unsigned char)6;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3175\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_begin_function_declaration(function_token, & function_name, 0,
                                     return_reference, (znode *)((void *)0));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3176\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->op_type = 1 << 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3177\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->u.op.var = get_temporary_variable(current_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3178\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  current_op = current_op_array->opcodes + current_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3179\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  current_op->opcode = (unsigned char)153;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3180\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_del_literal(current_op_array, (int )current_op->op2.constant);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3181\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  current_op->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3182\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3166\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    current_op->result_type = (unsigned char )result->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3167\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3163\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& result->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3164\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      current_op->result.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3165\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      current_op->result = result->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3183\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (is_static) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3168\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.active_op_array)->fn_flags |= 1U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3169\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3184\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (compiler_globals.active_op_array)->fn_flags |= 1048576U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3185\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_begin_lambda_function_declaration(znode *result ,
                                               znode *function_token ,
                                               int return_reference ,
                                               int is_static ) ;
void zend_init_compiler_context(void) ;
extern void __attribute__((__visibility__("default")))  init_op_array(zend_op_array *op_array ,
                                                                      zend_uchar type ,
                                                                      int initial_ops_size ) ;
void zend_do_begin_function_declaration(znode *function_token ,
                                        znode *function_name , int is_method ,
                                        int return_reference ,
                                        znode *fn_flags_znode ) 
{ zend_op_array op_array ;
  char *name ;
  int name_len ;
  int function_begin_line ;
  zend_uint fn_flags ;
  char const   *lcname ;
  zend_bool orig_interactive ;
  zend_bool use_heap ;
  char const   *tmp ;
  char const   *tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;
  int result ;
  char __attribute__((__visibility__("default")))  *tmp___2 ;
  char const __attribute__((__visibility__("default")))  *tmp___3 ;
  int __attribute__((__visibility__("default")))  tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;
  int tmp___6 ;
  int tmp___7 ;
  int tmp___8 ;
  int tmp___9 ;
  int tmp___10 ;
  int tmp___11 ;
  int tmp___12 ;
  char *class_lcname ;
  long tmp___13 ;
  void __attribute__((__visibility__("default")))  *tmp___14 ;
  void *tmp___15 ;
  long tmp___16 ;
  int tmp___17 ;
  int tmp___18 ;
  int tmp___19 ;
  int tmp___20 ;
  int tmp___21 ;
  int tmp___22 ;
  int tmp___23 ;
  int tmp___24 ;
  int tmp___25 ;
  int tmp___26 ;
  int tmp___27 ;
  long tmp___28 ;
  zend_op *opline ;
  zend_op *tmp___29 ;
  zval key ;
  znode tmp___30 ;
  char __attribute__((__visibility__("default")))  *tmp___31 ;
  char __attribute__((__visibility__("default")))  *tmp___32 ;
  int tmp___33 ;
  ulong __attribute__((__visibility__("default")))  tmp___34 ;
  zval _c ;
  char const   *__s ;
  int __l ;
  zval *__z ;
  char __attribute__((__visibility__("default")))  *tmp___35 ;
  int tmp___36 ;
  ulong __attribute__((__visibility__("default")))  tmp___37 ;
  zend_op *opline___0 ;
  zend_op *tmp___38 ;
  zend_switch_entry switch_entry ;
  zend_op dummy_opline ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3128\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  name = function_name->u.constant.value.str.val;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3129\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  name_len = function_name->u.constant.value.str.len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3130\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  function_begin_line = (int )function_token->u.op.opline_num;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3131\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (is_method) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2891\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((compiler_globals.active_class_entry)->ce_flags & 128U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2888\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (fn_flags_znode->u.constant.value.lval & -258L) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2886\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 6L,
                   "Access type for interface method %s::%s() must be omitted",
                   (compiler_globals.active_class_entry)->name,
                   function_name->u.constant.value.str.val);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2887\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2889\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      fn_flags_znode->u.constant.value.lval |= 2L;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2890\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2892\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    fn_flags = (unsigned int )fn_flags_znode->u.constant.value.lval;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2893\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    fn_flags = 0U;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3132\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (fn_flags & 1U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2904\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (fn_flags & 2U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2902\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (! ((compiler_globals.active_class_entry)->ce_flags & 128U)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2898\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (is_method) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2894\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp = "::";
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2895\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp = "";
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2899\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (is_method) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2896\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___0 = (compiler_globals.active_class_entry)->name;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2897\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___0 = "";
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2900\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 11L, "Static function %s%s%s() should not be abstract",
                   tmp___0, tmp, function_name->u.constant.value.str.val);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2901\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2903\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2905\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3133\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  function_token->u.op_array = compiler_globals.active_op_array;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3134\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  orig_interactive = (unsigned char )compiler_globals.interactive;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3135\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.interactive = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3136\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  init_op_array(& op_array, (unsigned char)2, 64);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3137\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.interactive = (int )orig_interactive;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3138\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  op_array.function_name = (char const   *)name;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3139\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (return_reference) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2906\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op_array.fn_flags |= 67108864U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2907\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3140\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  op_array.fn_flags |= fn_flags;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3141\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (is_method) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2908\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op_array.scope = compiler_globals.active_class_entry;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2909\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op_array.scope = (zend_class_entry *)((void *)0);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3142\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  op_array.prototype = (union _zend_function *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3143\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_get_compiled_lineno();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3144\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  op_array.line_start = (unsigned int )tmp___1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3145\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (is_method) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3063\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = zend_str_tolower_dup((char const   *)name, (unsigned int )name_len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3064\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = (*zend_new_interned_string)((char const   *)tmp___2, name_len + 1,
                                          1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3065\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    lcname = (char const   *)tmp___3;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3066\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )lcname >= (unsigned int )compiler_globals.interned_strings_start) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2914\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )lcname < (unsigned int )compiler_globals.interned_strings_end) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2910\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___4 = _zend_hash_quick_add_or_update(& (compiler_globals.active_class_entry)->function_table,
                                                 lcname,
                                                 (unsigned int )(name_len + 1),
                                                 ((Bucket *)((char *)lcname - sizeof(Bucket )))->h,
                                                 (void *)(& op_array),
                                                 sizeof(zend_op_array ),
                                                 (void **)(& compiler_globals.active_op_array),
                                                 1 << 1);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2911\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result = (int )tmp___4;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2912\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___5 = _zend_hash_add_or_update(& (compiler_globals.active_class_entry)->function_table,
                                           lcname,
                                           (unsigned int )(name_len + 1),
                                           (void *)(& op_array),
                                           sizeof(zend_op_array ),
                                           (void **)(& compiler_globals.active_op_array),
                                           1 << 1);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2913\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result = (int )tmp___5;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2915\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___5 = _zend_hash_add_or_update(& (compiler_globals.active_class_entry)->function_table,
                                         lcname, (unsigned int )(name_len + 1),
                                         (void *)(& op_array),
                                         sizeof(zend_op_array ),
                                         (void **)(& compiler_globals.active_op_array),
                                         1 << 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2916\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result = (int )tmp___5;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3067\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result == -1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2917\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Cannot redeclare %s::%s()",
                 (compiler_globals.active_class_entry)->name, name);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2918\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3068\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_stack_push(& compiler_globals.context_stack,
                    (void const   *)((void *)(& compiler_globals.context)),
                    (int )sizeof(compiler_globals.context));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3069\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_init_compiler_context();
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3070\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (fn_flags & 2U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2919\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (compiler_globals.active_class_entry)->ce_flags |= 16U;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2920\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3071\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! (fn_flags & 1792U)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2921\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      fn_flags |= 256U;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2922\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3072\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((compiler_globals.active_class_entry)->ce_flags & 128U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2968\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )name_len == sizeof("__call") - 1U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2926\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___12 = memcmp((void const   *)lcname, (void const   *)"__call",
                          sizeof("__call") - 1U);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2927\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___12) {
          goto _L___4;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2925\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (fn_flags & 1537U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2923\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_error(1 << 1L,
                       "The magic method __call() must have public visibility and cannot be static");
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2924\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2967\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___4: /* CIL Label */ 
        if ((unsigned int )name_len == sizeof("__callstatic") - 1U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2933\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___11 = memcmp((void const   *)lcname,
                            (void const   *)"__callstatic",
                            sizeof("__callstatic") - 1U);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2934\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___11) {
            goto _L___3;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2932\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (fn_flags & 1536U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2928\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              zend_error(1 << 1L,
                         "The magic method __callStatic() must have public visibility and be static");
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2931\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((fn_flags & 1U) == 0U) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2929\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                zend_error(1 << 1L,
                           "The magic method __callStatic() must have public visibility and be static");
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2930\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2966\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L___3: /* CIL Label */ 
          if ((unsigned int )name_len == sizeof("__get") - 1U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2938\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___10 = memcmp((void const   *)lcname, (void const   *)"__get",
                              sizeof("__get") - 1U);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2939\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (tmp___10) {
              goto _L___2;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2937\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (fn_flags & 1537U) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2935\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                zend_error(1 << 1L,
                           "The magic method __get() must have public visibility and cannot be static");
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2936\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2965\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___2: /* CIL Label */ 
            if ((unsigned int )name_len == sizeof("__set") - 1U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2943\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___9 = memcmp((void const   *)lcname, (void const   *)"__set",
                               sizeof("__set") - 1U);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2944\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (tmp___9) {
                goto _L___1;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2942\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (fn_flags & 1537U) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2940\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  zend_error(1 << 1L,
                             "The magic method __set() must have public visibility and cannot be static");
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2941\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2964\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L___1: /* CIL Label */ 
              if ((unsigned int )name_len == sizeof("__unset") - 1U) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2948\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___8 = memcmp((void const   *)lcname,
                                 (void const   *)"__unset",
                                 sizeof("__unset") - 1U);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2949\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (tmp___8) {
                  goto _L___0;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2947\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (fn_flags & 1537U) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2945\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    zend_error(1 << 1L,
                               "The magic method __unset() must have public visibility and cannot be static");
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2946\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2963\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                _L___0: /* CIL Label */ 
                if ((unsigned int )name_len == sizeof("__isset") - 1U) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2953\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___7 = memcmp((void const   *)lcname,
                                   (void const   *)"__isset",
                                   sizeof("__isset") - 1U);
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2954\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (tmp___7) {
                    goto _L;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2952\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (fn_flags & 1537U) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "2950\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      zend_error(1 << 1L,
                                 "The magic method __isset() must have public visibility and cannot be static");
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "2951\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }

                    }
                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2962\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  _L: /* CIL Label */ 
                  if ((unsigned int )name_len == sizeof("__tostring") - 1U) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2959\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___6 = memcmp((void const   *)lcname,
                                     (void const   *)"__tostring",
                                     sizeof("__tostring") - 1U);
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2960\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (tmp___6) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "2955\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }

                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "2958\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (fn_flags & 1537U) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "2956\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        zend_error(1 << 1L,
                                   "The magic method __toString() must have public visibility and cannot be static");
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "2957\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }

                      }
                    }
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2961\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                }
              }
            }
          }
        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3052\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___16 = __builtin_expect((long )((compiler_globals.active_class_entry)->name_length + 1U > 32768U),
                                  0L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3053\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      use_heap = (unsigned char )tmp___16;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3054\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (use_heap) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2969\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___14 = _emalloc((compiler_globals.active_class_entry)->name_length + 1U);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2970\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        class_lcname = (char *)tmp___14;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2971\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___15 = __builtin_alloca((compiler_globals.active_class_entry)->name_length + 1U);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2972\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        class_lcname = (char *)tmp___15;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3055\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_str_tolower_copy(class_lcname,
                            (compiler_globals.active_class_entry)->name,
                            (compiler_globals.active_class_entry)->name_length);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3056\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((compiler_globals.active_class_entry)->name_length == (zend_uint )name_len) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2978\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (((compiler_globals.active_class_entry)->ce_flags & 288U) != 288U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2976\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___27 = memcmp((void const   *)class_lcname,
                            (void const   *)lcname, (unsigned int )name_len);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2977\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___27) {
            goto _L___16;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2975\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (! (compiler_globals.active_class_entry)->constructor) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2973\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              (compiler_globals.active_class_entry)->constructor = (zend_function *)compiler_globals.active_op_array;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2974\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        } else {
          goto _L___16;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3047\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___16: /* CIL Label */ 
        _L___15: /* CIL Label */ 
        if ((unsigned int )name_len == sizeof("__construct") - 1U) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2983\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___26 = memcmp((void const   *)lcname,
                            (void const   *)"__construct", sizeof("__construct"));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2984\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___26) {
            goto _L___14;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2981\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((compiler_globals.active_class_entry)->constructor) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2979\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              zend_error(1 << 11L,
                         "Redefining already defined constructor for class %s",
                         (compiler_globals.active_class_entry)->name);
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2980\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2982\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.active_class_entry)->constructor = (zend_function *)compiler_globals.active_op_array;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3046\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L___14: /* CIL Label */ 
          if ((unsigned int )name_len == sizeof("__destruct") - 1U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2986\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___25 = memcmp((void const   *)lcname,
                              (void const   *)"__destruct",
                              sizeof("__destruct") - 1U);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2987\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (tmp___25) {
              goto _L___13;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2985\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              (compiler_globals.active_class_entry)->destructor = (zend_function *)compiler_globals.active_op_array;
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "3045\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___13: /* CIL Label */ 
            if ((unsigned int )name_len == sizeof("__clone") - 1U) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2989\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___24 = memcmp((void const   *)lcname,
                                (void const   *)"__clone",
                                sizeof("__clone") - 1U);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2990\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (tmp___24) {
                goto _L___12;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2988\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                (compiler_globals.active_class_entry)->clone = (zend_function *)compiler_globals.active_op_array;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "3044\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L___12: /* CIL Label */ 
              if ((unsigned int )name_len == sizeof("__call") - 1U) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2995\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___23 = memcmp((void const   *)lcname,
                                  (void const   *)"__call",
                                  sizeof("__call") - 1U);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2996\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (tmp___23) {
                  goto _L___11;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2993\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (fn_flags & 1537U) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2991\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    zend_error(1 << 1L,
                               "The magic method __call() must have public visibility and cannot be static");
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2992\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2994\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  (compiler_globals.active_class_entry)->__call = (zend_function *)compiler_globals.active_op_array;
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "3043\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                _L___11: /* CIL Label */ 
                if ((unsigned int )name_len == sizeof("__callstatic") - 1U) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "3003\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___22 = memcmp((void const   *)lcname,
                                    (void const   *)"__callstatic",
                                    sizeof("__callstatic") - 1U);
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "3004\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (tmp___22) {
                    goto _L___10;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "3001\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (fn_flags & 1536U) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "2997\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      zend_error(1 << 1L,
                                 "The magic method __callStatic() must have public visibility and be static");
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "3000\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if ((fn_flags & 1U) == 0U) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "2998\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        zend_error(1 << 1L,
                                   "The magic method __callStatic() must have public visibility and be static");
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "2999\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }

                      }
                    }
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "3002\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    (compiler_globals.active_class_entry)->__callstatic = (zend_function *)compiler_globals.active_op_array;
                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "3042\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  _L___10: /* CIL Label */ 
                  if ((unsigned int )name_len == sizeof("__get") - 1U) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "3009\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___21 = memcmp((void const   *)lcname,
                                      (void const   *)"__get",
                                      sizeof("__get") - 1U);
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "3010\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (tmp___21) {
                      goto _L___9;
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "3007\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (fn_flags & 1537U) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "3005\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        zend_error(1 << 1L,
                                   "The magic method __get() must have public visibility and cannot be static");
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "3006\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }

                      }
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "3008\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      (compiler_globals.active_class_entry)->__get = (zend_function *)compiler_globals.active_op_array;
                    }
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "3041\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    _L___9: /* CIL Label */ 
                    if ((unsigned int )name_len == sizeof("__set") - 1U) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "3015\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___20 = memcmp((void const   *)lcname,
                                        (void const   *)"__set",
                                        sizeof("__set") - 1U);
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "3016\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if (tmp___20) {
                        goto _L___8;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "3013\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if (fn_flags & 1537U) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "3011\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          zend_error(1 << 1L,
                                     "The magic method __set() must have public visibility and cannot be static");
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "3012\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }

                        }
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "3014\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        (compiler_globals.active_class_entry)->__set = (zend_function *)compiler_globals.active_op_array;
                      }
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "3040\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      _L___8: /* CIL Label */ 
                      if ((unsigned int )name_len == sizeof("__unset") - 1U) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "3021\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp___19 = memcmp((void const   *)lcname,
                                          (void const   *)"__unset",
                                          sizeof("__unset") - 1U);
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "3022\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if (tmp___19) {
                          goto _L___7;
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "3019\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          if (fn_flags & 1537U) {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "3017\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            zend_error(1 << 1L,
                                       "The magic method __unset() must have public visibility and cannot be static");
                          } else {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "3018\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }

                          }
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "3020\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          (compiler_globals.active_class_entry)->__unset = (zend_function *)compiler_globals.active_op_array;
                        }
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "3039\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        _L___7: /* CIL Label */ 
                        if ((unsigned int )name_len == sizeof("__isset") - 1U) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "3027\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp___18 = memcmp((void const   *)lcname,
                                            (void const   *)"__isset",
                                            sizeof("__isset") - 1U);
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "3028\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          if (tmp___18) {
                            goto _L___6;
                          } else {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "3025\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            if (fn_flags & 1537U) {
                              {
                              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                     "a");
                              fprintf(_coverage_fout, "3023\n");
                              fflush(_coverage_fout);
                              fclose(_coverage_fout);
                              }
                              zend_error(1 << 1L,
                                         "The magic method __isset() must have public visibility and cannot be static");
                            } else {
                              {
                              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                     "a");
                              fprintf(_coverage_fout, "3024\n");
                              fflush(_coverage_fout);
                              fclose(_coverage_fout);
                              }

                            }
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "3026\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            (compiler_globals.active_class_entry)->__isset = (zend_function *)compiler_globals.active_op_array;
                          }
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "3038\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          _L___6: /* CIL Label */ 
                          if ((unsigned int )name_len == sizeof("__tostring") - 1U) {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "3033\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            tmp___17 = memcmp((void const   *)lcname,
                                              (void const   *)"__tostring",
                                              sizeof("__tostring") - 1U);
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "3034\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            if (tmp___17) {
                              goto _L___5;
                            } else {
                              {
                              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                     "a");
                              fprintf(_coverage_fout, "3031\n");
                              fflush(_coverage_fout);
                              fclose(_coverage_fout);
                              }
                              if (fn_flags & 1537U) {
                                {
                                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                       "a");
                                fprintf(_coverage_fout, "3029\n");
                                fflush(_coverage_fout);
                                fclose(_coverage_fout);
                                }
                                zend_error(1 << 1L,
                                           "The magic method __toString() must have public visibility and cannot be static");
                              } else {
                                {
                                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                       "a");
                                fprintf(_coverage_fout, "3030\n");
                                fflush(_coverage_fout);
                                fclose(_coverage_fout);
                                }

                              }
                              {
                              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                     "a");
                              fprintf(_coverage_fout, "3032\n");
                              fflush(_coverage_fout);
                              fclose(_coverage_fout);
                              }
                              (compiler_globals.active_class_entry)->__tostring = (zend_function *)compiler_globals.active_op_array;
                            }
                          } else {
                            {
                            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                   "a");
                            fprintf(_coverage_fout, "3037\n");
                            fflush(_coverage_fout);
                            fclose(_coverage_fout);
                            }
                            _L___5: /* CIL Label */ 
                            if (! (fn_flags & 1U)) {
                              {
                              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                     "a");
                              fprintf(_coverage_fout, "3035\n");
                              fflush(_coverage_fout);
                              fclose(_coverage_fout);
                              }
                              (compiler_globals.active_op_array)->fn_flags |= 65536U;
                            } else {
                              {
                              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                     "a");
                              fprintf(_coverage_fout, "3036\n");
                              fflush(_coverage_fout);
                              fclose(_coverage_fout);
                              }

                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3057\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3050\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___28 = __builtin_expect((long )use_heap, 0L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3051\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___28) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3048\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _efree((void *)class_lcname);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3049\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3073\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3062\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )lcname >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3060\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )lcname < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3058\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3059\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _efree((void *)((char *)lcname));
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3061\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _efree((void *)((char *)lcname));
      }
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3099\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___29 = get_next_op(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3100\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline = tmp___29;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3101\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (compiler_globals.current_namespace) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3074\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___30.u.constant = *(compiler_globals.current_namespace);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3075\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zval_copy_ctor(& tmp___30.u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3076\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_do_build_namespace_name(& tmp___30, & tmp___30, function_name);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3077\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      op_array.function_name = (char const   *)tmp___30.u.constant.value.str.val;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3078\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      name_len = tmp___30.u.constant.value.str.len;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3079\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___31 = zend_str_tolower_dup((char const   *)tmp___30.u.constant.value.str.val,
                                      (unsigned int )name_len);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3080\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      lcname = (char const   *)tmp___31;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3081\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___32 = zend_str_tolower_dup((char const   *)name,
                                      (unsigned int )name_len);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3082\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      lcname = (char const   *)tmp___32;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3102\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)141;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3103\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char)1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3104\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    build_runtime_defined_function_key(& key, lcname, name_len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3105\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___33 = zend_add_literal(compiler_globals.active_op_array,
                                (zval const   *)(& key));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3106\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1.constant = (unsigned int )tmp___33;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3107\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___34 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                              (unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3108\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = (unsigned long )tmp___34;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3109\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char)1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3110\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3089\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3083\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __s = lcname;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3084\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __l = name_len;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3085\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z = & _c;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3086\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z->value.str.len = __l;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3087\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3088\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z->type = (unsigned char)6;
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3090\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___36 = zend_add_literal(compiler_globals.active_op_array,
                                  (zval const   *)(& _c));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3091\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___36;
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3111\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "3098\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3095\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3092\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val - sizeof(Bucket )))->h;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3093\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___37 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val,
                                    (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len + 1));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "3094\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = (unsigned long )tmp___37;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3096\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___37 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.val,
                                  (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "3097\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op2.constant)->constant))->hash_value = (unsigned long )tmp___37;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3112\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value = 141UL;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3113\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _zend_hash_quick_add_or_update(compiler_globals.function_table,
                                   (char const   *)key.value.str.val,
                                   (unsigned int )key.value.str.len,
                                   ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value,
                                   (void *)(& op_array), sizeof(zend_op_array ),
                                   (void **)(& compiler_globals.active_op_array),
                                   1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3114\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_stack_push(& compiler_globals.context_stack,
                    (void const   *)((void *)(& compiler_globals.context)),
                    (int )sizeof(compiler_globals.context));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3115\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_init_compiler_context();
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3146\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.compiler_options & 1U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3116\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___38 = get_next_op(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3117\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline___0 = tmp___38;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3118\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline___0->opcode = (unsigned char)104;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3119\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline___0->lineno = (unsigned int )function_begin_line;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3120\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline___0->op1_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3121\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline___0->op2_type = (unsigned char )(1 << 3);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3122\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3147\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  switch_entry.cond.op_type = 1 << 3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3148\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  switch_entry.default_case = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3149\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  switch_entry.control_var = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3150\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.switch_cond_stack,
                  (void const   *)((void *)(& switch_entry)),
                  (int )sizeof(switch_entry));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3151\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  dummy_opline.result_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3152\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  dummy_opline.op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3153\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.foreach_copy_stack,
                  (void const   *)((void *)(& dummy_opline)),
                  (int )sizeof(zend_op ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3154\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (compiler_globals.doc_comment) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3123\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.active_op_array)->doc_comment = (char const   *)compiler_globals.doc_comment;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3124\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.active_op_array)->doc_comment_len = compiler_globals.doc_comment_len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3125\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment = (char *)((void *)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3126\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.doc_comment_len = 0U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "3127\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3155\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
int zend_do_verify_access_types(znode const   *current_access_type ,
                                znode const   *new_modifier ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2880\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (current_access_type->u.constant.value.lval & 1792L) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2864\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (new_modifier->u.constant.value.lval & 1792L) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2862\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Multiple access type modifiers are not allowed");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2863\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2865\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2881\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (current_access_type->u.constant.value.lval & 2L) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2868\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (new_modifier->u.constant.value.lval & 2L) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2866\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Multiple abstract modifiers are not allowed");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2867\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2869\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2882\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (current_access_type->u.constant.value.lval & 1L) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2872\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (new_modifier->u.constant.value.lval & 1L) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2870\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Multiple static modifiers are not allowed");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2871\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2873\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2883\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (current_access_type->u.constant.value.lval & 4L) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2876\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (new_modifier->u.constant.value.lval & 4L) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2874\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Multiple final modifiers are not allowed");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2875\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2877\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2884\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (((current_access_type->u.constant.value.lval | new_modifier->u.constant.value.lval) & 6L) == 6L) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2878\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L,
               "Cannot use the final modifier on an abstract class member");
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2879\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2885\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )(current_access_type->u.constant.value.lval | new_modifier->u.constant.value.lval));
}
}
int zend_do_verify_access_types(znode const   *current_access_type ,
                                znode const   *new_modifier ) ;
void zend_do_free(znode *op1 ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;
  zend_op *opline___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2860\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (op1->op_type == 1 << 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2811\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = get_next_op(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2812\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline = tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2813\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)70;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2814\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2809\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1_type = (unsigned char )op1->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2810\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (op1->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2806\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                   (zval const   *)(& op1->u.constant));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2807\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1.constant = (unsigned int )tmp___0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2808\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1 = op1->u.op;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2815\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )(1 << 3);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2859\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op1->op_type == 1 << 2) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2853\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline___0 = (compiler_globals.active_op_array)->opcodes + ((compiler_globals.active_op_array)->last - 1U);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2854\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2821\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )opline___0->opcode == 58) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2816\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2820\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )opline___0->opcode == 103) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2817\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2819\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )opline___0->opcode == 137) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2818\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            } else {
              break;
            }
          }
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2822\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline___0 --;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2855\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )opline___0->result_type == 1 << 2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2837\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (opline___0->result.var == op1->u.op.var) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2836\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )opline___0->opcode == 80) {
            goto _L;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2835\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )opline___0->opcode == 81) {
              goto _L;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2834\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int )opline___0->opcode == 82) {
                goto _L;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2833\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )opline___0->opcode == 157) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2828\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  _L: /* CIL Label */ 
                  opline___0 = get_next_op(compiler_globals.active_op_array);
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2829\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  opline___0->opcode = (unsigned char)70;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2830\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  while (1) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2826\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    opline___0->op1_type = (unsigned char )op1->op_type;
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2827\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if (op1->op_type == 1) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "2823\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                                 (zval const   *)(& op1->u.constant));
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "2824\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      opline___0->op1.constant = (unsigned int )tmp___1;
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "2825\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      opline___0->op1 = op1->u.op;
                    }
                    break;
                  }
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2831\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  opline___0->op2_type = (unsigned char )(1 << 3);
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2832\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  opline___0->result_type = (unsigned char )((int )opline___0->result_type | (1 << 5));
                }
              }
            }
          }
        } else {
          goto _L___2;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2852\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___2: /* CIL Label */ 
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2849\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )opline___0 > (unsigned int )(compiler_globals.active_op_array)->opcodes) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2838\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            break;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2850\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )opline___0->opcode == 81) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2841\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )opline___0->op1_type == 1 << 2) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2840\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (opline___0->op1.var == op1->u.op.var) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2839\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                opline___0->extended_value = 0UL;
                break;
              } else {
                goto _L___1;
              }
            } else {
              goto _L___1;
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2848\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___1: /* CIL Label */ 
            _L___0: /* CIL Label */ 
            if ((int )opline___0->result_type == 1 << 2) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2846\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (opline___0->result.var == op1->u.op.var) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2844\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int )opline___0->opcode == 68) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2842\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  opline___0->result_type = (unsigned char )((int )opline___0->result_type | (1 << 5));
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2843\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2845\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2847\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2851\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline___0 --;
        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2858\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (op1->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2856\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _zval_dtor(& op1->u.constant);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2857\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2861\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_add_variable(znode *result , znode const   *op1 ,
                          znode const   *op2 ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2799\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2800\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2801\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)56;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2802\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (op1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2784\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2777\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1_type = (unsigned char )op1->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2778\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (op1->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2774\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                   & op1->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2775\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1.constant = (unsigned int )tmp___0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2776\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1 = (union _znode_op )op1->u.op;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2785\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2782\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result_type = (unsigned char )op1->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2783\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (op1->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2779\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                   & op1->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2780\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->result.constant = (unsigned int )tmp___1;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2781\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->result = (union _znode_op )op1->u.op;
      }
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2786\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2787\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )(1 << 1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2788\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2803\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2792\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )op2->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2793\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op2->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2789\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                 & op2->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2790\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___2;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2791\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2 = (union _znode_op )op2->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2804\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2797\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2798\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2794\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2795\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2796\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2805\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_add_variable(znode *result , znode const   *op1 ,
                          znode const   *op2 ) ;
void zend_do_add_string(znode *result , znode const   *op1 , znode *op2 ) 
{ zend_op *opline ;
  int ch ;
  zval *__z ;
  int tmp ;
  int tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2769\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (op2->u.constant.value.str.len > 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2732\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline = get_next_op(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2733\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->opcode = (unsigned char)55;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2743\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op2->u.constant.value.str.len == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2734\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ch = (int )*(op2->u.constant.value.str.val);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2735\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)op2->u.constant.value.str.val);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2736\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z = & op2->u.constant;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2737\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->value.lval = (long )ch;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2738\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->type = (unsigned char)1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2739\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline = get_next_op(compiler_globals.active_op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2740\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char)54;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2741\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)op2->u.constant.value.str.val);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2742\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2770\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (op1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2754\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2747\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1_type = (unsigned char )op1->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2748\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (op1->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2744\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = zend_add_literal(compiler_globals.active_op_array,
                               & op1->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2745\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1.constant = (unsigned int )tmp;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2746\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->op1 = (union _znode_op )op1->u.op;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2755\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2752\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->result_type = (unsigned char )op1->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2753\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (op1->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2749\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                   & op1->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2750\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->result.constant = (unsigned int )tmp___0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2751\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline->result = (union _znode_op )op1->u.op;
      }
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2756\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2757\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )(1 << 1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2758\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2771\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2762\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )op2->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2763\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op2->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2759\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& op2->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2760\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2761\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2 = op2->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2772\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2767\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2768\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2764\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2765\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2766\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2773\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_add_string(znode *result , znode const   *op1 , znode *op2 ) ;
void zend_do_end_variable_parse(znode *variable , int type , int arg_offset ) 
{ zend_llist *fetch_list_ptr ;
  zend_llist_element *le ;
  zend_op *opline ;
  zend_op *opline_ptr ;
  zend_uint this_var ;
  int tmp ;
  char __attribute__((__visibility__("default")))  *tmp___0 ;
  int tmp___1 ;
  zend_bool tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2724\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = (zend_op *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2725\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  this_var = (zend_uint )-1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2726\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.bp_stack),
                 (void **)(& fetch_list_ptr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2727\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  le = fetch_list_ptr->head;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2728\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (le) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2718\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline_ptr = (zend_op *)(le->data);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2719\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = opline_is_fetch_this((zend_op const   *)opline_ptr);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2720\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___2) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2668\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((compiler_globals.active_op_array)->last == 0U) {
        goto _L;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2667\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )((compiler_globals.active_op_array)->opcodes + ((compiler_globals.active_op_array)->last - 1U))->opcode != 57) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2658\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L: /* CIL Label */ 
          this_var = opline_ptr->result.var;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2659\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((compiler_globals.active_op_array)->this_var == 4294967295U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2649\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp = lookup_cv(compiler_globals.active_op_array,
                            ((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant.value.str.val,
                            ((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant.value.str.len,
                            ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant))->hash_value);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2650\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.active_op_array)->this_var = (unsigned int )tmp;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2651\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            ((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant.type = (unsigned char)0;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2652\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            zend_del_literal(compiler_globals.active_op_array,
                             (int )opline_ptr->op1.constant);
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2660\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          le = le->next;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2661\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (variable->op_type == 1 << 2) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2656\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (variable->u.op.var == this_var) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2653\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              variable->op_type = 1 << 4;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2654\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              variable->u.op.var = (compiler_globals.active_op_array)->this_var;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2655\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2657\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2666\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((compiler_globals.active_op_array)->this_var == 4294967295U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2662\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___0 = _estrndup("this", sizeof("this") - 1U);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2663\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___1 = lookup_cv(compiler_globals.active_op_array,
                                (char *)tmp___0, (int )(sizeof("this") - 1U),
                                275574653UL);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2664\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            (compiler_globals.active_op_array)->this_var = (unsigned int )tmp___1;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2665\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2669\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2721\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2705\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (le) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2670\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2706\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline_ptr = (zend_op *)(le->data);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2707\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )opline_ptr->opcode == 156) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2676\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (type != 0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2674\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (type != 3) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2671\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            opline = get_next_op(compiler_globals.active_op_array);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2672\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            memcpy((void */* __restrict  */)opline,
                   (void const   */* __restrict  */)opline_ptr, sizeof(zend_op ));
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2673\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2675\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2677\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        le = le->next;
        continue;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2678\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2708\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline = get_next_op(compiler_globals.active_op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2709\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      memcpy((void */* __restrict  */)opline,
             (void const   */* __restrict  */)opline_ptr, sizeof(zend_op ));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2710\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )opline->op1_type == 1 << 2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2682\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (opline->op1.var == this_var) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2679\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op1_type = (unsigned char )(1 << 4);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2680\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->op1.var = (compiler_globals.active_op_array)->this_var;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2681\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2683\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      switch (type) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2696\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 0: 
      if ((int )opline->opcode == 84) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2686\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )opline->op2_type == 1 << 3) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2684\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L, "Cannot use [] for reading");
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2685\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2687\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2697\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char )((int )opline->opcode - 3);
      break;
      case 1: 
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2698\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 2: 
      opline->opcode = (unsigned char )((int )opline->opcode + 3);
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2699\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 3: 
      if ((int )opline->opcode == 84) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2690\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )opline->op2_type == 1 << 3) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2688\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L, "Cannot use [] for reading");
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2689\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2691\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2700\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char )((int )opline->opcode + 6);
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2701\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 5: 
      opline->opcode = (unsigned char )((int )opline->opcode + 9);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2702\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->extended_value |= (unsigned long )arg_offset;
      break;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2703\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      case 6: 
      if ((int )opline->opcode == 84) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2694\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )opline->op2_type == 1 << 3) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2692\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L, "Cannot use [] for unsetting");
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2693\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2695\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2704\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char )((int )opline->opcode + 12);
      break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2711\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      le = le->next;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2722\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (opline) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2716\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2714\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (arg_offset) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2712\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline->extended_value |= 67108864UL;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2713\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2715\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2717\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2723\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2729\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_destroy(fetch_list_ptr);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2730\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_del_top(& compiler_globals.bp_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2731\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_begin_variable_parse(void) 
{ zend_llist fetch_list ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2646\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_init(& fetch_list, sizeof(zend_op ),
                  (void (*)(void * ))((void *)0), (unsigned char)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2647\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_push(& compiler_globals.bp_stack,
                  (void const   *)((void *)(& fetch_list)),
                  (int )sizeof(zend_llist ));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2648\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_check_writable_variable(znode const   *variable ) 
{ zend_uint type ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2642\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  type = (zend_uint )variable->EA;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2643\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (type & (unsigned int )(1 << 1)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2638\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Can\'t use method return value in write context");
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2639\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2644\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (type == (zend_uint )(1 << 3)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2640\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_error(1 << 6L, "Can\'t use function return value in write context");
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2641\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2645\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_if_end(void) 
{ int next_op_number ;
  int tmp ;
  zend_llist *jmp_list_ptr ;
  zend_llist_element *le ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2629\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2630\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  next_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2631\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.bp_stack),
                 (void **)(& jmp_list_ptr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2632\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  le = jmp_list_ptr->head;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2633\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2624\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (le) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2623\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2625\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->opcodes + *((int *)(le->data)))->op1.opline_num = (unsigned int )next_op_number;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2626\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    le = le->next;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2634\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_destroy(jmp_list_ptr);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2635\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_del_top(& compiler_globals.bp_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2636\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2627\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) --;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2628\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2637\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_if_after_statement(znode const   *closing_bracket_token ,
                                unsigned char initialize ) 
{ int if_end_op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *tmp___0 ;
  zend_llist *jmp_list_ptr ;
  zend_llist jmp_list ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2611\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2612\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if_end_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2613\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2614\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2615\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)42;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2616\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (initialize) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2608\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_llist_init(& jmp_list, sizeof(int ), (void (*)(void * ))((void *)0),
                    (unsigned char)0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2609\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_stack_push(& compiler_globals.bp_stack,
                    (void const   *)((void *)(& jmp_list)),
                    (int )sizeof(zend_llist ));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2610\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2617\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.bp_stack),
                 (void **)(& jmp_list_ptr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2618\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_add_element(jmp_list_ptr, (void *)(& if_end_op_number));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2619\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + closing_bracket_token->u.op.opline_num)->op2.opline_num = (unsigned int )(if_end_op_number + 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2620\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2621\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2622\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_if_after_statement(znode const   *closing_bracket_token ,
                                unsigned char initialize ) ;
void zend_do_if_cond(znode const   *cond , znode *closing_bracket_token ) 
{ int if_cond_op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2598\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2599\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if_cond_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2600\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2601\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2602\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)43;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2603\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2594\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )cond->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2595\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (cond->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2591\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & cond->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2592\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2593\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )cond->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2604\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  closing_bracket_token->u.op.opline_num = (unsigned int )if_cond_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2605\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2606\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2596\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) ++;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2597\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2607\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_if_cond(znode const   *cond , znode *closing_bracket_token ) ;
void zend_do_post_incdec(znode *result , znode const   *op1 , zend_uchar op ) 
{ int last_op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *last_op ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2580\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2581\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  last_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2582\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (last_op_number > 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2567\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op = (compiler_globals.active_op_array)->opcodes + (last_op_number - 1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2568\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )last_op->opcode == 88) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2561\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )op == 36) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2554\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        last_op->opcode = (unsigned char)134;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2555\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        last_op->opcode = (unsigned char)135;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2562\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      last_op->result_type = (unsigned char )(1 << 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2563\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      last_op->result.var = get_temporary_variable(compiler_globals.active_op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2564\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2559\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->op_type = (int )last_op->result_type;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2560\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (result->op_type == 1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2556\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->u.constant = ((compiler_globals.active_op_array)->literals + last_op->result.constant)->constant;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2557\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->u.op = last_op->result;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2558\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->EA = 0U;
        }
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2565\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2566\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2569\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2583\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2584\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = op;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2585\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2573\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )op1->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2574\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op1->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2570\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & op1->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2571\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2572\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )op1->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2586\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2587\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2588\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2589\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2578\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2579\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2575\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2576\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2577\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2590\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_post_incdec(znode *result , znode const   *op1 , zend_uchar op ) ;
void zend_do_pre_incdec(znode *result , znode const   *op1 , zend_uchar op ) 
{ int last_op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *last_op ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2543\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2544\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  last_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2545\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (last_op_number > 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2530\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op = (compiler_globals.active_op_array)->opcodes + (last_op_number - 1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2531\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )last_op->opcode == 88) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2524\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )op == 34) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2517\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        last_op->opcode = (unsigned char)132;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2518\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        last_op->opcode = (unsigned char)133;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2525\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      last_op->result_type = (unsigned char )(1 << 2);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2526\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      last_op->result.var = get_temporary_variable(compiler_globals.active_op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2527\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2522\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->op_type = (int )last_op->result_type;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2523\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (result->op_type == 1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2519\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->u.constant = ((compiler_globals.active_op_array)->literals + last_op->result.constant)->constant;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2520\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->u.op = last_op->result;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2521\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->EA = 0U;
        }
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2528\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2529\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2532\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2546\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2547\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = op;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2548\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2536\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )op1->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2537\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op1->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2533\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & op1->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2534\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2535\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )op1->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2549\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2550\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2551\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2552\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2541\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2542\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2538\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2539\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2540\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2553\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_pre_incdec(znode *result , znode const   *op1 , zend_uchar op ) ;
void zend_do_for_end(znode const   *second_semicolon_token ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2506\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2507\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2508\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)42;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2509\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1.opline_num = (unsigned int )(second_semicolon_token->u.op.opline_num + 1U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2510\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2511\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + second_semicolon_token->u.op.opline_num)->op2.opline_num = (unsigned int )tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2512\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2513\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2514\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  do_end_loop((int )(second_semicolon_token->u.op.opline_num + 1U), 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2515\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2504\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) --;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2505\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2516\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_for_end(znode const   *second_semicolon_token ) ;
void zend_do_for_before_statement(znode const   *cond_start ,
                                  znode const   *second_semicolon_token ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2493\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2494\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2495\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)42;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2496\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1.opline_num = (unsigned int )cond_start->u.op.opline_num;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2497\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2498\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + second_semicolon_token->u.op.opline_num)->extended_value = (unsigned long )tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2499\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2500\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2501\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  do_begin_loop();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2502\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2491\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) ++;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2492\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2503\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_for_before_statement(znode const   *cond_start ,
                                  znode const   *second_semicolon_token ) ;
void zend_do_for_cond(znode const   *expr , znode *second_semicolon_token ) 
{ int for_cond_op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2482\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2483\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  for_cond_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2484\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2485\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2486\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)45;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2487\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2480\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )expr->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2481\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2477\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & expr->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2478\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2479\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )expr->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2488\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  second_semicolon_token->u.op.opline_num = (unsigned int )for_cond_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2489\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2490\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_for_cond(znode const   *expr , znode *second_semicolon_token ) ;
void zend_do_while_end(znode const   *while_token ,
                       znode const   *close_bracket_token ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2466\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2467\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2468\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)42;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2469\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1.opline_num = (unsigned int )while_token->u.op.opline_num;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2470\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op1_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2471\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2472\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2473\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->opcodes + close_bracket_token->u.op.opline_num)->op2.opline_num = (unsigned int )tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2474\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  do_end_loop((int )while_token->u.op.opline_num, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2475\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2464\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) --;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2465\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2476\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_while_end(znode const   *while_token ,
                       znode const   *close_bracket_token ) ;
void zend_do_while_cond(znode const   *expr , znode *close_bracket_token ) 
{ int while_cond_op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2453\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2454\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while_cond_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2455\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2456\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2457\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)43;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2458\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2449\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )expr->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2450\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (expr->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2446\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & expr->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2447\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2448\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )expr->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2459\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  close_bracket_token->u.op.opline_num = (unsigned int )while_cond_op_number;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2460\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2461\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  do_begin_loop();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2462\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2451\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (compiler_globals.context.backpatch_count) ++;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2452\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2463\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_while_cond(znode const   *expr , znode *close_bracket_token ) ;
__inline static void do_end_loop(int cont_addr , int has_loop_var ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2441\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! has_loop_var) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2439\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->brk_cont_array + compiler_globals.context.current_brk_cont)->start = -1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2440\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2442\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->brk_cont_array + compiler_globals.context.current_brk_cont)->cont = cont_addr;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2443\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((compiler_globals.active_op_array)->brk_cont_array + compiler_globals.context.current_brk_cont)->brk = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2444\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.context.current_brk_cont = ((compiler_globals.active_op_array)->brk_cont_array + compiler_globals.context.current_brk_cont)->parent;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2445\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern zend_brk_cont_element *get_next_brk_cont_element(zend_op_array *op_array ) ;
__inline static void do_begin_loop(void) 
{ zend_brk_cont_element *brk_cont_element ;
  int parent ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2433\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  parent = compiler_globals.context.current_brk_cont;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2434\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.context.current_brk_cont = (compiler_globals.active_op_array)->last_brk_cont;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2435\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  brk_cont_element = get_next_brk_cont_element(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2436\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  brk_cont_element->start = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2437\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  brk_cont_element->parent = parent;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2438\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_assign_ref(znode *result , znode const   *lvar ,
                        znode const   *rvar ) 
{ zend_op *opline ;
  int last_op_number ;
  int tmp ;
  zend_bool tmp___0 ;
  zend_bool tmp___1 ;
  int tmp___2 ;
  int tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2424\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (lvar->op_type == (int const   )(1 << 4)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2389\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (lvar->u.op.var == (zend_uint const   )(compiler_globals.active_op_array)->this_var) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2387\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Cannot re-assign $this");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2388\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2400\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (lvar->op_type == (int const   )(1 << 2)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2396\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = get_next_op_number(compiler_globals.active_op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2397\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      last_op_number = tmp;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2398\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (last_op_number > 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2392\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline = (compiler_globals.active_op_array)->opcodes + (last_op_number - 1);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2393\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = opline_is_fetch_this((zend_op const   *)opline);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2394\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2390\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          zend_error(1 << 6L, "Cannot re-assign $this");
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2391\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2395\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2399\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2425\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2426\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)39;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2427\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_is_function_or_method_call(rvar);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2428\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2401\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->extended_value = 1UL;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2404\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (rvar->EA & (unsigned int const   )(1 << 6)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2402\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->extended_value = (unsigned long )(1 << 1);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2403\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->extended_value = 0UL;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2429\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (result) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2410\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )(1 << 2);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2411\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2412\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2408\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->op_type = (int )opline->result_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2409\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (result->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2405\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2406\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->u.op = opline->result;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2407\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->EA = 0U;
      }
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2413\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )((1 << 3) | (1 << 5));
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2430\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2417\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )lvar->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2418\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (lvar->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2414\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                 & lvar->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2415\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___2;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2416\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )lvar->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2431\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2422\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )rvar->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2423\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (rvar->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2419\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___3 = zend_add_literal(compiler_globals.active_op_array,
                                 & rvar->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2420\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___3;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2421\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2 = (union _znode_op )rvar->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2432\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static void zend_do_op_data(zend_op *data_op , znode const   *value ) ;
void zend_do_assign(znode *result , znode *variable , znode *value ) 
{ int last_op_number ;
  zend_op *opline ;
  zend_llist *fetch_list_ptr ;
  zval _c ;
  char const   *__s ;
  int __l ;
  zval *__z ;
  char __attribute__((__visibility__("default")))  *tmp ;
  int tmp___0 ;
  ulong __attribute__((__visibility__("default")))  tmp___1 ;
  int n ;
  zend_op *last_op ;
  int opline_no ;
  zend_uchar tmp___2 ;
  zend_uchar tmp___3 ;
  int opline_no___0 ;
  zend_uchar tmp___4 ;
  zend_uchar tmp___5 ;
  zend_bool tmp___6 ;
  int tmp___7 ;
  int tmp___8 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2375\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (value->op_type == 1 << 4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2285\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_stack_top((zend_stack const   *)(& compiler_globals.bp_stack),
                   (void **)(& fetch_list_ptr));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2286\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (fetch_list_ptr) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2283\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (fetch_list_ptr->head) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2280\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline = (zend_op *)((fetch_list_ptr->head)->data);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2281\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )opline->opcode == 84) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2278\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )opline->op1_type == 1 << 4) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2276\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (opline->op1.var == value->u.op.var) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2265\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              opline = get_next_op(compiler_globals.active_op_array);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2266\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              opline->opcode = (unsigned char)80;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2267\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              opline->result_type = (unsigned char )(1 << 2);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2268\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2269\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              opline->op1_type = (unsigned char)1;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2270\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              while (1) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2250\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                while (1) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2243\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  __s = ((compiler_globals.active_op_array)->vars + value->u.op.var)->name;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2244\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  __l = ((compiler_globals.active_op_array)->vars + value->u.op.var)->name_len;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2245\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  __z = & _c;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2246\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  __z->value.str.len = __l;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2247\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp = _estrndup(__s, (unsigned int )__l);
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2248\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  __z->value.str.val = (char *)tmp;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2249\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  __z->type = (unsigned char)6;
                  break;
                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2251\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                           (zval const   *)(& _c));
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2252\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                opline->op1.constant = (unsigned int )tmp___0;
                break;
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2271\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              while (1) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2259\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2256\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2253\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val - sizeof(Bucket )))->h;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2254\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    tmp___1 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                                             (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len + 1));
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2255\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = (unsigned long )tmp___1;
                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2257\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___1 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                                           (unsigned int )(((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len + 1));
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2258\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value = (unsigned long )tmp___1;
                }
                break;
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2272\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              opline->op2_type = (unsigned char )(1 << 3);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2273\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              opline->extended_value = 268435456UL;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2274\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              while (1) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2263\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                value->op_type = (int )opline->result_type;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2264\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (value->op_type == 1) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2260\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  value->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2261\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  value->u.op = opline->result;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2262\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  value->EA = 0U;
                }
                break;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2275\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2277\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2279\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2282\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2284\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2287\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2376\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_do_end_variable_parse(variable, 1, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2377\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  last_op_number = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2378\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2379\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (variable->op_type == 1 << 4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2290\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (variable->u.op.var == (compiler_globals.active_op_array)->this_var) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2288\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Cannot re-assign $this");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2289\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2359\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (variable->op_type == 1 << 2) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2356\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      n = 0;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2357\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2352\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (last_op_number - n > 0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2291\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2353\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        last_op = (compiler_globals.active_op_array)->opcodes + ((last_op_number - n) - 1);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2354\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )last_op->result_type == 1 << 2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2350\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (last_op->result.var == variable->u.op.var) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2348\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int )last_op->opcode == 85) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2311\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (n > 0) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2292\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                opline_no = (int )((unsigned int )(opline - (compiler_globals.active_op_array)->opcodes) / sizeof(*opline));
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2293\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                *opline = *last_op;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2294\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                last_op->opcode = (unsigned char)0;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2295\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                memset((void *)(& last_op->result), 0, sizeof(last_op->result));
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2296\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                memset((void *)(& last_op->op1), 0, sizeof(last_op->op1));
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2297\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                memset((void *)(& last_op->op2), 0, sizeof(last_op->op2));
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2298\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___3 = (unsigned char )(1 << 3);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2299\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                last_op->op2_type = tmp___3;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2300\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___2 = tmp___3;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2301\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                last_op->op1_type = tmp___2;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2302\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                last_op->result_type = tmp___2;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2303\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                opline = get_next_op(compiler_globals.active_op_array);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2304\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                last_op = (compiler_globals.active_op_array)->opcodes + opline_no;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2305\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2312\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              last_op->opcode = (unsigned char)136;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2313\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              zend_do_op_data(opline, (znode const   *)value);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2314\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              opline->result_type = (unsigned char )(1 << 3);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2315\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              while (1) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2309\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                result->op_type = (int )last_op->result_type;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2310\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (result->op_type == 1) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2306\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  result->u.constant = ((compiler_globals.active_op_array)->literals + last_op->result.constant)->constant;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2307\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  result->u.op = last_op->result;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2308\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  result->EA = 0U;
                }
                break;
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2316\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              return;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2347\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int )last_op->opcode == 84) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2336\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (n > 0) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2317\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  opline_no___0 = (int )((unsigned int )(opline - (compiler_globals.active_op_array)->opcodes) / sizeof(*opline));
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2318\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  *opline = *last_op;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2319\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  last_op->opcode = (unsigned char)0;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2320\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  memset((void *)(& last_op->result), 0, sizeof(last_op->result));
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2321\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  memset((void *)(& last_op->op1), 0, sizeof(last_op->op1));
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2322\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  memset((void *)(& last_op->op2), 0, sizeof(last_op->op2));
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2323\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___5 = (unsigned char )(1 << 3);
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2324\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  last_op->op2_type = tmp___5;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2325\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  tmp___4 = tmp___5;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2326\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  last_op->op1_type = tmp___4;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2327\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  last_op->result_type = tmp___4;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2328\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  opline = get_next_op(compiler_globals.active_op_array);
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2329\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  last_op = (compiler_globals.active_op_array)->opcodes + opline_no___0;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2330\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2337\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                last_op->opcode = (unsigned char)147;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2338\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                zend_do_op_data(opline, (znode const   *)value);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2339\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                opline->op2.var = get_temporary_variable(compiler_globals.active_op_array);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2340\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                opline->op2_type = (unsigned char )(1 << 2);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2341\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                opline->result_type = (unsigned char )(1 << 3);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2342\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                while (1) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2334\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  result->op_type = (int )last_op->result_type;
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2335\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (result->op_type == 1) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2331\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    result->u.constant = ((compiler_globals.active_op_array)->literals + last_op->result.constant)->constant;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2332\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    result->u.op = last_op->result;
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2333\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    result->EA = 0U;
                  }
                  break;
                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2343\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                return;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2345\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___6 = opline_is_fetch_this((zend_op const   *)last_op);
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2346\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (tmp___6) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2344\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  zend_error(1 << 6L, "Cannot re-assign $this");
                } else {
                  break;
                }
              }
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2349\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2351\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2355\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        n ++;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2358\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2380\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)38;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2381\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2363\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )variable->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2364\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (variable->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2360\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___7 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& variable->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2361\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___7;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2362\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = variable->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2382\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2368\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )value->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2369\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (value->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2365\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___8 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& value->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2366\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___8;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2367\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2 = value->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2383\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2384\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2385\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2373\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2374\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2370\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2371\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2372\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2386\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static zend_bool opline_is_fetch_this(zend_op const   *opline ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2242\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )opline->opcode == 83) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2240\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )opline->op1_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2238\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.type == 6) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2236\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant))->hash_value == 275574653UL) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2234\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.len == sizeof("this") - 1U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2231\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp = memcmp((void const   *)((compiler_globals.active_op_array)->literals + opline->op1.constant)->constant.value.str.val,
                         (void const   *)"this", sizeof("this"));
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2232\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (tmp) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2229\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              return ((unsigned char)0);
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2230\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              return ((unsigned char)1);
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2233\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return ((unsigned char)0);
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2235\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return ((unsigned char)0);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2237\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((unsigned char)0);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2239\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((unsigned char)0);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2241\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)0);
  }
}
}
void zend_do_abstract_method(znode const   *function_name , znode *modifiers ,
                             znode const   *body ) 
{ char *method_type ;
  zend_op *opline ;
  zend_op *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2226\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_class_entry)->ce_flags & 128U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2210\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    modifiers->u.constant.value.lval |= 2L;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2211\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    method_type = (char *)"Interface";
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2212\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    method_type = (char *)"Abstract";
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2227\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (modifiers->u.constant.value.lval & 2L) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2221\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (modifiers->u.constant.value.lval & 1024L) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2213\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "%s function %s::%s() cannot be declared private",
                 method_type, (compiler_globals.active_class_entry)->name,
                 function_name->u.constant.value.str.val);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2214\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2222\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (body->u.constant.value.lval == 2L) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2215\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = get_next_op(compiler_globals.active_op_array);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2216\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline = tmp;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2217\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->opcode = (unsigned char)142;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2218\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1_type = (unsigned char )(1 << 3);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2219\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2_type = (unsigned char )(1 << 3);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2220\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "%s function %s::%s() cannot contain body",
                 method_type, (compiler_globals.active_class_entry)->name,
                 function_name->u.constant.value.str.val);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2225\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (body->u.constant.value.lval == 2L) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2223\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 6L, "Non-abstract method %s::%s() must contain body",
                 (compiler_globals.active_class_entry)->name,
                 function_name->u.constant.value.str.val);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2224\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2228\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_abstract_method(znode const   *function_name , znode *modifiers ,
                             znode const   *body ) ;
void zend_do_echo(znode const   *arg ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2204\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2205\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2206\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)40;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2207\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2202\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )arg->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2203\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (arg->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2199\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & arg->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2200\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2201\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )arg->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2208\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2209\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_echo(znode const   *arg ) ;
void zend_do_print(znode *result , znode const   *arg ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2190\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2191\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2192\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2193\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2194\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = (unsigned char)41;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2195\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2183\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )arg->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2184\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (arg->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2180\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & arg->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2181\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2182\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )arg->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2196\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2197\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2188\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2189\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2185\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2186\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2187\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2198\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_print(znode *result , znode const   *arg ) ;
void fetch_array_dim(znode *result , znode const   *parent , znode const   *dim ) ;
void fetch_string_offset(znode *result , znode const   *parent ,
                         znode const   *offset ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2178\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fetch_array_dim(result, parent, offset);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2179\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void fetch_string_offset(znode *result , znode const   *parent ,
                         znode const   *offset ) ;
void fetch_array_dim(znode *result , znode const   *parent , znode const   *dim ) 
{ zend_op opline ;
  zend_llist *fetch_list_ptr ;
  int tmp ;
  zend_bool tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;
  ulong index___0 ;
  int numeric ;
  register char const   *tmp___3 ;
  char const   *end ;
  zval *__z ;
  ulong __attribute__((__visibility__("default")))  tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2165\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.bp_stack),
                 (void **)(& fetch_list_ptr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2166\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = zend_is_function_or_method_call(parent);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2167\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2090\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    init_op(& opline);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2091\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.opcode = (unsigned char)156;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2092\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2088\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline.op1_type = (unsigned char )parent->op_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2089\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (parent->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2085\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = zend_add_literal(compiler_globals.active_op_array,
                               & parent->u.constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2086\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline.op1.constant = (unsigned int )tmp;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2087\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline.op1 = (union _znode_op )parent->u.op;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2093\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.op2_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2094\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.result_type = (unsigned char )(1 << 2);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2095\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.result.var = opline.op1.var;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2096\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_llist_add_element(fetch_list_ptr, (void *)(& opline));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2097\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2168\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  init_op(& opline);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2169\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline.opcode = (unsigned char)84;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2170\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline.result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2171\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline.result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2172\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2101\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.op1_type = (unsigned char )parent->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2102\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (parent->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2098\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & parent->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2099\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline.op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2100\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline.op1 = (union _znode_op )parent->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2173\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2106\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.op2_type = (unsigned char )dim->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2107\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (dim->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2103\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                 & dim->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2104\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline.op2.constant = (unsigned int )tmp___2;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2105\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline.op2 = (union _znode_op )dim->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2174\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )opline.op2_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2158\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.type == 6) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2154\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      numeric = 0;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2155\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2139\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = (char const   *)((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.val;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2140\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*tmp___3 == 45) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2108\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___3 ++;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2109\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2141\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*tmp___3 >= 48) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2137\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*tmp___3 <= 57) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2131\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            end = (char const   *)(((((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.val + ((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.len) + 1) - 1);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2132\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*end != 0) {
              break;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2116\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*tmp___3 == 48) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2110\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.len + 1 > 2) {
                  break;
                } else {
                  goto _L;
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2115\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                _L: /* CIL Label */ 
                if (end - tmp___3 > 10) {
                  break;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2114\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if (end - tmp___3 == 10) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2112\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    if ((int const   )*tmp___3 > 50) {
                      break;
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "2111\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }

                    }
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2113\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                }
              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2133\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            index___0 = (unsigned long )((int const   )*tmp___3 - 48);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2134\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            while (1) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2120\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___3 ++;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2121\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((unsigned int )tmp___3 != (unsigned int )end) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2119\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp___3 >= 48) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2118\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*tmp___3 <= 57) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "2117\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  } else {
                    break;
                  }
                } else {
                  break;
                }
              } else {
                break;
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2122\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              index___0 = index___0 * 10UL + (ulong )((int const   )*tmp___3 - 48);
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2135\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )tmp___3 == (unsigned int )end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2128\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int )*(((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.val) == 45) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2124\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (index___0 - 1UL > 2147483647UL) {
                  break;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2123\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2125\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                index___0 = (unsigned long )(- ((long )index___0));
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "2127\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (index___0 > 2147483647UL) {
                  break;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "2126\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2129\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              numeric = 1;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2130\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2136\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2138\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2156\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (numeric) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2142\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _zval_dtor(& ((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2143\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z = & ((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2144\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z->value.lval = (long )index___0;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2145\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z->type = (unsigned char)1;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2153\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2152\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2149\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2146\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.val - sizeof(Bucket )))->h;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2147\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___4 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.val,
                                       (unsigned int )(((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.len + 1));
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2148\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant))->hash_value = (unsigned long )tmp___4;
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2150\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___4 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.val,
                                     (unsigned int )(((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant.value.str.len + 1));
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2151\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline.op2.constant)->constant))->hash_value = (unsigned long )tmp___4;
          }
          break;
        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2157\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2159\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2175\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2163\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline.result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2164\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2160\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline.result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2161\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline.result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2162\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2176\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_add_element(fetch_list_ptr, (void *)(& opline));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2177\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void fetch_array_begin(znode *result , znode *varname , znode *first_dim ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2082\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fetch_simple_variable(result, varname, 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2083\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fetch_array_dim(result, (znode const   *)result, (znode const   *)first_dim);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2084\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void fetch_array_begin(znode *result , znode *varname , znode *first_dim ) ;
extern char __attribute__((__visibility__("default")))  *_estrdup(char const   *s )  __attribute__((__malloc__)) ;
void zend_do_fetch_static_member(znode *result , znode *class_name ) 
{ znode class_node ;
  zend_llist *fetch_list_ptr ;
  zend_llist_element *le ;
  zend_op *opline_ptr ;
  zend_op opline ;
  int tmp ;
  zval _c ;
  char const   *__s ;
  char __attribute__((__visibility__("default")))  *tmp___0 ;
  int __l ;
  zval *__z ;
  char __attribute__((__visibility__("default")))  *tmp___1 ;
  int tmp___2 ;
  ulong __attribute__((__visibility__("default")))  tmp___3 ;
  void __attribute__((__visibility__("default")))  *tmp___4 ;
  int tmp___5 ;
  int tmp___6 ;
  zval _c___0 ;
  char const   *__s___0 ;
  char __attribute__((__visibility__("default")))  *tmp___7 ;
  int __l___0 ;
  zval *__z___0 ;
  char __attribute__((__visibility__("default")))  *tmp___8 ;
  int tmp___9 ;
  ulong __attribute__((__visibility__("default")))  tmp___10 ;
  void __attribute__((__visibility__("default")))  *tmp___11 ;
  int tmp___12 ;
  int tmp___13 ;
  void __attribute__((__visibility__("default")))  *tmp___14 ;
  int tmp___15 ;
  int tmp___16 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2078\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (class_name->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1943\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_get_class_fetch_type((char const   *)class_name->u.constant.value.str.val,
                                    (unsigned int )class_name->u.constant.value.str.len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1944\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (0 == tmp) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1940\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_resolve_class_name(class_name, 4UL, 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1941\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      class_node = *class_name;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1942\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_do_fetch_class(& class_node, class_name);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1945\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_fetch_class(& class_node, class_name);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2079\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_top((zend_stack const   *)(& compiler_globals.bp_stack),
                 (void **)(& fetch_list_ptr));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2080\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (result->op_type == 1 << 4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1987\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    init_op(& opline);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1988\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.opcode = (unsigned char)83;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1989\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.result_type = (unsigned char )(1 << 2);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1990\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.result.var = get_temporary_variable(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1991\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.op1_type = (unsigned char)1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1992\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1953\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1946\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = _estrdup(((compiler_globals.active_op_array)->vars + result->u.op.var)->name);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1947\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __s = (char const   *)tmp___0;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1948\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __l = ((compiler_globals.active_op_array)->vars + result->u.op.var)->name_len;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1949\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z = & _c;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1950\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z->value.str.len = __l;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1951\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1952\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __z->type = (unsigned char)6;
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1954\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& _c));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1955\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline.op1.constant = (unsigned int )tmp___2;
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1993\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1962\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1959\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1956\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.val - sizeof(Bucket )))->h;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1957\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___3 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.val,
                                   (unsigned int )(((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.len + 1));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1958\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant))->hash_value = (unsigned long )tmp___3;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1960\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.val,
                                 (unsigned int )(((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1961\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant))->hash_value = (unsigned long )tmp___3;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1994\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1970\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((compiler_globals.active_op_array)->literals + opline.op1.constant)->cache_slot = (unsigned int )(compiler_globals.active_op_array)->last_cache_slot;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1971\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (compiler_globals.active_op_array)->last_cache_slot += 2;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1972\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((compiler_globals.active_op_array)->fn_flags & 16U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1968\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((compiler_globals.active_op_array)->run_time_cache) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1963\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___4 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                              (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                              0);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1964\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___4;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1965\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1966\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 2)) = (void *)0;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1967\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1969\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1995\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (class_node.op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1973\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline.op2_type = (unsigned char)1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1974\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___5 = zend_add_class_name_literal(compiler_globals.active_op_array,
                                            (zval const   *)(& class_node.u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1975\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline.op2.constant = (unsigned int )tmp___5;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1981\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1979\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline.op2_type = (unsigned char )class_node.op_type;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1980\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (class_node.op_type == 1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1976\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___6 = zend_add_literal(compiler_globals.active_op_array,
                                     (zval const   *)(& class_node.u.constant));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1977\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline.op2.constant = (unsigned int )tmp___6;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1978\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline.op2 = class_node.u.op;
        }
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1996\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1985\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->op_type = (int )opline.result_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1986\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (result->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1982\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->u.constant = ((compiler_globals.active_op_array)->literals + opline.result.constant)->constant;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1983\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->u.op = opline.result;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1984\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->EA = 0U;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1997\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline.extended_value |= 805306368UL;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1998\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline_ptr = & opline;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1999\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_llist_add_element(fetch_list_ptr, (void *)(& opline));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2075\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    le = fetch_list_ptr->head;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2076\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline_ptr = (zend_op *)(le->data);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "2077\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )opline_ptr->opcode != 83) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2050\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )opline_ptr->op1_type == 1 << 4) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2038\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        init_op(& opline);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2039\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline.opcode = (unsigned char)83;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2040\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline.result_type = (unsigned char )(1 << 2);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2041\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline.result.var = get_temporary_variable(compiler_globals.active_op_array);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2042\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline.op1_type = (unsigned char)1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2043\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2007\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2000\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___7 = _estrdup(((compiler_globals.active_op_array)->vars + opline_ptr->op1.var)->name);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2001\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            __s___0 = (char const   *)tmp___7;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2002\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            __l___0 = ((compiler_globals.active_op_array)->vars + opline_ptr->op1.var)->name_len;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2003\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            __z___0 = & _c___0;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2004\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            __z___0->value.str.len = __l___0;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2005\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            __z___0->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s___0));
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2006\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            __z___0->type = (unsigned char)6;
            break;
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2008\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___9 = zend_add_literal(compiler_globals.active_op_array,
                                     (zval const   *)(& _c___0));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2009\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline.op1.constant = (unsigned int )tmp___9;
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2044\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2016\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2013\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2010\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.val - sizeof(Bucket )))->h;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2011\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___10 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.val,
                                        (unsigned int )(((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.len + 1));
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2012\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant))->hash_value = (unsigned long )tmp___10;
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2014\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___10 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.val,
                                      (unsigned int )(((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant.value.str.len + 1));
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2015\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline.op1.constant)->constant))->hash_value = (unsigned long )tmp___10;
          }
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2045\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2024\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((compiler_globals.active_op_array)->literals + opline.op1.constant)->cache_slot = (unsigned int )(compiler_globals.active_op_array)->last_cache_slot;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2025\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          (compiler_globals.active_op_array)->last_cache_slot += 2;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2026\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((compiler_globals.active_op_array)->fn_flags & 16U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2022\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((compiler_globals.active_op_array)->run_time_cache) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2017\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___11 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                                   (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                                   0);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2018\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___11;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2019\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2020\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 2)) = (void *)0;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2021\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2023\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2046\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (class_node.op_type == 1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2027\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline.op2_type = (unsigned char)1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2028\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___12 = zend_add_class_name_literal(compiler_globals.active_op_array,
                                                 (zval const   *)(& class_node.u.constant));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2029\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline.op2.constant = (unsigned int )tmp___12;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2035\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2033\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            opline.op2_type = (unsigned char )class_node.op_type;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2034\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (class_node.op_type == 1) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2030\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___13 = zend_add_literal(compiler_globals.active_op_array,
                                          (zval const   *)(& class_node.u.constant));
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2031\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              opline.op2.constant = (unsigned int )tmp___13;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2032\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              opline.op2 = class_node.u.op;
            }
            break;
          }
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2047\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline.extended_value |= 805306368UL;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2048\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2036\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline_ptr->op1_type = opline.result_type;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2037\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline_ptr->op1 = opline.result;
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2049\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_llist_prepend_element(fetch_list_ptr, (void *)(& opline));
      } else {
        goto _L;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2072\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L: /* CIL Label */ 
      if ((int )opline_ptr->op1_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2061\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2058\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->cache_slot = (unsigned int )(compiler_globals.active_op_array)->last_cache_slot;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2059\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          (compiler_globals.active_op_array)->last_cache_slot += 2;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2060\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((compiler_globals.active_op_array)->fn_flags & 16U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2056\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((compiler_globals.active_op_array)->run_time_cache) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2051\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___14 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                                   (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                                   0);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2052\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___14;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2053\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2054\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 2)) = (void *)0;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "2055\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2057\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          break;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2062\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2073\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (class_node.op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2063\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline_ptr->op2_type = (unsigned char)1;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2064\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___15 = zend_add_class_name_literal(compiler_globals.active_op_array,
                                               (zval const   *)(& class_node.u.constant));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2065\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        opline_ptr->op2.constant = (unsigned int )tmp___15;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "2071\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        while (1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2069\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          opline_ptr->op2_type = (unsigned char )class_node.op_type;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "2070\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (class_node.op_type == 1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2066\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___16 = zend_add_literal(compiler_globals.active_op_array,
                                        (zval const   *)(& class_node.u.constant));
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2067\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            opline_ptr->op2.constant = (unsigned int )tmp___16;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "2068\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            opline_ptr->op2 = class_node.u.op;
          }
          break;
        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "2074\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline_ptr->extended_value |= 805306368UL;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2081\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_fetch_static_member(znode *result , znode *class_name ) ;
void fetch_simple_variable(znode *result , znode *varname , int bp ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1938\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fetch_simple_variable_ex(result, varname, bp, (unsigned char)83);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1939\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void fetch_simple_variable_ex(znode *result , znode *varname , int bp ,
                              zend_uchar op ) 
{ zend_op opline ;
  zend_op *opline_ptr ;
  zend_llist *fetch_list_ptr ;
  ulong hash ;
  int tmp ;
  zend_bool __attribute__((__visibility__("default")))  tmp___0 ;
  int tmp___1 ;
  int tmp___2 ;
  ulong __attribute__((__visibility__("default")))  tmp___3 ;
  zend_bool __attribute__((__visibility__("default")))  tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1926\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (varname->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1892\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = (ulong )0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1893\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )varname->u.constant.type != 6) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1872\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )varname->u.constant.type != 6) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1870\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _convert_to_string(& varname->u.constant);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1871\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1877\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )varname->u.constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1875\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )varname->u.constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1873\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          hash = ((Bucket *)(varname->u.constant.value.str.val - sizeof(Bucket )))->h;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1874\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1876\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1894\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = zend_is_auto_global_quick((char const   *)varname->u.constant.value.str.val,
                                        (unsigned int )varname->u.constant.value.str.len,
                                        hash);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1895\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1878\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1891\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )varname->u.constant.value.str.len == sizeof("this") - 1U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1880\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = memcmp((void const   *)varname->u.constant.value.str.val,
                         (void const   *)"this", sizeof("this"));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1881\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          goto _L___0;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1879\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1890\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _L___0: /* CIL Label */ 
        if ((compiler_globals.active_op_array)->last == 0U) {
          goto _L;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1889\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )((compiler_globals.active_op_array)->opcodes + ((compiler_globals.active_op_array)->last - 1U))->opcode != 57) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1882\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L: /* CIL Label */ 
            result->op_type = 1 << 4;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1883\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp = lookup_cv(compiler_globals.active_op_array,
                            varname->u.constant.value.str.val,
                            varname->u.constant.value.str.len, hash);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1884\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            result->u.op.var = (unsigned int )tmp;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1885\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            varname->u.constant.value.str.val = (char *)((compiler_globals.active_op_array)->vars + result->u.op.var)->name;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1886\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            result->EA = 0U;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1887\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1888\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        }
      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1896\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1927\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (bp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1897\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline_ptr = & opline;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1898\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    init_op(opline_ptr);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1899\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline_ptr = get_next_op(compiler_globals.active_op_array);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1928\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline_ptr->opcode = op;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1929\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline_ptr->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1930\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline_ptr->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1931\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1903\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline_ptr->op1_type = (unsigned char )varname->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1904\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (varname->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1900\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& varname->u.constant));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1901\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline_ptr->op1.constant = (unsigned int )tmp___2;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1902\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline_ptr->op1 = varname->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1932\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1908\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline_ptr->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1909\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1905\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline_ptr->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1906\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline_ptr->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1907\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1933\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline_ptr->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1934\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline_ptr->extended_value = 268435456UL;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1935\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (varname->op_type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1919\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1916\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1913\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1910\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant.value.str.val - sizeof(Bucket )))->h;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1911\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___3 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant.value.str.val,
                                   (unsigned int )(((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant.value.str.len + 1));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1912\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant))->hash_value = (unsigned long )tmp___3;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1914\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___3 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant.value.str.val,
                                 (unsigned int )(((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1915\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant))->hash_value = (unsigned long )tmp___3;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1920\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___4 = zend_is_auto_global_quick((char const   *)varname->u.constant.value.str.val,
                                        (unsigned int )varname->u.constant.value.str.len,
                                        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + opline_ptr->op1.constant)->constant))->hash_value);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1921\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___4) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1917\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline_ptr->extended_value = 0UL;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1918\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1922\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1936\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (bp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1923\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_stack_top((zend_stack const   *)(& compiler_globals.bp_stack),
                   (void **)(& fetch_list_ptr));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1924\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_llist_add_element(fetch_list_ptr, (void *)opline_ptr);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1925\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1937\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_binary_assign_op(zend_uchar op , znode *result ,
                              znode const   *op1 , znode const   *op2 ) 
{ int last_op_number ;
  int tmp ;
  zend_op *opline ;
  zend_op *tmp___0 ;
  zend_op *last_op ;
  int tmp___1 ;
  int tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1858\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op_number(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1859\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  last_op_number = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1860\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1861\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1862\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (last_op_number > 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1841\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op = (compiler_globals.active_op_array)->opcodes + (last_op_number - 1);
    switch ((int )last_op->opcode) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1826\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 88: 
    last_op->opcode = op;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1827\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op->extended_value = 136UL;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1828\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_op_data(opline, op2);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1829\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1830\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1819\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->op_type = (int )last_op->result_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1820\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (result->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1816\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->u.constant = ((compiler_globals.active_op_array)->literals + last_op->result.constant)->constant;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1817\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->u.op = last_op->result;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1818\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->EA = 0U;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1831\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1832\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    case 87: 
    last_op->opcode = op;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1833\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    last_op->extended_value = 147UL;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1834\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_do_op_data(opline, op2);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1835\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2.var = get_temporary_variable(compiler_globals.active_op_array);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1836\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )(1 << 2);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1837\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->result_type = (unsigned char )(1 << 3);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1838\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1824\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->op_type = (int )last_op->result_type;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1825\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (result->op_type == 1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1821\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->u.constant = ((compiler_globals.active_op_array)->literals + last_op->result.constant)->constant;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1822\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->u.op = last_op->result;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1823\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->EA = 0U;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1839\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1840\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    default: ;
    break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1842\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1863\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = op;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1864\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1846\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )op1->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1847\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op1->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1843\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & op1->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1844\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1845\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )op1->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1865\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1851\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )op2->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1852\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op2->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1848\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_add_literal(compiler_globals.active_op_array,
                                 & op2->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1849\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___2;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1850\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2 = (union _znode_op )op2->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1866\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1867\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1868\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1856\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1857\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1853\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1854\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1855\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1869\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_binary_assign_op(zend_uchar op , znode *result ,
                              znode const   *op1 , znode const   *op2 ) ;
static void zend_do_op_data(zend_op *data_op , znode const   *value ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1812\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  data_op->opcode = (unsigned char)137;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1813\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1810\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    data_op->op1_type = (unsigned char )value->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1811\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (value->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1807\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = zend_add_literal(compiler_globals.active_op_array,
                             & value->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1808\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      data_op->op1.constant = (unsigned int )tmp;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1809\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      data_op->op1 = (union _znode_op )value->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1814\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  data_op->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1815\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_unary_op(zend_uchar op , znode *result , znode const   *op1 ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1798\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1799\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1800\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = op;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1801\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1802\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1803\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1791\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )op1->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1792\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op1->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1788\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & op1->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1789\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1790\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )op1->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1804\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1796\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1797\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1793\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1794\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1795\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1805\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->op2_type = (unsigned char )(1 << 3);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1806\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_unary_op(zend_uchar op , znode *result , znode const   *op1 ) ;
void zend_do_binary_op(zend_uchar op , znode *result , znode const   *op1 ,
                       znode const   *op2 ) 
{ zend_op *opline ;
  zend_op *tmp ;
  int tmp___0 ;
  int tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1779\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = get_next_op(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1780\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1781\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->opcode = op;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1782\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result_type = (unsigned char )(1 << 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1783\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  opline->result.var = get_temporary_variable(compiler_globals.active_op_array);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1784\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1767\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op1_type = (unsigned char )op1->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1768\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op1->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1764\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = zend_add_literal(compiler_globals.active_op_array,
                                 & op1->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1765\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1.constant = (unsigned int )tmp___0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1766\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op1 = (union _znode_op )op1->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1785\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1772\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    opline->op2_type = (unsigned char )op2->op_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1773\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op2->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1769\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_add_literal(compiler_globals.active_op_array,
                                 & op2->u.constant);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1770\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2.constant = (unsigned int )tmp___1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1771\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      opline->op2 = (union _znode_op )op2->u.op;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1786\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1777\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result->op_type = (int )opline->result_type;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1778\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (result->op_type == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1774\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.constant = ((compiler_globals.active_op_array)->literals + opline->result.constant)->constant;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1775\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->u.op = opline->result;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1776\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->EA = 0U;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1787\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_do_binary_op(zend_uchar op , znode *result , znode const   *op1 ,
                       znode const   *op2 ) ;
__inline static zend_bool zend_is_function_or_method_call(znode const   *variable ) 
{ zend_uint type ;
  int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1761\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  type = (zend_uint )variable->EA;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1762\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (type & (unsigned int )(1 << 1)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1757\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = 1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1760\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (type == (zend_uint )(1 << 3)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1758\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = 1;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1759\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = 0;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1763\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned char )tmp);
}
}
extern void __attribute__((__visibility__("default")))  zend_str_tolower(char *str ,
                                                                         unsigned int length ) ;
int zend_add_const_name_literal(zend_op_array *op_array , zval const   *zv ,
                                int unqualified ) 
{ int ret ;
  int tmp_literal ;
  char *name ;
  char *tmp_name ;
  char const   *ns_separator ;
  int name_len ;
  int ns_len ;
  zval c ;
  void const   *tmp ;
  char __attribute__((__visibility__("default")))  *tmp___0 ;
  char const   *__s ;
  int __l ;
  zval *__z ;
  char __attribute__((__visibility__("default")))  *tmp___1 ;
  ulong __attribute__((__visibility__("default")))  tmp___2 ;
  char __attribute__((__visibility__("default")))  *tmp___3 ;
  char const   *__s___0 ;
  int __l___0 ;
  zval *__z___0 ;
  char __attribute__((__visibility__("default")))  *tmp___4 ;
  ulong __attribute__((__visibility__("default")))  tmp___5 ;
  char __attribute__((__visibility__("default")))  *tmp___6 ;
  char const   *__s___1 ;
  int __l___1 ;
  zval *__z___1 ;
  char __attribute__((__visibility__("default")))  *tmp___7 ;
  ulong __attribute__((__visibility__("default")))  tmp___8 ;
  char __attribute__((__visibility__("default")))  *tmp___9 ;
  char const   *__s___2 ;
  int __l___2 ;
  zval *__z___2 ;
  char __attribute__((__visibility__("default")))  *tmp___10 ;
  ulong __attribute__((__visibility__("default")))  tmp___11 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1739\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (op_array->last_literal > 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1660\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )(& (op_array->literals + (op_array->last_literal - 1))->constant) == (unsigned int )zv) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1658\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((op_array->literals + (op_array->last_literal - 1))->cache_slot == 4294967295U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1656\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ret = op_array->last_literal - 1;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1657\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ret = zend_add_literal(op_array, zv);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1659\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ret = zend_add_literal(op_array, zv);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1661\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ret = zend_add_literal(op_array, zv);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1740\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*(zv->value.str.val + 0) == 92) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1662\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    name_len = (int )(zv->value.str.len - 1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1663\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    name = (char *)(zv->value.str.val + 1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1664\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    name_len = (int )zv->value.str.len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1665\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    name = (char *)zv->value.str.val;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1741\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_memrchr((void const   *)name, '\\', (unsigned int )name_len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1742\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ns_separator = (char const   *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1743\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ns_separator) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1666\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ns_len = ns_separator - (char const   *)name;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1667\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ns_len = 0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1744\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ns_len) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1694\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = _estrndup((char const   *)name, (unsigned int )name_len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1695\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp_name = (char *)tmp___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1696\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    zend_str_tolower(tmp_name, (unsigned int )ns_len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1697\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1668\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __s = (char const   *)tmp_name;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1669\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __l = name_len;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1670\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z = & c;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1671\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->value.str.len = __l;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1672\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1673\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z->type = (unsigned char)6;
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1698\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp_literal = zend_add_literal(compiler_globals.active_op_array,
                                   (zval const   *)(& c));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1699\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1680\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1677\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1674\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + tmp_literal)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val - sizeof(Bucket )))->h;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1675\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___2 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val,
                                   (unsigned int )(((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.len + 1));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1676\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + tmp_literal)->constant))->hash_value = (unsigned long )tmp___2;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1678\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___2 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val,
                                 (unsigned int )(((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1679\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + tmp_literal)->constant))->hash_value = (unsigned long )tmp___2;
      }
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1700\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = zend_str_tolower_dup((char const   *)name, (unsigned int )name_len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1701\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp_name = (char *)tmp___3;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1702\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1681\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __s___0 = (char const   *)tmp_name;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1682\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __l___0 = name_len;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1683\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z___0 = & c;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1684\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z___0->value.str.len = __l___0;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1685\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z___0->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s___0));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1686\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __z___0->type = (unsigned char)6;
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1703\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp_literal = zend_add_literal(compiler_globals.active_op_array,
                                   (zval const   *)(& c));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1704\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1693\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1690\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((unsigned int )((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1687\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + tmp_literal)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val - sizeof(Bucket )))->h;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1688\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___5 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val,
                                   (unsigned int )(((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.len + 1));
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1689\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + tmp_literal)->constant))->hash_value = (unsigned long )tmp___5;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1691\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___5 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val,
                                 (unsigned int )(((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1692\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + tmp_literal)->constant))->hash_value = (unsigned long )tmp___5;
      }
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1705\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1745\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (ns_len) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1708\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! unqualified) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1706\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (ret);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1707\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1709\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ns_len ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1710\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    name += ns_len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1711\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    name_len -= ns_len;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1712\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1746\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___6 = _estrndup((char const   *)name, (unsigned int )name_len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1747\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp_name = (char *)tmp___6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1748\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1713\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s___1 = (char const   *)tmp_name;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1714\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __l___1 = name_len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1715\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z___1 = & c;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1716\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z___1->value.str.len = __l___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1717\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z___1->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s___1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1718\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z___1->type = (unsigned char)6;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1749\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp_literal = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& c));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1750\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1725\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1722\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1719\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + tmp_literal)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val - sizeof(Bucket )))->h;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1720\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___8 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val,
                                 (unsigned int )(((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1721\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + tmp_literal)->constant))->hash_value = (unsigned long )tmp___8;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1723\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___8 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val,
                               (unsigned int )(((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.len + 1));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1724\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + tmp_literal)->constant))->hash_value = (unsigned long )tmp___8;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1751\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___9 = zend_str_tolower_dup((char const   *)name, (unsigned int )name_len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1752\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp_name = (char *)tmp___9;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1753\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1726\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s___2 = (char const   *)tmp_name;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1727\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __l___2 = name_len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1728\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z___2 = & c;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1729\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z___2->value.str.len = __l___2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1730\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z___2->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s___2));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1731\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z___2->type = (unsigned char)6;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1754\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp_literal = zend_add_literal(compiler_globals.active_op_array,
                                 (zval const   *)(& c));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1755\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1738\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1735\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1732\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + tmp_literal)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val - sizeof(Bucket )))->h;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1733\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___11 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val,
                                  (unsigned int )(((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1734\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + tmp_literal)->constant))->hash_value = (unsigned long )tmp___11;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1736\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___11 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.val,
                                (unsigned int )(((compiler_globals.active_op_array)->literals + tmp_literal)->constant.value.str.len + 1));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1737\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + tmp_literal)->constant))->hash_value = (unsigned long )tmp___11;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1756\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (ret);
}
}
int zend_add_class_name_literal(zend_op_array *op_array , zval const   *zv ) 
{ int ret ;
  char *lc_name ;
  int lc_len ;
  zval c ;
  int lc_literal ;
  char __attribute__((__visibility__("default")))  *tmp ;
  char __attribute__((__visibility__("default")))  *tmp___0 ;
  char const   *__s ;
  int __l ;
  zval *__z ;
  char __attribute__((__visibility__("default")))  *tmp___1 ;
  ulong __attribute__((__visibility__("default")))  tmp___2 ;
  int tmp___3 ;
  void __attribute__((__visibility__("default")))  *tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1649\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (op_array->last_literal > 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1618\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )(& (op_array->literals + (op_array->last_literal - 1))->constant) == (unsigned int )zv) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1616\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((op_array->literals + (op_array->last_literal - 1))->cache_slot == 4294967295U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1614\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ret = op_array->last_literal - 1;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1615\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ret = zend_add_literal(op_array, zv);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1617\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ret = zend_add_literal(op_array, zv);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1619\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ret = zend_add_literal(op_array, zv);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1650\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*(zv->value.str.val + 0) == 92) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1620\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    lc_len = (int )(zv->value.str.len - 1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1621\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_str_tolower_dup((char const   *)(zv->value.str.val + 1),
                               (unsigned int )lc_len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1622\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    lc_name = (char *)tmp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1623\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    lc_len = (int )zv->value.str.len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1624\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = zend_str_tolower_dup((char const   *)zv->value.str.val,
                                   (unsigned int )lc_len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1625\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    lc_name = (char *)tmp___0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1651\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1626\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s = (char const   *)lc_name;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1627\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __l = lc_len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1628\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z = & c;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1629\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->value.str.len = __l;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1630\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1631\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->type = (unsigned char)6;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1652\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lc_literal = zend_add_literal(compiler_globals.active_op_array,
                                (zval const   *)(& c));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1653\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1638\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1635\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1632\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + lc_literal)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val - sizeof(Bucket )))->h;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1633\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___2 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val,
                                 (unsigned int )(((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1634\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + lc_literal)->constant))->hash_value = (unsigned long )tmp___2;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1636\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val,
                               (unsigned int )(((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.len + 1));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1637\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + lc_literal)->constant))->hash_value = (unsigned long )tmp___2;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1654\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1645\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = (compiler_globals.active_op_array)->last_cache_slot;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1646\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->last_cache_slot) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1647\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ((compiler_globals.active_op_array)->literals + ret)->cache_slot = (unsigned int )tmp___3;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1648\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((compiler_globals.active_op_array)->fn_flags & 16U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1643\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((compiler_globals.active_op_array)->run_time_cache) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1639\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___4 = _erealloc((void *)(compiler_globals.active_op_array)->run_time_cache,
                            (unsigned int )(compiler_globals.active_op_array)->last_cache_slot * sizeof(void *),
                            0);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1640\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (compiler_globals.active_op_array)->run_time_cache = (void **)tmp___4;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1641\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *((compiler_globals.active_op_array)->run_time_cache + ((compiler_globals.active_op_array)->last_cache_slot - 1)) = (void *)0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1642\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1644\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1655\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (ret);
}
}
int zend_add_ns_func_name_literal(zend_op_array *op_array , zval const   *zv ) 
{ int ret ;
  char *lc_name ;
  char const   *ns_separator ;
  int lc_len ;
  zval c ;
  int lc_literal ;
  char __attribute__((__visibility__("default")))  *tmp ;
  char const   *__s ;
  int __l ;
  zval *__z ;
  char __attribute__((__visibility__("default")))  *tmp___0 ;
  ulong __attribute__((__visibility__("default")))  tmp___1 ;
  void const   *tmp___2 ;
  char __attribute__((__visibility__("default")))  *tmp___3 ;
  char const   *__s___0 ;
  int __l___0 ;
  zval *__z___0 ;
  char __attribute__((__visibility__("default")))  *tmp___4 ;
  ulong __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1599\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (op_array->last_literal > 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1571\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )(& (op_array->literals + (op_array->last_literal - 1))->constant) == (unsigned int )zv) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1569\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((op_array->literals + (op_array->last_literal - 1))->cache_slot == 4294967295U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1567\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ret = op_array->last_literal - 1;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1568\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ret = zend_add_literal(op_array, zv);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1570\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ret = zend_add_literal(op_array, zv);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1572\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ret = zend_add_literal(op_array, zv);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1600\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_str_tolower_dup((char const   *)zv->value.str.val,
                             (unsigned int )zv->value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1601\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lc_name = (char *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1602\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1573\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s = (char const   *)lc_name;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1574\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __l = (int )zv->value.str.len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1575\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z = & c;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1576\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->value.str.len = __l;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1577\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1578\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->type = (unsigned char)6;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1603\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lc_literal = zend_add_literal(compiler_globals.active_op_array,
                                (zval const   *)(& c));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1604\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1585\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1582\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1579\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + lc_literal)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val - sizeof(Bucket )))->h;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1580\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val,
                                 (unsigned int )(((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1581\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + lc_literal)->constant))->hash_value = (unsigned long )tmp___1;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1583\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val,
                               (unsigned int )(((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.len + 1));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1584\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + lc_literal)->constant))->hash_value = (unsigned long )tmp___1;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1605\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = zend_memrchr((void const   *)zv->value.str.val, '\\',
                         (unsigned int )zv->value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1606\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ns_separator = (char const   *)tmp___2 + 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1607\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lc_len = (int )(zv->value.str.len - (int const   )(ns_separator - (char const   *)zv->value.str.val));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1608\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = zend_str_tolower_dup(ns_separator, (unsigned int )lc_len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1609\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lc_name = (char *)tmp___3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1610\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1586\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s___0 = (char const   *)lc_name;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1587\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __l___0 = lc_len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1588\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z___0 = & c;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1589\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z___0->value.str.len = __l___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1590\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z___0->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s___0));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1591\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z___0->type = (unsigned char)6;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1611\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lc_literal = zend_add_literal(compiler_globals.active_op_array,
                                (zval const   *)(& c));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1612\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1598\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1595\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1592\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + lc_literal)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val - sizeof(Bucket )))->h;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1593\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___5 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val,
                                 (unsigned int )(((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1594\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + lc_literal)->constant))->hash_value = (unsigned long )tmp___5;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1596\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___5 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val,
                               (unsigned int )(((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.len + 1));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1597\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + lc_literal)->constant))->hash_value = (unsigned long )tmp___5;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1613\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (ret);
}
}
int zend_add_func_name_literal(zend_op_array *op_array , zval const   *zv ) 
{ int ret ;
  char *lc_name ;
  zval c ;
  int lc_literal ;
  char __attribute__((__visibility__("default")))  *tmp ;
  char const   *__s ;
  int __l ;
  zval *__z ;
  char __attribute__((__visibility__("default")))  *tmp___0 ;
  ulong __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1560\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (op_array->last_literal > 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1545\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )(& (op_array->literals + (op_array->last_literal - 1))->constant) == (unsigned int )zv) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1543\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((op_array->literals + (op_array->last_literal - 1))->cache_slot == 4294967295U) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1541\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ret = op_array->last_literal - 1;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1542\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ret = zend_add_literal(op_array, zv);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1544\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ret = zend_add_literal(op_array, zv);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1546\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ret = zend_add_literal(op_array, zv);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1561\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = zend_str_tolower_dup((char const   *)zv->value.str.val,
                             (unsigned int )zv->value.str.len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1562\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lc_name = (char *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1563\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1547\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s = (char const   *)lc_name;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1548\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __l = (int )zv->value.str.len;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1549\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z = & c;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1550\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->value.str.len = __l;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1551\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->value.str.val = (char *)((char __attribute__((__visibility__("default")))  *)((char *)__s));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1552\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __z->type = (unsigned char)6;
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1564\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  lc_literal = zend_add_literal(compiler_globals.active_op_array,
                                (zval const   *)(& c));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1565\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1559\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val >= (unsigned int )compiler_globals.interned_strings_start) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1556\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val < (unsigned int )compiler_globals.interned_strings_end) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1553\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + lc_literal)->constant))->hash_value = ((Bucket *)(((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val - sizeof(Bucket )))->h;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1554\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val,
                                 (unsigned int )(((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.len + 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1555\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + lc_literal)->constant))->hash_value = (unsigned long )tmp___1;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1557\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = zend_hash_func((char const   *)((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.val,
                               (unsigned int )(((compiler_globals.active_op_array)->literals + lc_literal)->constant.value.str.len + 1));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1558\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((zend_literal *)(& ((compiler_globals.active_op_array)->literals + lc_literal)->constant))->hash_value = (unsigned long )tmp___1;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1566\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (ret);
}
}
__inline static void zend_insert_literal(zend_op_array *op_array ,
                                         zval const   *zv ,
                                         int literal_position ) ;
int zend_append_individual_literal(zend_op_array *op_array , zval const   *zv ) 
{ int i ;
  void __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1535\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = op_array->last_literal;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1536\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (op_array->last_literal) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1537\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _erealloc((void *)op_array->literals,
                  (unsigned int )(i + 1) * sizeof(zend_literal ), 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1538\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  op_array->literals = (zend_literal *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1539\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_insert_literal(op_array, zv, i);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1540\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (i);
}
}
int zend_append_individual_literal(zend_op_array *op_array , zval const   *zv ) ;
int zend_add_literal(zend_op_array *op_array , zval const   *zv ) 
{ int i ;
  void __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1530\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = op_array->last_literal;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1531\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (op_array->last_literal) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1532\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (i >= compiler_globals.context.literals_size) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1526\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1524\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (i >= compiler_globals.context.literals_size) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1523\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1525\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      compiler_globals.context.literals_size += 16;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1527\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = _erealloc((void *)op_array->literals,
                    (unsigned int )compiler_globals.context.literals_size * sizeof(zend_literal ),
                    0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1528\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op_array->literals = (zend_literal *)tmp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1529\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1533\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_insert_literal(op_array, zv, i);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1534\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (i);
}
}
__inline static void zend_insert_literal(zend_op_array *op_array ,
                                         zval const   *zv ,
                                         int literal_position ) 
{ zval *z ;
  char const __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1516\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )zv->type == 6) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1508\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    z = (zval *)zv;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1509\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = (*zend_new_interned_string)((char const   *)zv->value.str.val,
                                      (int )(zv->value.str.len + 1), 1);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1510\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    z->value.str.val = (char *)tmp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1515\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )zv->type == 8) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1511\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      z = (zval *)zv;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1512\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = (*zend_new_interned_string)((char const   *)zv->value.str.val,
                                        (int )(zv->value.str.len + 1), 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1513\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      z->value.str.val = (char *)tmp;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1514\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1517\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (op_array->literals + literal_position)->constant = (struct _zval_struct )*zv;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1518\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zval_set_refcount_p(& (op_array->literals + literal_position)->constant, 2U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1519\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zval_set_isref_p(& (op_array->literals + literal_position)->constant);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1520\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (op_array->literals + literal_position)->hash_value = 0UL;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1521\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (op_array->literals + literal_position)->cache_slot = 4294967295U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1522\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_del_literal(zend_op_array *op_array , int n ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1505\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_dtor(& (op_array->literals + n)->constant);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1506\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (n + 1 == op_array->last_literal) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1503\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (op_array->last_literal) --;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1504\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (op_array->literals + n)->constant.type = (unsigned char)0;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1507\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static ulong zend_inline_hash_func(char const   *arKey ,
                                            uint nKeyLength ) ;
static int lookup_cv(zend_op_array *op_array , char *name , int name_len ,
                     ulong hash ) 
{ int i ;
  ulong hash_value ;
  ulong tmp ;
  ulong tmp___0 ;
  int tmp___1 ;
  void __attribute__((__visibility__("default")))  *tmp___2 ;
  char const __attribute__((__visibility__("default")))  *tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1491\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1492\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (hash) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1466\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = hash;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1467\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_inline_hash_func((char const   *)name,
                                (unsigned int )(name_len + 1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1468\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = tmp;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1493\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash_value = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1494\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1484\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (i < op_array->last_var) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1469\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1485\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )(op_array->vars + i)->name == (unsigned int )name) {
      goto _L;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1483\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((op_array->vars + i)->hash_value == hash_value) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1481\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((op_array->vars + i)->name_len == name_len) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1478\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___1 = memcmp((void const   *)(op_array->vars + i)->name,
                           (void const   *)name, (unsigned int )name_len);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1479\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (tmp___1 == 0) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1475\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L: /* CIL Label */ 
            while (1) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "1474\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((unsigned int )name >= (unsigned int )compiler_globals.interned_strings_start) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "1472\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((unsigned int )name < (unsigned int )compiler_globals.interned_strings_end) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "1470\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "1471\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  _efree((void *)name);
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "1473\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                _efree((void *)name);
              }
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1476\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return (i);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1477\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1480\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1482\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1486\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1495\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  i = op_array->last_var;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1496\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (op_array->last_var) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1497\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (op_array->last_var > compiler_globals.context.vars_size) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1487\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.context.vars_size += 16;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1488\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = _erealloc((void *)op_array->vars,
                        (unsigned int )compiler_globals.context.vars_size * sizeof(zend_compiled_variable ),
                        0);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1489\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    op_array->vars = (zend_compiled_variable *)tmp___2;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1490\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1498\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = (*zend_new_interned_string)((char const   *)name, name_len + 1, 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1499\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (op_array->vars + i)->name = (char const   *)tmp___3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1500\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (op_array->vars + i)->name_len = name_len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1501\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (op_array->vars + i)->hash_value = hash_value;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1502\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (i);
}
}
struct _HashPointer {
   HashPosition pos ;
   ulong h ;
};
typedef struct _HashPointer HashPointer;
struct __anonstruct_fe_54 {
   zval **ptr_ptr ;
   zval *ptr ;
   HashPointer fe_pos ;
};
struct __anonstruct_str_offset_53 {
   zval **ptr_ptr ;
   zval *str ;
   zend_uint offset ;
};
struct __anonstruct_var_52 {
   zval **ptr_ptr ;
   zval *ptr ;
   zend_bool fcall_returned_reference ;
};
union _temp_variable {
   zval tmp_var ;
   struct __anonstruct_var_52 var ;
   struct __anonstruct_str_offset_53 str_offset ;
   struct __anonstruct_fe_54 fe ;
   zend_class_entry *class_entry ;
};
typedef union _temp_variable temp_variable;
static zend_uint get_temporary_variable(zend_op_array *op_array ) 
{ zend_uint tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1463\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = op_array->T;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1464\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (op_array->T) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1465\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp * (((sizeof(temp_variable ) + 4U) - 1U) & 4294967292U));
}
}
zend_bool __attribute__((__visibility__("default")))  zend_is_compiling(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1462\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned char __attribute__((__visibility__("default")))  )compiler_globals.in_compilation);
}
}
zend_bool __attribute__((__visibility__("default")))  zend_is_compiling(void) ;
int __attribute__((__visibility__("default")))  zend_get_compiled_lineno(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1461\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int __attribute__((__visibility__("default")))  )compiler_globals.zend_lineno);
}
}
char __attribute__((__visibility__("default")))  *zend_get_compiled_filename(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1460\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((char __attribute__((__visibility__("default")))  *)compiler_globals.compiled_filename);
}
}
void __attribute__((__visibility__("default")))  zend_restore_compiled_filename(char *original_compiled_filename ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1458\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.compiled_filename = original_compiled_filename;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1459\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void __attribute__((__visibility__("default")))  zend_restore_compiled_filename(char *original_compiled_filename ) ;
char __attribute__((__visibility__("default")))  *zend_set_compiled_filename(char const   *new_compiled_filename ) 
{ char **pp ;
  char *p ;
  int length ;
  size_t tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  char __attribute__((__visibility__("default")))  *tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1449\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = strlen(new_compiled_filename);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1450\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  length = (int )tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1451\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = zend_hash_find((HashTable const   *)(& compiler_globals.filenames_table),
                           new_compiled_filename, (unsigned int )(length + 1),
                           (void **)(& pp));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1452\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___0 == (int __attribute__((__visibility__("default")))  )0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1446\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.compiled_filename = *pp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1447\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((char __attribute__((__visibility__("default")))  *)*pp);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1448\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1453\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = _estrndup(new_compiled_filename, (unsigned int )length);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1454\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p = (char *)tmp___1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1455\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zend_hash_add_or_update(& compiler_globals.filenames_table,
                           new_compiled_filename, (unsigned int )(length + 1),
                           (void *)(& p), sizeof(char *), (void **)(& pp), 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1456\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.compiled_filename = p;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1457\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((char __attribute__((__visibility__("default")))  *)p);
}
}
char __attribute__((__visibility__("default")))  *zend_set_compiled_filename(char const   *new_compiled_filename ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_destroy(zend_stack *stack ) ;
void shutdown_compiler(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1435\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_destroy(& compiler_globals.bp_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1436\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_destroy(& compiler_globals.function_call_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1437\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_destroy(& compiler_globals.switch_cond_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1438\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_destroy(& compiler_globals.foreach_copy_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1439\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_destroy(& compiler_globals.object_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1440\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_destroy(& compiler_globals.declare_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1441\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_destroy(& compiler_globals.list_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1442\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_hash_destroy(& compiler_globals.filenames_table);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1443\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_destroy(& compiler_globals.open_files);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1444\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_destroy(& compiler_globals.context_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1445\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void shutdown_compiler(void) ;
void __attribute__((__visibility__("default")))  file_handle_dtor(zend_file_handle *fh ) ;
struct _zend_mmap {
   size_t len ;
   size_t pos ;
   void *map ;
   char *buf ;
   void *old_handle ;
   void (*old_closer)(void *handle ) ;
};
typedef struct _zend_mmap zend_mmap;
struct _zend_stream {
   void *handle ;
   int isatty ;
   zend_mmap mmap ;
   size_t (*reader)(void *handle , char *buf , size_t len ) ;
   size_t (*fsizer)(void *handle ) ;
   void (*closer)(void *handle ) ;
};
typedef struct _zend_stream zend_stream;
union __anonunion_handle_44 {
   int fd ;
   FILE *fp ;
   zend_stream stream ;
};
enum __anonenum_zend_stream_type_43 {
    ZEND_HANDLE_FILENAME = 0,
    ZEND_HANDLE_FD = 1,
    ZEND_HANDLE_FP = 2,
    ZEND_HANDLE_STREAM = 3,
    ZEND_HANDLE_MAPPED = 4
} ;
typedef enum __anonenum_zend_stream_type_43 zend_stream_type;
struct _zend_file_handle {
   zend_stream_type type ;
   char const   *filename ;
   char *opened_path ;
   union __anonunion_handle_44 handle ;
   zend_bool free_filename ;
};
extern void __attribute__((__visibility__("default")))  free_estring(char **str_p ) ;
extern int zend_init_rsrc_list(void) ;
void zend_init_compiler_data_structures(void) ;
void init_compiler(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1428\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.active_op_array = (zend_op_array *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1429\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_init_compiler_data_structures();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1430\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_init_rsrc_list();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1431\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zend_hash_init(& compiler_globals.filenames_table, 5U,
                  (ulong (*)(char const   *arKey , uint nKeyLength ))((void *)0),
                  (void (*)(void *pDest ))(& free_estring), (unsigned char)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1432\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_init(& compiler_globals.open_files, sizeof(zend_file_handle ),
                  (void (*)(void * ))(& file_handle_dtor), (unsigned char)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1433\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.unclean_shutdown = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1434\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void init_compiler(void) ;
extern void __attribute__((__visibility__("default")))  zend_file_handle_dtor(zend_file_handle *fh ) ;
void __attribute__((__visibility__("default")))  file_handle_dtor(zend_file_handle *fh ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1426\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_file_handle_dtor(fh);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1427\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern int __attribute__((__visibility__("default")))  zend_stack_init(zend_stack *stack ) ;
static void init_compiler_declarables(void) ;
void zend_init_compiler_data_structures(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1406\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_init(& compiler_globals.bp_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1407\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_init(& compiler_globals.function_call_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1408\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_init(& compiler_globals.switch_cond_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1409\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_init(& compiler_globals.foreach_copy_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1410\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_init(& compiler_globals.object_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1411\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_init(& compiler_globals.declare_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1412\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.active_class_entry = (zend_class_entry *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1413\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_init(& compiler_globals.list_llist, sizeof(list_llist_element ),
                  (void (*)(void * ))((void *)0), (unsigned char)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1414\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_llist_init(& compiler_globals.dimension_llist, sizeof(int ),
                  (void (*)(void * ))((void *)0), (unsigned char)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1415\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_init(& compiler_globals.list_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1416\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.in_compilation = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1417\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.start_lineno = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1418\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.current_namespace = (zval *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1419\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.in_namespace = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1420\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.has_bracketed_namespaces = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1421\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.current_import = (HashTable *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1422\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  init_compiler_declarables();
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1423\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_stack_init(& compiler_globals.context_stack);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1424\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.encoding_declared = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1425\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
void zend_init_compiler_context(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1399\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->fn_flags & 16U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1397\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.context.opcodes_size = 8192U;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1398\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    compiler_globals.context.opcodes_size = 64U;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1400\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.context.vars_size = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1401\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.context.literals_size = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1402\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.context.current_brk_cont = -1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1403\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.context.backpatch_count = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1404\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.context.labels = (HashTable *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1405\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static void init_compiler_declarables(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1394\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.declarables.ticks.type = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1395\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compiler_globals.declarables.ticks.value.lval = 0L;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1396\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern  __attribute__((__nothrow__)) int sprintf(char * __restrict  __s ,
                                                 char const   * __restrict  __format 
                                                 , ...) ;
extern void __attribute__((__visibility__("default")))  *_safe_emalloc(size_t nmemb ,
                                                                       size_t size ,
                                                                       size_t offset )  __attribute__((__malloc__)) ;
static void build_runtime_defined_function_key(zval *result ,
                                               char const   *name ,
                                               int name_length ) 
{ char char_pos_buf[32] ;
  uint char_pos_len ;
  char const   *filename ;
  int tmp ;
  size_t tmp___0 ;
  void __attribute__((__visibility__("default")))  *tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1382\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = sprintf((char */* __restrict  */)(char_pos_buf),
                (char const   */* __restrict  */)"%p",
                language_scanner_globals.yy_text);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1383\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  char_pos_len = (unsigned int )tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1384\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((compiler_globals.active_op_array)->filename) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1380\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    filename = (compiler_globals.active_op_array)->filename;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1381\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    filename = "-";
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1385\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = strlen(filename);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1386\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->value.str.len = (int )(((size_t )(1 + name_length) + tmp___0) + char_pos_len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1387\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = _safe_emalloc((unsigned int )result->value.str.len, 1U, 1U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1388\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->value.str.val = (char *)tmp___1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1389\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *(result->value.str.val + 0) = (char )'\000';
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1390\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  sprintf((char */* __restrict  */)(result->value.str.val + 1),
          (char const   */* __restrict  */)"%s%s%s", name, filename,
          char_pos_buf);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1391\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result->type = (unsigned char)6;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1392\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zval_set_refcount_p(result, 1U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1393\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static void zend_destroy_property_info_internal(zend_property_info *property_info ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1378\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1377\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )((char *)property_info->name) >= (unsigned int )compiler_globals.interned_strings_start) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1375\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )((char *)property_info->name) < (unsigned int )compiler_globals.interned_strings_end) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1373\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1374\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        free((void *)((char *)property_info->name));
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1376\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      free((void *)((char *)property_info->name));
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1379\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static void zend_destroy_property_info(zend_property_info *property_info ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1370\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1367\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )property_info->name >= (unsigned int )compiler_globals.interned_strings_start) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1365\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((unsigned int )property_info->name < (unsigned int )compiler_globals.interned_strings_end) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1363\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1364\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        _efree((void *)((char *)property_info->name));
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1366\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)((char *)property_info->name));
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1371\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (property_info->doc_comment) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1368\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)((char *)property_info->doc_comment));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1369\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1372\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern char __attribute__((__visibility__("default")))  *zend_strndup(char const   *s ,
                                                                      unsigned int length )  __attribute__((__malloc__)) ;
static void zend_duplicate_property_info_internal(zend_property_info *property_info ) 
{ char __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1361\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )property_info->name >= (unsigned int )compiler_globals.interned_strings_start) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1358\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )property_info->name < (unsigned int )compiler_globals.interned_strings_end) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1355\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1356\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = zend_strndup(property_info->name,
                         (unsigned int )property_info->name_length);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1357\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      property_info->name = (char const   *)tmp;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1359\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zend_strndup(property_info->name,
                       (unsigned int )property_info->name_length);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1360\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    property_info->name = (char const   *)tmp;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1362\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
static void zend_duplicate_property_info(zend_property_info *property_info ) 
{ char __attribute__((__visibility__("default")))  *tmp ;
  char __attribute__((__visibility__("default")))  *tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1352\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )property_info->name >= (unsigned int )compiler_globals.interned_strings_start) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1346\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )property_info->name < (unsigned int )compiler_globals.interned_strings_end) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1343\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1344\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = _estrndup(property_info->name,
                      (unsigned int )property_info->name_length);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1345\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      property_info->name = (char const   *)tmp;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1347\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = _estrndup(property_info->name,
                    (unsigned int )property_info->name_length);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1348\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    property_info->name = (char const   *)tmp;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1353\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (property_info->doc_comment) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1349\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = _estrndup(property_info->doc_comment,
                        (unsigned int )property_info->doc_comment_len);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1350\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    property_info->doc_comment = (char const   *)tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1351\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1354\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern int __attribute__((__visibility__("default")))  zend_prepare_string_for_scanning(zval *str ,
                                                                                        char *filename ) ;
struct _zend_lex_state;
typedef struct _zend_lex_state zend_lex_state;
extern void __attribute__((__visibility__("default")))  zend_restore_lexical_state(zend_lex_state *lex_state ) ;
extern void __attribute__((__visibility__("default")))  zend_save_lexical_state(zend_lex_state *lex_state ) ;
struct _realpath_cache_bucket;
typedef struct _realpath_cache_bucket realpath_cache_bucket;
extern realpath_cache_bucket __attribute__((__visibility__("default")))  **realpath_cache_get_buckets(void) ;
extern int __attribute__((__visibility__("default")))  realpath_cache_max_buckets(void) ;
extern int __attribute__((__visibility__("default")))  realpath_cache_size(void) ;
typedef long __time_t;
typedef __time_t time_t;
extern realpath_cache_bucket __attribute__((__visibility__("default")))  *realpath_cache_lookup(char const   *path ,
                                                                                                int path_len ,
                                                                                                time_t t ) ;
extern void __attribute__((__visibility__("default")))  realpath_cache_del(char const   *path ,
                                                                           int path_len ) ;
extern void __attribute__((__visibility__("default")))  realpath_cache_clean(void) ;
struct _virtual_cwd_globals;
typedef struct _virtual_cwd_globals virtual_cwd_globals;
extern virtual_cwd_globals cwd_globals ;
extern char __attribute__((__visibility__("default")))  *tsrm_realpath(char const   *path ,
                                                                       char *real_path ) ;
struct _cwd_state;
typedef struct _cwd_state cwd_state;
extern int __attribute__((__visibility__("default")))  virtual_file_ex(cwd_state *state ,
                                                                       char const   *path ,
                                                                       int (*verify_path)(cwd_state const   * ) ,
                                                                       int use_realpath ) ;
typedef unsigned int __gid_t;
typedef __gid_t gid_t;
typedef unsigned int __uid_t;
typedef __uid_t uid_t;
extern int __attribute__((__visibility__("default")))  virtual_chown(char const   *filename ,
                                                                     uid_t owner ,
                                                                     gid_t group ,
                                                                     int link ) ;
typedef unsigned int __mode_t;
typedef __mode_t mode_t;
extern int __attribute__((__visibility__("default")))  virtual_chmod(char const   *filename ,
                                                                     mode_t mode ) ;
struct utimbuf;
extern int __attribute__((__visibility__("default")))  virtual_utime(char const   *filename ,
                                                                     struct utimbuf *buf ) ;
extern int __attribute__((__visibility__("default")))  virtual_access(char const   *pathname ,
                                                                      int mode ) ;
extern FILE __attribute__((__visibility__("default")))  *virtual_popen(char const   *command ,
                                                                       char const   *type ) ;
struct __dirstream;
typedef struct __dirstream DIR;
extern DIR __attribute__((__visibility__("default")))  *virtual_opendir(char const   *pathname ) ;
extern int __attribute__((__visibility__("default")))  virtual_rmdir(char const   *pathname ) ;
extern int __attribute__((__visibility__("default")))  virtual_mkdir(char const   *pathname ,
                                                                     mode_t mode ) ;
extern int __attribute__((__visibility__("default")))  virtual_unlink(char const   *path ) ;
struct stat;
extern int __attribute__((__visibility__("default")))  virtual_lstat(char const   *path ,
                                                                     struct stat *buf ) ;
extern int __attribute__((__visibility__("default")))  virtual_stat(char const   *path ,
                                                                    struct stat *buf ) ;
extern int __attribute__((__visibility__("default")))  virtual_rename(char *oldname ,
                                                                      char *newname ) ;
extern int __attribute__((__visibility__("default")))  virtual_creat(char const   *path ,
                                                                     mode_t mode ) ;
extern int __attribute__((__visibility__("default")))  virtual_open(char const   *path ,
                                                                    int flags 
                                                                    , ...) ;
extern FILE __attribute__((__visibility__("default")))  *virtual_fopen(char const   *path ,
                                                                       char const   *mode ) ;
extern char __attribute__((__visibility__("default")))  *virtual_realpath(char const   *path ,
                                                                          char *real_path ) ;
extern int __attribute__((__visibility__("default")))  virtual_filepath_ex(char const   *path ,
                                                                           char **filepath ,
                                                                           int (*verify_path)(cwd_state const   * ) ) ;
extern int __attribute__((__visibility__("default")))  virtual_filepath(char const   *path ,
                                                                        char **filepath ) ;
extern int __attribute__((__visibility__("default")))  virtual_chdir_file(char const   *path ,
                                                                          int (*p_chdir)(char const   *path ) ) ;
extern int __attribute__((__visibility__("default")))  virtual_chdir(char const   *path ) ;
extern char __attribute__((__visibility__("default")))  *virtual_getcwd(char *buf ,
                                                                        size_t size ) ;
extern char __attribute__((__visibility__("default")))  *virtual_getcwd_ex(size_t *length ) ;
extern void __attribute__((__visibility__("default")))  virtual_cwd_shutdown(void) ;
extern void __attribute__((__visibility__("default")))  virtual_cwd_startup(void) ;
struct dirent64;
extern  __attribute__((__nothrow__)) int versionsort64(struct dirent64  const  **__e1 ,
                                                       struct dirent64  const  **__e2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
struct dirent;
extern  __attribute__((__nothrow__)) int versionsort(struct dirent  const  **__e1 ,
                                                     struct dirent  const  **__e2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
typedef long long __quad_t;
typedef __quad_t __off64_t;
typedef int __ssize_t;
extern  __attribute__((__nothrow__)) __ssize_t getdirentries64(int __fd ,
                                                               char * __restrict  __buf ,
                                                               size_t __nbytes ,
                                                               __off64_t * __restrict  __basep )  __attribute__((__nonnull__(2,4))) ;
typedef long __off_t;
extern  __attribute__((__nothrow__)) __ssize_t getdirentries(int __fd ,
                                                             char * __restrict  __buf ,
                                                             size_t __nbytes ,
                                                             __off_t * __restrict  __basep )  __attribute__((__nonnull__(2,4))) ;
extern  __attribute__((__nothrow__)) int alphasort64(struct dirent64  const  **__e1 ,
                                                     struct dirent64  const  **__e2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int alphasort(struct dirent  const  **__e1 ,
                                                   struct dirent  const  **__e2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern int scandir64(char const   * __restrict  __dir ,
                     struct dirent64 *** __restrict  __namelist ,
                     int (*__selector)(struct dirent64  const  * ) ,
                     int (*__cmp)(struct dirent64  const  ** ,
                                  struct dirent64  const  ** ) )  __attribute__((__nonnull__(1,2))) ;
extern int scandir(char const   * __restrict  __dir ,
                   struct dirent *** __restrict  __namelist ,
                   int (*__selector)(struct dirent  const  * ) ,
                   int (*__cmp)(struct dirent  const  ** ,
                                struct dirent  const  ** ) )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int dirfd(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long telldir(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void seekdir(DIR *__dirp , long __pos )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void rewinddir(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern int readdir64_r(DIR * __restrict  __dirp ,
                       struct dirent64 * __restrict  __entry ,
                       struct dirent64 ** __restrict  __result )  __attribute__((__nonnull__(1,2,3))) ;
extern int readdir_r(DIR * __restrict  __dirp ,
                     struct dirent * __restrict  __entry ,
                     struct dirent ** __restrict  __result )  __attribute__((__nonnull__(1,2,3))) ;
extern struct dirent64 *readdir64(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern struct dirent *readdir(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern int closedir(DIR *__dirp )  __attribute__((__nonnull__(1))) ;
extern DIR *fdopendir(int __fd ) ;
extern DIR *opendir(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int utime(char const   *__file ,
                                               struct utimbuf  const  *__file_times )  __attribute__((__nonnull__(1))) ;
struct __locale_struct;
typedef struct __locale_struct *__locale_t;
extern  __attribute__((__nothrow__)) int toupper_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int __toupper_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int tolower_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int __tolower_l(int __c , __locale_t __l ) ;
extern  __attribute__((__nothrow__)) int isblank_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isxdigit_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isupper_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isspace_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int ispunct_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isprint_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isgraph_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int islower_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isdigit_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int iscntrl_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isalpha_l(int  , __locale_t  ) ;
extern  __attribute__((__nothrow__)) int isalnum_l(int  , __locale_t  ) ;
typedef int __int32_t;
extern  __attribute__((__nothrow__)) __int32_t const   **__ctype_toupper_loc(void)  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) int toupper(int __c ) ;
__inline extern int toupper(int __c ) 
{ __int32_t const   **tmp ;
  __int32_t tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1341\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (__c >= -128) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1339\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (__c < 256) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1336\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __ctype_toupper_loc();
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1337\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = *(*tmp + __c);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1338\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = (int const   )__c;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1340\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = (int const   )__c;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1342\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) int toupper(int __c ) ;
extern  __attribute__((__nothrow__)) __int32_t const   **__ctype_tolower_loc(void)  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) int tolower(int __c ) ;
__inline extern int tolower(int __c ) 
{ __int32_t const   **tmp ;
  __int32_t tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1334\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (__c >= -128) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1332\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (__c < 256) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1329\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __ctype_tolower_loc();
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1330\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = *(*tmp + __c);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1331\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = (int const   )__c;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1333\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = (int const   )__c;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1335\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) int tolower(int __c ) ;
extern  __attribute__((__nothrow__)) int _tolower(int  ) ;
extern  __attribute__((__nothrow__)) int _toupper(int  ) ;
extern  __attribute__((__nothrow__)) int toascii(int __c ) ;
extern  __attribute__((__nothrow__)) int isascii(int __c ) ;
extern  __attribute__((__nothrow__)) int isctype(int __c , int __mask ) ;
extern  __attribute__((__nothrow__)) int isblank(int  ) ;
extern  __attribute__((__nothrow__)) int isxdigit(int  ) ;
extern  __attribute__((__nothrow__)) int isupper(int  ) ;
extern  __attribute__((__nothrow__)) int isspace(int  ) ;
extern  __attribute__((__nothrow__)) int ispunct(int  ) ;
extern  __attribute__((__nothrow__)) int isprint(int  ) ;
extern  __attribute__((__nothrow__)) int isgraph(int  ) ;
extern  __attribute__((__nothrow__)) int islower(int  ) ;
extern  __attribute__((__nothrow__)) int isdigit(int  ) ;
extern  __attribute__((__nothrow__)) int iscntrl(int  ) ;
extern  __attribute__((__nothrow__)) int isalpha(int  ) ;
extern  __attribute__((__nothrow__)) int isalnum(int  ) ;
extern  __attribute__((__nothrow__)) unsigned short const   **__ctype_b_loc(void)  __attribute__((__const__)) ;
struct stat64;
extern  __attribute__((__nothrow__)) int __fxstatat64(int __ver , int __fildes ,
                                                      char const   *__filename ,
                                                      struct stat64 *__stat_buf ,
                                                      int __flag )  __attribute__((__nonnull__(3,4))) ;
__inline extern  __attribute__((__nothrow__)) int fstatat64(int __fd ,
                                                            char const   * __restrict  __filename ,
                                                            struct stat64 * __restrict  __statbuf ,
                                                            int __flag )  __attribute__((__nonnull__(2,3))) ;
__inline extern int fstatat64(int __fd , char const   * __restrict  __filename ,
                              struct stat64 * __restrict  __statbuf ,
                              int __flag ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1327\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __fxstatat64(3, __fd, (char const   *)__filename,
                     (struct stat64 *)__statbuf, __flag);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1328\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int fstatat64(int __fd ,
                                                            char const   * __restrict  __filename ,
                                                            struct stat64 * __restrict  __statbuf ,
                                                            int __flag )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) int __fxstat64(int __ver , int __fildes ,
                                                    struct stat64 *__stat_buf )  __attribute__((__nonnull__(3))) ;
__inline extern  __attribute__((__nothrow__)) int fstat64(int __fd ,
                                                          struct stat64 *__statbuf )  __attribute__((__nonnull__(2))) ;
__inline extern int fstat64(int __fd , struct stat64 *__statbuf ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1325\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __fxstat64(3, __fd, __statbuf);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1326\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int fstat64(int __fd ,
                                                          struct stat64 *__statbuf )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int __lxstat64(int __ver ,
                                                    char const   *__filename ,
                                                    struct stat64 *__stat_buf )  __attribute__((__nonnull__(2,3))) ;
__inline extern  __attribute__((__nothrow__)) int lstat64(char const   * __restrict  __path ,
                                                          struct stat64 * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
__inline extern int lstat64(char const   * __restrict  __path ,
                            struct stat64 * __restrict  __statbuf ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1323\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __lxstat64(3, (char const   *)__path, (struct stat64 *)__statbuf);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1324\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int lstat64(char const   * __restrict  __path ,
                                                          struct stat64 * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int __xstat64(int __ver ,
                                                   char const   *__filename ,
                                                   struct stat64 *__stat_buf )  __attribute__((__nonnull__(2,3))) ;
__inline extern  __attribute__((__nothrow__)) int stat64(char const   * __restrict  __path ,
                                                         struct stat64 * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
__inline extern int stat64(char const   * __restrict  __path ,
                           struct stat64 * __restrict  __statbuf ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1321\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __xstat64(3, (char const   *)__path, (struct stat64 *)__statbuf);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1322\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int stat64(char const   * __restrict  __path ,
                                                         struct stat64 * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
typedef unsigned long long __u_quad_t;
typedef __u_quad_t __dev_t;
extern  __attribute__((__nothrow__)) int __xmknodat(int __ver , int __fd ,
                                                    char const   *__path ,
                                                    __mode_t __mode ,
                                                    __dev_t *__dev )  __attribute__((__nonnull__(3,5))) ;
__inline extern  __attribute__((__nothrow__)) int mknodat(int __fd ,
                                                          char const   *__path ,
                                                          __mode_t __mode ,
                                                          __dev_t __dev )  __attribute__((__nonnull__(2))) ;
__inline extern int mknodat(int __fd , char const   *__path , __mode_t __mode ,
                            __dev_t __dev ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1319\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __xmknodat(1, __fd, __path, __mode, & __dev);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1320\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int mknodat(int __fd ,
                                                          char const   *__path ,
                                                          __mode_t __mode ,
                                                          __dev_t __dev )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int __xmknod(int __ver ,
                                                  char const   *__path ,
                                                  __mode_t __mode ,
                                                  __dev_t *__dev )  __attribute__((__nonnull__(2,4))) ;
__inline extern  __attribute__((__nothrow__)) int mknod(char const   *__path ,
                                                        __mode_t __mode ,
                                                        __dev_t __dev )  __attribute__((__nonnull__(1))) ;
__inline extern int mknod(char const   *__path , __mode_t __mode ,
                          __dev_t __dev ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1317\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __xmknod(1, __path, __mode, & __dev);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1318\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int mknod(char const   *__path ,
                                                        __mode_t __mode ,
                                                        __dev_t __dev )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int __fxstatat(int __ver , int __fildes ,
                                                    char const   *__filename ,
                                                    struct stat *__stat_buf ,
                                                    int __flag )  __attribute__((__nonnull__(3,4))) ;
__inline extern  __attribute__((__nothrow__)) int fstatat(int __fd ,
                                                          char const   * __restrict  __filename ,
                                                          struct stat * __restrict  __statbuf ,
                                                          int __flag )  __attribute__((__nonnull__(2,3))) ;
__inline extern int fstatat(int __fd , char const   * __restrict  __filename ,
                            struct stat * __restrict  __statbuf , int __flag ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1315\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __fxstatat(3, __fd, (char const   *)__filename,
                   (struct stat *)__statbuf, __flag);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1316\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int fstatat(int __fd ,
                                                          char const   * __restrict  __filename ,
                                                          struct stat * __restrict  __statbuf ,
                                                          int __flag )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) int __fxstat(int __ver , int __fildes ,
                                                  struct stat *__stat_buf )  __attribute__((__nonnull__(3))) ;
__inline extern  __attribute__((__nothrow__)) int fstat(int __fd ,
                                                        struct stat *__statbuf )  __attribute__((__nonnull__(2))) ;
__inline extern int fstat(int __fd , struct stat *__statbuf ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1313\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __fxstat(3, __fd, __statbuf);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1314\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int fstat(int __fd ,
                                                        struct stat *__statbuf )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int __lxstat(int __ver ,
                                                  char const   *__filename ,
                                                  struct stat *__stat_buf )  __attribute__((__nonnull__(2,3))) ;
__inline extern  __attribute__((__nothrow__)) int lstat(char const   * __restrict  __path ,
                                                        struct stat * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
__inline extern int lstat(char const   * __restrict  __path ,
                          struct stat * __restrict  __statbuf ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1311\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __lxstat(3, (char const   *)__path, (struct stat *)__statbuf);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1312\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int lstat(char const   * __restrict  __path ,
                                                        struct stat * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int __xstat(int __ver ,
                                                 char const   *__filename ,
                                                 struct stat *__stat_buf )  __attribute__((__nonnull__(2,3))) ;
__inline extern  __attribute__((__nothrow__)) int stat(char const   * __restrict  __path ,
                                                       struct stat * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
__inline extern int stat(char const   * __restrict  __path ,
                         struct stat * __restrict  __statbuf ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1309\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __xstat(3, (char const   *)__path, (struct stat *)__statbuf);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1310\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int stat(char const   * __restrict  __path ,
                                                       struct stat * __restrict  __statbuf )  __attribute__((__nonnull__(1,2))) ;
struct timespec;
extern  __attribute__((__nothrow__)) int futimens(int __fd ,
                                                  struct timespec  const  *__times ) ;
extern  __attribute__((__nothrow__)) int utimensat(int __fd ,
                                                   char const   *__path ,
                                                   struct timespec  const  *__times ,
                                                   int __flags )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int mkfifoat(int __fd ,
                                                  char const   *__path ,
                                                  __mode_t __mode )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int mkfifo(char const   *__path ,
                                                __mode_t __mode )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int mkdirat(int __fd ,
                                                 char const   *__path ,
                                                 __mode_t __mode )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int mkdir(char const   *__path ,
                                               __mode_t __mode )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) __mode_t getumask(void) ;
extern  __attribute__((__nothrow__)) __mode_t umask(__mode_t __mask ) ;
extern  __attribute__((__nothrow__)) int fchmodat(int __fd ,
                                                  char const   *__file ,
                                                  __mode_t __mode , int __flag )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int fchmod(int __fd , __mode_t __mode ) ;
extern  __attribute__((__nothrow__)) int lchmod(char const   *__file ,
                                                __mode_t __mode )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int chmod(char const   *__file ,
                                               __mode_t __mode )  __attribute__((__nonnull__(1))) ;
typedef __ssize_t ssize_t;
extern  __attribute__((__nothrow__)) void swab(void const   * __restrict  __from ,
                                               void * __restrict  __to ,
                                               ssize_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void encrypt(char *__block , int __edflag )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *crypt(char const   *__key ,
                                                 char const   *__salt )  __attribute__((__nonnull__(1,2))) ;
extern int fdatasync(int __fildes ) ;
extern int lockf64(int __fd , int __cmd , __off64_t __len ) ;
extern int lockf(int __fd , int __cmd , __off_t __len ) ;
extern  __attribute__((__nothrow__)) long syscall(long __sysno  , ...) ;
typedef int __intptr_t;
typedef __intptr_t intptr_t;
extern  __attribute__((__nothrow__)) void *sbrk(intptr_t __delta ) ;
extern  __attribute__((__nothrow__)) int brk(void *__addr ) ;
extern  __attribute__((__nothrow__)) int ftruncate64(int __fd ,
                                                     __off64_t __length ) ;
extern  __attribute__((__nothrow__)) int ftruncate(int __fd , __off_t __length ) ;
extern  __attribute__((__nothrow__)) int truncate64(char const   *__file ,
                                                    __off64_t __length )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int truncate(char const   *__file ,
                                                  __off_t __length )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int getdtablesize(void) ;
extern  __attribute__((__nothrow__)) int getpagesize(void)  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) void sync(void) ;
extern long gethostid(void) ;
extern int fsync(int __fd ) ;
extern char *getpass(char const   *__prompt )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int chroot(char const   *__path )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int daemon(int __nochdir , int __noclose ) ;
extern  __attribute__((__nothrow__)) void setusershell(void) ;
extern  __attribute__((__nothrow__)) void endusershell(void) ;
extern  __attribute__((__nothrow__)) char *getusershell(void) ;
extern  __attribute__((__nothrow__)) int acct(char const   *__name ) ;
extern  __attribute__((__nothrow__)) int profil(unsigned short *__sample_buffer ,
                                                size_t __size ,
                                                size_t __offset ,
                                                unsigned int __scale )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int revoke(char const   *__file )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int vhangup(void) ;
extern  __attribute__((__nothrow__)) int setdomainname(char const   *__name ,
                                                       size_t __len )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int getdomainname(char *__name ,
                                                       size_t __len )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int sethostid(long __id ) ;
extern  __attribute__((__nothrow__)) int sethostname(char const   *__name ,
                                                     size_t __len )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int gethostname(char *__name ,
                                                     size_t __len )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int getopt(int ___argc ,
                                                char * const  *___argv ,
                                                char const   *__shortopts ) ;
extern int optopt ;
extern int opterr ;
extern int optind ;
extern char *optarg ;
extern  __attribute__((__nothrow__)) int setlogin(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern int getlogin_r(char *__name , size_t __name_len )  __attribute__((__nonnull__(1))) ;
extern char *getlogin(void) ;
typedef int __pid_t;
extern  __attribute__((__nothrow__)) int tcsetpgrp(int __fd , __pid_t __pgrp_id ) ;
extern  __attribute__((__nothrow__)) __pid_t tcgetpgrp(int __fd ) ;
extern  __attribute__((__nothrow__)) int rmdir(char const   *__path )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int unlinkat(int __fd ,
                                                  char const   *__name ,
                                                  int __flag )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int unlink(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) ssize_t readlinkat(int __fd ,
                                                        char const   * __restrict  __path ,
                                                        char * __restrict  __buf ,
                                                        size_t __len )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) int symlinkat(char const   *__from ,
                                                   int __tofd ,
                                                   char const   *__to )  __attribute__((__nonnull__(1,3))) ;
extern  __attribute__((__nothrow__)) ssize_t readlink(char const   * __restrict  __path ,
                                                      char * __restrict  __buf ,
                                                      size_t __len )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int symlink(char const   *__from ,
                                                 char const   *__to )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int linkat(int __fromfd ,
                                                char const   *__from ,
                                                int __tofd ,
                                                char const   *__to ,
                                                int __flags )  __attribute__((__nonnull__(2,4))) ;
extern  __attribute__((__nothrow__)) int link(char const   *__from ,
                                              char const   *__to )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int ttyslot(void) ;
extern  __attribute__((__nothrow__)) int isatty(int __fd ) ;
extern  __attribute__((__nothrow__)) int ttyname_r(int __fd , char *__buf ,
                                                   size_t __buflen )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *ttyname(int __fd ) ;
extern  __attribute__((__nothrow__)) __pid_t vfork(void) ;
extern  __attribute__((__nothrow__)) __pid_t fork(void) ;
extern  __attribute__((__nothrow__)) int setresgid(__gid_t __rgid ,
                                                   __gid_t __egid ,
                                                   __gid_t __sgid ) ;
extern  __attribute__((__nothrow__)) int setresuid(__uid_t __ruid ,
                                                   __uid_t __euid ,
                                                   __uid_t __suid ) ;
extern  __attribute__((__nothrow__)) int getresgid(__gid_t *__rgid ,
                                                   __gid_t *__egid ,
                                                   __gid_t *__sgid ) ;
extern  __attribute__((__nothrow__)) int getresuid(__uid_t *__ruid ,
                                                   __uid_t *__euid ,
                                                   __uid_t *__suid ) ;
extern  __attribute__((__nothrow__)) int setegid(__gid_t __gid ) ;
extern  __attribute__((__nothrow__)) int setregid(__gid_t __rgid ,
                                                  __gid_t __egid ) ;
extern  __attribute__((__nothrow__)) int setgid(__gid_t __gid ) ;
extern  __attribute__((__nothrow__)) int seteuid(__uid_t __uid ) ;
extern  __attribute__((__nothrow__)) int setreuid(__uid_t __ruid ,
                                                  __uid_t __euid ) ;
extern  __attribute__((__nothrow__)) int setuid(__uid_t __uid ) ;
extern  __attribute__((__nothrow__)) int group_member(__gid_t __gid ) ;
extern  __attribute__((__nothrow__)) int getgroups(int __size , __gid_t *__list ) ;
extern  __attribute__((__nothrow__)) __gid_t getegid(void) ;
extern  __attribute__((__nothrow__)) __gid_t getgid(void) ;
extern  __attribute__((__nothrow__)) __uid_t geteuid(void) ;
extern  __attribute__((__nothrow__)) __uid_t getuid(void) ;
extern  __attribute__((__nothrow__)) __pid_t getsid(__pid_t __pid ) ;
extern  __attribute__((__nothrow__)) __pid_t setsid(void) ;
extern  __attribute__((__nothrow__)) int setpgrp(void) ;
extern  __attribute__((__nothrow__)) int setpgid(__pid_t __pid , __pid_t __pgid ) ;
extern  __attribute__((__nothrow__)) __pid_t getpgid(__pid_t __pid ) ;
extern  __attribute__((__nothrow__)) __pid_t __getpgid(__pid_t __pid ) ;
extern  __attribute__((__nothrow__)) __pid_t getpgrp(void) ;
extern  __attribute__((__nothrow__)) __pid_t getppid(void) ;
extern  __attribute__((__nothrow__)) __pid_t getpid(void) ;
extern  __attribute__((__nothrow__)) size_t confstr(int __name , char *__buf ,
                                                    size_t __len ) ;
extern  __attribute__((__nothrow__)) long sysconf(int __name ) ;
extern  __attribute__((__nothrow__)) long fpathconf(int __fd , int __name ) ;
extern  __attribute__((__nothrow__)) long pathconf(char const   *__path ,
                                                   int __name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__noreturn__)) void _exit(int __status ) ;
extern  __attribute__((__nothrow__)) int nice(int __inc ) ;
extern  __attribute__((__nothrow__)) int execvpe(char const   *__file ,
                                                 char * const  *__argv ,
                                                 char * const  *__envp )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execlp(char const   *__file ,
                                                char const   *__arg  , ...)  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execvp(char const   *__file ,
                                                char * const  *__argv )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execl(char const   *__path ,
                                               char const   *__arg  , ...)  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execle(char const   *__path ,
                                                char const   *__arg  , ...)  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int execv(char const   *__path ,
                                               char * const  *__argv )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int fexecve(int __fd ,
                                                 char * const  *__argv ,
                                                 char * const  *__envp )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int execve(char const   *__path ,
                                                char * const  *__argv ,
                                                char * const  *__envp )  __attribute__((__nonnull__(1,2))) ;
extern char **environ ;
extern char **__environ ;
extern  __attribute__((__nothrow__)) int dup3(int __fd , int __fd2 ,
                                              int __flags ) ;
extern  __attribute__((__nothrow__)) int dup2(int __fd , int __fd2 ) ;
extern  __attribute__((__nothrow__)) int dup(int __fd ) ;
extern  __attribute__((__nothrow__)) char *getwd(char *__buf )  __attribute__((__nonnull__(1),
__deprecated__)) ;
extern  __attribute__((__nothrow__)) char *get_current_dir_name(void) ;
extern  __attribute__((__nothrow__)) char *getcwd(char *__buf , size_t __size ) ;
extern  __attribute__((__nothrow__)) int fchdir(int __fd ) ;
extern  __attribute__((__nothrow__)) int chdir(char const   *__path )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int fchownat(int __fd ,
                                                  char const   *__file ,
                                                  __uid_t __owner ,
                                                  __gid_t __group , int __flag )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int lchown(char const   *__file ,
                                                __uid_t __owner ,
                                                __gid_t __group )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int fchown(int __fd , __uid_t __owner ,
                                                __gid_t __group ) ;
extern  __attribute__((__nothrow__)) int chown(char const   *__file ,
                                               __uid_t __owner ,
                                               __gid_t __group )  __attribute__((__nonnull__(1))) ;
extern int pause(void) ;
typedef unsigned int __useconds_t;
extern int usleep(__useconds_t __useconds ) ;
extern  __attribute__((__nothrow__)) __useconds_t ualarm(__useconds_t __value ,
                                                         __useconds_t __interval ) ;
extern unsigned int sleep(unsigned int __seconds ) ;
extern  __attribute__((__nothrow__)) unsigned int alarm(unsigned int __seconds ) ;
extern  __attribute__((__nothrow__)) int pipe2(int *__pipedes , int __flags ) ;
extern  __attribute__((__nothrow__)) int pipe(int *__pipedes ) ;
extern ssize_t pwrite64(int __fd , void const   *__buf , size_t __n ,
                        __off64_t __offset ) ;
extern ssize_t pread64(int __fd , void *__buf , size_t __nbytes ,
                       __off64_t __offset ) ;
extern ssize_t pwrite(int __fd , void const   *__buf , size_t __n ,
                      __off_t __offset ) ;
extern ssize_t pread(int __fd , void *__buf , size_t __nbytes ,
                     __off_t __offset ) ;
extern ssize_t write(int __fd , void const   *__buf , size_t __n ) ;
extern ssize_t read(int __fd , void *__buf , size_t __nbytes ) ;
extern int close(int __fd ) ;
extern  __attribute__((__nothrow__)) __off64_t lseek64(int __fd ,
                                                       __off64_t __offset ,
                                                       int __whence ) ;
extern  __attribute__((__nothrow__)) __off_t lseek(int __fd , __off_t __offset ,
                                                   int __whence ) ;
extern  __attribute__((__nothrow__)) int faccessat(int __fd ,
                                                   char const   *__file ,
                                                   int __type , int __flag )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int eaccess(char const   *__name ,
                                                 int __type )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int euidaccess(char const   *__name ,
                                                    int __type )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int access(char const   *__name ,
                                                int __type )  __attribute__((__nonnull__(1))) ;
extern int zend_spprintf(char **message , int max_len , char *format  , ...) ;
extern void __attribute__((__visibility__("default")))  zend_exception_error(zval *exception ,
                                                                             int severity ) ;
extern void __attribute__((__visibility__("default")))  (*zend_throw_exception_hook)(zval *ex ) ;
extern zval __attribute__((__visibility__("default")))  *zend_throw_error_exception(zend_class_entry *exception_ce ,
                                                                                    char *message ,
                                                                                    long code ,
                                                                                    int severity ) ;
extern void __attribute__((__visibility__("default")))  zend_clear_exception(void) ;
extern void __attribute__((__visibility__("default")))  zend_throw_exception_object(zval *exception ) ;
extern zval __attribute__((__visibility__("default")))  *zend_throw_exception_ex(zend_class_entry *exception_ce ,
                                                                                 long code ,
                                                                                 char *format 
                                                                                 , ...) ;
extern zval __attribute__((__visibility__("default")))  *zend_throw_exception(zend_class_entry *exception_ce ,
                                                                              char *message ,
                                                                              long code ) ;
extern void __attribute__((__visibility__("default")))  zend_register_default_classes(void) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_get_error_exception(void) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_exception_get_default(void) ;
extern void zend_register_default_exception(void) ;
extern void zend_throw_exception_internal(zval *exception ) ;
extern void __attribute__((__visibility__("default")))  zend_exception_restore(void) ;
extern void __attribute__((__visibility__("default")))  zend_exception_save(void) ;
extern void __attribute__((__visibility__("default")))  zend_exception_set_previous(zval *exception ,
                                                                                    zval *add_previous ) ;
extern void __attribute__((__visibility__("default")))  zif_display_disabled_class(int ht ,
                                                                                   zval *return_value ,
                                                                                   zval **return_value_ptr ,
                                                                                   zval *this_ptr ,
                                                                                   int return_value_used ) ;
extern void __attribute__((__visibility__("default")))  zif_display_disabled_function(int ht ,
                                                                                      zval *return_value ,
                                                                                      zval **return_value_ptr ,
                                                                                      zval *this_ptr ,
                                                                                      int return_value_used ) ;
extern void __attribute__((__visibility__("default")))  zend_rebuild_symbol_table(void) ;
extern void __attribute__((__visibility__("default")))  zend_reset_all_cv(HashTable *symbol_table ) ;
extern int __attribute__((__visibility__("default")))  zend_delete_global_variable_ex(char const   *name ,
                                                                                      int name_len ,
                                                                                      ulong hash_value ) ;
extern int __attribute__((__visibility__("default")))  zend_delete_global_variable(char const   *name ,
                                                                                   int name_len ) ;
extern void __attribute__((__visibility__("default")))  zend_delete_variable(zend_execute_data *ex ,
                                                                             HashTable *ht ,
                                                                             char const   *name ,
                                                                             int name_len ,
                                                                             ulong hash_value ) ;
extern int __attribute__((__visibility__("default")))  zend_set_hash_symbol(zval *symbol ,
                                                                            char const   *name ,
                                                                            int name_length ,
                                                                            zend_bool is_ref ,
                                                                            int num_symbol_tables 
                                                                            , ...) ;
struct _zend_fcall_info_cache;
typedef struct _zend_fcall_info_cache zend_fcall_info_cache;
struct _zend_fcall_info;
typedef struct _zend_fcall_info zend_fcall_info;
extern int __attribute__((__visibility__("default")))  zend_call_function(zend_fcall_info *fci ,
                                                                          zend_fcall_info_cache *fci_cache ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_call(zend_fcall_info *fci ,
                                                                            zend_fcall_info_cache *fcc ,
                                                                            zval **retval ,
                                                                            zval *args ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_argn(zend_fcall_info *fci ,
                                                                            int argc 
                                                                            , ...) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_argv(zend_fcall_info *fci ,
                                                                            int argc ,
                                                                            va_list *argv ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_argp(zend_fcall_info *fci ,
                                                                            int argc ,
                                                                            zval ***argv ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_args(zend_fcall_info *fci ,
                                                                            zval *args ) ;
extern void __attribute__((__visibility__("default")))  zend_fcall_info_args_restore(zend_fcall_info *fci ,
                                                                                     int param_count ,
                                                                                     zval ***params ) ;
extern void __attribute__((__visibility__("default")))  zend_fcall_info_args_save(zend_fcall_info *fci ,
                                                                                  int *param_count ,
                                                                                  zval ****params ) ;
extern void __attribute__((__visibility__("default")))  zend_fcall_info_args_clear(zend_fcall_info *fci ,
                                                                                   int free_mem ) ;
extern int __attribute__((__visibility__("default")))  zend_fcall_info_init(zval *callable ,
                                                                            uint check_flags ,
                                                                            zend_fcall_info *fci ,
                                                                            zend_fcall_info_cache *fcc ,
                                                                            char **callable_name ,
                                                                            char **error ) ;
extern zend_fcall_info_cache const __attribute__((__visibility__("default")))  empty_fcall_info_cache ;
extern zend_fcall_info const __attribute__((__visibility__("default")))  empty_fcall_info ;
extern int __attribute__((__visibility__("default")))  call_user_function_ex(HashTable *function_table ,
                                                                             zval **object_pp ,
                                                                             zval *function_name ,
                                                                             zval **retval_ptr_ptr ,
                                                                             zend_uint param_count ,
                                                                             zval ***params ,
                                                                             int no_separation ,
                                                                             HashTable *symbol_table ) ;
extern int __attribute__((__visibility__("default")))  call_user_function(HashTable *function_table ,
                                                                          zval **object_pp ,
                                                                          zval *function_name ,
                                                                          zval *retval_ptr ,
                                                                          zend_uint param_count ,
                                                                          zval **params ) ;
extern int __attribute__((__visibility__("default")))  add_property_zval_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ,
                                                                            zval *value ) ;
extern int __attribute__((__visibility__("default")))  add_property_stringl_ex(zval *arg ,
                                                                               char const   *key ,
                                                                               uint key_len ,
                                                                               char const   *str ,
                                                                               uint length ,
                                                                               int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_property_string_ex(zval *arg ,
                                                                              char const   *key ,
                                                                              uint key_len ,
                                                                              char const   *str ,
                                                                              int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_property_double_ex(zval *arg ,
                                                                              char const   *key ,
                                                                              uint key_len ,
                                                                              double d ) ;
extern int __attribute__((__visibility__("default")))  add_property_resource_ex(zval *arg ,
                                                                                char const   *key ,
                                                                                uint key_len ,
                                                                                long r ) ;
extern int __attribute__((__visibility__("default")))  add_property_bool_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ,
                                                                            int b ) ;
extern int __attribute__((__visibility__("default")))  add_property_null_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ) ;
extern int __attribute__((__visibility__("default")))  add_property_long_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ,
                                                                            long l ) ;
extern int __attribute__((__visibility__("default")))  add_get_index_stringl(zval *arg ,
                                                                             ulong idx ,
                                                                             char const   *str ,
                                                                             uint length ,
                                                                             void **dest ,
                                                                             int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_get_index_string(zval *arg ,
                                                                            ulong idx ,
                                                                            char const   *str ,
                                                                            void **dest ,
                                                                            int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_get_index_double(zval *arg ,
                                                                            ulong idx ,
                                                                            double d ,
                                                                            void **dest ) ;
extern int __attribute__((__visibility__("default")))  add_get_index_long(zval *arg ,
                                                                          ulong idx ,
                                                                          long l ,
                                                                          void **dest ) ;
extern int __attribute__((__visibility__("default")))  add_get_assoc_stringl_ex(zval *arg ,
                                                                                char const   *key ,
                                                                                uint key_len ,
                                                                                char const   *str ,
                                                                                uint length ,
                                                                                void **dest ,
                                                                                int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_get_assoc_string_ex(zval *arg ,
                                                                               char const   *key ,
                                                                               uint key_len ,
                                                                               char const   *str ,
                                                                               void **dest ,
                                                                               int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_zval(zval *arg ,
                                                                           zval *value ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_stringl(zval *arg ,
                                                                              char const   *str ,
                                                                              uint length ,
                                                                              int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_string(zval *arg ,
                                                                             char const   *str ,
                                                                             int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_double(zval *arg ,
                                                                             double d ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_resource(zval *arg ,
                                                                               int r ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_bool(zval *arg ,
                                                                           int b ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_null(zval *arg ) ;
extern int __attribute__((__visibility__("default")))  add_next_index_long(zval *arg ,
                                                                           long n ) ;
extern int __attribute__((__visibility__("default")))  add_index_zval(zval *arg ,
                                                                      ulong index ,
                                                                      zval *value ) ;
extern int __attribute__((__visibility__("default")))  add_index_stringl(zval *arg ,
                                                                         ulong idx ,
                                                                         char const   *str ,
                                                                         uint length ,
                                                                         int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_index_string(zval *arg ,
                                                                        ulong idx ,
                                                                        char const   *str ,
                                                                        int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_index_double(zval *arg ,
                                                                        ulong idx ,
                                                                        double d ) ;
extern int __attribute__((__visibility__("default")))  add_index_resource(zval *arg ,
                                                                          ulong idx ,
                                                                          int r ) ;
extern int __attribute__((__visibility__("default")))  add_index_bool(zval *arg ,
                                                                      ulong idx ,
                                                                      int b ) ;
extern int __attribute__((__visibility__("default")))  add_index_null(zval *arg ,
                                                                      ulong idx ) ;
extern int __attribute__((__visibility__("default")))  add_index_long(zval *arg ,
                                                                      ulong idx ,
                                                                      long n ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_zval_ex(zval *arg ,
                                                                         char const   *key ,
                                                                         uint key_len ,
                                                                         zval *value ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_stringl_ex(zval *arg ,
                                                                            char const   *key ,
                                                                            uint key_len ,
                                                                            char *str ,
                                                                            uint length ,
                                                                            int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_string_ex(zval *arg ,
                                                                           char const   *key ,
                                                                           uint key_len ,
                                                                           char *str ,
                                                                           int duplicate ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_double_ex(zval *arg ,
                                                                           char const   *key ,
                                                                           uint key_len ,
                                                                           double d ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_resource_ex(zval *arg ,
                                                                             char const   *key ,
                                                                             uint key_len ,
                                                                             int r ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_bool_ex(zval *arg ,
                                                                         char const   *key ,
                                                                         uint key_len ,
                                                                         int b ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_null_ex(zval *arg ,
                                                                         char const   *key ,
                                                                         uint key_len ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_long_ex(zval *arg ,
                                                                         char const   *key ,
                                                                         uint key_len ,
                                                                         long n ) ;
extern int __attribute__((__visibility__("default")))  add_assoc_function(zval *arg ,
                                                                          char const   *key ,
                                                                          void (*function_ptr)(int ht ,
                                                                                               zval *return_value ,
                                                                                               zval **return_value_ptr ,
                                                                                               zval *this_ptr ,
                                                                                               int return_value_used ) ) ;
extern void __attribute__((__visibility__("default")))  zend_merge_properties(zval *obj ,
                                                                              HashTable *properties ,
                                                                              int destroy_ht ) ;
struct _zend_object;
typedef struct _zend_object zend_object;
extern void __attribute__((__visibility__("default")))  object_properties_init(zend_object *object ,
                                                                               zend_class_entry *class_type ) ;
extern int __attribute__((__visibility__("default")))  _object_and_properties_init(zval *arg ,
                                                                                   zend_class_entry *ce ,
                                                                                   HashTable *properties ) ;
extern int __attribute__((__visibility__("default")))  _object_init_ex(zval *arg ,
                                                                       zend_class_entry *ce ) ;
extern int __attribute__((__visibility__("default")))  _object_init(zval *arg ) ;
extern int __attribute__((__visibility__("default")))  _array_init(zval *arg ,
                                                                   uint size ) ;
extern int __attribute__((__visibility__("default")))  zend_get_object_classname(zval const   *object ,
                                                                                 char const   **class_name ,
                                                                                 zend_uint *class_name_len ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_get_class_entry(zval const   *zobject ) ;
extern zval __attribute__((__visibility__("default")))  *zend_read_static_property(zend_class_entry *scope ,
                                                                                   char const   *name ,
                                                                                   int name_length ,
                                                                                   zend_bool silent ) ;
extern zval __attribute__((__visibility__("default")))  *zend_read_property(zend_class_entry *scope ,
                                                                            zval *object ,
                                                                            char const   *name ,
                                                                            int name_length ,
                                                                            zend_bool silent ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_stringl(zend_class_entry *scope ,
                                                                                           char const   *name ,
                                                                                           int name_length ,
                                                                                           char const   *value ,
                                                                                           int value_length ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_string(zend_class_entry *scope ,
                                                                                          char const   *name ,
                                                                                          int name_length ,
                                                                                          char const   *value ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_double(zend_class_entry *scope ,
                                                                                          char const   *name ,
                                                                                          int name_length ,
                                                                                          double value ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_long(zend_class_entry *scope ,
                                                                                        char const   *name ,
                                                                                        int name_length ,
                                                                                        long value ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_bool(zend_class_entry *scope ,
                                                                                        char const   *name ,
                                                                                        int name_length ,
                                                                                        long value ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property_null(zend_class_entry *scope ,
                                                                                        char const   *name ,
                                                                                        int name_length ) ;
extern int __attribute__((__visibility__("default")))  zend_update_static_property(zend_class_entry *scope ,
                                                                                   char const   *name ,
                                                                                   int name_length ,
                                                                                   zval *value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_stringl(zend_class_entry *scope ,
                                                                                     zval *object ,
                                                                                     char const   *name ,
                                                                                     int name_length ,
                                                                                     char const   *value ,
                                                                                     int value_length ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_string(zend_class_entry *scope ,
                                                                                    zval *object ,
                                                                                    char const   *name ,
                                                                                    int name_length ,
                                                                                    char const   *value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_double(zend_class_entry *scope ,
                                                                                    zval *object ,
                                                                                    char const   *name ,
                                                                                    int name_length ,
                                                                                    double value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_long(zend_class_entry *scope ,
                                                                                  zval *object ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  long value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_bool(zend_class_entry *scope ,
                                                                                  zval *object ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  long value ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property_null(zend_class_entry *scope ,
                                                                                  zval *object ,
                                                                                  char const   *name ,
                                                                                  int name_length ) ;
extern void __attribute__((__visibility__("default")))  zend_update_property(zend_class_entry *scope ,
                                                                             zval *object ,
                                                                             char const   *name ,
                                                                             int name_length ,
                                                                             zval *value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_string(zend_class_entry *ce ,
                                                                                          char const   *name ,
                                                                                          size_t name_length ,
                                                                                          char const   *value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_stringl(zend_class_entry *ce ,
                                                                                           char const   *name ,
                                                                                           size_t name_length ,
                                                                                           char const   *value ,
                                                                                           size_t value_length ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_double(zend_class_entry *ce ,
                                                                                          char const   *name ,
                                                                                          size_t name_length ,
                                                                                          double value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_bool(zend_class_entry *ce ,
                                                                                        char const   *name ,
                                                                                        size_t name_length ,
                                                                                        zend_bool value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_long(zend_class_entry *ce ,
                                                                                        char const   *name ,
                                                                                        size_t name_length ,
                                                                                        long value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant_null(zend_class_entry *ce ,
                                                                                        char const   *name ,
                                                                                        size_t name_length ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_class_constant(zend_class_entry *ce ,
                                                                                   char const   *name ,
                                                                                   size_t name_length ,
                                                                                   zval *value ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_stringl(zend_class_entry *ce ,
                                                                                     char const   *name ,
                                                                                     int name_length ,
                                                                                     char const   *value ,
                                                                                     int value_len ,
                                                                                     int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_string(zend_class_entry *ce ,
                                                                                    char const   *name ,
                                                                                    int name_length ,
                                                                                    char const   *value ,
                                                                                    int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_double(zend_class_entry *ce ,
                                                                                    char const   *name ,
                                                                                    int name_length ,
                                                                                    double value ,
                                                                                    int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_long(zend_class_entry *ce ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  long value ,
                                                                                  int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_bool(zend_class_entry *ce ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  long value ,
                                                                                  int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property_null(zend_class_entry *ce ,
                                                                                  char const   *name ,
                                                                                  int name_length ,
                                                                                  int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_declare_property(zend_class_entry *ce ,
                                                                             char const   *name ,
                                                                             int name_length ,
                                                                             zval *property ,
                                                                             int access_type ) ;
extern int __attribute__((__visibility__("default")))  zend_get_module_started(char const   *module_name ) ;
extern char const __attribute__((__visibility__("default")))  *zend_get_module_version(char const   *module_name ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_make_callable(zval *callable ,
                                                                                char **callable_name ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_callable(zval *callable ,
                                                                              uint check_flags ,
                                                                              char **callable_name ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_callable_ex(zval *callable ,
                                                                                 zval *object_ptr ,
                                                                                 uint check_flags ,
                                                                                 char **callable_name ,
                                                                                 int *callable_name_len ,
                                                                                 zend_fcall_info_cache *fcc ,
                                                                                 char **error ) ;
extern void __attribute__((__visibility__("default")))  zend_wrong_param_count(void) ;
extern int __attribute__((__visibility__("default")))  zend_disable_class(char *class_name ,
                                                                          uint class_name_length ) ;
extern int __attribute__((__visibility__("default")))  zend_disable_function(char *function_name ,
                                                                             uint function_name_length ) ;
extern int __attribute__((__visibility__("default")))  zend_register_class_alias_ex(char const   *name ,
                                                                                    int name_len ,
                                                                                    zend_class_entry *ce ) ;
extern void __attribute__((__visibility__("default")))  zend_class_implements(zend_class_entry *class_entry ,
                                                                              int num_interfaces 
                                                                              , ...) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_register_internal_interface(zend_class_entry *orig_class_entry ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_register_internal_class_ex(zend_class_entry *class_entry ,
                                                                                                     zend_class_entry *parent_ce ,
                                                                                                     char *parent_name ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_register_internal_class(zend_class_entry *class_entry ) ;
extern void __attribute__((__visibility__("default")))  zend_destroy_modules(void) ;
extern void __attribute__((__visibility__("default")))  zend_collect_module_handlers(void) ;
extern int __attribute__((__visibility__("default")))  zend_startup_modules(void) ;
typedef struct _zend_module_entry zend_module_entry;
extern int __attribute__((__visibility__("default")))  zend_startup_module_ex(zend_module_entry *module ) ;
extern zend_module_entry __attribute__((__visibility__("default")))  *zend_register_module_ex(zend_module_entry *module ) ;
extern zend_module_entry __attribute__((__visibility__("default")))  *zend_register_internal_module(zend_module_entry *module_entry ) ;
extern int __attribute__((__visibility__("default")))  zend_startup_module(zend_module_entry *module_entry ) ;
typedef struct _zend_function_entry zend_function_entry;
extern void __attribute__((__visibility__("default")))  zend_unregister_functions(zend_function_entry const   *functions ,
                                                                                  int count ,
                                                                                  HashTable *function_table ) ;
extern int __attribute__((__visibility__("default")))  zend_register_functions(zend_class_entry *scope ,
                                                                               zend_function_entry const   *functions ,
                                                                               HashTable *function_table ,
                                                                               int type ) ;
extern int __attribute__((__visibility__("default")))  zend_parse_method_parameters_ex(int flags ,
                                                                                       int num_args ,
                                                                                       zval *this_ptr ,
                                                                                       char const   *type_spec 
                                                                                       , ...) ;
extern int __attribute__((__visibility__("default")))  zend_parse_method_parameters(int num_args ,
                                                                                    zval *this_ptr ,
                                                                                    char const   *type_spec 
                                                                                    , ...) ;
extern char __attribute__((__visibility__("default")))  *zend_zval_type_name(zval const   *arg ) ;
extern int __attribute__((__visibility__("default")))  zend_parse_parameters_ex(int flags ,
                                                                                int num_args ,
                                                                                char const   *type_spec 
                                                                                , ...) ;
extern int __attribute__((__visibility__("default")))  zend_parse_parameters(int num_args ,
                                                                             char const   *type_spec 
                                                                             , ...) ;
extern int __attribute__((__visibility__("default")))  zend_copy_parameters_array(int param_count ,
                                                                                  zval *argument_array ) ;
extern int __attribute__((__visibility__("default")))  _zend_get_parameters_array_ex(int param_count ,
                                                                                     zval ***argument_array ) ;
extern int __attribute__((__visibility__("default"),
__deprecated__))  zend_get_parameters_ex(int param_count  , ...) ;
extern int __attribute__((__visibility__("default")))  _zend_get_parameters_array(int ht ,
                                                                                  int param_count ,
                                                                                  zval **argument_array ) ;
extern int __attribute__((__visibility__("default")))  zend_get_parameters(int ht ,
                                                                           int param_count 
                                                                           , ...) ;
extern int zend_next_free_module(void) ;
extern int __attribute__((__visibility__("default")))  zend_do_fcall(zend_execute_data *execute_data ) ;
struct _zend_free_op;
typedef struct _zend_free_op zend_free_op;
union _temp_variable;
extern zval __attribute__((__visibility__("default")))  **zend_get_zval_ptr_ptr(int op_type ,
                                                                                znode_op const   *node ,
                                                                                temp_variable const   *Ts ,
                                                                                zend_free_op *should_free ,
                                                                                int type ) ;
extern zval __attribute__((__visibility__("default")))  *zend_get_zval_ptr(int op_type ,
                                                                           znode_op const   *node ,
                                                                           temp_variable const   *Ts ,
                                                                           zend_free_op *should_free ,
                                                                           int type ) ;
typedef int (*user_opcode_handler_t)(zend_execute_data *execute_data );
extern user_opcode_handler_t __attribute__((__visibility__("default")))  zend_get_user_opcode_handler(zend_uchar opcode ) ;
extern int __attribute__((__visibility__("default")))  zend_set_user_opcode_handler(zend_uchar opcode ,
                                                                                    int (*handler)(zend_execute_data *execute_data ) ) ;
extern zval __attribute__((__visibility__("default")))  **zend_get_compiled_variable_value(zend_execute_data const   *execute_data_ptr ,
                                                                                           zend_uint var ) ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_fetch_class_by_name(char const   *class_name ,
                                                                                              uint class_name_len ,
                                                                                              zend_literal const   *key ,
                                                                                              int fetch_type ) ;
extern void __attribute__((__visibility__("default")))  zend_timeout(int dummy ) ;
extern void __attribute__((__visibility__("default")))  zend_unset_timeout(void) ;
extern void __attribute__((__visibility__("default")))  zend_set_timeout(long seconds ,
                                                                         int reset_signals ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_is_executing(void) ;
extern uint __attribute__((__visibility__("default")))  zend_get_executed_lineno(void) ;
extern char const __attribute__((__visibility__("default")))  *zend_get_executed_filename(void) ;
extern char const __attribute__((__visibility__("default")))  *get_active_function_name(void) ;
extern char const __attribute__((__visibility__("default")))  *get_active_class_name(char const   **space ) ;
extern void execute_new_code(void) ;
struct _zend_ptr_stack;
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_3_pop)(zend_ptr_stack *stack ,
                                                                                void **a ,
                                                                                void **b ,
                                                                                void **c ) ;
__inline static void ( __attribute__((__always_inline__)) zend_arg_types_stack_3_pop)(zend_ptr_stack *stack ,
                                                                                      zend_class_entry **called_scope ,
                                                                                      zval **object ,
                                                                                      zend_function **fbc ) 
{ void *a ;
  void *b ;
  void *c ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1304\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_ptr_stack_3_pop(stack, & a, & b, & c);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1305\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *called_scope = (zend_class_entry *)a;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1306\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *object = (zval *)b;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1307\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *fbc = (zend_function *)c;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1308\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_arg_types_stack_3_pop)(zend_ptr_stack *stack ,
                                                                                      zend_class_entry **called_scope ,
                                                                                      zval **object ,
                                                                                      zend_function **fbc ) ;
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_2_pop)(zend_ptr_stack *stack ,
                                                                                void **a ,
                                                                                void **b ) ;
__inline static void ( __attribute__((__always_inline__)) zend_arg_types_stack_2_pop)(zend_ptr_stack *stack ,
                                                                                      zval **object ,
                                                                                      zend_function **fbc ) 
{ void *a ;
  void *b ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1300\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_ptr_stack_2_pop(stack, & a, & b);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1301\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *object = (zval *)a;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1302\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *fbc = (zend_function *)b;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1303\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_arg_types_stack_2_pop)(zend_ptr_stack *stack ,
                                                                                      zval **object ,
                                                                                      zend_function **fbc ) ;
struct _zend_function_state {
   zend_function *function ;
   void **arguments ;
};
typedef struct _zend_function_state zend_function_state;
struct _zend_execute_data {
   struct _zend_op *opline ;
   zend_function_state function_state ;
   zend_function *fbc ;
   zend_class_entry *called_scope ;
   zend_op_array *op_array ;
   zval *object ;
   union _temp_variable *Ts ;
   zval ***CVs ;
   HashTable *symbol_table ;
   struct _zend_execute_data *prev_execute_data ;
   zval *old_error_reporting ;
   zend_bool nested ;
   zval **original_return_value ;
   zend_class_entry *current_scope ;
   zend_class_entry *current_called_scope ;
   zval *current_this ;
   zval *current_object ;
};
__inline static zval **( __attribute__((__always_inline__)) zend_vm_stack_get_arg)(int requested_arg ) 
{ void **p ;
  int arg_count ;
  long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1295\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p = ((executor_globals.current_execute_data)->prev_execute_data)->function_state.arguments;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1296\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arg_count = (int )((unsigned long )*p);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1297\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_expect((long )(requested_arg > arg_count), 0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1298\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1293\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((zval **)((void *)0));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1294\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1299\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((((zval **)p - arg_count) + requested_arg) - 1);
}
}
__inline static zval **( __attribute__((__always_inline__)) zend_vm_stack_get_arg)(int requested_arg ) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_free_int)(void *ptr ) ;
__inline static void ( __attribute__((__always_inline__)) i_zval_ptr_dtor)(zval *zval_ptr ) ;
struct _zend_vm_stack {
   void **top ;
   void **end ;
   zend_vm_stack prev ;
};
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_clear_multiple)(void) 
{ void **p ;
  int delete_count ;
  zval *q ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1288\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p = (executor_globals.argument_stack)->top - 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1289\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  delete_count = (int )((unsigned long )*p);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1290\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1282\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    delete_count --;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1283\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (delete_count >= 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1281\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1284\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p --;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1285\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    q = *((zval **)p);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1286\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *p = (void *)0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1287\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    i_zval_ptr_dtor(q);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1291\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_vm_stack_free_int((void *)p);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1292\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_clear_multiple)(void) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_extend)(int count ) ;
__inline static void **( __attribute__((__always_inline__)) zend_vm_stack_push_args)(int count ) 
{ zend_vm_stack p ;
  void *data ;
  zend_vm_stack r ;
  long tmp ;
  int tmp___0 ;
  void **tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  void **tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1275\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = __builtin_expect((long )((executor_globals.argument_stack)->top - (void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U)) < count),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1276\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___2) {
    goto _L;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1273\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((unsigned int )(executor_globals.argument_stack)->top == (unsigned int )(executor_globals.argument_stack)->end),
                               0L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1274\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1264\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _L: /* CIL Label */ 
      p = executor_globals.argument_stack;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1265\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_vm_stack_extend(count + 1);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1266\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (executor_globals.argument_stack)->top += count;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1267\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *((executor_globals.argument_stack)->top) = (void *)((unsigned long )count);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1268\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1256\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = count;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1257\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        count --;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1258\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___0 > 0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1250\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1259\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        (p->top) --;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1260\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        data = *(p->top);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1261\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = __builtin_expect((long )((unsigned int )p->top == (unsigned int )((void **)((char *)p + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U)))),
                               0L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1262\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1251\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          r = p;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1252\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          (executor_globals.argument_stack)->prev = p->prev;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1253\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          p = p->prev;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1254\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _efree((void *)r);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1255\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1263\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *((void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U)) + count) = data;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1269\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = (executor_globals.argument_stack)->top;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1270\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ((executor_globals.argument_stack)->top) ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1271\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (tmp___1);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1272\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1277\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *((executor_globals.argument_stack)->top) = (void *)((unsigned long )count);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1278\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = (executor_globals.argument_stack)->top;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1279\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((executor_globals.argument_stack)->top) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1280\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___4);
}
}
__inline static void **( __attribute__((__always_inline__)) zend_vm_stack_push_args)(int count ) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_free)(void *ptr ) 
{ zend_vm_stack p ;
  long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1247\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_expect((long )((unsigned int )((void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U))) == (unsigned int )((void **)ptr)),
                         0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1248\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1240\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p = executor_globals.argument_stack;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1241\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    executor_globals.argument_stack = p->prev;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1242\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)p);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1246\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (4U > sizeof(void *)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1243\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ptr = (void *)((char *)ptr - (((sizeof(void *) + 4U) - 1U) & 4294967292U));
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1244\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (executor_globals.argument_stack)->top = *((void ***)ptr);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1245\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (executor_globals.argument_stack)->top = (void **)ptr;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1249\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_free)(void *ptr ) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_free_int)(void *ptr ) 
{ zend_vm_stack p ;
  long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1237\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_expect((long )((unsigned int )((void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U))) == (unsigned int )((void **)ptr)),
                         0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1238\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1233\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p = executor_globals.argument_stack;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1234\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    executor_globals.argument_stack = p->prev;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1235\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)p);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1236\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (executor_globals.argument_stack)->top = (void **)ptr;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1239\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void *( __attribute__((__always_inline__)) zend_vm_stack_alloc)(size_t size ) 
{ void *ret ;
  int extra ;
  void **old_top ;
  long tmp ;
  long tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1228\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  size = (size + (sizeof(void *) - 1U)) / sizeof(void *);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1229\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (4U > sizeof(void *)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1220\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    extra = (int )((4UL - ((unsigned long )(executor_globals.argument_stack)->top & 3UL)) / (unsigned long )sizeof(void *));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1221\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = __builtin_expect((long )((unsigned long )((size + (size_t )extra) + (((sizeof(void *) + 4U) - 1U) & 4294967292U) / sizeof(void *)) > (unsigned long )((executor_globals.argument_stack)->end - (executor_globals.argument_stack)->top)),
                           0L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1222\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1215\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_vm_stack_extend((int )size);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1216\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      old_top = (executor_globals.argument_stack)->top;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1217\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (executor_globals.argument_stack)->top += extra;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1218\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *((executor_globals.argument_stack)->top) = (void *)old_top;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1219\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (executor_globals.argument_stack)->top += (((sizeof(void *) + 4U) - 1U) & 4294967292U) / sizeof(void *);
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1227\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1225\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = __builtin_expect((long )((int )size > (executor_globals.argument_stack)->end - (executor_globals.argument_stack)->top),
                                 0L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1226\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1223\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_vm_stack_extend((int )size);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1224\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1230\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ret = (void *)(executor_globals.argument_stack)->top;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1231\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (executor_globals.argument_stack)->top += size;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1232\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (ret);
}
}
__inline static void *( __attribute__((__always_inline__)) zend_vm_stack_alloc)(size_t size ) ;
__inline static void *( __attribute__((__always_inline__)) zend_vm_stack_pop)(void) 
{ void *el ;
  zend_vm_stack p ;
  long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1210\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((executor_globals.argument_stack)->top) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1211\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  el = *((executor_globals.argument_stack)->top);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1212\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_expect((long )((unsigned int )(executor_globals.argument_stack)->top == (unsigned int )((void **)((char *)executor_globals.argument_stack + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U)))),
                         0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1213\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1206\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p = executor_globals.argument_stack;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1207\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    executor_globals.argument_stack = p->prev;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1208\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)p);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1209\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1214\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (el);
}
}
__inline static void *( __attribute__((__always_inline__)) zend_vm_stack_pop)(void) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_push_nocheck)(void *ptr ) 
{ void **tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1202\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = (executor_globals.argument_stack)->top;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1203\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((executor_globals.argument_stack)->top) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1204\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp = ptr;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1205\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_push_nocheck)(void *ptr ) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_push)(void *ptr ) 
{ long tmp ;
  void **tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1197\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1195\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = __builtin_expect((long )(1 > (executor_globals.argument_stack)->end - (executor_globals.argument_stack)->top),
                           0L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1196\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1193\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_vm_stack_extend(1);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1194\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1198\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = (executor_globals.argument_stack)->top;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1199\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ((executor_globals.argument_stack)->top) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1200\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___0 = ptr;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1201\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_push)(void *ptr ) ;
__inline static void **( __attribute__((__always_inline__)) zend_vm_stack_top)(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1192\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((executor_globals.argument_stack)->top);
}
}
__inline static void **( __attribute__((__always_inline__)) zend_vm_stack_top)(void) ;
__inline static zend_vm_stack ( __attribute__((__always_inline__)) zend_vm_stack_new_page)(int count ) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_extend)(int count ) 
{ zend_vm_stack p ;
  int tmp ;
  zend_vm_stack tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1186\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (count >= 16368) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1184\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = count;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1185\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = 16368;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1187\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = zend_vm_stack_new_page(tmp);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1188\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1189\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p->prev = executor_globals.argument_stack;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1190\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  executor_globals.argument_stack = p;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1191\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_destroy)(void) 
{ zend_vm_stack stack ;
  zend_vm_stack p ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1181\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  stack = executor_globals.argument_stack;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1182\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1177\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )stack != (unsigned int )((void *)0)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1176\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1178\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p = stack->prev;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1179\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _efree((void *)stack);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1180\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack = p;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1183\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_destroy)(void) ;
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_init)(void) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1174\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  executor_globals.argument_stack = zend_vm_stack_new_page(16368);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1175\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_vm_stack_init)(void) ;
__inline static zend_vm_stack ( __attribute__((__always_inline__)) zend_vm_stack_new_page)(int count ) 
{ zend_vm_stack page ;
  void __attribute__((__visibility__("default")))  *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1168\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _emalloc((((sizeof(*page) + 4U) - 1U) & 4294967292U) + sizeof(void *) * (unsigned int )count);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1169\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  page = (struct _zend_vm_stack *)tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1170\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  page->top = (void **)((char *)page + (((sizeof(struct _zend_vm_stack ) + 4U) - 1U) & 4294967292U));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1171\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  page->end = page->top + count;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1172\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  page->prev = (struct _zend_vm_stack *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1173\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (page);
}
}
extern int __attribute__((__visibility__("default")))  zval_update_constant_no_inline_change(zval **pp ,
                                                                                             void *arg ) ;
extern int __attribute__((__visibility__("default")))  zval_update_constant_inline_change(zval **pp ,
                                                                                          void *arg ) ;
extern int __attribute__((__visibility__("default")))  zval_update_constant(zval **pp ,
                                                                            void *arg ) ;
extern void __attribute__((__visibility__("default")))  convert_to_boolean(zval *op ) ;
struct _zend_object_handlers {
   void (*add_ref)(zval *object ) ;
   void (*del_ref)(zval *object ) ;
   zend_object_value (*clone_obj)(zval *object ) ;
   zval *(*read_property)(zval *object , zval *member , int type ,
                          struct _zend_literal  const  *key ) ;
   void (*write_property)(zval *object , zval *member , zval *value ,
                          struct _zend_literal  const  *key ) ;
   zval *(*read_dimension)(zval *object , zval *offset , int type ) ;
   void (*write_dimension)(zval *object , zval *offset , zval *value ) ;
   zval **(*get_property_ptr_ptr)(zval *object , zval *member ,
                                  struct _zend_literal  const  *key ) ;
   zval *(*get)(zval *object ) ;
   void (*set)(zval **object , zval *value ) ;
   int (*has_property)(zval *object , zval *member , int has_set_exists ,
                       struct _zend_literal  const  *key ) ;
   void (*unset_property)(zval *object , zval *member ,
                          struct _zend_literal  const  *key ) ;
   int (*has_dimension)(zval *object , zval *member , int check_empty ) ;
   void (*unset_dimension)(zval *object , zval *offset ) ;
   HashTable *(*get_properties)(zval *object ) ;
   union _zend_function *(*get_method)(zval **object_ptr , char *method ,
                                       int method_len ,
                                       struct _zend_literal  const  *key ) ;
   int (*call_method)(char const   *method , int ht , zval *return_value ,
                      zval **return_value_ptr , zval *this_ptr ,
                      int return_value_used ) ;
   union _zend_function *(*get_constructor)(zval *object ) ;
   zend_class_entry *(*get_class_entry)(zval const   *object ) ;
   int (*get_class_name)(zval const   *object , char const   **class_name ,
                         zend_uint *class_name_len , int parent ) ;
   int (*compare_objects)(zval *object1 , zval *object2 ) ;
   int (*cast_object)(zval *readobj , zval *retval , int type ) ;
   int (*count_elements)(zval *object , long *count ) ;
   HashTable *(*get_debug_info)(zval *object , int *is_temp ) ;
   int (*get_closure)(zval *obj , zend_class_entry **ce_ptr ,
                      union _zend_function **fptr_ptr , zval **zobj_ptr ) ;
   HashTable *(*get_gc)(zval *object , zval ***table , int *n ) ;
};
__inline static int ( __attribute__((__always_inline__)) i_zend_is_true)(zval *op ) 
{ int result ;
  int __attribute__((__visibility__("default")))  tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  zval tmp___1 ;
  int tmp___2 ;
  zval *tmp___3 ;
  zval *tmp___4 ;

  {
  switch ((int )op->type) {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1158\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 0: 
  result = 0;
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1159\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 1: 
  case 3: 
  case 7: 
  if (op->value.lval) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1129\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1130\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 0;
  }
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1160\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 2: 
  if (op->value.dval) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1131\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1132\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 0;
  }
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1161\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 6: 
  if (op->value.str.len == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1133\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1138\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (op->value.str.len == 1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1136\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*(op->value.str.val + 0) == 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1134\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result = 0;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1135\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result = 1;
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1137\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result = 1;
    }
  }
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1162\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 4: 
  tmp___0 = zend_hash_num_elements((HashTable const   *)op->value.ht);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1163\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1139\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 1;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1140\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    result = 0;
  }
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1164\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 5: 
  if ((int )op->type == 5) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1156\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )(op->value.obj.handlers)->get_class_entry != (unsigned int )((void *)0)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1154\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((op->value.obj.handlers)->cast_object) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1143\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___2 = (*((op->value.obj.handlers)->cast_object))(op, & tmp___1, 3);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1144\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___2 == 0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1141\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result = (int )tmp___1.value.lval;
          break;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1142\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1153\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((op->value.obj.handlers)->get) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1149\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___4 = (*((op->value.obj.handlers)->get))(op);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1150\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___3 = tmp___4;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1151\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )tmp___3->type != 5) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1145\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            convert_to_boolean(tmp___3);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1146\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            result = (int )tmp___3->value.lval;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1147\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _zval_ptr_dtor(& tmp___3);
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "1148\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1152\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1155\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1157\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1165\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  result = 1;
  break;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1166\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  default: 
  result = 0;
  break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1167\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (result);
}
}
__inline static int ( __attribute__((__always_inline__)) i_zend_is_true)(zval *op ) ;
__inline static void ( __attribute__((__always_inline__)) gc_zval_check_possible_root)(zval *z ) ;
__inline static void ( __attribute__((__always_inline__)) i_zval_ptr_dtor)(zval *zval_ptr ) 
{ zend_uint tmp ;
  zend_uint tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1126\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = zval_delref_p(zval_ptr);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1127\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1116\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = zval_refcount_p(zval_ptr);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1117\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp == 1U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1114\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zval_unset_isref_p(zval_ptr);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1115\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1118\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    gc_zval_check_possible_root(zval_ptr);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1125\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )zval_ptr != (unsigned int )(& executor_globals.uninitialized_zval)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1121\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((gc_root_buffer *)((unsigned long )((zval_gc_info *)zval_ptr)->u.buffered & 4294967292UL)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1119\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        gc_remove_zval_from_buffer(zval_ptr);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1120\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1122\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _zval_dtor(zval_ptr);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1123\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)zval_ptr);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1124\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1128\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern int __attribute__((__visibility__("default")))  zend_verify_arg_error(int error_type ,
                                                                             zend_function const   *zf ,
                                                                             zend_uint arg_num ,
                                                                             char const   *need_msg ,
                                                                             char const   *need_kind ,
                                                                             char const   *given_msg ,
                                                                             char const   *given_kind ) ;
extern char __attribute__((__visibility__("default")))  *zend_verify_arg_class_kind(zend_arg_info const   *cur_arg_info ,
                                                                                    ulong fetch_type ,
                                                                                    char const   **class_name ,
                                                                                    zend_class_entry **pce ) ;
extern int __attribute__((__visibility__("default")))  zend_eval_stringl_ex(char *str ,
                                                                            int str_len ,
                                                                            zval *retval_ptr ,
                                                                            char *string_name ,
                                                                            int handle_exceptions ) ;
extern int __attribute__((__visibility__("default")))  zend_eval_string_ex(char *str ,
                                                                           zval *retval_ptr ,
                                                                           char *string_name ,
                                                                           int handle_exceptions ) ;
extern int __attribute__((__visibility__("default")))  zend_eval_stringl(char *str ,
                                                                         int str_len ,
                                                                         zval *retval_ptr ,
                                                                         char *string_name ) ;
extern int __attribute__((__visibility__("default")))  zend_eval_string(char *str ,
                                                                        zval *retval_ptr ,
                                                                        char *string_name ) ;
extern int __attribute__((__visibility__("default")))  zend_lookup_class_ex(char const   *name ,
                                                                            int name_length ,
                                                                            zend_literal const   *key ,
                                                                            int use_autoload ,
                                                                            zend_class_entry ***ce ) ;
__inline static void ( __attribute__((__always_inline__)) safe_free_zval_ptr_rel)(zval *p ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1112\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )p != (unsigned int )executor_globals.uninitialized_zval_ptr) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1110\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1108\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((gc_root_buffer *)((unsigned long )((zval_gc_info *)p)->u.buffered & 4294967292UL)) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1106\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        gc_remove_zval_from_buffer(p);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1107\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1109\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      _efree((void *)p);
      break;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1111\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1113\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) safe_free_zval_ptr_rel)(zval *p ) ;
extern int __attribute__((__visibility__("default")))  zend_is_true(zval *op ) ;
extern void __attribute__((__visibility__("default")))  execute_internal(zend_execute_data *execute_data_ptr ,
                                                                         int return_value_used ) ;
extern void __attribute__((__visibility__("default")))  execute(zend_op_array *op_array ) ;
extern void shutdown_destructors(void) ;
extern void shutdown_executor(void) ;
extern void init_executor(void) ;
extern void __attribute__((__visibility__("default")))  (*zend_execute_internal)(zend_execute_data *execute_data_ptr ,
                                                                                 int return_value_used ) ;
extern void __attribute__((__visibility__("default")))  (*zend_execute)(zend_op_array *op_array ) ;
extern int __attribute__((__visibility__("default")))  le_index_ptr ;
extern int __attribute__((__visibility__("default")))  zend_fetch_list_dtor_id(char *type_name ) ;
extern char const __attribute__((__visibility__("default")))  *zend_rsrc_list_get_rsrc_type(int resource ) ;
extern void __attribute__((__visibility__("default")))  *zend_fetch_resource(zval **passed_id ,
                                                                             int default_id ,
                                                                             char const   *resource_type_name ,
                                                                             int *found_resource_type ,
                                                                             int num_resource_types 
                                                                             , ...) ;
extern int __attribute__((__visibility__("default")))  zend_register_resource(zval *rsrc_result ,
                                                                              void *rsrc_pointer ,
                                                                              int rsrc_type ) ;
extern void __attribute__((__visibility__("default")))  *_zend_list_find(int id ,
                                                                         int *type ) ;
extern int __attribute__((__visibility__("default")))  _zend_list_delete(int id ) ;
extern int __attribute__((__visibility__("default")))  _zend_list_addref(int id ) ;
extern int __attribute__((__visibility__("default")))  zend_list_insert(void *ptr ,
                                                                        int type ) ;
extern void zend_destroy_rsrc_list_dtors(void) ;
extern int zend_init_rsrc_list_dtors(void) ;
extern void zend_destroy_rsrc_list(HashTable *ht ) ;
extern int zend_init_rsrc_plist(void) ;
extern void zend_clean_module_rsrc_dtors(int module_number ) ;
extern void plist_entry_destructor(void *ptr ) ;
extern void list_entry_destructor(void *ptr ) ;
struct _zend_rsrc_list_entry;
typedef struct _zend_rsrc_list_entry zend_rsrc_list_entry;
extern int __attribute__((__visibility__("default")))  zend_register_list_destructors_ex(void (*ld)(zend_rsrc_list_entry *rsrc ) ,
                                                                                         void (*pld)(zend_rsrc_list_entry *rsrc ) ,
                                                                                         char const   *type_name ,
                                                                                         int module_number ) ;
extern int __attribute__((__visibility__("default")))  zend_register_list_destructors(void (*ld)(void * ) ,
                                                                                      void (*pld)(void * ) ,
                                                                                      int module_number ) ;
extern zend_constant *zend_quick_get_constant(zend_literal const   *key ,
                                              ulong flags ) ;
extern void copy_zend_constant(zend_constant *c ) ;
extern void zend_copy_constants(HashTable *target , HashTable *sourc ) ;
extern int __attribute__((__visibility__("default")))  zend_register_constant(zend_constant *c ) ;
extern void __attribute__((__visibility__("default")))  zend_register_stringl_constant(char const   *name ,
                                                                                       uint name_len ,
                                                                                       char *strval ,
                                                                                       uint strlen ,
                                                                                       int flags ,
                                                                                       int module_number ) ;
extern void __attribute__((__visibility__("default")))  zend_register_string_constant(char const   *name ,
                                                                                      uint name_len ,
                                                                                      char *strval ,
                                                                                      int flags ,
                                                                                      int module_number ) ;
extern void __attribute__((__visibility__("default")))  zend_register_double_constant(char const   *name ,
                                                                                      uint name_len ,
                                                                                      double dval ,
                                                                                      int flags ,
                                                                                      int module_number ) ;
extern int __attribute__((__visibility__("default")))  zend_get_constant_ex(char const   *name ,
                                                                            uint name_len ,
                                                                            zval *result ,
                                                                            zend_class_entry *scope ,
                                                                            ulong flags ) ;
extern int __attribute__((__visibility__("default")))  zend_get_constant(char const   *name ,
                                                                         uint name_len ,
                                                                         zval *result ) ;
extern void clean_non_persistent_constants(void) ;
extern void zend_register_standard_constants(void) ;
extern int zend_shutdown_constants(void) ;
extern int zend_startup_constants(void) ;
extern void free_zend_constant(zend_constant *c ) ;
extern void clean_module_constants(int module_number ) ;
extern char __attribute__((__visibility__("default")))  *zend_make_compiled_string_description(char const   *name ) ;
extern void print_op_array(zend_op_array *op_array , int optimizations ) ;
extern int print_class(zend_class_entry *class_entry ) ;
extern void zend_class_add_ref(zend_class_entry **ce ) ;
extern void __attribute__((__visibility__("default")))  destroy_zend_class(zend_class_entry **pce ) ;
extern void __attribute__((__visibility__("default")))  destroy_zend_function(zend_function *function ) ;
extern int __attribute__((__visibility__("default")))  zend_cleanup_function_data_full(zend_function *function ) ;
extern int __attribute__((__visibility__("default")))  zend_cleanup_function_data(zend_function *function ) ;
extern void __attribute__((__visibility__("default")))  zend_cleanup_internal_classes(void) ;
extern void __attribute__((__visibility__("default")))  zend_cleanup_internal_class_data(zend_class_entry *ce ) ;
extern int __attribute__((__visibility__("default")))  zend_cleanup_user_class_data(zend_class_entry **pce ) ;
extern int __attribute__((__visibility__("default")))  zend_cleanup_class_data(zend_class_entry **pce ) ;
extern void __attribute__((__visibility__("default")))  zend_destroy_file_handle(zend_file_handle *file_handle ) ;
extern void __attribute__((__visibility__("default")))  destroy_op_array(zend_op_array *op_array ) ;
extern int __attribute__((__visibility__("default")))  open_file_for_scanning(zend_file_handle *file_handle ) ;
extern int __attribute__((__visibility__("default")))  zend_execute_scripts(int type ,
                                                                            zval **retval ,
                                                                            int file_count 
                                                                            , ...) ;
extern zend_op_array __attribute__((__visibility__("default")))  *compile_filename(int type ,
                                                                                   zval *filename ) ;
extern zend_op_array __attribute__((__visibility__("default")))  *compile_string(zval *source_string ,
                                                                                 char *filename ) ;
extern zend_op_array __attribute__((__visibility__("default")))  *compile_file(zend_file_handle *file_handle ,
                                                                               int type ) ;
extern void init_trait_alias(znode *result , znode const   *method_name ,
                             znode const   *alias , znode const   *modifiers ) ;
extern void add_trait_alias(znode *result , znode const   *trait_alias ) ;
extern void init_trait_alias_list(znode *result , znode const   *trait_alias ) ;
typedef int (*binary_op_type)(zval * , zval * , zval * );
extern binary_op_type __attribute__((__visibility__("default")))  get_binary_op(int opcode ) ;
typedef int (*unary_op_type)(zval * , zval * );
extern unary_op_type __attribute__((__visibility__("default")))  get_unary_op(int opcode ) ;
extern void shutdown_scanner(void) ;
extern void startup_scanner(void) ;
zend_op_array __attribute__((__visibility__("default")))  *(*zend_compile_string)(zval *source_string ,
                                                                                  char *filename )  ;
zend_op_array __attribute__((__visibility__("default")))  *(*zend_compile_string)(zval *source_string ,
                                                                                  char *filename ) ;
zend_op_array __attribute__((__visibility__("default")))  *(*zend_compile_file)(zend_file_handle *file_handle ,
                                                                                int type )  ;
zend_op_array __attribute__((__visibility__("default")))  *(*zend_compile_file)(zend_file_handle *file_handle ,
                                                                                int type ) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_set_script_encoding_by_string(char const   *new_value ,
                                                                                                    size_t new_value_length ) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_set_internal_encoding(zend_encoding const   *encoding ) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_set_script_encoding(zend_encoding const   **encoding_list ,
                                                                                          size_t encoding_list_size ) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_get_script_encoding(void) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_get_internal_encoding(void) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_parse_encoding_list(char const   *encoding_list ,
                                                                                          size_t encoding_list_len ,
                                                                                          zend_encoding const   ***return_list ,
                                                                                          size_t *return_size ,
                                                                                          int persistent ) ;
extern size_t __attribute__((__visibility__("default")))  zend_multibyte_encoding_converter(unsigned char **to ,
                                                                                            size_t *to_length ,
                                                                                            unsigned char const   *from ,
                                                                                            size_t from_length ,
                                                                                            zend_encoding const   *encoding_to ,
                                                                                            zend_encoding const   *encoding_from ) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_detector(unsigned char const   *string ,
                                                                                                         size_t length ,
                                                                                                         zend_encoding const   **list ,
                                                                                                         size_t list_size ) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_check_lexer_compatibility(zend_encoding const   *encoding ) ;
extern char const __attribute__((__visibility__("default")))  *zend_multibyte_get_encoding_name(zend_encoding const   *encoding ) ;
struct _zend_multibyte_functions;
typedef struct _zend_multibyte_functions zend_multibyte_functions;
extern zend_multibyte_functions const __attribute__((__visibility__("default")))  *zend_multibyte_get_functions(void) ;
extern int __attribute__((__visibility__("default")))  zend_multibyte_set_functions(zend_multibyte_functions const   *functions ) ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf8 ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf16le ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf16be ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf32le ;
extern zend_encoding const __attribute__((__visibility__("default")))  *zend_multibyte_encoding_utf32be ;
extern fpu_control_t __fpu_control ;
extern void __attribute__((__visibility__("default")))  zend_ensure_fpu_mode(void) ;
extern void __attribute__((__visibility__("default")))  zend_shutdown_fpu(void) ;
extern void __attribute__((__visibility__("default")))  zend_init_fpu(void) ;
extern int module_registry_unload_temp(zend_module_entry const   *module ) ;
extern int module_registry_request_startup(zend_module_entry *module ) ;
extern int module_registry_cleanup(zend_module_entry *module ) ;
extern void module_destructor(zend_module_entry *module ) ;
extern HashTable __attribute__((__visibility__("default")))  module_registry ;
extern zend_object_handlers __attribute__((__visibility__("default")))  *zend_get_std_object_handlers(void) ;
extern zval __attribute__((__visibility__("default")))  *zend_object_create_proxy(zval *object ,
                                                                                  zval *member ) ;
struct _zend_objects_store;
extern void __attribute__((__visibility__("default")))  zend_objects_store_free_object_storage(zend_objects_store *objects ) ;
extern void __attribute__((__visibility__("default")))  zend_object_store_ctor_failed(zval *zobject ) ;
extern void __attribute__((__visibility__("default")))  zend_object_store_set_object(zval *zobject ,
                                                                                     void *object ) ;
extern void __attribute__((__visibility__("default")))  *zend_object_store_get_object_by_handle(zend_object_handle handle ) ;
extern void __attribute__((__visibility__("default")))  *zend_object_store_get_object(zval const   *object ) ;
struct _zend_object_value;
extern zend_object_value __attribute__((__visibility__("default")))  zend_objects_store_clone_obj(zval *object ) ;
extern zend_uint __attribute__((__visibility__("default")))  zend_objects_store_get_refcount(zval *object ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_del_ref_by_handle_ex(zend_object_handle handle ,
                                                                                                zend_object_handlers const   *handlers ) ;
__inline static void ( __attribute__((__always_inline__)) zend_objects_store_del_ref_by_handle)(zend_object_handle handle ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1104\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  zend_objects_store_del_ref_by_handle_ex(handle,
                                          (zend_object_handlers const   *)((void *)0));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1105\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_objects_store_del_ref_by_handle)(zend_object_handle handle ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_add_ref_by_handle(zend_object_handle handle ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_del_ref(zval *object ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_add_ref(zval *object ) ;
extern zend_object_handle __attribute__((__visibility__("default")))  zend_objects_store_put(void *object ,
                                                                                             void (*dtor)(void *object ,
                                                                                                          zend_object_handle handle ) ,
                                                                                             void (*storage)(void *object ) ,
                                                                                             void (*clone)(void *object ,
                                                                                                           void **object_clone ) ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_destroy(zend_objects_store *objects ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_mark_destructed(zend_objects_store *objects ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_call_destructors(zend_objects_store *objects ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_store_init(zend_objects_store *objects ,
                                                                                zend_uint init_size ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_free_object_storage(zend_object *object ) ;
extern zend_object_value __attribute__((__visibility__("default")))  zend_objects_clone_obj(zval *object ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_clone_members(zend_object *new_object ,
                                                                                   zend_object_value new_obj_val ,
                                                                                   zend_object *old_object ,
                                                                                   zend_object_handle handle ) ;
extern zend_object __attribute__((__visibility__("default")))  *zend_objects_get_address(zval const   *object ) ;
extern void __attribute__((__visibility__("default")))  zend_objects_destroy_object(zend_object *object ,
                                                                                    zend_object_handle handle ) ;
extern zend_object_value __attribute__((__visibility__("default")))  zend_objects_new(zend_object **object ,
                                                                                      zend_class_entry *class_type ) ;
extern void __attribute__((__visibility__("default")))  zend_object_std_dtor(zend_object *object ) ;
extern void __attribute__((__visibility__("default")))  zend_object_std_init(zend_object *object ,
                                                                             zend_class_entry *ce ) ;
__inline static void *( __attribute__((__always_inline__)) zend_ptr_stack_pop)(zend_ptr_stack *stack ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1101\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1102\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1103\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (*(stack->top_element));
}
}
__inline static void *( __attribute__((__always_inline__)) zend_ptr_stack_pop)(zend_ptr_stack *stack ) ;
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_push)(zend_ptr_stack *stack ,
                                                                               void *ptr ) 
{ void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  void *tmp___1 ;
  void **tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1095\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (stack->top + 1 > stack->max) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1090\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1084\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      stack->max += 64;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1085\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (stack->top + 1 > stack->max) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1083\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1091\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (stack->persistent) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1086\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __zend_realloc((void *)stack->elements,
                           sizeof(void *) * (unsigned int )stack->max);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1087\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = tmp;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1088\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = _erealloc((void *)stack->elements,
                          sizeof(void *) * (unsigned int )stack->max, 0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1089\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = (void *)tmp___0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1092\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack->elements = (void **)tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1093\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack->top_element = stack->elements + stack->top;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1094\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1096\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1097\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = stack->top_element;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1098\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1099\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___2 = ptr;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1100\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_push)(zend_ptr_stack *stack ,
                                                                               void *ptr ) ;
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_2_pop)(zend_ptr_stack *stack ,
                                                                                void **a ,
                                                                                void **b ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1077\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1078\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *a = *(stack->top_element);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1079\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1080\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *b = *(stack->top_element);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1081\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  stack->top -= 2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1082\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_3_pop)(zend_ptr_stack *stack ,
                                                                                void **a ,
                                                                                void **b ,
                                                                                void **c ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1069\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1070\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *a = *(stack->top_element);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1071\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1072\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *b = *(stack->top_element);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1073\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1074\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *c = *(stack->top_element);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1075\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  stack->top -= 3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1076\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_2_push)(zend_ptr_stack *stack ,
                                                                                 void *a ,
                                                                                 void *b ) 
{ void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  void *tmp___1 ;
  void **tmp___2 ;
  void **tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1060\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (stack->top + 2 > stack->max) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1055\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1049\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      stack->max += 64;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1050\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (stack->top + 2 > stack->max) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1048\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1056\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (stack->persistent) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1051\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __zend_realloc((void *)stack->elements,
                           sizeof(void *) * (unsigned int )stack->max);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1052\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = tmp;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1053\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = _erealloc((void *)stack->elements,
                          sizeof(void *) * (unsigned int )stack->max, 0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1054\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = (void *)tmp___0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1057\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack->elements = (void **)tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1058\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack->top_element = stack->elements + stack->top;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1059\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1061\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  stack->top += 2;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1062\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = stack->top_element;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1063\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1064\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___2 = a;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1065\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = stack->top_element;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1066\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1067\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___3 = b;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1068\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_2_push)(zend_ptr_stack *stack ,
                                                                                 void *a ,
                                                                                 void *b ) ;
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_3_push)(zend_ptr_stack *stack ,
                                                                                 void *a ,
                                                                                 void *b ,
                                                                                 void *c ) 
{ void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;
  void *tmp___1 ;
  void **tmp___2 ;
  void **tmp___3 ;
  void **tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1036\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (stack->top + 3 > stack->max) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1031\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1025\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      stack->max += 64;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1026\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (stack->top + 3 > stack->max) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1024\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1032\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (stack->persistent) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1027\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __zend_realloc((void *)stack->elements,
                           sizeof(void *) * (unsigned int )stack->max);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1028\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = tmp;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1029\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = _erealloc((void *)stack->elements,
                          sizeof(void *) * (unsigned int )stack->max, 0);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1030\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___1 = (void *)tmp___0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1033\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack->elements = (void **)tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1034\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    stack->top_element = stack->elements + stack->top;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1035\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1037\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  stack->top += 3;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1038\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = stack->top_element;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1039\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1040\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___2 = a;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1041\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = stack->top_element;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1042\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1043\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___3 = b;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1044\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = stack->top_element;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1045\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (stack->top_element) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1046\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *tmp___4 = c;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1047\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) zend_ptr_stack_3_push)(zend_ptr_stack *stack ,
                                                                                 void *a ,
                                                                                 void *b ,
                                                                                 void *c ) ;
extern int __attribute__((__visibility__("default")))  zend_ptr_stack_num_elements(zend_ptr_stack *stack ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_clean(zend_ptr_stack *stack ,
                                                                             void (*func)(void * ) ,
                                                                             zend_bool free_elements ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_apply(zend_ptr_stack *stack ,
                                                                             void (*func)(void * ) ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_destroy(zend_ptr_stack *stack ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_n_pop(zend_ptr_stack *stack ,
                                                                             int count 
                                                                             , ...) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_n_push(zend_ptr_stack *stack ,
                                                                              int count 
                                                                              , ...) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_init_ex(zend_ptr_stack *stack ,
                                                                               zend_bool persistent ) ;
extern void __attribute__((__visibility__("default")))  zend_ptr_stack_init(zend_ptr_stack *stack ) ;
extern void __attribute__((__visibility__("default")))  zend_stack_apply_with_argument(zend_stack *stack ,
                                                                                       int type ,
                                                                                       int (*apply_function)(void *element ,
                                                                                                             void *arg ) ,
                                                                                       void *arg ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_count(zend_stack const   *stack ) ;
extern void __attribute__((__visibility__("default")))  **zend_stack_base(zend_stack const   *stack ) ;
extern int __attribute__((__visibility__("default")))  zend_stack_int_top(zend_stack const   *stack ) ;
struct _zend_ini_scanner_globals;
typedef struct _zend_ini_scanner_globals zend_ini_scanner_globals;
extern zend_ini_scanner_globals __attribute__((__visibility__("default")))  ini_scanner_globals ;
zend_executor_globals __attribute__((__visibility__("default")))  executor_globals  ;
extern int zendparse(void) ;
struct _zend_compiler_globals  __attribute__((__visibility__("default"))) compiler_globals  ;
struct __jmp_buf_tag;
extern  __attribute__((__nothrow__,
__noreturn__)) void siglongjmp(struct __jmp_buf_tag *__env , int __val ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void _longjmp(struct __jmp_buf_tag *__env , int __val ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void longjmp(struct __jmp_buf_tag *__env , int __val ) ;
extern  __attribute__((__nothrow__)) int _setjmp(struct __jmp_buf_tag *__env ) ;
extern  __attribute__((__nothrow__)) int __sigsetjmp(struct __jmp_buf_tag *__env ,
                                                     int __savemask ) ;
extern  __attribute__((__nothrow__)) int setjmp(struct __jmp_buf_tag *__env ) ;
typedef int ( __attribute__((__fastcall__)) (*opcode_handler_t))(zend_execute_data *execute_data );
extern opcode_handler_t __attribute__((__visibility__("default")))  *zend_opcode_handlers ;
struct __anonstruct_zend_error_handling_48;
typedef struct __anonstruct_zend_error_handling_48 zend_error_handling;
extern void __attribute__((__visibility__("default")))  zend_restore_error_handling(zend_error_handling *saved ) ;
enum __anonenum_zend_error_handling_t_47;
extern void __attribute__((__visibility__("default")))  zend_replace_error_handling(zend_error_handling_t error_handling ,
                                                                                    zend_class_entry *exception_class ,
                                                                                    zend_error_handling *current ) ;
extern void __attribute__((__visibility__("default")))  zend_save_error_handling(zend_error_handling *current ) ;
extern void __attribute__((__visibility__("default")))  _zval_dtor_wrapper(zval *zvalue ) ;
extern void __attribute__((__visibility__("default")))  _zval_internal_dtor(zval *zvalue ) ;
extern int __attribute__((__visibility__("default")))  zend_print_variable(zval *var ) ;
extern int __attribute__((__visibility__("default")))  zval_copy_static_var(zval **p ,
                                                                            int num_args ,
                                                                            va_list args ,
                                                                            zend_hash_key *key ) ;
extern void __attribute__((__visibility__("default")))  _zval_copy_ctor_func(zval *zvalue ) ;
__inline static void ( __attribute__((__always_inline__)) _zval_copy_ctor)(zval *zvalue ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1021\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )zvalue->type <= 3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1019\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1020\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1022\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_copy_ctor_func(zvalue);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1023\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  _zval_dtor_func(zval *zvalue ) ;
__inline static void ( __attribute__((__always_inline__)) _zval_dtor)(zval *zvalue ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1016\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )zvalue->type <= 3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1014\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1015\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1017\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  _zval_dtor_func(zvalue);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1018\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static int ( __attribute__((__always_inline__)) fast_is_smaller_or_equal_function)(zval *result ,
                                                                                            zval *op1 ,
                                                                                            zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1010\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1011\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "998\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "999\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "993\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (op1->value.lval <= op2->value.lval);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "996\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "997\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "994\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((double )op1->value.lval <= op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "995\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1008\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "1009\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1005\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1006\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1000\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (op1->value.dval <= op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1003\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "1004\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1001\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (op1->value.dval <= (double )op2->value.lval);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "1002\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "1007\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1012\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compare_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1013\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (result->value.lval <= 0L);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_is_smaller_or_equal_function)(zval *result ,
                                                                                            zval *op1 ,
                                                                                            zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_is_smaller_function)(zval *result ,
                                                                                   zval *op1 ,
                                                                                   zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "989\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "990\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "977\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "978\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "972\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (op1->value.lval < op2->value.lval);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "975\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "976\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "973\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((double )op1->value.lval < op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "974\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "987\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "988\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "984\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "985\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "979\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (op1->value.dval < op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "982\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "983\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "980\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (op1->value.dval < (double )op2->value.lval);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "981\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "986\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "991\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compare_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "992\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (result->value.lval < 0L);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_is_smaller_function)(zval *result ,
                                                                                   zval *op1 ,
                                                                                   zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_not_equal_function)(zval *result ,
                                                                                  zval *op1 ,
                                                                                  zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "968\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "969\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "956\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "957\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "951\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (op1->value.lval != op2->value.lval);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "954\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "955\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "952\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((double )op1->value.lval != op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "953\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "966\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "967\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "963\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "964\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "958\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (op1->value.dval != op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "961\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "962\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "959\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (op1->value.dval != (double )op2->value.lval);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "960\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "965\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "970\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compare_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "971\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (result->value.lval != 0L);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_not_equal_function)(zval *result ,
                                                                                  zval *op1 ,
                                                                                  zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_equal_function)(zval *result ,
                                                                              zval *op1 ,
                                                                              zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "947\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "948\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "935\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "936\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "930\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (op1->value.lval == op2->value.lval);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "933\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "934\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "931\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((double )op1->value.lval == op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "932\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "945\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "946\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "942\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "943\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "937\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (op1->value.dval == op2->value.dval);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "940\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "941\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "938\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (op1->value.dval == (double )op2->value.lval);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "939\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "944\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "949\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  compare_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "950\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (result->value.lval == 0L);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_equal_function)(zval *result ,
                                                                              zval *op1 ,
                                                                              zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  mod_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_mod_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  int __attribute__((__visibility__("default")))  tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "926\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___2 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "927\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___2) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "923\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "924\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "917\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = __builtin_expect((long )(op2->value.lval == 0L), 0L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "918\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "907\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        zend_error(1 << 1L, "Division by zero");
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "908\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.lval = 0L;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "909\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)3;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "910\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (-1);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "915\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = __builtin_expect((long )(op2->value.lval == -1L), 0L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "916\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "911\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->value.lval = 0L;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "912\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->type = (unsigned char)1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "913\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (0);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "914\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "919\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->value.lval = op1->value.lval % op2->value.lval;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "920\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      result->type = (unsigned char)1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "921\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "922\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "925\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "928\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = mod_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "929\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___3);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_mod_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  div_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_div_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long tmp ;
  int tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;
  long tmp___5 ;
  long tmp___6 ;
  long tmp___7 ;
  long tmp___8 ;
  long tmp___9 ;
  long tmp___10 ;
  long tmp___11 ;
  int __attribute__((__visibility__("default")))  tmp___12 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "903\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___11 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "904\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___11) {
    goto _L;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "901\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _L: /* CIL Label */ 
    tmp___10 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "902\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___10) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "899\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "900\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "905\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___12 = div_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "906\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___12);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_div_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  mul_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_mul_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long overflow ;
  long __tmpvar ;
  long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "895\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "896\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "879\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "880\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "870\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "866\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __asm__  ("imul %3,%0\n"
                  "adc $0,%1": "=r" (__tmpvar),
                  "=r" (overflow): "0" (op1->value.lval), "r" (op2->value.lval),
                  "1" (0));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "867\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (overflow) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "864\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->value.dval = (double )op1->value.lval * (double )op2->value.lval;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "865\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->value.lval = __tmpvar;
        }
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "871\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (overflow) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "868\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "869\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)1;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "872\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "877\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "878\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "873\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.dval = (double )op1->value.lval * op2->value.dval;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "874\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "875\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "876\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "893\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "894\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "890\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "891\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "881\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.dval = op1->value.dval * op2->value.dval;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "882\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "883\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "888\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "889\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "884\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->value.dval = op1->value.dval * (double )op2->value.lval;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "885\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->type = (unsigned char)2;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "886\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (0);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "887\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "892\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "897\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___5 = mul_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "898\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___5);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_mul_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  sub_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_sub_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "860\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "861\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "844\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "845\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "836\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __asm__  ("movl\t(%1), %%eax\n\t"
                "subl   (%2), %%eax\n\t"
                "jo     0f\n\t"
                "movl   %%eax, (%0)\n\t"
                "movb   $0x1,0xc(%0)\n\t"
                "jmp    1f\n"
                "0:\n\t"
                "fildl\t(%2)\n\t"
                "fildl\t(%1)\n\t"
                "fsubp\t%%st, %%st(1)\n\t"
                "movb   $0x2,0xc(%0)\n\t"
                "fstpl\t(%0)\n"
                "1:": : "r" (result), "r" (op1), "r" (op2): "eax");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "837\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "842\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "843\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "838\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.dval = (double )op1->value.lval - op2->value.dval;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "839\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "840\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "841\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "858\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "859\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "855\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "856\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "846\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.dval = op1->value.dval - op2->value.dval;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "847\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "848\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "853\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "854\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "849\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->value.dval = op1->value.dval - (double )op2->value.lval;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "850\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->type = (unsigned char)2;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "851\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (0);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "852\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "857\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "862\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___5 = sub_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "863\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___5);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_sub_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  add_function(zval *result ,
                                                                    zval *op1 ,
                                                                    zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_add_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) 
{ long tmp ;
  long tmp___0 ;
  long tmp___1 ;
  long tmp___2 ;
  long tmp___3 ;
  long tmp___4 ;
  int __attribute__((__visibility__("default")))  tmp___5 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "832\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "833\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "816\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __builtin_expect((long )((int )op2->type == 1), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "817\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "808\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __asm__  ("movl\t(%1), %%eax\n\t"
                "addl   (%2), %%eax\n\t"
                "jo     0f\n\t"
                "movl   %%eax, (%0)\n\t"
                "movb   $0x1,0xc(%0)\n\t"
                "jmp    1f\n"
                "0:\n\t"
                "fildl\t(%1)\n\t"
                "fildl\t(%2)\n\t"
                "faddp\t%%st, %%st(1)\n\t"
                "movb   $0x2,0xc(%0)\n\t"
                "fstpl\t(%0)\n"
                "1:": : "r" (result), "r" (op1), "r" (op2): "eax");
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "809\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return (0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "814\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "815\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "810\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.dval = (double )op1->value.lval + op2->value.dval;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "811\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "812\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "813\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "830\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = __builtin_expect((long )((int )op1->type == 2), 1L);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "831\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (tmp___3) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "827\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___2 = __builtin_expect((long )((int )op2->type == 2), 1L);
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "828\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (tmp___2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "818\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->value.dval = op1->value.dval + op2->value.dval;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "819\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        result->type = (unsigned char)2;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "820\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return (0);
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "825\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___1 = __builtin_expect((long )((int )op2->type == 1), 1L);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "826\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___1) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "821\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->value.dval = op1->value.dval + (double )op2->value.lval;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "822\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          result->type = (unsigned char)2;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "823\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (0);
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "824\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "829\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "834\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___5 = add_function(result, op1, op2);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "835\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___5);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_add_function)(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  decrement_function(zval *op2 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_decrement_function)(zval *op1 ) 
{ long tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "804\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "805\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "801\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __asm__  ("decl (%0)\n\t"
              "jno  0f\n\t"
              "movl $0x00200000, (%0)\n\t"
              "movl $0xc1e00000, 0x4(%0)\n\t"
              "movb $0x2,0xc(%0)\n"
              "0:": : "r" (op1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "802\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return (0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "803\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "806\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = decrement_function(op1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "807\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___0);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_decrement_function)(zval *op1 ) ;
extern int __attribute__((__visibility__("default")))  increment_function(zval *op1 ) ;
__inline static int ( __attribute__((__always_inline__)) fast_increment_function)(zval *op1 ) 
{ long tmp ;
  int __attribute__((__visibility__("default")))  tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "797\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_expect((long )((int )op1->type == 1), 1L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "798\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "794\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __asm__  ("incl (%0)\n\t"
              "jno  0f\n\t"
              "movl $0x0, (%0)\n\t"
              "movl $0x41e00000, 0x4(%0)\n\t"
              "movb $0x2,0xc(%0)\n"
              "0:": : "r" (op1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "795\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return (0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "796\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "799\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = increment_function(op1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "800\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___0);
}
}
__inline static int ( __attribute__((__always_inline__)) fast_increment_function)(zval *op1 ) ;
extern void __attribute__((__visibility__("default")))  zend_locale_sprintf_double(zval *op ) ;
extern long __attribute__((__visibility__("default")))  zend_atol(char const   *str ,
                                                                  int str_len ) ;
extern int __attribute__((__visibility__("default")))  zend_atoi(char const   *str ,
                                                                 int str_len ) ;
extern void __attribute__((__visibility__("default")))  zend_compare_objects(zval *result ,
                                                                             zval *o1 ,
                                                                             zval *o2 ) ;
extern void __attribute__((__visibility__("default")))  zend_compare_arrays(zval *result ,
                                                                            zval *a1 ,
                                                                            zval *a2 ) ;
extern void __attribute__((__visibility__("default")))  zend_compare_symbol_tables(zval *result ,
                                                                                   HashTable *ht1 ,
                                                                                   HashTable *ht2 ) ;
extern void __attribute__((__visibility__("default")))  zendi_smart_strcmp(zval *result ,
                                                                           zval *s1 ,
                                                                           zval *s2 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_strncasecmp(char const   *s1 ,
                                                                               uint len1 ,
                                                                               char const   *s2 ,
                                                                               uint len2 ,
                                                                               uint length ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_strncmp(char const   *s1 ,
                                                                           uint len1 ,
                                                                           char const   *s2 ,
                                                                           uint len2 ,
                                                                           uint length ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_strcmp(char const   *s1 ,
                                                                          uint len1 ,
                                                                          char const   *s2 ,
                                                                          uint len2 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_zval_strncasecmp(zval *s1 ,
                                                                                    zval *s2 ,
                                                                                    zval *s3 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_zval_strcasecmp(zval *s1 ,
                                                                                   zval *s2 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_zval_strncmp(zval *s1 ,
                                                                                zval *s2 ,
                                                                                zval *s3 ) ;
extern int __attribute__((__visibility__("default")))  zend_binary_zval_strcmp(zval *s1 ,
                                                                               zval *s2 ) ;
extern int __attribute__((__visibility__("default")))  string_locale_compare_function(zval *result ,
                                                                                      zval *op1 ,
                                                                                      zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  string_case_compare_function(zval *result ,
                                                                                    zval *op1 ,
                                                                                    zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  string_compare_function(zval *result ,
                                                                               zval *op1 ,
                                                                               zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  string_compare_function_ex(zval *result ,
                                                                                  zval *op1 ,
                                                                                  zval *op2 ,
                                                                                  zend_bool case_insensitive ) ;
extern int __attribute__((__visibility__("default")))  numeric_compare_function(zval *result ,
                                                                                zval *op1 ,
                                                                                zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  zval_is_true(zval *op ) ;
extern double __attribute__((__visibility__("default")))  zend_string_to_double(char const   *number ,
                                                                                zend_uint length ) ;
extern int __attribute__((__visibility__("default")))  add_string_to_string(zval *result ,
                                                                            zval const   *op1 ,
                                                                            zval const   *op2 ) ;
extern int __attribute__((__visibility__("default")))  add_char_to_string(zval *result ,
                                                                          zval const   *op1 ,
                                                                          zval const   *op2 ) ;
extern void __attribute__((__visibility__("default")))  multi_convert_to_string_ex(int argc 
                                                                                   , ...) ;
extern void __attribute__((__visibility__("default")))  multi_convert_to_double_ex(int argc 
                                                                                   , ...) ;
extern void __attribute__((__visibility__("default")))  multi_convert_to_long_ex(int argc 
                                                                                 , ...) ;
extern void __attribute__((__visibility__("default")))  convert_to_object(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_to_array(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_to_null(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_to_long_base(zval *op ,
                                                                             int base ) ;
extern void __attribute__((__visibility__("default")))  convert_to_double(zval *op ) ;
extern void __attribute__((__visibility__("default")))  convert_scalar_to_number(zval *op ) ;
__inline static void const   *zend_memrchr(void const   *s , int c , size_t n ) 
{ register unsigned char const   *e ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "790\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (n <= 0U) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "782\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((void const   *)((void *)0));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "783\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "791\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  e = ((unsigned char const   *)s + n) - 1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "792\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "787\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )e >= (unsigned int )((unsigned char const   *)s)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "784\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "788\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*e == (int const   )((unsigned char const   )c)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "785\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((void const   *)e);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "786\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "789\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    e --;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "793\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((void const   *)((void *)0));
}
}
__inline static char *zend_memnstr(char *haystack , char *needle ,
                                   int needle_len , char *end ) 
{ char *p ;
  char ne ;
  void *tmp ;
  int tmp___0 ;
  void *tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "775\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p = haystack;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "776\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ne = *(needle + (needle_len - 1));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "777\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (needle_len == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "754\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = memchr((void const   *)p, (int )*needle, (unsigned int )(end - p));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "755\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((char *)tmp);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "756\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "778\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (needle_len > end - haystack) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "757\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((char *)((void *)0));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "758\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "779\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  end -= needle_len;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "780\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "769\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )p <= (unsigned int )end) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "759\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "770\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = memchr((void const   *)p, (int )*needle,
                     (unsigned int )((end - p) + 1));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "771\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p = (char *)tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "772\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (p) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "765\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )ne == (int )*(p + (needle_len - 1))) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "762\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___0 = memcmp((void const   *)needle, (void const   *)p,
                         (unsigned int )(needle_len - 1));
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "763\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "760\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "761\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return (p);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "764\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "766\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "773\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )p == (unsigned int )((void *)0)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "767\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((char *)((void *)0));
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "768\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "774\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    p ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "781\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((char *)((void *)0));
}
}
__inline static char *zend_memnstr(char *haystack , char *needle ,
                                   int needle_len , char *end ) ;
extern  __attribute__((__nothrow__)) long strtol(char const   * __restrict  __nptr ,
                                                 char ** __restrict  __endptr ,
                                                 int __base )  __attribute__((__nonnull__(1))) ;
extern double __attribute__((__visibility__("default")))  zend_strtod(char const   *s00 ,
                                                                      char const   **se ) ;
static char const   long_min_digits[11] ;
extern double __attribute__((__visibility__("default")))  zend_hex_strtod(char const   *str ,
                                                                          char const   **endptr ) ;
__inline static zend_uchar is_numeric_string(char const   *str , int length ,
                                             long *lval , double *dval ,
                                             int allow_errors ) 
{ char const   *ptr ;
  int base ;
  int digits ;
  int dp_or_e ;
  double local_dval ;
  zend_uchar type ;
  char const   *e ;
  char const   *tmp ;
  double __attribute__((__visibility__("default")))  tmp___0 ;
  double __attribute__((__visibility__("default")))  tmp___1 ;
  char const   *tmp___2 ;
  char const   *tmp___3 ;
  int cmp ;
  size_t __s1_len ;
  size_t __s2_len ;
  int tmp___4 ;
  int tmp___5 ;
  int tmp___6 ;
  unsigned char const   *__s2 ;
  register int __result ;
  int tmp___7 ;
  unsigned char const   *__s1 ;
  register int __result___0 ;
  int tmp___8 ;
  int tmp___9 ;
  int tmp___10 ;
  double __attribute__((__visibility__("default")))  tmp___11 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "744\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  base = 10;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "745\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  digits = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "746\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  dp_or_e = 0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "747\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (! length) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "590\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "591\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "748\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "603\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*str == 32) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "592\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "602\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*str == 9) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "593\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "601\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*str == 10) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "594\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "600\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*str == 13) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "595\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "599\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*str == 11) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "596\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "598\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*str == 12) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "597\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              } else {
                break;
              }
            }
          }
        }
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "604\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    str ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "605\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    length --;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "749\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  ptr = str;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "750\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )*ptr == 45) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "606\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    ptr ++;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "609\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*ptr == 43) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "607\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      ptr ++;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "608\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "751\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )*ptr >= 48) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "678\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*ptr <= 57) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "673\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (length > 2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "618\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*str == 48) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "616\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*(str + 1) == 120) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "610\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            base = 16;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "611\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            ptr += 2;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "615\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*(str + 1) == 88) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "612\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              base = 16;
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "613\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              ptr += 2;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "614\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "617\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "619\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "674\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "621\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*ptr == 48) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "620\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "622\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ptr ++;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "675\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      type = (unsigned char)1;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "676\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      while (1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "656\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (digits >= 11) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "625\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (dval) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "624\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (allow_errors == 1) {
              break;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "623\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "626\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "657\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        check_digits: 
        if ((int const   )*ptr >= 48) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "627\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*ptr <= 57) {
            goto __Cont;
          } else {
            goto _L___3;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "655\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L___3: /* CIL Label */ 
          if (base == 16) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "631\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*ptr >= 65) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "628\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*ptr <= 70) {
                goto __Cont;
              } else {
                goto _L___2;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "630\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L___2: /* CIL Label */ 
              if ((int const   )*ptr >= 97) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "629\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*ptr <= 102) {
                  goto __Cont;
                } else {
                  goto _L___1;
                }
              } else {
                goto _L___1;
              }
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "654\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___1: /* CIL Label */ 
            if (base == 10) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "652\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*ptr == 46) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "632\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (dp_or_e < 1) {
                  goto process_double;
                } else {
                  goto _L___0;
                }
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "651\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                _L___0: /* CIL Label */ 
                if ((int const   )*ptr == 101) {
                  goto _L;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "650\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*ptr == 69) {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "648\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }
                    _L: /* CIL Label */ 
                    if (dp_or_e < 2) {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "644\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      e = ptr + 1;
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "645\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if ((int const   )*e == 45) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "633\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        tmp = e;
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "634\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        e ++;
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "635\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        ptr = tmp;
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "640\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if ((int const   )*e == 43) {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "636\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          tmp = e;
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "637\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          e ++;
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "638\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }
                          ptr = tmp;
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "639\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }

                        }
                      }
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "646\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }
                      if ((int const   )*e >= 48) {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "642\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }
                        if ((int const   )*e <= 57) {
                          goto process_double;
                        } else {
                          {
                          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                                 "a");
                          fprintf(_coverage_fout, "641\n");
                          fflush(_coverage_fout);
                          fclose(_coverage_fout);
                          }

                        }
                      } else {
                        {
                        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                               "a");
                        fprintf(_coverage_fout, "643\n");
                        fflush(_coverage_fout);
                        fclose(_coverage_fout);
                        }

                      }
                    } else {
                      {
                      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                             "a");
                      fprintf(_coverage_fout, "647\n");
                      fflush(_coverage_fout);
                      fclose(_coverage_fout);
                      }

                    }
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "649\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                }
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "653\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        }
        break;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "658\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __Cont: /* CIL Label */ 
        digits ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "659\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        ptr ++;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "677\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (base == 10) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "662\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (digits >= 11) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "660\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          dp_or_e = -1;
          goto process_double;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "661\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "672\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (digits < 8) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "663\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "671\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (digits == 8) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "665\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*(ptr + - digits) <= 55) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "664\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            } else {
              goto _L___4;
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "669\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___4: /* CIL Label */ 
            if (dval) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "666\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___0 = zend_hex_strtod(str, & ptr);
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "667\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              local_dval = (double )tmp___0;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "668\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "670\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            type = (unsigned char)2;
          }
        }
      }
    } else {
      goto _L___5;
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "697\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    _L___5: /* CIL Label */ 
    if ((int const   )*ptr == 46) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "695\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*(ptr + 1) >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "693\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*(ptr + 1) <= 57) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "690\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          process_double: 
          type = (unsigned char)2;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "691\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (dval) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "679\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___1 = zend_strtod(str, & ptr);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "680\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            local_dval = (double )tmp___1;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "689\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if (allow_errors != 1) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "687\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (dp_or_e != -1) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "683\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___3 = ptr;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "684\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                ptr ++;
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "685\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp___3 == 46) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "681\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  dp_or_e = 1;
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "682\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  dp_or_e = 2;
                }
                goto check_digits;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "686\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "688\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }

            }
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "692\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return ((unsigned char)0);
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "694\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        return ((unsigned char)0);
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "696\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((unsigned char)0);
    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "752\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )ptr != (unsigned int )(str + length)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "702\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (! allow_errors) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "698\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((unsigned char)0);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "699\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "703\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (allow_errors == -1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "700\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      zend_error(1 << 3L, "A non well formed numeric value encountered");
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "701\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "704\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "753\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )type == 1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "737\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (digits == 10) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "731\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "716\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __s1_len = strlen(ptr + - digits);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "717\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __s2_len = strlen(long_min_digits);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "718\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (! ((unsigned int )((void const   *)((ptr + - digits) + 1)) - (unsigned int )((void const   *)(ptr + - digits)) == 1U)) {
          goto _L___7;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "711\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if (__s1_len >= 4U) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "709\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            _L___7: /* CIL Label */ 
            if (! ((unsigned int )((void const   *)(long_min_digits + 1)) - (unsigned int )((void const   *)(long_min_digits)) == 1U)) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "705\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              tmp___10 = 1;
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "708\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (__s2_len >= 4U) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "706\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___10 = 1;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "707\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                tmp___10 = 0;
              }
            }
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "710\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___10 = 0;
          }
        }
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "719\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (tmp___10) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "712\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___5 = __builtin_strcmp(ptr + - digits, long_min_digits);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "713\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___9 = tmp___5;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "714\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___8 = __builtin_strcmp(ptr + - digits, long_min_digits);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "715\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp___9 = tmp___8;
        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "720\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___8 = __builtin_strcmp(ptr + - digits, long_min_digits);
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "721\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp___9 = tmp___8;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "732\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      cmp = tmp___9;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "733\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if (cmp < 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "722\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "730\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if (cmp == 0) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "724\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*str == 45) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "723\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            goto _L___8;
          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "728\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          _L___8: /* CIL Label */ 
          if (dval) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "725\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___11 = zend_strtod(str, (char const   **)((void *)0));
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "726\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *dval = (double )tmp___11;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "727\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "729\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          return ((unsigned char)2);
        }
      }
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "734\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "738\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (lval) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "735\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *lval = strtol((char const   */* __restrict  */)str,
                     (char **/* __restrict  */)((void *)0), base);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "736\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "739\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)1);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "742\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (dval) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "740\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *dval = local_dval;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "741\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "743\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((unsigned char)2);
  }
}
}
__inline static zend_uchar is_numeric_string(char const   *str , int length ,
                                             long *lval , double *dval ,
                                             int allow_errors ) ;
__inline static long ( __attribute__((__always_inline__)) zend_dval_to_lval)(double d ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "588\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (d > (double )2147483647L) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "584\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return ((long )((unsigned long )((long long )d)));
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "587\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (d < (double )(-0x7FFFFFFF-1)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "585\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      return ((long )((unsigned long )((long long )d)));
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "586\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "589\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((long )d);
}
}
extern zend_bool __attribute__((__visibility__("default")))  instanceof_function(zend_class_entry const   *instance_ce ,
                                                                                 zend_class_entry const   *ce ) ;
extern zend_bool __attribute__((__visibility__("default")))  instanceof_function_ex(zend_class_entry const   *instance_ce ,
                                                                                    zend_class_entry const   *ce ,
                                                                                    zend_bool interfaces_only ) ;
extern int __attribute__((__visibility__("default")))  is_smaller_or_equal_function(zval *result ,
                                                                                    zval *op1 ,
                                                                                    zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_smaller_function(zval *result ,
                                                                           zval *op1 ,
                                                                           zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_not_equal_function(zval *result ,
                                                                             zval *op1 ,
                                                                             zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_not_identical_function(zval *result ,
                                                                                 zval *op1 ,
                                                                                 zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_identical_function(zval *result ,
                                                                             zval *op1 ,
                                                                             zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  is_equal_function(zval *result ,
                                                                         zval *op1 ,
                                                                         zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  concat_function(zval *result ,
                                                                       zval *op1 ,
                                                                       zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  shift_right_function(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  shift_left_function(zval *result ,
                                                                           zval *op1 ,
                                                                           zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  bitwise_xor_function(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  bitwise_and_function(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  bitwise_or_function(zval *result ,
                                                                           zval *op1 ,
                                                                           zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  bitwise_not_function(zval *result ,
                                                                            zval *op1 ) ;
extern int __attribute__((__visibility__("default")))  boolean_not_function(zval *result ,
                                                                            zval *op1 ) ;
extern int __attribute__((__visibility__("default")))  boolean_xor_function(zval *result ,
                                                                            zval *op1 ,
                                                                            zval *op2 ) ;
extern int __attribute__((__visibility__("default")))  zend_shutdown_strtod(void) ;
extern int __attribute__((__visibility__("default")))  zend_startup_strtod(void) ;
extern double __attribute__((__visibility__("default")))  zend_bin_strtod(char const   *str ,
                                                                          char const   **endptr ) ;
extern double __attribute__((__visibility__("default")))  zend_oct_strtod(char const   *str ,
                                                                          char const   **endptr ) ;
extern char __attribute__((__visibility__("default")))  *zend_dtoa(double _d ,
                                                                   int mode ,
                                                                   int ndigits ,
                                                                   int *decpt ,
                                                                   int *sign ,
                                                                   char **rve ) ;
extern void __attribute__((__visibility__("default")))  zend_freedtoa(char *s ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void __assert(char const   *__assertion , char const   *__file ,
                             int __line ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void __assert_perror_fail(int __errnum , char const   *__file ,
                                         unsigned int __line ,
                                         char const   *__function ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void __assert_fail(char const   *__assertion ,
                                  char const   *__file , unsigned int __line ,
                                  char const   *__function ) ;
extern char *program_invocation_short_name ;
extern char *program_invocation_name ;
extern  __attribute__((__nothrow__)) int *__errno_location(void)  __attribute__((__const__)) ;
struct _zend_gc_globals;
typedef struct _zend_gc_globals zend_gc_globals;
extern zend_gc_globals __attribute__((__visibility__("default")))  gc_globals ;
union __anonunion_u_45 {
   zval *pz ;
   zend_object_handlers const   *handlers ;
};
struct _gc_root_buffer {
   struct _gc_root_buffer *prev ;
   struct _gc_root_buffer *next ;
   zend_object_handle handle ;
   union __anonunion_u_45 u ;
};
struct _zend_gc_globals {
   zend_bool gc_enabled ;
   zend_bool gc_active ;
   gc_root_buffer *buf ;
   gc_root_buffer roots ;
   gc_root_buffer *unused ;
   gc_root_buffer *first_unused ;
   gc_root_buffer *last_unused ;
   zval_gc_info *zval_to_free ;
   zval_gc_info *free_list ;
   zval_gc_info *next_to_free ;
   zend_uint gc_runs ;
   zend_uint collected ;
};
__inline static void ( __attribute__((__always_inline__)) gc_remove_from_buffer)(gc_root_buffer *root ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "579\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (root->next)->prev = root->prev;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "580\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (root->prev)->next = root->next;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "581\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  root->prev = gc_globals.unused;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "582\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  gc_globals.unused = root;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "583\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
__inline static void ( __attribute__((__always_inline__)) gc_remove_from_buffer)(gc_root_buffer *root ) ;
extern void __attribute__((__visibility__("default")))  gc_zval_possible_root(zval *zv ) ;
__inline static void ( __attribute__((__always_inline__)) gc_zval_check_possible_root)(zval *z ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "577\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )z->type == 4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "573\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    gc_zval_possible_root(z);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "576\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )z->type == 5) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "574\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      gc_zval_possible_root(z);
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "575\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "578\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return;
}
}
extern void __attribute__((__visibility__("default")))  gc_reset(void) ;
extern void __attribute__((__visibility__("default")))  gc_init(void) ;
extern void __attribute__((__visibility__("default")))  gc_globals_dtor(void) ;
extern void __attribute__((__visibility__("default")))  gc_globals_ctor(void) ;
extern void __attribute__((__visibility__("default")))  gc_zobj_possible_root(zval *zv ) ;
extern int __attribute__((__visibility__("default")))  gc_collect_cycles(void) ;
extern int __attribute__((__visibility__("default")))  zend_get_configuration_directive(char const   *name ,
                                                                                        uint name_length ,
                                                                                        zval *contents ) ;
extern void __attribute__((__visibility__("default")))  zend_message_dispatcher(long message ,
                                                                                void const   *data ) ;
struct _zend_utility_values;
typedef struct _zend_utility_values zend_utility_values;
extern zend_utility_values __attribute__((__visibility__("default")))  zend_uv ;
extern zend_class_entry __attribute__((__visibility__("default")))  *zend_standard_class_def ;
extern void zenderror(char const   *error ) ;
extern char __attribute__((__visibility__("default")))  *(*zend_resolve_path)(char const   *filename ,
                                                                              int filename_len ) ;
extern char __attribute__((__visibility__("default")))  *(*zend_getenv)(char *name ,
                                                                        size_t name_len ) ;
extern int (*zend_vspprintf)(char **pbuf , size_t max_len ,
                             char const   *format , va_list ap ) ;
extern int __attribute__((__visibility__("default")))  (*zend_stream_open_function)(char const   *filename ,
                                                                                    zend_file_handle *handle ) ;
extern void __attribute__((__visibility__("default")))  (*zend_on_timeout)(int seconds ) ;
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  (*zend_error_cb))(int type ,
                                                                                                    char const   *error_filename ,
                                                                                                    uint error_lineno ,
                                                                                                    char const   *format ,
                                                                                                    va_list args ) ;
extern void __attribute__((__visibility__("default")))  (*zend_ticks_function)(int ticks ) ;
extern void __attribute__((__visibility__("default")))  (*zend_unblock_interruptions)(void) ;
extern void __attribute__((__visibility__("default")))  (*zend_block_interruptions)(void) ;
extern FILE __attribute__((__visibility__("default")))  *(*zend_fopen)(char const   *filename ,
                                                                       char **opened_path ) ;
extern int (* __attribute__((__visibility__("default"))) zend_write)(char const   *str ,
                                                                     uint str_length ) ;
extern int __attribute__((__visibility__("default")))  ( /* format attribute */  (*zend_printf))(char const   *format 
                                                                                                 , ...) ;
extern void zend_post_deactivate_modules(void) ;
extern void zend_deactivate_modules(void) ;
extern void zend_activate_modules(void) ;
extern void zend_call_destructors(void) ;
extern void zend_deactivate(void) ;
extern void zend_activate(void) ;
extern void __attribute__((__visibility__("default")))  ( /* format attribute */  zend_output_debug_string)(zend_bool trigger_break ,
                                                                                                            char const   *format 
                                                                                                            , ...) ;
extern void __attribute__((__visibility__("default")))  zend_print_zval_r_ex(int (*write_func)(char const   *str ,
                                                                                               uint str_length ) ,
                                                                             zval *expr ,
                                                                             int indent ) ;
extern void __attribute__((__visibility__("default")))  zend_print_flat_zval_r(zval *expr ) ;
extern void __attribute__((__visibility__("default")))  zend_print_zval_r(zval *expr ,
                                                                          int indent ) ;
extern int __attribute__((__visibility__("default")))  zend_print_zval_ex(int (*write_func)(char const   *str ,
                                                                                            uint str_length ) ,
                                                                          zval *expr ,
                                                                          int indent ) ;
extern int __attribute__((__visibility__("default")))  zend_print_zval(zval *expr ,
                                                                       int indent ) ;
extern char __attribute__((__visibility__("default")))  *get_zend_version(void) ;
extern void __attribute__((__visibility__("default")))  _zend_bailout(char *filename ,
                                                                      uint lineno ) ;
extern void zend_set_utility_values(zend_utility_values *utility_values ) ;
extern void zend_post_startup(void) ;
extern void zend_register_standard_ini_entries(void) ;
extern void zend_shutdown(void) ;
struct _zend_utility_functions;
typedef struct _zend_utility_functions zend_utility_functions;
extern int zend_startup(zend_utility_functions *utility_functions ,
                        char **extensions ) ;
extern int __attribute__((__visibility__("default")))  zend_compare_file_handles(zend_file_handle *fh1 ,
                                                                                 zend_file_handle *fh2 ) ;
extern int __attribute__((__visibility__("default")))  zend_stream_fixup(zend_file_handle *file_handle ,
                                                                         char **buf ,
                                                                         size_t *len ) ;
extern int __attribute__((__visibility__("default")))  zend_stream_open(char const   *filename ,
                                                                        zend_file_handle *handle ) ;
extern void __attribute__((__visibility__("default")))  zend_register_iterator_wrapper(void) ;
extern zval __attribute__((__visibility__("default")))  *zend_iterator_wrap(zend_object_iterator *iter ) ;
enum zend_object_iterator_kind;
extern enum zend_object_iterator_kind  __attribute__((__visibility__("default"))) zend_iterator_unwrap(zval *array_ptr ,
                                                                                                       zend_object_iterator **iter ) ;
__inline static zend_bool ( __attribute__((__always_inline__)) zval_set_isref_to_p)(zval *pz ,
                                                                                    zend_bool isref ) 
{ zend_uchar tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "570\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = isref;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "571\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  pz->is_ref__gc = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "572\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline static zend_bool ( __attribute__((__always_inline__)) zval_set_isref_to_p)(zval *pz ,
                                                                                    zend_bool isref ) ;
__inline static zend_bool ( __attribute__((__always_inline__)) zval_unset_isref_p)(zval *pz ) 
{ zend_uchar tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "567\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = (unsigned char)0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "568\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  pz->is_ref__gc = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "569\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline static zend_bool ( __attribute__((__always_inline__)) zval_set_isref_p)(zval *pz ) 
{ zend_uchar tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "564\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = (unsigned char)1;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "565\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  pz->is_ref__gc = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "566\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline static zend_bool ( __attribute__((__always_inline__)) zval_isref_p)(zval *pz ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "563\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (pz->is_ref__gc);
}
}
__inline static zend_uint ( __attribute__((__always_inline__)) zval_delref_p)(zval *pz ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "561\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (pz->refcount__gc) --;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "562\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (pz->refcount__gc);
}
}
__inline static zend_uint ( __attribute__((__always_inline__)) zval_addref_p)(zval *pz ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "559\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  (pz->refcount__gc) ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "560\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (pz->refcount__gc);
}
}
__inline static zend_uint ( __attribute__((__always_inline__)) zval_set_refcount_p)(zval *pz ,
                                                                                    zend_uint rc ) 
{ zend_uint tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "556\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = rc;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "557\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  pz->refcount__gc = tmp;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "558\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline static zend_uint ( __attribute__((__always_inline__)) zval_refcount_p)(zval *pz ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "555\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (pz->refcount__gc);
}
}
extern void __attribute__((__visibility__("default")))  zend_std_call_user_call(int ht ,
                                                                                zval *return_value ,
                                                                                zval **return_value_ptr ,
                                                                                zval *this_ptr ,
                                                                                int return_value_used ) ;
extern int __attribute__((__visibility__("default")))  zend_check_property_access(zend_object *zobj ,
                                                                                  char const   *prop_info_name ,
                                                                                  int prop_info_name_len ) ;
extern int __attribute__((__visibility__("default")))  zend_check_protected(zend_class_entry *ce ,
                                                                            zend_class_entry *scope ) ;
extern int __attribute__((__visibility__("default")))  zend_check_private(union _zend_function *fbc ,
                                                                          zend_class_entry *ce ,
                                                                          char *function_name_strval ,
                                                                          int function_name_strlen ) ;
extern void __attribute__((__visibility__("default")))  rebuild_object_properties(zend_object *zobj ) ;
extern void __attribute__((__visibility__("default")))  zend_std_write_property(zval *object ,
                                                                                zval *member ,
                                                                                zval *value ,
                                                                                struct _zend_literal  const  *key ) ;
extern int __attribute__((__visibility__("default")))  zend_std_cast_object_tostring(zval *readobj ,
                                                                                     zval *writeobj ,
                                                                                     int type ) ;
extern HashTable __attribute__((__visibility__("default")))  *zend_std_get_debug_info(zval *object ,
                                                                                      int *is_temp ) ;
extern HashTable __attribute__((__visibility__("default")))  *zend_std_get_properties(zval *object ) ;
extern struct _zend_property_info  __attribute__((__visibility__("default"))) *zend_get_property_info(zend_class_entry *ce ,
                                                                                                      zval *member ,
                                                                                                      int silent ) ;
extern union _zend_function  __attribute__((__visibility__("default"))) *zend_std_get_constructor(zval *object ) ;
extern zend_bool __attribute__((__visibility__("default")))  zend_std_unset_static_property(zend_class_entry *ce ,
                                                                                            char const   *property_name ,
                                                                                            int property_name_len ,
                                                                                            struct _zend_literal  const  *key ) ;
extern zval __attribute__((__visibility__("default")))  **zend_std_get_static_property(zend_class_entry *ce ,
                                                                                       char const   *property_name ,
                                                                                       int property_name_len ,
                                                                                       zend_bool silent ,
                                                                                       struct _zend_literal  const  *key ) ;
extern union _zend_function  __attribute__((__visibility__("default"))) *zend_std_get_static_method(zend_class_entry *ce ,
                                                                                                    char const   *function_name_strval ,
                                                                                                    int function_name_strlen ,
                                                                                                    struct _zend_literal  const  *key ) ;
extern zend_object_handlers __attribute__((__visibility__("default")))  std_object_handlers ;
extern  __attribute__((__noreturn__)) void zend_error_noreturn(int type ,
                                                               char const   *format 
                                                               , ...) ;
typedef zend_llist_element *zend_llist_position;
extern void __attribute__((__visibility__("default")))  *zend_llist_get_prev_ex(zend_llist *l ,
                                                                                zend_llist_position *pos ) ;
extern void __attribute__((__visibility__("default")))  *zend_llist_get_next_ex(zend_llist *l ,
                                                                                zend_llist_position *pos ) ;
extern void __attribute__((__visibility__("default")))  *zend_llist_get_last_ex(zend_llist *l ,
                                                                                zend_llist_position *pos ) ;
extern void __attribute__((__visibility__("default")))  *zend_llist_get_first_ex(zend_llist *l ,
                                                                                 zend_llist_position *pos ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_sort(zend_llist *l ,
                                                                        int (*comp_func)(zend_llist_element const   ** ,
                                                                                         zend_llist_element const   ** ) ) ;
extern int __attribute__((__visibility__("default")))  zend_llist_count(zend_llist *l ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_apply_with_arguments(zend_llist *l ,
                                                                                        void (*func)(void *data ,
                                                                                                     int num_args ,
                                                                                                     va_list args ) ,
                                                                                        int num_args 
                                                                                        , ...) ;
extern void __attribute__((__visibility__("default")))  zend_llist_apply_with_argument(zend_llist *l ,
                                                                                       void (*func)(void *data ,
                                                                                                    void *arg ) ,
                                                                                       void *arg ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_apply_with_del(zend_llist *l ,
                                                                                  int (*func)(void *data ) ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_apply(zend_llist *l ,
                                                                         void (*func)(void * ) ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_clean(zend_llist *l ) ;
extern void __attribute__((__visibility__("default")))  zend_llist_del_element(zend_llist *l ,
                                                                               void *element ,
                                                                               int (*compare)(void *element1 ,
                                                                                              void *element2 ) ) ;
extern ulong __attribute__((__visibility__("default")))  zend_ts_hash_func(char *arKey ,
                                                                           uint nKeyLength ) ;
struct _zend_ts_hashtable;
typedef struct _zend_ts_hashtable TsHashTable;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_rehash(TsHashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_num_elements(TsHashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_minmax(TsHashTable *ht ,
                                                                           int (*compar)(void const   * ,
                                                                                         void const   * ) ,
                                                                           int flag ,
                                                                           void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_compare(TsHashTable *ht1 ,
                                                                            TsHashTable *ht2 ,
                                                                            int (*compar)(void const   * ,
                                                                                          void const   * ) ,
                                                                            zend_bool ordered ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_sort(TsHashTable *ht ,
                                                                         void (*sort_func)(void * ,
                                                                                           size_t  ,
                                                                                           size_t  ,
                                                                                           int (*)(void const   * ,
                                                                                                   void const   * ) ) ,
                                                                         int (*compare_func)(void const   * ,
                                                                                             void const   * ) ,
                                                                         int renumber ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_merge_ex(TsHashTable *target ,
                                                                              TsHashTable *source ,
                                                                              void (*pCopyConstructor)(void *pElement ) ,
                                                                              uint size ,
                                                                              zend_bool (*pMergeSource)(HashTable *target_ht ,
                                                                                                        void *source_data ,
                                                                                                        zend_hash_key *hash_key ,
                                                                                                        void *pParam ) ,
                                                                              void *pParam ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_merge(TsHashTable *target ,
                                                                           TsHashTable *source ,
                                                                           void (*pCopyConstructor)(void *pElement ) ,
                                                                           void *tmp ,
                                                                           uint size ,
                                                                           int overwrite ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_copy_to_hash(HashTable *target ,
                                                                                  TsHashTable *source ,
                                                                                  void (*pCopyConstructor)(void *pElement ) ,
                                                                                  void *tmp ,
                                                                                  uint size ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_copy(TsHashTable *target ,
                                                                          TsHashTable *source ,
                                                                          void (*pCopyConstructor)(void *pElement ) ,
                                                                          void *tmp ,
                                                                          uint size ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_index_exists(TsHashTable *ht ,
                                                                                 ulong h ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_exists(TsHashTable *ht ,
                                                                           char *arKey ,
                                                                           uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_index_find(TsHashTable *ht ,
                                                                               ulong h ,
                                                                               void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_quick_find(TsHashTable *ht ,
                                                                               char *arKey ,
                                                                               uint nKeyLength ,
                                                                               ulong h ,
                                                                               void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_find(TsHashTable *ht ,
                                                                         char *arKey ,
                                                                         uint nKeyLength ,
                                                                         void **pData ) ;
extern ulong __attribute__((__visibility__("default")))  zend_ts_get_hash_value(TsHashTable *ht ,
                                                                                char *arKey ,
                                                                                uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_del_key_or_index(TsHashTable *ht ,
                                                                                     char *arKey ,
                                                                                     uint nKeyLength ,
                                                                                     ulong h ,
                                                                                     int flag ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_reverse_apply(TsHashTable *ht ,
                                                                                   int (*apply_func)(void *pDest ) ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_apply_with_arguments(TsHashTable *ht ,
                                                                                          int (*apply_func)(void *pDest ,
                                                                                                            int num_args ,
                                                                                                            va_list args ,
                                                                                                            zend_hash_key *hash_key ) ,
                                                                                          int  
                                                                                          , ...) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_apply_with_argument(TsHashTable *ht ,
                                                                                         int (*apply_func)(void *pDest ,
                                                                                                           void *argument ) ,
                                                                                         void * ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_apply(TsHashTable *ht ,
                                                                           int (*apply_func)(void *pDest ) ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_graceful_destroy(TsHashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_ts_hash_add_empty_element(TsHashTable *ht ,
                                                                                      char *arKey ,
                                                                                      uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_index_update_or_next_insert(TsHashTable *ht ,
                                                                                                 ulong h ,
                                                                                                 void *pData ,
                                                                                                 uint nDataSize ,
                                                                                                 void **pDest ,
                                                                                                 int flag ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_quick_add_or_update(TsHashTable *ht ,
                                                                                         char *arKey ,
                                                                                         uint nKeyLength ,
                                                                                         ulong h ,
                                                                                         void *pData ,
                                                                                         uint nDataSize ,
                                                                                         void **pDest ,
                                                                                         int flag ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_add_or_update(TsHashTable *ht ,
                                                                                   char *arKey ,
                                                                                   uint nKeyLength ,
                                                                                   void *pData ,
                                                                                   uint nDataSize ,
                                                                                   void **pDest ,
                                                                                   int flag ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_clean(TsHashTable *ht ) ;
extern void __attribute__((__visibility__("default")))  zend_ts_hash_destroy(TsHashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_init_ex(TsHashTable *ht ,
                                                                             uint nSize ,
                                                                             ulong (*pHashFunction)(char const   *arKey ,
                                                                                                    uint nKeyLength ) ,
                                                                             void (*pDestructor)(void *pDest ) ,
                                                                             zend_bool persistent ,
                                                                             zend_bool bApplyProtection ) ;
extern int __attribute__((__visibility__("default")))  _zend_ts_hash_init(TsHashTable *ht ,
                                                                          uint nSize ,
                                                                          ulong (*pHashFunction)(char const   *arKey ,
                                                                                                 uint nKeyLength ) ,
                                                                          void (*pDestructor)(void *pDest ) ,
                                                                          zend_bool persistent ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_update_current_key_ex(HashTable *ht ,
                                                                                       int key_type ,
                                                                                       char const   *str_index ,
                                                                                       uint str_length ,
                                                                                       ulong num_index ,
                                                                                       int mode ,
                                                                                       HashPosition *pos ) ;
__inline static int zend_symtable_update_current_key_ex(HashTable *ht ,
                                                        char const   *arKey ,
                                                        uint nKeyLength ,
                                                        int mode ,
                                                        HashPosition *pos ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "552\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "551\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "548\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = arKey;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "549\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "516\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "517\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "550\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "546\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "540\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "541\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "524\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "518\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "523\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "522\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "520\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "519\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "521\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "542\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "543\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "528\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "529\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "527\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "526\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "525\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "530\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "544\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "536\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "532\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "531\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "533\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "535\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "534\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "537\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___0 = zend_hash_update_current_key_ex(ht, 2,
                                                      (char const   *)((void *)0),
                                                      0U, idx, mode, pos);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "538\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "539\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "545\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "547\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "553\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_hash_update_current_key_ex(ht, 1, arKey, nKeyLength, 0UL, mode,
                                            pos);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "554\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
__inline static int zend_symtable_update_current_key_ex(HashTable *ht ,
                                                        char const   *arKey ,
                                                        uint nKeyLength ,
                                                        int mode ,
                                                        HashPosition *pos ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_index_exists(HashTable const   *ht ,
                                                                              ulong h ) ;
__inline static int zend_symtable_exists(HashTable *ht , char const   *arKey ,
                                         uint nKeyLength ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "513\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "512\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "509\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = arKey;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "510\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "477\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "478\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "511\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "507\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "501\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "502\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "485\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "479\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "484\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "483\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "481\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "480\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "482\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "503\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "504\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "489\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "490\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "488\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "487\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "486\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "491\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "505\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "497\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "493\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "492\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "494\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "496\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "495\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "498\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___0 = zend_hash_index_exists((HashTable const   *)ht, idx);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "499\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "500\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "506\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "508\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "514\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_hash_exists((HashTable const   *)ht, arKey, nKeyLength);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "515\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
__inline static int zend_symtable_exists(HashTable *ht , char const   *arKey ,
                                         uint nKeyLength ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_index_find(HashTable const   *ht ,
                                                                            ulong h ,
                                                                            void **pData ) ;
__inline static int zend_symtable_find(HashTable *ht , char const   *arKey ,
                                       uint nKeyLength , void **pData ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "474\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "473\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "470\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = arKey;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "471\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "438\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "439\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "472\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "468\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "462\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "463\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "446\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "440\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "445\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "444\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "442\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "441\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "443\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "464\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "465\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "450\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "451\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "449\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "448\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "447\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "452\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "466\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "458\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "454\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "453\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "455\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "457\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "456\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "459\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___0 = zend_hash_index_find((HashTable const   *)ht, idx, pData);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "460\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "461\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "467\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "469\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "475\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_hash_find((HashTable const   *)ht, arKey, nKeyLength, pData);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "476\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
__inline static int zend_symtable_find(HashTable *ht , char const   *arKey ,
                                       uint nKeyLength , void **pData ) ;
__inline static int zend_symtable_del(HashTable *ht , char const   *arKey ,
                                      uint nKeyLength ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "435\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "434\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "431\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = arKey;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "432\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "399\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "400\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "433\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "429\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "423\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "424\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "407\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "401\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "406\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "405\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "403\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "402\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "404\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "425\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "426\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "411\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "412\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "410\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "409\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "408\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "413\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "427\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "419\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "415\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "414\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "416\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "418\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "417\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "420\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___0 = zend_hash_del_key_or_index(ht,
                                                 (char const   *)((void *)0),
                                                 0U, idx, 1);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "421\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "422\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "428\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "430\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "436\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = zend_hash_del_key_or_index(ht, arKey, nKeyLength, 0UL, 0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "437\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
__inline static int zend_symtable_del(HashTable *ht , char const   *arKey ,
                                      uint nKeyLength ) ;
__inline static int zend_symtable_update(HashTable *ht , char const   *arKey ,
                                         uint nKeyLength , void *pData ,
                                         uint nDataSize , void **pDest ) 
{ ulong idx ;
  register char const   *tmp ;
  char const   *end ;
  int __attribute__((__visibility__("default")))  tmp___0 ;
  int __attribute__((__visibility__("default")))  tmp___1 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "396\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "395\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "392\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = arKey;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "393\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp == 45) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "360\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp ++;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "361\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "394\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*tmp >= 48) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "390\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*tmp <= 57) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "384\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          end = (arKey + nKeyLength) - 1;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "385\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*end != 0) {
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "368\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*tmp == 48) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "362\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (nKeyLength > 2U) {
                break;
              } else {
                goto _L;
              }
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "367\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              _L: /* CIL Label */ 
              if (end - tmp > 10) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "366\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if (end - tmp == 10) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "364\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }
                  if ((int const   )*tmp > 50) {
                    break;
                  } else {
                    {
                    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                           "a");
                    fprintf(_coverage_fout, "363\n");
                    fflush(_coverage_fout);
                    fclose(_coverage_fout);
                    }

                  }
                } else {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "365\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                }
              }
            }
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "386\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          idx = (unsigned long )((int const   )*tmp - 48);
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "387\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          while (1) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "372\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "373\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((unsigned int )tmp != (unsigned int )end) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "371\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if ((int const   )*tmp >= 48) {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "370\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }
                if ((int const   )*tmp <= 57) {
                  {
                  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                         "a");
                  fprintf(_coverage_fout, "369\n");
                  fflush(_coverage_fout);
                  fclose(_coverage_fout);
                  }

                } else {
                  break;
                }
              } else {
                break;
              }
            } else {
              break;
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "374\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            idx = idx * 10UL + (ulong )((int const   )*tmp - 48);
          }
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "388\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((unsigned int )tmp == (unsigned int )end) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "380\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            if ((int const   )*arKey == 45) {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "376\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx - 1UL > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "375\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "377\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              idx = (unsigned long )(- ((long )idx));
            } else {
              {
              _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                     "a");
              fprintf(_coverage_fout, "379\n");
              fflush(_coverage_fout);
              fclose(_coverage_fout);
              }
              if (idx > 2147483647UL) {
                break;
              } else {
                {
                _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                       "a");
                fprintf(_coverage_fout, "378\n");
                fflush(_coverage_fout);
                fclose(_coverage_fout);
                }

              }
            }
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "381\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp___0 = _zend_hash_index_update_or_next_insert(ht, idx, pData,
                                                             nDataSize, pDest, 1);
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "382\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            return ((int )tmp___0);
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "383\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "389\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "391\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      break;
    }
    break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "397\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___1 = _zend_hash_add_or_update(ht, arKey, nKeyLength, pData, nDataSize,
                                     pDest, 1);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "398\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp___1);
}
}
__inline static ulong zend_inline_hash_func(char const   *arKey ,
                                            uint nKeyLength ) 
{ register ulong hash ;
  char const   *tmp ;
  char const   *tmp___0 ;
  char const   *tmp___1 ;
  char const   *tmp___2 ;
  char const   *tmp___3 ;
  char const   *tmp___4 ;
  char const   *tmp___5 ;
  char const   *tmp___6 ;
  char const   *tmp___7 ;
  char const   *tmp___8 ;
  char const   *tmp___9 ;
  char const   *tmp___10 ;
  char const   *tmp___11 ;
  char const   *tmp___12 ;
  char const   *tmp___13 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "357\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = (ulong )5381;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "358\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "310\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (nKeyLength >= 8U) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "309\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "311\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "312\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "313\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "314\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "315\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "316\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "317\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "318\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "319\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___1;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "320\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "321\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "322\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "323\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "324\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "325\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___3;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "326\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___4 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "327\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "328\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___4;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "329\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___5 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "330\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "331\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___5;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "332\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___6 = arKey;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "333\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    arKey ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "334\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    hash = ((hash << 5) + hash) + (ulong )*tmp___6;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "335\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    nKeyLength -= 8U;
  }
  switch ((int )nKeyLength) {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "336\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 7: 
  tmp___7 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "337\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "338\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___7;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "339\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 6: 
  tmp___8 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "340\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "341\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___8;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "342\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 5: 
  tmp___9 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "343\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "344\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___9;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "345\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 4: 
  tmp___10 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "346\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "347\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___10;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "348\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 3: 
  tmp___11 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "349\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "350\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___11;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "351\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 2: 
  tmp___12 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "352\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "353\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___12;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "354\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  case 1: 
  tmp___13 = arKey;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "355\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  arKey ++;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "356\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  hash = ((hash << 5) + hash) + (ulong )*tmp___13;
  break;
  case 0: 
  break;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "359\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (hash);
}
}
extern int __attribute__((__visibility__("default")))  zend_hash_rehash(HashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_minmax(HashTable const   *ht ,
                                                                        int (*compar)(void const   * ,
                                                                                      void const   * ) ,
                                                                        int flag ,
                                                                        void **pData ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_compare(HashTable *ht1 ,
                                                                         HashTable *ht2 ,
                                                                         int (*compar)(void const   * ,
                                                                                       void const   * ) ,
                                                                         zend_bool ordered ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_sort(HashTable *ht ,
                                                                      void (*sort_func)(void * ,
                                                                                        size_t  ,
                                                                                        size_t  ,
                                                                                        int (*)(void const   * ,
                                                                                                void const   * ) ) ,
                                                                      int (*compare_func)(void const   * ,
                                                                                          void const   * ) ,
                                                                      int renumber ) ;
struct _HashPointer;
extern int __attribute__((__visibility__("default")))  zend_hash_set_pointer(HashTable *ht ,
                                                                             HashPointer const   *ptr ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_get_pointer(HashTable const   *ht ,
                                                                             HashPointer *ptr ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_internal_pointer_end_ex(HashTable *ht ,
                                                                                          HashPosition *pos ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_get_current_key_type_ex(HashTable *ht ,
                                                                                         HashPosition *pos ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_get_current_key_ex(HashTable const   *ht ,
                                                                                    char **str_index ,
                                                                                    uint *str_length ,
                                                                                    ulong *num_index ,
                                                                                    zend_bool duplicate ,
                                                                                    HashPosition *pos ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_move_backwards_ex(HashTable *ht ,
                                                                                   HashPosition *pos ) ;
extern ulong __attribute__((__visibility__("default")))  zend_hash_next_free_element(HashTable const   *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_quick_exists(HashTable const   *ht ,
                                                                              char const   *arKey ,
                                                                              uint nKeyLength ,
                                                                              ulong h ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_reverse_apply(HashTable *ht ,
                                                                                int (*apply_func)(void *pDest ) ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_apply_with_argument(HashTable *ht ,
                                                                                      int (*apply_func)(void *pDest ,
                                                                                                        void *argument ) ,
                                                                                      void * ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_graceful_reverse_destroy(HashTable *ht ) ;
extern int __attribute__((__visibility__("default")))  zend_hash_add_empty_element(HashTable *ht ,
                                                                                   char const   *arKey ,
                                                                                   uint nKeyLength ) ;
extern void __attribute__((__visibility__("default")))  zend_hash_clean(HashTable *ht ) ;
static char const   long_min_digits[11]  = 
  {      (char const   )'2',      (char const   )'1',      (char const   )'4',      (char const   )'7', 
        (char const   )'4',      (char const   )'8',      (char const   )'3',      (char const   )'6', 
        (char const   )'4',      (char const   )'8',      (char const   )'\000'};
extern void zend_interned_strings_dtor(void) ;
extern void zend_interned_strings_init(void) ;
extern void __attribute__((__visibility__("default")))  (*zend_interned_strings_restore)(void) ;
extern void __attribute__((__visibility__("default")))  (*zend_interned_strings_snapshot)(void) ;
struct _zend_mm_heap;
typedef struct _zend_mm_heap zend_mm_heap;
extern void __attribute__((__visibility__("default")))  zend_mm_set_custom_handlers(zend_mm_heap *heap ,
                                                                                    void *(*_malloc)(size_t  ) ,
                                                                                    void (*_free)(void * ) ,
                                                                                    void *(*_realloc)(void * ,
                                                                                                      size_t  ) ) ;
struct _zend_mm_storage;
typedef struct _zend_mm_storage zend_mm_storage;
extern zend_mm_storage __attribute__((__visibility__("default")))  *zend_mm_get_storage(zend_mm_heap *heap ) ;
extern zend_mm_heap __attribute__((__visibility__("default")))  *zend_mm_set_heap(zend_mm_heap *new_heap ) ;
struct _zend_mm_mem_handlers;
typedef struct _zend_mm_mem_handlers zend_mm_mem_handlers;
extern zend_mm_heap __attribute__((__visibility__("default")))  *zend_mm_startup_ex(zend_mm_mem_handlers const   *handlers ,
                                                                                    size_t block_size ,
                                                                                    size_t reserve_size ,
                                                                                    int internal ,
                                                                                    void *params ) ;
extern size_t __attribute__((__visibility__("default")))  _zend_mm_block_size(zend_mm_heap *heap ,
                                                                              void *p ) ;
extern void __attribute__((__visibility__("default")))  *_zend_mm_realloc(zend_mm_heap *heap ,
                                                                          void *p ,
                                                                          size_t size ) ;
extern void __attribute__((__visibility__("default")))  _zend_mm_free(zend_mm_heap *heap ,
                                                                      void *p ) ;
extern void __attribute__((__visibility__("default")))  *_zend_mm_alloc(zend_mm_heap *heap ,
                                                                        size_t size )  __attribute__((__malloc__)) ;
extern void __attribute__((__visibility__("default")))  zend_mm_shutdown(zend_mm_heap *heap ,
                                                                         int full_shutdown ,
                                                                         int silent ) ;
extern zend_mm_heap __attribute__((__visibility__("default")))  *zend_mm_startup(void) ;
extern size_t __attribute__((__visibility__("default")))  zend_memory_peak_usage(int real_usage ) ;
extern size_t __attribute__((__visibility__("default")))  zend_memory_usage(int real_usage ) ;
extern int __attribute__((__visibility__("default")))  is_zend_mm(void) ;
extern void __attribute__((__visibility__("default")))  shutdown_memory_manager(int silent ,
                                                                                int full_shutdown ) ;
extern void __attribute__((__visibility__("default")))  start_memory_manager(void) ;
extern int __attribute__((__visibility__("default")))  zend_set_memory_limit(size_t memory_limit ) ;
extern  __attribute__((__nothrow__, __noreturn__)) void exit(int __status ) ;
extern struct _IO_FILE *stderr ;
__inline static void *__zend_realloc(void *p , size_t len ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "305\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  p = realloc(p, len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "306\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (p) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "303\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return (p);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "304\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "307\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fprintf((FILE */* __restrict  */)stderr,
          (char const   */* __restrict  */)"Out of memory\n");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "308\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  exit(1);
}
}
extern void __attribute__((__visibility__("default")))  *_safe_malloc(size_t nmemb ,
                                                                      size_t size ,
                                                                      size_t offset )  __attribute__((__malloc__)) ;
__inline static void *__zend_calloc(size_t nmemb , size_t len ) 
{ void *tmp ;
  void __attribute__((__visibility__("default")))  *tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "299\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = _safe_malloc(nmemb, len, 0U);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "300\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = (void *)tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "301\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  memset(tmp, 0, nmemb * len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "302\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline static void *__zend_calloc(size_t nmemb , size_t len ) ;
__inline static void *__zend_malloc(size_t len ) 
{ void *tmp ;
  void *tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "294\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___0 = malloc(len);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "295\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = tmp___0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "296\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "292\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    return (tmp);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "293\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "297\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  fprintf((FILE */* __restrict  */)stderr,
          (char const   */* __restrict  */)"Out of memory\n");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "298\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  exit(1);
}
}
extern size_t __attribute__((__visibility__("default")))  _zend_mem_block_size(void *ptr ) ;
extern void __attribute__((__visibility__("default")))  *_safe_realloc(void *ptr ,
                                                                       size_t nmemb ,
                                                                       size_t size ,
                                                                       size_t offset ) ;
extern void __attribute__((__visibility__("default")))  *_safe_erealloc(void *ptr ,
                                                                        size_t nmemb ,
                                                                        size_t size ,
                                                                        size_t offset ) ;
extern void __attribute__((__visibility__("default")))  *_ecalloc(size_t nmemb ,
                                                                  size_t size )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) int dlinfo(void * __restrict  __handle ,
                                                int __request ,
                                                void * __restrict  __arg )  __attribute__((__nonnull__(1,3))) ;
struct __anonstruct_Dl_info_34;
typedef struct __anonstruct_Dl_info_34 Dl_info;
extern  __attribute__((__nothrow__)) int dladdr1(void const   *__address ,
                                                 Dl_info *__info ,
                                                 void **__extra_info ,
                                                 int __flags )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int dladdr(void const   *__address ,
                                                Dl_info *__info )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *dlerror(void) ;
extern  __attribute__((__nothrow__)) void *dlvsym(void * __restrict  __handle ,
                                                  char const   * __restrict  __name ,
                                                  char const   * __restrict  __version )  __attribute__((__nonnull__(2,3))) ;
typedef long Lmid_t;
extern  __attribute__((__nothrow__)) void *dlmopen(Lmid_t __nsid ,
                                                   char const   *__file ,
                                                   int __mode ) ;
extern  __attribute__((__nothrow__)) void *dlsym(void * __restrict  __handle ,
                                                 char const   * __restrict  __name )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int dlclose(void *__handle )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *dlopen(char const   *__file ,
                                                  int __mode ) ;
extern  __attribute__((__nothrow__)) void _dl_mcount_wrapper_check(void *__selfpc ) ;
typedef void _IO_lock_t;
struct _IO_marker;
struct _IO_FILE {
   int _flags ;
   char *_IO_read_ptr ;
   char *_IO_read_end ;
   char *_IO_read_base ;
   char *_IO_write_base ;
   char *_IO_write_ptr ;
   char *_IO_write_end ;
   char *_IO_buf_base ;
   char *_IO_buf_end ;
   char *_IO_save_base ;
   char *_IO_backup_base ;
   char *_IO_save_end ;
   struct _IO_marker *_markers ;
   struct _IO_FILE *_chain ;
   int _fileno ;
   int _flags2 ;
   __off_t _old_offset ;
   unsigned short _cur_column ;
   signed char _vtable_offset ;
   char _shortbuf[1] ;
   _IO_lock_t *_lock ;
   __off64_t _offset ;
   void *__pad1 ;
   void *__pad2 ;
   void *__pad3 ;
   void *__pad4 ;
   size_t __pad5 ;
   int _mode ;
   char _unused2[(15U * sizeof(int ) - 4U * sizeof(void *)) - sizeof(size_t )] ;
};
__inline extern  __attribute__((__nothrow__)) int ferror_unlocked(FILE *__stream ) ;
__inline extern int ferror_unlocked(FILE *__stream ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "291\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((__stream->_flags & 0x20) != 0);
}
}
__inline extern  __attribute__((__nothrow__)) int ferror_unlocked(FILE *__stream ) ;
__inline extern  __attribute__((__nothrow__)) int feof_unlocked(FILE *__stream ) ;
__inline extern int feof_unlocked(FILE *__stream ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "290\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((__stream->_flags & 0x10) != 0);
}
}
__inline extern  __attribute__((__nothrow__)) int feof_unlocked(FILE *__stream ) ;
extern __ssize_t __getdelim(char ** __restrict  __lineptr ,
                            size_t * __restrict  __n , int __delimiter ,
                            FILE * __restrict  __stream ) ;
__inline extern __ssize_t getline(char ** __restrict  __lineptr ,
                                  size_t * __restrict  __n ,
                                  FILE * __restrict  __stream ) 
{ __ssize_t tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "288\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __getdelim(__lineptr, __n, '\n', __stream);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "289\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern __ssize_t getline(char ** __restrict  __lineptr ,
                                  size_t * __restrict  __n ,
                                  FILE * __restrict  __stream ) ;
extern struct _IO_FILE *stdout ;
typedef struct _IO_FILE _IO_FILE;
extern int __overflow(_IO_FILE * , int  ) ;
__inline extern int putchar_unlocked(int __c ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  char tmp___2 ;
  int tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "285\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((unsigned int )stdout->_IO_write_ptr >= (unsigned int )stdout->_IO_write_end),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "286\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "278\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __overflow(stdout, (int )((unsigned char )__c));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "279\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "280\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = stdout->_IO_write_ptr;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "281\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (stdout->_IO_write_ptr) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "282\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = (char )__c;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "283\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *tmp___1 = tmp___2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "284\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = (int )((unsigned char )tmp___2);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "287\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___3);
}
}
__inline extern int putchar_unlocked(int __c ) ;
__inline extern int putc_unlocked(int __c , FILE *__stream ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  char tmp___2 ;
  int tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "275\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((unsigned int )__stream->_IO_write_ptr >= (unsigned int )__stream->_IO_write_end),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "276\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "268\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __overflow(__stream, (int )((unsigned char )__c));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "269\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "270\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = __stream->_IO_write_ptr;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "271\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (__stream->_IO_write_ptr) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "272\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = (char )__c;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "273\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *tmp___1 = tmp___2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "274\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = (int )((unsigned char )tmp___2);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "277\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___3);
}
}
__inline extern int putc_unlocked(int __c , FILE *__stream ) ;
__inline extern int fputc_unlocked(int __c , FILE *__stream ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  char tmp___2 ;
  int tmp___3 ;
  long tmp___4 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "265\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___4 = __builtin_expect((long )((unsigned int )__stream->_IO_write_ptr >= (unsigned int )__stream->_IO_write_end),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "266\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___4) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "258\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __overflow(__stream, (int )((unsigned char )__c));
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "259\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "260\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = __stream->_IO_write_ptr;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "261\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (__stream->_IO_write_ptr) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "262\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = (char )__c;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "263\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *tmp___1 = tmp___2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "264\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___3 = (int )((unsigned char )tmp___2);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "267\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___3);
}
}
__inline extern int fputc_unlocked(int __c , FILE *__stream ) ;
extern int _IO_putc(int __c , _IO_FILE *__fp ) ;
__inline extern int putchar(int __c ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "256\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _IO_putc(__c, stdout);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "257\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern int putchar(int __c ) ;
extern struct _IO_FILE *stdin ;
extern int __uflow(_IO_FILE * ) ;
__inline extern int getchar_unlocked(void) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  int tmp___2 ;
  long tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "253\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = __builtin_expect((long )((unsigned int )stdin->_IO_read_ptr >= (unsigned int )stdin->_IO_read_end),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "254\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "248\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __uflow(stdin);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "249\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "250\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = stdin->_IO_read_ptr;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "251\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (stdin->_IO_read_ptr) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "252\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = (int )*((unsigned char *)tmp___1);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "255\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___2);
}
}
__inline extern int getchar_unlocked(void) ;
__inline extern int getc_unlocked(FILE *__fp ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  int tmp___2 ;
  long tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "245\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = __builtin_expect((long )((unsigned int )__fp->_IO_read_ptr >= (unsigned int )__fp->_IO_read_end),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "246\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "240\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __uflow(__fp);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "241\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "242\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = __fp->_IO_read_ptr;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "243\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (__fp->_IO_read_ptr) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "244\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = (int )*((unsigned char *)tmp___1);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "247\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___2);
}
}
__inline extern int getc_unlocked(FILE *__fp ) ;
__inline extern int fgetc_unlocked(FILE *__fp ) 
{ long tmp ;
  int tmp___0 ;
  char *tmp___1 ;
  int tmp___2 ;
  long tmp___3 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "237\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp___3 = __builtin_expect((long )((unsigned int )__fp->_IO_read_ptr >= (unsigned int )__fp->_IO_read_end),
                             0L);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "238\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (tmp___3) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "232\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = __uflow(__fp);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "233\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = tmp___0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "234\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___1 = __fp->_IO_read_ptr;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "235\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    (__fp->_IO_read_ptr) ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "236\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = (int )*((unsigned char *)tmp___1);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "239\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___2);
}
}
__inline extern int fgetc_unlocked(FILE *__fp ) ;
extern int _IO_getc(_IO_FILE *__fp ) ;
__inline extern int getchar(void) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "230\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = _IO_getc(stdin);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "231\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern int getchar(void) ;
extern int vfprintf(FILE * __restrict  __s ,
                    char const   * __restrict  __format , __gnuc_va_list __arg ) ;
__inline extern int vprintf(char const   * __restrict  __fmt ,
                            __gnuc_va_list __arg ) 
{ int tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "228\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = vfprintf((FILE */* __restrict  */)stdout, __fmt, __arg);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "229\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern int vprintf(char const   * __restrict  __fmt ,
                            __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) void funlockfile(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int ftrylockfile(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) void flockfile(FILE *__stream ) ;
struct obstack;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  obstack_vprintf)(struct obstack * __restrict  __obstack ,
                                                                                    char const   * __restrict  __format ,
                                                                                    __gnuc_va_list __args ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  obstack_printf)(struct obstack * __restrict  __obstack ,
                                                                                   char const   * __restrict  __format 
                                                                                   , ...) ;
extern char *cuserid(char *__s ) ;
extern  __attribute__((__nothrow__)) char *ctermid(char *__s ) ;
extern int pclose(FILE *__stream ) ;
extern FILE *popen(char const   *__command , char const   *__modes ) ;
extern  __attribute__((__nothrow__)) int fileno_unlocked(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int fileno(FILE *__stream ) ;
extern char const   * const  _sys_errlist[] ;
extern int _sys_nerr ;
extern char const   * const  sys_errlist[] ;
extern int sys_nerr ;
extern void perror(char const   *__s ) ;
extern  __attribute__((__nothrow__)) void clearerr_unlocked(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int ferror(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int feof(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) void clearerr(FILE *__stream ) ;
struct __anonstruct__G_fpos64_t_32;
typedef struct __anonstruct__G_fpos64_t_32 _G_fpos64_t;
typedef _G_fpos64_t fpos64_t;
extern int fsetpos64(FILE *__stream , fpos64_t const   *__pos ) ;
extern int fgetpos64(FILE * __restrict  __stream , fpos64_t * __restrict  __pos ) ;
extern __off64_t ftello64(FILE *__stream ) ;
extern int fseeko64(FILE *__stream , __off64_t __off , int __whence ) ;
struct __anonstruct__G_fpos_t_31;
typedef struct __anonstruct__G_fpos_t_31 _G_fpos_t;
typedef _G_fpos_t fpos_t;
extern int fsetpos(FILE *__stream , fpos_t const   *__pos ) ;
extern int fgetpos(FILE * __restrict  __stream , fpos_t * __restrict  __pos ) ;
extern __off_t ftello(FILE *__stream ) ;
extern int fseeko(FILE *__stream , __off_t __off , int __whence ) ;
extern void rewind(FILE *__stream ) ;
extern long ftell(FILE *__stream ) ;
extern int fseek(FILE *__stream , long __off , int __whence ) ;
extern size_t fwrite_unlocked(void const   * __restrict  __ptr , size_t __size ,
                              size_t __n , FILE * __restrict  __stream ) ;
extern size_t fread_unlocked(void * __restrict  __ptr , size_t __size ,
                             size_t __n , FILE * __restrict  __stream ) ;
extern int fputs_unlocked(char const   * __restrict  __s ,
                          FILE * __restrict  __stream ) ;
extern size_t fwrite(void const   * __restrict  __ptr , size_t __size ,
                     size_t __n , FILE * __restrict  __s ) ;
extern size_t fread(void * __restrict  __ptr , size_t __size , size_t __n ,
                    FILE * __restrict  __stream ) ;
extern int ungetc(int __c , FILE *__stream ) ;
extern int puts(char const   *__s ) ;
extern int fputs(char const   * __restrict  __s , FILE * __restrict  __stream ) ;
extern __ssize_t getdelim(char ** __restrict  __lineptr ,
                          size_t * __restrict  __n , int __delimiter ,
                          FILE * __restrict  __stream ) ;
extern char *fgets_unlocked(char * __restrict  __s , int __n ,
                            FILE * __restrict  __stream ) ;
extern char *gets(char *__s ) ;
extern char *fgets(char * __restrict  __s , int __n ,
                   FILE * __restrict  __stream ) ;
extern int putw(int __w , FILE *__stream ) ;
extern int getw(FILE *__stream ) ;
extern int putc(int __c , FILE *__stream ) ;
extern int fputc(int __c , FILE *__stream ) ;
extern int getc(FILE *__stream ) ;
extern int fgetc(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  vsscanf)(char const   * __restrict  __s ,
                                                                            char const   * __restrict  __format ,
                                                                            __gnuc_va_list __arg ) ;
extern int ( /* format attribute */  vscanf)(char const   * __restrict  __format ,
                                             __gnuc_va_list __arg ) ;
extern int ( /* format attribute */  vfscanf)(FILE * __restrict  __s ,
                                              char const   * __restrict  __format ,
                                              __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int sscanf(char const   * __restrict  __s ,
                                                char const   * __restrict  __format 
                                                , ...) ;
extern int scanf(char const   * __restrict  __format  , ...) ;
extern int fscanf(FILE * __restrict  __stream ,
                  char const   * __restrict  __format  , ...) ;
extern int ( /* format attribute */  dprintf)(int __fd ,
                                              char const   * __restrict  __fmt 
                                              , ...) ;
extern int ( /* format attribute */  vdprintf)(int __fd ,
                                               char const   * __restrict  __fmt ,
                                               __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  asprintf)(char ** __restrict  __ptr ,
                                                                             char const   * __restrict  __fmt 
                                                                             , ...) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  __asprintf)(char ** __restrict  __ptr ,
                                                                               char const   * __restrict  __fmt 
                                                                               , ...) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  vasprintf)(char ** __restrict  __ptr ,
                                                                              char const   * __restrict  __f ,
                                                                              __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  vsnprintf)(char * __restrict  __s ,
                                                                              size_t __maxlen ,
                                                                              char const   * __restrict  __format ,
                                                                              __gnuc_va_list __arg ) ;
extern  __attribute__((__nothrow__)) int ( /* format attribute */  snprintf)(char * __restrict  __s ,
                                                                             size_t __maxlen ,
                                                                             char const   * __restrict  __format 
                                                                             , ...) ;
extern  __attribute__((__nothrow__)) int vsprintf(char * __restrict  __s ,
                                                  char const   * __restrict  __format ,
                                                  __gnuc_va_list __arg ) ;
extern int printf(char const   * __restrict  __format  , ...) ;
extern  __attribute__((__nothrow__)) void setlinebuf(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) void setbuffer(FILE * __restrict  __stream ,
                                                    char * __restrict  __buf ,
                                                    size_t __size ) ;
extern  __attribute__((__nothrow__)) int setvbuf(FILE * __restrict  __stream ,
                                                 char * __restrict  __buf ,
                                                 int __modes , size_t __n ) ;
extern  __attribute__((__nothrow__)) void setbuf(FILE * __restrict  __stream ,
                                                 char * __restrict  __buf ) ;
extern  __attribute__((__nothrow__)) FILE *open_memstream(char **__bufloc ,
                                                          size_t *__sizeloc ) ;
extern  __attribute__((__nothrow__)) FILE *fmemopen(void *__s , size_t __len ,
                                                    char const   *__modes ) ;
struct __anonstruct__IO_cookie_io_functions_t_33;
typedef struct __anonstruct__IO_cookie_io_functions_t_33 _IO_cookie_io_functions_t;
extern  __attribute__((__nothrow__)) FILE *fopencookie(void * __restrict  __magic_cookie ,
                                                       char const   * __restrict  __modes ,
                                                       _IO_cookie_io_functions_t __io_funcs ) ;
extern  __attribute__((__nothrow__)) FILE *fdopen(int __fd ,
                                                  char const   *__modes ) ;
extern FILE *freopen64(char const   * __restrict  __filename ,
                       char const   * __restrict  __modes ,
                       FILE * __restrict  __stream ) ;
extern FILE *fopen64(char const   * __restrict  __filename ,
                     char const   * __restrict  __modes ) ;
extern FILE *freopen(char const   * __restrict  __filename ,
                     char const   * __restrict  __modes ,
                     FILE * __restrict  __stream ) ;
extern int fcloseall(void) ;
extern int fflush_unlocked(FILE *__stream ) ;
extern  __attribute__((__nothrow__)) char *tempnam(char const   *__dir ,
                                                   char const   *__pfx )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) char *tmpnam_r(char *__s ) ;
extern  __attribute__((__nothrow__)) char *tmpnam(char *__s ) ;
extern FILE *tmpfile64(void) ;
extern FILE *tmpfile(void) ;
extern  __attribute__((__nothrow__)) int renameat(int __oldfd ,
                                                  char const   *__old ,
                                                  int __newfd ,
                                                  char const   *__new ) ;
extern  __attribute__((__nothrow__)) int rename(char const   *__old ,
                                                char const   *__new ) ;
extern  __attribute__((__nothrow__)) int remove(char const   *__filename ) ;
extern  __attribute__((__nothrow__)) void _IO_free_backup_area(_IO_FILE * ) ;
extern __off64_t _IO_seekpos(_IO_FILE * , __off64_t  , int  ) ;
extern __off64_t _IO_seekoff(_IO_FILE * , __off64_t  , int  , int  ) ;
extern size_t _IO_sgetn(_IO_FILE * , void * , size_t  ) ;
extern __ssize_t _IO_padn(_IO_FILE * , int  , __ssize_t  ) ;
extern int _IO_vfprintf(_IO_FILE * __restrict   , char const   * __restrict   ,
                        __gnuc_va_list  ) ;
extern int _IO_vfscanf(_IO_FILE * __restrict   , char const   * __restrict   ,
                       __gnuc_va_list  , int * __restrict   ) ;
extern  __attribute__((__nothrow__)) int _IO_ftrylockfile(_IO_FILE * ) ;
extern  __attribute__((__nothrow__)) void _IO_funlockfile(_IO_FILE * ) ;
extern  __attribute__((__nothrow__)) void _IO_flockfile(_IO_FILE * ) ;
extern int _IO_peekc_locked(_IO_FILE *__fp ) ;
extern  __attribute__((__nothrow__)) int _IO_ferror(_IO_FILE *__fp ) ;
extern  __attribute__((__nothrow__)) int _IO_feof(_IO_FILE *__fp ) ;
extern int __underflow(_IO_FILE * ) ;
struct _IO_cookie_file;
extern void _IO_cookie_init(struct _IO_cookie_file *__cfile , int __read_write ,
                            void *__cookie , _IO_cookie_io_functions_t __fns ) ;
struct _IO_FILE_plus;
extern struct _IO_FILE_plus _IO_2_1_stderr_ ;
extern struct _IO_FILE_plus _IO_2_1_stdout_ ;
extern struct _IO_FILE_plus _IO_2_1_stdin_ ;
union __anonunion_28 {
   double __d ;
   int __i[2] ;
};
__inline extern  __attribute__((__nothrow__)) int __finite(double __x )  __attribute__((__const__)) ;
__inline extern int __finite(double __x ) 
{ union __anonunion_28 __constr_expr_0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "226\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __constr_expr_0.__d = __x;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "227\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )((((unsigned int )__constr_expr_0.__i[1] | 0x800fffffu) + 1U) >> 31));
}
}
__inline extern  __attribute__((__nothrow__)) int __finite(double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long long llrintl(long double __x ) ;
__inline extern long long llrintl(long double __x ) 
{ long long __llrintres ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "224\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fistpll %0": "=m" (__llrintres): "t" (__x): "st");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "225\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__llrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long long llrintl(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long long llrint(double __x ) ;
__inline extern long long llrint(double __x ) 
{ long long __llrintres ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "222\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fistpll %0": "=m" (__llrintres): "t" (__x): "st");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "223\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__llrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long long llrint(double __x ) ;
__inline extern  __attribute__((__nothrow__)) long long llrintf(float __x ) ;
__inline extern long long llrintf(float __x ) 
{ long long __llrintres ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "220\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fistpll %0": "=m" (__llrintres): "t" (__x): "st");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "221\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__llrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long long llrintf(float __x ) ;
__inline extern  __attribute__((__nothrow__)) long lrintl(long double __x ) ;
__inline extern long lrintl(long double __x ) 
{ long __lrintres ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "218\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fistpl %0": "=m" (__lrintres): "t" (__x): "st");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "219\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__lrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long lrintl(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long lrint(double __x ) ;
__inline extern long lrint(double __x ) 
{ long __lrintres ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "216\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fistpl %0": "=m" (__lrintres): "t" (__x): "st");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "217\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__lrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long lrint(double __x ) ;
__inline extern  __attribute__((__nothrow__)) long lrintf(float __x ) ;
__inline extern long lrintf(float __x ) 
{ long __lrintres ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "214\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fistpl %0": "=m" (__lrintres): "t" (__x): "st");
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "215\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__lrintres);
}
}
__inline extern  __attribute__((__nothrow__)) long lrintf(float __x ) ;
__inline extern  __attribute__((__nothrow__)) long double ceill(long double __x )  __attribute__((__const__)) ;
__inline extern long double ceill(long double __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "212\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0800, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "213\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__value);
}
}
__inline extern  __attribute__((__nothrow__)) long double ceill(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) float ceilf(float __x )  __attribute__((__const__)) ;
__inline extern float ceilf(float __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "210\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0800, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "211\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((float )__value);
}
}
__inline extern  __attribute__((__nothrow__)) float ceilf(float __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) double ceil(double __x )  __attribute__((__const__)) ;
__inline extern double ceil(double __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "208\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0800, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "209\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((double )__value);
}
}
__inline extern  __attribute__((__nothrow__)) double ceil(double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long double floorl(long double __x )  __attribute__((__const__)) ;
__inline extern long double floorl(long double __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "206\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0400, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "207\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__value);
}
}
__inline extern  __attribute__((__nothrow__)) long double floorl(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) float floorf(float __x )  __attribute__((__const__)) ;
__inline extern float floorf(float __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "204\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0400, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "205\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((float )__value);
}
}
__inline extern  __attribute__((__nothrow__)) float floorf(float __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) double floor(double __x )  __attribute__((__const__)) ;
__inline extern double floor(double __x ) 
{ register long double __value ;
  register int __ignore ;
  unsigned short __cw ;
  unsigned short __cwtmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "202\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __asm__  volatile   ("fnstcw %3\n\t"
                       "movzwl %3, %1\n\t"
                       "andl $0xf3ff, %1\n\t"
                       "orl $0x0400, %1\n\t"
                       "movw %w1, %2\n\t"
                       "fldcw %2\n\t"
                       "frndint\n\t"
                       "fldcw %3": "=t" (__value), "=&q" (__ignore),
                       "=m" (__cwtmp), "=m" (__cw): "0" (__x));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "203\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((double )__value);
}
}
__inline extern  __attribute__((__nothrow__)) double floor(double __x )  __attribute__((__const__)) ;
union __anonunion___n_27 {
   long double __xld ;
   unsigned int __xi[3] ;
};
__inline extern  __attribute__((__nothrow__)) long double __sgn1l(long double __x ) ;
__inline extern long double __sgn1l(long double __x ) 
{ union __anonunion___n_27 __n ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "197\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __n.__xld = __x;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "198\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __n.__xi[2] = (__n.__xi[2] & 32768U) | 16383U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "199\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __n.__xi[1] = 0x80000000;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "200\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __n.__xi[0] = 0U;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "201\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__n.__xld);
}
}
__inline extern  __attribute__((__nothrow__)) long double __sgn1l(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long double __fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern long double __fabsl(long double __x ) 
{ long double tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "195\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_fabsl(__x);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "196\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long double __fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long double fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern long double fabsl(long double __x ) 
{ long double tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "193\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_fabsl(__x);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "194\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long double fabsl(long double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) float fabsf(float __x )  __attribute__((__const__)) ;
__inline extern float fabsf(float __x ) 
{ float tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "191\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_fabsf(__x);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "192\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) float fabsf(float __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) double fabs(double __x )  __attribute__((__const__)) ;
__inline extern double fabs(double __x ) 
{ double tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "189\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_fabs(__x);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "190\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) double fabs(double __x )  __attribute__((__const__)) ;
__inline extern  __attribute__((__nothrow__)) long double __atan2l(long double __y ,
                                                                   long double __x ) ;
__inline extern long double __atan2l(long double __y , long double __x ) 
{ long double tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "187\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = __builtin_atan2l(__y, __x);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "188\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long double __atan2l(long double __y ,
                                                                   long double __x ) ;
__inline extern  __attribute__((__nothrow__)) long double __sgnl(long double __x ) ;
__inline extern long double __sgnl(long double __x ) 
{ double tmp ;
  double tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "185\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (__x == (long double )0.0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "180\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = 0.0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "183\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (__x > (long double )0.0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "181\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = 1.0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "182\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = - 1.0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "184\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = tmp;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "186\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((long double )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) long double __sgnl(long double __x ) ;
__inline extern  __attribute__((__nothrow__)) float __sgnf(float __x ) ;
__inline extern float __sgnf(float __x ) 
{ double tmp ;
  double tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "178\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((double )__x == 0.0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "173\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = 0.0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "176\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((double )__x > 0.0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "174\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = 1.0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "175\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = - 1.0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "177\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = tmp;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "179\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((float )tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) float __sgnf(float __x ) ;
__inline extern  __attribute__((__nothrow__)) double __sgn(double __x ) ;
__inline extern double __sgn(double __x ) 
{ double tmp ;
  double tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "171\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if (__x == 0.0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "166\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = 0.0;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "169\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if (__x > 0.0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "167\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = 1.0;
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "168\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = - 1.0;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "170\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = tmp;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "172\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp___0);
}
}
__inline extern  __attribute__((__nothrow__)) double __sgn(double __x ) ;
union __anonunion___u_26 {
   long double __l ;
   int __i[3] ;
};
__inline extern  __attribute__((__nothrow__)) int __signbitl(long double __x )  __attribute__((__const__)) ;
__inline extern int __signbitl(long double __x ) 
{ union __anonunion___u_26 __u ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "164\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __u.__l = __x;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "165\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((__u.__i[2] & 0x8000) != 0);
}
}
__inline extern  __attribute__((__nothrow__)) int __signbitl(long double __x )  __attribute__((__const__)) ;
union __anonunion___u_25 {
   double __d ;
   int __i[2] ;
};
__inline extern  __attribute__((__nothrow__)) int __signbit(double __x )  __attribute__((__const__)) ;
__inline extern int __signbit(double __x ) 
{ union __anonunion___u_25 __u ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "162\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __u.__d = __x;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "163\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__u.__i[1] < 0);
}
}
__inline extern  __attribute__((__nothrow__)) int __signbit(double __x )  __attribute__((__const__)) ;
union __anonunion___u_24 {
   float __f ;
   int __i ;
};
__inline extern  __attribute__((__nothrow__)) int __signbitf(float __x )  __attribute__((__const__)) ;
__inline extern int __signbitf(float __x ) 
{ union __anonunion___u_24 __u ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "160\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __u.__f = __x;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "161\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__u.__i < 0);
}
}
__inline extern  __attribute__((__nothrow__)) int __signbitf(float __x )  __attribute__((__const__)) ;
struct exception;
extern int matherr(struct exception *__exc ) ;
enum __anonenum__LIB_VERSION_TYPE_23;
typedef enum __anonenum__LIB_VERSION_TYPE_23 _LIB_VERSION_TYPE;
extern _LIB_VERSION_TYPE _LIB_VERSION ;
extern int signgam ;
extern  __attribute__((__nothrow__)) long double __scalbl(long double __x ,
                                                          long double __n ) ;
extern  __attribute__((__nothrow__)) long double scalbl(long double __x ,
                                                        long double __n ) ;
extern  __attribute__((__nothrow__)) long double __fmal(long double __x ,
                                                        long double __y ,
                                                        long double __z ) ;
extern  __attribute__((__nothrow__)) long double fmal(long double __x ,
                                                      long double __y ,
                                                      long double __z ) ;
extern  __attribute__((__nothrow__)) int __fpclassifyl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __fminl(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double fminl(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __fmaxl(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double fmaxl(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __fdiml(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double fdiml(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long long __llroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long long llroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long __lroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long lroundl(long double __x ) ;
extern  __attribute__((__nothrow__)) long long __llrintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long __lrintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __remquol(long double __x ,
                                                           long double __y ,
                                                           int *__quo ) ;
extern  __attribute__((__nothrow__)) long double remquol(long double __x ,
                                                         long double __y ,
                                                         int *__quo ) ;
extern  __attribute__((__nothrow__)) long double __truncl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double truncl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __roundl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double roundl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __nearbyintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double nearbyintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __scalblnl(long double __x ,
                                                            long __n ) ;
extern  __attribute__((__nothrow__)) long double scalblnl(long double __x ,
                                                          long __n ) ;
extern  __attribute__((__nothrow__)) int __ilogbl(long double __x ) ;
extern  __attribute__((__nothrow__)) int ilogbl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __scalbnl(long double __x ,
                                                           int __n ) ;
extern  __attribute__((__nothrow__)) long double scalbnl(long double __x ,
                                                         int __n ) ;
extern  __attribute__((__nothrow__)) long double __remainderl(long double __x ,
                                                              long double __y ) ;
extern  __attribute__((__nothrow__)) long double remainderl(long double __x ,
                                                            long double __y ) ;
extern  __attribute__((__nothrow__)) long double __nexttowardl(long double __x ,
                                                               long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double nexttowardl(long double __x ,
                                                             long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __nextafterl(long double __x ,
                                                              long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double nextafterl(long double __x ,
                                                            long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __rintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double rintl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __lgammal_r(long double  ,
                                                             int *__signgamp ) ;
extern  __attribute__((__nothrow__)) long double lgammal_r(long double  ,
                                                           int *__signgamp ) ;
extern  __attribute__((__nothrow__)) long double __gammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double gammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double __tgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double tgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double __lgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double lgammal(long double  ) ;
extern  __attribute__((__nothrow__)) long double __erfcl(long double  ) ;
extern  __attribute__((__nothrow__)) long double erfcl(long double  ) ;
extern  __attribute__((__nothrow__)) long double __erfl(long double  ) ;
extern  __attribute__((__nothrow__)) long double erfl(long double  ) ;
extern  __attribute__((__nothrow__)) long double __ynl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double ynl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double __y1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double y1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __y0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double y0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __jnl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double jnl(int  , long double  ) ;
extern  __attribute__((__nothrow__)) long double __j1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double j1l(long double  ) ;
extern  __attribute__((__nothrow__)) long double __j0l(long double  ) ;
extern  __attribute__((__nothrow__)) long double j0l(long double  ) ;
extern  __attribute__((__nothrow__)) int isnanl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isnanl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __nanl(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double nanl(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __copysignl(long double __x ,
                                                             long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double copysignl(long double __x ,
                                                           long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __significandl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double significandl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __dreml(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double dreml(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) int finitel(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isinfl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __finitel(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isinfl(long double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __fmodl(long double __x ,
                                                         long double __y ) ;
extern  __attribute__((__nothrow__)) long double fmodl(long double __x ,
                                                       long double __y ) ;
extern  __attribute__((__nothrow__)) long double __floorl(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __ceill(long double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long double __cbrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double cbrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __hypotl(long double __x ,
                                                          long double __y ) ;
extern  __attribute__((__nothrow__)) long double hypotl(long double __x ,
                                                        long double __y ) ;
extern  __attribute__((__nothrow__)) long double __sqrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double sqrtl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __powl(long double __x ,
                                                        long double __y ) ;
extern  __attribute__((__nothrow__)) long double powl(long double __x ,
                                                      long double __y ) ;
extern  __attribute__((__nothrow__)) long double __log2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double log2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __exp2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double exp2l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __logbl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double logbl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __log1pl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double log1pl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __expm1l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double expm1l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __pow10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double pow10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __exp10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double exp10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __modfl(long double __x ,
                                                         long double *__iptr ) ;
extern  __attribute__((__nothrow__)) long double modfl(long double __x ,
                                                       long double *__iptr ) ;
extern  __attribute__((__nothrow__)) long double __log10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double log10l(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __logl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double logl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __ldexpl(long double __x ,
                                                          int __exponent ) ;
extern  __attribute__((__nothrow__)) long double ldexpl(long double __x ,
                                                        int __exponent ) ;
extern  __attribute__((__nothrow__)) long double __frexpl(long double __x ,
                                                          int *__exponent ) ;
extern  __attribute__((__nothrow__)) long double frexpl(long double __x ,
                                                        int *__exponent ) ;
extern  __attribute__((__nothrow__)) long double __expl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double expl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __atanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double atanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __asinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double asinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __acoshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double acoshl(long double __x ) ;
extern  __attribute__((__nothrow__)) void __sincosl(long double __x ,
                                                    long double *__sinx ,
                                                    long double *__cosx ) ;
extern  __attribute__((__nothrow__)) void sincosl(long double __x ,
                                                  long double *__sinx ,
                                                  long double *__cosx ) ;
extern  __attribute__((__nothrow__)) long double __tanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double tanhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __sinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double sinhl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __coshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double coshl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __tanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double tanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __sinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double sinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __cosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double cosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double atan2l(long double __y ,
                                                        long double __x ) ;
extern  __attribute__((__nothrow__)) long double __atanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double atanl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __asinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double asinl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double __acosl(long double __x ) ;
extern  __attribute__((__nothrow__)) long double acosl(long double __x ) ;
extern  __attribute__((__nothrow__)) float __scalbf(float __x , float __n ) ;
extern  __attribute__((__nothrow__)) float scalbf(float __x , float __n ) ;
extern  __attribute__((__nothrow__)) float __fmaf(float __x , float __y ,
                                                  float __z ) ;
extern  __attribute__((__nothrow__)) float fmaf(float __x , float __y ,
                                                float __z ) ;
extern  __attribute__((__nothrow__)) int __fpclassifyf(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __fminf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float fminf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __fmaxf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float fmaxf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __fdimf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float fdimf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) long long __llroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long long llroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long __lroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long lroundf(float __x ) ;
extern  __attribute__((__nothrow__)) long long __llrintf(float __x ) ;
extern  __attribute__((__nothrow__)) long __lrintf(float __x ) ;
extern  __attribute__((__nothrow__)) float __remquof(float __x , float __y ,
                                                     int *__quo ) ;
extern  __attribute__((__nothrow__)) float remquof(float __x , float __y ,
                                                   int *__quo ) ;
extern  __attribute__((__nothrow__)) float __truncf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float truncf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __roundf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float roundf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __nearbyintf(float __x ) ;
extern  __attribute__((__nothrow__)) float nearbyintf(float __x ) ;
extern  __attribute__((__nothrow__)) float __scalblnf(float __x , long __n ) ;
extern  __attribute__((__nothrow__)) float scalblnf(float __x , long __n ) ;
extern  __attribute__((__nothrow__)) int __ilogbf(float __x ) ;
extern  __attribute__((__nothrow__)) int ilogbf(float __x ) ;
extern  __attribute__((__nothrow__)) float __scalbnf(float __x , int __n ) ;
extern  __attribute__((__nothrow__)) float scalbnf(float __x , int __n ) ;
extern  __attribute__((__nothrow__)) float __remainderf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float remainderf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __nexttowardf(float __x ,
                                                         long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float nexttowardf(float __x ,
                                                       long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __nextafterf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float nextafterf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __rintf(float __x ) ;
extern  __attribute__((__nothrow__)) float rintf(float __x ) ;
extern  __attribute__((__nothrow__)) float __lgammaf_r(float  , int *__signgamp ) ;
extern  __attribute__((__nothrow__)) float lgammaf_r(float  , int *__signgamp ) ;
extern  __attribute__((__nothrow__)) float __gammaf(float  ) ;
extern  __attribute__((__nothrow__)) float gammaf(float  ) ;
extern  __attribute__((__nothrow__)) float __tgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float tgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float __lgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float lgammaf(float  ) ;
extern  __attribute__((__nothrow__)) float __erfcf(float  ) ;
extern  __attribute__((__nothrow__)) float erfcf(float  ) ;
extern  __attribute__((__nothrow__)) float __erff(float  ) ;
extern  __attribute__((__nothrow__)) float erff(float  ) ;
extern  __attribute__((__nothrow__)) float __ynf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float ynf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float __y1f(float  ) ;
extern  __attribute__((__nothrow__)) float y1f(float  ) ;
extern  __attribute__((__nothrow__)) float __y0f(float  ) ;
extern  __attribute__((__nothrow__)) float y0f(float  ) ;
extern  __attribute__((__nothrow__)) float __jnf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float jnf(int  , float  ) ;
extern  __attribute__((__nothrow__)) float __j1f(float  ) ;
extern  __attribute__((__nothrow__)) float j1f(float  ) ;
extern  __attribute__((__nothrow__)) float __j0f(float  ) ;
extern  __attribute__((__nothrow__)) float j0f(float  ) ;
extern  __attribute__((__nothrow__)) int isnanf(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isnanf(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __nanf(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float nanf(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __copysignf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float copysignf(float __x , float __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __significandf(float __x ) ;
extern  __attribute__((__nothrow__)) float significandf(float __x ) ;
extern  __attribute__((__nothrow__)) float __dremf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float dremf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) int finitef(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isinff(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __finitef(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isinff(float __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __fmodf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float fmodf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __floorf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __fabsf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __ceilf(float __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) float __cbrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float cbrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float __hypotf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float hypotf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __sqrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float sqrtf(float __x ) ;
extern  __attribute__((__nothrow__)) float __powf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float powf(float __x , float __y ) ;
extern  __attribute__((__nothrow__)) float __log2f(float __x ) ;
extern  __attribute__((__nothrow__)) float log2f(float __x ) ;
extern  __attribute__((__nothrow__)) float __exp2f(float __x ) ;
extern  __attribute__((__nothrow__)) float exp2f(float __x ) ;
extern  __attribute__((__nothrow__)) float __logbf(float __x ) ;
extern  __attribute__((__nothrow__)) float logbf(float __x ) ;
extern  __attribute__((__nothrow__)) float __log1pf(float __x ) ;
extern  __attribute__((__nothrow__)) float log1pf(float __x ) ;
extern  __attribute__((__nothrow__)) float __expm1f(float __x ) ;
extern  __attribute__((__nothrow__)) float expm1f(float __x ) ;
extern  __attribute__((__nothrow__)) float __pow10f(float __x ) ;
extern  __attribute__((__nothrow__)) float pow10f(float __x ) ;
extern  __attribute__((__nothrow__)) float __exp10f(float __x ) ;
extern  __attribute__((__nothrow__)) float exp10f(float __x ) ;
extern  __attribute__((__nothrow__)) float __modff(float __x , float *__iptr ) ;
extern  __attribute__((__nothrow__)) float modff(float __x , float *__iptr ) ;
extern  __attribute__((__nothrow__)) float __log10f(float __x ) ;
extern  __attribute__((__nothrow__)) float log10f(float __x ) ;
extern  __attribute__((__nothrow__)) float __logf(float __x ) ;
extern  __attribute__((__nothrow__)) float logf(float __x ) ;
extern  __attribute__((__nothrow__)) float __ldexpf(float __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) float ldexpf(float __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) float __frexpf(float __x , int *__exponent ) ;
extern  __attribute__((__nothrow__)) float frexpf(float __x , int *__exponent ) ;
extern  __attribute__((__nothrow__)) float __expf(float __x ) ;
extern  __attribute__((__nothrow__)) float expf(float __x ) ;
extern  __attribute__((__nothrow__)) float __atanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float atanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __asinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float asinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __acoshf(float __x ) ;
extern  __attribute__((__nothrow__)) float acoshf(float __x ) ;
extern  __attribute__((__nothrow__)) void __sincosf(float __x , float *__sinx ,
                                                    float *__cosx ) ;
extern  __attribute__((__nothrow__)) void sincosf(float __x , float *__sinx ,
                                                  float *__cosx ) ;
extern  __attribute__((__nothrow__)) float __tanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float tanhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __sinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float sinhf(float __x ) ;
extern  __attribute__((__nothrow__)) float __coshf(float __x ) ;
extern  __attribute__((__nothrow__)) float coshf(float __x ) ;
extern  __attribute__((__nothrow__)) float __tanf(float __x ) ;
extern  __attribute__((__nothrow__)) float tanf(float __x ) ;
extern  __attribute__((__nothrow__)) float __sinf(float __x ) ;
extern  __attribute__((__nothrow__)) float sinf(float __x ) ;
extern  __attribute__((__nothrow__)) float __cosf(float __x ) ;
extern  __attribute__((__nothrow__)) float cosf(float __x ) ;
extern  __attribute__((__nothrow__)) float __atan2f(float __y , float __x ) ;
extern  __attribute__((__nothrow__)) float atan2f(float __y , float __x ) ;
extern  __attribute__((__nothrow__)) float __atanf(float __x ) ;
extern  __attribute__((__nothrow__)) float atanf(float __x ) ;
extern  __attribute__((__nothrow__)) float __asinf(float __x ) ;
extern  __attribute__((__nothrow__)) float asinf(float __x ) ;
extern  __attribute__((__nothrow__)) float __acosf(float __x ) ;
extern  __attribute__((__nothrow__)) float acosf(float __x ) ;
extern  __attribute__((__nothrow__)) double __scalb(double __x , double __n ) ;
extern  __attribute__((__nothrow__)) double scalb(double __x , double __n ) ;
extern  __attribute__((__nothrow__)) double __fma(double __x , double __y ,
                                                  double __z ) ;
extern  __attribute__((__nothrow__)) double fma(double __x , double __y ,
                                                double __z ) ;
extern  __attribute__((__nothrow__)) int __fpclassify(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __fmin(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double fmin(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __fmax(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double fmax(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __fdim(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double fdim(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) long long __llround(double __x ) ;
extern  __attribute__((__nothrow__)) long long llround(double __x ) ;
extern  __attribute__((__nothrow__)) long __lround(double __x ) ;
extern  __attribute__((__nothrow__)) long lround(double __x ) ;
extern  __attribute__((__nothrow__)) long long __llrint(double __x ) ;
extern  __attribute__((__nothrow__)) long __lrint(double __x ) ;
extern  __attribute__((__nothrow__)) double __remquo(double __x , double __y ,
                                                     int *__quo ) ;
extern  __attribute__((__nothrow__)) double remquo(double __x , double __y ,
                                                   int *__quo ) ;
extern  __attribute__((__nothrow__)) double __trunc(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double trunc(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __round(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double round(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __nearbyint(double __x ) ;
extern  __attribute__((__nothrow__)) double nearbyint(double __x ) ;
extern  __attribute__((__nothrow__)) double __scalbln(double __x , long __n ) ;
extern  __attribute__((__nothrow__)) double scalbln(double __x , long __n ) ;
extern  __attribute__((__nothrow__)) int __ilogb(double __x ) ;
extern  __attribute__((__nothrow__)) int ilogb(double __x ) ;
extern  __attribute__((__nothrow__)) double __scalbn(double __x , int __n ) ;
extern  __attribute__((__nothrow__)) double scalbn(double __x , int __n ) ;
extern  __attribute__((__nothrow__)) double __remainder(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double remainder(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __nexttoward(double __x ,
                                                         long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double nexttoward(double __x ,
                                                       long double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __nextafter(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double nextafter(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __rint(double __x ) ;
extern  __attribute__((__nothrow__)) double rint(double __x ) ;
extern  __attribute__((__nothrow__)) double __lgamma_r(double  ,
                                                       int *__signgamp ) ;
extern  __attribute__((__nothrow__)) double lgamma_r(double  , int *__signgamp ) ;
extern  __attribute__((__nothrow__)) double __gamma(double  ) ;
extern  __attribute__((__nothrow__)) double gamma(double  ) ;
extern  __attribute__((__nothrow__)) double __tgamma(double  ) ;
extern  __attribute__((__nothrow__)) double tgamma(double  ) ;
extern  __attribute__((__nothrow__)) double __lgamma(double  ) ;
extern  __attribute__((__nothrow__)) double lgamma(double  ) ;
extern  __attribute__((__nothrow__)) double __erfc(double  ) ;
extern  __attribute__((__nothrow__)) double erfc(double  ) ;
extern  __attribute__((__nothrow__)) double __erf(double  ) ;
extern  __attribute__((__nothrow__)) double erf(double  ) ;
extern  __attribute__((__nothrow__)) double __yn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double yn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double __y1(double  ) ;
extern  __attribute__((__nothrow__)) double y1(double  ) ;
extern  __attribute__((__nothrow__)) double __y0(double  ) ;
extern  __attribute__((__nothrow__)) double y0(double  ) ;
extern  __attribute__((__nothrow__)) double __jn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double jn(int  , double  ) ;
extern  __attribute__((__nothrow__)) double __j1(double  ) ;
extern  __attribute__((__nothrow__)) double j1(double  ) ;
extern  __attribute__((__nothrow__)) double __j0(double  ) ;
extern  __attribute__((__nothrow__)) double j0(double  ) ;
extern  __attribute__((__nothrow__)) int isnan(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isnan(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __nan(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double nan(char const   *__tagb )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __copysign(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double copysign(double __x , double __y )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __significand(double __x ) ;
extern  __attribute__((__nothrow__)) double significand(double __x ) ;
extern  __attribute__((__nothrow__)) double __drem(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double drem(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) int finite(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int isinf(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int __isinf(double __value )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __fmod(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double fmod(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __floor(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __fabs(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __ceil(double __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) double __cbrt(double __x ) ;
extern  __attribute__((__nothrow__)) double cbrt(double __x ) ;
extern  __attribute__((__nothrow__)) double __hypot(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double hypot(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __sqrt(double __x ) ;
extern  __attribute__((__nothrow__)) double sqrt(double __x ) ;
extern  __attribute__((__nothrow__)) double __pow(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double pow(double __x , double __y ) ;
extern  __attribute__((__nothrow__)) double __log2(double __x ) ;
extern  __attribute__((__nothrow__)) double log2(double __x ) ;
extern  __attribute__((__nothrow__)) double __exp2(double __x ) ;
extern  __attribute__((__nothrow__)) double exp2(double __x ) ;
extern  __attribute__((__nothrow__)) double __logb(double __x ) ;
extern  __attribute__((__nothrow__)) double logb(double __x ) ;
extern  __attribute__((__nothrow__)) double __log1p(double __x ) ;
extern  __attribute__((__nothrow__)) double log1p(double __x ) ;
extern  __attribute__((__nothrow__)) double __expm1(double __x ) ;
extern  __attribute__((__nothrow__)) double expm1(double __x ) ;
extern  __attribute__((__nothrow__)) double __pow10(double __x ) ;
extern  __attribute__((__nothrow__)) double pow10(double __x ) ;
extern  __attribute__((__nothrow__)) double __exp10(double __x ) ;
extern  __attribute__((__nothrow__)) double exp10(double __x ) ;
extern  __attribute__((__nothrow__)) double __modf(double __x , double *__iptr ) ;
extern  __attribute__((__nothrow__)) double modf(double __x , double *__iptr ) ;
extern  __attribute__((__nothrow__)) double __log10(double __x ) ;
extern  __attribute__((__nothrow__)) double log10(double __x ) ;
extern  __attribute__((__nothrow__)) double __log(double __x ) ;
extern  __attribute__((__nothrow__)) double log(double __x ) ;
extern  __attribute__((__nothrow__)) double __ldexp(double __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) double ldexp(double __x , int __exponent ) ;
extern  __attribute__((__nothrow__)) double __frexp(double __x ,
                                                    int *__exponent ) ;
extern  __attribute__((__nothrow__)) double frexp(double __x , int *__exponent ) ;
extern  __attribute__((__nothrow__)) double __exp(double __x ) ;
extern  __attribute__((__nothrow__)) double exp(double __x ) ;
extern  __attribute__((__nothrow__)) double __atanh(double __x ) ;
extern  __attribute__((__nothrow__)) double atanh(double __x ) ;
extern  __attribute__((__nothrow__)) double __asinh(double __x ) ;
extern  __attribute__((__nothrow__)) double asinh(double __x ) ;
extern  __attribute__((__nothrow__)) double __acosh(double __x ) ;
extern  __attribute__((__nothrow__)) double acosh(double __x ) ;
extern  __attribute__((__nothrow__)) void __sincos(double __x , double *__sinx ,
                                                   double *__cosx ) ;
extern  __attribute__((__nothrow__)) void sincos(double __x , double *__sinx ,
                                                 double *__cosx ) ;
extern  __attribute__((__nothrow__)) double __tanh(double __x ) ;
extern  __attribute__((__nothrow__)) double tanh(double __x ) ;
extern  __attribute__((__nothrow__)) double __sinh(double __x ) ;
extern  __attribute__((__nothrow__)) double sinh(double __x ) ;
extern  __attribute__((__nothrow__)) double __cosh(double __x ) ;
extern  __attribute__((__nothrow__)) double cosh(double __x ) ;
extern  __attribute__((__nothrow__)) double __tan(double __x ) ;
extern  __attribute__((__nothrow__)) double tan(double __x ) ;
extern  __attribute__((__nothrow__)) double __sin(double __x ) ;
extern  __attribute__((__nothrow__)) double sin(double __x ) ;
extern  __attribute__((__nothrow__)) double __cos(double __x ) ;
extern  __attribute__((__nothrow__)) double cos(double __x ) ;
extern  __attribute__((__nothrow__)) double __atan2(double __y , double __x ) ;
extern  __attribute__((__nothrow__)) double atan2(double __y , double __x ) ;
extern  __attribute__((__nothrow__)) double __atan(double __x ) ;
extern  __attribute__((__nothrow__)) double atan(double __x ) ;
extern  __attribute__((__nothrow__)) double __asin(double __x ) ;
extern  __attribute__((__nothrow__)) double asin(double __x ) ;
extern  __attribute__((__nothrow__)) double __acos(double __x ) ;
extern  __attribute__((__nothrow__)) double acos(double __x ) ;
extern  __attribute__((__nothrow__)) char *__strndup(char const   *__string ,
                                                     size_t __n )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) char *__strdup(char const   *__string )  __attribute__((__malloc__)) ;
__inline extern char *__strsep_3c(char **__s , char __reject1 , char __reject2 ,
                                  char __reject3 ) 
{ register char *__retval ;
  register char *__cp ;
  char *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "157\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __retval = *__s;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "158\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )__retval != (unsigned int )((void *)0)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "153\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __cp = __retval;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "154\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "150\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*__cp == 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "136\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __cp = (char *)((void *)0);
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "137\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "151\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*__cp == (int )__reject1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "138\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = __cp;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "139\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __cp ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "140\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp = (char )'\000';
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "149\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*__cp == (int )__reject2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "141\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp = __cp;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "142\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          __cp ++;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "143\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *tmp = (char )'\000';
          break;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "148\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int )*__cp == (int )__reject3) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "144\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            tmp = __cp;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "145\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            __cp ++;
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "146\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }
            *tmp = (char )'\000';
            break;
          } else {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "147\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          }
        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "152\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __cp ++;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "155\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *__s = __cp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "156\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "159\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__retval);
}
}
__inline extern char *__strsep_3c(char **__s , char __reject1 , char __reject2 ,
                                  char __reject3 ) ;
__inline extern char *__strsep_2c(char **__s , char __reject1 , char __reject2 ) 
{ register char *__retval ;
  register char *__cp ;
  char *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "133\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __retval = *__s;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "134\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )__retval != (unsigned int )((void *)0)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "129\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __cp = __retval;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "130\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "126\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*__cp == 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "116\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __cp = (char *)((void *)0);
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "117\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "127\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*__cp == (int )__reject1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "118\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        tmp = __cp;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "119\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        __cp ++;
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "120\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *tmp = (char )'\000';
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "125\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int )*__cp == (int )__reject2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "121\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          tmp = __cp;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "122\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          __cp ++;
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "123\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          *tmp = (char )'\000';
          break;
        } else {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "124\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        }
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "128\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __cp ++;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "131\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *__s = __cp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "132\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "135\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__retval);
}
}
__inline extern char *__strsep_2c(char **__s , char __reject1 , char __reject2 ) ;
__inline extern char *__strsep_1c(char **__s , char __reject ) 
{ register char *__retval ;
  char *tmp ;
  char *tmp___0 ;
  void *tmp___1 ;
  char *tmp___2 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "113\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __retval = *__s;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "114\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )__retval != (unsigned int )((void *)0)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "108\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___2 = __builtin_strchr(__retval, (int )__reject);
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "109\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp___0 = tmp___2;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "110\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    *__s = tmp___0;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "111\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((unsigned int )tmp___0 != (unsigned int )((void *)0)) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "104\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp = *__s;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "105\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      (*__s) ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "106\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      *tmp = (char )'\000';
    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "107\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "112\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "115\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__retval);
}
}
__inline extern char *__strsep_1c(char **__s , char __reject ) ;
extern char *__strsep_g(char **__stringp , char const   *__delim ) ;
__inline extern char *__strtok_r_1c(char *__s , char __sep , char **__nextp ) 
{ char *__result ;
  char *tmp ;
  char *tmp___0 ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "98\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((unsigned int )__s == (unsigned int )((void *)0)) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "81\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s = *__nextp;
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "82\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "99\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "84\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int )*__s == (int )__sep) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "83\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "85\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "100\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (char *)((void *)0);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "101\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int )*__s != 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "93\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = __s;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "94\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s ++;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "95\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result = tmp;
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "96\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    while (1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "89\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*__s != 0) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "86\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "90\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      tmp___0 = __s;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "91\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      __s ++;
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "92\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int )*tmp___0 == (int )__sep) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "87\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        *(__s + -1) = (char )'\000';
        break;
      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "88\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      }
    }
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "97\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }

  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "102\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  *__nextp = __s;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "103\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
__inline extern char *__strtok_r_1c(char *__s , char __sep , char **__nextp ) ;
__inline extern char *__strpbrk_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) 
{ char *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "78\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "74\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*__s != 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "73\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*__s != (int const   )__accept1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "72\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*__s != (int const   )__accept2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "71\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*__s != (int const   )__accept3) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "70\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            break;
          }
        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "75\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "79\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )*__s == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "76\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = (char *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "77\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = (char *)((unsigned int )__s);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "80\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern char *__strpbrk_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) ;
__inline extern char *__strpbrk_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) 
{ char *tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "67\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "63\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*__s != 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "62\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*__s != (int const   )__accept1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "61\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*__s != (int const   )__accept2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "60\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "64\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __s ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "68\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  if ((int const   )*__s == 0) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "65\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = (char *)((void *)0);
  } else {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "66\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    tmp = (char *)((unsigned int )__s);
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "69\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern char *__strpbrk_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) ;
__inline extern size_t __strspn_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) 
{ register size_t __result ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "57\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "58\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "55\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(__s + __result) == (int const   )__accept1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "50\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "54\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*(__s + __result) == (int const   )__accept2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "51\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "53\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*(__s + __result) == (int const   )__accept3) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "52\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "56\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "59\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strspn_c3(char const   *__s , int __accept1 ,
                                   int __accept2 , int __accept3 ) ;
__inline extern size_t __strspn_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) 
{ register size_t __result ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "47\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "48\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "45\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(__s + __result) == (int const   )__accept1) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "42\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "44\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*(__s + __result) == (int const   )__accept2) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "43\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "46\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "49\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strspn_c2(char const   *__s , int __accept1 ,
                                   int __accept2 ) ;
__inline extern size_t __strspn_c1(char const   *__s , int __accept ) 
{ register size_t __result ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "39\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "40\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "37\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(__s + __result) == (int const   )__accept) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "36\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }

    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "38\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "41\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strspn_c1(char const   *__s , int __accept ) ;
__inline extern size_t __strcspn_c3(char const   *__s , int __reject1 ,
                                    int __reject2 , int __reject3 ) 
{ register size_t __result ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "33\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "34\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "31\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(__s + __result) != 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "30\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*(__s + __result) != (int const   )__reject1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "29\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*(__s + __result) != (int const   )__reject2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "28\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }
          if ((int const   )*(__s + __result) != (int const   )__reject3) {
            {
            _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                   "a");
            fprintf(_coverage_fout, "27\n");
            fflush(_coverage_fout);
            fclose(_coverage_fout);
            }

          } else {
            break;
          }
        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "32\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "35\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strcspn_c3(char const   *__s , int __reject1 ,
                                    int __reject2 , int __reject3 ) ;
__inline extern size_t __strcspn_c2(char const   *__s , int __reject1 ,
                                    int __reject2 ) 
{ register size_t __result ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "24\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "25\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "22\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(__s + __result) != 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "21\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*(__s + __result) != (int const   )__reject1) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "20\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }
        if ((int const   )*(__s + __result) != (int const   )__reject2) {
          {
          _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                                 "a");
          fprintf(_coverage_fout, "19\n");
          fflush(_coverage_fout);
          fclose(_coverage_fout);
          }

        } else {
          break;
        }
      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "23\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "26\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strcspn_c2(char const   *__s , int __reject1 ,
                                    int __reject2 ) ;
__inline extern size_t __strcspn_c1(char const   *__s , int __reject ) 
{ register size_t __result ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "16\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  __result = (size_t )0;
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "17\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  while (1) {
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "14\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    if ((int const   )*(__s + __result) != 0) {
      {
      _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                             "a");
      fprintf(_coverage_fout, "13\n");
      fflush(_coverage_fout);
      fclose(_coverage_fout);
      }
      if ((int const   )*(__s + __result) != (int const   )__reject) {
        {
        _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                               "a");
        fprintf(_coverage_fout, "12\n");
        fflush(_coverage_fout);
        fclose(_coverage_fout);
        }

      } else {
        break;
      }
    } else {
      break;
    }
    {
    _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                           "a");
    fprintf(_coverage_fout, "15\n");
    fflush(_coverage_fout);
    fclose(_coverage_fout);
    }
    __result ++;
  }
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "18\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (__result);
}
}
__inline extern size_t __strcspn_c1(char const   *__s , int __reject ) ;
extern void *__rawmemchr(void const   *__s , int __c ) ;
extern  __attribute__((__nothrow__)) char *basename(char const   *__filename )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *memfrob(void *__s , size_t __n )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *strfry(char *__string )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int strverscmp(char const   *__s1 ,
                                                    char const   *__s2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *stpncpy(char * __restrict  __dest ,
                                                   char const   * __restrict  __src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *__stpncpy(char * __restrict  __dest ,
                                                     char const   * __restrict  __src ,
                                                     size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *stpcpy(char * __restrict  __dest ,
                                                  char const   * __restrict  __src )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *__stpcpy(char * __restrict  __dest ,
                                                    char const   * __restrict  __src )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strsignal(int __sig ) ;
extern  __attribute__((__nothrow__)) char *strsep(char ** __restrict  __stringp ,
                                                  char const   * __restrict  __delim )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int strncasecmp_l(char const   *__s1 ,
                                                       char const   *__s2 ,
                                                       size_t __n ,
                                                       __locale_t __loc )  __attribute__((__pure__,
__nonnull__(1,2,4))) ;
extern  __attribute__((__nothrow__)) int strcasecmp_l(char const   *__s1 ,
                                                      char const   *__s2 ,
                                                      __locale_t __loc )  __attribute__((__pure__,
__nonnull__(1,2,3))) ;
extern  __attribute__((__nothrow__)) int strncasecmp(char const   *__s1 ,
                                                     char const   *__s2 ,
                                                     size_t __n )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int ffsll(long long __ll )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int ffsl(long __l )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int ffs(int __i )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) char *rindex(char const   *__s , int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *index(char const   *__s , int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int bcmp(void const   *__s1 ,
                                              void const   *__s2 , size_t __n )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void bzero(void *__s , size_t __n )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void bcopy(void const   *__src ,
                                                void *__dest , size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void __bzero(void *__s , size_t __n )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *strerror_l(int __errnum ,
                                                      __locale_t __l ) ;
extern  __attribute__((__nothrow__)) char *strerror_r(int __errnum ,
                                                      char *__buf ,
                                                      size_t __buflen )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *strerror(int __errnum ) ;
extern  __attribute__((__nothrow__)) size_t strnlen(char const   *__string ,
                                                    size_t __maxlen )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *mempcpy(void * __restrict  __dest ,
                                                   void const   * __restrict  __src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void *__mempcpy(void * __restrict  __dest ,
                                                     void const   * __restrict  __src ,
                                                     size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void *memmem(void const   *__haystack ,
                                                  size_t __haystacklen ,
                                                  void const   *__needle ,
                                                  size_t __needlelen )  __attribute__((__pure__,
__nonnull__(1,3))) ;
extern  __attribute__((__nothrow__)) char *strcasestr(char const   *__haystack ,
                                                      char const   *__needle )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strtok_r(char * __restrict  __s ,
                                                    char const   * __restrict  __delim ,
                                                    char ** __restrict  __save_ptr )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) char *__strtok_r(char * __restrict  __s ,
                                                      char const   * __restrict  __delim ,
                                                      char ** __restrict  __save_ptr )  __attribute__((__nonnull__(2,3))) ;
extern  __attribute__((__nothrow__)) char *strtok(char * __restrict  __s ,
                                                  char const   * __restrict  __delim )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *strstr(char const   *__haystack ,
                                                  char const   *__needle )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strpbrk(char const   *__s ,
                                                   char const   *__accept )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) size_t strspn(char const   *__s ,
                                                   char const   *__accept )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) size_t strcspn(char const   *__s ,
                                                    char const   *__reject )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strchrnul(char const   *__s ,
                                                     int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *strrchr(char const   *__s , int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *strchr(char const   *__s , int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *strndup(char const   *__string ,
                                                   size_t __n )  __attribute__((__nonnull__(1),
__malloc__)) ;
extern  __attribute__((__nothrow__)) char *strdup(char const   *__s )  __attribute__((__nonnull__(1),
__malloc__)) ;
extern  __attribute__((__nothrow__)) size_t strxfrm_l(char *__dest ,
                                                      char const   *__src ,
                                                      size_t __n ,
                                                      __locale_t __l )  __attribute__((__nonnull__(2,4))) ;
extern  __attribute__((__nothrow__)) int strcoll_l(char const   *__s1 ,
                                                   char const   *__s2 ,
                                                   __locale_t __l )  __attribute__((__pure__,
__nonnull__(1,2,3))) ;
extern  __attribute__((__nothrow__)) size_t strxfrm(char * __restrict  __dest ,
                                                    char const   * __restrict  __src ,
                                                    size_t __n )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int strcoll(char const   *__s1 ,
                                                 char const   *__s2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int strcmp(char const   *__s1 ,
                                                char const   *__s2 )  __attribute__((__pure__,
__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strncat(char * __restrict  __dest ,
                                                   char const   * __restrict  __src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strcat(char * __restrict  __dest ,
                                                  char const   * __restrict  __src )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strncpy(char * __restrict  __dest ,
                                                   char const   * __restrict  __src ,
                                                   size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *strcpy(char * __restrict  __dest ,
                                                  char const   * __restrict  __src )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void *memrchr(void const   *__s , int __c ,
                                                   size_t __n )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *rawmemchr(void const   *__s ,
                                                     int __c )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *memccpy(void * __restrict  __dest ,
                                                   void const   * __restrict  __src ,
                                                   int __c , size_t __n )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int getloadavg(double *__loadavg ,
                                                    int __nelem )  __attribute__((__nonnull__(1))) ;
extern int getpt(void) ;
extern  __attribute__((__nothrow__)) int ptsname_r(int __fd , char *__buf ,
                                                   size_t __buflen )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) char *ptsname(int __fd ) ;
extern  __attribute__((__nothrow__)) int unlockpt(int __fd ) ;
extern  __attribute__((__nothrow__)) int grantpt(int __fd ) ;
extern int posix_openpt(int __oflag ) ;
extern  __attribute__((__nothrow__)) void setkey(char const   *__key )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int getsubopt(char ** __restrict  __optionp ,
                                                   char * const  * __restrict  __tokens ,
                                                   char ** __restrict  __valuep )  __attribute__((__nonnull__(1,2,3))) ;
extern  __attribute__((__nothrow__)) int rpmatch(char const   *__response )  __attribute__((__nonnull__(1))) ;
typedef long wchar_t;
extern  __attribute__((__nothrow__)) size_t wcstombs(char * __restrict  __s ,
                                                     wchar_t const   * __restrict  __pwcs ,
                                                     size_t __n ) ;
extern  __attribute__((__nothrow__)) size_t mbstowcs(wchar_t * __restrict  __pwcs ,
                                                     char const   * __restrict  __s ,
                                                     size_t __n ) ;
extern  __attribute__((__nothrow__)) int wctomb(char *__s , wchar_t __wchar ) ;
extern  __attribute__((__nothrow__)) int mbtowc(wchar_t * __restrict  __pwc ,
                                                char const   * __restrict  __s ,
                                                size_t __n ) ;
extern  __attribute__((__nothrow__)) int mblen(char const   *__s , size_t __n ) ;
extern  __attribute__((__nothrow__)) int qfcvt_r(long double __value ,
                                                 int __ndigit ,
                                                 int * __restrict  __decpt ,
                                                 int * __restrict  __sign ,
                                                 char * __restrict  __buf ,
                                                 size_t __len )  __attribute__((__nonnull__(3,4,5))) ;
extern  __attribute__((__nothrow__)) int qecvt_r(long double __value ,
                                                 int __ndigit ,
                                                 int * __restrict  __decpt ,
                                                 int * __restrict  __sign ,
                                                 char * __restrict  __buf ,
                                                 size_t __len )  __attribute__((__nonnull__(3,4,5))) ;
extern  __attribute__((__nothrow__)) int fcvt_r(double __value , int __ndigit ,
                                                int * __restrict  __decpt ,
                                                int * __restrict  __sign ,
                                                char * __restrict  __buf ,
                                                size_t __len )  __attribute__((__nonnull__(3,4,5))) ;
extern  __attribute__((__nothrow__)) int ecvt_r(double __value , int __ndigit ,
                                                int * __restrict  __decpt ,
                                                int * __restrict  __sign ,
                                                char * __restrict  __buf ,
                                                size_t __len )  __attribute__((__nonnull__(3,4,5))) ;
extern  __attribute__((__nothrow__)) char *qgcvt(long double __value ,
                                                 int __ndigit , char *__buf )  __attribute__((__nonnull__(3))) ;
extern  __attribute__((__nothrow__)) char *qfcvt(long double __value ,
                                                 int __ndigit ,
                                                 int * __restrict  __decpt ,
                                                 int * __restrict  __sign )  __attribute__((__nonnull__(3,4))) ;
extern  __attribute__((__nothrow__)) char *qecvt(long double __value ,
                                                 int __ndigit ,
                                                 int * __restrict  __decpt ,
                                                 int * __restrict  __sign )  __attribute__((__nonnull__(3,4))) ;
extern  __attribute__((__nothrow__)) char *gcvt(double __value , int __ndigit ,
                                                char *__buf )  __attribute__((__nonnull__(3))) ;
extern  __attribute__((__nothrow__)) char *fcvt(double __value , int __ndigit ,
                                                int * __restrict  __decpt ,
                                                int * __restrict  __sign )  __attribute__((__nonnull__(3,4))) ;
extern  __attribute__((__nothrow__)) char *ecvt(double __value , int __ndigit ,
                                                int * __restrict  __decpt ,
                                                int * __restrict  __sign )  __attribute__((__nonnull__(3,4))) ;
struct __anonstruct_lldiv_t_6;
typedef struct __anonstruct_lldiv_t_6 lldiv_t;
extern  __attribute__((__nothrow__)) lldiv_t lldiv(long long __numer ,
                                                   long long __denom )  __attribute__((__const__)) ;
struct __anonstruct_ldiv_t_5;
typedef struct __anonstruct_ldiv_t_5 ldiv_t;
extern  __attribute__((__nothrow__)) ldiv_t ldiv(long __numer , long __denom )  __attribute__((__const__)) ;
struct __anonstruct_div_t_4;
typedef struct __anonstruct_div_t_4 div_t;
extern  __attribute__((__nothrow__)) div_t div(int __numer , int __denom )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long long llabs(long long __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) long labs(long __x )  __attribute__((__const__)) ;
extern  __attribute__((__nothrow__)) int abs(int __x )  __attribute__((__const__)) ;
extern void qsort_r(void *__base , size_t __nmemb , size_t __size ,
                    int (*__compar)(void const   * , void const   * , void * ) ,
                    void *__arg )  __attribute__((__nonnull__(1,4))) ;
extern void qsort(void *__base , size_t __nmemb , size_t __size ,
                  int (*__compar)(void const   * , void const   * ) )  __attribute__((__nonnull__(1,4))) ;
extern void *bsearch(void const   *__key , void const   *__base ,
                     size_t __nmemb , size_t __size ,
                     int (*__compar)(void const   * , void const   * ) )  __attribute__((__nonnull__(1,2,5))) ;
extern  __attribute__((__nothrow__)) char *realpath(char const   * __restrict  __name ,
                                                    char * __restrict  __resolved ) ;
extern  __attribute__((__nothrow__)) char *canonicalize_file_name(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern int system(char const   *__command ) ;
extern int mkostemps64(char *__template , int __suffixlen , int __flags )  __attribute__((__nonnull__(1))) ;
extern int mkostemps(char *__template , int __suffixlen , int __flags )  __attribute__((__nonnull__(1))) ;
extern int mkostemp64(char *__template , int __flags )  __attribute__((__nonnull__(1))) ;
extern int mkostemp(char *__template , int __flags )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *mkdtemp(char *__template )  __attribute__((__nonnull__(1))) ;
extern int mkstemps64(char *__template , int __suffixlen )  __attribute__((__nonnull__(1))) ;
extern int mkstemps(char *__template , int __suffixlen )  __attribute__((__nonnull__(1))) ;
extern int mkstemp64(char *__template )  __attribute__((__nonnull__(1))) ;
extern int mkstemp(char *__template )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *mktemp(char *__template )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int clearenv(void) ;
extern  __attribute__((__nothrow__)) int unsetenv(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int setenv(char const   *__name ,
                                                char const   *__value ,
                                                int __replace )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int putenv(char *__string )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *__secure_getenv(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *getenv(char const   *__name )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__, __noreturn__)) void _Exit(int __status ) ;
extern  __attribute__((__nothrow__,
__noreturn__)) void quick_exit(int __status ) ;
extern  __attribute__((__nothrow__)) int on_exit(void (*__func)(int __status ,
                                                                void *__arg ) ,
                                                 void *__arg )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int at_quick_exit(void (*__func)(void) )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) int atexit(void (*__func)(void) )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__, __noreturn__)) void abort(void) ;
extern  __attribute__((__nothrow__)) int posix_memalign(void **__memptr ,
                                                        size_t __alignment ,
                                                        size_t __size )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void *valloc(size_t __size )  __attribute__((__malloc__)) ;
extern  __attribute__((__nothrow__)) void *alloca(size_t __size ) ;
extern  __attribute__((__nothrow__)) void cfree(void *__ptr ) ;
extern  __attribute__((__nothrow__)) void *calloc(size_t __nmemb ,
                                                  size_t __size )  __attribute__((__malloc__)) ;
struct drand48_data;
extern  __attribute__((__nothrow__)) int lcong48_r(unsigned short *__param ,
                                                   struct drand48_data *__buffer )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int seed48_r(unsigned short *__seed16v ,
                                                  struct drand48_data *__buffer )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int srand48_r(long __seedval ,
                                                   struct drand48_data *__buffer )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) int jrand48_r(unsigned short *__xsubi ,
                                                   struct drand48_data * __restrict  __buffer ,
                                                   long * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int mrand48_r(struct drand48_data * __restrict  __buffer ,
                                                   long * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int nrand48_r(unsigned short *__xsubi ,
                                                   struct drand48_data * __restrict  __buffer ,
                                                   long * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int lrand48_r(struct drand48_data * __restrict  __buffer ,
                                                   long * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int erand48_r(unsigned short *__xsubi ,
                                                   struct drand48_data * __restrict  __buffer ,
                                                   double * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int drand48_r(struct drand48_data * __restrict  __buffer ,
                                                   double * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) void lcong48(unsigned short *__param )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) unsigned short *seed48(unsigned short *__seed16v )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) void srand48(long __seedval ) ;
extern  __attribute__((__nothrow__)) long jrand48(unsigned short *__xsubi )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long mrand48(void) ;
extern  __attribute__((__nothrow__)) long nrand48(unsigned short *__xsubi )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long lrand48(void) ;
extern  __attribute__((__nothrow__)) double erand48(unsigned short *__xsubi )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) double drand48(void) ;
extern  __attribute__((__nothrow__)) int rand_r(unsigned int *__seed ) ;
extern  __attribute__((__nothrow__)) void srand(unsigned int __seed ) ;
extern  __attribute__((__nothrow__)) int rand(void) ;
struct random_data;
extern  __attribute__((__nothrow__)) int setstate_r(char * __restrict  __statebuf ,
                                                    struct random_data * __restrict  __buf )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) int initstate_r(unsigned int __seed ,
                                                     char * __restrict  __statebuf ,
                                                     size_t __statelen ,
                                                     struct random_data * __restrict  __buf )  __attribute__((__nonnull__(2,4))) ;
extern  __attribute__((__nothrow__)) int srandom_r(unsigned int __seed ,
                                                   struct random_data *__buf )  __attribute__((__nonnull__(2))) ;
typedef int int32_t;
extern  __attribute__((__nothrow__)) int random_r(struct random_data * __restrict  __buf ,
                                                  int32_t * __restrict  __result )  __attribute__((__nonnull__(1,2))) ;
extern  __attribute__((__nothrow__)) char *setstate(char *__statebuf )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *initstate(unsigned int __seed ,
                                                     char *__statebuf ,
                                                     size_t __statelen )  __attribute__((__nonnull__(2))) ;
extern  __attribute__((__nothrow__)) void srandom(unsigned int __seed ) ;
extern  __attribute__((__nothrow__)) long random(void) ;
__inline extern  __attribute__((__nothrow__)) unsigned long long gnu_dev_makedev(unsigned int __major ,
                                                                                 unsigned int __minor ) ;
__inline extern unsigned long long gnu_dev_makedev(unsigned int __major ,
                                                   unsigned int __minor ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "11\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (((unsigned long long )((__minor & 255U) | ((__major & 4095U) << 8)) | ((unsigned long long )(__minor & 4294967040U) << 12)) | ((unsigned long long )(__major & 4294963200U) << 32));
}
}
__inline extern  __attribute__((__nothrow__)) unsigned long long gnu_dev_makedev(unsigned int __major ,
                                                                                 unsigned int __minor ) ;
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_minor(unsigned long long __dev ) ;
__inline extern unsigned int gnu_dev_minor(unsigned long long __dev ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "10\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned int )((__dev & 255ULL) | (unsigned long long )((unsigned int )(__dev >> 12) & 4294967040U)));
}
}
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_minor(unsigned long long __dev ) ;
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_major(unsigned long long __dev ) ;
__inline extern unsigned int gnu_dev_major(unsigned long long __dev ) 
{ 

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "9\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((unsigned int )(((__dev >> 8) & 4095ULL) | (unsigned long long )((unsigned int )(__dev >> 32) & 4294963200U)));
}
}
__inline extern  __attribute__((__nothrow__)) unsigned int gnu_dev_major(unsigned long long __dev ) ;
struct __anonstruct___sigset_t_8;
struct __anonstruct_fd_set_9;
typedef struct __anonstruct_fd_set_9 fd_set;
extern int pselect(int __nfds , fd_set * __restrict  __readfds ,
                   fd_set * __restrict  __writefds ,
                   fd_set * __restrict  __exceptfds ,
                   struct timespec  const  * __restrict  __timeout ,
                   __sigset_t const   * __restrict  __sigmask ) ;
struct timeval;
extern int select(int __nfds , fd_set * __restrict  __readfds ,
                  fd_set * __restrict  __writefds ,
                  fd_set * __restrict  __exceptfds ,
                  struct timeval * __restrict  __timeout ) ;
extern  __attribute__((__nothrow__)) long a64l(char const   *__s )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) char *l64a(long __n ) ;
extern  __attribute__((__nothrow__)) long long strtoll(char const   * __restrict  __nptr ,
                                                       char ** __restrict  __endptr ,
                                                       int __base )  __attribute__((__nonnull__(1))) ;
__inline extern  __attribute__((__nothrow__)) long long atoll(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern long long atoll(char const   *__nptr ) 
{ long long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "7\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = strtoll((char const   */* __restrict  */)__nptr,
                (char **/* __restrict  */)((char **)((void *)0)), 10);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "8\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long long atoll(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern  __attribute__((__nothrow__)) long atol(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern long atol(char const   *__nptr ) 
{ long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "5\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = strtol((char const   */* __restrict  */)__nptr,
               (char **/* __restrict  */)((char **)((void *)0)), 10);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "6\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) long atol(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern  __attribute__((__nothrow__)) int atoi(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern int atoi(char const   *__nptr ) 
{ long tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "3\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = strtol((char const   */* __restrict  */)__nptr,
               (char **/* __restrict  */)((char **)((void *)0)), 10);
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "4\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return ((int )tmp);
}
}
__inline extern  __attribute__((__nothrow__)) int atoi(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) double strtod(char const   * __restrict  __nptr ,
                                                   char ** __restrict  __endptr )  __attribute__((__nonnull__(1))) ;
__inline extern  __attribute__((__nothrow__)) double atof(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
__inline extern double atof(char const   *__nptr ) 
{ double tmp ;

  {
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "1\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  tmp = strtod((char const   */* __restrict  */)__nptr,
               (char **/* __restrict  */)((char **)((void *)0)));
  {
  _coverage_fout = fopen("/root/mountpoint-genprog/genprog-many-bugs/php-bug-2012-03-04-bda5ea7111-60dfd64bf2/coverage/coverage.path",
                         "a");
  fprintf(_coverage_fout, "2\n");
  fflush(_coverage_fout);
  fclose(_coverage_fout);
  }
  return (tmp);
}
}
__inline extern  __attribute__((__nothrow__)) double atof(char const   *__nptr )  __attribute__((__pure__,
__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long double strtold_l(char const   * __restrict  __nptr ,
                                                           char ** __restrict  __endptr ,
                                                           __locale_t __loc )  __attribute__((__nonnull__(1,3))) ;
extern  __attribute__((__nothrow__)) float strtof_l(char const   * __restrict  __nptr ,
                                                    char ** __restrict  __endptr ,
                                                    __locale_t __loc )  __attribute__((__nonnull__(1,3))) ;
extern  __attribute__((__nothrow__)) double strtod_l(char const   * __restrict  __nptr ,
                                                     char ** __restrict  __endptr ,
                                                     __locale_t __loc )  __attribute__((__nonnull__(1,3))) ;
extern  __attribute__((__nothrow__)) unsigned long long strtoull_l(char const   * __restrict  __nptr ,
                                                                   char ** __restrict  __endptr ,
                                                                   int __base ,
                                                                   __locale_t __loc )  __attribute__((__nonnull__(1,4))) ;
extern  __attribute__((__nothrow__)) long long strtoll_l(char const   * __restrict  __nptr ,
                                                         char ** __restrict  __endptr ,
                                                         int __base ,
                                                         __locale_t __loc )  __attribute__((__nonnull__(1,4))) ;
extern  __attribute__((__nothrow__)) unsigned long strtoul_l(char const   * __restrict  __nptr ,
                                                             char ** __restrict  __endptr ,
                                                             int __base ,
                                                             __locale_t __loc )  __attribute__((__nonnull__(1,4))) ;
extern  __attribute__((__nothrow__)) long strtol_l(char const   * __restrict  __nptr ,
                                                   char ** __restrict  __endptr ,
                                                   int __base ,
                                                   __locale_t __loc )  __attribute__((__nonnull__(1,4))) ;
extern  __attribute__((__nothrow__)) unsigned long long strtoull(char const   * __restrict  __nptr ,
                                                                 char ** __restrict  __endptr ,
                                                                 int __base )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) unsigned long long strtouq(char const   * __restrict  __nptr ,
                                                                char ** __restrict  __endptr ,
                                                                int __base )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long long strtoq(char const   * __restrict  __nptr ,
                                                      char ** __restrict  __endptr ,
                                                      int __base )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) unsigned long strtoul(char const   * __restrict  __nptr ,
                                                           char ** __restrict  __endptr ,
                                                           int __base )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) long double strtold(char const   * __restrict  __nptr ,
                                                         char ** __restrict  __endptr )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) float strtof(char const   * __restrict  __nptr ,
                                                  char ** __restrict  __endptr )  __attribute__((__nonnull__(1))) ;
extern  __attribute__((__nothrow__)) size_t __ctype_get_mb_cur_max(void) ;
struct _zend_lex_state {
   unsigned int yy_leng ;
   unsigned char *yy_start ;
   unsigned char *yy_text ;
   unsigned char *yy_cursor ;
   unsigned char *yy_marker ;
   unsigned char *yy_limit ;
   int yy_state ;
   zend_stack state_stack ;
   zend_file_handle *in ;
   uint lineno ;
   char *filename ;
   unsigned char *script_org ;
   size_t script_org_size ;
   unsigned char *script_filtered ;
   size_t script_filtered_size ;
   size_t (*input_filter)(unsigned char **str , size_t *str_length ,
                          unsigned char const   *buf , size_t length ) ;
   size_t (*output_filter)(unsigned char **str , size_t *str_length ,
                           unsigned char const   *buf , size_t length ) ;
   zend_encoding const   *script_encoding ;
};
struct _cwd_state {
   char *cwd ;
   int cwd_length ;
};
struct _virtual_cwd_globals {
   cwd_state cwd ;
   long realpath_cache_size ;
   long realpath_cache_size_limit ;
   long realpath_cache_ttl ;
   realpath_cache_bucket *realpath_cache[1024] ;
};
struct _realpath_cache_bucket {
   unsigned long key ;
   char *path ;
   int path_len ;
   char *realpath ;
   int realpath_len ;
   int is_dir ;
   time_t expires ;
   struct _realpath_cache_bucket *next ;
};
typedef int (*verify_path_func)(cwd_state const   * );
enum __anonenum_59 {
    DT_UNKNOWN = 0,
    DT_FIFO = 1,
    DT_CHR = 2,
    DT_DIR = 4,
    DT_BLK = 6,
    DT_REG = 8,
    DT_LNK = 10,
    DT_SOCK = 12,
    DT_WHT = 14
} ;
enum __anonenum_59;
typedef __u_quad_t __ino64_t;
struct dirent64 {
   __ino64_t d_ino ;
   __off64_t d_off ;
   unsigned short d_reclen ;
   unsigned char d_type ;
   char d_name[256] ;
};
typedef unsigned long __ino_t;
struct dirent {
   __ino_t d_ino ;
   __off_t d_off ;
   unsigned short d_reclen ;
   unsigned char d_type ;
   char d_name[256] ;
};
struct utimbuf {
   __time_t actime ;
   __time_t modtime ;
};
enum __anonenum_58 {
    _ISupper = 256,
    _ISlower = 512,
    _ISalpha = 1024,
    _ISdigit = 2048,
    _ISxdigit = 4096,
    _ISspace = 8192,
    _ISprint = 16384,
    _ISgraph = 32768,
    _ISblank = 1,
    _IScntrl = 2,
    _ISpunct = 4,
    _ISalnum = 8
} ;
enum __anonenum_58;
struct timespec {
   __time_t tv_sec ;
   long tv_nsec ;
};
typedef __quad_t __blkcnt64_t;
typedef long __blksize_t;
typedef unsigned int __nlink_t;
struct stat64 {
   __dev_t st_dev ;
   unsigned int __pad1 ;
   __ino_t __st_ino ;
   __mode_t st_mode ;
   __nlink_t st_nlink ;
   __uid_t st_uid ;
   __gid_t st_gid ;
   __dev_t st_rdev ;
   unsigned int __pad2 ;
   __off64_t st_size ;
   __blksize_t st_blksize ;
   __blkcnt64_t st_blocks ;
   struct timespec st_atim ;
   struct timespec st_mtim ;
   struct timespec st_ctim ;
   __ino64_t st_ino ;
};
typedef long __blkcnt_t;
struct stat {
   __dev_t st_dev ;
   unsigned short __pad1 ;
   __ino_t st_ino ;
   __mode_t st_mode ;
   __nlink_t st_nlink ;
   __uid_t st_uid ;
   __gid_t st_gid ;
   __dev_t st_rdev ;
   unsigned short __pad2 ;
   __off_t st_size ;
   __blksize_t st_blksize ;
   __blkcnt_t st_blocks ;
   struct timespec st_atim ;
   struct timespec st_mtim ;
   struct timespec st_ctim ;
   unsigned long __unused4 ;
   unsigned long __unused5 ;
};
enum __anonenum_57 {
    _CS_PATH = 0,
    _CS_V6_WIDTH_RESTRICTED_ENVS = 1,
    _CS_GNU_LIBC_VERSION = 2,
    _CS_GNU_LIBPTHREAD_VERSION = 3,
    _CS_V5_WIDTH_RESTRICTED_ENVS = 4,
    _CS_V7_WIDTH_RESTRICTED_ENVS = 5,
    _CS_LFS_CFLAGS = 1000,
    _CS_LFS_LDFLAGS = 1001,
    _CS_LFS_LIBS = 1002,
    _CS_LFS_LINTFLAGS = 1003,
    _CS_LFS64_CFLAGS = 1004,
    _CS_LFS64_LDFLAGS = 1005,
    _CS_LFS64_LIBS = 1006,
    _CS_LFS64_LINTFLAGS = 1007,
    _CS_XBS5_ILP32_OFF32_CFLAGS = 1100,
    _CS_XBS5_ILP32_OFF32_LDFLAGS = 1101,
    _CS_XBS5_ILP32_OFF32_LIBS = 1102,
    _CS_XBS5_ILP32_OFF32_LINTFLAGS = 1103,
    _CS_XBS5_ILP32_OFFBIG_CFLAGS = 1104,
    _CS_XBS5_ILP32_OFFBIG_LDFLAGS = 1105,
    _CS_XBS5_ILP32_OFFBIG_LIBS = 1106,
    _CS_XBS5_ILP32_OFFBIG_LINTFLAGS = 1107,
    _CS_XBS5_LP64_OFF64_CFLAGS = 1108,
    _CS_XBS5_LP64_OFF64_LDFLAGS = 1109,
    _CS_XBS5_LP64_OFF64_LIBS = 1110,
    _CS_XBS5_LP64_OFF64_LINTFLAGS = 1111,
    _CS_XBS5_LPBIG_OFFBIG_CFLAGS = 1112,
    _CS_XBS5_LPBIG_OFFBIG_LDFLAGS = 1113,
    _CS_XBS5_LPBIG_OFFBIG_LIBS = 1114,
    _CS_XBS5_LPBIG_OFFBIG_LINTFLAGS = 1115,
    _CS_POSIX_V6_ILP32_OFF32_CFLAGS = 1116,
    _CS_POSIX_V6_ILP32_OFF32_LDFLAGS = 1117,
    _CS_POSIX_V6_ILP32_OFF32_LIBS = 1118,
    _CS_POSIX_V6_ILP32_OFF32_LINTFLAGS = 1119,
    _CS_POSIX_V6_ILP32_OFFBIG_CFLAGS = 1120,
    _CS_POSIX_V6_ILP32_OFFBIG_LDFLAGS = 1121,
    _CS_POSIX_V6_ILP32_OFFBIG_LIBS = 1122,
    _CS_POSIX_V6_ILP32_OFFBIG_LINTFLAGS = 1123,
    _CS_POSIX_V6_LP64_OFF64_CFLAGS = 1124,
    _CS_POSIX_V6_LP64_OFF64_LDFLAGS = 1125,
    _CS_POSIX_V6_LP64_OFF64_LIBS = 1126,
    _CS_POSIX_V6_LP64_OFF64_LINTFLAGS = 1127,
    _CS_POSIX_V6_LPBIG_OFFBIG_CFLAGS = 1128,
    _CS_POSIX_V6_LPBIG_OFFBIG_LDFLAGS = 1129,
    _CS_POSIX_V6_LPBIG_OFFBIG_LIBS = 1130,
    _CS_POSIX_V6_LPBIG_OFFBIG_LINTFLAGS = 1131,
    _CS_POSIX_V7_ILP32_OFF32_CFLAGS = 1132,
    _CS_POSIX_V7_ILP32_OFF32_LDFLAGS = 1133,
    _CS_POSIX_V7_ILP32_OFF32_LIBS = 1134,
    _CS_POSIX_V7_ILP32_OFF32_LINTFLAGS = 1135,
    _CS_POSIX_V7_ILP32_OFFBIG_CFLAGS = 1136,
    _CS_POSIX_V7_ILP32_OFFBIG_LDFLAGS = 1137,
    _CS_POSIX_V7_ILP32_OFFBIG_LIBS = 1138,
    _CS_POSIX_V7_ILP32_OFFBIG_LINTFLAGS = 1139,
    _CS_POSIX_V7_LP64_OFF64_CFLAGS = 1140,
    _CS_POSIX_V7_LP64_OFF64_LDFLAGS = 1141,
    _CS_POSIX_V7_LP64_OFF64_LIBS = 1142,
    _CS_POSIX_V7_LP64_OFF64_LINTFLAGS = 1143,
    _CS_POSIX_V7_LPBIG_OFFBIG_CFLAGS = 1144,
    _CS_POSIX_V7_LPBIG_OFFBIG_LDFLAGS = 1145,
    _CS_POSIX_V7_LPBIG_OFFBIG_LIBS = 1146,
    _CS_POSIX_V7_LPBIG_OFFBIG_LINTFLAGS = 1147,
    _CS_V6_ENV = 1148,
    _CS_V7_ENV = 1149
} ;
enum __anonenum_57;
enum __anonenum_56 {
    _SC_ARG_MAX = 0,
    _SC_CHILD_MAX = 1,
    _SC_CLK_TCK = 2,
    _SC_NGROUPS_MAX = 3,
    _SC_OPEN_MAX = 4,
    _SC_STREAM_MAX = 5,
    _SC_TZNAME_MAX = 6,
    _SC_JOB_CONTROL = 7,
    _SC_SAVED_IDS = 8,
    _SC_REALTIME_SIGNALS = 9,
    _SC_PRIORITY_SCHEDULING = 10,
    _SC_TIMERS = 11,
    _SC_ASYNCHRONOUS_IO = 12,
    _SC_PRIORITIZED_IO = 13,
    _SC_SYNCHRONIZED_IO = 14,
    _SC_FSYNC = 15,
    _SC_MAPPED_FILES = 16,
    _SC_MEMLOCK = 17,
    _SC_MEMLOCK_RANGE = 18,
    _SC_MEMORY_PROTECTION = 19,
    _SC_MESSAGE_PASSING = 20,
    _SC_SEMAPHORES = 21,
    _SC_SHARED_MEMORY_OBJECTS = 22,
    _SC_AIO_LISTIO_MAX = 23,
    _SC_AIO_MAX = 24,
    _SC_AIO_PRIO_DELTA_MAX = 25,
    _SC_DELAYTIMER_MAX = 26,
    _SC_MQ_OPEN_MAX = 27,
    _SC_MQ_PRIO_MAX = 28,
    _SC_VERSION = 29,
    _SC_PAGESIZE = 30,
    _SC_RTSIG_MAX = 31,
    _SC_SEM_NSEMS_MAX = 32,
    _SC_SEM_VALUE_MAX = 33,
    _SC_SIGQUEUE_MAX = 34,
    _SC_TIMER_MAX = 35,
    _SC_BC_BASE_MAX = 36,
    _SC_BC_DIM_MAX = 37,
    _SC_BC_SCALE_MAX = 38,
    _SC_BC_STRING_MAX = 39,
    _SC_COLL_WEIGHTS_MAX = 40,
    _SC_EQUIV_CLASS_MAX = 41,
    _SC_EXPR_NEST_MAX = 42,
    _SC_LINE_MAX = 43,
    _SC_RE_DUP_MAX = 44,
    _SC_CHARCLASS_NAME_MAX = 45,
    _SC_2_VERSION = 46,
    _SC_2_C_BIND = 47,
    _SC_2_C_DEV = 48,
    _SC_2_FORT_DEV = 49,
    _SC_2_FORT_RUN = 50,
    _SC_2_SW_DEV = 51,
    _SC_2_LOCALEDEF = 52,
    _SC_PII = 53,
    _SC_PII_XTI = 54,
    _SC_PII_SOCKET = 55,
    _SC_PII_INTERNET = 56,
    _SC_PII_OSI = 57,
    _SC_POLL = 58,
    _SC_SELECT = 59,
    _SC_UIO_MAXIOV = 60,
    _SC_IOV_MAX = 60,
    _SC_PII_INTERNET_STREAM = 61,
    _SC_PII_INTERNET_DGRAM = 62,
    _SC_PII_OSI_COTS = 63,
    _SC_PII_OSI_CLTS = 64,
    _SC_PII_OSI_M = 65,
    _SC_T_IOV_MAX = 66,
    _SC_THREADS = 67,
    _SC_THREAD_SAFE_FUNCTIONS = 68,
    _SC_GETGR_R_SIZE_MAX = 69,
    _SC_GETPW_R_SIZE_MAX = 70,
    _SC_LOGIN_NAME_MAX = 71,
    _SC_TTY_NAME_MAX = 72,
    _SC_THREAD_DESTRUCTOR_ITERATIONS = 73,
    _SC_THREAD_KEYS_MAX = 74,
    _SC_THREAD_STACK_MIN = 75,
    _SC_THREAD_THREADS_MAX = 76,
    _SC_THREAD_ATTR_STACKADDR = 77,
    _SC_THREAD_ATTR_STACKSIZE = 78,
    _SC_THREAD_PRIORITY_SCHEDULING = 79,
    _SC_THREAD_PRIO_INHERIT = 80,
    _SC_THREAD_PRIO_PROTECT = 81,
    _SC_THREAD_PROCESS_SHARED = 82,
    _SC_NPROCESSORS_CONF = 83,
    _SC_NPROCESSORS_ONLN = 84,
    _SC_PHYS_PAGES = 85,
    _SC_AVPHYS_PAGES = 86,
    _SC_ATEXIT_MAX = 87,
    _SC_PASS_MAX = 88,
    _SC_XOPEN_VERSION = 89,
    _SC_XOPEN_XCU_VERSION = 90,
    _SC_XOPEN_UNIX = 91,
    _SC_XOPEN_CRYPT = 92,
    _SC_XOPEN_ENH_I18N = 93,
    _SC_XOPEN_SHM = 94,
    _SC_2_CHAR_TERM = 95,
    _SC_2_C_VERSION = 96,
    _SC_2_UPE = 97,
    _SC_XOPEN_XPG2 = 98,
    _SC_XOPEN_XPG3 = 99,
    _SC_XOPEN_XPG4 = 100,
    _SC_CHAR_BIT = 101,
    _SC_CHAR_MAX = 102,
    _SC_CHAR_MIN = 103,
    _SC_INT_MAX = 104,
    _SC_INT_MIN = 105,
    _SC_LONG_BIT = 106,
    _SC_WORD_BIT = 107,
    _SC_MB_LEN_MAX = 108,
    _SC_NZERO = 109,
    _SC_SSIZE_MAX = 110,
    _SC_SCHAR_MAX = 111,
    _SC_SCHAR_MIN = 112,
    _SC_SHRT_MAX = 113,
    _SC_SHRT_MIN = 114,
    _SC_UCHAR_MAX = 115,
    _SC_UINT_MAX = 116,
    _SC_ULONG_MAX = 117,
    _SC_USHRT_MAX = 118,
    _SC_NL_ARGMAX = 119,
    _SC_NL_LANGMAX = 120,
    _SC_NL_MSGMAX = 121,
    _SC_NL_NMAX = 122,
    _SC_NL_SETMAX = 123,
    _SC_NL_TEXTMAX = 124,
    _SC_XBS5_ILP32_OFF32 = 125,
    _SC_XBS5_ILP32_OFFBIG = 126,
    _SC_XBS5_LP64_OFF64 = 127,
    _SC_XBS5_LPBIG_OFFBIG = 128,
    _SC_XOPEN_LEGACY = 129,
    _SC_XOPEN_REALTIME = 130,
    _SC_XOPEN_REALTIME_THREADS = 131,
    _SC_ADVISORY_INFO = 132,
    _SC_BARRIERS = 133,
    _SC_BASE = 134,
    _SC_C_LANG_SUPPORT = 135,
    _SC_C_LANG_SUPPORT_R = 136,
    _SC_CLOCK_SELECTION = 137,
    _SC_CPUTIME = 138,
    _SC_THREAD_CPUTIME = 139,
    _SC_DEVICE_IO = 140,
    _SC_DEVICE_SPECIFIC = 141,
    _SC_DEVICE_SPECIFIC_R = 142,
    _SC_FD_MGMT = 143,
    _SC_FIFO = 144,
    _SC_PIPE = 145,
    _SC_FILE_ATTRIBUTES = 146,
    _SC_FILE_LOCKING = 147,
    _SC_FILE_SYSTEM = 148,
    _SC_MONOTONIC_CLOCK = 149,
    _SC_MULTI_PROCESS = 150,
    _SC_SINGLE_PROCESS = 151,
    _SC_NETWORKING = 152,
    _SC_READER_WRITER_LOCKS = 153,
    _SC_SPIN_LOCKS = 154,
    _SC_REGEXP = 155,
    _SC_REGEX_VERSION = 156,
    _SC_SHELL = 157,
    _SC_SIGNALS = 158,
    _SC_SPAWN = 159,
    _SC_SPORADIC_SERVER = 160,
    _SC_THREAD_SPORADIC_SERVER = 161,
    _SC_SYSTEM_DATABASE = 162,
    _SC_SYSTEM_DATABASE_R = 163,
    _SC_TIMEOUTS = 164,
    _SC_TYPED_MEMORY_OBJECTS = 165,
    _SC_USER_GROUPS = 166,
    _SC_USER_GROUPS_R = 167,
    _SC_2_PBS = 168,
    _SC_2_PBS_ACCOUNTING = 169,
    _SC_2_PBS_LOCATE = 170,
    _SC_2_PBS_MESSAGE = 171,
    _SC_2_PBS_TRACK = 172,
    _SC_SYMLOOP_MAX = 173,
    _SC_STREAMS = 174,
    _SC_2_PBS_CHECKPOINT = 175,
    _SC_V6_ILP32_OFF32 = 176,
    _SC_V6_ILP32_OFFBIG = 177,
    _SC_V6_LP64_OFF64 = 178,
    _SC_V6_LPBIG_OFFBIG = 179,
    _SC_HOST_NAME_MAX = 180,
    _SC_TRACE = 181,
    _SC_TRACE_EVENT_FILTER = 182,
    _SC_TRACE_INHERIT = 183,
    _SC_TRACE_LOG = 184,
    _SC_LEVEL1_ICACHE_SIZE = 185,
    _SC_LEVEL1_ICACHE_ASSOC = 186,
    _SC_LEVEL1_ICACHE_LINESIZE = 187,
    _SC_LEVEL1_DCACHE_SIZE = 188,
    _SC_LEVEL1_DCACHE_ASSOC = 189,
    _SC_LEVEL1_DCACHE_LINESIZE = 190,
    _SC_LEVEL2_CACHE_SIZE = 191,
    _SC_LEVEL2_CACHE_ASSOC = 192,
    _SC_LEVEL2_CACHE_LINESIZE = 193,
    _SC_LEVEL3_CACHE_SIZE = 194,
    _SC_LEVEL3_CACHE_ASSOC = 195,
    _SC_LEVEL3_CACHE_LINESIZE = 196,
    _SC_LEVEL4_CACHE_SIZE = 197,
    _SC_LEVEL4_CACHE_ASSOC = 198,
    _SC_LEVEL4_CACHE_LINESIZE = 199,
    _SC_IPV6 = 235,
    _SC_RAW_SOCKETS = 236,
    _SC_V7_ILP32_OFF32 = 237,
    _SC_V7_ILP32_OFFBIG = 238,
    _SC_V7_LP64_OFF64 = 239,
    _SC_V7_LPBIG_OFFBIG = 240,
    _SC_SS_REPL_MAX = 241,
    _SC_TRACE_EVENT_NAME_MAX = 242,
    _SC_TRACE_NAME_MAX = 243,
    _SC_TRACE_SYS_MAX = 244,
    _SC_TRACE_USER_EVENT_MAX = 245,
    _SC_XOPEN_STREAMS = 246,
    _SC_THREAD_ROBUST_PRIO_INHERIT = 247,
    _SC_THREAD_ROBUST_PRIO_PROTECT = 248
} ;
enum __anonenum_56;
enum __anonenum_55 {
    _PC_LINK_MAX = 0,
    _PC_MAX_CANON = 1,
    _PC_MAX_INPUT = 2,
    _PC_NAME_MAX = 3,
    _PC_PATH_MAX = 4,
    _PC_PIPE_BUF = 5,
    _PC_CHOWN_RESTRICTED = 6,
    _PC_NO_TRUNC = 7,
    _PC_VDISABLE = 8,
    _PC_SYNC_IO = 9,
    _PC_ASYNC_IO = 10,
    _PC_PRIO_IO = 11,
    _PC_SOCK_MAXBUF = 12,
    _PC_FILESIZEBITS = 13,
    _PC_REC_INCR_XFER_SIZE = 14,
    _PC_REC_MAX_XFER_SIZE = 15,
    _PC_REC_MIN_XFER_SIZE = 16,
    _PC_REC_XFER_ALIGN = 17,
    _PC_ALLOC_SIZE_MIN = 18,
    _PC_SYMLINK_MAX = 19,
    _PC_2_SYMLINKS = 20
} ;
enum __anonenum_55;
typedef unsigned int __socklen_t;
typedef __socklen_t socklen_t;
struct _zend_fcall_info_cache {
   zend_bool initialized ;
   zend_function *function_handler ;
   zend_class_entry *calling_scope ;
   zend_class_entry *called_scope ;
   zval *object_ptr ;
};
struct _zend_fcall_info {
   size_t size ;
   HashTable *function_table ;
   zval *function_name ;
   HashTable *symbol_table ;
   zval **retval_ptr_ptr ;
   zend_uint param_count ;
   zval ***params ;
   zval *object_ptr ;
   zend_bool no_separation ;
};
struct _zend_function_entry {
   char const   *fname ;
   void (*handler)(int ht , zval *return_value , zval **return_value_ptr ,
                   zval *this_ptr , int return_value_used ) ;
   struct _zend_arg_info  const  *arg_info ;
   zend_uint num_args ;
   zend_uint flags ;
};
struct _zend_free_op {
   zval *var ;
};
struct __anonstruct_fe_54;
struct __anonstruct_str_offset_53;
struct __anonstruct_var_52;
struct _zend_rsrc_list_dtors_entry;
typedef struct _zend_rsrc_list_dtors_entry zend_rsrc_list_dtors_entry;
struct _zend_rsrc_list_dtors_entry {
   void (*list_dtor)(void * ) ;
   void (*plist_dtor)(void * ) ;
   void (*list_dtor_ex)(zend_rsrc_list_entry *rsrc ) ;
   void (*plist_dtor_ex)(zend_rsrc_list_entry *rsrc ) ;
   char const   *type_name ;
   int module_number ;
   int resource_id ;
   unsigned char type ;
};
typedef void (*rsrc_dtor_func_t)(zend_rsrc_list_entry *rsrc );
struct _zend_rsrc_list_entry {
   void *ptr ;
   int type ;
   int refcount ;
};
typedef zend_bool (*zend_auto_global_callback)(char const   *name ,
                                               uint name_len );
struct _zend_ini_scanner_globals {
   zend_file_handle *yy_in ;
   zend_file_handle *yy_out ;
   unsigned int yy_leng ;
   unsigned char *yy_start ;
   unsigned char *yy_text ;
   unsigned char *yy_cursor ;
   unsigned char *yy_marker ;
   unsigned char *yy_limit ;
   int yy_state ;
   zend_stack state_stack ;
   char *filename ;
   int lineno ;
   int scanner_mode ;
};
struct _zend_multibyte_functions {
   char const   *provider_name ;
   zend_encoding const   *(*encoding_fetcher)(char const   *encoding_name ) ;
   char const   *(*encoding_name_getter)(zend_encoding const   *encoding ) ;
   int (*lexer_compatibility_checker)(zend_encoding const   *encoding ) ;
   zend_encoding const   *(*encoding_detector)(unsigned char const   *string ,
                                               size_t length ,
                                               zend_encoding const   **list ,
                                               size_t list_size ) ;
   size_t (*encoding_converter)(unsigned char **to , size_t *to_length ,
                                unsigned char const   *from ,
                                size_t from_length ,
                                zend_encoding const   *encoding_to ,
                                zend_encoding const   *encoding_from ) ;
   int (*encoding_list_parser)(char const   *encoding_list ,
                               size_t encoding_list_len ,
                               zend_encoding const   ***return_list ,
                               size_t *return_size , int persistent ) ;
   zend_encoding const   *(*internal_encoding_getter)(void) ;
   int (*internal_encoding_setter)(zend_encoding const   *encoding ) ;
};
typedef int (*zend_encoding_internal_encoding_setter)(zend_encoding const   *encoding );
typedef zend_encoding const   *(*zend_encoding_internal_encoding_getter)(void);
typedef int (*zend_encoding_list_parser)(char const   *encoding_list ,
                                         size_t encoding_list_len ,
                                         zend_encoding const   ***return_list ,
                                         size_t *return_size , int persistent );
typedef size_t (*zend_encoding_converter)(unsigned char **to ,
                                          size_t *to_length ,
                                          unsigned char const   *from ,
                                          size_t from_length ,
                                          zend_encoding const   *encoding_to ,
                                          zend_encoding const   *encoding_from );
typedef zend_encoding const   *(*zend_encoding_detector)(unsigned char const   *string ,
                                                         size_t length ,
                                                         zend_encoding const   **list ,
                                                         size_t list_size );
typedef int (*zend_encoding_lexer_compatibility_checker)(zend_encoding const   *encoding );
typedef char const   *(*zend_encoding_name_getter)(zend_encoding const   *encoding );
typedef zend_encoding const   *(*zend_encoding_fetcher)(char const   *encoding_name );
typedef size_t (*zend_encoding_filter)(unsigned char **str ,
                                       size_t *str_length ,
                                       unsigned char const   *buf ,
                                       size_t length );
struct _zend_module_dep {
   char const   *name ;
   char const   *rel ;
   char const   *version ;
   unsigned char type ;
};
struct _zend_module_dep;
struct _zend_module_entry {
   unsigned short size ;
   unsigned int zend_api ;
   unsigned char zend_debug ;
   unsigned char zts ;
   struct _zend_ini_entry  const  *ini_entry ;
   struct _zend_module_dep  const  *deps ;
   char const   *name ;
   struct _zend_function_entry  const  *functions ;
   int (*module_startup_func)(int type , int module_number ) ;
   int (*module_shutdown_func)(int type , int module_number ) ;
   int (*request_startup_func)(int type , int module_number ) ;
   int (*request_shutdown_func)(int type , int module_number ) ;
   void (*info_func)(zend_module_entry *zend_module ) ;
   char const   *version ;
   size_t globals_size ;
   void *globals_ptr ;
   void (*globals_ctor)(void *global ) ;
   void (*globals_dtor)(void *global ) ;
   int (*post_deactivate_func)(void) ;
   int module_started ;
   unsigned char type ;
   void *handle ;
   int module_number ;
   char const   *build_id ;
};
typedef struct _zend_module_dep zend_module_dep;
struct __anonstruct_free_list_51 {
   int next ;
};
struct _store_object {
   void *object ;
   void (*dtor)(void *object , zend_object_handle handle ) ;
   void (*free_storage)(void *object ) ;
   void (*clone)(void *object , void **object_clone ) ;
   zend_object_handlers const   *handlers ;
   zend_uint refcount ;
   gc_root_buffer *buffered ;
};
union _store_bucket {
   struct _store_object obj ;
   struct __anonstruct_free_list_51 free_list ;
};
struct _zend_object_store_bucket {
   zend_bool destructor_called ;
   zend_bool valid ;
   union _store_bucket bucket ;
};
union _store_bucket;
struct __anonstruct_free_list_51;
struct _store_object;
typedef void (*zend_objects_store_clone_t)(void *object , void **object_clone );
typedef void (*zend_objects_free_object_storage_t)(void *object );
typedef void (*zend_objects_store_dtor_t)(void *object ,
                                          zend_object_handle handle );
typedef struct _zend_compiler_globals zend_compiler_globals;
typedef struct __jmp_buf_tag sigjmp_buf[1];
struct _zend_function_state;
struct __anonstruct_common_50;
struct _zend_internal_function;
struct _zend_internal_function_info;
typedef struct _zend_internal_function_info zend_internal_function_info;
struct _zend_internal_function_info {
   char const   *_name ;
   zend_uint _name_len ;
   char const   *_class_name ;
   zend_uint required_num_args ;
   zend_uchar _type_hint ;
   zend_bool return_reference ;
   zend_bool pass_rest_by_reference ;
};
union __anonunion_u_49;
struct __anonstruct_zend_error_handling_48 {
   zend_error_handling_t handling ;
   zend_class_entry *exception ;
   zval *user_handler ;
};
typedef int error_t;
union __anonunion_u_46;
union __anonunion_u_45;
typedef int (*zend_write_func_t)(char const   *str , uint str_length );
struct _zend_utility_values {
   char *import_use_extension ;
   uint import_use_extension_length ;
   zend_bool html_errors ;
};
struct _zend_utility_functions {
   void ( /* format attribute */  (*error_function))(int type ,
                                                     char const   *error_filename ,
                                                     uint error_lineno ,
                                                     char const   *format ,
                                                     va_list args ) ;
   int ( /* format attribute */  (*printf_function))(char const   *format  , ...) ;
   int (*write_function)(char const   *str , uint str_length ) ;
   FILE *(*fopen_function)(char const   *filename , char **opened_path ) ;
   void (*message_handler)(long message , void const   *data ) ;
   void (*block_interruptions)(void) ;
   void (*unblock_interruptions)(void) ;
   int (*get_configuration_directive)(char const   *name , uint name_length ,
                                      zval *contents ) ;
   void (*ticks_function)(int ticks ) ;
   void (*on_timeout)(int seconds ) ;
   int (*stream_open_function)(char const   *filename ,
                               zend_file_handle *handle ) ;
   int (*vspprintf_function)(char **pbuf , size_t max_len ,
                             char const   *format , va_list ap ) ;
   char *(*getenv_function)(char *name , size_t name_len ) ;
   char *(*resolve_path_function)(char const   *filename , int filename_len ) ;
};
union __anonunion_handle_44;
struct _zend_stream;
struct _zend_mmap;
enum __anonenum_zend_stream_type_43;
typedef void (*zend_stream_closer_t)(void *handle );
typedef size_t (*zend_stream_reader_t)(void *handle , char *buf , size_t len );
typedef size_t (*zend_stream_fsizer_t)(void *handle );
union __anonunion_info_40;
struct __anonstruct_internal_42;
struct __anonstruct_user_41;
enum zend_object_iterator_kind {
    ZEND_ITER_INVALID = 0,
    ZEND_ITER_PLAIN_ARRAY = 1,
    ZEND_ITER_PLAIN_OBJECT = 2,
    ZEND_ITER_OBJECT = 3
} ;
struct _zend_class_iterator_funcs;
struct _zend_object_iterator {
   void *data ;
   zend_object_iterator_funcs *funcs ;
   ulong index ;
};
struct _zend_object_iterator_funcs {
   void (*dtor)(zend_object_iterator *iter ) ;
   int (*valid)(zend_object_iterator *iter ) ;
   void (*get_current_data)(zend_object_iterator *iter , zval ***data ) ;
   int (*get_current_key)(zend_object_iterator *iter , char **str_key ,
                          uint *str_key_len , ulong *int_key ) ;
   void (*move_forward)(zend_object_iterator *iter ) ;
   void (*rewind)(zend_object_iterator *iter ) ;
   void (*invalidate_current)(zend_object_iterator *iter ) ;
};
union _zvalue_value;
struct __anonstruct_str_39;
typedef HashTable *(*zend_object_get_gc_t)(zval *object , zval ***table ,
                                           int *n );
typedef int (*zend_object_get_closure_t)(zval *obj , zend_class_entry **ce_ptr ,
                                         union _zend_function **fptr_ptr ,
                                         zval **zobj_ptr );
typedef int (*zend_object_count_elements_t)(zval *object , long *count );
typedef int (*zend_object_cast_t)(zval *readobj , zval *retval , int type );
typedef int (*zend_object_compare_t)(zval *object1 , zval *object2 );
typedef int (*zend_object_get_class_name_t)(zval const   *object ,
                                            char const   **class_name ,
                                            zend_uint *class_name_len ,
                                            int parent );
typedef zend_class_entry *(*zend_object_get_class_entry_t)(zval const   *object );
typedef zend_object_value (*zend_object_clone_obj_t)(zval *object );
typedef void (*zend_object_delete_obj_t)(zval *object );
typedef void (*zend_object_del_ref_t)(zval *object );
typedef void (*zend_object_add_ref_t)(zval *object );
typedef union _zend_function *(*zend_object_get_constructor_t)(zval *object );
typedef union _zend_function *(*zend_object_get_method_t)(zval **object_ptr ,
                                                          char *method ,
                                                          int method_len ,
                                                          struct _zend_literal  const  *key );
typedef int (*zend_object_call_method_t)(char const   *method , int ht ,
                                         zval *return_value ,
                                         zval **return_value_ptr ,
                                         zval *this_ptr , int return_value_used );
typedef HashTable *(*zend_object_get_debug_info_t)(zval *object , int *is_temp );
typedef HashTable *(*zend_object_get_properties_t)(zval *object );
typedef void (*zend_object_unset_dimension_t)(zval *object , zval *offset );
typedef void (*zend_object_unset_property_t)(zval *object , zval *member ,
                                             struct _zend_literal  const  *key );
typedef int (*zend_object_has_dimension_t)(zval *object , zval *member ,
                                           int check_empty );
typedef int (*zend_object_has_property_t)(zval *object , zval *member ,
                                          int has_set_exists ,
                                          struct _zend_literal  const  *key );
typedef zval *(*zend_object_get_t)(zval *object );
typedef void (*zend_object_set_t)(zval **object , zval *value );
typedef zval **(*zend_object_get_property_ptr_ptr_t)(zval *object ,
                                                     zval *member ,
                                                     struct _zend_literal  const  *key );
typedef void (*zend_object_write_dimension_t)(zval *object , zval *offset ,
                                              zval *value );
typedef void (*zend_object_write_property_t)(zval *object , zval *member ,
                                             zval *value ,
                                             struct _zend_literal  const  *key );
typedef zval *(*zend_object_read_dimension_t)(zval *object , zval *offset ,
                                              int type );
typedef zval *(*zend_object_read_property_t)(zval *object , zval *member ,
                                             int type ,
                                             struct _zend_literal  const  *key );
struct _zend_object {
   zend_class_entry *ce ;
   HashTable *properties ;
   zval **properties_table ;
   HashTable *guards ;
};
struct _zend_guard;
typedef struct _zend_guard zend_guard;
struct _zend_guard {
   zend_bool in_get ;
   zend_bool in_set ;
   zend_bool in_unset ;
   zend_bool in_isset ;
   zend_bool dummy ;
};
typedef void (*llist_apply_func_t)(void * );
typedef void (*llist_apply_with_arg_func_t)(void *data , void *arg );
typedef void (*llist_apply_with_args_func_t)(void *data , int num_args ,
                                             va_list args );
typedef int (*llist_compare_func_t)(zend_llist_element const   ** ,
                                    zend_llist_element const   ** );
typedef void (*llist_dtor_func_t)(void * );
struct _zend_ts_hashtable {
   HashTable hash ;
   zend_uint reader ;
};
typedef int (*apply_func_args_t)(void *pDest , int num_args , va_list args ,
                                 zend_hash_key *hash_key );
typedef int (*apply_func_arg_t)(void *pDest , void *argument );
typedef int (*apply_func_t)(void *pDest );
typedef zend_bool (*merge_checker_func_t)(HashTable *target_ht ,
                                          void *source_data ,
                                          zend_hash_key *hash_key ,
                                          void *pParam );
typedef void (*copy_ctor_param_func_t)(void *pElement , void *pParam );
typedef void (*copy_ctor_func_t)(void *pElement );
typedef void (*dtor_func_t)(void *pDest );
typedef void (*sort_func_t)(void * , size_t  , size_t  ,
                            int (*)(void const   * , void const   * ) );
typedef int (*compare_func_t)(void const   * , void const   * );
typedef ulong (*hash_func_t)(char const   *arKey , uint nKeyLength );
typedef unsigned long zend_uintptr_t;
typedef long zend_intptr_t;
typedef unsigned long long zend_ulong64;
typedef long long zend_long64;
typedef unsigned short zend_ushort;
struct _zend_mm_storage {
   zend_mm_mem_handlers const   *handlers ;
   void *data ;
};
struct _zend_mm_segment;
typedef struct _zend_mm_segment zend_mm_segment;
struct _zend_mm_mem_handlers {
   char const   *name ;
   zend_mm_storage *(*init)(void *params ) ;
   void (*dtor)(zend_mm_storage *storage ) ;
   void (*compact)(zend_mm_storage *storage ) ;
   zend_mm_segment *(*_alloc)(zend_mm_storage *storage , size_t size ) ;
   zend_mm_segment *(*_realloc)(zend_mm_storage *storage ,
                                zend_mm_segment *ptr , size_t size ) ;
   void (*_free)(zend_mm_storage *storage , zend_mm_segment *ptr ) ;
};
struct _zend_mm_segment {
   size_t size ;
   struct _zend_mm_segment *next_segment ;
};
struct _zend_leak_info;
typedef struct _zend_leak_info zend_leak_info;
struct _zend_leak_info {
   void *addr ;
   size_t size ;
   char const   *filename ;
   uint lineno ;
   char const   *orig_filename ;
   uint orig_lineno ;
};
typedef unsigned long tsrm_uintptr_t;
typedef long tsrm_intptr_t;
struct __anonstruct_Dl_serinfo_38;
typedef struct __anonstruct_Dl_serinfo_38 Dl_serinfo;
struct __anonstruct_Dl_serpath_37 {
   char *dls_name ;
   unsigned int dls_flags ;
};
typedef struct __anonstruct_Dl_serpath_37 Dl_serpath;
struct __anonstruct_Dl_serinfo_38 {
   size_t dls_size ;
   unsigned int dls_cnt ;
   Dl_serpath dls_serpath[1] ;
};
struct __anonstruct_Dl_serpath_37;
enum __anonenum_36 {
    RTLD_DI_LMID = 1,
    RTLD_DI_LINKMAP = 2,
    RTLD_DI_CONFIGADDR = 3,
    RTLD_DI_SERINFO = 4,
    RTLD_DI_SERINFOSIZE = 5,
    RTLD_DI_ORIGIN = 6,
    RTLD_DI_PROFILENAME = 7,
    RTLD_DI_PROFILEOUT = 8,
    RTLD_DI_TLS_MODID = 9,
    RTLD_DI_TLS_DATA = 10,
    RTLD_DI_MAX = 10
} ;
enum __anonenum_36;
enum __anonenum_35 {
    RTLD_DL_SYMENT = 1,
    RTLD_DL_LINKMAP = 2
} ;
enum __anonenum_35;
struct __anonstruct_Dl_info_34 {
   char const   *dli_fname ;
   void *dli_fbase ;
   char const   *dli_sname ;
   void *dli_saddr ;
};
typedef _IO_cookie_io_functions_t cookie_io_functions_t;
typedef int __io_close_fn(void *__cookie );
typedef int __io_seek_fn(void *__cookie , __off64_t *__pos , int __w );
typedef __ssize_t __io_write_fn(void *__cookie , char const   *__buf ,
                                size_t __n );
typedef __ssize_t __io_read_fn(void *__cookie , char *__buf , size_t __nbytes );
struct __anonstruct__IO_cookie_io_functions_t_33 {
   __io_read_fn *read ;
   __io_write_fn *write ;
   __io_seek_fn *seek ;
   __io_close_fn *close ;
};
typedef __io_close_fn cookie_close_function_t;
typedef __io_seek_fn cookie_seek_function_t;
typedef __io_write_fn cookie_write_function_t;
typedef __io_read_fn cookie_read_function_t;
enum __codecvt_result {
    __codecvt_ok = 0,
    __codecvt_partial = 1,
    __codecvt_error = 2,
    __codecvt_noconv = 3
} ;
enum __codecvt_result;
struct _IO_marker {
   struct _IO_marker *_next ;
   struct _IO_FILE *_sbuf ;
   int _pos ;
};
struct _IO_jump_t;
typedef unsigned int _G_uint32_t;
typedef unsigned short _G_uint16_t;
typedef int _G_int32_t;
typedef short _G_int16_t;
union __anonunion___value_30 {
   unsigned int __wch ;
   char __wchb[4] ;
};
struct __anonstruct___mbstate_t_29 {
   int __count ;
   union __anonunion___value_30 __value ;
};
typedef struct __anonstruct___mbstate_t_29 __mbstate_t;
struct __anonstruct__G_fpos64_t_32 {
   __off64_t __pos ;
   __mbstate_t __state ;
};
struct __anonstruct__G_fpos_t_31 {
   __off_t __pos ;
   __mbstate_t __state ;
};
struct __anonstruct___mbstate_t_29;
union __anonunion___value_30;
typedef struct _IO_FILE __FILE;
union __anonunion_28;
union __anonunion___n_27;
union __anonunion___u_26;
union __anonunion___u_25;
union __anonunion___u_24;
struct exception {
   int type ;
   char *name ;
   double arg1 ;
   double arg2 ;
   double retval ;
};
enum __anonenum__LIB_VERSION_TYPE_23 {
    _IEEE_ = -1,
    _SVID_ = 0,
    _XOPEN_ = 1,
    _POSIX_ = 2,
    _ISOC_ = 3
} ;
enum __anonenum_22 {
    FP_NAN = 0,
    FP_INFINITE = 1,
    FP_ZERO = 2,
    FP_SUBNORMAL = 3,
    FP_NORMAL = 4
} ;
enum __anonenum_22;
typedef long double double_t;
typedef long double float_t;
typedef int (*__compar_d_fn_t)(void const   * , void const   * , void * );
typedef int (*comparison_fn_t)(void const   * , void const   * );
typedef int (*__compar_fn_t)(void const   * , void const   * );
struct drand48_data {
   unsigned short __x[3] ;
   unsigned short __old_x[3] ;
   unsigned short __c ;
   unsigned short __init ;
   unsigned long long __a ;
};
struct random_data {
   int32_t *fptr ;
   int32_t *rptr ;
   int32_t *state ;
   int rand_type ;
   int rand_deg ;
   int rand_sep ;
   int32_t *end_ptr ;
};
union __anonunion_pthread_barrierattr_t_21;
typedef union __anonunion_pthread_barrierattr_t_21 pthread_barrierattr_t;
union __anonunion_pthread_barrierattr_t_21 {
   char __size[4] ;
   int __align ;
};
union __anonunion_pthread_barrier_t_20;
typedef union __anonunion_pthread_barrier_t_20 pthread_barrier_t;
union __anonunion_pthread_barrier_t_20 {
   char __size[20] ;
   long __align ;
};
typedef int volatile   pthread_spinlock_t;
union __anonunion_pthread_rwlockattr_t_19;
typedef union __anonunion_pthread_rwlockattr_t_19 pthread_rwlockattr_t;
union __anonunion_pthread_rwlockattr_t_19 {
   char __size[8] ;
   long __align ;
};
union __anonunion_pthread_rwlock_t_17;
typedef union __anonunion_pthread_rwlock_t_17 pthread_rwlock_t;
struct __anonstruct___data_18 {
   int __lock ;
   unsigned int __nr_readers ;
   unsigned int __readers_wakeup ;
   unsigned int __writer_wakeup ;
   unsigned int __nr_readers_queued ;
   unsigned int __nr_writers_queued ;
   unsigned char __flags ;
   unsigned char __shared ;
   unsigned char __pad1 ;
   unsigned char __pad2 ;
   int __writer ;
};
union __anonunion_pthread_rwlock_t_17 {
   struct __anonstruct___data_18 __data ;
   char __size[32] ;
   long __align ;
};
struct __anonstruct___data_18;
typedef int pthread_once_t;
typedef unsigned int pthread_key_t;
union __anonunion_pthread_condattr_t_16;
typedef union __anonunion_pthread_condattr_t_16 pthread_condattr_t;
union __anonunion_pthread_condattr_t_16 {
   char __size[4] ;
   long __align ;
};
union __anonunion_pthread_cond_t_14;
typedef union __anonunion_pthread_cond_t_14 pthread_cond_t;
struct __anonstruct___data_15 {
   int __lock ;
   unsigned int __futex ;
   unsigned long long __total_seq ;
   unsigned long long __wakeup_seq ;
   unsigned long long __woken_seq ;
   void *__mutex ;
   unsigned int __nwaiters ;
   unsigned int __broadcast_seq ;
};
union __anonunion_pthread_cond_t_14 {
   struct __anonstruct___data_15 __data ;
   char __size[48] ;
   long long __align ;
};
struct __anonstruct___data_15;
union __anonunion_pthread_mutexattr_t_13;
typedef union __anonunion_pthread_mutexattr_t_13 pthread_mutexattr_t;
union __anonunion_pthread_mutexattr_t_13 {
   char __size[4] ;
   long __align ;
};
union __anonunion_pthread_mutex_t_11;
typedef union __anonunion_pthread_mutex_t_11 pthread_mutex_t;
struct __pthread_internal_slist {
   struct __pthread_internal_slist *__next ;
};
typedef struct __pthread_internal_slist __pthread_slist_t;
union __anonunion____missing_field_name_12 {
   int __spins ;
   __pthread_slist_t __list ;
};
struct __pthread_mutex_s {
   int __lock ;
   unsigned int __count ;
   int __owner ;
   int __kind ;
   unsigned int __nusers ;
   union __anonunion____missing_field_name_12 __annonCompField1 ;
};
union __anonunion_pthread_mutex_t_11 {
   struct __pthread_mutex_s __data ;
   char __size[24] ;
   long __align ;
};
struct __pthread_mutex_s;
union __anonunion____missing_field_name_12;
struct __pthread_internal_slist;
union __anonunion_pthread_attr_t_10;
typedef union __anonunion_pthread_attr_t_10 pthread_attr_t;
union __anonunion_pthread_attr_t_10 {
   char __size[36] ;
   long __align ;
};
typedef unsigned long pthread_t;
typedef __u_quad_t __fsfilcnt64_t;
typedef __fsfilcnt64_t fsfilcnt64_t;
typedef __u_quad_t __fsblkcnt64_t;
typedef __fsblkcnt64_t fsblkcnt64_t;
typedef __blkcnt64_t blkcnt64_t;
typedef unsigned long __fsfilcnt_t;
typedef __fsfilcnt_t fsfilcnt_t;
typedef unsigned long __fsblkcnt_t;
typedef __fsblkcnt_t fsblkcnt_t;
typedef __blkcnt_t blkcnt_t;
typedef __blksize_t blksize_t;
typedef long __fd_mask;
typedef __fd_mask fd_mask;
struct __anonstruct_fd_set_9 {
   __fd_mask fds_bits[1024 / (8 * (int )sizeof(__fd_mask ))] ;
};
typedef long __suseconds_t;
struct timeval {
   __time_t tv_sec ;
   __suseconds_t tv_usec ;
};
typedef __sigset_t sigset_t;
typedef int __sig_atomic_t;
typedef int register_t;
typedef unsigned long long u_int64_t;
typedef unsigned int u_int32_t;
typedef unsigned short u_int16_t;
typedef unsigned char u_int8_t;
typedef long long int64_t;
typedef short int16_t;
typedef signed char int8_t;
typedef unsigned short ushort;
typedef __suseconds_t suseconds_t;
typedef __useconds_t useconds_t;
typedef void *__timer_t;
typedef __timer_t timer_t;
typedef int __clockid_t;
typedef __clockid_t clockid_t;
typedef long __clock_t;
typedef __clock_t clock_t;
typedef int __key_t;
typedef __key_t key_t;
typedef char *__caddr_t;
typedef __caddr_t caddr_t;
typedef int __daddr_t;
typedef __daddr_t daddr_t;
typedef unsigned int __id_t;
typedef __id_t id_t;
typedef __pid_t pid_t;
typedef __off64_t off64_t;
typedef __off_t off_t;
typedef __nlink_t nlink_t;
typedef __dev_t dev_t;
typedef __ino64_t ino64_t;
typedef __ino_t ino_t;
typedef __off64_t __loff_t;
typedef __loff_t loff_t;
struct __anonstruct___fsid_t_7;
typedef struct __anonstruct___fsid_t_7 __fsid_t;
typedef __fsid_t fsid_t;
typedef __u_quad_t u_quad_t;
typedef __quad_t quad_t;
typedef unsigned long __u_long;
typedef __u_long u_long;
typedef unsigned int __u_int;
typedef __u_int u_int;
typedef unsigned short __u_short;
typedef __u_short u_short;
typedef unsigned char __u_char;
typedef __u_char u_char;
typedef __quad_t *__qaddr_t;
typedef long __swblk_t;
typedef __u_quad_t __rlim64_t;
typedef unsigned long __rlim_t;
struct __anonstruct___fsid_t_7 {
   int __val[2] ;
};
typedef unsigned long long __uint64_t;
typedef long long __int64_t;
typedef unsigned int __uint32_t;
typedef unsigned short __uint16_t;
typedef short __int16_t;
typedef unsigned char __uint8_t;
typedef signed char __int8_t;
typedef __locale_t locale_t;
struct __locale_data;
struct __locale_struct {
   struct __locale_data *__locales[13] ;
   unsigned short const   *__ctype_b ;
   int const   *__ctype_tolower ;
   int const   *__ctype_toupper ;
   char const   *__names[13] ;
};
struct __anonstruct_lldiv_t_6 {
   long long quot ;
   long long rem ;
};
struct __anonstruct_ldiv_t_5 {
   long quot ;
   long rem ;
};
struct __anonstruct_div_t_4 {
   int quot ;
   int rem ;
};
union __anonunion___WAIT_STATUS_3;
typedef union __anonunion___WAIT_STATUS_3  __attribute__((__transparent_union__)) __WAIT_STATUS;
union wait;
union __anonunion___WAIT_STATUS_3 {
   union wait *__uptr ;
   int *__iptr ;
};
struct __anonstruct___wait_stopped_2 {
   unsigned int __w_stopval : 8 ;
   unsigned int __w_stopsig : 8 ;
   unsigned int  : 16 ;
};
struct __anonstruct___wait_terminated_1 {
   unsigned int __w_termsig : 7 ;
   unsigned int __w_coredump : 1 ;
   unsigned int __w_retcode : 8 ;
   unsigned int  : 16 ;
};
union wait {
   int w_status ;
   struct __anonstruct___wait_terminated_1 __wait_terminated ;
   struct __anonstruct___wait_stopped_2 __wait_stopped ;
};
struct __anonstruct___wait_stopped_2;
struct __anonstruct___wait_terminated_1;
typedef int YYSTYPE;
enum yytokentype {
    END = 0,
    T_REQUIRE_ONCE = 258,
    T_REQUIRE = 259,
    T_EVAL = 260,
    T_INCLUDE_ONCE = 261,
    T_INCLUDE = 262,
    T_LOGICAL_OR = 263,
    T_LOGICAL_XOR = 264,
    T_LOGICAL_AND = 265,
    T_PRINT = 266,
    T_SR_EQUAL = 267,
    T_SL_EQUAL = 268,
    T_XOR_EQUAL = 269,
    T_OR_EQUAL = 270,
    T_AND_EQUAL = 271,
    T_MOD_EQUAL = 272,
    T_CONCAT_EQUAL = 273,
    T_DIV_EQUAL = 274,
    T_MUL_EQUAL = 275,
    T_MINUS_EQUAL = 276,
    T_PLUS_EQUAL = 277,
    T_BOOLEAN_OR = 278,
    T_BOOLEAN_AND = 279,
    T_IS_NOT_IDENTICAL = 280,
    T_IS_IDENTICAL = 281,
    T_IS_NOT_EQUAL = 282,
    T_IS_EQUAL = 283,
    T_IS_GREATER_OR_EQUAL = 284,
    T_IS_SMALLER_OR_EQUAL = 285,
    T_SR = 286,
    T_SL = 287,
    T_INSTANCEOF = 288,
    T_UNSET_CAST = 289,
    T_BOOL_CAST = 290,
    T_OBJECT_CAST = 291,
    T_ARRAY_CAST = 292,
    T_STRING_CAST = 293,
    T_DOUBLE_CAST = 294,
    T_INT_CAST = 295,
    T_DEC = 296,
    T_INC = 297,
    T_CLONE = 298,
    T_NEW = 299,
    T_EXIT = 300,
    T_IF = 301,
    T_ELSEIF = 302,
    T_ELSE = 303,
    T_ENDIF = 304,
    T_LNUMBER = 305,
    T_DNUMBER = 306,
    T_STRING = 307,
    T_STRING_VARNAME = 308,
    T_VARIABLE = 309,
    T_NUM_STRING = 310,
    T_INLINE_HTML = 311,
    T_CHARACTER = 312,
    T_BAD_CHARACTER = 313,
    T_ENCAPSED_AND_WHITESPACE = 314,
    T_CONSTANT_ENCAPSED_STRING = 315,
    T_ECHO = 316,
    T_DO = 317,
    T_WHILE = 318,
    T_ENDWHILE = 319,
    T_FOR = 320,
    T_ENDFOR = 321,
    T_FOREACH = 322,
    T_ENDFOREACH = 323,
    T_DECLARE = 324,
    T_ENDDECLARE = 325,
    T_AS = 326,
    T_SWITCH = 327,
    T_ENDSWITCH = 328,
    T_CASE = 329,
    T_DEFAULT = 330,
    T_BREAK = 331,
    T_CONTINUE = 332,
    T_GOTO = 333,
    T_FUNCTION = 334,
    T_CONST = 335,
    T_RETURN = 336,
    T_TRY = 337,
    T_CATCH = 338,
    T_THROW = 339,
    T_USE = 340,
    T_INSTEADOF = 341,
    T_GLOBAL = 342,
    T_PUBLIC = 343,
    T_PROTECTED = 344,
    T_PRIVATE = 345,
    T_FINAL = 346,
    T_ABSTRACT = 347,
    T_STATIC = 348,
    T_VAR = 349,
    T_UNSET = 350,
    T_ISSET = 351,
    T_EMPTY = 352,
    T_HALT_COMPILER = 353,
    T_CLASS = 354,
    T_TRAIT = 355,
    T_INTERFACE = 356,
    T_EXTENDS = 357,
    T_IMPLEMENTS = 358,
    T_OBJECT_OPERATOR = 359,
    T_DOUBLE_ARROW = 360,
    T_LIST = 361,
    T_ARRAY = 362,
    T_CALLABLE = 363,
    T_CLASS_C = 364,
    T_TRAIT_C = 365,
    T_METHOD_C = 366,
    T_FUNC_C = 367,
    T_LINE = 368,
    T_FILE = 369,
    T_COMMENT = 370,
    T_DOC_COMMENT = 371,
    T_OPEN_TAG = 372,
    T_OPEN_TAG_WITH_ECHO = 373,
    T_CLOSE_TAG = 374,
    T_WHITESPACE = 375,
    T_START_HEREDOC = 376,
    T_END_HEREDOC = 377,
    T_DOLLAR_OPEN_CURLY_BRACES = 378,
    T_CURLY_OPEN = 379,
    T_PAAMAYIM_NEKUDOTAYIM = 380,
    T_NAMESPACE = 381,
    T_NS_C = 382,
    T_DIR = 383,
    T_NS_SEPARATOR = 384
} ;
enum yytokentype;
struct _IO_FILE *_coverage_fout  ;
