#include <stdlib.h>
#include <stdio.h>

char * TESTS[] =
 {
    "ext/sockets/tests/mcast_ipv4_recv-detect.phpt",
    "ext/sockets/tests/bug46360-detect.phpt",
    "ext/sockets/tests/socket_create_pair-wrongparams-detect.phpt",
    "ext/sockets/tests/ipv4loop-detect.phpt",
    "ext/sockets/tests/socket_import_stream-2-detect.phpt",
    "ext/sockets/tests/mcast_ipv4_send_error-detect.phpt",
    "ext/sockets/tests/socket_bind_params-detect.phpt",
    "ext/sockets/tests/socket_write_params-detect.phpt",
    "ext/sockets/tests/socket_connect_error-detect.phpt",
    "ext/sockets/tests/socket_import_stream-3-detect.phpt",
    "ext/sockets/tests/socket_listen_params-detect.phpt",
    "ext/sockets/tests/socket_select-wrongparams-2-detect.phpt",
    "ext/sockets/tests/socket_create_params-detect.phpt",
    "ext/sockets/tests/socket_set_nonblock-wrongparams-detect.phpt",
    "ext/sockets/tests/socket_select-wrongparams-1-detect.phpt",
    "ext/sockets/tests/mcast_ipv4_send-detect.phpt",
    "ext/sockets/tests/socket_create_listen_used-detect.phpt",
    "ext/sockets/tests/socket_create_listen-wrongparams-detect.phpt",
    "ext/sockets/tests/socket_bind-detect.phpt",
    "ext/sockets/tests/socket_set_option_rcvtimeo-detect.phpt",
    "ext/sockets/tests/socket_import_stream-5-detect.phpt",
    "ext/sockets/tests/socket_create_listen-nobind-detect.phpt",
    "ext/sockets/tests/socket_getpeername_ipv6loop-detect.phpt",
    "ext/sockets/tests/unixloop-detect.phpt",
    "ext/sockets/tests/socket_select-detect.phpt",
    "ext/sockets/tests/socket_getpeername-detect.phpt",
    "ext/sockets/tests/socket_getpeername_ipv4loop-detect.phpt",
    "ext/sockets/tests/ipv6loop-detect.phpt",
    "ext/sockets/tests/socket_accept-wrongparams-detect.phpt",
    "ext/sockets/tests/socket_select-wrongparams-3-detect.phpt",
    "ext/sockets/tests/socket_getsockname-detect.phpt",
    "ext/sockets/tests/socket_set_block-retval-detect.phpt",
    "ext/sockets/tests/socket_listen-wrongparams-detect.phpt",
    "ext/sockets/tests/socket_create_listen-detect.phpt",
    "ext/sockets/tests/mcast_ipv6_recv_limited-detect.phpt",
    "ext/sockets/tests/socket_read_params-detect.phpt",
    "ext/sockets/tests/socket_strerror-detect.phpt",
    "ext/sockets/tests/socket_set_nonblock-retval-detect.phpt",
    "ext/sockets/tests/socket_sentto_recvfrom_ipv4_udp-detect.phpt",
    "ext/sockets/tests/socket_select-wrongparams-4-detect.phpt",
    "ext/sockets/tests/socket_create_listen_params-detect.phpt",
    "ext/sockets/tests/socket_set_option_sndtimeo-detect.phpt",
    "ext/sockets/tests/socket_set_block-wrongparams-detect.phpt",
    "ext/sockets/tests/bug51958-detect.phpt",
    "ext/sockets/tests/mcast_ipv6_send-detect.phpt",
    "ext/sockets/tests/socket_set_option_seolinger-detect.phpt",
    "ext/sockets/tests/socket_create_listen-win32-detect.phpt",
    "ext/sockets/tests/mcast_ipv6_recv-detect.phpt",
    "ext/sockets/tests/socket_close_params-detect.phpt",
    "ext/sockets/tests/socket_import_stream-1-detect.phpt",
    "ext/sockets/tests/socket_sentto_recvfrom_ipv6_udp-detect.phpt",
    "ext/sockets/tests/socket_import_stream-4-win-detect.phpt",
    "ext/sockets/tests/socket_select-wrongparams-1-win32-detect.phpt",
    "ext/sockets/tests/socket_create_pair-detect.phpt",
    "ext/sockets/tests/socket_import_stream-4-detect.phpt",
    "ext/sockets/tests/socket_connect_params-detect.phpt",
    "ext/sockets/tests/socket_create_pair-wrongparams-win32-detect.phpt",
    "ext/sockets/tests/socket_sentto_recvfrom_unix-detect.phpt",
    "ext/sockets/tests/socket_set_nonblock-detect.phpt",
    "ext/sockets/tests/socket_set_option_error_socket_option-detect.phpt",
    "ext/enchant/tests/broker_init-detect.phpt",
    "ext/enchant/tests/broker_free-detect.phpt",
    "ext/enchant/tests/dict_quick_check-detect.phpt",
    "ext/enchant/tests/broker_describe-detect.phpt",
    "ext/enchant/tests/broker_request_dict-detect.phpt",
    "ext/enchant/tests/bug13181-detect.phpt",
    "ext/sybase_ct/tests/bug28354-detect.phpt",
    "ext/sybase_ct/tests/test_close_notopen-detect.phpt",
    "ext/sybase_ct/tests/bug30312-withfree-detect.phpt",
    "ext/sybase_ct/tests/bug30312-detect.phpt",
    "ext/sybase_ct/tests/test_msghandler_handled-detect.phpt",
    "ext/sybase_ct/tests/test_unbuffered_no_full_fetch-detect.phpt",
    "ext/sybase_ct/tests/test_connection_caching-detect.phpt",
    "ext/sybase_ct/tests/test_unbuffered_query-detect.phpt",
    "ext/sybase_ct/tests/test_query_nostore-detect.phpt",
    "ext/sybase_ct/tests/bug27843-detect.phpt",
    "ext/sybase_ct/tests/bug43578-detect.phpt",
    "ext/sybase_ct/tests/bug29064-detect.phpt",
    "ext/sybase_ct/tests/test_close_default-detect.phpt",
    "ext/sybase_ct/tests/test_long-detect.phpt",
    "ext/sybase_ct/tests/test_fetch_object-detect.phpt",
    "ext/sybase_ct/tests/bug22403-detect.phpt",
    "ext/sybase_ct/tests/test_msghandler-detect.phpt",
    "ext/sybase_ct/tests/test_close-detect.phpt",
    "ext/sybase_ct/tests/test_fields-detect.phpt",
    "ext/sybase_ct/tests/bug26407-detect.phpt",
    "ext/sybase_ct/tests/test_connectionbased_msghandler-detect.phpt",
    "ext/sybase_ct/tests/test_connect-detect.phpt",
    "ext/sybase_ct/tests/test_types-detect.phpt",
    "ext/sybase_ct/tests/test_appname-detect.phpt",
    "ext/sybase_ct/tests/bug6339-detect.phpt",
    "ext/skeleton/tests/001-detect.phpt",
    "ext/sqlite3/tests/sqlite3_32_createAggregate_paramCount-detect.phpt",
    "ext/sqlite3/tests/sqlite3_close_with_params-detect.phpt",
    "ext/sqlite3/tests/sqlite3_22_loadextension-detect.phpt",
    "ext/sqlite3/tests/sqlite3_33_reset-detect.phpt",
    "ext/sqlite3/tests/sqlite3_19_columninfo-detect.phpt",
    "ext/sqlite3/tests/sqlite3_32_last_insert_rowid_param-detect.phpt",
    "ext/sqlite3/tests/sqlite3_close_error-detect.phpt",
    "ext/sqlite3/tests/bug45798-detect.phpt",
    "ext/sqlite3/tests/sqlite3_27_reset_prepared_stmt_result-detect.phpt",
    "ext/sqlite3/tests/sqlite3stmt_reset_params-detect.phpt",
    "ext/sqlite3/tests/sqlite3_loadextension_with_wrong_param-detect.phpt",
    "ext/sqlite3/tests/sqlite3_21_security-detect.phpt",
    "ext/sqlite3/tests/sqlite3_36_create_collation-detect.phpt",
    "ext/sqlite3/tests/sqlite3_06_prepared_stmt-detect.phpt",
    "ext/sqlite3/tests/sqlite3result_fetcharray_with_two_params_fails-detect.phpt",
    "ext/sqlite3/tests/sqlite3_prepare_with_empty_string-detect.phpt",
    "ext/sqlite3/tests/sqlite3_version_noparam-detect.phpt",
    "ext/sqlite3/tests/sqlite3_31_open-detect.phpt",
    "ext/sqlite3/tests/sqlite3_02_create-detect.phpt",
    "ext/sqlite3/tests/sqlite3_11_numrows-detect.phpt",
    "ext/sqlite3/tests/sqlite3_07_prepared_stmt-detect.phpt",
    "ext/sqlite3/tests/sqlite3_12_unfinalized_stmt_cleanup-detect.phpt",
    "ext/sqlite3/tests/sqlite3_01_open-detect.phpt",
    "ext/sqlite3/tests/sqlite3_18_changes-detect.phpt",
    "ext/sqlite3/tests/sqlite3_13_skip_all_cleanup-detect.phpt",
    "ext/sqlite3/tests/sqlite3_14_querysingle-detect.phpt",
    "ext/sqlite3/tests/sqlite3_26_reset_prepared_stmt-detect.phpt",
    "ext/sqlite3/tests/sqlite3_enable_exceptions-detect.phpt",
    "ext/sqlite3/tests/sqlite3_15_open_error-detect.phpt",
    "ext/sqlite3/tests/sqlite3_02_open-detect.phpt",
    "ext/sqlite3/tests/sqlite3_32_changes-detect.phpt",
    "ext/sqlite3/tests/sqlite3_31_changes-detect.phpt",
    "ext/sqlite3/tests/sqlite3_lasterrormsg_with_params-detect.phpt",
    "ext/sqlite3/tests/bug47159-detect.phpt",
    "ext/sqlite3/tests/sqlite3stmt_paramCount_basic-detect.phpt",
    "ext/sqlite3/tests/sqlite3_34_load_extension_ext_dir-detect.phpt",
    "ext/sqlite3/tests/sqlite3_17_version-detect.phpt",
    "ext/sqlite3/tests/sqlite3_04_update-detect.phpt",
    "ext/sqlite3/tests/sqlite3_open_empty_string-detect.phpt",
    "ext/sqlite3/tests/sqlite3_exec_wrongparams-detect.phpt",
    "ext/sqlite3/tests/sqlite3_prepare_faultystmt-detect.phpt",
    "ext/sqlite3/tests/sqlite3_30_blobopen-detect.phpt",
    "ext/sqlite3/tests/sqlite3stmt_paramCount_error-detect.phpt",
    "ext/sqlite3/tests/sqlite3_15_open_error-win-detect.phpt",
    "ext/sqlite3/tests/sqlite3_16_select_no_results-detect.phpt",
    "ext/sqlite3/tests/sqlite3_05_delete-detect.phpt",
    "ext/sqlite3/tests/sqlite3_24_last_insert_rowid-detect.phpt",
    "ext/sqlite3/tests/sqlite3_28_clear_bindings-detect.phpt",
    "ext/sqlite3/tests/sqlite3_29_createfunction-detect.phpt",
    "ext/sqlite3/tests/bug53463-detect.phpt",
    "ext/sqlite3/tests/sqlite3_querysingle_error-detect.phpt",
    "ext/sqlite3/tests/sqlite3_03_insert-detect.phpt",
    "ext/sqlite3/tests/sqlite3result_reset_with_params_fails-detect.phpt",
    "ext/sqlite3/tests/sqlite3_10_bound_value_name-detect.phpt",
    "ext/sqlite3/tests/sqlite3_25_create_aggregate-detect.phpt",
    "ext/sqlite3/tests/sqlite3_openblob_wrongparams-detect.phpt",
    "ext/sqlite3/tests/sqlite3result_numcolumns_error-detect.phpt",
    "ext/sqlite3/tests/sqlite3_lasterrorcode_with_params-detect.phpt",
    "ext/sqlite3/tests/sqlite3_query_error-detect.phpt",
    "ext/sqlite3/tests/sqlite3_08_udf-detect.phpt",
    "ext/sqlite3/tests/sqlite3_prepared_stmt_clear_with_params-detect.phpt",
    "ext/sqlite3/tests/sqlite3_09_blob_bound_param-detect.phpt",
    "ext/sqlite3/tests/sqlite3_20_error-detect.phpt",
    "ext/sqlite3/tests/sqlite3_33_createAggregate_notcallable-detect.phpt",
    "ext/sqlite3/tests/sqlite3_prepare_wrongparams-detect.phpt",
    "ext/sqlite3/tests/sqlite3_prepare_001-detect.phpt",
    "ext/sqlite3/tests/sqlite3_35_stmt_readonly-detect.phpt",
    "ext/sqlite3/tests/sqlite3_33_load_extension_param-detect.phpt",
    "ext/sqlite3/tests/sqlite3_23_escape_string-detect.phpt",
    "ext/xmlrpc/tests/bug45555-detect.phpt",
    "ext/xmlrpc/tests/bug42189-detect.phpt",
    "ext/xmlrpc/tests/bug51288-detect.phpt",
    "ext/xmlrpc/tests/bug18916-detect.phpt",
    "ext/xmlrpc/tests/bug50761-detect.phpt",
    "ext/xmlrpc/tests/bug40576_64bit-detect.phpt",
    "ext/xmlrpc/tests/bug47818-detect.phpt",
    "ext/xmlrpc/tests/bug40576-detect.phpt",
    "ext/xmlrpc/tests/001-detect.phpt",
    "ext/xmlrpc/tests/bug61097-detect.phpt",
    "ext/xmlrpc/tests/bug50282-detect.phpt",
    "ext/xmlrpc/tests/bug44996-detect.phpt",
    "ext/xmlrpc/tests/bug42736-detect.phpt",
    "ext/xmlrpc/tests/bug38431-detect.phpt",
    "ext/xmlrpc/tests/bug37057-detect.phpt",
    "ext/xmlrpc/tests/bug45556-detect.phpt",
    "ext/xmlrpc/tests/002-detect.phpt",
    "ext/xmlrpc/tests/bug61264-detect.phpt",
    "ext/xmlrpc/tests/bug50285-detect.phpt",
    "ext/xmlrpc/tests/bug45226-detect.phpt",
    "ext/pdo_oci/tests/pdo_oci_quote1-detect.phpt",
    "ext/pdo_oci/tests/pdo_oci_attr_client-detect.phpt",
    "ext/pdo_oci/tests/pdo_oci_attr_prefetch_1-detect.phpt",
    "ext/pdo_oci/tests/pdo_oci_attr_prefetch_2-detect.phpt",
    "ext/pdo_oci/tests/bug_33707-detect.phpt",
    "ext/pdo_oci/tests/bug41996-detect.phpt",
    "ext/pdo_oci/tests/pdo_oci_stream_1-detect.phpt",
    "ext/pdo_oci/tests/bug46274_2-detect.phpt",
    "ext/pdo_oci/tests/pdo_oci_attr_autocommit_2-detect.phpt",
    "ext/pdo_oci/tests/pecl_bug_6364-detect.phpt",
    "ext/pdo_oci/tests/pdo_oci_attr_autocommit_1-detect.phpt",
    "ext/pdo_oci/tests/bug46274-detect.phpt",
    "ext/pdo_oci/tests/common-detect.phpt",
    "ext/pdo_oci/tests/pdo_oci_attr_autocommit_3-detect.phpt",
    "ext/pdo_oci/tests/pdo_oci_attr_server-detect.phpt",
    "ext/pdo_oci/tests/bug44301-detect.phpt",
    "ext/pdo_oci/tests/pdo_oci_attr_nulls_1-detect.phpt",
    "ext/pdo_oci/tests/pdo_oci_attr_case-detect.phpt",
    "ext/pdo_oci/tests/pdo_oci_fread_1-detect.phpt",
    "ext/pdo_oci/tests/pecl_bug_11345-detect.phpt",
    "ext/pdo_oci/tests/pdo_oci_attr_drivername-detect.phpt",
    "ext/date/tests/bug32086-detect.phpt",
    "ext/date/tests/date_diff1-detect.phpt",
    "ext/date/tests/bug34087-detect.phpt",
    "ext/date/tests/DateTime_setTime_variation2-detect.phpt",
    "ext/date/tests/bug36988-detect.phpt",
    "ext/date/tests/gmstrftime_variation22-detect.phpt",
    "ext/date/tests/date_timezone_set_variation2-detect.phpt",
    "ext/date/tests/checkdate_variation2-detect.phpt",
    "ext/date/tests/DateTime_format_error-detect.phpt",
    "ext/date/tests/oo_001-detect.phpt",
    "ext/date/tests/date_create_from_format_basic2-detect.phpt",
    "ext/date/tests/DateTime_format_basic1-detect.phpt",
    "ext/date/tests/bug49059-detect.phpt",
    "ext/date/tests/DateTime_construct-dst-overlap-detect.phpt",
    "ext/date/tests/bug48476-detect.phpt",
    "ext/date/tests/010-detect.phpt",
    "ext/date/tests/bug33536-detect.phpt",
    "ext/date/tests/date-parse-by-format001-detect.phpt",
    "ext/date/tests/DateTimeZone_getOffset_variation1-detect.phpt",
    "ext/date/tests/bug33578-detect.phpt",
    "ext/date/tests/bug43003-detect.phpt",
    "ext/date/tests/gmstrftime_basic-detect.phpt",
    "ext/date/tests/bug27780-detect.phpt",
    "ext/date/tests/DateTime_days-absolute-detect.phpt",
    "ext/date/tests/DateTime_sub-spring-type3-type3-detect.phpt",
    "ext/date/tests/DatePeriod_wrong_constructor-detect.phpt",
    "ext/date/tests/DateTime_setDate_variation3-detect.phpt",
    "ext/date/tests/date_sunrise_variation4-detect.phpt",
    "ext/date/tests/bug41599-detect.phpt",
    "ext/date/tests/oo_002-detect.phpt",
    "ext/date/tests/time_error-detect.phpt",
    "ext/date/tests/strtotime2-detect.phpt",
    "ext/date/tests/005-detect.phpt",
    "ext/date/tests/DateTimeZone_getTransitions_basic1-detect.phpt",
    "ext/date/tests/DateTime_diff-february-detect.phpt",
    "ext/date/tests/date_isodate_set_variation2-detect.phpt",
    "ext/date/tests/timezone_offset_get_error-detect.phpt",
    "ext/date/tests/gmmktime_variation5-detect.phpt",
    "ext/date/tests/DateTime_setDate_basic1-detect.phpt",
    "ext/date/tests/bug48058-detect.phpt",
    "ext/date/tests/gmstrftime_variation7-detect.phpt",
    "ext/date/tests/getdate_basic-detect.phpt",
    "ext/date/tests/bug52454-detect.phpt",
    "ext/date/tests/gmstrftime_variation11-detect.phpt",
    "ext/date/tests/timezone_name_from_abbr_basic1-detect.phpt",
    "ext/date/tests/014-detect.phpt",
    "ext/date/tests/gmmktime_variation6-detect.phpt",
    "ext/date/tests/bug43808-detect.phpt",
    "ext/date/tests/DateTime_days-fall-type2-type3-detect.phpt",
    "ext/date/tests/DateTime_clone_basic4-detect.phpt",
    "ext/date/tests/date_time_set_variation3-detect.phpt",
    "ext/date/tests/bug29595-detect.phpt",
    "ext/date/tests/bug41523-64bit-detect.phpt",
    "ext/date/tests/getdate_error-detect.phpt",
    "ext/date/tests/gmstrftime_variation8-detect.phpt",
    "ext/date/tests/localtime_basic-detect.phpt",
    "ext/date/tests/gmstrftime_variation18-detect.phpt",
    "ext/date/tests/bug41842-detect.phpt",
    "ext/date/tests/bug46108-detect.phpt",
    "ext/date/tests/date_parse_basic1-detect.phpt",
    "ext/date/tests/DateTime_sub-fall-type3-type3-detect.phpt",
    "ext/date/tests/DateTimeZone_clone_basic3-detect.phpt",
    "ext/date/tests/011-detect.phpt",
    "ext/date/tests/DateTime_modify_variation1-detect.phpt",
    "ext/date/tests/bug29150-detect.phpt",
    "ext/date/tests/date_date_set_variation3-detect.phpt",
    "ext/date/tests/strtotime-detect.phpt",
    "ext/date/tests/bug43075-detect.phpt",
    "ext/date/tests/strtotime_basic-detect.phpt",
    "ext/date/tests/DateTime_setISODate_variation3-detect.phpt",
    "ext/date/tests/default-timezone-1-detect.phpt",
    "ext/date/tests/test-parse-from-format-detect.phpt",
    "ext/date/tests/localtime_variation3-detect.phpt",
    "ext/date/tests/getdate_variation5-detect.phpt",
    "ext/date/tests/DateTime_diff-dates-detect.phpt",
    "ext/date/tests/gmstrftime_error-detect.phpt",
    "ext/date/tests/bug21399-detect.phpt",
    "ext/date/tests/gmstrftime_variation3-detect.phpt",
    "ext/date/tests/DateTimeZone_compare_basic1-detect.phpt",
    "ext/date/tests/gmdate_variation4-detect.phpt",
    "ext/date/tests/date_create-2-detect.phpt",
    "ext/date/tests/DateTime_sub-spring-type2-type2-detect.phpt",
    "ext/date/tests/strftime_error-detect.phpt",
    "ext/date/tests/date_variation1-detect.phpt",
    "ext/date/tests/DateTime_modify_basic1-detect.phpt",
    "ext/date/tests/gmdate_variation10-detect.phpt",
    "ext/date/tests/bug35624-detect.phpt",
    "ext/date/tests/bug30096-detect.phpt",
    "ext/date/tests/gmstrftime_variation5-detect.phpt",
    "ext/date/tests/gmdate_variation2-detect.phpt",
    "ext/date/tests/timezone_offset_get_variation2-detect.phpt",
    "ext/date/tests/DateTime_add-fall-type2-type2-detect.phpt",
    "ext/date/tests/date_diff-detect.phpt",
    "ext/date/tests/strtotime-mysql-64bit-detect.phpt",
    "ext/date/tests/date_sunrise_variation7-detect.phpt",
    "ext/date/tests/DateTime_add-february-detect.phpt",
    "ext/date/tests/DateTime_getOffset_basic1-detect.phpt",
    "ext/date/tests/timezone_transitions_get_basic1-detect.phpt",
    "ext/date/tests/gmstrftime_variation17-detect.phpt",
    "ext/date/tests/strftime_variation12-detect.phpt",
    "ext/date/tests/DateTimeZone_getTransitions_error-detect.phpt",
    "ext/date/tests/timezone_offset_get_variation1-detect.phpt",
    "ext/date/tests/gmdate_variation12-detect.phpt",
    "ext/date/tests/bug51393-detect.phpt",
    "ext/date/tests/strtotime_variation_scottish-detect.phpt",
    "ext/date/tests/DateTime_setTime_variation3-detect.phpt",
    "ext/date/tests/date_create_basic-detect.phpt",
    "ext/date/tests/date_sunrise_variation5-detect.phpt",
    "ext/date/tests/bug46874-detect.phpt",
    "ext/date/tests/bug43960-detect.phpt",
    "ext/date/tests/bug62852-detect.phpt",
    "ext/date/tests/date_isodate_set_variation3-detect.phpt",
    "ext/date/tests/gmdate_error-detect.phpt",
    "ext/date/tests/format-negative-timestamp-detect.phpt",
    "ext/date/tests/009_win32-detect.phpt",
    "ext/date/tests/getdate_variation7-detect.phpt",
    "ext/date/tests/date_sun_info_variation3-detect.phpt",
    "ext/date/tests/idate_error-detect.phpt",
    "ext/date/tests/DateTime_format_variation1-detect.phpt",
    "ext/date/tests/bug34771-detect.phpt",
    "ext/date/tests/DateTime_diff-massive-detect.phpt",
    "ext/date/tests/bug36510-detect.phpt",
    "ext/date/tests/bug35885-detect.phpt",
    "ext/date/tests/date_sunset_variation4-detect.phpt",
    "ext/date/tests/gmdate_variation14-detect.phpt",
    "ext/date/tests/idate_variation3-detect.phpt",
    "ext/date/tests/gmdate_variation3-detect.phpt",
    "ext/date/tests/date_timezone_get_variation1-detect.phpt",
    "ext/date/tests/DateTime_diff-fall-type2-type3-detect.phpt",
    "ext/date/tests/bug33452-detect.phpt",
    "ext/date/tests/date_date_set_variation1-detect.phpt",
    "ext/date/tests/bug43527-detect.phpt",
    "ext/date/tests/DateTime_diff-spring-type3-type2-detect.phpt",
    "ext/date/tests/bug45554-detect.phpt",
    "ext/date/tests/strtotime-relative-detect.phpt",
    "ext/date/tests/DateTimeZone_extends_basic1-detect.phpt",
    "ext/date/tests/bug33415-2-detect.phpt",
    "ext/date/tests/date_sunrise_variation2-detect.phpt",
    "ext/date/tests/date_timestamp_get-detect.phpt",
    "ext/date/tests/015-detect.phpt",
    "ext/date/tests/bug35887-detect.phpt",
    "ext/date/tests/date_timezone_set_error-detect.phpt",
    "ext/date/tests/DateTimeZone_getName_basic1-detect.phpt",
    "ext/date/tests/date_variation2-detect.phpt",
    "ext/date/tests/bug35218-detect.phpt",
    "ext/date/tests/DateTime_format_basic2-detect.phpt",
    "ext/date/tests/date_sub_basic-detect.phpt",
    "ext/date/tests/DateTimeZone_construct_basic-detect.phpt",
    "ext/date/tests/date_add_basic-detect.phpt",
    "ext/date/tests/strftime_variation22-detect.phpt",
    "ext/date/tests/date_sunset_variation7-detect.phpt",
    "ext/date/tests/bug51819-detect.phpt",
    "ext/date/tests/mktime_variation1-detect.phpt",
    "ext/date/tests/004-detect.phpt",
    "ext/date/tests/gmmktime_variation9-detect.phpt",
    "ext/date/tests/DateTime_compare_basic1-detect.phpt",
    "ext/date/tests/gmstrftime_variation2-detect.phpt",
    "ext/date/tests/DateTime_days-fall-type3-type2-detect.phpt",
    "ext/date/tests/mktime_variation2-detect.phpt",
    "ext/date/tests/bug30532-detect.phpt",
    "ext/date/tests/DateTime_days-spring-type3-type2-detect.phpt",
    "ext/date/tests/strftime_variation7-detect.phpt",
    "ext/date/tests/DateTime_serialize-detect.phpt",
    "ext/date/tests/DateTime_diff-fall-type3-type3-detect.phpt",
    "ext/date/tests/date_modify_variation1-detect.phpt",
    "ext/date/tests/idate_variation4-detect.phpt",
    "ext/date/tests/date_sunrise_variation1-detect.phpt",
    "ext/date/tests/date_format_variation2-detect.phpt",
    "ext/date/tests/default-timezone-2-detect.phpt",
    "ext/date/tests/date_offset_get_variation1-detect.phpt",
    "ext/date/tests/bug27719-detect.phpt",
    "ext/date/tests/DateTimeZone_construct_variation1-detect.phpt",
    "ext/date/tests/timezones-detect.phpt",
    "ext/date/tests/bug52808-detect.phpt",
    "ext/date/tests/date_default_timezone_get_error-detect.phpt",
    "ext/date/tests/date_sun_info_001-detect.phpt",
    "ext/date/tests/date_constants-detect.phpt",
    "ext/date/tests/DateTimeZone_getName_error-detect.phpt",
    "ext/date/tests/bug49081-detect.phpt",
    "ext/date/tests/date_create_variation1-detect.phpt",
    "ext/date/tests/DateTime_add-massive-detect.phpt",
    "ext/date/tests/DateTime_sub-fall-type3-type2-detect.phpt",
    "ext/date/tests/bug33415-1-detect.phpt",
    "ext/date/tests/DateTimeZone_construct_error-detect.phpt",
    "ext/date/tests/DateTime_diff-fall-type3-type2-detect.phpt",
    "ext/date/tests/date_parse_001-detect.phpt",
    "ext/date/tests/bug33869-detect.phpt",
    "ext/date/tests/strftime_variation6-detect.phpt",
    "ext/date/tests/getdate_variation1-detect.phpt",
    "ext/date/tests/bug52062-64bit-detect.phpt",
    "ext/date/tests/strftime_variation4-detect.phpt",
    "ext/date/tests/date-lenient-create-detect.phpt",
    "ext/date/tests/bug35705-detect.phpt",
    "ext/date/tests/DateTime_verify-detect.phpt",
    "ext/date/tests/gmdate_variation7-detect.phpt",
    "ext/date/tests/bug38229-detect.phpt",
    "ext/date/tests/timezones-list-detect.phpt",
    "ext/date/tests/bug48276-detect.phpt",
    "ext/date/tests/bug26320-detect.phpt",
    "ext/date/tests/date_timezone_set_variation1-detect.phpt",
    "ext/date/tests/date_isodate_set_variation1-detect.phpt",
    "ext/date/tests/rfc-datetime_and_daylight_saving_time-type1-detect.phpt",
    "ext/date/tests/date_create-1-detect.phpt",
    "ext/date/tests/DateTimeZone_getOffset_basic1-detect.phpt",
    "ext/date/tests/mktime-3-64bit-detect.phpt",
    "ext/date/tests/bug28088-detect.phpt",
    "ext/date/tests/date_time_set_basic1-detect.phpt",
    "ext/date/tests/bug45529-detect.phpt",
    "ext/date/tests/date_sunset_variation3-detect.phpt",
    "ext/date/tests/DateTime_extends_basic1-detect.phpt",
    "ext/date/tests/bug35630-detect.phpt",
    "ext/date/tests/bug44742-detect.phpt",
    "ext/date/tests/gmdate_variation13-detect.phpt",
    "ext/date/tests/date_sunrise_variation8-detect.phpt",
    "ext/date/tests/003-detect.phpt",
    "ext/date/tests/timezone_name_get_error-detect.phpt",
    "ext/date/tests/bug35143-detect.phpt",
    "ext/date/tests/date_get_last_errors_basic-detect.phpt",
    "ext/date/tests/gmmktime_variation7-detect.phpt",
    "ext/date/tests/DateTime_setTime_basic1-detect.phpt",
    "ext/date/tests/bug42910-detect.phpt",
    "ext/date/tests/bug28599-detect.phpt",
    "ext/date/tests/gmstrftime_variation21-detect.phpt",
    "ext/date/tests/date_modify-2-detect.phpt",
    "ext/date/tests/DateTime_clone_basic3-detect.phpt",
    "ext/date/tests/bug45682-detect.phpt",
    "ext/date/tests/date_modify_error-detect.phpt",
    "ext/date/tests/bug52062-detect.phpt",
    "ext/date/tests/bug52290-detect.phpt",
    "ext/date/tests/date_sunrise_variation3-detect.phpt",
    "ext/date/tests/bug62500-detect.phpt",
    "ext/date/tests/gmdate_variation9-detect.phpt",
    "ext/date/tests/gmstrftime_variation16-detect.phpt",
    "ext/date/tests/DateTime_sub-spring-type3-type2-detect.phpt",
    "ext/date/tests/date_sunset_variation9-detect.phpt",
    "ext/date/tests/strftime_variation1-detect.phpt",
    "ext/date/tests/strftime_variation14-detect.phpt",
    "ext/date/tests/date_basic1-detect.phpt",
    "ext/date/tests/date_sunset_variation8-detect.phpt",
    "ext/date/tests/DateTime_setDate_variation1-detect.phpt",
    "ext/date/tests/date_modify_variation2-detect.phpt",
    "ext/date/tests/DateTimeZone_listAbbreviations_basic1-detect.phpt",
    "ext/date/tests/rfc-datetime_and_daylight_saving_time-type3-detect.phpt",
    "ext/date/tests/mktime_variation4-detect.phpt",
    "ext/date/tests/strftime_variation17-detect.phpt",
    "ext/date/tests/bug50055-detect.phpt",
    "ext/date/tests/bug34676-detect.phpt",
    "ext/date/tests/timezone_abbreviations_list_basic1-detect.phpt",
    "ext/date/tests/gmdate_variation11-detect.phpt",
    "ext/date/tests/DateTime_setTimezone_error-detect.phpt",
    "ext/date/tests/timezone_name_from_abbr_error-detect.phpt",
    "ext/date/tests/bug26090-detect.phpt",
    "ext/date/tests/mktime_variation5-detect.phpt",
    "ext/date/tests/mktime_variation7-detect.phpt",
    "ext/date/tests/date_time_set_variation1-detect.phpt",
    "ext/date/tests/date_sunrise_error-detect.phpt",
    "ext/date/tests/006-detect.phpt",
    "ext/date/tests/date_isodate_set_basic1-detect.phpt",
    "ext/date/tests/date_timezone_get_basic1-detect.phpt",
    "ext/date/tests/date_sunset_variation1-detect.phpt",
    "ext/date/tests/bug33562-detect.phpt",
    "ext/date/tests/bug49585-detect.phpt",
    "ext/date/tests/DateTime_days-massive-detect.phpt",
    "ext/date/tests/strftime_variation16-detect.phpt",
    "ext/date/tests/bug49778-detect.phpt",
    "ext/date/tests/strftime_variation2-detect.phpt",
    "ext/date/tests/timezone_open_error-detect.phpt",
    "ext/date/tests/gmstrftime_variation1-detect.phpt",
    "ext/date/tests/date_sunset_variation6-detect.phpt",
    "ext/date/tests/getdate_variation4-detect.phpt",
    "ext/date/tests/strftime_variation5-detect.phpt",
    "ext/date/tests/localtime_variation5-detect.phpt",
    "ext/date/tests/date_sun_info_variation2-detect.phpt",
    "ext/date/tests/timezone_name_from_abbr_variation3-detect.phpt",
    "ext/date/tests/date-time-modify-times-detect.phpt",
    "ext/date/tests/strftime_variation10-detect.phpt",
    "ext/date/tests/bug52668-detect.phpt",
    "ext/date/tests/date_create_error-detect.phpt",
    "ext/date/tests/DateTime_add-spring-type3-type3-detect.phpt",
    "ext/date/tests/date_parse_from_format_basic-detect.phpt",
    "ext/date/tests/bug14561-detect.phpt",
    "ext/date/tests/idate_variation1-detect.phpt",
    "ext/date/tests/DateTime_diff-spring-type3-type3-detect.phpt",
    "ext/date/tests/DateTime_construct_basic1-detect.phpt",
    "ext/date/tests/date_default_timezone_set-1-detect.phpt",
    "ext/date/tests/strftime_variation11-detect.phpt",
    "ext/date/tests/gettimeofday_variation1-detect.phpt",
    "ext/date/tests/date_sun_info_002-detect.phpt",
    "ext/date/tests/gmstrftime_variation15-detect.phpt",
    "ext/date/tests/getdate_variation6-detect.phpt",
    "ext/date/tests/cal_days_in_month_invalid_date-detect.phpt",
    "ext/date/tests/DateTime_setTimezone_basic1-detect.phpt",
    "ext/date/tests/bug37368-detect.phpt",
    "ext/date/tests/DateTime_extends_basic3-detect.phpt",
    "ext/date/tests/date_format_error-detect.phpt",
    "ext/date/tests/bug46268-detect.phpt",
    "ext/date/tests/checkdate_error-detect.phpt",
    "ext/date/tests/DateTime_diff-spring-type2-type3-detect.phpt",
    "ext/date/tests/mktime_basic1-detect.phpt",
    "ext/date/tests/bug48097-detect.phpt",
    "ext/date/tests/DateTime_setISODate_error-detect.phpt",
    "ext/date/tests/bug40743-detect.phpt",
    "ext/date/tests/DateTime_modify_error-detect.phpt",
    "ext/date/tests/date_offset_get_error-detect.phpt",
    "ext/date/tests/strftime_variation23-detect.phpt",
    "ext/date/tests/mktime-1-detect.phpt",
    "ext/date/tests/microtime_basic-detect.phpt",
    "ext/date/tests/bug33532-detect.phpt",
    "ext/date/tests/bug21966-detect.phpt",
    "ext/date/tests/bug54597-detect.phpt",
    "ext/date/tests/bug54316-detect.phpt",
    "ext/date/tests/bug13142-detect.phpt",
    "ext/date/tests/timestamp-in-dst-detect.phpt",
    "ext/date/tests/DateTime_add-fall-type2-type3-detect.phpt",
    "ext/date/tests/timezone_location_get-detect.phpt",
    "ext/date/tests/microtime_error-detect.phpt",
    "ext/date/tests/date_isodate_set_variation4-detect.phpt",
    "ext/date/tests/gmstrftime_variation13-detect.phpt",
    "ext/date/tests/idate_variation2-detect.phpt",
    "ext/date/tests/timezone_open_variation1-detect.phpt",
    "ext/date/tests/strftime_variation20-detect.phpt",
    "ext/date/tests/007-detect.phpt",
    "ext/date/tests/bug26694-detect.phpt",
    "ext/date/tests/sunfuncts-detect.phpt",
    "ext/date/tests/gmstrftime_variation12-detect.phpt",
    "ext/date/tests/DateTime_days-spring-type2-type2-detect.phpt",
    "ext/date/tests/bug20382-1-detect.phpt",
    "ext/date/tests/DateTime_setISODate_variation2-detect.phpt",
    "ext/date/tests/gmmktime_variation3-detect.phpt",
    "ext/date/tests/DateTime_clone_basic2-detect.phpt",
    "ext/date/tests/DateTime_add-fall-type3-type2-detect.phpt",
    "ext/date/tests/bug33414-1-detect.phpt",
    "ext/date/tests/DateInterval_format-detect.phpt",
    "ext/date/tests/bug37747-detect.phpt",
    "ext/date/tests/bug20382-2-detect.phpt",
    "ext/date/tests/idate_variation5-detect.phpt",
    "ext/date/tests/strftime_variation9-detect.phpt",
    "ext/date/tests/gmmktime_basic-detect.phpt",
    "ext/date/tests/timezone_open_basic1-detect.phpt",
    "ext/date/tests/date_default_timezone_get-1-win32-detect.phpt",
    "ext/date/tests/checkdate_variation3-detect.phpt",
    "ext/date/tests/DateTimeZone_clone_basic4-detect.phpt",
    "ext/date/tests/bug37616-detect.phpt",
    "ext/date/tests/DateTime_getOffset_error-detect.phpt",
    "ext/date/tests/strtotime3-64bit-detect.phpt",
    "ext/date/tests/date_date_set_basic1-detect.phpt",
    "ext/date/tests/strftime_variation15-detect.phpt",
    "ext/date/tests/bug35456-detect.phpt",
    "ext/date/tests/strtotime-mysql-detect.phpt",
    "ext/date/tests/bug50680-detect.phpt",
    "ext/date/tests/mktime_variation3-detect.phpt",
    "ext/date/tests/DateTime_construct_variation2-detect.phpt",
    "ext/date/tests/DateTime_sub-fall-type2-type3-detect.phpt",
    "ext/date/tests/bug32270-detect.phpt",
    "ext/date/tests/002-detect.phpt",
    "ext/date/tests/gmstrftime_variation19-detect.phpt",
    "ext/date/tests/bug26198-detect.phpt",
    "ext/date/tests/008-detect.phpt",
    "ext/date/tests/date_create_from_format_basic-detect.phpt",
    "ext/date/tests/DateTime_construct_error-detect.phpt",
    "ext/date/tests/date_sunset_error-detect.phpt",
    "ext/date/tests/DateTime_diff-fall-type2-type2-detect.phpt",
    "ext/date/tests/mktime_variation6-detect.phpt",
    "ext/date/tests/bug35699-detect.phpt",
    "ext/date/tests/DateTime_add-spring-type3-type2-detect.phpt",
    "ext/date/tests/DateTime_add-dates-detect.phpt",
    "ext/date/tests/date_date_set_error-detect.phpt",
    "ext/date/tests/date_add_basic2-detect.phpt",
    "ext/date/tests/strftime_variation13-detect.phpt",
    "ext/date/tests/date_sunset_variation2-detect.phpt",
    "ext/date/tests/date_sun_info_variation1-detect.phpt",
    "ext/date/tests/date_timezone_get_error-detect.phpt",
    "ext/date/tests/bug54851-detect.phpt",
    "ext/date/tests/date_parse_variation1-detect.phpt",
    "ext/date/tests/bug52342-detect.phpt",
    "ext/date/tests/timezone_transitions_get_error-detect.phpt",
    "ext/date/tests/date_create-relative-detect.phpt",
    "ext/date/tests/mktime_error-detect.phpt",
    "ext/date/tests/date_parse_error-detect.phpt",
    "ext/date/tests/bug50475-detect.phpt",
    "ext/date/tests/date_date_set_variation2-detect.phpt",
    "ext/date/tests/DateTime_sub-massive-detect.phpt",
    "ext/date/tests/date_error-detect.phpt",
    "ext/date/tests/bug41523-detect.phpt",
    "ext/date/tests/date_date_set_variation4-detect.phpt",
    "ext/date/tests/timezone_transitions_get_variation2-detect.phpt",
    "ext/date/tests/DateTime_setTimezone_variation1-detect.phpt",
    "ext/date/tests/bug60236-detect.phpt",
    "ext/date/tests/gmstrftime_variation6-detect.phpt",
    "ext/date/tests/bug50392-detect.phpt",
    "ext/date/tests/bug41709-detect.phpt",
    "ext/date/tests/DateTime_clone_basic1-detect.phpt",
    "ext/date/tests/bug33414-2-detect.phpt",
    "ext/date/tests/013-detect.phpt",
    "ext/date/tests/checkdate_basic1-detect.phpt",
    "ext/date/tests/gmmktime_error-detect.phpt",
    "ext/date/tests/DateTimeZone_clone_basic1-detect.phpt",
    "ext/date/tests/gmstrftime_variation4-detect.phpt",
    "ext/date/tests/bug40861-detect.phpt",
    "ext/date/tests/date_default_timezone_set_error-detect.phpt",
    "ext/date/tests/idate_variation6-detect.phpt",
    "ext/date/tests/gettimeofday_basic-detect.phpt",
    "ext/date/tests/DateTime_days-fall-type2-type2-detect.phpt",
    "ext/date/tests/bug33563-detect.phpt",
    "ext/date/tests/date_format_variation1-detect.phpt",
    "ext/date/tests/date-lenient-detect.phpt",
    "ext/date/tests/DateTime_days-dates-detect.phpt",
    "ext/date/tests/bug52577-detect.phpt",
    "ext/date/tests/bug35414-detect.phpt",
    "ext/date/tests/strftime_variation19-detect.phpt",
    "ext/date/tests/getdate_variation3-detect.phpt",
    "ext/date/tests/DateTime_add-spring-type2-type3-detect.phpt",
    "ext/date/tests/bug36224-detect.phpt",
    "ext/date/tests/gmdate_variation5-detect.phpt",
    "ext/date/tests/DateTime_getTimeZone_basic1-detect.phpt",
    "ext/date/tests/bug34304-detect.phpt",
    "ext/date/tests/date_offset_get_basic1-detect.phpt",
    "ext/date/tests/timezone_transitions_get_variation3-detect.phpt",
    "ext/date/tests/DateTime_setISODate_basic1-detect.phpt",
    "ext/date/tests/DateTime_setDate_error-detect.phpt",
    "ext/date/tests/date_sunrise_variation9-detect.phpt",
    "ext/date/tests/bug49700-detect.phpt",
    "ext/date/tests/localtime_variation1-detect.phpt",
    "ext/date/tests/timezone_identifiers_list_basic1-detect.phpt",
    "ext/date/tests/strtotime3-detect.phpt",
    "ext/date/tests/DateTime_add-spring-type2-type2-detect.phpt",
    "ext/date/tests/getdate_variation2-detect.phpt",
    "ext/date/tests/DateTimeZone_getOffset_error-detect.phpt",
    "ext/date/tests/date_sunrise_variation6-detect.phpt",
    "ext/date/tests/date-detect.phpt",
    "ext/date/tests/bug45866-detect.phpt",
    "ext/date/tests/012-detect.phpt",
    "ext/date/tests/bug32555-detect.phpt",
    "ext/date/tests/date_time_set_variation4-detect.phpt",
    "ext/date/tests/DateTimeZone_verify-detect.phpt",
    "ext/date/tests/gmdate_variation6-detect.phpt",
    "ext/date/tests/bug32588-detect.phpt",
    "ext/date/tests/gmdate_variation1-detect.phpt",
    "ext/date/tests/bug36599-detect.phpt",
    "ext/date/tests/DateTime_diff-spring-type2-type2-detect.phpt",
    "ext/date/tests/strftime_variation18-detect.phpt",
    "ext/date/tests/date_default_timezone_set_variation1-detect.phpt",
    "ext/date/tests/bug35422-detect.phpt",
    "ext/date/tests/mktime-2-detect.phpt",
    "ext/date/tests/bug41844-detect.phpt",
    "ext/date/tests/bug33056-detect.phpt",
    "ext/date/tests/time_basic-detect.phpt",
    "ext/date/tests/DateTime_sub-spring-type2-type3-detect.phpt",
    "ext/date/tests/DateTime_construct_variation1-detect.phpt",
    "ext/date/tests/gmstrftime_variation14-detect.phpt",
    "ext/date/tests/bug53437-detect.phpt",
    "ext/date/tests/bug17988-detect.phpt",
    "ext/date/tests/date_timezone_set_basic1-detect.phpt",
    "ext/date/tests/DateTime_add-fall-type3-type3-detect.phpt",
    "ext/date/tests/gmstrftime_variation9-detect.phpt",
    "ext/date/tests/strftime_variation8-detect.phpt",
    "ext/date/tests/bug46111-detect.phpt",
    "ext/date/tests/DateTime_setTime_variation1-detect.phpt",
    "ext/date/tests/DateTime_days-spring-type3-type3-detect.phpt",
    "ext/date/tests/timezone_transitions_get_variation1-detect.phpt",
    "ext/date/tests/bug51994-detect.phpt",
    "ext/date/tests/bug53502-detect.phpt",
    "ext/date/tests/bug54340-detect.phpt",
    "ext/date/tests/bug43452-detect.phpt",
    "ext/date/tests/bug28024-detect.phpt",
    "ext/date/tests/DateTimeZone_listIdentifiers_basic1-detect.phpt",
    "ext/date/tests/DateTime_sub-fall-type2-type2-detect.phpt",
    "ext/date/tests/localtime_variation4-detect.phpt",
    "ext/date/tests/gmmktime_variation1-detect.phpt",
    "ext/date/tests/DateTime_setTime_error-detect.phpt",
    "ext/date/tests/DateTime_setDate_variation2-detect.phpt",
    "ext/date/tests/DateTime_extends_basic2-detect.phpt",
    "ext/date/tests/timezone_offset_get_basic1-detect.phpt",
    "ext/date/tests/date_create_variation2-detect.phpt",
    "ext/date/tests/checkdate_variation1-detect.phpt",
    "ext/date/tests/date_time_set_error-detect.phpt",
    "ext/date/tests/bug37514-detect.phpt",
    "ext/date/tests/date_sunset_variation5-detect.phpt",
    "ext/date/tests/localtime_error-detect.phpt",
    "ext/date/tests/date_sun_info_error-detect.phpt",
    "ext/date/tests/DateTime_diff-absolute-detect.phpt",
    "ext/date/tests/bug35425-detect.phpt",
    "ext/date/tests/date-set-timestamp-detect.phpt",
    "ext/date/tests/cal_days_in_month_invalid_calendar-detect.phpt",
    "ext/date/tests/gmdate_basic-detect.phpt",
    "ext/date/tests/DateTime_days-spring-type2-type3-detect.phpt",
    "ext/date/tests/idate_basic-detect.phpt",
    "ext/date/tests/gmmktime_variation2-detect.phpt",
    "ext/date/tests/bug52113-detect.phpt",
    "ext/date/tests/timezone_name_from_abbr_variation1-detect.phpt",
    "ext/date/tests/DateTimeZone_serialize-detect.phpt",
    "ext/date/tests/bug48187-detect.phpt",
    "ext/date/tests/bug33957-detect.phpt",
    "ext/date/tests/DateTime_setISODate_variation1-detect.phpt",
    "ext/date/tests/bug35499-detect.phpt",
    "ext/date/tests/strtotime_basic2-detect.phpt",
    "ext/date/tests/bug55253-detect.phpt",
    "ext/date/tests/DateTime_sub-dates-detect.phpt",
    "ext/date/tests/DateInterval_format_a-detect.phpt",
    "ext/date/tests/DateTimeZone_clone_basic2-detect.phpt",
    "ext/date/tests/date_sunrise_and_sunset_basic-detect.phpt",
    "ext/date/tests/bug44562-detect.phpt",
    "ext/date/tests/bug41964-detect.phpt",
    "ext/date/tests/date_time_set_variation2-detect.phpt",
    "ext/date/tests/DateTime_days-fall-type3-type3-detect.phpt",
    "ext/date/tests/date_modify-1-detect.phpt",
    "ext/date/tests/date_format_basic1-detect.phpt",
    "ext/date/tests/date_default_timezone_get-2-detect.phpt",
    "ext/date/tests/bug29585-detect.phpt",
    "ext/date/tests/date_isodate_set_error-detect.phpt",
    "ext/date/tests/strftime_variation3-detect.phpt",
    "ext/date/tests/timezone_identifiers_list_wrong_constructor-detect.phpt",
    "ext/date/tests/date_period-detect.phpt",
    "ext/date/tests/bug52063-detect.phpt",
    "ext/date/tests/gmdate_variation8-detect.phpt",
    "ext/date/tests/bug37017-detect.phpt",
    "ext/date/tests/big_year-detect.phpt",
    "ext/date/tests/rfc-datetime_and_daylight_saving_time-type2-detect.phpt",
    "ext/date/tests/gmstrftime_variation20-detect.phpt",
    "ext/date/tests/strftime_basic-detect.phpt",
    "ext/date/tests/localtime_variation2-detect.phpt",
    "ext/date/tests/bug51096-detect.phpt",
    "ext/date/tests/009-detect.phpt",
    "ext/date/tests/bug51866-detect.phpt",
    "ext/date/tests/bug39782-detect.phpt",
    "ext/date/tests/timezone-configuration-detect.phpt",
    "ext/date/tests/date_modify_basic1-detect.phpt",
    "ext/date/tests/bug54283-detect.phpt",
    "ext/date/tests/date_default_timezone_get-3-detect.phpt",
    "ext/date/tests/date_default_timezone_get-1-detect.phpt",
    "ext/date/tests/DateTime_sub-february-detect.phpt",
    "ext/date/tests/gmmktime_variation8-detect.phpt",
    "ext/date/tests/gettimeofday_error-detect.phpt",
    "ext/date/tests/DateTime_days-february-detect.phpt",
    "ext/date/tests/mktime-3-detect.phpt",
    "ext/date/tests/strftime_variation21-detect.phpt",
    "ext/date/tests/bug52738-detect.phpt",
    "ext/date/tests/gmmktime_variation4-detect.phpt",
    "ext/date/tests/gmstrftime_variation10-detect.phpt",
    "ext/date/tests/bug52430-detect.phpt",
    "ext/date/tests/bug48678-detect.phpt",
    "ext/date/tests/timezone_name_from_abbr_variation2-detect.phpt",
    "ext/date/tests/bug26317-detect.phpt",
    "ext/ldap/tests/ldap_error_basic-detect.phpt",
    "ext/ldap/tests/ldap_mod_add_error-detect.phpt",
    "ext/ldap/tests/ldap_get_entries_variation-detect.phpt",
    "ext/ldap/tests/ldap_modify_error-detect.phpt",
    "ext/ldap/tests/ldap_get_entries_basic-detect.phpt",
    "ext/ldap/tests/ldap_connect_basic-detect.phpt",
    "ext/ldap/tests/ldap_set_rebind_proc_error-detect.phpt",
    "ext/ldap/tests/ldap_add_basic-detect.phpt",
    "ext/ldap/tests/ldap_delete_error-detect.phpt",
    "ext/ldap/tests/ldap_errno_error-detect.phpt",
    "ext/ldap/tests/ldap_control_paged_results_variation3-detect.phpt",
    "ext/ldap/tests/ldap_rename_error-detect.phpt",
    "ext/ldap/tests/ldap_read_error-detect.phpt",
    "ext/ldap/tests/ldap_search_error-detect.phpt",
    "ext/ldap/tests/ldap_get_option_basic-detect.phpt",
    "ext/ldap/tests/ldap_mod_replace_error-detect.phpt",
    "ext/ldap/tests/ldap_error_error-detect.phpt",
    "ext/ldap/tests/ldap_modify_basic-detect.phpt",
    "ext/ldap/tests/ldap_unbind_error-detect.phpt",
    "ext/ldap/tests/ldap_get_values_len_error-detect.phpt",
    "ext/ldap/tests/ldap_free_result_basic-detect.phpt",
    "ext/ldap/tests/ldap_dn2ufn-detect.phpt",
    "ext/ldap/tests/ldap_next_reference_basic-detect.phpt",
    "ext/ldap/tests/ldap_first_attribute_basic-detect.phpt",
    "ext/ldap/tests/ldap_count_entries_basic-detect.phpt",
    "ext/ldap/tests/ldap_compare_basic-detect.phpt",
    "ext/ldap/tests/ldap_parse_result_basic-detect.phpt",
    "ext/ldap/tests/ldap_search_variation1-detect.phpt",
    "ext/ldap/tests/ldap_get_entries_error-detect.phpt",
    "ext/ldap/tests/ldap_explode_dn-detect.phpt",
    "ext/ldap/tests/ldap_free_result_error-detect.phpt",
    "ext/ldap/tests/ldap_unbind_basic-detect.phpt",
    "ext/ldap/tests/ldap_search_variation3-detect.phpt",
    "ext/ldap/tests/ldap_set_rebind_proc_basic-detect.phpt",
    "ext/ldap/tests/ldap_search_variation6-detect.phpt",
    "ext/ldap/tests/ldap_get_dn_error-detect.phpt",
    "ext/ldap/tests/ldap_bind_error-detect.phpt",
    "ext/ldap/tests/ldap_parse_result_error-detect.phpt",
    "ext/ldap/tests/ldap_search_basic-detect.phpt",
    "ext/ldap/tests/ldap_connect_error-detect.phpt",
    "ext/ldap/tests/ldap_mod_replace_basic-detect.phpt",
    "ext/ldap/tests/ldap_first_entry_basic-detect.phpt",
    "ext/ldap/tests/ldap_first_attribute_error-detect.phpt",
    "ext/ldap/tests/ldap_err2str_error-detect.phpt",
    "ext/ldap/tests/ldap_get_attributes_error-detect.phpt",
    "ext/ldap/tests/ldap_get_option_error-detect.phpt",
    "ext/ldap/tests/ldap_rename_basic-detect.phpt",
    "ext/ldap/tests/ldap_add_error-detect.phpt",
    "ext/ldap/tests/ldap_set_option_basic-detect.phpt",
    "ext/ldap/tests/ldap_unbind_variation-detect.phpt",
    "ext/ldap/tests/bug48441-detect.phpt",
    "ext/ldap/tests/ldap_next_entry_error-detect.phpt",
    "ext/ldap/tests/ldap_count_entries_error-detect.phpt",
    "ext/ldap/tests/ldap_control_paged_results_variation1-detect.phpt",
    "ext/ldap/tests/ldap_compare_error-detect.phpt",
    "ext/ldap/tests/ldap_connect_variation-detect.phpt",
    "ext/ldap/tests/ldap_read_basic-detect.phpt",
    "ext/ldap/tests/ldap_search_variation2-detect.phpt",
    "ext/ldap/tests/ldap_mod_del_error-detect.phpt",
    "ext/ldap/tests/ldap_next_attribute_error-detect.phpt",
    "ext/ldap/tests/ldap_sasl_bind_error-detect.phpt",
    "ext/ldap/tests/bug48696-detect.phpt",
    "ext/ldap/tests/ldap_get_dn_basic-detect.phpt",
    "ext/ldap/tests/ldap_set_option_error-detect.phpt",
    "ext/ldap/tests/ldap_search_variation4-detect.phpt",
    "ext/ldap/tests/ldap_get_option_variation-detect.phpt",
    "ext/ldap/tests/ldap_sort_error-detect.phpt",
    "ext/ldap/tests/ldap_sasl_bind_basic-detect.phpt",
    "ext/ldap/tests/ldap_start_tls_error-detect.phpt",
    "ext/ldap/tests/ldap_set_option_variation-detect.phpt",
    "ext/ldap/tests/ldap_err2str_basic-detect.phpt",
    "ext/ldap/tests/ldap_mod_add_basic-detect.phpt",
    "ext/ldap/tests/ldap_control_paged_results_variation2-detect.phpt",
    "ext/ldap/tests/ldap_get_attributes_basic-detect.phpt",
    "ext/ldap/tests/ldap_bind_variation-detect.phpt",
    "ext/ldap/tests/ldap_get_values_len_basic-detect.phpt",
    "ext/ldap/tests/ldap_bind_basic-detect.phpt",
    "ext/ldap/tests/ldap_mod_del_basic-detect.phpt",
    "ext/ldap/tests/ldap_next_attribute_basic-detect.phpt",
    "ext/ldap/tests/ldap_first_entry_error-detect.phpt",
    "ext/ldap/tests/ldap_first_reference_error-detect.phpt",
    "ext/ldap/tests/ldap_list_error-detect.phpt",
    "ext/ldap/tests/ldap_sort_variation-detect.phpt",
    "ext/ldap/tests/ldap_parse_reference_basic-detect.phpt",
    "ext/ldap/tests/ldap_search_variation5-detect.phpt",
    "ext/ldap/tests/ldap_sort_basic-detect.phpt",
    "ext/ldap/tests/ldap_parse_reference_error-detect.phpt",
    "ext/ldap/tests/ldap_first_reference_basic-detect.phpt",
    "ext/ldap/tests/ldap_next_reference_error-detect.phpt",
    "ext/ldap/tests/ldap_next_entry_basic-detect.phpt",
    "ext/ldap/tests/ldap_start_tls_basic-detect.phpt",
    "ext/ldap/tests/ldap_list_basic-detect.phpt",
    "ext/ldap/tests/ldap_delete_basic-detect.phpt",
    "ext/ldap/tests/ldap_errno_basic-detect.phpt",
    "ext/imap/tests/imap_fetch_overview_variation3-detect.phpt",
    "ext/imap/tests/imap_lsub_basic-detect.phpt",
    "ext/imap/tests/imap_fetch_overview_variation5-detect.phpt",
    "ext/imap/tests/imap_clearflag_full_basic-detect.phpt",
    "ext/imap/tests/imap_close_error-detect.phpt",
    "ext/imap/tests/imap_fetchbody_variation4-detect.phpt",
    "ext/imap/tests/imap_fetchbody_basic-detect.phpt",
    "ext/imap/tests/imap_num_msg_error-detect.phpt",
    "ext/imap/tests/imap_timeout_basic-detect.phpt",
    "ext/imap/tests/imap_mail_copy_basic-detect.phpt",
    "ext/imap/tests/imap_fetchbody_variation1-detect.phpt",
    "ext/imap/tests/imap_fetch_overview_variation4-detect.phpt",
    "ext/imap/tests/bug31142_1-detect.phpt",
    "ext/imap/tests/imap_createmailbox_basic-detect.phpt",
    "ext/imap/tests/bug44098-detect.phpt",
    "ext/imap/tests/imap_fetchheader_variation2-detect.phpt",
    "ext/imap/tests/imap_fetchbody_variation5-detect.phpt",
    "ext/imap/tests/imap_headerinfo_error-detect.phpt",
    "ext/imap/tests/bug46918-detect.phpt",
    "ext/imap/tests/bug45705_1-detect.phpt",
    "ext/imap/tests/imap_append_basic-detect.phpt",
    "ext/imap/tests/imap_rfc822_parse_headers_basic-detect.phpt",
    "ext/imap/tests/imap_errors_basic-detect.phpt",
    "ext/imap/tests/imap_fetchbody_variation2-detect.phpt",
    "ext/imap/tests/imap_close_variation2-detect.phpt",
    "ext/imap/tests/imap_body-detect.phpt",
    "ext/imap/tests/bug45705_2-detect.phpt",
    "ext/imap/tests/imap_fetchheader_variation3-detect.phpt",
    "ext/imap/tests/imap_close_variation3-detect.phpt",
    "ext/imap/tests/imap_close_basic-detect.phpt",
    "ext/imap/tests/imap_alerts_error-detect.phpt",
    "ext/imap/tests/imap_utf8-detect.phpt",
    "ext/imap/tests/imap_mutf7_to_utf8-detect.phpt",
    "ext/imap/tests/imap_undelete_error-detect.phpt",
    "ext/imap/tests/bug53377-detect.phpt",
    "ext/imap/tests/imap_expunge_error-detect.phpt",
    "ext/imap/tests/imap_gc_error-detect.phpt",
    "ext/imap/tests/imap_fetchbody_error-detect.phpt",
    "ext/imap/tests/imap_mail_move-detect.phpt",
    "ext/imap/tests/imap_base64_basic-detect.phpt",
    "ext/imap/tests/imap_close_variation1-detect.phpt",
    "ext/imap/tests/imap_body_basic-detect.phpt",
    "ext/imap/tests/imap_undelete_basic-detect.phpt",
    "ext/imap/tests/imap_bodystruct_basic-detect.phpt",
    "ext/imap/tests/imap_savebody_basic-detect.phpt",
    "ext/imap/tests/imap_fetch_overview_error-detect.phpt",
    "ext/imap/tests/imap_fetch_overview_variation2-detect.phpt",
    "ext/imap/tests/imap_close_variation4-detect.phpt",
    "ext/imap/tests/bug35669-detect.phpt",
    "ext/imap/tests/imap_binary_basic-detect.phpt",
    "ext/imap/tests/imap_headers-detect.phpt",
    "ext/imap/tests/imap_ping_error-detect.phpt",
    "ext/imap/tests/imap_fetchbody_variation3-detect.phpt",
    "ext/imap/tests/imap_mail_copy-detect.phpt",
    "ext/imap/tests/imap_fetch_overview_variation6-detect.phpt",
    "ext/imap/tests/imap_fetchheader_variation4-detect.phpt",
    "ext/imap/tests/imap_mail_move_basic-detect.phpt",
    "ext/imap/tests/imap_8bit_basic-detect.phpt",
    "ext/imap/tests/imap_open_error-detect.phpt",
    "ext/imap/tests/imap_headerinfo_basic-detect.phpt",
    "ext/imap/tests/imap_fetch_overview_basic-detect.phpt",
    "ext/imap/tests/imap_list_basic-detect.phpt",
    "ext/imap/tests/imap_fetchstructure_basic-detect.phpt",
    "ext/imap/tests/imap_fetchheader_variation1-detect.phpt",
    "ext/imap/tests/imap_fetchbody_variation6-detect.phpt",
    "ext/imap/tests/imap_fetchheader_error-detect.phpt",
    "ext/imap/tests/imap_fetchheader_basic-detect.phpt",
    "ext/imap/tests/bug32589-detect.phpt",
    "ext/imap/tests/imap_num_recent_error-detect.phpt",
    "ext/imap/tests/imap_fetchheader_variation5-detect.phpt",
    "ext/imap/tests/imap_utf8_to_mutf7_basic-detect.phpt",
    "ext/imap/tests/imap_fetch_overview_variation1-detect.phpt",
    "ext/imap/tests/bug31142_2-detect.phpt",
    "ext/imap/tests/imap_renamemailbox_basic-detect.phpt",
    "ext/imap/tests/bug40854-detect.phpt",
    "ext/imap/tests/imap_getsubscribed_basic-detect.phpt",
    "ext/pdo_odbc/tests/common-detect.phpt",
    "ext/pdo_odbc/tests/long_columns-detect.phpt",
    "ext/zlib/tests/gzfile_variation12-detect.phpt",
    "ext/zlib/tests/zlib_scheme_unlink_basic-detect.phpt",
    "ext/zlib/tests/readgzfile_variation14-detect.phpt",
    "ext/zlib/tests/readgzfile_variation7-detect.phpt",
    "ext/zlib/tests/gzfile_variation1-detect.phpt",
    "ext/zlib/tests/gzencode_variation2-detect.phpt",
    "ext/zlib/tests/gzfile_basic-detect.phpt",
    "ext/zlib/tests/005-detect.phpt",
    "ext/zlib/tests/gzuncompress_error1-detect.phpt",
    "ext/zlib/tests/readgzfile_basic2-detect.phpt",
    "ext/zlib/tests/gzinflate_length-detect.phpt",
    "ext/zlib/tests/gzinflate-bug42663-detect.phpt",
    "ext/zlib/tests/gztell_error-detect.phpt",
    "ext/zlib/tests/gzfile_variation2-detect.phpt",
    "ext/zlib/tests/zlib_scheme_file_basic-detect.phpt",
    "ext/zlib/tests/gzread_error2-detect.phpt",
    "ext/zlib/tests/readgzfile_variation8-detect.phpt",
    "ext/zlib/tests/zlib_scheme_fopen_variation1-detect.phpt",
    "ext/zlib/tests/gzopen_basic-detect.phpt",
    "ext/zlib/tests/gzfile_error-detect.phpt",
    "ext/zlib/tests/gzfile_variation8-detect.phpt",
    "ext/zlib/tests/zlib_scheme_stat_basic-detect.phpt",
    "ext/zlib/tests/gzfile_variation4-detect.phpt",
    "ext/zlib/tests/zlib_filter_inflate2-detect.phpt",
    "ext/zlib/tests/bug51269-detect.phpt",
    "ext/zlib/tests/bug_40189-detect.phpt",
    "ext/zlib/tests/bug_52944-detect.phpt",
    "ext/zlib/tests/zlib_scheme_file_read_file_basic-detect.phpt",
    "ext/zlib/tests/gzseek_basic-detect.phpt",
    "ext/zlib/tests/readgzfile_variation2-detect.phpt",
    "ext/zlib/tests/zlib_wrapper_ftruncate_basic-detect.phpt",
    "ext/zlib/tests/gzeof_basic-detect.phpt",
    "ext/zlib/tests/gzcompress_variation1-detect.phpt",
    "ext/zlib/tests/bug61443-detect.phpt",
    "ext/zlib/tests/gzgetc_error-detect.phpt",
    "ext/zlib/tests/001-detect.phpt",
    "ext/zlib/tests/readgzfile_variation13-detect.phpt",
    "ext/zlib/tests/bug55544-detect.phpt",
    "ext/zlib/tests/gzopen_variation9-detect.phpt",
    "ext/zlib/tests/004-detect.phpt",
    "ext/zlib/tests/gzseek_variation5-detect.phpt",
    "ext/zlib/tests/readgzfile_basic-detect.phpt",
    "ext/zlib/tests/readgzfile_variation12-detect.phpt",
    "ext/zlib/tests/bug60761-detect.phpt",
    "ext/zlib/tests/gztell_basic2-detect.phpt",
    "ext/zlib/tests/readgzfile_error-detect.phpt",
    "ext/zlib/tests/gzopen_variation8-detect.phpt",
    "ext/zlib/tests/gzopen_variation6-detect.phpt",
    "ext/zlib/tests/gzclose_error-detect.phpt",
    "ext/zlib/tests/readgzfile_variation1-detect.phpt",
    "ext/zlib/tests/gzfile_variation13-detect.phpt",
    "ext/zlib/tests/gzfile_variation11-detect.phpt",
    "ext/zlib/tests/gzopen_error-detect.phpt",
    "ext/zlib/tests/gzclose_basic-detect.phpt",
    "ext/zlib/tests/gzrewind_error-detect.phpt",
    "ext/zlib/tests/gzseek_error-detect.phpt",
    "ext/zlib/tests/bug61287-detect.phpt",
    "ext/zlib/tests/gzputs_basic-detect.phpt",
    "ext/zlib/tests/003-detect.phpt",
    "ext/zlib/tests/gzopen_basic2-detect.phpt",
    "ext/zlib/tests/gzgets_basic-detect.phpt",
    "ext/zlib/tests/gzdeflate_variation1-detect.phpt",
    "ext/zlib/tests/gzcompress_error1-detect.phpt",
    "ext/zlib/tests/gzopen_variation1-detect.phpt",
    "ext/zlib/tests/gzfile_basic2-detect.phpt",
    "ext/zlib/tests/gzrewind_basic-detect.phpt",
    "ext/zlib/tests/gzreadgzwriteplain-detect.phpt",
    "ext/zlib/tests/gzcompress_basic1-detect.phpt",
    "ext/zlib/tests/006-detect.phpt",
    "ext/zlib/tests/gzeof_variation1-detect.phpt",
    "ext/zlib/tests/gzseek_variation7-detect.phpt",
    "ext/zlib/tests/gzfile_variation7-detect.phpt",
    "ext/zlib/tests/gzgets_error-detect.phpt",
    "ext/zlib/tests/ob_002-detect.phpt",
    "ext/zlib/tests/compress_zlib_wrapper-detect.phpt",
    "ext/zlib/tests/gzopen_variation4-detect.phpt",
    "ext/zlib/tests/gzseek_variation1-detect.phpt",
    "ext/zlib/tests/readgzfile_variation9-detect.phpt",
    "ext/zlib/tests/zlib_scheme_copy_basic-detect.phpt",
    "ext/zlib/tests/gzpassthru_basic-detect.phpt",
    "ext/zlib/tests/007-detect.phpt",
    "ext/zlib/tests/gzfile_variation3-detect.phpt",
    "ext/zlib/tests/readgzfile_variation3-detect.phpt",
    "ext/zlib/tests/gzrewind_basic2-detect.phpt",
    "ext/zlib/tests/gzread_error-detect.phpt",
    "ext/zlib/tests/gzfile_variation10-detect.phpt",
    "ext/zlib/tests/gzopen_variation2-detect.phpt",
    "ext/zlib/tests/gzgetc_basic-detect.phpt",
    "ext/zlib/tests/gzencode_variation1-detect.phpt",
    "ext/zlib/tests/ob_gzhandler_legacy_002-detect.phpt",
    "ext/zlib/tests/gzseek_basic2-detect.phpt",
    "ext/zlib/tests/gztell_basic-detect.phpt",
    "ext/zlib/tests/zlib_filter_deflate-detect.phpt",
    "ext/zlib/tests/ob_005-detect.phpt",
    "ext/zlib/tests/002-detect.phpt",
    "ext/zlib/tests/readgzfile_variation10-detect.phpt",
    "ext/zlib/tests/008-detect.phpt",
    "ext/zlib/tests/readgzfile_variation5-detect.phpt",
    "ext/zlib/tests/gzinflate_error1-detect.phpt",
    "ext/zlib/tests/gzread_basic-detect.phpt",
    "ext/zlib/tests/gzseek_variation2-detect.phpt",
    "ext/zlib/tests/readgzfile_variation11-detect.phpt",
    "ext/zlib/tests/zlib_wrapper_meta_data_basic-detect.phpt",
    "ext/zlib/tests/gzfilegzreadfile-detect.phpt",
    "ext/zlib/tests/gzpassthru_error-detect.phpt",
    "ext/zlib/tests/zlib_wrapper_fstat_basic-detect.phpt",
    "ext/zlib/tests/zlib_scheme_copy_variation2-detect.phpt",
    "ext/zlib/tests/gzdeflate_error1-detect.phpt",
    "ext/zlib/tests/zlib_scheme_file_get_contents_basic-detect.phpt",
    "ext/zlib/tests/zlib_wrapper_fflush_basic-detect.phpt",
    "ext/zlib/tests/gzeof_error-detect.phpt",
    "ext/zlib/tests/bug61139-detect.phpt",
    "ext/zlib/tests/zlib_filter_inflate-detect.phpt",
    "ext/zlib/tests/gzencode_variation2-win32-detect.phpt",
    "ext/zlib/tests/gzencode_error1-detect.phpt",
    "ext/zlib/tests/gzseek_variation6-detect.phpt",
    "ext/zlib/tests/gzfile_variation6-detect.phpt",
    "ext/zlib/tests/gzreadgzwrite-detect.phpt",
    "ext/zlib/tests/gzuncompress_basic1-detect.phpt",
    "ext/zlib/tests/zlib_scheme_dir_basic-detect.phpt",
    "ext/zlib/tests/zlib_scheme_file_put_contents_basic-detect.phpt",
    "ext/zlib/tests/gzseek_variation4-detect.phpt",
    "ext/zlib/tests/readgzfile_variation4-detect.phpt",
    "ext/zlib/tests/gzseek_variation3-detect.phpt",
    "ext/zlib/tests/gzdeflate_basic1-detect.phpt",
    "ext/zlib/tests/gzwrite_basic-detect.phpt",
    "ext/zlib/tests/ob_001-detect.phpt",
    "ext/zlib/tests/zlib_scheme_rename_basic-detect.phpt",
    "ext/zlib/tests/gzgetc_basic_1-detect.phpt",
    "ext/zlib/tests/zlib_wrapper_flock_basic-detect.phpt",
    "ext/zlib/tests/zlib_filter_deflate2-detect.phpt",
    "ext/zlib/tests/bug61820-detect.phpt",
    "ext/zlib/tests/gzfile_variation15-detect.phpt",
    "ext/zlib/tests/gzopen_variation7-detect.phpt",
    "ext/zlib/tests/ob_003-detect.phpt",
    "ext/zlib/tests/readgzfile_variation6-detect.phpt",
    "ext/zlib/tests/gzwrite_error-detect.phpt",
    "ext/zlib/tests/gzopen_variation5-detect.phpt",
    "ext/zlib/tests/gzwrite_error2-detect.phpt",
    "ext/zlib/tests/gzwrite_variation1-detect.phpt",
    "ext/zlib/tests/gzfile_variation9-detect.phpt",
    "ext/zlib/tests/gzread_variation1-detect.phpt",
    "ext/zlib/tests/zlib_scheme_fopen_basic-detect.phpt",
    "ext/zlib/tests/gzopen_variation3-detect.phpt",
    "ext/zlib/tests/gzencode_basic1-detect.phpt",
    "ext/zlib/tests/gzfile_variation14-detect.phpt",
    "ext/zlib/tests/readgzfile_variation15-detect.phpt",
    "ext/zlib/tests/ob_004-detect.phpt",
    "ext/zlib/tests/bug_34821-detect.phpt",
    "ext/zlib/tests/gzrewind_variation1-detect.phpt",
    "ext/zlib/tests/zlib_scheme_copy_variation1-detect.phpt",
    "ext/zlib/tests/zlib_scheme_stat_basic2-detect.phpt",
    "ext/zlib/tests/gzencode_variation1-win32-detect.phpt",
    "ext/zlib/tests/gzfile_variation5-detect.phpt",
    "ext/pdo/tests/pdo_025-detect.phpt",
    "ext/pdo/tests/pdo_033-detect.phpt",
    "ext/pdo/tests/pdo_017-detect.phpt",
    "ext/pdo/tests/pdo_028-detect.phpt",
    "ext/pdo/tests/bug_40285-detect.phpt",
    "ext/pdo/tests/pdo_027-detect.phpt",
    "ext/pdo/tests/bug_50458-detect.phpt",
    "ext/pdo/tests/bug_44173-detect.phpt",
    "ext/pdo/tests/bug_34630-detect.phpt",
    "ext/pdo/tests/pdo_003-detect.phpt",
    "ext/pdo/tests/bug_39656-detect.phpt",
    "ext/pdo/tests/pdo_037-detect.phpt",
    "ext/pdo/tests/pdo_026-detect.phpt",
    "ext/pdo/tests/pdo_009-detect.phpt",
    "ext/pdo/tests/pdo_021-detect.phpt",
    "ext/pdo/tests/pdo_035-detect.phpt",
    "ext/pdo/tests/pdo_031-detect.phpt",
    "ext/pdo/tests/bug_36428-detect.phpt",
    "ext/pdo/tests/pdo_011-detect.phpt",
    "ext/pdo/tests/pdo_007-detect.phpt",
    "ext/pdo/tests/pdo_019-detect.phpt",
    "ext/pdo/tests/bug_44861-detect.phpt",
    "ext/pdo/tests/pdo_023-detect.phpt",
    "ext/pdo/tests/bug_42917-detect.phpt",
    "ext/pdo/tests/pdo_029-detect.phpt",
    "ext/pdo/tests/pdo_013-detect.phpt",
    "ext/pdo/tests/pecl_bug_5217-detect.phpt",
    "ext/pdo/tests/pdo_022-detect.phpt",
    "ext/pdo/tests/pdo_002-detect.phpt",
    "ext/pdo/tests/pdo_034-detect.phpt",
    "ext/pdo/tests/pdo_030-detect.phpt",
    "ext/pdo/tests/pecl_bug_5772-detect.phpt",
    "ext/pdo/tests/bug61292-detect.phpt",
    "ext/pdo/tests/bug_38253-detect.phpt",
    "ext/pdo/tests/pdo_001-detect.phpt",
    "ext/pdo/tests/bug47769-detect.phpt",
    "ext/pdo/tests/bug_34687-detect.phpt",
    "ext/pdo/tests/pdo_032-detect.phpt",
    "ext/pdo/tests/bug_43130-detect.phpt",
    "ext/pdo/tests/bug_44159-detect.phpt",
    "ext/pdo/tests/pdo_016a-detect.phpt",
    "ext/pdo/tests/pdo_018-detect.phpt",
    "ext/pdo/tests/bug_35671-detect.phpt",
    "ext/pdo/tests/bug_38394-detect.phpt",
    "ext/pdo/tests/pdo_014-detect.phpt",
    "ext/pdo/tests/pdo_010-detect.phpt",
    "ext/pdo/tests/pdo_004-detect.phpt",
    "ext/pdo/tests/bug_36798-detect.phpt",
    "ext/pdo/tests/bug_43663-detect.phpt",
    "ext/pdo/tests/pecl_bug_5809-detect.phpt",
    "ext/pdo/tests/pdo_012-detect.phpt",
    "ext/pdo/tests/bug_44409-detect.phpt",
    "ext/pdo/tests/pdo_006-detect.phpt",
    "ext/pdo/tests/pdo_016-detect.phpt",
    "ext/pdo/tests/pdo_020-detect.phpt",
    "ext/pdo/tests/bug_39398-detect.phpt",
    "ext/pdo/tests/pdo_024-detect.phpt",
    "ext/pdo/tests/pdorow-detect.phpt",
    "ext/pdo/tests/pdo_015-detect.phpt",
    "ext/pdo/tests/pdo_008-detect.phpt",
    "ext/pdo/tests/pdo_005-detect.phpt",
    "ext/pdo/tests/bug_43139-detect.phpt",
    "ext/pdo/tests/pdo_036-detect.phpt",
    "ext/sysvmsg/tests/005-detect.phpt",
    "ext/sysvmsg/tests/001-detect.phpt",
    "ext/sysvmsg/tests/004-detect.phpt",
    "ext/sysvmsg/tests/003-detect.phpt",
    "ext/sysvmsg/tests/006-detect.phpt",
    "ext/sysvmsg/tests/002-detect.phpt",
    "ext/mysqli/tests/050-detect.phpt",
    "ext/mysqli/tests/027-detect.phpt",
    "ext/mysqli/tests/mysqli_fork-detect.phpt",
    "ext/mysqli/tests/mysqli_get_server_version-detect.phpt",
    "ext/mysqli/tests/mysqli_send_query-detect.phpt",
    "ext/mysqli/tests/bug31141-detect.phpt",
    "ext/mysqli/tests/mysqli_debug_ini-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_close-detect.phpt",
    "ext/mysqli/tests/mysqli_pconn_disabled-detect.phpt",
    "ext/mysqli/tests/033-detect.phpt",
    "ext/mysqli/tests/bug46109-detect.phpt",
    "ext/mysqli/tests/mysqli_get_connection_stats-detect.phpt",
    "ext/mysqli/tests/mysqli_get_connection_stats_off-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_short_len-detect.phpt",
    "ext/mysqli/tests/mysqli_real_connect_pconn-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_bind_param_many_columns-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_get_result_bit-detect.phpt",
    "ext/mysqli/tests/010-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_get_result2-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_bind_param_references-detect.phpt",
    "ext/mysqli/tests/066-detect.phpt",
    "ext/mysqli/tests/mysqli_real_escape_string_euckr-detect.phpt",
    "ext/mysqli/tests/mysqli_real_escape_string_gbk-detect.phpt",
    "ext/mysqli/tests/045-detect.phpt",
    "ext/mysqli/tests/mysqli_connect_oo_warnings-detect.phpt",
    "ext/mysqli/tests/bug36745-detect.phpt",
    "ext/mysqli/tests/005-detect.phpt",
    "ext/mysqli/tests/bug35759-detect.phpt",
    "ext/mysqli/tests/mysqli_ps_select_union-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_negative_len-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_replace_buffer-detect.phpt",
    "ext/mysqli/tests/mysqli_errno_oo-detect.phpt",
    "ext/mysqli/tests/mysqli_use_result-detect.phpt",
    "ext/mysqli/tests/mysqli_num_fields-detect.phpt",
    "ext/mysqli/tests/054-detect.phpt",
    "ext/mysqli/tests/mysqli_connect-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_result_metadata-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_closures-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_bind_param_call_user_func-detect.phpt",
    "ext/mysqli/tests/mysqli_get_client_stats_skipped-detect.phpt",
    "ext/mysqli/tests/bug54221-detect.phpt",
    "ext/mysqli/tests/014-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_bind_param-detect.phpt",
    "ext/mysqli/tests/044-detect.phpt",
    "ext/mysqli/tests/029-detect.phpt",
    "ext/mysqli/tests/mysqli_warning_unclonable-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_get_result_types-detect.phpt",
    "ext/mysqli/tests/mysqli_warning_count-detect.phpt",
    "ext/mysqli/tests/bug62885-detect.phpt",
    "ext/mysqli/tests/bug51647-detect.phpt",
    "ext/mysqli/tests/073-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_object_oo-detect.phpt",
    "ext/mysqli/tests/mysqli_get_client_stats-detect.phpt",
    "ext/mysqli/tests/072-detect.phpt",
    "ext/mysqli/tests/mysqli_options-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_assoc_zerofill-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_error-detect.phpt",
    "ext/mysqli/tests/mysqli_insert_packet_overflow-detect.phpt",
    "ext/mysqli/tests/mysqli_sqlstate-detect.phpt",
    "ext/mysqli/tests/011-detect.phpt",
    "ext/mysqli/tests/mysqli_get_server_info-detect.phpt",
    "ext/mysqli/tests/mysqli_get_proto_info-detect.phpt",
    "ext/mysqli/tests/bug50772-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_assoc_oo-detect.phpt",
    "ext/mysqli/tests/mysqli_get_host_info-detect.phpt",
    "ext/mysqli/tests/mysqli_num_rows-detect.phpt",
    "ext/mysqli/tests/062-detect.phpt",
    "ext/mysqli/tests/mysqli_driver-detect.phpt",
    "ext/mysqli/tests/mysqli_poll_reference-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_bind_result_bit-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_fields-detect.phpt",
    "ext/mysqli/tests/017-detect.phpt",
    "ext/mysqli/tests/mysqli_class_mysqli_warning_reflection-detect.phpt",
    "ext/mysqli/tests/mysqli_error_unicode-detect.phpt",
    "ext/mysqli/tests/mysqli_kill-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_get_result_geom-detect.phpt",
    "ext/mysqli/tests/mysqli_result_references-detect.phpt",
    "ext/mysqli/tests/mysqli_class_mysqli_properties_no_conn-detect.phpt",
    "ext/mysqli/tests/036-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_attr_get_prefetch-detect.phpt",
    "ext/mysqli/tests/bug36420-detect.phpt",
    "ext/mysqli/tests/mysqli_class_mysqli_interface-detect.phpt",
    "ext/mysqli/tests/mysqli_result_references_mysqlnd-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_execute_stored_proc-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_bind_limits-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_send_long_data_packet_size_libmysql-detect.phpt",
    "ext/mysqli/tests/mysqli_connect_error-detect.phpt",
    "ext/mysqli/tests/mysqli_errno-detect.phpt",
    "ext/mysqli/tests/mysqli_dump_debug_info-detect.phpt",
    "ext/mysqli/tests/bug54674-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_buffer_overflow-detect.phpt",
    "ext/mysqli/tests/mysqli_connect_twice-detect.phpt",
    "ext/mysqli/tests/mysqli_stat-detect.phpt",
    "ext/mysqli/tests/mysqli_debug_mysqlnd_only-detect.phpt",
    "ext/mysqli/tests/bug38710-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_nofileop-detect.phpt",
    "ext/mysqli/tests/mysqli_change_user_set_names-detect.phpt",
    "ext/mysqli/tests/mysqli_get_warnings-detect.phpt",
    "ext/mysqli/tests/mysqli_debug_append-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_assoc_no_alias_utf8-detect.phpt",
    "ext/mysqli/tests/mysqli_constants_categories-detect.phpt",
    "ext/mysqli/tests/mysqli_field_tell-detect.phpt",
    "ext/mysqli/tests/043-detect.phpt",
    "ext/mysqli/tests/028-detect.phpt",
    "ext/mysqli/tests/060-detect.phpt",
    "ext/mysqli/tests/068-detect.phpt",
    "ext/mysqli/tests/001-detect.phpt",
    "ext/mysqli/tests/041-detect.phpt",
    "ext/mysqli/tests/mysqli_pconnect-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_array_oo-detect.phpt",
    "ext/mysqli/tests/bug33491-detect.phpt",
    "ext/mysqli/tests/mysqli_class_mysqli_result_reflection-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_attr_get-detect.phpt",
    "ext/mysqli/tests/mysqli_affected_rows_oo-detect.phpt",
    "ext/mysqli/tests/mysqli_change_user_insert_id-detect.phpt",
    "ext/mysqli/tests/bug_mysql_49406-detect.phpt",
    "ext/mysqli/tests/015-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_field_oo-detect.phpt",
    "ext/mysqli/tests/026-detect.phpt",
    "ext/mysqli/tests/bug35103-detect.phpt",
    "ext/mysqli/tests/004-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_field_types-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_insert_id-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_free_result-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_store_result-detect.phpt",
    "ext/mysqli/tests/mysqli_pconn_limits-detect.phpt",
    "ext/mysqli/tests/bug55859-detect.phpt",
    "ext/mysqli/tests/mysqli_info-detect.phpt",
    "ext/mysqli/tests/bug33090-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_fetch_fields_win32_unicode-detect.phpt",
    "ext/mysqli/tests/mysqli_real_escape_string_unicode-detect.phpt",
    "ext/mysqli/tests/mysqli_multi_query-detect.phpt",
    "ext/mysqli/tests/bug47050-detect.phpt",
    "ext/mysqli/tests/mysqli_get_client_version-detect.phpt",
    "ext/mysqli/tests/bug55582-detect.phpt",
    "ext/mysqli/tests/mysqli_free_result-detect.phpt",
    "ext/mysqli/tests/mysqli_get_client_stats_implicit_free-detect.phpt",
    "ext/mysqli/tests/mysqli_debug_control_string-detect.phpt",
    "ext/mysqli/tests/mysqli_connect_oo_defaults-detect.phpt",
    "ext/mysqli/tests/mysqli_prepare-detect.phpt",
    "ext/mysqli/tests/mysqli_mysqli_result_invalid_mode-detect.phpt",
    "ext/mysqli/tests/mysqli_options_openbasedir-detect.phpt",
    "ext/mysqli/tests/061-detect.phpt",
    "ext/mysqli/tests/048-detect.phpt",
    "ext/mysqli/tests/mysqli_enable_reads_from_master-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_unregister-detect.phpt",
    "ext/mysqli/tests/mysqli_last_insert_id-detect.phpt",
    "ext/mysqli/tests/mysqli_field_count-detect.phpt",
    "ext/mysqli/tests/mysqli_character_set_name-detect.phpt",
    "ext/mysqli/tests/bug31668-detect.phpt",
    "ext/mysqli/tests/mysqli_driver_unclonable-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_datatype_change-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_fetch_geom-detect.phpt",
    "ext/mysqli/tests/bug49027-detect.phpt",
    "ext/mysqli/tests/003-detect.phpt",
    "ext/mysqli/tests/bug45019-detect.phpt",
    "ext/mysqli/tests/mysqli_connect_errno-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_row-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_get_warnings-detect.phpt",
    "ext/mysqli/tests/mysqli_pconn_max_links-detect.phpt",
    "ext/mysqli/tests/mysqli_poll_kill-detect.phpt",
    "ext/mysqli/tests/bug52891-detect.phpt",
    "ext/mysqli/tests/049-detect.phpt",
    "ext/mysqli/tests/mysqli_init-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_send_long_data-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_prepare-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_new_query-detect.phpt",
    "ext/mysqli/tests/bug55283-detect.phpt",
    "ext/mysqli/tests/064-detect.phpt",
    "ext/mysqli/tests/mysqli_embedded_connect-detect.phpt",
    "ext/mysqli/tests/020-detect.phpt",
    "ext/mysqli/tests/mysqli_no_reconnect-detect.phpt",
    "ext/mysqli/tests/022-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_bind_result_format-detect.phpt",
    "ext/mysqli/tests/mysqli_error-detect.phpt",
    "ext/mysqli/tests/006-detect.phpt",
    "ext/mysqli/tests/031-detect.phpt",
    "ext/mysqli/tests/mysqli_insert_id-detect.phpt",
    "ext/mysqli/tests/071-detect.phpt",
    "ext/mysqli/tests/mysqli_result_unclonable-detect.phpt",
    "ext/mysqli/tests/023-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_errno-detect.phpt",
    "ext/mysqli/tests/058-detect.phpt",
    "ext/mysqli/tests/046-detect.phpt",
    "ext/mysqli/tests/mysqli_real_escape_string-detect.phpt",
    "ext/mysqli/tests/mysqli_max_links-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_field_direct-detect.phpt",
    "ext/mysqli/tests/bug45289-detect.phpt",
    "ext/mysqli/tests/mysqli_disable_reads_from_master-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_assoc_no_alias-detect.phpt",
    "ext/mysqli/tests/mysqli_change_user_rollback-detect.phpt",
    "ext/mysqli/tests/mysqli_autocommit_oo-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_field_count-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_default-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_object_no_constructor-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_get_result_seek-detect.phpt",
    "ext/mysqli/tests/mysqli_options_int_and_float_native-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_openbasedir-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_num_rows-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_get_result_metadata_fetch_field-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_array_large-detect.phpt",
    "ext/mysqli/tests/mysqli_query_unicode-detect.phpt",
    "ext/mysqli/tests/bug34810-detect.phpt",
    "ext/mysqli/tests/bug38003-detect.phpt",
    "ext/mysqli/tests/mysqli_query_local_infile_large-detect.phpt",
    "ext/mysqli/tests/069-detect.phpt",
    "ext/mysqli/tests/007-detect.phpt",
    "ext/mysqli/tests/mysqli_select_db-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_bind_param_type_juggling-detect.phpt",
    "ext/mysqli/tests/mysqli_report_wo_ps-detect.phpt",
    "ext/mysqli/tests/mysqli_change_user_oo-detect.phpt",
    "ext/mysqli/tests/bug37090-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_reset-detect.phpt",
    "ext/mysqli/tests/074-detect.phpt",
    "ext/mysqli/tests/mysqli_pconn_kill-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_sqlstate-detect.phpt",
    "ext/mysqli/tests/mysqli_real_escape_string_nobackslash-detect.phpt",
    "ext/mysqli/tests/mysqli_change_user-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_array_assoc-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_all_oo-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_array-detect.phpt",
    "ext/mysqli/tests/mysqli_change_user_prepared_statements-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_nested_call-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_execute_stored_proc_next_result-detect.phpt",
    "ext/mysqli/tests/mysqli_mysqlnd_read_timeout_zero-detect.phpt",
    "ext/mysqli/tests/mysqli_thread_id-detect.phpt",
    "ext/mysqli/tests/mysqli_poll-detect.phpt",
    "ext/mysqli/tests/bug32405-detect.phpt",
    "ext/mysqli/tests/002-detect.phpt",
    "ext/mysqli/tests/mysqli_poll_mixing_insert_select-detect.phpt",
    "ext/mysqli/tests/bug44897-detect.phpt",
    "ext/mysqli/tests/008-detect.phpt",
    "ext/mysqli/tests/018-detect.phpt",
    "ext/mysqli/tests/mysqli_query_stored_proc-detect.phpt",
    "ext/mysqli/tests/mysqli_character_set_name_oo-detect.phpt",
    "ext/mysqli/tests/mysqli_affected_rows-detect.phpt",
    "ext/mysqli/tests/040-detect.phpt",
    "ext/mysqli/tests/mysqli_real_escape_string_gb2312-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_get_result-detect.phpt",
    "ext/mysqli/tests/056-detect.phpt",
    "ext/mysqli/tests/mysqli_rollback-detect.phpt",
    "ext/mysqli/tests/mysqli_insert_id_variation-detect.phpt",
    "ext/mysqli/tests/034-detect.phpt",
    "ext/mysqli/tests/bug33263-detect.phpt",
    "ext/mysqli/tests/039-detect.phpt",
    "ext/mysqli/tests/035-detect.phpt",
    "ext/mysqli/tests/070-detect.phpt",
    "ext/mysqli/tests/mysqli_dump_debug_info_oo-detect.phpt",
    "ext/mysqli/tests/mysqli_phpinfo-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_array_many_rows-detect.phpt",
    "ext/mysqli/tests/mysqli_class_mysqli_warning-detect.phpt",
    "ext/mysqli/tests/mysqli_commit_oo-detect.phpt",
    "ext/mysqli/tests/013-detect.phpt",
    "ext/mysqli/tests/mysqli_real_query-detect.phpt",
    "ext/mysqli/tests/037-detect.phpt",
    "ext/mysqli/tests/016-detect.phpt",
    "ext/mysqli/tests/mysqli_commit-detect.phpt",
    "ext/mysqli/tests/mysqli_store_result-detect.phpt",
    "ext/mysqli/tests/mysqli_change_user_get_lock-detect.phpt",
    "ext/mysqli/tests/bug29311-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_unclonable-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_bind_result-detect.phpt",
    "ext/mysqli/tests/mysqli_unclonable-detect.phpt",
    "ext/mysqli/tests/055-detect.phpt",
    "ext/mysqli/tests/mysqli_class_mysqli_stmt_interface-detect.phpt",
    "ext/mysqli/tests/bug53503-detect.phpt",
    "ext/mysqli/tests/047-detect.phpt",
    "ext/mysqli/tests/bug49442-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_close_link-detect.phpt",
    "ext/mysqli/tests/mysqli_constants-detect.phpt",
    "ext/mysqli/tests/bug48909-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_data_seek-detect.phpt",
    "ext/mysqli/tests/mysqli_class_mysqli_driver_interface-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_get_result_non_select-detect.phpt",
    "ext/mysqli/tests/059-detect.phpt",
    "ext/mysqli/tests/mysqli_ping-detect.phpt",
    "ext/mysqli/tests/mysqli_real_escape_string_eucjpms-detect.phpt",
    "ext/mysqli/tests/025-detect.phpt",
    "ext/mysqli/tests/mysqli_query_iterators-detect.phpt",
    "ext/mysqli/tests/mysqli_data_seek-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_get_result_field_count-detect.phpt",
    "ext/mysqli/tests/012-detect.phpt",
    "ext/mysqli/tests/038-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_lengths-detect.phpt",
    "ext/mysqli/tests/bug46614-detect.phpt",
    "ext/mysqli/tests/mysqli_class_mysqli_driver_reflection-detect.phpt",
    "ext/mysqli/tests/mysqli_ssl_set-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_lengths_oo-detect.phpt",
    "ext/mysqli/tests/021-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_bind_result_references-detect.phpt",
    "ext/mysqli/tests/bug36949-detect.phpt",
    "ext/mysqli/tests/052-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_assoc_bit-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_kill_link-detect.phpt",
    "ext/mysqli/tests/mysqli_mysqlnd_read_timeout-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_attr_set-detect.phpt",
    "ext/mysqli/tests/mysqli_pconn_twice-detect.phpt",
    "ext/mysqli/tests/mysqli_query-detect.phpt",
    "ext/mysqli/tests/mysqli_report-detect.phpt",
    "ext/mysqli/tests/bug30967-detect.phpt",
    "ext/mysqli/tests/bug52082-detect.phpt",
    "ext/mysqli/tests/067-detect.phpt",
    "ext/mysqli/tests/mysqli_autocommit-detect.phpt",
    "ext/mysqli/tests/mysqli_options_init_command-detect.phpt",
    "ext/mysqli/tests/mysqli_error_oo-detect.phpt",
    "ext/mysqli/tests/mysqli_more_results-detect.phpt",
    "ext/mysqli/tests/mysqli_get_client_info-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_closefile-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_bind_result_zerofill-detect.phpt",
    "ext/mysqli/tests/bug36802-detect.phpt",
    "ext/mysqli/tests/mysqli_set_opt-detect.phpt",
    "ext/mysqli/tests/057-detect.phpt",
    "ext/mysqli/tests/mysqli_get_client_stats_ps-detect.phpt",
    "ext/mysqli/tests/mysqli_get_client_stats_off-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_object-detect.phpt",
    "ext/mysqli/tests/030-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_field_direct_oo-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_all-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_fetch-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_send_long_data_packet_size_mysqlnd-detect.phpt",
    "ext/mysqli/tests/mysqli_data_seek_oo-detect.phpt",
    "ext/mysqli/tests/bug39457-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_fetch_bit-detect.phpt",
    "ext/mysqli/tests/bug34785-detect.phpt",
    "ext/mysqli/tests/mysqli_debug_mysqlnd_control_string-detect.phpt",
    "ext/mysqli/tests/bug42378-detect.phpt",
    "ext/mysqli/tests/bug35517-detect.phpt",
    "ext/mysqli/tests/mysqli_next_result-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_get_result_metadata-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_object_no_object-detect.phpt",
    "ext/mysqli/tests/063-detect.phpt",
    "ext/mysqli/tests/032-detect.phpt",
    "ext/mysqli/tests/mysqli_prepare_no_object-detect.phpt",
    "ext/mysqli/tests/bug42548-detect.phpt",
    "ext/mysqli/tests/bug62046-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_affected_rows-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_result_metadata_sqltests-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_bind_param_check_param_no_change-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_execute_stored_proc_out-detect.phpt",
    "ext/mysqli/tests/019-detect.phpt",
    "ext/mysqli/tests/mysqli_field_seek-detect.phpt",
    "ext/mysqli/tests/mysqli_class_mysqli_result_interface-detect.phpt",
    "ext/mysqli/tests/mysqli_set_local_infile_handler_bad_character-detect.phpt",
    "ext/mysqli/tests/bug51605-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_assoc-detect.phpt",
    "ext/mysqli/tests/mysqli_set_charset-detect.phpt",
    "ext/mysqli/tests/mysqli_mysqlnd_read_timeout_long-detect.phpt",
    "ext/mysqli/tests/mysqli_explain_metadata-detect.phpt",
    "ext/mysqli/tests/mysqli_thread_safe-detect.phpt",
    "ext/mysqli/tests/mysqli_real_escape_string_sjis-detect.phpt",
    "ext/mysqli/tests/mysqli_connect_oo-detect.phpt",
    "ext/mysqli/tests/053-detect.phpt",
    "ext/mysqli/tests/051-detect.phpt",
    "ext/mysqli/tests/bug55653-detect.phpt",
    "ext/mysqli/tests/mysqli_pconn_conn_multiple-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_field-detect.phpt",
    "ext/mysqli/tests/024-detect.phpt",
    "ext/mysqli/tests/mysqli_auth_pam-detect.phpt",
    "ext/mysqli/tests/mysqli_fetch_field_flags-detect.phpt",
    "ext/mysqli/tests/065-detect.phpt",
    "ext/mysqli/tests/mysqli_change_user_locks_temporary-detect.phpt",
    "ext/mysqli/tests/009-detect.phpt",
    "ext/mysqli/tests/mysqli_real_connect-detect.phpt",
    "ext/mysqli/tests/mysqli_class_mysqli_reflection-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_execute-detect.phpt",
    "ext/mysqli/tests/mysqli_pconn_reuse-detect.phpt",
    "ext/mysqli/tests/mysqli_reconnect-detect.phpt",
    "ext/mysqli/tests/mysqli_close_oo-detect.phpt",
    "ext/mysqli/tests/042-detect.phpt",
    "ext/mysqli/tests/mysqli_debug-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_param_count-detect.phpt",
    "ext/mysqli/tests/mysqli_close-detect.phpt",
    "ext/mysqli/tests/mysqli_get_charset-detect.phpt",
    "ext/mysqli/tests/mysqli_character_set-detect.phpt",
    "ext/mysqli/tests/mysqli_real_escape_string_big5-detect.phpt",
    "ext/mysqli/tests/mysqli_stmt_init-detect.phpt",
    "ext/mysqli/tests/bug28817-detect.phpt",
    "ext/gettext/tests/dcngettext-detect.phpt",
    "ext/gettext/tests/gettext_gettext_error_wrongparams-detect.phpt",
    "ext/gettext/tests/gettext_bindtextdomain-emptydomain-detect.phpt",
    "ext/gettext/tests/gettext_basic-detect.phpt",
    "ext/gettext/tests/gettext_dcgettext-detect.phpt",
    "ext/gettext/tests/gettext_textdomain-retval-detect.phpt",
    "ext/gettext/tests/gettext_bindtextdomain-cwd-detect.phpt",
    "ext/gettext/tests/gettext_dngettext-plural-detect.phpt",
    "ext/gettext/tests/gettext_dngettext-wrongparams-detect.phpt",
    "ext/gettext/tests/gettext_dgettext-detect.phpt",
    "ext/gettext/tests/gettext_bindtextdomain-wrongparams-detect.phpt",
    "ext/gettext/tests/gettext_dgettext_error_wrongparams-detect.phpt",
    "ext/gettext/tests/gettext_ngettext-wrongparams-detect.phpt",
    "ext/gettext/tests/gettext_bind_textdomain_codeset-retval-detect.phpt",
    "ext/gettext/tests/gettext_ngettext-detect.phpt",
    "ext/gettext/tests/gettext_bindtextdomain-path-detect.phpt",
    "ext/gettext/tests/44938-detect.phpt",
    "ext/gettext/tests/gettext_phpinfo-detect.phpt",
    "ext/gettext/tests/gettext_dcgettext-wrongparams-detect.phpt",
    "ext/gettext/tests/gettext_bind_textdomain_codeset-wrongparams-detect.phpt",
    "ext/gettext/tests/gettext_basic-enus-detect.phpt",
    "ext/gettext/tests/gettext_textdomain-wrongparams-detect.phpt",
    "ext/pgsql/tests/pg_delete_001-detect.phpt",
    "ext/pgsql/tests/80_bug42783-detect.phpt",
    "ext/pgsql/tests/bug60244-detect.phpt",
    "ext/pgsql/tests/07optional-detect.phpt",
    "ext/pgsql/tests/80_bug14383-detect.phpt",
    "ext/pgsql/tests/16pg_result_status-detect.phpt",
    "ext/pgsql/tests/28large_object_import_oid-detect.phpt",
    "ext/pgsql/tests/80_bug36625-detect.phpt",
    "ext/pgsql/tests/bug37100-detect.phpt",
    "ext/pgsql/tests/20pg_get_pid-detect.phpt",
    "ext/pgsql/tests/12pg_insert-detect.phpt",
    "ext/pgsql/tests/80_bug32223-detect.phpt",
    "ext/pgsql/tests/01createdb-detect.phpt",
    "ext/pgsql/tests/03sync_query-detect.phpt",
    "ext/pgsql/tests/80_bug39971-detect.phpt",
    "ext/pgsql/tests/22pg_fetch_object-detect.phpt",
    "ext/pgsql/tests/80_bug24499-detect.phpt",
    "ext/pgsql/tests/17result-detect.phpt",
    "ext/pgsql/tests/12pg_insert_85-detect.phpt",
    "ext/pgsql/tests/pg_insert_001-detect.phpt",
    "ext/pgsql/tests/21pg_get_notify-detect.phpt",
    "ext/pgsql/tests/18pg_escape_bytea-detect.phpt",
    "ext/pgsql/tests/98old_api-detect.phpt",
    "ext/pgsql/tests/13pg_select_85-detect.phpt",
    "ext/pgsql/tests/24sync_query_prepared-detect.phpt",
    "ext/pgsql/tests/09notice-detect.phpt",
    "ext/pgsql/tests/15pg_delete-detect.phpt",
    "ext/pgsql/tests/pg_select_001-detect.phpt",
    "ext/pgsql/tests/14pg_update_85-detect.phpt",
    "ext/pgsql/tests/26async_query_prepared-detect.phpt",
    "ext/pgsql/tests/pg_update_001-detect.phpt",
    "ext/pgsql/tests/80_bug27597-detect.phpt",
    "ext/pgsql/tests/23sync_query_params-detect.phpt",
    "ext/pgsql/tests/06copy-detect.phpt",
    "ext/pgsql/tests/13pg_select-detect.phpt",
    "ext/pgsql/tests/14pg_update-detect.phpt",
    "ext/pgsql/tests/11pg_meta_data-detect.phpt",
    "ext/pgsql/tests/08escape-detect.phpt",
    "ext/pgsql/tests/04async_query-detect.phpt",
    "ext/pgsql/tests/05large_object-detect.phpt",
    "ext/pgsql/tests/pg_meta_data_001-detect.phpt",
    "ext/pgsql/tests/10pg_convert_85-detect.phpt",
    "ext/pgsql/tests/bug37100_85-detect.phpt",
    "ext/pgsql/tests/80_bug32223b-detect.phpt",
    "ext/pgsql/tests/25async_query_params-detect.phpt",
    "ext/pgsql/tests/02connection-detect.phpt",
    "ext/pgsql/tests/19pg_ping-detect.phpt",
    "ext/pgsql/tests/9999dropdb-detect.phpt",
    "ext/pgsql/tests/bug47199-detect.phpt",
    "ext/pgsql/tests/10pg_convert-detect.phpt",
    "ext/pgsql/tests/27large_object_oid-detect.phpt",
    "ext/spl/tests/bug48493-detect.phpt",
    "ext/spl/tests/SplFileObject_rewind_error001-detect.phpt",
    "ext/spl/tests/bug45622-detect.phpt",
    "ext/spl/tests/multiple_iterator_001-detect.phpt",
    "ext/spl/tests/bug62616-detect.phpt",
    "ext/spl/tests/iterator_035-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_offsetGet_param_string-detect.phpt",
    "ext/spl/tests/dit_005-detect.phpt",
    "ext/spl/tests/SplFixedArray_offsetSet_one_invalid_parameter-detect.phpt",
    "ext/spl/tests/bug53071-detect.phpt",
    "ext/spl/tests/spl_iterator_apply_error-detect.phpt",
    "ext/spl/tests/fixedarray_019-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_current-detect.phpt",
    "ext/spl/tests/DirectoryIterator_getBasename_basic_test-detect.phpt",
    "ext/spl/tests/arrayObject_getFlags_basic2-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_setIteratorMode_param_SplDoublyLinkedList-detect.phpt",
    "ext/spl/tests/SplDoublylinkedlist_offsetunset_first002-detect.phpt",
    "ext/spl/tests/recursiveIteratorIterator_endchildren_error-detect.phpt",
    "ext/spl/tests/fileobject_001-detect.phpt",
    "ext/spl/tests/iterator_049b-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_isEmpty_empty-detect.phpt",
    "ext/spl/tests/bug41692-detect.phpt",
    "ext/spl/tests/spl_iterator_apply_error_001-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_getIteratorMode-detect.phpt",
    "ext/spl/tests/iterator_067-detect.phpt",
    "ext/spl/tests/array_007-detect.phpt",
    "ext/spl/tests/observer_003-detect.phpt",
    "ext/spl/tests/pqueue_002-detect.phpt",
    "ext/spl/tests/limititerator_seek-detect.phpt",
    "ext/spl/tests/bug46053-detect.phpt",
    "ext/spl/tests/pqueue_001-detect.phpt",
    "ext/spl/tests/iterator_002-detect.phpt",
    "ext/spl/tests/bug33136-detect.phpt",
    "ext/spl/tests/SplFileObject_key_error002-detect.phpt",
    "ext/spl/tests/bug48361-detect.phpt",
    "ext/spl/tests/bug31926-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_top_pass_array-detect.phpt",
    "ext/spl/tests/dit_001-detect.phpt",
    "ext/spl/tests/fixedarray_008-detect.phpt",
    "ext/spl/tests/iterator_004-detect.phpt",
    "ext/spl/tests/pqueue_003-detect.phpt",
    "ext/spl/tests/SplFileObject_fputcsv_variation7-detect.phpt",
    "ext/spl/tests/arrayObject_magicMethods3-detect.phpt",
    "ext/spl/tests/iterator_017-detect.phpt",
    "ext/spl/tests/SplObjectStorage_contains_invalid_parameter-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_isEmpty_not-empty-with-parameter-detect.phpt",
    "ext/spl/tests/SplFixedArray_fromarray_non_indexes-detect.phpt",
    "ext/spl/tests/DirectoryIterator_by_reference-detect.phpt",
    "ext/spl/tests/bug31346-detect.phpt",
    "ext/spl/tests/SplFileObject_getflags_error002-detect.phpt",
    "ext/spl/tests/spl_heap_iteration_error-detect.phpt",
    "ext/spl/tests/array_026-detect.phpt",
    "ext/spl/tests/SplFileObject_seek_error002-detect.phpt",
    "ext/spl/tests/iterator_038-detect.phpt",
    "ext/spl/tests/spl_cachingiterator___toString_basic-detect.phpt",
    "ext/spl/tests/SplFileObject_fputcsv_variation8-detect.phpt",
    "ext/spl/tests/SplFileObject_next_basic-detect.phpt",
    "ext/spl/tests/SplHeap_extract_invalid_parameter-detect.phpt",
    "ext/spl/tests/SplFileObject_fputcsv-detect.phpt",
    "ext/spl/tests/heap_004-detect.phpt",
    "ext/spl/tests/array_002-detect.phpt",
    "ext/spl/tests/SplObjectStorage_getHash-detect.phpt",
    "ext/spl/tests/SplFixedArray_construct_param_SplFixedArray-detect.phpt",
    "ext/spl/tests/spl_iterator_to_array_basic-detect.phpt",
    "ext/spl/tests/fixedarray_012-detect.phpt",
    "ext/spl/tests/heap_top_variation_002-detect.phpt",
    "ext/spl/tests/iterator_065-detect.phpt",
    "ext/spl/tests/SplFileObject_fseek_error_001-detect.phpt",
    "ext/spl/tests/iterator_032-detect.phpt",
    "ext/spl/tests/SplFileObject_current_variation001-detect.phpt",
    "ext/spl/tests/recursiveiteratoriterator_enditeration_basic-detect.phpt",
    "ext/spl/tests/bug51532-detect.phpt",
    "ext/spl/tests/spl_iterator_to_array_error-detect.phpt",
    "ext/spl/tests/bug38618-detect.phpt",
    "ext/spl/tests/spl_autoload_010-detect.phpt",
    "ext/spl/tests/SplObjectStorage_removeAllExcept_basic-detect.phpt",
    "ext/spl/tests/SplFixedArray_setsize_shrink-detect.phpt",
    "ext/spl/tests/SplTempFileObject_constructor_memory_lt1_variation-detect.phpt",
    "ext/spl/tests/SplFixedArray_getSize_pass_param-detect.phpt",
    "ext/spl/tests/iterator_031-detect.phpt",
    "ext/spl/tests/bug41828-detect.phpt",
    "ext/spl/tests/SplFixedArray_setSize_reduce-detect.phpt",
    "ext/spl/tests/iterator_025-detect.phpt",
    "ext/spl/tests/SplObjectStorage_removeAllExcept_invalid_parameter-detect.phpt",
    "ext/spl/tests/recursive_tree_iterator_006-detect.phpt",
    "ext/spl/tests/SplFixedArray_setSize_param_float-detect.phpt",
    "ext/spl/tests/pqueue_compare_basic-detect.phpt",
    "ext/spl/tests/arrayObject_ksort_basic2-detect.phpt",
    "ext/spl/tests/spl_pq_top_error_corrupt-detect.phpt",
    "ext/spl/tests/iterator_011-detect.phpt",
    "ext/spl/tests/SplFileObject_getchildren_error001-detect.phpt",
    "ext/spl/tests/spl_004-detect.phpt",
    "ext/spl/tests/heap_005-detect.phpt",
    "ext/spl/tests/SplObjectStorage_offsetGet-detect.phpt",
    "ext/spl/tests/regexiterator_setpregflags_exception-detect.phpt",
    "ext/spl/tests/SplFileObject_fgetcsv_enclosure_error-detect.phpt",
    "ext/spl/tests/spl_006-detect.phpt",
    "ext/spl/tests/SplObjectStorage_addAll_invalid_parameter-detect.phpt",
    "ext/spl/tests/iterator_063-detect.phpt",
    "ext/spl/tests/array_019-detect.phpt",
    "ext/spl/tests/SplFileObject_seek_basic-detect.phpt",
    "ext/spl/tests/SplFileObject_haschildren_basic-detect.phpt",
    "ext/spl/tests/bug40036-detect.phpt",
    "ext/spl/tests/arrayObject_magicMethods5-detect.phpt",
    "ext/spl/tests/iterator_030-detect.phpt",
    "ext/spl/tests/arrayObject___construct_basic5-detect.phpt",
    "ext/spl/tests/fileobject_003-detect.phpt",
    "ext/spl/tests/bug52238-detect.phpt",
    "ext/spl/tests/arrayObject_uasort_basic1-detect.phpt",
    "ext/spl/tests/fixedarray_003-detect.phpt",
    "ext/spl/tests/iterator_033-detect.phpt",
    "ext/spl/tests/arrayObject_uksort_error1-detect.phpt",
    "ext/spl/tests/SplFixedArray_rewind_param-detect.phpt",
    "ext/spl/tests/dit_004-detect.phpt",
    "ext/spl/tests/spl_pq_top_basic-detect.phpt",
    "ext/spl/tests/bug54323-detect.phpt",
    "ext/spl/tests/array_009-detect.phpt",
    "ext/spl/tests/SplObjectStorage_getInfo_empty_storage-detect.phpt",
    "ext/spl/tests/fixedarray_004-detect.phpt",
    "ext/spl/tests/bug62262-detect.phpt",
    "ext/spl/tests/iterator_029-detect.phpt",
    "ext/spl/tests/SplObjectStorage_detach_invalid_parameter-detect.phpt",
    "ext/spl/tests/SplFixedArray_next_param-detect.phpt",
    "ext/spl/tests/array_021-detect.phpt",
    "ext/spl/tests/SplObjectStorage_current_empty_storage-detect.phpt",
    "ext/spl/tests/SplFixedArray_setSize_filled_to_smaller-detect.phpt",
    "ext/spl/tests/dllist_006-detect.phpt",
    "ext/spl/tests/dit_006-detect.phpt",
    "ext/spl/tests/heap_011-detect.phpt",
    "ext/spl/tests/heap_corruption-detect.phpt",
    "ext/spl/tests/SplFileObject_getflags_variation001-detect.phpt",
    "ext/spl/tests/arrayObject___construct_basic6-detect.phpt",
    "ext/spl/tests/bug54291-detect.phpt",
    "ext/spl/tests/bug61418-detect.phpt",
    "ext/spl/tests/regexIterator_flags_basic-detect.phpt",
    "ext/spl/tests/bug54970-detect.phpt",
    "ext/spl/tests/recursiveiteratoriterator_nextelement_basic-detect.phpt",
    "ext/spl/tests/fixedarray_006-detect.phpt",
    "ext/spl/tests/iterator_005-detect.phpt",
    "ext/spl/tests/bug48023-detect.phpt",
    "ext/spl/tests/iterator_061-detect.phpt",
    "ext/spl/tests/bug45826-detect.phpt",
    "ext/spl/tests/recursive_tree_iterator_005-detect.phpt",
    "ext/spl/tests/observer_007-detect.phpt",
    "ext/spl/tests/heap_009-detect.phpt",
    "ext/spl/tests/bug49723-detect.phpt",
    "ext/spl/tests/spl_iterator_getcallchildren-detect.phpt",
    "ext/spl/tests/SplFileObject_haschildren_error001-detect.phpt",
    "ext/spl/tests/SplFixedArray_offsetGet_invalid_parameter-detect.phpt",
    "ext/spl/tests/observer_005-detect.phpt",
    "ext/spl/tests/fixedarray_018-detect.phpt",
    "ext/spl/tests/spl_heap_count_error-detect.phpt",
    "ext/spl/tests/array_015-detect.phpt",
    "ext/spl/tests/observer_006-detect.phpt",
    "ext/spl/tests/heap_isempty_variation_001-detect.phpt",
    "ext/spl/tests/SplFixedArray_setSize_param_array-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_count_param_SplDoublyLinkedList-detect.phpt",
    "ext/spl/tests/fileobject_setmaxlinelen_error003-detect.phpt",
    "ext/spl/tests/SplFixedArray_fromArray_invalid_parameter_002-detect.phpt",
    "ext/spl/tests/recursive_tree_iterator_002-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_top_pass_float-detect.phpt",
    "ext/spl/tests/SplFileObject_fwrite_variation_001-detect.phpt",
    "ext/spl/tests/SplPriorityQueue_insert_invalid_parameter-detect.phpt",
    "ext/spl/tests/dit_001_noglob-detect.phpt",
    "ext/spl/tests/iterator_006-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_top_pass_null-detect.phpt",
    "ext/spl/tests/SplFileObject_fwrite_variation_002-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_top_pass_integer-detect.phpt",
    "ext/spl/tests/heap_it_current_empty-detect.phpt",
    "ext/spl/tests/SplFixedArray_fromArray_invalid_parameter_001-detect.phpt",
    "ext/spl/tests/array_023-detect.phpt",
    "ext/spl/tests/SplFixedArray_fromarray_indexes-detect.phpt",
    "ext/spl/tests/iterator_012-detect.phpt",
    "ext/spl/tests/bug47534-detect.phpt",
    "ext/spl/tests/SplFixedArray_offsetExists_invalid_parameter-detect.phpt",
    "ext/spl/tests/arrayObject_count_basic1-detect.phpt",
    "ext/spl/tests/bug37457-detect.phpt",
    "ext/spl/tests/spl_heap_count_basic-detect.phpt",
    "ext/spl/tests/fileobject_getsize_basic-detect.phpt",
    "ext/spl/tests/arrayObject_exchangeArray_basic2-detect.phpt",
    "ext/spl/tests/observer_002-detect.phpt",
    "ext/spl/tests/spl_minheap_compare_error-detect.phpt",
    "ext/spl/tests/bug36941-detect.phpt",
    "ext/spl/tests/bug42364-detect.phpt",
    "ext/spl/tests/regexiterator_setpregflags-detect.phpt",
    "ext/spl/tests/recursiveiteratoriterator_getsubiterator_basic-detect.phpt",
    "ext/spl/tests/SplFileInfo_getOwner_error-detect.phpt",
    "ext/spl/tests/array_001-detect.phpt",
    "ext/spl/tests/spl_005-detect.phpt",
    "ext/spl/tests/bug61453-detect.phpt",
    "ext/spl/tests/SplFileObject_fputcsv_variation5-detect.phpt",
    "ext/spl/tests/observer_009-detect.phpt",
    "ext/spl/tests/bug42654-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_offsetUnset_negative-parameter-detect.phpt",
    "ext/spl/tests/dllist_007-detect.phpt",
    "ext/spl/tests/arrayObject_offsetExists_nullcheck-detect.phpt",
    "ext/spl/tests/DirectoryIterator_getInode_basic-detect.phpt",
    "ext/spl/tests/recursive_tree_iterator_008-detect.phpt",
    "ext/spl/tests/dllist_002-detect.phpt",
    "ext/spl/tests/SplFixedArray__construct_param_string-detect.phpt",
    "ext/spl/tests/DirectoryIterator_getExtension_basic-detect.phpt",
    "ext/spl/tests/dllist_010-detect.phpt",
    "ext/spl/tests/iterator_050-detect.phpt",
    "ext/spl/tests/heap_002-detect.phpt",
    "ext/spl/tests/arrayObject___construct_basic4-detect.phpt",
    "ext/spl/tests/iterator_040-detect.phpt",
    "ext/spl/tests/SplFileObject_fgetcsv_delimiter_error-detect.phpt",
    "ext/spl/tests/SplObjectStorage_attach_invalid_parameter-detect.phpt",
    "ext/spl/tests/recursiveiteratoriterator_getsubiterator_variation_002-detect.phpt",
    "ext/spl/tests/array_024-detect.phpt",
    "ext/spl/tests/bug45216-detect.phpt",
    "ext/spl/tests/iterator_047-detect.phpt",
    "ext/spl/tests/spl_007-detect.phpt",
    "ext/spl/tests/bug42703-detect.phpt",
    "ext/spl/tests/bug46115-detect.phpt",
    "ext/spl/tests/iterator_008-detect.phpt",
    "ext/spl/tests/SplFixedArray_setSize_param_null-detect.phpt",
    "ext/spl/tests/bug61527-detect.phpt",
    "ext/spl/tests/fixedarray_010-detect.phpt",
    "ext/spl/tests/SplFixedArray_current_param-detect.phpt",
    "ext/spl/tests/SplObjectStorage_unserialize_nested-detect.phpt",
    "ext/spl/tests/iterator_041-detect.phpt",
    "ext/spl/tests/iterator_036-detect.phpt",
    "ext/spl/tests/bug62904-detect.phpt",
    "ext/spl/tests/spl_fileinfo_getlinktarget_basic-detect.phpt",
    "ext/spl/tests/fixedarray_021-detect.phpt",
    "ext/spl/tests/iterator_018-detect.phpt",
    "ext/spl/tests/iterator_042-detect.phpt",
    "ext/spl/tests/iterator_052-detect.phpt",
    "ext/spl/tests/spl_003-detect.phpt",
    "ext/spl/tests/arrayObject_getIteratorClass_basic1-detect.phpt",
    "ext/spl/tests/SplObjectStorage_unserialize_invalid_parameter3-detect.phpt",
    "ext/spl/tests/spl_heap_extract_parameter_error-detect.phpt",
    "ext/spl/tests/bug53144-detect.phpt",
    "ext/spl/tests/arrayObject_setIteratorClass_error1-detect.phpt",
    "ext/spl/tests/bug50579-detect.phpt",
    "ext/spl/tests/bug49263-detect.phpt",
    "ext/spl/tests/SplFileObject_next_variation001-detect.phpt",
    "ext/spl/tests/countable_count_variation1-detect.phpt",
    "ext/spl/tests/SplFileObject_fgetcsv_delimiter_basic-detect.phpt",
    "ext/spl/tests/SplObjectStorage_removeAll_invalid_parameter-detect.phpt",
    "ext/spl/tests/bug41691-detect.phpt",
    "ext/spl/tests/iterator_037-detect.phpt",
    "ext/spl/tests/recursiveIteratorIterator_beginchildren_error-detect.phpt",
    "ext/spl/tests/class_uses_variation1-detect.phpt",
    "ext/spl/tests/iterator_055-detect.phpt",
    "ext/spl/tests/array_017-detect.phpt",
    "ext/spl/tests/iterator_041a-detect.phpt",
    "ext/spl/tests/spl_heap_isempty-detect.phpt",
    "ext/spl/tests/iterator_066-detect.phpt",
    "ext/spl/tests/spl_heap_recoverfromcorruption_arguments-detect.phpt",
    "ext/spl/tests/array_010-detect.phpt",
    "ext/spl/tests/iterator_014-detect.phpt",
    "ext/spl/tests/SplFileObject_next_variation002-detect.phpt",
    "ext/spl/tests/SplFileObject_fgetcsv_enclosure_basic-detect.phpt",
    "ext/spl/tests/iterator_022-detect.phpt",
    "ext/spl/tests/SplFileInfo_getInode_error-detect.phpt",
    "ext/spl/tests/SplFixedArray_setsize_grow-detect.phpt",
    "ext/spl/tests/SplFileObject_getchildren_basic-detect.phpt",
    "ext/spl/tests/SplFixedArray_key_setsize-detect.phpt",
    "ext/spl/tests/SplFileObject_setCsvControl_error003-detect.phpt",
    "ext/spl/tests/regexIterator_mode_basic-detect.phpt",
    "ext/spl/tests/array_013-detect.phpt",
    "ext/spl/tests/bug31185-detect.phpt",
    "ext/spl/tests/bug54281-detect.phpt",
    "ext/spl/tests/iterator_044-detect.phpt",
    "ext/spl/tests/iterator_013-detect.phpt",
    "ext/spl/tests/SplFileInfo_getPerms_error-detect.phpt",
    "ext/spl/tests/SplHeap_insert_invalid_parameter-detect.phpt",
    "ext/spl/tests/SplFileInfo_getInode_basic-detect.phpt",
    "ext/spl/tests/arrayObject_magicMethods4-detect.phpt",
    "ext/spl/tests/bug31348-detect.phpt",
    "ext/spl/tests/class_implements_variation1-detect.phpt",
    "ext/spl/tests/fileobject_setmaxlinelen_error001-detect.phpt",
    "ext/spl/tests/iterator_048-detect.phpt",
    "ext/spl/tests/fixedarray_017-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_lifoMode-detect.phpt",
    "ext/spl/tests/spl_maxheap_compare_basic-detect.phpt",
    "ext/spl/tests/array_022-detect.phpt",
    "ext/spl/tests/SplQueue_setIteratorMode_param_lifo-detect.phpt",
    "ext/spl/tests/splDoublyLinkedList_shift_noParams-detect.phpt",
    "ext/spl/tests/bug61347-detect.phpt",
    "ext/spl/tests/iterator_016-detect.phpt",
    "ext/spl/tests/spl_heap_is_empty_basic-detect.phpt",
    "ext/spl/tests/dit_002-detect.phpt",
    "ext/spl/tests/SplFileObject_current_error001-detect.phpt",
    "ext/spl/tests/SplFixedArray_setsize_001-detect.phpt",
    "ext/spl/tests/fixedarray_009-detect.phpt",
    "ext/spl/tests/DirectoryIterator_getBasename_pass_array-detect.phpt",
    "ext/spl/tests/bug32134-detect.phpt",
    "ext/spl/tests/SplTempFileObject_constructor_error-detect.phpt",
    "ext/spl/tests/SplFileObject_fputcsv_variation14-detect.phpt",
    "ext/spl/tests/class_implements_variation-detect.phpt",
    "ext/spl/tests/iterator_057-detect.phpt",
    "ext/spl/tests/SplFileObject_rewind_basic-detect.phpt",
    "ext/spl/tests/array_003-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_offsetGet_param_array-detect.phpt",
    "ext/spl/tests/spl_caching_iterator_constructor_flags-detect.phpt",
    "ext/spl/tests/iterator_059-detect.phpt",
    "ext/spl/tests/SplObjectStorage_unserialize_invalid_parameter2-detect.phpt",
    "ext/spl/tests/SplFixedArray_count_param_int-detect.phpt",
    "ext/spl/tests/SplFixedArray_offsetUnset_invalid_parameter-detect.phpt",
    "ext/spl/tests/fixedarray_015-detect.phpt",
    "ext/spl/tests/fixedarray_020-detect.phpt",
    "ext/spl/tests/DirectoryIterator_empty_constructor-detect.phpt",
    "ext/spl/tests/recursive_tree_iterator_001-detect.phpt",
    "ext/spl/tests/iterator_071-detect.phpt",
    "ext/spl/tests/SplPriorityQueue_extract_invalid_parameter-detect.phpt",
    "ext/spl/tests/bug62073-detect.phpt",
    "ext/spl/tests/array_011-detect.phpt",
    "ext/spl/tests/SplFileObject_fgetcsv_escape_default-detect.phpt",
    "ext/spl/tests/array_016-detect.phpt",
    "ext/spl/tests/spl_autoload_call_basic-detect.phpt",
    "ext/spl/tests/bug28822-detect.phpt",
    "ext/spl/tests/SplDoublylinkedlist_offsetunset_first-detect.phpt",
    "ext/spl/tests/SplFileObject_fputcsv_variation10-detect.phpt",
    "ext/spl/tests/bug44615-detect.phpt",
    "ext/spl/tests/SplFixedArray__construct_param_float-detect.phpt",
    "ext/spl/tests/SplFileObject_fputcsv_variation12-detect.phpt",
    "ext/spl/tests/spl_recursive_iterator_iterator_key_case-detect.phpt",
    "ext/spl/tests/filesystemiterator_flags-detect.phpt",
    "ext/spl/tests/spl_autoload_011-detect.phpt",
    "ext/spl/tests/SplFixedArray_toArray_with-params-detect.phpt",
    "ext/spl/tests/SplFixedArray_count_checkParams-detect.phpt",
    "ext/spl/tests/spl_iterator_caching_count_error-detect.phpt",
    "ext/spl/tests/iterator_001-detect.phpt",
    "ext/spl/tests/SplObjectStorage_unserialize_invalid_parameter1-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_serialization-detect.phpt",
    "ext/spl/tests/iterator_023-detect.phpt",
    "ext/spl/tests/SplFileObject_ftruncate_error_001-detect.phpt",
    "ext/spl/tests/CallbackFilterIteratorTest-detect.phpt",
    "ext/spl/tests/fileobject_getcurrentline_basic-detect.phpt",
    "ext/spl/tests/SplObjectStorage_unserialize_bad-detect.phpt",
    "ext/spl/tests/bug51119-detect.phpt",
    "ext/spl/tests/recursive_tree_iterator_003-detect.phpt",
    "ext/spl/tests/observer_008-detect.phpt",
    "ext/spl/tests/spl_limit_iterator_check_limits-detect.phpt",
    "ext/spl/tests/bug49972-detect.phpt",
    "ext/spl/tests/regexIterator_setMode_error-detect.phpt",
    "ext/spl/tests/recursiveiteratoriterator_beginiteration_basic-detect.phpt",
    "ext/spl/tests/spl_autoload_001-detect.phpt",
    "ext/spl/tests/SplFileObject_fgetcsv_escape_error-detect.phpt",
    "ext/spl/tests/array_025-detect.phpt",
    "ext/spl/tests/arrayObject_magicMethods2-detect.phpt",
    "ext/spl/tests/bug40091-detect.phpt",
    "ext/spl/tests/recursive_tree_iterator_007-detect.phpt",
    "ext/spl/tests/SplTempFileObject_constructor_maxmemory_basic-detect.phpt",
    "ext/spl/tests/fileobject_getmaxlinelen_basic-detect.phpt",
    "ext/spl/tests/bug36825-detect.phpt",
    "ext/spl/tests/bug53362-detect.phpt",
    "ext/spl/tests/DirectoryIterator_getInode_error-detect.phpt",
    "ext/spl/tests/iterator_043-detect.phpt",
    "ext/spl/tests/iterator_068-detect.phpt",
    "ext/spl/tests/iterator_051-detect.phpt",
    "ext/spl/tests/fileobject_setmaxlinelen_basic-detect.phpt",
    "ext/spl/tests/pqueue_current_error-detect.phpt",
    "ext/spl/tests/ArrayObject_unserialize_empty_string-detect.phpt",
    "ext/spl/tests/arrayObject_uksort_basic1-detect.phpt",
    "ext/spl/tests/SplObjectStorage_var_dump-detect.phpt",
    "ext/spl/tests/SplFileObject_setCsvControl_error001-detect.phpt",
    "ext/spl/tests/dit_003-detect.phpt",
    "ext/spl/tests/spl_autoload_002-detect.phpt",
    "ext/spl/tests/SplFileObject_key_basic-detect.phpt",
    "ext/spl/tests/bug32394-detect.phpt",
    "ext/spl/tests/bug54304-detect.phpt",
    "ext/spl/tests/array_012-detect.phpt",
    "ext/spl/tests/SplFixedArray_offsetUnset_string-detect.phpt",
    "ext/spl/tests/SplFileObject_seek_error_001-detect.phpt",
    "ext/spl/tests/class_uses_basic2-detect.phpt",
    "ext/spl/tests/arrayObject___construct_error2-detect.phpt",
    "ext/spl/tests/fileobject_002-detect.phpt",
    "ext/spl/tests/heap_012-detect.phpt",
    "ext/spl/tests/fixedarray_001-detect.phpt",
    "ext/spl/tests/iterator_028-detect.phpt",
    "ext/spl/tests/iterator_046-detect.phpt",
    "ext/spl/tests/iterator_069-detect.phpt",
    "ext/spl/tests/iterator_count-detect.phpt",
    "ext/spl/tests/arrayObject_natsort_basic1-detect.phpt",
    "ext/spl/tests/spl_pq_top_error_empty-detect.phpt",
    "ext/spl/tests/heap_007-detect.phpt",
    "ext/spl/tests/arrayObject___construct_basic1-detect.phpt",
    "ext/spl/tests/iterator_053-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_isEmpty_not-empty-detect.phpt",
    "ext/spl/tests/arrayObject_natcasesort_basic1-detect.phpt",
    "ext/spl/tests/bug51374-detect.phpt",
    "ext/spl/tests/arrayObject_uasort_error1-detect.phpt",
    "ext/spl/tests/spl_autoload_007-detect.phpt",
    "ext/spl/tests/fixedarray_013-detect.phpt",
    "ext/spl/tests/SplFileObject_fputcsv_error-detect.phpt",
    "ext/spl/tests/bug46051-detect.phpt",
    "ext/spl/tests/fixedarray_011-detect.phpt",
    "ext/spl/tests/SplFileObject_fputcsv_variation13-detect.phpt",
    "ext/spl/tests/bug45614-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_offsetGet_empty-detect.phpt",
    "ext/spl/tests/SplFileObject_fputcsv_variation6-detect.phpt",
    "ext/spl/tests/spl_heap_insert_basic-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_bottom_pass_integer-detect.phpt",
    "ext/spl/tests/spl_autoload_009-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_current_empty-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_bottom_pass_array-detect.phpt",
    "ext/spl/tests/spl_autoload_005-detect.phpt",
    "ext/spl/tests/heap_001-detect.phpt",
    "ext/spl/tests/CallbackFilterIteratorTest-002-detect.phpt",
    "ext/spl/tests/fileobject_checktype_basic-detect.phpt",
    "ext/spl/tests/SplFixedArray_key_param-detect.phpt",
    "ext/spl/tests/SplFixedArray_fromarray_param_boolean-detect.phpt",
    "ext/spl/tests/arrayObject_asort_basic1-detect.phpt",
    "ext/spl/tests/heap_top_variation_001-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_push_missing_parameter-detect.phpt",
    "ext/spl/tests/spl_iterator_iterator_constructor-detect.phpt",
    "ext/spl/tests/spl_autoload_008-detect.phpt",
    "ext/spl/tests/bug62433-detect.phpt",
    "ext/spl/tests/arrayObject_magicMethods1-detect.phpt",
    "ext/spl/tests/SplFixedArray__construct_param_array-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_bottom_pass_null-detect.phpt",
    "ext/spl/tests/SplFileObject_fgetcsv_escape_basic-detect.phpt",
    "ext/spl/tests/bug62328-detect.phpt",
    "ext/spl/tests/array_004-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_offsetExists_success-detect.phpt",
    "ext/spl/tests/class_uses_basic-detect.phpt",
    "ext/spl/tests/dllist_008-detect.phpt",
    "ext/spl/tests/SplFileObject_key_error001-detect.phpt",
    "ext/spl/tests/arrayObject_exchangeArray_basic1-detect.phpt",
    "ext/spl/tests/spl_autoload_003-detect.phpt",
    "ext/spl/tests/iterator_045-detect.phpt",
    "ext/spl/tests/fileobject_getmaxlinelen_error001-detect.phpt",
    "ext/spl/tests/spl_recursiveIteratorIterator_setMaxDepth_parameter_count-detect.phpt",
    "ext/spl/tests/heap_008-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_pop_noParams-detect.phpt",
    "ext/spl/tests/SplObjectStorage_offsetGet_missing_object-detect.phpt",
    "ext/spl/tests/iterator_058-detect.phpt",
    "ext/spl/tests/spl_autoload_004-detect.phpt",
    "ext/spl/tests/SplFileInfo_001-detect.phpt",
    "ext/spl/tests/SplArray_fromArray-detect.phpt",
    "ext/spl/tests/recursiveIteratorIterator_nextelement_error-detect.phpt",
    "ext/spl/tests/SplQueue_setIteratorMode-detect.phpt",
    "ext/spl/tests/array_006-detect.phpt",
    "ext/spl/tests/bug54971-detect.phpt",
    "ext/spl/tests/spl_autoload_bug48541-detect.phpt",
    "ext/spl/tests/array_020-detect.phpt",
    "ext/spl/tests/SplFixedArray__construct_param_null-detect.phpt",
    "ext/spl/tests/splpriorityqueue_setextractflags-detect.phpt",
    "ext/spl/tests/bug45622b-detect.phpt",
    "ext/spl/tests/dllist_001-detect.phpt",
    "ext/spl/tests/bug46031-detect.phpt",
    "ext/spl/tests/class_uses_variation2-detect.phpt",
    "ext/spl/tests/bug60201-detect.phpt",
    "ext/spl/tests/bug46088-detect.phpt",
    "ext/spl/tests/recursive_tree_iterator_004-detect.phpt",
    "ext/spl/tests/arrayObject_clone_basic3-detect.phpt",
    "ext/spl/tests/SplFileObject_getflags_basic-detect.phpt",
    "ext/spl/tests/SplFileObject_getCsvControl_basic_001-detect.phpt",
    "ext/spl/tests/array_027-detect.phpt",
    "ext/spl/tests/dllist_011-detect.phpt",
    "ext/spl/tests/spl_autoload_014-detect.phpt",
    "ext/spl/tests/spl_autoload_013-detect.phpt",
    "ext/spl/tests/recursiveiteratoriterator_getsubiterator_error-detect.phpt",
    "ext/spl/tests/iterator_024-detect.phpt",
    "ext/spl/tests/splpriorityqueue_extract-detect.phpt",
    "ext/spl/tests/SplObjectStorage_offsetGet_invalid_parameter-detect.phpt",
    "ext/spl/tests/SplFileObject_fflush_basic_001-detect.phpt",
    "ext/spl/tests/SplFileObject_fputcsv_variation1-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_pop_params-detect.phpt",
    "ext/spl/tests/countable_class_basic1-detect.phpt",
    "ext/spl/tests/bug36258-detect.phpt",
    "ext/spl/tests/splfixedarray_offsetExists_larger-detect.phpt",
    "ext/spl/tests/SplFileObject_setCsvControl_variation001-detect.phpt",
    "ext/spl/tests/iterator_060-detect.phpt",
    "ext/spl/tests/regexiterator_setflags_exception-detect.phpt",
    "ext/spl/tests/heap_003-detect.phpt",
    "ext/spl/tests/iterator_049-detect.phpt",
    "ext/spl/tests/iterator_054-detect.phpt",
    "ext/spl/tests/bug53515-detect.phpt",
    "ext/spl/tests/class_implements_basic-detect.phpt",
    "ext/spl/tests/recursiveiteratoriterator_getsubiterator_variation-detect.phpt",
    "ext/spl/tests/array_014-detect.phpt",
    "ext/spl/tests/spl_autoload_006-detect.phpt",
    "ext/spl/tests/bug38325-detect.phpt",
    "ext/spl/tests/bug52573-detect.phpt",
    "ext/spl/tests/bug61326-detect.phpt",
    "ext/spl/tests/array_018-detect.phpt",
    "ext/spl/tests/spl_pq_top_error_args-detect.phpt",
    "ext/spl/tests/fixedarray_005-detect.phpt",
    "ext/spl/tests/SplTempFileObject_constructor_basic-detect.phpt",
    "ext/spl/tests/heap_current_variation_001-detect.phpt",
    "ext/spl/tests/SplFileObject_setCsvControl_error002-detect.phpt",
    "ext/spl/tests/iterator_034-detect.phpt",
    "ext/spl/tests/pqueue_compare_error-detect.phpt",
    "ext/spl/tests/bug40442-detect.phpt",
    "ext/spl/tests/observer_001-detect.phpt",
    "ext/spl/tests/SplFileInfo_getOwner_basic-detect.phpt",
    "ext/spl/tests/SplHeap_isEmpty_invalid_parameter-detect.phpt",
    "ext/spl/tests/SplFixedArray_offsetSet_invalid_parameter-detect.phpt",
    "ext/spl/tests/class_implements_variation2-detect.phpt",
    "ext/spl/tests/pqueue_004-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_offsetSet_invalid_parameter-detect.phpt",
    "ext/spl/tests/SplHeap_count_invalid_parameter-detect.phpt",
    "ext/spl/tests/iterator_to_array-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_unshift_missing_parameter-detect.phpt",
    "ext/spl/tests/spl_iterator_recursive_getiterator_error-detect.phpt",
    "ext/spl/tests/iterator_010-detect.phpt",
    "ext/spl/tests/SplObjectStorage_setInfo_invalid_parameter-detect.phpt",
    "ext/spl/tests/arrayObject_clone_basic1-detect.phpt",
    "ext/spl/tests/fixedarray_002-detect.phpt",
    "ext/spl/tests/arrayObject___construct_error1-detect.phpt",
    "ext/spl/tests/SplHeap_isEmpty-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_bottom_pass_float-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_debug-info-detect.phpt",
    "ext/spl/tests/heap_top_variation_003-detect.phpt",
    "ext/spl/tests/SplFileObject_key_error003-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_offsetSet_one_invalid_parameter-detect.phpt",
    "ext/spl/tests/fixedarray_016-detect.phpt",
    "ext/spl/tests/SplFileObject_fputcsv_variation11-detect.phpt",
    "ext/spl/tests/dllist_003-detect.phpt",
    "ext/spl/tests/iterator_070-detect.phpt",
    "ext/spl/tests/arrayObject_setFlags_basic2-detect.phpt",
    "ext/spl/tests/SplFileObject_current_basic-detect.phpt",
    "ext/spl/tests/arrayObject_clone_basic2-detect.phpt",
    "ext/spl/tests/arrayObject___construct_basic3-detect.phpt",
    "ext/spl/tests/array_005-detect.phpt",
    "ext/spl/tests/bug41528-detect.phpt",
    "ext/spl/tests/SplFileObject_fgetcsv_basic-detect.phpt",
    "ext/spl/tests/iterator_009-detect.phpt",
    "ext/spl/tests/fixedarray_007-detect.phpt",
    "ext/spl/tests/SplDoublylinkedlist_offsetunset_last-detect.phpt",
    "ext/spl/tests/SplFileInfo_getGroup_error-detect.phpt",
    "ext/spl/tests/spl_iterator_caching_getcache_error-detect.phpt",
    "ext/spl/tests/recursive_tree_iterator_setprefixpart-detect.phpt",
    "ext/spl/tests/class_uses_variation-detect.phpt",
    "ext/spl/tests/regexiterator_getregex-detect.phpt",
    "ext/spl/tests/iterator_019-detect.phpt",
    "ext/spl/tests/bug40872-detect.phpt",
    "ext/spl/tests/fileobject_005-detect.phpt",
    "ext/spl/tests/arrayObject___construct_basic2-detect.phpt",
    "ext/spl/tests/arrayObject_asort_basic2-detect.phpt",
    "ext/spl/tests/fixedarray_014-detect.phpt",
    "ext/spl/tests/SplFileObject_fwrite_error_001-detect.phpt",
    "ext/spl/tests/iterator_039-detect.phpt",
    "ext/spl/tests/recursiveIteratorIterator_callHasChildren_error-detect.phpt",
    "ext/spl/tests/iterator_056-detect.phpt",
    "ext/spl/tests/SplFileInfo_getPerms_basic-detect.phpt",
    "ext/spl/tests/fileobject_setmaxlinelen_error002-detect.phpt",
    "ext/spl/tests/dllist_004-detect.phpt",
    "ext/spl/tests/bug36287-detect.phpt",
    "ext/spl/tests/SplFileObject_fpassthru_basic-detect.phpt",
    "ext/spl/tests/arrayObject_magicMethods6-detect.phpt",
    "ext/spl/tests/class_implements_basic2-detect.phpt",
    "ext/spl/tests/heap_010-detect.phpt",
    "ext/spl/tests/SplFileInfo_getExtension_basic-detect.phpt",
    "ext/spl/tests/arrayObject_ksort_basic1-detect.phpt",
    "ext/spl/tests/iterator_026-detect.phpt",
    "ext/spl/tests/recursiveiteratoriterator_getsubiterator_variation_003-detect.phpt",
    "ext/spl/tests/spl_iterator_caching_count_basic-detect.phpt",
    "ext/spl/tests/bug46160-detect.phpt",
    "ext/spl/tests/SplFixedArray_offsetExists_less_than_zero-detect.phpt",
    "ext/spl/tests/regexiterator_getpregflags-detect.phpt",
    "ext/spl/tests/iterator_007-detect.phpt",
    "ext/spl/tests/SplFixedArray_fromarray_param_multiarray-detect.phpt",
    "ext/spl/tests/RecursiveCallbackFilterIteratorTest-detect.phpt",
    "ext/spl/tests/array_009a-detect.phpt",
    "ext/spl/tests/SplFileObject_fscanf_basic-detect.phpt",
    "ext/spl/tests/heap_006-detect.phpt",
    "ext/spl/tests/arrayObject_setFlags_basic1-detect.phpt",
    "ext/spl/tests/fileobject_getbasename_basic-detect.phpt",
    "ext/spl/tests/iterator_003-detect.phpt",
    "ext/spl/tests/bug44144-detect.phpt",
    "ext/spl/tests/spl_002-detect.phpt",
    "ext/spl/tests/iterator_062-detect.phpt",
    "ext/spl/tests/iterator_015-detect.phpt",
    "ext/spl/tests/SplFileObject_getflags_error001-detect.phpt",
    "ext/spl/tests/spl_autoload_012-detect.phpt",
    "ext/spl/tests/SplFileObject_rewind_variation001-detect.phpt",
    "ext/spl/tests/iterator_021-detect.phpt",
    "ext/spl/tests/array_008-detect.phpt",
    "ext/spl/tests/dllist_memleak-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_offsetUnset_parameter-larger-num-elements-detect.phpt",
    "ext/spl/tests/fileobject_getfileinfo_basic-detect.phpt",
    "ext/spl/tests/iterator_064-detect.phpt",
    "ext/spl/tests/fileobject_004-detect.phpt",
    "ext/spl/tests/spl_001-detect.phpt",
    "ext/spl/tests/dllist_012-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_offsetExists_invalid_parameter-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_offsetGet_missing_param-detect.phpt",
    "ext/spl/tests/observer_004-detect.phpt",
    "ext/spl/tests/spl_priorityqeue_insert_two_params_error-detect.phpt",
    "ext/spl/tests/SplFileInfo_getGroup_basic-detect.phpt",
    "ext/spl/tests/SplFileObject_fputcsv_002-detect.phpt",
    "ext/spl/tests/spl_classes-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_isEmpty_empty-with-parameter-detect.phpt",
    "ext/spl/tests/SplStack_setIteratorMode-detect.phpt",
    "ext/spl/tests/bug54384-detect.phpt",
    "ext/spl/tests/iterator_027-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_getIteratorMode_error-detect.phpt",
    "ext/spl/tests/arrayObject_getFlags_basic1-detect.phpt",
    "ext/spl/tests/bug34548-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_count-detect.phpt",
    "ext/spl/tests/spl_cachingiterator_setFlags_basic-detect.phpt",
    "ext/spl/tests/iterator_020-detect.phpt",
    "ext/spl/tests/iterator_041b-detect.phpt",
    "ext/spl/tests/arrayObject_exchangeArray_basic3-detect.phpt",
    "ext/spl/tests/SplFileObject_setCsvControl_basic-detect.phpt",
    "ext/spl/tests/dllist_005-detect.phpt",
    "ext/spl/tests/bug54292-detect.phpt",
    "ext/spl/tests/SplDoublyLinkedList_offsetUnset_greater_than_elements-detect.phpt",
    "ext/spl/tests/SplObjectStorage_setInfo_empty_storage-detect.phpt",
    "ext/spl/examples/tests/dualiterator_001-detect.phpt",
    "ext/mysql/tests/mysql_fetch_row-detect.phpt",
    "ext/mysql/tests/mysql_field_len-detect.phpt",
    "ext/mysql/tests/mysql_info-detect.phpt",
    "ext/mysql/tests/mysql_escape_string-detect.phpt",
    "ext/mysql/tests/mysql_trace_mode-detect.phpt",
    "ext/mysql/tests/mysql_list_processes-detect.phpt",
    "ext/mysql/tests/mysql_fetch_object-detect.phpt",
    "ext/mysql/tests/bug51242-detect.phpt",
    "ext/mysql/tests/bug55473-detect.phpt",
    "ext/mysql/tests/mysql_affected_rows-detect.phpt",
    "ext/mysql/tests/mysql_mysqlnd_read_timeout_long-detect.phpt",
    "ext/mysql/tests/mysql_phpinfo-detect.phpt",
    "ext/mysql/tests/mysql_real_escape_string-detect.phpt",
    "ext/mysql/tests/mysql_fetch_lengths-detect.phpt",
    "ext/mysql/tests/mysql_pconn_reuse-detect.phpt",
    "ext/mysql/tests/001-detect.phpt",
    "ext/mysql/tests/mysql_data_seek-detect.phpt",
    "ext/mysql/tests/mysql_max_links-detect.phpt",
    "ext/mysql/tests/mysql_select_db-detect.phpt",
    "ext/mysql/tests/mysql_list_tables-detect.phpt",
    "ext/mysql/tests/mysql_query-detect.phpt",
    "ext/mysql/tests/mysql_fetch_field-detect.phpt",
    "ext/mysql/tests/mysql_fetch_array-detect.phpt",
    "ext/mysql/tests/mysql_max_persistent-detect.phpt",
    "ext/mysql/tests/mysql_insert_id-detect.phpt",
    "ext/mysql/tests/bug53649-detect.phpt",
    "ext/mysql/tests/003-detect.phpt",
    "ext/mysql/tests/mysql_fetch_assoc-detect.phpt",
    "ext/mysql/tests/mysql_ping-detect.phpt",
    "ext/mysql/tests/mysql_get_client_info-detect.phpt",
    "ext/mysql/tests/mysql_free_result-detect.phpt",
    "ext/mysql/tests/mysql_errno-detect.phpt",
    "ext/mysql/tests/mysql_pconnect-detect.phpt",
    "ext/mysql/tests/bug47438-detect.phpt",
    "ext/mysql/tests/mysql_thread_id-detect.phpt",
    "ext/mysql/tests/mysql_get_server_info-detect.phpt",
    "ext/mysql/tests/mysql_db_name-detect.phpt",
    "ext/mysql/tests/mysql_client_encoding-detect.phpt",
    "ext/mysql/tests/mysql_pconn_disable-detect.phpt",
    "ext/mysql/tests/mysql_result-detect.phpt",
    "ext/mysql/tests/mysql_unbuffered_query-detect.phpt",
    "ext/mysql/tests/mysql_get_host_info-detect.phpt",
    "ext/mysql/tests/mysql_deprecated_api-detect.phpt",
    "ext/mysql/tests/mysql_set_charset-detect.phpt",
    "ext/mysql/tests/mysql_error-detect.phpt",
    "ext/mysql/tests/002-detect.phpt",
    "ext/mysql/tests/mysql_pconn_kill-detect.phpt",
    "ext/mysql/tests/mysql_constants-detect.phpt",
    "ext/mysql/tests/mysql_query_load_data_openbasedir-detect.phpt",
    "ext/mysql/tests/mysql_field_table-detect.phpt",
    "ext/mysql/tests/mysql_stat-detect.phpt",
    "ext/mysql/tests/mysql_sql_safe_mode-detect.phpt",
    "ext/mysql/tests/mysql_field_type-detect.phpt",
    "ext/mysql/tests/mysql_pconn_max_links-detect.phpt",
    "ext/mysql/tests/mysql_list_dbs-detect.phpt",
    "ext/mysql/tests/mysql_num_rows-detect.phpt",
    "ext/mysql/tests/mysql_field_seek-detect.phpt",
    "ext/mysql/tests/mysql_connect-detect.phpt",
    "ext/mysql/tests/mysql_list_fields-detect.phpt",
    "ext/mysql/tests/mysql_drop_db-detect.phpt",
    "ext/mysql/tests/mysql_tablename-detect.phpt",
    "ext/mysql/tests/mysql_get_proto_info-detect.phpt",
    "ext/mysql/tests/mysql_close-detect.phpt",
    "ext/mysql/tests/mysql_num_fields-detect.phpt",
    "ext/mysql/tests/mysql_db_query-detect.phpt",
    "ext/mysql/tests/mysql_field_name-detect.phpt",
    "ext/mysql/tests/mysql_field_flags-detect.phpt",
    "ext/mysql/tests/bug48754-detect.phpt",
    "ext/mysql/tests/mysql_create_db-detect.phpt",
    "ext/pdo_pgsql/tests/large_objects-detect.phpt",
    "ext/pdo_pgsql/tests/bug46274_2-detect.phpt",
    "ext/pdo_pgsql/tests/is_in_transaction-detect.phpt",
    "ext/pdo_pgsql/tests/bug43925-detect.phpt",
    "ext/pdo_pgsql/tests/bug36727-detect.phpt",
    "ext/pdo_pgsql/tests/bug48764-detect.phpt",
    "ext/pdo_pgsql/tests/bug46274-detect.phpt",
    "ext/pdo_pgsql/tests/common-detect.phpt",
    "ext/pdo_pgsql/tests/bug_33876-detect.phpt",
    "ext/pdo_pgsql/tests/bug_49985-detect.phpt",
    "ext/pdo_pgsql/tests/copy_from-detect.phpt",
    "ext/pdo_pgsql/tests/bug61267-detect.phpt",
    "ext/pdo_pgsql/tests/copy_to-detect.phpt",
    "ext/session/tests/027-detect.phpt",
    "ext/session/tests/bug60634_error_2-detect.phpt",
    "ext/session/tests/rfc1867_sid_get-detect.phpt",
    "ext/session/tests/session_save_path_variation5-detect.phpt",
    "ext/session/tests/session_set_cookie_params_error-detect.phpt",
    "ext/session/tests/session_save_path_basic-detect.phpt",
    "ext/session/tests/rfc1867_invalid_settings_2-win-detect.phpt",
    "ext/session/tests/session_unset_error-detect.phpt",
    "ext/session/tests/rfc1867_sid_only_cookie_2-detect.phpt",
    "ext/session/tests/session_commit_variation1-detect.phpt",
    "ext/session/tests/session_module_name_variation2-detect.phpt",
    "ext/session/tests/bug26862-detect.phpt",
    "ext/session/tests/010-detect.phpt",
    "ext/session/tests/rfc1867-detect.phpt",
    "ext/session/tests/rfc1867_sid_cookie-detect.phpt",
    "ext/session/tests/session_encode_error2-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_005-detect.phpt",
    "ext/session/tests/005-detect.phpt",
    "ext/session/tests/session_start_variation1-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_002-detect.phpt",
    "ext/session/tests/rfc1867_disabled_2-detect.phpt",
    "ext/session/tests/session_encode_variation4-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_006-detect.phpt",
    "ext/session/tests/session_save_path_variation2-detect.phpt",
    "ext/session/tests/014-detect.phpt",
    "ext/session/tests/029-detect.phpt",
    "ext/session/tests/session_id_error4-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_012-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_001-detect.phpt",
    "ext/session/tests/011-detect.phpt",
    "ext/session/tests/session_encode_variation7-detect.phpt",
    "ext/session/tests/017-detect.phpt",
    "ext/session/tests/session_decode_variation3-detect.phpt",
    "ext/session/tests/session_set_save_handler_error-detect.phpt",
    "ext/session/tests/session_module_name_error-detect.phpt",
    "ext/session/tests/session_cache_limiter_variation2-detect.phpt",
    "ext/session/tests/session_encode_basic-detect.phpt",
    "ext/session/tests/session_module_name_variation3-detect.phpt",
    "ext/session/tests/sessionhandler_open_001-detect.phpt",
    "ext/session/tests/session_decode_basic-detect.phpt",
    "ext/session/tests/session_id_error-detect.phpt",
    "ext/session/tests/session_start_variation7-detect.phpt",
    "ext/session/tests/028-detect.phpt",
    "ext/session/tests/001-detect.phpt",
    "ext/session/tests/session_decode_error-detect.phpt",
    "ext/session/tests/session_set_save_handler_error2-detect.phpt",
    "ext/session/tests/015-detect.phpt",
    "ext/session/tests/session_encode_variation6-detect.phpt",
    "ext/session/tests/026-detect.phpt",
    "ext/session/tests/session_name_variation2-detect.phpt",
    "ext/session/tests/004-detect.phpt",
    "ext/session/tests/session_save_path_variation1-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_004-detect.phpt",
    "ext/session/tests/session_set_save_handler_closures-detect.phpt",
    "ext/session/tests/session_name_basic-detect.phpt",
    "ext/session/tests/session_id_error3-detect.phpt",
    "ext/session/tests/session_cache_limiter_basic-detect.phpt",
    "ext/session/tests/session_name_error-detect.phpt",
    "ext/session/tests/session_name_variation1-detect.phpt",
    "ext/session/tests/session_decode_variation2-detect.phpt",
    "ext/session/tests/session_status_disabled-detect.phpt",
    "ext/session/tests/session_set_cookie_params_variation1-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_015-detect.phpt",
    "ext/session/tests/session_id_error2-detect.phpt",
    "ext/session/tests/session_write_close_variation4-detect.phpt",
    "ext/session/tests/session_set_save_handler_basic-detect.phpt",
    "ext/session/tests/session_start_variation5-detect.phpt",
    "ext/session/tests/session_encode_error-detect.phpt",
    "ext/session/tests/session_set_save_handler_variation3-detect.phpt",
    "ext/session/tests/rfc1867_disabled-detect.phpt",
    "ext/session/tests/bug51338-detect.phpt",
    "ext/session/tests/session_cache_expire_variation2-detect.phpt",
    "ext/session/tests/session_save_path_error-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_003-detect.phpt",
    "ext/session/tests/bug60634_error_5-detect.phpt",
    "ext/session/tests/rfc1867_sid_invalid-detect.phpt",
    "ext/session/tests/bug24592-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_008-detect.phpt",
    "ext/session/tests/020-detect.phpt",
    "ext/session/tests/rfc1867_invalid_settings-detect.phpt",
    "ext/session/tests/022-detect.phpt",
    "ext/session/tests/session_regenerate_id_error-detect.phpt",
    "ext/session/tests/006-detect.phpt",
    "ext/session/tests/031-detect.phpt",
    "ext/session/tests/bug41600-detect.phpt",
    "ext/session/tests/session_cache_expire_variation1-detect.phpt",
    "ext/session/tests/session_commit_variation4-detect.phpt",
    "ext/session/tests/023-detect.phpt",
    "ext/session/tests/session_cache_limiter_variation3-detect.phpt",
    "ext/session/tests/bug60860-detect.phpt",
    "ext/session/tests/bug60634_error_3-detect.phpt",
    "ext/session/tests/session_cache_expire_variation3-detect.phpt",
    "ext/session/tests/session_destroy_variation3-detect.phpt",
    "ext/session/tests/session_unset_variation1-detect.phpt",
    "ext/session/tests/bug38377-detect.phpt",
    "ext/session/tests/session_commit_error-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_014-detect.phpt",
    "ext/session/tests/session_write_close_variation1-detect.phpt",
    "ext/session/tests/rfc1867_sid_get_2-detect.phpt",
    "ext/session/tests/rfc1867_invalid_settings-win-detect.phpt",
    "ext/session/tests/session_set_cookie_params_variation3-detect.phpt",
    "ext/session/tests/session_set_save_handler_iface_001-detect.phpt",
    "ext/session/tests/bug60634_error_4-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_013-detect.phpt",
    "ext/session/tests/session_write_close_variation3-detect.phpt",
    "ext/session/tests/bug60634-detect.phpt",
    "ext/session/tests/session_cache_limiter_variation1-detect.phpt",
    "ext/session/tests/session_start_variation9-detect.phpt",
    "ext/session/tests/session_commit_variation3-detect.phpt",
    "ext/session/tests/session_encode_variation8-detect.phpt",
    "ext/session/tests/session_cache_expire_error-detect.phpt",
    "ext/session/tests/bug42596-detect.phpt",
    "ext/session/tests/session_decode_variation1-detect.phpt",
    "ext/session/tests/session_set_cookie_params_variation4-detect.phpt",
    "ext/session/tests/session_decode_variation4-detect.phpt",
    "ext/session/tests/session_start_variation3-detect.phpt",
    "ext/session/tests/session_encode_variation3-detect.phpt",
    "ext/session/tests/002-detect.phpt",
    "ext/session/tests/session_commit_variation2-detect.phpt",
    "ext/session/tests/session_unset_basic-detect.phpt",
    "ext/session/tests/session_status-detect.phpt",
    "ext/session/tests/session_save_path_variation4-detect.phpt",
    "ext/session/tests/018-detect.phpt",
    "ext/session/tests/session_save_path_variation3-detect.phpt",
    "ext/session/tests/session_encode_variation5-detect.phpt",
    "ext/session/tests/session_start_variation2-detect.phpt",
    "ext/session/tests/rfc1867_sid_only_cookie-detect.phpt",
    "ext/session/tests/session_module_name_variation1-detect.phpt",
    "ext/session/tests/session_start_variation4-detect.phpt",
    "ext/session/tests/013-detect.phpt",
    "ext/session/tests/bug36459-detect.phpt",
    "ext/session/tests/016-detect.phpt",
    "ext/session/tests/bug32330-detect.phpt",
    "ext/session/tests/session_get_cookie_params_error-detect.phpt",
    "ext/session/tests/rfc1867_cleanup-detect.phpt",
    "ext/session/tests/rfc1867_inter-detect.phpt",
    "ext/session/tests/session_set_save_handler_variation2-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_010-detect.phpt",
    "ext/session/tests/025-detect.phpt",
    "ext/session/tests/session_id_variation1-detect.phpt",
    "ext/session/tests/session_set_save_handler_variation1-detect.phpt",
    "ext/session/tests/012-detect.phpt",
    "ext/session/tests/session_module_name_basic-detect.phpt",
    "ext/session/tests/session_start_error-detect.phpt",
    "ext/session/tests/rfc1867_no_name-detect.phpt",
    "ext/session/tests/session_set_cookie_params_variation2-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_011-detect.phpt",
    "ext/session/tests/021-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_007-detect.phpt",
    "ext/session/tests/bug53141-detect.phpt",
    "ext/session/tests/session_cache_limiter_error-detect.phpt",
    "ext/session/tests/bug60634_error_1-detect.phpt",
    "ext/session/tests/session_set_save_handler_variation4-detect.phpt",
    "ext/session/tests/session_id_basic-detect.phpt",
    "ext/session/tests/bug55688-detect.phpt",
    "ext/session/tests/session_destroy_variation2-detect.phpt",
    "ext/session/tests/bug61728-detect.phpt",
    "ext/session/tests/session_start_variation6-detect.phpt",
    "ext/session/tests/session_destroy_error-detect.phpt",
    "ext/session/tests/rfc1867_invalid_settings_2-detect.phpt",
    "ext/session/tests/030-detect.phpt",
    "ext/session/tests/session_write_close_basic-detect.phpt",
    "ext/session/tests/session_destroy_variation1-detect.phpt",
    "ext/session/tests/bug31454-detect.phpt",
    "ext/session/tests/session_set_save_handler_error3-detect.phpt",
    "ext/session/tests/session_set_save_handler_class_009-detect.phpt",
    "ext/session/tests/session_start_variation8-detect.phpt",
    "ext/session/tests/session_id_variation2-detect.phpt",
    "ext/session/tests/session_regenerate_id_basic-detect.phpt",
    "ext/session/tests/rfc1867_sid_post-detect.phpt",
    "ext/session/tests/session_encode_variation1-detect.phpt",
    "ext/session/tests/019-detect.phpt",
    "ext/session/tests/session_module_name_variation4-detect.phpt",
    "ext/session/tests/session_cache_expire_basic-detect.phpt",
    "ext/session/tests/session_get_cookie_params_basic-detect.phpt",
    "ext/session/tests/session_set_cookie_params_variation5-detect.phpt",
    "ext/session/tests/024-detect.phpt",
    "ext/session/tests/009-detect.phpt",
    "ext/session/tests/session_decode_error2-detect.phpt",
    "ext/session/tests/session_write_close_error-detect.phpt",
    "ext/session/tests/session_regenerate_id_variation1-detect.phpt",
    "ext/session/tests/session_write_close_variation2-detect.phpt",
    "ext/session/tests/session_commit_basic-detect.phpt",
    "ext/session/tests/session_encode_variation2-detect.phpt",
    "ext/session/tests/session_set_save_handler_error4-detect.phpt",
    "ext/session/tests/session_get_cookie_params_variation1-detect.phpt",
    "ext/session/tests/session_set_save_handler_iface_002-detect.phpt",
    "ext/session/tests/session_set_cookie_params_basic-detect.phpt",
    "ext/hash/tests/md4-detect.phpt",
    "ext/hash/tests/whirlpool-detect.phpt",
    "ext/hash/tests/sha1-detect.phpt",
    "ext/hash/tests/fnv132-detect.phpt",
    "ext/hash/tests/hash_file_error-detect.phpt",
    "ext/hash/tests/ripemd160-detect.phpt",
    "ext/hash/tests/md2-detect.phpt",
    "ext/hash/tests/sha384-detect.phpt",
    "ext/hash/tests/gost-detect.phpt",
    "ext/hash/tests/hash_hmac_error-detect.phpt",
    "ext/hash/tests/hash_hmac_file_basic-detect.phpt",
    "ext/hash/tests/bug52240-detect.phpt",
    "ext/hash/tests/tiger-detect.phpt",
    "ext/hash/tests/hash_hmac_file_error-detect.phpt",
    "ext/hash/tests/ripemd256-detect.phpt",
    "ext/hash/tests/hash_error-detect.phpt",
    "ext/hash/tests/haval-detect.phpt",
    "ext/hash/tests/fnv164-detect.phpt",
    "ext/hash/tests/hash_file_basic1-detect.phpt",
    "ext/hash/tests/hash_file_basic-detect.phpt",
    "ext/hash/tests/mhash_003-detect.phpt",
    "ext/hash/tests/hmac-md5-detect.phpt",
    "ext/hash/tests/mhash_002-detect.phpt",
    "ext/hash/tests/joaat-detect.phpt",
    "ext/hash/tests/hash_algos-detect.phpt",
    "ext/hash/tests/mhash_004-detect.phpt",
    "ext/hash/tests/hash_copy_001-detect.phpt",
    "ext/hash/tests/sha512-detect.phpt",
    "ext/hash/tests/md5-detect.phpt",
    "ext/hash/tests/snefru-detect.phpt",
    "ext/hash/tests/hash_hmac_basic-detect.phpt",
    "ext/hash/tests/crc32-detect.phpt",
    "ext/hash/tests/ripemd128-detect.phpt",
    "ext/hash/tests/adler32-detect.phpt",
    "ext/hash/tests/mhash_001-detect.phpt",
    "ext/hash/tests/ripemd320-detect.phpt",
    "ext/hash/tests/sha224-detect.phpt",
    "ext/hash/tests/hash_copy_002-detect.phpt",
    "ext/hash/tests/sha256-detect.phpt",
    "ext/pdo_dblib/tests/bug_38955-detect.phpt",
    "ext/pdo_dblib/tests/bug_47588-detect.phpt",
    "ext/pdo_dblib/tests/bug_45876-detect.phpt",
    "ext/pdo_dblib/tests/bug_50755-detect.phpt",
    "ext/mcrypt/tests/mcrypt_ofb-detect.phpt",
    "ext/mcrypt/tests/mcrypt_decrypt_error-detect.phpt",
    "ext/mcrypt/tests/mcrypt_list_modes-detect.phpt",
    "ext/mcrypt/tests/bug35496-detect.phpt",
    "ext/mcrypt/tests/mcrypt_module_is_block_algorithm-detect.phpt",
    "ext/mcrypt/tests/bug8040-detect.phpt",
    "ext/mcrypt/tests/mcrypt_enc_get_iv_size-detect.phpt",
    "ext/mcrypt/tests/mcrypt_encrypt_variation2-detect.phpt",
    "ext/mcrypt/tests/mcrypt_module_get_algo_key_size-detect.phpt",
    "ext/mcrypt/tests/mcrypt_ecb_variation5-detect.phpt",
    "ext/mcrypt/tests/mcrypt_enc_is_block_algorithm-detect.phpt",
    "ext/mcrypt/tests/mcrypt_get_key_size-detect.phpt",
    "ext/mcrypt/tests/mcrypt_ecb_3des_encrypt-detect.phpt",
    "ext/mcrypt/tests/mcrypt_rijndael128_128BitKey-detect.phpt",
    "ext/mcrypt/tests/mcrypt_module_get_supported_key_sizes-detect.phpt",
    "ext/mcrypt/tests/mcrypt_module_is_block_mode-detect.phpt",
    "ext/mcrypt/tests/mcrypt_ecb_variation2-detect.phpt",
    "ext/mcrypt/tests/mcrypt_get_cipher_name-detect.phpt",
    "ext/mcrypt/tests/mcrypt_cbc_variation5-detect.phpt",
    "ext/mcrypt/tests/mcrypt_encrypt_variation3-detect.phpt",
    "ext/mcrypt/tests/mcrypt_module_open-detect.phpt",
    "ext/mcrypt/tests/mcrypt_encrypt_variation5-detect.phpt",
    "ext/mcrypt/tests/mcrypt_enc_get_supported_key_sizes-detect.phpt",
    "ext/mcrypt/tests/mcrypt_ecb_variation3-detect.phpt",
    "ext/mcrypt/tests/mcrypt_encrypt_variation1-detect.phpt",
    "ext/mcrypt/tests/mcrypt_get_iv_size-detect.phpt",
    "ext/mcrypt/tests/mcrypt_cbc_variation2-detect.phpt",
    "ext/mcrypt/tests/mcrypt_enc_get_key_size-detect.phpt",
    "ext/mcrypt/tests/mcrypt_ecb_error-detect.phpt",
    "ext/mcrypt/tests/mcrypt_cbc_3des_encrypt-detect.phpt",
    "ext/mcrypt/tests/mcrypt_decrypt_variation5-detect.phpt",
    "ext/mcrypt/tests/mcrypt_ecb_variation1-detect.phpt",
    "ext/mcrypt/tests/mcrypt_list_algorithms-detect.phpt",
    "ext/mcrypt/tests/mcrypt_decrypt_variation4-detect.phpt",
    "ext/mcrypt/tests/mcrypt_module_self_test-detect.phpt",
    "ext/mcrypt/tests/mcrypt_decrypt-detect.phpt",
    "ext/mcrypt/tests/mcrypt_module_is_block_algorithm_mode-detect.phpt",
    "ext/mcrypt/tests/mcrypt_cbc_3des_decrypt-detect.phpt",
    "ext/mcrypt/tests/mcrypt_cbc_error-detect.phpt",
    "ext/mcrypt/tests/mcrypt_ecb_3des_decrypt-detect.phpt",
    "ext/mcrypt/tests/bug55169-detect.phpt",
    "ext/mcrypt/tests/blowfish-detect.phpt",
    "ext/mcrypt/tests/mcrypt_enc_get_block_size-detect.phpt",
    "ext/mcrypt/tests/mcrypt_enc_get_mode_name-detect.phpt",
    "ext/mcrypt/tests/mcrypt_rijndael128_256BitKey-detect.phpt",
    "ext/mcrypt/tests/mcrypt_decrypt_variation1-detect.phpt",
    "ext/mcrypt/tests/mcrypt_module_get_algo_block_size-detect.phpt",
    "ext/mcrypt/tests/mcrypt_ecb_variation4-detect.phpt",
    "ext/mcrypt/tests/bug43143-detect.phpt",
    "ext/mcrypt/tests/mcrypt_decrypt_3des_ecb-detect.phpt",
    "ext/mcrypt/tests/mcrypt_create_iv-detect.phpt",
    "ext/mcrypt/tests/mcrypt_cbc_variation3-detect.phpt",
    "ext/mcrypt/tests/mcrypt_enc_get_algorithms_name-detect.phpt",
    "ext/mcrypt/tests/mcrypt_enc_is_block_algorithm_mode-detect.phpt",
    "ext/mcrypt/tests/mcrypt_cbc_variation4-detect.phpt",
    "ext/mcrypt/tests/mcrypt_enc_is_block_mode-detect.phpt",
    "ext/mcrypt/tests/mcrypt_cbc-detect.phpt",
    "ext/mcrypt/tests/mcrypt_cfb-detect.phpt",
    "ext/mcrypt/tests/bug46010-detect.phpt",
    "ext/mcrypt/tests/mcrypt_encrypt_3des_ecb-detect.phpt",
    "ext/mcrypt/tests/mcrypt_encrypt_error-detect.phpt",
    "ext/mcrypt/tests/bug49738-detect.phpt",
    "ext/mcrypt/tests/mcrypt_encrypt_3des_cbc-detect.phpt",
    "ext/mcrypt/tests/mcrypt_ecb-detect.phpt",
    "ext/mcrypt/tests/mcrypt_encrypt_variation4-detect.phpt",
    "ext/mcrypt/tests/mcrypt_cbc_variation1-detect.phpt",
    "ext/mcrypt/tests/mcrypt_enc_self_test-detect.phpt",
    "ext/mcrypt/tests/mcrypt_filters-detect.phpt",
    "ext/mcrypt/tests/mcrypt_decrypt_3des_cbc-detect.phpt",
    "ext/mcrypt/tests/mcrypt_decrypt_variation2-detect.phpt",
    "ext/mcrypt/tests/mcrypt_get_block_size-detect.phpt",
    "ext/mcrypt/tests/mcrypt_decrypt_variation3-detect.phpt",
    "ext/mcrypt/tests/bug37595-detect.phpt",
    "ext/mcrypt/tests/bug41252-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-04-detect.phpt",
    "ext/mbstring/tests/mb_stristr_variation1-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_pattern-12-detect.phpt",
    "ext/mbstring/tests/mb_stripos_variation1-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-12-detect.phpt",
    "ext/mbstring/tests/mb_eregi-detect.phpt",
    "ext/mbstring/tests/overload02-detect.phpt",
    "ext/mbstring/tests/mb_strrchr_error2-detect.phpt",
    "ext/mbstring/tests/mb_strrchr_variation6-detect.phpt",
    "ext/mbstring/tests/mb_strstr_basic-detect.phpt",
    "ext/mbstring/tests/bug25140-detect.phpt",
    "ext/mbstring/tests/mb_convert_encoding-detect.phpt",
    "ext/mbstring/tests/bug54494-detect.phpt",
    "ext/mbstring/tests/mb_substr_count_variation4-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace_variation4-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-11-detect.phpt",
    "ext/mbstring/tests/mb_ereg_variation3-detect.phpt",
    "ext/mbstring/tests/mb_regex_encoding_variation1-detect.phpt",
    "ext/mbstring/tests/mb_internal_encoding_ini_basic2-detect.phpt",
    "ext/mbstring/tests/mb_substr_variation1-detect.phpt",
    "ext/mbstring/tests/mb_stristr_variation5-detect.phpt",
    "ext/mbstring/tests/mb_parse_str-detect.phpt",
    "ext/mbstring/tests/bug31911-detect.phpt",
    "ext/mbstring/tests/mb_strstr_error1-detect.phpt",
    "ext/mbstring/tests/mb_strtolower_error1-detect.phpt",
    "ext/mbstring/tests/mb_decode_mimeheader_variation2-detect.phpt",
    "ext/mbstring/tests/mb_substitute_character_variation2-detect.phpt",
    "ext/mbstring/tests/mb_stristr_error2-detect.phpt",
    "ext/mbstring/tests/mb_strrichr_variation5-detect.phpt",
    "ext/mbstring/tests/mb_substitute_character-detect.phpt",
    "ext/mbstring/tests/mb_substitute_character_basic-detect.phpt",
    "ext/mbstring/tests/mb_ereg_error-detect.phpt",
    "ext/mbstring/tests/mb_strpos_basic-detect.phpt",
    "ext/mbstring/tests/mb_substr_count_basic-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_pattern-01-detect.phpt",
    "ext/mbstring/tests/mb_strtolower_error2-detect.phpt",
    "ext/mbstring/tests/bug52981-detect.phpt",
    "ext/mbstring/tests/mb_internal_encoding_variation2-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-compat-08-detect.phpt",
    "ext/mbstring/tests/mb_encode_mimeheader_basic2-detect.phpt",
    "ext/mbstring/tests/mb_internal_encoding_error2-detect.phpt",
    "ext/mbstring/tests/mb_send_mail01-detect.phpt",
    "ext/mbstring/tests/mb_split_error-detect.phpt",
    "ext/mbstring/tests/php_gr_jp_dev_884-1-detect.phpt",
    "ext/mbstring/tests/mb_substr_count-detect.phpt",
    "ext/mbstring/tests/mb_strrichr_error2-detect.phpt",
    "ext/mbstring/tests/mb_substr_variation7-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-detect.phpt",
    "ext/mbstring/tests/mb_internal_encoding_error1-detect.phpt",
    "ext/mbstring/tests/mb_ereg-detect.phpt",
    "ext/mbstring/tests/mb_regex_set_options-detect.phpt",
    "ext/mbstring/tests/mb_stripos_variation5_Bug45923-detect.phpt",
    "ext/mbstring/tests/mb_send_mail03-detect.phpt",
    "ext/mbstring/tests/mb_str_functions_opt-parameter-detect.phpt",
    "ext/mbstring/tests/mb_strrichr_variation3-detect.phpt",
    "ext/mbstring/tests/mb_strrchr_variation4-detect.phpt",
    "ext/mbstring/tests/mb_strrichr_error1-detect.phpt",
    "ext/mbstring/tests/mb_substr_count_error1-detect.phpt",
    "ext/mbstring/tests/mb_stristr_variation4-detect.phpt",
    "ext/mbstring/tests/mb_strpos_variation5-detect.phpt",
    "ext/mbstring/tests/mb_strlen_error1-detect.phpt",
    "ext/mbstring/tests/ini_language-detect.phpt",
    "ext/mbstring/tests/mb_substr_count_variation1-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_pattern-04-detect.phpt",
    "ext/mbstring/tests/mb_substr_count_variation2-detect.phpt",
    "ext/mbstring/tests/mb_strtolower_variation1-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace_variation1-detect.phpt",
    "ext/mbstring/tests/mb_ereg3-detect.phpt",
    "ext/mbstring/tests/mb_strlen_basic-detect.phpt",
    "ext/mbstring/tests/mb_strripos_variation2-detect.phpt",
    "ext/mbstring/tests/bug43993-detect.phpt",
    "ext/mbstring/tests/mb_convert_kana-detect.phpt",
    "ext/mbstring/tests/mb_send_mail02-detect.phpt",
    "ext/mbstring/tests/mb_strripos_basic-detect.phpt",
    "ext/mbstring/tests/mb_stristr_basic-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace_callback-detect.phpt",
    "ext/mbstring/tests/mb_decode_mimeheader_variation1-detect.phpt",
    "ext/mbstring/tests/mb_stripos_error2-detect.phpt",
    "ext/mbstring/tests/mb_ereg_variation2-detect.phpt",
    "ext/mbstring/tests/mb_strlen_variation2-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-03-detect.phpt",
    "ext/mbstring/tests/mb_strpos_error1-detect.phpt",
    "ext/mbstring/tests/mb_send_mail07-detect.phpt",
    "ext/mbstring/tests/mb_strrchr_basic-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-compat-05-detect.phpt",
    "ext/mbstring/tests/mb_strstr_error2-detect.phpt",
    "ext/mbstring/tests/mb_internal_encoding_basic-detect.phpt",
    "ext/mbstring/tests/mb_ereg_search_pos-detect.phpt",
    "ext/mbstring/tests/mb_strripos_variation3_Bug45923-detect.phpt",
    "ext/mbstring/tests/bug46843-detect.phpt",
    "ext/mbstring/tests/mb_strtoupper_variation2-detect.phpt",
    "ext/mbstring/tests/mb_detect_encoding-detect.phpt",
    "ext/mbstring/tests/bug20087-detect.phpt",
    "ext/mbstring/tests/mb_ereg_search_regs-detect.phpt",
    "ext/mbstring/tests/mb_eregi_replace-detect.phpt",
    "ext/mbstring/tests/mb_strpos_error2-detect.phpt",
    "ext/mbstring/tests/mb_ereg_match_error1-detect.phpt",
    "ext/mbstring/tests/bug52931-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-02-detect.phpt",
    "ext/mbstring/tests/mb_ereg_variation7-detect.phpt",
    "ext/mbstring/tests/mb_ereg-compat-01-detect.phpt",
    "ext/mbstring/tests/mb_substr_variation3-detect.phpt",
    "ext/mbstring/tests/mb_strtolower_basic-detect.phpt",
    "ext/mbstring/tests/bug48645-detect.phpt",
    "ext/mbstring/tests/php_gr_jp_16242-detect.phpt",
    "ext/mbstring/tests/bug28220-detect.phpt",
    "ext/mbstring/tests/mb_substitute_character_variation1-detect.phpt",
    "ext/mbstring/tests/mb_strripos_error1-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace_variation3-detect.phpt",
    "ext/mbstring/tests/mb_substr_variation2-detect.phpt",
    "ext/mbstring/tests/mb_split_variation3-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-08-detect.phpt",
    "ext/mbstring/tests/mb_strrchr_variation1-detect.phpt",
    "ext/mbstring/tests/mb_substr_basic-detect.phpt",
    "ext/mbstring/tests/bug49354-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_shift_jis-detect.phpt",
    "ext/mbstring/tests/mb_substr_error2-detect.phpt",
    "ext/mbstring/tests/mb_strimwidth-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_pattern-10-detect.phpt",
    "ext/mbstring/tests/mb_encode_mimeheader_variation5-detect.phpt",
    "ext/mbstring/tests/mb_strstr_variation1-detect.phpt",
    "ext/mbstring/tests/mb_substr_error1-detect.phpt",
    "ext/mbstring/tests/mb_stripos_variation2-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_runtime_ini_alteration-02-detect.phpt",
    "ext/mbstring/tests/mb_strstr_variation3-detect.phpt",
    "ext/mbstring/tests/mb_ereg_variation5-detect.phpt",
    "ext/mbstring/tests/bug49528-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace_basic-detect.phpt",
    "ext/mbstring/tests/mb_encode_mimeheader_variation3-detect.phpt",
    "ext/mbstring/tests/mb_strtolower_variation4-detect.phpt",
    "ext/mbstring/tests/bug43841-detect.phpt",
    "ext/mbstring/tests/mb_strlen_variation1-detect.phpt",
    "ext/mbstring/tests/mb_stripos_basic2-detect.phpt",
    "ext/mbstring/tests/bug30549-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-13-detect.phpt",
    "ext/mbstring/tests/mb_strtoupper_variation3-detect.phpt",
    "ext/mbstring/tests/mb_ereg_variation6-detect.phpt",
    "ext/mbstring/tests/mb_encode_mimeheader_variation1-detect.phpt",
    "ext/mbstring/tests/mb_ereg2-detect.phpt",
    "ext/mbstring/tests/mb_strrpos_variation4-detect.phpt",
    "ext/mbstring/tests/mb_strtoupper_error2-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_pattern-09-detect.phpt",
    "ext/mbstring/tests/mb_send_mail06-detect.phpt",
    "ext/mbstring/tests/mb_strripos_variation4-detect.phpt",
    "ext/mbstring/tests/bug52861-detect.phpt",
    "ext/mbstring/tests/mb_regex_encoding_variation2-detect.phpt",
    "ext/mbstring/tests/mb_substr_variation4-detect.phpt",
    "ext/mbstring/tests/bug43840-detect.phpt",
    "ext/mbstring/tests/mb_strripos_variation1-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_pattern-08-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-09-detect.phpt",
    "ext/mbstring/tests/mb_substr_variation6-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-compat-03-detect.phpt",
    "ext/mbstring/tests/mb_convert_encoding_basic-detect.phpt",
    "ext/mbstring/tests/mb_strrpos_variation3-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-compat-01-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-05-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace_variation2-detect.phpt",
    "ext/mbstring/tests/mb_encoding_aliases-detect.phpt",
    "ext/mbstring/tests/mb_substr_count_error2-detect.phpt",
    "ext/mbstring/tests/mb_parse_str02-detect.phpt",
    "ext/mbstring/tests/bug60306-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-compat-04-detect.phpt",
    "ext/mbstring/tests/mb_substr_variation5-detect.phpt",
    "ext/mbstring/tests/mb_strrpos_error2-detect.phpt",
    "ext/mbstring/tests/mb_regex_encoding_error2-detect.phpt",
    "ext/mbstring/tests/mb_strpos_variation4-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-compat-02-detect.phpt",
    "ext/mbstring/tests/mb_list_encodings-detect.phpt",
    "ext/mbstring/tests/mb_encode_mimeheader_basic3-detect.phpt",
    "ext/mbstring/tests/mb_preferred_mime_name-detect.phpt",
    "ext/mbstring/tests/bug47399-detect.phpt",
    "ext/mbstring/tests/mb_strtoupper_basic-detect.phpt",
    "ext/mbstring/tests/mb_strrchr_variation5-detect.phpt",
    "ext/mbstring/tests/mb_get_info-detect.phpt",
    "ext/mbstring/tests/mb_substitute_character_error-detect.phpt",
    "ext/mbstring/tests/mb_strstr-detect.phpt",
    "ext/mbstring/tests/mb_strrichr_variation2-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-compat-10-detect.phpt",
    "ext/mbstring/tests/mb_strrichr_variation4-detect.phpt",
    "ext/mbstring/tests/mb_ereg_basic-detect.phpt",
    "ext/mbstring/tests/mb_ereg-compat-02-detect.phpt",
    "ext/mbstring/tests/mb_ereg1-detect.phpt",
    "ext/mbstring/tests/mb_ereg_variation4-detect.phpt",
    "ext/mbstring/tests/bug45722-detect.phpt",
    "ext/mbstring/tests/mb_substr_count_variation3-detect.phpt",
    "ext/mbstring/tests/mb_convert_encoding_stateful-detect.phpt",
    "ext/mbstring/tests/mb_strpos_variation3-detect.phpt",
    "ext/mbstring/tests/mb_stristr_variation3-detect.phpt",
    "ext/mbstring/tests/mb_regex_encoding_basic-detect.phpt",
    "ext/mbstring/tests/mb_strpos-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-07-detect.phpt",
    "ext/mbstring/tests/empty_pattern-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_pattern-02-detect.phpt",
    "ext/mbstring/tests/mb_http_output-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_runtime_ini_alteration-01-detect.phpt",
    "ext/mbstring/tests/bug43998-detect.phpt",
    "ext/mbstring/tests/mb_ereg_search-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-10-detect.phpt",
    "ext/mbstring/tests/mb_strrpos_variation1-detect.phpt",
    "ext/mbstring/tests/mb_strtoupper_variation4-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-14-detect.phpt",
    "ext/mbstring/tests/simpletest-detect.phpt",
    "ext/mbstring/tests/mb_strtolower_variation2-detect.phpt",
    "ext/mbstring/tests/mb_stristr_error1-detect.phpt",
    "ext/mbstring/tests/mb_internal_encoding_variation1-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_euc_jp-detect.phpt",
    "ext/mbstring/tests/pictogram1-detect.phpt",
    "ext/mbstring/tests/mb_encode_numericentity-detect.phpt",
    "ext/mbstring/tests/php_gr_jp_10830-detect.phpt",
    "ext/mbstring/tests/mb_stripos_variation3-detect.phpt",
    "ext/mbstring/tests/mb_split-detect.phpt",
    "ext/mbstring/tests/mb_strtoupper_error1-detect.phpt",
    "ext/mbstring/tests/mb_strlen_error2-detect.phpt",
    "ext/mbstring/tests/bug48697-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-compat-11-detect.phpt",
    "ext/mbstring/tests/mb_internal_encoding-detect.phpt",
    "ext/mbstring/tests/mb_strlen-detect.phpt",
    "ext/mbstring/tests/mb_strcut_missing_boundary_check-detect.phpt",
    "ext/mbstring/tests/bug43994-detect.phpt",
    "ext/mbstring/tests/mb_strrchr_variation3-detect.phpt",
    "ext/mbstring/tests/mb_ereg_search_xxx-detect.phpt",
    "ext/mbstring/tests/bug46806-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-compat-13-detect.phpt",
    "ext/mbstring/tests/mb_http_input-detect.phpt",
    "ext/mbstring/tests/mb_strrichr_basic-detect.phpt",
    "ext/mbstring/tests/mb_strrpos_variation2-detect.phpt",
    "ext/mbstring/tests/bug49536-detect.phpt",
    "ext/mbstring/tests/mb_strpos_variation1-detect.phpt",
    "ext/mbstring/tests/htmlent-detect.phpt",
    "ext/mbstring/tests/illformed_utf_sequences-detect.phpt",
    "ext/mbstring/tests/mb_split-compat-01-detect.phpt",
    "ext/mbstring/tests/mb_convert_variables-detect.phpt",
    "ext/mbstring/tests/mb_encode_mimeheader_basic-detect.phpt",
    "ext/mbstring/tests/overload01-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_pattern-11-detect.phpt",
    "ext/mbstring/tests/mb_stripos_error1-detect.phpt",
    "ext/mbstring/tests/mb_split_variation2-detect.phpt",
    "ext/mbstring/tests/mb_eregi_invalid_arguments-detect.phpt",
    "ext/mbstring/tests/mb_encode_mimeheader_variation4-detect.phpt",
    "ext/mbstring/tests/mb_encode_mimeheader_indent-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-06-detect.phpt",
    "ext/mbstring/tests/bug40685-detect.phpt",
    "ext/mbstring/tests/mb_strrpos_basic-detect.phpt",
    "ext/mbstring/tests/bug26639-detect.phpt",
    "ext/mbstring/tests/mb_ereg_match_basic-detect.phpt",
    "ext/mbstring/tests/mb_stripos_variation4-detect.phpt",
    "ext/mbstring/tests/zend_multibyte-01-detect.phpt",
    "ext/mbstring/tests/mb_stripos-detect.phpt",
    "ext/mbstring/tests/mb_strstr_variation6-detect.phpt",
    "ext/mbstring/tests/mb_stristr_variation2-detect.phpt",
    "ext/mbstring/tests/mb_strripos_basic2-detect.phpt",
    "ext/mbstring/tests/mb_decode_numericentity-detect.phpt",
    "ext/mbstring/tests/mb_strwidth-detect.phpt",
    "ext/mbstring/tests/mb_strstr_variation5-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-compat-07-detect.phpt",
    "ext/mbstring/tests/mb_regex_encoding_error1-detect.phpt",
    "ext/mbstring/tests/mb_strrchr_variation2-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_pattern-03-detect.phpt",
    "ext/mbstring/tests/mb_encode_mimeheader_variation2-detect.phpt",
    "ext/mbstring/tests/mb_ereg4-detect.phpt",
    "ext/mbstring/tests/mb_strcut-detect.phpt",
    "ext/mbstring/tests/mb_decode_mimeheader_basic-detect.phpt",
    "ext/mbstring/tests/mb_decode_mimeheader_error-detect.phpt",
    "ext/mbstring/tests/mb_strripos_variation5_Bug45923-detect.phpt",
    "ext/mbstring/tests/mb_detect_order-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace_error-detect.phpt",
    "ext/mbstring/tests/mb_strstr_variation2-detect.phpt",
    "ext/mbstring/tests/casefold-detect.phpt",
    "ext/mbstring/tests/mb_strpos_variation2-detect.phpt",
    "ext/mbstring/tests/mb_stripos_basic-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_pattern-05-detect.phpt",
    "ext/mbstring/tests/mb_strstr_variation4-detect.phpt",
    "ext/mbstring/tests/mb_send_mail05-detect.phpt",
    "ext/mbstring/tests/mb_strrchr_error1-detect.phpt",
    "ext/mbstring/tests/mb_strrpos_variation5-detect.phpt",
    "ext/mbstring/tests/mb_strtolower_variation3-detect.phpt",
    "ext/mbstring/tests/mb_encode_mimeheader_variation6-detect.phpt",
    "ext/mbstring/tests/mb_send_mail04-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-compat-09-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-compat-12-detect.phpt",
    "ext/mbstring/tests/mb_strtoupper_variation1-detect.phpt",
    "ext/mbstring/tests/mb_ereg_variation1-detect.phpt",
    "ext/mbstring/tests/mb_strlen_variation3-detect.phpt",
    "ext/mbstring/tests/mb_strripos_error2-detect.phpt",
    "ext/mbstring/tests/php_gr_jp_dev_884-2-detect.phpt",
    "ext/mbstring/tests/bug45923-detect.phpt",
    "ext/mbstring/tests/mb_decode_mimeheader_variation3-detect.phpt",
    "ext/mbstring/tests/bug43301-detect.phpt",
    "ext/mbstring/tests/mb_substr-detect.phpt",
    "ext/mbstring/tests/bug45239-detect.phpt",
    "ext/mbstring/tests/mb_split_variation1-detect.phpt",
    "ext/mbstring/tests/mb_strrpos_error1-detect.phpt",
    "ext/mbstring/tests/mb_strrichr_variation1-detect.phpt",
    "ext/mbstring/tests/mb_ereg_replace-compat-06-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_pattern-06-detect.phpt",
    "ext/mbstring/tests/mb_output_handler_pattern-07-detect.phpt",
    "ext/mbstring/tests/mb_convert_encoding_variation1-detect.phpt",
    "ext/readline/tests/readline_write_history_001-detect.phpt",
    "ext/readline/tests/readline_add_history_001-detect.phpt",
    "ext/readline/tests/readline_clear_history_001-detect.phpt",
    "ext/readline/tests/readline_completion_function_001-detect.phpt",
    "ext/readline/tests/readline_list_history_001-detect.phpt",
    "ext/readline/tests/readline_read_history_001-detect.phpt",
    "ext/readline/tests/readline_callback_handler_remove_001-detect.phpt",
    "ext/readline/tests/readline_callback_handler_install_001-detect.phpt",
    "ext/readline/tests/readline_info_001-detect.phpt",
    "ext/curl/tests/curl_copy_handle_basic_001-detect.phpt",
    "ext/curl/tests/curl_multi_getcontent_basic3-detect.phpt",
    "ext/curl/tests/curl_multi_getcontent_error3-detect.phpt",
    "ext/curl/tests/bug52202-detect.phpt",
    "ext/curl/tests/curl_multi_select_basic1-detect.phpt",
    "ext/curl/tests/curl_basic_001-detect.phpt",
    "ext/curl/tests/curl_basic_006-detect.phpt",
    "ext/curl/tests/curl_setopt_CURLOPT_READFUNCTION-detect.phpt",
    "ext/curl/tests/bug46739-detect.phpt",
    "ext/curl/tests/curl_writeheader_callback-detect.phpt",
    "ext/curl/tests/curl_basic_007-detect.phpt",
    "ext/curl/tests/curl_setopt_error-detect.phpt",
    "ext/curl/tests/curl_setopt_array_basic-detect.phpt",
    "ext/curl/tests/curl_basic_018-detect.phpt",
    "ext/curl/tests/curl_copy_handle_basic_002-detect.phpt",
    "ext/curl/tests/bug46711-detect.phpt",
    "ext/curl/tests/bug27023-detect.phpt",
    "ext/curl/tests/curl_write_file-detect.phpt",
    "ext/curl/tests/curl_basic_015-detect.phpt",
    "ext/curl/tests/bug55767-detect.phpt",
    "ext/curl/tests/curl_basic_005-detect.phpt",
    "ext/curl/tests/curl_copy_handle_variation1-detect.phpt",
    "ext/curl/tests/curl_copy_handle_variation2-detect.phpt",
    "ext/curl/tests/curl_setopt_basic004-detect.phpt",
    "ext/curl/tests/curl_basic_004-detect.phpt",
    "ext/curl/tests/curl_version_variation1-detect.phpt",
    "ext/curl/tests/curl_file_deleted_before_curl_close-detect.phpt",
    "ext/curl/tests/bug48203-detect.phpt",
    "ext/curl/tests/curl_basic_020-detect.phpt",
    "ext/curl/tests/curl_basic_009-detect.phpt",
    "ext/curl/tests/curl_basic_002-detect.phpt",
    "ext/curl/tests/curl_multi_getcontent_error2-detect.phpt",
    "ext/curl/tests/curl_copy_handle_basic-detect.phpt",
    "ext/curl/tests/bug62839-detect.phpt",
    "ext/curl/tests/curl_basic_008-detect.phpt",
    "ext/curl/tests/curl_copy_handle_basic_008-detect.phpt",
    "ext/curl/tests/curl_basic_016-detect.phpt",
    "ext/curl/tests/curl_copy_handle_basic_004-detect.phpt",
    "ext/curl/tests/curl_multi_init_basic-detect.phpt",
    "ext/curl/tests/curl_multi_getcontent_error1-detect.phpt",
    "ext/curl/tests/curl_multi_getcontent_error4-detect.phpt",
    "ext/curl/tests/curl_copy_handle_basic_007-detect.phpt",
    "ext/curl/tests/curl_setopt_basic003-detect.phpt",
    "ext/curl/tests/curl_copy_handle_basic_006-detect.phpt",
    "ext/curl/tests/curl_basic_014-detect.phpt",
    "ext/curl/tests/curl_copy_handle_basic_005-detect.phpt",
    "ext/curl/tests/curl_basic_019-detect.phpt",
    "ext/curl/tests/curl_basic_017-detect.phpt",
    "ext/curl/tests/curl_setopt_CURLOPT_FOLLOWLOCATION_open_basedir-detect.phpt",
    "ext/curl/tests/curl_basic_012-detect.phpt",
    "ext/curl/tests/bug54798-detect.phpt",
    "ext/curl/tests/curl_CURLOPT_READDATA-detect.phpt",
    "ext/curl/tests/curl_write_callback-detect.phpt",
    "ext/curl/tests/curl_basic_010-detect.phpt",
    "ext/curl/tests/bug48514-detect.phpt",
    "ext/curl/tests/curl_ftp_pasv-detect.phpt",
    "ext/curl/tests/curl_setopt_basic002-detect.phpt",
    "ext/curl/tests/curl_basic_013-detect.phpt",
    "ext/curl/tests/curl_basic_011-detect.phpt",
    "ext/curl/tests/curl_basic_003-detect.phpt",
    "ext/curl/tests/bug52827-detect.phpt",
    "ext/curl/tests/curl_multi_close_basic-detect.phpt",
    "ext/curl/tests/curl_close_basic-detect.phpt",
    "ext/curl/tests/bug48207-detect.phpt",
    "ext/curl/tests/bug45161-detect.phpt",
    "ext/curl/tests/curl_write_stdout-detect.phpt",
    "ext/curl/tests/bug48203_multi-detect.phpt",
    "ext/curl/tests/bug61948-detect.phpt",
    "ext/curl/tests/curl_error_basic-detect.phpt",
    "ext/curl/tests/curl_basic_021-detect.phpt",
    "ext/curl/tests/curl_version_error-detect.phpt",
    "ext/curl/tests/bug61948-win32-detect.phpt",
    "ext/curl/tests/curl_write_return-detect.phpt",
    "ext/dba/tests/bug36436-detect.phpt",
    "ext/dba/tests/dba_qdbm-detect.phpt",
    "ext/dba/tests/dba_db4_006-detect.phpt",
    "ext/dba/tests/dba_ndbm-detect.phpt",
    "ext/dba/tests/dba_db4_011-detect.phpt",
    "ext/dba/tests/bug49125-detect.phpt",
    "ext/dba/tests/dba_db4_sync-detect.phpt",
    "ext/dba/tests/dba_db4_001-detect.phpt",
    "ext/dba/tests/dba_db4_004-detect.phpt",
    "ext/dba/tests/dba012-detect.phpt",
    "ext/dba/tests/dba_db4_013-detect.phpt",
    "ext/dba/tests/dba015-detect.phpt",
    "ext/dba/tests/dba006-detect.phpt",
    "ext/dba/tests/dba010-detect.phpt",
    "ext/dba/tests/dba_db4_017-detect.phpt",
    "ext/dba/tests/dba007-detect.phpt",
    "ext/dba/tests/bug38698-detect.phpt",
    "ext/dba/tests/dba_db4_018-detect.phpt",
    "ext/dba/tests/dba_db4_015-detect.phpt",
    "ext/dba/tests/dba_db4_handlers-detect.phpt",
    "ext/dba/tests/dba_db3-detect.phpt",
    "ext/dba/tests/dba_db4_010-detect.phpt",
    "ext/dba/tests/dba_cdb_read-detect.phpt",
    "ext/dba/tests/dba_gdbm-detect.phpt",
    "ext/dba/tests/dba_db4_007-detect.phpt",
    "ext/dba/tests/dba_inifile-detect.phpt",
    "ext/dba/tests/dba004-detect.phpt",
    "ext/dba/tests/dba_optimize-detect.phpt",
    "ext/dba/tests/dba_handlers-detect.phpt",
    "ext/dba/tests/dba_cdb_001-detect.phpt",
    "ext/dba/tests/dba_dbm-detect.phpt",
    "ext/dba/tests/dba_db1-detect.phpt",
    "ext/dba/tests/dba_db4_003-detect.phpt",
    "ext/dba/tests/dba_sync-detect.phpt",
    "ext/dba/tests/dba_db4_002-detect.phpt",
    "ext/dba/tests/dba_db4_016-detect.phpt",
    "ext/dba/tests/dba_db4_000-detect.phpt",
    "ext/dba/tests/dba_db4_012-detect.phpt",
    "ext/dba/tests/dba_db4_009-detect.phpt",
    "ext/dba/tests/bug48240-detect.phpt",
    "ext/dba/tests/dba002-detect.phpt",
    "ext/dba/tests/dba011-detect.phpt",
    "ext/dba/tests/dba_flatfile-detect.phpt",
    "ext/dba/tests/dba_db4_optimize-detect.phpt",
    "ext/dba/tests/dba_db4_008-detect.phpt",
    "ext/dba/tests/dba014-detect.phpt",
    "ext/dba/tests/dba_cdb_make-detect.phpt",
    "ext/dba/tests/dba005-detect.phpt",
    "ext/dba/tests/dba_tcadb-detect.phpt",
    "ext/dba/tests/dba016-detect.phpt",
    "ext/dba/tests/dba013-detect.phpt",
    "ext/dba/tests/dba_db4_014-detect.phpt",
    "ext/dba/tests/dba008-detect.phpt",
    "ext/dba/tests/dba_split-detect.phpt",
    "ext/dba/tests/dba_cdb-detect.phpt",
    "ext/dba/tests/dba001-detect.phpt",
    "ext/dba/tests/dba009-detect.phpt",
    "ext/dba/tests/dba_db2-detect.phpt",
    "ext/dba/tests/dba_db4_005-detect.phpt",
    "ext/dba/tests/dba003-detect.phpt",
    "ext/ftp/tests/005-detect.phpt",
    "ext/ftp/tests/ftp_chmod_basic-detect.phpt",
    "ext/ftp/tests/bug39583-2-detect.phpt",
    "ext/ftp/tests/ftp_exec_basic-detect.phpt",
    "ext/ftp/tests/001-detect.phpt",
    "ext/ftp/tests/ftp_fget_basic1-detect.phpt",
    "ext/ftp/tests/004-detect.phpt",
    "ext/ftp/tests/bug37799-detect.phpt",
    "ext/ftp/tests/ftp_get_basic-detect.phpt",
    "ext/ftp/tests/ftp_nb_fget_basic1-detect.phpt",
    "ext/ftp/tests/003-detect.phpt",
    "ext/ftp/tests/ftp_rmdir_basic-detect.phpt",
    "ext/ftp/tests/006-detect.phpt",
    "ext/ftp/tests/bug39583-detect.phpt",
    "ext/ftp/tests/ftp_fget_basic-detect.phpt",
    "ext/ftp/tests/ftp_nb_fget_basic2-detect.phpt",
    "ext/ftp/tests/ftp_rawlist_basic2-detect.phpt",
    "ext/ftp/tests/ftp_nb_fget_basic3-detect.phpt",
    "ext/ftp/tests/bug7216-detect.phpt",
    "ext/ftp/tests/002-detect.phpt",
    "ext/ftp/tests/bug7216-2-detect.phpt",
    "ext/ftp/tests/bug27809-detect.phpt",
    "ext/ftp/tests/ftp_alloc_basic1-detect.phpt",
    "ext/ftp/tests/ftp_mdtm_basic-detect.phpt",
    "ext/ftp/tests/ftp_fget_basic2-detect.phpt",
    "ext/ftp/tests/bug39458-2-detect.phpt",
    "ext/ftp/tests/ftp_alloc_basic2-detect.phpt",
    "ext/ftp/tests/bug39458-detect.phpt",
    "ext/ftp/tests/ftp_fget_basic3-detect.phpt",
    "ext/pdo_sqlite/tests/pdo_fetch_func_001-detect.phpt",
    "ext/pdo_sqlite/tests/bug46542-detect.phpt",
    "ext/pdo_sqlite/tests/debugdumpparams_001-detect.phpt",
    "ext/pdo_sqlite/tests/bug46139-detect.phpt",
    "ext/pdo_sqlite/tests/pdo_sqlite_transaction-detect.phpt",
    "ext/pdo_sqlite/tests/pdo_sqlite_createcollation-detect.phpt",
    "ext/pdo_sqlite/tests/pdo_sqlite_createaggregate_002-detect.phpt",
    "ext/pdo_sqlite/tests/bug33841-detect.phpt",
    "ext/pdo_sqlite/tests/bug44327_3-detect.phpt",
    "ext/pdo_sqlite/tests/pdo_sqlite_get_attribute-detect.phpt",
    "ext/pdo_sqlite/tests/common-detect.phpt",
    "ext/pdo_sqlite/tests/bug43831-detect.phpt",
    "ext/pdo_sqlite/tests/bug60104-detect.phpt",
    "ext/pdo_sqlite/tests/pdo_sqlite_lastinsertid-detect.phpt",
    "ext/pdo_sqlite/tests/bug52487-detect.phpt",
    "ext/pdo_sqlite/tests/pdo_sqlite_createaggregate-detect.phpt",
    "ext/pdo_sqlite/tests/pdo_sqlite_createfunction-detect.phpt",
    "ext/pdo_sqlite/tests/bug48773-detect.phpt",
    "ext/pdo_sqlite/tests/bug_42589-detect.phpt",
    "ext/pdo_sqlite/tests/bug44327_2-detect.phpt",
    "ext/pdo_sqlite/tests/bug50728-detect.phpt",
    "ext/pdo_sqlite/tests/bug35336-detect.phpt",
    "ext/openssl/tests/bug25614-detect.phpt",
    "ext/openssl/tests/bug54992-detect.phpt",
    "ext/openssl/tests/005-detect.phpt",
    "ext/openssl/tests/bug41353-detect.phpt",
    "ext/openssl/tests/014-detect.phpt",
    "ext/openssl/tests/011-detect.phpt",
    "ext/openssl/tests/017-detect.phpt",
    "ext/openssl/tests/bug61124-detect.phpt",
    "ext/openssl/tests/openssl_x509_parse_basic-detect.phpt",
    "ext/openssl/tests/001-detect.phpt",
    "ext/openssl/tests/openssl_encrypt_error-detect.phpt",
    "ext/openssl/tests/015-detect.phpt",
    "ext/openssl/tests/004-detect.phpt",
    "ext/openssl/tests/openssl_random_pseudo_bytes-detect.phpt",
    "ext/openssl/tests/sni_001-detect.phpt",
    "ext/openssl/tests/003-detect.phpt",
    "ext/openssl/tests/022-detect.phpt",
    "ext/openssl/tests/006-detect.phpt",
    "ext/openssl/tests/023-detect.phpt",
    "ext/openssl/tests/bug54060-detect.phpt",
    "ext/openssl/tests/007-detect.phpt",
    "ext/openssl/tests/openssl_digest_basic-detect.phpt",
    "ext/openssl/tests/bug47828-detect.phpt",
    "ext/openssl/tests/002-detect.phpt",
    "ext/openssl/tests/bug41033-detect.phpt",
    "ext/openssl/tests/008-detect.phpt",
    "ext/openssl/tests/018-detect.phpt",
    "ext/openssl/tests/bug36732-detect.phpt",
    "ext/openssl/tests/openssl_decrypt_error-detect.phpt",
    "ext/openssl/tests/bug38261-detect.phpt",
    "ext/openssl/tests/013-detect.phpt",
    "ext/openssl/tests/016-detect.phpt",
    "ext/openssl/tests/bug39217-detect.phpt",
    "ext/openssl/tests/bug37820-detect.phpt",
    "ext/openssl/tests/bug46127-detect.phpt",
    "ext/openssl/tests/025-detect.phpt",
    "ext/openssl/tests/012-detect.phpt",
    "ext/openssl/tests/021-detect.phpt",
    "ext/openssl/tests/bug55646-detect.phpt",
    "ext/openssl/tests/bug48182-detect.phpt",
    "ext/openssl/tests/openssl_encrypt_crash-detect.phpt",
    "ext/openssl/tests/openssl_x509_parse_basic_v9-detect.phpt",
    "ext/openssl/tests/bug54061-detect.phpt",
    "ext/openssl/tests/019-detect.phpt",
    "ext/openssl/tests/024-detect.phpt",
    "ext/openssl/tests/bug38255-detect.phpt",
    "ext/openssl/tests/009-detect.phpt",
    "ext/openssl/tests/bug28382-detect.phpt",
    "ext/pdo_firebird/tests/bug_48877-detect.phpt",
    "ext/pdo_firebird/tests/common-detect.phpt",
    "ext/pdo_firebird/tests/connect-detect.phpt",
    "ext/pdo_firebird/tests/bug_53280-detect.phpt",
    "ext/pdo_firebird/tests/rowCount-detect.phpt",
    "ext/pdo_firebird/tests/execute-detect.phpt",
    "ext/pdo_firebird/tests/bug_47415-detect.phpt",
    "ext/pdo_firebird/tests/ddl-detect.phpt",
    "ext/com_dotnet/tests/bug45280-detect.phpt",
    "ext/com_dotnet/tests/bug39596-detect.phpt",
    "ext/com_dotnet/tests/bug34272-detect.phpt",
    "ext/com_dotnet/tests/variants-detect.phpt",
    "ext/com_dotnet/tests/bug39606-detect.phpt",
    "ext/com_dotnet/tests/bug49192-detect.phpt",
    "ext/com_dotnet/tests/bug33386-detect.phpt",
    "ext/com_dotnet/tests/27974-detect.phpt",
    "ext/gd/tests/imagefilltoborder_error6-detect.phpt",
    "ext/gd/tests/imageinterlace_variation2-detect.phpt",
    "ext/gd/tests/imagestringup_basic-detect.phpt",
    "ext/gd/tests/imagechar_error4-detect.phpt",
    "ext/gd/tests/imagedashedline_basic-detect.phpt",
    "ext/gd/tests/imageellipse_error6-detect.phpt",
    "ext/gd/tests/imagestring_basic-detect.phpt",
    "ext/gd/tests/imagecreatetruecolor_error2-detect.phpt",
    "ext/gd/tests/bug48801-detect.phpt",
    "ext/gd/tests/bug48555-detect.phpt",
    "ext/gd/tests/bug24594-detect.phpt",
    "ext/gd/tests/imagerectangle_error8-detect.phpt",
    "ext/gd/tests/imagefilter_error9-detect.phpt",
    "ext/gd/tests/imagestring_error2-detect.phpt",
    "ext/gd/tests/imagefilter_error2-detect.phpt",
    "ext/gd/tests/imageinterlace_error2-detect.phpt",
    "ext/gd/tests/imagestring_error4-detect.phpt",
    "ext/gd/tests/imagefilter_error3-detect.phpt",
    "ext/gd/tests/libgd00086_extern-detect.phpt",
    "ext/gd/tests/imageellipse_error2-detect.phpt",
    "ext/gd/tests/crafted_gd2-detect.phpt",
    "ext/gd/tests/gif2jpg-detect.phpt",
    "ext/gd/tests/bug39508-detect.phpt",
    "ext/gd/tests/imagesetbrush_basic-detect.phpt",
    "ext/gd/tests/bug28147-detect.phpt",
    "ext/gd/tests/bug44849-detect.phpt",
    "ext/gd/tests/imagecolorallocatealpha_error4-detect.phpt",
    "ext/gd/tests/gd_info_error-detect.phpt",
    "ext/gd/tests/imageellipse_basic-detect.phpt",
    "ext/gd/tests/imagefilter_error15-detect.phpt",
    "ext/gd/tests/imagefilter-detect.phpt",
    "ext/gd/tests/imagetruecolortopalette_basic-detect.phpt",
    "ext/gd/tests/imagecolorclosesthwb-detect.phpt",
    "ext/gd/tests/imagecharup_error2-detect.phpt",
    "ext/gd/tests/imagestringup_error5-detect.phpt",
    "ext/gd/tests/imagechar_basic-detect.phpt",
    "ext/gd/tests/imagecolorallocatealpha_error1-detect.phpt",
    "ext/gd/tests/imagerectangle_error7-detect.phpt",
    "ext/gd/tests/imagearc_basic-detect.phpt",
    "ext/gd/tests/bug43073-detect.phpt",
    "ext/gd/tests/bug45799-detect.phpt",
    "ext/gd/tests/imagestring_error5-detect.phpt",
    "ext/gd/tests/createfromwbmp2-detect.phpt",
    "ext/gd/tests/imagefilter_error12-detect.phpt",
    "ext/gd/tests/bug37360-detect.phpt",
    "ext/gd/tests/bug38112-detect.phpt",
    "ext/gd/tests/imagerectangle_error6-detect.phpt",
    "ext/gd/tests/imagecolorallocatealpha_error2-detect.phpt",
    "ext/gd/tests/imageconvolution_error2-detect.phpt",
    "ext/gd/tests/imagegammacorrect_basic-detect.phpt",
    "ext/gd/tests/png2wbmp_error2-detect.phpt",
    "ext/gd/tests/imagecolordeallocate_basic-detect.phpt",
    "ext/gd/tests/xbm2png-detect.phpt",
    "ext/gd/tests/bug39366-detect.phpt",
    "ext/gd/tests/imageellipse_error7-detect.phpt",
    "ext/gd/tests/imagecolorallocate_variation4-detect.phpt",
    "ext/gd/tests/imagecolorset_basic-detect.phpt",
    "ext/gd/tests/bug43121-detect.phpt",
    "ext/gd/tests/bug38212-detect.phpt",
    "ext/gd/tests/imageellipse_error1-detect.phpt",
    "ext/gd/tests/bug19366-detect.phpt",
    "ext/gd/tests/imageellipse_error8-detect.phpt",
    "ext/gd/tests/imagefilter_error1-detect.phpt",
    "ext/gd/tests/bug39780_extern-detect.phpt",
    "ext/gd/tests/imageinterlace_basic-detect.phpt",
    "ext/gd/tests/types-detect.phpt",
    "ext/gd/tests/libgd00094-detect.phpt",
    "ext/gd/tests/001-detect.phpt",
    "ext/gd/tests/imagecolordeallocate_error3-detect.phpt",
    "ext/gd/tests/imagefilledarc_variation2-detect.phpt",
    "ext/gd/tests/imagestringup_error2-detect.phpt",
    "ext/gd/tests/imagecolormatch_basic-detect.phpt",
    "ext/gd/tests/imagearc_variation2-detect.phpt",
    "ext/gd/tests/libgd00100-detect.phpt",
    "ext/gd/tests/imagefilter_error10-detect.phpt",
    "ext/gd/tests/colorat-detect.phpt",
    "ext/gd/tests/bug48732-detect.phpt",
    "ext/gd/tests/bug36697-detect.phpt",
    "ext/gd/tests/imagefilledarc_basic-detect.phpt",
    "ext/gd/tests/imagestringup_error3-detect.phpt",
    "ext/gd/tests/imagefilter_error4-detect.phpt",
    "ext/gd/tests/imagetruecolortopalette_error4-detect.phpt",
    "ext/gd/tests/imagesetthickness_error2-detect.phpt",
    "ext/gd/tests/imagelayereffect_error3-detect.phpt",
    "ext/gd/tests/colorexact-detect.phpt",
    "ext/gd/tests/imagefilter_error13-detect.phpt",
    "ext/gd/tests/imagecharup_basic-detect.phpt",
    "ext/gd/tests/imagefilter_error16-detect.phpt",
    "ext/gd/tests/bug51671-detect.phpt",
    "ext/gd/tests/imagecolordeallocate_error4-detect.phpt",
    "ext/gd/tests/libgd00086-detect.phpt",
    "ext/gd/tests/bug49600-detect.phpt",
    "ext/gd/tests/imageistruecolor_error1-detect.phpt",
    "ext/gd/tests/imageconvolution_basic-detect.phpt",
    "ext/gd/tests/gd_info_variation1-detect.phpt",
    "ext/gd/tests/bug39273-detect.phpt",
    "ext/gd/tests/copyresized-detect.phpt",
    "ext/gd/tests/imagefttext-detect.phpt",
    "ext/gd/tests/imagefilter_error20-detect.phpt",
    "ext/gd/tests/imagefilltoborder_basic-detect.phpt",
    "ext/gd/tests/imagelayereffect_error1-detect.phpt",
    "ext/gd/tests/imagefontwidth_basic-detect.phpt",
    "ext/gd/tests/bug39780-detect.phpt",
    "ext/gd/tests/imageantialias_error2-detect.phpt",
    "ext/gd/tests/jpeg2wbmp_error3-detect.phpt",
    "ext/gd/tests/imageellipse_error5-detect.phpt",
    "ext/gd/tests/imagecolorstotal_basic-detect.phpt",
    "ext/gd/tests/createfromwbmp-detect.phpt",
    "ext/gd/tests/imagechar_error1-detect.phpt",
    "ext/gd/tests/imagerectangle_error2-detect.phpt",
    "ext/gd/tests/imagerectangle_error1-detect.phpt",
    "ext/gd/tests/bug39286-detect.phpt",
    "ext/gd/tests/imagecolorallocate_variation1-detect.phpt",
    "ext/gd/tests/imagestring_error6-detect.phpt",
    "ext/gd/tests/imagecolordeallocate_error2-detect.phpt",
    "ext/gd/tests/imagefilter_error7-detect.phpt",
    "ext/gd/tests/imageellipse_error3-detect.phpt",
    "ext/gd/tests/imagecharup_error5-detect.phpt",
    "ext/gd/tests/imagestringup_error4-detect.phpt",
    "ext/gd/tests/imagefilltoborder_error5-detect.phpt",
    "ext/gd/tests/imagelayereffect_basic-detect.phpt",
    "ext/gd/tests/imagepolygon_negative-detect.phpt",
    "ext/gd/tests/imagecolorallocate_variation3-detect.phpt",
    "ext/gd/tests/imagegammacorrect_error1-detect.phpt",
    "ext/gd/tests/lines-detect.phpt",
    "ext/gd/tests/imagecopyresampled_basic-detect.phpt",
    "ext/gd/tests/imagecharup_error3-detect.phpt",
    "ext/gd/tests/jpeg2png-detect.phpt",
    "ext/gd/tests/imagechar_error6-detect.phpt",
    "ext/gd/tests/png2wbmp_error3-detect.phpt",
    "ext/gd/tests/imagecharup_error4-detect.phpt",
    "ext/gd/tests/imagechar_error5-detect.phpt",
    "ext/gd/tests/imagerectangle_error4-detect.phpt",
    "ext/gd/tests/imagecolorallocate_error-detect.phpt",
    "ext/gd/tests/imagefontheight_error1-detect.phpt",
    "ext/gd/tests/imagegammacorrect_error4-detect.phpt",
    "ext/gd/tests/bug51263-detect.phpt",
    "ext/gd/tests/imagecharup_error6-detect.phpt",
    "ext/gd/tests/imagecolourstotal_basic-detect.phpt",
    "ext/gd/tests/colorresolve-detect.phpt",
    "ext/gd/tests/imagefilledarc_variation1-detect.phpt",
    "ext/gd/tests/libgd00101-detect.phpt",
    "ext/gd/tests/imagefilter_error17-detect.phpt",
    "ext/gd/tests/bug41442-detect.phpt",
    "ext/gd/tests/imagefilltoborder_error1-detect.phpt",
    "ext/gd/tests/imagearc_error1-detect.phpt",
    "ext/gd/tests/imageloadfont_invalid-detect.phpt",
    "ext/gd/tests/imagesetthickness_error1-detect.phpt",
    "ext/gd/tests/imagecolorallocatealpha_error3-detect.phpt",
    "ext/gd/tests/colormatch-detect.phpt",
    "ext/gd/tests/imagechar_error7-detect.phpt",
    "ext/gd/tests/imagewbmp-detect.phpt",
    "ext/gd/tests/libgd00186-detect.phpt",
    "ext/gd/tests/imageantialias_error1-detect.phpt",
    "ext/gd/tests/imagearc_variation1-detect.phpt",
    "ext/gd/tests/bug38179-detect.phpt",
    "ext/gd/tests/dashedlines-detect.phpt",
    "ext/gd/tests/bug24155-detect.phpt",
    "ext/gd/tests/imagecolormatch_error1-detect.phpt",
    "ext/gd/tests/gif2png-detect.phpt",
    "ext/gd/tests/bug27582_1-detect.phpt",
    "ext/gd/tests/imagegammacorrect_error3-detect.phpt",
    "ext/gd/tests/bug37346-detect.phpt",
    "ext/gd/tests/imagefilter_error5-detect.phpt",
    "ext/gd/tests/colorclosest-detect.phpt",
    "ext/gd/tests/imagerectangle_error3-detect.phpt",
    "ext/gd/tests/imagetruecolortopalette_error3-detect.phpt",
    "ext/gd/tests/imagecolorallocate_variation5-detect.phpt",
    "ext/gd/tests/imagecolorallocate_variation6-detect.phpt",
    "ext/gd/tests/imagefilltoborder_error7-detect.phpt",
    "ext/gd/tests/imageloadfont_error2-detect.phpt",
    "ext/gd/tests/imagecopymerge_basic-detect.phpt",
    "ext/gd/tests/imagecopymerge_error-detect.phpt",
    "ext/gd/tests/imagestring_error3-detect.phpt",
    "ext/gd/tests/gif2gd-detect.phpt",
    "ext/gd/tests/imagecharup_error7-detect.phpt",
    "ext/gd/tests/jpeg2wbmp_error1-detect.phpt",
    "ext/gd/tests/imagefilter_error19-detect.phpt",
    "ext/gd/tests/bug40764-detect.phpt",
    "ext/gd/tests/libgd00106-detect.phpt",
    "ext/gd/tests/png2wbmp_error1-detect.phpt",
    "ext/gd/tests/imagetruecolortopalette_error2-detect.phpt",
    "ext/gd/tests/imageinterlace_variation1-detect.phpt",
    "ext/gd/tests/imagecreatetruecolor_error1-detect.phpt",
    "ext/gd/tests/imagefilltoborder_error3-detect.phpt",
    "ext/gd/tests/imagefilltoborder_error4-detect.phpt",
    "ext/gd/tests/png2gd-detect.phpt",
    "ext/gd/tests/gif-detect.phpt",
    "ext/gd/tests/bug39082-detect.phpt",
    "ext/gd/tests/imageloadfont_error1-detect.phpt",
    "ext/gd/tests/jpeg2wbmp_error2-detect.phpt",
    "ext/gd/tests/imagefilter_error6-detect.phpt",
    "ext/gd/tests/imagecreatetruecolor_error3-detect.phpt",
    "ext/gd/tests/imagecolordeallocate_error1-detect.phpt",
    "ext/gd/tests/imagestringup_error6-detect.phpt",
    "ext/gd/tests/copypalette-detect.phpt",
    "ext/gd/tests/libgd00191-detect.phpt",
    "ext/gd/tests/imagefilledpolygon_basic-detect.phpt",
    "ext/gd/tests/pngcomp-detect.phpt",
    "ext/gd/tests/xpm2gd-detect.phpt",
    "ext/gd/tests/imagecreatetruecolor_basic-detect.phpt",
    "ext/gd/tests/imagecolorallocate_basic-detect.phpt",
    "ext/gd/tests/imagerotate_overflow-detect.phpt",
    "ext/gd/tests/imagestring_error7-detect.phpt",
    "ext/gd/tests/imageistruecolor_basic-detect.phpt",
    "ext/gd/tests/createfromwbmp2_extern-detect.phpt",
    "ext/gd/tests/imagecolormatch_error3-detect.phpt",
    "ext/gd/tests/imagerectangle_basic-detect.phpt",
    "ext/gd/tests/imagestringup_error7-detect.phpt",
    "ext/gd/tests/imagefilter_error8-detect.phpt",
    "ext/gd/tests/imagegammacorrect_variation1-detect.phpt",
    "ext/gd/tests/imagetruecolortopalette_error1-detect.phpt",
    "ext/gd/tests/bug42434-detect.phpt",
    "ext/gd/tests/imagecolormatch_error2-detect.phpt",
    "ext/gd/tests/imageellipse_error4-detect.phpt",
    "ext/gd/tests/imagecolorallocatealpha_error5-detect.phpt",
    "ext/gd/tests/imagecolormatch_error4-detect.phpt",
    "ext/gd/tests/bug22544-detect.phpt",
    "ext/gd/tests/imagesetthickness_basic-detect.phpt",
    "ext/gd/tests/xpm2jpg-detect.phpt",
    "ext/gd/tests/copy-detect.phpt",
    "ext/gd/tests/imageconvolution_error3-detect.phpt",
    "ext/gd/tests/imagefontwidth_error1-detect.phpt",
    "ext/gd/tests/imageconvolution_error1-detect.phpt",
    "ext/gd/tests/imagecharup_error1-detect.phpt",
    "ext/gd/tests/imagestringup_error1-detect.phpt",
    "ext/gd/tests/imagestring_error1-detect.phpt",
    "ext/gd/tests/imagefilter_error14-detect.phpt",
    "ext/gd/tests/imagechar_error3-detect.phpt",
    "ext/gd/tests/bug60160-detect.phpt",
    "ext/gd/tests/imageinterlace_error1-detect.phpt",
    "ext/gd/tests/imagefilltoborder_error2-detect.phpt",
    "ext/gd/tests/imagefontheight_basic-detect.phpt",
    "ext/gd/tests/imagelayereffect_error2-detect.phpt",
    "ext/gd/tests/imagefilter_error11-detect.phpt",
    "ext/gd/tests/xpm2png-detect.phpt",
    "ext/gd/tests/imagefilledarc_error1-detect.phpt",
    "ext/gd/tests/createfromstring-detect.phpt",
    "ext/gd/tests/imagepolygon_basic-detect.phpt",
    "ext/gd/tests/imagecolorallocatealpha_basic-detect.phpt",
    "ext/gd/tests/imagegammacorrect_error2-detect.phpt",
    "ext/gd/tests/imagerectangle_error5-detect.phpt",
    "ext/gd/tests/imagechar_error2-detect.phpt",
    "ext/gd/tests/imagecolorstotal_error-detect.phpt",
    "ext/gd/tests/imagecolorallocate_variation2-detect.phpt",
    "ext/gd/tests/imagefilter_error18-detect.phpt",
    "ext/gd/tests/imagefilledpolygon_negative-detect.phpt",
    "ext/gd/tests/gd_info_basic-detect.phpt",
    "ext/gd/tests/imagefill_1-detect.phpt",
    "ext/gd/tests/createfromgd2-detect.phpt",
    "ext/gd/tests/jpg2gd-detect.phpt",
    "ext/wddx/tests/005-detect.phpt",
    "ext/wddx/tests/bug41283-detect.phpt",
    "ext/wddx/tests/001-64bit-detect.phpt",
    "ext/wddx/tests/bug48562-detect.phpt",
    "ext/wddx/tests/bug35410_64bit-detect.phpt",
    "ext/wddx/tests/001-detect.phpt",
    "ext/wddx/tests/bug27287-detect.phpt",
    "ext/wddx/tests/004-detect.phpt",
    "ext/wddx/tests/003-detect.phpt",
    "ext/wddx/tests/bug35410-detect.phpt",
    "ext/wddx/tests/bug37587-detect.phpt",
    "ext/wddx/tests/bug37569-detect.phpt",
    "ext/wddx/tests/bug34306-detect.phpt",
    "ext/wddx/tests/002-detect.phpt",
    "ext/wddx/tests/bug45901-detect.phpt",
    "ext/wddx/tests/bug41527-detect.phpt",
    "ext/wddx/tests/bug52468-detect.phpt",
    "ext/standard/tests/http/bug48929-detect.phpt",
    "ext/standard/tests/http/bug60570-detect.phpt",
    "ext/standard/tests/http/ignore_errors-detect.phpt",
    "ext/standard/tests/http/bug38802-detect.phpt",
    "ext/standard/tests/http/bug43510-detect.phpt",
    "ext/standard/tests/http/bug53198-detect.phpt",
    "ext/standard/tests/php_real_logo_guid-detect.phpt",
    "ext/standard/tests/mail/mail_variation_alt1-win32-detect.phpt",
    "ext/standard/tests/mail/mail_basic_alt2-win32-detect.phpt",
    "ext/standard/tests/mail/mail_basic4-detect.phpt",
    "ext/standard/tests/mail/mail_basic_alt3-win32-detect.phpt",
    "ext/standard/tests/mail/mail_variation_alt3-win32-detect.phpt",
    "ext/standard/tests/mail/mail_basic3-detect.phpt",
    "ext/standard/tests/mail/mail_basic-detect.phpt",
    "ext/standard/tests/mail/ezmlm_hash_variation1-detect.phpt",
    "ext/standard/tests/mail/mail_basic2-detect.phpt",
    "ext/standard/tests/mail/mail_variation2-detect.phpt",
    "ext/standard/tests/mail/mail_variation1-detect.phpt",
    "ext/standard/tests/mail/ezmlm_hash_basic-detect.phpt",
    "ext/standard/tests/mail/mail_basic_alt4-win32-detect.phpt",
    "ext/standard/tests/mail/ezmlm_hash_basic_64bit-detect.phpt",
    "ext/standard/tests/mail/mail_variation_alt2-win32-detect.phpt",
    "ext/standard/tests/mail/mail_error-detect.phpt",
    "ext/standard/tests/mail/mail_basic_alt1-win32-detect.phpt",
    "ext/standard/tests/mail/ezmlm_hash_error-detect.phpt",
    "ext/standard/tests/mail/mail_basic5-detect.phpt",
    "ext/standard/tests/mail/bug51604-detect.phpt",
    "ext/standard/tests/directory/DirectoryClass_error_001-detect.phpt",
    "ext/standard/tests/directory/DirectoryClass_basic_001-detect.phpt",
    "ext/standard/tests/directory/directory_constants-win32-detect.phpt",
    "ext/standard/tests/directory/directory_constants-detect.phpt",
    "ext/standard/tests/php_ini_loaded_file-detect.phpt",
    "ext/standard/tests/bug49244-detect.phpt",
    "ext/standard/tests/file/glob_error_002-win32-detect.phpt",
    "ext/standard/tests/file/touch_basic-win32-detect.phpt",
    "ext/standard/tests/file/include_streams-detect.phpt",
    "ext/standard/tests/file/copy_variation15-detect.phpt",
    "ext/standard/tests/file/rename_variation2-detect.phpt",
    "ext/standard/tests/file/fscanf_variation14-detect.phpt",
    "ext/standard/tests/file/fgetss_error-detect.phpt",
    "ext/standard/tests/file/disk_free_space_variation-detect.phpt",
    "ext/standard/tests/file/fscanf_variation24-detect.phpt",
    "ext/standard/tests/file/fopen_variation9-detect.phpt",
    "ext/standard/tests/file/fscanf-detect.phpt",
    "ext/standard/tests/file/file_put_contents_variation8-detect.phpt",
    "ext/standard/tests/file/005_variation-win32-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation3-detect.phpt",
    "ext/standard/tests/file/glob_basic-detect.phpt",
    "ext/standard/tests/file/bug45181-detect.phpt",
    "ext/standard/tests/file/fscanf_variation1-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_variation8-win32-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation8-detect.phpt",
    "ext/standard/tests/file/stat_variation6-win32-detect.phpt",
    "ext/standard/tests/file/copy_variation7-detect.phpt",
    "ext/standard/tests/file/touch_variation3-detect.phpt",
    "ext/standard/tests/file/disk_free_space_error-detect.phpt",
    "ext/standard/tests/file/rename_variation3-win32-detect.phpt",
    "ext/standard/tests/file/readfile_variation9-detect.phpt",
    "ext/standard/tests/file/007_variation6-detect.phpt",
    "ext/standard/tests/file/popen_pclose_error-win32-detect.phpt",
    "ext/standard/tests/file/copy_variation14-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation4-detect.phpt",
    "ext/standard/tests/file/fopen_variation15-detect.phpt",
    "ext/standard/tests/file/file_get_contents_variation3-detect.phpt",
    "ext/standard/tests/file/file_exists_variation1-detect.phpt",
    "ext/standard/tests/file/copy_variation6-detect.phpt",
    "ext/standard/tests/file/is_executable_basic-detect.phpt",
    "ext/standard/tests/file/move_uploaded_file_basic-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation7-detect.phpt",
    "ext/standard/tests/file/fscanf_variation23-detect.phpt",
    "ext/standard/tests/file/file_get_contents_file_put_contents_variation1-detect.phpt",
    "ext/standard/tests/file/flock_basic-detect.phpt",
    "ext/standard/tests/file/fgets_variation2-detect.phpt",
    "ext/standard/tests/file/fwrite_variation5-detect.phpt",
    "ext/standard/tests/file/fwrite_variation3-detect.phpt",
    "ext/standard/tests/file/readlink_realpath_variation3-detect.phpt",
    "ext/standard/tests/file/unlink_variation8-detect.phpt",
    "ext/standard/tests/file/bug12556-detect.phpt",
    "ext/standard/tests/file/bug44034-detect.phpt",
    "ext/standard/tests/file/fpassthru_variation1-detect.phpt",
    "ext/standard/tests/file/fopen_variation5-detect.phpt",
    "ext/standard/tests/file/file_basic-detect.phpt",
    "ext/standard/tests/file/fgetss_variation4-detect.phpt",
    "ext/standard/tests/file/touch_variation5-win32-detect.phpt",
    "ext/standard/tests/file/fgets_variation5-detect.phpt",
    "ext/standard/tests/file/fscanf_variation8-detect.phpt",
    "ext/standard/tests/file/bug38450_1-detect.phpt",
    "ext/standard/tests/file/disk_free_space_error-win32-detect.phpt",
    "ext/standard/tests/file/userstreams_005-detect.phpt",
    "ext/standard/tests/file/readfile_variation3-detect.phpt",
    "ext/standard/tests/file/rmdir_variation2-detect.phpt",
    "ext/standard/tests/file/mkdir_variation5-win32-detect.phpt",
    "ext/standard/tests/file/stream_get_line-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation13-detect.phpt",
    "ext/standard/tests/file/fgets_socket_variation2-detect.phpt",
    "ext/standard/tests/file/fscanf_variation15-detect.phpt",
    "ext/standard/tests/file/stream_002-detect.phpt",
    "ext/standard/tests/file/umask_basic-detect.phpt",
    "ext/standard/tests/file/file_get_contents_variation7-detect.phpt",
    "ext/standard/tests/file/copy_variation16-win32-detect.phpt",
    "ext/standard/tests/file/parse_ini_file_variation2-detect.phpt",
    "ext/standard/tests/file/glob_variation2-detect.phpt",
    "ext/standard/tests/file/filetype_variation-detect.phpt",
    "ext/standard/tests/file/fclose_variation1-detect.phpt",
    "ext/standard/tests/file/is_uploaded_file_basic-detect.phpt",
    "ext/standard/tests/file/readlink_realpath_variation1-detect.phpt",
    "ext/standard/tests/file/bug26938-detect.phpt",
    "ext/standard/tests/file/chmod_error-detect.phpt",
    "ext/standard/tests/file/disk_total_space_error-win32-detect.phpt",
    "ext/standard/tests/file/file_get_contents_error001-detect.phpt",
    "ext/standard/tests/file/007_variation16-detect.phpt",
    "ext/standard/tests/file/fgetcsv_error-detect.phpt",
    "ext/standard/tests/file/ftruncate-detect.phpt",
    "ext/standard/tests/file/rename_variation1-detect.phpt",
    "ext/standard/tests/file/001-win32-detect.phpt",
    "ext/standard/tests/file/is_readable_error-detect.phpt",
    "ext/standard/tests/file/file_error-detect.phpt",
    "ext/standard/tests/file/readlink_realpath_variation2-detect.phpt",
    "ext/standard/tests/file/copy_variation4-detect.phpt",
    "ext/standard/tests/file/rename_variation12-win32-detect.phpt",
    "ext/standard/tests/file/007_variation12-win32-detect.phpt",
    "ext/standard/tests/file/bug44607-detect.phpt",
    "ext/standard/tests/file/fscanf_variation4-detect.phpt",
    "ext/standard/tests/file/copy_variation5-detect.phpt",
    "ext/standard/tests/file/chmod_basic-detect.phpt",
    "ext/standard/tests/file/file_put_contents_variation4-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation31-detect.phpt",
    "ext/standard/tests/file/fscanf_variation19-detect.phpt",
    "ext/standard/tests/file/file_get_contents_variation8-win32-detect.phpt",
    "ext/standard/tests/file/readlink_realpath_basic1-detect.phpt",
    "ext/standard/tests/file/fscanf_variation35-detect.phpt",
    "ext/standard/tests/file/fscanf_variation32-detect.phpt",
    "ext/standard/tests/file/stat_basic-win32-detect.phpt",
    "ext/standard/tests/file/umask_variation1-detect.phpt",
    "ext/standard/tests/file/file_get_contents_variation6-detect.phpt",
    "ext/standard/tests/file/stat_variation1-win32-detect.phpt",
    "ext/standard/tests/file/file_get_contents_variation2-detect.phpt",
    "ext/standard/tests/file/stream_rfc2397_007-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_basic1-detect.phpt",
    "ext/standard/tests/file/fgetc_variation1-detect.phpt",
    "ext/standard/tests/file/rename_basic-detect.phpt",
    "ext/standard/tests/file/pathinfo_basic1-win32-detect.phpt",
    "ext/standard/tests/file/symlink_link_linkinfo_is_link_variation3-detect.phpt",
    "ext/standard/tests/file/is_file_basic-detect.phpt",
    "ext/standard/tests/file/rename_variation2-win32-detect.phpt",
    "ext/standard/tests/file/fstat_variation6-detect.phpt",
    "ext/standard/tests/file/flock_error-detect.phpt",
    "ext/standard/tests/file/parse_ini_file_variation5-detect.phpt",
    "ext/standard/tests/file/fgetss_variation3-detect.phpt",
    "ext/standard/tests/file/007_variation14-detect.phpt",
    "ext/standard/tests/file/rename_variation12-detect.phpt",
    "ext/standard/tests/file/get_current_user-detect.phpt",
    "ext/standard/tests/file/fgetss_basic2-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation7-win32-detect.phpt",
    "ext/standard/tests/file/unlink_variation7-detect.phpt",
    "ext/standard/tests/file/filetype_variation3-detect.phpt",
    "ext/standard/tests/file/rename_variation13-detect.phpt",
    "ext/standard/tests/file/005_error-detect.phpt",
    "ext/standard/tests/file/symlink_link_linkinfo_is_link_variation7-detect.phpt",
    "ext/standard/tests/file/file_put_contents_variation9-detect.phpt",
    "ext/standard/tests/file/fscanf_variation22-detect.phpt",
    "ext/standard/tests/file/fscanf_variation54-detect.phpt",
    "ext/standard/tests/file/symlink_to_symlink-detect.phpt",
    "ext/standard/tests/file/fopen_variation14-detect.phpt",
    "ext/standard/tests/file/readfile_basic-detect.phpt",
    "ext/standard/tests/file/file_put_contents_variation8-win32-detect.phpt",
    "ext/standard/tests/file/file_put_contents_variation1-detect.phpt",
    "ext/standard/tests/file/bug45303-detect.phpt",
    "ext/standard/tests/file/006_basic-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation10-detect.phpt",
    "ext/standard/tests/file/bug39538-detect.phpt",
    "ext/standard/tests/file/file_variation2-detect.phpt",
    "ext/standard/tests/file/rename_variation1-win32-detect.phpt",
    "ext/standard/tests/file/symlink_link_linkinfo_is_link_variation9-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation21-detect.phpt",
    "ext/standard/tests/file/mkdir-001-detect.phpt",
    "ext/standard/tests/file/bug41693-detect.phpt",
    "ext/standard/tests/file/file_put_contents_variation3-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_error1-detect.phpt",
    "ext/standard/tests/file/fgetss1-detect.phpt",
    "ext/standard/tests/file/popen_pclose_basic-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation3-detect.phpt",
    "ext/standard/tests/file/fscanf_variation51-detect.phpt",
    "ext/standard/tests/file/filesize_variation1-detect.phpt",
    "ext/standard/tests/file/007_variation18-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation21-detect.phpt",
    "ext/standard/tests/file/tempnam_variation1-win32-detect.phpt",
    "ext/standard/tests/file/copy_variation1-detect.phpt",
    "ext/standard/tests/file/fopen_variation13-detect.phpt",
    "ext/standard/tests/file/symlink-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation15-detect.phpt",
    "ext/standard/tests/file/statcache-corruption-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation18-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation11-detect.phpt",
    "ext/standard/tests/file/dirname_variation1-detect.phpt",
    "ext/standard/tests/file/is_readable_variation3-detect.phpt",
    "ext/standard/tests/file/windows_acls/bug44859_2-detect.phpt",
    "ext/standard/tests/file/windows_acls/bug44859_4-detect.phpt",
    "ext/standard/tests/file/windows_acls/bug44859_3-detect.phpt",
    "ext/standard/tests/file/windows_acls/bug44859-detect.phpt",
    "ext/standard/tests/file/rmdir_variation1-win32-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation9-detect.phpt",
    "ext/standard/tests/file/copy_variation3-win32-detect.phpt",
    "ext/standard/tests/file/file_get_contents_variation7-win32-detect.phpt",
    "ext/standard/tests/file/bug27619-detect.phpt",
    "ext/standard/tests/file/readfile_basic-win32-detect.phpt",
    "ext/standard/tests/file/007_variation11-detect.phpt",
    "ext/standard/tests/file/popen_pclose_error-detect.phpt",
    "ext/standard/tests/file/file_put_contents_variation7-detect.phpt",
    "ext/standard/tests/file/copy_variation12-detect.phpt",
    "ext/standard/tests/file/file_get_contents_file_put_contents_basic-detect.phpt",
    "ext/standard/tests/file/bug52624-detect.phpt",
    "ext/standard/tests/file/mkdir_rmdir_variation2-detect.phpt",
    "ext/standard/tests/file/fgetss-detect.phpt",
    "ext/standard/tests/file/fopen_variation7-detect.phpt",
    "ext/standard/tests/file/readfile_variation2-detect.phpt",
    "ext/standard/tests/file/unlink_variation6-detect.phpt",
    "ext/standard/tests/file/file_get_contents_variation8-detect.phpt",
    "ext/standard/tests/file/001-detect.phpt",
    "ext/standard/tests/file/fputcsv-detect.phpt",
    "ext/standard/tests/file/fgetss_basic2-win32-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation12-detect.phpt",
    "ext/standard/tests/file/file_get_contents_variation5-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation5-win32-detect.phpt",
    "ext/standard/tests/file/fopen_variation11-win32-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_variation1-detect.phpt",
    "ext/standard/tests/file/parse_ini_file_variation6-win32-detect.phpt",
    "ext/standard/tests/file/bug41815-detect.phpt",
    "ext/standard/tests/file/file_get_contents_error002-detect.phpt",
    "ext/standard/tests/file/glob_error-detect.phpt",
    "ext/standard/tests/file/is_writable_error-detect.phpt",
    "ext/standard/tests/file/rename_variation13-win32-detect.phpt",
    "ext/standard/tests/file/fwrite_variation3-win32-detect.phpt",
    "ext/standard/tests/file/unlink_variation1-detect.phpt",
    "ext/standard/tests/file/disk_free_space_basic-detect.phpt",
    "ext/standard/tests/file/fflush_variation1-detect.phpt",
    "ext/standard/tests/file/fopen_variation16-detect.phpt",
    "ext/standard/tests/file/chroot_001-detect.phpt",
    "ext/standard/tests/file/file_put_contents_variation6-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_variation8-detect.phpt",
    "ext/standard/tests/file/mkdir-002-detect.phpt",
    "ext/standard/tests/file/unlink_variation10-detect.phpt",
    "ext/standard/tests/file/proc_open01-detect.phpt",
    "ext/standard/tests/file/basename_variation2-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation14-detect.phpt",
    "ext/standard/tests/file/readfile_variation7-detect.phpt",
    "ext/standard/tests/file/rename_error-detect.phpt",
    "ext/standard/tests/file/004-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation6-win32-detect.phpt",
    "ext/standard/tests/file/007_variation21-detect.phpt",
    "ext/standard/tests/file/realpath_variation-win32-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_variation6-detect.phpt",
    "ext/standard/tests/file/userstreams_004-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation3-detect.phpt",
    "ext/standard/tests/file/007_variation23-detect.phpt",
    "ext/standard/tests/file/fopen_variation14-win32-detect.phpt",
    "ext/standard/tests/file/mkdir_variation1-win32-detect.phpt",
    "ext/standard/tests/file/005_variation2-win32-detect.phpt",
    "ext/standard/tests/file/fopen_variation15-win32-detect.phpt",
    "ext/standard/tests/file/realpath_basic2-detect.phpt",
    "ext/standard/tests/file/readfile_variation10-win32-detect.phpt",
    "ext/standard/tests/file/lchown_basic-detect.phpt",
    "ext/standard/tests/file/bug43137-detect.phpt",
    "ext/standard/tests/file/userstreams_006-detect.phpt",
    "ext/standard/tests/file/file_variation6-detect.phpt",
    "ext/standard/tests/file/bug41655_2-detect.phpt",
    "ext/standard/tests/file/chmod_variation3-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation5-detect.phpt",
    "ext/standard/tests/file/bug43216-detect.phpt",
    "ext/standard/tests/file/fwrite_variation1-detect.phpt",
    "ext/standard/tests/file/copy_variation13-detect.phpt",
    "ext/standard/tests/file/file_variation8-win32-detect.phpt",
    "ext/standard/tests/file/stream_rfc2397_001-detect.phpt",
    "ext/standard/tests/file/fputcsv_002-detect.phpt",
    "ext/standard/tests/file/chown-detect.phpt",
    "ext/standard/tests/file/filesize_variation2-win32-detect.phpt",
    "ext/standard/tests/file/copy_variation6-win32-detect.phpt",
    "ext/standard/tests/file/fstat_variation1-detect.phpt",
    "ext/standard/tests/file/fnmatch_basic-detect.phpt",
    "ext/standard/tests/file/stream_rfc2397_002-detect.phpt",
    "ext/standard/tests/file/unlink_variation5-detect.phpt",
    "ext/standard/tests/file/bug38086-detect.phpt",
    "ext/standard/tests/file/file_put_contents_variation2-detect.phpt",
    "ext/standard/tests/file/userdirstream-detect.phpt",
    "ext/standard/tests/file/fileinode_variation1-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation7-detect.phpt",
    "ext/standard/tests/file/file_variation5-win32-detect.phpt",
    "ext/standard/tests/file/basename_variation1-win32-detect.phpt",
    "ext/standard/tests/file/realpath_variation2-detect.phpt",
    "ext/standard/tests/file/bug47767-detect.phpt",
    "ext/standard/tests/file/unlink_variation9-win32-detect.phpt",
    "ext/standard/tests/file/stat_error-win32-detect.phpt",
    "ext/standard/tests/file/bug37158-detect.phpt",
    "ext/standard/tests/file/stream_rfc2397_006-detect.phpt",
    "ext/standard/tests/file/stream_001-detect.phpt",
    "ext/standard/tests/file/fflush_variation1-win32-detect.phpt",
    "ext/standard/tests/file/is_writable_variation1-detect.phpt",
    "ext/standard/tests/file/filetype_variation2-detect.phpt",
    "ext/standard/tests/file/php_fd_wrapper_01-detect.phpt",
    "ext/standard/tests/file/fscanf_variation3-detect.phpt",
    "ext/standard/tests/file/stat_variation8-win32-detect.phpt",
    "ext/standard/tests/file/copy_basic-detect.phpt",
    "ext/standard/tests/file/bug43522-detect.phpt",
    "ext/standard/tests/file/006_variation2-detect.phpt",
    "ext/standard/tests/file/basename-win32-detect.phpt",
    "ext/standard/tests/file/rename_variation-detect.phpt",
    "ext/standard/tests/file/copy_variation16-detect.phpt",
    "ext/standard/tests/file/filesize_basic-detect.phpt",
    "ext/standard/tests/file/stream_copy_to_stream-detect.phpt",
    "ext/standard/tests/file/fgetcsv-detect.phpt",
    "ext/standard/tests/file/file_variation8-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation19-detect.phpt",
    "ext/standard/tests/file/symlink_link_linkinfo_is_link_basic1-detect.phpt",
    "ext/standard/tests/file/realpath_cache_win32-detect.phpt",
    "ext/standard/tests/file/bug20424-detect.phpt",
    "ext/standard/tests/file/dirname_basic-detect.phpt",
    "ext/standard/tests/file/symlink_link_linkinfo_is_link_variation4-detect.phpt",
    "ext/standard/tests/file/fscanf_variation43-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation1-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation27-detect.phpt",
    "ext/standard/tests/file/bug41655_1-detect.phpt",
    "ext/standard/tests/file/pathinfo_variation1-detect.phpt",
    "ext/standard/tests/file/fgetss_variation3-win32-detect.phpt",
    "ext/standard/tests/file/fscanf_variation47-detect.phpt",
    "ext/standard/tests/file/fwrite_basic-detect.phpt",
    "ext/standard/tests/file/fscanf_variation13-detect.phpt",
    "ext/standard/tests/file/pathinfo_basic1-detect.phpt",
    "ext/standard/tests/file/php_fd_wrapper_04-detect.phpt",
    "ext/standard/tests/file/rename_variation11-win32-detect.phpt",
    "ext/standard/tests/file/unlink_basic-detect.phpt",
    "ext/standard/tests/file/copy_variation5-win32-detect.phpt",
    "ext/standard/tests/file/stream_rfc2397_005-detect.phpt",
    "ext/standard/tests/file/003-detect.phpt",
    "ext/standard/tests/file/007_variation20-detect.phpt",
    "ext/standard/tests/file/fileperms_variation1-detect.phpt",
    "ext/standard/tests/file/touch_basic-detect.phpt",
    "ext/standard/tests/file/chmod_variation1-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation3-win32-detect.phpt",
    "ext/standard/tests/file/tempnam_variation2-win32-detect.phpt",
    "ext/standard/tests/file/symlink_link_linkinfo_is_link_variation8-detect.phpt",
    "ext/standard/tests/file/file_put_contents_variation5-detect.phpt",
    "ext/standard/tests/file/fgetc_variation3-detect.phpt",
    "ext/standard/tests/file/ftruncate_error-detect.phpt",
    "ext/standard/tests/file/popen_pclose_error-win32-debug-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation18-detect.phpt",
    "ext/standard/tests/file/fopen_variation19-detect.phpt",
    "ext/standard/tests/file/fscanf_variation21-detect.phpt",
    "ext/standard/tests/file/fscanf_variation50-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation25-detect.phpt",
    "ext/standard/tests/file/fstat_variation8-detect.phpt",
    "ext/standard/tests/file/fscanf_variation31-detect.phpt",
    "ext/standard/tests/file/is_file_variation3-detect.phpt",
    "ext/standard/tests/file/fscanf_variation34-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation8-detect.phpt",
    "ext/standard/tests/file/tempnam_error-detect.phpt",
    "ext/standard/tests/file/bug51094-detect.phpt",
    "ext/standard/tests/file/fscanf_variation26-detect.phpt",
    "ext/standard/tests/file/feof_basic-detect.phpt",
    "ext/standard/tests/file/filetype_basic-detect.phpt",
    "ext/standard/tests/file/fscanf_variation11-detect.phpt",
    "ext/standard/tests/file/fflush_variation2-detect.phpt",
    "ext/standard/tests/file/fread_error-detect.phpt",
    "ext/standard/tests/file/007_variation4-detect.phpt",
    "ext/standard/tests/file/fgetss_basic1-detect.phpt",
    "ext/standard/tests/file/bug32160-detect.phpt",
    "ext/standard/tests/file/readfile_variation1-detect.phpt",
    "ext/standard/tests/file/007_variation7-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation10-detect.phpt",
    "ext/standard/tests/file/bug35781-detect.phpt",
    "ext/standard/tests/file/readfile_variation10-detect.phpt",
    "ext/standard/tests/file/rename_variation7-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_variation6-win32-detect.phpt",
    "ext/standard/tests/file/fileinode_error-detect.phpt",
    "ext/standard/tests/file/fscanf_variation18-detect.phpt",
    "ext/standard/tests/file/symlink_link_linkinfo_is_link_error2-detect.phpt",
    "ext/standard/tests/file/fscanf_variation41-detect.phpt",
    "ext/standard/tests/file/touch_variation2-detect.phpt",
    "ext/standard/tests/file/basename_variation4-detect.phpt",
    "ext/standard/tests/file/bug39673-detect.phpt",
    "ext/standard/tests/file/fscanf_variation30-detect.phpt",
    "ext/standard/tests/file/unlink_variation2-win32-detect.phpt",
    "ext/standard/tests/file/unlink_variation2-detect.phpt",
    "ext/standard/tests/file/stat_variation3-win32-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation16-detect.phpt",
    "ext/standard/tests/file/fgetss_variation5-detect.phpt",
    "ext/standard/tests/file/fscanf_variation17-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation6-detect.phpt",
    "ext/standard/tests/file/fscanf_variation33-detect.phpt",
    "ext/standard/tests/file/readfile_variation5-detect.phpt",
    "ext/standard/tests/file/fscanf_variation36-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation10-detect.phpt",
    "ext/standard/tests/file/is_readable_basic-detect.phpt",
    "ext/standard/tests/file/bug40374-detect.phpt",
    "ext/standard/tests/file/fileinode_basic-detect.phpt",
    "ext/standard/tests/file/is_writable_variation2-detect.phpt",
    "ext/standard/tests/file/tempnam_variation7-detect.phpt",
    "ext/standard/tests/file/copy_variation8-detect.phpt",
    "ext/standard/tests/file/touch_variation1-detect.phpt",
    "ext/standard/tests/file/readfile_variation4-detect.phpt",
    "ext/standard/tests/file/fwrite_basic-win32-detect.phpt",
    "ext/standard/tests/file/tempnam_variation3-win32-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation8-detect.phpt",
    "ext/standard/tests/file/bug22382-detect.phpt",
    "ext/standard/tests/file/fopen_variation3-detect.phpt",
    "ext/standard/tests/file/fwrite_variation2-detect.phpt",
    "ext/standard/tests/file/touch_variation3-win32-detect.phpt",
    "ext/standard/tests/file/php_fd_wrapper_02-detect.phpt",
    "ext/standard/tests/file/file_get_contents_basic001-detect.phpt",
    "ext/standard/tests/file/file_get_contents_variation4-detect.phpt",
    "ext/standard/tests/file/parse_ini_file_error-detect.phpt",
    "ext/standard/tests/file/fstat_variation5-detect.phpt",
    "ext/standard/tests/file/fscanf_variation7-detect.phpt",
    "ext/standard/tests/file/fscanf_variation2-detect.phpt",
    "ext/standard/tests/file/readlink_realpath_basic2-detect.phpt",
    "ext/standard/tests/file/005_basic-detect.phpt",
    "ext/standard/tests/file/stream_enclosed-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation23-detect.phpt",
    "ext/standard/tests/file/007_variation5-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation12-detect.phpt",
    "ext/standard/tests/file/fscanf_variation45-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation1-win32-detect.phpt",
    "ext/standard/tests/file/fscanf_variation25-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation22-detect.phpt",
    "ext/standard/tests/file/fscanf_variation10-detect.phpt",
    "ext/standard/tests/file/link_win32-detect.phpt",
    "ext/standard/tests/file/include_userstream_002-detect.phpt",
    "ext/standard/tests/file/userstreams_007-detect.phpt",
    "ext/standard/tests/file/fscanf_variation29-detect.phpt",
    "ext/standard/tests/file/fwrite_error-detect.phpt",
    "ext/standard/tests/file/chmod_variation2-win32-detect.phpt",
    "ext/standard/tests/file/unlink_variation3-detect.phpt",
    "ext/standard/tests/file/007_variation24-detect.phpt",
    "ext/standard/tests/file/fileperms_variation2-detect.phpt",
    "ext/standard/tests/file/fstat_variation3-detect.phpt",
    "ext/standard/tests/file/fopencookie-detect.phpt",
    "ext/standard/tests/file/fgetss_variation2-detect.phpt",
    "ext/standard/tests/file/filesize_variation2-detect.phpt",
    "ext/standard/tests/file/file_variation5-detect.phpt",
    "ext/standard/tests/file/unlink_error-win32-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_variation2-win32-detect.phpt",
    "ext/standard/tests/file/is_executable_variation1-detect.phpt",
    "ext/standard/tests/file/mkdir_rmdir_variation1-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation29-detect.phpt",
    "ext/standard/tests/file/fread_variation2-detect.phpt",
    "ext/standard/tests/file/include_userstream_001-detect.phpt",
    "ext/standard/tests/file/bug52820-detect.phpt",
    "ext/standard/tests/file/touch-detect.phpt",
    "ext/standard/tests/file/filegroup_variation2-detect.phpt",
    "ext/standard/tests/file/007_variation11-win32-detect.phpt",
    "ext/standard/tests/file/fscanf_variation16-detect.phpt",
    "ext/standard/tests/file/is_file_variation4-detect.phpt",
    "ext/standard/tests/file/bug22414-detect.phpt",
    "ext/standard/tests/file/rmdir_variation3-detect.phpt",
    "ext/standard/tests/file/rename_variation8-win32-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_basic2-detect.phpt",
    "ext/standard/tests/file/fileowner_error-detect.phpt",
    "ext/standard/tests/file/dirname_basic-win32-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation9-detect.phpt",
    "ext/standard/tests/file/fflush_variation3-detect.phpt",
    "ext/standard/tests/file/lstat_stat_error-detect.phpt",
    "ext/standard/tests/file/006_error-detect.phpt",
    "ext/standard/tests/file/tempnam_variation5-detect.phpt",
    "ext/standard/tests/file/mkdir_variation4-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation2-detect.phpt",
    "ext/standard/tests/file/fseek_variation2-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation2-detect.phpt",
    "ext/standard/tests/file/fwrite_variation4-win32-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation1-detect.phpt",
    "ext/standard/tests/file/fread_variation4-detect.phpt",
    "ext/standard/tests/file/fileowner_basic-detect.phpt",
    "ext/standard/tests/file/mkdir_rmdir_variation-win32-detect.phpt",
    "ext/standard/tests/file/fgetc_basic-detect.phpt",
    "ext/standard/tests/file/fgetss_variation1-detect.phpt",
    "ext/standard/tests/file/fopen_variation8-detect.phpt",
    "ext/standard/tests/file/mkdir_rmdir_error-detect.phpt",
    "ext/standard/tests/file/bug39367-detect.phpt",
    "ext/standard/tests/file/parse_ini_file_variation1-detect.phpt",
    "ext/standard/tests/file/fopen_variation1-detect.phpt",
    "ext/standard/tests/file/file_variation4-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation6-detect.phpt",
    "ext/standard/tests/file/fileinode_variation2-detect.phpt",
    "ext/standard/tests/file/filegroup_variation1-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation26-detect.phpt",
    "ext/standard/tests/file/umask_variation3-detect.phpt",
    "ext/standard/tests/file/parse_ini_file_variation6-detect.phpt",
    "ext/standard/tests/file/symlink_link_linkinfo_is_link_variation5-detect.phpt",
    "ext/standard/tests/file/filesize_variation5-detect.phpt",
    "ext/standard/tests/file/bug43353-win32-detect.phpt",
    "ext/standard/tests/file/fgetc_variation4-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation14-detect.phpt",
    "ext/standard/tests/file/fread_variation3-win32-detect.phpt",
    "ext/standard/tests/file/rename_variation5-detect.phpt",
    "ext/standard/tests/file/is_executable_variation2-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation4-win32-detect.phpt",
    "ext/standard/tests/file/fscanf_variation37-detect.phpt",
    "ext/standard/tests/file/mkdir_variation2-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation19-detect.phpt",
    "ext/standard/tests/file/symlink_link_linkinfo_is_link_error1-detect.phpt",
    "ext/standard/tests/file/007_variation1-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation1-detect.phpt",
    "ext/standard/tests/file/fwrite_variation2-win32-detect.phpt",
    "ext/standard/tests/file/directory_wrapper_fstat_basic-detect.phpt",
    "ext/standard/tests/file/fgetss_variation1-win32-detect.phpt",
    "ext/standard/tests/file/fstat-detect.phpt",
    "ext/standard/tests/file/filegroup_error-detect.phpt",
    "ext/standard/tests/file/007_variation8-detect.phpt",
    "ext/standard/tests/file/parse_ini_file_variation4-detect.phpt",
    "ext/standard/tests/file/rename_variation6-detect.phpt",
    "ext/standard/tests/file/copy_variation10-detect.phpt",
    "ext/standard/tests/file/flock_variation-detect.phpt",
    "ext/standard/tests/file/stat_variation4-win32-detect.phpt",
    "ext/standard/tests/file/bug39551-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_variation4-detect.phpt",
    "ext/standard/tests/file/symlink_link_linkinfo_is_link_basic2-detect.phpt",
    "ext/standard/tests/file/filesize_variation1-win32-detect.phpt",
    "ext/standard/tests/file/popen_pclose_error-sunos-detect.phpt",
    "ext/standard/tests/file/file_put_contents_variation7-win32-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation5-detect.phpt",
    "ext/standard/tests/file/bug38450-detect.phpt",
    "ext/standard/tests/file/is_readable_basic-win32-detect.phpt",
    "ext/standard/tests/file/fgetss_variation5-win32-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_error3-detect.phpt",
    "ext/standard/tests/file/bug24482-detect.phpt",
    "ext/standard/tests/file/007_variation13-detect.phpt",
    "ext/standard/tests/file/filesize_variation3-win32-detect.phpt",
    "ext/standard/tests/file/tempnam_variation3-detect.phpt",
    "ext/standard/tests/file/chmod_basic-win32-detect.phpt",
    "ext/standard/tests/file/umask_error-detect.phpt",
    "ext/standard/tests/file/fread_socket_variation1-detect.phpt",
    "ext/standard/tests/file/fopen_variation6-detect.phpt",
    "ext/standard/tests/file/file_get_contents_error-detect.phpt",
    "ext/standard/tests/file/basename_variation3-detect.phpt",
    "ext/standard/tests/file/statpage-detect.phpt",
    "ext/standard/tests/file/002-detect.phpt",
    "ext/standard/tests/file/bug61961-detect.phpt",
    "ext/standard/tests/file/fileowner_variation2-detect.phpt",
    "ext/standard/tests/file/filegroup_variation3-detect.phpt",
    "ext/standard/tests/file/fwrite-detect.phpt",
    "ext/standard/tests/file/file_variation7-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation9-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation6-detect.phpt",
    "ext/standard/tests/file/tempnam_variation5-win32-detect.phpt",
    "ext/standard/tests/file/file_put_contents-detect.phpt",
    "ext/standard/tests/file/filegroup_basic-detect.phpt",
    "ext/standard/tests/file/bug46347-detect.phpt",
    "ext/standard/tests/file/is_file_variation2-detect.phpt",
    "ext/standard/tests/file/filesize_variation4-win32-detect.phpt",
    "ext/standard/tests/file/is_readable_variation1-detect.phpt",
    "ext/standard/tests/file/copy_variation18-detect.phpt",
    "ext/standard/tests/file/bug37864-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation2-detect.phpt",
    "ext/standard/tests/file/clearstatcache_001-detect.phpt",
    "ext/standard/tests/file/unlink_variation8-win32-detect.phpt",
    "ext/standard/tests/file/disk_total_space_error-detect.phpt",
    "ext/standard/tests/file/copy_variation11-detect.phpt",
    "ext/standard/tests/file/bug41874-detect.phpt",
    "ext/standard/tests/file/basename_variation2-win32-detect.phpt",
    "ext/standard/tests/file/fread_variation4-win32-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation30-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation11-detect.phpt",
    "ext/standard/tests/file/007_error-detect.phpt",
    "ext/standard/tests/file/disk-detect.phpt",
    "ext/standard/tests/file/realpath_basic-win32-detect.phpt",
    "ext/standard/tests/file/filestat-detect.phpt",
    "ext/standard/tests/file/readlink_realpath_error-detect.phpt",
    "ext/standard/tests/file/file_get_contents_file_put_contents_variation2-detect.phpt",
    "ext/standard/tests/file/stream_rfc2397_003-detect.phpt",
    "ext/standard/tests/file/bug38450_2-detect.phpt",
    "ext/standard/tests/file/bug43353-detect.phpt",
    "ext/standard/tests/file/mkdir_variation3-detect.phpt",
    "ext/standard/tests/file/007_variation13-win32-detect.phpt",
    "ext/standard/tests/file/007_basic-detect.phpt",
    "ext/standard/tests/file/touch_variation4-win32-detect.phpt",
    "ext/standard/tests/file/fileperms_variation3-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation5-detect.phpt",
    "ext/standard/tests/file/copy_variation3-detect.phpt",
    "ext/standard/tests/file/unlink_variation1-win32-detect.phpt",
    "ext/standard/tests/file/rename_variation7-win32-detect.phpt",
    "ext/standard/tests/file/005_variation2-detect.phpt",
    "ext/standard/tests/file/fstat_variation2-detect.phpt",
    "ext/standard/tests/file/fgetc_error-detect.phpt",
    "ext/standard/tests/file/mkdir-004-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_basic2-win32-detect.phpt",
    "ext/standard/tests/file/mkdir-006-detect.phpt",
    "ext/standard/tests/file/bug35740-detect.phpt",
    "ext/standard/tests/file/dirname_error-detect.phpt",
    "ext/standard/tests/file/bug30362-detect.phpt",
    "ext/standard/tests/file/chmod_variation2-detect.phpt",
    "ext/standard/tests/file/fflush_variation4-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation4-detect.phpt",
    "ext/standard/tests/file/copy_variation9-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation13-detect.phpt",
    "ext/standard/tests/file/is_dir_variation4-detect.phpt",
    "ext/standard/tests/file/007_variation22-detect.phpt",
    "ext/standard/tests/file/fread_variation3-detect.phpt",
    "ext/standard/tests/file/fgets_variation3-detect.phpt",
    "ext/standard/tests/file/rename_variation10-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation7-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation3-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation22-detect.phpt",
    "ext/standard/tests/file/fscanf_variation42-detect.phpt",
    "ext/standard/tests/file/fscanf_variation49-detect.phpt",
    "ext/standard/tests/file/pathinfo_basic2-detect.phpt",
    "ext/standard/tests/file/fgets_variation6-win32-detect.phpt",
    "ext/standard/tests/file/pathinfo_error-detect.phpt",
    "ext/standard/tests/file/fputcsv_error-detect.phpt",
    "ext/standard/tests/file/fflush_basic-detect.phpt",
    "ext/standard/tests/file/is_dir_error-detect.phpt",
    "ext/standard/tests/file/touch_variation6-win32-detect.phpt",
    "ext/standard/tests/file/lstat_stat_basic-detect.phpt",
    "ext/standard/tests/file/fread_basic-detect.phpt",
    "ext/standard/tests/file/pclose_variation1-detect.phpt",
    "ext/standard/tests/file/rename_variation3-detect.phpt",
    "ext/standard/tests/file/fseek_dir_basic-detect.phpt",
    "ext/standard/tests/file/bug41874_1-detect.phpt",
    "ext/standard/tests/file/tempnam_variation1-detect.phpt",
    "ext/standard/tests/file/fread_fwrite_basic-detect.phpt",
    "ext/standard/tests/file/bug49047-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation7-detect.phpt",
    "ext/standard/tests/file/fscanf_variation53-detect.phpt",
    "ext/standard/tests/file/fileowner_variation3-detect.phpt",
    "ext/standard/tests/file/chgrp-detect.phpt",
    "ext/standard/tests/file/fscanf_variation46-detect.phpt",
    "ext/standard/tests/file/is_dir_basic-detect.phpt",
    "ext/standard/tests/file/fscanf_variation44-detect.phpt",
    "ext/standard/tests/file/file_get_contents_variation1-detect.phpt",
    "ext/standard/tests/file/basename_basic-win32-detect.phpt",
    "ext/standard/tests/file/bug24313-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation2-win32-detect.phpt",
    "ext/standard/tests/file/is_file_variation1-detect.phpt",
    "ext/standard/tests/file/is_executable_variation3-detect.phpt",
    "ext/standard/tests/file/symlink_link_linkinfo_is_link_variation2-detect.phpt",
    "ext/standard/tests/file/mkdir-003-detect.phpt",
    "ext/standard/tests/file/fnmatch_error-detect.phpt",
    "ext/standard/tests/file/bug53848-detect.phpt",
    "ext/standard/tests/file/fscanf_variation20-detect.phpt",
    "ext/standard/tests/file/tempnam_variation8-win32-detect.phpt",
    "ext/standard/tests/file/bug40501-detect.phpt",
    "ext/standard/tests/file/bug41874_2-detect.phpt",
    "ext/standard/tests/file/fgetc_variation2-detect.phpt",
    "ext/standard/tests/file/copy_error-detect.phpt",
    "ext/standard/tests/file/fgets_error-detect.phpt",
    "ext/standard/tests/file/rename_variation-win32-detect.phpt",
    "ext/standard/tests/file/touch_error-detect.phpt",
    "ext/standard/tests/file/fstat_error-detect.phpt",
    "ext/standard/tests/file/fscanf_variation55-detect.phpt",
    "ext/standard/tests/file/readfile_variation8-win32-detect.phpt",
    "ext/standard/tests/file/fscanf_variation6-detect.phpt",
    "ext/standard/tests/file/mkdir_variation1-detect.phpt",
    "ext/standard/tests/file/is_file_error-detect.phpt",
    "ext/standard/tests/file/stat_variation2-win32-detect.phpt",
    "ext/standard/tests/file/lchown_error-detect.phpt",
    "ext/standard/tests/file/fscanf_variation5-detect.phpt",
    "ext/standard/tests/file/bug39863-detect.phpt",
    "ext/standard/tests/file/copy_variation17-detect.phpt",
    "ext/standard/tests/file/fstat_basic-detect.phpt",
    "ext/standard/tests/file/fscanf_variation27-detect.phpt",
    "ext/standard/tests/file/include_userstream_003-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation14-detect.phpt",
    "ext/standard/tests/file/006_variation1-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation20-detect.phpt",
    "ext/standard/tests/file/copy_variation2-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation6-detect.phpt",
    "ext/standard/tests/file/pathinfo_variation2-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation1-detect.phpt",
    "ext/standard/tests/file/007_variation17-detect.phpt",
    "ext/standard/tests/file/bug43248-detect.phpt",
    "ext/standard/tests/file/007_variation2-detect.phpt",
    "ext/standard/tests/file/fscanf_variation28-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_variation4-win32-detect.phpt",
    "ext/standard/tests/file/fread_variation1-detect.phpt",
    "ext/standard/tests/file/is_executable_error-detect.phpt",
    "ext/standard/tests/file/fscanf_variation12-detect.phpt",
    "ext/standard/tests/file/realpath_basic4-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_variation5-detect.phpt",
    "ext/standard/tests/file/fileowner_variation1-detect.phpt",
    "ext/standard/tests/file/fpassthru_error-detect.phpt",
    "ext/standard/tests/file/php_fd_wrapper_03-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_variation2-detect.phpt",
    "ext/standard/tests/file/readfile_variation6-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation4-detect.phpt",
    "ext/standard/tests/file/copy_variation2-win32-detect.phpt",
    "ext/standard/tests/file/pathinfo_variaton-detect.phpt",
    "ext/standard/tests/file/fopen_variation10-win32-detect.phpt",
    "ext/standard/tests/file/filesize_variation4-detect.phpt",
    "ext/standard/tests/file/userstreams_002-detect.phpt",
    "ext/standard/tests/file/rename_variation8-detect.phpt",
    "ext/standard/tests/file/file_variation3-detect.phpt",
    "ext/standard/tests/file/windows_links/bug48746_2-detect.phpt",
    "ext/standard/tests/file/windows_links/bug48746_1-detect.phpt",
    "ext/standard/tests/file/windows_links/bug48746-detect.phpt",
    "ext/standard/tests/file/windows_links/bug48746_3-detect.phpt",
    "ext/standard/tests/file/glob_variation-detect.phpt",
    "ext/standard/tests/file/pathinfo_variation3-detect.phpt",
    "ext/standard/tests/file/fileinode_variation-detect.phpt",
    "ext/standard/tests/file/bug60120-detect.phpt",
    "ext/standard/tests/file/file_variation-detect.phpt",
    "ext/standard/tests/file/fscanf_variation38-detect.phpt",
    "ext/standard/tests/file/fpassthru_basic-detect.phpt",
    "ext/standard/tests/file/realpath_cache-detect.phpt",
    "ext/standard/tests/file/stat_variation5-win32-detect.phpt",
    "ext/standard/tests/file/fopen_variation4-detect.phpt",
    "ext/standard/tests/file/fopen_variation12-detect.phpt",
    "ext/standard/tests/file/fscanf_variation39-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation17-detect.phpt",
    "ext/standard/tests/file/copy_variation12-win32-detect.phpt",
    "ext/standard/tests/file/umask_variation2-detect.phpt",
    "ext/standard/tests/file/rmdir_variation3-win32-detect.phpt",
    "ext/standard/tests/file/pathinfo_basic-detect.phpt",
    "ext/standard/tests/file/bug26615-detect.phpt",
    "ext/standard/tests/file/touch_variation5-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation5-detect.phpt",
    "ext/standard/tests/file/007_variation10-detect.phpt",
    "ext/standard/tests/file/fscanf_variation48-detect.phpt",
    "ext/standard/tests/file/pathinfo_basic2-win32-detect.phpt",
    "ext/standard/tests/file/fflush_error-detect.phpt",
    "ext/standard/tests/file/fgets_variation6-detect.phpt",
    "ext/standard/tests/file/filetype_error-detect.phpt",
    "ext/standard/tests/file/fopen_variation17-detect.phpt",
    "ext/standard/tests/file/is_writable_basic-detect.phpt",
    "ext/standard/tests/file/is_executable_basic-win32-detect.phpt",
    "ext/standard/tests/file/basename_error-detect.phpt",
    "ext/standard/tests/file/unlink_error-detect.phpt",
    "ext/standard/tests/file/filesize_variation3-detect.phpt",
    "ext/standard/tests/file/is_readable_variation2-detect.phpt",
    "ext/standard/tests/file/bug53241-detect.phpt",
    "ext/standard/tests/file/lchgrp_basic-detect.phpt",
    "ext/standard/tests/file/fpassthru_variation-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation15-detect.phpt",
    "ext/standard/tests/file/stream_supports_lock-detect.phpt",
    "ext/standard/tests/file/symlink_link_linkinfo_is_link_variation1-detect.phpt",
    "ext/standard/tests/file/bug43008-detect.phpt",
    "ext/standard/tests/file/lstat_stat_variation16-detect.phpt",
    "ext/standard/tests/file/is_writable_variation3-detect.phpt",
    "ext/standard/tests/file/clearstatcache_error-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_variation7-detect.phpt",
    "ext/standard/tests/file/bug41874_3-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation28-detect.phpt",
    "ext/standard/tests/file/bug55124-detect.phpt",
    "ext/standard/tests/file/file_get_contents_variation9-detect.phpt",
    "ext/standard/tests/file/rename_variation4-detect.phpt",
    "ext/standard/tests/file/realpath_error-win32-detect.phpt",
    "ext/standard/tests/file/disk_total_space_basic-detect.phpt",
    "ext/standard/tests/file/bug38450_3-detect.phpt",
    "ext/standard/tests/file/fwrite_variation1-win32-detect.phpt",
    "ext/standard/tests/file/userwrapper-detect.phpt",
    "ext/standard/tests/file/unlink_variation4-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation2-detect.phpt",
    "ext/standard/tests/file/rmdir_variation1-detect.phpt",
    "ext/standard/tests/file/realpath_basic3-detect.phpt",
    "ext/standard/tests/file/file_get_contents_basic-detect.phpt",
    "ext/standard/tests/file/fgets_basic-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_error2-detect.phpt",
    "ext/standard/tests/file/fstat_variation4-detect.phpt",
    "ext/standard/tests/file/007_variation15-detect.phpt",
    "ext/standard/tests/file/rename_variation6-win32-detect.phpt",
    "ext/standard/tests/file/file_variation9-detect.phpt",
    "ext/standard/tests/file/fgets_variation1-detect.phpt",
    "ext/standard/tests/file/tempnam_variation6-detect.phpt",
    "ext/standard/tests/file/readlink_variation1-detect.phpt",
    "ext/standard/tests/file/007_variation9-detect.phpt",
    "ext/standard/tests/file/popen_pclose_basic-win32-detect.phpt",
    "ext/standard/tests/file/007_variation3-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation12-detect.phpt",
    "ext/standard/tests/file/fnmatch_variation-detect.phpt",
    "ext/standard/tests/file/is_dir_variation3-detect.phpt",
    "ext/standard/tests/file/fileinode_variation3-detect.phpt",
    "ext/standard/tests/file/readfile_error-detect.phpt",
    "ext/standard/tests/file/userfilters-detect.phpt",
    "ext/standard/tests/file/tempnam_variation2-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation13-detect.phpt",
    "ext/standard/tests/file/fscanf_variation9-detect.phpt",
    "ext/standard/tests/file/bug27508-detect.phpt",
    "ext/standard/tests/file/flock-detect.phpt",
    "ext/standard/tests/file/fseek_variation3-detect.phpt",
    "ext/standard/tests/file/tempnam_variation6-win32-detect.phpt",
    "ext/standard/tests/file/touch_variation4-detect.phpt",
    "ext/standard/tests/file/file_get_contents_file_put_contents_error-detect.phpt",
    "ext/standard/tests/file/stat_variation7-win32-detect.phpt",
    "ext/standard/tests/file/007_variation19-detect.phpt",
    "ext/standard/tests/file/fscanf_error-detect.phpt",
    "ext/standard/tests/file/fputcsv_variation11-detect.phpt",
    "ext/standard/tests/file/fscanf_variation40-detect.phpt",
    "ext/standard/tests/file/rename_variation9-detect.phpt",
    "ext/standard/tests/file/disk_total_space_variation-detect.phpt",
    "ext/standard/tests/file/fscanf_variation52-detect.phpt",
    "ext/standard/tests/file/filesize_error-detect.phpt",
    "ext/standard/tests/file/stream_rfc2397_004-detect.phpt",
    "ext/standard/tests/file/glob_variation-win32-detect.phpt",
    "ext/standard/tests/file/is_dir_variation2-detect.phpt",
    "ext/standard/tests/file/is_dir_variation1-detect.phpt",
    "ext/standard/tests/file/bug44805-detect.phpt",
    "ext/standard/tests/file/parse_ini_file-detect.phpt",
    "ext/standard/tests/file/userstreams_003-detect.phpt",
    "ext/standard/tests/file/symlink_link_linkinfo_is_link_variation6-detect.phpt",
    "ext/standard/tests/file/tempnam_variation4-detect.phpt",
    "ext/standard/tests/file/basename-detect.phpt",
    "ext/standard/tests/file/userstreams-detect.phpt",
    "ext/standard/tests/file/ftruncate_variation4-detect.phpt",
    "ext/standard/tests/file/fstat_variation7-detect.phpt",
    "ext/standard/tests/file/fwrite_variation4-detect.phpt",
    "ext/standard/tests/file/basename_variation1-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation24-detect.phpt",
    "ext/standard/tests/file/parse_ini_file_variation3-detect.phpt",
    "ext/standard/tests/file/basename_basic-detect.phpt",
    "ext/standard/tests/file/fgetcsv_variation20-detect.phpt",
    "ext/standard/tests/file/tempnam_variation7-win32-detect.phpt",
    "ext/standard/tests/file/mkdir-005-detect.phpt",
    "ext/standard/tests/file/fgets_variation4-detect.phpt",
    "ext/standard/tests/file/007_variation12-detect.phpt",
    "ext/standard/tests/file/bug26003-detect.phpt",
    "ext/standard/tests/file/fseek_variation1-detect.phpt",
    "ext/standard/tests/file/fgets_variation4-win32-detect.phpt",
    "ext/standard/tests/file/005_variation-detect.phpt",
    "ext/standard/tests/file/fgets_socket_variation1-detect.phpt",
    "ext/standard/tests/file/chmod_variation4-detect.phpt",
    "ext/standard/tests/file/file_exists_error-detect.phpt",
    "ext/standard/tests/file/bug45985-detect.phpt",
    "ext/standard/tests/file/mkdir_variation5-detect.phpt",
    "ext/standard/tests/file/fseek_ftell_rewind_variation3-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_object1-detect.phpt",
    "ext/standard/tests/array/bug44181-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation1-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation5-detect.phpt",
    "ext/standard/tests/array/array_fill_keys_variation3-detect.phpt",
    "ext/standard/tests/array/asort_variation2-detect.phpt",
    "ext/standard/tests/array/array_keys_variation_002-detect.phpt",
    "ext/standard/tests/array/arsort_variation4-detect.phpt",
    "ext/standard/tests/array/array_shift_variation4-detect.phpt",
    "ext/standard/tests/array/array_product_variation3-detect.phpt",
    "ext/standard/tests/array/usort_variation9-detect.phpt",
    "ext/standard/tests/array/array_fill_keys-detect.phpt",
    "ext/standard/tests/array/ksort_variation7-detect.phpt",
    "ext/standard/tests/array/array_change_key_case_errors-detect.phpt",
    "ext/standard/tests/array/bug36975-detect.phpt",
    "ext/standard/tests/array/array_unique_variation7-detect.phpt",
    "ext/standard/tests/array/array_pad_variation1-detect.phpt",
    "ext/standard/tests/array/array_push_variation1-detect.phpt",
    "ext/standard/tests/array/array_slice_variation6-detect.phpt",
    "ext/standard/tests/array/natcasesort_variation2-detect.phpt",
    "ext/standard/tests/array/asort_variation6-detect.phpt",
    "ext/standard/tests/array/arsort_object2-detect.phpt",
    "ext/standard/tests/array/natcasesort_error-detect.phpt",
    "ext/standard/tests/array/array_diff_key_variation4-detect.phpt",
    "ext/standard/tests/array/array_rand_variation3-detect.phpt",
    "ext/standard/tests/array/array_key_exists_variation6-detect.phpt",
    "ext/standard/tests/array/sort_variation11-detect.phpt",
    "ext/standard/tests/array/array_pad_variation3-detect.phpt",
    "ext/standard/tests/array/array_multisort_error-detect.phpt",
    "ext/standard/tests/array/array_search_variation1-detect.phpt",
    "ext/standard/tests/array/array_intersect_assoc_variation7-detect.phpt",
    "ext/standard/tests/array/array_uintersect_variation4-detect.phpt",
    "ext/standard/tests/array/array_key_exists_object1-detect.phpt",
    "ext/standard/tests/array/array_unshift_variation5-detect.phpt",
    "ext/standard/tests/array/array_shift_error-detect.phpt",
    "ext/standard/tests/array/array_merge_recursive_variation7-detect.phpt",
    "ext/standard/tests/array/array_uintersect_assoc_variation4-detect.phpt",
    "ext/standard/tests/array/key_exists_variation1-detect.phpt",
    "ext/standard/tests/array/array_multisort_natural_case-detect.phpt",
    "ext/standard/tests/array/array_combine_error2-detect.phpt",
    "ext/standard/tests/array/reset_error-detect.phpt",
    "ext/standard/tests/array/array_map_001-detect.phpt",
    "ext/standard/tests/array/krsort_variation4-detect.phpt",
    "ext/standard/tests/array/bug24897-detect.phpt",
    "ext/standard/tests/array/asort_variation11-detect.phpt",
    "ext/standard/tests/array/array_flip_variation3-detect.phpt",
    "ext/standard/tests/array/max_error-detect.phpt",
    "ext/standard/tests/array/array_filter_variation7-detect.phpt",
    "ext/standard/tests/array/bug39576-detect.phpt",
    "ext/standard/tests/array/bug32021-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_object2-detect.phpt",
    "ext/standard/tests/array/005-detect.phpt",
    "ext/standard/tests/array/array_unique_variation4-detect.phpt",
    "ext/standard/tests/array/asort_object1-detect.phpt",
    "ext/standard/tests/array/array_chunk-detect.phpt",
    "ext/standard/tests/array/array_unshift_variation8-detect.phpt",
    "ext/standard/tests/array/array_diff_key_variation6-detect.phpt",
    "ext/standard/tests/array/extract_variation9-detect.phpt",
    "ext/standard/tests/array/array_filter_variation4-detect.phpt",
    "ext/standard/tests/array/array_intersect_assoc_error-detect.phpt",
    "ext/standard/tests/array/min_error-detect.phpt",
    "ext/standard/tests/array/bug22463-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation17-detect.phpt",
    "ext/standard/tests/array/array_splice_basic-detect.phpt",
    "ext/standard/tests/array/array_multisort_variation8-detect.phpt",
    "ext/standard/tests/array/key_variation1-detect.phpt",
    "ext/standard/tests/array/array_map_variation19-detect.phpt",
    "ext/standard/tests/array/bug33989-detect.phpt",
    "ext/standard/tests/array/array_key_exists_variation8-detect.phpt",
    "ext/standard/tests/array/ksort_variation11-detect.phpt",
    "ext/standard/tests/array/bug41121-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_variation8-detect.phpt",
    "ext/standard/tests/array/uasort_variation10-detect.phpt",
    "ext/standard/tests/array/array_intersect_assoc_variation10-detect.phpt",
    "ext/standard/tests/array/ksort_variation4-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation23-detect.phpt",
    "ext/standard/tests/array/array_diff_key_variation3-detect.phpt",
    "ext/standard/tests/array/sizeof_variation5-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_basic1-detect.phpt",
    "ext/standard/tests/array/array_intersect_assoc_basic-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation22-detect.phpt",
    "ext/standard/tests/array/ksort_variation8-detect.phpt",
    "ext/standard/tests/array/bug25758-detect.phpt",
    "ext/standard/tests/array/asort_basic-detect.phpt",
    "ext/standard/tests/array/ksort_variation1-detect.phpt",
    "ext/standard/tests/array/krsort_variation1-detect.phpt",
    "ext/standard/tests/array/asort_error-detect.phpt",
    "ext/standard/tests/array/array_fill_variation2-detect.phpt",
    "ext/standard/tests/array/array_values_variation2-detect.phpt",
    "ext/standard/tests/array/rsort_variation1-detect.phpt",
    "ext/standard/tests/array/array_sum_variation4-detect.phpt",
    "ext/standard/tests/array/rsort_basic-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_error2-detect.phpt",
    "ext/standard/tests/array/array_user_key_compare-detect.phpt",
    "ext/standard/tests/array/natcasesort_variation11-detect.phpt",
    "ext/standard/tests/array/prev_error2-detect.phpt",
    "ext/standard/tests/array/array_values_variation7-detect.phpt",
    "ext/standard/tests/array/array_splice_errors-detect.phpt",
    "ext/standard/tests/array/var_export-detect.phpt",
    "ext/standard/tests/array/extract_variation1-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation7-detect.phpt",
    "ext/standard/tests/array/compact_variation2-detect.phpt",
    "ext/standard/tests/array/array_sum_variation6-detect.phpt",
    "ext/standard/tests/array/array_key_exists_variation1-detect.phpt",
    "ext/standard/tests/array/asort_variation1-detect.phpt",
    "ext/standard/tests/array/min_basiclong_64bit-detect.phpt",
    "ext/standard/tests/array/array_intersect_key_variation3-detect.phpt",
    "ext/standard/tests/array/current_variation3-detect.phpt",
    "ext/standard/tests/array/array_push_basic-detect.phpt",
    "ext/standard/tests/array/natcasesort_variation3-detect.phpt",
    "ext/standard/tests/array/bug35022-detect.phpt",
    "ext/standard/tests/array/array_map_variation3-detect.phpt",
    "ext/standard/tests/array/shuffle_variation2-detect.phpt",
    "ext/standard/tests/array/array_chunk_error-detect.phpt",
    "ext/standard/tests/array/bug50006-detect.phpt",
    "ext/standard/tests/array/array_pad_variation5-detect.phpt",
    "ext/standard/tests/array/bug21182-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_variation5-detect.phpt",
    "ext/standard/tests/array/array_reduce_error-detect.phpt",
    "ext/standard/tests/array/array_slice_variation4-detect.phpt",
    "ext/standard/tests/array/end_error-detect.phpt",
    "ext/standard/tests/array/array_udiff_uassoc_variation3-detect.phpt",
    "ext/standard/tests/array/array_diff_assoc_variation8-detect.phpt",
    "ext/standard/tests/array/key_basic-detect.phpt",
    "ext/standard/tests/array/array_reverse_variation1-detect.phpt",
    "ext/standard/tests/array/bug31158-detect.phpt",
    "ext/standard/tests/array/array_intersect_assoc_variation4-detect.phpt",
    "ext/standard/tests/array/array_map_object1-detect.phpt",
    "ext/standard/tests/array/array_unique_variation5-detect.phpt",
    "ext/standard/tests/array/key_variation4-detect.phpt",
    "ext/standard/tests/array/array_diff_key-detect.phpt",
    "ext/standard/tests/array/array_filter_variation9-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation8-detect.phpt",
    "ext/standard/tests/array/shuffle_basic1-detect.phpt",
    "ext/standard/tests/array/array_intersect_1-detect.phpt",
    "ext/standard/tests/array/uasort_variation5-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation32-detect.phpt",
    "ext/standard/tests/array/arsort_variation2-detect.phpt",
    "ext/standard/tests/array/key_error-detect.phpt",
    "ext/standard/tests/array/array_filter_object-detect.phpt",
    "ext/standard/tests/array/array_keys_variation_004-detect.phpt",
    "ext/standard/tests/array/array_pad_variation6-detect.phpt",
    "ext/standard/tests/array/asort_object2-detect.phpt",
    "ext/standard/tests/array/compact-detect.phpt",
    "ext/standard/tests/array/bug42177-detect.phpt",
    "ext/standard/tests/array/array_unshift_variation1-detect.phpt",
    "ext/standard/tests/array/rsort_variation10-detect.phpt",
    "ext/standard/tests/array/array_diff_key_variation8-detect.phpt",
    "ext/standard/tests/array/array_values_variation1-detect.phpt",
    "ext/standard/tests/array/array_walk-detect.phpt",
    "ext/standard/tests/array/array_key_exists_variation7-detect.phpt",
    "ext/standard/tests/array/array_splice_variation2-detect.phpt",
    "ext/standard/tests/array/array_merge_variation1-detect.phpt",
    "ext/standard/tests/array/sort_variation2-detect.phpt",
    "ext/standard/tests/array/asort_variation5-detect.phpt",
    "ext/standard/tests/array/sort_object1-detect.phpt",
    "ext/standard/tests/array/bug34982-detect.phpt",
    "ext/standard/tests/array/array_sum_variation7-detect.phpt",
    "ext/standard/tests/array/array_udiff_uassoc_variation2-detect.phpt",
    "ext/standard/tests/array/rsort_variation7-detect.phpt",
    "ext/standard/tests/array/array_combine_variation2-detect.phpt",
    "ext/standard/tests/array/array_intersect_assoc_variation2-detect.phpt",
    "ext/standard/tests/array/array_intersect_key_error-detect.phpt",
    "ext/standard/tests/array/array_map_variation2-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation6-detect.phpt",
    "ext/standard/tests/array/array_udiff_uassoc_variation6-detect.phpt",
    "ext/standard/tests/array/array_map_variation8-detect.phpt",
    "ext/standard/tests/array/array_walk_rec_objects-detect.phpt",
    "ext/standard/tests/array/array_slice_variation11-detect.phpt",
    "ext/standard/tests/array/ksort_basic-detect.phpt",
    "ext/standard/tests/array/usort_variation1-detect.phpt",
    "ext/standard/tests/array/array_pad_variation7-detect.phpt",
    "ext/standard/tests/array/array_key_exists_variation2-detect.phpt",
    "ext/standard/tests/array/array_intersect_variation6-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_variation9-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation25-detect.phpt",
    "ext/standard/tests/array/array_keys_variation_001-detect.phpt",
    "ext/standard/tests/array/bug24220-detect.phpt",
    "ext/standard/tests/array/array_diff_variation5-detect.phpt",
    "ext/standard/tests/array/array_values-detect.phpt",
    "ext/standard/tests/array/array_udiff_uassoc_error-detect.phpt",
    "ext/standard/tests/array/bug52534-detect.phpt",
    "ext/standard/tests/array/array_merge_recursive_basic2-detect.phpt",
    "ext/standard/tests/array/bug12776-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation26-detect.phpt",
    "ext/standard/tests/array/array_udiff_assoc_variation2-detect.phpt",
    "ext/standard/tests/array/arsort_variation5-detect.phpt",
    "ext/standard/tests/array/array_values_variation5-detect.phpt",
    "ext/standard/tests/array/asort_variation10-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation13-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation12-detect.phpt",
    "ext/standard/tests/array/array_unique_variation1-detect.phpt",
    "ext/standard/tests/array/array_values_errors-detect.phpt",
    "ext/standard/tests/array/array_multisort_variation7-detect.phpt",
    "ext/standard/tests/array/array_map_variation13-detect.phpt",
    "ext/standard/tests/array/array_fill_variation3-detect.phpt",
    "ext/standard/tests/array/array_rand_variation6-detect.phpt",
    "ext/standard/tests/array/array_values_variation4-detect.phpt",
    "ext/standard/tests/array/array_multisort_natural_incase-detect.phpt",
    "ext/standard/tests/array/array_pop_errors-detect.phpt",
    "ext/standard/tests/array/usort_object2-detect.phpt",
    "ext/standard/tests/array/bug61730-detect.phpt",
    "ext/standard/tests/array/array_count_values-detect.phpt",
    "ext/standard/tests/array/array_splice_variation1-detect.phpt",
    "ext/standard/tests/array/array_flip_basic-detect.phpt",
    "ext/standard/tests/array/array_uintersect_assoc_variation2-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation15-detect.phpt",
    "ext/standard/tests/array/array_unshift_basic1-detect.phpt",
    "ext/standard/tests/array/bug40709-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation18-detect.phpt",
    "ext/standard/tests/array/array_intersect_ukey_variation1-detect.phpt",
    "ext/standard/tests/array/bug44929-detect.phpt",
    "ext/standard/tests/array/array_diff_assoc_variation10-detect.phpt",
    "ext/standard/tests/array/001-detect.phpt",
    "ext/standard/tests/array/bug33382-detect.phpt",
    "ext/standard/tests/array/array_intersect_ukey_variation4-detect.phpt",
    "ext/standard/tests/array/max_basiclong_64bit-detect.phpt",
    "ext/standard/tests/array/array_diff_ukey_variation2-detect.phpt",
    "ext/standard/tests/array/array_intersect_uassoc_variation4-detect.phpt",
    "ext/standard/tests/array/array_diff_variation9-detect.phpt",
    "ext/standard/tests/array/compact_basic-detect.phpt",
    "ext/standard/tests/array/array_walk_variation1-detect.phpt",
    "ext/standard/tests/array/krsort_variation5-detect.phpt",
    "ext/standard/tests/array/bug52719-detect.phpt",
    "ext/standard/tests/array/reset_variation3-detect.phpt",
    "ext/standard/tests/array/array_map_basic-detect.phpt",
    "ext/standard/tests/array/array_product_variation4-detect.phpt",
    "ext/standard/tests/array/sort_variation10-detect.phpt",
    "ext/standard/tests/array/array_slice_variation9-detect.phpt",
    "ext/standard/tests/array/array_count_values_variation-detect.phpt",
    "ext/standard/tests/array/current_basic-detect.phpt",
    "ext/standard/tests/array/extract_variation2-detect.phpt",
    "ext/standard/tests/array/array_merge_basic-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation2-detect.phpt",
    "ext/standard/tests/array/array_merge_recursive_variation3-detect.phpt",
    "ext/standard/tests/array/array_udiff_uassoc_variation5-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation8-detect.phpt",
    "ext/standard/tests/array/array_intersect_uassoc_variation2-detect.phpt",
    "ext/standard/tests/array/array_merge_variation6-detect.phpt",
    "ext/standard/tests/array/array_udiff_uassoc_basic-detect.phpt",
    "ext/standard/tests/array/array_diff_ukey_variation3-detect.phpt",
    "ext/standard/tests/array/array_merge_variation7-detect.phpt",
    "ext/standard/tests/array/array_reduce_variation2-detect.phpt",
    "ext/standard/tests/array/array_splice_variation3-detect.phpt",
    "ext/standard/tests/array/array_walk_error2-detect.phpt",
    "ext/standard/tests/array/004-detect.phpt",
    "ext/standard/tests/array/bug34227-detect.phpt",
    "ext/standard/tests/array/shuffle_basic2-detect.phpt",
    "ext/standard/tests/array/array_diff_ukey_variation8-detect.phpt",
    "ext/standard/tests/array/array_keys_variation_002_64bit-detect.phpt",
    "ext/standard/tests/array/in_array_variation4-detect.phpt",
    "ext/standard/tests/array/array_slice_basic-detect.phpt",
    "ext/standard/tests/array/reset_basic-detect.phpt",
    "ext/standard/tests/array/array_udiff_basic-detect.phpt",
    "ext/standard/tests/array/asort_variation3-detect.phpt",
    "ext/standard/tests/array/array_merge-detect.phpt",
    "ext/standard/tests/array/array_fill_variation1_64bit-detect.phpt",
    "ext/standard/tests/array/array_walk_object1-detect.phpt",
    "ext/standard/tests/array/array_diff_variation1-detect.phpt",
    "ext/standard/tests/array/array_diff_key_variation5-detect.phpt",
    "ext/standard/tests/array/bug35014_64bit-detect.phpt",
    "ext/standard/tests/array/array_merge_recursive_variation6-detect.phpt",
    "ext/standard/tests/array/sizeof_variation4-detect.phpt",
    "ext/standard/tests/array/array_uintersect_error-detect.phpt",
    "ext/standard/tests/array/natcasesort_object1-detect.phpt",
    "ext/standard/tests/array/natcasesort_variation8-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive1-detect.phpt",
    "ext/standard/tests/array/array_walk_variation3-detect.phpt",
    "ext/standard/tests/array/array_intersect_ukey_variation6-detect.phpt",
    "ext/standard/tests/array/reset_variation2-detect.phpt",
    "ext/standard/tests/array/array_reverse_variation4-detect.phpt",
    "ext/standard/tests/array/array_replace-detect.phpt",
    "ext/standard/tests/array/each_variation1-detect.phpt",
    "ext/standard/tests/array/var_export3-detect.phpt",
    "ext/standard/tests/array/array_values_variation6-detect.phpt",
    "ext/standard/tests/array/end_variation3-detect.phpt",
    "ext/standard/tests/array/array_reduce-detect.phpt",
    "ext/standard/tests/array/extract_error-detect.phpt",
    "ext/standard/tests/array/array_map_variation5-detect.phpt",
    "ext/standard/tests/array/arsort_variation9-detect.phpt",
    "ext/standard/tests/array/count_basic-detect.phpt",
    "ext/standard/tests/array/array_intersect_ukey_variation2-detect.phpt",
    "ext/standard/tests/array/array_merge_recursive_variation10-detect.phpt",
    "ext/standard/tests/array/array_flip_variation4-detect.phpt",
    "ext/standard/tests/array/array_intersect_variation2-detect.phpt",
    "ext/standard/tests/array/array_intersect_basic-detect.phpt",
    "ext/standard/tests/array/usort_variation2-detect.phpt",
    "ext/standard/tests/array/array_map_variation12-detect.phpt",
    "ext/standard/tests/array/bug35014-detect.phpt",
    "ext/standard/tests/array/arsort_variation1-detect.phpt",
    "ext/standard/tests/array/range_variation-detect.phpt",
    "ext/standard/tests/array/each_variation2-detect.phpt",
    "ext/standard/tests/array/array_rand_variation4-detect.phpt",
    "ext/standard/tests/array/array_fill_variation1-detect.phpt",
    "ext/standard/tests/array/bug20865-detect.phpt",
    "ext/standard/tests/array/bug23581-detect.phpt",
    "ext/standard/tests/array/ksort_variation3-detect.phpt",
    "ext/standard/tests/array/array_udiff_uassoc_variation1-detect.phpt",
    "ext/standard/tests/array/array_intersect_key_variation5-detect.phpt",
    "ext/standard/tests/array/extract_safety-detect.phpt",
    "ext/standard/tests/array/array_diff_variation4-detect.phpt",
    "ext/standard/tests/array/shuffle_variation1-detect.phpt",
    "ext/standard/tests/array/key_exists_basic-detect.phpt",
    "ext/standard/tests/array/arsort_variation8-detect.phpt",
    "ext/standard/tests/array/arsort_variation6-detect.phpt",
    "ext/standard/tests/array/array_unshift_variation3-detect.phpt",
    "ext/standard/tests/array/array_intersect_uassoc_error-detect.phpt",
    "ext/standard/tests/array/rsort_variation6-detect.phpt",
    "ext/standard/tests/array/array_udiff_variation2-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation1-detect.phpt",
    "ext/standard/tests/array/ksort_variation6-detect.phpt",
    "ext/standard/tests/array/array_unique_variation2-detect.phpt",
    "ext/standard/tests/array/array_intersect_uassoc_basic-detect.phpt",
    "ext/standard/tests/array/array_map_variation10-detect.phpt",
    "ext/standard/tests/array/bug21918-detect.phpt",
    "ext/standard/tests/array/krsort_variation9-detect.phpt",
    "ext/standard/tests/array/array_unique_basic-detect.phpt",
    "ext/standard/tests/array/extract_variation6-detect.phpt",
    "ext/standard/tests/array/array_map_object2-detect.phpt",
    "ext/standard/tests/array/array_rand_variation2-detect.phpt",
    "ext/standard/tests/array/max_variation1-detect.phpt",
    "ext/standard/tests/array/array_multisort_basic2-detect.phpt",
    "ext/standard/tests/array/array_diff_basic-detect.phpt",
    "ext/standard/tests/array/array_reverse_variation2-detect.phpt",
    "ext/standard/tests/array/array_sum_variation3-detect.phpt",
    "ext/standard/tests/array/extract_variation8-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation2-detect.phpt",
    "ext/standard/tests/array/array_filter_basic-detect.phpt",
    "ext/standard/tests/array/array_fill_keys_variation1-detect.phpt",
    "ext/standard/tests/array/bug54459-detect.phpt",
    "ext/standard/tests/array/var_export2-detect.phpt",
    "ext/standard/tests/array/array_push-detect.phpt",
    "ext/standard/tests/array/array_merge_recursive_variation9-detect.phpt",
    "ext/standard/tests/array/array_shift_variation3-detect.phpt",
    "ext/standard/tests/array/array_walk_object2-detect.phpt",
    "ext/standard/tests/array/array_uintersect_assoc_error-detect.phpt",
    "ext/standard/tests/array/003-detect.phpt",
    "ext/standard/tests/array/extract_variation4-detect.phpt",
    "ext/standard/tests/array/array_combine_variation3-detect.phpt",
    "ext/standard/tests/array/array_map_variation18-detect.phpt",
    "ext/standard/tests/array/array_udiff_error-detect.phpt",
    "ext/standard/tests/array/array_diff_assoc_variation6-detect.phpt",
    "ext/standard/tests/array/array_map_variation17-detect.phpt",
    "ext/standard/tests/array/array_intersect_key_variation7-detect.phpt",
    "ext/standard/tests/array/array_intersect_key_basic-detect.phpt",
    "ext/standard/tests/array/array_flip_variation2-detect.phpt",
    "ext/standard/tests/array/sizeof_variation3-detect.phpt",
    "ext/standard/tests/array/array_intersect_ukey_variation7-detect.phpt",
    "ext/standard/tests/array/array_shuffle_basic-detect.phpt",
    "ext/standard/tests/array/bug22088-detect.phpt",
    "ext/standard/tests/array/array_search1-detect.phpt",
    "ext/standard/tests/array/array_uintersect_uassoc_variation2-detect.phpt",
    "ext/standard/tests/array/rsort_error-detect.phpt",
    "ext/standard/tests/array/bug24980-detect.phpt",
    "ext/standard/tests/array/array_filter_error-detect.phpt",
    "ext/standard/tests/array/sort_basic-detect.phpt",
    "ext/standard/tests/array/array_merge_variation9-detect.phpt",
    "ext/standard/tests/array/array_reduce_variation1-detect.phpt",
    "ext/standard/tests/array/array_diff_key_variation2-detect.phpt",
    "ext/standard/tests/array/array_fill_object-detect.phpt",
    "ext/standard/tests/array/array_uintersect_uassoc_variation6-detect.phpt",
    "ext/standard/tests/array/asort_variation7-detect.phpt",
    "ext/standard/tests/array/krsort_object-detect.phpt",
    "ext/standard/tests/array/array_uintersect_uassoc_variation4-detect.phpt",
    "ext/standard/tests/array/array_merge_recursive_error-detect.phpt",
    "ext/standard/tests/array/array_unshift-detect.phpt",
    "ext/standard/tests/array/array_udiff_variation5-detect.phpt",
    "ext/standard/tests/array/array_uintersect_assoc_variation5-detect.phpt",
    "ext/standard/tests/array/uasort_variation1-detect.phpt",
    "ext/standard/tests/array/array_filter_variation5-detect.phpt",
    "ext/standard/tests/array/bug29493-detect.phpt",
    "ext/standard/tests/array/in_array_variation3-detect.phpt",
    "ext/standard/tests/array/array_slice_variation1-detect.phpt",
    "ext/standard/tests/array/array_multisort_natural-detect.phpt",
    "ext/standard/tests/array/array_intersect_assoc_variation1-detect.phpt",
    "ext/standard/tests/array/array_intersect_key_variation2-detect.phpt",
    "ext/standard/tests/array/usort_error2-detect.phpt",
    "ext/standard/tests/array/array_uintersect_assoc_basic-detect.phpt",
    "ext/standard/tests/array/max_variation2-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation9-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation7-detect.phpt",
    "ext/standard/tests/array/usort_variation8-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation11-detect.phpt",
    "ext/standard/tests/array/array_intersect_uassoc_variation8-detect.phpt",
    "ext/standard/tests/array/krsort_variation8-detect.phpt",
    "ext/standard/tests/array/bug35821-detect.phpt",
    "ext/standard/tests/array/array_reduce_variation3-detect.phpt",
    "ext/standard/tests/array/arsort_variation11-detect.phpt",
    "ext/standard/tests/array/array_pad_variation2-detect.phpt",
    "ext/standard/tests/array/array_fill_error-detect.phpt",
    "ext/standard/tests/array/array_splice_variation4-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation9-detect.phpt",
    "ext/standard/tests/array/array_uintersect_basic-detect.phpt",
    "ext/standard/tests/array/array_intersect_uassoc_variation1-detect.phpt",
    "ext/standard/tests/array/array_walk_error1-detect.phpt",
    "ext/standard/tests/array/end_basic-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation24-detect.phpt",
    "ext/standard/tests/array/shuffle_variation5-detect.phpt",
    "ext/standard/tests/array/bug50006_2-detect.phpt",
    "ext/standard/tests/array/array_udiff_assoc_error-detect.phpt",
    "ext/standard/tests/array/key_exists_variation2-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation3-detect.phpt",
    "ext/standard/tests/array/count_error-detect.phpt",
    "ext/standard/tests/array/006-detect.phpt",
    "ext/standard/tests/array/array_slice_variation10-detect.phpt",
    "ext/standard/tests/array/prev_variation1-detect.phpt",
    "ext/standard/tests/array/array_walk_basic1-detect.phpt",
    "ext/standard/tests/array/array_intersect_variation7-detect.phpt",
    "ext/standard/tests/array/shuffle_variation4-detect.phpt",
    "ext/standard/tests/array/array_merge_recursive_variation2-detect.phpt",
    "ext/standard/tests/array/bug51552-detect.phpt",
    "ext/standard/tests/array/array_intersect_uassoc_variation11-detect.phpt",
    "ext/standard/tests/array/array_udiff_variation4-detect.phpt",
    "ext/standard/tests/array/array_intersect_uassoc_variation9-detect.phpt",
    "ext/standard/tests/array/array_diff_variation6-detect.phpt",
    "ext/standard/tests/array/count_variation3-detect.phpt",
    "ext/standard/tests/array/arsort_object1-detect.phpt",
    "ext/standard/tests/array/array_walk_basic2-detect.phpt",
    "ext/standard/tests/array/sizeof_error-detect.phpt",
    "ext/standard/tests/array/array_fill-detect.phpt",
    "ext/standard/tests/array/array_reverse_basic1-detect.phpt",
    "ext/standard/tests/array/array_walk_objects-detect.phpt",
    "ext/standard/tests/array/array_unique_variation6-detect.phpt",
    "ext/standard/tests/array/krsort_variation3-detect.phpt",
    "ext/standard/tests/array/ksort_variation9-detect.phpt",
    "ext/standard/tests/array/array_slice_variation8-detect.phpt",
    "ext/standard/tests/array/ksort_object-detect.phpt",
    "ext/standard/tests/array/array_count_values_error-detect.phpt",
    "ext/standard/tests/array/array_rand_variation5-detect.phpt",
    "ext/standard/tests/array/krsort_variation2-detect.phpt",
    "ext/standard/tests/array/in_array_variation2-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation28-detect.phpt",
    "ext/standard/tests/array/arsort_basic-detect.phpt",
    "ext/standard/tests/array/array_reverse_variation6-detect.phpt",
    "ext/standard/tests/array/bug24766-detect.phpt",
    "ext/standard/tests/array/array_push_variation3-detect.phpt",
    "ext/standard/tests/array/bug14580-detect.phpt",
    "ext/standard/tests/array/array_map_variation4-detect.phpt",
    "ext/standard/tests/array/bug48484-detect.phpt",
    "ext/standard/tests/array/current_variation4-detect.phpt",
    "ext/standard/tests/array/array_pad_error-detect.phpt",
    "ext/standard/tests/array/array_pop-detect.phpt",
    "ext/standard/tests/array/array_push_variation6-detect.phpt",
    "ext/standard/tests/array/extract_variation11-detect.phpt",
    "ext/standard/tests/array/extract_variation5-detect.phpt",
    "ext/standard/tests/array/bug43541-detect.phpt",
    "ext/standard/tests/array/array_diff_assoc_basic-detect.phpt",
    "ext/standard/tests/array/array_uintersect_variation2-detect.phpt",
    "ext/standard/tests/array/array_filter_variation8-detect.phpt",
    "ext/standard/tests/array/ksort_variation10-detect.phpt",
    "ext/standard/tests/array/min_basic-detect.phpt",
    "ext/standard/tests/array/array_unique_variation3-detect.phpt",
    "ext/standard/tests/array/range_variation1-detect.phpt",
    "ext/standard/tests/array/extract_variation7-detect.phpt",
    "ext/standard/tests/array/sort_variation4-detect.phpt",
    "ext/standard/tests/array/sort_variation1-detect.phpt",
    "ext/standard/tests/array/end_64bit-detect.phpt",
    "ext/standard/tests/array/extract_variation10-detect.phpt",
    "ext/standard/tests/array/uasort_variation9-detect.phpt",
    "ext/standard/tests/array/array_merge_recursive_variation1-detect.phpt",
    "ext/standard/tests/array/array_udiff_uassoc_variation4-detect.phpt",
    "ext/standard/tests/array/array_change_key_case_variation4-detect.phpt",
    "ext/standard/tests/array/bug29253-detect.phpt",
    "ext/standard/tests/array/array_slice_variation2-detect.phpt",
    "ext/standard/tests/array/bug46873-detect.phpt",
    "ext/standard/tests/array/array_intersect_variation8-detect.phpt",
    "ext/standard/tests/array/array_walk_variation5-detect.phpt",
    "ext/standard/tests/array/array_push_error2-detect.phpt",
    "ext/standard/tests/array/bug50006_1-detect.phpt",
    "ext/standard/tests/array/bug43495-detect.phpt",
    "ext/standard/tests/array/end_variation2-detect.phpt",
    "ext/standard/tests/array/sort_object2-detect.phpt",
    "ext/standard/tests/array/arsort_error-detect.phpt",
    "ext/standard/tests/array/bug26458-detect.phpt",
    "ext/standard/tests/array/asort_variation8-detect.phpt",
    "ext/standard/tests/array/next_variation1-detect.phpt",
    "ext/standard/tests/array/array_unshift_variation4-detect.phpt",
    "ext/standard/tests/array/array_uintersect_assoc_basic2-detect.phpt",
    "ext/standard/tests/array/array_reverse_variation5-detect.phpt",
    "ext/standard/tests/array/each_variation3-detect.phpt",
    "ext/standard/tests/array/array_map_variation14-detect.phpt",
    "ext/standard/tests/array/natcasesort_basic-detect.phpt",
    "ext/standard/tests/array/array_sum_variation2-detect.phpt",
    "ext/standard/tests/array/array_next_error1-detect.phpt",
    "ext/standard/tests/array/array_udiff_assoc_variation4-detect.phpt",
    "ext/standard/tests/array/locale_sort-detect.phpt",
    "ext/standard/tests/array/sizeof_variation2-detect.phpt",
    "ext/standard/tests/array/array_combine_variation6-detect.phpt",
    "ext/standard/tests/array/array_diff_variation2-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive-detect.phpt",
    "ext/standard/tests/array/array_diff_key_error-detect.phpt",
    "ext/standard/tests/array/array_diff_assoc_variation2-detect.phpt",
    "ext/standard/tests/array/bug45312-detect.phpt",
    "ext/standard/tests/array/krsort_error-detect.phpt",
    "ext/standard/tests/array/array_intersect_uassoc_variation3-detect.phpt",
    "ext/standard/tests/array/array_combine_variation5-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation10-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_variation1-detect.phpt",
    "ext/standard/tests/array/usort_basic-detect.phpt",
    "ext/standard/tests/array/array_intersect_variation10-detect.phpt",
    "ext/standard/tests/array/array_multisort_variation1-detect.phpt",
    "ext/standard/tests/array/bug61058-detect.phpt",
    "ext/standard/tests/array/natcasesort_variation10-detect.phpt",
    "ext/standard/tests/array/007-detect.phpt",
    "ext/standard/tests/array/bug42850-detect.phpt",
    "ext/standard/tests/array/key_exists_error-detect.phpt",
    "ext/standard/tests/array/array_diff_ukey_variation4-detect.phpt",
    "ext/standard/tests/array/bug42233-detect.phpt",
    "ext/standard/tests/array/array_uintersect_assoc_variation1-detect.phpt",
    "ext/standard/tests/array/array_values_error-detect.phpt",
    "ext/standard/tests/array/next_basic-detect.phpt",
    "ext/standard/tests/array/array_intersect_ukey_variation8-detect.phpt",
    "ext/standard/tests/array/bug38464-detect.phpt",
    "ext/standard/tests/array/current_variation2-detect.phpt",
    "ext/standard/tests/array/krsort_variation11-detect.phpt",
    "ext/standard/tests/array/array_key_exists_variation3-detect.phpt",
    "ext/standard/tests/array/bug28974-detect.phpt",
    "ext/standard/tests/array/in_array_errors-detect.phpt",
    "ext/standard/tests/array/in_array_variation1-detect.phpt",
    "ext/standard/tests/array/krsort_variation10-detect.phpt",
    "ext/standard/tests/array/uksort_error-detect.phpt",
    "ext/standard/tests/array/array_push_variation2-detect.phpt",
    "ext/standard/tests/array/sizeof_variation1-detect.phpt",
    "ext/standard/tests/array/sizeof_basic1-detect.phpt",
    "ext/standard/tests/array/array_intersect_uassoc_variation7-detect.phpt",
    "ext/standard/tests/array/array_multisort_variation10-detect.phpt",
    "ext/standard/tests/array/array_rand_basic2-detect.phpt",
    "ext/standard/tests/array/array_diff_assoc_variation7-detect.phpt",
    "ext/standard/tests/array/array_diff_error-detect.phpt",
    "ext/standard/tests/array/array_combine-detect.phpt",
    "ext/standard/tests/array/array_udiff_variation3-detect.phpt",
    "ext/standard/tests/array/array_intersect_assoc_variation6-detect.phpt",
    "ext/standard/tests/array/array_search_variation3-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_error-detect.phpt",
    "ext/standard/tests/array/array_keys_variation_003-detect.phpt",
    "ext/standard/tests/array/array_rand-detect.phpt",
    "ext/standard/tests/array/array_unshift_variation6-detect.phpt",
    "ext/standard/tests/array/array_map_variation6-detect.phpt",
    "ext/standard/tests/array/array_multisort_variation5-detect.phpt",
    "ext/standard/tests/array/array_multisort_case-detect.phpt",
    "ext/standard/tests/array/array_slice_variation7-detect.phpt",
    "ext/standard/tests/array/uasort_basic2-detect.phpt",
    "ext/standard/tests/array/array_diff_ukey_variation11-detect.phpt",
    "ext/standard/tests/array/array_intersect_key_variation6-detect.phpt",
    "ext/standard/tests/array/array_key_exists_object2-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation11-detect.phpt",
    "ext/standard/tests/array/current_variation1-detect.phpt",
    "ext/standard/tests/array/array_search_variation4-detect.phpt",
    "ext/standard/tests/array/uasort_variation8-detect.phpt",
    "ext/standard/tests/array/array_udiff_variation1-detect.phpt",
    "ext/standard/tests/array/array_merge_recursive_basic1-detect.phpt",
    "ext/standard/tests/array/array_diff_ukey_variation5-detect.phpt",
    "ext/standard/tests/array/array_diff_ukey_variation6-detect.phpt",
    "ext/standard/tests/array/002-detect.phpt",
    "ext/standard/tests/array/array_fill_keys_variation4-detect.phpt",
    "ext/standard/tests/array/008-detect.phpt",
    "ext/standard/tests/array/usort_variation6-detect.phpt",
    "ext/standard/tests/array/array_intersect_ukey_variation10-detect.phpt",
    "ext/standard/tests/array/array_values_variation-detect.phpt",
    "ext/standard/tests/array/uksort_variation1-detect.phpt",
    "ext/standard/tests/array/natcasesort_variation7-detect.phpt",
    "ext/standard/tests/array/min_variation2-detect.phpt",
    "ext/standard/tests/array/uksort_variation2-detect.phpt",
    "ext/standard/tests/array/array_map_variation9-detect.phpt",
    "ext/standard/tests/array/array_diff_assoc_variation1-detect.phpt",
    "ext/standard/tests/array/prev_error1-detect.phpt",
    "ext/standard/tests/array/array_diff_key_basic-detect.phpt",
    "ext/standard/tests/array/array_fill_object_2_4-detect.phpt",
    "ext/standard/tests/array/usort_error1-detect.phpt",
    "ext/standard/tests/array/next_variation2-detect.phpt",
    "ext/standard/tests/array/array_diff_ukey_basic-detect.phpt",
    "ext/standard/tests/array/sort_variation3-detect.phpt",
    "ext/standard/tests/array/array_walk_variation4-detect.phpt",
    "ext/standard/tests/array/array_uintersect_variation5-detect.phpt",
    "ext/standard/tests/array/natcasesort_object2-detect.phpt",
    "ext/standard/tests/array/array_combine_variation4-detect.phpt",
    "ext/standard/tests/array/array_sum-detect.phpt",
    "ext/standard/tests/array/array_multisort_variation3-detect.phpt",
    "ext/standard/tests/array/array_intersect_variation3-detect.phpt",
    "ext/standard/tests/array/bug34066-detect.phpt",
    "ext/standard/tests/array/sizeof_basic2-detect.phpt",
    "ext/standard/tests/array/array_map_variation15-detect.phpt",
    "ext/standard/tests/array/array_sum_variation5-detect.phpt",
    "ext/standard/tests/array/sort_variation9-detect.phpt",
    "ext/standard/tests/array/array_multisort_variation11-detect.phpt",
    "ext/standard/tests/array/array_merge_variation8-detect.phpt",
    "ext/standard/tests/array/array_values_variation3-detect.phpt",
    "ext/standard/tests/array/array_merge_recursive_variation8-detect.phpt",
    "ext/standard/tests/array/usort_variation5-detect.phpt",
    "ext/standard/tests/array/array_diff_key2-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation4-detect.phpt",
    "ext/standard/tests/array/array_reverse_error-detect.phpt",
    "ext/standard/tests/array/array_merge_recursive_variation4-detect.phpt",
    "ext/standard/tests/array/array_shift_variation1-detect.phpt",
    "ext/standard/tests/array/array_change_key_case_variation-detect.phpt",
    "ext/standard/tests/array/array_merge_recursive_variation5-detect.phpt",
    "ext/standard/tests/array/uasort_object2-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_error1-detect.phpt",
    "ext/standard/tests/array/next_error-detect.phpt",
    "ext/standard/tests/array/array_map_variation11-detect.phpt",
    "ext/standard/tests/array/array_intersect_key_variation8-detect.phpt",
    "ext/standard/tests/array/array_multisort_variation4-detect.phpt",
    "ext/standard/tests/array/each_error-detect.phpt",
    "ext/standard/tests/array/array_shift_variation2-detect.phpt",
    "ext/standard/tests/array/each_variation5-detect.phpt",
    "ext/standard/tests/array/ksort_error-detect.phpt",
    "ext/standard/tests/array/array_keys_variation_005-detect.phpt",
    "ext/standard/tests/array/array_push_error1-detect.phpt",
    "ext/standard/tests/array/array_merge_variation10-detect.phpt",
    "ext/standard/tests/array/array_sum_error-detect.phpt",
    "ext/standard/tests/array/krsort_variation6-detect.phpt",
    "ext/standard/tests/array/bug43505-detect.phpt",
    "ext/standard/tests/array/array_map_variation16-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation13-detect.phpt",
    "ext/standard/tests/array/array_change_key_case_variation7-detect.phpt",
    "ext/standard/tests/array/array_intersect_variation9-detect.phpt",
    "ext/standard/tests/array/rsort_variation4-detect.phpt",
    "ext/standard/tests/array/array_intersect_ukey_variation5-detect.phpt",
    "ext/standard/tests/array/sizeof_object2-detect.phpt",
    "ext/standard/tests/array/array_combine_basic-detect.phpt",
    "ext/standard/tests/array/array_chunk2-detect.phpt",
    "ext/standard/tests/array/array_key_exists_variation4-detect.phpt",
    "ext/standard/tests/array/reset_variation1-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation31-detect.phpt",
    "ext/standard/tests/array/bug25359-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation6-detect.phpt",
    "ext/standard/tests/array/bug30833-detect.phpt",
    "ext/standard/tests/array/array_shift_variation5-detect.phpt",
    "ext/standard/tests/array/array_uintersect_variation1-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation16-detect.phpt",
    "ext/standard/tests/array/array_pad-detect.phpt",
    "ext/standard/tests/array/shuffle_variation3-detect.phpt",
    "ext/standard/tests/array/array_unique_error-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation12-detect.phpt",
    "ext/standard/tests/array/array_map_object3-detect.phpt",
    "ext/standard/tests/array/array_change_key_case_variation6-detect.phpt",
    "ext/standard/tests/array/current_error-detect.phpt",
    "ext/standard/tests/array/array_pop_variation-detect.phpt",
    "ext/standard/tests/array/unexpected_array_mod_bug-detect.phpt",
    "ext/standard/tests/array/bug31213-detect.phpt",
    "ext/standard/tests/array/arsort_variation10-detect.phpt",
    "ext/standard/tests/array/array_diff_ukey_variation1-detect.phpt",
    "ext/standard/tests/array/array_flip_error-detect.phpt",
    "ext/standard/tests/array/array_diff_ukey_variation9-detect.phpt",
    "ext/standard/tests/array/bug42838-detect.phpt",
    "ext/standard/tests/array/uasort_object1-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation27-detect.phpt",
    "ext/standard/tests/array/array_walk_variation6-detect.phpt",
    "ext/standard/tests/array/array_change_key_case-detect.phpt",
    "ext/standard/tests/array/array_values_basic-detect.phpt",
    "ext/standard/tests/array/array_change_key_case_variation5-detect.phpt",
    "ext/standard/tests/array/usort_variation10-detect.phpt",
    "ext/standard/tests/array/array_sum_basic-detect.phpt",
    "ext/standard/tests/array/array_combine_variation1-detect.phpt",
    "ext/standard/tests/array/array_walk_variation9-detect.phpt",
    "ext/standard/tests/array/bug41686-detect.phpt",
    "ext/standard/tests/array/array_reverse_basic2-detect.phpt",
    "ext/standard/tests/array/rsort_variation2-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation14-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation21-detect.phpt",
    "ext/standard/tests/array/array_intersect_variation5-detect.phpt",
    "ext/standard/tests/array/array_intersect_assoc_variation9-detect.phpt",
    "ext/standard/tests/array/array_intersect_uassoc_variation6-detect.phpt",
    "ext/standard/tests/array/krsort_variation7-detect.phpt",
    "ext/standard/tests/array/end_variation1-detect.phpt",
    "ext/standard/tests/array/uasort_variation2-detect.phpt",
    "ext/standard/tests/array/uasort_variation11-detect.phpt",
    "ext/standard/tests/array/array_unshift_object-detect.phpt",
    "ext/standard/tests/array/array_merge_variation2-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation5-detect.phpt",
    "ext/standard/tests/array/arsort_variation7-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation4-detect.phpt",
    "ext/standard/tests/array/array_chunk_basic1-detect.phpt",
    "ext/standard/tests/array/sort_variation7-detect.phpt",
    "ext/standard/tests/array/array_intersect_variation1-detect.phpt",
    "ext/standard/tests/array/array_multisort_incase-detect.phpt",
    "ext/standard/tests/array/rsort_variation8-detect.phpt",
    "ext/standard/tests/array/bug48224-detect.phpt",
    "ext/standard/tests/array/array_fill_variation5-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_variation14-detect.phpt",
    "ext/standard/tests/array/array_keys_error-detect.phpt",
    "ext/standard/tests/array/arsort_variation3-detect.phpt",
    "ext/standard/tests/array/array_unshift_basic2-detect.phpt",
    "ext/standard/tests/array/array_product_variation5-detect.phpt",
    "ext/standard/tests/array/min_variation1-detect.phpt",
    "ext/standard/tests/array/array_combine_error1-detect.phpt",
    "ext/standard/tests/array/prev_error3-detect.phpt",
    "ext/standard/tests/array/array_product_error-detect.phpt",
    "ext/standard/tests/array/array_diff_key_variation7-detect.phpt",
    "ext/standard/tests/array/array_unshift_variation2-detect.phpt",
    "ext/standard/tests/array/array_fill_keys_variation2-detect.phpt",
    "ext/standard/tests/array/array_intersect_key_variation4-detect.phpt",
    "ext/standard/tests/array/array_filter_variation3-detect.phpt",
    "ext/standard/tests/array/array_diff_variation10-detect.phpt",
    "ext/standard/tests/array/array_reverse_variation3-detect.phpt",
    "ext/standard/tests/array/bug44182-detect.phpt",
    "ext/standard/tests/array/bug20381-detect.phpt",
    "ext/standard/tests/array/rsort_variation9-detect.phpt",
    "ext/standard/tests/array/array_diff_1-detect.phpt",
    "ext/standard/tests/array/each_variation4-detect.phpt",
    "ext/standard/tests/array/array_filter-detect.phpt",
    "ext/standard/tests/array/array_shift_variation7-detect.phpt",
    "ext/standard/tests/array/uksort_basic-detect.phpt",
    "ext/standard/tests/array/array_intersect_error-detect.phpt",
    "ext/standard/tests/array/array_uintersect_uassoc_basic-detect.phpt",
    "ext/standard/tests/array/usort_variation4-detect.phpt",
    "ext/standard/tests/array/array_merge_variation4-detect.phpt",
    "ext/standard/tests/array/array_change_key_case_variation8-detect.phpt",
    "ext/standard/tests/array/array_rand_error-detect.phpt",
    "ext/standard/tests/array/array_change_key_case_variation3-detect.phpt",
    "ext/standard/tests/array/array_udiff_assoc_variation-detect.phpt",
    "ext/standard/tests/array/key_variation3-detect.phpt",
    "ext/standard/tests/array/usort_variation3-detect.phpt",
    "ext/standard/tests/array/array_filter_variation1-detect.phpt",
    "ext/standard/tests/array/array_udiff_assoc_basic-detect.phpt",
    "ext/standard/tests/array/uasort_variation7-detect.phpt",
    "ext/standard/tests/array/uasort_variation6-detect.phpt",
    "ext/standard/tests/array/array_uintersect_uassoc_variation1-detect.phpt",
    "ext/standard/tests/array/array_multisort_variation9-detect.phpt",
    "ext/standard/tests/array/array_intersect_ukey_variation9-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation20-detect.phpt",
    "ext/standard/tests/array/prev_variation2-detect.phpt",
    "ext/standard/tests/array/array_uintersect_assoc_variation3-detect.phpt",
    "ext/standard/tests/array/shuffle_error-detect.phpt",
    "ext/standard/tests/array/array_intersect_assoc_variation5-detect.phpt",
    "ext/standard/tests/array/array_diff_variation3-detect.phpt",
    "ext/standard/tests/array/array_merge_variation5-detect.phpt",
    "ext/standard/tests/array/array_unique_variation8-detect.phpt",
    "ext/standard/tests/array/array_shift_basic-detect.phpt",
    "ext/standard/tests/array/array_flip_variation1-detect.phpt",
    "ext/standard/tests/array/asort_variation9-detect.phpt",
    "ext/standard/tests/array/each-detect.phpt",
    "ext/standard/tests/array/array_uintersect_uassoc_error-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_variation7-detect.phpt",
    "ext/standard/tests/array/count_recursive-detect.phpt",
    "ext/standard/tests/array/compact_variation1-detect.phpt",
    "ext/standard/tests/array/array_map_error-detect.phpt",
    "ext/standard/tests/array/array_diff_variation8-detect.phpt",
    "ext/standard/tests/array/bug30074-detect.phpt",
    "ext/standard/tests/array/array_intersect_uassoc_variation10-detect.phpt",
    "ext/standard/tests/array/count_variation1-detect.phpt",
    "ext/standard/tests/array/bug34066_1-detect.phpt",
    "ext/standard/tests/array/array_walk_variation8-detect.phpt",
    "ext/standard/tests/array/array_pad_variation4-detect.phpt",
    "ext/standard/tests/array/uasort_variation3-detect.phpt",
    "ext/standard/tests/array/array_intersect_uassoc_variation5-detect.phpt",
    "ext/standard/tests/array/bug48854-detect.phpt",
    "ext/standard/tests/array/bug25708-detect.phpt",
    "ext/standard/tests/array/array_intersect_assoc_variation3-detect.phpt",
    "ext/standard/tests/array/end-detect.phpt",
    "ext/standard/tests/array/array_map_variation1-detect.phpt",
    "ext/standard/tests/array/array_flip-detect.phpt",
    "ext/standard/tests/array/array_diff_assoc_variation3-detect.phpt",
    "ext/standard/tests/array/array_search_variation2-detect.phpt",
    "ext/standard/tests/array/array_shift_variation8-detect.phpt",
    "ext/standard/tests/array/array_diff_assoc_variation4-detect.phpt",
    "ext/standard/tests/array/array_values_variation_64bit-detect.phpt",
    "ext/standard/tests/array/array_key_exists_basic-detect.phpt",
    "ext/standard/tests/array/sizeof_object1-detect.phpt",
    "ext/standard/tests/array/array_udiff_assoc_variation5-detect.phpt",
    "ext/standard/tests/array/natcasesort_variation1-detect.phpt",
    "ext/standard/tests/array/max_basic-detect.phpt",
    "ext/standard/tests/array/natcasesort_variation6-detect.phpt",
    "ext/standard/tests/array/array_keys_basic-detect.phpt",
    "ext/standard/tests/array/array_uintersect_uassoc_variation3-detect.phpt",
    "ext/standard/tests/array/each_basic-detect.phpt",
    "ext/standard/tests/array/array_filter_variation6-detect.phpt",
    "ext/standard/tests/array/natsort_basic-detect.phpt",
    "ext/standard/tests/array/rsort_variation3-detect.phpt",
    "ext/standard/tests/array/array_diff_assoc_variation5-detect.phpt",
    "ext/standard/tests/array/uasort_error-detect.phpt",
    "ext/standard/tests/array/array_next_error2-detect.phpt",
    "ext/standard/tests/array/array_intersect_key-detect.phpt",
    "ext/standard/tests/array/array_diff_assoc-detect.phpt",
    "ext/standard/tests/array/array_fill_keys_error-detect.phpt",
    "ext/standard/tests/array/array_intersect_ukey_variation3-detect.phpt",
    "ext/standard/tests/array/compact_error-detect.phpt",
    "ext/standard/tests/array/array_search-detect.phpt",
    "ext/standard/tests/array/prev_basic-detect.phpt",
    "ext/standard/tests/array/array_count_values2-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_variation2-detect.phpt",
    "ext/standard/tests/array/bug23788-detect.phpt",
    "ext/standard/tests/array/array_multisort_basic1-detect.phpt",
    "ext/standard/tests/array/sort_variation5-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_basic2-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_variation6-detect.phpt",
    "ext/standard/tests/array/array_diff_ukey_variation10-detect.phpt",
    "ext/standard/tests/array/max-detect.phpt",
    "ext/standard/tests/array/ksort_variation2-detect.phpt",
    "ext/standard/tests/array/array_change_key_case_variation2-detect.phpt",
    "ext/standard/tests/array/array_sum_variation1-detect.phpt",
    "ext/standard/tests/array/array_push_variation5-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation3-detect.phpt",
    "ext/standard/tests/array/array_intersect_variation4-detect.phpt",
    "ext/standard/tests/array/array_unshift_variation7-detect.phpt",
    "ext/standard/tests/array/array_diff_variation7-detect.phpt",
    "ext/standard/tests/array/array_udiff_assoc_variation1-detect.phpt",
    "ext/standard/tests/array/array_product_variation1-detect.phpt",
    "ext/standard/tests/array/count_variation2-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_variation4-detect.phpt",
    "ext/standard/tests/array/array_unshift_error-detect.phpt",
    "ext/standard/tests/array/array_product_variation2-detect.phpt",
    "ext/standard/tests/array/each_variation6-detect.phpt",
    "ext/standard/tests/array/bug21998-detect.phpt",
    "ext/standard/tests/array/krsort_basic-detect.phpt",
    "ext/standard/tests/array/array_uintersect_variation3-detect.phpt",
    "ext/standard/tests/array/array_chunk_basic2-detect.phpt",
    "ext/standard/tests/array/array_flip_variation5-detect.phpt",
    "ext/standard/tests/array/uasort_variation4-detect.phpt",
    "ext/standard/tests/array/asort_variation4-detect.phpt",
    "ext/standard/tests/array/sort_variation6-detect.phpt",
    "ext/standard/tests/array/array_intersect_assoc_variation8-detect.phpt",
    "ext/standard/tests/array/sort_error-detect.phpt",
    "ext/standard/tests/array/array_walk_variation2-detect.phpt",
    "ext/standard/tests/array/ksort_variation5-detect.phpt",
    "ext/standard/tests/array/array_diff_uassoc_basic-detect.phpt",
    "ext/standard/tests/array/rsort_variation11-detect.phpt",
    "ext/standard/tests/array/array_intersect_key_variation1-detect.phpt",
    "ext/standard/tests/array/array_intersect_ukey_error-detect.phpt",
    "ext/standard/tests/array/array_fill_basic-detect.phpt",
    "ext/standard/tests/array/sort_variation8-detect.phpt",
    "ext/standard/tests/array/array_slice_error-detect.phpt",
    "ext/standard/tests/array/array_slice_variation3-detect.phpt",
    "ext/standard/tests/array/array_udiff_assoc_variation3-detect.phpt",
    "ext/standard/tests/array/rsort_object2-detect.phpt",
    "ext/standard/tests/array/range-detect.phpt",
    "ext/standard/tests/array/rsort_object1-detect.phpt",
    "ext/standard/tests/array/natcasesort_variation4-detect.phpt",
    "ext/standard/tests/array/array_key_exists_variation5-detect.phpt",
    "ext/standard/tests/array/array_rand_variation1-detect.phpt",
    "ext/standard/tests/array/array_walk_variation7-detect.phpt",
    "ext/standard/tests/array/array_multisort_variation2-detect.phpt",
    "ext/standard/tests/array/range_variation1_64bit-detect.phpt",
    "ext/standard/tests/array/array_merge_variation3-detect.phpt",
    "ext/standard/tests/array/array_uintersect_uassoc_variation5-detect.phpt",
    "ext/standard/tests/array/key_variation2-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation19-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation29-detect.phpt",
    "ext/standard/tests/array/009-detect.phpt",
    "ext/standard/tests/array/array_diff_ukey_variation7-detect.phpt",
    "ext/standard/tests/array/rsort_variation5-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation10-detect.phpt",
    "ext/standard/tests/array/array_intersect_ukey_basic-detect.phpt",
    "ext/standard/tests/array/array_diff_key_variation1-detect.phpt",
    "ext/standard/tests/array/natcasesort_variation9-detect.phpt",
    "ext/standard/tests/array/usort_variation7-detect.phpt",
    "ext/standard/tests/array/array_unshift_variation9-detect.phpt",
    "ext/standard/tests/array/array_change_key_case_variation1-detect.phpt",
    "ext/standard/tests/array/array_fill_variation4-detect.phpt",
    "ext/standard/tests/array/array_rand_basic1-detect.phpt",
    "ext/standard/tests/array/bug24198-detect.phpt",
    "ext/standard/tests/array/array_diff_assoc_error-detect.phpt",
    "ext/standard/tests/array/min-detect.phpt",
    "ext/standard/tests/array/array_filter_variation2-detect.phpt",
    "ext/standard/tests/array/array_merge_error-detect.phpt",
    "ext/standard/tests/array/bug28739-detect.phpt",
    "ext/standard/tests/array/extract_variation3-detect.phpt",
    "ext/standard/tests/array/array_slice-detect.phpt",
    "ext/standard/tests/array/array_walk_recursive_variation3-detect.phpt",
    "ext/standard/tests/array/usort_object1-detect.phpt",
    "ext/standard/tests/array/array_multisort_variation6-detect.phpt",
    "ext/standard/tests/array/array_shift_variation6-detect.phpt",
    "ext/standard/tests/array/array_map_variation7-detect.phpt",
    "ext/standard/tests/array/bug30266-detect.phpt",
    "ext/standard/tests/array/current_variation5-detect.phpt",
    "ext/standard/tests/array/array_diff_ukey_error-detect.phpt",
    "ext/standard/tests/array/array_chunk_variation30-detect.phpt",
    "ext/standard/tests/array/array_search_errors-detect.phpt",
    "ext/standard/tests/array/bug40191-detect.phpt",
    "ext/standard/tests/array/natcasesort_variation5-detect.phpt",
    "ext/standard/tests/array/array_diff_assoc_variation9-detect.phpt",
    "ext/standard/tests/array/range_errors-detect.phpt",
    "ext/standard/tests/array/array_key_exists_error-detect.phpt",
    "ext/standard/tests/array/array_slice_variation5-detect.phpt",
    "ext/standard/tests/array/uasort_basic1-detect.phpt",
    "ext/standard/tests/array/array_key_exists-detect.phpt",
    "ext/standard/tests/strings/bug60965-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation8-detect.phpt",
    "ext/standard/tests/strings/vfprintf_basic3-detect.phpt",
    "ext/standard/tests/strings/bug26973-detect.phpt",
    "ext/standard/tests/strings/substr_replace-detect.phpt",
    "ext/standard/tests/strings/ltrim_variation1-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation10-detect.phpt",
    "ext/standard/tests/strings/trim_error-detect.phpt",
    "ext/standard/tests/strings/strncasecmp_variation10-detect.phpt",
    "ext/standard/tests/strings/strrchr_variation1-detect.phpt",
    "ext/standard/tests/strings/strtr_variation6-detect.phpt",
    "ext/standard/tests/strings/strspn_variation5-detect.phpt",
    "ext/standard/tests/strings/strval_variation1-detect.phpt",
    "ext/standard/tests/strings/printf_variation1-detect.phpt",
    "ext/standard/tests/strings/hebrev_error-detect.phpt",
    "ext/standard/tests/strings/sha1_file-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation11_64bit-detect.phpt",
    "ext/standard/tests/strings/bug45166-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation8-detect.phpt",
    "ext/standard/tests/strings/fprintf_variation_005-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation9-detect.phpt",
    "ext/standard/tests/strings/sprintf_f-detect.phpt",
    "ext/standard/tests/strings/strip_tags_variation4-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation1-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation19-detect.phpt",
    "ext/standard/tests/strings/convert_uudecode_basic-detect.phpt",
    "ext/standard/tests/strings/htmlentities06-detect.phpt",
    "ext/standard/tests/strings/show_source_basic-detect.phpt",
    "ext/standard/tests/strings/str_split_variation6_64bit-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation16_64bit-detect.phpt",
    "ext/standard/tests/strings/money_format_basic1-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation52-detect.phpt",
    "ext/standard/tests/strings/convert_uuencode_variation1-detect.phpt",
    "ext/standard/tests/strings/ltrim_variation2-detect.phpt",
    "ext/standard/tests/strings/strings001-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation20-detect.phpt",
    "ext/standard/tests/strings/str_split_variation1-detect.phpt",
    "ext/standard/tests/strings/fprintf_variation_002-detect.phpt",
    "ext/standard/tests/strings/vsprintf_basic1-detect.phpt",
    "ext/standard/tests/strings/setlocale_variation3-detect.phpt",
    "ext/standard/tests/strings/strtr_variation8-detect.phpt",
    "ext/standard/tests/strings/get_html_translation_table_error-detect.phpt",
    "ext/standard/tests/strings/sprintf_f_3-detect.phpt",
    "ext/standard/tests/strings/setlocale_variation5-detect.phpt",
    "ext/standard/tests/strings/strripos_variation4-detect.phpt",
    "ext/standard/tests/strings/explode_variation4-detect.phpt",
    "ext/standard/tests/strings/sha1-detect.phpt",
    "ext/standard/tests/strings/wordwrap_variation5-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation7-detect.phpt",
    "ext/standard/tests/strings/005-detect.phpt",
    "ext/standard/tests/strings/uuencode-detect.phpt",
    "ext/standard/tests/strings/strtoupper1-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation15-detect.phpt",
    "ext/standard/tests/strings/http_build_query-detect.phpt",
    "ext/standard/tests/strings/nl2br_error-detect.phpt",
    "ext/standard/tests/strings/trim1-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation19-detect.phpt",
    "ext/standard/tests/strings/chunk_split_variation11-detect.phpt",
    "ext/standard/tests/strings/bug22187-detect.phpt",
    "ext/standard/tests/strings/strtr_variation1-detect.phpt",
    "ext/standard/tests/strings/str_replace_variation3-detect.phpt",
    "ext/standard/tests/strings/crc32_variation1-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation9-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation20-detect.phpt",
    "ext/standard/tests/strings/addcslashes_002-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation8-detect.phpt",
    "ext/standard/tests/strings/str_pad-detect.phpt",
    "ext/standard/tests/strings/stripos_variation12-detect.phpt",
    "ext/standard/tests/strings/printf-detect.phpt",
    "ext/standard/tests/strings/parse_str_basic3-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation11-detect.phpt",
    "ext/standard/tests/strings/get_html_translation_table_basic4-detect.phpt",
    "ext/standard/tests/strings/chunk_split-detect.phpt",
    "ext/standard/tests/strings/stripos_variation6-detect.phpt",
    "ext/standard/tests/strings/convert_uuencode_basic-detect.phpt",
    "ext/standard/tests/strings/strlen-detect.phpt",
    "ext/standard/tests/strings/html_entity_decode_koi8-r-detect.phpt",
    "ext/standard/tests/strings/get_html_translation_table_basic9-detect.phpt",
    "ext/standard/tests/strings/strspn_variation11-detect.phpt",
    "ext/standard/tests/strings/soundex_error-detect.phpt",
    "ext/standard/tests/strings/str_shuffle_variation1-detect.phpt",
    "ext/standard/tests/strings/wordwrap-detect.phpt",
    "ext/standard/tests/strings/nl_langinfo_basic-detect.phpt",
    "ext/standard/tests/strings/vprintf_basic2-detect.phpt",
    "ext/standard/tests/strings/get_html_translation_table_basic2-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation43-detect.phpt",
    "ext/standard/tests/strings/crc32_variation2-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation7-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation8-detect.phpt",
    "ext/standard/tests/strings/htmlentities-utf-detect.phpt",
    "ext/standard/tests/strings/chop_basic-detect.phpt",
    "ext/standard/tests/strings/hebrevc_variation2-detect.phpt",
    "ext/standard/tests/strings/bug47481-detect.phpt",
    "ext/standard/tests/strings/chop_variation1-detect.phpt",
    "ext/standard/tests/strings/bug37244-detect.phpt",
    "ext/standard/tests/strings/bug21338-detect.phpt",
    "ext/standard/tests/strings/stripslashes_variation1-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation15_64bit-detect.phpt",
    "ext/standard/tests/strings/strripos-detect.phpt",
    "ext/standard/tests/strings/strpos_number-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation14_64bit-detect.phpt",
    "ext/standard/tests/strings/strspn_variation4-detect.phpt",
    "ext/standard/tests/strings/strtr_variation5-detect.phpt",
    "ext/standard/tests/strings/str_replace_error-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation34-detect.phpt",
    "ext/standard/tests/strings/bug47322-detect.phpt",
    "ext/standard/tests/strings/strspn_variation12-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation12-detect.phpt",
    "ext/standard/tests/strings/strrev_variation3-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation49-detect.phpt",
    "ext/standard/tests/strings/sprintf_basic7-detect.phpt",
    "ext/standard/tests/strings/strip_tags_variation10-detect.phpt",
    "ext/standard/tests/strings/vfprintf_error2-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation2-detect.phpt",
    "ext/standard/tests/strings/convert_cyr_string_variation1-detect.phpt",
    "ext/standard/tests/strings/str_word_count-detect.phpt",
    "ext/standard/tests/strings/vfprintf_error3-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation51-detect.phpt",
    "ext/standard/tests/strings/sprintf_basic5-detect.phpt",
    "ext/standard/tests/strings/bug54055-detect.phpt",
    "ext/standard/tests/strings/parse_str_error1-detect.phpt",
    "ext/standard/tests/strings/bug42107-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation12-detect.phpt",
    "ext/standard/tests/strings/strncasecmp_variation3-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation21-detect.phpt",
    "ext/standard/tests/strings/bug61764-detect.phpt",
    "ext/standard/tests/strings/sprintf_basic2-detect.phpt",
    "ext/standard/tests/strings/strlen_error-detect.phpt",
    "ext/standard/tests/strings/strcspn_variation10-detect.phpt",
    "ext/standard/tests/strings/bug27295-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation15_64bit-detect.phpt",
    "ext/standard/tests/strings/nl2br_variation5-detect.phpt",
    "ext/standard/tests/strings/setlocale_variation1-detect.phpt",
    "ext/standard/tests/strings/strtr_variation9-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation7-detect.phpt",
    "ext/standard/tests/strings/rtrim_variation2-detect.phpt",
    "ext/standard/tests/strings/strtok_variation7-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation47-detect.phpt",
    "ext/standard/tests/strings/chunk_split_variation4-detect.phpt",
    "ext/standard/tests/strings/html_entity_decode_macroman-detect.phpt",
    "ext/standard/tests/strings/fprintf_variation_007_64bit-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation19_64bit-detect.phpt",
    "ext/standard/tests/strings/sscanf_basic3-detect.phpt",
    "ext/standard/tests/strings/bug22227-detect.phpt",
    "ext/standard/tests/strings/substr_count_error-detect.phpt",
    "ext/standard/tests/strings/strrpos_offset-detect.phpt",
    "ext/standard/tests/strings/strtoupper1-win32-detect.phpt",
    "ext/standard/tests/strings/bug38770-detect.phpt",
    "ext/standard/tests/strings/strrpos-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation4_64bit-detect.phpt",
    "ext/standard/tests/strings/chr_ord-detect.phpt",
    "ext/standard/tests/strings/crypt_blowfish_variation1-detect.phpt",
    "ext/standard/tests/strings/nl_langinfo_variation1-detect.phpt",
    "ext/standard/tests/strings/htmlspecialchars_decode_variation5-detect.phpt",
    "ext/standard/tests/strings/vsprintf_basic8-detect.phpt",
    "ext/standard/tests/strings/bug23894-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation10-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation9-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation40_64bit-detect.phpt",
    "ext/standard/tests/strings/count_chars_variation1-detect.phpt",
    "ext/standard/tests/strings/bug62443-detect.phpt",
    "ext/standard/tests/strings/printf_basic2-detect.phpt",
    "ext/standard/tests/strings/bug28386-detect.phpt",
    "ext/standard/tests/strings/vprintf_basic1-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation11_64bit-detect.phpt",
    "ext/standard/tests/strings/strtr_variation4-detect.phpt",
    "ext/standard/tests/strings/printf_basic7-detect.phpt",
    "ext/standard/tests/strings/htmlentities18-detect.phpt",
    "ext/standard/tests/strings/htmlspecialchars_decode_error-detect.phpt",
    "ext/standard/tests/strings/str_rot13_error-detect.phpt",
    "ext/standard/tests/strings/strip_tags_variation3-detect.phpt",
    "ext/standard/tests/strings/stripos_variation9-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation12_64bit-detect.phpt",
    "ext/standard/tests/strings/strncmp_variation6-detect.phpt",
    "ext/standard/tests/strings/bug28633-detect.phpt",
    "ext/standard/tests/strings/setlocale_variation4-detect.phpt",
    "ext/standard/tests/strings/strncasecmp_variation2-detect.phpt",
    "ext/standard/tests/strings/chr_error-detect.phpt",
    "ext/standard/tests/strings/get_html_translation_table_variation1-detect.phpt",
    "ext/standard/tests/strings/strcspn-detect.phpt",
    "ext/standard/tests/strings/substr-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation1-detect.phpt",
    "ext/standard/tests/strings/stristr_variation1-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation16_64bit-detect.phpt",
    "ext/standard/tests/strings/ucwords_variation3-detect.phpt",
    "ext/standard/tests/strings/crypt_blowfish_variation2-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation19-detect.phpt",
    "ext/standard/tests/strings/strrchr_variation7-detect.phpt",
    "ext/standard/tests/strings/setlocale_error-detect.phpt",
    "ext/standard/tests/strings/http_build_query_variation2-detect.phpt",
    "ext/standard/tests/strings/count_chars-detect.phpt",
    "ext/standard/tests/strings/crypt-detect.phpt",
    "ext/standard/tests/strings/bug38322-detect.phpt",
    "ext/standard/tests/strings/wordwrap_basic-detect.phpt",
    "ext/standard/tests/strings/sscanf_variation1-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation1-detect.phpt",
    "ext/standard/tests/strings/stripos_variation15-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation3-detect.phpt",
    "ext/standard/tests/strings/quotemeta_basic-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation42-detect.phpt",
    "ext/standard/tests/strings/str_shuffle_error-detect.phpt",
    "ext/standard/tests/strings/printf_64bit-detect.phpt",
    "ext/standard/tests/strings/parse_str_basic4-detect.phpt",
    "ext/standard/tests/strings/ucwords_variation4-detect.phpt",
    "ext/standard/tests/strings/trim_variation2-detect.phpt",
    "ext/standard/tests/strings/vfprintf_basic2-detect.phpt",
    "ext/standard/tests/strings/bug40432-detect.phpt",
    "ext/standard/tests/strings/nl2br-detect.phpt",
    "ext/standard/tests/strings/md5_basic2-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation2-detect.phpt",
    "ext/standard/tests/strings/crypt_blowfish-detect.phpt",
    "ext/standard/tests/strings/strspn_error-detect.phpt",
    "ext/standard/tests/strings/str_replace_basic-detect.phpt",
    "ext/standard/tests/strings/strncmp_variation4-detect.phpt",
    "ext/standard/tests/strings/bug20108-detect.phpt",
    "ext/standard/tests/strings/addslashes_variation2-detect.phpt",
    "ext/standard/tests/strings/bug36306-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation14-detect.phpt",
    "ext/standard/tests/strings/htmlentities07-detect.phpt",
    "ext/standard/tests/strings/strncasecmp_basic-detect.phpt",
    "ext/standard/tests/strings/str_replace_variation2-detect.phpt",
    "ext/standard/tests/strings/strspn_basic-detect.phpt",
    "ext/standard/tests/strings/pathinfo-detect.phpt",
    "ext/standard/tests/strings/htmlentities17-detect.phpt",
    "ext/standard/tests/strings/chunk_split_variation10-detect.phpt",
    "ext/standard/tests/strings/bug27675-detect.phpt",
    "ext/standard/tests/strings/sscanf_basic2-detect.phpt",
    "ext/standard/tests/strings/strstr2-detect.phpt",
    "ext/standard/tests/strings/004-detect.phpt",
    "ext/standard/tests/strings/bug22224-detect.phpt",
    "ext/standard/tests/strings/url_t-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation15-detect.phpt",
    "ext/standard/tests/strings/vprintf_basic3-detect.phpt",
    "ext/standard/tests/strings/chop_variation4-detect.phpt",
    "ext/standard/tests/strings/strcspn_error-detect.phpt",
    "ext/standard/tests/strings/strrchr_variation5-detect.phpt",
    "ext/standard/tests/strings/strip_tags_variation2-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation32-detect.phpt",
    "ext/standard/tests/strings/similar_text_error-detect.phpt",
    "ext/standard/tests/strings/bug21744-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation14_64bit-detect.phpt",
    "ext/standard/tests/strings/bug54721-detect.phpt",
    "ext/standard/tests/strings/vfprintf_basic4-detect.phpt",
    "ext/standard/tests/strings/chop_error-detect.phpt",
    "ext/standard/tests/strings/strip_tags_variation7-detect.phpt",
    "ext/standard/tests/strings/strspn_variation9-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation18-detect.phpt",
    "ext/standard/tests/strings/sscanf_basic4-detect.phpt",
    "ext/standard/tests/strings/sscanf_variation2-detect.phpt",
    "ext/standard/tests/strings/strcspn_variation3-detect.phpt",
    "ext/standard/tests/strings/htmlentities12-detect.phpt",
    "ext/standard/tests/strings/stripos_variation1-detect.phpt",
    "ext/standard/tests/strings/join_variation2-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation29-detect.phpt",
    "ext/standard/tests/strings/vsprintf_basic4-detect.phpt",
    "ext/standard/tests/strings/crypt_sha512-detect.phpt",
    "ext/standard/tests/strings/strrchr_variation10-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation17-detect.phpt",
    "ext/standard/tests/strings/printf_basic5-detect.phpt",
    "ext/standard/tests/strings/strcspn_variation11-detect.phpt",
    "ext/standard/tests/strings/strcspn_variation4-detect.phpt",
    "ext/standard/tests/strings/strspn_variation7-detect.phpt",
    "ext/standard/tests/strings/addcslashes_004-detect.phpt",
    "ext/standard/tests/strings/vprintf_error-detect.phpt",
    "ext/standard/tests/strings/strncmp_variation8-detect.phpt",
    "ext/standard/tests/strings/stripcslashes_error-detect.phpt",
    "ext/standard/tests/strings/strtok_variation3-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation3-detect.phpt",
    "ext/standard/tests/strings/convert_cyr_string_basic-detect.phpt",
    "ext/standard/tests/strings/bug22207-detect.phpt",
    "ext/standard/tests/strings/str_split_variation4-detect.phpt",
    "ext/standard/tests/strings/basename_variation-detect.phpt",
    "ext/standard/tests/strings/strcspn_variation7-detect.phpt",
    "ext/standard/tests/strings/convert_cyr_string_error-detect.phpt",
    "ext/standard/tests/strings/vprintf_basic7-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation6-detect.phpt",
    "ext/standard/tests/strings/htmlentities02-detect.phpt",
    "ext/standard/tests/strings/stripos_error-detect.phpt",
    "ext/standard/tests/strings/bug26819-detect.phpt",
    "ext/standard/tests/strings/stripos_variation2-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation16-detect.phpt",
    "ext/standard/tests/strings/chunk_split_variation3-detect.phpt",
    "ext/standard/tests/strings/count_chars_variation2-detect.phpt",
    "ext/standard/tests/strings/sprintf_basic1-detect.phpt",
    "ext/standard/tests/strings/bug50052-detect.phpt",
    "ext/standard/tests/strings/nl2br_variation4-detect.phpt",
    "ext/standard/tests/strings/strripos_basic1-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation16_64bit-detect.phpt",
    "ext/standard/tests/strings/vsprintf_basic7_64bit-detect.phpt",
    "ext/standard/tests/strings/stripslashes_basic-detect.phpt",
    "ext/standard/tests/strings/sscanf_error-detect.phpt",
    "ext/standard/tests/strings/strtok_variation6-detect.phpt",
    "ext/standard/tests/strings/bug54238-detect.phpt",
    "ext/standard/tests/strings/strtr_variation2-detect.phpt",
    "ext/standard/tests/strings/htmlentities23-detect.phpt",
    "ext/standard/tests/strings/strip_tags_variation5-detect.phpt",
    "ext/standard/tests/strings/printf_basic3-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation11_64bit-detect.phpt",
    "ext/standard/tests/strings/htmlentities01-detect.phpt",
    "ext/standard/tests/strings/quoted_printable_encode_001-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation5-detect.phpt",
    "ext/standard/tests/strings/strtr_variation3-detect.phpt",
    "ext/standard/tests/strings/strncasecmp_variation4-detect.phpt",
    "ext/standard/tests/strings/strcspn_variation8-detect.phpt",
    "ext/standard/tests/strings/htmlentities22-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation44-detect.phpt",
    "ext/standard/tests/strings/bug21453-detect.phpt",
    "ext/standard/tests/strings/str_pad_variation2-detect.phpt",
    "ext/standard/tests/strings/dirname_basic-detect.phpt",
    "ext/standard/tests/strings/crc32_error-detect.phpt",
    "ext/standard/tests/strings/bug45485-detect.phpt",
    "ext/standard/tests/strings/chop_variation2-detect.phpt",
    "ext/standard/tests/strings/bug51059-detect.phpt",
    "ext/standard/tests/strings/get_html_translation_table_basic3-detect.phpt",
    "ext/standard/tests/strings/bug53319-detect.phpt",
    "ext/standard/tests/strings/bug54454-detect.phpt",
    "ext/standard/tests/strings/strtoupper-detect.phpt",
    "ext/standard/tests/strings/vfprintf_error1-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation13-detect.phpt",
    "ext/standard/tests/strings/sha1_basic-detect.phpt",
    "ext/standard/tests/strings/explode_variation3-detect.phpt",
    "ext/standard/tests/strings/vfprintf_basic1-detect.phpt",
    "ext/standard/tests/strings/http_build_query_variation1-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation23-detect.phpt",
    "ext/standard/tests/strings/vprintf_basic6-detect.phpt",
    "ext/standard/tests/strings/hebrevc_basic-detect.phpt",
    "ext/standard/tests/strings/quoted_printable_decode_variation1-detect.phpt",
    "ext/standard/tests/strings/vfprintf_basic7-detect.phpt",
    "ext/standard/tests/strings/chunk_split_error-detect.phpt",
    "ext/standard/tests/strings/bug44703-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation5-detect.phpt",
    "ext/standard/tests/strings/bug20169-detect.phpt",
    "ext/standard/tests/strings/fprintf_variation_009-detect.phpt",
    "ext/standard/tests/strings/bug46578-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation16-detect.phpt",
    "ext/standard/tests/strings/sprintf_error-detect.phpt",
    "ext/standard/tests/strings/ucwords_variation2-detect.phpt",
    "ext/standard/tests/strings/stripos_basic2-detect.phpt",
    "ext/standard/tests/strings/strtr_variation7-detect.phpt",
    "ext/standard/tests/strings/stripos_variation4-detect.phpt",
    "ext/standard/tests/strings/strip_tags_error-detect.phpt",
    "ext/standard/tests/strings/strrchr-detect.phpt",
    "ext/standard/tests/strings/bug37262-detect.phpt",
    "ext/standard/tests/strings/fprintf_variation_007-detect.phpt",
    "ext/standard/tests/strings/strlen_variation1-detect.phpt",
    "ext/standard/tests/strings/fprintf_error-detect.phpt",
    "ext/standard/tests/strings/quoted_printable_encode_002-detect.phpt",
    "ext/standard/tests/strings/fprintf_variation_004-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation35_64bit-detect.phpt",
    "ext/standard/tests/strings/bug36392-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation5-detect.phpt",
    "ext/standard/tests/strings/number_format_error-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation18-detect.phpt",
    "ext/standard/tests/strings/dirname_variation-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation41-detect.phpt",
    "ext/standard/tests/strings/strrchr_variation3-detect.phpt",
    "ext/standard/tests/strings/ord_variation1-detect.phpt",
    "ext/standard/tests/strings/strtr-detect.phpt",
    "ext/standard/tests/strings/chr_variation1-detect.phpt",
    "ext/standard/tests/strings/get_html_translation_table_basic7-detect.phpt",
    "ext/standard/tests/strings/strrpos_error-detect.phpt",
    "ext/standard/tests/strings/strncmp_variation1-detect.phpt",
    "ext/standard/tests/strings/hebrev_variation1-detect.phpt",
    "ext/standard/tests/strings/strstr-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation11-detect.phpt",
    "ext/standard/tests/strings/vprintf_basic5-detect.phpt",
    "ext/standard/tests/strings/htmlspecialchars_decode_variation4-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation24-detect.phpt",
    "ext/standard/tests/strings/printf_variation2-detect.phpt",
    "ext/standard/tests/strings/strripos_variation5-detect.phpt",
    "ext/standard/tests/strings/strspn_variation1-detect.phpt",
    "ext/standard/tests/strings/sprintf_basic7_64bit-detect.phpt",
    "ext/standard/tests/strings/strspn-detect.phpt",
    "ext/standard/tests/strings/trim_variation1-detect.phpt",
    "ext/standard/tests/strings/bug55871-detect.phpt",
    "ext/standard/tests/strings/str_pad_variation1-detect.phpt",
    "ext/standard/tests/strings/stripos_variation5-detect.phpt",
    "ext/standard/tests/strings/strrpos_basic2-detect.phpt",
    "ext/standard/tests/strings/bug39621-detect.phpt",
    "ext/standard/tests/strings/fprintf_variation_006_64bit-detect.phpt",
    "ext/standard/tests/strings/quoted_printable_decode_basic-detect.phpt",
    "ext/standard/tests/strings/chunk_split_variation6-detect.phpt",
    "ext/standard/tests/strings/strtok_variation2-detect.phpt",
    "ext/standard/tests/strings/strtok_error-detect.phpt",
    "ext/standard/tests/strings/html_entity_decode_win1252-detect.phpt",
    "ext/standard/tests/strings/strripos_offset-detect.phpt",
    "ext/standard/tests/strings/006-detect.phpt",
    "ext/standard/tests/strings/str_replace-detect.phpt",
    "ext/standard/tests/strings/html_entity_decode2-detect.phpt",
    "ext/standard/tests/strings/strncasecmp_variation11-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation14_64bit-detect.phpt",
    "ext/standard/tests/strings/setlocale_basic1-detect.phpt",
    "ext/standard/tests/strings/ltrim_error-detect.phpt",
    "ext/standard/tests/strings/stripos-detect.phpt",
    "ext/standard/tests/strings/strripos_basic2-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation28-detect.phpt",
    "ext/standard/tests/strings/strncasecmp_variation7-detect.phpt",
    "ext/standard/tests/strings/convert_cyr_string-detect.phpt",
    "ext/standard/tests/strings/implode-detect.phpt",
    "ext/standard/tests/strings/unpack_error-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation22-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation27-detect.phpt",
    "ext/standard/tests/strings/ord_basic-detect.phpt",
    "ext/standard/tests/strings/join_variation5-detect.phpt",
    "ext/standard/tests/strings/setlocale_variation2-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation46-detect.phpt",
    "ext/standard/tests/strings/bug27276-detect.phpt",
    "ext/standard/tests/strings/vsprintf_basic2-detect.phpt",
    "ext/standard/tests/strings/strcspn_basic-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation30-detect.phpt",
    "ext/standard/tests/strings/bug40915-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation39-detect.phpt",
    "ext/standard/tests/strings/stripcslashes_basic-detect.phpt",
    "ext/standard/tests/strings/substr_replace_error-detect.phpt",
    "ext/standard/tests/strings/explode_variation2-detect.phpt",
    "ext/standard/tests/strings/htmlentities13-detect.phpt",
    "ext/standard/tests/strings/explode_variation6-detect.phpt",
    "ext/standard/tests/strings/strrev_variation4-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation5-detect.phpt",
    "ext/standard/tests/strings/strncmp_variation2-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation14-detect.phpt",
    "ext/standard/tests/strings/vsprintf_basic3-detect.phpt",
    "ext/standard/tests/strings/strrchr_error-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation10-detect.phpt",
    "ext/standard/tests/strings/strncasecmp_variation8-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation15-detect.phpt",
    "ext/standard/tests/strings/convert_uudecode_variation1-detect.phpt",
    "ext/standard/tests/strings/stripos_basic1-detect.phpt",
    "ext/standard/tests/strings/hebrev_variation2-detect.phpt",
    "ext/standard/tests/strings/str_split_basic-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation1-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation9-detect.phpt",
    "ext/standard/tests/strings/htmlentities16-detect.phpt",
    "ext/standard/tests/strings/sprintf_basic8_64bit-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation6-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation35-detect.phpt",
    "ext/standard/tests/strings/sscanf_basic1-detect.phpt",
    "ext/standard/tests/strings/bug24312-detect.phpt",
    "ext/standard/tests/strings/htmlentities03-detect.phpt",
    "ext/standard/tests/strings/chop_variation3-detect.phpt",
    "ext/standard/tests/strings/bug35817-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation4_64bit-detect.phpt",
    "ext/standard/tests/strings/soundex-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation8-detect.phpt",
    "ext/standard/tests/strings/parse_str_basic1-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation13_64bit-detect.phpt",
    "ext/standard/tests/strings/str_split_variation2-detect.phpt",
    "ext/standard/tests/strings/sha1_variation2-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation12_64bit-detect.phpt",
    "ext/standard/tests/strings/strncmp_variation3-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation41_64bit-detect.phpt",
    "ext/standard/tests/strings/strspn_variation2-detect.phpt",
    "ext/standard/tests/strings/bin2hex_error-detect.phpt",
    "ext/standard/tests/strings/strpbrk_error-detect.phpt",
    "ext/standard/tests/strings/ucwords_error-detect.phpt",
    "ext/standard/tests/strings/strncmp_variation5-detect.phpt",
    "ext/standard/tests/strings/strrchr_variation6-detect.phpt",
    "ext/standard/tests/strings/explode1-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation4-detect.phpt",
    "ext/standard/tests/strings/strncasecmp_variation6-detect.phpt",
    "ext/standard/tests/strings/chunk_split_variation2-detect.phpt",
    "ext/standard/tests/strings/vsprintf_basic7-detect.phpt",
    "ext/standard/tests/strings/explode_variation1-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation14-detect.phpt",
    "ext/standard/tests/strings/sprintf_basic6-detect.phpt",
    "ext/standard/tests/strings/money_format_error-detect.phpt",
    "ext/standard/tests/strings/rtrim_error-detect.phpt",
    "ext/standard/tests/strings/fprintf_variation_003_64bit-detect.phpt",
    "ext/standard/tests/strings/htmlentities_html5-detect.phpt",
    "ext/standard/tests/strings/explode_variation5-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation10-detect.phpt",
    "ext/standard/tests/strings/get_html_translation_table_variation2-detect.phpt",
    "ext/standard/tests/strings/crc32_variation3-detect.phpt",
    "ext/standard/tests/strings/addslashes_basic-detect.phpt",
    "ext/standard/tests/strings/strrev_variation2-detect.phpt",
    "ext/standard/tests/strings/setlocale_basic2-detect.phpt",
    "ext/standard/tests/strings/md5_error-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation53-detect.phpt",
    "ext/standard/tests/strings/printf_basic9-detect.phpt",
    "ext/standard/tests/strings/sscanf_basic5-detect.phpt",
    "ext/standard/tests/strings/007-detect.phpt",
    "ext/standard/tests/strings/sprintf_basic3-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation15-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation26-detect.phpt",
    "ext/standard/tests/strings/stristr_error-detect.phpt",
    "ext/standard/tests/strings/strncasecmp_variation5-detect.phpt",
    "ext/standard/tests/strings/get_meta_tags-detect.phpt",
    "ext/standard/tests/strings/strval_basic-detect.phpt",
    "ext/standard/tests/strings/html_entity_decode3-detect.phpt",
    "ext/standard/tests/strings/strcoll_error-detect.phpt",
    "ext/standard/tests/strings/convert_uuencode_error-detect.phpt",
    "ext/standard/tests/strings/bug54332-detect.phpt",
    "ext/standard/tests/strings/nl2br_variation2-detect.phpt",
    "ext/standard/tests/strings/bug27278-detect.phpt",
    "ext/standard/tests/strings/strcasecmp-detect.phpt",
    "ext/standard/tests/strings/bin2hex_basic-detect.phpt",
    "ext/standard/tests/strings/ltrim_basic-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation11-detect.phpt",
    "ext/standard/tests/strings/sprintf_basic9-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation16-detect.phpt",
    "ext/standard/tests/strings/crc32_variation4-detect.phpt",
    "ext/standard/tests/strings/vfprintf_basic6-detect.phpt",
    "ext/standard/tests/strings/strrchr_variation9-detect.phpt",
    "ext/standard/tests/strings/http_build_query_variation3-detect.phpt",
    "ext/standard/tests/strings/bug40637-detect.phpt",
    "ext/standard/tests/strings/pack-detect.phpt",
    "ext/standard/tests/strings/strip_tags_variation11-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation11-detect.phpt",
    "ext/standard/tests/strings/strval_variation2-detect.phpt",
    "ext/standard/tests/strings/chunk_split_variation1-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation9-detect.phpt",
    "ext/standard/tests/strings/strspn_variation3-detect.phpt",
    "ext/standard/tests/strings/strrev_basic-detect.phpt",
    "ext/standard/tests/strings/str_split_variation6-detect.phpt",
    "ext/standard/tests/strings/strrpos_basic1-detect.phpt",
    "ext/standard/tests/strings/htmlspecialchars_decode_variation1-detect.phpt",
    "ext/standard/tests/strings/htmlentities-detect.phpt",
    "ext/standard/tests/strings/md5_basic1-detect.phpt",
    "ext/standard/tests/strings/htmlentities20-detect.phpt",
    "ext/standard/tests/strings/strspn_variation6-detect.phpt",
    "ext/standard/tests/strings/stripos_variation11-detect.phpt",
    "ext/standard/tests/strings/get_html_translation_table_basic8-detect.phpt",
    "ext/standard/tests/strings/strip_tags_variation6-detect.phpt",
    "ext/standard/tests/strings/print_basic-detect.phpt",
    "ext/standard/tests/strings/str_split_variation5-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation50-detect.phpt",
    "ext/standard/tests/strings/bug50847-detect.phpt",
    "ext/standard/tests/strings/str_rot13_basic-detect.phpt",
    "ext/standard/tests/strings/lcfirst-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation19_64bit-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation14-detect.phpt",
    "ext/standard/tests/strings/strip_tags_basic2-detect.phpt",
    "ext/standard/tests/strings/get_html_translation_table_basic6-detect.phpt",
    "ext/standard/tests/strings/vfprintf_basic5-detect.phpt",
    "ext/standard/tests/strings/fprintf_variation_006-detect.phpt",
    "ext/standard/tests/strings/bug48709-detect.phpt",
    "ext/standard/tests/strings/htmlentities19-detect.phpt",
    "ext/standard/tests/strings/bug24281-detect.phpt",
    "ext/standard/tests/strings/sscanf_basic7-detect.phpt",
    "ext/standard/tests/strings/str_pad_variation5-detect.phpt",
    "ext/standard/tests/strings/strtr_error-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation33-detect.phpt",
    "ext/standard/tests/strings/strncasecmp_error-detect.phpt",
    "ext/standard/tests/strings/bug33605-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation3-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation13-detect.phpt",
    "ext/standard/tests/strings/fprintf_variation_003-detect.phpt",
    "ext/standard/tests/strings/bug53021-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation15-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation4-detect.phpt",
    "ext/standard/tests/strings/ucwords_basic-detect.phpt",
    "ext/standard/tests/strings/setlocale_basic3-detect.phpt",
    "ext/standard/tests/strings/substr_count_basic-detect.phpt",
    "ext/standard/tests/strings/ucwords_variation1-detect.phpt",
    "ext/standard/tests/strings/str_pad_variation3-detect.phpt",
    "ext/standard/tests/strings/rtrim_basic-detect.phpt",
    "ext/standard/tests/strings/strtolower-win32-detect.phpt",
    "ext/standard/tests/strings/addcslashes_003-detect.phpt",
    "ext/standard/tests/strings/similar_text_basic-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation4-detect.phpt",
    "ext/standard/tests/strings/show_source_variation1-detect.phpt",
    "ext/standard/tests/strings/addslashes_error-detect.phpt",
    "ext/standard/tests/strings/str_shuffle-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation10-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation17-detect.phpt",
    "ext/standard/tests/strings/sscanf_basic8-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation4-detect.phpt",
    "ext/standard/tests/strings/ucfirst-detect.phpt",
    "ext/standard/tests/strings/strripos_variation3-detect.phpt",
    "ext/standard/tests/strings/stristr-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation19-detect.phpt",
    "ext/standard/tests/strings/strnatcmp_error-detect.phpt",
    "ext/standard/tests/strings/vfprintf_error4-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation15_64bit-detect.phpt",
    "ext/standard/tests/strings/htmlspecialchars_basic-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation13-detect.phpt",
    "ext/standard/tests/strings/add-and-stripcslashes-detect.phpt",
    "ext/standard/tests/strings/stripslashes_variation2-detect.phpt",
    "ext/standard/tests/strings/stripslashes_variation3-detect.phpt",
    "ext/standard/tests/strings/explode-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation6-detect.phpt",
    "ext/standard/tests/strings/bug27457-detect.phpt",
    "ext/standard/tests/strings/strtok_variation5-detect.phpt",
    "ext/standard/tests/strings/strtok_variation4-detect.phpt",
    "ext/standard/tests/strings/nl2br_variation3-detect.phpt",
    "ext/standard/tests/strings/vsprintf_basic9-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation37-detect.phpt",
    "ext/standard/tests/strings/str_split_variation2_64bit-detect.phpt",
    "ext/standard/tests/strings/chunk_split_variation5-detect.phpt",
    "ext/standard/tests/strings/htmlentities10-detect.phpt",
    "ext/standard/tests/strings/show_source_variation2-detect.phpt",
    "ext/standard/tests/strings/str_split_variation7_64bit-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation13_64bit-detect.phpt",
    "ext/standard/tests/strings/chunk_split_variation7-detect.phpt",
    "ext/standard/tests/strings/bug36148-detect.phpt",
    "ext/standard/tests/strings/strlen_basic-detect.phpt",
    "ext/standard/tests/strings/strnatcasecmp_basic-detect.phpt",
    "ext/standard/tests/strings/unpack-detect.phpt",
    "ext/standard/tests/strings/printf_error-detect.phpt",
    "ext/standard/tests/strings/strrchr_variation2-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation36-detect.phpt",
    "ext/standard/tests/strings/strcspn_variation1-detect.phpt",
    "ext/standard/tests/strings/bug23650-detect.phpt",
    "ext/standard/tests/strings/strrchr_basic-detect.phpt",
    "ext/standard/tests/strings/chunk_split_variation13-detect.phpt",
    "ext/standard/tests/strings/bug40754-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation3-detect.phpt",
    "ext/standard/tests/strings/007-win32-detect.phpt",
    "ext/standard/tests/strings/html_entity_decode1-detect.phpt",
    "ext/standard/tests/strings/dirname_error-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation40-detect.phpt",
    "ext/standard/tests/strings/trim_basic-detect.phpt",
    "ext/standard/tests/strings/bug20934-detect.phpt",
    "ext/standard/tests/strings/strncmp_variation7-detect.phpt",
    "ext/standard/tests/strings/join_variation3-detect.phpt",
    "ext/standard/tests/strings/levenshtein-detect.phpt",
    "ext/standard/tests/strings/htmlspecialchars_decode_variation7-detect.phpt",
    "ext/standard/tests/strings/money_format_variation1-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation1-detect.phpt",
    "ext/standard/tests/strings/sha1_error-detect.phpt",
    "ext/standard/tests/strings/number_format_basic-detect.phpt",
    "ext/standard/tests/strings/wordwrap_variation3-detect.phpt",
    "ext/standard/tests/strings/htmlentities15-detect.phpt",
    "ext/standard/tests/strings/stripos_variation8-detect.phpt",
    "ext/standard/tests/strings/bug47168-detect.phpt",
    "ext/standard/tests/strings/fprintf_variation_001-detect.phpt",
    "ext/standard/tests/strings/md5-detect.phpt",
    "ext/standard/tests/strings/strrchr_variation12-detect.phpt",
    "ext/standard/tests/strings/bug62462-detect.phpt",
    "ext/standard/tests/strings/parse_str_basic2-detect.phpt",
    "ext/standard/tests/strings/strnatcasecmp_error-detect.phpt",
    "ext/standard/tests/strings/stripos_variation3-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation45-detect.phpt",
    "ext/standard/tests/strings/print_variation1-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation7-detect.phpt",
    "ext/standard/tests/strings/strip_tags_variation8-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation13_64bit-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation5-detect.phpt",
    "ext/standard/tests/strings/strcspn_variation2-detect.phpt",
    "ext/standard/tests/strings/strripos_variation1-detect.phpt",
    "ext/standard/tests/strings/bug39873-detect.phpt",
    "ext/standard/tests/strings/bug25671-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation31-detect.phpt",
    "ext/standard/tests/strings/bug21730-detect.phpt",
    "ext/standard/tests/strings/vsprintf_basic5-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation12_64bit-detect.phpt",
    "ext/standard/tests/strings/str_getcsv_001-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation38-detect.phpt",
    "ext/standard/tests/strings/htmlentities_html4-detect.phpt",
    "ext/standard/tests/strings/stristr2-detect.phpt",
    "ext/standard/tests/strings/htmlentities05-detect.phpt",
    "ext/standard/tests/strings/bug44242-detect.phpt",
    "ext/standard/tests/strings/wordwrap_error-detect.phpt",
    "ext/standard/tests/strings/strspn_variation10-detect.phpt",
    "ext/standard/tests/strings/strncasecmp_variation1-detect.phpt",
    "ext/standard/tests/strings/strcoll-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation17-detect.phpt",
    "ext/standard/tests/strings/str_shuffle_basic-detect.phpt",
    "ext/standard/tests/strings/htmlentities21-detect.phpt",
    "ext/standard/tests/strings/chunk_split_variation9-detect.phpt",
    "ext/standard/tests/strings/strval-detect.phpt",
    "ext/standard/tests/strings/bug61374-detect.phpt",
    "ext/standard/tests/strings/strrchr_variation4-detect.phpt",
    "ext/standard/tests/strings/stristr_variation2-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation16-detect.phpt",
    "ext/standard/tests/strings/strnatcasecmp_variation1-detect.phpt",
    "ext/standard/tests/strings/str_replace_variation1-detect.phpt",
    "ext/standard/tests/strings/htmlspecialchars_decode_variation2-detect.phpt",
    "ext/standard/tests/strings/strtok_basic-detect.phpt",
    "ext/standard/tests/strings/htmlentities08-detect.phpt",
    "ext/standard/tests/strings/strrev_variation1-detect.phpt",
    "ext/standard/tests/strings/htmlspecialchars_decode_variation6-detect.phpt",
    "ext/standard/tests/strings/bug40704-detect.phpt",
    "ext/standard/tests/strings/join_variation6-detect.phpt",
    "ext/standard/tests/strings/sha1raw-detect.phpt",
    "ext/standard/tests/strings/count_chars_basic-detect.phpt",
    "ext/standard/tests/strings/htmlentities11-detect.phpt",
    "ext/standard/tests/strings/bug24208-detect.phpt",
    "ext/standard/tests/strings/strtolower-detect.phpt",
    "ext/standard/tests/strings/printf_basic6-detect.phpt",
    "ext/standard/tests/strings/highlight_file-detect.phpt",
    "ext/standard/tests/strings/quotemeta_error-detect.phpt",
    "ext/standard/tests/strings/chop_variation5-detect.phpt",
    "ext/standard/tests/strings/add-and-stripslashes-detect.phpt",
    "ext/standard/tests/strings/crc32-detect.phpt",
    "ext/standard/tests/strings/strncasecmp_variation9-detect.phpt",
    "ext/standard/tests/strings/html_entity_decode_html5-detect.phpt",
    "ext/standard/tests/strings/quoted_printable_decode_error-detect.phpt",
    "ext/standard/tests/strings/strcspn_variation12-detect.phpt",
    "ext/standard/tests/strings/strrchr_variation11-detect.phpt",
    "ext/standard/tests/strings/str_split_error-detect.phpt",
    "ext/standard/tests/strings/strip_tags_variation1-detect.phpt",
    "ext/standard/tests/strings/sha1_variation1-detect.phpt",
    "ext/standard/tests/strings/addcslashes_001-detect.phpt",
    "ext/standard/tests/strings/html_entity_decode_iso8859-15-detect.phpt",
    "ext/standard/tests/strings/strncmp_error-detect.phpt",
    "ext/standard/tests/strings/moneyformat-detect.phpt",
    "ext/standard/tests/strings/stripos_variation13-detect.phpt",
    "ext/standard/tests/strings/crc32_basic-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation13-detect.phpt",
    "ext/standard/tests/strings/bug33076-detect.phpt",
    "ext/standard/tests/strings/wordwrap_variation4-detect.phpt",
    "ext/standard/tests/strings/vfprintf_basic7_64bit-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation12-detect.phpt",
    "ext/standard/tests/strings/sprintf_basic4-detect.phpt",
    "ext/standard/tests/strings/str_pad_variation4-detect.phpt",
    "ext/standard/tests/strings/bug29538-detect.phpt",
    "ext/standard/tests/strings/printf_basic4-detect.phpt",
    "ext/standard/tests/strings/vprintf_basic8-detect.phpt",
    "ext/standard/tests/strings/html_entity_decode_html4-detect.phpt",
    "ext/standard/tests/strings/stripos_variation7-detect.phpt",
    "ext/standard/tests/strings/fprintf_variation_008-detect.phpt",
    "ext/standard/tests/strings/bin2hex_variation1-detect.phpt",
    "ext/standard/tests/strings/htmlspecialchars-detect.phpt",
    "ext/standard/tests/strings/count_chars_error-detect.phpt",
    "ext/standard/tests/strings/crypt_sha256-detect.phpt",
    "ext/standard/tests/strings/bug47443-detect.phpt",
    "ext/standard/tests/strings/hebrevc_variation1-detect.phpt",
    "ext/standard/tests/strings/hebrev_basic-detect.phpt",
    "ext/standard/tests/strings/htmlspecialchars_decode_basic-detect.phpt",
    "ext/standard/tests/strings/bug51899-detect.phpt",
    "ext/standard/tests/strings/ord_error-detect.phpt",
    "ext/standard/tests/strings/bug20927-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation29_64bit-detect.phpt",
    "ext/standard/tests/strings/join_basic-detect.phpt",
    "ext/standard/tests/strings/strncmp_variation9-detect.phpt",
    "ext/standard/tests/strings/strcspn_variation5-detect.phpt",
    "ext/standard/tests/strings/htmlspecialchars_decode_variation3-detect.phpt",
    "ext/standard/tests/strings/strpbrk_basic-detect.phpt",
    "ext/standard/tests/strings/stristr_basic-detect.phpt",
    "ext/standard/tests/strings/ltrim-detect.phpt",
    "ext/standard/tests/strings/bug54322-detect.phpt",
    "ext/standard/tests/strings/bug43927-detect.phpt",
    "ext/standard/tests/strings/chr_basic-detect.phpt",
    "ext/standard/tests/strings/vfprintf_basic9-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation21-detect.phpt",
    "ext/standard/tests/strings/strtr_basic-detect.phpt",
    "ext/standard/tests/strings/strspn_variation8-detect.phpt",
    "ext/standard/tests/strings/strcmp-detect.phpt",
    "ext/standard/tests/strings/bug55674-detect.phpt",
    "ext/standard/tests/strings/bug20261-detect.phpt",
    "ext/standard/tests/strings/implode1-detect.phpt",
    "ext/standard/tests/strings/sscanf_basic6-detect.phpt",
    "ext/standard/tests/strings/strip_tags_basic1-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation18-detect.phpt",
    "ext/standard/tests/strings/basename_error-detect.phpt",
    "ext/standard/tests/strings/explode_bug-detect.phpt",
    "ext/standard/tests/strings/get_html_translation_table_basic5-detect.phpt",
    "ext/standard/tests/strings/bug60801-detect.phpt",
    "ext/standard/tests/strings/substr_count_variation_002-detect.phpt",
    "ext/standard/tests/strings/htmlentities04-detect.phpt",
    "ext/standard/tests/strings/str_repeat-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation2-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation12-detect.phpt",
    "ext/standard/tests/strings/htmlentities09-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation18-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation48-detect.phpt",
    "ext/standard/tests/strings/substr_replace_array-detect.phpt",
    "ext/standard/tests/strings/bug25707-detect.phpt",
    "ext/standard/tests/strings/strrchr_variation8-detect.phpt",
    "ext/standard/tests/strings/strcspn_variation9-detect.phpt",
    "ext/standard/tests/strings/bug24098-detect.phpt",
    "ext/standard/tests/strings/explode_error-detect.phpt",
    "ext/standard/tests/strings/soundex_basic-detect.phpt",
    "ext/standard/tests/strings/strpos-detect.phpt",
    "ext/standard/tests/strings/html_entity_decode_cp866-detect.phpt",
    "ext/standard/tests/strings/strrev_error-detect.phpt",
    "ext/standard/tests/strings/str_split_variation7-detect.phpt",
    "ext/standard/tests/strings/str_word_count1-detect.phpt",
    "ext/standard/tests/strings/join_variation4-detect.phpt",
    "ext/standard/tests/strings/strrev-detect.phpt",
    "ext/standard/tests/strings/bug47842-detect.phpt",
    "ext/standard/tests/strings/strnatcmp_basic-detect.phpt",
    "ext/standard/tests/strings/stripcslashes_variation1-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation3-detect.phpt",
    "ext/standard/tests/strings/convert_uudecode_error-detect.phpt",
    "ext/standard/tests/strings/md5_file-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation4_64bit-detect.phpt",
    "ext/standard/tests/strings/bug29075-detect.phpt",
    "ext/standard/tests/strings/htmlentities-utf-3-detect.phpt",
    "ext/standard/tests/strings/sprintf_f_2-detect.phpt",
    "ext/standard/tests/strings/strripos_variation2-detect.phpt",
    "ext/standard/tests/strings/strncmp_basic-detect.phpt",
    "ext/standard/tests/strings/strripos_error-detect.phpt",
    "ext/standard/tests/strings/str_split_variation3-detect.phpt",
    "ext/standard/tests/strings/vprintf_basic4-detect.phpt",
    "ext/standard/tests/strings/crypt_variation1-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation28_64bit-detect.phpt",
    "ext/standard/tests/strings/rtrim-detect.phpt",
    "ext/standard/tests/strings/html_entity_decode_iso8859-5-detect.phpt",
    "ext/standard/tests/strings/stripos_variation10-detect.phpt",
    "ext/standard/tests/strings/join_error-detect.phpt",
    "ext/standard/tests/strings/vprintf_variation2-detect.phpt",
    "ext/standard/tests/strings/chunk_split_basic-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation34_64bit-detect.phpt",
    "ext/standard/tests/strings/chunk_split_variation8-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation7-detect.phpt",
    "ext/standard/tests/strings/printf_basic8-detect.phpt",
    "ext/standard/tests/strings/strip_tags_variation9-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation14-detect.phpt",
    "ext/standard/tests/strings/htmlentities-utf-2-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation44_64bit-detect.phpt",
    "ext/standard/tests/strings/printf_basic1-detect.phpt",
    "ext/standard/tests/strings/htmlentities24-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation11-detect.phpt",
    "ext/standard/tests/strings/addslashes_variation1-detect.phpt",
    "ext/standard/tests/strings/vfprintf_basic8-detect.phpt",
    "ext/standard/tests/strings/bug36944-detect.phpt",
    "ext/standard/tests/strings/bin2hex-detect.phpt",
    "ext/standard/tests/strings/bug29119-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation6-detect.phpt",
    "ext/standard/tests/strings/vsprintf_error-detect.phpt",
    "ext/standard/tests/strings/join_variation1-detect.phpt",
    "ext/standard/tests/strings/vsprintf_basic6-detect.phpt",
    "ext/standard/tests/strings/bug61660-detect.phpt",
    "ext/standard/tests/strings/get_html_translation_table_basic1-detect.phpt",
    "ext/standard/tests/strings/fprintf_variation_008_64bit-detect.phpt",
    "ext/standard/tests/strings/vfprintf_basic-detect.phpt",
    "ext/standard/tests/strings/bug26817-detect.phpt",
    "ext/standard/tests/strings/htmlentities14-detect.phpt",
    "ext/standard/tests/strings/php_strip_whitespace-detect.phpt",
    "ext/standard/tests/strings/vprintf_basic7_64bit-detect.phpt",
    "ext/standard/tests/strings/md5raw-detect.phpt",
    "ext/standard/tests/strings/chunk_split_variation12-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation19_64bit-detect.phpt",
    "ext/standard/tests/strings/wordwrap_variation2-detect.phpt",
    "ext/standard/tests/strings/rtrim_variation1-detect.phpt",
    "ext/standard/tests/strings/strrpos_variation6-detect.phpt",
    "ext/standard/tests/strings/str_ireplace-detect.phpt",
    "ext/standard/tests/strings/bug49785-detect.phpt",
    "ext/standard/tests/strings/vprintf_basic9-detect.phpt",
    "ext/standard/tests/strings/strcspn_variation6-detect.phpt",
    "ext/standard/tests/strings/substr_compare-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation17-detect.phpt",
    "ext/standard/tests/strings/bug34214-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation25-detect.phpt",
    "ext/standard/tests/strings/bug42208-detect.phpt",
    "ext/standard/tests/strings/basename-detect.phpt",
    "ext/standard/tests/strings/bug39032-detect.phpt",
    "ext/standard/tests/strings/nl2br_variation1-detect.phpt",
    "ext/standard/tests/strings/money_format_variation2-detect.phpt",
    "ext/standard/tests/strings/strip_tags-detect.phpt",
    "ext/standard/tests/strings/trim-detect.phpt",
    "ext/standard/tests/strings/strtok_variation1-detect.phpt",
    "ext/standard/tests/strings/bug26878-detect.phpt",
    "ext/standard/tests/strings/html_entity_decode_win1251-detect.phpt",
    "ext/standard/tests/strings/substr_count_variation_001-detect.phpt",
    "ext/standard/tests/strings/metaphone-detect.phpt",
    "ext/standard/tests/strings/basename_basic-detect.phpt",
    "ext/standard/tests/strings/wordwrap_variation1-detect.phpt",
    "ext/standard/tests/strings/stripslashes_error-detect.phpt",
    "ext/standard/tests/strings/vsprintf_variation4-detect.phpt",
    "ext/standard/tests/strings/sprintf_variation4_64bit-detect.phpt",
    "ext/standard/tests/strings/sprintf_basic8-detect.phpt",
    "ext/standard/tests/strings/hebrevc_error-detect.phpt",
    "ext/standard/tests/strings/bug22008-detect.phpt",
    "ext/standard/tests/strings/nl_langinfo_error1-detect.phpt",
    "ext/standard/tests/strings/bug39350-detect.phpt",
    "ext/standard/tests/strings/crypt_chars-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation12-detect.phpt",
    "ext/standard/tests/strings/vfprintf_variation13-detect.phpt",
    "ext/standard/tests/strings/stripos_variation14-detect.phpt",
    "ext/standard/tests/strings/strval_error-detect.phpt",
    "ext/standard/tests/strings/stripslashes_variation4-detect.phpt",
    "ext/standard/tests/general_functions/parse_ini_string_001-detect.phpt",
    "ext/standard/tests/general_functions/escapeshellarg_error-detect.phpt",
    "ext/standard/tests/general_functions/bug44295-detect.phpt",
    "ext/standard/tests/general_functions/proc_nice_error-detect.phpt",
    "ext/standard/tests/general_functions/bug60227_3-detect.phpt",
    "ext/standard/tests/general_functions/bug44667-detect.phpt",
    "ext/standard/tests/general_functions/getrusage_error-detect.phpt",
    "ext/standard/tests/general_functions/php_uname_error-detect.phpt",
    "ext/standard/tests/general_functions/callbacks_002-detect.phpt",
    "ext/standard/tests/general_functions/is_callable_basic1-detect.phpt",
    "ext/standard/tests/general_functions/bug29038-detect.phpt",
    "ext/standard/tests/general_functions/gettype_settype_variation4-detect.phpt",
    "ext/standard/tests/general_functions/var_export_basic1-detect.phpt",
    "ext/standard/tests/general_functions/getopt_002-detect.phpt",
    "ext/standard/tests/general_functions/005-detect.phpt",
    "ext/standard/tests/general_functions/var_export_basic6-detect.phpt",
    "ext/standard/tests/general_functions/gettype_settype_basic-detect.phpt",
    "ext/standard/tests/general_functions/bug41445-detect.phpt",
    "ext/standard/tests/general_functions/ob_get_flush_basic-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation4-detect.phpt",
    "ext/standard/tests/general_functions/sys_getloadavg-detect.phpt",
    "ext/standard/tests/general_functions/var_export_basic5-detect.phpt",
    "ext/standard/tests/general_functions/bug44394_2-detect.phpt",
    "ext/standard/tests/general_functions/get_defined_constants_error-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation9-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation11-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation6-detect.phpt",
    "ext/standard/tests/general_functions/ob_start_closures-detect.phpt",
    "ext/standard/tests/general_functions/phpcredits2-detect.phpt",
    "ext/standard/tests/general_functions/import_request3-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation1-detect.phpt",
    "ext/standard/tests/general_functions/intval_variation1-detect.phpt",
    "ext/standard/tests/general_functions/gettype_settype_error-detect.phpt",
    "ext/standard/tests/general_functions/get_cfg_var_error-detect.phpt",
    "ext/standard/tests/general_functions/get_cfg_var_variation5-detect.phpt",
    "ext/standard/tests/general_functions/bug49692-detect.phpt",
    "ext/standard/tests/general_functions/usleep_basic-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_basic-detect.phpt",
    "ext/standard/tests/general_functions/getopt_004-detect.phpt",
    "ext/standard/tests/general_functions/gettype_settype_variation6-detect.phpt",
    "ext/standard/tests/general_functions/proc_nice_variation7-detect.phpt",
    "ext/standard/tests/general_functions/is_callable_variation2-detect.phpt",
    "ext/standard/tests/general_functions/bug43293_1-detect.phpt",
    "ext/standard/tests/general_functions/var_export_basic3-detect.phpt",
    "ext/standard/tests/general_functions/php_uname_basic-detect.phpt",
    "ext/standard/tests/general_functions/sleep_basic-detect.phpt",
    "ext/standard/tests/general_functions/get_extension_funcs_basic-detect.phpt",
    "ext/standard/tests/general_functions/getopt_005-detect.phpt",
    "ext/standard/tests/general_functions/bug41445_1-detect.phpt",
    "ext/standard/tests/general_functions/is_null-detect.phpt",
    "ext/standard/tests/general_functions/getservbyport_variation1-detect.phpt",
    "ext/standard/tests/general_functions/phpinfo-detect.phpt",
    "ext/standard/tests/general_functions/import_request1-detect.phpt",
    "ext/standard/tests/general_functions/floatval_variation1-detect.phpt",
    "ext/standard/tests/general_functions/getrusage_basic-detect.phpt",
    "ext/standard/tests/general_functions/sleep_error-detect.phpt",
    "ext/standard/tests/general_functions/001-detect.phpt",
    "ext/standard/tests/general_functions/is_resource-detect.phpt",
    "ext/standard/tests/general_functions/escapeshellarg_basic-detect.phpt",
    "ext/standard/tests/general_functions/getopt_003-detect.phpt",
    "ext/standard/tests/general_functions/get_cfg_var_variation4-detect.phpt",
    "ext/standard/tests/general_functions/get_resource_type_error-detect.phpt",
    "ext/standard/tests/general_functions/intval_variation2-detect.phpt",
    "ext/standard/tests/general_functions/bug41518-detect.phpt",
    "ext/standard/tests/general_functions/bug25038-detect.phpt",
    "ext/standard/tests/general_functions/call_user_method_002-detect.phpt",
    "ext/standard/tests/general_functions/004-detect.phpt",
    "ext/standard/tests/general_functions/bug27678-detect.phpt",
    "ext/standard/tests/general_functions/parse_ini_string_002-detect.phpt",
    "ext/standard/tests/general_functions/get_defined_vars_basic-detect.phpt",
    "ext/standard/tests/general_functions/is_bool-detect.phpt",
    "ext/standard/tests/general_functions/get_cfg_var_variation7-detect.phpt",
    "ext/standard/tests/general_functions/var_export_error1-detect.phpt",
    "ext/standard/tests/general_functions/is_scalar-detect.phpt",
    "ext/standard/tests/general_functions/get_cfg_var_variation3-detect.phpt",
    "ext/standard/tests/general_functions/proc_open02-detect.phpt",
    "ext/standard/tests/general_functions/bug35229-detect.phpt",
    "ext/standard/tests/general_functions/get_cfg_var_variation9-detect.phpt",
    "ext/standard/tests/general_functions/call_user_func_return-detect.phpt",
    "ext/standard/tests/general_functions/import_request-detect.phpt",
    "ext/standard/tests/general_functions/bug44295-win-detect.phpt",
    "ext/standard/tests/general_functions/rand-detect.phpt",
    "ext/standard/tests/general_functions/usleep_error-detect.phpt",
    "ext/standard/tests/general_functions/get_extension_funcs_error-detect.phpt",
    "ext/standard/tests/general_functions/phpinfo2-detect.phpt",
    "ext/standard/tests/general_functions/is_numeric-detect.phpt",
    "ext/standard/tests/general_functions/bug31190-detect.phpt",
    "ext/standard/tests/general_functions/highlight_heredoc-detect.phpt",
    "ext/standard/tests/general_functions/is_array-detect.phpt",
    "ext/standard/tests/general_functions/is_resource_error-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation10-detect.phpt",
    "ext/standard/tests/general_functions/isset_basic2-detect.phpt",
    "ext/standard/tests/general_functions/bug40752-detect.phpt",
    "ext/standard/tests/general_functions/get_include_path_basic-detect.phpt",
    "ext/standard/tests/general_functions/ob_get_flush_error-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation7-detect.phpt",
    "ext/standard/tests/general_functions/003-detect.phpt",
    "ext/standard/tests/general_functions/bug48660-detect.phpt",
    "ext/standard/tests/general_functions/proc_nice_variation6-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation3-detect.phpt",
    "ext/standard/tests/general_functions/get_resource_type_basic-detect.phpt",
    "ext/standard/tests/general_functions/get_loaded_extensions_error-detect.phpt",
    "ext/standard/tests/general_functions/bug48768-detect.phpt",
    "ext/standard/tests/general_functions/bug47857-detect.phpt",
    "ext/standard/tests/general_functions/get_resource_type_variation1-detect.phpt",
    "ext/standard/tests/general_functions/proc_nice_basic-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation8-detect.phpt",
    "ext/standard/tests/general_functions/getservbyport_error-detect.phpt",
    "ext/standard/tests/general_functions/bug49056-detect.phpt",
    "ext/standard/tests/general_functions/getmypid_basic-detect.phpt",
    "ext/standard/tests/general_functions/debug_zval_dump_v-detect.phpt",
    "ext/standard/tests/general_functions/get_cfg_var_variation8-detect.phpt",
    "ext/standard/tests/general_functions/debug_zval_dump_o-detect.phpt",
    "ext/standard/tests/general_functions/gettype_settype_variation1-detect.phpt",
    "ext/standard/tests/general_functions/006-detect.phpt",
    "ext/standard/tests/general_functions/getrusage-detect.phpt",
    "ext/standard/tests/general_functions/floatval_basic-detect.phpt",
    "ext/standard/tests/general_functions/debug_zval_dump_b-detect.phpt",
    "ext/standard/tests/general_functions/escapeshellarg_variation1-win32-detect.phpt",
    "ext/standard/tests/general_functions/get_included_files-detect.phpt",
    "ext/standard/tests/general_functions/bug47027-detect.phpt",
    "ext/standard/tests/general_functions/get_defined_constants_basic-detect.phpt",
    "ext/standard/tests/general_functions/error_get_last-detect.phpt",
    "ext/standard/tests/general_functions/bug43293_3-detect.phpt",
    "ext/standard/tests/general_functions/is_callable_basic2-detect.phpt",
    "ext/standard/tests/general_functions/getrusage_variation1-detect.phpt",
    "ext/standard/tests/general_functions/callbacks_001-detect.phpt",
    "ext/standard/tests/general_functions/bug60227_1-detect.phpt",
    "ext/standard/tests/general_functions/is_float_64bit-detect.phpt",
    "ext/standard/tests/general_functions/bug60227_4-detect.phpt",
    "ext/standard/tests/general_functions/floatval-detect.phpt",
    "ext/standard/tests/general_functions/print_r-detect.phpt",
    "ext/standard/tests/general_functions/is_string-detect.phpt",
    "ext/standard/tests/general_functions/is_object-detect.phpt",
    "ext/standard/tests/general_functions/bug44394-detect.phpt",
    "ext/standard/tests/general_functions/007-detect.phpt",
    "ext/standard/tests/general_functions/var_dump-detect.phpt",
    "ext/standard/tests/general_functions/is_int_64bit-detect.phpt",
    "ext/standard/tests/general_functions/import_request2-detect.phpt",
    "ext/standard/tests/general_functions/debug_zval_dump_e-detect.phpt",
    "ext/standard/tests/general_functions/escapeshellarg_variation1-detect.phpt",
    "ext/standard/tests/general_functions/call_user_func_array_variation_003-detect.phpt",
    "ext/standard/tests/general_functions/gettype_settype_variation3-detect.phpt",
    "ext/standard/tests/general_functions/intval-detect.phpt",
    "ext/standard/tests/general_functions/002-detect.phpt",
    "ext/standard/tests/general_functions/bug60227_2-detect.phpt",
    "ext/standard/tests/general_functions/008-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation12-detect.phpt",
    "ext/standard/tests/general_functions/get_cfg_var_variation2-detect.phpt",
    "ext/standard/tests/general_functions/gettype_settype_variation7-detect.phpt",
    "ext/standard/tests/general_functions/include_path-detect.phpt",
    "ext/standard/tests/general_functions/get_loaded_extensions_basic-detect.phpt",
    "ext/standard/tests/general_functions/putenv-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation14-detect.phpt",
    "ext/standard/tests/general_functions/is_int-detect.phpt",
    "ext/standard/tests/general_functions/get_magic_quotes_runtime-detect.phpt",
    "ext/standard/tests/general_functions/phpcredits-detect.phpt",
    "ext/standard/tests/general_functions/escapeshellarg_basic-win32-detect.phpt",
    "ext/standard/tests/general_functions/getopt-detect.phpt",
    "ext/standard/tests/general_functions/var_export_error3-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation13-detect.phpt",
    "ext/standard/tests/general_functions/floatval_error-detect.phpt",
    "ext/standard/tests/general_functions/set_magic_quotes_runtime_error-detect.phpt",
    "ext/standard/tests/general_functions/var_export_basic8-detect.phpt",
    "ext/standard/tests/general_functions/var_export_basic9-detect.phpt",
    "ext/standard/tests/general_functions/bug50732-detect.phpt",
    "ext/standard/tests/general_functions/type-detect.phpt",
    "ext/standard/tests/general_functions/php_uname_variation1-detect.phpt",
    "ext/standard/tests/general_functions/uniqid_error-detect.phpt",
    "ext/standard/tests/general_functions/bug46587-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation2-detect.phpt",
    "ext/standard/tests/general_functions/head-detect.phpt",
    "ext/standard/tests/general_functions/var_export_basic4-detect.phpt",
    "ext/standard/tests/general_functions/strval-detect.phpt",
    "ext/standard/tests/general_functions/print_r_64bit-detect.phpt",
    "ext/standard/tests/general_functions/gettype_settype_variation5-detect.phpt",
    "ext/standard/tests/general_functions/bug43293_2-detect.phpt",
    "ext/standard/tests/general_functions/bug34794-detect.phpt",
    "ext/standard/tests/general_functions/var_dump_64bit-detect.phpt",
    "ext/standard/tests/general_functions/var_export-locale-detect.phpt",
    "ext/standard/tests/general_functions/get_extension_funcs_variation-detect.phpt",
    "ext/standard/tests/general_functions/isset_basic1-detect.phpt",
    "ext/standard/tests/general_functions/proc_nice_variation5-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_error-detect.phpt",
    "ext/standard/tests/general_functions/proc_nice_variation2-detect.phpt",
    "ext/standard/tests/general_functions/bug36011-detect.phpt",
    "ext/standard/tests/general_functions/var_export_basic7-detect.phpt",
    "ext/standard/tests/general_functions/bug41037-detect.phpt",
    "ext/standard/tests/general_functions/set_magic_quotes_runtime_basic-detect.phpt",
    "ext/standard/tests/general_functions/call_user_method-detect.phpt",
    "ext/standard/tests/general_functions/var_export_basic2-detect.phpt",
    "ext/standard/tests/general_functions/dl-cve-2007-4887-detect.phpt",
    "ext/standard/tests/general_functions/bug47859-detect.phpt",
    "ext/standard/tests/general_functions/proc_open-detect.phpt",
    "ext/standard/tests/general_functions/bug42272-detect.phpt",
    "ext/standard/tests/general_functions/is_resource_basic-detect.phpt",
    "ext/standard/tests/general_functions/parse_ini_booleans-detect.phpt",
    "ext/standard/tests/general_functions/gettype_settype_variation8-detect.phpt",
    "ext/standard/tests/general_functions/ini_get_all-detect.phpt",
    "ext/standard/tests/general_functions/get_magic_quotes_gpc-detect.phpt",
    "ext/standard/tests/general_functions/is_callable_variation1-detect.phpt",
    "ext/standard/tests/general_functions/bug50690-detect.phpt",
    "ext/standard/tests/general_functions/closures_002-detect.phpt",
    "ext/standard/tests/general_functions/getservbyname_variation5-detect.phpt",
    "ext/standard/tests/general_functions/call_user_func_array_variation_001-detect.phpt",
    "ext/standard/tests/general_functions/bug55371-detect.phpt",
    "ext/standard/tests/general_functions/is_float-detect.phpt",
    "ext/standard/tests/general_functions/get_cfg_var_basic-detect.phpt",
    "ext/standard/tests/general_functions/proc_nice_variation3-detect.phpt",
    "ext/standard/tests/general_functions/var_export_error2-detect.phpt",
    "ext/standard/tests/general_functions/escapeshellcmd-win32-detect.phpt",
    "ext/standard/tests/general_functions/parse_ini_basic-detect.phpt",
    "ext/standard/tests/general_functions/bug52138-detect.phpt",
    "ext/standard/tests/general_functions/get_cfg_var_variation6-detect.phpt",
    "ext/standard/tests/general_functions/009-detect.phpt",
    "ext/standard/tests/general_functions/parse_ini_file-detect.phpt",
    "ext/standard/tests/general_functions/proc_nice_variation1-detect.phpt",
    "ext/standard/tests/general_functions/bug40398-detect.phpt",
    "ext/standard/tests/general_functions/is_callable_error-detect.phpt",
    "ext/standard/tests/general_functions/gettype_settype_variation2-detect.phpt",
    "ext/standard/tests/general_functions/ob_get_length_basic-detect.phpt",
    "ext/standard/tests/general_functions/call_user_func_array_variation_002-detect.phpt",
    "ext/standard/tests/general_functions/get_cfg_var_variation1-detect.phpt",
    "ext/standard/tests/general_functions/bug49847-detect.phpt",
    "ext/standard/tests/general_functions/debug_zval_dump_b_64bit-detect.phpt",
    "ext/standard/tests/general_functions/bug44461-detect.phpt",
    "ext/standard/tests/general_functions/getservbyport_basic-detect.phpt",
    "ext/standard/tests/general_functions/bug41970-detect.phpt",
    "ext/standard/tests/general_functions/bug44487-detect.phpt",
    "ext/standard/tests/image/image_type_to_mime_type_variation4-detect.phpt",
    "ext/standard/tests/image/image_type_to_mime_type_variation3-detect.phpt",
    "ext/standard/tests/image/image_type_to_extension-detect.phpt",
    "ext/standard/tests/image/image_type_to_mime_type_basic-detect.phpt",
    "ext/standard/tests/image/image_type_to_mime_type-detect.phpt",
    "ext/standard/tests/image/image_type_to_mime_type_error-detect.phpt",
    "ext/standard/tests/image/image_type_to_mime_type_variation2-detect.phpt",
    "ext/standard/tests/image/getimagesize_variation3-detect.phpt",
    "ext/standard/tests/image/getimagesize_error1-detect.phpt",
    "ext/standard/tests/image/getimagesize_swc-detect.phpt",
    "ext/standard/tests/image/getimagesize_variation_005-detect.phpt",
    "ext/standard/tests/image/getimagesize_246x247-detect.phpt",
    "ext/standard/tests/image/image_type_to_mime_type_variation1-detect.phpt",
    "ext/standard/tests/image/getimagesize_tif_mm-detect.phpt",
    "ext/standard/tests/image/iptcembed_001-detect.phpt",
    "ext/standard/tests/image/getimagesize_384x385-detect.phpt",
    "ext/standard/tests/image/getimagesize_variation_003-detect.phpt",
    "ext/standard/tests/image/bug13213-detect.phpt",
    "ext/standard/tests/image/getimagesize_variation1-detect.phpt",
    "ext/standard/tests/image/getimagesizefromstring1-detect.phpt",
    "ext/standard/tests/image/getimagesize_variation4-detect.phpt",
    "ext/standard/tests/image/getimagesize_xbm-detect.phpt",
    "ext/standard/tests/image/getimagesize_wbmp-detect.phpt",
    "ext/standard/tests/image/getimagesize_jpgapp-detect.phpt",
    "ext/standard/tests/image/getimagesize_basic-detect.phpt",
    "ext/standard/tests/image/getimagesize-detect.phpt",
    "ext/standard/tests/image/getimagesize_variation2-detect.phpt",
    "ext/standard/tests/class_object/get_parent_class_error_001-detect.phpt",
    "ext/standard/tests/class_object/get_class_vars_error-detect.phpt",
    "ext/standard/tests/class_object/is_a_error_001-detect.phpt",
    "ext/standard/tests/class_object/get_class_methods_basic_002-detect.phpt",
    "ext/standard/tests/class_object/trait_exists_basic_001-detect.phpt",
    "ext/standard/tests/class_object/class_exists_variation_002-detect.phpt",
    "ext/standard/tests/class_object/is_a_variation_002-detect.phpt",
    "ext/standard/tests/class_object/get_declared_traits_error_001-detect.phpt",
    "ext/standard/tests/class_object/get_declared_classes_variation1-detect.phpt",
    "ext/standard/tests/class_object/is_subclass_of_variation_004-detect.phpt",
    "ext/standard/tests/class_object/interface_exists_error-detect.phpt",
    "ext/standard/tests/class_object/get_class_methods_basic_001-detect.phpt",
    "ext/standard/tests/class_object/is_subclass_of_variation_002-detect.phpt",
    "ext/standard/tests/class_object/get_class_variation_002-detect.phpt",
    "ext/standard/tests/class_object/class_exists_variation_003-detect.phpt",
    "ext/standard/tests/class_object/is_subclass_of_variation_001-detect.phpt",
    "ext/standard/tests/class_object/get_declared_classes_error_001-detect.phpt",
    "ext/standard/tests/class_object/get_class_error_001-detect.phpt",
    "ext/standard/tests/class_object/forward_static_call_002-detect.phpt",
    "ext/standard/tests/class_object/get_parent_class_variation_002-detect.phpt",
    "ext/standard/tests/class_object/interface_exists_variation1-detect.phpt",
    "ext/standard/tests/class_object/method_exists_error_001-detect.phpt",
    "ext/standard/tests/class_object/trait_class_exists_variation_003-detect.phpt",
    "ext/standard/tests/class_object/trait_exists_variation_002-detect.phpt",
    "ext/standard/tests/class_object/get_declared_classes_basic_001-detect.phpt",
    "ext/standard/tests/class_object/get_object_vars_variation_003-detect.phpt",
    "ext/standard/tests/class_object/get_class_methods_variation_002-detect.phpt",
    "ext/standard/tests/class_object/get_class_vars_variation1-detect.phpt",
    "ext/standard/tests/class_object/method_exists_basic_003-detect.phpt",
    "ext/standard/tests/class_object/class_exists_error_001-detect.phpt",
    "ext/standard/tests/class_object/interface_exists_variation2-detect.phpt",
    "ext/standard/tests/class_object/trait_exists_variation_001-detect.phpt",
    "ext/standard/tests/class_object/forward_static_call_001-detect.phpt",
    "ext/standard/tests/class_object/get_parent_class_variation_001-detect.phpt",
    "ext/standard/tests/class_object/get_declared_traits_variation1-detect.phpt",
    "ext/standard/tests/class_object/is_subclass_of_variation_003-detect.phpt",
    "ext/standard/tests/class_object/is_a-detect.phpt",
    "ext/standard/tests/class_object/get_object_vars_basic_001-detect.phpt",
    "ext/standard/tests/class_object/get_object_vars_basic_002-detect.phpt",
    "ext/standard/tests/class_object/is_a_variation_001-detect.phpt",
    "ext/standard/tests/class_object/class_exists_variation_001-detect.phpt",
    "ext/standard/tests/class_object/get_object_vars_variation_001-detect.phpt",
    "ext/standard/tests/class_object/get_declared_interfaces_basic_001-detect.phpt",
    "ext/standard/tests/class_object/get_declared_interfaces_error_001-detect.phpt",
    "ext/standard/tests/class_object/class_exists_basic_001-detect.phpt",
    "ext/standard/tests/class_object/trait_exists_error_001-detect.phpt",
    "ext/standard/tests/class_object/interface_exists_variation3-detect.phpt",
    "ext/standard/tests/class_object/method_exists_basic_002-detect.phpt",
    "ext/standard/tests/class_object/method_exists_basic_001-detect.phpt",
    "ext/standard/tests/class_object/property_exists_variation1-detect.phpt",
    "ext/standard/tests/class_object/is_subclass_of_error_001-detect.phpt",
    "ext/standard/tests/class_object/method_exists_variation_003-detect.phpt",
    "ext/standard/tests/class_object/forward_static_call_003-detect.phpt",
    "ext/standard/tests/class_object/get_declared_interfaces_variation1-detect.phpt",
    "ext/standard/tests/class_object/get_class_methods_error_001-detect.phpt",
    "ext/standard/tests/class_object/get_declared_traits_basic_001-detect.phpt",
    "ext/standard/tests/class_object/interface_exists_variation4-detect.phpt",
    "ext/standard/tests/class_object/is_a_variation_003-detect.phpt",
    "ext/standard/tests/class_object/get_class_vars_variation2-detect.phpt",
    "ext/standard/tests/class_object/method_exists_variation_001-detect.phpt",
    "ext/standard/tests/class_object/get_class_methods_variation_001-detect.phpt",
    "ext/standard/tests/class_object/get_class_variation_001-detect.phpt",
    "ext/standard/tests/class_object/get_object_vars_variation_002-detect.phpt",
    "ext/standard/tests/class_object/get_object_vars_error_001-detect.phpt",
    "ext/standard/tests/class_object/method_exists_variation_002-detect.phpt",
    "ext/standard/tests/class_object/property_exists_error-detect.phpt",
    "ext/standard/tests/class_object/get_class_methods_basic_003-detect.phpt",
    "ext/standard/tests/assert/assert_closures-detect.phpt",
    "ext/standard/tests/assert/assert_basic1-detect.phpt",
    "ext/standard/tests/assert/assert_error1-detect.phpt",
    "ext/standard/tests/assert/assert_basic4-detect.phpt",
    "ext/standard/tests/assert/assert03-detect.phpt",
    "ext/standard/tests/assert/assert_variation-detect.phpt",
    "ext/standard/tests/assert/assert_basic2-detect.phpt",
    "ext/standard/tests/assert/assert_basic3-detect.phpt",
    "ext/standard/tests/assert/assert02-detect.phpt",
    "ext/standard/tests/assert/assert_basic-detect.phpt",
    "ext/standard/tests/assert/assert_error-detect.phpt",
    "ext/standard/tests/assert/assert04-detect.phpt",
    "ext/standard/tests/assert/assert-detect.phpt",
    "ext/standard/tests/assert/assert_basic5-detect.phpt",
    "ext/standard/tests/streams/bug48309-detect.phpt",
    "ext/standard/tests/streams/bug49936_win32-detect.phpt",
    "ext/standard/tests/streams/stream_get_meta_data_process_basic-detect.phpt",
    "ext/standard/tests/streams/stream_get_meta_data_file_variation1-detect.phpt",
    "ext/standard/tests/streams/stream_socket_pair-detect.phpt",
    "ext/standard/tests/streams/stream_get_meta_data_file_variation5-detect.phpt",
    "ext/standard/tests/streams/stream_copy_to_stream_socket-detect.phpt",
    "ext/standard/tests/streams/bug54946-detect.phpt",
    "ext/standard/tests/streams/stream_get_contents_002-detect.phpt",
    "ext/standard/tests/streams/bug46024-detect.phpt",
    "ext/standard/tests/streams/stream_get_meta_data_socket_variation2-detect.phpt",
    "ext/standard/tests/streams/bug61115-2-detect.phpt",
    "ext/standard/tests/streams/bug46426-detect.phpt",
    "ext/standard/tests/streams/stream_context_set_option_basic-detect.phpt",
    "ext/standard/tests/streams/bug49936-detect.phpt",
    "ext/standard/tests/streams/stream_get_meta_data_socket_variation1-detect.phpt",
    "ext/standard/tests/streams/stream_context_get_params_001-detect.phpt",
    "ext/standard/tests/streams/bug47997-detect.phpt",
    "ext/standard/tests/streams/bug44818-detect.phpt",
    "ext/standard/tests/streams/stream_set_chunk_size-detect.phpt",
    "ext/standard/tests/streams/stream_is_local-detect.phpt",
    "ext/standard/tests/streams/bug60817-detect.phpt",
    "ext/standard/tests/streams/stream_context_set_option_error_002-detect.phpt",
    "ext/standard/tests/streams/stream_context_set_option_error_001-detect.phpt",
    "ext/standard/tests/streams/bug61115-detect.phpt",
    "ext/standard/tests/streams/bug60455_03-detect.phpt",
    "ext/standard/tests/streams/stream_get_meta_data_file_error-detect.phpt",
    "ext/standard/tests/streams/stream_get_line_nb-detect.phpt",
    "ext/standard/tests/streams/bug60455_04-detect.phpt",
    "ext/standard/tests/streams/stream_resolve_include_path-detect.phpt",
    "ext/standard/tests/streams/bug53427-detect.phpt",
    "ext/standard/tests/streams/bug60455_01-detect.phpt",
    "ext/standard/tests/streams/stream_get_meta_data_socket_basic-detect.phpt",
    "ext/standard/tests/streams/bug40459-detect.phpt",
    "ext/standard/tests/streams/bug53903-detect.phpt",
    "ext/standard/tests/streams/stream_get_line_NUL_delimiter-detect.phpt",
    "ext/standard/tests/streams/stream_get_meta_data_socket_variation3-detect.phpt",
    "ext/standard/tests/streams/bug61371-win-detect.phpt",
    "ext/standard/tests/streams/bug61115-1-detect.phpt",
    "ext/standard/tests/streams/bug44712-detect.phpt",
    "ext/standard/tests/streams/stream_get_contents_001-detect.phpt",
    "ext/standard/tests/streams/bug54623-detect.phpt",
    "ext/standard/tests/streams/stream_set_timeout_error-detect.phpt",
    "ext/standard/tests/streams/stream_get_meta_data_socket_variation4-detect.phpt",
    "ext/standard/tests/streams/bug60455_02-detect.phpt",
    "ext/standard/tests/streams/stream_get_meta_data_file_variation2-detect.phpt",
    "ext/standard/tests/streams/bug61371-detect.phpt",
    "ext/standard/tests/streams/stream_get_meta_data_file_basic-detect.phpt",
    "ext/standard/tests/streams/stream_get_meta_data_dir_basic-detect.phpt",
    "ext/standard/tests/streams/stream_get_meta_data_file_variation4-detect.phpt",
    "ext/standard/tests/math/abs_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/deg2rad_error-detect.phpt",
    "ext/standard/tests/math/mt_rand_error-detect.phpt",
    "ext/standard/tests/math/bindec_error-detect.phpt",
    "ext/standard/tests/math/floor_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/hexdec_variation1_64bit-detect.phpt",
    "ext/standard/tests/math/round_large_exp-detect.phpt",
    "ext/standard/tests/math/dechex_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/bug30695-detect.phpt",
    "ext/standard/tests/math/bindec_variation1_64bit-detect.phpt",
    "ext/standard/tests/math/cosh_error-detect.phpt",
    "ext/standard/tests/math/log1p_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/tanh_variation-detect.phpt",
    "ext/standard/tests/math/decoct_basic-detect.phpt",
    "ext/standard/tests/math/round_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/fmod_variation2-detect.phpt",
    "ext/standard/tests/math/acos_variation-detect.phpt",
    "ext/standard/tests/math/decbin_error-detect.phpt",
    "ext/standard/tests/math/bug21523-detect.phpt",
    "ext/standard/tests/math/octdec_error-detect.phpt",
    "ext/standard/tests/math/cosh_basic-detect.phpt",
    "ext/standard/tests/math/tan_basic-detect.phpt",
    "ext/standard/tests/math/cos_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/acos_error-detect.phpt",
    "ext/standard/tests/math/hypot_error-detect.phpt",
    "ext/standard/tests/math/octdec_basic_64bit-detect.phpt",
    "ext/standard/tests/math/acosh_basic-detect.phpt",
    "ext/standard/tests/math/mt_srand_error-detect.phpt",
    "ext/standard/tests/math/sqrt_variation-detect.phpt",
    "ext/standard/tests/math/bug25694-detect.phpt",
    "ext/standard/tests/math/round_prerounding-detect.phpt",
    "ext/standard/tests/math/abs_error-detect.phpt",
    "ext/standard/tests/math/ceil_basic-detect.phpt",
    "ext/standard/tests/math/hexdec_error-detect.phpt",
    "ext/standard/tests/math/srand_variation1-detect.phpt",
    "ext/standard/tests/math/tanh_basic-detect.phpt",
    "ext/standard/tests/math/pow_variation1-detect.phpt",
    "ext/standard/tests/math/decbin_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/decbin_variation1-detect.phpt",
    "ext/standard/tests/math/log_variation2-detect.phpt",
    "ext/standard/tests/math/dechex_variation1-detect.phpt",
    "ext/standard/tests/math/round-detect.phpt",
    "ext/standard/tests/math/fmod_variation1-detect.phpt",
    "ext/standard/tests/math/rad2deg_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/dechex_error-detect.phpt",
    "ext/standard/tests/math/hypot_variation1-detect.phpt",
    "ext/standard/tests/math/floor_variation1-detect.phpt",
    "ext/standard/tests/math/log10_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/pow_variation2-detect.phpt",
    "ext/standard/tests/math/asinh_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/log-detect.phpt",
    "ext/standard/tests/math/abs-detect.phpt",
    "ext/standard/tests/math/hexdec_variation1-detect.phpt",
    "ext/standard/tests/math/is_nan_variation1-detect.phpt",
    "ext/standard/tests/math/atan_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/pow_error-detect.phpt",
    "ext/standard/tests/math/getrandmax_error-detect.phpt",
    "ext/standard/tests/math/sqrt_basic-detect.phpt",
    "ext/standard/tests/math/acos_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/atan2_variation1-detect.phpt",
    "ext/standard/tests/math/sin_variation-detect.phpt",
    "ext/standard/tests/math/atan2_error-detect.phpt",
    "ext/standard/tests/math/acosh_variation-detect.phpt",
    "ext/standard/tests/math/dechex_variation1_64bit-detect.phpt",
    "ext/standard/tests/math/round_variation2-detect.phpt",
    "ext/standard/tests/math/sqrt_error-detect.phpt",
    "ext/standard/tests/math/round_variation1-detect.phpt",
    "ext/standard/tests/math/cos_basic-detect.phpt",
    "ext/standard/tests/math/srand_basic-detect.phpt",
    "ext/standard/tests/math/is_infinite_basic-detect.phpt",
    "ext/standard/tests/math/atanh_variation-detect.phpt",
    "ext/standard/tests/math/atan_basic-detect.phpt",
    "ext/standard/tests/math/acosh_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/log10_variation-detect.phpt",
    "ext/standard/tests/math/rand_basic-detect.phpt",
    "ext/standard/tests/math/is_infinite_variation1-detect.phpt",
    "ext/standard/tests/math/pow_basic2-detect.phpt",
    "ext/standard/tests/math/is_finite_error-detect.phpt",
    "ext/standard/tests/math/asinh_basic-detect.phpt",
    "ext/standard/tests/math/is_finite_basic-detect.phpt",
    "ext/standard/tests/math/round_basic-detect.phpt",
    "ext/standard/tests/math/acos_basic-detect.phpt",
    "ext/standard/tests/math/fmod_error-detect.phpt",
    "ext/standard/tests/math/fmod_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/constants-detect.phpt",
    "ext/standard/tests/math/decoct_error-detect.phpt",
    "ext/standard/tests/math/ceil_variation1-detect.phpt",
    "ext/standard/tests/math/hexdec-detect.phpt",
    "ext/standard/tests/math/tanh_error-detect.phpt",
    "ext/standard/tests/math/asin_variation-detect.phpt",
    "ext/standard/tests/math/hypot_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/floor_basic-detect.phpt",
    "ext/standard/tests/math/atan2_variation2-detect.phpt",
    "ext/standard/tests/math/log_error-detect.phpt",
    "ext/standard/tests/math/hypot_basic-detect.phpt",
    "ext/standard/tests/math/is_nan_basic-detect.phpt",
    "ext/standard/tests/math/exp_basic-detect.phpt",
    "ext/standard/tests/math/atanh_error-detect.phpt",
    "ext/standard/tests/math/pow_basic-detect.phpt",
    "ext/standard/tests/math/base_convert_error-detect.phpt",
    "ext/standard/tests/math/srand_error-detect.phpt",
    "ext/standard/tests/math/floor_error-detect.phpt",
    "ext/standard/tests/math/log_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/tan_variation-detect.phpt",
    "ext/standard/tests/math/rad2deg_variation-detect.phpt",
    "ext/standard/tests/math/exp_variation1-detect.phpt",
    "ext/standard/tests/math/mt_srand_variation1-detect.phpt",
    "ext/standard/tests/math/lcg_value_basic-detect.phpt",
    "ext/standard/tests/math/atan_error-detect.phpt",
    "ext/standard/tests/math/constants_basic-detect.phpt",
    "ext/standard/tests/math/is_nan_error-detect.phpt",
    "ext/standard/tests/math/decbin_variation1_64bit-detect.phpt",
    "ext/standard/tests/math/atanh_basic-detect.phpt",
    "ext/standard/tests/math/hexdec_basic-detect.phpt",
    "ext/standard/tests/math/pow_variation1_64bit-detect.phpt",
    "ext/standard/tests/math/bug25665-detect.phpt",
    "ext/standard/tests/math/exp_error-detect.phpt",
    "ext/standard/tests/math/is_infinite_error-detect.phpt",
    "ext/standard/tests/math/log1p_variation1-detect.phpt",
    "ext/standard/tests/math/hexdec_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/bug45712-detect.phpt",
    "ext/standard/tests/math/log10_basic-detect.phpt",
    "ext/standard/tests/math/log1p_basic-detect.phpt",
    "ext/standard/tests/math/is_nan_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/deg2rad_variation-detect.phpt",
    "ext/standard/tests/math/mt_getrandmax_basic-detect.phpt",
    "ext/standard/tests/math/mt_srand_basic-detect.phpt",
    "ext/standard/tests/math/atanh_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/sin_basic-detect.phpt",
    "ext/standard/tests/math/is_finite_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/asin_basic-detect.phpt",
    "ext/standard/tests/math/fmod_basic-detect.phpt",
    "ext/standard/tests/math/asinh_error-detect.phpt",
    "ext/standard/tests/math/sin_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/log_basic-detect.phpt",
    "ext/standard/tests/math/getrandmax_basic-detect.phpt",
    "ext/standard/tests/math/asin_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/base_convert_variation3-detect.phpt",
    "ext/standard/tests/math/expm1_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/bug62112-detect.phpt",
    "ext/standard/tests/math/rand_variation1-detect.phpt",
    "ext/standard/tests/math/cosh_variation-detect.phpt",
    "ext/standard/tests/math/floorceil-detect.phpt",
    "ext/standard/tests/math/cosh_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/deg2rad_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/hexdec_basic_64bit-detect.phpt",
    "ext/standard/tests/math/sinh_basic-detect.phpt",
    "ext/standard/tests/math/hypot_variation2-detect.phpt",
    "ext/standard/tests/math/sqrt_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/rand_error-detect.phpt",
    "ext/standard/tests/math/pow-detect.phpt",
    "ext/standard/tests/math/number_format_basic-detect.phpt",
    "ext/standard/tests/math/ceil_error-detect.phpt",
    "ext/standard/tests/math/octdec_basic-detect.phpt",
    "ext/standard/tests/math/tan_error-detect.phpt",
    "ext/standard/tests/math/ceil_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/log1p_error-detect.phpt",
    "ext/standard/tests/math/mt_getrandmax_error-detect.phpt",
    "ext/standard/tests/math/atan_variation-detect.phpt",
    "ext/standard/tests/math/is_finite_variation1-detect.phpt",
    "ext/standard/tests/math/sinh_error-detect.phpt",
    "ext/standard/tests/math/number_format_multichar-detect.phpt",
    "ext/standard/tests/math/cos_error-detect.phpt",
    "ext/standard/tests/math/dechex_basic-detect.phpt",
    "ext/standard/tests/math/log_variation1-detect.phpt",
    "ext/standard/tests/math/tanh_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/octdec_variation1-detect.phpt",
    "ext/standard/tests/math/decoct_variation1_64bit-detect.phpt",
    "ext/standard/tests/math/round_error-detect.phpt",
    "ext/standard/tests/math/abs_variation-detect.phpt",
    "ext/standard/tests/math/mt_rand_basic-detect.phpt",
    "ext/standard/tests/math/rand_variation2-detect.phpt",
    "ext/standard/tests/math/bug27646-detect.phpt",
    "ext/standard/tests/math/decbin_basic-detect.phpt",
    "ext/standard/tests/math/log10_error-detect.phpt",
    "ext/standard/tests/math/pow_basic_64bit-detect.phpt",
    "ext/standard/tests/math/bindec_basic-detect.phpt",
    "ext/standard/tests/math/atan2_basic-detect.phpt",
    "ext/standard/tests/math/expm1_variation1-detect.phpt",
    "ext/standard/tests/math/bug28228-detect.phpt",
    "ext/standard/tests/math/deg2rad_basic-detect.phpt",
    "ext/standard/tests/math/base_convert_variation2-detect.phpt",
    "ext/standard/tests/math/octdec_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/cos_variation-detect.phpt",
    "ext/standard/tests/math/base_convert_basic-detect.phpt",
    "ext/standard/tests/math/expm1_basic-detect.phpt",
    "ext/standard/tests/math/sinh_variation-detect.phpt",
    "ext/standard/tests/math/pi_basic-detect.phpt",
    "ext/standard/tests/math/exp_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/expm1_error-detect.phpt",
    "ext/standard/tests/math/sinh_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/atan2_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/asinh_variation-detect.phpt",
    "ext/standard/tests/math/sin_error-detect.phpt",
    "ext/standard/tests/math/decoct_variation1-detect.phpt",
    "ext/standard/tests/math/bindec_basic_64bit-detect.phpt",
    "ext/standard/tests/math/round_modes-detect.phpt",
    "ext/standard/tests/math/bindec_variation1-detect.phpt",
    "ext/standard/tests/math/pow_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/acosh_error-detect.phpt",
    "ext/standard/tests/math/is_infinite_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/tan_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/rad2deg_error-detect.phpt",
    "ext/standard/tests/math/bug30069-detect.phpt",
    "ext/standard/tests/math/base_convert_variation1-detect.phpt",
    "ext/standard/tests/math/decoct_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/bug24142-detect.phpt",
    "ext/standard/tests/math/rad2deg_basic-detect.phpt",
    "ext/standard/tests/math/mt_rand_variation1-detect.phpt",
    "ext/standard/tests/math/bindec_basiclong_64bit-detect.phpt",
    "ext/standard/tests/math/mt_rand_variation2-detect.phpt",
    "ext/standard/tests/math/abs_basic-detect.phpt",
    "ext/standard/tests/math/asin_error-detect.phpt",
    "ext/standard/tests/versioning/version_compare-detect.phpt",
    "ext/standard/tests/versioning/php_sapi_name_variation001-detect.phpt",
    "ext/standard/tests/versioning/php_sapi_name-detect.phpt",
    "ext/standard/tests/time/001-detect.phpt",
    "ext/standard/tests/time/bug38524-detect.phpt",
    "ext/standard/tests/time/strptime_error-detect.phpt",
    "ext/standard/tests/time/idate-detect.phpt",
    "ext/standard/tests/time/strptime_basic-detect.phpt",
    "ext/standard/tests/time/bug60222-detect.phpt",
    "ext/standard/tests/time/strptime_parts-detect.phpt",
    "ext/standard/tests/php_logo_guid-detect.phpt",
    "ext/standard/tests/dir/closedir_variation2-detect.phpt",
    "ext/standard/tests/dir/scandir_variation1-win32-detect.phpt",
    "ext/standard/tests/dir/readdir_variation1-detect.phpt",
    "ext/standard/tests/dir/dir_error-detect.phpt",
    "ext/standard/tests/dir/scandir_variation4-detect.phpt",
    "ext/standard/tests/dir/dir_variation7-detect.phpt",
    "ext/standard/tests/dir/scandir_variation6-win32-detect.phpt",
    "ext/standard/tests/dir/dir_variation4-detect.phpt",
    "ext/standard/tests/dir/closedir_variation3-detect.phpt",
    "ext/standard/tests/dir/scandir_basic-detect.phpt",
    "ext/standard/tests/dir/scandir_variation6-detect.phpt",
    "ext/standard/tests/dir/opendir_variation4-detect.phpt",
    "ext/standard/tests/dir/scandir_variation1-detect.phpt",
    "ext/standard/tests/dir/chdir_error2-detect.phpt",
    "ext/standard/tests/dir/dir_variation5-win32-detect.phpt",
    "ext/standard/tests/dir/chdir_error1-detect.phpt",
    "ext/standard/tests/dir/closedir_variation1-detect.phpt",
    "ext/standard/tests/dir/opendir_error2-win32-detect.phpt",
    "ext/standard/tests/dir/readdir_variation6-detect.phpt",
    "ext/standard/tests/dir/rewinddir_basic-detect.phpt",
    "ext/standard/tests/dir/dir_variation9-win32-detect.phpt",
    "ext/standard/tests/dir/scandir_error1-detect.phpt",
    "ext/standard/tests/dir/readdir_variation7-detect.phpt",
    "ext/standard/tests/dir/readdir_variation5-detect.phpt",
    "ext/standard/tests/dir/readdir_variation3-detect.phpt",
    "ext/standard/tests/dir/scandir_variation5-detect.phpt",
    "ext/standard/tests/dir/opendir_error2-detect.phpt",
    "ext/standard/tests/dir/opendir_variation7-detect.phpt",
    "ext/standard/tests/dir/scandir_variation2-detect.phpt",
    "ext/standard/tests/dir/getcwd_basic-detect.phpt",
    "ext/standard/tests/dir/opendir_variation6-detect.phpt",
    "ext/standard/tests/dir/opendir_variation3-detect.phpt",
    "ext/standard/tests/dir/dir_variation8-detect.phpt",
    "ext/standard/tests/dir/dir_variation1-detect.phpt",
    "ext/standard/tests/dir/readdir_basic-detect.phpt",
    "ext/standard/tests/dir/dir_variation9-detect.phpt",
    "ext/standard/tests/dir/chdir_variation2-detect.phpt",
    "ext/standard/tests/dir/opendir_error1-detect.phpt",
    "ext/standard/tests/dir/scandir_variation3-detect.phpt",
    "ext/standard/tests/dir/readdir_error-detect.phpt",
    "ext/standard/tests/dir/opendir_basic-detect.phpt",
    "ext/standard/tests/dir/dir_variation6-win32-detect.phpt",
    "ext/standard/tests/dir/scandir_variation10-detect.phpt",
    "ext/standard/tests/dir/dir_basic-detect.phpt",
    "ext/standard/tests/dir/closedir_error-detect.phpt",
    "ext/standard/tests/dir/dir_variation3-detect.phpt",
    "ext/standard/tests/dir/opendir_variation2-detect.phpt",
    "ext/standard/tests/dir/readdir_variation2-detect.phpt",
    "ext/standard/tests/dir/scandir_error2-detect.phpt",
    "ext/standard/tests/dir/opendir_variation5-detect.phpt",
    "ext/standard/tests/dir/opendir_variation1-win32-detect.phpt",
    "ext/standard/tests/dir/scandir_error2-win32-detect.phpt",
    "ext/standard/tests/dir/closedir_basic-detect.phpt",
    "ext/standard/tests/dir/chdir_variation1-detect.phpt",
    "ext/standard/tests/dir/rewinddir_error-detect.phpt",
    "ext/standard/tests/dir/opendir_variation6-win32-detect.phpt",
    "ext/standard/tests/dir/rewinddir_variation3-detect.phpt",
    "ext/standard/tests/dir/chdir_basic-detect.phpt",
    "ext/standard/tests/dir/readdir_variation4-detect.phpt",
    "ext/standard/tests/dir/scandir_variation8-detect.phpt",
    "ext/standard/tests/dir/dir_variation2-detect.phpt",
    "ext/standard/tests/dir/opendir_variation1-detect.phpt",
    "ext/standard/tests/dir/getcwd_error-detect.phpt",
    "ext/standard/tests/dir/dir_variation1-win32-detect.phpt",
    "ext/standard/tests/dir/scandir_variation9-detect.phpt",
    "ext/standard/tests/dir/scandir_variation7-detect.phpt",
    "ext/standard/tests/dir/rewinddir_variation1-detect.phpt",
    "ext/standard/tests/dir/rewinddir_variation2-detect.phpt",
    "ext/standard/tests/dir/dir_variation8-win32-detect.phpt",
    "ext/standard/tests/dir/dir_variation5-detect.phpt",
    "ext/standard/tests/dir/dir_variation6-detect.phpt",
    "ext/standard/tests/zend_logo_guid-detect.phpt",
    "ext/standard/tests/filters/bug22538-detect.phpt",
    "ext/standard/tests/filters/php_user_filter_03-detect.phpt",
    "ext/standard/tests/filters/001-detect.phpt",
    "ext/standard/tests/filters/bug35916-detect.phpt",
    "ext/standard/tests/filters/php_user_filter_02-detect.phpt",
    "ext/standard/tests/filters/chunked_001-detect.phpt",
    "ext/standard/tests/filters/stream_filter_remove_error-detect.phpt",
    "ext/standard/tests/filters/basic-detect.phpt",
    "ext/standard/tests/filters/filter_errors_user-detect.phpt",
    "ext/standard/tests/filters/bug46164-2-detect.phpt",
    "ext/standard/tests/filters/bug50363-detect.phpt",
    "ext/standard/tests/filters/php_user_filter_01-detect.phpt",
    "ext/standard/tests/filters/stream_filter_remove_basic-detect.phpt",
    "ext/standard/tests/filters/bug46164-1-detect.phpt",
    "ext/standard/tests/filters/filter_errors_convert_base64_decode-detect.phpt",
    "ext/standard/tests/filters/read-detect.phpt",
    "ext/standard/tests/filters/filter_errors_zlib_inflate-detect.phpt",
    "ext/standard/tests/serialize/serialization_miscTypes_001-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_014-detect.phpt",
    "ext/standard/tests/serialize/005-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_009-detect.phpt",
    "ext/standard/tests/serialize/unserializeS-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_015-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_004-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_005-detect.phpt",
    "ext/standard/tests/serialize/bug30234-detect.phpt",
    "ext/standard/tests/serialize/precision-detect.phpt",
    "ext/standard/tests/serialize/serialization_arrays_001-detect.phpt",
    "ext/standard/tests/serialize/001-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_011-detect.phpt",
    "ext/standard/tests/serialize/bug36424-detect.phpt",
    "ext/standard/tests/serialize/serialization_arrays_002-detect.phpt",
    "ext/standard/tests/serialize/004-detect.phpt",
    "ext/standard/tests/serialize/bug14293-detect.phpt",
    "ext/standard/tests/serialize/bug25378-detect.phpt",
    "ext/standard/tests/serialize/bug31442-detect.phpt",
    "ext/standard/tests/serialize/bug21957-detect.phpt",
    "ext/standard/tests/serialize/serialization_resources_001-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_012-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_010-detect.phpt",
    "ext/standard/tests/serialize/003-detect.phpt",
    "ext/standard/tests/serialize/bug23298-detect.phpt",
    "ext/standard/tests/serialize/006-detect.phpt",
    "ext/standard/tests/serialize/bug43614-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_008-detect.phpt",
    "ext/standard/tests/serialize/bug62836_2-detect.phpt",
    "ext/standard/tests/serialize/serialization_error_001-detect.phpt",
    "ext/standard/tests/serialize/bug42919-detect.phpt",
    "ext/standard/tests/serialize/serialization_precision_002-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_013-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_006-detect.phpt",
    "ext/standard/tests/serialize/bug26762-detect.phpt",
    "ext/standard/tests/serialize/bug37947-detect.phpt",
    "ext/standard/tests/serialize/serialization_arrays_003-detect.phpt",
    "ext/standard/tests/serialize/bug31402-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_007-detect.phpt",
    "ext/standard/tests/serialize/002-detect.phpt",
    "ext/standard/tests/serialize/bug62373-detect.phpt",
    "ext/standard/tests/serialize/bug62836_1-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_003-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_002-detect.phpt",
    "ext/standard/tests/serialize/bug24063-detect.phpt",
    "ext/standard/tests/serialize/bug55798-detect.phpt",
    "ext/standard/tests/serialize/bug28325-detect.phpt",
    "ext/standard/tests/serialize/bug27469-detect.phpt",
    "ext/standard/tests/serialize/serialization_precision_001-detect.phpt",
    "ext/standard/tests/serialize/bug46882-detect.phpt",
    "ext/standard/tests/serialize/bug45706-detect.phpt",
    "ext/standard/tests/serialize/serialization_objects_001-detect.phpt",
    "ext/standard/tests/serialize/serialization_arrays_004-detect.phpt",
    "ext/standard/tests/serialize/incomplete_class-detect.phpt",
    "ext/standard/tests/serialize/serialization_arrays_005-detect.phpt",
    "ext/standard/tests/url/rawurlencode_error_001-detect.phpt",
    "ext/standard/tests/url/parse_url_variation_002_32bit-detect.phpt",
    "ext/standard/tests/url/parse_url_basic_001-detect.phpt",
    "ext/standard/tests/url/parse_url_error_001-detect.phpt",
    "ext/standard/tests/url/base64_decode_error_001-detect.phpt",
    "ext/standard/tests/url/parse_url_basic_008-detect.phpt",
    "ext/standard/tests/url/parse_url_variation_001-detect.phpt",
    "ext/standard/tests/url/bug53248-detect.phpt",
    "ext/standard/tests/url/parse_url_basic_005-detect.phpt",
    "ext/standard/tests/url/urlencode_error_001-detect.phpt",
    "ext/standard/tests/url/get_headers_error_001-detect.phpt",
    "ext/standard/tests/url/get_headers_error_002-detect.phpt",
    "ext/standard/tests/url/rawurldecode_variation_001-detect.phpt",
    "ext/standard/tests/url/base64_encode_basic_002-detect.phpt",
    "ext/standard/tests/url/base64_decode_basic_001-detect.phpt",
    "ext/standard/tests/url/parse_url_basic_009-detect.phpt",
    "ext/standard/tests/url/rawurldecode_error_001-detect.phpt",
    "ext/standard/tests/url/base64_decode_basic_002-detect.phpt",
    "ext/standard/tests/url/rawurlencode_variation_001-detect.phpt",
    "ext/standard/tests/url/parse_url_basic_006-detect.phpt",
    "ext/standard/tests/url/urlencode_variation_001-detect.phpt",
    "ext/standard/tests/url/parse_url_relative_scheme-detect.phpt",
    "ext/standard/tests/url/urldecode_error_001-detect.phpt",
    "ext/standard/tests/url/urldecode_variation_001-detect.phpt",
    "ext/standard/tests/url/bug55399-detect.phpt",
    "ext/standard/tests/url/base64_decode_variation_001-detect.phpt",
    "ext/standard/tests/url/parse_url_basic_004-detect.phpt",
    "ext/standard/tests/url/base64_encode_basic_001-detect.phpt",
    "ext/standard/tests/url/bug47174-detect.phpt",
    "ext/standard/tests/url/base64_encode_error_001-detect.phpt",
    "ext/standard/tests/url/base64_decode_variation_002-detect.phpt",
    "ext/standard/tests/url/parse_url_error_002-detect.phpt",
    "ext/standard/tests/url/bug52327-detect.phpt",
    "ext/standard/tests/url/parse_url_basic_003-detect.phpt",
    "ext/standard/tests/url/parse_url_basic_010-detect.phpt",
    "ext/standard/tests/url/bug55273-detect.phpt",
    "ext/standard/tests/url/parse_url_basic_007-detect.phpt",
    "ext/standard/tests/url/parse_url_variation_002_64bit-detect.phpt",
    "ext/standard/tests/url/base64_encode_variation_001-detect.phpt",
    "ext/standard/tests/url/bug54180-detect.phpt",
    "ext/standard/tests/url/parse_url_basic_002-detect.phpt",
    "ext/standard/tests/network/inet_ipv6-detect.phpt",
    "ext/standard/tests/network/udgloop-detect.phpt",
    "ext/standard/tests/network/long2ip_error-detect.phpt",
    "ext/standard/tests/network/udp4loop-detect.phpt",
    "ext/standard/tests/network/gethostbyname_error003-detect.phpt",
    "ext/standard/tests/network/gethostbyname_error005-detect.phpt",
    "ext/standard/tests/network/fsockopen_error-detect.phpt",
    "ext/standard/tests/network/gethostbyaddr_error-detect.phpt",
    "ext/standard/tests/network/tcp6loop-detect.phpt",
    "ext/standard/tests/network/ip2long_variation1-detect.phpt",
    "ext/standard/tests/network/closelog_error-detect.phpt",
    "ext/standard/tests/network/gethostbyname_basic003-detect.phpt",
    "ext/standard/tests/network/shutdown-detect.phpt",
    "ext/standard/tests/network/gethostbyname_basic002-detect.phpt",
    "ext/standard/tests/network/syslog_error-detect.phpt",
    "ext/standard/tests/network/gethostbyname_error006-detect.phpt",
    "ext/standard/tests/network/unixloop-detect.phpt",
    "ext/standard/tests/network/gethostbynamel_error-detect.phpt",
    "ext/standard/tests/network/gethostbyname_error001-detect.phpt",
    "ext/standard/tests/network/closelog_basic-detect.phpt",
    "ext/standard/tests/network/ip-detect.phpt",
    "ext/standard/tests/network/fsockopen_basic-detect.phpt",
    "ext/standard/tests/network/gethostbynamel_basic1-detect.phpt",
    "ext/standard/tests/network/gethostbyname_error004-detect.phpt",
    "ext/standard/tests/network/fsockopen_variation2-detect.phpt",
    "ext/standard/tests/network/bug41347-detect.phpt",
    "ext/standard/tests/network/tcp4loop-detect.phpt",
    "ext/standard/tests/network/socket_get_status_basic-detect.phpt",
    "ext/standard/tests/network/gethostbyaddr_basic1-detect.phpt",
    "ext/standard/tests/network/gethostbyname_error002-detect.phpt",
    "ext/standard/tests/network/syslog_basic-win32-detect.phpt",
    "ext/standard/tests/network/udp6loop-detect.phpt",
    "ext/standard/tests/network/gethostbyname_basic001-detect.phpt",
    "ext/standard/tests/network/ip2long_error-detect.phpt",
    "ext/standard/tests/network/bug20134-detect.phpt",
    "ext/standard/tests/network/http-stream-detect.phpt",
    "ext/standard/tests/network/long2ip_variation1-detect.phpt",
    "ext/standard/tests/network/inet-detect.phpt",
    "ext/standard/tests/network/ip_x86_64-detect.phpt",
    "ext/standard/tests/network/fsockopen_variation1-detect.phpt",
    "ext/standard/tests/network/getmxrr-detect.phpt",
    "ext/standard/tests/misc/get_browser_basic-detect.phpt",
    "ext/standard/tests/misc/time_nanosleep_error3-detect.phpt",
    "ext/standard/tests/misc/time_nanosleep_error5-detect.phpt",
    "ext/standard/tests/misc/time_nanosleep_error2-detect.phpt",
    "ext/standard/tests/misc/time_nanosleep_basic-detect.phpt",
    "ext/standard/tests/misc/time_nanosleep_error1-detect.phpt",
    "ext/standard/tests/misc/syslog_basic-detect.phpt",
    "ext/standard/tests/misc/get_browser_error-detect.phpt",
    "ext/standard/tests/misc/get_browser_variation1-detect.phpt",
    "ext/standard/tests/misc/time_nanosleep_error4-detect.phpt",
    "ext/standard/tests/misc/syslog_parameters-detect.phpt",
    "ext/odbc/tests/odbc_free_result_001-detect.phpt",
    "ext/odbc/tests/odbc_data_source_001-detect.phpt",
    "ext/odbc/tests/odbc_columns_001-detect.phpt",
    "ext/odbc/tests/odbc_columnprivileges_001-detect.phpt",
    "ext/odbc/tests/odbc_exec_002-detect.phpt",
    "ext/odbc/tests/odbc_tables_001-detect.phpt",
    "ext/odbc/tests/odbc_exec_001-detect.phpt",
    "ext/xmlreader/tests/010-detect.phpt",
    "ext/xmlreader/tests/005-detect.phpt",
    "ext/xmlreader/tests/expand-detect.phpt",
    "ext/xmlreader/tests/011-detect.phpt",
    "ext/xmlreader/tests/001-detect.phpt",
    "ext/xmlreader/tests/004-detect.phpt",
    "ext/xmlreader/tests/bug36743-detect.phpt",
    "ext/xmlreader/tests/003-detect.phpt",
    "ext/xmlreader/tests/006-detect.phpt",
    "ext/xmlreader/tests/007-detect.phpt",
    "ext/xmlreader/tests/002-detect.phpt",
    "ext/xmlreader/tests/008-detect.phpt",
    "ext/xmlreader/tests/013-detect.phpt",
    "ext/xmlreader/tests/012-detect.phpt",
    "ext/xmlreader/tests/bug42139-detect.phpt",
    "ext/xmlreader/tests/009-detect.phpt",
    "ext/pcntl/tests/pcntl_alarm-detect.phpt",
    "ext/pcntl/tests/pcntl_get_last_error-detect.phpt",
    "ext/pcntl/tests/pcntl_wait-detect.phpt",
    "ext/pcntl/tests/001-detect.phpt",
    "ext/pcntl/tests/pcntl_fork_basic-detect.phpt",
    "ext/pcntl/tests/pcntl_exec_2-detect.phpt",
    "ext/pcntl/tests/003-detect.phpt",
    "ext/pcntl/tests/pcntl_exec_3-detect.phpt",
    "ext/pcntl/tests/bug47566-detect.phpt",
    "ext/pcntl/tests/002-detect.phpt",
    "ext/pcntl/tests/signal_closure_handler-detect.phpt",
    "ext/pcntl/tests/pcntl_signal_dispatch-detect.phpt",
    "ext/pcntl/tests/pcntl_exec-detect.phpt",
    "ext/pcntl/tests/pcntl_fork_variation-detect.phpt",
    "ext/pcntl/tests/pcntl_signal-detect.phpt",
    "ext/gmp/tests/027-detect.phpt",
    "ext/gmp/tests/033-detect.phpt",
    "ext/gmp/tests/010-detect.phpt",
    "ext/gmp/tests/bug32773-detect.phpt",
    "ext/gmp/tests/005-detect.phpt",
    "ext/gmp/tests/014-detect.phpt",
    "ext/gmp/tests/029-detect.phpt",
    "ext/gmp/tests/011-detect.phpt",
    "ext/gmp/tests/017-detect.phpt",
    "ext/gmp/tests/036-detect.phpt",
    "ext/gmp/tests/028-detect.phpt",
    "ext/gmp/tests/001-detect.phpt",
    "ext/gmp/tests/015-detect.phpt",
    "ext/gmp/tests/026-detect.phpt",
    "ext/gmp/tests/004-detect.phpt",
    "ext/gmp/tests/bug50283-detect.phpt",
    "ext/gmp/tests/003-detect.phpt",
    "ext/gmp/tests/020-detect.phpt",
    "ext/gmp/tests/022-detect.phpt",
    "ext/gmp/tests/006-detect.phpt",
    "ext/gmp/tests/031-detect.phpt",
    "ext/gmp/tests/023-detect.phpt",
    "ext/gmp/tests/007-detect.phpt",
    "ext/gmp/tests/gmp_nextprime-detect.phpt",
    "ext/gmp/tests/002-detect.phpt",
    "ext/gmp/tests/008-detect.phpt",
    "ext/gmp/tests/018-detect.phpt",
    "ext/gmp/tests/040-detect.phpt",
    "ext/gmp/tests/034-detect.phpt",
    "ext/gmp/tests/039-detect.phpt",
    "ext/gmp/tests/035-detect.phpt",
    "ext/gmp/tests/013-detect.phpt",
    "ext/gmp/tests/037-detect.phpt",
    "ext/gmp/tests/016-detect.phpt",
    "ext/gmp/tests/025-detect.phpt",
    "ext/gmp/tests/012-detect.phpt",
    "ext/gmp/tests/038-detect.phpt",
    "ext/gmp/tests/021-detect.phpt",
    "ext/gmp/tests/030-detect.phpt",
    "ext/gmp/tests/032-detect.phpt",
    "ext/gmp/tests/019-detect.phpt",
    "ext/gmp/tests/024-detect.phpt",
    "ext/gmp/tests/009-detect.phpt",
    "ext/gmp/tests/bug52906-detect.phpt",
    "ext/ereg/tests/eregi_variation_003-detect.phpt",
    "ext/ereg/tests/split_basic_001-detect.phpt",
    "ext/ereg/tests/010-detect.phpt",
    "ext/ereg/tests/spliti_variation_001-detect.phpt",
    "ext/ereg/tests/005-detect.phpt",
    "ext/ereg/tests/eregi_error_002-detect.phpt",
    "ext/ereg/tests/spliti_variation_003-detect.phpt",
    "ext/ereg/tests/ereg_replace_variation_001-detect.phpt",
    "ext/ereg/tests/014-detect.phpt",
    "ext/ereg/tests/split_variation_003-detect.phpt",
    "ext/ereg/tests/split_error_002-detect.phpt",
    "ext/ereg/tests/spliti_basic_002-detect.phpt",
    "ext/ereg/tests/011-detect.phpt",
    "ext/ereg/tests/eregi_variation_001-detect.phpt",
    "ext/ereg/tests/001-detect.phpt",
    "ext/ereg/tests/ereg_replace_variation_003-detect.phpt",
    "ext/ereg/tests/015-detect.phpt",
    "ext/ereg/tests/ereg_error_001-detect.phpt",
    "ext/ereg/tests/004-detect.phpt",
    "ext/ereg/tests/spliti_error_002-detect.phpt",
    "ext/ereg/tests/split_variation_002-detect.phpt",
    "ext/ereg/tests/spliti_variation_004-detect.phpt",
    "ext/ereg/tests/eregi_variation_002-detect.phpt",
    "ext/ereg/tests/spliti_basic_003-detect.phpt",
    "ext/ereg/tests/eregi_error_001-detect.phpt",
    "ext/ereg/tests/spliti_basic_001-detect.phpt",
    "ext/ereg/tests/ereg_replace_error_001-detect.phpt",
    "ext/ereg/tests/eregi_replace_variation_002-detect.phpt",
    "ext/ereg/tests/ereg_basic_001-detect.phpt",
    "ext/ereg/tests/003-detect.phpt",
    "ext/ereg/tests/eregi_variation_004-detect.phpt",
    "ext/ereg/tests/eregi_basic_003-detect.phpt",
    "ext/ereg/tests/ereg_variation_001-detect.phpt",
    "ext/ereg/tests/006-detect.phpt",
    "ext/ereg/tests/eregi_basic-detect.phpt",
    "ext/ereg/tests/split_error_001-detect.phpt",
    "ext/ereg/tests/spliti_basic_004-detect.phpt",
    "ext/ereg/tests/ereg_variation_003-detect.phpt",
    "ext/ereg/tests/spliti_error_001-detect.phpt",
    "ext/ereg/tests/sql_regcase_basic_001-detect.phpt",
    "ext/ereg/tests/sql_regcase_error_001-detect.phpt",
    "ext/ereg/tests/007-detect.phpt",
    "ext/ereg/tests/split_variation_004-detect.phpt",
    "ext/ereg/tests/002-detect.phpt",
    "ext/ereg/tests/008-detect.phpt",
    "ext/ereg/tests/split_basic_002-detect.phpt",
    "ext/ereg/tests/eregi_replace_basic-detect.phpt",
    "ext/ereg/tests/eregi_replace_variation_001-detect.phpt",
    "ext/ereg/tests/spliti_variation_002-detect.phpt",
    "ext/ereg/tests/013-detect.phpt",
    "ext/ereg/tests/eregi_replace_basic_001-detect.phpt",
    "ext/ereg/tests/016-detect.phpt",
    "ext/ereg/tests/ereg_replace_basic_001-detect.phpt",
    "ext/ereg/tests/eregi_basic_001-detect.phpt",
    "ext/ereg/tests/eregi_replace_variation_003-detect.phpt",
    "ext/ereg/tests/012-detect.phpt",
    "ext/ereg/tests/split_basic_003-detect.phpt",
    "ext/ereg/tests/sql_regcase_variation_001-detect.phpt",
    "ext/ereg/tests/eregi_replace_basic_002-detect.phpt",
    "ext/ereg/tests/eregi_basic_002-detect.phpt",
    "ext/ereg/tests/ereg_error_002-detect.phpt",
    "ext/ereg/tests/eregi_replace_error_001-detect.phpt",
    "ext/ereg/tests/ereg_replace_error_002-detect.phpt",
    "ext/ereg/tests/ereg_basic_004-detect.phpt",
    "ext/ereg/tests/eregi_replace_error_002-detect.phpt",
    "ext/ereg/tests/ereg_basic_003-detect.phpt",
    "ext/ereg/tests/ereg_replace_basic_002-detect.phpt",
    "ext/ereg/tests/ereg_replace_variation_002-detect.phpt",
    "ext/ereg/tests/eregi_basic_004-detect.phpt",
    "ext/ereg/tests/ereg_basic_002-detect.phpt",
    "ext/ereg/tests/009-detect.phpt",
    "ext/ereg/tests/ereg_variation_004-detect.phpt",
    "ext/ereg/tests/split_variation_001-detect.phpt",
    "ext/ereg/tests/ereg_variation_002-detect.phpt",
    "ext/ctype/tests/ctype_upper_variation2-detect.phpt",
    "ext/ctype/tests/ctype_alnum_error-detect.phpt",
    "ext/ctype/tests/ctype_cntrl_variation1-detect.phpt",
    "ext/ctype/tests/ctype_graph_basic-detect.phpt",
    "ext/ctype/tests/ctype_punct_variation1-detect.phpt",
    "ext/ctype/tests/ctype_alnum_variation4-detect.phpt",
    "ext/ctype/tests/ctype_alpha_variation4-detect.phpt",
    "ext/ctype/tests/ctype_digit_variation2-detect.phpt",
    "ext/ctype/tests/ctype_graph_variation3-detect.phpt",
    "ext/ctype/tests/ctype_xdigit_basic-detect.phpt",
    "ext/ctype/tests/ctype_digit_variation3-detect.phpt",
    "ext/ctype/tests/bug25745-detect.phpt",
    "ext/ctype/tests/ctype_upper_error-detect.phpt",
    "ext/ctype/tests/ctype_space_variation4-detect.phpt",
    "ext/ctype/tests/ctype_graph_variation2-detect.phpt",
    "ext/ctype/tests/001-detect.phpt",
    "ext/ctype/tests/ctype_lower_basic-detect.phpt",
    "ext/ctype/tests/ctype_digit_error-detect.phpt",
    "ext/ctype/tests/ctype_space_variation3-detect.phpt",
    "ext/ctype/tests/ctype_lower_variation4-detect.phpt",
    "ext/ctype/tests/ctype_upper_variation4-detect.phpt",
    "ext/ctype/tests/ctype_space_variation2-detect.phpt",
    "ext/ctype/tests/ctype_cntrl_variation4-detect.phpt",
    "ext/ctype/tests/ctype_cntrl_basic-detect.phpt",
    "ext/ctype/tests/ctype_print_variation2-detect.phpt",
    "ext/ctype/tests/ctype_digit_variation1-detect.phpt",
    "ext/ctype/tests/ctype_graph_variation4-detect.phpt",
    "ext/ctype/tests/ctype_xdigit_variation2-detect.phpt",
    "ext/ctype/tests/ctype_xdigit_variation4-detect.phpt",
    "ext/ctype/tests/ctype_alpha_variation1-detect.phpt",
    "ext/ctype/tests/ctype_digit_basic-detect.phpt",
    "ext/ctype/tests/ctype_digit_variation4-detect.phpt",
    "ext/ctype/tests/ctype_cntrl_error-detect.phpt",
    "ext/ctype/tests/ctype_alpha_variation2-detect.phpt",
    "ext/ctype/tests/ctype_cntrl_variation3-detect.phpt",
    "ext/ctype/tests/ctype_graph_variation1-detect.phpt",
    "ext/ctype/tests/ctype_punct_variation3-detect.phpt",
    "ext/ctype/tests/ctype_upper_basic-detect.phpt",
    "ext/ctype/tests/ctype_space_variation1-detect.phpt",
    "ext/ctype/tests/ctype_lower_variation1-detect.phpt",
    "ext/ctype/tests/ctype_alpha_basic-detect.phpt",
    "ext/ctype/tests/ctype_alnum_variation2-detect.phpt",
    "ext/ctype/tests/ctype_xdigit_variation3-detect.phpt",
    "ext/ctype/tests/ctype_alnum_variation3-detect.phpt",
    "ext/ctype/tests/002-detect.phpt",
    "ext/ctype/tests/ctype_alnum_basic-detect.phpt",
    "ext/ctype/tests/ctype_graph_error-detect.phpt",
    "ext/ctype/tests/ctype_alpha_variation3-detect.phpt",
    "ext/ctype/tests/ctype_punct_error-detect.phpt",
    "ext/ctype/tests/ctype_lower_error-detect.phpt",
    "ext/ctype/tests/ctype_alpha_error-detect.phpt",
    "ext/ctype/tests/ctype_xdigit_variation1-detect.phpt",
    "ext/ctype/tests/ctype_lower_variation2-detect.phpt",
    "ext/ctype/tests/ctype_space_error-detect.phpt",
    "ext/ctype/tests/ctype_punct_variation4-detect.phpt",
    "ext/ctype/tests/ctype_alnum_variation1-detect.phpt",
    "ext/ctype/tests/ctype_upper_variation3-detect.phpt",
    "ext/ctype/tests/ctype_print_error-detect.phpt",
    "ext/ctype/tests/bug34645-detect.phpt",
    "ext/ctype/tests/ctype_punct_variation2-detect.phpt",
    "ext/ctype/tests/ctype_cntrl_variation2-detect.phpt",
    "ext/ctype/tests/ctype_print_variation4-detect.phpt",
    "ext/ctype/tests/ctype_xdigit_error-detect.phpt",
    "ext/ctype/tests/ctype_space_basic-detect.phpt",
    "ext/ctype/tests/ctype_punct_basic-detect.phpt",
    "ext/ctype/tests/ctype_print_basic-detect.phpt",
    "ext/ctype/tests/ctype_lower_variation3-detect.phpt",
    "ext/ctype/tests/ctype_print_variation3-detect.phpt",
    "ext/ctype/tests/ctype_upper_variation1-detect.phpt",
    "ext/ctype/tests/ctype_print_variation1-detect.phpt",
    "ext/fileinfo/tests/finfo_file_regex-win32-detect.phpt",
    "ext/fileinfo/tests/bug61173-detect.phpt",
    "ext/fileinfo/tests/finfo_close_basic-detect.phpt",
    "ext/fileinfo/tests/finfo_open_error-detect.phpt",
    "ext/fileinfo/tests/finfo_phpinfo_basic-detect.phpt",
    "ext/fileinfo/tests/mime_content_type_001-detect.phpt",
    "ext/fileinfo/tests/finfo_open_basic-detect.phpt",
    "ext/fileinfo/tests/finfo_close_error-detect.phpt",
    "ext/fileinfo/tests/bug61964-detect.phpt",
    "ext/fileinfo/tests/finfo_open_002-detect.phpt",
    "ext/fileinfo/tests/mime_content_type_002-detect.phpt",
    "ext/fileinfo/tests/finfo_set_flags_basic-detect.phpt",
    "ext/fileinfo/tests/finfo_file_stream_001-detect.phpt",
    "ext/fileinfo/tests/finfo_file_basic-detect.phpt",
    "ext/fileinfo/tests/finfo_file_001-detect.phpt",
    "ext/fileinfo/tests/finfo_buffer_error-detect.phpt",
    "ext/fileinfo/tests/finfo_open_error-win32-detect.phpt",
    "ext/fileinfo/tests/finfo_open_001-detect.phpt",
    "ext/fileinfo/tests/finfo_open_variation1-detect.phpt",
    "ext/fileinfo/tests/finfo_buffer_variation1-detect.phpt",
    "ext/fileinfo/tests/finfo_file_regex-detect.phpt",
    "ext/fileinfo/tests/finfo_buffer_basic-detect.phpt",
    "ext/fileinfo/tests/finfo_file_002-detect.phpt",
    "ext/reflection/tests/ReflectionObject_getConstructor_error-detect.phpt",
    "ext/reflection/tests/bug41061-detect.phpt",
    "ext/reflection/tests/bug30146-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_006-detect.phpt",
    "ext/reflection/tests/bug37816-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_getFileName001-detect.phpt",
    "ext/reflection/tests/bug60367-detect.phpt",
    "ext/reflection/tests/ReflectionExtension_constructor_error-detect.phpt",
    "ext/reflection/tests/ReflectionClass_newInstanceWithoutConstructor-detect.phpt",
    "ext/reflection/tests/bug49719-detect.phpt",
    "ext/reflection/tests/bug45139-detect.phpt",
    "ext/reflection/tests/010-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_getDocComment_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getName_error1-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getMethods_002-detect.phpt",
    "ext/reflection/tests/ReflectionObject_getName_error-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isAbstract_basic-detect.phpt",
    "ext/reflection/tests/bug36308-detect.phpt",
    "ext/reflection/tests/ReflectionClass_export_basic1-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_export_basic-detect.phpt",
    "ext/reflection/tests/005-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_invalidMethodInConstructor-detect.phpt",
    "ext/reflection/tests/closures_003-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getMethods_003-detect.phpt",
    "ext/reflection/tests/014-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getDocComment_002-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_export_basic-detect.phpt",
    "ext/reflection/tests/ReflectionObject_isInstantiable_variation-detect.phpt",
    "ext/reflection/tests/ReflectionObject_isInstance_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_hasConstant_001-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isInternal_basic-detect.phpt",
    "ext/reflection/tests/bug52854-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getMethods_001-detect.phpt",
    "ext/reflection/tests/ReflectionExtension_getClassNames_variation1-detect.phpt",
    "ext/reflection/tests/bug51911-detect.phpt",
    "ext/reflection/tests/bug39884-detect.phpt",
    "ext/reflection/tests/bug26695-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getNamespaceName-detect.phpt",
    "ext/reflection/tests/ReflectionClass_constructor_001-detect.phpt",
    "ext/reflection/tests/011-detect.phpt",
    "ext/reflection/tests/ReflectionExtension_info_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_FileInfo_basic-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_invokeArgs_basic-detect.phpt",
    "ext/reflection/tests/bug30148-detect.phpt",
    "ext/reflection/tests/bug38942-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getConstants_error-detect.phpt",
    "ext/reflection/tests/property_exists-detect.phpt",
    "ext/reflection/tests/ReflectionClass_modifiers_002-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isSubclassOf_error1-detect.phpt",
    "ext/reflection/tests/bug45765-detect.phpt",
    "ext/reflection/tests/ReflectionObject_isInternal_basic-detect.phpt",
    "ext/reflection/tests/017-detect.phpt",
    "ext/reflection/tests/bug53915-detect.phpt",
    "ext/reflection/tests/ReflectionExtension_isTemporary-detect.phpt",
    "ext/reflection/tests/ReflectionClass_newInstance_001-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getProperty_003-detect.phpt",
    "ext/reflection/tests/ReflectionClass_toString_001-detect.phpt",
    "ext/reflection/tests/ReflectionClass_setStaticPropertyValue_001_2_4-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isInternal_error-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_getExtensionName-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getConstructor_error-detect.phpt",
    "ext/reflection/tests/bug52057-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getConstant_error-detect.phpt",
    "ext/reflection/tests/ReflectionClass_toString_002-detect.phpt",
    "ext/reflection/tests/traits001-detect.phpt",
    "ext/reflection/tests/bug38465-detect.phpt",
    "ext/reflection/tests/bug60357-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_setAccessible-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_getModifiers_basic-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_getDocComment_error-detect.phpt",
    "ext/reflection/tests/bug37964-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_export_error2-detect.phpt",
    "ext/reflection/tests/001-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_export_error3-detect.phpt",
    "ext/reflection/tests/bug39001-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getConstructor_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_hasMethod_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getProperties_002-detect.phpt",
    "ext/reflection/tests/015-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isInterface_basic-detect.phpt",
    "ext/reflection/tests/traits004-detect.phpt",
    "ext/reflection/tests/026-detect.phpt",
    "ext/reflection/tests/004-detect.phpt",
    "ext/reflection/tests/ReflectionObject_isUserDefined_error-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getConstants_basic-detect.phpt",
    "ext/reflection/tests/bug46064_2-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getInterfaceNames_basic-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_canBePassedByValue-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getInterfaces_004-detect.phpt",
    "ext/reflection/tests/bug49074-detect.phpt",
    "ext/reflection/tests/closures_001-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getInterfaces_002-detect.phpt",
    "ext/reflection/tests/bug48336-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_returnsReference_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getStaticPropertyValue_001_2_4-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getProperty_004-detect.phpt",
    "ext/reflection/tests/bug40794-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_basic1-detect.phpt",
    "ext/reflection/tests/ReflectionObject_getName_basic-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_construct001-detect.phpt",
    "ext/reflection/tests/ReflectionClass_hasProperty_001-detect.phpt",
    "ext/reflection/tests/ReflectionClass_newInstanceArgs_002-detect.phpt",
    "ext/reflection/tests/bug33312-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getModifiers_basic-detect.phpt",
    "ext/reflection/tests/ReflectionExtension_getDependencies_variation2-detect.phpt",
    "ext/reflection/tests/ReflectionObject_isSubclassOf002-detect.phpt",
    "ext/reflection/tests/bug39067-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isInstantiable_basic-detect.phpt",
    "ext/reflection/tests/bug62384-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_isDefault_basic-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_DefaultValueConstant_error-detect.phpt",
    "ext/reflection/tests/parameters_002-detect.phpt",
    "ext/reflection/tests/bug33389-detect.phpt",
    "ext/reflection/tests/ReflectionClass_hasConstant_basic-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_getPosition_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isIterateable_001-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getDefaultProperties_001-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_constructor_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isUserDefined_error-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getProperty_001-detect.phpt",
    "ext/reflection/tests/bug38217-detect.phpt",
    "ext/reflection/tests/bug49092-detect.phpt",
    "ext/reflection/tests/bug30209-detect.phpt",
    "ext/reflection/tests/bug38132-detect.phpt",
    "ext/reflection/tests/003-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getExtension_basic-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_001-detect.phpt",
    "ext/reflection/tests/ReflectionObject_export_basic1-detect.phpt",
    "ext/reflection/tests/bug31651-detect.phpt",
    "ext/reflection/tests/ReflectionObject_isSubclassOf_error-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_getClosureThis-detect.phpt",
    "ext/reflection/tests/bug29986-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_isDeprecated_basic-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_getFileName002-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_getClosure_error-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_isDisabled_basic-detect.phpt",
    "ext/reflection/tests/bug38653-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getInterfaces_001-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_getDocComment_error-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_error-detect.phpt",
    "ext/reflection/tests/bug48757-detect.phpt",
    "ext/reflection/tests/020-detect.phpt",
    "ext/reflection/tests/022-detect.phpt",
    "ext/reflection/tests/parameters_001-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_constructor_error-detect.phpt",
    "ext/reflection/tests/bug40431-detect.phpt",
    "ext/reflection/tests/bug43926-detect.phpt",
    "ext/reflection/tests/006-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_invoke_error1-detect.phpt",
    "ext/reflection/tests/ReflectionObject_isInstantiable_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getModifierNames_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getName_basic-detect.phpt",
    "ext/reflection/tests/bug32981-detect.phpt",
    "ext/reflection/tests/ReflectionClass_newInstanceArgs_001-detect.phpt",
    "ext/reflection/tests/023-detect.phpt",
    "ext/reflection/tests/bug36337-detect.phpt",
    "ext/reflection/tests/ReflectionObject_isSubclassOf_basic-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_constructor_variation1-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getStaticPropertyValue_002-detect.phpt",
    "ext/reflection/tests/ReflectionObject_constructor_error-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_basic4-detect.phpt",
    "ext/reflection/tests/bug38194-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isCloneable_001-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_getDeclaringClass_basic-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_export_error-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_003-detect.phpt",
    "ext/reflection/tests/bug45571-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_invoke_basic-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_basic2-detect.phpt",
    "ext/reflection/tests/bug26640-detect.phpt",
    "ext/reflection/tests/ReflectionObject_getConstants_basic-detect.phpt",
    "ext/reflection/tests/bug61388-detect.phpt",
    "ext/reflection/tests/ReflectionObject_constructor_basic-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_getDeclaringClass_variation1-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_getValue_error-detect.phpt",
    "ext/reflection/tests/ReflectionObject_isInstantiable_error-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_basic3-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getParentClass_001-detect.phpt",
    "ext/reflection/tests/ReflectionClass_constructor_002-detect.phpt",
    "ext/reflection/tests/ReflectionObject_isInstance_error-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_001-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isInstance_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getDocComment_001-detect.phpt",
    "ext/reflection/tests/ReflectionExtension_export_basic-detect.phpt",
    "ext/reflection/tests/ReflectionObject_getConstant_error-detect.phpt",
    "ext/reflection/tests/007-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_invoke_error2-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getProperties_001-detect.phpt",
    "ext/reflection/tests/ReflectionObject___toString_basic1-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isSubclassOf_error-detect.phpt",
    "ext/reflection/tests/ReflectionClass_setStaticPropertyValue_001-detect.phpt",
    "ext/reflection/tests/ReflectionObject_getConstructor_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_hasMethod_001-detect.phpt",
    "ext/reflection/tests/bug51905-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_isClosure_basic-detect.phpt",
    "ext/reflection/tests/ReflectionObject_getName_error1-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_getModifiers_basic-detect.phpt",
    "ext/reflection/tests/ReflectionExtension_getClassNames_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_hasConstant_002-detect.phpt",
    "ext/reflection/tests/002-detect.phpt",
    "ext/reflection/tests/static_properties_002-detect.phpt",
    "ext/reflection/tests/008-detect.phpt",
    "ext/reflection/tests/018-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_constructor_error1-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_getClosure_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_export_basic2-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getDefaultProperties_002-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_getStaticVariables_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isFinal_basic-detect.phpt",
    "ext/reflection/tests/ReflectionExtension_getVersion_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getName_error-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_getNamespaceName-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_invokeArgs_error1-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getProperties_003-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isInstantiable_variation-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getMethod_002-detect.phpt",
    "ext/reflection/tests/013-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_getClosureScopeClass-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_getModifiers001-detect.phpt",
    "ext/reflection/tests/016-detect.phpt",
    "ext/reflection/tests/ReflectionClass_modifiers_001-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_invokeArgs_error3-detect.phpt",
    "ext/reflection/tests/ReflectionObject_getConstants_error-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getParentClass-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getProperty_002-detect.phpt",
    "ext/reflection/tests/ReflectionClass_implementsInterface_001-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_DefaultValueConstant_basic2-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_invokeArgs_error2-detect.phpt",
    "ext/reflection/tests/closures_003_v1-detect.phpt",
    "ext/reflection/tests/bug30856-detect.phpt",
    "ext/reflection/tests/025-detect.phpt",
    "ext/reflection/tests/bug36434-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_DefaultValueConstant_basic1-detect.phpt",
    "ext/reflection/tests/ReflectionExtension_constructor_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isIterateable_variation1-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getStaticPropertyValue_001-detect.phpt",
    "ext/reflection/tests/012-detect.phpt",
    "ext/reflection/tests/ReflectionObject_export_basic2-detect.phpt",
    "ext/reflection/tests/021-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getExtension_variation-detect.phpt",
    "ext/reflection/tests/ReflectionClass_FileInfo_error-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getInterfaces_003-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_getClosure_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_hasProperty_basic-detect.phpt",
    "ext/reflection/tests/bug29268-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isSubclassOf_002-detect.phpt",
    "ext/reflection/tests/traits002-detect.phpt",
    "ext/reflection/tests/ReflectionObject_isInternal_error-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_getClosureThis-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isCloneable_002-detect.phpt",
    "ext/reflection/tests/traits005-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_getDocComment001-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_export_error-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_002-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_getExtension-detect.phpt",
    "ext/reflection/tests/ReflectionObject_export_basic3-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isInstantiable_error-detect.phpt",
    "ext/reflection/tests/bug42976-detect.phpt",
    "ext/reflection/tests/closures_004-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isIterateable_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_CannotClone_basic-detect.phpt",
    "ext/reflection/tests/ReflectionFunction_getClosure_error-detect.phpt",
    "ext/reflection/tests/ReflectionExtension_getClasses_basic-detect.phpt",
    "ext/reflection/tests/bug46205-detect.phpt",
    "ext/reflection/tests/ReflectionClass_setStaticPropertyValue_002-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_setValue_error-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_basic1-detect.phpt",
    "ext/reflection/tests/ReflectionClass_toString_003-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_basic2-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_getDeclaringFunction_basic-detect.phpt",
    "ext/reflection/tests/closures_002-detect.phpt",
    "ext/reflection/tests/ReflectionMethod_constructor_error2-detect.phpt",
    "ext/reflection/tests/ReflectionObject_getConstant_basic-detect.phpt",
    "ext/reflection/tests/019-detect.phpt",
    "ext/reflection/tests/ReflectionExtension_isPersistant-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_setAccessible-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getConstant_basic-detect.phpt",
    "ext/reflection/tests/ReflectionObject___toString_basic2-detect.phpt",
    "ext/reflection/tests/bug62715-detect.phpt",
    "ext/reflection/tests/ReflectionExtension_getName_basic-detect.phpt",
    "ext/reflection/tests/ReflectionProperty_getDocComment_basic-detect.phpt",
    "ext/reflection/tests/bug29523-detect.phpt",
    "ext/reflection/tests/ReflectionObject_FileInfo_error-detect.phpt",
    "ext/reflection/tests/024-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isSubclassOf_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isUserDefined_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getMethod_001-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_isDefault-detect.phpt",
    "ext/reflection/tests/traits003-detect.phpt",
    "ext/reflection/tests/ReflectionParameter_toString_basic-detect.phpt",
    "ext/reflection/tests/009-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getExtensionName_variation-detect.phpt",
    "ext/reflection/tests/ReflectionClass_hasProperty_002-detect.phpt",
    "ext/reflection/tests/bug46064-detect.phpt",
    "ext/reflection/tests/ReflectionClass_getExtensionName_basic-detect.phpt",
    "ext/reflection/tests/ReflectionExtension_getDependencies_basic-detect.phpt",
    "ext/reflection/tests/bug29828-detect.phpt",
    "ext/reflection/tests/bug47254-detect.phpt",
    "ext/reflection/tests/ReflectionClass_hasMethod_002-detect.phpt",
    "ext/reflection/tests/ReflectionObject_FileInfo_basic-detect.phpt",
    "ext/reflection/tests/ReflectionClass_isInstance_error-detect.phpt",
    "ext/reflection/tests/bug41884-detect.phpt",
    "ext/reflection/tests/ReflectionObject_isUserDefined_basic-detect.phpt",
    "ext/reflection/tests/bug53366-detect.phpt",
    "ext/reflection/tests/bug30961-detect.phpt",
    "ext/xsl/tests/xslt012-detect.phpt",
    "ext/xsl/tests/xsltprocessor_registerPHPFunctions-null-detect.phpt",
    "ext/xsl/tests/xslt006-detect.phpt",
    "ext/xsl/tests/xsltprocessor_removeParameter-detect.phpt",
    "ext/xsl/tests/xslt011-detect.phpt",
    "ext/xsl/tests/xsltprocessor_registerPHPFunctions-string-notallowed-detect.phpt",
    "ext/xsl/tests/xsltprocessor_registerPHPFunctions-string-detect.phpt",
    "ext/xsl/tests/xsl-phpinfo-detect.phpt",
    "ext/xsl/tests/xsltprocessor_registerPHPFunctions-funcundef-detect.phpt",
    "ext/xsl/tests/xslt004-detect.phpt",
    "ext/xsl/tests/xslt003-detect.phpt",
    "ext/xsl/tests/xslt001-detect.phpt",
    "ext/xsl/tests/xsltprocessor_setparameter-nostring-detect.phpt",
    "ext/xsl/tests/xslt010-detect.phpt",
    "ext/xsl/tests/xsltprocessor_removeParameter-wrongparams-detect.phpt",
    "ext/xsl/tests/xsltprocessor_registerPHPFunctions-array-detect.phpt",
    "ext/xsl/tests/xsltprocessor_getParameter-wrongparam-detect.phpt",
    "ext/xsl/tests/xslt008-detect.phpt",
    "ext/xsl/tests/bug26384-detect.phpt",
    "ext/xsl/tests/xslt007-detect.phpt",
    "ext/xsl/tests/xsltprocessor_registerPHPFunctions-array-multiple-detect.phpt",
    "ext/xsl/tests/bug33853-detect.phpt",
    "ext/xsl/tests/xsltprocessor_registerPHPFunctions-funcnostring-detect.phpt",
    "ext/xsl/tests/bug54446_with_ini-detect.phpt",
    "ext/xsl/tests/xsltprocessor_registerPHPFunctions-allfuncs-detect.phpt",
    "ext/xsl/tests/xslt009-detect.phpt",
    "ext/xsl/tests/xslt002-detect.phpt",
    "ext/xsl/tests/bug48221-detect.phpt",
    "ext/xsl/tests/xslt005-detect.phpt",
    "ext/xsl/tests/xsltprocessor_registerPHPFunctions-string-multiple-detect.phpt",
    "ext/xsl/tests/xsltprocessor_registerPHPFunctions-array-notallowed-detect.phpt",
    "ext/xsl/tests/xsltprocessor_setparameter-errorquote-detect.phpt",
    "ext/xsl/tests/xsltprocessor_getParameter-detect.phpt",
    "ext/xsl/tests/xsltprocessor_getParameter-invalidparam-detect.phpt",
    "ext/xsl/tests/bug54446-detect.phpt",
    "ext/xsl/tests/xsltprocessor_removeParameter-invalidparam-detect.phpt",
    "ext/sysvshm/tests/005-detect.phpt",
    "ext/sysvshm/tests/001-detect.phpt",
    "ext/sysvshm/tests/004-detect.phpt",
    "ext/sysvshm/tests/003-detect.phpt",
    "ext/sysvshm/tests/006-detect.phpt",
    "ext/sysvshm/tests/007-detect.phpt",
    "ext/sysvshm/tests/002-detect.phpt",
    "ext/filter/tests/050-detect.phpt",
    "ext/filter/tests/027-detect.phpt",
    "ext/filter/tests/bug48762-detect.phpt",
    "ext/filter/tests/033-detect.phpt",
    "ext/filter/tests/bug39846-detect.phpt",
    "ext/filter/tests/filter_data-detect.phpt",
    "ext/filter/tests/010-detect.phpt",
    "ext/filter/tests/045-detect.phpt",
    "ext/filter/tests/005-detect.phpt",
    "ext/filter/tests/bug53150-detect.phpt",
    "ext/filter/tests/054-detect.phpt",
    "ext/filter/tests/callback_closure-detect.phpt",
    "ext/filter/tests/014-detect.phpt",
    "ext/filter/tests/044-detect.phpt",
    "ext/filter/tests/029-detect.phpt",
    "ext/filter/tests/bug53037-detect.phpt",
    "ext/filter/tests/011-detect.phpt",
    "ext/filter/tests/017-detect.phpt",
    "ext/filter/tests/036-detect.phpt",
    "ext/filter/tests/bug55478-detect.phpt",
    "ext/filter/tests/bug42718-2-detect.phpt",
    "ext/filter/tests/043-detect.phpt",
    "ext/filter/tests/028-detect.phpt",
    "ext/filter/tests/001-detect.phpt",
    "ext/filter/tests/041-detect.phpt",
    "ext/filter/tests/bug49274-detect.phpt",
    "ext/filter/tests/015-detect.phpt",
    "ext/filter/tests/bug7715-detect.phpt",
    "ext/filter/tests/026-detect.phpt",
    "ext/filter/tests/004-detect.phpt",
    "ext/filter/tests/bug7733-detect.phpt",
    "ext/filter/tests/bug44445-detect.phpt",
    "ext/filter/tests/048-detect.phpt",
    "ext/filter/tests/bug8315-detect.phpt",
    "ext/filter/tests/003-detect.phpt",
    "ext/filter/tests/049-detect.phpt",
    "ext/filter/tests/020-detect.phpt",
    "ext/filter/tests/022-detect.phpt",
    "ext/filter/tests/006-detect.phpt",
    "ext/filter/tests/031-detect.phpt",
    "ext/filter/tests/023-detect.phpt",
    "ext/filter/tests/046-detect.phpt",
    "ext/filter/tests/callback_non_modified_var-detect.phpt",
    "ext/filter/tests/bug46973-detect.phpt",
    "ext/filter/tests/bug52929-detect.phpt",
    "ext/filter/tests/007-detect.phpt",
    "ext/filter/tests/bug44779-detect.phpt",
    "ext/filter/tests/bug42718-detect.phpt",
    "ext/filter/tests/PMOPB45-detect.phpt",
    "ext/filter/tests/002-detect.phpt",
    "ext/filter/tests/008-detect.phpt",
    "ext/filter/tests/018-detect.phpt",
    "ext/filter/tests/040-detect.phpt",
    "ext/filter/tests/034-detect.phpt",
    "ext/filter/tests/039-detect.phpt",
    "ext/filter/tests/035-detect.phpt",
    "ext/filter/tests/013-detect.phpt",
    "ext/filter/tests/037-detect.phpt",
    "ext/filter/tests/016-detect.phpt",
    "ext/filter/tests/bug52209-detect.phpt",
    "ext/filter/tests/047-detect.phpt",
    "ext/filter/tests/025-detect.phpt",
    "ext/filter/tests/012-detect.phpt",
    "ext/filter/tests/038-detect.phpt",
    "ext/filter/tests/bug50632-detect.phpt",
    "ext/filter/tests/bug50158-detect.phpt",
    "ext/filter/tests/021-detect.phpt",
    "ext/filter/tests/052-detect.phpt",
    "ext/filter/tests/bug7586-detect.phpt",
    "ext/filter/tests/030-detect.phpt",
    "ext/filter/tests/032-detect.phpt",
    "ext/filter/tests/019-detect.phpt",
    "ext/filter/tests/053-detect.phpt",
    "ext/filter/tests/051-detect.phpt",
    "ext/filter/tests/024-detect.phpt",
    "ext/filter/tests/009-detect.phpt",
    "ext/filter/tests/bug47745-detect.phpt",
    "ext/filter/tests/042-detect.phpt",
    "ext/filter/tests/bug51192-detect.phpt",
    "ext/filter/tests/bug47435-detect.phpt",
    "ext/simplexml/tests/027-detect.phpt",
    "ext/simplexml/tests/sxe_003-detect.phpt",
    "ext/simplexml/tests/profile06-detect.phpt",
    "ext/simplexml/tests/033-detect.phpt",
    "ext/simplexml/tests/010-detect.phpt",
    "ext/simplexml/tests/005-detect.phpt",
    "ext/simplexml/tests/profile09-detect.phpt",
    "ext/simplexml/tests/bug41582-detect.phpt",
    "ext/simplexml/tests/014-detect.phpt",
    "ext/simplexml/tests/029-detect.phpt",
    "ext/simplexml/tests/profile04-detect.phpt",
    "ext/simplexml/tests/simplexml_load_file-detect.phpt",
    "ext/simplexml/tests/011-detect.phpt",
    "ext/simplexml/tests/profile03-detect.phpt",
    "ext/simplexml/tests/017-detect.phpt",
    "ext/simplexml/tests/bug38354-detect.phpt",
    "ext/simplexml/tests/036-detect.phpt",
    "ext/simplexml/tests/SimpleXMLElement_addAttribute_basic-detect.phpt",
    "ext/simplexml/tests/bug42369-detect.phpt",
    "ext/simplexml/tests/bug39760-detect.phpt",
    "ext/simplexml/tests/feature55218-detect.phpt",
    "ext/simplexml/tests/profile10-detect.phpt",
    "ext/simplexml/tests/028-detect.phpt",
    "ext/simplexml/tests/profile13-detect.phpt",
    "ext/simplexml/tests/001-detect.phpt",
    "ext/simplexml/tests/profile02-detect.phpt",
    "ext/simplexml/tests/009b-detect.phpt",
    "ext/simplexml/tests/015-detect.phpt",
    "ext/simplexml/tests/bug41867-detect.phpt",
    "ext/simplexml/tests/026-detect.phpt",
    "ext/simplexml/tests/004-detect.phpt",
    "ext/simplexml/tests/bug44478-detect.phpt",
    "ext/simplexml/tests/016a-detect.phpt",
    "ext/simplexml/tests/bug51615-detect.phpt",
    "ext/simplexml/tests/000-detect.phpt",
    "ext/simplexml/tests/profile05-detect.phpt",
    "ext/simplexml/tests/bug24392-detect.phpt",
    "ext/simplexml/tests/bug45553-detect.phpt",
    "ext/simplexml/tests/003-detect.phpt",
    "ext/simplexml/tests/profile11-detect.phpt",
    "ext/simplexml/tests/020-detect.phpt",
    "ext/simplexml/tests/022-detect.phpt",
    "ext/simplexml/tests/006-detect.phpt",
    "ext/simplexml/tests/031-detect.phpt",
    "ext/simplexml/tests/bug41175-detect.phpt",
    "ext/simplexml/tests/bug41947-detect.phpt",
    "ext/simplexml/tests/023-detect.phpt",
    "ext/simplexml/tests/simplexml_import_dom-detect.phpt",
    "ext/simplexml/tests/sxe_002-detect.phpt",
    "ext/simplexml/tests/bug37076_1-detect.phpt",
    "ext/simplexml/tests/bug38406-detect.phpt",
    "ext/simplexml/tests/bug48601-detect.phpt",
    "ext/simplexml/tests/bug38424-detect.phpt",
    "ext/simplexml/tests/007-detect.phpt",
    "ext/simplexml/tests/profile12-detect.phpt",
    "ext/simplexml/tests/bug37076-detect.phpt",
    "ext/simplexml/tests/002-detect.phpt",
    "ext/simplexml/tests/SimpleXMLElement_addAttribute_required_attribute_name-detect.phpt",
    "ext/simplexml/tests/bug40451-detect.phpt",
    "ext/simplexml/tests/008-detect.phpt",
    "ext/simplexml/tests/profile07-detect.phpt",
    "ext/simplexml/tests/018-detect.phpt",
    "ext/simplexml/tests/034-detect.phpt",
    "ext/simplexml/tests/035-detect.phpt",
    "ext/simplexml/tests/profile01-detect.phpt",
    "ext/simplexml/tests/013-detect.phpt",
    "ext/simplexml/tests/016-detect.phpt",
    "ext/simplexml/tests/bug26976-detect.phpt",
    "ext/simplexml/tests/bug36611-detect.phpt",
    "ext/simplexml/tests/bug46003-detect.phpt",
    "ext/simplexml/tests/bug27010-detect.phpt",
    "ext/simplexml/tests/025-detect.phpt",
    "ext/simplexml/tests/012-detect.phpt",
    "ext/simplexml/tests/bug41861-detect.phpt",
    "ext/simplexml/tests/021-detect.phpt",
    "ext/simplexml/tests/014a-detect.phpt",
    "ext/simplexml/tests/sxe_005-detect.phpt",
    "ext/simplexml/tests/bug37565-detect.phpt",
    "ext/simplexml/tests/bug37386-detect.phpt",
    "ext/simplexml/tests/bug46048-detect.phpt",
    "ext/simplexml/tests/030-detect.phpt",
    "ext/simplexml/tests/bug46047-detect.phpt",
    "ext/simplexml/tests/profile08-detect.phpt",
    "ext/simplexml/tests/bug38347-detect.phpt",
    "ext/simplexml/tests/sxe_004-detect.phpt",
    "ext/simplexml/tests/032-detect.phpt",
    "ext/simplexml/tests/bug43221-detect.phpt",
    "ext/simplexml/tests/014b-detect.phpt",
    "ext/simplexml/tests/019-detect.phpt",
    "ext/simplexml/tests/024-detect.phpt",
    "ext/simplexml/tests/009-detect.phpt",
    "ext/simplexml/tests/bug35785-detect.phpt",
    "ext/simplexml/tests/sxe_001-detect.phpt",
    "ext/simplexml/tests/bug42259-detect.phpt",
    "ext/simplexml/tests/bug39662-detect.phpt",
    "ext/bcmath/tests/bcpowmod_error1-detect.phpt",
    "ext/bcmath/tests/bcadd_error1-detect.phpt",
    "ext/bcmath/tests/bcmod_error2-detect.phpt",
    "ext/bcmath/tests/bcdiv_error1-detect.phpt",
    "ext/bcmath/tests/bug60377-detect.phpt",
    "ext/bcmath/tests/bcsqrt-detect.phpt",
    "ext/bcmath/tests/bcscale_variation001-detect.phpt",
    "ext/bcmath/tests/bcpow-detect.phpt",
    "ext/bcmath/tests/bcpowmod_error3-detect.phpt",
    "ext/bcmath/tests/bcpowmod_error2-detect.phpt",
    "ext/bcmath/tests/bcmod-detect.phpt",
    "ext/bcmath/tests/bcscale_variation002-detect.phpt",
    "ext/bcmath/tests/bccomp_variation002-detect.phpt",
    "ext/bcmath/tests/bcpow_variation001-detect.phpt",
    "ext/bcmath/tests/bcsqrt_error1-detect.phpt",
    "ext/bcmath/tests/bcmul-detect.phpt",
    "ext/bcmath/tests/bccomp_variation001-detect.phpt",
    "ext/bcmath/tests/bcpowmod-detect.phpt",
    "ext/bcmath/tests/bcdiv_error2-detect.phpt",
    "ext/bcmath/tests/bcdiv-detect.phpt",
    "ext/bcmath/tests/bcpow_error3-detect.phpt",
    "ext/bcmath/tests/bcadd_variation001-detect.phpt",
    "ext/bcmath/tests/bcsub-detect.phpt",
    "ext/bcmath/tests/bcscale-detect.phpt",
    "ext/bcmath/tests/bcmod_error1-detect.phpt",
    "ext/bcmath/tests/bcsub_error1-detect.phpt",
    "ext/bcmath/tests/bccomp-detect.phpt",
    "ext/bcmath/tests/bcmul_error1-detect.phpt",
    "ext/bcmath/tests/bcsqrt_variation001-detect.phpt",
    "ext/bcmath/tests/bcadd-detect.phpt",
    "ext/bcmath/tests/bcsqrt_error2-detect.phpt",
    "ext/bz2/tests/with_files-detect.phpt",
    "ext/bz2/tests/005-detect.phpt",
    "ext/bz2/tests/bz2_filter_compress-detect.phpt",
    "ext/bz2/tests/001-detect.phpt",
    "ext/bz2/tests/004-detect.phpt",
    "ext/bz2/tests/bz2_filter_decompress-detect.phpt",
    "ext/bz2/tests/003-detect.phpt",
    "ext/bz2/tests/002-detect.phpt",
    "ext/bz2/tests/bug51997-detect.phpt",
    "ext/bz2/tests/with_strings-detect.phpt",
    "ext/libxml/tests/libxml_set_external_entity_loader_variation1-detect.phpt",
    "ext/libxml/tests/001-detect.phpt",
    "ext/libxml/tests/libxml_set_external_entity_loader_error1-detect.phpt",
    "ext/libxml/tests/bug42112-detect.phpt",
    "ext/libxml/tests/004-detect.phpt",
    "ext/libxml/tests/libxml_set_external_entity_loader_basic-detect.phpt",
    "ext/libxml/tests/003-detect.phpt",
    "ext/libxml/tests/002-detect.phpt",
    "ext/libxml/tests/bug61367-read-detect.phpt",
    "ext/libxml/tests/bug54440-detect.phpt",
    "ext/libxml/tests/bug61367-write-detect.phpt",
    "ext/libxml/tests/libxml_set_external_entity_loader_variation2-detect.phpt",
    "ext/phar/tests/027-detect.phpt",
    "ext/phar/tests/rename_dir-detect.phpt",
    "ext/phar/tests/open_for_write_newfile_b-detect.phpt",
    "ext/phar/tests/pharfileinfo_construct-detect.phpt",
    "ext/phar/tests/frontcontroller16-detect.phpt",
    "ext/phar/tests/bug46060-detect.phpt",
    "ext/phar/tests/033-detect.phpt",
    "ext/phar/tests/phar_get_supportedcomp4-detect.phpt",
    "ext/phar/tests/bug48377-detect.phpt",
    "ext/phar/tests/mkdir-detect.phpt",
    "ext/phar/tests/010-detect.phpt",
    "ext/phar/tests/phar_buildfromdirectory4-detect.phpt",
    "ext/phar/tests/frontcontroller32-detect.phpt",
    "ext/phar/tests/opendir-detect.phpt",
    "ext/phar/tests/phar_unlinkarchive-detect.phpt",
    "ext/phar/tests/phar_offset_check-detect.phpt",
    "ext/phar/tests/ini_set-detect.phpt",
    "ext/phar/tests/005-detect.phpt",
    "ext/phar/tests/bug46178-detect.phpt",
    "ext/phar/tests/phar_buildfromiterator8-detect.phpt",
    "ext/phar/tests/nophar_web-detect.phpt",
    "ext/phar/tests/fopen_edgecases-detect.phpt",
    "ext/phar/tests/security-detect.phpt",
    "ext/phar/tests/create_new_phar_c-detect.phpt",
    "ext/phar/tests/phar_setdefaultstub-detect.phpt",
    "ext/phar/tests/phar_oo_003-detect.phpt",
    "ext/phar/tests/phar_construct_invalidurl-detect.phpt",
    "ext/phar/tests/frontcontroller1-detect.phpt",
    "ext/phar/tests/bug483772-detect.phpt",
    "ext/phar/tests/014-detect.phpt",
    "ext/phar/tests/phar_oo_getcontentsgz-detect.phpt",
    "ext/phar/tests/frontcontroller2-detect.phpt",
    "ext/phar/tests/029-detect.phpt",
    "ext/phar/tests/phar_stub_write-detect.phpt",
    "ext/phar/tests/phar_gobyebye-detect.phpt",
    "ext/phar/tests/phar_buildfromiterator5-detect.phpt",
    "ext/phar/tests/phar_convert_tar-detect.phpt",
    "ext/phar/tests/phar_oo_getcontents-detect.phpt",
    "ext/phar/tests/frontcontroller25-detect.phpt",
    "ext/phar/tests/phar_oo_005_52-detect.phpt",
    "ext/phar/tests/phar_begin_setstub_commitU-detect.phpt",
    "ext/phar/tests/phpinfo_004-detect.phpt",
    "ext/phar/tests/phar_oo_iswriteable-detect.phpt",
    "ext/phar/tests/011-detect.phpt",
    "ext/phar/tests/phar_ctx_001-detect.phpt",
    "ext/phar/tests/phar_convert_tar3-detect.phpt",
    "ext/phar/tests/019bU-detect.phpt",
    "ext/phar/tests/phar_create_in_cwd-detect.phpt",
    "ext/phar/tests/017-detect.phpt",
    "ext/phar/tests/phar_extract3-detect.phpt",
    "ext/phar/tests/phar_oo_002U-detect.phpt",
    "ext/phar/tests/open_for_write_existing_c-detect.phpt",
    "ext/phar/tests/open_for_write_newfile_b_5_2-detect.phpt",
    "ext/phar/tests/metadata_write-detect.phpt",
    "ext/phar/tests/bug13727-detect.phpt",
    "ext/phar/tests/cached_manifest_1U-detect.phpt",
    "ext/phar/tests/phar_copy-detect.phpt",
    "ext/phar/tests/pharfileinfo_compression-detect.phpt",
    "ext/phar/tests/frontcontroller18-detect.phpt",
    "ext/phar/tests/metadata_writeU-detect.phpt",
    "ext/phar/tests/phar_gzip-detect.phpt",
    "ext/phar/tests/opendir_edgecases-detect.phpt",
    "ext/phar/tests/open_for_write_newfile_c-detect.phpt",
    "ext/phar/tests/phar_oo_002-detect.phpt",
    "ext/phar/tests/phar_oo_007-detect.phpt",
    "ext/phar/tests/readfile_edgecases-detect.phpt",
    "ext/phar/tests/phar_buildfromiterator1-detect.phpt",
    "ext/phar/tests/frontcontroller5-detect.phpt",
    "ext/phar/tests/pharfileinfo_setmetadata-detect.phpt",
    "ext/phar/tests/028-detect.phpt",
    "ext/phar/tests/bug45218_SLOWTESTU-detect.phpt",
    "ext/phar/tests/mounteddirU-detect.phpt",
    "ext/phar/tests/phar_metadata_read-detect.phpt",
    "ext/phar/tests/open_for_write_existing_b-detect.phpt",
    "ext/phar/tests/file_get_contents-detect.phpt",
    "ext/phar/tests/001-detect.phpt",
    "ext/phar/tests/readfile-detect.phpt",
    "ext/phar/tests/pharfileinfo_getcrc32-detect.phpt",
    "ext/phar/tests/bug47085-detect.phpt",
    "ext/phar/tests/015-detect.phpt",
    "ext/phar/tests/frontcontroller12-detect.phpt",
    "ext/phar/tests/026-detect.phpt",
    "ext/phar/tests/phar_oo_005-detect.phpt",
    "ext/phar/tests/withphar-detect.phpt",
    "ext/phar/tests/frontcontroller33-detect.phpt",
    "ext/phar/tests/phar_setalias-detect.phpt",
    "ext/phar/tests/004-detect.phpt",
    "ext/phar/tests/phar_oo_012_confirm-detect.phpt",
    "ext/phar/tests/badparameters-detect.phpt",
    "ext/phar/tests/delete_in_phar_b-detect.phpt",
    "ext/phar/tests/stat-detect.phpt",
    "ext/phar/tests/phar_oo_010-detect.phpt",
    "ext/phar/tests/phar_begin_setstub_commit-detect.phpt",
    "ext/phar/tests/frontcontroller29-detect.phpt",
    "ext/phar/tests/frontcontroller19-detect.phpt",
    "ext/phar/tests/phar_oo_005U-detect.phpt",
    "ext/phar/tests/open_for_write_newfile-detect.phpt",
    "ext/phar/tests/phar_buildfromiterator2-detect.phpt",
    "ext/phar/tests/webphar_compilefail-detect.phpt",
    "ext/phar/tests/phar_buildfromdirectory5-detect.phpt",
    "ext/phar/tests/frontcontroller34-detect.phpt",
    "ext/phar/tests/bug53872-detect.phpt",
    "ext/phar/tests/metadata_read-detect.phpt",
    "ext/phar/tests/frontcontroller17-detect.phpt",
    "ext/phar/tests/phar_oo_uncompressall-detect.phpt",
    "ext/phar/tests/phar_get_supportedcomp2-detect.phpt",
    "ext/phar/tests/test_signaturealgos-detect.phpt",
    "ext/phar/tests/003-detect.phpt",
    "ext/phar/tests/frontcontroller20-detect.phpt",
    "ext/phar/tests/phar_setsignaturealgo1-detect.phpt",
    "ext/phar/tests/phar_extract2-detect.phpt",
    "ext/phar/tests/stat2-detect.phpt",
    "ext/phar/tests/phar_oo_compressallgz-detect.phpt",
    "ext/phar/tests/phar_dotted_path-detect.phpt",
    "ext/phar/tests/phar_decompress-detect.phpt",
    "ext/phar/tests/open_for_write_existing_c_5_2-detect.phpt",
    "ext/phar/tests/phar_gzipU-detect.phpt",
    "ext/phar/tests/frontcontroller11-detect.phpt",
    "ext/phar/tests/phar_extract-detect.phpt",
    "ext/phar/tests/rmdir-detect.phpt",
    "ext/phar/tests/frontcontroller14-detect.phpt",
    "ext/phar/tests/015b-detect.phpt",
    "ext/phar/tests/frontcontroller9-detect.phpt",
    "ext/phar/tests/027U-detect.phpt",
    "ext/phar/tests/phar_gobyebye-win32-detect.phpt",
    "ext/phar/tests/fopen_edgecases2-detect.phpt",
    "ext/phar/tests/020-detect.phpt",
    "ext/phar/tests/022-detect.phpt",
    "ext/phar/tests/phar_convert_repeated-detect.phpt",
    "ext/phar/tests/006-detect.phpt",
    "ext/phar/tests/031-detect.phpt",
    "ext/phar/tests/phar_oo_011-detect.phpt",
    "ext/phar/tests/frontcontroller22-detect.phpt",
    "ext/phar/tests/test_alias_unset-detect.phpt",
    "ext/phar/tests/phar_buildfromdirectory6-detect.phpt",
    "ext/phar/tests/bug60164-detect.phpt",
    "ext/phar/tests/phar_convert_repeated_b-detect.phpt",
    "ext/phar/tests/003a-detect.phpt",
    "ext/phar/tests/phar_buildfromiterator4-detect.phpt",
    "ext/phar/tests/invalid_setstubalias-detect.phpt",
    "ext/phar/tests/023-detect.phpt",
    "ext/phar/tests/phar_setalias2-detect.phpt",
    "ext/phar/tests/019c-detect.phpt",
    "ext/phar/tests/phar_oo_012-detect.phpt",
    "ext/phar/tests/phar_get_supported_signatures_001a-detect.phpt",
    "ext/phar/tests/ini_set_off-detect.phpt",
    "ext/phar/tests/phar_stub-detect.phpt",
    "ext/phar/tests/fgc_edgecases-detect.phpt",
    "ext/phar/tests/phar_buildfromiterator7-detect.phpt",
    "ext/phar/tests/phar_dir_iterate-detect.phpt",
    "ext/phar/tests/stat2_53-detect.phpt",
    "ext/phar/tests/refcount1_5_2-detect.phpt",
    "ext/phar/tests/frontcontroller6-detect.phpt",
    "ext/phar/tests/metadata_write_commit-detect.phpt",
    "ext/phar/tests/phar_createdefaultstub-detect.phpt",
    "ext/phar/tests/phar_stub_write_file-detect.phpt",
    "ext/phar/tests/bug52013-detect.phpt",
    "ext/phar/tests/frontcontroller7-detect.phpt",
    "ext/phar/tests/phar_oo_001U-detect.phpt",
    "ext/phar/tests/phpinfo_002-detect.phpt",
    "ext/phar/tests/cached_manifest_1-detect.phpt",
    "ext/phar/tests/016b-detect.phpt",
    "ext/phar/tests/phar_oo_compressed_002-detect.phpt",
    "ext/phar/tests/phar_buildfromdirectory1-detect.phpt",
    "ext/phar/tests/007-detect.phpt",
    "ext/phar/tests/test_unset-detect.phpt",
    "ext/phar/tests/phar_oo_getmodified-detect.phpt",
    "ext/phar/tests/phar_oo_compressed_001b-detect.phpt",
    "ext/phar/tests/readfile_edgecasesU-detect.phpt",
    "ext/phar/tests/create_new_phar-detect.phpt",
    "ext/phar/tests/019b-detect.phpt",
    "ext/phar/tests/delete_in_phar-detect.phpt",
    "ext/phar/tests/bug54395-detect.phpt",
    "ext/phar/tests/phar_get_supportedcomp1-detect.phpt",
    "ext/phar/tests/tar/rename_dir-detect.phpt",
    "ext/phar/tests/tar/open_for_write_newfile_b-detect.phpt",
    "ext/phar/tests/tar/033-detect.phpt",
    "ext/phar/tests/tar/tar_openssl_hash-detect.phpt",
    "ext/phar/tests/tar/tar_nostub-detect.phpt",
    "ext/phar/tests/tar/frontcontroller19phar-detect.phpt",
    "ext/phar/tests/tar/frontcontroller17phar-detect.phpt",
    "ext/phar/tests/tar/frontcontroller18phar-detect.phpt",
    "ext/phar/tests/tar/phar_buildfromiterator8-detect.phpt",
    "ext/phar/tests/tar/phar_setdefaultstub-detect.phpt",
    "ext/phar/tests/tar/frontcontroller13phar-detect.phpt",
    "ext/phar/tests/tar/phar_buildfromiterator5-detect.phpt",
    "ext/phar/tests/tar/frontcontroller5phar-detect.phpt",
    "ext/phar/tests/tar/phar_begin_setstub_commitU-detect.phpt",
    "ext/phar/tests/tar/links6-detect.phpt",
    "ext/phar/tests/tar/tar_makegz-detect.phpt",
    "ext/phar/tests/tar/tar_004U-detect.phpt",
    "ext/phar/tests/tar/frontcontroller20phar-detect.phpt",
    "ext/phar/tests/tar/frontcontroller11phar-detect.phpt",
    "ext/phar/tests/tar/open_for_write_existing_c-detect.phpt",
    "ext/phar/tests/tar/open_for_write_newfile_b_5_2-detect.phpt",
    "ext/phar/tests/tar/frontcontroller21phar-detect.phpt",
    "ext/phar/tests/tar/phar_copy-detect.phpt",
    "ext/phar/tests/tar/tar_bz2-detect.phpt",
    "ext/phar/tests/tar/open_for_write_newfile_c-detect.phpt",
    "ext/phar/tests/tar/links3-detect.phpt",
    "ext/phar/tests/tar/open_for_write_existing_b-detect.phpt",
    "ext/phar/tests/tar/exists_as_phar-detect.phpt",
    "ext/phar/tests/tar/frontcontroller2phar-detect.phpt",
    "ext/phar/tests/tar/links5-detect.phpt",
    "ext/phar/tests/tar/phar_setalias-detect.phpt",
    "ext/phar/tests/tar/delete_in_phar_b-detect.phpt",
    "ext/phar/tests/tar/frontcontroller9phar-detect.phpt",
    "ext/phar/tests/tar/phar_begin_setstub_commit-detect.phpt",
    "ext/phar/tests/tar/bignames_overflow-detect.phpt",
    "ext/phar/tests/tar/bug49910-detect.phpt",
    "ext/phar/tests/tar/open_for_write_newfile-detect.phpt",
    "ext/phar/tests/tar/allU-detect.phpt",
    "ext/phar/tests/tar/frontcontroller3phar-detect.phpt",
    "ext/phar/tests/tar/require_hash-detect.phpt",
    "ext/phar/tests/tar/phar_convert_phar3-detect.phpt",
    "ext/phar/tests/tar/open_for_write_existing_c_5_2-detect.phpt",
    "ext/phar/tests/tar/tar_002-detect.phpt",
    "ext/phar/tests/tar/rmdir-detect.phpt",
    "ext/phar/tests/tar/phar_convert_phar-detect.phpt",
    "ext/phar/tests/tar/truncated-detect.phpt",
    "ext/phar/tests/tar/phar_buildfromiterator4-detect.phpt",
    "ext/phar/tests/tar/bignames-detect.phpt",
    "ext/phar/tests/tar/phar_setalias2-detect.phpt",
    "ext/phar/tests/tar/tar_makebz2-detect.phpt",
    "ext/phar/tests/tar/phar_stub-detect.phpt",
    "ext/phar/tests/tar/phar_buildfromiterator7-detect.phpt",
    "ext/phar/tests/tar/refcount1_5_2-detect.phpt",
    "ext/phar/tests/tar/frontcontroller8phar-detect.phpt",
    "ext/phar/tests/tar/links2-detect.phpt",
    "ext/phar/tests/tar/frontcontroller16phar-detect.phpt",
    "ext/phar/tests/tar/tar_001-detect.phpt",
    "ext/phar/tests/tar/tar_gzip-detect.phpt",
    "ext/phar/tests/tar/delete_in_phar-detect.phpt",
    "ext/phar/tests/tar/delete-detect.phpt",
    "ext/phar/tests/tar/phar_magic-detect.phpt",
    "ext/phar/tests/tar/open_for_write_newfile_c_5_2-detect.phpt",
    "ext/phar/tests/tar/frontcontroller12phar-detect.phpt",
    "ext/phar/tests/tar/frontcontroller10phar-detect.phpt",
    "ext/phar/tests/tar/tar_004-detect.phpt",
    "ext/phar/tests/tar/phar_setsignaturealgo2-detect.phpt",
    "ext/phar/tests/tar/create_new_and_modify-detect.phpt",
    "ext/phar/tests/tar/dir-detect.phpt",
    "ext/phar/tests/tar/create_new_phar_b-detect.phpt",
    "ext/phar/tests/tar/badalias-detect.phpt",
    "ext/phar/tests/tar/phar_convert_phar2-detect.phpt",
    "ext/phar/tests/tar/rename-detect.phpt",
    "ext/phar/tests/tar/refcount1-detect.phpt",
    "ext/phar/tests/tar/tar_003-detect.phpt",
    "ext/phar/tests/tar/frontcontroller7phar-detect.phpt",
    "ext/phar/tests/tar/frontcontroller4phar-detect.phpt",
    "ext/phar/tests/tar/links4-detect.phpt",
    "ext/phar/tests/tar/phar_convert_phar4-detect.phpt",
    "ext/phar/tests/tar/tar_nohash-detect.phpt",
    "ext/phar/tests/tar/alias_acrobatics-detect.phpt",
    "ext/phar/tests/tar/033a-detect.phpt",
    "ext/phar/tests/tar/open_for_write_existing_b_5_2-detect.phpt",
    "ext/phar/tests/tar/frontcontroller15phar-detect.phpt",
    "ext/phar/tests/tar/phar_buildfromiterator9-detect.phpt",
    "ext/phar/tests/tar/phar_buildfromiterator6-detect.phpt",
    "ext/phar/tests/tar/badchecksum-detect.phpt",
    "ext/phar/tests/tar/links-detect.phpt",
    "ext/phar/tests/tar/frontcontroller6phar-detect.phpt",
    "ext/phar/tests/tar/frontcontroller1phar-detect.phpt",
    "ext/phar/tests/tar/phar_commitwrite-detect.phpt",
    "ext/phar/tests/tar/phar_stub_error-detect.phpt",
    "ext/phar/tests/tar/tar_bz2U-detect.phpt",
    "ext/phar/tests/tar/delete_in_phar_confirm-detect.phpt",
    "ext/phar/tests/tar/tar_gzipU-detect.phpt",
    "ext/phar/tests/tar/all-detect.phpt",
    "ext/phar/tests/tar/frontcontroller14phar-detect.phpt",
    "ext/phar/tests/tar/open_for_write_existing-detect.phpt",
    "ext/phar/tests/frontcontroller23-detect.phpt",
    "ext/phar/tests/delete-detect.phpt",
    "ext/phar/tests/002-detect.phpt",
    "ext/phar/tests/frontcontroller15-detect.phpt",
    "ext/phar/tests/phar_magic-detect.phpt",
    "ext/phar/tests/008-detect.phpt",
    "ext/phar/tests/open_for_write_newfile_c_5_2-detect.phpt",
    "ext/phar/tests/phar_oo_004U-detect.phpt",
    "ext/phar/tests/018-detect.phpt",
    "ext/phar/tests/phar_convert_zip-detect.phpt",
    "ext/phar/tests/phar_offset_get_error-detect.phpt",
    "ext/phar/tests/metadata_write_commitU-detect.phpt",
    "ext/phar/tests/phar_buildfromdirectory2-detect.phpt",
    "ext/phar/tests/phpinfo_003-detect.phpt",
    "ext/phar/tests/phar_oo_nosig-detect.phpt",
    "ext/phar/tests/rename_dir_and_mount-detect.phpt",
    "ext/phar/tests/include_path-detect.phpt",
    "ext/phar/tests/phar_oo_004-detect.phpt",
    "ext/phar/tests/013-detect.phpt",
    "ext/phar/tests/phar_setsignaturealgo2-detect.phpt",
    "ext/phar/tests/016-detect.phpt",
    "ext/phar/tests/bug45218_SLOWTEST-detect.phpt",
    "ext/phar/tests/phar_mount-detect.phpt",
    "ext/phar/tests/phar_get_supported_signatures_002a-detect.phpt",
    "ext/phar/tests/phpinfo_001-detect.phpt",
    "ext/phar/tests/create_new_and_modify-detect.phpt",
    "ext/phar/tests/fopen52-detect.phpt",
    "ext/phar/tests/fopen_edgecases2U-detect.phpt",
    "ext/phar/tests/phar_get_supported_signatures_002-detect.phpt",
    "ext/phar/tests/018U-detect.phpt",
    "ext/phar/tests/withphar_web-detect.phpt",
    "ext/phar/tests/dir-detect.phpt",
    "ext/phar/tests/phar_convert_again-detect.phpt",
    "ext/phar/tests/phar_buildfromiterator3-detect.phpt",
    "ext/phar/tests/create_new_phar_b-detect.phpt",
    "ext/phar/tests/025-detect.phpt",
    "ext/phar/tests/phar_buildfromdirectory3-detect.phpt",
    "ext/phar/tests/012-detect.phpt",
    "ext/phar/tests/phar_metadata_readU-detect.phpt",
    "ext/phar/tests/addfuncs-detect.phpt",
    "ext/phar/tests/phar_oo_009-detect.phpt",
    "ext/phar/tests/019cU-detect.phpt",
    "ext/phar/tests/fopen-detect.phpt",
    "ext/phar/tests/phar_oo_001-detect.phpt",
    "ext/phar/tests/rename-detect.phpt",
    "ext/phar/tests/nophar-detect.phpt",
    "ext/phar/tests/phar_metadata_writeU-detect.phpt",
    "ext/phar/tests/phar_oo_006-detect.phpt",
    "ext/phar/tests/021-detect.phpt",
    "ext/phar/tests/refcount1-detect.phpt",
    "ext/phar/tests/frontcontroller13-detect.phpt",
    "ext/phar/tests/frontcontroller27-detect.phpt",
    "ext/phar/tests/mounteddir-detect.phpt",
    "ext/phar/tests/phar_isvalidpharfilename-detect.phpt",
    "ext/phar/tests/phar_oo_012b-detect.phpt",
    "ext/phar/tests/frontcontroller30-detect.phpt",
    "ext/phar/tests/phar_buildfromiterator10-detect.phpt",
    "ext/phar/tests/alias_acrobatics-detect.phpt",
    "ext/phar/tests/033a-detect.phpt",
    "ext/phar/tests/frontcontroller4-detect.phpt",
    "ext/phar/tests/open_for_write_existing_b_5_2-detect.phpt",
    "ext/phar/tests/fatal_error_webphar-detect.phpt",
    "ext/phar/tests/phar_metadata_write-detect.phpt",
    "ext/phar/tests/phar_buildfromiterator9-detect.phpt",
    "ext/phar/tests/pharfileinfo_destruct-detect.phpt",
    "ext/phar/tests/phar_buildfromiterator6-detect.phpt",
    "ext/phar/tests/030-detect.phpt",
    "ext/phar/tests/phar_buildfromdirectory2-win-detect.phpt",
    "ext/phar/tests/phar_oo_compressed_001-detect.phpt",
    "ext/phar/tests/bug60261-detect.phpt",
    "ext/phar/tests/ini_set_offU-detect.phpt",
    "ext/phar/tests/frontcontroller8-detect.phpt",
    "ext/phar/tests/metadata_readU-detect.phpt",
    "ext/phar/tests/bug13786-detect.phpt",
    "ext/phar/tests/phar_oo_011b-detect.phpt",
    "ext/phar/tests/create_path_error-detect.phpt",
    "ext/phar/tests/frontcontroller21-detect.phpt",
    "ext/phar/tests/032-detect.phpt",
    "ext/phar/tests/phar_commitwrite-detect.phpt",
    "ext/phar/tests/phar_stub_error-detect.phpt",
    "ext/phar/tests/phar_oo_008-detect.phpt",
    "ext/phar/tests/019-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite25phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller16-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite3phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller32-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite22phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller1-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller2-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller25-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller18-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller5-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite15phar-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite18phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller12-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller33-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller29-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller19-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller34-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller17-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller20-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite20phar-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite7phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller11-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller14-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller9-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller22-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite14phar-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite13phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller6-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller7-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite2phar-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite5phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller23-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite19phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller15-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite10phar-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite21phar-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite4a-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite16phar-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite4phar-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite23phar-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite1phar-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite17phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller13-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller27-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller30-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite9phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller4-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite11phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller8-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller21-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite12phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller26-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller31-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller10-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite6phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller3-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller28-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite8phar-detect.phpt",
    "ext/phar/tests/cache_list/frontcontroller24-detect.phpt",
    "ext/phar/tests/cache_list/copyonwrite24phar-detect.phpt",
    "ext/phar/tests/frontcontroller26-detect.phpt",
    "ext/phar/tests/phar_oo_compressallbz2-detect.phpt",
    "ext/phar/tests/phar_running-detect.phpt",
    "ext/phar/tests/zf_test-detect.phpt",
    "ext/phar/tests/delete_in_phar_confirm-detect.phpt",
    "ext/phar/tests/frontcontroller31-detect.phpt",
    "ext/phar/tests/phar_get_supported_signatures_001-detect.phpt",
    "ext/phar/tests/frontcontroller10-detect.phpt",
    "ext/phar/tests/frontcontroller3-detect.phpt",
    "ext/phar/tests/frontcontroller28-detect.phpt",
    "ext/phar/tests/024-detect.phpt",
    "ext/phar/tests/frontphar-detect.phpt",
    "ext/phar/tests/phar_oo_compressed_002b-detect.phpt",
    "ext/phar/tests/phar_convert_tar2-detect.phpt",
    "ext/phar/tests/bug46032-detect.phpt",
    "ext/phar/tests/009-detect.phpt",
    "ext/phar/tests/phar_bz2-detect.phpt",
    "ext/phar/tests/invalid_alias-detect.phpt",
    "ext/phar/tests/include_path_advanced-detect.phpt",
    "ext/phar/tests/017U-detect.phpt",
    "ext/phar/tests/zip/rename_dir-detect.phpt",
    "ext/phar/tests/zip/open_for_write_newfile_b-detect.phpt",
    "ext/phar/tests/zip/033-detect.phpt",
    "ext/phar/tests/zip/corrupt_004-detect.phpt",
    "ext/phar/tests/zip/frontcontroller19phar-detect.phpt",
    "ext/phar/tests/zip/frontcontroller17phar-detect.phpt",
    "ext/phar/tests/zip/frontcontroller18phar-detect.phpt",
    "ext/phar/tests/zip/phar_buildfromiterator8-detect.phpt",
    "ext/phar/tests/zip/phar_setdefaultstub-detect.phpt",
    "ext/phar/tests/zip/frontcontroller13phar-detect.phpt",
    "ext/phar/tests/zip/phar_buildfromiterator5-detect.phpt",
    "ext/phar/tests/zip/frontcontroller5phar-detect.phpt",
    "ext/phar/tests/zip/phar_begin_setstub_commitU-detect.phpt",
    "ext/phar/tests/zip/phar_magicU-detect.phpt",
    "ext/phar/tests/zip/frontcontroller20phar-detect.phpt",
    "ext/phar/tests/zip/frontcontroller11phar-detect.phpt",
    "ext/phar/tests/zip/open_for_write_existing_c-detect.phpt",
    "ext/phar/tests/zip/notphar-detect.phpt",
    "ext/phar/tests/zip/open_for_write_newfile_b_5_2-detect.phpt",
    "ext/phar/tests/zip/frontcontroller21phar-detect.phpt",
    "ext/phar/tests/zip/phar_copy-detect.phpt",
    "ext/phar/tests/zip/getalias-detect.phpt",
    "ext/phar/tests/zip/open_for_write_newfile_c-detect.phpt",
    "ext/phar/tests/zip/open_for_write_existing_b-detect.phpt",
    "ext/phar/tests/zip/exists_as_phar-detect.phpt",
    "ext/phar/tests/zip/frontcontroller2phar-detect.phpt",
    "ext/phar/tests/zip/phar_setalias-detect.phpt",
    "ext/phar/tests/zip/delete_in_phar_b-detect.phpt",
    "ext/phar/tests/zip/frontcontroller9phar-detect.phpt",
    "ext/phar/tests/zip/phar_begin_setstub_commit-detect.phpt",
    "ext/phar/tests/zip/odt-detect.phpt",
    "ext/phar/tests/zip/open_for_write_newfile-detect.phpt",
    "ext/phar/tests/zip/allU-detect.phpt",
    "ext/phar/tests/zip/frontcontroller3phar-detect.phpt",
    "ext/phar/tests/zip/corrupt_005-detect.phpt",
    "ext/phar/tests/zip/phar_oo_compressallgz-detect.phpt",
    "ext/phar/tests/zip/corrupt_003-detect.phpt",
    "ext/phar/tests/zip/open_for_write_existing_c_5_2-detect.phpt",
    "ext/phar/tests/zip/rmdir-detect.phpt",
    "ext/phar/tests/zip/phar_convert_phar-detect.phpt",
    "ext/phar/tests/zip/bzip2-detect.phpt",
    "ext/phar/tests/zip/corrupt_010-detect.phpt",
    "ext/phar/tests/zip/phar_buildfromiterator4-detect.phpt",
    "ext/phar/tests/zip/corrupt_001-detect.phpt",
    "ext/phar/tests/zip/corrupt_002-detect.phpt",
    "ext/phar/tests/zip/phar_setalias2-detect.phpt",
    "ext/phar/tests/zip/corrupt_009-detect.phpt",
    "ext/phar/tests/zip/corrupt_008-detect.phpt",
    "ext/phar/tests/zip/phar_stub-detect.phpt",
    "ext/phar/tests/zip/phar_buildfromiterator7-detect.phpt",
    "ext/phar/tests/zip/refcount1_5_2-detect.phpt",
    "ext/phar/tests/zip/metadata_write_commit-detect.phpt",
    "ext/phar/tests/zip/zlib-detect.phpt",
    "ext/phar/tests/zip/frontcontroller8phar-detect.phpt",
    "ext/phar/tests/zip/frontcontroller16phar-detect.phpt",
    "ext/phar/tests/zip/corrupt_007-detect.phpt",
    "ext/phar/tests/zip/delete_in_phar-detect.phpt",
    "ext/phar/tests/zip/largezip-detect.phpt",
    "ext/phar/tests/zip/bug48791-detect.phpt",
    "ext/phar/tests/zip/delete-detect.phpt",
    "ext/phar/tests/zip/phar_magic-detect.phpt",
    "ext/phar/tests/zip/open_for_write_newfile_c_5_2-detect.phpt",
    "ext/phar/tests/zip/frontcontroller12phar-detect.phpt",
    "ext/phar/tests/zip/metadata_write_commitU-detect.phpt",
    "ext/phar/tests/zip/frontcontroller10phar-detect.phpt",
    "ext/phar/tests/zip/phar_setsignaturealgo2-detect.phpt",
    "ext/phar/tests/zip/create_new_and_modify-detect.phpt",
    "ext/phar/tests/zip/dir-detect.phpt",
    "ext/phar/tests/zip/create_new_phar_b-detect.phpt",
    "ext/phar/tests/zip/badalias-detect.phpt",
    "ext/phar/tests/zip/rename-detect.phpt",
    "ext/phar/tests/zip/unixzip-detect.phpt",
    "ext/phar/tests/zip/refcount1-detect.phpt",
    "ext/phar/tests/zip/frontcontroller7phar-detect.phpt",
    "ext/phar/tests/zip/frontcontroller4phar-detect.phpt",
    "ext/phar/tests/zip/corrupt_006-detect.phpt",
    "ext/phar/tests/zip/alias_acrobatics-detect.phpt",
    "ext/phar/tests/zip/033a-detect.phpt",
    "ext/phar/tests/zip/open_for_write_existing_b_5_2-detect.phpt",
    "ext/phar/tests/zip/frontcontroller15phar-detect.phpt",
    "ext/phar/tests/zip/phar_buildfromiterator9-detect.phpt",
    "ext/phar/tests/zip/phar_buildfromiterator6-detect.phpt",
    "ext/phar/tests/zip/frontcontroller6phar-detect.phpt",
    "ext/phar/tests/zip/frontcontroller1phar-detect.phpt",
    "ext/phar/tests/zip/phar_commitwrite-detect.phpt",
    "ext/phar/tests/zip/phar_stub_error-detect.phpt",
    "ext/phar/tests/zip/phar_oo_compressallbz2-detect.phpt",
    "ext/phar/tests/zip/delete_in_phar_confirm-detect.phpt",
    "ext/phar/tests/zip/all-detect.phpt",
    "ext/phar/tests/zip/frontcontroller14phar-detect.phpt",
    "ext/phar/tests/zip/open_for_write_existing-detect.phpt",
    "ext/phar/tests/phar_get_supportedcomp3-detect.phpt",
    "ext/phar/tests/frontcontroller24-detect.phpt",
    "ext/phar/tests/open_for_write_existing-detect.phpt",
    "ext/phar/tests/pharfileinfo_chmod-detect.phpt",
    "ext/sysvsem/tests/sysv-detect.phpt",
    "ext/dom/tests/DOMDocument_createProcessingInstruction_error-detect.phpt",
    "ext/dom/tests/DOMDocumentType_internalSubset_error_001-detect.phpt",
    "ext/dom/tests/DOMCharacterData_appendData_error_001-detect.phpt",
    "ext/dom/tests/DOMImplementation_createDocumentType_basic-detect.phpt",
    "ext/dom/tests/dom002-detect.phpt",
    "ext/dom/tests/DOMDocument_saveHTMLFile_invalid_filename-detect.phpt",
    "ext/dom/tests/bug36756-detect.phpt",
    "ext/dom/tests/bug42082-detect.phpt",
    "ext/dom/tests/DOMDocument_schemaValidateSource_error4-detect.phpt",
    "ext/dom/tests/DOMCharacterData_data_error_002-detect.phpt",
    "ext/dom/tests/DOMComment_appendData_basic-detect.phpt",
    "ext/dom/tests/dom006-detect.phpt",
    "ext/dom/tests/bug38474-detect.phpt",
    "ext/dom/tests/bug34276-detect.phpt",
    "ext/dom/tests/DOMDocument_relaxNGValidateSource_basic-detect.phpt",
    "ext/dom/tests/DOMDocument_createAttribute_basic-detect.phpt",
    "ext/dom/tests/DOMDocumentType_notations_error_001-detect.phpt",
    "ext/dom/tests/DOMDocument_preserveWhiteSpace_basic-detect.phpt",
    "ext/dom/tests/DOMComment_construct_basic_001-detect.phpt",
    "ext/dom/tests/DOMComment_construct_error_001-detect.phpt",
    "ext/dom/tests/bug50661-detect.phpt",
    "ext/dom/tests/dom001-detect.phpt",
    "ext/dom/tests/bug35342-detect.phpt",
    "ext/dom/tests/DOMComment_replaceData_error2-detect.phpt",
    "ext/dom/tests/DOMNode_hasChildNodes_basic-detect.phpt",
    "ext/dom/tests/domxpath-detect.phpt",
    "ext/dom/tests/DOMDocumentType_publicId_error_001-detect.phpt",
    "ext/dom/tests/DOMDocument_save_basic-detect.phpt",
    "ext/dom/tests/dom_set_attr_node-detect.phpt",
    "ext/dom/tests/domdocument_createentityreference_002-detect.phpt",
    "ext/dom/tests/DOMDocumentFragment_construct_basic_001-detect.phpt",
    "ext/dom/tests/bug47430-detect.phpt",
    "ext/dom/tests/DOMNode_insertBefore_error1-detect.phpt",
    "ext/dom/tests/DOMDocument_loadHTML_error2-detect.phpt",
    "ext/dom/tests/DOMDocument_saveHTML_error2-detect.phpt",
    "ext/dom/tests/domdocument_createentityreference_001-detect.phpt",
    "ext/dom/tests/DOMNode_normalize_basic-detect.phpt",
    "ext/dom/tests/bug38949-detect.phpt",
    "ext/dom/tests/dom007-detect.phpt",
    "ext/dom/tests/DOMDocument_schemaValidate_error1-detect.phpt",
    "ext/dom/tests/domdocument_createcomment_error_001-detect.phpt",
    "ext/dom/tests/DOMDocument_createEntityReference_basic-detect.phpt",
    "ext/dom/tests/DOMDocumentType_basic_001-detect.phpt",
    "ext/dom/tests/DOMImplementation_createDocument_basic-detect.phpt",
    "ext/dom/tests/DOMDocument_implementationRead_basic-detect.phpt",
    "ext/dom/tests/DOMDocument_loadHTML_basic-detect.phpt",
    "ext/dom/tests/bug41374-detect.phpt",
    "ext/dom/tests/DOMDocument_createAttribute_variation-detect.phpt",
    "ext/dom/tests/dom_comment_variation-detect.phpt",
    "ext/dom/tests/bug44648-detect.phpt",
    "ext/dom/tests/DOMComment_insertData_error2-detect.phpt",
    "ext/dom/tests/DOMDocument_encoding_basic-detect.phpt",
    "ext/dom/tests/DOMDocument_resolveExternals_basic-detect.phpt",
    "ext/dom/tests/dom004-detect.phpt",
    "ext/dom/tests/DOMCharacterData_substringData_basic_001-detect.phpt",
    "ext/dom/tests/regsiter_node_class-detect.phpt",
    "ext/dom/tests/DOMCharacterData_appendData_basic-detect.phpt",
    "ext/dom/tests/DOMNode_getLineNo_basic-detect.phpt",
    "ext/dom/tests/DOMDocumentType_publicId_basic_001-detect.phpt",
    "ext/dom/tests/DOMCharacterData_replaceData_error_001-detect.phpt",
    "ext/dom/tests/DOMDocument_schemaValidate_error4-detect.phpt",
    "ext/dom/tests/DOMDocument_loadHTML_error1-detect.phpt",
    "ext/dom/tests/DOMAttr_ownerElement_error_001-detect.phpt",
    "ext/dom/tests/dom003-detect.phpt",
    "ext/dom/tests/bug52656-detect.phpt",
    "ext/dom/tests/DOMDocumentFragment_appendXML_basic_001-detect.phpt",
    "ext/dom/tests/dom_comment_basic-detect.phpt",
    "ext/dom/tests/DOMDocument_relaxNGValidateSource_error1-detect.phpt",
    "ext/dom/tests/dom_xinclude-detect.phpt",
    "ext/dom/tests/domchardata-detect.phpt",
    "ext/dom/tests/dom005-detect.phpt",
    "ext/dom/tests/DOMDocument_preserveWhiteSpace_variations-detect.phpt",
    "ext/dom/tests/DOMDocument_validate_error1-detect.phpt",
    "ext/dom/tests/dom_import_simplexml-detect.phpt",
    "ext/dom/tests/DOMDocument_schemaValidate_basic-detect.phpt",
    "ext/dom/tests/bug37456-detect.phpt",
    "ext/dom/tests/DOMDocument_validate_on_parse_basic-detect.phpt",
    "ext/dom/tests/DOMAttr_value_basic_002-detect.phpt",
    "ext/dom/tests/DOMDocument_savexml_basic-detect.phpt",
    "ext/dom/tests/DOMComment_replaceData_error1-detect.phpt",
    "ext/dom/tests/DOMNode_C14NFile_basic-detect.phpt",
    "ext/dom/tests/DOMNode_getNodePath_basic-detect.phpt",
    "ext/dom/tests/DOMDocumentFragment_construct_basic_002-detect.phpt",
    "ext/dom/tests/bug49463-detect.phpt",
    "ext/dom/tests/DOMDocument_saveHTML_variant1-detect.phpt",
    "ext/dom/tests/DOMNode_issamenode_basic-detect.phpt",
    "ext/dom/tests/DOMDocument_saveHTMLFile_formatOutput-detect.phpt",
    "ext/dom/tests/DOMCharacterData_deleteData_basic_001-detect.phpt",
    "ext/dom/tests/DOMDocumentFragment_appendXML_error_002-detect.phpt",
    "ext/dom/tests/bug38438-detect.phpt",
    "ext/dom/tests/bug41257-detect.phpt",
    "ext/dom/tests/canonicalization-detect.phpt",
    "ext/dom/tests/DOMDocumentFragment_construct_error_001-detect.phpt",
    "ext/dom/tests/DOMDocument_standalone_basic-detect.phpt",
    "ext/dom/tests/DOMComment_appendData_basic_Sullivan-detect.phpt",
    "ext/dom/tests/DOMDocumentType_systemId_error_001-detect.phpt",
    "ext/dom/tests/DOMDocumentFragment_appendXML_hasChildNodes_basic-detect.phpt",
    "ext/dom/tests/DOMDocument_saveHTMLFile_error2-detect.phpt",
    "ext/dom/tests/DOMComment_insertData_basic-detect.phpt",
    "ext/dom/tests/DOMCharacterData_length_error_001-detect.phpt",
    "ext/dom/tests/domattributes-detect.phpt",
    "ext/dom/tests/bug43364-detect.phpt",
    "ext/dom/tests/domobject_debug_handler-detect.phpt",
    "ext/dom/tests/DOMNode_hasChildNodes-detect.phpt",
    "ext/dom/tests/DOMDocument_documentURI_basic-detect.phpt",
    "ext/dom/tests/DOMComment_replaceData_basic-detect.phpt",
    "ext/dom/tests/DOMDocument_validate_external_dtd-detect.phpt",
    "ext/dom/tests/bug32615-detect.phpt",
    "ext/dom/tests/bug49490-detect.phpt",
    "ext/dom/tests/DOMDocumentFragment_appendXML_error_001-detect.phpt",
    "ext/dom/tests/DOMDocument_relaxNGValidate_basic-detect.phpt",
    "ext/dom/tests/DOMCharacterData_insertData_error_001-detect.phpt",
    "ext/dom/tests/DOMDocument_saveHTMLFile_error1-detect.phpt",
    "ext/dom/tests/DOMDocument_schemaValidate_error5-detect.phpt",
    "ext/dom/tests/bug46185-detect.phpt",
    "ext/dom/tests/DOMAttr_value_basic_001-detect.phpt",
    "ext/dom/tests/DOMCharacterData_deleteData_error_002-detect.phpt",
    "ext/dom/tests/DOMDocument_schemaValidateSource_error3-detect.phpt",
    "ext/dom/tests/bug46849-detect.phpt",
    "ext/dom/tests/bug37277-detect.phpt",
    "ext/dom/tests/DOMDocument_createAttribute_error1-detect.phpt",
    "ext/dom/tests/bug47848-detect.phpt",
    "ext/dom/tests/bug35673-detect.phpt",
    "ext/dom/tests/DOMDocument_schemaValidate_error2-detect.phpt",
    "ext/dom/tests/DOMComment_insertData_error1-detect.phpt",
    "ext/dom/tests/bug47849-detect.phpt",
    "ext/dom/tests/DOMNode_insertBefore-detect.phpt",
    "ext/dom/tests/DOMAttr_name_basic_001-detect.phpt",
    "ext/dom/tests/bug45251-detect.phpt",
    "ext/dom/tests/DOMDocument_schemaValidate_error3-detect.phpt",
    "ext/dom/tests/bug40836-detect.phpt",
    "ext/dom/tests/DOMDocument_relaxNGValidateSource_error2-detect.phpt",
    "ext/dom/tests/DOMDocumentType_entities_error_001-detect.phpt",
    "ext/dom/tests/DOMDocument_strictErrorChecking_basic-detect.phpt",
    "ext/dom/tests/DOMText_appendData_basic-detect.phpt",
    "ext/dom/tests/DOMNode_C14N_basic-detect.phpt",
    "ext/dom/tests/DOMDocument_schemaValidateSource_error1-detect.phpt",
    "ext/dom/tests/DOMNode_cloneNode_basic-detect.phpt",
    "ext/dom/tests/dom_create_element-detect.phpt",
    "ext/dom/tests/DOMDocument_saveHTML_variant2-detect.phpt",
    "ext/dom/tests/DOMDocument_schemaValidateSource_basic-detect.phpt",
    "ext/dom/tests/DOMCharacterData_data_basic_002-detect.phpt",
    "ext/dom/tests/bug28721-detect.phpt",
    "ext/dom/tests/domelement-detect.phpt",
    "ext/dom/tests/DOMAttr_construct_error_001-detect.phpt",
    "ext/dom/tests/DOMDocument_relaxNGValidate_error1-detect.phpt",
    "ext/dom/tests/DOMDocumentType_name_error_001-detect.phpt",
    "ext/dom/tests/DOMDocument_schemaValidateSource_error2-detect.phpt",
    "ext/dom/tests/DOMDocument_createAttribute_error-detect.phpt",
    "ext/dom/tests/bug54601-detect.phpt",
    "ext/dom/tests/DOMNode_replaceChild_basic-detect.phpt",
    "ext/dom/tests/DOMElement_hasAttributes_basic-detect.phpt",
    "ext/dom/tests/DOMDocument_createProcessingInstruction_basic-detect.phpt",
    "ext/dom/tests/bug38850-detect.phpt",
    "ext/dom/tests/DOMCDATASection_construct_error_001-detect.phpt",
    "ext/dom/tests/DOMDocument_relaxNGValidate_error2-detect.phpt",
    "ext/dom/tests/DOMNode_removeChild_basic-detect.phpt",
    "ext/dom/tests/DOMDocument_strictErrorChecking_variation-detect.phpt",
    "ext/dom/tests/DOMDocument_validate_error2-detect.phpt",
    "ext/dom/tests/DOMImplementation_hasFeature_basic-detect.phpt",
    "ext/dom/tests/DOMDocument_validate_basic-detect.phpt",
    "ext/dom/tests/DOMCharacterData_deleteData_error_001-detect.phpt",
    "ext/dom/tests/DOMDocument_saveHTML_basic-detect.phpt",
    "ext/dom/tests/DOMDocument_validate_on_parse_variation-detect.phpt",
    "ext/dom/tests/DOMDocument_saveHTMLFile_basic-detect.phpt",
    "ext/dom/tests/DOMDocumentType_systemId_basic_001-detect.phpt",
    "ext/dom/tests/bug46335-detect.phpt",
    "ext/dom/tests/bug28817-detect.phpt",
    "ext/dom/tests/DOMDocumentFragment_appendXML_error_003-detect.phpt",
    "ext/dom/tests/DOMDocument_config_basic-detect.phpt",
    "ext/pcre/tests/preg_quote_basic-detect.phpt",
    "ext/pcre/tests/preg_grep_basic-detect.phpt",
    "ext/pcre/tests/preg_match_error1-detect.phpt",
    "ext/pcre/tests/preg_match_variation1-detect.phpt",
    "ext/pcre/tests/bug21732-detect.phpt",
    "ext/pcre/tests/bug44925-detect.phpt",
    "ext/pcre/tests/005-detect.phpt",
    "ext/pcre/tests/locales-detect.phpt",
    "ext/pcre/tests/split2-detect.phpt",
    "ext/pcre/tests/preg_match_all_error3-detect.phpt",
    "ext/pcre/tests/null_bytes-detect.phpt",
    "ext/pcre/tests/bug38600-detect.phpt",
    "ext/pcre/tests/preg_replace_callback2-detect.phpt",
    "ext/pcre/tests/bug40909-detect.phpt",
    "ext/pcre/tests/bug44214_2-detect.phpt",
    "ext/pcre/tests/preg_match_basic_edit-detect.phpt",
    "ext/pcre/tests/pcre_extended-detect.phpt",
    "ext/pcre/tests/pcre_anchored-detect.phpt",
    "ext/pcre/tests/bug44214-detect.phpt",
    "ext/pcre/tests/preg_match_all_error1-detect.phpt",
    "ext/pcre/tests/preg_match_all_error-detect.phpt",
    "ext/pcre/tests/preg_match_all_edit_basic-detect.phpt",
    "ext/pcre/tests/bug20528-detect.phpt",
    "ext/pcre/tests/preg_quote_error-detect.phpt",
    "ext/pcre/tests/preg_split_error2-detect.phpt",
    "ext/pcre/tests/ungreedy-detect.phpt",
    "ext/pcre/tests/cache_limit-detect.phpt",
    "ext/pcre/tests/001-detect.phpt",
    "ext/pcre/tests/study-detect.phpt",
    "ext/pcre/tests/multiline-detect.phpt",
    "ext/pcre/tests/bug47229-detect.phpt",
    "ext/pcre/tests/004-detect.phpt",
    "ext/pcre/tests/preg_replace-detect.phpt",
    "ext/pcre/tests/bug47662-detect.phpt",
    "ext/pcre/tests/preg_match_all_basic-detect.phpt",
    "ext/pcre/tests/bug21758-detect.phpt",
    "ext/pcre/tests/recursion_limit-detect.phpt",
    "ext/pcre/tests/dollar_endonly-detect.phpt",
    "ext/pcre/tests/preg_split_basic-detect.phpt",
    "ext/pcre/tests/preg_last_error_error-detect.phpt",
    "ext/pcre/tests/split-detect.phpt",
    "ext/pcre/tests/invalid_utf8-detect.phpt",
    "ext/pcre/tests/003-detect.phpt",
    "ext/pcre/tests/bug26927-detect.phpt",
    "ext/pcre/tests/preg_replace_callback-detect.phpt",
    "ext/pcre/tests/preg_split_error-detect.phpt",
    "ext/pcre/tests/pcre_extra-detect.phpt",
    "ext/pcre/tests/preg_replace_callback_error-detect.phpt",
    "ext/pcre/tests/bug41148-detect.phpt",
    "ext/pcre/tests/006-detect.phpt",
    "ext/pcre/tests/bug44191-detect.phpt",
    "ext/pcre/tests/backtrack_limit-detect.phpt",
    "ext/pcre/tests/preg_quote_error1-detect.phpt",
    "ext/pcre/tests/preg_grep_error2-detect.phpt",
    "ext/pcre/tests/preg_match_basic-detect.phpt",
    "ext/pcre/tests/preg_replace_edit_basic-detect.phpt",
    "ext/pcre/tests/delimiters-detect.phpt",
    "ext/pcre/tests/bug52971-detect.phpt",
    "ext/pcre/tests/preg_replace_basic-detect.phpt",
    "ext/pcre/tests/007-detect.phpt",
    "ext/pcre/tests/preg_replace_error1-detect.phpt",
    "ext/pcre/tests/bug37800-detect.phpt",
    "ext/pcre/tests/invalid_utf8_offset-detect.phpt",
    "ext/pcre/tests/002-detect.phpt",
    "ext/pcre/tests/match_flags3-detect.phpt",
    "ext/pcre/tests/preg_grep_error-detect.phpt",
    "ext/pcre/tests/preg_split_error1-detect.phpt",
    "ext/pcre/tests/bug37911-detect.phpt",
    "ext/pcre/tests/pcreconstants-detect.phpt",
    "ext/pcre/tests/match_flags-detect.phpt",
    "ext/pcre/tests/bug40195-detect.phpt",
    "ext/pcre/tests/preg_filter-detect.phpt",
    "ext/pcre/tests/bug34790-detect.phpt",
    "ext/pcre/tests/preg_replace_callback_basic-detect.phpt",
    "ext/pcre/tests/match_flags2-detect.phpt",
    "ext/pcre/tests/preg_grep_error1-detect.phpt",
    "ext/pcre/tests/preg_match_error2-detect.phpt",
    "ext/pcre/tests/bug42945-detect.phpt",
    "ext/pcre/tests/preg_match_basic_002-detect.phpt",
    "ext/pcre/tests/preg_replace2-detect.phpt",
    "ext/pcre/tests/bug41638-detect.phpt",
    "ext/pcre/tests/preg_replace_callback3-detect.phpt",
    "ext/pcre/tests/grep2-detect.phpt",
    "ext/pcre/tests/preg_match_all_error2-detect.phpt",
    "ext/pcre/tests/bug41050-detect.phpt",
    "ext/pcre/tests/preg_replace_callback_error1-detect.phpt",
    "ext/pcre/tests/bug42298-detect.phpt",
    "ext/pcre/tests/pcre_count-detect.phpt",
    "ext/pcre/tests/preg_match_error-detect.phpt",
    "ext/pcre/tests/grep-detect.phpt",
    "ext/pcre/tests/preg_replace_error-detect.phpt",
    "ext/pcre/tests/bug27103-detect.phpt",
    "ext/pcre/tests/bug52732-detect.phpt",
    "ext/pcre/tests/preg_replace_variation1-detect.phpt",
    "ext/pcre/tests/bug42737-detect.phpt",
    "ext/pcre/tests/bug27011-detect.phpt",
    "ext/pcre/tests/preg_replace_error2-detect.phpt",
    "ext/oci8/tests/coll_003-detect.phpt",
    "ext/oci8/tests/bug40415-detect.phpt",
    "ext/oci8/tests/bug36096-detect.phpt",
    "ext/oci8/tests/field_funcs0-detect.phpt",
    "ext/oci8/tests/coll_007-detect.phpt",
    "ext/oci8/tests/array_bind_002-detect.phpt",
    "ext/oci8/tests/function_aliases-detect.phpt",
    "ext/oci8/tests/dbmsoutput-detect.phpt",
    "ext/oci8/tests/bug27303_3-detect.phpt",
    "ext/oci8/tests/connect_1_old-detect.phpt",
    "ext/oci8/tests/bug38173-detect.phpt",
    "ext/oci8/tests/bug42134-detect.phpt",
    "ext/oci8/tests/array_bind_str1-detect.phpt",
    "ext/oci8/tests/drcp_connection_class-detect.phpt",
    "ext/oci8/tests/error-detect.phpt",
    "ext/oci8/tests/bug37220-detect.phpt",
    "ext/oci8/tests/field_funcs_old-detect.phpt",
    "ext/oci8/tests/coll_005-detect.phpt",
    "ext/oci8/tests/lob_014-detect.phpt",
    "ext/oci8/tests/array_bind_003-detect.phpt",
    "ext/oci8/tests/coll_004-detect.phpt",
    "ext/oci8/tests/define_old-detect.phpt",
    "ext/oci8/tests/cursor_bind-detect.phpt",
    "ext/oci8/tests/bind_char_1_11gR1-detect.phpt",
    "ext/oci8/tests/array_bind_010-detect.phpt",
    "ext/oci8/tests/fetch_all3-detect.phpt",
    "ext/oci8/tests/lob_036-detect.phpt",
    "ext/oci8/tests/fetch_object-detect.phpt",
    "ext/oci8/tests/driver_name-detect.phpt",
    "ext/oci8/tests/lob_039-detect.phpt",
    "ext/oci8/tests/coll_014_func-detect.phpt",
    "ext/oci8/tests/lob_010-detect.phpt",
    "ext/oci8/tests/define3-detect.phpt",
    "ext/oci8/tests/drcp_conn_close2-detect.phpt",
    "ext/oci8/tests/coll_002-detect.phpt",
    "ext/oci8/tests/lob_null-detect.phpt",
    "ext/oci8/tests/pecl_bug10194_blob-detect.phpt",
    "ext/oci8/tests/bind_sqltchr_2-detect.phpt",
    "ext/oci8/tests/drcp_pconn_close1-detect.phpt",
    "ext/oci8/tests/coll_009-detect.phpt",
    "ext/oci8/tests/drcp_scope5-detect.phpt",
    "ext/oci8/tests/bind_rowid-detect.phpt",
    "ext/oci8/tests/field_funcs-detect.phpt",
    "ext/oci8/tests/lob_034-detect.phpt",
    "ext/oci8/tests/num-detect.phpt",
    "ext/oci8/tests/connect_with_charset_001-detect.phpt",
    "ext/oci8/tests/bind_long_raw-detect.phpt",
    "ext/oci8/tests/drcp_conn_close1-detect.phpt",
    "ext/oci8/tests/cursor_bind_err-detect.phpt",
    "ext/oci8/tests/lob_002-detect.phpt",
    "ext/oci8/tests/connect_scope_try3-detect.phpt",
    "ext/oci8/tests/minfo-detect.phpt",
    "ext/oci8/tests/coll_013_func-detect.phpt",
    "ext/oci8/tests/lob_007-detect.phpt",
    "ext/oci8/tests/fetch_all4-detect.phpt",
    "ext/oci8/tests/password_2-detect.phpt",
    "ext/oci8/tests/bind_query-detect.phpt",
    "ext/oci8/tests/bind_char_1-detect.phpt",
    "ext/oci8/tests/coll_010_func-detect.phpt",
    "ext/oci8/tests/coll_012_func-detect.phpt",
    "ext/oci8/tests/bug36403-detect.phpt",
    "ext/oci8/tests/array_bind_float1-detect.phpt",
    "ext/oci8/tests/bug38161-detect.phpt",
    "ext/oci8/tests/drcp_scope4-detect.phpt",
    "ext/oci8/tests/array_bind_014-detect.phpt",
    "ext/oci8/tests/bug36010-detect.phpt",
    "ext/oci8/tests/bug27303_4-detect.phpt",
    "ext/oci8/tests/bug35973-detect.phpt",
    "ext/oci8/tests/bind_long-detect.phpt",
    "ext/oci8/tests/uncommitted-detect.phpt",
    "ext/oci8/tests/coll_015-detect.phpt",
    "ext/oci8/tests/refcur_prefetch_3-detect.phpt",
    "ext/oci8/tests/error_old-detect.phpt",
    "ext/oci8/tests/descriptors-detect.phpt",
    "ext/oci8/tests/define-detect.phpt",
    "ext/oci8/tests/array_bind_001-detect.phpt",
    "ext/oci8/tests/bind_sqltint-detect.phpt",
    "ext/oci8/tests/lob_011-detect.phpt",
    "ext/oci8/tests/pecl_bug10194_blob_64-detect.phpt",
    "ext/oci8/tests/connect_without_oracle_home_old-detect.phpt",
    "ext/oci8/tests/password-detect.phpt",
    "ext/oci8/tests/drcp_newconnect-detect.phpt",
    "ext/oci8/tests/oci_execute_segfault-detect.phpt",
    "ext/oci8/tests/cursors_old-detect.phpt",
    "ext/oci8/tests/bind_char_4_11gR1-detect.phpt",
    "ext/oci8/tests/statement_type_old-detect.phpt",
    "ext/oci8/tests/refcur_prefetch_4-detect.phpt",
    "ext/oci8/tests/drcp_privileged-detect.phpt",
    "ext/oci8/tests/array_bind_006-detect.phpt",
    "ext/oci8/tests/coll_017-detect.phpt",
    "ext/oci8/tests/conn_attr_3-detect.phpt",
    "ext/oci8/tests/privileged_connect1-detect.phpt",
    "ext/oci8/tests/error_parse-detect.phpt",
    "ext/oci8/tests/lob_temp1-detect.phpt",
    "ext/oci8/tests/cursors-detect.phpt",
    "ext/oci8/tests/array_bind_date1-detect.phpt",
    "ext/oci8/tests/bug37581-detect.phpt",
    "ext/oci8/tests/serverversion-detect.phpt",
    "ext/oci8/tests/pecl_bug10194-detect.phpt",
    "ext/oci8/tests/coll_002_func-detect.phpt",
    "ext/oci8/tests/fetch_assoc-detect.phpt",
    "ext/oci8/tests/bug43492_2-detect.phpt",
    "ext/oci8/tests/fetch_row-detect.phpt",
    "ext/oci8/tests/bug42496_2-detect.phpt",
    "ext/oci8/tests/bug51253-detect.phpt",
    "ext/oci8/tests/lob_020-detect.phpt",
    "ext/oci8/tests/coll_003_func-detect.phpt",
    "ext/oci8/tests/bug44113-detect.phpt",
    "ext/oci8/tests/define2-detect.phpt",
    "ext/oci8/tests/coll_016-detect.phpt",
    "ext/oci8/tests/dupcolnames-detect.phpt",
    "ext/oci8/tests/testping-detect.phpt",
    "ext/oci8/tests/null_byte_3-detect.phpt",
    "ext/oci8/tests/bug44206-detect.phpt",
    "ext/oci8/tests/coll_012-detect.phpt",
    "ext/oci8/tests/commit_old-detect.phpt",
    "ext/oci8/tests/lob_018-detect.phpt",
    "ext/oci8/tests/bind_empty-detect.phpt",
    "ext/oci8/tests/coll_006_func-detect.phpt",
    "ext/oci8/tests/fetch_all5-detect.phpt",
    "ext/oci8/tests/coll_011_func-detect.phpt",
    "ext/oci8/tests/coll_017_func-detect.phpt",
    "ext/oci8/tests/lob_031-detect.phpt",
    "ext/oci8/tests/lob_030-detect.phpt",
    "ext/oci8/tests/array_bind_008-detect.phpt",
    "ext/oci8/tests/bug51291_1-detect.phpt",
    "ext/oci8/tests/lob_028-detect.phpt",
    "ext/oci8/tests/field_funcs1-detect.phpt",
    "ext/oci8/tests/fetch_object_1-detect.phpt",
    "ext/oci8/tests/lob_009-detect.phpt",
    "ext/oci8/tests/password_new-detect.phpt",
    "ext/oci8/tests/lob_042-detect.phpt",
    "ext/oci8/tests/fetch_array-detect.phpt",
    "ext/oci8/tests/fetch_all2-detect.phpt",
    "ext/oci8/tests/conn_attr_1-detect.phpt",
    "ext/oci8/tests/array_bind_str-detect.phpt",
    "ext/oci8/tests/connect_1-detect.phpt",
    "ext/oci8/tests/coll_006-detect.phpt",
    "ext/oci8/tests/b47243_2-detect.phpt",
    "ext/oci8/tests/statement_type-detect.phpt",
    "ext/oci8/tests/lob_038-detect.phpt",
    "ext/oci8/tests/error1-detect.phpt",
    "ext/oci8/tests/fetch-detect.phpt",
    "ext/oci8/tests/error2-detect.phpt",
    "ext/oci8/tests/array_bind_bdouble-detect.phpt",
    "ext/oci8/tests/old_oci_close-detect.phpt",
    "ext/oci8/tests/bug27303_4_11gR1-detect.phpt",
    "ext/oci8/tests/coll_010-detect.phpt",
    "ext/oci8/tests/lob_032-detect.phpt",
    "ext/oci8/tests/connect_scope_try5-detect.phpt",
    "ext/oci8/tests/fetch_into-detect.phpt",
    "ext/oci8/tests/lob_023-detect.phpt",
    "ext/oci8/tests/array_bind_float-detect.phpt",
    "ext/oci8/tests/pecl_bug8816-detect.phpt",
    "ext/oci8/tests/bind_unsupported_3-detect.phpt",
    "ext/oci8/tests/bug45458-detect.phpt",
    "ext/oci8/tests/prefetch-detect.phpt",
    "ext/oci8/tests/bug47189-detect.phpt",
    "ext/oci8/tests/lob_017-detect.phpt",
    "ext/oci8/tests/bug42173-detect.phpt",
    "ext/oci8/tests/coll_008-detect.phpt",
    "ext/oci8/tests/define6-detect.phpt",
    "ext/oci8/tests/lob_005-detect.phpt",
    "ext/oci8/tests/define5-detect.phpt",
    "ext/oci8/tests/connect-detect.phpt",
    "ext/oci8/tests/coll_016_func-detect.phpt",
    "ext/oci8/tests/extauth_01-detect.phpt",
    "ext/oci8/tests/statement_cache-detect.phpt",
    "ext/oci8/tests/lob_037-detect.phpt",
    "ext/oci8/tests/define1-detect.phpt",
    "ext/oci8/tests/connect_scope_try1-detect.phpt",
    "ext/oci8/tests/select_null-detect.phpt",
    "ext/oci8/tests/conn_attr_4-detect.phpt",
    "ext/oci8/tests/lob_016-detect.phpt",
    "ext/oci8/tests/lob_026-detect.phpt",
    "ext/oci8/tests/bug42496_1-detect.phpt",
    "ext/oci8/tests/pecl_bug6109-detect.phpt",
    "ext/oci8/tests/default_prefetch2-detect.phpt",
    "ext/oci8/tests/bind_number-detect.phpt",
    "ext/oci8/tests/bug40078-detect.phpt",
    "ext/oci8/tests/lob_041-detect.phpt",
    "ext/oci8/tests/error3-detect.phpt",
    "ext/oci8/tests/close-detect.phpt",
    "ext/oci8/tests/fetch_into1-detect.phpt",
    "ext/oci8/tests/lob_033-detect.phpt",
    "ext/oci8/tests/coll_009_func-detect.phpt",
    "ext/oci8/tests/bind_sqltchr_1-detect.phpt",
    "ext/oci8/tests/commit_001-detect.phpt",
    "ext/oci8/tests/default_prefetch0-detect.phpt",
    "ext/oci8/tests/connect_old-detect.phpt",
    "ext/oci8/tests/bug27303_1-detect.phpt",
    "ext/oci8/tests/bug46994-detect.phpt",
    "ext/oci8/tests/lob_008-detect.phpt",
    "ext/oci8/tests/coll_018-detect.phpt",
    "ext/oci8/tests/conn_attr_5-detect.phpt",
    "ext/oci8/tests/drcp_pconn_close2-detect.phpt",
    "ext/oci8/tests/array_bind_009-detect.phpt",
    "ext/oci8/tests/edition_2-detect.phpt",
    "ext/oci8/tests/refcur_prefetch_1-detect.phpt",
    "ext/oci8/tests/bug26133-detect.phpt",
    "ext/oci8/tests/null_byte_2-detect.phpt",
    "ext/oci8/tests/bug43497_92-detect.phpt",
    "ext/oci8/tests/lob_012-detect.phpt",
    "ext/oci8/tests/bug32325-detect.phpt",
    "ext/oci8/tests/array_bind_011-detect.phpt",
    "ext/oci8/tests/coll_013-detect.phpt",
    "ext/oci8/tests/bug47281_tt-detect.phpt",
    "ext/oci8/tests/connect_without_oracle_home_old_11-detect.phpt",
    "ext/oci8/tests/drcp_characterset-detect.phpt",
    "ext/oci8/tests/drcp_cclass1-detect.phpt",
    "ext/oci8/tests/lob_001-detect.phpt",
    "ext/oci8/tests/array_bind_012-detect.phpt",
    "ext/oci8/tests/array_bind_int-detect.phpt",
    "ext/oci8/tests/prefetch_old-detect.phpt",
    "ext/oci8/tests/b47243_1-detect.phpt",
    "ext/oci8/tests/lob_aliases-detect.phpt",
    "ext/oci8/tests/password_old-detect.phpt",
    "ext/oci8/tests/connect_scope1-detect.phpt",
    "ext/oci8/tests/commit_002-detect.phpt",
    "ext/oci8/tests/lob_027-detect.phpt",
    "ext/oci8/tests/field_funcs2-detect.phpt",
    "ext/oci8/tests/bug27303_2-detect.phpt",
    "ext/oci8/tests/lob_035-detect.phpt",
    "ext/oci8/tests/xmltype_02-detect.phpt",
    "ext/oci8/tests/coll_011-detect.phpt",
    "ext/oci8/tests/ini_1-detect.phpt",
    "ext/oci8/tests/bind_char_4-detect.phpt",
    "ext/oci8/tests/lob_temp-detect.phpt",
    "ext/oci8/tests/b47243_3-detect.phpt",
    "ext/oci8/tests/drcp_connect1-detect.phpt",
    "ext/oci8/tests/coll_015_func-detect.phpt",
    "ext/oci8/tests/drcp_scope1-detect.phpt",
    "ext/oci8/tests/bug47281-detect.phpt",
    "ext/oci8/tests/clientversion-detect.phpt",
    "ext/oci8/tests/conn_attr_2-detect.phpt",
    "ext/oci8/tests/bug41069-detect.phpt",
    "ext/oci8/tests/coll_004_func-detect.phpt",
    "ext/oci8/tests/coll_001-detect.phpt",
    "ext/oci8/tests/drcp_scope2-detect.phpt",
    "ext/oci8/tests/bug51291_2-detect.phpt",
    "ext/oci8/tests/lob_024-detect.phpt",
    "ext/oci8/tests/lob_044-detect.phpt",
    "ext/oci8/tests/bind_unsupported_1-detect.phpt",
    "ext/oci8/tests/array_bind_bfloat-detect.phpt",
    "ext/oci8/tests/connect_scope2-detect.phpt",
    "ext/oci8/tests/fetch_object_2-detect.phpt",
    "ext/oci8/tests/array_bind_int1-detect.phpt",
    "ext/oci8/tests/lob_043-detect.phpt",
    "ext/oci8/tests/fetch_all1-detect.phpt",
    "ext/oci8/tests/array_bind_004-detect.phpt",
    "ext/oci8/tests/bind_misccoltypes-detect.phpt",
    "ext/oci8/tests/lob_015-detect.phpt",
    "ext/oci8/tests/bind_sqltnum-detect.phpt",
    "ext/oci8/tests/error_bind-detect.phpt",
    "ext/oci8/tests/bug27303_1_11gR1-detect.phpt",
    "ext/oci8/tests/lob_006-detect.phpt",
    "ext/oci8/tests/default_prefetch1-detect.phpt",
    "ext/oci8/tests/edition_1-detect.phpt",
    "ext/oci8/tests/array_bind_uin-detect.phpt",
    "ext/oci8/tests/bind_misccoltypes_errs-detect.phpt",
    "ext/oci8/tests/bind_char_2_11gR1-detect.phpt",
    "ext/oci8/tests/bind_char_3_11gR1-detect.phpt",
    "ext/oci8/tests/connect_scope_try6-detect.phpt",
    "ext/oci8/tests/exec_fetch-detect.phpt",
    "ext/oci8/tests/bug43497-detect.phpt",
    "ext/oci8/tests/reflection2-detect.phpt",
    "ext/oci8/tests/fetch_into2-detect.phpt",
    "ext/oci8/tests/array_bind_013-detect.phpt",
    "ext/oci8/tests/lob_019-detect.phpt",
    "ext/oci8/tests/connect_scope_try4-detect.phpt",
    "ext/oci8/tests/default_prefetch-detect.phpt",
    "ext/oci8/tests/bug44008-detect.phpt",
    "ext/oci8/tests/bug42841-detect.phpt",
    "ext/oci8/tests/coll_014-detect.phpt",
    "ext/oci8/tests/bind_char_3-detect.phpt",
    "ext/oci8/tests/xmltype_01-detect.phpt",
    "ext/oci8/tests/array_bind_date-detect.phpt",
    "ext/oci8/tests/null_byte_1-detect.phpt",
    "ext/oci8/tests/lob_025-detect.phpt",
    "ext/oci8/tests/extauth_04-detect.phpt",
    "ext/oci8/tests/connect_scope_try2-detect.phpt",
    "ext/oci8/tests/extauth_02-detect.phpt",
    "ext/oci8/tests/define4-detect.phpt",
    "ext/oci8/tests/lob_029-detect.phpt",
    "ext/oci8/tests/lob_004-detect.phpt",
    "ext/oci8/tests/array_bind_005-detect.phpt",
    "ext/oci8/tests/connect_without_oracle_home_11-detect.phpt",
    "ext/oci8/tests/pecl_bug16842-detect.phpt",
    "ext/oci8/tests/execute_mode-detect.phpt",
    "ext/oci8/tests/lob_021-detect.phpt",
    "ext/oci8/tests/reflection1-detect.phpt",
    "ext/oci8/tests/define0-detect.phpt",
    "ext/oci8/tests/bind_char_2-detect.phpt",
    "ext/oci8/tests/lob_022-detect.phpt",
    "ext/oci8/tests/bug27303_2_11gR1-detect.phpt",
    "ext/oci8/tests/bind_raw-detect.phpt",
    "ext/oci8/tests/lob_003-detect.phpt",
    "ext/oci8/tests/field_funcs3-detect.phpt",
    "ext/oci8/tests/connect_without_oracle_home-detect.phpt",
    "ext/oci8/tests/clientversion_92-detect.phpt",
    "ext/oci8/tests/debug-detect.phpt",
    "ext/oci8/tests/extauth_03-detect.phpt",
    "ext/oci8/tests/bug43492-detect.phpt",
    "ext/oci8/tests/bind_unsupported_2-detect.phpt",
    "ext/oci8/tests/persistent-detect.phpt",
    "ext/oci8/tests/lob_040-detect.phpt",
    "ext/oci8/tests/fetch_all-detect.phpt",
    "ext/oci8/tests/bind_raw_2-detect.phpt",
    "ext/oci8/tests/pecl_bug16035-detect.phpt",
    "ext/oci8/tests/coll_019-detect.phpt",
    "ext/oci8/tests/bind_sqltafc-detect.phpt",
    "ext/oci8/tests/refcur_prefetch_2-detect.phpt",
    "ext/oci8/tests/array_bind_007-detect.phpt",
    "ext/oci8/tests/drcp_scope3-detect.phpt",
    "ext/oci8/tests/old_oci_close1-detect.phpt",
    "ext/oci8/tests/privileged_connect-detect.phpt",
    "ext/oci8/tests/lob_013-detect.phpt",
    "ext/exif/tests/bug34704-detect.phpt",
    "ext/exif/tests/exif015-detect.phpt",
    "ext/exif/tests/exif004-detect.phpt",
    "ext/exif/tests/exif027-detect.phpt",
    "ext/exif/tests/exif_imagetype_basic-detect.phpt",
    "ext/exif/tests/exif026-detect.phpt",
    "ext/exif/tests/exif008-detect.phpt",
    "ext/exif/tests/exif013-detect.phpt",
    "ext/exif/tests/exif024-detect.phpt",
    "ext/exif/tests/exif021-detect.phpt",
    "ext/exif/tests/exif014-detect.phpt",
    "ext/exif/tests/exif017-detect.phpt",
    "ext/exif/tests/exif025-detect.phpt",
    "ext/exif/tests/exif000-detect.phpt",
    "ext/exif/tests/exif012-detect.phpt",
    "ext/exif/tests/exif002-detect.phpt",
    "ext/exif/tests/exif009-detect.phpt",
    "ext/exif/tests/exif022-detect.phpt",
    "ext/exif/tests/exif_imagetype_error-detect.phpt",
    "ext/exif/tests/bug60150-detect.phpt",
    "ext/exif/tests/exif003-detect.phpt",
    "ext/exif/tests/exif016-detect.phpt",
    "ext/exif/tests/exif011-detect.phpt",
    "ext/exif/tests/exif_read_exif_data_basic-detect.phpt",
    "ext/exif/tests/exif018-detect.phpt",
    "ext/exif/tests/exif_imagetype_variation1-detect.phpt",
    "ext/exif/tests/exif_tagname_basic-detect.phpt",
    "ext/exif/tests/exif001-detect.phpt",
    "ext/exif/tests/bug54002-detect.phpt",
    "ext/exif/tests/exif010-detect.phpt",
    "ext/exif/tests/exif005-detect.phpt",
    "ext/exif/tests/bug48378-detect.phpt",
    "ext/exif/tests/exif007-detect.phpt",
    "ext/exif/tests/exif020-detect.phpt",
    "ext/exif/tests/exif_tagname_error-detect.phpt",
    "ext/exif/tests/exif_tagname_variation1-detect.phpt",
    "ext/exif/tests/exif023-detect.phpt",
    "ext/json/tests/bug42785-detect.phpt",
    "ext/json/tests/005-detect.phpt",
    "ext/json/tests/bug41504-detect.phpt",
    "ext/json/tests/bug54058-detect.phpt",
    "ext/json/tests/bug42090-detect.phpt",
    "ext/json/tests/json_decode_basic-detect.phpt",
    "ext/json/tests/pass003-detect.phpt",
    "ext/json/tests/serialize-detect.phpt",
    "ext/json/tests/001-detect.phpt",
    "ext/json/tests/004-detect.phpt",
    "ext/json/tests/pass0011_64bit-detect.phpt",
    "ext/json/tests/bug43941-detect.phpt",
    "ext/json/tests/bug41067-detect.phpt",
    "ext/json/tests/json_encode_numeric-detect.phpt",
    "ext/json/tests/json_encode_pretty_print-detect.phpt",
    "ext/json/tests/003-detect.phpt",
    "ext/json/tests/json_encode_basic-detect.phpt",
    "ext/json/tests/bug41567-detect.phpt",
    "ext/json/tests/bug47644-detect.phpt",
    "ext/json/tests/006-detect.phpt",
    "ext/json/tests/json_encode_error-detect.phpt",
    "ext/json/tests/json_encode_unescaped_slashes-detect.phpt",
    "ext/json/tests/bug46944-detect.phpt",
    "ext/json/tests/bug40503-detect.phpt",
    "ext/json/tests/007-detect.phpt",
    "ext/json/tests/pass0011-detect.phpt",
    "ext/json/tests/bug45791-detect.phpt",
    "ext/json/tests/002-detect.phpt",
    "ext/json/tests/008-detect.phpt",
    "ext/json/tests/bug61978-detect.phpt",
    "ext/json/tests/bug55543-detect.phpt",
    "ext/json/tests/json_encode_basic_utf8-detect.phpt",
    "ext/json/tests/bug54484-detect.phpt",
    "ext/json/tests/bug41403-detect.phpt",
    "ext/json/tests/bug53946-detect.phpt",
    "ext/json/tests/json_decode_error-detect.phpt",
    "ext/json/tests/bug46215-detect.phpt",
    "ext/json/tests/fail001-detect.phpt",
    "ext/json/tests/bug41034-detect.phpt",
    "ext/json/tests/pass001-detect.phpt",
    "ext/json/tests/pass002-detect.phpt",
    "ext/pspell/tests/005-detect.phpt",
    "ext/pspell/tests/001-detect.phpt",
    "ext/pspell/tests/004-detect.phpt",
    "ext/pspell/tests/003-detect.phpt",
    "ext/pspell/tests/002-detect.phpt",
    "ext/intl/tests/resourcebundle_internal-detect.phpt",
    "ext/intl/tests/formatter_get_locale-detect.phpt",
    "ext/intl/tests/locale_get_keywords2-detect.phpt",
    "ext/intl/tests/dateformat_get_locale-detect.phpt",
    "ext/intl/tests/dateformat_format_parse-detect.phpt",
    "ext/intl/tests/formatter_get_set_attribute-detect.phpt",
    "ext/intl/tests/formatter_parse-detect.phpt",
    "ext/intl/tests/resourcebundle_create-detect.phpt",
    "ext/intl/tests/transliterator_get_error_message_error-detect.phpt",
    "ext/intl/tests/bug62081-detect.phpt",
    "ext/intl/tests/collator_compare-detect.phpt",
    "ext/intl/tests/symfony_format_type_double_intl2-detect.phpt",
    "ext/intl/tests/transliterator_get_error_code_basic-detect.phpt",
    "ext/intl/tests/locale_get_display_variant-detect.phpt",
    "ext/intl/tests/transliterator_create_inverse_error-detect.phpt",
    "ext/intl/tests/locale_get_display_region2-detect.phpt",
    "ext/intl/tests/symfony_format_type_int64_intl1-detect.phpt",
    "ext/intl/tests/locale_get_script-detect.phpt",
    "ext/intl/tests/locale_get_display_script2-detect.phpt",
    "ext/intl/tests/bug60192-getsortkey-detect.phpt",
    "ext/intl/tests/dateformat_get_set_calendar-detect.phpt",
    "ext/intl/tests/transliterator_property_id-detect.phpt",
    "ext/intl/tests/bug62082-detect.phpt",
    "ext/intl/tests/transliterator_create_basic-detect.phpt",
    "ext/intl/tests/normalizer_normalize-detect.phpt",
    "ext/intl/tests/formatter_get_set_symbol2-detect.phpt",
    "ext/intl/tests/formatter_get_set_pattern-detect.phpt",
    "ext/intl/tests/locale_get_display_variant2-detect.phpt",
    "ext/intl/tests/msgfmt_get_error-detect.phpt",
    "ext/intl/tests/msgfmt_parse-detect.phpt",
    "ext/intl/tests/regression_sort_eq-detect.phpt",
    "ext/intl/tests/idn-detect.phpt",
    "ext/intl/tests/dateformat_clone-detect.phpt",
    "ext/intl/tests/dateformat_get_datetype-detect.phpt",
    "ext/intl/tests/collator_sort-detect.phpt",
    "ext/intl/tests/bug53512-detect.phpt",
    "ext/intl/tests/collator_get_error_code-detect.phpt",
    "ext/intl/tests/dateformat_set_timezone_id-detect.phpt",
    "ext/intl/tests/bug12887-detect.phpt",
    "ext/intl/tests/locale_get_display_name-detect.phpt",
    "ext/intl/tests/transliterator_transliterate_error-detect.phpt",
    "ext/intl/tests/bug60192-sort-detect.phpt",
    "ext/intl/tests/idn_uts46_errors-detect.phpt",
    "ext/intl/tests/symfony_format_type_int32_intl3-detect.phpt",
    "ext/intl/tests/symfony_format_type_int32_intl6-detect.phpt",
    "ext/intl/tests/msgfmt_fail-detect.phpt",
    "ext/intl/tests/locale_get_display_script-detect.phpt",
    "ext/intl/tests/transliterator_get_error_message_basic-detect.phpt",
    "ext/intl/tests/locale_accept-detect.phpt",
    "ext/intl/tests/formatter_fail-detect.phpt",
    "ext/intl/tests/intl_icu_data_version_constant-detect.phpt",
    "ext/intl/tests/msgfmt_clone_bad_obj-detect.phpt",
    "ext/intl/tests/dateformat_parse-detect.phpt",
    "ext/intl/tests/dateformat_clone2-detect.phpt",
    "ext/intl/tests/formatter_format_currency-detect.phpt",
    "ext/intl/tests/dateformat_get_timetype-detect.phpt",
    "ext/intl/tests/regression_sortwsk_eq-detect.phpt",
    "ext/intl/tests/collator_asort-detect.phpt",
    "ext/intl/tests/bug62070-detect.phpt",
    "ext/intl/tests/bug61487-detect.phpt",
    "ext/intl/tests/locale_get_display_name2-detect.phpt",
    "ext/intl/tests/spoofchecker_001-detect.phpt",
    "ext/intl/tests/msgfmt_clone-detect.phpt",
    "ext/intl/tests/symfony_format_type_double_intl1-detect.phpt",
    "ext/intl/tests/locale_get_display_region-detect.phpt",
    "ext/intl/tests/formatter_format_currency2-detect.phpt",
    "ext/intl/tests/spoofchecker_003-detect.phpt",
    "ext/intl/tests/bug62915-2-detect.phpt",
    "ext/intl/tests/transliterator_get_error_code_error-detect.phpt",
    "ext/intl/tests/transliterator_transliterate_basic-detect.phpt",
    "ext/intl/tests/formatter_format-detect.phpt",
    "ext/intl/tests/collator_get_set_strength-detect.phpt",
    "ext/intl/tests/resourcebundle_iterator-detect.phpt",
    "ext/intl/tests/symfony_format_type_double_intl3-detect.phpt",
    "ext/intl/tests/locale_get_all_variants-detect.phpt",
    "ext/intl/tests/resourcebundle_locales-detect.phpt",
    "ext/intl/tests/bug62017-detect.phpt",
    "ext/intl/tests/collation_customization-detect.phpt",
    "ext/intl/tests/symfony_format_type_int64_intl8-detect.phpt",
    "ext/intl/tests/bug14562-detect.phpt",
    "ext/intl/tests/dateformat_get_timezone_id-detect.phpt",
    "ext/intl/tests/locale_lookup-detect.phpt",
    "ext/intl/tests/bug48227-detect.phpt",
    "ext/intl/tests/intl_is_failure-detect.phpt",
    "ext/intl/tests/formatter_get_set_symbol-detect.phpt",
    "ext/intl/tests/collator_create-detect.phpt",
    "ext/intl/tests/symfony_format_type_int64_intl6-detect.phpt",
    "ext/intl/tests/locale_parse_locale2-detect.phpt",
    "ext/intl/tests/dateformat_parse_localtime_parsepos-detect.phpt",
    "ext/intl/tests/symfony_format_type_double_intl4-detect.phpt",
    "ext/intl/tests/collator_get_set_attribute-detect.phpt",
    "ext/intl/tests/collator_get_locale2-detect.phpt",
    "ext/intl/tests/locale_get_keywords-detect.phpt",
    "ext/intl/tests/dateformat_clone_bad_obj-detect.phpt",
    "ext/intl/tests/locale_parse_locale-detect.phpt",
    "ext/intl/tests/collator_get_locale-detect.phpt",
    "ext/intl/tests/intl_error_name-detect.phpt",
    "ext/intl/tests/intl_get_error_code-detect.phpt",
    "ext/intl/tests/badargs-detect.phpt",
    "ext/intl/tests/formatter_parse_currency-detect.phpt",
    "ext/intl/tests/transliterator_clone-detect.phpt",
    "ext/intl/tests/resourcebundle_traversable-detect.phpt",
    "ext/intl/tests/locale_compose_locale-detect.phpt",
    "ext/intl/tests/locale_get_primary_language-detect.phpt",
    "ext/intl/tests/formatter_clone_bad_obj-detect.phpt",
    "ext/intl/tests/locale_set_default-detect.phpt",
    "ext/intl/tests/transliterator_transliterate_variant1-detect.phpt",
    "ext/intl/tests/msgfmt_get_locale-detect.phpt",
    "ext/intl/tests/symfony_format_type_int32_intl4-detect.phpt",
    "ext/intl/tests/transliterator_create_inverse_basic-detect.phpt",
    "ext/intl/tests/collator_sort_with_sort_keys-detect.phpt",
    "ext/intl/tests/idn_uts46_basic-detect.phpt",
    "ext/intl/tests/symfony_format_type_int32_intl1-detect.phpt",
    "ext/intl/tests/symfony_format_type_int32_intl2-detect.phpt",
    "ext/intl/tests/dateformat_calendars-detect.phpt",
    "ext/intl/tests/dateformat_localtime-detect.phpt",
    "ext/intl/tests/formatter_format_conv-detect.phpt",
    "ext/intl/tests/dateformat_get_set_pattern2-detect.phpt",
    "ext/intl/tests/formatter_format2-detect.phpt",
    "ext/intl/tests/regression_sort_and_cow-detect.phpt",
    "ext/intl/tests/bug60192-getlocale-detect.phpt",
    "ext/intl/tests/locale_get_display_language-detect.phpt",
    "ext/intl/tests/transliterator_list_ids_basic-detect.phpt",
    "ext/intl/tests/symfony_format_type_int64_intl5-detect.phpt",
    "ext/intl/tests/bug50590-detect.phpt",
    "ext/intl/tests/dateformat_parse_timestamp_parsepos-detect.phpt",
    "ext/intl/tests/collator_get_sort_key-detect.phpt",
    "ext/intl/tests/intl_icu_version_constant-detect.phpt",
    "ext/intl/tests/transliterator_create_from_rule_basic-detect.phpt",
    "ext/intl/tests/dateformat_set_timezone_id2-detect.phpt",
    "ext/intl/tests/grapheme-detect.phpt",
    "ext/intl/tests/locale_get_region-detect.phpt",
    "ext/intl/tests/bug59597_64-detect.phpt",
    "ext/intl/tests/bug62083-detect.phpt",
    "ext/intl/tests/spoofchecker_004-detect.phpt",
    "ext/intl/tests/regression_sortwsk_and_cow-detect.phpt",
    "ext/intl/tests/intl_get_error_message-detect.phpt",
    "ext/intl/tests/transliterator_create_error-detect.phpt",
    "ext/intl/tests/formatter_get_error-detect.phpt",
    "ext/intl/tests/collator_create2-detect.phpt",
    "ext/intl/tests/dateformat_is_set_lenient-detect.phpt",
    "ext/intl/tests/locale_filter_matches-detect.phpt",
    "ext/intl/tests/symfony_format_type_int64_intl4-detect.phpt",
    "ext/intl/tests/spoofchecker_002-detect.phpt",
    "ext/intl/tests/transliterator_list_ids_error-detect.phpt",
    "ext/intl/tests/symfony_format_type_int32_intl5-detect.phpt",
    "ext/intl/tests/resourcebundle_individual-detect.phpt",
    "ext/intl/tests/symfony_format_type_int32_intl7-detect.phpt",
    "ext/intl/tests/transliterator_create_from_rule_error-detect.phpt",
    "ext/intl/tests/collator_get_error_message-detect.phpt",
    "ext/intl/tests/symfony_format_type_int64_intl7-detect.phpt",
    "ext/intl/tests/bug59597_32-detect.phpt",
    "ext/intl/tests/formatter_get_set_text_attribute-detect.phpt",
    "ext/intl/tests/resourcebundle_null_mandatory_args-detect.phpt",
    "ext/intl/tests/locale_filter_matches2-detect.phpt",
    "ext/intl/tests/formatter_clone-detect.phpt",
    "ext/intl/tests/bug60192-compare-detect.phpt",
    "ext/intl/tests/dateformat_format-detect.phpt",
    "ext/intl/tests/msgfmt_get_set_pattern-detect.phpt",
    "ext/intl/tests/symfony_format_type_int64_intl3-detect.phpt",
    "ext/intl/tests/msgfmt_fail2-detect.phpt",
    "ext/intl/tests/locale_get_default-detect.phpt",
    "ext/intl/tests/bug60192-sortwithsortkeys-detect.phpt",
    "ext/intl/tests/bug55562-detect.phpt",
    "ext/intl/tests/msgfmt_format-detect.phpt",
    "ext/intl/tests/dateformat_get_set_pattern-detect.phpt",
    "ext/intl/tests/resourcebundle_arrayaccess-detect.phpt",
    "ext/intl/tests/symfony_format_type_int64_intl2-detect.phpt",
    "ext/intl/tests/locale_get_display_script3-detect.phpt",
    "ext/soap/tests/server009-detect.phpt",
    "ext/soap/tests/classmap004-detect.phpt",
    "ext/soap/tests/server021-detect.phpt",
    "ext/soap/tests/classmap003-detect.phpt",
    "ext/soap/tests/server006-detect.phpt",
    "ext/soap/tests/typemap004-detect.phpt",
    "ext/soap/tests/server030-detect.phpt",
    "ext/soap/tests/typemap008-detect.phpt",
    "ext/soap/tests/server022-detect.phpt",
    "ext/soap/tests/bugs/bug37013-detect.phpt",
    "ext/soap/tests/bugs/bug31755-detect.phpt",
    "ext/soap/tests/bugs/bug54911-detect.phpt",
    "ext/soap/tests/bugs/bug29109-detect.phpt",
    "ext/soap/tests/bugs/bug35273-detect.phpt",
    "ext/soap/tests/bugs/bug41004-detect.phpt",
    "ext/soap/tests/bugs/bug46419-detect.phpt",
    "ext/soap/tests/bugs/bug32776-detect.phpt",
    "ext/soap/tests/bugs/bug36614-detect.phpt",
    "ext/soap/tests/bugs/bug42692-detect.phpt",
    "ext/soap/tests/bugs/bug37278-detect.phpt",
    "ext/soap/tests/bugs/bug42326-detect.phpt",
    "ext/soap/tests/bugs/bug30994-detect.phpt",
    "ext/soap/tests/bugs/bug50698_4-detect.phpt",
    "ext/soap/tests/bugs/bug42488-detect.phpt",
    "ext/soap/tests/bugs/bug36999-detect.phpt",
    "ext/soap/tests/bugs/bug34643-detect.phpt",
    "ext/soap/tests/bugs/bug42183-detect.phpt",
    "ext/soap/tests/bugs/multiport-detect.phpt",
    "ext/soap/tests/bugs/bug50698_1-detect.phpt",
    "ext/soap/tests/bugs/bug47273-detect.phpt",
    "ext/soap/tests/bugs/bug41337_2-detect.phpt",
    "ext/soap/tests/bugs/bug27742-detect.phpt",
    "ext/soap/tests/bugs/bug41477-detect.phpt",
    "ext/soap/tests/bugs/bug37083-detect.phpt",
    "ext/soap/tests/bugs/bug38536-detect.phpt",
    "ext/soap/tests/bugs/bug55323-detect.phpt",
    "ext/soap/tests/bugs/bug38067-detect.phpt",
    "ext/soap/tests/bugs/bug41097-detect.phpt",
    "ext/soap/tests/bugs/bug31422-win-detect.phpt",
    "ext/soap/tests/bugs/bug31422-detect.phpt",
    "ext/soap/tests/bugs/bug31695-detect.phpt",
    "ext/soap/tests/bugs/bug40609-detect.phpt",
    "ext/soap/tests/bugs/bug44686-detect.phpt",
    "ext/soap/tests/bugs/bug34449-detect.phpt",
    "ext/soap/tests/bugs/bug36226-2-detect.phpt",
    "ext/soap/tests/bugs/bug28969-detect.phpt",
    "ext/soap/tests/bugs/bug36575-detect.phpt",
    "ext/soap/tests/bugs/bug29830-detect.phpt",
    "ext/soap/tests/bugs/bug42359-detect.phpt",
    "ext/soap/tests/bugs/bug36629-detect.phpt",
    "ext/soap/tests/bugs/bug46427-detect.phpt",
    "ext/soap/tests/bugs/bug42086-detect.phpt",
    "ext/soap/tests/bugs/bug30928-detect.phpt",
    "ext/soap/tests/bugs/bug44811-detect.phpt",
    "ext/soap/tests/bugs/bug36226-detect.phpt",
    "ext/soap/tests/bugs/bug36908-detect.phpt",
    "ext/soap/tests/bugs/bug42151-detect.phpt",
    "ext/soap/tests/bugs/bug34453-detect.phpt",
    "ext/soap/tests/bugs/bug41337-detect.phpt",
    "ext/soap/tests/bugs/bug44882-detect.phpt",
    "ext/soap/tests/bugs/bug50698_3-detect.phpt",
    "ext/soap/tests/bugs/bug29061-detect.phpt",
    "ext/soap/tests/bugs/bug39832-detect.phpt",
    "ext/soap/tests/bugs/bug35142-detect.phpt",
    "ext/soap/tests/bugs/bug39121-detect.phpt",
    "ext/soap/tests/bugs/bug47049-detect.phpt",
    "ext/soap/tests/bugs/bug39815-detect.phpt",
    "ext/soap/tests/bugs/bug50762-detect.phpt",
    "ext/soap/tests/bugs/bug30106-detect.phpt",
    "ext/soap/tests/bugs/bug28985-detect.phpt",
    "ext/soap/tests/bugs/bug29839-detect.phpt",
    "ext/soap/tests/bugs/bug32941-detect.phpt",
    "ext/soap/tests/bugs/bug30799-detect.phpt",
    "ext/soap/tests/bugs/bug38055-detect.phpt",
    "ext/soap/tests/bugs/bug42214-detect.phpt",
    "ext/soap/tests/bugs/bug30045-detect.phpt",
    "ext/soap/tests/bugs/bug38005-detect.phpt",
    "ext/soap/tests/bugs/bug29795-detect.phpt",
    "ext/soap/tests/bugs/bug38004-detect.phpt",
    "ext/soap/tests/bugs/bug41566-detect.phpt",
    "ext/soap/tests/bugs/bug29236-detect.phpt",
    "ext/soap/tests/bugs/bug50698_2-detect.phpt",
    "ext/soap/tests/bugs/bug30175-detect.phpt",
    "ext/soap/tests/bugs/bug28751-detect.phpt",
    "ext/soap/tests/bugs/bug27722-detect.phpt",
    "ext/soap/tests/bugs/bug29844-detect.phpt",
    "ext/soap/tests/bugs/bug43045-detect.phpt",
    "ext/soap/tests/bugs/bug34657-detect.phpt",
    "ext/soap/tests/server015-detect.phpt",
    "ext/soap/tests/server014-detect.phpt",
    "ext/soap/tests/server016-detect.phpt",
    "ext/soap/tests/bug46760-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupE/r3_groupE_list_002w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupE/r3_groupE_list_006w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupE/r3_groupE_list_005w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupE/r3_groupE_list_001w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupE/r3_groupE_list_003w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupE/r3_groupE_list_004w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_rpcenc_002w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_rpcenc_001w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_doclitparams_001w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_compound1_002w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_import2_001w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_import3_001w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_doclit_002w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_compound1_001w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_import1_001w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_rpcenc_003w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_doclit_003w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_doclitparams_004w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_import3_002w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_compound2_001w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_doclit_001w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_compound1_003w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_doclitparams_002w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_doclitparams_003w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_rpcenc_004w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_doclit_004w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupD/r3_groupD_emptysa_001w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupF/r3_groupF_headers_001w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupF/r3_groupF_headers_002w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupF/r3_groupF_headers_004w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupF/r3_groupF_headers_003w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupF/r3_groupF_ext_001w-detect.phpt",
    "ext/soap/tests/interop/Round3/GroupF/r3_groupF_extreq_001w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_029w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_022w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_008w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_027w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_013w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_015w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_025w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_010w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_009w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_012w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_018w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_020w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_017w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_005w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_030w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_014w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_023w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_028w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_016w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_007w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_024w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_004w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_031w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_026w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_011w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_033w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_034w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_002w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_006w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_021w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_001w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_003w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_019w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_035w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupI/r4_groupI_xsd_032w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_rpcenc_005w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_soapfault_001w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_012w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_017w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_rpcenc_009w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_015w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_014w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_005w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_rpcenc_007w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_doclit_002w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_doclit_009w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_001w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_soapfault_003w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_001w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_009w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_012w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_rpcenc_010w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_rpcenc_008w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_004w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_doclit_010w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_002w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_doclit_005w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_doclit_008w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_003w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_015w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_017w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_005w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_rpcenc_006w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_013w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_doclit_006w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_016w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_003w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_rpcenc_001w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_soapfault_002w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_007w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_010w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_009w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_002w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_010w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_008w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_soapfault_004w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_011w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_011w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_007w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_008w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_016w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_rpcenc_003w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_doclit_004w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_rpcenc_002w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_rpcenc_004w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_014w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_doclit_007w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_006w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_013w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_rpcenc_004w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_doclit_003w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_complex_doclit_001w-detect.phpt",
    "ext/soap/tests/interop/Round4/GroupH/r4_groupH_simple_doclit_006w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_011w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_011s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_002w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_016p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_020w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_001p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_015p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_017w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_003s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_015s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_007w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_007s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_017p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_005w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_001w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_011p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_021w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_024p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_024s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_021s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_004s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_012w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_004p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_022w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_013w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_023w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_013s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_023s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_002s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_013p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_003p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_010s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_019w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_009s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_005s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_021p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_003w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_008w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_019p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_023p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_005p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_012p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_008p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_010w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_020p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_018w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_002p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_018p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_019s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_007p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_022p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_006p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_014s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_015w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_009p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_010p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_012s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_006w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_024w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_014p-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_001s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_008s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_020s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_006s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_016s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_018s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_014w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_009w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_022s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_017s-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_004w-detect.phpt",
    "ext/soap/tests/interop/Round2/Base/r2_base_016w-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_005p-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_002w-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_002p-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_001p-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_004s-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_004w-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_003w-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_001s-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_005w-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_001w-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_005s-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_003s-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_002s-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_004p-detect.phpt",
    "ext/soap/tests/interop/Round2/GroupB/r2_groupB_003p-detect.phpt",
    "ext/soap/tests/server023-detect.phpt",
    "ext/soap/tests/setheaders-detect.phpt",
    "ext/soap/tests/typemap007-detect.phpt",
    "ext/soap/tests/server025-detect.phpt",
    "ext/soap/tests/classmap002-detect.phpt",
    "ext/soap/tests/typemap001-detect.phpt",
    "ext/soap/tests/server010-detect.phpt",
    "ext/soap/tests/server004-detect.phpt",
    "ext/soap/tests/server020-detect.phpt",
    "ext/soap/tests/server005-detect.phpt",
    "ext/soap/tests/server001-detect.phpt",
    "ext/soap/tests/server012-detect.phpt",
    "ext/soap/tests/server007-detect.phpt",
    "ext/soap/tests/any-detect.phpt",
    "ext/soap/tests/typemap013-detect.phpt",
    "ext/soap/tests/server028-detect.phpt",
    "ext/soap/tests/server026-detect.phpt",
    "ext/soap/tests/server018-detect.phpt",
    "ext/soap/tests/transport001-detect.phpt",
    "ext/soap/tests/server029-detect.phpt",
    "ext/soap/tests/bug48557-detect.phpt",
    "ext/soap/tests/typemap012-detect.phpt",
    "ext/soap/tests/server017-detect.phpt",
    "ext/soap/tests/typemap010-detect.phpt",
    "ext/soap/tests/typemap011-detect.phpt",
    "ext/soap/tests/server008-detect.phpt",
    "ext/soap/tests/soap12/T10-detect.phpt",
    "ext/soap/tests/soap12/T80-detect.phpt",
    "ext/soap/tests/soap12/T28-detect.phpt",
    "ext/soap/tests/soap12/T45-detect.phpt",
    "ext/soap/tests/soap12/T14-detect.phpt",
    "ext/soap/tests/soap12/T41-detect.phpt",
    "ext/soap/tests/soap12/T36-detect.phpt",
    "ext/soap/tests/soap12/T12-detect.phpt",
    "ext/soap/tests/soap12/T30-detect.phpt",
    "ext/soap/tests/soap12/T40-detect.phpt",
    "ext/soap/tests/soap12/T66-detect.phpt",
    "ext/soap/tests/soap12/T77_1-detect.phpt",
    "ext/soap/tests/soap12/T72-detect.phpt",
    "ext/soap/tests/soap12/T55-detect.phpt",
    "ext/soap/tests/soap12/T74-detect.phpt",
    "ext/soap/tests/soap12/T15-detect.phpt",
    "ext/soap/tests/soap12/T13-detect.phpt",
    "ext/soap/tests/soap12/T48-detect.phpt",
    "ext/soap/tests/soap12/T02-detect.phpt",
    "ext/soap/tests/soap12/T65-detect.phpt",
    "ext/soap/tests/soap12/T03-detect.phpt",
    "ext/soap/tests/soap12/T56-detect.phpt",
    "ext/soap/tests/soap12/T04-detect.phpt",
    "ext/soap/tests/soap12/T78-detect.phpt",
    "ext/soap/tests/soap12/T34-detect.phpt",
    "ext/soap/tests/soap12/T35-detect.phpt",
    "ext/soap/tests/soap12/T76_1-detect.phpt",
    "ext/soap/tests/soap12/T75-detect.phpt",
    "ext/soap/tests/soap12/T73-detect.phpt",
    "ext/soap/tests/soap12/T58-detect.phpt",
    "ext/soap/tests/soap12/T26-detect.phpt",
    "ext/soap/tests/soap12/T24-detect.phpt",
    "ext/soap/tests/soap12/T47-detect.phpt",
    "ext/soap/tests/soap12/T38_1-detect.phpt",
    "ext/soap/tests/soap12/T46-detect.phpt",
    "ext/soap/tests/soap12/T69-detect.phpt",
    "ext/soap/tests/soap12/T64-detect.phpt",
    "ext/soap/tests/soap12/T76_2-detect.phpt",
    "ext/soap/tests/soap12/T61-detect.phpt",
    "ext/soap/tests/soap12/T77_3-detect.phpt",
    "ext/soap/tests/soap12/T33-detect.phpt",
    "ext/soap/tests/soap12/T23-detect.phpt",
    "ext/soap/tests/soap12/T25-detect.phpt",
    "ext/soap/tests/soap12/T54-detect.phpt",
    "ext/soap/tests/soap12/T49-detect.phpt",
    "ext/soap/tests/soap12/T50-detect.phpt",
    "ext/soap/tests/soap12/T05-detect.phpt",
    "ext/soap/tests/soap12/T59-detect.phpt",
    "ext/soap/tests/soap12/T22-detect.phpt",
    "ext/soap/tests/soap12/T42-detect.phpt",
    "ext/soap/tests/soap12/T29-detect.phpt",
    "ext/soap/tests/soap12/T52-detect.phpt",
    "ext/soap/tests/soap12/T70-detect.phpt",
    "ext/soap/tests/soap12/T63-detect.phpt",
    "ext/soap/tests/soap12/T60-detect.phpt",
    "ext/soap/tests/soap12/T71-detect.phpt",
    "ext/soap/tests/soap12/T19-detect.phpt",
    "ext/soap/tests/soap12/T67-detect.phpt",
    "ext/soap/tests/soap12/T51-detect.phpt",
    "ext/soap/tests/soap12/T31-detect.phpt",
    "ext/soap/tests/soap12/T39-detect.phpt",
    "ext/soap/tests/soap12/T77_2-detect.phpt",
    "ext/soap/tests/soap12/T37-detect.phpt",
    "ext/soap/tests/soap12/T11-detect.phpt",
    "ext/soap/tests/soap12/T32-detect.phpt",
    "ext/soap/tests/soap12/T27-detect.phpt",
    "ext/soap/tests/soap12/T57-detect.phpt",
    "ext/soap/tests/soap12/T68-detect.phpt",
    "ext/soap/tests/soap12/T38_2-detect.phpt",
    "ext/soap/tests/soap12/T53-detect.phpt",
    "ext/soap/tests/soap12/T44-detect.phpt",
    "ext/soap/tests/soap12/T01-detect.phpt",
    "ext/soap/tests/soap12/T43-detect.phpt",
    "ext/soap/tests/typemap009-detect.phpt",
    "ext/soap/tests/typemap006-detect.phpt",
    "ext/soap/tests/typemap002-detect.phpt",
    "ext/soap/tests/typemap005-detect.phpt",
    "ext/soap/tests/server024-detect.phpt",
    "ext/soap/tests/server019-detect.phpt",
    "ext/soap/tests/typemap003-detect.phpt",
    "ext/soap/tests/server027-detect.phpt",
    "ext/soap/tests/schema/schema039-detect.phpt",
    "ext/soap/tests/schema/schema051-detect.phpt",
    "ext/soap/tests/schema/schema047-detect.phpt",
    "ext/soap/tests/schema/schema019-detect.phpt",
    "ext/soap/tests/schema/schema043-detect.phpt",
    "ext/soap/tests/schema/schema052-detect.phpt",
    "ext/soap/tests/schema/schema042-detect.phpt",
    "ext/soap/tests/schema/schema054-detect.phpt",
    "ext/soap/tests/schema/schema006-detect.phpt",
    "ext/soap/tests/schema/schema009-detect.phpt",
    "ext/soap/tests/schema/schema071-detect.phpt",
    "ext/soap/tests/schema/schema059-detect.phpt",
    "ext/soap/tests/schema/schema074-detect.phpt",
    "ext/soap/tests/schema/schema016-detect.phpt",
    "ext/soap/tests/schema/schema004-detect.phpt",
    "ext/soap/tests/schema/schema023-detect.phpt",
    "ext/soap/tests/schema/schema002-detect.phpt",
    "ext/soap/tests/schema/schema044-detect.phpt",
    "ext/soap/tests/schema/schema063-detect.phpt",
    "ext/soap/tests/schema/schema028-detect.phpt",
    "ext/soap/tests/schema/schema005-detect.phpt",
    "ext/soap/tests/schema/schema049-detect.phpt",
    "ext/soap/tests/schema/schema025-detect.phpt",
    "ext/soap/tests/schema/schema073-detect.phpt",
    "ext/soap/tests/schema/schema079-detect.phpt",
    "ext/soap/tests/schema/schema008-detect.phpt",
    "ext/soap/tests/schema/schema072-detect.phpt",
    "ext/soap/tests/schema/schema027-detect.phpt",
    "ext/soap/tests/schema/schema018-detect.phpt",
    "ext/soap/tests/schema/schema045-detect.phpt",
    "ext/soap/tests/schema/schema003-detect.phpt",
    "ext/soap/tests/schema/schema015-detect.phpt",
    "ext/soap/tests/schema/schema011-detect.phpt",
    "ext/soap/tests/schema/schema032-detect.phpt",
    "ext/soap/tests/schema/schema055-detect.phpt",
    "ext/soap/tests/schema/schema058-detect.phpt",
    "ext/soap/tests/schema/schema010-detect.phpt",
    "ext/soap/tests/schema/schema048-detect.phpt",
    "ext/soap/tests/schema/schema029-detect.phpt",
    "ext/soap/tests/schema/schema060-detect.phpt",
    "ext/soap/tests/schema/schema017-detect.phpt",
    "ext/soap/tests/schema/schema040-detect.phpt",
    "ext/soap/tests/schema/schema068-detect.phpt",
    "ext/soap/tests/schema/schema014-detect.phpt",
    "ext/soap/tests/schema/schema007-detect.phpt",
    "ext/soap/tests/schema/schema066-detect.phpt",
    "ext/soap/tests/schema/schema070-detect.phpt",
    "ext/soap/tests/schema/schema065-detect.phpt",
    "ext/soap/tests/schema/schema035-detect.phpt",
    "ext/soap/tests/schema/schema022-detect.phpt",
    "ext/soap/tests/schema/schema082-detect.phpt",
    "ext/soap/tests/schema/schema081-detect.phpt",
    "ext/soap/tests/schema/schema085-detect.phpt",
    "ext/soap/tests/schema/schema026-detect.phpt",
    "ext/soap/tests/schema/schema080-detect.phpt",
    "ext/soap/tests/schema/schema069-detect.phpt",
    "ext/soap/tests/schema/schema030-detect.phpt",
    "ext/soap/tests/schema/schema057-detect.phpt",
    "ext/soap/tests/schema/schema083-detect.phpt",
    "ext/soap/tests/schema/schema034-detect.phpt",
    "ext/soap/tests/schema/schema076-detect.phpt",
    "ext/soap/tests/schema/schema031-detect.phpt",
    "ext/soap/tests/schema/schema024-detect.phpt",
    "ext/soap/tests/schema/schema075-detect.phpt",
    "ext/soap/tests/schema/schema053-detect.phpt",
    "ext/soap/tests/schema/schema033-detect.phpt",
    "ext/soap/tests/schema/schema062-detect.phpt",
    "ext/soap/tests/schema/schema036-detect.phpt",
    "ext/soap/tests/schema/schema020-detect.phpt",
    "ext/soap/tests/schema/schema050-detect.phpt",
    "ext/soap/tests/schema/schema021-detect.phpt",
    "ext/soap/tests/schema/schema046-detect.phpt",
    "ext/soap/tests/schema/schema077-detect.phpt",
    "ext/soap/tests/schema/schema037-detect.phpt",
    "ext/soap/tests/schema/schema056-detect.phpt",
    "ext/soap/tests/schema/schema013-detect.phpt",
    "ext/soap/tests/schema/schema012-detect.phpt",
    "ext/soap/tests/schema/schema061-detect.phpt",
    "ext/soap/tests/schema/schema084-detect.phpt",
    "ext/soap/tests/schema/schema038-detect.phpt",
    "ext/soap/tests/schema/schema064-detect.phpt",
    "ext/soap/tests/schema/schema041-detect.phpt",
    "ext/soap/tests/schema/schema001-detect.phpt",
    "ext/soap/tests/schema/schema067-detect.phpt",
    "ext/soap/tests/schema/schema078-detect.phpt",
    "ext/soap/tests/server031-detect.phpt",
    "ext/soap/tests/server013-detect.phpt",
    "ext/soap/tests/server002-detect.phpt",
    "ext/soap/tests/server003-detect.phpt",
    "ext/soap/tests/classmap001-detect.phpt",
    "ext/soap/tests/server011-detect.phpt",
    "ext/shmop/tests/001-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation10-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation5-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation11-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation1-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation14-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation7-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation8-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation3-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation13-detect.phpt",
    "ext/tokenizer/tests/001-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation4-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation12-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation2-detect.phpt",
    "ext/tokenizer/tests/token_get_all_basic-detect.phpt",
    "ext/tokenizer/tests/token_get_all_error-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation6-detect.phpt",
    "ext/tokenizer/tests/003-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation9-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation17-detect.phpt",
    "ext/tokenizer/tests/bug54089-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation16-detect.phpt",
    "ext/tokenizer/tests/002-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation19-detect.phpt",
    "ext/tokenizer/tests/bug26463-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation15-detect.phpt",
    "ext/tokenizer/tests/token_get_all_variation18-detect.phpt",
    "ext/snmp/tests/snmp_set_enum_print-detect.phpt",
    "ext/snmp/tests/snmpset-nomib-detect.phpt",
    "ext/snmp/tests/snmpgetnext-detect.phpt",
    "ext/snmp/tests/snmp3-error-detect.phpt",
    "ext/snmp/tests/snmp3-detect.phpt",
    "ext/snmp/tests/bug60749-detect.phpt",
    "ext/snmp/tests/snmp2_getnext-detect.phpt",
    "ext/snmp/tests/snmp-object-error-detect.phpt",
    "ext/snmp/tests/snmp-object-setSecurity_error-detect.phpt",
    "ext/snmp/tests/snmp2_set-detect.phpt",
    "ext/snmp/tests/wrong_hostname-detect.phpt",
    "ext/snmp/tests/snmp2_real_walk-detect.phpt",
    "ext/snmp/tests/snmprealwalk-detect.phpt",
    "ext/snmp/tests/snmp-object-errno-errstr-detect.phpt",
    "ext/snmp/tests/generic_timeout_error-detect.phpt",
    "ext/snmp/tests/snmpwalk-detect.phpt",
    "ext/snmp/tests/snmp_get_quick_print-detect.phpt",
    "ext/snmp/tests/snmp2_walk-detect.phpt",
    "ext/snmp/tests/snmp2_get-detect.phpt",
    "ext/snmp/tests/snmp-object-detect.phpt",
    "ext/snmp/tests/snmpset-detect.phpt",
    "ext/snmp/tests/snmp_set_oid_output_format-detect.phpt",
    "ext/snmp/tests/snmp_get_valueretrieval-detect.phpt",
    "ext/snmp/tests/snmp-object-properties-detect.phpt",
    "ext/snmp/tests/snmpget-detect.phpt",
    "ext/snmp/tests/snmp_getvalue-detect.phpt",
    "ext/snmp/tests/snmp_read_mib-detect.phpt",
    "ext/snmp/tests/ipv6-detect.phpt",
    "ext/snmp/tests/snmp2_set-nomib-detect.phpt",
    "ext/xml/tests/xml_set_character_data_handler_variation1-detect.phpt",
    "ext/xml/tests/xml_parser_get_option_variation2-detect.phpt",
    "ext/xml/tests/xml_set_end_namespace_decl_handler_error-detect.phpt",
    "ext/xml/tests/bug26614_libxml-detect.phpt",
    "ext/xml/tests/xml_set_notation_decl_handler_variation1-detect.phpt",
    "ext/xml/tests/xml_set_object_variation2-detect.phpt",
    "ext/xml/tests/xml_parser_create_error-detect.phpt",
    "ext/xml/tests/bug27908-detect.phpt",
    "ext/xml/tests/xml_set_external_entity_ref_handler_error-detect.phpt",
    "ext/xml/tests/utf8_decode_error-detect.phpt",
    "ext/xml/tests/bug35447-detect.phpt",
    "ext/xml/tests/xml_get_error_code_error-detect.phpt",
    "ext/xml/tests/bug25666-detect.phpt",
    "ext/xml/tests/xml_set_start_namespace_decl_handler_error-detect.phpt",
    "ext/xml/tests/xml_set_element_handler_variation1-detect.phpt",
    "ext/xml/tests/xml011-detect.phpt",
    "ext/xml/tests/xml_parser_create_ns_variation1-detect.phpt",
    "ext/xml/tests/xml_get_current_column_number_error-detect.phpt",
    "ext/xml/tests/xml_set_start_namespace_decl_handler_variation1-detect.phpt",
    "ext/xml/tests/xml_set_character_data_handler_error-detect.phpt",
    "ext/xml/tests/xml003-detect.phpt",
    "ext/xml/tests/xml001-detect.phpt",
    "ext/xml/tests/xml_parser_create_variation1-detect.phpt",
    "ext/xml/tests/xml_set_unparsed_entity_decl_handler_variation1-detect.phpt",
    "ext/xml/tests/xml_parser_set_option_basic-detect.phpt",
    "ext/xml/tests/xml_set_default_handler_variation1-detect.phpt",
    "ext/xml/tests/xml_set_processing_instruction_handler_basic-detect.phpt",
    "ext/xml/tests/xml_get_current_byte_index_error-detect.phpt",
    "ext/xml/tests/xml_set_end_namespace_decl_handler_variation1-detect.phpt",
    "ext/xml/tests/xml_parser_free_error-detect.phpt",
    "ext/xml/tests/xml_parser_set_option_variation1-detect.phpt",
    "ext/xml/tests/bug26528-detect.phpt",
    "ext/xml/tests/xml_set_unparsed_entity_decl_handler_error-detect.phpt",
    "ext/xml/tests/bug32001b-detect.phpt",
    "ext/xml/tests/xml_parser_set_option_variation2-detect.phpt",
    "ext/xml/tests/xml_parse_into_struct_variation1-detect.phpt",
    "ext/xml/tests/xml_parser_get_option_error-detect.phpt",
    "ext/xml/tests/xml_parser_set_option_variation3-detect.phpt",
    "ext/xml/tests/xml002-detect.phpt",
    "ext/xml/tests/xml_get_error_code_variation1-detect.phpt",
    "ext/xml/tests/xml_parser_create_ns_error-detect.phpt",
    "ext/xml/tests/bug26614-detect.phpt",
    "ext/xml/tests/xml_get_current_column_number_variation1-detect.phpt",
    "ext/xml/tests/xml010-detect.phpt",
    "ext/xml/tests/bug43957-detect.phpt",
    "ext/xml/tests/xml_parse_into_struct_variation-detect.phpt",
    "ext/xml/tests/xml_set_processing_instruction_handler_variation1-detect.phpt",
    "ext/xml/tests/xml_closures_001-detect.phpt",
    "ext/xml/tests/xml_parse_into_struct_error-detect.phpt",
    "ext/xml/tests/xml007-detect.phpt",
    "ext/xml/tests/xml_set_start_namespace_decl_handler_basic-detect.phpt",
    "ext/xml/tests/xml_parser_set_option_error-detect.phpt",
    "ext/xml/tests/xml_error_string_variation1-detect.phpt",
    "ext/xml/tests/utf8_decode_variation1-detect.phpt",
    "ext/xml/tests/xml_parser_get_option_variation1-detect.phpt",
    "ext/xml/tests/xml006-detect.phpt",
    "ext/xml/tests/xml_parse_variation1-detect.phpt",
    "ext/xml/tests/utf8_encode_variation1-detect.phpt",
    "ext/xml/tests/xml_set_element_handler_error-detect.phpt",
    "ext/xml/tests/xml_set_notation_decl_handler_error-detect.phpt",
    "ext/xml/tests/utf8_encode_error-detect.phpt",
    "ext/xml/tests/bug49687-detect.phpt",
    "ext/xml/tests/xml009-detect.phpt",
    "ext/xml/tests/bug32001-detect.phpt",
    "ext/xml/tests/xml_set_object_variation1-detect.phpt",
    "ext/xml/tests/bug62328-detect.phpt",
    "ext/xml/tests/xml_parser_free_variation1-detect.phpt",
    "ext/xml/tests/xml_set_processing_instruction_handler_error-detect.phpt",
    "ext/xml/tests/xml_set_object_error-detect.phpt",
    "ext/xml/tests/xml_parse_error-detect.phpt",
    "ext/xml/tests/xml_set_external_entity_ref_handler_variation1-detect.phpt",
    "ext/xml/tests/bug46699-detect.phpt",
    "ext/xml/tests/xml_error_string_error-detect.phpt",
    "ext/xml/tests/xml_set_default_handler_error-detect.phpt",
    "ext/xml/tests/xml_set_notation_decl_handler_basic-detect.phpt",
    "ext/xml/tests/xml004-detect.phpt",
    "ext/xml/tests/xml_get_current_byte_index_variation1-detect.phpt",
    "ext/xml/tests/xml_get_current_line_number_error-detect.phpt",
    "ext/xml/tests/bug50576-detect.phpt",
    "ext/xml/tests/xml_get_current_line_number_variation1-detect.phpt",
    "ext/xml/tests/bug30266-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_nextrowset-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_phpinfo-detect.phpt",
    "ext/pdo_mysql/tests/bug_42499-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_fetch_serialize_simple-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql___construct_options_libmysql-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_fetch_non_select-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_pconnect-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_case-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_blobs-detect.phpt",
    "ext/pdo_mysql/tests/show_tables-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_types-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_connection_status-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_prepare_match_against-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_bindvalue-detect.phpt",
    "ext/pdo_mysql/tests/bug_41698-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_prepare_native_placeholder_everywhere-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_fetchobject-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_prepare_native-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_variable_columncount-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_fetch_table_names-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_exec_load_data-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_exec_select-detect.phpt",
    "ext/pdo_mysql/tests/bug_51670-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_class_constants-detect.phpt",
    "ext/pdo_mysql/tests/bug_61755-detect.phpt",
    "ext/pdo_mysql/tests/bug_41997-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_columncount-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_autocommit-detect.phpt",
    "ext/pdo_mysql/tests/bug_39858-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_bindparam-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_exec-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_prepare_native_named_placeholder-detect.phpt",
    "ext/pdo_mysql/tests/bug53782-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_prepare_load_data-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_errorinfo-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_prepare_native_dup_named_placeholder-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_driver_name-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_prepare_emulated_anonymous-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_begintransaction-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_statement_class-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_prefetch-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_prepare_native_column-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_errorcode-detect.phpt",
    "ext/pdo_mysql/tests/bug_33689-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_client_version-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_prepare_emulated-detect.phpt",
    "ext/pdo_mysql/tests/bug_39483-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_prepare_emulated_placeholder_everywhere-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql___construct_options-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_prepare_native_mixed_style-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql___construct_uri-detect.phpt",
    "ext/pdo_mysql/tests/bug_61411-detect.phpt",
    "ext/pdo_mysql/tests/common-detect.phpt",
    "ext/pdo_mysql/tests/bug_61207-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_bindparam_types-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_rowcount-detect.phpt",
    "ext/pdo_mysql/tests/bug_43371-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_subclass-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_prepare_native_clear_error-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_blobfromsteam-detect.phpt",
    "ext/pdo_mysql/tests/bug_37445-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_interface-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_errorcode-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_getcolumnmeta-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_connect_charset-detect.phpt",
    "ext/pdo_mysql/tests/pecl_bug_5802-detect.phpt",
    "ext/pdo_mysql/tests/bug_pecl_7976-detect.phpt",
    "ext/pdo_mysql/tests/bug_44454-detect.phpt",
    "ext/pdo_mysql/tests/pecl_bug_5780-detect.phpt",
    "ext/pdo_mysql/tests/bug54929-detect.phpt",
    "ext/pdo_mysql/tests/bug44327-detect.phpt",
    "ext/pdo_mysql/tests/bug41125-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_max_buffer_size-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_oracle_nulls-detect.phpt",
    "ext/pdo_mysql/tests/bug_44707-detect.phpt",
    "ext/pdo_mysql/tests/pecl_bug_5200-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_bit-detect.phpt",
    "ext/pdo_mysql/tests/bug53551-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_unbuffered_2050-detect.phpt",
    "ext/pdo_mysql/tests/bug_pecl_12925-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_bindcolumn-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_closecursor-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_commit-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_get_attribute-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql___construct_ini-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_rollback-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_server_version-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_init_command-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_fetch_serialize-detect.phpt",
    "ext/pdo_mysql/tests/last_insert_id-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_types_zerofill-detect.phpt",
    "ext/pdo_mysql/tests/bug_50323-detect.phpt",
    "ext/pdo_mysql/tests/bug_41125-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_closecursor_empty-detect.phpt",
    "ext/pdo_mysql/tests/bug46292-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_server_info-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_multiquery-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_stmt_errorinfo-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_fetch_both-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_exec_ddl-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_last_insert_id-detect.phpt",
    "ext/pdo_mysql/tests/bug_45120-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql_attr_errmode-detect.phpt",
    "ext/pdo_mysql/tests/pdo_mysql___construct-detect.phpt",
    "ext/interbase/tests/ibase_rollback_001-detect.phpt",
    "ext/interbase/tests/005-detect.phpt",
    "ext/interbase/tests/ibase_num_fields_001-detect.phpt",
    "ext/interbase/tests/ibase_free_query_001-detect.phpt",
    "ext/interbase/tests/bug45575-detect.phpt",
    "ext/interbase/tests/004-detect.phpt",
    "ext/interbase/tests/ibase_affected_rows_001-detect.phpt",
    "ext/interbase/tests/ibase_param_info_001-detect.phpt",
    "ext/interbase/tests/ibase_close_001-detect.phpt",
    "ext/interbase/tests/003-detect.phpt",
    "ext/interbase/tests/ibase_trans_001-detect.phpt",
    "ext/interbase/tests/ibase_num_params_001-detect.phpt",
    "ext/interbase/tests/006-detect.phpt",
    "ext/interbase/tests/007-detect.phpt",
    "ext/interbase/tests/002-detect.phpt",
    "ext/interbase/tests/008-detect.phpt",
    "ext/interbase/tests/ibase_errmsg_001-detect.phpt",
    "ext/interbase/tests/bug46247-detect.phpt",
    "ext/interbase/tests/bug45373-detect.phpt",
    "ext/interbase/tests/ibase_drop_db_001-detect.phpt",
    "ext/interbase/tests/bug46543-detect.phpt",
    "ext/interbase/tests/ibase_trans_002-detect.phpt",
    "ext/tidy/tests/027-detect.phpt",
    "ext/tidy/tests/033-detect.phpt",
    "ext/tidy/tests/010-detect.phpt",
    "ext/tidy/tests/005-detect.phpt",
    "ext/tidy/tests/014-detect.phpt",
    "ext/tidy/tests/029-detect.phpt",
    "ext/tidy/tests/011-detect.phpt",
    "ext/tidy/tests/tidy_error1-detect.phpt",
    "ext/tidy/tests/bug_50558-detect.phpt",
    "ext/tidy/tests/017-detect.phpt",
    "ext/tidy/tests/tidy_error-detect.phpt",
    "ext/tidy/tests/028-detect.phpt",
    "ext/tidy/tests/001-detect.phpt",
    "ext/tidy/tests/015-detect.phpt",
    "ext/tidy/tests/026-detect.phpt",
    "ext/tidy/tests/004-detect.phpt",
    "ext/tidy/tests/bug54682-detect.phpt",
    "ext/tidy/tests/003-detect.phpt",
    "ext/tidy/tests/020-detect.phpt",
    "ext/tidy/tests/022-detect.phpt",
    "ext/tidy/tests/006-detect.phpt",
    "ext/tidy/tests/031-detect.phpt",
    "ext/tidy/tests/023-detect.phpt",
    "ext/tidy/tests/007-detect.phpt",
    "ext/tidy/tests/002-detect.phpt",
    "ext/tidy/tests/008-detect.phpt",
    "ext/tidy/tests/018-detect.phpt",
    "ext/tidy/tests/034-detect.phpt",
    "ext/tidy/tests/035-detect.phpt",
    "ext/tidy/tests/013-detect.phpt",
    "ext/tidy/tests/016-detect.phpt",
    "ext/tidy/tests/025-detect.phpt",
    "ext/tidy/tests/012-detect.phpt",
    "ext/tidy/tests/021-detect.phpt",
    "ext/tidy/tests/030-detect.phpt",
    "ext/tidy/tests/032-detect.phpt",
    "ext/tidy/tests/019-detect.phpt",
    "ext/tidy/tests/024-detect.phpt",
    "ext/tidy/tests/009-detect.phpt",
    "ext/iconv/tests/iconv_strpos_variation2-detect.phpt",
    "ext/iconv/tests/iconv_strrpos-charset-length-cve-2007-4840-detect.phpt",
    "ext/iconv/tests/iconv_substr_basic-detect.phpt",
    "ext/iconv/tests/iconv_substr_error2-detect.phpt",
    "ext/iconv/tests/eucjp2sjis-detect.phpt",
    "ext/iconv/tests/iconv_mime_decode_headers_variation2-detect.phpt",
    "ext/iconv/tests/iconv_strpos-detect.phpt",
    "ext/iconv/tests/iconv_mime_decode_variation3-detect.phpt",
    "ext/iconv/tests/iconv_strrpos_error2-detect.phpt",
    "ext/iconv/tests/iconv_strlen_variation1-detect.phpt",
    "ext/iconv/tests/iconv_mime_decode_headers_variation3-detect.phpt",
    "ext/iconv/tests/iconv_strrpos_variation3-detect.phpt",
    "ext/iconv/tests/translit-failure-detect.phpt",
    "ext/iconv/tests/eucjp2utf8-detect.phpt",
    "ext/iconv/tests/iconv004-detect.phpt",
    "ext/iconv/tests/iconv_mime_decode_variation1-detect.phpt",
    "ext/iconv/tests/iconv_strpos_error2-detect.phpt",
    "ext/iconv/tests/ob_iconv_handler-charset-length-cve-2007-4840-detect.phpt",
    "ext/iconv/tests/iconv_set_encoding-charset-length-cve-2007-4840-detect.phpt",
    "ext/iconv/tests/iconv_strlen_basic-detect.phpt",
    "ext/iconv/tests/translit-utf8-detect.phpt",
    "ext/iconv/tests/bug48289-detect.phpt",
    "ext/iconv/tests/iconv_mime_decode_headers-detect.phpt",
    "ext/iconv/tests/bug37773-detect.phpt",
    "ext/iconv/tests/iconv_mime_decode-detect.phpt",
    "ext/iconv/tests/iconv_strpos_variation5-detect.phpt",
    "ext/iconv/tests/bug53304-detect.phpt",
    "ext/iconv/tests/iconv_mime_decode_headers-charset-length-cve-2007-4840-detect.phpt",
    "ext/iconv/tests/iconv_strlen_error2-detect.phpt",
    "ext/iconv/tests/iconv_strlen_error1-detect.phpt",
    "ext/iconv/tests/iconv_strpos_variation3_64bit-detect.phpt",
    "ext/iconv/tests/iconv_get_encoding_basic-detect.phpt",
    "ext/iconv/tests/iconv_strpos_basic-detect.phpt",
    "ext/iconv/tests/iconv003-detect.phpt",
    "ext/iconv/tests/iconv_strlen_variation2-detect.phpt",
    "ext/iconv/tests/ob_iconv_handler-detect.phpt",
    "ext/iconv/tests/iconv_strpos_error1-detect.phpt",
    "ext/iconv/tests/iconv_substr_error1-detect.phpt",
    "ext/iconv/tests/iconv_strlen-charset-length-cve-2007-4840-detect.phpt",
    "ext/iconv/tests/iconv_strrpos-detect.phpt",
    "ext/iconv/tests/iconv-charset-length-cve-2007-4840-detect.phpt",
    "ext/iconv/tests/iconv_basic-detect.phpt",
    "ext/iconv/tests/iconv001-detect.phpt",
    "ext/iconv/tests/iconv002-detect.phpt",
    "ext/iconv/tests/iconv_get_encoding_error-detect.phpt",
    "ext/iconv/tests/iconv_stream_filter_delimiter-detect.phpt",
    "ext/iconv/tests/iconv_mime_decode-charset-length-cve-2007-4840-detect.phpt",
    "ext/iconv/tests/iconv_strrpos_error1-detect.phpt",
    "ext/iconv/tests/iconv_mime_encode-detect.phpt",
    "ext/iconv/tests/iconv_strpos-charset-length-cve-2007-4840-detect.phpt",
    "ext/iconv/tests/bug52941-detect.phpt",
    "ext/iconv/tests/iconv_strpos_variation4-detect.phpt",
    "ext/iconv/tests/iconv_strpos_variation3-detect.phpt",
    "ext/iconv/tests/iconv_set_encoding_error-detect.phpt",
    "ext/iconv/tests/iconv_substr-detect.phpt",
    "ext/iconv/tests/bug52211-detect.phpt",
    "ext/iconv/tests/bug51250-detect.phpt",
    "ext/iconv/tests/bug37176-detect.phpt",
    "ext/iconv/tests/iconv_strrpos_variation2-detect.phpt",
    "ext/iconv/tests/iconv_strrpos_variation1-detect.phpt",
    "ext/iconv/tests/iconv_encoding_basic-detect.phpt",
    "ext/iconv/tests/iconv_mime_decode_headers_variation1-detect.phpt",
    "ext/iconv/tests/iconv_set_encoding_variation-detect.phpt",
    "ext/iconv/tests/iconv_strlen-detect.phpt",
    "ext/iconv/tests/iconv_strpos_variation1-detect.phpt",
    "ext/iconv/tests/iconv_strrpos_basic-detect.phpt",
    "ext/iconv/tests/iconv_substr-charset-length-cve-2007-4783-detect.phpt",
    "ext/iconv/tests/iconv_mime_decode_variation2-detect.phpt",
    "ext/iconv/tests/eucjp2iso2022jp-detect.phpt",
    "ext/iconv/tests/iconv_stream_filter-detect.phpt",
    "ext/zip/tests/zip_open-detect.phpt",
    "ext/zip/tests/zip_entry_open-detect.phpt",
    "ext/zip/tests/oo_getnameindex-detect.phpt",
    "ext/zip/tests/zip_entry_name-detect.phpt",
    "ext/zip/tests/oo_addfile-detect.phpt",
    "ext/zip/tests/zip_entry_compressedsize-detect.phpt",
    "ext/zip/tests/001-detect.phpt",
    "ext/zip/tests/bug38944-detect.phpt",
    "ext/zip/tests/oo_getstatusstring-detect.phpt",
    "ext/zip/tests/oo_open-detect.phpt",
    "ext/zip/tests/zip_entry_compressionmethod-detect.phpt",
    "ext/zip/tests/oo_delete-detect.phpt",
    "ext/zip/tests/zip_read-detect.phpt",
    "ext/zip/tests/bug53579-detect.phpt",
    "ext/zip/tests/zip_entry_filesize-detect.phpt",
    "ext/zip/tests/bug53885-detect.phpt",
    "ext/zip/tests/bug51353-detect.phpt",
    "ext/zip/tests/oo_rename-detect.phpt",
    "ext/zip/tests/bug53603-detect.phpt",
    "ext/zip/tests/zip_close-detect.phpt",
    "ext/zip/tests/zip_open_error-detect.phpt",
    "ext/zip/tests/bug38943-detect.phpt",
    "ext/zip/tests/oo_extract-detect.phpt",
    "ext/zip/tests/bug40228-detect.phpt",
    "ext/zip/tests/oo_properties-detect.phpt",
    "ext/zip/tests/pecl12414-detect.phpt",
    "ext/zip/tests/bug7658-detect.phpt",
    "ext/zip/tests/bug7214-detect.phpt",
    "ext/zip/tests/bug8009-detect.phpt",
    "ext/zip/tests/bug47667-detect.phpt",
    "ext/zip/tests/bug14962-detect.phpt",
    "ext/zip/tests/oo_setcomment-detect.phpt",
    "ext/zip/tests/oo_getcomment-detect.phpt",
    "ext/zip/tests/oo_namelocate-detect.phpt",
    "ext/zip/tests/bug49072-detect.phpt",
    "ext/zip/tests/stream_meta_data-detect.phpt",
    "ext/zip/tests/oo_ext_zip-detect.phpt",
    "ext/zip/tests/zip_entry_read-detect.phpt",
    "ext/zip/tests/bug8700-detect.phpt",
    "ext/zip/tests/oo_close-detect.phpt",
    "ext/zip/tests/bug53854-detect.phpt",
    "ext/zip/tests/oo_addemptydir-detect.phpt",
    "ext/zip/tests/bug11216-detect.phpt",
    "ext/zip/tests/oo_stream-detect.phpt",
    "ext/xmlwriter/tests/010-detect.phpt",
    "ext/xmlwriter/tests/005-detect.phpt",
    "ext/xmlwriter/tests/OO_004-detect.phpt",
    "ext/xmlwriter/tests/OO_007-detect.phpt",
    "ext/xmlwriter/tests/xmlwriter_write_dtd_basic_001-detect.phpt",
    "ext/xmlwriter/tests/OO_003-detect.phpt",
    "ext/xmlwriter/tests/001-detect.phpt",
    "ext/xmlwriter/tests/bug41326-detect.phpt",
    "ext/xmlwriter/tests/004-detect.phpt",
    "ext/xmlwriter/tests/OO_006-detect.phpt",
    "ext/xmlwriter/tests/OO_008-detect.phpt",
    "ext/xmlwriter/tests/bug41287-detect.phpt",
    "ext/xmlwriter/tests/003-detect.phpt",
    "ext/xmlwriter/tests/OO_002-detect.phpt",
    "ext/xmlwriter/tests/006-detect.phpt",
    "ext/xmlwriter/tests/bug48204-detect.phpt",
    "ext/xmlwriter/tests/xmlwriter_open_uri_error_003-detect.phpt",
    "ext/xmlwriter/tests/xmlwriter_set_indent_string_basic_001-detect.phpt",
    "ext/xmlwriter/tests/bug39504-detect.phpt",
    "ext/xmlwriter/tests/007-detect.phpt",
    "ext/xmlwriter/tests/OO_005-detect.phpt",
    "ext/xmlwriter/tests/xmlwriter_open_uri_error_005-detect.phpt",
    "ext/xmlwriter/tests/002-detect.phpt",
    "ext/xmlwriter/tests/008-detect.phpt",
    "ext/xmlwriter/tests/xmlwriter_write_dtd_error_001-detect.phpt",
    "ext/xmlwriter/tests/xmlwriter_open_uri_error_002-detect.phpt",
    "ext/xmlwriter/tests/xmlwriter_open_uri_error_004-detect.phpt",
    "ext/xmlwriter/tests/xmlwriter_open_uri_error_001-detect.phpt",
    "ext/xmlwriter/tests/OO_001-detect.phpt",
    "ext/xmlwriter/tests/xmlwriter_write_attribute_ns_basic_001-detect.phpt",
    "ext/xmlwriter/tests/xmlwriter_set_indent_string_error_001-detect.phpt",
    "ext/xmlwriter/tests/OO_009-detect.phpt",
    "ext/xmlwriter/tests/xmlwriter_write_attribute_ns_error_001-detect.phpt",
    "ext/xmlwriter/tests/009-detect.phpt",
    "ext/calendar/tests/jdtojulian-detect.phpt",
    "ext/calendar/tests/jdtofrench-detect.phpt",
    "ext/calendar/tests/juliantojd-detect.phpt",
    "ext/calendar/tests/jdtomonthname-detect.phpt",
    "ext/calendar/tests/jddayofweek-detect.phpt",
    "ext/calendar/tests/easter_days-detect.phpt",
    "ext/calendar/tests/gregoriantojd-detect.phpt",
    "ext/calendar/tests/frenchtojd-detect.phpt",
    "ext/calendar/tests/cal_info-detect.phpt",
    "ext/calendar/tests/bug55797_2-detect.phpt",
    "ext/calendar/tests/bug52744-detect.phpt",
    "ext/calendar/tests/easter_date-detect.phpt",
    "ext/calendar/tests/jdmonthname-detect.phpt",
    "ext/calendar/tests/bug53574_2-detect.phpt",
    "ext/calendar/tests/jewishtojd-detect.phpt",
    "ext/calendar/tests/bug53574_1-detect.phpt",
    "ext/calendar/tests/cal_from_jd-detect.phpt",
    "ext/calendar/tests/bug55797_1-detect.phpt",
    "ext/calendar/tests/jdtojewish-detect.phpt",
    "ext/calendar/tests/cal_to_jd-detect.phpt",
    "ext/calendar/tests/unixtojd-detect.phpt",
    "ext/calendar/tests/cal_days_in_month-detect.phpt",
    "ext/calendar/tests/jdtogregorian-detect.phpt",
    "ext/calendar/tests/jdtounix-detect.phpt",
    "tests/func/010-detect.phpt",
    "tests/func/001-detect.phpt",
    "tests/func/004-detect.phpt",
    "tests/func/003-detect.phpt",
    "tests/func/006-detect.phpt",
    "tests/func/007-detect.phpt",
    "tests/func/002-detect.phpt",
    "tests/func/008-detect.phpt",
    "tests/func/ini_alter-detect.phpt",
    "tests/func/009-detect.phpt",
    "tests/security/open_basedir_fileinode-detect.phpt",
    "tests/security/open_basedir_error_log-detect.phpt",
    "tests/security/open_basedir_dir-detect.phpt",
    "tests/security/open_basedir_filegroup-detect.phpt",
    "tests/security/open_basedir_tempnam-detect.phpt",
    "tests/security/open_basedir_is_file-detect.phpt",
    "tests/security/open_basedir_fileperms-detect.phpt",
    "tests/security/open_basedir_symlink-detect.phpt",
    "tests/security/open_basedir_fileowner-detect.phpt",
    "tests/security/open_basedir_unlink-detect.phpt",
    "tests/security/open_basedir_filectime-detect.phpt",
    "tests/security/open_basedir_lstat-detect.phpt",
    "tests/security/open_basedir_filemtime-detect.phpt",
    "tests/security/open_basedir_file_put_contents-detect.phpt",
    "tests/security/open_basedir_linkinfo-detect.phpt",
    "tests/security/bug53226-detect.phpt",
    "tests/security/open_basedir_filesize-detect.phpt",
    "tests/security/open_basedir_fopen-detect.phpt",
    "tests/security/open_basedir_file_get_contents-detect.phpt",
    "tests/security/open_basedir_touch-detect.phpt",
    "tests/security/open_basedir_stat-detect.phpt",
    "tests/security/open_basedir_parse_ini_file-detect.phpt",
    "tests/security/open_basedir_chmod-detect.phpt",
    "tests/security/open_basedir_file-detect.phpt",
    "tests/security/open_basedir_filetype-detect.phpt",
    "tests/security/open_basedir_copy_variation1-detect.phpt",
    "tests/security/open_basedir_is_executable-detect.phpt",
    "tests/security/open_basedir_chdir-detect.phpt",
    "tests/security/open_basedir_opendir-detect.phpt",
    "tests/security/open_basedir_is_readable-detect.phpt",
    "tests/security/open_basedir_disk_free_space-detect.phpt",
    "tests/security/open_basedir_rename-detect.phpt",
    "tests/security/open_basedir_mkdir-detect.phpt",
    "tests/security/open_basedir_rmdir-detect.phpt",
    "tests/security/open_basedir_001-detect.phpt",
    "tests/security/open_basedir_error_log_variation-detect.phpt",
    "tests/security/open_basedir_glob-detect.phpt",
    "tests/security/open_basedir_readlink-detect.phpt",
    "tests/security/open_basedir_fileatime-detect.phpt",
    "tests/security/open_basedir_file_exists-detect.phpt",
    "tests/security/open_basedir_glob_variation-detect.phpt",
    "tests/security/open_basedir_is_dir-detect.phpt",
    "tests/security/open_basedir_is_link-detect.phpt",
    "tests/security/open_basedir_scandir-detect.phpt",
    "tests/security/open_basedir_link-detect.phpt",
    "tests/security/open_basedir_is_writable-detect.phpt",
    "tests/security/open_basedir_realpath-detect.phpt",
    "tests/security/open_basedir_copy-detect.phpt",
    "tests/basic/027-detect.phpt",
    "tests/basic/010-detect.phpt",
    "tests/basic/bug51709_1-detect.phpt",
    "tests/basic/005-detect.phpt",
    "tests/basic/bug45986-detect.phpt",
    "tests/basic/014-detect.phpt",
    "tests/basic/029-detect.phpt",
    "tests/basic/php_real_logo_guid-detect.phpt",
    "tests/basic/bug61000-detect.phpt",
    "tests/basic/011-detect.phpt",
    "tests/basic/017-detect.phpt",
    "tests/basic/028-detect.phpt",
    "tests/basic/001-detect.phpt",
    "tests/basic/bug53180-detect.phpt",
    "tests/basic/015-detect.phpt",
    "tests/basic/026-detect.phpt",
    "tests/basic/004-detect.phpt",
    "tests/basic/rfc1867_anonymous_upload-detect.phpt",
    "tests/basic/rfc1867_garbled_mime_headers-detect.phpt",
    "tests/basic/rfc1867_file_upload_disabled-detect.phpt",
    "tests/basic/bug54514-detect.phpt",
    "tests/basic/php_egg_logo_guid-detect.phpt",
    "tests/basic/003-detect.phpt",
    "tests/basic/enable_post_data_reading_01-detect.phpt",
    "tests/basic/rfc1867_max_file_uploads_empty_files-detect.phpt",
    "tests/basic/rfc1867_array_upload-detect.phpt",
    "tests/basic/rfc1867_post_max_size-detect.phpt",
    "tests/basic/020-detect.phpt",
    "tests/basic/022-detect.phpt",
    "tests/basic/rfc1867_empty_upload-detect.phpt",
    "tests/basic/006-detect.phpt",
    "tests/basic/031-detect.phpt",
    "tests/basic/rfc1867_missing_boundary_2-detect.phpt",
    "tests/basic/023-detect.phpt",
    "tests/basic/rfc1867_missing_boundary-detect.phpt",
    "tests/basic/rfc1867_max_file_size-detect.phpt",
    "tests/basic/rfc1867_boundary_1-detect.phpt",
    "tests/basic/007-detect.phpt",
    "tests/basic/enable_post_data_reading_04-detect.phpt",
    "tests/basic/bug51709_2-detect.phpt",
    "tests/basic/rfc1867_invalid_boundary-detect.phpt",
    "tests/basic/002-detect.phpt",
    "tests/basic/008-detect.phpt",
    "tests/basic/018-detect.phpt",
    "tests/basic/bug29971-detect.phpt",
    "tests/basic/rfc1867_malicious_input-detect.phpt",
    "tests/basic/013-detect.phpt",
    "tests/basic/rfc1867_post_max_filesize-detect.phpt",
    "tests/basic/016-detect.phpt",
    "tests/basic/025-detect.phpt",
    "tests/basic/012-detect.phpt",
    "tests/basic/021-detect.phpt",
    "tests/basic/enable_post_data_reading_02-detect.phpt",
    "tests/basic/bug55500-detect.phpt",
    "tests/basic/rfc1867_max_file_uploads_empty_files_debug-detect.phpt",
    "tests/basic/php_logo_guid-detect.phpt",
    "tests/basic/030-detect.phpt",
    "tests/basic/req44164-detect.phpt",
    "tests/basic/032-detect.phpt",
    "tests/basic/rfc1867_boundary_2-detect.phpt",
    "tests/basic/019-detect.phpt",
    "tests/basic/zend_logo_guid-detect.phpt",
    "tests/basic/bug20539-detect.phpt",
    "tests/basic/024-detect.phpt",
    "tests/basic/009-detect.phpt",
    "tests/basic/enable_post_data_reading_03-detect.phpt",
    "tests/output/bug46897-detect.phpt",
    "tests/output/ob_008-detect.phpt",
    "tests/output/ob_get_length_error_001-detect.phpt",
    "tests/output/ob_end_flush_basic_001-detect.phpt",
    "tests/output/bug60322-detect.phpt",
    "tests/output/ob_implicit_flush_variation_001-detect.phpt",
    "tests/output/ob_012-detect.phpt",
    "tests/output/ob_013-detect.phpt",
    "tests/output/ob_007-detect.phpt",
    "tests/output/ob_009-detect.phpt",
    "tests/output/ob_get_contents_error_001-detect.phpt",
    "tests/output/bug60282-detect.phpt",
    "tests/output/ob_006-detect.phpt",
    "tests/output/ob_end_flush_error_001-detect.phpt",
    "tests/output/ob_017-detect.phpt",
    "tests/output/ob_011-detect.phpt",
    "tests/output/ob_start_callbacks-detect.phpt",
    "tests/output/flush_error_001-detect.phpt",
    "tests/output/ob_implicit_flush_error_001-detect.phpt",
    "tests/output/ob_end_clean_basic_001-detect.phpt",
    "tests/output/ob_start_basic_unerasable_003-detect.phpt",
    "tests/output/bug60768-detect.phpt",
    "tests/output/ob_get_level_basic_001-detect.phpt",
    "tests/output/bug60321-detect.phpt",
    "tests/output/ob_get_contents_basic_001-detect.phpt",
    "tests/output/ob_start_error_001-detect.phpt",
    "tests/output/ob_implicit_flush_basic_002-detect.phpt",
    "tests/output/ob_020-detect.phpt",
    "tests/output/ob_get_length_basic_001-detect.phpt",
    "tests/output/ob_start_error_002-detect.phpt",
    "tests/output/ob_clean_basic_001-detect.phpt",
    "tests/output/ob_start_basic_006-detect.phpt",
    "tests/output/ob_002-detect.phpt",
    "tests/output/ob_start_basic_001-detect.phpt",
    "tests/output/ob_flush_basic_001-detect.phpt",
    "tests/output/ob_clean_error_001-detect.phpt",
    "tests/output/ob_get_clean_error_001-detect.phpt",
    "tests/output/ob_start_basic_unerasable_002-detect.phpt",
    "tests/output/ob_start_basic_003-detect.phpt",
    "tests/output/flush_basic_001-detect.phpt",
    "tests/output/ob_start_basic_unerasable_001-detect.phpt",
    "tests/output/ob_end_clean_error_001-detect.phpt",
    "tests/output/ob_005-detect.phpt",
    "tests/output/ob_get_level_error_001-detect.phpt",
    "tests/output/ob_start_error_003-detect.phpt",
    "tests/output/ob_015-detect.phpt",
    "tests/output/ob_flush_error_001-detect.phpt",
    "tests/output/ob_get_clean_basic_001-detect.phpt",
    "tests/output/ob_start_basic_unerasable_005-detect.phpt",
    "tests/output/ob_implicit_flush_basic_001-detect.phpt",
    "tests/output/ob_start_basic_005-detect.phpt",
    "tests/output/ob_010-detect.phpt",
    "tests/output/ob_001-detect.phpt",
    "tests/output/ob_003-detect.phpt",
    "tests/output/ob_start_error_005-detect.phpt",
    "tests/output/ob_get_status-detect.phpt",
    "tests/output/ob_014-detect.phpt",
    "tests/output/ob_start_basic_002-detect.phpt",
    "tests/output/ob_start_basic_unerasable_004-detect.phpt",
    "tests/output/ob_get_clean_basic_002-detect.phpt",
    "tests/output/ob_004-detect.phpt",
    "tests/output/ob_start_error_004-detect.phpt",
    "tests/output/ob_018-detect.phpt",
    "tests/output/ob_start_basic_004-detect.phpt",
    "tests/strings/bug22592-detect.phpt",
    "tests/strings/offsets_chaining_3-detect.phpt",
    "tests/strings/001-detect.phpt",
    "tests/strings/004-detect.phpt",
    "tests/strings/bug26703-detect.phpt",
    "tests/strings/offsets_chaining_4-detect.phpt",
    "tests/strings/offsets_chaining_1-detect.phpt",
    "tests/strings/002-detect.phpt",
    "tests/strings/offsets_general-detect.phpt",
    "tests/strings/offsets_chaining_2-detect.phpt",
    "tests/strings/offsets_chaining_5-detect.phpt",
    "tests/classes/interfaces_001-detect.phpt",
    "tests/classes/inheritance_005-detect.phpt",
    "tests/classes/final-detect.phpt",
    "tests/classes/iterators_006-detect.phpt",
    "tests/classes/constants_scope_001-detect.phpt",
    "tests/classes/private_006b-detect.phpt",
    "tests/classes/autoload_018-detect.phpt",
    "tests/classes/property_recreate_private-detect.phpt",
    "tests/classes/destructor_and_exceptions-detect.phpt",
    "tests/classes/protected_002-detect.phpt",
    "tests/classes/constants_basic_003-detect.phpt",
    "tests/classes/type_hinting_005d-detect.phpt",
    "tests/classes/factory_and_singleton_004-detect.phpt",
    "tests/classes/final_abstract-detect.phpt",
    "tests/classes/factory_and_singleton_009-detect.phpt",
    "tests/classes/autoload_006-detect.phpt",
    "tests/classes/static_this-detect.phpt",
    "tests/classes/new_001-detect.phpt",
    "tests/classes/ctor_failure-detect.phpt",
    "tests/classes/property_override_protectedStatic_protected-detect.phpt",
    "tests/classes/final_redeclare-detect.phpt",
    "tests/classes/clone_003-detect.phpt",
    "tests/classes/tostring_004-detect.phpt",
    "tests/classes/iterators_003-detect.phpt",
    "tests/classes/interface_and_extends-detect.phpt",
    "tests/classes/visibility_002b-detect.phpt",
    "tests/classes/interface_method_private-detect.phpt",
    "tests/classes/property_override_protectedStatic_privateStatic-detect.phpt",
    "tests/classes/visibility_000a-detect.phpt",
    "tests/classes/property_override_private_privateStatic-detect.phpt",
    "tests/classes/interface_instantiate-detect.phpt",
    "tests/classes/inheritance-detect.phpt",
    "tests/classes/factory_and_singleton_005-detect.phpt",
    "tests/classes/factory_001-detect.phpt",
    "tests/classes/iterators_005-detect.phpt",
    "tests/classes/static_properties_undeclared_assign-detect.phpt",
    "tests/classes/class_stdclass-detect.phpt",
    "tests/classes/__set__get_001-detect.phpt",
    "tests/classes/abstract_derived-detect.phpt",
    "tests/classes/property_override_privateStatic_publicStatic-detect.phpt",
    "tests/classes/property_override_private_public-detect.phpt",
    "tests/classes/destructor_inheritance-detect.phpt",
    "tests/classes/abstract_not_declared-detect.phpt",
    "tests/classes/autoload_016-detect.phpt",
    "tests/classes/__set_data_corrupt-detect.phpt",
    "tests/classes/destructor_and_globals-detect.phpt",
    "tests/classes/visibility_001b-detect.phpt",
    "tests/classes/array_access_001-detect.phpt",
    "tests/classes/static_properties_003_error3-detect.phpt",
    "tests/classes/autoload_007-detect.phpt",
    "tests/classes/private_003-detect.phpt",
    "tests/classes/final_ctor2-detect.phpt",
    "tests/classes/property_override_protected_private-detect.phpt",
    "tests/classes/interface_must_be_implemented-detect.phpt",
    "tests/classes/static_properties_003_error2-detect.phpt",
    "tests/classes/private_004b-detect.phpt",
    "tests/classes/method_override_optional_arg_001-detect.phpt",
    "tests/classes/__call_003-detect.phpt",
    "tests/classes/constants_basic_004-detect.phpt",
    "tests/classes/factory_and_singleton_007-detect.phpt",
    "tests/classes/property_override_private_publicStatic-detect.phpt",
    "tests/classes/property_recreate_protected-detect.phpt",
    "tests/classes/iterators_007-detect.phpt",
    "tests/classes/bug29446-detect.phpt",
    "tests/classes/array_access_012-detect.phpt",
    "tests/classes/private_members-detect.phpt",
    "tests/classes/interface_method_final-detect.phpt",
    "tests/classes/autoload_004-detect.phpt",
    "tests/classes/autoload_014-detect.phpt",
    "tests/classes/private_006-detect.phpt",
    "tests/classes/type_hinting_002-detect.phpt",
    "tests/classes/interfaces_002-detect.phpt",
    "tests/classes/property_override_protected_privateStatic-detect.phpt",
    "tests/classes/private_003b-detect.phpt",
    "tests/classes/tostring_001-detect.phpt",
    "tests/classes/factory_and_singleton_003-detect.phpt",
    "tests/classes/ctor_dtor-detect.phpt",
    "tests/classes/interface_optional_arg-detect.phpt",
    "tests/classes/static_properties_undeclared_assignRef-detect.phpt",
    "tests/classes/clone_006-detect.phpt",
    "tests/classes/static_properties_003_error1-detect.phpt",
    "tests/classes/interface_optional_arg_002-detect.phpt",
    "tests/classes/interface_member-detect.phpt",
    "tests/classes/property_override_public_privateStatic-detect.phpt",
    "tests/classes/type_hinting_005c-detect.phpt",
    "tests/classes/visibility_004a-detect.phpt",
    "tests/classes/class_abstract-detect.phpt",
    "tests/classes/visibility_004b-detect.phpt",
    "tests/classes/type_hinting_005a-detect.phpt",
    "tests/classes/protected_001-detect.phpt",
    "tests/classes/private_005-detect.phpt",
    "tests/classes/static_properties_undeclared_inc-detect.phpt",
    "tests/classes/__call_006-detect.phpt",
    "tests/classes/iterators_004-detect.phpt",
    "tests/classes/visibility_000c-detect.phpt",
    "tests/classes/this-detect.phpt",
    "tests/classes/inheritance_004-detect.phpt",
    "tests/classes/incdec_property_003-detect.phpt",
    "tests/classes/interface_optional_arg_003-detect.phpt",
    "tests/classes/array_access_008-detect.phpt",
    "tests/classes/private_002-detect.phpt",
    "tests/classes/autoload_008-detect.phpt",
    "tests/classes/visibility_002a-detect.phpt",
    "tests/classes/autoload_002-detect.phpt",
    "tests/classes/interface_constant_inheritance_001-detect.phpt",
    "tests/classes/factory_and_singleton_010-detect.phpt",
    "tests/classes/visibility_003c-detect.phpt",
    "tests/classes/array_access_011-detect.phpt",
    "tests/classes/bug23951-detect.phpt",
    "tests/classes/interface_constant_inheritance_002-detect.phpt",
    "tests/classes/property_override_privateStatic_protected-detect.phpt",
    "tests/classes/type_hinting_001-detect.phpt",
    "tests/classes/destructor_and_echo-detect.phpt",
    "tests/classes/implicit_instantiation_001-detect.phpt",
    "tests/classes/private_004-detect.phpt",
    "tests/classes/clone_004-detect.phpt",
    "tests/classes/final_ctor1-detect.phpt",
    "tests/classes/__set__get_005-detect.phpt",
    "tests/classes/visibility_001a-detect.phpt",
    "tests/classes/property_override_privateStatic_privateStatic-detect.phpt",
    "tests/classes/bug27504-detect.phpt",
    "tests/classes/ctor_in_interface_01-detect.phpt",
    "tests/classes/clone_005-detect.phpt",
    "tests/classes/interface_doubled-detect.phpt",
    "tests/classes/bug24399-detect.phpt",
    "tests/classes/property_override_publicStatic_privateStatic-detect.phpt",
    "tests/classes/interface_constant_inheritance_003-detect.phpt",
    "tests/classes/autoload_013-detect.phpt",
    "tests/classes/property_override_public_public-detect.phpt",
    "tests/classes/type_hinting_003-detect.phpt",
    "tests/classes/abstract_final-detect.phpt",
    "tests/classes/ctor_name_clash-detect.phpt",
    "tests/classes/factory_and_singleton_002-detect.phpt",
    "tests/classes/interface_implemented-detect.phpt",
    "tests/classes/array_access_010-detect.phpt",
    "tests/classes/autoload_017-detect.phpt",
    "tests/classes/autoload_010-detect.phpt",
    "tests/classes/autoload_011-detect.phpt",
    "tests/classes/interfaces_003-detect.phpt",
    "tests/classes/inheritance_003-detect.phpt",
    "tests/classes/interface_constant_inheritance_004-detect.phpt",
    "tests/classes/__set__get_002-detect.phpt",
    "tests/classes/array_access_009-detect.phpt",
    "tests/classes/abstract_by_interface_002-detect.phpt",
    "tests/classes/class_example-detect.phpt",
    "tests/classes/bug27468-detect.phpt",
    "tests/classes/property_override_publicStatic_public-detect.phpt",
    "tests/classes/property_override_protectedStatic_private-detect.phpt",
    "tests/classes/inheritance_006-detect.phpt",
    "tests/classes/abstract_inherit-detect.phpt",
    "tests/classes/dereferencing_001-detect.phpt",
    "tests/classes/destructor_and_references-detect.phpt",
    "tests/classes/final_ctor3-detect.phpt",
    "tests/classes/autoload_001-detect.phpt",
    "tests/classes/autoload_003-detect.phpt",
    "tests/classes/tostring_002-detect.phpt",
    "tests/classes/arrayobject_001-detect.phpt",
    "tests/classes/destructor_visibility_001-detect.phpt",
    "tests/classes/visibility_003a-detect.phpt",
    "tests/classes/__call_002-detect.phpt",
    "tests/classes/property_override_public_publicStatic-detect.phpt",
    "tests/classes/property_override_protected_public-detect.phpt",
    "tests/classes/visibility_002c-detect.phpt",
    "tests/classes/abstract_class-detect.phpt",
    "tests/classes/ctor_visibility-detect.phpt",
    "tests/classes/array_access_013-detect.phpt",
    "tests/classes/array_access_002-detect.phpt",
    "tests/classes/property_override_publicStatic_publicStatic-detect.phpt",
    "tests/classes/incdec_property_004-detect.phpt",
    "tests/classes/array_access_005-detect.phpt",
    "tests/classes/__call_007-detect.phpt",
    "tests/classes/inheritance_007-detect.phpt",
    "tests/classes/constants_error_006-detect.phpt",
    "tests/classes/clone_002-detect.phpt",
    "tests/classes/autoload_015-detect.phpt",
    "tests/classes/ctor_in_interface_02-detect.phpt",
    "tests/classes/property_override_public_protected-detect.phpt",
    "tests/classes/interface_class-detect.phpt",
    "tests/classes/static_properties_undeclared_isset-detect.phpt",
    "tests/classes/method_override_optional_arg_002-detect.phpt",
    "tests/classes/factory_and_singleton_008-detect.phpt",
    "tests/classes/property_override_privateStatic_protectedStatic-detect.phpt",
    "tests/classes/abstract_redeclare-detect.phpt",
    "tests/classes/array_access_004-detect.phpt",
    "tests/classes/visibility_003b-detect.phpt",
    "tests/classes/factory_and_singleton_001-detect.phpt",
    "tests/classes/abstract_user_call-detect.phpt",
    "tests/classes/constants_error_004-detect.phpt",
    "tests/classes/array_access_006-detect.phpt",
    "tests/classes/inheritance_002-detect.phpt",
    "tests/classes/constants_error_005-detect.phpt",
    "tests/classes/property_override_protectedStatic_publicStatic-detect.phpt",
    "tests/classes/static_properties_004-detect.phpt",
    "tests/classes/__call_004-detect.phpt",
    "tests/classes/constants_basic_006-detect.phpt",
    "tests/classes/static_properties_003_error4-detect.phpt",
    "tests/classes/property_override_publicStatic_private-detect.phpt",
    "tests/classes/factory_and_singleton_006-detect.phpt",
    "tests/classes/property_override_publicStatic_protectedStatic-detect.phpt",
    "tests/classes/property_override_protected_protectedStatic-detect.phpt",
    "tests/classes/iterators_002-detect.phpt",
    "tests/classes/bug24445-detect.phpt",
    "tests/classes/class_final-detect.phpt",
    "tests/classes/iterators_001-detect.phpt",
    "tests/classes/static_mix_2-detect.phpt",
    "tests/classes/property_override_private_protected-detect.phpt",
    "tests/classes/autoload_009-detect.phpt",
    "tests/classes/private_007-detect.phpt",
    "tests/classes/object_reference_001-detect.phpt",
    "tests/classes/property_override_private_protectedStatic-detect.phpt",
    "tests/classes/autoload_005-detect.phpt",
    "tests/classes/property_override_protectedStatic_protectedStatic-detect.phpt",
    "tests/classes/__call_005-detect.phpt",
    "tests/classes/constants_error_007-detect.phpt",
    "tests/classes/property_override_public_private-detect.phpt",
    "tests/classes/constants_basic_005-detect.phpt",
    "tests/classes/__set__get_003-detect.phpt",
    "tests/classes/protected_001b-detect.phpt",
    "tests/classes/constants_error_003-detect.phpt",
    "tests/classes/autoload_019-detect.phpt",
    "tests/classes/static_mix_1-detect.phpt",
    "tests/classes/destructor_visibility_003-detect.phpt",
    "tests/classes/method_call_variation_001-detect.phpt",
    "tests/classes/static_properties_undeclared_read-detect.phpt",
    "tests/classes/static_properties_001-detect.phpt",
    "tests/classes/visibility_005-detect.phpt",
    "tests/classes/autoload_020-detect.phpt",
    "tests/classes/array_access_007-detect.phpt",
    "tests/classes/property_override_protected_protected-detect.phpt",
    "tests/classes/static_properties_undeclared_assignInc-detect.phpt",
    "tests/classes/destructor_visibility_002-detect.phpt",
    "tests/classes/ctor_in_interface_03-detect.phpt",
    "tests/classes/property_override_privateStatic_private-detect.phpt",
    "tests/classes/abstract_by_interface_001-detect.phpt",
    "tests/classes/ctor_in_interface_04-detect.phpt",
    "tests/classes/ctor_dtor_inheritance-detect.phpt",
    "tests/classes/serialize_001-detect.phpt",
    "tests/classes/tostring_003-detect.phpt",
    "tests/classes/private_007b-detect.phpt",
    "tests/classes/property_override_protectedStatic_public-detect.phpt",
    "tests/classes/__set__get_004-detect.phpt",
    "tests/classes/property_override_private_private-detect.phpt",
    "tests/classes/visibility_000b-detect.phpt",
    "tests/classes/incdec_property_001-detect.phpt",
    "tests/classes/abstract_static-detect.phpt",
    "tests/classes/property_override_privateStatic_public-detect.phpt",
    "tests/classes/private_005b-detect.phpt",
    "tests/classes/type_hinting_004-detect.phpt",
    "tests/classes/visibility_004c-detect.phpt",
    "tests/classes/abstract-detect.phpt",
    "tests/classes/singleton_001-detect.phpt",
    "tests/classes/constants_basic_001-detect.phpt",
    "tests/classes/bug26737-detect.phpt",
    "tests/classes/iterators_008-detect.phpt",
    "tests/classes/constants_basic_002-detect.phpt",
    "tests/classes/autoload_012-detect.phpt",
    "tests/classes/property_override_publicStatic_protected-detect.phpt",
    "tests/classes/property_override_protected_publicStatic-detect.phpt",
    "tests/classes/visibility_001c-detect.phpt",
    "tests/classes/assign_op_property_001-detect.phpt",
    "tests/classes/static_properties_003-detect.phpt",
    "tests/classes/type_hinting_005b-detect.phpt",
    "tests/classes/private_001-detect.phpt",
    "tests/classes/property_override_public_protectedStatic-detect.phpt",
    "tests/classes/__call_001-detect.phpt",
    "tests/classes/clone_001-detect.phpt",
    "tests/classes/constants_error_002-detect.phpt",
    "tests/classes/interface_method-detect.phpt",
    "tests/classes/constants_error_001-detect.phpt",
    "tests/classes/incdec_property_002-detect.phpt",
    "tests/classes/array_access_003-detect.phpt",
    "tests/classes/private_redeclare-detect.phpt",
    "tests/lang/027-detect.phpt",
    "tests/lang/passByReference_005-detect.phpt",
    "tests/lang/returnByReference007-detect.phpt",
    "tests/lang/foreachLoopObjects001-detect.phpt",
    "tests/lang/engine_assignExecutionOrder_007-detect.phpt",
    "tests/lang/bug35382-detect.phpt",
    "tests/lang/bug7515-detect.phpt",
    "tests/lang/bug30862-detect.phpt",
    "tests/lang/033-detect.phpt",
    "tests/lang/foreachLoop003-detect.phpt",
    "tests/lang/010-detect.phpt",
    "tests/lang/foreachLoop005-detect.phpt",
    "tests/lang/engine_assignExecutionOrder_008-detect.phpt",
    "tests/lang/engine_assignExecutionOrder_003-detect.phpt",
    "tests/lang/foreachLoop008-detect.phpt",
    "tests/lang/005-detect.phpt",
    "tests/lang/bug21669-detect.phpt",
    "tests/lang/returnByReference002-detect.phpt",
    "tests/lang/foreachLoop009-detect.phpt",
    "tests/lang/bug24396-detect.phpt",
    "tests/lang/014-detect.phpt",
    "tests/lang/044-detect.phpt",
    "tests/lang/bug22592-detect.phpt",
    "tests/lang/func_get_args002-detect.phpt",
    "tests/lang/func_get_arg002-detect.phpt",
    "tests/lang/bug17115-detect.phpt",
    "tests/lang/error_2_exception_001-detect.phpt",
    "tests/lang/func_get_args004-detect.phpt",
    "tests/lang/type_hints_002-detect.phpt",
    "tests/lang/bug28213-detect.phpt",
    "tests/lang/foreachLoopIteratorAggregate002-detect.phpt",
    "tests/lang/011-detect.phpt",
    "tests/lang/script_tag-detect.phpt",
    "tests/lang/bug30578-detect.phpt",
    "tests/lang/017-detect.phpt",
    "tests/lang/passByReference_007-detect.phpt",
    "tests/lang/foreachLoop011-detect.phpt",
    "tests/lang/array_shortcut_002-detect.phpt",
    "tests/lang/bug24403-detect.phpt",
    "tests/lang/036-detect.phpt",
    "tests/lang/func_num_args004-detect.phpt",
    "tests/lang/func_num_args001-detect.phpt",
    "tests/lang/bug24652-detect.phpt",
    "tests/lang/bug24054-detect.phpt",
    "tests/lang/foreachLoopIteratorAggregate003-detect.phpt",
    "tests/lang/func_get_arg003-detect.phpt",
    "tests/lang/bug28800-detect.phpt",
    "tests/lang/engine_assignExecutionOrder_006-detect.phpt",
    "tests/lang/returnByReference004-detect.phpt",
    "tests/lang/bug26696-detect.phpt",
    "tests/lang/043-detect.phpt",
    "tests/lang/bug25652-detect.phpt",
    "tests/lang/028-detect.phpt",
    "tests/lang/foreachLoopIterator001-detect.phpt",
    "tests/lang/001-detect.phpt",
    "tests/lang/041-detect.phpt",
    "tests/lang/015-detect.phpt",
    "tests/lang/bug22690-detect.phpt",
    "tests/lang/026-detect.phpt",
    "tests/lang/foreachLoopObjects005-detect.phpt",
    "tests/lang/004-detect.phpt",
    "tests/lang/returnByReference003-detect.phpt",
    "tests/lang/bug24499-detect.phpt",
    "tests/lang/passByReference_012-detect.phpt",
    "tests/lang/array_shortcut_003-detect.phpt",
    "tests/lang/static_variation_002-detect.phpt",
    "tests/lang/short_tags002-detect.phpt",
    "tests/lang/engine_assignExecutionOrder_002-detect.phpt",
    "tests/lang/bug26869-detect.phpt",
    "tests/lang/bug23922-detect.phpt",
    "tests/lang/bug25922-detect.phpt",
    "tests/lang/short_tags003-detect.phpt",
    "tests/lang/include_variation3-detect.phpt",
    "tests/lang/array_shortcut_001-detect.phpt",
    "tests/lang/bug26182-detect.phpt",
    "tests/lang/bug18872-detect.phpt",
    "tests/lang/short_tags004-detect.phpt",
    "tests/lang/bug26866-detect.phpt",
    "tests/lang/passByReference_002-detect.phpt",
    "tests/lang/string_decimals_001-detect.phpt",
    "tests/lang/operators/operator_equals_variation_64bit-detect.phpt",
    "tests/lang/operators/operator_lt_or_equal_variation-detect.phpt",
    "tests/lang/operators/subtract_basiclong_64bit-detect.phpt",
    "tests/lang/operators/bitwiseOr_variationStr-detect.phpt",
    "tests/lang/operators/bitwiseShiftLeft_variationStr-detect.phpt",
    "tests/lang/operators/bitwiseShiftRight_variationStr-detect.phpt",
    "tests/lang/operators/operator_gt_variation-detect.phpt",
    "tests/lang/operators/add_variationStr-detect.phpt",
    "tests/lang/operators/operator_equals_basic-detect.phpt",
    "tests/lang/operators/operator_lt_or_equal_variation_64bit-detect.phpt",
    "tests/lang/operators/operator_lt_variation-detect.phpt",
    "tests/lang/operators/predec_basiclong_64bit-detect.phpt",
    "tests/lang/operators/negate_basiclong_64bit-detect.phpt",
    "tests/lang/operators/bitwiseAnd_variationStr-detect.phpt",
    "tests/lang/operators/negate_variationStr-detect.phpt",
    "tests/lang/operators/operator_gt_variation_64bit-detect.phpt",
    "tests/lang/operators/modulus_variationStr-detect.phpt",
    "tests/lang/operators/predec_variationStr-detect.phpt",
    "tests/lang/operators/preinc_basiclong_64bit-detect.phpt",
    "tests/lang/operators/operator_lt_or_equal_basic-detect.phpt",
    "tests/lang/operators/operator_equals_variation-detect.phpt",
    "tests/lang/operators/bitwiseXor_variationStr-detect.phpt",
    "tests/lang/operators/operator_gt_or_equal_variation_64bit-detect.phpt",
    "tests/lang/operators/operator_notequals_variation_64bit-detect.phpt",
    "tests/lang/operators/operator_gt_or_equal_variation-detect.phpt",
    "tests/lang/operators/postinc_variationStr-detect.phpt",
    "tests/lang/operators/operator_identical_variation_64bit-detect.phpt",
    "tests/lang/operators/operator_gt_basic-detect.phpt",
    "tests/lang/operators/preinc_variationStr-detect.phpt",
    "tests/lang/operators/operator_notidentical_variation_64bit-detect.phpt",
    "tests/lang/operators/bitwiseNot_basiclong_64bit-detect.phpt",
    "tests/lang/operators/operator_notequals_variation-detect.phpt",
    "tests/lang/operators/multiply_variationStr-detect.phpt",
    "tests/lang/operators/operator_notidentical_basic-detect.phpt",
    "tests/lang/operators/add_basiclong_64bit-detect.phpt",
    "tests/lang/operators/bitwiseShiftLeft_basiclong_64bit-detect.phpt",
    "tests/lang/operators/bitwiseShiftLeft_variationStr_64bit-detect.phpt",
    "tests/lang/operators/bitwiseOr_basiclong_64bit-detect.phpt",
    "tests/lang/operators/bitwiseXor_basiclong_64bit-detect.phpt",
    "tests/lang/operators/bitwiseNot_variationStr-detect.phpt",
    "tests/lang/operators/operator_lt_variation_64bit-detect.phpt",
    "tests/lang/operators/operator_notidentical_variation-detect.phpt",
    "tests/lang/operators/operator_notequals_basic-detect.phpt",
    "tests/lang/operators/divide_basiclong_64bit-detect.phpt",
    "tests/lang/operators/postinc_basiclong_64bit-detect.phpt",
    "tests/lang/operators/divide_variationStr-detect.phpt",
    "tests/lang/operators/operator_lt_basic-detect.phpt",
    "tests/lang/operators/operator_gt_or_equal_basic-detect.phpt",
    "tests/lang/operators/postdec_basiclong_64bit-detect.phpt",
    "tests/lang/operators/multiply_basiclong_64bit-detect.phpt",
    "tests/lang/operators/operator_identical_basic-detect.phpt",
    "tests/lang/operators/bitwiseShiftRight_basiclong_64bit-detect.phpt",
    "tests/lang/operators/modulus_basiclong_64bit-detect.phpt",
    "tests/lang/operators/bitwiseAnd_basiclong_64bit-detect.phpt",
    "tests/lang/operators/postdec_variationStr-detect.phpt",
    "tests/lang/operators/operator_identical_variation-detect.phpt",
    "tests/lang/operators/subtract_variationStr-detect.phpt",
    "tests/lang/comments-detect.phpt",
    "tests/lang/compare_objects_basic2-detect.phpt",
    "tests/lang/engine_assignExecutionOrder_001-detect.phpt",
    "tests/lang/003-detect.phpt",
    "tests/lang/bug45392-detect.phpt",
    "tests/lang/engine_assignExecutionOrder_005-detect.phpt",
    "tests/lang/bug32828-detect.phpt",
    "tests/lang/foreachLoop014-detect.phpt",
    "tests/lang/passByReference_006-detect.phpt",
    "tests/lang/foreachLoopObjects002-detect.phpt",
    "tests/lang/func_get_args003-detect.phpt",
    "tests/lang/throw_variation_001-detect.phpt",
    "tests/lang/bug43958-detect.phpt",
    "tests/lang/bug24951-detect.phpt",
    "tests/lang/bug32924-detect.phpt",
    "tests/lang/compare_objects_basic1-detect.phpt",
    "tests/lang/returnByReference006-detect.phpt",
    "tests/lang/020-detect.phpt",
    "tests/lang/type_hints_001-detect.phpt",
    "tests/lang/022-detect.phpt",
    "tests/lang/006-detect.phpt",
    "tests/lang/031-detect.phpt",
    "tests/lang/bug23624-detect.phpt",
    "tests/lang/bug27535-detect.phpt",
    "tests/lang/bug30726-detect.phpt",
    "tests/lang/comments2-detect.phpt",
    "tests/lang/023-detect.phpt",
    "tests/lang/passByReference_004-detect.phpt",
    "tests/lang/foreachLoop006-detect.phpt",
    "tests/lang/this_assignment-detect.phpt",
    "tests/lang/static_basic_001-detect.phpt",
    "tests/lang/func_num_args002-detect.phpt",
    "tests/lang/returnByReference001-detect.phpt",
    "tests/lang/bug24573-detect.phpt",
    "tests/lang/catchable_error_002-detect.phpt",
    "tests/lang/foreachLoopObjects004-detect.phpt",
    "tests/lang/foreachLoop004-detect.phpt",
    "tests/lang/bug22231-detect.phpt",
    "tests/lang/each_binary_safety-detect.phpt",
    "tests/lang/bug24658-detect.phpt",
    "tests/lang/007-detect.phpt",
    "tests/lang/returnByReference008-detect.phpt",
    "tests/lang/bug19943-detect.phpt",
    "tests/lang/bug29944-detect.phpt",
    "tests/lang/bug21820-detect.phpt",
    "tests/lang/bug23584-detect.phpt",
    "tests/lang/bug29566-detect.phpt",
    "tests/lang/bug25547-detect.phpt",
    "tests/lang/type_hints_003-detect.phpt",
    "tests/lang/func_get_args001-detect.phpt",
    "tests/lang/002-detect.phpt",
    "tests/lang/empty_variation-detect.phpt",
    "tests/lang/bug23489-detect.phpt",
    "tests/lang/008-detect.phpt",
    "tests/lang/passByReference_008-detect.phpt",
    "tests/lang/018-detect.phpt",
    "tests/lang/040-detect.phpt",
    "tests/lang/passByReference_009-detect.phpt",
    "tests/lang/catchable_error_001-detect.phpt",
    "tests/lang/bug25145-detect.phpt",
    "tests/lang/returnByReference009-detect.phpt",
    "tests/lang/foreachLoop016-detect.phpt",
    "tests/lang/foreachLoop010-detect.phpt",
    "tests/lang/034-detect.phpt",
    "tests/lang/039-detect.phpt",
    "tests/lang/035-detect.phpt",
    "tests/lang/foreachLoopObjects006-detect.phpt",
    "tests/lang/bug20175-detect.phpt",
    "tests/lang/013-detect.phpt",
    "tests/lang/037-detect.phpt",
    "tests/lang/016-detect.phpt",
    "tests/lang/engine_assignExecutionOrder_004-detect.phpt",
    "tests/lang/static_basic_002-detect.phpt",
    "tests/lang/bug19566-detect.phpt",
    "tests/lang/bug21849-detect.phpt",
    "tests/lang/array_shortcut_005-detect.phpt",
    "tests/lang/bison1-detect.phpt",
    "tests/lang/func_get_arg004-detect.phpt",
    "tests/lang/025-detect.phpt",
    "tests/lang/func_get_arg_variation-detect.phpt",
    "tests/lang/bug44654-detect.phpt",
    "tests/lang/bug21961-detect.phpt",
    "tests/lang/foreachLoopIterator002-detect.phpt",
    "tests/lang/foreachLoopObjects003-detect.phpt",
    "tests/lang/foreachLoop013-detect.phpt",
    "tests/lang/012-detect.phpt",
    "tests/lang/038-detect.phpt",
    "tests/lang/bug23524-detect.phpt",
    "tests/lang/bug21600-detect.phpt",
    "tests/lang/bug35176-detect.phpt",
    "tests/lang/foreachLoopIteratorAggregate001-detect.phpt",
    "tests/lang/021-detect.phpt",
    "tests/lang/bug27443-detect.phpt",
    "tests/lang/returnByReference005-detect.phpt",
    "tests/lang/passByReference_001-detect.phpt",
    "tests/lang/include_variation2-detect.phpt",
    "tests/lang/foreachLoopIteratorAggregate004-detect.phpt",
    "tests/lang/bug24640-detect.phpt",
    "tests/lang/bug24783-detect.phpt",
    "tests/lang/foreach_with_object_001-detect.phpt",
    "tests/lang/passByReference_010-detect.phpt",
    "tests/lang/bug38579-detect.phpt",
    "tests/lang/foreach_with_references_001-detect.phpt",
    "tests/lang/func_get_arg005-detect.phpt",
    "tests/lang/bug24908-detect.phpt",
    "tests/lang/030-detect.phpt",
    "tests/lang/foreachLoop002-detect.phpt",
    "tests/lang/bug55754-detect.phpt",
    "tests/lang/foreachLoop007-detect.phpt",
    "tests/lang/bug24926-detect.phpt",
    "tests/lang/032-detect.phpt",
    "tests/lang/bug22510-detect.phpt",
    "tests/lang/passByReference_003-detect.phpt",
    "tests/lang/019-detect.phpt",
    "tests/lang/func_get_arg001-detect.phpt",
    "tests/lang/bug23384-detect.phpt",
    "tests/lang/include_variation1-detect.phpt",
    "tests/lang/bug44827-detect.phpt",
    "tests/lang/bug27354-detect.phpt",
    "tests/lang/foreachLoop012-detect.phpt",
    "tests/lang/024-detect.phpt",
    "tests/lang/bug23279-detect.phpt",
    "tests/lang/bug29893-detect.phpt",
    "tests/lang/foreachLoop001-detect.phpt",
    "tests/lang/009-detect.phpt",
    "tests/lang/042-detect.phpt",
    "tests/lang/engine_assignExecutionOrder_009-detect.phpt",
    "tests/lang/foreachLoop017-detect.phpt",
    "tests/lang/func_num_args003-detect.phpt",
    "tests/lang/short_tags001-detect.phpt",
    "tests/lang/foreachLoop015-detect.phpt",
    "tests/lang/static_variation_001-detect.phpt",
    "tests/lang/bug21094-detect.phpt",
    "tests/lang/bug30638-detect.phpt",
    "tests/lang/execution_order-detect.phpt",
    "tests/lang/bug24436-detect.phpt",
    "tests/lang/bug27439-detect.phpt",
    "tests/run-test/test010-detect.phpt",
    "tests/run-test/test009-detect.phpt",
    "tests/run-test/test004-detect.phpt",
    "tests/run-test/test007-detect.phpt",
    "tests/run-test/test008a-detect.phpt",
    "tests/run-test/test001-detect.phpt",
    "tests/run-test/test005-detect.phpt",
    "tests/run-test/test002-detect.phpt",
    "tests/run-test/test008-detect.phpt",
    "tests/run-test/test006-detect.phpt",
    "tests/run-test/test003-detect.phpt",
    "Zend/tests/027-detect.phpt",
    "Zend/tests/catch_004-detect.phpt",
    "Zend/tests/bug54547-detect.phpt",
    "Zend/tests/ns_064-detect.phpt",
    "Zend/tests/bug54265-detect.phpt",
    "Zend/tests/instanceof_001-detect.phpt",
    "Zend/tests/bug54804-detect.phpt",
    "Zend/tests/nowdoc_016-detect.phpt",
    "Zend/tests/gc_012-detect.phpt",
    "Zend/tests/bug31683-detect.phpt",
    "Zend/tests/bug43344_2-detect.phpt",
    "Zend/tests/ns_044-detect.phpt",
    "Zend/tests/dereference_009-detect.phpt",
    "Zend/tests/closure_025-detect.phpt",
    "Zend/tests/closure_015-detect.phpt",
    "Zend/tests/bug32290-detect.phpt",
    "Zend/tests/anonymous_func_001-detect.phpt",
    "Zend/tests/bug42817-detect.phpt",
    "Zend/tests/exception_handler_004-detect.phpt",
    "Zend/tests/gc_016-detect.phpt",
    "Zend/tests/bug53347-detect.phpt",
    "Zend/tests/objects_030-detect.phpt",
    "Zend/tests/bug36214-detect.phpt",
    "Zend/tests/bug39445-detect.phpt",
    "Zend/tests/bug26802-detect.phpt",
    "Zend/tests/bug48770_2-detect.phpt",
    "Zend/tests/isset_001-detect.phpt",
    "Zend/tests/objects_015-detect.phpt",
    "Zend/tests/033-detect.phpt",
    "Zend/tests/error_reporting08-detect.phpt",
    "Zend/tests/errmsg_032-detect.phpt",
    "Zend/tests/bug38234-detect.phpt",
    "Zend/tests/bug39602-detect.phpt",
    "Zend/tests/bug34199-detect.phpt",
    "Zend/tests/assign_to_var_002-detect.phpt",
    "Zend/tests/bug44184-detect.phpt",
    "Zend/tests/bug32252-detect.phpt",
    "Zend/tests/bug53748-detect.phpt",
    "Zend/tests/call_static-detect.phpt",
    "Zend/tests/bug62097-detect.phpt",
    "Zend/tests/bug33116-detect.phpt",
    "Zend/tests/heredoc_002-detect.phpt",
    "Zend/tests/gc_010-detect.phpt",
    "Zend/tests/ns_077_3-detect.phpt",
    "Zend/tests/closure_033-detect.phpt",
    "Zend/tests/unset_cv08-detect.phpt",
    "Zend/tests/ns_082-detect.phpt",
    "Zend/tests/010-detect.phpt",
    "Zend/tests/mul_001-detect.phpt",
    "Zend/tests/lsb_019-detect.phpt",
    "Zend/tests/objects_001-detect.phpt",
    "Zend/tests/bug39017-detect.phpt",
    "Zend/tests/add_003-detect.phpt",
    "Zend/tests/gc_011-detect.phpt",
    "Zend/tests/closure_004-detect.phpt",
    "Zend/tests/bug30889-detect.phpt",
    "Zend/tests/closure_011-detect.phpt",
    "Zend/tests/bug54358-detect.phpt",
    "Zend/tests/bug55007-detect.phpt",
    "Zend/tests/bug44141-detect.phpt",
    "Zend/tests/bug40236-detect.phpt",
    "Zend/tests/ns_011-detect.phpt",
    "Zend/tests/bug54624-detect.phpt",
    "Zend/tests/bug52237-detect.phpt",
    "Zend/tests/bug51822-detect.phpt",
    "Zend/tests/bug26229-detect.phpt",
    "Zend/tests/anonymous_func_002-detect.phpt",
    "Zend/tests/bug33710-detect.phpt",
    "Zend/tests/bug34358-detect.phpt",
    "Zend/tests/ns_039-detect.phpt",
    "Zend/tests/nowdoc_013-detect.phpt",
    "Zend/tests/ns_065-detect.phpt",
    "Zend/tests/bug55825-detect.phpt",
    "Zend/tests/access_modifiers_004-detect.phpt",
    "Zend/tests/foreach_undefined-detect.phpt",
    "Zend/tests/bug41026-detect.phpt",
    "Zend/tests/bug44899_2-detect.phpt",
    "Zend/tests/jump05-detect.phpt",
    "Zend/tests/errmsg_044-detect.phpt",
    "Zend/tests/gc_008-detect.phpt",
    "Zend/tests/errmsg_041-detect.phpt",
    "Zend/tests/005-detect.phpt",
    "Zend/tests/dynamic_call_004-detect.phpt",
    "Zend/tests/bug49893-detect.phpt",
    "Zend/tests/ns_026-detect.phpt",
    "Zend/tests/globals_001-detect.phpt",
    "Zend/tests/foreach-detect.phpt",
    "Zend/tests/trait_exists_003-detect.phpt",
    "Zend/tests/class_alias_006-detect.phpt",
    "Zend/tests/bug54367-detect.phpt",
    "Zend/tests/bug27669-detect.phpt",
    "Zend/tests/bug55509-detect.phpt",
    "Zend/tests/bug36037-detect.phpt",
    "Zend/tests/bug32833-detect.phpt",
    "Zend/tests/class_exists_001-detect.phpt",
    "Zend/tests/class_alias_010-detect.phpt",
    "Zend/tests/bug43344_12-detect.phpt",
    "Zend/tests/get_parent_class_001-detect.phpt",
    "Zend/tests/bug54013-detect.phpt",
    "Zend/tests/bug45805-detect.phpt",
    "Zend/tests/heredoc_011-detect.phpt",
    "Zend/tests/clone_003-detect.phpt",
    "Zend/tests/bug40757-detect.phpt",
    "Zend/tests/bug32993-detect.phpt",
    "Zend/tests/014-detect.phpt",
    "Zend/tests/bug35239-detect.phpt",
    "Zend/tests/list_006-detect.phpt",
    "Zend/tests/objects_026-detect.phpt",
    "Zend/tests/bug40899-detect.phpt",
    "Zend/tests/bug41209-detect.phpt",
    "Zend/tests/bug50394-detect.phpt",
    "Zend/tests/bug51176-detect.phpt",
    "Zend/tests/bug40833-detect.phpt",
    "Zend/tests/029-detect.phpt",
    "Zend/tests/bug62653-detect.phpt",
    "Zend/tests/empty_str_offset-detect.phpt",
    "Zend/tests/bug47699-detect.phpt",
    "Zend/tests/bug52484-detect.phpt",
    "Zend/tests/globals_003-detect.phpt",
    "Zend/tests/bug52160-detect.phpt",
    "Zend/tests/settype_string-detect.phpt",
    "Zend/tests/bug46246-detect.phpt",
    "Zend/tests/exception_006-detect.phpt",
    "Zend/tests/ns_047-detect.phpt",
    "Zend/tests/bug43344_7-detect.phpt",
    "Zend/tests/bug35437-detect.phpt",
    "Zend/tests/bug20242-detect.phpt",
    "Zend/tests/jump11-detect.phpt",
    "Zend/tests/isset_003_2_4-detect.phpt",
    "Zend/tests/closure_040-detect.phpt",
    "Zend/tests/bug38623-detect.phpt",
    "Zend/tests/errmsg_018-detect.phpt",
    "Zend/tests/bug30451-detect.phpt",
    "Zend/tests/bug41813-detect.phpt",
    "Zend/tests/unset_cv09-detect.phpt",
    "Zend/tests/objects_019-detect.phpt",
    "Zend/tests/heredoc_009-detect.phpt",
    "Zend/tests/bug41421-detect.phpt",
    "Zend/tests/exception_handler_006-detect.phpt",
    "Zend/tests/list_001-detect.phpt",
    "Zend/tests/bug44913-detect.phpt",
    "Zend/tests/indirect_method_call_003-detect.phpt",
    "Zend/tests/errmsg_023-detect.phpt",
    "Zend/tests/bug62680-detect.phpt",
    "Zend/tests/bug43323-detect.phpt",
    "Zend/tests/011-detect.phpt",
    "Zend/tests/call_user_func_002-detect.phpt",
    "Zend/tests/bug62005-detect.phpt",
    "Zend/tests/bug43343-detect.phpt",
    "Zend/tests/bug36303-detect.phpt",
    "Zend/tests/indirect_method_call_002-detect.phpt",
    "Zend/tests/bug31102-detect.phpt",
    "Zend/tests/heredoc_014-detect.phpt",
    "Zend/tests/bug53511-detect.phpt",
    "Zend/tests/heredoc_010-detect.phpt",
    "Zend/tests/selfParent_002-detect.phpt",
    "Zend/tests/bug29210-detect.phpt",
    "Zend/tests/declare_003-detect.phpt",
    "Zend/tests/bug61095-detect.phpt",
    "Zend/tests/jump06-detect.phpt",
    "Zend/tests/bug26698-detect.phpt",
    "Zend/tests/objects_021-detect.phpt",
    "Zend/tests/access_modifiers_001-detect.phpt",
    "Zend/tests/bug32660-detect.phpt",
    "Zend/tests/bug43344_13-detect.phpt",
    "Zend/tests/ns_084-detect.phpt",
    "Zend/tests/get_defined_vars-detect.phpt",
    "Zend/tests/bug38942-detect.phpt",
    "Zend/tests/bug49866-detect.phpt",
    "Zend/tests/property_exists-detect.phpt",
    "Zend/tests/get_class_vars_006-detect.phpt",
    "Zend/tests/list_003-detect.phpt",
    "Zend/tests/bug29883-detect.phpt",
    "Zend/tests/bug48004-detect.phpt",
    "Zend/tests/class_alias_012-detect.phpt",
    "Zend/tests/ns_045-detect.phpt",
    "Zend/tests/bug30346-detect.phpt",
    "Zend/tests/bug60611-detect.phpt",
    "Zend/tests/bug20240-detect.phpt",
    "Zend/tests/get_required_files-detect.phpt",
    "Zend/tests/indirect_call_array_003-detect.phpt",
    "Zend/tests/gc_003-detect.phpt",
    "Zend/tests/017-detect.phpt",
    "Zend/tests/heredoc_015-detect.phpt",
    "Zend/tests/settype_bool-detect.phpt",
    "Zend/tests/dereference_012-detect.phpt",
    "Zend/tests/bug39542-detect.phpt",
    "Zend/tests/bug43918-detect.phpt",
    "Zend/tests/halt01-detect.phpt",
    "Zend/tests/gc_032-detect.phpt",
    "Zend/tests/objects_007-detect.phpt",
    "Zend/tests/lsb_014-detect.phpt",
    "Zend/tests/closure_026-detect.phpt",
    "Zend/tests/bug30828-detect.phpt",
    "Zend/tests/bug43332_1-detect.phpt",
    "Zend/tests/offset_object-detect.phpt",
    "Zend/tests/bug24635-detect.phpt",
    "Zend/tests/decrement_001-detect.phpt",
    "Zend/tests/bug55578-detect.phpt",
    "Zend/tests/bug36568-detect.phpt",
    "Zend/tests/objects_008-detect.phpt",
    "Zend/tests/bug54039-detect.phpt",
    "Zend/tests/closure_043-detect.phpt",
    "Zend/tests/exception_001-detect.phpt",
    "Zend/tests/036-detect.phpt",
    "Zend/tests/each_001-detect.phpt",
    "Zend/tests/class_alias_007-detect.phpt",
    "Zend/tests/bug28072-detect.phpt",
    "Zend/tests/class_alias_017-detect.phpt",
    "Zend/tests/bug47880-detect.phpt",
    "Zend/tests/ns_067-detect.phpt",
    "Zend/tests/decrement_001_64bit-detect.phpt",
    "Zend/tests/closure_046-detect.phpt",
    "Zend/tests/closure_041-detect.phpt",
    "Zend/tests/bug33999-detect.phpt",
    "Zend/tests/error_reporting04-detect.phpt",
    "Zend/tests/offset_long-detect.phpt",
    "Zend/tests/class_exists_002-detect.phpt",
    "Zend/tests/heredoc_007-detect.phpt",
    "Zend/tests/function_exists_error-detect.phpt",
    "Zend/tests/bug48215_2-detect.phpt",
    "Zend/tests/ns_002-detect.phpt",
    "Zend/tests/bug46241-detect.phpt",
    "Zend/tests/bug41351-detect.phpt",
    "Zend/tests/increment_001-detect.phpt",
    "Zend/tests/objects_011-detect.phpt",
    "Zend/tests/objects_006-detect.phpt",
    "Zend/tests/cast_to_int-detect.phpt",
    "Zend/tests/heredoc_013-detect.phpt",
    "Zend/tests/offset_bool-detect.phpt",
    "Zend/tests/constants_003-detect.phpt",
    "Zend/tests/cast_to_string-detect.phpt",
    "Zend/tests/dereference_007-detect.phpt",
    "Zend/tests/call_static_007-detect.phpt",
    "Zend/tests/argument_restriction_003-detect.phpt",
    "Zend/tests/constants_007-detect.phpt",
    "Zend/tests/lsb_020-detect.phpt",
    "Zend/tests/lsb_001-detect.phpt",
    "Zend/tests/bug61165-detect.phpt",
    "Zend/tests/closure_007-detect.phpt",
    "Zend/tests/function_exists_basic-detect.phpt",
    "Zend/tests/objects_024-detect.phpt",
    "Zend/tests/dynamic_call_002-detect.phpt",
    "Zend/tests/ns_009-detect.phpt",
    "Zend/tests/ns_070-detect.phpt",
    "Zend/tests/div_001-detect.phpt",
    "Zend/tests/ns_081-detect.phpt",
    "Zend/tests/anonymous_func_003-detect.phpt",
    "Zend/tests/bug32429-detect.phpt",
    "Zend/tests/bug29104-detect.phpt",
    "Zend/tests/nowdoc_008-detect.phpt",
    "Zend/tests/class_alias_001-detect.phpt",
    "Zend/tests/ns_017-detect.phpt",
    "Zend/tests/bug52940-detect.phpt",
    "Zend/tests/ns_074-detect.phpt",
    "Zend/tests/gc_020-detect.phpt",
    "Zend/tests/class_constants_002-detect.phpt",
    "Zend/tests/ns_006-detect.phpt",
    "Zend/tests/bug26696-detect.phpt",
    "Zend/tests/bug37138-detect.phpt",
    "Zend/tests/bug31525-detect.phpt",
    "Zend/tests/get_class_vars_004-detect.phpt",
    "Zend/tests/bug43483-detect.phpt",
    "Zend/tests/ns_003-detect.phpt",
    "Zend/tests/bug47353-detect.phpt",
    "Zend/tests/bug43344_8-detect.phpt",
    "Zend/tests/bug53958-detect.phpt",
    "Zend/tests/bug52508-detect.phpt",
    "Zend/tests/error_reporting09-detect.phpt",
    "Zend/tests/bug33732-detect.phpt",
    "Zend/tests/bug34065-detect.phpt",
    "Zend/tests/028-detect.phpt",
    "Zend/tests/bug55137-detect.phpt",
    "Zend/tests/bug62358-detect.phpt",
    "Zend/tests/bug30162-detect.phpt",
    "Zend/tests/bug42143-detect.phpt",
    "Zend/tests/magic_by_ref_005-detect.phpt",
    "Zend/tests/class_exists_003-detect.phpt",
    "Zend/tests/bug45877-detect.phpt",
    "Zend/tests/globals_004-detect.phpt",
    "Zend/tests/001-detect.phpt",
    "Zend/tests/error_reporting10-detect.phpt",
    "Zend/tests/errmsg_040-detect.phpt",
    "Zend/tests/interface_exists_001-detect.phpt",
    "Zend/tests/gc_025-detect.phpt",
    "Zend/tests/bug60569-detect.phpt",
    "Zend/tests/exception_handler_005-detect.phpt",
    "Zend/tests/dereference_008-detect.phpt",
    "Zend/tests/class_alias_018-detect.phpt",
    "Zend/tests/errmsg_011-detect.phpt",
    "Zend/tests/bug52361-detect.phpt",
    "Zend/tests/gc_030-detect.phpt",
    "Zend/tests/mod_001-detect.phpt",
    "Zend/tests/bug54372-detect.phpt",
    "Zend/tests/lsb_009-detect.phpt",
    "Zend/tests/access_modifiers_005-detect.phpt",
    "Zend/tests/bug55305-detect.phpt",
    "Zend/tests/gc_006-detect.phpt",
    "Zend/tests/bug29890-detect.phpt",
    "Zend/tests/gc_031-detect.phpt",
    "Zend/tests/add_004-detect.phpt",
    "Zend/tests/ns_010-detect.phpt",
    "Zend/tests/exception_003-detect.phpt",
    "Zend/tests/bug27798-detect.phpt",
    "Zend/tests/bug27268-detect.phpt",
    "Zend/tests/bug42798-detect.phpt",
    "Zend/tests/015-detect.phpt",
    "Zend/tests/closure_019-detect.phpt",
    "Zend/tests/callable_type_hint_002-detect.phpt",
    "Zend/tests/bug26697-detect.phpt",
    "Zend/tests/bug60573-detect.phpt",
    "Zend/tests/dereference_003-detect.phpt",
    "Zend/tests/gc_007-detect.phpt",
    "Zend/tests/unset_cv05-detect.phpt",
    "Zend/tests/bug40509-detect.phpt",
    "Zend/tests/026-detect.phpt",
    "Zend/tests/jump01-detect.phpt",
    "Zend/tests/jump09-detect.phpt",
    "Zend/tests/oct_overflow_32bit-detect.phpt",
    "Zend/tests/dynamic_call_003-detect.phpt",
    "Zend/tests/bug35393-detect.phpt",
    "Zend/tests/bug26077-detect.phpt",
    "Zend/tests/access_modifiers_002-detect.phpt",
    "Zend/tests/closure_005-detect.phpt",
    "Zend/tests/bug39036-detect.phpt",
    "Zend/tests/constants_008-detect.phpt",
    "Zend/tests/004-detect.phpt",
    "Zend/tests/bug61225-detect.phpt",
    "Zend/tests/int_underflow_64bit-detect.phpt",
    "Zend/tests/call_static_006-detect.phpt",
    "Zend/tests/errmsg_007-detect.phpt",
    "Zend/tests/closure_039-detect.phpt",
    "Zend/tests/call_user_func_001-detect.phpt",
    "Zend/tests/bug43344_1-detect.phpt",
    "Zend/tests/bug39449-detect.phpt",
    "Zend/tests/compare_003_64bit-detect.phpt",
    "Zend/tests/xor_001-detect.phpt",
    "Zend/tests/compare_005_64bit-detect.phpt",
    "Zend/tests/strict_002-detect.phpt",
    "Zend/tests/bug29015-detect.phpt",
    "Zend/tests/bug46106-detect.phpt",
    "Zend/tests/indirect_property_access-detect.phpt",
    "Zend/tests/bug43053-detect.phpt",
    "Zend/tests/dereference_010-detect.phpt",
    "Zend/tests/ns_036-detect.phpt",
    "Zend/tests/bug48428-detect.phpt",
    "Zend/tests/add_005-detect.phpt",
    "Zend/tests/object_handlers-detect.phpt",
    "Zend/tests/bug39304-detect.phpt",
    "Zend/tests/settype_null-detect.phpt",
    "Zend/tests/closure_021-detect.phpt",
    "Zend/tests/bug36071-detect.phpt",
    "Zend/tests/bug38772-detect.phpt",
    "Zend/tests/bug43651-detect.phpt",
    "Zend/tests/bug43703-detect.phpt",
    "Zend/tests/closure_003-detect.phpt",
    "Zend/tests/ns_029-detect.phpt",
    "Zend/tests/bug24699-detect.phpt",
    "Zend/tests/offset_string-detect.phpt",
    "Zend/tests/ns_008-detect.phpt",
    "Zend/tests/gc_026-detect.phpt",
    "Zend/tests/ns_028-detect.phpt",
    "Zend/tests/bug41351_2-detect.phpt",
    "Zend/tests/compare_002-detect.phpt",
    "Zend/tests/ns_068-detect.phpt",
    "Zend/tests/ns_040-detect.phpt",
    "Zend/tests/bug30922-detect.phpt",
    "Zend/tests/bug38146-detect.phpt",
    "Zend/tests/bug54268-detect.phpt",
    "Zend/tests/callable_type_hint_003-detect.phpt",
    "Zend/tests/bug37811-detect.phpt",
    "Zend/tests/bug28444-detect.phpt",
    "Zend/tests/bug36513-detect.phpt",
    "Zend/tests/class_alias_011-detect.phpt",
    "Zend/tests/errmsg_043-detect.phpt",
    "Zend/tests/bug43344_6-detect.phpt",
    "Zend/tests/gc_029_zts-detect.phpt",
    "Zend/tests/lsb_010-detect.phpt",
    "Zend/tests/bug32596-detect.phpt",
    "Zend/tests/class_alias_014-detect.phpt",
    "Zend/tests/lsb_005-detect.phpt",
    "Zend/tests/isset_str_offset-detect.phpt",
    "Zend/tests/closure_024-detect.phpt",
    "Zend/tests/error_reporting06-detect.phpt",
    "Zend/tests/argument_restriction_005-detect.phpt",
    "Zend/tests/errmsg_028-detect.phpt",
    "Zend/tests/ns_077_7-detect.phpt",
    "Zend/tests/bug50383-detect.phpt",
    "Zend/tests/halt02-detect.phpt",
    "Zend/tests/get_class_vars_001-detect.phpt",
    "Zend/tests/bug21888-detect.phpt",
    "Zend/tests/object-null-detect.phpt",
    "Zend/tests/heredoc_018-detect.phpt",
    "Zend/tests/array_append_COW-detect.phpt",
    "Zend/tests/heredoc_001-detect.phpt",
    "Zend/tests/heredoc_017-detect.phpt",
    "Zend/tests/bug29689-detect.phpt",
    "Zend/tests/heredoc_006-detect.phpt",
    "Zend/tests/gc_014-detect.phpt",
    "Zend/tests/bug30407-detect.phpt",
    "Zend/tests/bug22836-detect.phpt",
    "Zend/tests/bug50146-detect.phpt",
    "Zend/tests/ns_023-detect.phpt",
    "Zend/tests/ns_043-detect.phpt",
    "Zend/tests/bug47516-detect.phpt",
    "Zend/tests/nowdoc_002-detect.phpt",
    "Zend/tests/exception_008-detect.phpt",
    "Zend/tests/declare_002-detect.phpt",
    "Zend/tests/ns_042-detect.phpt",
    "Zend/tests/errmsg_030-detect.phpt",
    "Zend/tests/inter_03-detect.phpt",
    "Zend/tests/isset_002-detect.phpt",
    "Zend/tests/bug41117_1-detect.phpt",
    "Zend/tests/bug41929-detect.phpt",
    "Zend/tests/lsb_012-detect.phpt",
    "Zend/tests/inter_06-detect.phpt",
    "Zend/tests/errmsg_016-detect.phpt",
    "Zend/tests/closure_032-detect.phpt",
    "Zend/tests/bug40809-detect.phpt",
    "Zend/tests/bug52939-detect.phpt",
    "Zend/tests/exception_handler_002-detect.phpt",
    "Zend/tests/bug46238-detect.phpt",
    "Zend/tests/bug32296-detect.phpt",
    "Zend/tests/bug44653-detect.phpt",
    "Zend/tests/bug47836-detect.phpt",
    "Zend/tests/bug62892-detect.phpt",
    "Zend/tests/ns_086-detect.phpt",
    "Zend/tests/get_called_class_001-detect.phpt",
    "Zend/tests/bug38047-detect.phpt",
    "Zend/tests/unset_cv01-detect.phpt",
    "Zend/tests/bug48215-detect.phpt",
    "Zend/tests/bug33802-detect.phpt",
    "Zend/tests/ns_035-detect.phpt",
    "Zend/tests/bug30820-detect.phpt",
    "Zend/tests/compare_003-detect.phpt",
    "Zend/tests/nowdoc_010-detect.phpt",
    "Zend/tests/bug43344_9-detect.phpt",
    "Zend/tests/objects_029-detect.phpt",
    "Zend/tests/function_arguments_001-detect.phpt",
    "Zend/tests/bug33558-detect.phpt",
    "Zend/tests/debug_backtrace_options-detect.phpt",
    "Zend/tests/class_alias_019-detect.phpt",
    "Zend/tests/bug30702-detect.phpt",
    "Zend/tests/bug35411-detect.phpt",
    "Zend/tests/ns_030-detect.phpt",
    "Zend/tests/access_modifiers_003-detect.phpt",
    "Zend/tests/bug55086-detect.phpt",
    "Zend/tests/closure_012-detect.phpt",
    "Zend/tests/bug34062-detect.phpt",
    "Zend/tests/magic_methods_006-detect.phpt",
    "Zend/tests/compare_001-detect.phpt",
    "Zend/tests/objects_013-detect.phpt",
    "Zend/tests/bug45744-detect.phpt",
    "Zend/tests/bug43175-detect.phpt",
    "Zend/tests/instanceof_002-detect.phpt",
    "Zend/tests/argument_restriction_006-detect.phpt",
    "Zend/tests/nowdoc_005-detect.phpt",
    "Zend/tests/bug38779_1-detect.phpt",
    "Zend/tests/bug35509-detect.phpt",
    "Zend/tests/bug52879-detect.phpt",
    "Zend/tests/ns_079-detect.phpt",
    "Zend/tests/003-detect.phpt",
    "Zend/tests/bug46381-detect.phpt",
    "Zend/tests/bug43200-detect.phpt",
    "Zend/tests/bug38220-detect.phpt",
    "Zend/tests/lsb_007-detect.phpt",
    "Zend/tests/ns_012-detect.phpt",
    "Zend/tests/bug45180-detect.phpt",
    "Zend/tests/compare_004-detect.phpt",
    "Zend/tests/exception_005-detect.phpt",
    "Zend/tests/bug62907-detect.phpt",
    "Zend/tests/closure_029-detect.phpt",
    "Zend/tests/unset_cv12-detect.phpt",
    "Zend/tests/ns_037-detect.phpt",
    "Zend/tests/bug39003-detect.phpt",
    "Zend/tests/list_007-detect.phpt",
    "Zend/tests/bug45910-detect.phpt",
    "Zend/tests/objects_014-detect.phpt",
    "Zend/tests/bug41919-detect.phpt",
    "Zend/tests/ns_053-detect.phpt",
    "Zend/tests/indirect_method_call_005-detect.phpt",
    "Zend/tests/jump08-detect.phpt",
    "Zend/tests/cast_to_bool-detect.phpt",
    "Zend/tests/bug40770-detect.phpt",
    "Zend/tests/bug35017-detect.phpt",
    "Zend/tests/class_alias_005-detect.phpt",
    "Zend/tests/bug38469-detect.phpt",
    "Zend/tests/indirect_call_array_001-detect.phpt",
    "Zend/tests/bug43201-detect.phpt",
    "Zend/tests/ns_062-detect.phpt",
    "Zend/tests/nowdoc_015-detect.phpt",
    "Zend/tests/bug51394-detect.phpt",
    "Zend/tests/bug38287-detect.phpt",
    "Zend/tests/call_static_004-detect.phpt",
    "Zend/tests/bug41075-detect.phpt",
    "Zend/tests/bug41633_4-detect.phpt",
    "Zend/tests/bug43426-detect.phpt",
    "Zend/tests/ns_061-detect.phpt",
    "Zend/tests/errmsg_034-detect.phpt",
    "Zend/tests/bug31177-2-detect.phpt",
    "Zend/tests/int_overflow_32bit-detect.phpt",
    "Zend/tests/bug28442-detect.phpt",
    "Zend/tests/closure_037-detect.phpt",
    "Zend/tests/errmsg_013-detect.phpt",
    "Zend/tests/objects_009-detect.phpt",
    "Zend/tests/halt_compiler4-detect.phpt",
    "Zend/tests/bug48930-detect.phpt",
    "Zend/tests/errmsg_009-detect.phpt",
    "Zend/tests/bug43027-detect.phpt",
    "Zend/tests/ns_077_5-detect.phpt",
    "Zend/tests/heredoc_004-detect.phpt",
    "Zend/tests/ns_022-detect.phpt",
    "Zend/tests/020-detect.phpt",
    "Zend/tests/ns_076-detect.phpt",
    "Zend/tests/bug53727-detect.phpt",
    "Zend/tests/022-detect.phpt",
    "Zend/tests/clone_004-detect.phpt",
    "Zend/tests/closure_045-detect.phpt",
    "Zend/tests/bug42818-detect.phpt",
    "Zend/tests/bug34467-detect.phpt",
    "Zend/tests/bug42211-detect.phpt",
    "Zend/tests/errmsg_033-detect.phpt",
    "Zend/tests/006-detect.phpt",
    "Zend/tests/031-detect.phpt",
    "Zend/tests/magic_methods_005-detect.phpt",
    "Zend/tests/exception_handler_003-detect.phpt",
    "Zend/tests/bug34064-detect.phpt",
    "Zend/tests/objects_010-detect.phpt",
    "Zend/tests/bug31720-detect.phpt",
    "Zend/tests/indexing_001-detect.phpt",
    "Zend/tests/objects_032-detect.phpt",
    "Zend/tests/constants_004-detect.phpt",
    "Zend/tests/traits/bug61052-detect.phpt",
    "Zend/tests/traits/bug55554c-detect.phpt",
    "Zend/tests/traits/error_013-detect.phpt",
    "Zend/tests/traits/static_get_called_class-detect.phpt",
    "Zend/tests/traits/bug60173-detect.phpt",
    "Zend/tests/traits/methods_002-detect.phpt",
    "Zend/tests/traits/property002-detect.phpt",
    "Zend/tests/traits/get_declared_traits_001-detect.phpt",
    "Zend/tests/traits/conflict003-detect.phpt",
    "Zend/tests/traits/bug55524-detect.phpt",
    "Zend/tests/traits/bug60165c-detect.phpt",
    "Zend/tests/traits/static_002-detect.phpt",
    "Zend/tests/traits/property009-detect.phpt",
    "Zend/tests/traits/error_001-detect.phpt",
    "Zend/tests/traits/methods_003-detect.phpt",
    "Zend/tests/traits/bug55355-detect.phpt",
    "Zend/tests/traits/language008b-detect.phpt",
    "Zend/tests/traits/property008-detect.phpt",
    "Zend/tests/traits/methods_001-detect.phpt",
    "Zend/tests/traits/interface_002-detect.phpt",
    "Zend/tests/traits/conflict002-detect.phpt",
    "Zend/tests/traits/bugs/abstract-methods06-detect.phpt",
    "Zend/tests/traits/bugs/abstract-methods05-detect.phpt",
    "Zend/tests/traits/bugs/case-sensitive-detect.phpt",
    "Zend/tests/traits/bugs/abstract-methods01-detect.phpt",
    "Zend/tests/traits/bugs/alias-semantics-detect.phpt",
    "Zend/tests/traits/bugs/alias01-detect.phpt",
    "Zend/tests/traits/bugs/missing-trait-detect.phpt",
    "Zend/tests/traits/bugs/abstract-methods04-detect.phpt",
    "Zend/tests/traits/bugs/overridding-conflicting-property-initializer-detect.phpt",
    "Zend/tests/traits/bugs/interfaces-detect.phpt",
    "Zend/tests/traits/bugs/alias-semantics02-detect.phpt",
    "Zend/tests/traits/bugs/abstract-methods02-detect.phpt",
    "Zend/tests/traits/bugs/overridding-conflicting-methods-detect.phpt",
    "Zend/tests/traits/bugs/abstract-methods03-detect.phpt",
    "Zend/tests/traits/error_016-detect.phpt",
    "Zend/tests/traits/bug55137-detect.phpt",
    "Zend/tests/traits/language001-detect.phpt",
    "Zend/tests/traits/bug55554b-detect.phpt",
    "Zend/tests/traits/static_forward_static_call-detect.phpt",
    "Zend/tests/traits/interface_003-detect.phpt",
    "Zend/tests/traits/error_012-detect.phpt",
    "Zend/tests/traits/inheritance003-detect.phpt",
    "Zend/tests/traits/bug55554a-detect.phpt",
    "Zend/tests/traits/static_001-detect.phpt",
    "Zend/tests/traits/bug55554e-detect.phpt",
    "Zend/tests/traits/bug55372-detect.phpt",
    "Zend/tests/traits/error_004-detect.phpt",
    "Zend/tests/traits/property007-detect.phpt",
    "Zend/tests/traits/static_003-detect.phpt",
    "Zend/tests/traits/property004-detect.phpt",
    "Zend/tests/traits/language007-detect.phpt",
    "Zend/tests/traits/error_002-detect.phpt",
    "Zend/tests/traits/flattening002-detect.phpt",
    "Zend/tests/traits/bug60165d-detect.phpt",
    "Zend/tests/traits/language010-detect.phpt",
    "Zend/tests/traits/bug55554g-detect.phpt",
    "Zend/tests/traits/bug55554d-detect.phpt",
    "Zend/tests/traits/error_015-detect.phpt",
    "Zend/tests/traits/property006-detect.phpt",
    "Zend/tests/traits/get_declared_traits_003-detect.phpt",
    "Zend/tests/traits/language002-detect.phpt",
    "Zend/tests/traits/error_005-detect.phpt",
    "Zend/tests/traits/bug60217c-detect.phpt",
    "Zend/tests/traits/error_009-detect.phpt",
    "Zend/tests/traits/bug55424-detect.phpt",
    "Zend/tests/traits/language003-detect.phpt",
    "Zend/tests/traits/error_008-detect.phpt",
    "Zend/tests/traits/bug61998-detect.phpt",
    "Zend/tests/traits/bug60165b-detect.phpt",
    "Zend/tests/traits/bug55554f-detect.phpt",
    "Zend/tests/traits/language009-detect.phpt",
    "Zend/tests/traits/property001-detect.phpt",
    "Zend/tests/traits/bug55214-detect.phpt",
    "Zend/tests/traits/language005-detect.phpt",
    "Zend/tests/traits/error_014-detect.phpt",
    "Zend/tests/traits/language004-detect.phpt",
    "Zend/tests/traits/trait_constant_001-detect.phpt",
    "Zend/tests/traits/bug60217b-detect.phpt",
    "Zend/tests/traits/language011-detect.phpt",
    "Zend/tests/traits/language013-detect.phpt",
    "Zend/tests/traits/error_003-detect.phpt",
    "Zend/tests/traits/error_010-detect.phpt",
    "Zend/tests/traits/conflict001-detect.phpt",
    "Zend/tests/traits/bug60809-detect.phpt",
    "Zend/tests/traits/noctor001-detect.phpt",
    "Zend/tests/traits/language008a-detect.phpt",
    "Zend/tests/traits/error_011-detect.phpt",
    "Zend/tests/traits/inheritance001-detect.phpt",
    "Zend/tests/traits/bug60369-detect.phpt",
    "Zend/tests/traits/static_004-detect.phpt",
    "Zend/tests/traits/error_006-detect.phpt",
    "Zend/tests/traits/flattening003-detect.phpt",
    "Zend/tests/traits/interface_001-detect.phpt",
    "Zend/tests/traits/bug60165a-detect.phpt",
    "Zend/tests/traits/bug60145-detect.phpt",
    "Zend/tests/traits/bug60217a-detect.phpt",
    "Zend/tests/traits/get_declared_traits_002-detect.phpt",
    "Zend/tests/traits/inheritance002-detect.phpt",
    "Zend/tests/traits/error_007-detect.phpt",
    "Zend/tests/traits/property005-detect.phpt",
    "Zend/tests/traits/bug54441-detect.phpt",
    "Zend/tests/traits/property003-detect.phpt",
    "Zend/tests/traits/trait_constant_002-detect.phpt",
    "Zend/tests/traits/bug60153-detect.phpt",
    "Zend/tests/traits/bug60717-detect.phpt",
    "Zend/tests/traits/flattening001-detect.phpt",
    "Zend/tests/traits/language012-detect.phpt",
    "Zend/tests/traits/language006-detect.phpt",
    "Zend/tests/unset_cv10-detect.phpt",
    "Zend/tests/ns_048-detect.phpt",
    "Zend/tests/bug40621-detect.phpt",
    "Zend/tests/023-detect.phpt",
    "Zend/tests/heredoc_008-detect.phpt",
    "Zend/tests/exception_007-detect.phpt",
    "Zend/tests/halt_compiler3-detect.phpt",
    "Zend/tests/ns_077_1-detect.phpt",
    "Zend/tests/bug37715-detect.phpt",
    "Zend/tests/ns_031-detect.phpt",
    "Zend/tests/gc_019-detect.phpt",
    "Zend/tests/dereference_006-detect.phpt",
    "Zend/tests/heredoc_005-detect.phpt",
    "Zend/tests/bug47343-detect.phpt",
    "Zend/tests/catch-detect.phpt",
    "Zend/tests/bug60362-detect.phpt",
    "Zend/tests/jump10-detect.phpt",
    "Zend/tests/bug39775-detect.phpt",
    "Zend/tests/bug52484_3-detect.phpt",
    "Zend/tests/or_001-detect.phpt",
    "Zend/tests/ns_077_2-detect.phpt",
    "Zend/tests/bug48770-detect.phpt",
    "Zend/tests/error_reporting03-detect.phpt",
    "Zend/tests/bug47320-detect.phpt",
    "Zend/tests/magic_by_ref_004-detect.phpt",
    "Zend/tests/inter_02-detect.phpt",
    "Zend/tests/bug48408-detect.phpt",
    "Zend/tests/bug38808-detect.phpt",
    "Zend/tests/bug61011-detect.phpt",
    "Zend/tests/bug33282-detect.phpt",
    "Zend/tests/div_002-detect.phpt",
    "Zend/tests/errmsg_036-detect.phpt",
    "Zend/tests/ns_051-detect.phpt",
    "Zend/tests/gc_021-detect.phpt",
    "Zend/tests/bug46308-detect.phpt",
    "Zend/tests/errmsg_014-detect.phpt",
    "Zend/tests/bug46196-detect.phpt",
    "Zend/tests/errmsg_029-detect.phpt",
    "Zend/tests/bug36759-detect.phpt",
    "Zend/tests/method_exists_002-detect.phpt",
    "Zend/tests/jump12-detect.phpt",
    "Zend/tests/dereference_002-detect.phpt",
    "Zend/tests/errmsg_038-detect.phpt",
    "Zend/tests/declare_004-detect.phpt",
    "Zend/tests/bug30707-detect.phpt",
    "Zend/tests/closure_022-detect.phpt",
    "Zend/tests/ns_013-detect.phpt",
    "Zend/tests/cast_to_array-detect.phpt",
    "Zend/tests/ns_024-detect.phpt",
    "Zend/tests/closure_034-detect.phpt",
    "Zend/tests/get_class_methods_001-detect.phpt",
    "Zend/tests/bug34617-detect.phpt",
    "Zend/tests/ns_038-detect.phpt",
    "Zend/tests/class_constants_003-detect.phpt",
    "Zend/tests/bug47593-detect.phpt",
    "Zend/tests/offset_null-detect.phpt",
    "Zend/tests/ns_046-detect.phpt",
    "Zend/tests/add_007-detect.phpt",
    "Zend/tests/bug32674-detect.phpt",
    "Zend/tests/class_alias_020-detect.phpt",
    "Zend/tests/bug34873-detect.phpt",
    "Zend/tests/bug48533-detect.phpt",
    "Zend/tests/add_006-detect.phpt",
    "Zend/tests/errmsg_037-detect.phpt",
    "Zend/tests/bug39297-detect.phpt",
    "Zend/tests/ns_001-detect.phpt",
    "Zend/tests/get_class_vars_003-detect.phpt",
    "Zend/tests/nowdoc_011-detect.phpt",
    "Zend/tests/bug60536_001-detect.phpt",
    "Zend/tests/magic_by_ref_001-detect.phpt",
    "Zend/tests/offset_assign-detect.phpt",
    "Zend/tests/trait_exists_001-detect.phpt",
    "Zend/tests/ns_078-detect.phpt",
    "Zend/tests/compare_002_64bit-detect.phpt",
    "Zend/tests/get_defined_functions_basic-detect.phpt",
    "Zend/tests/callable_type_hint_001-detect.phpt",
    "Zend/tests/bug26801-detect.phpt",
    "Zend/tests/closure_006-detect.phpt",
    "Zend/tests/ns_087-detect.phpt",
    "Zend/tests/bug39944-detect.phpt",
    "Zend/tests/bug22725-detect.phpt",
    "Zend/tests/bug34879-detect.phpt",
    "Zend/tests/lsb_013-detect.phpt",
    "Zend/tests/exception_009-detect.phpt",
    "Zend/tests/ns_077_4-detect.phpt",
    "Zend/tests/bug40815-detect.phpt",
    "Zend/tests/errmsg_003-detect.phpt",
    "Zend/tests/constants_009-detect.phpt",
    "Zend/tests/indirect_reference_this-detect.phpt",
    "Zend/tests/bug35163_2-detect.phpt",
    "Zend/tests/call_user_func_004-detect.phpt",
    "Zend/tests/binary-32bit-detect.phpt",
    "Zend/tests/method_static_var-detect.phpt",
    "Zend/tests/ns_054-detect.phpt",
    "Zend/tests/bug50816-detect.phpt",
    "Zend/tests/closure_018-detect.phpt",
    "Zend/tests/binary-detect.phpt",
    "Zend/tests/bug46665-detect.phpt",
    "Zend/tests/call_static_005-detect.phpt",
    "Zend/tests/bug33996-detect.phpt",
    "Zend/tests/bug28377-detect.phpt",
    "Zend/tests/bug32799-detect.phpt",
    "Zend/tests/bug55445-detect.phpt",
    "Zend/tests/bug33318-detect.phpt",
    "Zend/tests/closure_036-detect.phpt",
    "Zend/tests/bug27598-detect.phpt",
    "Zend/tests/exception_002-detect.phpt",
    "Zend/tests/multibyte/multibyte_encoding_004-detect.phpt",
    "Zend/tests/multibyte/multibyte_encoding_005-detect.phpt",
    "Zend/tests/multibyte/multibyte_encoding_003-detect.phpt",
    "Zend/tests/multibyte/multibyte_encoding_006-detect.phpt",
    "Zend/tests/multibyte/multibyte_encoding_002-detect.phpt",
    "Zend/tests/multibyte/multibyte_encoding_001-detect.phpt",
    "Zend/tests/get_class_vars_007-detect.phpt",
    "Zend/tests/jump13-detect.phpt",
    "Zend/tests/007-detect.phpt",
    "Zend/tests/bug31341-detect.phpt",
    "Zend/tests/unset_cv06-detect.phpt",
    "Zend/tests/bug48899-detect.phpt",
    "Zend/tests/bug33257-detect.phpt",
    "Zend/tests/str_offset_001-detect.phpt",
    "Zend/tests/ns_019-detect.phpt",
    "Zend/tests/and_001-detect.phpt",
    "Zend/tests/magic_by_ref_007-detect.phpt",
    "Zend/tests/bug60169-detect.phpt",
    "Zend/tests/function_exists_variation1-detect.phpt",
    "Zend/tests/error_reporting05-detect.phpt",
    "Zend/tests/bug44660-detect.phpt",
    "Zend/tests/float_prec_001-detect.phpt",
    "Zend/tests/nowdoc_009-detect.phpt",
    "Zend/tests/nowdoc_004-detect.phpt",
    "Zend/tests/objects_027-detect.phpt",
    "Zend/tests/bug32427-detect.phpt",
    "Zend/tests/array_type_hint_001-detect.phpt",
    "Zend/tests/int_underflow_32bit-detect.phpt",
    "Zend/tests/dereference_005-detect.phpt",
    "Zend/tests/objects_005-detect.phpt",
    "Zend/tests/bug29944-detect.phpt",
    "Zend/tests/double_to_string_64bit-detect.phpt",
    "Zend/tests/bug60444-detect.phpt",
    "Zend/tests/argument_restriction_004-detect.phpt",
    "Zend/tests/settype_resource-detect.phpt",
    "Zend/tests/get_class_vars_002-detect.phpt",
    "Zend/tests/bug44899-detect.phpt",
    "Zend/tests/bug61761-detect.phpt",
    "Zend/tests/bug47704-detect.phpt",
    "Zend/tests/is_a-detect.phpt",
    "Zend/tests/errmsg_020-detect.phpt",
    "Zend/tests/bug60138-detect.phpt",
    "Zend/tests/bug37251-detect.phpt",
    "Zend/tests/unset_cv02-detect.phpt",
    "Zend/tests/bug42819-detect.phpt",
    "Zend/tests/bug34893-detect.phpt",
    "Zend/tests/ns_049-detect.phpt",
    "Zend/tests/bug48667_2-detect.phpt",
    "Zend/tests/bug52193-detect.phpt",
    "Zend/tests/objects_023-detect.phpt",
    "Zend/tests/bug43183-detect.phpt",
    "Zend/tests/bug54585-detect.phpt",
    "Zend/tests/bug41633_3-detect.phpt",
    "Zend/tests/errmsg_024-detect.phpt",
    "Zend/tests/settype_int-detect.phpt",
    "Zend/tests/closure_001-detect.phpt",
    "Zend/tests/ns_075-detect.phpt",
    "Zend/tests/dereference_004-detect.phpt",
    "Zend/tests/catch_003-detect.phpt",
    "Zend/tests/bug32226-detect.phpt",
    "Zend/tests/errmsg_021-detect.phpt",
    "Zend/tests/bug54305-detect.phpt",
    "Zend/tests/sub_001-detect.phpt",
    "Zend/tests/shift_002-detect.phpt",
    "Zend/tests/closure_030-detect.phpt",
    "Zend/tests/dtor_scope-detect.phpt",
    "Zend/tests/errmsg_004-detect.phpt",
    "Zend/tests/bug55135-detect.phpt",
    "Zend/tests/bug33171-detect.phpt",
    "Zend/tests/inter_01-detect.phpt",
    "Zend/tests/call_with_refs-detect.phpt",
    "Zend/tests/access_modifiers_012-detect.phpt",
    "Zend/tests/002-detect.phpt",
    "Zend/tests/gc_004-detect.phpt",
    "Zend/tests/bug49269-detect.phpt",
    "Zend/tests/declare_001-detect.phpt",
    "Zend/tests/bug48693-detect.phpt",
    "Zend/tests/ns_057-detect.phpt",
    "Zend/tests/indirect_method_call_001-detect.phpt",
    "Zend/tests/objects_004-detect.phpt",
    "Zend/tests/008-detect.phpt",
    "Zend/tests/bug49908-detect.phpt",
    "Zend/tests/method_exists-detect.phpt",
    "Zend/tests/gc_023-detect.phpt",
    "Zend/tests/bug47801-detect.phpt",
    "Zend/tests/bug52614-detect.phpt",
    "Zend/tests/closure_002-detect.phpt",
    "Zend/tests/constants_005-detect.phpt",
    "Zend/tests/lsb_008-detect.phpt",
    "Zend/tests/018-detect.phpt",
    "Zend/tests/magic_methods_002-detect.phpt",
    "Zend/tests/class_alias_008-detect.phpt",
    "Zend/tests/bug39018-detect.phpt",
    "Zend/tests/exception_004-detect.phpt",
    "Zend/tests/errmsg_015-detect.phpt",
    "Zend/tests/closure_031-detect.phpt",
    "Zend/tests/bug52060-detect.phpt",
    "Zend/tests/ns_072-detect.phpt",
    "Zend/tests/ns_071-detect.phpt",
    "Zend/tests/bug38461-detect.phpt",
    "Zend/tests/bug30140-detect.phpt",
    "Zend/tests/bug60536_003-detect.phpt",
    "Zend/tests/bug43344_5-detect.phpt",
    "Zend/tests/jump02-detect.phpt",
    "Zend/tests/bug43851-detect.phpt",
    "Zend/tests/debug_print_backtrace_limit-detect.phpt",
    "Zend/tests/bug24773-detect.phpt",
    "Zend/tests/error_reporting07-detect.phpt",
    "Zend/tests/bug45186-detect.phpt",
    "Zend/tests/bug48228-detect.phpt",
    "Zend/tests/objects_018-detect.phpt",
    "Zend/tests/closure_014-detect.phpt",
    "Zend/tests/clone_002-detect.phpt",
    "Zend/tests/034-detect.phpt",
    "Zend/tests/ns_083-detect.phpt",
    "Zend/tests/gc_002-detect.phpt",
    "Zend/tests/objects_022-detect.phpt",
    "Zend/tests/lsb_017-detect.phpt",
    "Zend/tests/bug49472-detect.phpt",
    "Zend/tests/bug38211-detect.phpt",
    "Zend/tests/035-detect.phpt",
    "Zend/tests/compare_005-detect.phpt",
    "Zend/tests/errmsg_022-detect.phpt",
    "Zend/tests/closure_035-detect.phpt",
    "Zend/tests/bug42820-detect.phpt",
    "Zend/tests/bug47981-detect.phpt",
    "Zend/tests/bug30791-detect.phpt",
    "Zend/tests/indirect_method_call_004-detect.phpt",
    "Zend/tests/access_modifiers_010-detect.phpt",
    "Zend/tests/bug24884-detect.phpt",
    "Zend/tests/ns_004-detect.phpt",
    "Zend/tests/compare_006_64bit-detect.phpt",
    "Zend/tests/closure_017-detect.phpt",
    "Zend/tests/class_alias_009-detect.phpt",
    "Zend/tests/lsb_022-detect.phpt",
    "Zend/tests/bug47572-detect.phpt",
    "Zend/tests/bug42802-detect.phpt",
    "Zend/tests/ns_034-detect.phpt",
    "Zend/tests/bug27731-detect.phpt",
    "Zend/tests/exception_010-detect.phpt",
    "Zend/tests/nowdoc_007-detect.phpt",
    "Zend/tests/bug34137-detect.phpt",
    "Zend/tests/closure_044-detect.phpt",
    "Zend/tests/unexpected_ref_bug-detect.phpt",
    "Zend/tests/bug40784-detect.phpt",
    "Zend/tests/013-detect.phpt",
    "Zend/tests/lsb_002-detect.phpt",
    "Zend/tests/ns_085-detect.phpt",
    "Zend/tests/037-detect.phpt",
    "Zend/tests/016-detect.phpt",
    "Zend/tests/bug48667_1-detect.phpt",
    "Zend/tests/list_004-detect.phpt",
    "Zend/tests/bug52484_2-detect.phpt",
    "Zend/tests/ns_041-detect.phpt",
    "Zend/tests/bug43344_3-detect.phpt",
    "Zend/tests/xor_002-detect.phpt",
    "Zend/tests/nowdoc_003-detect.phpt",
    "Zend/tests/errmsg_019-detect.phpt",
    "Zend/tests/bug37632-detect.phpt",
    "Zend/tests/bug29505-detect.phpt",
    "Zend/tests/bug50005-detect.phpt",
    "Zend/tests/bug60139-detect.phpt",
    "Zend/tests/errmsg_012-detect.phpt",
    "Zend/tests/gc_001-detect.phpt",
    "Zend/tests/bug39304_2_4-detect.phpt",
    "Zend/tests/zend_strtod-detect.phpt",
    "Zend/tests/bug44414-detect.phpt",
    "Zend/tests/bug31177-detect.phpt",
    "Zend/tests/bug53305-detect.phpt",
    "Zend/tests/heredoc_016-detect.phpt",
    "Zend/tests/constants_001-detect.phpt",
    "Zend/tests/errmsg_001-detect.phpt",
    "Zend/tests/errmsg_006-detect.phpt",
    "Zend/tests/bug48248-detect.phpt",
    "Zend/tests/class_constants_004-detect.phpt",
    "Zend/tests/bug51421-detect.phpt",
    "Zend/tests/xor_003-detect.phpt",
    "Zend/tests/bug29896-detect.phpt",
    "Zend/tests/bug26010-detect.phpt",
    "Zend/tests/bug_debug_backtrace-detect.phpt",
    "Zend/tests/bug43344_10-detect.phpt",
    "Zend/tests/call_user_func_005-detect.phpt",
    "Zend/tests/get_class_methods_002-detect.phpt",
    "Zend/tests/gc_024-detect.phpt",
    "Zend/tests/bug36006-detect.phpt",
    "Zend/tests/bug40705-detect.phpt",
    "Zend/tests/bug29674-detect.phpt",
    "Zend/tests/objects_002-detect.phpt",
    "Zend/tests/constants_002-detect.phpt",
    "Zend/tests/access_modifiers_007-detect.phpt",
    "Zend/tests/bug34518-detect.phpt",
    "Zend/tests/ns_016-detect.phpt",
    "Zend/tests/bug37667-detect.phpt",
    "Zend/tests/exception_handler_001-detect.phpt",
    "Zend/tests/magic_by_ref_002-detect.phpt",
    "Zend/tests/bug62763-detect.phpt",
    "Zend/tests/025-detect.phpt",
    "Zend/tests/heredoc_012-detect.phpt",
    "Zend/tests/foreach_unset_globals-detect.phpt",
    "Zend/tests/dereference_001-detect.phpt",
    "Zend/tests/bug47714-detect.phpt",
    "Zend/tests/unset_cv07-detect.phpt",
    "Zend/tests/gc_013-detect.phpt",
    "Zend/tests/constants/dir-constant-eval-detect.phpt",
    "Zend/tests/constants/dir-constant-nested_includes-detect.phpt",
    "Zend/tests/constants/dir-constant-includes-detect.phpt",
    "Zend/tests/constants/dir-constant-normal-detect.phpt",
    "Zend/tests/bug41372-detect.phpt",
    "Zend/tests/lsb_003-detect.phpt",
    "Zend/tests/bug38779-detect.phpt",
    "Zend/tests/bug45910_2-detect.phpt",
    "Zend/tests/012-detect.phpt",
    "Zend/tests/objects_028-detect.phpt",
    "Zend/tests/bug30080-detect.phpt",
    "Zend/tests/038-detect.phpt",
    "Zend/tests/call_static_003-detect.phpt",
    "Zend/tests/bug32428-detect.phpt",
    "Zend/tests/bug51791-detect.phpt",
    "Zend/tests/bug34310-detect.phpt",
    "Zend/tests/ns_069-detect.phpt",
    "Zend/tests/bug38624-detect.phpt",
    "Zend/tests/bug43332_2-detect.phpt",
    "Zend/tests/not_001-detect.phpt",
    "Zend/tests/errmsg_025-detect.phpt",
    "Zend/tests/str_offset_002-detect.phpt",
    "Zend/tests/errmsg_035-detect.phpt",
    "Zend/tests/bug60099-detect.phpt",
    "Zend/tests/dereference_014-detect.phpt",
    "Zend/tests/class_alias_002-detect.phpt",
    "Zend/tests/bug53629-detect.phpt",
    "Zend/tests/abstract-static-detect.phpt",
    "Zend/tests/closure_016-detect.phpt",
    "Zend/tests/bug60536_005-detect.phpt",
    "Zend/tests/bug61273-detect.phpt",
    "Zend/tests/closure_008-detect.phpt",
    "Zend/tests/errmsg_027-detect.phpt",
    "Zend/tests/unset_cv04-detect.phpt",
    "Zend/tests/ns_025-detect.phpt",
    "Zend/tests/ns_055-detect.phpt",
    "Zend/tests/021-detect.phpt",
    "Zend/tests/bug23104-detect.phpt",
    "Zend/tests/bug55247-detect.phpt",
    "Zend/tests/bug34260-detect.phpt",
    "Zend/tests/dereference_013-detect.phpt",
    "Zend/tests/debug_backtrace_limit-detect.phpt",
    "Zend/tests/bug30161-detect.phpt",
    "Zend/tests/halt_compiler2-detect.phpt",
    "Zend/tests/class_alias_003-detect.phpt",
    "Zend/tests/class_alias_016-detect.phpt",
    "Zend/tests/ns_027-detect.phpt",
    "Zend/tests/ns_066-detect.phpt",
    "Zend/tests/argument_restriction_002-detect.phpt",
    "Zend/tests/add_001-detect.phpt",
    "Zend/tests/ns_060-detect.phpt",
    "Zend/tests/call_static_002-detect.phpt",
    "Zend/tests/fr47160-detect.phpt",
    "Zend/tests/jump07-detect.phpt",
    "Zend/tests/closure_028-detect.phpt",
    "Zend/tests/interface_exists_002-detect.phpt",
    "Zend/tests/bug60536_004-detect.phpt",
    "Zend/tests/bug34678-detect.phpt",
    "Zend/tests/bug53632-detect.phpt",
    "Zend/tests/bug35106-detect.phpt",
    "Zend/tests/array_unshift_COW-detect.phpt",
    "Zend/tests/bug45742-detect.phpt",
    "Zend/tests/bug37144-detect.phpt",
    "Zend/tests/bug62956-detect.phpt",
    "Zend/tests/gc_017-detect.phpt",
    "Zend/tests/heredoc_003-detect.phpt",
    "Zend/tests/ns_052-detect.phpt",
    "Zend/tests/halt_compiler1-detect.phpt",
    "Zend/tests/bug47596-detect.phpt",
    "Zend/tests/bug46811-detect.phpt",
    "Zend/tests/assign_to_var_003-detect.phpt",
    "Zend/tests/indirect_call_array_002-detect.phpt",
    "Zend/tests/ns_077_8-detect.phpt",
    "Zend/tests/closure_013-detect.phpt",
    "Zend/tests/ns_056-detect.phpt",
    "Zend/tests/bug55156-detect.phpt",
    "Zend/tests/bug43344_11-detect.phpt",
    "Zend/tests/bug41351_3-detect.phpt",
    "Zend/tests/int_overflow_64bit-detect.phpt",
    "Zend/tests/bug53971-detect.phpt",
    "Zend/tests/zend_operators-detect.phpt",
    "Zend/tests/list_005-detect.phpt",
    "Zend/tests/lsb_004-detect.phpt",
    "Zend/tests/bug50810-detect.phpt",
    "Zend/tests/settype_double-detect.phpt",
    "Zend/tests/errmsg_005-detect.phpt",
    "Zend/tests/bug61087-detect.phpt",
    "Zend/tests/errmsg_031-detect.phpt",
    "Zend/tests/gc_022-detect.phpt",
    "Zend/tests/ns_007-detect.phpt",
    "Zend/tests/access_modifiers_009-detect.phpt",
    "Zend/tests/objects_003-detect.phpt",
    "Zend/tests/offset_array-detect.phpt",
    "Zend/tests/assign_to_var_001-detect.phpt",
    "Zend/tests/class_constants_001-detect.phpt",
    "Zend/tests/bug40261-detect.phpt",
    "Zend/tests/assign_to_var_004-detect.phpt",
    "Zend/tests/lsb_006-detect.phpt",
    "Zend/tests/settype_array-detect.phpt",
    "Zend/tests/closure_020-detect.phpt",
    "Zend/tests/class_alias_013-detect.phpt",
    "Zend/tests/shift_001-detect.phpt",
    "Zend/tests/ns_032-detect.phpt",
    "Zend/tests/class_alias_015-detect.phpt",
    "Zend/tests/bug47054-detect.phpt",
    "Zend/tests/constants_006-detect.phpt",
    "Zend/tests/bug41961-detect.phpt",
    "Zend/tests/bug50261-detect.phpt",
    "Zend/tests/gc_009-detect.phpt",
    "Zend/tests/gc_005-detect.phpt",
    "Zend/tests/ns_077_6-detect.phpt",
    "Zend/tests/bug52041-detect.phpt",
    "Zend/tests/closure_009-detect.phpt",
    "Zend/tests/bug60613-detect.phpt",
    "Zend/tests/compare_004_64bit-detect.phpt",
    "Zend/tests/ns_059-detect.phpt",
    "Zend/tests/lsb_016-detect.phpt",
    "Zend/tests/get_class_methods_003-detect.phpt",
    "Zend/tests/access_modifiers_006-detect.phpt",
    "Zend/tests/indirect_call_array_004-detect.phpt",
    "Zend/tests/bug26281-detect.phpt",
    "Zend/tests/bug39990-detect.phpt",
    "Zend/tests/bug36268-detect.phpt",
    "Zend/tests/bug41633_2-detect.phpt",
    "Zend/tests/magic_methods_008-detect.phpt",
    "Zend/tests/objects_020-detect.phpt",
    "Zend/tests/bug35634-detect.phpt",
    "Zend/tests/bug42772-detect.phpt",
    "Zend/tests/closure_023-detect.phpt",
    "Zend/tests/bug27304-detect.phpt",
    "Zend/tests/bug44830-detect.phpt",
    "Zend/tests/bug19859-detect.phpt",
    "Zend/tests/jump14-detect.phpt",
    "Zend/tests/bug54262-detect.phpt",
    "Zend/tests/bug26166-detect.phpt",
    "Zend/tests/bug30725-detect.phpt",
    "Zend/tests/030-detect.phpt",
    "Zend/tests/jump03-detect.phpt",
    "Zend/tests/bug43128-detect.phpt",
    "Zend/tests/bug43200_2-detect.phpt",
    "Zend/tests/closure_042-detect.phpt",
    "Zend/tests/bug44069-detect.phpt",
    "Zend/tests/bug30394-detect.phpt",
    "Zend/tests/compare_006-detect.phpt",
    "Zend/tests/nowdoc_017-detect.phpt",
    "Zend/tests/double_to_string-detect.phpt",
    "Zend/tests/bug37046-detect.phpt",
    "Zend/tests/lsb_018-detect.phpt",
    "Zend/tests/bug52051-detect.phpt",
    "Zend/tests/lsb_021-detect.phpt",
    "Zend/tests/bug41640-detect.phpt",
    "Zend/tests/jump04-detect.phpt",
    "Zend/tests/access_modifiers_008-detect.phpt",
    "Zend/tests/bug46304-detect.phpt",
    "Zend/tests/gc_018-detect.phpt",
    "Zend/tests/magic_by_ref_003-detect.phpt",
    "Zend/tests/bug60978-detect.phpt",
    "Zend/tests/bug34045-detect.phpt",
    "Zend/tests/bug50255-detect.phpt",
    "Zend/tests/ns_020-detect.phpt",
    "Zend/tests/gc_027-detect.phpt",
    "Zend/tests/bug32322-detect.phpt",
    "Zend/tests/bug45186_2-detect.phpt",
    "Zend/tests/bug39438-detect.phpt",
    "Zend/tests/gc_029-detect.phpt",
    "Zend/tests/ns_014-detect.phpt",
    "Zend/tests/add_002-detect.phpt",
    "Zend/tests/bug52001-detect.phpt",
    "Zend/tests/bug48770_3-detect.phpt",
    "Zend/tests/concat_001-detect.phpt",
    "Zend/tests/each_002-detect.phpt",
    "Zend/tests/bug39018_2-detect.phpt",
    "Zend/tests/bug42767-detect.phpt",
    "Zend/tests/catch_002-detect.phpt",
    "Zend/tests/magic_methods_003-detect.phpt",
    "Zend/tests/errmsg_017-detect.phpt",
    "Zend/tests/cast_to_object-detect.phpt",
    "Zend/tests/foreach_002-detect.phpt",
    "Zend/tests/bug33512-detect.phpt",
    "Zend/tests/bug35655-detect.phpt",
    "Zend/tests/bug48912-detect.phpt",
    "Zend/tests/bug45147-detect.phpt",
    "Zend/tests/032-detect.phpt",
    "Zend/tests/bug31098-detect.phpt",
    "Zend/tests/bug29368-detect.phpt",
    "Zend/tests/ns_058-detect.phpt",
    "Zend/tests/bug47165-detect.phpt",
    "Zend/tests/objects_017-detect.phpt",
    "Zend/tests/closure_027-detect.phpt",
    "Zend/tests/ns_063-detect.phpt",
    "Zend/tests/bug50174-detect.phpt",
    "Zend/tests/ns_018-detect.phpt",
    "Zend/tests/error_reporting01-detect.phpt",
    "Zend/tests/settype_object-detect.phpt",
    "Zend/tests/errmsg_039-detect.phpt",
    "Zend/tests/bug60825-detect.phpt",
    "Zend/tests/constructor_args-detect.phpt",
    "Zend/tests/errmsg_026-detect.phpt",
    "Zend/tests/bug35163-detect.phpt",
    "Zend/tests/access_modifiers_011-detect.phpt",
    "Zend/tests/magic_methods_004-detect.phpt",
    "Zend/tests/ns_021-detect.phpt",
    "Zend/tests/019-detect.phpt",
    "Zend/tests/nowdoc_014-detect.phpt",
    "Zend/tests/unset_cv03-detect.phpt",
    "Zend/tests/list_002-detect.phpt",
    "Zend/tests/lsb_015-detect.phpt",
    "Zend/tests/argument_restriction_001-detect.phpt",
    "Zend/tests/bug47771-detect.phpt",
    "Zend/tests/magic_methods_010-detect.phpt",
    "Zend/tests/bug44827-detect.phpt",
    "Zend/tests/bug33771-detect.phpt",
    "Zend/tests/magic_methods_009-detect.phpt",
    "Zend/tests/bug37212-detect.phpt",
    "Zend/tests/inter_007-detect.phpt",
    "Zend/tests/bug41401-detect.phpt",
    "Zend/tests/bug60536_002-detect.phpt",
    "Zend/tests/gc_015-detect.phpt",
    "Zend/tests/bug35470-detect.phpt",
    "Zend/tests/bug60771-detect.phpt",
    "Zend/tests/inter_05-detect.phpt",
    "Zend/tests/selfParent_001-detect.phpt",
    "Zend/tests/bug43450-detect.phpt",
    "Zend/tests/halt03-detect.phpt",
    "Zend/tests/each_003-detect.phpt",
    "Zend/tests/lsb_011-detect.phpt",
    "Zend/tests/compare_001_64bit-detect.phpt",
    "Zend/tests/errmsg_042-detect.phpt",
    "Zend/tests/bug39127-detect.phpt",
    "Zend/tests/bug30998-detect.phpt",
    "Zend/tests/globals_002-detect.phpt",
    "Zend/tests/magic_methods_001-detect.phpt",
    "Zend/tests/bug48409-detect.phpt",
    "Zend/tests/objects_031-detect.phpt",
    "Zend/tests/closure_010-detect.phpt",
    "Zend/tests/nowdoc_012-detect.phpt",
    "Zend/tests/bug34786-detect.phpt",
    "Zend/tests/not_002-detect.phpt",
    "Zend/tests/class_alias_021-detect.phpt",
    "Zend/tests/ns_005-detect.phpt",
    "Zend/tests/closure_038-detect.phpt",
    "Zend/tests/ns_015-detect.phpt",
    "Zend/tests/bug46409-detect.phpt",
    "Zend/tests/024-detect.phpt",
    "Zend/tests/bug33277-detect.phpt",
    "Zend/tests/bug45178-detect.phpt",
    "Zend/tests/inter_04-detect.phpt",
    "Zend/tests/error_reporting02-detect.phpt",
    "Zend/tests/bug43344_4-detect.phpt",
    "Zend/tests/nowdoc_001-detect.phpt",
    "Zend/tests/dynamic_call_001-detect.phpt",
    "Zend/tests/instanceof-detect.phpt",
    "Zend/tests/bug35163_3-detect.phpt",
    "Zend/tests/strict_001-detect.phpt",
    "Zend/tests/bug60350-detect.phpt",
    "Zend/tests/bug37707-detect.phpt",
    "Zend/tests/009-detect.phpt",
    "Zend/tests/unset_cv11-detect.phpt",
    "Zend/tests/bug45862-detect.phpt",
    "Zend/tests/bug21478-detect.phpt",
    "Zend/tests/bug46701-detect.phpt",
    "Zend/tests/bug39346-detect.phpt",
    "Zend/tests/bug46813-detect.phpt",
    "Zend/tests/get_defined_functions_error-detect.phpt",
    "Zend/tests/dereference_011-detect.phpt",
    "Zend/tests/ns_050-detect.phpt",
    "Zend/tests/call_user_func_003-detect.phpt",
    "Zend/tests/get_class_vars_005-detect.phpt",
    "Zend/tests/result_unused-detect.phpt",
    "Zend/tests/ns_080-detect.phpt",
    "Zend/tests/bug54910-detect.phpt",
    "Zend/tests/bug42859-detect.phpt",
    "Zend/tests/errmsg_008-detect.phpt",
    "Zend/tests/objects_012-detect.phpt",
    "Zend/tests/bug42937-detect.phpt",
    "Zend/tests/ns_033-detect.phpt",
    "Zend/tests/class_alias_004-detect.phpt",
    "Zend/tests/magic_by_ref_006-detect.phpt",
    "Zend/tests/bug30519-detect.phpt",
    "Zend/tests/objects_025-detect.phpt",
    "Zend/tests/bug41633_1-detect.phpt",
    "Zend/tests/clone_001-detect.phpt",
    "Zend/tests/bug47109-detect.phpt",
    "Zend/tests/bug39721-detect.phpt",
    "Zend/tests/ns_073-detect.phpt",
    "Zend/tests/magic_methods_007-detect.phpt",
    "Zend/tests/bug61782-detect.phpt",
    "Zend/tests/gc_028-detect.phpt",
    "Zend/tests/bug39825-detect.phpt",
    "Zend/tests/errmsg_010-detect.phpt",
    "Zend/tests/nowdoc_006-detect.phpt",
    "Zend/tests/trait_exists_002-detect.phpt",
    "Zend/tests/cast_to_double-detect.phpt",
    "Zend/tests/hex_overflow_32bit-detect.phpt",
    "Zend/tests/bug55705-detect.phpt",
    "Zend/tests/bug24436-detect.phpt",
    "Zend/tests/errmsg_002-detect.phpt",
    "Zend/tests/function_arguments_002-detect.phpt",
    "Zend/tests/isset_003-detect.phpt",
    "sapi/cgi/tests/010-detect.phpt",
    "sapi/cgi/tests/005-detect.phpt",
    "sapi/cgi/tests/011-detect.phpt",
    "sapi/cgi/tests/001-detect.phpt",
    "sapi/cgi/tests/004-detect.phpt",
    "sapi/cgi/tests/003-detect.phpt",
    "sapi/cgi/tests/bug61605-detect.phpt",
    "sapi/cgi/tests/006-detect.phpt",
    "sapi/cgi/tests/apache_request_headers-detect.phpt",
    "sapi/cgi/tests/007-detect.phpt",
    "sapi/cgi/tests/002-detect.phpt",
    "sapi/cgi/tests/008-detect.phpt",
    "sapi/cgi/tests/009-detect.phpt",
    "sapi/tests/test004-detect.phpt",
    "sapi/tests/test007-detect.phpt",
    "sapi/tests/test001-detect.phpt",
    "sapi/tests/test005-detect.phpt",
    "sapi/tests/test002-detect.phpt",
    "sapi/tests/test006-detect.phpt",
    "sapi/tests/test003-detect.phpt",
    "sapi/cli/tests/002-win32-detect.phpt",
    "sapi/cli/tests/php_cli_server_005-detect.phpt",
    "sapi/cli/tests/010-detect.phpt",
    "sapi/cli/tests/005-detect.phpt",
    "sapi/cli/tests/php_cli_server_003-detect.phpt",
    "sapi/cli/tests/php_cli_server_017-detect.phpt",
    "sapi/cli/tests/014-detect.phpt",
    "sapi/cli/tests/011-detect.phpt",
    "sapi/cli/tests/003-2-detect.phpt",
    "sapi/cli/tests/017-detect.phpt",
    "sapi/cli/tests/php_cli_server_007-detect.phpt",
    "sapi/cli/tests/010-2-detect.phpt",
    "sapi/cli/tests/php_cli_server_006-detect.phpt",
    "sapi/cli/tests/001-detect.phpt",
    "sapi/cli/tests/php_cli_server_002-detect.phpt",
    "sapi/cli/tests/php_cli_server_004-detect.phpt",
    "sapi/cli/tests/015-detect.phpt",
    "sapi/cli/tests/bug61546-detect.phpt",
    "sapi/cli/tests/004-detect.phpt",
    "sapi/cli/tests/php_cli_server_015-detect.phpt",
    "sapi/cli/tests/php_cli_server_011-detect.phpt",
    "sapi/cli/tests/003-detect.phpt",
    "sapi/cli/tests/020-detect.phpt",
    "sapi/cli/tests/022-detect.phpt",
    "sapi/cli/tests/006-detect.phpt",
    "sapi/cli/tests/php_cli_server_009-detect.phpt",
    "sapi/cli/tests/007-detect.phpt",
    "sapi/cli/tests/002-detect.phpt",
    "sapi/cli/tests/008-detect.phpt",
    "sapi/cli/tests/018-detect.phpt",
    "sapi/cli/tests/php_cli_server_010-detect.phpt",
    "sapi/cli/tests/013-detect.phpt",
    "sapi/cli/tests/016-detect.phpt",
    "sapi/cli/tests/012-detect.phpt",
    "sapi/cli/tests/php_cli_server_013-detect.phpt",
    "sapi/cli/tests/021-detect.phpt",
    "sapi/cli/tests/bug61977-detect.phpt",
    "sapi/cli/tests/bug44564-detect.phpt",
    "sapi/cli/tests/php_cli_server_008-detect.phpt",
    "sapi/cli/tests/php_cli_server_014-detect.phpt",
    "sapi/cli/tests/019-detect.phpt",
    "sapi/cli/tests/php_cli_server_001-detect.phpt",
    "sapi/cli/tests/php_cli_server_016-detect.phpt",
    "sapi/cli/tests/009-detect.phpt",
    "sapi/cli/tests/php_cli_server_012-detect.phpt",
};

int run_test(char* name) {
    char buffer[1024];
    sprintf(buffer, "./sapi/cli/php ../php-helper.php -p ./sapi/cli/php -q %s", name);
    int killed = system("killall php &> /dev/null");
    if (killed == 0) {
        system("echo A php process was killed > A-PHP-Process-Was-Killed");    }
    int res = system(buffer);

if (res != 0) {        printf("PASS: %s\n", name);
        return 0;
    } else {
        printf("FAIL: %s\n", name);
        return 1;
    }
}

/*
 * Usage:
 * ./php-run-tests TESTNUM [True]
 * ./php-run-tests length [True]
 *     TESTNUM is an integer specifying the test to run. If 'length' is given,
 *       we return the total number of tests to stdout then exit.
 *     The optional third argument should be True if we want to run
 *       abreviated tests.
*/
int main(int argc, char** argv) {
    // If we wanted the length, atoi will return 0 which is just fine
    int num = atoi(argv[1]) - 1;

    char** tests;
    int length;
    tests = TESTS;
    length = sizeof(TESTS)/sizeof(char*);
    // Check if we just wanted the number of tests, if so give it and die
    if (argv[1][0] == 'l') {
        printf("%d", length);
        exit(0);
    }

    if (num < 0 || num >= length) {
        fprintf(stderr, "Invalid test number: %d\n", num);
        exit(255);
    }
    char* name = tests[num];
    return run_test(name);
}
